;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	CONVERT_E2M_PIA
;
; PURPOSE:
;	Convert the ERD2MAP output data into a phtscp structure 
;
; CATEGORY:
;       PIA/E2M - Interface routine
;
; CALLING SEQUENCE:
;	convert_e2m_pia, unit, det, admn, ordr, hdr
;
; INPUTS:
;	unit:  	measurement's start time 
;	det:	detector ('C1' or 'C2')
;	admn:	measurement's internal name
;	ordr:	position of measurement in observation
;	hdr:	measurement's header
;	
; OUTPUTS:
;	(none) 
;	
; COMMON BLOCKS:
;	common_all	Commons from ERDTOMAP programs
;	drift_struct	E2M-PIA I/F common
;	
; RESTRICTIONS:
;	(none)
;	
; PROCEDURES USED:
;	PIA LIB:
;		fill_scp
;	
; MODIFICATION HISTORY:
;	Written by: C.Gabriel (ESA/VILSPA)		April 2000
;	Bugs fixed: including open and resistor pixel, raster point
;	            correctly dimensioned - October 2000
;	V9.1
;	Using erpid (extended RPID) instead of rpid for coping with 
;	 data reduction case.  [CG, Feb. 2001]			
;-

PRO convert_e2m_pia, unit, det, admn, ordr, hdr

@common_all
COMMON drift_struct, drift_struct
@phtscp
;rjt	IF N_ELEMENTS(step) NE N_ELEMENTS(drift_struct.modelfit(0,*)) THEN BEGIN
        IF N_ELEMENTS(estep) NE N_ELEMENTS(drift_struct.modelfit(0,*)) THEN BEGIN    
	   PRINT, 'ERROR: different sizes of STEP and DRIFT_STRUCT, returning'
	   RETURN
	ENDIF

; array bad is used for flagging glitches, probably ebad is not updated
; so we have to use bad for it - 
	itrans = WHERE(drift_struct.itrans EQ 1, ntrans)
        l_maskout = drift_struct.maskout
        IF ntrans EQ N_ELEMENTS(bad(0,*)) THEN BEGIN
	   npix = N_ELEMENTS(bad(*,0))
           FOR ipix=0,npix-1 DO BEGIN
		ibad = WHERE(bad(ipix,*) NE 0, nbad)
	        IF nbad GT 0 THEN $
	            l_maskout(ipix,itrans(ibad)) = 1
           ENDFOR
        ENDIF ELSE $
		PRINT, 'WARNING,WARNING, nr of transmitted points NE arrays '

;rjt	dif_step = step(1:*) - step(0:N_ELEMENTS(step)-2)
        dif_step = estep(1:*) - estep(0:N_ELEMENTS(estep)-2)
	dif_step = [1,dif_step]
;rjt	ind_step = WHERE(dif_step NE 0 AND fpt EQ 1, dims)
        ind_step = WHERE(dif_step NE 0 AND fpoint EQ 1, dims)

; illumination for all chopper plateaus
        ii=where(drift_struct.itrans(ind_step) eq 1,nii)
; take only transmitted points
	ind_step = ind_step(ii)

		
	all_model = drift_struct.modelfit(*,ind_step) * $
	             drift_struct.respcorr(*,ind_step)

	FXADDPAR, hdr, 'PIA_E2MD', 1, ' SCP derived from P32-Special Processing',$
		BEFORE='XTENSION'
	
	ired = WHERE(STRMID(hdr,0,3) EQ 'END', nnn)
	IF nnn GT 0 THEN hdr = hdr(0:ired(0))
	
; use the two dimensionals maskout and msk for getting the indices

	ind_bad = WHERE(l_maskout+drift_struct.msk NE 0, nbad)
	ind_good = WHERE(l_maskout+drift_struct.msk EQ 0, ngood)

; but define badp and goodp also as 2-dimensionals
	badp =  BYTE(drift_struct.maskout) * 0B 
	goodp = badp
	IF nbad GT 0 THEN badp(ind_bad) = 1B
	IF ngood GT 0 THEN goodp(ind_good) = 1B

; take only transmitted points for establishing step size	
	dif_TRANS_step = step(1:*) - step(0:N_ELEMENTS(step)-2)
	ind_med_step = WHERE(dif_trans_step NE 0 AND fpt EQ 1)
	dif_ims = ind_med_step(1:*) - ind_med_step(0:N_ELEMENTS(ind_med_step)-2)
	mmed_step = FIX(MEDIAN(dif_ims))
	
	ind_fur = ind_step + mmed_step
; the last step can contain less points
	ind_fur(N_ELEMENTS(ind_fur)-1) = N_ELEMENTS(badp(0,*)) -1
	
	npix = N_ELEMENTS(s(*,0))
	flaggi = BYTE(all_model)*0B
	
	comb_rms = all_model * 0.
	
	FOR ipix=0, npix-1 DO $
	 FOR icp = 0,N_ELEMENTS(ind_fur)-1 DO BEGIN
	  
	  flaggi(ipix,icp) = MIN(badp(ipix,ind_step(icp):ind_fur(icp))) 
	  
	  sum_of_good = TOTAL(goodp(ipix,ind_step(icp):ind_fur(icp)))
	  IF sum_of_good NE 0 THEN $ 
	           comb_rms(ipix,icp) = TOTAL( $
	  		rms(ind_step(icp):ind_fur(icp)) * $
			goodp(ind_step(icp):ind_fur(icp)) ) / $
			 sum_of_good / SQRT(sum_of_good)
			 
	 ENDFOR
	FLAGGI = FLAGGI*2
;rjt	dummy = CHOP(ind_step)*0
        dummy = ECHOP(ind_step)*0

; preserve the same structure as normal SCP, including open and resistor pixel
        scpmod = FLTARR(npix+2,N_ELEMENTS(ind_step))
        scpcomb_rms = scpmod
	scprms = scpmod
	scpflag = FIX(scpmod)

	scpmod(0:npix-1,*) = all_model
        scpcomb_rms(0:npix-1,*) = comb_rms
        scprms(0:npix-1,*) = rms(*,ind_step)
	scpflag(0:npix-1,*) = flaggi 
        
	PHTSCPT = {			$
	  UNIT: UNIT, 			$
	  DET:  DET,			$
	  ADMN: ADMN,			$
	  ORDR: ORDR,			$
	  DIMS: dims,			$
	  HDR: HDR,			$
	  TIME: etime(ind_step),	$
	  AVER: scpmod,			$
	  AVUN: scpcomb_rms,		$
	  MEDI: scpmod,			$
	  Q1ME: scpmod+scprms,		$
	  Q3ME: scpmod-scprms,		$
	  FLAG: scpflag,				$
	  CHOP: ECHOP(ind_step),		$
	  STEP: ESTEP(ind_step),		$
	  FCSN: dummy,				$
	  TEMP: dummy,				$
	  FCS1: dummy,				$
	  FCS2: dummy,				$
	  TKEY: dummy,				$
	  RPID: erpid(*,ind_step),		$
	  ARSI: dummy,				$
	  ANDR: dummy,				$
	  AVSD: dummy  }
	  
	fill_scp, phtscpt
	SCPPOS = N_ELEMENTS(PHTSCP) - 1
	SCPPS = SCPPOS 

	
	RETURN
	END
	
	
