;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       CORR_RESET_INT
;
; PURPOSE:
;	Apply the reset interval correction to an array
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       CORR_RESET_INT, IN_SIG, DET, INTT, DRED, OUT_SIG, CORR
;
; INPUT:
;       IN_SIG:	signal input array
;	DET:	detector (eg 'C1', 'C100', 'P1')
;	INTT:	reset interval - INTEGER (PHT nomenclature: t_int=1/2^(INTT-7))
;	DRED:	data reduction factor - INTEGER
;
; OUTPUT:
;	OUT_SIG: corrected signal array
;	CORR:	 structure (corr.a0, corr.a1) containing the used parameters
;
; KEYWORD:
;	UNCERT:	error on in_sig by input - two elements array of syst.unc. by 
;		output (from a0 and a1)
;
; RESTRICTIONS:
;	(none)
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;        phtintt_corr       
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel  (ESA-SAI)		May 1997
;	V6.3
;	Returning also uncertainties (CG)		September 1997
;	V6.5
;	Allowing for larger data red as in table (CG)	December 1997
;	V6.6
;	UNCERT changed in output (CG)			March 1998
;	Returning dummy corr structure if PHT-S		April 1988
;	V7.1
;	Bug for case of uncert=0 by entry fixed (CG)	May 1998
;	V7.2
;
;-
PRO corr_reset_int, in_sig, det, intt, dred, out_sig, corr, uncert=uncert

COMMON phtintt_corr, phtintt_corr


	out_sig = in_sig
	error = 0	
	corr = {a0:0., a1:0.}

	IF STRMID(det,0,1) EQ 'S' THEN RETURN

	IF N_ELEMENTS(phtintt_corr) EQ 0 THEN phtintt_corr_init, error=error
	IF error THEN BEGIN
	  corr = {a0:0., a1:0.}
	  MESSAGE, /info, 'Calibration file not found, returning'
	  RETURN
	ENDIF
	
	ind = WHERE(STRMID(phtintt_corr.det,0,2)  EQ STRMID(det,0,2)  AND $
		    phtintt_corr.intt EQ intt AND $
		    phtintt_corr.dred EQ dred, nind)
	
	IF nind NE 1 THEN BEGIN
	   ind = WHERE(STRMID(phtintt_corr.det,0,2)  EQ STRMID(det,0,2)  AND $
		    phtintt_corr.intt EQ intt, nind)
	   IF nind EQ 0 THEN BEGIN 
	    corr = {a0:0., a1:0.}
	    MESSAGE,/info, 'No correction possible, combination not found '
	    RETURN
	   ENDIF
	   
	   indind = WHERE(phtintt_corr.dred(ind) EQ $
	                     max([phtintt_corr.dred(ind)]),nindind)
	   IF nindind EQ 1 then ind=ind(indind)
	   	   
	   MESSAGE,/info, $
	      'Warning: combination not found, max data red in table taken'
	   
	ENDIF
	
	ind = ind(0)
	
	a0 = phtintt_corr.a0(ind)
	a1 = phtintt_corr.a1(ind)
	
	out_sig = a0 + a1 * in_sig
	corr = {a0:a0, a1:a1}
	
;	IF KEYWORD_SET(uncert) THEN BEGIN
	  new_unc = FLTARR(2)
	  new_unc(0) = phtintt_corr.a0er(ind) 
	  new_unc(1) = avg([SQRT(phtintt_corr.a1er(ind)^2 * in_sig^2 + $
	  			a1^2*uncert^2)])
          help,new_unc
	  uncert = new_unc
          help,uncert
;	ENDIF
	
	RETURN
	END
