pro createcomposite,ipt1,iptend1,fromorig,toorig,keyporig,vigporig,nplateau,$
t,signal,rms,itrans,mask,keypoint,ech,finep,pident,eytrue,eztrue,fpoint,$
vig,vigc,finepc,pidentc,eytruec,eztruec,fpointc,nplatc,nplateauc,$
signalc,tc,rmsc,itransc,echc,maskc,inrampidentc,keypointc,iptc1,iptc2,$
toc,fromc,vigpc,keypc,nc
;
;save,/xdr,ipt1,iptend1,fromorig,toorig,keyporig,$
;t,signal,rms,itrans,mask,keypoint,ech,file='diag_createcomposite.xdr'
;
; creates composite dataset of one chopper sweep by stacking all data
; within a fine pointing. This routine assumes a sawtooth pattern for the P32
; chopper motion. For fast read modes with data reduction this composite
; sweep is only an approximation, as the number of reads on each sky direction
; are not all the same on different chopper sweeps. The pattern for the first
; complete sweep is taken.
;
; input:  
;         SIGNAL      (array in V/s over all observation) measured detector 
;                      signal at times in T since illumination step at time 
;                      zero (expanded time line)
;         T           (array in s over all observation) time since flux step 
;                      at start of plateau at time 0. T refers to the time at 
;         RMS         (array in V/s over all observation) RMS uncertainty in 
;                      SIGNAL
;         PIDENT     (integer identifier of plateau) (array dimension expanded
;                     timeline)
;         EYTRUE      (1D array in arcsec - dimension full dataset)
;         EZTRUE      (1D array in arcsec - dimension full dataset) 
;         ITRANS     (1D array INTEGER of whole timeline of observation). 
;                     This is set to 1 for data which is
;                     transmitted to the ground and zero for data not
;                     transmitted (in the fast readout modes).  
;         ECH        (1D array INTEGER of whole timeline of observation). 
;                      Chopper plateau identifier (eg for P32
;                      1 to 13 for C100 1 to 7 for C200) for each time sample
;         MASK       (1D array integer of whole timeline of observation)
;                     = 1 for data to be ignored. We have
;                     to import all time sampling points in the time array
;                     T in order to follow the illumination history
;                     so for ease of programming we import all data
;                     too, necessitating a mask.
;         VIG         full array
;-----------------------------------------------------------------------------
;         the following plateau dependent arrays are input as their values
;         as input to DRIFTCORR (ie they do not reflect any changes in the 
;         plateau structure due to the drift trap in DRIFTCORR). This is
;         done to prevent a solution being found on the basis of only NPLATINIT
;         elementary samples of a virtual slew defined in the glitch trap.
;         On input to DRIFTCORR the plateau structure should be the original.
;         TOORIG   - (NPLAT) identifier of first point on a plateau
;         FROMORIG - (NPLAT) identifier of last point on a plateau
;         VIGPORIG - (NPLAT) value of vignetting factor (normally <=1) on a 
;                     plateau
;         KEYPORIG - (NPLAT) value of natural grid identifier on a plateau
;         FINEP    - (NPLAT) 0 or 1 
;-----------------------------------------------------------------------------
;         KEYPOINT   (1D array (INTEGER) of whole timeline of observation).  
;                     For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains the integer key K
;                     to the nearest direction on the P32 natural grid.
;                     K= iy + iz * nygrid,  where iy,iz are the 2D
;                     pointing positions and the grid has dimensions NYGRID,
;                     NZGRID, though this information is not needed, and not
;                     used by this routine.
;         IPT1         First sample of s/c fine pointing being 
;                      processed in calling (DRIFTCORR) routine.
;         IPTEND1      Last sample of s/c fine pointing being 
;                      processed in calling (DRIFTCORR) routine.
;         ARRAY   (common STEERIN) = 'c100' or 'c200'
;         NINRAMP (common STEERIN)
;         DTIME   (common TIMER) ; elementary read interval
;             
; output:
;         arrays over a singlee chopper sweep, starting with 
;         lowest chopper position:
;         SIGNALC      
;         TC
;         RMSC
;         ITRANSC
;         ECHC
;         VIGC
;         MASKC
;         NC   number of points used in average when obtaining composite 
;         KEYPOINTC
;         IPTC1 starting sample of array (set to 0)
;         IPTC2 
;               plateau arrays (dimension of number of chopper positions
;               in a single sweep
;         TOC   
;         FROMC
;         PIDENTC
;         VIGPC
;         KEYPC
;
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_timer
;common timer,time,dtime,complete
;
; find first and last plateaus IRAMP1 IRAMP2 corresponding to IPT1 and IPTEND1
;
findramp,fromorig,toorig,ipt1,jpt1,iramp1
findramp,fromorig,toorig,iptend1,jpt2,iramp2
;help,ipt1,iptend1, jpt1,iramp1,jpt2,iramp2
;
; only consider complete stabilised plateaus
; 
iramp1=iramp1+1
iramp2=iramp2-1
;
; find range of input chopper pointings.
;
kmin=min(keyporig(iramp1:iramp2))
kmax=max(keyporig(iramp1:iramp2))
;
ktot=kmax-kmin+1
nplatinsweep=7
if(array eq 'c100')then nplatinsweep=13
print,'... ipt1,iptend1,iramp1,iramp2,kmin,kmax = ',ipt1,iptend1,iramp1,$
iramp2,kmin,kmax
if(ktot ne nplatinsweep)then begin
if(array eq 'c200')then $
print,'CREATECOMPOSITE: aborting - number of pointings not 7 in C200 sweep'
print,'... ipt1,iptend1,iramp1,iramp2,kmin,kmax = ',ipt1,iptend1,iramp1,$
iramp2,kmin,kmax
if(array eq 'c100')then $
print,'CREATECOMPOSITE: aborting - number of pointings not 13 in C100 sweep'
print,'... ipt1,iptend1,iramp1,iramp2,kmin,kmax = ',ipt1,iptend1,iramp1,$
iramp2,kmin,kmax
goto,c999
endif ; ktot ne nplatinsweep 
;
; set up READIDENT array - a counter starting at 0 at the start of each plateau
;
n=toorig(iramp1)-fromorig(iramp1)+1
readident=lindgen(n)
nramplat=n/ninramp ; number of read cycles on each chopper plateau
incinrampident=lindgen(ninramp)
;
if(nramplat gt 1)then begin
for i=1,nramplat-1 do incinrampident=[incinrampident,lindgen(ninramp)] 
; identifier of position of read in ramp
endif ; nramplat gt 1
;
inrampident=incinrampident
iflag=0
if(keyporig(iramp1) eq kmin and iflag eq 0)then begin
iflag=1
ifirstrampkmin=iramp1
if(ech(fromorig(ifirstrampkmin)) ne 1)then begin
print,'CREATECOMPOSITE: aborting - most negative Y position in sweep'
print,'             ... does not coincide with chopper position 1'
goto,c999 
endif ; ech(ifirstrampkmin) ne 1
endif ; keyporig(iramp1) eq kmin and iflag eq 0
;
for iramp=iramp1+1,iramp2 do begin
;
if(keyporig(iramp) eq kmin and iflag eq 0)then begin
iflag=1
ifirstrampkmin=iramp
;
if(ech(fromorig(ifirstrampkmin)) ne 1)then begin
print,'CREATECOMPOSITE: aborting - most negative Y position in sweep'
print,'             ... does not coincide with chopper position 1'
goto,c999 
endif ; ech(ifirstrampkmin) ne 1
;
endif ; keyporig(iramp) eq kmin and iflag eq 0
;
nprev=n
n=toorig(iramp)-fromorig(iramp)+1
if(n ne nprev)then begin
;print,'CREATECOMPOSITE: warning - successive plateaus have different lengths'
endif ; n ne nprev
;
readident=[readident,lindgen(n)]
;
nramplat=n/ninramp ; number of read cycles on each chopper plateau
incinrampident=lindgen(ninramp)
;
if(nramplat gt 0)then begin
for i=1,nramplat-1 do incinrampident=[incinrampident,lindgen(ninramp)] 
; identifier of position of read in ramp
;
inrampident=[inrampident,incinrampident]
endif ; nramplat gt 1
;
endfor ; iramp=iramp1,iramp2
;
kp1=ifirstrampkmin
kp2=ifirstrampkmin+nplatinsweep-1
k1=fromorig(kp1)
k2=toorig(kp2)
nref=lonarr(nplatinsweep)
for k=kp1,kp2 do nref(k-kp1)=toorig(k)-fromorig(k)+1 
;help,nref
ncomposite=k2-k1+1
;help,kp1,kp2,k1,k2,ncomposite,t,signal,dtime
tc=t(k1:k2)-t(k1)+dtime
itransc=itrans(k1:k2)
echc=ech(k1:k2)
vigc=vig(K1:k2)
keypointc=keypoint(k1:k2)
eytruec=eytrue(k1:k2)
eztruec=eztrue(k1:k2)
pidentc=pident(k1:k2)-pident(k1) ; to get running from 0 to number in ramp
fpointc=fpoint(k1:k2)
inrampidentc=inrampident(0:ncomposite-1) ; identifier of read within read cycle
iptc1=0
iptc2=iptc1+ncomposite-1
signalc=fltarr(ncomposite)
;help,signalc
maskc=lonarr(ncomposite)
rmsc=signalc
nc=maskc
nplatc=kp2-kp1+1
nplateauc=nplateau(kp1:kp2)
toc=toorig(kp1:kp2)-fromorig(kp1)
fromc=fromorig(kp1:kp2)-fromorig(kp1)
keypc=keyporig(kp1:kp2)
vigpc=vigporig(kp1:kp2)
finepc=finep(kp1:kp2)
;
icomposite=-1 ; counter for output composite sweep arrays
;
for k=kmin,kmax do begin
;
ii=iramp1+where(keyporig(iramp1:iramp2) eq k,nii)
if(nii le 0)then begin
print,'CREATECOMPOSITE: aborting - sweep incomplete'
goto, c999
endif ; nii le 0
;
;print,'nii=',nii
for i=0,nii-1 do begin
;
i1=fromorig(ii(i))
i2=toorig(ii(i))
i3=i1-fromorig(iramp1)
i4=i2-fromorig(iramp1)
;
if(i eq 0)then begin
swk=signal(i1:i2)
maskwk=mask(i1:i2)
itranswk=itrans(i1:i2)
identwk=readident(i3:i4)
endif ; i eq 0
;
if(i gt 0)then begin
;
swk=[swk,signal(i1:i2)]
maskwk=[maskwk,mask(i1:i2)]
itranswk=[itranswk,itrans(i1:i2)]
identwk=[identwk,readident(i3:i4)]
;
endif ; i gt 0
;
endfor ;  i=0,nii-1
;help,swk,maskwk, itranswk,identwk
;
;help,nread,k,kmin,ii,nii,toorig,fromorig
;nread=toorig(ii(k-kmin))-fromorig(ii(k-kmin))+1 ; number of samples in 
;                               chopper plateau k in output composite sweep
;nread =toorig(ii(0))-fromorig(ii(0))+1 ; take number in first instance of
; pointing
;
nread=nref(k-kmin)
;
for iread=0,nread-1 do begin
;

ir=where(swk gt 0.0 and identwk eq iread and maskwk eq 0 and itranswk eq 1,nir)

;help,k,icomposite,nir,nc
icomposite=icomposite+1
;if(k eq kmin+5)then 
;help,k,icomposite,nir,nc
nc(icomposite)=nir
;
if(nir gt 0)then begin
signalc(icomposite)=total(swk(ir))/float(nir)
maskc(icomposite)=0
rmsc(icomposite)=sqrt(total(rms(ir)*rms(ir)))/float(nir)
;rmsc(icomposite)=total(rms(ir))/float(nir) ; gives
;test=swk(ir)
;save,swk,ir,nir,icomposite,file='testdiag.xdr'
;stat,test,testmin,testmax,tq2,tq1,tq3,tsigequiv,testav,testsig,0
;rmsc(icomposite)=testsig/sqrt(float(nir))
endif ; nir gt 0
;
if(nir le 0)then begin
signalc(icomposite)=0.0
maskc(icomposite)=1
rmsc(icomposite)=0.0
endif ; 
;
endfor ;  iread=0,nread-1
tstind = where(signalc eq 0.,tstcnt)
;
endfor ; k=kmin,kmax
;
; optionally relativise the ramp (for irel=1)
;
irel=0
;
if(irel eq 1)then begin
;
relfactor=fltarr(ninramp-1) ; destructive read at end of read cycle always bad
relfactor(*)=1.0 ; multiplicative
;relfactor(*)=0.0 ; additive
;
signalcr=fltarr(ninramp-1,nplatinsweep*nramplat)
badcr=lonarr(ninramp-1,nplatinsweep*nramplat)
signalall=fltarr(nplatinsweep*nramplat)
nsignalall=lonarr(nplatinsweep*nramplat)
;
for i=0,ninramp-2 do begin
ii=where(inrampidentc eq i,nii)
signalcr(i,*)=signalc(ii)
badcr(i,*)=maskc(ii)
tbad=badcr(i,*)
sigcr=signalcr(i,*)
igood=where(tbad(*) eq 0 and sigcr(*) gt 0.0,nigood)
if(nigood gt 0)then begin
signalall(igood)=signalall(igood)+signalcr(i,igood)
nsignalall(igood)=nsignalall(igood)+1
endif ; nigood gt 0
endfor ;  i=0,ninramp-2
;
ii=where(nsignalall gt 0,nii)
signalall(ii)=signalall(ii)/float(nsignalall(ii))
;
nd=1 ; dimension of poly_fit (=1 for linear)
funfit=fltarr(ninramp-1,nd+1)
;
for i=0,ninramp-2 do begin
tbad=badcr(i,*)
sigcr=signalcr(i,*)
ii=where(nsignalall gt 0 and tbad(*) eq 0 and sigcr(*) gt 0.0,nii)
relfactor(i)=median(signalcr(i,ii)/signalall(ii)) ; for multiplitive
;relfactor(i)=median(signalcr(i,ii)-signalall(ii)) ; for additive
x=signalall(ii)
y=x
y(*)=signalcr(i,ii)-signalall(ii)
iplot=1
;if(i eq iplot)then window,1
;if(i eq iplot)then plot,x(sort(x)),y(sort(x)),xran=[min([0.,min(x)]),max(x)],$
;yran=[min(y),max(y)],linesty=i+1
;if(i eq 0)then oplot,[0.,max(x)],[0.,max(y)],linesty=0
;if(i ne 0)then oplot,x(sort(x)),y(sort(x)),linestyle=i+1
;result=poly_fit(x,y,nd,yfit)
;if(i eq iplot)then oplot,x,result(0)+result(1)*x
for id=0,nd do funfit(i,id)=result(id)
endfor ;  i=0,ninramp-2
;
relfactor=relfactor*float(ninramp-1)/total(relfactor) ; for multiplitive
;relfactor=relfactor-total(relfactor)/float(ninramp-1) ; additive
;
for i=0,ninramp-2 do begin
ii=where(inrampidentc eq i,nii)
;signalc(ii)=signalc(ii)/relfactor(i) ; multiplicative
;rmsc(ii)=rmsc(ii)/relfactor(i) ; multiplictive
;signalc(ii)=signalc(ii)-relfactor(i) ; additive
iii=where(inrampidentc eq i and maskc eq 0,niii)
fit=funfit(i,0)+funfit(i,1)*signalc(iii)   ; linear polyfit
corr=signalc(iii)-fit ; deviation of signal from the linear fit of unity
;                     gradient through origin which should apply
signalc(iii)=signalc(iii)+corr
endfor ;  i=0,ninramp-2
;
endif ; irel eq 1
;
c999:
;
return
end








