pro createtemplate,imethodin,par1,par2,par3,par4,par5,template
;
; creates template over the P32 natural grid with value of either 1 or -1 
; (according to whether IMETHODIN is positive or negative, respectively) for 
; a "background" direction and value 0 for other directions. A choice of 
; method is available, according to the values of IMETHODIN, PAR1, PAR2, PAR3,
; PAR4 and PAR5:
;
; ABS(IMETHODIN)=1 sets par1 columns at LH side of map (lowest Y) and 
;                  par2 columns at RH side of map (highest Y) as background.
; ABS(IMETHODIN)=2 sets corner regions with side P1 * P2 map pixels in Y * Z
;                  as background.
; ABS(IMETHODIN)=3 sets as background region outside ellipsoid, symmetrical 
;                  about map pixel in IDL notation (P4,P5), of semi-major axis
;                  P1 and semi-minor axis P2 at PA P3 degrees (anticlockwise 
;                  from the upwards direction on the map. For PA 90 the major 
;                  axis is in Y, and pixels more than P1 from the map centre 
;                  along Y through the map centre are deemed to be background.
;                  P4, P5 are real, allowing the mask to be centred at the map
;                  centre even for NYGRID and MZGRID being even (for this case
;                  specify P4=float(nygrid-1)/2.0 P5=float(nzgrid-1)/2.0)
; ABS(IMETHODIN)=4 a statistical background in which the lowest fraction PAR1
;                  are deemed to be the background. This makes use of MAKEMAP,
;                  so it is advisable that a flat field with FLAT_INSCAN,
;                  FLAT_INSCAN_TD, FLAT_CROSSSCAN and FLAT_CROSSSCAN_TD is 
;                  first implemented, to avoid detector pixel bias eg at 
;                  corners of map which are measured only by one detector 
;                  pixel.
;
; If IMETHODIN is positive the background is set to 1 and all other directions
;              are set to zero
; If IMETHODIN is negative the background is set to 0 and all other directions
;              are set to 1.
;
; In this way templates can be defined for both source and background for
; use in aperture photometry, in such a way as to optimise the detection
; statistic witout biasing the photometry. This typically means defining
; a template TEMPLATESOURCE which is the minimum area enclosing a source,
; and a template TEMPLATEBACK which is the maximumarea outside the area
; defined by TEMPLATESOURCE over which the systematic level of the background
; reflects that under the source.  
;
; eg ycen=float(nygrid-1)/2. ; specify aperture centred at map centre
;    zcen=float(nzgrid-1)/2. 
;    createtemplate,3,100.0,100.0,0.0,ycen,zcen,templateback ; create template
;    TEMPLATEBACK of a circular apearture of semiaxis 100 arcsec centred on 
;    the map centre, with value 0 inside the aperture and value 1 outside. 
;    createtemplate,-3,50.0,50.0,0.0,ycen,zcen,templatesource ; create template
;    TEMPLATESOURCE of a circular aperture of semiaxis 50 arcsec centred on 
;    the map centre, with value 1 inside the aperture and value 0 outside.
;    TEMPLATEBACK and TEMPLATESOURCE can then be used as input to MAPBACK
;    to perform photometry.
;
;    To create a background annulus on a larger map around a source at map
;    centre do (for example):
;    createtemplate,-3,200.0,200.0,0.0,ycen,zcen,temp1
;    createtemplate,-3,100.0,100.0,0.0,ycen,zcen,temp2
;    templateback=temp2-temp1
;
@common_steer1
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_mapper
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
;
;
if(n_elements(imethodin) le 0)then begin
print,'createtemplate,imethodin,par1,par2,par3,par4,par5,template'
print,'   set imethodin positive to set background to 1 and source to 0'
print,'   set imethodin negative to set background to 0 and source to 1'
print,'abs(imethodin) = 1 for column eg createtemplate,1,1,1,0,0,0,temp'
print,'abs(imethodin) = 2 for corner eg createtemplate,2,3,1,0,0,0,temp'
print,'abs(imethodin) = 3 for ellipse '
print,' eg createtemplate,-3,200.,200.,45.,float(nygrid-1)/2.,float(nzgrid-1)/2.,temp'
print,'abs(imethodin) = 4 for statistical eg createtemplate,4,0.3,2.,0.,0.,0.,temp'
goto,c999
endif
;
imethod=abs(imethodin)
;
template=lonarr(nygrid,nzgrid)
template(*,*)=0
p1=par1
p2=par2
p3=par3
p4=par4
p5=par5
;
if(imethod eq 1)then begin 
p1=long(p1)
p2=long(p2)
print,'CREATEBACK (imethod=1): '
print,'LH cols, RH cols set to background = ',p1,p2
template(0:p1-1,*)=1
template(nygrid-p2:nygrid-1,*)=1
endif ; imethod eq 1
;
if(imethod eq 2)then begin 
p1=long(p1)
p2=long(p2)
print,'CREATEBACK (imethod=2):' 
print,'Y * Z map pixels set to background at each corner = ',p1,p2
template(0:p1-1,0:p2-1)=1
template(0:p1-1,nzgrid-p2:nzgrid-1)=1
template(nygrid-p1:nygrid-1,0:p2-1)=1
template(nygrid-p1:nygrid-1,nzgrid-p2:nzgrid-1)=1
endif ; imethod eq 2
;
if(imethod eq 3)then begin 
p1=float(p1)
p2=float(p2)
p3=float(p3)
p4=float(p4)
p5=float(p5)
print,'CREATEBACK (imethod=3): 
print,'semi-major, semi-minor (arcsec) =',p1,p2
print,'PA (deg), centre (pixels) =',p3,p4,p5
;zc=float(nzgrid-1)/2.0
;yc=float(nygrid-1)/2.0
yc=p4 ; units of input map pixels
zc=p5
yc=yc*sgridy ; offset in arcsec from bottom LH corner
zc=zc*sgridz
pa=p3
parad=pa*!pi/180.
cospa=cos(parad)
sinpa=sin(parad)
p22=p2*p2
p11=p1*p1
for i=0,nygrid-1 do begin
y=float(i)*sgridy-yc ; Y offset of map pixel from ellipse centre in arcsec
for j=0,nzgrid-1 do begin
z=float(j)*sgridz-zc ; Z offset of map pixel from ellipse centre in arcsec
ztrans=z*cospa-y*sinpa
testz=ztrans*ztrans/p11
ytrans=y*cospa+z*sinpa
testy=ytrans*ytrans/p22
if(testz+testy gt 1.)then template(i,j)=1 
;template(i,j)=testz+testy
endfor ; j
endfor ; i
template=long(template)
endif ; imethod eq 3
;
if(imethod eq 4)then begin 
;
p1=float(p1)
print,'CREATETEMP (imethod=4): background fraction = ',p1
if(p1 ge 1.0 or p1 le 0.0)then begin
print,'CREATETEMP: background fraction out of bounds - aborting'
goto,c999
endif ; p1 ge 1.0 or p1 le 0.0
makemap,npix,lindgen(npix),mapb,mapr,mapd,mapy,mapz,mapn,0,1 ; otf on
ii=where(mapn gt 0,nii)
if(nii le 0)then begin
print,'CREATETEMP: no good points on map - aborting'
goto,c999
endif ; nii le 0
dum=mapb(ii)
dum=dum(sort(dum))
nthresh=round(p1*float(nii))
thresh=dum(nthresh)
iback=where(mapn gt 0 and mapb lt thresh,niback)
print,'CREATETEMPLATE: background threshold = ',thresh
if(niback gt 0)then begin
template(iback)=1
endif ; niback gt 0
;
endif ; imethod eq 4
;
iback=where(template eq 1,niback)
print,'CREATEBACK: number of background directions set = ',niback
if(niback le npix)then begin
print,'CREATETEMPLATE warning - number of background directions less than'
print,'... number of detector pixels'
endif ; niback le npix
if(niback ge nygrid*nzgrid)then begin
print,'CREATETEMPLATE warning - all map directions set as background'
endif ; niback le npix
;
; plot template
;
if(imethodin lt 0)then begin
template=1-template
endif ; invert template
;
nmag=200
nmag=min([nmag/nygrid,nmag/nzgrid])
ny=nmag*nygrid
nz=nmag*nzgrid
;print,'CREATETEMPLATE: nmag,ny,nz = ',nmag,ny,nz
window,0,tit='TEMPLATE',xs=ny,ys=nz
dum=congrid(template,ny,nz)
;help,dum
tvscl,dum
;
c999:
;
return
end



