pro drift32,processfrom,processto,idefcal,iverbose,pixarray,niter,$
icalmethod,tplatback,tplatforward,nlookback,ical1,ical2,isolveselfcal,$
irmscorr,implementdrift,istartnewsol,ndetinit,icompositecorr_in, $
USE_BUFFER=use_buffer,dr_init=dr_init   
;
; Solves for detector responsivity versus time and vignetting for P32 data
; Also implements the corrections to S and RMS common arrays if IMPLEMENT=1
; 
; input:
; from common NAMES:
;    TDTNAM: string denoting name of map
;              would be  PROCESSFROM=[5,4,3,3] PROCESSTO=[5,5,4,3], which
;              gives a symetrical weighting to each pointing while allowing
;              all slews to be processed and all pointings to have an 
;              optimisation of the initial state
;    IVERBOSE = 1 to print numerical results during iterations
;    PIXARRAY = (integer*NPIX) array of pixels to correct for 
;               (useful for diagnostics
;    XIPLOT1,XIPLOT2 = fractional range of elementary non-destructive reads 
;                      to be plotted (unexpanded). No plot is made if 
;                      XIPLOT2 < XIPLOT1
;    NLOOKBACK   number of previous instances of a fine pointing
;                plateau to use in determining illumination history
;                eg put one to use one chopper sweep 2 for 2 chopper sweeps
;                etc           
;    NDETINIT     = number of iterations in DRIFTCORR to find the detector
;                starting state from the redundancy within the first pointing.
;                Often NDETINIT=2 is a good compromise between accuracy 
;                achieved and speed. It also gives a guide to convergence
;                by comparing the results for thw first and second iterations.
;    ICALMETHOD  = 0 to use default model parameters
;                = 1 to find model parameters from fcs exposures
;                = 2 to find model parameters from data of a user-supplied
;                    fine pointing sequence ICAL1 to ICAL2 
;                = 3 combination self cal with FCS optimisation
;                    (not yet tested as slow)
;                = 4 override with detector parameters from ***OVERRIDE arrays,
;                   (overwritten in calling script by user after initialisation
;                     in DETDEFAULTSETUP, which calls INITDETPARS)
;                = 5 override with detector parameters from ***OVERSC arrays,
;                   (overwritten in last call to DETSELFCAL for pixel 
;                    considered, after initialisation in DETDEFAULTSETUP)
;    ISOLVESELFCAL integer array defining combination of detector parameters
;                to ind in the self calibration search:
;              ISOLVESELFCAL(0)=1 search in beta =0 supress beta search
;              ISOLVESELFCAL(1)=1 search in tau =0 supress tau search
;              ISOLVESELFCAL(2)=1 search in gammaexp =0 supress search gammaexp
;    ICAL1,ICAL2 (1D integer arrays dimension number of detector pixels, 
;                 even if  PIXARRAY has lower dimension) 
;                 range of s/c fine pointing to use for selfcalibration
;                 in each pixel (still to be implemented). Note that
;                 the s/c pointings are labeled in terms of a linear array
;                 of dimension NYRASTER*NZRASTER in IFDL notation increasing 
;                 in time order.
;    ICOMPOSITECORR_IN =1 to perform self calibration on composite sweep
;                 (only used when DETSELFCAL called on a single pointing - ie 
;                  ical1(ipix)=ical2(ipix) and icalmethod > 1
;    IRMSCORR     = 0 to leave RMS array untouched
;                 = 1 to multiply RMS array by multiplative responsivity
;                 drift correction (It's not obvious what is right) 
;    IMPLEMENTDRIFT = 1 to implement the drift solution for the whole timeline
;                 for the pixels in PIXARRAY. This can only be done once.
;                 A replacement implementation for any pixel combination can 
;                 only be done by re-running the ERDTOMAP signal conditioning 
;                 routine again prior to re-running DRIFT32 for that pixel 
;                 combination. IMPLEMENTDRIFT allows the user to make various
;                 trials before retaining the best solution.
;    ISTARTNEWSOL =1 to initialise the common block DRIFTRESULT for the pixels
;                 in PIXARRAY, and to wholly initialise the common block
;                 the first time DRIFT32 is called. 
;                 =0 to avoid initialisation so that previous solutions can
;                 be retained. Typically, this is useful if DRIFTCORR has
;                 to be run more than once for a given pixel for different 
;                 sets or ranges of fine pointings, according to the values
;                 of PROCESSFROM and PROCESSTO.
;    NITER        number of iterations in DRIFTCORR
;    BACKPRED    (common DRIFTPAR) predicted background level in MJy/sterad. 
;                Should be found from
;                IRSKY taken around position of observations or, if hook 
;                response is no problem, from previous run of FINDBACK 
;                program). Only used for INEGMETHOD=1
;    ISLEWMETHOD (Integer flag common DRIFTPAR).
;                      Determined method to get the illumination history
;                      during slews. This is particularly problematic when
;                      there is no transmission to ground at intervals during 
;                      a slew. The input parameter is used for iterations
;                      other than the first, for which ISLEWMETHOD = 2
;                      should be used 
;                  = 1 use the nearest P32 natural grid pixel
;                      to define the illumination for all points 
;                      (all combinations of FPOINT ITRANS)
;                  = 2 Use the measured sky points where they exist
;                      (from SKYPASS array)
;                      (ie ITRANS=1) and linearly interpolate in time
;                      where the points don't exist (ITRANS=0). Neighboring
;                      fine pointings not used.
;                  = 3 Perform a spline fit versus position for the points 
;                      taken from SKYPASS array in the slew constraining by ;  
;         TPLATBACK   time in s over which to recalculate illumination
;                     history on discovery of an illegal plateau illumination
;                     (normally due to a glitch on a previous plateau). 
;         TPLATFORWARD time in s over after a glitch over which to ignore
;                      derived illumination when calculating BESTSKY for
;                      the illumination history
;     BACKPRED predicted background level in MJy/sterad. Should be found from
;                IRSKY taken around position of observations or, if hook 
;                response is no problem, from previous run of FINDBACK 
;                program). Only used for INEGMETHOD=1
;                      surrounding fine pointings too from BESTSKY (as
;                      the fine pointings are calculated before the points
;                      on the fly).
;                  = 4 use linear interpolation between nearest P32 natural
;                      grid pixels
;                  = 5 as for 2 but neighboring fine pointings are used  
;    NPIX    (common STEER1) = number of pixels in array
;    NPTIN   (common STEERIN) = total number of points in uncompressed data
; from common SIGNAL:
;    S       (common SIGNAL) (NPIX,NPTIN)
;    RMS     (common SIGNAL) (NPIX,NPTIN)
; from common TIMER
;      TIME    (common SIGNAL) (NPIX,NPTIN)
;      DTIME (non-destructive read interval in s)
; from common STEERIN
;       array = 'c100' or 'c200'
; from common block DRIFTPAR: (input parameter
; from common MOREPAR
;         GAMMAEXP   (common MOREPAR) detector model constant. For GAMMAEXP=1
;                    the Foukes function is exactly reproduced for IMODEL=1
;                    (subroutine FOUKES). For
;                    GAMMAEXP < 1 the detector changes more rapidly at first, 
;                    thus mimicing the observed tendency for short response
;                    timescales immediately after the flux step, during 
;                    (eg) P32 chopper plateaus of ca. 1/2 second, and long 
;                    timescales thereafter; (eg) on FCS plateaus of ca. 32s.
;                    scales thereafter 
;         ZEROBIAS   (common MOREPAR) illumination level about which noise 
;                    fluctuations 
;                    should have a symetrical relaxation times for positive
;                    and negative fluctuations. This is normally the estimated
;                    background level on the map - the minimum real sky
;                    illumination encountered
;       NDRIFTSOLVE > 0 if routine has been previously run, in which case
;                  DRIFT32 returns unaltered values for RESPC and S. 
;
;       set in routine: IFINEMETHOD
; from common block DRIFTPAR: (set in routine)
;       BETA(ipix)  default values on input
;       TAU(ipix)
;       IMODEL     = 1 for Foukes Schubert = 2 for exponential
;       INEGMETHOD = 1 to scale mean level to BACKPRED in V/S in
;                      routine PREPAREDRIFT.
;                      (should be used in underexposed or high dynamic range
;                       observations if hook response makes level negative
;                       or wrong
;                  = 0 to avoid scaling to BACKPRED.
;                  Note that either all pixels are scaled or none. 
;                  Routine should if possible be run with INEGMETHOD=0, 
;                  certainly for the first run.
;       NYGRID,NZGRID (common MAPPER) dimensions of P32 natural grid
; from common block PLOTTER:
;       OTFSTART,OTFSTOP both (ny,nz) where nz and nz are the number of s/c
;                        pointings. Starting and finishing reads where
;                        internally defined OTF is set.
; from common block TESTDRIFTCORR SKYPASS
;
; output:
;       NDRIFTSOLVE (common driftpar; array dimension total number of detector,
;                   initialised in ERDTOMAP. Implemented by +1 for pixels in
;                   PIXARRAY if IMPLEMENTDRIFT=1
;       RESPC(npix,nptin) (common DRIFTRESULT)
;       ILLC(npix,nptin)  (common DRIFTRESULT)
;       MASKD(npix,nptin) (common DRIFTRESULT) = 0 where a good drift solution
;                      was found = 1 otherwise
;       IMPLE(npix,nptin) (common DRIFTRESULT) = 1 where a drift solution
;                      was searched for = 0 otherwise
;       S(npix,nptin), RMS((npix,nptin) (common SIGNAL) 
;                     multiplied by RESPC (can only be done once due to
;                     the NDRIFTSOLVE parameter).
;       BAD(npix,nptin) (common SIGNAL) = BAD on input + MASKD
; to common FILSEQ: ILLUMSKYPREVFILTER (real of dimension number of
;                        FILTERS in AOT) - denotes the signal response
;                        of detector immediately before the raster starts
;                        in chopper rest position. The value for the current
;                        ISEQ is output)
; useful internal arrays
;         FITRESULT    (1D real array) an output parameter normalised to RMS 
;                      indicating how good the fit is for each elementary 
;                      sample points (ie all individual points on plateau 
;                      calculated according to 
;                      the value of the internal parameter ISQUARE.
;                      The lower the better. It can be used to identify 
;                      plateaus not fitted by the data. This enables plateaus 
;                      to be masked out by the calling program if required
;                      by the application eg mapping. Note that for plateaus 
;                      with only 
;                      one data point (eg points taken on the fly) FITRESULT
;                      will tend to zero, rather than RMS, as the number of 
;                      search iterations increases. Because there is no
;                      statistical information on the goodness of fit for
;                      single points, FITRESULT is set to 0.0 for single points
;                      The overall goodness of fit can be calculated by the
;                      calling program as appropriate for the investigation
;                      being conducted (eg one could just use fine pointing
;                      points and take some statistical difference between
;                      the input data and MODELFIT.
;         MODELFIT     (array in V/s): best fit model
;                      for the data. 
;         RESPCORR     (array, dimensioness) illumination/response calculated 
;                      from the vignetting and the model. This is the basic
;                      transient correction result.;
; to common block DRIFTPAR: 
;       BETA(ipix)  for ICALMETHOD=1 or 2 BETA and TAU are overwritten 
;       TAU(ipix)   values obtained from either the FCS or analysis of signal
;                   at s/c pointings ICAL1(npix) ICAL2(npix). This is done
;                   to ensure consistency if beta and tau are use by other 
;                   routines (eg in establishing corrections to fcs 
;                   calibration).
;
;UPDATE to 06/06/00 (CG)
@common_rms_imp
;common rms_imp,bestrms,bestsolrms
@common_testdriftcorr
;common testdriftcorr,skyfound,skyfoundfitres,skyfoundnpt,skypass,nskyfound,$
;ndimsky,skyfoundplat,platskyuse
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;common names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
@common_steer1
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;common driftpar,beta,tau,imodel,ndriftsolve,islewmethod,backpred,inegmethod
@common_driftpar
;common driftresult,respc,illc,imple,maskd
@common_driftresult
;common e32,es,ireduce,eytrue,eztrue,etime,fpoint,echop,estep,ebad,neptin
@common_e32
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_signal
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
@common_mapper
@common_timer
;common timer,time,dtime,complete
; added JUL00 RJT
@common_map3dref
;common map3dref,map3dref,map3drefrms,map3drefd,map3drefy,map3drefz,map3drefn
@common_plotter
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
@common_detcal
;common detcal,betadef,taudef,gammaexpdef,$
;betashootdef,taushootdef,gammaexpshootdef,$
;betacoeffdef,betaexpdef,tauoffsetdef,$
;betacoeffshootdef,betaexpshootdef,tauoffsetshootdef,$
;betaupsc,betadownsc,tauupsc,taudownsc,gammaexpupsc,gammaexpdownsc,$
;betashootupsc,betashootdownsc,taushootupsc,taushootdownsc,$
;gammaexpshootupsc,gammaexpshootdownsc,$
;betacoeffupsc,betacoeffdownsc,betaexpupsc,betaexpdownsc,$
;tauoffsetupsc,tauoffsetdownsc,betacoeffshootupsc,betacoeffshootdownsc,$
;betaexpshootupsc,betaexpshootdownsc,tauoffsetshootupsc,tauoffsetshootdownsc
@common_detdefault
;common detdefault,betadefault,taudefault,gammaexpdefault,$
;betashootdefault,taushootdefault,gammaexpshootdefault,$
;betacoeffdefault,betaexpdefault,tauoffsetdefault,$
;betacoeffshootdefault,betaexpshootdefault,tauoffsetshootdefault,$
;betaup,betadown,tauup,taudown,gammaexpup,gammaexpdown,$
;betashootup,betashootdown,taushootup,taushootdown,$
;gammaexpshootup,gammaexpshootdown,$
;betacoeffup,betacoeffdown,betaexpup,betaexpdown,$
;tauoffsetup,tauoffsetdown,betacoeffshootup,betacoeffshootdown,$
;betaexpshootup,betaexpshootdown,tauoffsetshootup,tauoffsetshootdown,$
;betaprec,tauprec,gammaexpprec,betashootprec,taushootprec,gammaexpshootprec,$
;betacoeffprec,betaexpprec,tauoffsetprec,$
;betacoeffshootprec,betaexpshootprec,tauoffsetshootprec
@common_detpars
;common detpars,betaoverride,tauoverride,gammaexpoverride,$
;betashootoverride,taushootoverride,gammaexpshootoverride,$
;betacoeffoverride,betaexpoverride,tauoffsetoverride,$
;betacoeffshootoverride,betaexpshootoverride,tauoffsetshootoverride,$
;betaoversc,tauoversc,gammaexpoversc,$
;betashootoversc,taushootoversc,gammaexpshootoversc,$
;betacoeffoversc,betaexpoversc,tauoffsetoversc,$
;betacoeffshootoversc,betaexpshootoversc,tauoffsetshootoversc
;
@common_dg2block
;common dg2block,idg2badvalue1,idg2badvalue2,dg2bad
;
COMMON drift_struct, drift_struct
if(n_elements(processfrom) le 0)then begin
print,'drift32,processfrom,processto,idefcal,iverbose,pixarray,niter,$'
print,'icalmethod,tplatback,tplatforward,nlookback,ical1,ical2,isolveselfcal,$'
print,'irmscorr,implementdrift,istartnewsol,ndetinit,icompositecorr_in'
goto,c999
endif
;
IF KEYWORD_SET(use_buffer) THEN use_buffer=1 ELSE use_buffer=0

;print, 'input', processfrom,processto,idefcal,iverbose,pixarray,niter
;print,'calmethod',icalmethod,'params',isolveselfcal
;print,icalmethod,tplatback,tplatforward,nlookback,ical1,ical2,isolveselfcal
;print,irmscorr,implementdrift,istartnewsol,ndetinit,icompositecorr_in
;help,s

npixloop=n_elements(pixarray)
savearray=array ; for TIMELINEFIGURE saves
;
; repository for output diagnostics from fcs fits:
;
fcsbeta0=fltarr(npixloop)
fcstau0=fcsbeta0
fcserr0=fcsbeta0
fcssky0=fcsbeta0
fcsill0=fcsbeta0
fcsbeta1=fcsbeta0
fcstau1=fcsbeta0
fcserr1=fcsbeta0
fcssky1=fcsbeta0
fcsill1=fcsbeta0

;
for pix=0,npixloop-1 do begin
ipix=pixarray(pix)
;if(ndriftsolve(ipix) ne 0 and implementdrift eq 1)then begin
;print,'DRIFT32: attempt to correct already corrected data for pixel ',ipix
;print,'... drift correction not implemented'
;goto, c999
;endif ; ndriftsolve(ipix) ne 0 and implementdrift eq 1
endfor ; pix=0,npixloop-1
;
if(n_elements(processfrom) lt 1)then begin
print,'DRIFT32 - aborting: must specify at least one range of s/c pointings'
goto,c999
endif ; n_elements(processfrom) lt 1
if(n_elements(processfrom) ne n_elements(processto))then begin
print,'DRIFT32 - aborting: must specify same number of starting and ...'
print,'      ... end points for range of s/c pointings'
goto,c999
endif ;  n_elements(processfrom) ne n_elements(processto)
;
nrdc=n_elements(processfrom)
for i=0,nrdc-1 do begin
if(processfrom(i) gt processto(i))then begin
print,'DRIFT32 - aborting: must specify same number of starting and ...'
print,'      ... end points for range of s/c pointings'
goto,c999
endif ; processfrom(i) lt processto(i)
endfor ; i=0,nrdc-1 do begin
;
if(max(processto) ge nyraster*nzraster or min(processfrom) lt 0)then begin
print,'DRIFT32 - aborting: one or more fine pointing ranges ...'
print,'... specified by PROCESSFROM, PROCESSTO is out of range ...'
print,'processfrom,processto = ',processfrom,processto
goto, c999
endif ; max(processto) ge nyraster*nzraster or min(processfrom) lt 0
;
if(istartnewsol eq 1)then begin ; partially or wholly initialise common block 
;                                 DRIFTRESULT
ntest=n_elements(respc)
;
;print,'debug:',ntest,npix,nptin
if(ntest ne npix*nptin)then begin ; first call of all to DRIFT32, so wholly initialise
respc=fltarr(npix,nptin)             
illc=respc
respc(*,*)=1.0
maskd=lonarr(npix,nptin)
imple=maskd
maskd(*,*)=1 ; initialise all points bad
imple(*,*)=0 ; initialise processing tag=0 
endif ; ntest le 0
;
if(ntest eq npix*nptin)then begin 
;     common block already exists so only initialise
;     for the detector pixel in PIXARRAY 
for pix=0,npixloop-1 do begin
ipix=pixarray(pix)
;
respc(ipix,*)=1.0
illc(ipix,*)=0.0
maskd(ipix,*)=1
imple(ipix,*)=0
;
endfor ; pix=0,npixloop-1
;
endif ; ntest gt 0
;
endif ; istartnewsol eq 1
;
if(istartnewsol ne 1)then begin ; no initialisation to allow
; results from a previous run of DRIFTCORR for thee pixels in PIXARRAY to be
; retained, and supleementing by new values (typically for a different
; set of fine pointings of a different fine pointing range).
;
; check arrays exist
;
ntest=n_elements(respc)
;
if(ntest le 0)then begin
print,'DRIFT32 - aborting: ISTARTNEWSOL cannot be non-zero for the first ...'
print,'... call to DRIFT32'
goto, c999
endif ; ntest le 0
;
endif ; istartnewsol ne 1
;
beta=fltarr(npix)
;
; fill common block DETDEFAULT according to value of IMODEL. 
; DETDEFAULT contains detector model parameters, and search ranges and search 
; precision for the parameters in FCS and self cal applications.
;
imodeldum=imodel
detdefaultsetup,0,imodeldum
;
beta=betadefault
tau=taudefault
betashoot=betashootdefault
;
if(iverbose gt 0)then begin
print,'DRIFT32: setting default and search detector parameters for',array
if(imodel lt 3)then $
print,'betaprec,tauprec,gammaexpprec      = ',betaprec,tauprec,gammaexpprec
if(imodel eq 3)then begin
print,'betaprec,betacoeffprec,betaexpprec = ',$
betaprec,betacoeffprec,betaexpprec
print,'tauoffsetprec,tauprec,gammaexpprec = ',$
tauoffsetprec,tauprec,gammaexpprec
print,'betaprec,betacoeffprec,betaexprec(sh)',$
betashootprec,betacoeffshootprec,betaexpshootprec
print,'tauoffsetprec,tauprec,gammaexprec(sh)',$
tauoffsetshootprec,taushootprec,gammaexpshootprec
endif ; imodel eq 3
if(imodel lt 3)then begin 
print,'beta           = ',betadefault
print,'tau            = ',taudefault
print,'gammaexp       = ',gammaexpdefault
print,'betaup         = ',betaup
print,'betadown       = ',betadown
print,'tauup          = ',tauup
print,'taudown        = ',taudown
print,'gammaexpdown   = ',gammaexpdown
print,'gammaexpup     = ',gammaexpup
endif ; imodel lt 3
if(imodel eq 3)then begin
print,'beta           = ',betadefault
print,'betaup         = ',betaup
print,'betadown       = ',betadown
;
print,'betacoeff      = ',betacoeffdefault
print,'betacoeffup    = ',betacoeffup
print,'betacoeffdown  = ',betacoeffdown
;
print,'betaexp        = ',betaexpdefault
print,'betaexpup      = ',betaexpup
print,'betaexpdown    = ',betaexpdown
;
print,'tauoffset      = ',tauoffsetdefault
print,'tauoffsetup    = ',tauoffsetup
print,'tauoffsetdown  = ',tauoffsetdown
;
print,'tau            = ',taudefault
print,'tauup          = ',tauup
print,'taudown        = ',taudown
;
print,'gammaexp       = ',gammaexpdefault
print,'gammaexpup     = ',gammaexpup
print,'gammaexpdown   = ',gammaexpdown
;
print,'betashoot      = ',betashootdefault
print,'betashootup    = ',betashootup
print,'betashootdown  = ',betashootdown
;
print,'betacoeffshoot = ',betacoeffshootdefault
print,'betacoeffshootup ',betacoeffshootup
print,'betacoeffshootdow',betacoeffshootdown
;
print,'betaexpshoot   = ',betaexpshootdefault
print,'betaexpshootup = ',betaexpshootup
print,'betaexpshootdown ',betaexpshootdown
;
print,'tauoffsetshoot = ',tauoffsetshootdefault
print,'tauoffsetshootup ',tauoffsetshootup
print,'tauoffsetshootdow',tauoffsetshootdown
;
print,'taushoot       = ',taushootdefault
print,'taushootup     = ',taushootup
print,'taushootdown   = ',taushootdown
;
print,'gammaexpshoot  = ',gammaexpshootdefault
print,'gammaexpshootdown',gammaexpshootdown
print,'gammaexpshootup =',gammaexpshootup
endif ; imodel eq 3
endif ; iverbose gt 0
;
isquare=1 ; simple rms statistic
;
; NB unlike IFINEMETHOD, ISLEWMETHOD must be set in common prior to call of 
; DRIFT32
;islewmethod=4 ; linear interpolation on the fly (surface, using p3).
;               Use when noise limited
;islewmethod=2 ; linear interpolation fromo elementary points on the fly.
              ; Use in high s/n situatioons when the planar interpolation
              ; between widely spaced fine pointings of islewmethod=4 will
              ; give large interpolation errors compared to linear
              ; interpolation between closely spaced on the fly points
ifinemethod=2 ; use means when recovering illuminatation hisory in FINDILLHIST
;ifinemethod=4 ; use median - NB there is a bug here in FINDILLHIST and for 
              ; some reason this will not work.
;ifinemethod=3 ; weight according to goodness of fit
;
ngrid=nygrid*nzgrid
bestsolrms=fltarr(npix,ngrid) ; initialise
;
if(ngrid le 0)then begin
print,'DRIFT32 - cannot run without grid definition - aborting'
goto,c999
endif ; ngrid le 0
;
if(iverbose gt 0)then $
print,'DRIFT32: islewmethod,backpred,inegmethod = ',$
islewmethod,backpred,inegmethod
;
preparedrift,idriftflag,$
driftincrem,isdriftsave,sdriftsave,idefcal,backminv,backpredv,pixarray
; (overwrite points negative or too faint in S after (for INEGMETHOD=1)
;  adding temporary increments in V/s DRIFTINCREM(ipix). The routine also
;  creates flag array IDRIFTFLAG where 1 is assigned to points with
;  BAD=0 on input but which, even after applying increments are still negative
;  or too faint for the drift correction routine. IDRIFTFLAG is added to BAD).
; The same points
; are set to a low positive value (same for all pixels) which are stored in
; the 1D array SDRIFTSAVE (exactly the right length to accommodate the values)
; and the corresponding indices of S are stored in the 1D long array
; ISDRIFTSAVE. IDRIFTFLAG is not added to BAD, as we wish to use the points
; marked 1 in IDRIFTFLAG in the drift correction.
;
if(n_elements(backminv) lt 1)then begin
print,'DRIFT32: No value for BACKMINV returned by PREPAREDRIFT - aborting'
goto,c999
endif
;
expand32,1 ; for real data
print,'expanded points', neptin
;
; CG
;
; Initialize structure using expanded data
 IF array EQ 'c100' THEN pixels=9 ELSE pixels=4
 IF dr_init THEN BEGIN       ;add at ipac(irk) to allow successive info)
   farr = FLTARR(neptin)
   ffarr = FLTARR(pixels,neptin)
   drift_struct = {name:'', time:farr, fpoint:farr, msk:ffarr, itrans:farr,$
		   modelfit:ffarr, respcorr:ffarr, signal:ffarr, $
		   maskout:ffarr, ech:farr, savearray:array, $
		   skypasssav:ffarr}
 ENDIF
;
icount=lindgen(neptin)
;
;findstartdet,sig0,sill0 ; find sillum0 and simple estimate for slow response
;                         signal0 from uncorrected sky brightness in S 
;                         for all pixels
;
; check sig0 and sill0 greater than the small positive amount BACKMINV 
; for pixels specified in PIXARRAY
sig0=dblarr(npix)
sill0=sig0
sigstart0=sig0
sigshoot0=sig0 ; *** RJT JUL00
;
for pix=0,npixloop-1 do begin
ipix=pixarray(pix)
;
; find estimate for starting state on fine pointing processfrom(0)
;
ipointstart=processfrom(0)
ipointstop=ipointstart
;
; find starting and ending samples (istartnoexpand, istopnoexpand in original
; sampling and istartexpand, istopexpand in expanded sample) 
; for the first and last plateaus with fpoint=1 everywhere in the plateau for 
; the fine pointing sequence starting at s/c fine pointing IPOINTSTART and 
; ending at s/c fine pointing IPOINTSTOP. 
; 
startendpoint,ipointstart,ipointstop,$
irasterstart,irasterstop,istartnoexpand,istopnoexpand,$
irasterstarte,irasterstope,istartexpand,istopexpand 
;
; following changed by RJT JUL00
;
backminvthispix=backminv(ipix)
; findstart is overwrites only the ipix element of sig0,sigshoot0,sill0
findstart,istartnoexpand,backminvthispix,ipix,imodel,icalmethod,$
sig0,sigshoot0,sill0
;
endfor ; pix=0,npixloop-1
;
; *** end changes RJT JUL00
;
t=etime-etime(0)+dtime 
;
; define arrays needed by DRIFTCORR and FINDPLATAU
;
eyt=fltarr(neptin)
ezt=eyt
msk=lonarr(neptin)
signal=eyt
erms=eyt 
erms(*)=0.0
;
iorig=where(ireduce eq 0,niorig) ; list of transmitted points in EXPAND32
if(niorig ne nptin)then begin
print,'DRIFT32 aborting:  number of good points in expanded data not NPTIN'
goto, c999
endif ; niorig ne nptin
;
; itrans has opposite definition from ireduce
itrans=ireduce
itrans(*)=0
itrans(iorig)=1
;
; chopper plateau identifier (1 to 13 for C100, 1 to 7 for C200)
;
ech=estep
;
if(array eq 'c100')then nplatinit=13 ; DRIFTCORR to find detector initial 
;                       from one chopper sweep if NDETINIT is GT 0 
if(array eq 'c200')then nplatinit=7 ; DRIFTCORR to find detector initial 
;                       from one chopper sweep if NDETINIT is GT 0 
print,'DRIFT32: processfrom = ',processfrom
print,'         processto   = ',processto
print,'         pixarray    = ',pixarray
print,'         implementdrift,istartnewsol = ',byte(implementdrift),$
                                                byte(istartnewsol)
print,'         ndetinit,nplatinit = ',byte(ndetinit),byte(nplatinit)
;
for pix=0,npixloop-1 do begin
ipix=pixarray(pix)
;
print,'DRIFT32: processing pixel ',ipix
;
; the following parameters are fixed, not changable via FCS cal or self cal:
;
;tauoffset=tauoffsetdefault(ipix) ; set independently of imodel, to initialise,
;betaexp=betaexpdefault(ipix)     ; even though only used for imodel=3
;betacoeff=betacoeffdefault(ipix)
;
; output file name for storing data for TIMELINEFIGURE after each run of
; DRIFTCORR:
if(ipix eq 0)then tlfnam='timelinefigure0.xdr'
if(ipix eq 1)then tlfnam='timelinefigure1.xdr'
if(ipix eq 2)then tlfnam='timelinefigure2.xdr'
if(ipix eq 3)then tlfnam='timelinefigure3.xdr'
if(ipix eq 4)then tlfnam='timelinefigure4.xdr'
if(ipix eq 5)then tlfnam='timelinefigure5.xdr'
if(ipix eq 6)then tlfnam='timelinefigure6.xdr'
if(ipix eq 7)then tlfnam='timelinefigure7.xdr'
if(ipix eq 8)then tlfnam='timelinefigure8.xdr'
;
findkeypoint,ipix,eytrue,eztrue,keypoint ; find KEYPOINT array for pixel ipix
;
eyt(*)=eytrue(ipix,*)
ezt(*)=eztrue(ipix,*)
msk(*)=ebad(ipix,*)
ii=where(msk ne 0,nii)
if(nii gt 0)then msk(ii)=1
;
if(array eq 'c100')then vignecorr32,1,ipix,echop,vig
if(array eq 'c200')then vignecorr32,2,ipix,echop,vig
;
signal(*)=es(ipix,*)
rmsmed=median(rms(ipix,*))
erms(iorig)=rms(ipix,iorig) ; real rms
iitest=where(erms(*) le 0.0,niitest)
if(niitest gt 0)then erms(iitest)=rmsmed
;erms(*)=rmsmed ; set to single value medium. Really a short cut but
; should not matter as rms only used to find search space and to normalise
; the result. We keep it vectorised to allow a more sophisticated future
; treatment. We comment this out since rms can show strong variations
; and SOLVEILLUM1 can go astray if the input rms is much larger than the true 
; rms.
;
backminvthispix=backminv(ipix) ; lower limit of search range in DRIFTCORR
backpredvthispix=backpredv(ipix) ; best estimate for background
;                                  same value independent of raster point
;
; find pointing plateaus from FPOINT and KEYPOINT arrays for this pixel
; (this is pixel dependent due to the vigp keyp pident arrays)
;
;
findplateau,fpoint,keypoint,vig,$
nplat,nplateau,from,to,vigp,$
keyp,finep,pident,0 ; notverbose
;
;
; *** JUL00 RJT insertion
;
; set up MOREPAR common block according to values of ipix and icalmethod:
setparcal,icalmethod,imodel,ipix,betawk,tauwk
;
betacoeffwk=betacoeff
betaexpwk=betaexp
tauoffsetwk=tauoffset
gammaexpwk=gammaexp 
betashootwk=betashoot
betacoeffshootwk=betacoeffshoot
betaexpshootwk=betaexpshoot
tauoffsetshootwk=tauoffsetshoot
taushootwk=taushoot 
gammaexpshootwk=gammaexpshoot
;
; *** end JUL00 RJT insertion
;
if(icalmethod eq 0)then begin ; take default beta tau
; 
; following commented JUL00 RJT
;betawk=betadefault(ipix) ; prevents overwriting of defaults
;betacoeffwk=betacoeffdefault(ipix)
;betaexpwk=betaexpdefault(ipix)
;tauoffsetwk=tauoffsetdefault(ipix)
;tauwk=taudefault(ipix)   ; with detector parameters found from dataset
;gammaexpwk=gammaexpdefault(ipix) 
;betashootwk=betashootdefault(ipix)
;betacoeffshootwk=betacoeffshootdefault(ipix)
;betaexpshootwk=betaexpshootdefault(ipix)
;tauoffsetshootwk=tauoffsetshootdefault(ipix)
;taushootwk=taushootdefault(ipix) 
;gammaexpshootwk=gammaexpshootdefault(ipix)
;
; initialise MOREPAR common block
;betacoeff=betacoeffdefault(ipix)
;betaexp=betaexpdefault(ipix)
;tauoffset=tauoffsetdefault(ipix)
;gammaexp=gammaexpdefault(ipix) 
;betashoot=betashootdefault(ipix) 
;betacoeffshoot=betacoeffshootdefault(ipix) 
;betaexpshoot=betaexpshootdefault(ipix)
;tauoffsetshoot=tauoffsetshootdefault(ipix) 
;taushoot=taushootdefault(ipix)
;gammaexpshoot=gammaexpshootdefault(ipix)  
;
;end comment JUL00 RJT
;
print,'DRIFT32 for pixel',byte(ipix),' using default parameters:'
print,'beta betacoeff betaexp tauoffset tau gammaexp (slow): '
print,float(betawk),float(betacoeffwk),float(betaexpwk),$
float(tauoffsetwk),float(tauwk),float(gammaexpwk)
print,'beta betacoeff betaexp tauoffset tau gammaexp (shoot): '
print,float(betashootwk),float(betacoeffshootwk),float(betaexpshootwk),$
float(tauoffsetshootwk),float(taushootwk),float(gammaexpshootwk)

if(iverbose gt -1)then begin
if(imodel eq 1 or imodel eq 2)then begin
print,'DRIFT32 using default values for ipix,beta,tau,gammaexp of: '
print,'... ',byte(ipix),float(betawk),float(tauwk),float(gammaexpwk)
endif ; imodel eq 1 or imodel eq 2
if(imodel eq 3)then begin
print,'DRIFT32 for pixel',byte(ipix),' using default parameters:'
print,'beta betacoeff betaexp tauoffset tau gammaexp (slow): '
print,float(betawk),float(betacoeffwk),float(betaexpwk),$
float(tauoffsetwk),float(tauwk),float(gammaexpwk)
print,'beta betacoeff betaexp tauoffset tau gammaexp (shoot): '
print,float(betashootwk),float(betacoeffshootwk),float(betaexpshootwk),$
float(tauoffsetshootwk),float(taushootwk),float(gammaexpshootwk)
endif ; imodel eq 3
endif ; iverbose gt -1
;
endif ; icalmethod eq 0
;
if(icalmethod eq 4 or icalmethod eq 5)then begin 
; take detector parameters from OVERRIDE or OVERSC arrays in common DETPARS
; 
; following cmmented JUL00 RJT
;if(icalmethod eq 4)then begin ; override from OVERRIDE arrays (user steering)
;betawk=betaoverride(ipix) ; prevents overwriting of overrides
;betacoeffwk=betacoeffoverride(ipix)
;betaexpwk=betaexpoverride(ipix)
;tauoffsetwk=tauoffsetoverride(ipix)
;tauwk=tauoverride(ipix)   ; with detector parameters found from dataset
;gammaexpwk=gammaexpoverride(ipix) 
;betashootwk=betashootoverride(ipix)
;betacoeffshootwk=betacoeffshootoverride(ipix)
;betaexpshootwk=betaexpshootoverride(ipix)
;tauoffsetshootwk=tauoffsetshootoverride(ipix)
;taushootwk=taushootoverride(ipix) 
;gammaexpshootwk=gammaexpshootoverride(ipix)
;endif ; icalmethod eq 4
;
;if(icalmethod eq 5)then begin ; overwrite from OVERSC arrays (deposited from
;                               selfcals)
;betawk=betaoversc(ipix) ; prevents overwriting of overrides
;betacoeffwk=betacoeffoversc(ipix)
;betaexpwk=betaexpoversc(ipix)
;tauoffsetwk=tauoffsetoversc(ipix)
;tauwk=tauoversc(ipix)   ; with detector parameters found from dataset
;gammaexpwk=gammaexpoversc(ipix) 
;betashootwk=betashootoversc(ipix)
;betacoeffshootwk=betacoeffshootoversc(ipix)
;betaexpshootwk=betaexpshootoversc(ipix)
;tauoffsetshootwk=tauoffsetshootoversc(ipix)
;taushootwk=taushootoversc(ipix) 
;gammaexpshootwk=gammaexpshootoversc(ipix)
;endif ; icalmethod eq 5
;
; initialise MOREPAR common block
;betacoeff=betacoeffwk
;betaexp=betaexpwk
;tauoffset=tauoffsetwk
;gammaexp=gammaexpwk 
;betashoot=betashootwk 
;betacoeffshoot=betacoeffshootwk 
;betaexpshoot=betaexpshootwk
;tauoffsetshoot=tauoffsetshootwk 
;taushoot=taushootwk
;gammaexpshoot=gammaexpshootwk  
;
; end comment JUL00 RJT
;
if(iverbose gt -1)then begin
if(imodel eq 1 or imodel eq 2)then begin
if(icalmethod eq 4)then print,$
'DRIFT32 using override (user) values for ipix,beta,tau,gammaexp of: '
if(icalmethod eq 5)then print,$
'DRIFT32 using override (self cal) values for ipix,beta,tau,gammaexp of: '
print,'... ',byte(ipix),float(betawk),float(tauwk),float(gammaexpwk)
endif ; imodel eq 1 or imodel eq 2
if(imodel eq 3)then begin
if(icalmethod eq 4)then print,$
'DRIFT32 for pixel',byte(ipix),' using override (user) parameters:'
if(icalmethod eq 5)then print,$
'DRIFT32 for pixel',byte(ipix),' using override (self cal) parameters:'
print,'beta betacoeff betaexp tauoffset tau gammaexp (slow): '
print,float(betawk),float(betacoeffwk),float(betaexpwk),$
float(tauoffsetwk),float(tauwk),float(gammaexpwk)
print,'beta betacoeff betaexp tauoffset tau gammaexp (shoot): '
print,float(betashootwk),float(betacoeffshootwk),float(betaexpshootwk),$
float(tauoffsetshootwk),float(taushootwk),float(gammaexpshootwk)
endif ; imodel eq 3
endif ; iverbose gt -1
;
endif ; icalmethod eq 4 or icalmethod eq 5
;
if(icalmethod eq 1)then begin ; find slow beta tau parameters from FCS
;
fcsfraction=1.0 ; for ICALMETHOD=1 (fcs calibration) use only first fraction 
;                 FCSFRACTION of an fcs exposure to find beta tau 
imedfilter=1
iplotverbose=1
taudownfcs=taudown(ipix)
tauupfcs=tauup(ipix)
betaupfcs=betaup(ipix)
betadownfcs=betadown(ipix)
betacoeff=betacoeffdefault(ipix)
betaexp=betaexpdefault(ipix)
tauoffset=tauoffsetdefault(ipix)
gammaexp=gammaexpdefault(ipix) ; these are set before the findillumbeforefcs
betashoot=betashootdefault(ipix)  ;   since they are not recalculated by 
betacoeffshoot=betacoeffshootdefault(ipix) ; findillumbeforefcs
betaexpshoot=betaexpshootdefault(ipix)
tauoffsetshoot=tauoffsetshootdefault(ipix) 
taushoot=taushootdefault(ipix)
gammaexpshoot=gammaexpshootdefault(ipix)  
;                                
; find illumsky from central position of chopper prior to raster
ibefore=1
findillumbeforefcs,ipix,ibefore,bestsky,nbestsky,illumsky,ifound,0,1 ; verbose
illumskyprevfilter=illumsky ; approximation (only affects 2nd or subsequent
;                             filter and only for ibefore=1 
betabefore=[betadefault(ipix)]
taubefore=[taudefault(ipix)]
;
illumskybefore=illumsky
findbetataufcs,[ipix],imodel,isquare,ibefore,illumsky,illumskyprevfilter,$
betadownfcs,betaupfcs,taudownfcs,tauupfcs,betabefore,taubefore,$
sillumoutbefore,sillumbeforeraster,sillumshootbeforeraster,fcsfraction,$
imedfilter,fitresultoutbefore,iplotverbose, use_buffer=use_buffer
;
betafter=[betadefault(ipix)]
tauafter=[taudefault(ipix)]
ibefore=0
findillumbeforefcs,ipix,ibefore,bestsky,nbestsky,illumsky,ifound,0,1 ; verbose
illumskyprevfilter=illumsky ; approximation (only affects 2nd or subsequent
;                             filter and only for ibefore=1 
illumskyafter=illumsky
findbetataufcs,[ipix],imodel,isquare,ibefore,illumsky,illumskyprevfilter,$
betadownfcs,betaupfcs,taudownfcs,tauupfcs,betafter,tauafter,$
sillumoutafter,sillumbeforeraster,sillumshootbeforeraster,fcsfraction,$
imedfilter,fitresultoutafter,iplotverbose, use_buffer=use_buffer
;
betawk=(betabefore+betafter)/2.0
tauwk=(taubefore+tauafter)/2.0
betawk=betawk(0)
tauwk=tauwk(0)
sillumout=(sillumoutbefore+sillumoutafter)/2.0
;
; add results for to repository for printing later
;
; ibefore=0
fcsbeta0(pix)=betafter
fcstau0(pix)=tauafter
fcserr0(pix)=fitresultoutafter
fcssky0(pix)=illumskyafter
fcsill0(pix)=sillumoutafter
;
fcsbeta1(pix)=betabefore
fcstau1(pix)=taubefore
fcserr1(pix)=fitresultoutbefore
fcssky1(pix)=illumskybefore
fcsill1(pix)=sillumoutbefore
;
betacoeffwk=betacoeff
betaexpwk=betaexp
tauoffsetwk=tauoffset
gammaexpwk=gammaexp
betashootwk=betashoot ; here the shooting constants are just for the print out
betacoeffshootwk=betacoeffshoot
betaexpshootwk=betaexpshoot
tauoffsetshootwk=tauoffsetshoot
taushootwk=taushoot
gammaexpshootwk=gammaexpshoot
;
if(iverbose gt -1)then begin
;
if(imodel eq 1 or imodel eq 2)then begin
print,'DRIFT32 using fcscal values for ipix,beta,tau,gammaexp of: '
print,'... ',byte(ipix),float(betawk),float(tauwk),float(gammaexpwk)
endif ; imodel eq 1 or imodel eq 2
if(imodel eq 3)then begin
print,'DRIFT32 for pixel',byte(ipix),' using FCS calibration:'
print,'beta betacoeff betaexp tauoffset tau gammaexp (slow): '
print,float(betawk),float(betacoeffwk),float(betaexpwk),$
float(tauoffsetwk),float(tauwk),float(gammaexpwk)
print,'beta betacoeff betaexp tauoffset tau gammaexp (shoot): '
print,float(betashootwk),float(betacoeffshootwk),float(betaexpshootwk),$
float(tauoffsetshootwk),float(taushootwk),float(gammaexpshootwk)
endif ; imodel eq 3
;
endif ; iverbose gt -1
;
endif ; icalmethod eq 1
;
if(icalmethod eq 2 or icalmethod eq 3)then begin 
; selfcalibration on source between pointings ICAL1 ICAL2. Also constrained
; by FCS measurements for icalmethod=3
;
icalpt1=ical1(ipix)
icalpt2=ical2(ipix)
;
icompositecorr=icompositecorr_in
if(icalpt1 ne icalpt2 and icompositecorr eq 1)then begin
print,'DRIFT32: *** warning - cannot perform self calibration on'
print,'         ... composite sweep when more than one pointing specified ... '
print,'         ... setting icalpt2 to icalpt1 ***'
;goto,c999 
icalpt2=icalpt1
endif ; icalpt1 ne icalpt2
;
; find estimate for starting state on fine pointing processfrom(0)
;
ipointstart=icalpt1
ipointstop=ipointstart
;
; find starting and ending samples (istartnoexpand, istopnoexpand in original
; sampling and istartexpand, istopexpand in expanded sample) 
; for the first and last plateaus with fpoint=1 everywhere in the plateau for 
; the fine pointing sequence starting at s/c fine pointing IPOINTSTART and 
; ending at s/c fine pointing IPOINTSTOP. 
; 
startendpoint,ipointstart,ipointstop,$
irasterstart,irasterstop,istartnoexpand,istopnoexpand,$
irasterstarte,irasterstope,istartexpand,istopexpand 
;
;
; *** RJT JUL00
;
; findstart is overwrites only the ipix element of sig0,sigshoot0,sill0
findstart,istartnoexpand,backminvthispix,ipix,imodel,icalmethod,$
sig0,sigshoot0,sill0
;
;findstartdetnew,istartnoexpand,ipix,siginit,sillinit,sigstart
;sig0(ipix)=siginit
;sill0(ipix)=sillinit
;
; *** end RJT JUL00
;
betaupwk=betaup(ipix)
betadownwk=betadown(ipix)
betacoeffupwk=betacoeffup(ipix)
betacoeffdownwk=betacoeffdown(ipix)
betaexpupwk=betaexpup(ipix)
betaexpdownwk=betaexpdown(ipix)
tauoffsetupwk=tauoffsetup(ipix)
tauoffsetdownwk=tauoffsetdown(ipix)
tauupwk=tauup(ipix)
taudownwk=taudown(ipix)
gammaexpupwk=gammaexpup(ipix)
gammaexpdownwk=gammaexpdown(ipix)
betashootupwk=betashootup(ipix)
betashootdownwk=betashootdown(ipix)
betacoeffshootupwk=betacoeffshootup(ipix)
betacoeffshootdownwk=betacoeffshootdown(ipix)
betaexpshootupwk=betaexpshootup(ipix)
betaexpshootdownwk=betaexpshootdown(ipix)
tauoffsetshootupwk=tauoffsetshootup(ipix)
tauoffsetshootdownwk=tauoffsetshootdown(ipix)
taushootupwk=taushootup(ipix)
taushootdownwk=taushootdown(ipix)
gammaexpshootupwk=gammaexpshootup(ipix)
gammaexpshootdownwk=gammaexpshootdown(ipix)
;
;tauwk=taudefault(ipix)    ; set to default values to cover case they are not
;betawk=betadefault(ipix)  ; contained in ISOLVESELFCAL
;gammaexp=gammaexpdefault(ipix)
;gammaexpwk=gammaexpdefault(ipix)
;taushootwk=taushootdefault(ipix)
;betashootwk=betashootdefault(ipix)
;gammaexpshoot=gammaexpshootdefault(ipix)
;gammaexpshootwk=gammaexpshootdefault(ipix)
;
;
; JUN00 RJT changed the following from ***default assignments to
;                                      ***oversc assignments
;
; JUL00 RJT commented the following
;betawk=betaoversc(ipix) ; set to oversc values to cover case they are not
;betacoeffwk=betacoeffoversc(ipix) ; contained in ISOLVESELFCAL
;betaexpwk=betaexpoversc(ipix)
;tauoffsetwk=tauoffsetoversc(ipix)
;tauwk=tauoversc(ipix)   ; with detector parameters found from dataset
;gammaexpwk=gammaexpoversc(ipix) 
;betashootwk=betashootoversc(ipix)
;betacoeffshootwk=betacoeffshootoversc(ipix)
;betaexpshootwk=betaexpshootoversc(ipix)
;tauoffsetshootwk=tauoffsetshootoversc(ipix)
;taushootwk=taushootoversc(ipix) 
;gammaexpshootwk=gammaexpshootoversc(ipix)
;
; in followin ***oversc changed to MOREPAR values JUL00
betadef=betawk          ; fill common DETCAL (needed to prevent
taudef=tauwk            ; compilation error due to too many passed 
gammaexpdef=gammaexp  ; parameters in DETSELFCAL from MOREPAR array
tauoffsetdef=tauoffset 
betashootdef=betashoot
taushootdef=taushoot
gammaexpshootdef=gammaexpshoot
betacoeffdef=betacoeff
betaexpdef=betaexp
betacoeffshootdef=betacoeffshoot
betaexpshootdef=betaexpshoot
tauoffsetshootdef=tauoffsetshoot
; end substitution JUL00 from oversc to morepar parameters
;
betaupsc=betaupwk
betadownsc=betadownwk
tauupsc=tauupwk
taudownsc=taudownwk
gammaexpupsc=gammaexpupwk
gammaexpdownsc=gammaexpdownwk
betashootupsc=betashootupwk
betashootdownsc=betashootdownwk
taushootupsc=taushootupwk
taushootdownsc=taushootdownwk
gammaexpshootupsc=gammaexpshootupwk
gammaexpshootdownsc=gammaexpshootdownwk
betacoeffupsc=betacoeffupwk
betacoeffdownsc=betacoeffdownwk
betaexpupsc=betaexpupwk
betaexpdownsc=betaexpdownwk
tauoffsetupsc=tauoffsetupwk
tauoffsetdownsc=tauoffsetdownwk
betacoeffshootupsc=betacoeffshootupwk
betacoeffshootdownsc=betacoeffshootdownwk
betaexpshootupsc=betaexpshootupwk
betaexpshootdownsc=betaexpshootdownwk
tauoffsetshootupsc=tauoffsetshootupwk
tauoffsetshootdownsc=tauoffsetshootdownwk
;
; folowing commented JUL00 RJT
; initialise MOREPAR common block
;betacoeff=betacoeffoversc(ipix)
;betaexp=betaexpoversc(ipix)
;tauoffset=tauoffsetoversc(ipix)
;gammaexp=gammaexpoversc(ipix) 
;betashoot=betashootoversc(ipix) 
;betacoeffshoot=betacoeffshootoversc(ipix) 
;betaexpshoot=betaexpshootoversc(ipix)
;tauoffsetshoot=tauoffsetshootoversc(ipix) 
;taushoot=taushootoversc(ipix)
;gammaexpshoot=gammaexpshootoversc(ipix)  
;
;
; end of RJT JUN00 change from ***default to ***oversc
;
backpredf=backpred
;
detselfcal,isolveselfcal,icalmethod,idefcal,icalpt1,icalpt2,ipix,chimin,$
bestsky,nbestsky,$
betawk,tauwk,gammaexpwk,betashootwk,taushootwk,gammaexpshootwk,betacoeffwk,$
betaexpwk,tauoffsetwk,betacoeffshootwk,betaexpshootwk,tauoffsetshootwk,$
sig0(ipix),sigshoot0(ipix),sill0(ipix),ngrid,$
from,to,nplat,nplateau,vigp,keyp,finep,pident,$
keypoint,fpoint,itrans,ech,$
eyt,ezt,msk,vig,t,signal,erms,imodel,isquare,islewmethod,ifinemethod,$
fitresult,modelfit,respcorr,maskout,niter,$
ndetinit,nplatinit,nlookback,backminvthispix,backpredf,backpredvthispix,$
icompositecorr
;
gammaexp=gammaexpwk ; must set these since no equivalence with common
gammaexpshoot=gammaexpshootwk
betashoot=betashootwk
taushoot=taushootwk
betacoeff=betacoeffwk
betaexp=betaexpwk
tauoffset=tauoffsetwk
betacoeffshoot=betacoeffshootwk
betaexpshoot=betaexpshootwk
tauoffsetshoot=tauoffsetshootwk
;
if(iverbose gt -1)then begin
;
if(imodel eq 1 or imodel eq 2)then begin
print,'DRIFT32 using fcscal values for ipix,beta,tau,gammaexp of: '
print,'... ',byte(ipix),float(betawk),float(tauwk),float(gammaexpwk)
endif ; imodel eq 1 or imodel eq 2
if(imodel eq 3)then begin
print,'DRIFT32 for pixel',byte(ipix),' using self calibration:'
print,'beta betacoeff betaexp tauoffset tau gammaexp (slow): '
print,float(betawk),float(betacoeffwk),float(betaexpwk),$
float(tauoffsetwk),float(tauwk),float(gammaexpwk)
print,'beta betacoeff betaexp tauoffset tau gammaexp (shoot): '
print,float(betashootwk),float(betacoeffshootwk),float(betaexpshootwk),$
float(tauoffsetshootwk),float(taushootwk),float(gammaexpshootwk)
endif ; imodel eq 3
;
endif ; iverbose gt -1
;
endif ; icalmethod eq 2 or icalmethod eq 3
;

for irdc=0,nrdc-1 do begin
;
ipointstart=processfrom(irdc)
ipointstop=processto(irdc)
;
;print,'************************************'
;print,'Processfrom = ',processfrom
;print,'Processto   = ',processto
;print,'ipointstart,ipointstop = ',ipointstart,ipointstop
;print,'************************************'
;
; find starting and ending samples (istartnoexpand, istopnoexpand in original
; sampling and istartexpand, istopexpand in expanded sample) 
; for the first and last plateaus with fpoint=1 everywhere in the plateau for 
; the fine pointing sequence starting at s/c fine pointing IPOINTSTART and 
; ending at s/c fine pointing IPOINTSTOP. 
; 
startendpoint,ipointstart,ipointstop,$
irasterstart,irasterstop,istartnoexpand,istopnoexpand,$
irasterstarte,irasterstope,istartexpand,istopexpand 
;
istopexpandfirst=irasterstope(ipointstart) ; last expanded sample of 
;        first s/c pointing, needed for determination of detector initial state
;        by DRIFTCORR with option NDETINIT GT 0
;
; find initial estimate for starting state on first pointing of 
; current fine pointing range
;
;
; *** RJT JUL00
;
; findstart is overwrites only the ipix element of sig0,sigshoot0,sill0
findstart,istartnoexpand,backminvthispix,ipix,imodel,icalmethod,$
sig0,sigshoot0,sill0
;
;findstartdetnew,istartnoexpand,ipix,siginit,sillinit,sigstart
;sig0(ipix)=siginit
;sill0(ipix)=sillinit
;if(imodel eq 3)then begin
;sigshoot0(ipix)=0.0
;if(sigstart ne 0.0)then sigshoot0(ipix)=sigstart-sillinit
;endif ; imodel eq 3
;
; *** end RJT JUL00
;
findramp,from,to,istartexpand,itest,iplat1
if(itest ne istartexpand)then begin
print,'DRIFT32: warning - fine pointing does not begin at start of plateau...'
print,'... istartexpand,itest,iplat1 = ',istartexpand,itest,iplat1
endif ; itest ne istartexpand
;
if(istopexpand lt neptin-1)then begin ; it is not the last point
findramp,from,to,istopexpand+1,itest,iplattest2
iplat2=iplattest2-1
if(itest ne istopexpand+1)then begin
print,'DRIFT32: warning - fine pointing does not end at end of plateau...'
print,'... istopexpand,itest,iplat2 = ',istopexpand,itest,iplat2
endif ; itest ne istopexpand+1
endif ; istopexpand lt neptin-1
if(istopexpand eq neptin-1)then begin ; it is the last point
findramp,from,to,istopexpand,itest,iplat2
endif ; istopexpand eq neptin-1
;
if(istopexpandfirst lt neptin-1)then begin ; it is not the last point
findramp,from,to,istopexpandfirst+1,itest,iplattest2
iiplat2=iplattest2-1
if(itest ne istopexpandfirst+1)then begin
print,'DRIFT32: warning - fine pointing does not end at end of plateau...'
print,'... istopexpandfirst,itest,iiplat2 = ',istopexpandfirst,itest,iiplat2
endif ; itest ne istopexpandfirst+1
endif ; istopexpandfirst lt neptin-1
if(istopexpandfirst eq neptin-1)then begin ; it is the last point
findramp,from,to,istopexpandfirst,itest,iiplat2
endif ; istopexpandfirst eq neptin-1
;
if(iverbose gt 0)then begin
print,'DRIFT32: ipix,ipointstart,ipointstop = ',ipix,ipointstart,ipointstop
print,'... iplat1,iplat2,istartexpand,istopexpandfirst,istopexpand ='
print,iplat1,iplat2,istartexpand,istopexpandfirst,istopexpand
endif ; iverbose gt 0
;
gammaexp=gammaexpwk ; must set these since no equivalence with common
gammaexpshoot=gammaexpshootwk
betashoot=betashootwk
taushoot=taushootwk
betacoeff=betacoeffwk
betaexp=betaexpwk
tauoffset=tauoffsetwk
betacoeffshoot=betacoeffshootwk
betaexpshoot=betaexpshootwk
tauoffsetshoot=tauoffsetshootwk
;
driftcorr,betawk,tauwk,sig0(ipix),sigshoot0(ipix),sill0(ipix),ngrid,$
from,to,nplat,nplateau,vigp,keyp,finep,pident,$
keypoint,fpoint,itrans,ech,$
eyt,ezt,msk,vig,t,signal,erms,imodel,isquare,islewmethod,ifinemethod,$
fitresult,modelfit,modelfitshoot,modelfitslow,respcorr,maskout,niter,$
istartexpand,istopexpand,istopexpandfirst,iplat1,iplat2,ndetinit,nplatinit,$
ipix,tplatback,tplatforward,nlookback,backminvthispix,backpredvthispix,ierr,$
iverbose,chi
;
isolgrid=where(bestrms gt 0.0, nisolgrid)
if(nisolgrid gt 0)then $
bestsolrms(ipix,isolgrid)=bestrms(isolgrid) ; update rms per elementary sample
;                               on each P32 natural grid point for which
;                               statistics couldd be found as this may be 
;                               needed by DRIFTRMS, below. Note that for
;                               sequential mode like DRIFTCORR repeated
;                               for individual pointings BESTRMS will overwrite
;                               overwritten in DROUINIT called by DRIFTCORR,
;                               so IRMSCORR=2 or =3 is not recommended for
;                               such sequential processing.
;
ii=where(itrans eq 1 and icount ge istartexpand and icount le istopexpand,nii)
imple(ipix,istartnoexpand:istopnoexpand)=1 ; update list of samples to be 
;                                            corrected at original sampling
respc(ipix,istartnoexpand:istopnoexpand)=respcorr(ii) ; compression from e32 
;                                                       length
maskd(ipix,istartnoexpand:istopnoexpand)=maskout(ii) ; unmask points with 
;                                                     reasonable drift solution
illc(ipix,istartnoexpand:istopnoexpand)=respcorr(ii)*modelfit(ii) ;
;                                           illumination history
; Save data for TIMELINEFIGURE:
; supress diagnostic output in the IDL session from SAVE and RESTORE?
;
skypasssav=skypass
IF iverbose GT 0 THEN BEGIN
  help,time,fpoint,msk,itrans,modelfit,respcorr,signal,maskout
  print,'pix=',pix,ipix, istartexpand, istopexpand, istartnoexpand,istopnoexpand
  print,'use_buffer', use_buffer
ENDIF

IF NOT use_buffer THEN BEGIN
 save,/xdr,time,fpoint,msk,itrans,modelfit,respcorr,signal,maskout,ech,$
 savearray,skypasssav,file=!INTDIR+tlfnam
; DON't save data, put it into a structure, pass it further in common
ENDIF ELSE BEGIN
 IF pix EQ 0 THEN BEGIN
 drift_struct.name = tdtnam
 drift_struct.time = time
 drift_struct.fpoint(istartexpand:istopexpand) = fpoint(istartexpand:istopexpand)
 drift_struct.ech(istartexpand:istopexpand) = estep(istartexpand:istopexpand)
 drift_struct.itrans = itrans
 ;drift_struct.ech = ech    
 drift_struct.modelfit(ipix,istartexpand:istopexpand) = modelfit(istartexpand:istopexpand)
 drift_struct.respcorr(ipix,istartexpand:istopexpand) = respcorr(istartexpand:istopexpand)
 drift_struct.signal(ipix,istartexpand:istopexpand) = signal(istartexpand:istopexpand)
 drift_struct.maskout(ipix,istartexpand:istopexpand) = maskout(istartexpand:istopexpand)
 drift_struct.msk(ipix,istartexpand:istopexpand) = msk(istartexpand:istopexpand)
 drift_struct.skypasssav(ipix,istartexpand:istopexpand) = skypasssav(istartexpand:istopexpand)
ENDIF ELSE BEGIN
 drift_struct.fpoint(istartexpand:istopexpand) = fpoint(istartexpand:istopexpand)
 drift_struct.modelfit(ipix,istartexpand:istopexpand) = modelfit(istartexpand:istopexpand)
 drift_struct.respcorr(ipix,istartexpand:istopexpand) = respcorr(istartexpand:istopexpand)
 drift_struct.signal(ipix,istartexpand:istopexpand) = signal(istartexpand:istopexpand)
 drift_struct.maskout(ipix,istartexpand:istopexpand) = maskout(istartexpand:istopexpand)
 drift_struct.msk(ipix,istartexpand:istopexpand) = msk(istartexpand:istopexpand)
 drift_struct.skypasssav(ipix,istartexpand:istopexpand) = skypasssav(istartexpand:istopexpand)
ENDELSE  
ENDELSE

;
; diagnostic for ipix to window ipix+1 made before zero levels and low values 
; restored in S
;timelinefigure,ipix+1,ipointstart,ipointstop,'fine',ipix,'' 
;
endfor ; irdc=0,nrdc-1
;
beta(ipix)=betawk
tau(ipix)=tauwk
;
endfor ; pix=0,npixloop-1
;
; this ends the calculation of the rersponsivity drift. Now follows
; the implementation into the S RMS and BAD arrays of common block SIGNAL:
;
; restore original values in S by reversing the manipulations in PREPAREDRIFT
; irrespective of whether we implement the solution or not. First 
; re-assign negative or low data in a single assignment for all detector 
; pixels (ie outside the pix loop):
;
;help,isdriftsave
;help,sdriftsave
if(isdriftsave(0) ne -1)then begin
s(isdriftsave)=sdriftsave ; isdriftsave=-1 if no
;                  low points were found in PREPAREDRIFT.
print,'DRIFT32: restoring low data to original values'
endif ; isdriftsave(0) ne -1
;
; Then restore original zero level for each detector pixel considered, and
; implement responsivity drift corrections in S and RMS for each 
; detector pixel, if requested (IMPLEMENTDRIFT=1), and if not done previously 
; (this should be done only once, after the full RESPC array has been 
; assembled in previous runs of DRIFT32 with IMPLEMENTDRIFT=0. In this step 
; the vignetting correction is also inplemented (resp = illum/modelfit does
; not contain the vignetting correction)
;
;print,'*** DRIFTINCREM = ',driftincrem
for pix=0,npixloop-1 do begin
ipix=pixarray(pix)
;
print,'DRIFT32: subtracting ',driftincrem(ipix),' from pixel ',ipix
s(ipix,*)=s(ipix,*)-driftincrem(ipix)
; added JUL00 RJT
map3dref(ipix,*,*)=map3dref(ipix,*,*)-driftincrem(ipix)
;
; finally make patched SCP plateaus invisible again to DRIFT32 by adding
; DG2BAD to bad (reversing subtraction applied in PREPAREDRIFT
bad=bad+dg2bad
;
; optionally correct S and RMS for responsivity drifts for pixels considered:
;
if(implementdrift eq 1)then begin ; implement solution for all 
;                                   timeline of pixel ipix
;if(ndriftsolve(ipix) ne 0)then begin
;print,'DRIFT32: attempt to correct already corrected data for pixel ',ipix
;print,'... drift correction not implemented ... aborting ... '
;goto, c999
;endif ; ndriftsolve(ipix) ne 0
;
iimp=where(imple(ipix,*) eq 1,niimp)
iimpno=where(imple(ipix,*) eq 0,niimpno)
nnnnn=n_elements(s(0,*))
if(niimp ne nnnnn)then begin
print,'DRIFT32: warning - responsivity drift solution only implemented '
print,'... for ',niimp,' out of ',nnnnn,' samples for pixel ',ipix
endif ; niimp ne nnnnn
s(ipix,iimp)=s(ipix,iimp)*respc(ipix,iimp)
if(array eq 'c100')then vignecorr32,1,ipix,echop,vig ; don't save the vig 
if(array eq 'c200')then vignecorr32,2,ipix,echop,vig ; arrays to save space
s(ipix,iimp)=s(ipix,iimp)/vig(iimp) ; vignetting correction
bad(ipix,iimp)=bad(ipix,iimp)+maskd(ipix,iimp)
;if(niimpno gt 1)then bad(ipix,iimpno)=2 ; set samples without correction 
; as bad - we don't do this as DRIFT32 may be run many times in a sequence.
; this should be set by calling program after the drift correction has benn
; finished
ndriftsolve(ipix)=ndriftsolve(ipix)+1
;if(irmscorr eq 1)then rms(ipix,iimp)=rms(ipix,iimp)*respc(ipix,iimp)
;
if(irmscorr ge 1) then driftrms,ipix,irmscorr
;
; correct rms for vignetting for all IRMSCORR
rms(ipix,iimp)=rms(ipix,iimp)/vig(iimp) 
;
endif ; implementdrift eq 1
;
endfor ; pix
;
; print out summary of detector fit results
;
if(icalmethod eq 1)then begin
print,'SUMMARY OF FCS FIT RESULTS  for TDT,ISEQ: ',tdtnam,iseq
print,pixarray
print,'ibefore=1:'
print,'BETA:',fcsbeta1
print,'TAU: ',fcstau1
print,'CHI: ',fcserr1
print,'SKY: ',fcssky1
print,'FCS: ',fcsill1
print,'ibefore=0:'
print,'BETA:',fcsbeta0
print,'TAU: ',fcstau0
print,'CHI: ',fcserr0
print,'SKY: ',fcssky0
print,'FCS: ',fcsill0
endif ; icalmethod eq 1
;
c999:
;
return
end
