pro driftcorr,beta,tau,signal0,signalshoot0,sillum0,ngrid,$
from,to,nplat,nplateau,vigp,keyp,finep,pident,$
keypoint,fpoint,itrans,ech,$
ytrue,ztrue,mask,vig,t,signal,rms,imodel,isquare,islewmethod,ifinemethod,$
fitresult,modelfit,modelfitshoot,modelfitslow,respcorr,maskout,niter,$
ipt1,ipt2,iptend1,iplat1,iplat2,ndetinit,nplatinit,$
ipix,tplatback,tplatforward,nlookback,backminv,backpredv,ierr,iverbose,chi
;
;save,/xdr,beta,tau,signal0,signalshoot0,sillum0,ngrid,$
;from,to,nplat,nplateau,vigp,keyp,finep,pident,$
;keypoint,fpoint,itrans,ech,$
;ytrue,ztrue,mask,vig,t,signal,rms,imodel,isquare,islewmethod,ifinemethod,$
;niter,ipt1,ipt2,iptend1,iplat1,iplat2,ndetinit,nplatinit,$
;ipix,tplatback,tplatforward,nlookback,backminv,backpredv,iverbose,$
;file='driftcorr.xdr'
;
; DRIFTCORR solves for a detector responsivity drift model on a specified
; range of pointing plateaus seen within an observation by a single 
; detector pixel. It should be AOT independent (provided the detector
; models and parameters are appropriate for the application), though it has 
; only been tested on P32 data. The routine must be started at the
; start of a plateau at the start of, or within, a fine pointing sequence
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         GAMMAEXP (common MOREPAR) third detector parameter
;         ZEROBIAS (common MOREPAR) fourth detector parameter
;         NDETINIT     = number of iterations to find the detector
;                starting state from the redundancy within the first pointing.
;         SIGNAL0     (single number in V/s): input guess for actual 
;                     (noiseless) slow component of detector response at 
;                     time zero prior to the 1st data sample. For IMODEL=1 or 2
;                     this is also the total detector response. Note this 
;                     number would normally not be numerically identical to 
;                     the first measured point for IMODEL=1,2 due to
;                     (1) noise on the measured point (2) due to any 
;                     residual drifts between time zero and the time of the
;                     first measured point for the case that the detector
;                     was not stabilized before the observation started and
;                     (3) because the vignetting may have been different 
;                     if the chopper was moving at time zero. Setting SIGNAL0
;                     equal to SILLUM0 means the detector would be stabilized.
;         SIGNALSHOOT0 (single number in V/s): (only used for imodel gt 2).
;                     estimate of shooting detector response at time DTIME 
;                     prior to the 1st data sample at ipt1. Only 
;                     overwritten if NDETINIT is GT 0.
;         SILLUM0     (single number in V/s): input guess at illumination
;                     prior to the first data point. Must be positive.
;                     One would expect SILLUM0 to be close (within the
;                     vignetting variations) to the illumination on the first
;                     plateau.
;         BACKMINV    estimated minimum background in V/s. From this is 
;                     derived the search range in V/s.
;         BACKPREDV   estimated background in V/s. This is 
;                     used to avoid the program getting stuck at illuminations
;                     much lower than BACKPREV were the time constant for
;                     recovery is very long
;         TPLATBACK   time interval in s over which to recalculate illumination
;                     history on discovery of an illegal plateau illumination
;                     (normally due to a glitch on a previous plateau).
;                      Glitch trap is ignored if  TPLATFORWARD and TPLATBACK
;                      are zero
;         TPLATFORWARD time in s over after a glitch over which to ignore
;                      derived illumination when calculating BESTSKY for
;                      the illumination history. Glitch trap ignored if 
;                      TPLATFORWARD and TPLATBACK are zero
;         NLOOKBACK   number of previous instances of a fine pointing
;                     plateau to use in determining illumination history
;                eg put one to use one chopper sweep 2 for 2 chopper sweeps
;                etc 
;         the following parameters define the plateau structure and plateau
;         dependent variables used by DRIFTCORR. This corresponds to the
;         whole timeline of an observation irrespective of whether a drift 
;         solution is sought only for part of this timeline or not.
;------------------------------------
;         NPLAT    - number of plateaus
;         NPLATEAU - an integer array (NPLAT) containing number of samples
;                    on each plateau 
;         TO       - (NPLAT) identifier of first point on a plateau
;         FROM     - (NPLAT) identifier of last point on a plateau
;         VIGP     - (NPLAT) value of vigneting factor (normally <=1) on a 
;                     plateau
;         KEYP     - (NPLAT) value of natural grid identifier on a plateau
;         FINEP    - (NPLAT) = 1 for fine pointing plateau = 0 for on the fly
;         PIDENT   - (integer array of dimension numberof elementary points)
;                     containing plateau identifier (not to be confused with 
;                     the identifier of the P32 fine pointing)
;         IPLAT1,IPLAT2 first and last plateaus including any "incomplete"
;                     plateaus 
;------------------------------------
;         IPT1,IPT2   identifiers of first and last points in entire range
;                     to be processed. These are also the first and last 
;                     samples in IPLAT1 and IPLAT2. Note that the first
;                     and last plateaus may be shorter than normal fine 
;                     pointing plateaus since the chopper is not synchronised
;                     with the spacecraft fine pointings. The range may 
;                     start and end within a plateau in the case of fine 
;                     pointings.
;         IPTEND1     last identifier of first s/c fine pointing. 
;                     The first identifier of first s/c fine pointing is 
;                     IPT1 if the data starts at a fine pointing.
;         IPIX        identity of detector pixel (only needed for labeling of
;                     the printout of chi)
;         NITER       number of passes through the data. Should be at least 2,
;                     as early determinations of plateau illuminations
;                     in the first pass are not constrained by the information
;                     on these plateaus gathered later in the dataset when
;                     the detector pixel passes again over the same sky
;                     position. 
;         NGRID       number of map pixels on the P32 natural grid
;                     (NGRID = NYGRID * NZGRID)
;         KEYPOINT   (1D array (INTEGER) of whole timeline of observation).  
;                     For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains the integer key K
;                     to the nearest direction on the P32 natural grid.
;                     K= iy + iz * nygrid,  where iy,iz are the 2D
;                     pointing positions and the grid has dimensions NYGRID,
;                     NZGRID, though this information is not needed, and not
;                     used by this routine.
;         FPOINT     (1D array INTEGER of whole timeline of observation ).
;                     For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains 1 if the 
;                     on target flag (OTF) was set (as defined by ERDTOMAP)
;                     and zero if the OTF is not set - ie when the data
;                     is taken on the fly. Beware that FPOINT is not 
;                     overwritten or changed by DRIFTCORR, even though the 
;                     plateau dependent quantities NPLAT NPLATEAU KEYP
;                     VIGP FINEP PIDENT FROM TO IPLAT1 IPLAT2 may take altered
;                     values during processing by DRIFTCORR if the glitchtrap 
;                     is operating. However NPLAT NPLATEAU KEYP VIGP FINEP
;                     PIDENT FROM TO IPLAT1 IPLAT2 are overwritten with their
;                     input values on exit from DRIFTCORR if the glitch trap
;                     has been operating
;         ECH        (1D array INTEGER of whole timeline of observation). 
;                      Chopper plateau identifier
;                     (1 to 13 for C100 1 to 7 for C200) for each time sample
;         ITRANS     (1D array INTEGER of whole timeline of observation). 
;                     This is set to 1 for data which is
;                     transmitted to the ground and zero for data not
;                     transmitted (in the fast readout modes).  
;         MASK       (1D array integer of whole timeline of observation)
;                     = 1 for data to be ignored. We have
;                     to import all time sampling points in the time array
;                     T in order to follow the illumination history
;                     so for ease of programming we import all data
;                     too, necessitating a mask.
;         VIG        (1D floating array of whole timeline of observation).
;                     This is set to the attenuation
;                     due to vignetting (VIG <= 1) as a function of chopper
;                     position in the sense detector signal = vig * sky
;                     As the same sky direction is viewed by the same pixel
;                     in different chopper offsets, the detector pixel
;                     illumination can vary even if the same sky direction is
;                     viewed.
;         T           (array in s of whole timeline of observation). 
;                      This array contains times for each read
;                     (including times where data is read on board but not
;                      transmitted since time=0 at start of observation.
;                      T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be a small positive
;                      number, normally the non-destructive read interval. 
;                      *** BEWARE that
;                      if T(0) is zero spurious results can be obtained.
;         SIGNAL       (real 1D aray of whole timeline of observation) 
;                      measured detector signal at times in T 
;                      since time zero. All points must be
;                      positive 
;         RMS          (array in V/s for whole timeline of observation).
;                      RMS uncertainty in SIGNAL
;         YTRUE        (1D array in arcsec - dimension full dataset)
;         ZTRUE        (1D array in arcsec - dimension full dataset) 
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;          IVERBOSE    =1 to print numerical diagnostics during the 
;                      iterations. =0 for reduced printing of diagnostics.
;                      =-1 for bare minimum. Printing of errorss are not
;                      affected by IVERBOSE.
;          ISLEWMETHOD (Integer flag).
;                      Determined method to get the illumination history
;                      during slews. This is particularly problematic when
;                      there is no transmission to ground at intervals during 
;                      a slew. The input parameter is used for iterations
;                      other than the first, for which ISLEWMETHOD = 2
;                      should be used 
;                  = 1 use the nearest P32 natural grid pixel
;                      to define the illumination for all points 
;                      (all combinations of FPOINT ITRANS)
;                  = 2 Use the measured sky points where they exist
;                      (from SKYPASS array)
;                      (ie ITRANS=1) and linearly interpolate in time
;                      where the points don't exist (ITRANS=0). Neighboring
;                      fine pointings not used.
;                  = 3 Perform a spline fit versus position for the points 
;                      taken from SKYPASS array in the slew constraining by 
;                      surrounding fine pointings too from BESTSKY (as
;                      the fine pointings are calculated before the points
;                      on the fly).
;                  = 4 use linear interpolation between nearest P32 natural
;                      grid pixels
;                  = 5 as for 2 but neighboring fine pointings are used
; Note that ISLEWMETHOD only controls information for the calculation of
; detector state at the start of each plateau. The calculation of restored
; data on the slew is not changed in any way - ie not treated as samples
; on the nearest grid point and not interpolated.
;
;         IFINEMETHOD (Integer Flag)
;                      Determines method to get the illumination
;                      on fine pointings when finding illumination history with
;                      FINDILLHIST
;                  = 1 mean
;                  = 2 weighted mean (using skyfoundnpt as weight)
;                  = 3 weighted mean (using 1/(skyfoundfitres)^2 as weight)
;                  = 4 median (using MEDIANTRUE routine)
;
;         ILLCON1,ILLCON2,IDRIFTRANCON (common SCCONSTRAIN)
;                      if IDRIFTRANCON is 1 then the search range is forced
;                      to be ILLCON1 to ILLCON2 and ZEROBIAS is set to zero.
;                      temporarily within DRIFTCORR
; ------------------------------------------------------------------------
; OUTPUT
;
;         BESTSKY      (common TESTDRIFTCORR)
;                      (1D array NGID in V/s) This is the drift corrected
;                      illumination that each point on the natural sky grid 
;                      would receive in the
;                      absence of vignetting, calculated from the accumulated 
;                      information in The dimension of sky is the
;                      total number of points on the natural P32 grid. 
;                      BESTSKY is constrained to be positive, and is found
;                      from the last call of FINDILLHIST from DRIFTCORR.
;         NBESTSKY     (common TESTDRIFTCORR)
;                      (ID integer array) Total number of elementary 
;                      data samples contributing to the value for BESTSKY.
;                      Note that FINDILLHIST always tries to find plausible
;                      values for BESTSKY even for directions for which
;                      NBESTSKY is zero.
;         FITRESULT    (1D real array of dimension all timeline of observation)
;                      NOT YET IMPLEMENTED
;                      an output parameter normalised to RMS 
;                      indicating how good the fit is for each elementary 
;                      sample points (ie all individual points on plateau 
;                      calculated according to 
;                      the value of the input parameter ISQUARE (see above). 
;                      The lower the better. It can be used to identify 
;                      plateaus not fitted by the data. This enables plateaus 
;                      to be masked out by the calling program if required
;                      by the application eg mapping. Note that for plateaus 
;                      with only 
;                      one data point (eg points taken on the fly) FITRESULT
;                      will tend to zero, rather than RMS, as the number of 
;                      search iterations increases. Because there is no
;                      statistical information on the goodness of fit for
;                      single points, FITRESULT is set to 0.0 for single points
;                      The overall goodness of fit can be calculated by the
;                      calling program as appropriate for the investigation
;                      being conducted (eg one could just use fine pointing
;                      points and take some statistical difference between
;                      the input data and MODELFIT.
;         MODELFIT     (expanded array in V/s all timeline of observation): 
;                      best fit model for the data. Set to response calculated
;                      by FOUKES (for imodel=1) or EXPFUN (IMODEL=2) to 
;                      illumination history calculated from FINDILLHIST in 
;                      range IPT1:IPT2. Outside IPT1:IPT2 set to zero.
;                      Unreliable points set to 1 in MASKOUT. MODELFIT
;                      is completely overwritten each time DRIFTCORR is called.
;                      diagnostics overlaying data with model etc.
;                      For imodel=3 MODELFIT=MODELFITSHOOT+MODELFITSLOW
;         MODELFITSHOOT shooting component of MODELFIT (imodel=3; set to zero
;                       otherwise.
;         MODELFITSLOW long component of MODELFIT (imodel=3; set to zero
;                       otherwise.
;         RESPCORR     (array, dimensioness all timeline of observation) 
;                      illumination/response calculated 
;                      from the vignetting and the model for the specified
;                      data range IPT1:IPT2. Samples where no solution could
;                      be found are marked 1 in MASKOUT and set to 1.0 in
;                      RESPCORR. RESPCORR iss set to 0.0 outside IPT1:IPT2.
;                      This is the basic transient correction result.
;                      RESPCOR iss completely overwritten each time DRIFTCORR
;                      is run.
;         MASKOUT      output MASK (all timeline). points within IPT1:IPT2 for
;                      which a responsivity correction could not be found are
;                      set to 1 in MASKOUT. Outside IPT1:IPT2 MASKOUT is set
;                      to zero. MASKOUT is completely overwritten each time
;                      DRIFTCORR is run. 
;         SIGNAL0     (single number in V/s): estimate of slow
;                     (noiseless) detector response at time DTIME prior to the
;                     1st data sample at ipt1.  This input parameter is only
;                     overwritten if NDETINIT is GT 0. Note that because of the
;                     convention of defining SIGNAL0 and SILLUM0 at DTIME prior
;                     to ipt1 the actual last preceding good transmitted data 
;                     point prior to ipt1 may precede ipt1 by more than DTIME 
;                     and so the values for SIGNAL0 and SILLUM0 may not be
;                     applicable. Also, the detector state is specified 
;                     uniquely by a combination of SIGNAL0 and SILLUM0 for
;                     imodel=1,2 (see comments for SOLVEBACKILLUM1), 
;                     so use of SIGNAL0, SILLUM0 output by DRIFTCORR has to 
;                     be made with care.
;         SIGNALSHOOT0 (single number in V/s): (only used for imodel gt 2).
;                     estimate of shooting detector response at time DTIME 
;                     prior to the 1st data sample at ipt1. Only 
;                     overwritten if NDETINIT is GT 0.
;         SILLUM0     (single number in V/s): illumination
;                     prior to the first data point, calculated from the 
;                     redundancy in the
;                     sky pointing pattern on the first fine pointing in the
;                     input data. This input parameter is only
;                     overwritten if NDETINIT is GT 0. See also comments 
;                     for SIGNAL0, above.
;         IERR         =1 if an error occurred
;
; ---- The following plateau related parameters may be changed during
;      DRIFTCORR if the glitch trap is in operation. The arrays refer to the 
;      plateau or sample structure for the whole timeline of the observation.
;      However, DRIFTCORR always returns these quantities with their
;      original values on exit even if the glitch trap has been operating.
;
;         NPLAT    - number of plateaus
;         NPLATEAU - an integer array (NPLAT) containing number of samples
;                    on each plateau 
;         TO       - (NPLAT) identifier of first point on a plateau
;         FROM     - (NPLAT) identifier of last point on a plateau
;         VIGP     - (NPLAT) value of vigneting factor (normally <=1) on a 
;                     plateau
;         KEYP     - (NPLAT) value of natural grid identifier on a plateau
;         FINEP    - (NPLAT) = 1 for fine pointing plateau = 0 for on the fly
;         PIDENT   - (integer array of dimension numberof elementary points)
;                     containing plateau identifier (not to be confused with 
;                     the identifier of the P32 fine pointing)          
;         IPLAT1,IPLAT2 first and last plateaus in the sequence to be processed
;
;----------------------------------------------------------------------------
;
;common_morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
@common_morepar
;common timer,time,dtime,complete
@common_timer
;common testdriftcorr,skyfound,skyfoundfitres,skyfoundnpt,skypass,nskyfound,$
;ndimsky,skyfoundplat,platskyuse
@common_testdriftcorr
@common_scconstrain
;common scconstrain,illcon1,illcon2,betalongconmin,betalongconmax,$
;betashortconmin,betashortconmax,taulongconmin,taulongconmax,$
;taushortconmin,taushortconmax,idriftrancon ; this common block
; only used for illcon1,illcon2 and idriftrancon
;
ntotal=n_elements(t) ; total number of data points in whole data stream 
                     ; (encompassing range of points IPT1:IPT2 to be processed)
;
tsubtract=t(ipt1)-dtime
t=t-tsubtract ;  t array updated such that t(ipt1) is time since last good
;                transmitted point This will mean that there may be negative 
;                 values in t. However these will not be used by driftcorr.
;                The assignment of t(ipt1)=dtime is just a convention. In fact
;                the last good transmitted point prior to ipt1 may precede ipt1
;                by more than this. This convention means however that any
;                output values for s and SILLUM0 will apply for a time
;                DTIME prior to ipt1 and not necessarily to the actual
;                preceding data point.
;
if(tplatback gt 0.0 or tplatforward gt 0.0)then begin
nplatorig=nplat
nplateauorig=nplateau
toorig=to
fromorig=from
vigporig=vigp
keyporig=keyp
fineporig=finep
pidentorig=pident
iplat1orig=iplat1
iplat2orig=iplat2
endif ; tplatback gt 0.0 or tplatforward gt 0.0
;
if(ipt1 lt 0 or ipt1 ge ipt2 or ipt2 ge ntotal)then begin
print,'DRIFTCORR: error in specified sample range'
print,'... ipt1,ipt2 = ',ipt1,ipt2
goto, c999
endif ; ipt1 lt 0 or ipt1 ge ipt2 or ipt2 ge ntotal
;
if(iverbose gt -1)then print,'running DRIFTCORR for sample range ',ipt1,ipt2,'
;
if(iplat1 lt 0 or iplat2 lt iplat1 or iplat2 gt nplat-1)$
then begin
print,'DRIFTCORR: error in specified plateau range or plateau structure:'
print,'... iplat1,iplat2,nplat =',iplat1,iplat2,nplat
goto, c999
endif ; iplat1 lt 0 or iplat2 lt iplat1 or iplat2 gt nplat-1
;
if(iverbose gt -1)then $
print,'....             and plateau range ',iplat1,iplat2
if(iverbose gt -1)then $
print,'.... and SIGNAL0,SIGNALSHOOT0,SILLUM0 ',$
float(signal0),float(signalshoot0),float(sillum0)
;
; note that the set of complete plateaus will in general not actually start 
; and end at IPT1 and IPT2. We wish to process for
; all the fine pointing plateaus including any shorter plateaus
; as these can be photometrically
; very important, representing the end points of the maps with a good view of
; the background outside the source.
;
findramp,from,to,ipt1,ipttest1,iplattest1
;
if(ipt1 ne ipttest1 or iplat1 ne iplattest1)then begin
print,'DRIFTCORR: probable error in plateau structure: '
print,'... ipt1,ipttest1,iplat1,iplattest1 = ',$
ipt1,ipttest1,iplat1,iplattest1
goto, c999
endif ; ipt1 ne ipttest1 or iplat1 ne iplattest1
;
if(ipt2 lt ntotal-1)then begin ; it is not the last point in detector stream
findramp,from,to,ipt2+1,ipttest2,iplattest2
;
if(ipt2+1 ne ipttest2 or iplat2+1 ne iplattest2)then begin
print,'DRIFTCORR: probable error in plateau structure: '
print,'... ipt2+1,ipttest2,iplat2+1,iplattest2 = ',$
ipt2+1,ipttest2,iplat2+1,iplattest2
goto, c999
endif ; ipt2+1 ne ipttest2 or iplat2 ne iplattest22
;
endif ; ipt2 lt ntotal-1
;
maskout=mask 
maskout(*)=0L
fpointwk=fpoint ; internal working array for fpoint. FPOINTWK is used
;  to determine whether we are slewing or not. Onle fine pointing plateau
;  illuminations are used to derive the solution foro the brightness
;  distribution. If a glitch is detectd on a fine pointing plateau, these
;  points are masked in MASKOUT and FPOINTWK set to 0 to create a virtual
;  slew. In this way the effect of the glitch on the illumination history can
;  be calculated without affecting the brighness solution and preventing
;  propagation of the glitch into the solution for succeeding good fine
;  pointings
;
; find SKYMAX (single REAL in V/s) which indicates maximum possible
; brightness which could be encountered. SKYMAX is used to define search
; space in the solution for the illumination for each plateau.
;
; internal parameters
;
igoodall=where(mask(ipt1:ipt2) eq 0 and itrans(ipt1:ipt2) eq 1,nigoodall) 
;
if(nigoodall le 0)then begin
print,'DRIFTCORR: aborting: no good points in specified data range'
ierr=1
goto,c999
endif ; nigoodall le 0
;
signalchi=signal(ipt1+igoodall) ; needed throughout subr. for calculating chi
rmschi=rms(ipt1+igoodall)
;
smax=max(signal(ipt1+igoodall))
;skymax=2.5*smax ; factor of 2.5 absolute maximum for single step and beta 0.4
skymax=4.0*smax ; extra safety factor needed for C100 array to avoid clipping
;                 noise on slews 
skymax=min([skymax,1.0/dtime]) ; otherwise saturation
;
if(iverbose gt 0)then $
print,'DRIFTCORR: upper limit for maximum signal set to ',skymax
if(smax*dtime ge 1.0)then print,'DRIFTCORR: warning: looks like saturated data'
;
; for each chopper plateau optimisation
; SILMIN,SILMAX (used minimum and maximum possible values for SILLUM 
; (used to control the cutting algorithm)
;
silmax=skymax
silmin=backminv ; backminv is the V/s corrsponding to the extreme limit
;           (eg) 0.1 MJy/sterad set for BACKMIN in PREPAREDRIFT 
silmin=backpredv/4.0 ; backpredv is the V/s corrsponding to the user supplied 
;                      estimate of the background BACKPRED 
;                      NB this meant as a safeguard against spurious
;                      dips (eg due to residual glitches) to an unphysically
;                      low solution but is dangerous in low s/n situations
;                      since it biases signal higher. It wwouold be better
;                      to bias the time constant for solutions 
;                      less than silmin so that any spuriouss signal does not
;                      have long lasting effects in the calculation of the 
;                      illumination history.
;                      On the other hand silmin should be less than zerobias
;
;dummy=backpredv/4.0
;silmin=backpredv-5.0*median(rmschi)
;if(silmin lt dummy)then silmin=dummy ; prevents eg negative values
;
; New procedure to allow silmin to be furher below offset pedastal
;
dummy=backpredv-5.0*median(rmschi)
if(silmin lt backminv)then silmin=backminv ; prevents eg negative values
;
;
; note, in setting silmin closer to the user estimate we make it more difficult
; for a wrong detector model to succeed in DETSELFCAL, since we increase the
; likelyhood of the solution reaching SILMIN. In running DRIFT32 the user
; should first give a conservatively low estimate for BACKPRED and then raise
; it according to what the background seen on the maps realy looks like.
;
;
; for IMODEL eq 3 avoid search in illumination regime for which shooting
;                 detector time constant becomes negative
;
; the following IF loop should have become redundant due to 
; the new statement ensuring positivity of the time scale long and
; shoot parameters in setshoottime_model3.pro. We leave it in as 
; it is not time consuming
; 
if(imodel eq 3)then begin
;
setrange,imodel,tau,silmin,silmax,slimit,ilower
;
if(slimit gt -999.0)then begin ; reset search range
;
if(ilower eq 1)then begin ; reset lower limit
if(iverbose gt -1)then begin
print,'DRIFTCORR Warning - due to negative shooting time constants '
print,'               .... resetting silmin from ',silmin,' to ',slimit
print,' .... solution unlikely to be satisfactory except when self-calibrating'
endif ; iverbose gt -1
silmin=slimit
endif ; ilower eq 1
;
if(ilower ne 1)then begin ; reset upper limit
;
;  no need to extend range to accommodate vignetting via 
;  slimit=slimit/min(vigp)as this can cause big inaccuracies 
;  for time constants very sensitive to illumination).
;
if(iverbose gt -1)then begin
print,'DRIFTCORR Warning - due to negative shooting time constants '
print,' .... resetting silmax from ',silmax,' to ',slimit
print,' .... solution unlikely to be satisfactory except when self-calibrating'
endif ; iverbose gt -1
silmax=slimit
endif ; ilower ne 1
;
endif ; slimit gt -999.0
;
endif ; imodel eq 3
;
;print,'DRIFTCORR: illcon1, illcon2',illcon1,illcon2
IF N_ELEMENTS(idriftrancon) EQ 0 THEN BEGIN
  print, 'WARNING: setting "idriftrancon" to 0 - not defined before'
  idriftrancon = 0
ENDIF
if(idriftrancon eq 1)then begin ; set manual limits
if(illcon1 le 0.0 or illcon2 le illcon1)then begin
print,'DRIFTCORR aborting - attempt to set manual limits of ',illcon1,illcon2
goto, c999
endif ; illcon1 le 0.0 or illcon2 le illcon1
if(iverbose gt -1)then begin
print,$
'DRIFTCORR: manual signal search range with zerobias=0 for idriftrancon = 1'
endif ; iverbose gt -1
silmin=illcon1
silmax=illcon2
endif ; idriftrancon eq 1
;
if(iverbose gt -1)then print,'DRIFTCORR: signal search range = ',silmin,silmax
;
zerobias_in=zerobias 
zerobias=(backpredv+silmin)/2.0
;zerobias=0.0
if(idriftrancon eq 1)then zerobias=0.0
;
; biasing parameter for detector model to prevent too
; slow a recovery from spurious low solutions.
; we can bias solution from correct solution aroundd zero level if
; zerobias is set above the real zero.
; Zerobias should be between the predicted background and SILMIN to have an
; effect. Thus, by specifying different values for BACKPRED (but never
; above the real minimum brightness encountered, or we would get a wrong 
; solution) the user can control the convergence of the solution and influence
; how often SILMIN is hit. As stated above, the user should start with 
; conservativley low BACKPRED and gradually raise it.
;
fitprec=0.1
;         fitprec is internal constant giving gives the precision to 
;         which the best fit is found. The interval of search for 
;         ILLUM=SKYSOL*VIG for each plateau used by SOLVEILLUM1
;         ILLUM is RMS*FITPREC. Where SKYSOL is the solution for the
;         plateau, VIG is the vignetting for the plateau (<=1) and
;         RMS is the detector noise for the plateau. Here fitprec is set
;         to the precision desired for the final iteration. For the
;         first iteration a higher value may be set to speed up the solution
;         when the bounding illumination tolerances for the illumination
;         search in SOLVEILLUM1 are still high.
;
if(min(rms(ipt1+igoodall)) le 0.0)then begin
print,'DRIFTCORR: rms array zero or negative '
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(isquare lt 0 or isquare gt 2)then begin
print,'DRIFTCORR: isquare out of range - isquare = ',isquare
ierr=1
goto,c999
endif
;
if(imodel ne 1 and imodel ne 2 and imodel ne 3)then begin
print,'DRIFTCORR: model must be Foukes-Schubert or exponential or expshoot'
ierr=1
goto,c999
endif
;
if(backminv le 0.0)then begin
print,'DRIFTCORR: aborting: detector data must be positive - '
print,'backminv = ',backminv
ierr=1
goto,c999
endif
;
; the following requirement on positivity of data is not needed
;
;if(min(signal(ipt1+igoodall)) lt backminv)then begin
;print,'DRIFTCORR: aborting: detector data must be at least backminv - '
;print,'backminv, min(signal(ipt1+igoodall)),nigoodall = ',$
;backminv,min(signal(ipt1+igoodall)),nigoodall
;ierr=1
;goto,c999
;endif
;
if(imodel eq 1 or imodel eq 2)then begin
if(beta le 0.0 or beta gt 1.0)then begin
print,'DRIFTCORR: aborting: beta out of range - beta = ',beta
ierr=1
goto,c999
endif
endif ; imodel eq 1 or imodel eq 2
;
if(fpointwk(ipt1) eq 0)then begin
print,'DRIFTCORR warning: observation does not start on a fine pointing'
endif
;
if(imodel lt 3 and tau le 0.0)then begin
print,'DRIFTCORR: aborting: tau negative - tau = ',tau
ierr=1
goto,c999
endif
;
if(signal0 le 0.0)then begin
print,'DRIFTCORR: aborting: SIGNAL0 must be positive - signal0 = ',$
signal0
ierr=1
goto,c999
endif
;
;if(signalshoot0 lt 0.0)then begin ; this condition does not have to be
;                                   strictly met, maybe.
;print,'DRIFTCORR: aborting: SIGNALSHOOT0 must be positive on input'
;print,' - signalshoot0 = ',signalshoot0
;ierr=1
;goto,c999
;endif
;
if(sillum0 le 0.0)then begin
print,'DRIFTCORR: '
print,'aborting: SILLUM0 must be positive - sillumprev = ',sillumprev
ierr=1
goto,c999
endif
;
;if(min(t(igoodall)) le 0.0)then begin
if(min(t(ipt1+igoodall)) le 0.0)then begin
print,'DRIFTCORR: aborting: time array t must be a positive interval since'
print,'the last destructive read - min(t) = ',min(t(ipt1+igoodall))
ierr=1
goto,c999
endif
;
if(niter le 1 and iverbose gt 0)then begin
print,'DRIFTCORR warning: need at least two iterations: niter = ',niter
;ierr=1
;goto,c999
endif
;
if(nigoodall gt 0)then begin
skydummy=median(signal(ipt1+igoodall))
if(iverbose gt 0)then print,'DRIFTCORR: skydummy = ',skydummy
endif ; nigoodall gt 0
;
; copy of input parameters dependent on plateau structure
;
nplatorig=nplat
nplateauorig=nplateau
toorig=to
fromorig=from
vigporig=vigp
keyporig=keyp
fineporig=finep
pidentorig=pident
iplat1orig=iplat1
iplat2orig=iplat2
;
; set up YK ZK needed by findillumhist outside future niter loop
;
findkyz,yk,zk ; yk and zk have dimensions of whole naturalgrid 
;
; initialize arrays to hold the accumulating information of estimates of 
; true sky illumination and errors etc in these estimates. We dimension 
; these arrays according to 
; [safety factor]/[(number of pointing plateaus not on fly)*(number of natural
;                                                            map grid points)] 
; first dimension arrays holding sky measurements (find NDIMSKY):
;
safe=10.0
ii=where(nplateau gt 1,nmore1)
if(nmore1 le 0)then begin
print,'DRIFTCORR warning: no pointing plateau has more than one point'
ndimsky=safe*2.0*float(nplat)/float(ngrid)
endif ; nmore1 le 0
if(nmore1 gt 0)then begin
ndimsky=safe*float(nmore1)/float(ngrid)
endif ; nmore1 gt 0
ndimsky=long(ndimsky+1.)
;
; solve for the plateau illumination and sky brightness, updating sky 
; brightness stack as go along:
;
If(iverbose gt 0)then $
print,'DRIFTCORR: iplat from    to nigood keyp finep sigplat   skyguess',$
'    signalprev  sillumprev'
;
; follow illumination history back a maximum of JJFOLLOW samples. This
; is needed to speed things up.
;
if(iverbose gt -1)then print,'DRIFTCORR: imodel,beta,tau,gammaexp,zerobias:'
if(iverbose gt -1)then print,'...',byte(imodel),beta,tau,gammaexp,zerobias
if(iverbose gt -1 and imodel eq 3)then begin
print,'... betaexp,betacoeff,tauoffset'
print,'...',betaexp,betacoeff,tauoffset
print,'... betashoot,betacoeffshoot,betaexpshoot'
print,'...',betashoot,betacoeffshoot,betaexpshoot
print,'... tauoffsetshoot,taushoot,gammaexpshoot'
print,'...',tauoffsetshoot,taushoot,gammaexpshoot
endif ; iverbose gt -1 and imodel eq 3
if(iverbose gt 0)then print,'DRIFTCORR: iter = 0'
;
iter=0 ; iteractin coutner
kfineall=lindgen(ngrid)
idetinit=-1
;
; start of minor iteration loop IDETINIT for detector initial state. Call
; DETINIT only after an intial pass has filled SKYPASS but prior to
; initialisation of arrays by DROUINIT
;
c322:
;
idetinit=idetinit+1
;
if(ndetinit gt 0)then begin
;
if(idetinit gt 0)then detinit,beta,tau,signal0,signalshoot0,sillum0,nplatinit,$
ipt1,iptend1,fromorig,toorig,keyporig,vigporig,$
t,signal,rms,itrans,mask,keypoint,ech,$
bestsky,skypass,fitprec,imodel,silmin,silmax,isquare
;
if(idetinit ge 1)then $
if(iverbose gt -1)then $
print,'subiter',byte(idetinit),$
' : sig0,sigshoot0,sill0 =',float(signal0),float(signalshoot0),$
float(sillum0)
if(idetinit eq 0)then $
if(iverbose gt -1)then $
;print,'... sig0,sigshoot0,sill0 =',$
print,'subiter',byte(idetinit),$
' : sig0,sigshoot0,sill0 =',$
float(signal0),float(signalshoot0),float(sillum0)
;
endif ; ndetinit gt 0
;
; initialise arrays in common TESTDRIFTCORR before entering plateau loop
; for each minor iteration in IDETINIT (for NDETINIT iterations for detector
; starting state SIGNAL0 SIGNALSHOOT0 SILLUM0
;
; call updated JUL00 RJT to include nskytry,skytryplat
drouinit,iplatprevarray,iplatprevprevarray,jjhistfollowarray,$
illhistf,resphistf,resphistslowf,resphistshootf,$
kfinelast,jjhistfollow,iplatprev,iplatprevprev,$
iplatlastfine,tlastfine,nsolve,ipt1,nskytry,skytryplat,$
bestsky,nbestsky,t,ngrid,nplat
;
;save,/xdr,beta,tau,signal0,signalshoot0,sillum0,ngrid,$
;from,to,nplat,nplateau,vigp,keyp,finep,pident,$
;keypoint,fpoint,itrans,ech,$
;ytrue,ztrue,mask,vig,t,signal,rms,imodel,isquare,islewmethod,ifinemethod,$
;niter,ipt1,ipt2,iptend1,iplat1,iplat2,ndetinit,nplatinit,$
;ipix,tplatback,tplatforward,nlookback,backminv,backpredv,iverbose,$
;file='driftcorr.xdr'
;
iplat=iplat1-1L
;
c444:
;
; revert to start of timeline to find a new estimate of detector starting state
;
if(iplat ge 0)then begin ; here iplat refers to the last plateau processed
if(to(iplat) ge iptend1 and idetinit lt ndetinit)then goto,c322
endif ; iplat ge 0
;
if(iplat ge iplat2)then goto, c555
;
iplat=iplat+1
;
platskyuse(iplat)=0 ; needed in case glitchrecover upset the indexing
jjhistfollowarray(iplat)=jjhistfollow
iplatprevarray(iplat)=iplatprev
iplatprevprevarray(iplat)=iplatprevprev
;
ninplat=nplateau(iplat)
itransplat=itrans(from(iplat):to(iplat))
mplat=mask(from(iplat):to(iplat))
igood=where(mplat eq 0 and itransplat eq 1,nigood)
;
if(nigood ge 1)then begin
;
nsolve=nsolve+1
;
; Find SIGNALPREV (long response), SIGNALPREVSHOOT (short response)
; (sum=SIGNALPREVTOT) and SILLUMPREV from 
; all previous photometric information
;
; find best guess for sky BESTSKY(0:ngrid-1) (where ngrid is number of sample 
; points on natural p32 grid)from accumulated previous results using
; information in SKYPASS and in SKYFOUND, SKYFOUNDFITRES, SKYFOUNDNPT. Using
; BESTSKY and the inferred sky brightnesses encountered during slews stored in
; SKYPASS calculate illumination history ILLHIST
; up to point kk=from(iplat)-1. For plateaus with no information 
; (due to glitches or no transmission of data) use median of
; all SKYFOUND arrays or a simple interpolation between neighboring points
; which have information (program FINDILLHIST gives a warning). 
; If there has been NO previous results (eg all points flagged bad on first 
; plateau) then use SILLUM0.
;
; establish range jj to kk to follow illumination history, making sure jj
; is at a point where two successive previous estimates of illumination have
; been made (ie at a fine pointing following a previous fine pointing).
;
kk=from(iplat)-1
jj=jjhistfollow
;
if(nsolve gt 0)then begin ; we are not at first plateau containing a solution
;                           so must calculate illumination history
;
; even for fine pointing use islewmethod=2 for iter=0 rather than input value
; since for islewmethod=4 findillstep1 can give a misleading interpolation 
; on noisy slews preceding the fine pointing. 
; 
if(finep(iplat) eq 1)then begin 
;
; replaced 2 with islewmethod JUL00 RJT (to allow use with islewmethod=1)
;findillhist,2,skydummy,$
;from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
;ngrid,yk,zk,bestsky,jj,kk,pident,illhist,ipix,ierr
;
findillhist,islewmethod,skydummy,$
from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
ngrid,yk,zk,bestsky,jj,kk,pident,illhist,ipix,ierr
;
endif ; finep(iplat) eq 1
;
; for on the fly use islewmethod=2 as tests show islewmethod=4 gives poor
; results when slewing over sources. Also, if we wish photometric information 
; from slews to be independent, we should use islewmethod=2, avoiding 
; use of photometry from BESTSKY, which is exclusively from fine pointings.
; Treat fine pointing plateaus with only
; one sample as taken on the fly where the finep(iplat)=1 assignment
; is just a chance close crossing to a natural grid pixel.
;
if(finep(iplat) ne 1)then begin ; we are slewing
;
if(finep(iplat) ne 1 and finep(iplatprev) eq 1)then begin
iplatlastfine=iplatprev
tlastfine=t(from(iplatlastfine)) ; label according to time due to
;                         possible redefinition of plateaus by GLITCHRECOVER
endif ; finep(iplat) ne 1 and finep(iplatprev) eq 1
;
jjslew=jjhistfollow
;
if(islewmethod eq 2 and finep(iplatprev) eq 0)then begin
;                    we are at at least the second point in slew
;jjslew=jjhistfollow ; standard solution, most accurate
;jjslew=from(iplatlastfine) ; for better accuracy find illumination history
;                            from last plateau with a fine pointing
;                            This is exactly as accurate as jjslew=jjhistfollow
nslewhistback=10 ; should be set to typical number of transmitted and non-
;                  transmitted samples between the plateaus for which 
;                  solveillum1 was run
ipba=iplatprev-nslewhistback
if(ipba lt iplatlastfine)then ipba=iplatlastfine ; no need to go back
;                                           further than last fine pointing
;jjslew=from(ipba) ; calculate history from up to the last nslewhistback
                  ; transmitted good plateaus
ipp=iplatprevprev
if(ipp lt iplatlastfine)then ipp=iplatlastfine
jjslew=from(ipp) ; use last two values for which solveillum1 was run, thus 
;                  getting a linear interpolation rather than just an 
;                  extrapolation. This is a significant improvement.
;
;jjslew=from(iplatprev) 
;                set to last point 
;                in slew with good transmitted data. This means the 
;                illumination history will only be calculated within slew 
;                between this last point and the current point. Remember
;                the current point may be displaced from previous point by 
;                more than one sample in the expanded time history due to
;                glitches and or non-transmitted points. Restricting
;                consideration to just this short time range may be reasonable
;                for the case ISLEWMETHOD is 2 since this in any case just
;                interpolates between nearest samples of SKYPASS, just taking
;                the preceding SKYPASS value when encountering a range with
;                a preceding but no succeeding data point, as at the very end
;                of the illumination history. (Before this, where there
;                are both preceding and succeeding points in the slew, the
;                solution will be slightly different as an interpolatiion
;                rather than a simple extension of the last value. This 
;                could be catered for by always calculating from two 
;                plateaus back if the slew were long enough).
;                Taking this smaller time range compared with the full
;                calculation from jjhistfollow speeds up the calculation
;                compared to a repeated re-entry to FINDILLHIST for each 
;                point of slew, especially for the slew part of the 
;                illumination history. Note that for the first fine pointing
;                after a slew we do calculate the illumination history
;                all the way back to jjhistfollow (since jjj always set to 
;                jjhistfollow at the start of the major iplat loop), thus 
;                getting linear interpolations between each good transmitted 
;                sample on the slew.
;                
endif ; islewmethod eq 2
;
; replaced 2 with islewmethod JUL00 RJT
;findillhist,2,skydummy,$
;from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
;ngrid,yk,zk,bestsky,jjslew,kk,pident,illhist,ipix,ierr
;
findillhist,islewmethod,skydummy,$
from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
ngrid,yk,zk,bestsky,jjslew,kk,pident,illhist,ipix,ierr
;
jj=jjslew
;
endif ; finep(iplat) ne 1
;
illhistf(jj:kk)=illhist(jj:kk) ; running update of illumination
;                                history stored in ILLHISTF
sillumcalc=illhist(jj:kk)
;
if(jj eq ipt1)then begin
tcalc=t(jj:kk)-t(jj)+dtime
sigstart=signal0 ; long response (also total response for imodel=1 or 2
sigshootstart=signalshoot0 ; fast response
sillstart=sillum0
endif ; jj eq ipt1
;
if(jj gt ipt1)then begin
;
tcalc=t(jj:kk)-t(jj-1)
;
if(imodel eq 1 or imodel eq 2)then sigstart=resphistf(jj-1) 
; long resp is total resp for imodel=1 or 2
if(imodel eq 3)then begin
sigstart=resphistslowf(jj-1) ; slow response
sigshootstart=resphistshootf(jj-1)
endif ; imodel eq 3
;
sillstart=illhistf(jj-1)
;
endif ; jj gt ipt1
;
; calculate detector response to illumination history up to 
; sample kk=from(iplat)-1
;
if(imodel eq 1)then foukes,beta,tau,sigstart,sillstart,$
sillumcalc,tcalc,resphist
if(imodel eq 2)then expfun,beta,tau,sigstart,sillstart,$
sillumcalc,tcalc,resphist
if(imodel eq 3)then expshoot,beta,tau,sigstart,sigshootstart,sillstart,$
sillumcalc,tcalc,resphist,resphistslow,resphistshoot
;
resphistf(jj:kk)=resphist(*) ; total response (for all models)
if(imodel eq 3)then begin
resphistslowf(jj:kk)=resphistslow(*) ; slow component of response
resphistshootf(jj:kk)=resphistshoot(*) ; fast component of response
endif ; imodel eq 3
;
; We now must find SIGNALPREVLONG SIGNALPREVSHOOT and SILLUMPREV to use in 
; SOLVEILLUM1. An 
; asignment from sample kk=from(iplat)-1, and the corresponding assignment of 
; the time array as tplat=tplat-t(kk) for SOLVEILLUM1, can however lead 
; to a biased solution from SOLVEILLUM1 on slews. For a fixed definition of kk
; as from(iplat)-1 this time assignment is only true for a slew plateau 
; following a slew plateau when there is no intervening non-transmitted or bad
; time sample. This can be seen by a simple example. Suppose the detector
; is out of equilibrium in the slew with constant illumination (constant sky)
; but with a high, declining signal (eg from a previous transit of a source).
; Then suppose we are in a data reduction factor mode where 
; kk=from(iplat)-1 = (say) 1000, so that the last good transmitted data point 
; was point 992 and the current good transmitted point was sample 1001. 
; If we now try to search for an illumination step at sample 1000 the 
; SOLVEILLUM1 routine will implicitly solve for the response to a 
; signal history with SIGNALPREV set at point 992, though it was found
; at point 992. Thus we are forcing a constant signal betweeun 992 and 1000, 
; which does not apply,
; and this will induce a repeated first order cumulative bias in the data.
; One possible solution would be to treat each chopper plateau as a   
; constant illumination. However, this would give bad errors when slewing
; over sources in the rapid read out modes. The progamatic solution adopted 
; here is simply to solve for a constant illumination between each sucessive
; data point (ie avoiding extrapolation of signal) rather than between the 
; current data point and the end of the preceeding plateau which may have no
; data point. This also applies for the first plateau in a fine pointing
; sequence following a slew. 
;
; An alternative is to invoke the SOLVESLEW1
; routine (uncomment the ********** loop below). SOLVESLEW1 forces a 
; solution to be a linear interpolation between the new and the current
; illumination while on a chopper plateau, stepping only between chopper
; plateaus. However though SOLVESLEW1 is slightly more accurate, it is slower.
; Hence we use SOLVEILLUM1 with initial conditions depending on whether the
; preceding point was on a slew or not:
; tplat=t(from(iplat):to(iplat))-t(to(iplatprev) ; if fpoint(iplatprev)=0
; tplat=t(from(iplat):to(iplat))-t(kk) ; if fpoint(iplatprev)=1
;
; iplatprev exists within cirrent nsolve condition of nsolve gt 0
;
if(finep(iplatprev) eq 0)then begin
;                                            
tplat=t(from(iplat):to(iplat))-t(kk) ; timed from last point of previous
;    plateau independent of whether this last point was a good and transmitted 
;
if(imodel eq 1 or imodel eq 2)then signalprev=resphistf(kk) ; slow = full resp.
if(imodel eq 3)then begin
signalprev=resphistslowf(kk) ; slow
signalprevshoot=resphistshootf(kk) ; fast
endif ; imodel eq 3
;
sillumprev=illhistf(kk)
;
endif ; finep(iplatprev) eq 0
;
if(finep(iplatprev) eq 1)then begin
;
tplat=t(from(iplat):to(iplat))-t(kk) ; timed from last point of previous
;    plateau independent of whether this last point was a good and transmitted 
;
if(imodel eq 1 or imodel eq 2)then signalprev=resphistf(kk) ; slow=full
if(imodel eq 3)then begin
signalprev=resphistslowf(kk) ; slow
signalprevshoot=resphistshootf(kk) ; fast
endif ; imodel eq 3
;
sillumprev=illhistf(kk)
;
endif ; finep(iplatprev) eq 1
;
endif ; nsolve gt 0
;
if(nsolve le 0)then begin
;
if(imodel eq 1 or imodel eq 2)then signalprev=signal0 ; slow=full
if(imodel eq 3)then begin
signalprev=signal0 ; slow
signalprevshoot=signalshoot0 ; fast
endif ; imodel eq 3
;
sillumprev=sillum0
tplat=t(from(iplat):to(iplat))-t(from(iplat))+dtime 
;
endif ; nsolve le 0
;
; this completes assignment of TPLAT SIGNALPREV SIGNALPREVSHOOT
; and SILLUMPREV. Now set up 
; remaining input for SOLVEILLUM1:
;
mplat=mask(from(iplat):to(iplat))
sigplat=signal(from(iplat):to(iplat))
rmsplat=rms(from(iplat):to(iplat))
tplat=tplat(igood)
mplat=mplat(igood)
sigplat=sigplat(igood)
rmsplat=rmsplat(igood)
;
; find SILLUMPLAT and hence SKYGUESS with SOLVEILLUM1. For first iteration
; use larger fitprec than for subsequent iterations to improve speed
;
fitprecsave=fitprec
fitprec=0.1
;
solveillum1,beta,tau,signalprev,signalprevshoot,sillumprev,sillumplat,tplat,$
sigplat,rmsplat,fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;
iplatprevprev=iplatprev
iplatprev=iplat ; only set when a solution is found (either for slew
;                 or for fine pointing
;
ifoundglitch=0
;
delta=median(rmsplat)/100.0
delta=double(delta)
;
iproblem=0
if(sillumplat le silmin+delta and nsolve gt 0)then iproblem=1
if(sillumplat ge silmax-delta and nsolve gt 0)then iproblem=1
; note - always use same silmin,silmax as this is needed by FINDILLHIST 
iproblem=0 ; this admits all solutions into FINDBESTSKY, including
           ; those which are running into silmin, silmax in SOLVEILLUM1
           ; if this statement commented we only uss solutions found
           ; within the SILLMIN SILLMAX range. In practice, when one has
           ; an imperfect model, overshhots are common. However, this does not
           ; mean the overshoots should be ignored, as they are telling us of a
           ; a tendency for the illumination to be high or low. This means that
           ; the overall map is often better if solutions running into SILLMIN
           ; and SILLMAX are admitted. However, the machinary is in place 
           ; to ignore plateau ss\olutions running into SILLMIN and SILLMAX
           ; by commenting out this statement (and the one in the second loop 
           ; below)
;
if(iproblem eq 1)then begin 
if(tplatback gt 0.0 or tplatforward gt 0.0)then begin
;
ifoundglitch=1 ; to prevent storage in SKYFOUND and pollution of BESTSKY
; note that if the first fine-pointing plateau on a sky pointing direction
; has either no data or has ifoundglitch=1, the next call of FINDILLHIST
; will issue a warning that the illumination history for this plateau
; has to be guessed. The user should react to this by setting tplatforward
; and tplatback to reasonable positive values, according to the output
; of TIMELINEFIGURE
;
if(iverbose gt -1)then $
print,'DRIFTCORR glitch trap: iplat,t(s),silmin,silmax,sillumplat = ',$
iplat,t(kk+1),silmin,silmax,sillumplat 
;
npointback=tplatback/dtime
jstart=kk+1-npointback
findramp,from,to,jstart,jout,jramp ; jramp is plateau containing sample jstart
nplatback=iplat-jramp
nplatback=max([1,nplatback]) ; ensures go back at least one plateau
npb=nplatback ; number of plateaus to go back
;if(iplat-nplatback lt 0)then npb=iplat
if(iplat-nplatback lt iplat1)then npb=iplat-iplat1 ; can't go back beyond
;                                 start of data sequence
;
npointforward=tplatforward/dtime
jstart=kk+1+npointforward
findramp,from,to,jstart,jout,jramp
nplatforward=1+jramp-iplat ; the +1 is to count from the end of current ramp
nplatforward=max([1,nplatforward])
npf=nplatforward ; number of plateaus to go forward
;if(iplat+nplatforward ge nplat)then npf=nplat-iplat-1
if(iplat+nplatforward ge iplat2+1)then npf=iplat2-iplat
;
; nskytry,skytryplat added to parameters of glitchrecover call RJT JUL00
glitchrecover,iplat,npb,npf,maskout,$   
skyfound,nskyfound,skyfoundfitres,skyfoundnpt,platskyuse,skyfoundplat,$
nskytry,skytryplat,bestsky,nbestsky,skypass,$
fpointwk,keypoint,vig,nplat,nplateau,from,to,vigp,keyp,finep,pident,$
iplatprevarray,iplatprevprevarray,jjhistfollowarray,igoback
;
; make sure iplat2 takes a value consistent with any changes to the
; plateau structure made by GLITCHRECOVER. This should be the only
; change necessary to the succeeding plateau definitions outside of 
; GLITCHRECOVER. This is slightly inefficient since it is only needed if
; the plateau structure is changed, which is not always the case, for
; instance if fpointwk is only assigned during an existing slew. 
; Unnecessary calls to findramp could be avoided by making NFINEFORWARD
; and NFINEBACK output parameters of GLITCHRECOVER and only recalculating
; iplat2 if either of NFINEBACK or NFINEFORWARD were not 0.
;
findramp,from,to,ipt2,itest,iplat2test
if(iplat2test ne iplat2)then begin
if(iverbose gt -1)then $
print,'DRIFTCORR - resetting iplat2 from ',iplat2,' to ',iplat2test
iplat2=iplat2test
endif ; iplat2test ne iplat2
if(fpointwk(ipt2) ne 1)then begin
if(iverbose gt -1)then begin
print,'DRIFTCORR warning - in redefinition of plateau structure in the glitch'
print,'... trap the final fine pointing plateau has been set to a virtual'
print,'... slew!!' 
endif ; iverbose gt -1
endif ; fpointwk(ipt2) ne 1
;
if(igoback eq 1)then begin ; a fine pointing was encountered
;                          within the last NPB plateaus and the illumination
;                          history there censored
; now need to update all quantities depending on plateau identifiers where
; this was not done by GLITCHRECOVER:
;
; note the following iiii does not necessarily correspond to a real plateau
; containing good transmitted data since npb is, in that context, arbitrary.
; This is why iplatprevarray iplatprevprevarray jjhistfollowarray is updated 
; in the main iplat
; loop for each iplat, not just for the iplats corresponding to transmitted
; good data.
;
iiii=max([0,iplat-npb])
iplatprevprev=iplatprevprevarray(iiii)
iplatprev=iplatprevarray(iiii) ; set iplattprev to the correct value for 
;                                     the first iplat to be recalculated
; reset starting plateau to one before the first 
; to be recalculated and go back to start of iplat loop
;
iplat=max([0,iplat-npb-1]) 
;
; update iplat1. Note that the sample definitions ipt1 ipt2 are independent
; of the plateau structure
;
findramp,from,to,ipt1,itest,iplat1test
if(iverbose gt -1)then $
print,'DRIFTCORR: resetting iplat1 from ',iplat1,' to ',iplat1test
iplat1=iplat1test
if(itest ne ipt1 or fpointwk(ipt1) ne 1)then begin
if(iverbose gt -1)then begin
print,'DRIFTCORR warning - in redefinition of plateau structure in the glitch'
print,'... trap the starting fine pointing plateau has been set to a virtual'
print,'... slew!!' 
endif ; iverbose gt -1
endif ; itest ne ipt1
;
if(iverbose gt -1)then begin
print,'DRIFTCORR reverting to iplat,iplatprev,iplatprevprev,iplatlastfine = '
print,'... ',iplat,iplatprev,iplatprevprev,iplatlastfine,jjhistfollow
print,'... iplat1 updated to ',iplat1
endif ; iverbose gt -1
;
jjhistfollow=jjhistfollowarray(iplatprev)
;
itest=where(t eq tlastfine,nitest)
if(nitest gt 0)then begin ; a value for iplatlastfine has been set
;                      and must be recalculated due to the redefinition of 
;                      plateaus
iitest=itest(0)
findramp,from,to,iitest,iitestout,iitestramp
iplatlastfine=iitestramp
;
if(iplatlastfine ge iplat+1)then begin ; we have gone back further than the
                                   ; last setting of iplatlastfine, tlastfine 
iii=where(finep(iplat1:iplat) eq 1,niii)
if(niii gt 0)then iplatlastfine=max(iii)
if(niii le 0)then iplatlastfine=iplat1
ttto=to(iplatlastfine)
tlastfine=t(ttto)
;
endif ; iplatlastfine ge iplat+1
;
endif ; nitest gt 0
;
goto,c444
;
endif ; igoback eq 1
;
; for igoback lt 1 the plateaus have been restructured by the resetting of 
; fpointwk only in forward dirction so there is no need to go back.
;
; However if a fine pointing has been changed to a slew and we don't go back 
; we should recalculate igood and the solution sillumplat.
; However we don't do this as the sequential solution for the
; original points IGOOD on the fine pointing will not be very different from 
; the fine pointing solution. In any case only the interpolation is affected
; up to the next point on the newly created virtual slew and all points
; on the slew will be recalculated anyway. The presence of a majority
; of SILLMIN values in SKYFOUND means that the median option for FINDBESTSKY
; (IFINEMETHOD=4, set in DRIFT32) may be unsatisfactory, and thaat it is best
; to work with IFINEMETHOD=2 (weighting proportional to samples).
;
endif ; tplatback gt 0.0 or tplatforward gt 0.0
endif ; iproblem eq 1 (recalculate)
;
skyguess=sillumplat/vigp(iplat)
fitprec=fitprecsave
;
if(iverbose gt 0)then $
print,iplat,from(iplat),to(iplat),byte(nigood),byte(keyp(iplat)),$sinput
byte(finep(iplat)),float(median(sigplat)),float(skyguess),$
float(signalprev),float(sillumprev)
;
; update arrays of previous results if on fine pointing and if there
; was no glitch found 
;
if(ifoundglitch eq 0)then begin
;
if(finep(iplat) eq 1)then begin
;
kp=keyp(iplat) ; natural grid 1D pointing identifier
;
; changes made from here RJT JUL00
;
;nskyfound(kp)=nskyfound(kp)+1 ; commented RJT JUL00
;nsf=nskyfound(kp) ; commented rjt JUL00
;
nsftry=nskytry(kp)+1 ; added rjt JUL00
skytryplat(kp,nsftry-1)=iplat ; added rjt JUL00
; following if clause added JUL00 rjt
if(nsftry gt ndimsky)then begin
print,'DRIFTCORR error: array dimension SKYTRYPLAT too small'
ierr=1
goto, c999
endif ; nsftry gt ndimsky
;
if(iproblem eq 0)then begin ; added rjt JUL00
;
nskyfound(kp)=nskyfound(kp)+1 ; added RJT JUL00
nsf=nskyfound(kp) ; added RJT JUL00
if(nsf gt ndimsky)then begin
print,'DRIFTCORR error: array dimension SKYFOUND too small'
ierr=1
goto, c999
endif ; nsf gt ndimsky
platskyuse(iplat)=1
skyfound(kp,nsf-1)=skyguess
skyfoundfitres(kp,nsf-1)=fitresult 
skyfoundplat(kp,nsf-1)=iplat
skyfoundnpt(kp,nsf-1)=nigood ; number of data points contributing to fit
kfinelast=kp
;
; update bestsky used to update illumination history on next plateau
;
findbestsky,ifinemethod,kfinelast,$
nskyfound,skyfound,skyfoundfitres,skyfoundnpt,ngrid,bestsky,nbestsky
;
endif ; iproblem eq 0 ; added RJT JUL00
;
; find jjhistfollow, the starting sample from which to find the illumination
; history on the next value of iplat
;
; following commented RJT JUL00
;ijjh=nsf-1-nlookback+1 ; for nlookback=1 it should be start of plateau
                       ; just stored = nsf-1
;
ijjh=nsftry-1-nlookback+1 ;  added RJT JUL00
if(ijjh lt 0)then ijjh=0
;
;jjhistfollow=from(skyfoundplat(kp,ijjh)) ; commented RJT JUL00
;
jjhistfollow=from(skytryplat(kp,ijjh)) ; added RJT JUL00
;
endif ; finep(iplat) eq 1
;
endif ; ifoundglitch eq 0
;
; save solution as a simple sequence in i (to deal with slews with various
; options specified by ISLEWMETHOD in the FINDILLHIST routine, and as input to
; the DETINIT routine. This is done irrespective of whether the glitchtrap
; came into operation or not for slews since the glitch must be bridged
; for slews.
;
iskypass=igood+from(iplat)
skypass(iskypass)=skyguess ; fill points on the plateau including slews
;                            irrespective of whether they are glitchy or not
;
if(finep(iplat) eq 1)then skypass(from(iplat):to(iplat))=skyguess 
; can fill all points on a fine pointing
;
endif ; nigood ge 1
;
goto,c444
;
c555:
;
; in subsequent iterations a common solution for BESTSKY from the previous
; iteration is used for all fine pointing plateaus. Chi may be nominally 
; worsened, due to the application of this redundancy constraint, especially 
; if the solution for SIGNAL0, SIGNALSHOOT0, SILLUM0 (or input values if 
; DETINIT was not 
; called) are not accurate. Chi will also be worsened if there are other
; effects not covered by the detector model such as long term responsivity
; drifts. However, for a perfect detector model with no long term drifts and
; a determination of SIGNAL0 SIGNALSHOOT0 SILLUM0 to well within the noise 
; uncertainty,
; chi should be improved. No glitchtrap is set for these subsequent
; iterations, though any revised plateau structure (virtual slews) set up by 
; the glitch trap in iteration 0 is used.
;
if(niter gt 1)then begin
;
for iter=1,niter-1 do begin
;
; update complete illumination history ILLHIST and find improved BESTSKY
; using estimate of drift-corrected 
; photometry from previous iteration using input ISLEWMETHOD for all points
; including those on slew and nskyfound,skyfound,skyfoundfitres,skyfoundnpt
; and skypass (though this should not be needed for islewmethod=4)
; from previous iteration. 
;
;
; the following call should not make a difference! commented out RJT JUL00
;findbestsky,ifinemethod,kfineall,$
;nskyfound,skyfound,skyfoundfitres,skyfoundnpt,ngrid,bestsky,nbestsky
;
bestskysave=bestsky ; for detinit
;
kk=ipt2
jj=ipt1
;
findillhist,islewmethod,skydummy,$
from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
ngrid,yk,zk,bestsky,jj,kk,pident,illhist,ipix,ierr
;
; recalculate detector initial state
;
idetinit=-1
;
c3222:
;
idetinit=idetinit+1
;
if(ndetinit gt 0)then begin
;
if(idetinit gt 0)then begin
;
detinit,beta,tau,signal0,signalshoot0,sillum0,nplatinit,$
ipt1,iptend1,fromorig,toorig,keyporig,vigporig,$
t,signal,rms,itrans,mask,keypoint,ech,$
bestskysave,skypass,fitprec,imodel,silmin,silmax,isquare
;
if(iverbose gt -1)then $
print,'subiter',byte(idetinit),$
' : sig0,sigshoot0,sill0 =',float(signal0),float(signalshoot0),$
float(sillum0)
;
endif ; idetinit gt 0
;
endif ; ndetinit gt 0
;
; reset nskyfound,skyfound,skyfoundfitres,skyfoundnpt,skypass etc
;
; updated JUL00 RJT to include nskytry,skytryplat
drouinit,iplatprevarray,iplatprevprevarray,jjhistfollowarray,$
illhistf,resphistf,resphistslowf,resphistshootf,$
kfinelast,jjhistfollow,iplatprev,iplatprevprev,$
iplatlastfine,tlastfine,nsolve,ipt1,nskytry,skytryplat,$
bestsky,nbestsky,t,ngrid,nplat
;
; find predicted total, slow and fast detector responses 
; RESPHIST, RESPHISTSLOW, RESPHISTSHOOT 
;
tcalc=t(ipt1:ipt2)-t(ipt1)+dtime
sillumcalc=illhist(ipt1:ipt2)
if(imodel eq 1)then foukes,beta,tau,signal0,sillum0,sillumcalc,tcalc,resphistf
if(imodel eq 2)then expfun,beta,tau,signal0,sillum0,sillumcalc,tcalc,resphistf
if(imodel eq 3)then expshoot,beta,tau,signal0,signalshoot0,sillum0,$
sillumcalc,tcalc,resphistf,resphistslowf,resphistshootf
;
resphist=t
resphist(*)=0.0
resphistslow=resphist
resphistshoot=resphist
resphist(ipt1:ipt2)=resphistf(*)
if(imodel eq 3)then begin
resphistslow(ipt1:ipt2)=resphistslowf(*)
resphistshoot(ipt1:ipt2)=resphistshootf(*)
endif ; imodel eq 3
resphistchi=resphist(ipt1+igoodall)
findfitresult,signalchi,rmschi,resphistchi,fitresult,isquare
chi=fitresult
;
if(iverbose gt -1 and idetinit eq 0)then begin
print,'DRIFTCORR: major iteration with ipix,iter,chi = ',$
byte(ipix),byte(iter-1),float(chi)
endif ; iverbose gt -1
;
iplat=iplat1-1L
;
c4444:
;
;for iplat=iplat1,iplat2 do begin
;
; revert to start of timeline to find a new estimate of detector starting state
;
if(iplat ge 0)then begin ; here iplat refers to the last plateau processed
if(to(iplat) ge iptend1 and idetinit lt ndetinit)then goto,c3222
endif ; iplat ge 0
;
if(iplat ge iplat2)then goto, c5555
;
iplat=iplat+1
;
platskyuse(iplat)=0 ; needed in case glitchrecover upset the indexing
jjhistfollowarray(iplat)=jjhistfollow
iplatprevarray(iplat)=iplatprev
iplatprevprevarray(iplat)=iplatprevprev
;
ninplat=nplateau(iplat)
itransplat=itrans(from(iplat):to(iplat))
mplat=mask(from(iplat):to(iplat))
igood=where(mplat eq 0 and itransplat eq 1,nigood)
;
if(nigood ge 1)then begin ; there are good data on this plateau:
;
nsolve=nsolve+1
kk=from(iplat)-1 ; identifier of last destructive read of previous plateau
;
if(nsolve gt 0)then begin ; we are not at first plateau containing a solution
;                           so must calculate illumination history
;
if(finep(iplatprev) eq 0)then begin
;
tplat=t(from(iplat):to(iplat))-t(kk) ; timed from last point of previous
;    plateau independent of whether this last point was a good and transmitted 
;
if(imodel eq 1 or imodel eq 2)then signalprev=resphist(kk) ; total=long resp
if(imodel eq 3)then begin
signalprev=resphistslow(kk) ; slow
signalprevshoot=resphistshoot(kk) ; fast
endif ; imodel eq 3
;
sillumprev=illhist(kk)
;
endif ; finep(iplatprev) eq 0
;
if(finep(iplatprev) eq 1)then begin
;
tplat=t(from(iplat):to(iplat))-t(kk) ; timed from last point of previous
;    plateau independent of whether this last point was a good and transmitted 
;
if(imodel eq 1 or imodel eq 2)then signalprev=resphist(kk) ; total=slow
if(imodel eq 3)then begin
signalprev=resphistslow(kk) ; slow
signalprevshoot=resphistshoot(kk) ; fast
endif ; imodel eq 3
;
sillumprev=illhist(kk)
;
endif ; finep(iplatprev) eq 1
;
endif ; nsolve gt 0
;
if(nsolve le 0)then begin
;
signalprev=signal0 ; slow response (imodel=3) or total response (imodel=1,2)
signalprevshoot=signalshoot0 ; fast response (imodel=3)
sillumprev=sillum0
tplat=t(from(iplat):to(iplat))-t(from(iplat))+dtime 
;
endif ; nsolve le 0
;
sigplat=signal(from(iplat):to(iplat))
rmsplat=rms(from(iplat):to(iplat))
tplat=tplat(igood)
mplat=mplat(igood)
sigplat=sigplat(igood)
rmsplat=rmsplat(igood)
k=keyp(iplat) ; pointing direction on P32 grid
;
silminfast=max([bestsky(k)/3.0,silmin]) ; the tolerance has to be large 
;        if we are to re-enter the routine to optimise beta on the sky.
silmaxfast=min([bestsky(k)*3.0,silmax])
silminfast=silmin
silmaxfast=silmax
if(bestsky(k) eq 0.0)then begin ; can be encountered on slews
silminfast=silmin
silmaxfast=silmax
endif ; bestsky(k) eq 0.0
;
solveillum1,beta,tau,signalprev,signalprevshoot,sillumprev,sillumplat,tplat,$
sigplat,rmsplat,fitprec,imodel,silminfast,silmaxfast,isquare,fitresult,$
modelfit,ierr
;
iplatprevprev=iplatprev
iplatprev=iplat ; only set when a solution is found (either for slew
;                 or for fine pointing
;
skyguess=sillumplat/vigp(iplat)
;
; *** added JUL00 RJT
delta=median(rmsplat)/100.0
delta=double(delta)
;
iproblem=0
if(sillumplat le silmin+delta and nsolve gt 0)then iproblem=1
if(sillumplat ge silmax-delta and nsolve gt 0)then iproblem=1
; note - always use same silmin,silmax as this is needed by FINDILLHIST 
iproblem=0 ; this admits all solutions into FINDBESTSKY, including
           ; those which are running into silmin, silmax in SOLVEILLUM1
           ; if this statement commented we only uss solutions found
           ; within the SILLMIN SILLMAX range.
;
; *** end added JUL00 RJT
;
if(iverbose gt 0)then $
print,iplat,from(iplat),to(iplat),byte(nigood),byte(keyp(iplat)),$
byte(finep(iplat)),float(median(sigplat)),float(skyguess),$
float(signalprev),float(sillumprev)
;
; update arrays of previous results if on fine pointing and if we have found
; a valid solution from SOLVEILLUM1
;
if(finep(iplat) eq 1)then begin
;
if(iproblem eq 0)then begin ; *** added JUL00 RJT
;
kp=keyp(iplat) ; natural grid 1D pointing identifier
nskyfound(kp)=nskyfound(kp)+1
nsf=nskyfound(kp)
if(nsf gt ndimsky)then begin
print,'DRIFTCORR error: array dimension SKYFOUND too small (in niter > 1)'
ierr=1
;print,'iplat,kp,nsf = ',iplat,kp,nsf
goto, c999
endif ; nsf gt ndimsky
platskyuse(iplat)=1
skyfoundplat(kp,nsf-1)=iplat
skyfound(kp,nsf-1)=skyguess
skyfoundfitres(kp,nsf-1)=fitresult 
skyfoundnpt(kp,nsf-1)=nigood ; number of data points contributing to fit
;
endif ; iproblem eq 0 ; *** added JUL00 RJT
;
endif ; finep(iplat) eq 1
;
; save solution as a simple sequence in i (to deal with slews with various
; options specified by ISLEWMETHOD in the FINDILLHIST routine.
;
iskypass=igood+from(iplat)
skypass(iskypass)=skyguess ; fill points on the plateau including slews
;                            irrespective of whether they are glitchy or not
;
if(finep(iplat) eq 1)then skypass(from(iplat):to(iplat))=skyguess 
; can fill all points on a fine pointing
;
endif ; nigood
;
goto,c4444
;
c5555:
;
;endfor ; iplat
;
endfor ; iter
;
endif ; niter
;
; here calculate products FITRESULT MODELFIT RESPCORR by running 
; FINDILLUMHIST and (imodel eq 1) FOUKES again.
;
findbestsky,ifinemethod,kfineall,nskyfound,$
skyfound,skyfoundfitres,skyfoundnpt,ngrid,bestsky,nbestsky
;
jj=ipt1
kk=ipt2
findillhist,islewmethod,skydummy,$
from,to,t,vig,keypoint,fpointwk,ytrue,ztrue,skypass,ech,$
ngrid,yk,zk,bestsky,jj,kk,pident,illhist,ipix,ierr
;
tcalc=t(ipt1:ipt2)-t(ipt1)+dtime
sillumcalc=illhist(ipt1:ipt2)
;
if(imodel eq 1)then foukes,beta,tau,signal0,sillum0,sillumcalc,tcalc,resphistf
if(imodel eq 2)then expfun,beta,tau,signal0,sillum0,sillumcalc,tcalc,resphistf
if(imodel eq 3)then expshoot,beta,tau,signal0,signalshoot0,sillum0,sillumcalc,$
tcalc,resphistf,resphistslowf,resphistshootf
;
modelfit=t
modelfit(*)=0.0
modelfitshoot=modelfit
modelfitslow=modelfit
modelfit(ipt1:ipt2)=resphistf(*)
if(imodel eq 3)then begin
modelfitshoot(ipt1:ipt2)=resphistshootf(*)
modelfitslow(ipt1:ipt2)=resphistslowf(*)
endif ; imodel eq 3
modelfitchi=modelfit(ipt1+igoodall)
findfitresult,signalchi,rmschi,modelfitchi,fitresult,isquare
chi=fitresult
if(iverbose gt -1)then $
print,'DRIFTCORR: major iteration with ipix,iter,chi = ',$
byte(ipix),byte(iter-1),float(chi)
;
; find respcorr = illumination/response calculated 
; from the vignetting and the model for the input ipt1:ipt2 range. Mask
; points (MASKOUT array) where SOLVEILLUM1 gave an out of bounds result
; for points with MASK=0 and ITRANS=1 on input. 
;
respcorr=t
respcorr(*)=0.0 ; for safety so that errors in assignment sample list in 
;                 this and calling program will quickly be seen
respcorr(ipt1:ipt2)=1.0
;
delta=median(rmsplat)/100.0
delta=double(delta)
min_modelfit=0.0
min_skypass=silmin+delta
; modelfit dim. = full time
ii=where(skypass(ipt1:ipt2)*vig(ipt1:ipt2) le min_skypass and $
mask(ipt1:ipt2) eq 0 and itrans(ipt1:ipt2) eq 1,nii)
;
if(nii gt 0)then begin
if(iverbose gt -1)then begin
print,'DRIFTCORR: model detector response masked for',nii,' samples'
endif ; iverbose gt -1
maskout(ipt1+ii)=1
endif ; nii gt 0
;
ii=where(modelfit(ipt1:ipt2) gt min_modelfit,nii) ; avoid zeros
if(nii gt 0)then respcorr(ipt1+ii)=illhist(ipt1+ii)/modelfit(ipt1+ii)
if(nii le 0)then begin
print,'DRIFTCORR: warning: no restored samples above threshold of',min_modelfit
endif ; nii le 0
;
c999:
;
; restore plateau dependent quantities to their original values to be 
; consistent with the unaltered FPOINT
;
if(tplatback gt 0.0 or tplatforward gt 0.0)then begin
nplat=nplatorig
nplateau=nplateauorig
to=toorig
from=fromorig
vigp=vigporig
keyp=keyporig
finep=fineporig
pident=pidentorig
iplat1=iplat1orig
iplat2=iplat2orig
endif ; tplatback gt 0.0 or tplatforward gt 0.0
;
zerobias=zerobias_in ; reset detector parameter to input value
t=t+tsubtract ; restore t array to input values
;
return
end






