pro expand32,iflag
;
; creates arrays in common block E32 which span whole data
;   measured on the satellite, including data not transmitted.
;
; modified 23/02/01 to include erpid array
; modified 28/02/01 (CG) to initialize erpid with rpid in case of no datared
;
; needed for drift reconstruction taking account of illumination
; when telemetry not being transmitted, and for simulation (eg glitch
; tails outside of telemetry transmission windows. EXPAND32 can be
; applied just after SIGCREATE and after the chopper correction
; and before relativise for use in the 
; simulation with iflag=0 (puts S_IN into ES array) and/or just prior 
; to DRIFTCORR32 for use in the drift correction (puts S into ES array). 
; 
; input from command line
;  IFLAG = 0 for S_in to ES and = 1 for S to ES
;
; input from common block SIGNAL:
;  s_in(npix,nptin) raw signal
;  s(npix,nptin) ; relativised raw signal
;  badin(npix,nptin) ; 0 = good
;  bad(npix,nptin)
; input from common PINFO: FPT(nptin) = 1 on fine pointing = 0 otherwise
; input from common block PLOTTER
;  chop(nptin) (needed to check for change of chopper position
;               at end of a telemetry break interval which would
;               affect the pointing interpolation through the break).
;               Remember chop has some jitter at the arcsec level.
;  yrel(npix,nptin)  
;  zrel(npix,nptin)
;  rpid(2,nptin)
; input from common block TIMER
;  time(nptin)
;  dtime = read interval (note reads with reset last slightly longer)
;  complete : < 1 if not all data transmitted
; input from common block STEER1
;  npix
; input from common block STEERIN
;  nptin
;  ninramp = number of reads in ramp
;
; output to common block e32:
;  es(npix,neptin) (copied from either s_in or s with points
;                   with no telemetry filled out with zeros)
;  ireduce(neptin) = 0 for transmitted =1 otherwise  
;  eytrue(npix,neptin) with linear interpolation for pointing
;  eztrue(npix,neptin) ... where no data 
;  etime(neptin)
;  estep(neptin) (from plotter: chopper position identifier (1 to 13 for C100,
;                               1 to 7 for C200 array) 
;  echop(neptin) (from chop: assumes chopper changes just 
;                 before telemetry resumed)
;  ebad(npix,neptin) (from bad: set to 0 when there is no telemetry)
;  neptin: number of samplings by detector >= nptin=number transmitted 
;  fpoint(neptin) = 1 if ERDTOMAP OTF set =0 otherwise
;  erpid(2,neptin)
;
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_signal
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
@common_plotter
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_steerin
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_steer1
;common timer,time,dtime,complete
@common_timer
;common e32,es,ireduce,eytrue,eztrue,etime,fpoint,echop,estep,ebad,neptin,erpid
@common_e32
@common_pinfo
; common pinfo,fpt
;
xcomplete=0.9
;
if(complete gt xcomplete)then begin ; we just copy data to E32 common block
;
print,'EXPAND32: simple copy of s_in array into E32 common block ...'
neptin=nptin
if(iflag eq 0)then begin
es=s_in
print,'EXPAND32: simple copy of s_in array into E32 common block ...'
endif
if(iflag eq 1)then begin
es=s
print,'EXPAND32: simple copy of s array into E32 common block ...'
endif
ireduce=lonarr(neptin)
eytrue=ytrue
eztrue=ztrue
etime=time
fpoint=fpt
echop=chop
estep=step ; bug caught 230201
;estep=ireduce
erpid=rpid  ; added by CG for initialization
ebad=bad
;
endif
;
if(complete le xcomplete)then begin ; we must add gaps in data transmission
;
if(iflag eq 0)then print,'EXPAND32: ',$
'expanding s_in array into E32 common block ...'
if(iflag eq 1)then print,'EXPAND32: ',$
'expanding s array into E32 common block ...'
difftime=time
difftime(0)=dtime
difftime(1:nptin-1)=time(1:nptin-1)-time(0:nptin-2)
;plot,difftime
ibreak=where(difftime gt 1.5*dtime,nbreak)
print,'EXPAND32: number of breaks in telemetry = ',nbreak
xnbreak=fltarr(nbreak)
;
for i=0,nbreak-1 do begin
;
; remember that signal(i) is volts(i)-volts(i-1)/integration time
;
ii1=ibreak(i) ; input read identifier of first read following break
tstop=time(ii1) ; time of last read before break
tstart=time(ii1-1) ; time of first read after break
ninbreak=round((tstop-tstart)/dtime) - 1 ; number of reads in break
xnbreak(i)=ninbreak
;if(i le 10)then print,i,tstart,tstop,dtime,xnbreak(i)
;
endfor ; i
;
ninbreak=LONG(median(xnbreak)) ; The gap after the occasional chopper plateau change
                         ; adds time for which there is no read. In
                         ; reality there is a constant number of reads
                         ; in each telemetry gap
neptin=nptin+nbreak*ninbreak
print,'EXPAND32: number of non-transmitted points in each break = ',ninbreak
print,'EXPAND32: total number of transmitted and lost points    = ',neptin
es=dblarr(npix,neptin)
eytrue=es
eztrue=eytrue
etime=dblarr(neptin)
echop=etime
fpoint=etime
ireduce=lonarr(neptin)
estep=ireduce
ebad=lonarr(npix,neptin)
erpid=lonarr(2,neptin)
;
; real data before first break
;
for ipix=0,npix-1 do begin
if(iflag eq 0)then es(ipix,0:ibreak(0)-1)=s_in(ipix,0:ibreak(0)-1)
if(iflag eq 1)then es(ipix,0:ibreak(0)-1)=s(ipix,0:ibreak(0)-1)
eytrue(ipix,0:ibreak(0)-1)=ytrue(ipix,0:ibreak(0)-1)
eztrue(ipix,0:ibreak(0)-1)=ztrue(ipix,0:ibreak(0)-1)
ebad(ipix,0:ibreak(0)-1)=bad(ipix,0:ibreak(0)-1)
endfor ; ipix
etime(0:ibreak(0)-1)=time(0:ibreak(0)-1)
fpoint(0:ibreak(0)-1)=fpt(0:ibreak(0)-1)
echop(0:ibreak(0)-1)=chop(0:ibreak(0)-1)
estep(0:ibreak(0)-1)=step(0:ibreak(0)-1)
erpid(0,0:ibreak(0)-1)=rpid(0,0:ibreak(0)-1)
erpid(1,0:ibreak(0)-1)=rpid(1,0:ibreak(0)-1)
ireduce(0:ibreak(0)-1)=0
;
ib=ibreak(0)-1 ; last good read (output counter) before break
;
for i=0,nbreak-1 do begin
;
ii1=ibreak(i) ; input read identifier of first read following break
;ii1= ii1-1 ; (virtual) input read identifier of last read of break
tstart=time(ii1-1)+dtime ; time of first virtual read in beak
tstop=tstart+float(ninbreak-1)*dtime
;
; write "data" during break
;
; first see whether or not chopper changes positon between last
; good read of previous sequence and first good read of next good
; sequence: Find chop(ii1)-chop(ii1-1)
;
dchop=chop(ii1)-chop(ii1-1)
dfp=fpt(ii1)-fpt(ii1-1)
;
for ipix=0,npix-1 do begin
es(ipix,ib+1:ib+ninbreak)=0.0
y1=ytrue(ipix,ii1-1)
y2=ytrue(ipix,ii1)
z1=ztrue(ipix,ii1-1)
z2=ztrue(ipix,ii1)
if(abs(dchop) gt 0.5*dcy)then y2=y2-dchop ; assume chopper moves at
;                                           end of period of no transmission 
; note the following cannont take into account the exact point at which a
; slew may start, nor does it take into account the extra time for 
; chopper synchronisation (both of which are however expected to be
; small effects for fast readouts. 
;
if(ipix eq 0)then begin ; for diagnostics
y10=y1
y20=y2
endif
;
eytrue(ipix,ib+1:ib+ninbreak)=$
y1+(y2-y1)*(1.0+findgen(ninbreak))/float(ninbreak+1)
eztrue(ipix,ib+1:ib+ninbreak)=$
z1+(z2-z1)*(1.0+findgen(ninbreak))/float(ninbreak+1)
;ebad(ipix,ib+1:ib+ninbreak)=0 ; comment out since initialized to 0
endfor ; ipix
etime(ib+1:ib+ninbreak)=tstart+findgen(ninbreak)*dtime
echop(ib+1:ib+ninbreak)=chop(ii1-1) ; set to value of last good point
estep(ib+1:ib+ninbreak)=step(ii1-1) ; set to value of last good point
erpid(0,ib+1:ib+ninbreak)=rpid(0,ii1-1) ; set to value of last good point
erpid(1,ib+1:ib+ninbreak)=rpid(1,ii1-1) ; set to value of last good point
; if pointing flag changes during transmission break assume all
; transmission break is taken on the fly.
if(fpt(ii1-1) eq fpt(ii1))then fpoint(ib+1:ib+ninbreak)=fpt(ii1-1)
if(fpt(ii1-1) ne fpt(ii1))then fpoint(ib+1:ib+ninbreak)=0
ireduce(ib+1:ib+ninbreak)=1
;
;if(abs(y10-y20) gt 1.0)then begin ; test print
;print,ib+1,ib+ninbreak,ii1
;print,y10,y20,dchop
;print,eytrue(0,ib+1:ib+ninbreak)
;endif ; i le 8
;
ib=ib+ninbreak ; update ib to last data before end of break (output counter)
;
; write good data up to next break or until the end of the dataset
; following last break
;
if(i eq nbreak-1)then begin ; write good data until the end of the dataset
;
nremain=nptin-ii1
for ipix=0,npix-1 do begin
if(iflag eq 0)then es(ipix,ib+1:ib+nremain)=s_in(ipix,ii1:nptin-1)
if(iflag eq 1)then es(ipix,ib+1:ib+nremain)=s(ipix,ii1:nptin-1)
eytrue(ipix,ib+1:ib+nremain)=ytrue(ipix,ii1:nptin-1)
eztrue(ipix,ib+1:ib+nremain)=ztrue(ipix,ii1:nptin-1)
ebad(ipix,ib+1:ib+nremain)=bad(ipix,ii1:nptin-1)
endfor ; ipix
etime(ib+1:ib+nremain)=time(ii1:nptin-1)
fpoint(ib+1:ib+nremain)=fpt(ii1:nptin-1)
echop(ib+1:ib+nremain)=chop(ii1:nptin-1)
estep(ib+1:ib+nremain)=step(ii1:nptin-1)
erpid(0,ib+1:ib+nremain)=rpid(0,ii1:nptin-1)
erpid(1,ib+1:ib+nremain)=rpid(1,ii1:nptin-1)
ireduce(ib+1:ib+nremain)=0
;if(i le 8)then begin ; test print
;print,ib+1,ib+nremain,ii1,nptin-1
;print,ytrue(0,ii1:nptin-1)
;print,eytrue(0,ib+1,ib+nremain)
;endif ; i le 8
;
ib=ib+nremain ; last data point written
;
endif ; i eq nbreak-1
;
if(i lt nbreak-1)then begin ; write good data until the next break 
; and update ib
ii2=ibreak(i+1)-1 ; input read identifier of last read in period of good data
                  ; following break 
nremain=ii2-ii1+1
for ipix=0,npix-1 do begin
if(iflag eq 0)then es(ipix,ib+1:ib+nremain)=s_in(ipix,ii1:ii2)
if(iflag eq 1)then es(ipix,ib+1:ib+nremain)=s(ipix,ii1:ii2)
eytrue(ipix,ib+1:ib+nremain)=ytrue(ipix,ii1:ii2)
eztrue(ipix,ib+1:ib+nremain)=ztrue(ipix,ii1:ii2)
ebad(ipix,ib+1:ib+nremain)=bad(ipix,ii1:ii2)
endfor ; ipix
etime(ib+1:ib+nremain)=time(ii1:ii2)
fpoint(ib+1:ib+nremain)=fpt(ii1:ii2)
echop(ib+1:ib+nremain)=chop(ii1:ii2)
estep(ib+1:ib+nremain)=step(ii1:ii2)
erpid(0,ib+1:ib+nremain)=rpid(0,ii1:ii2)
erpid(1,ib+1:ib+nremain)=rpid(1,ii1:ii2)
ireduce(ib+1:ib+nremain)=0
endif ; i lt nbreak-1
;if(i le 6)then print,ii1,ii2,ib,nremain
;if(i le 6)then print,eytrue(0,ib+1:ib+nremain)
;if(i le 6)then print,ytrue(0,ii1:ii2)
ib=ib+nremain
;
endfor ; i
;
endif ; complete lt xcomplete
;
return
end
