pro expfun,beta,tau,signalprev,sillumprev,sillum,t,signal
;
;
; calculates signal response SIGNAL (real array or real single number) 
; to set of illuminations or single illumination SILLUM, at a set of times or
; single time T since an illumination change from SILLUMPREV to SILLUM at time
; zero. SIGNALPREV is input as the best estimate of signal response
; of the detector at time zero minus a small fraction of the read interval
; (ie at time 0 but before the flux step). T represents the time at the end of
; each elementary read interval.
; 
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         GAMMAEXP   (common MOREPAR) detector model constant. For GAMMAEXP=1
;                    the Foukes function is exactly reproduced. For
;                    GAMMAEXP < 1 the detector changes more rapidly at first, 
;                    thus mimicing the observed tendency for short response
;                    timescales immediately after the flux step, during 
;                    (eg) P32 chopper plateaus of ca. 1/2 second, and long 
;                    timescales thereafter; (eg) on FCS plateaus of ca. 32s.
;                    scales thereafter 
;         ZEROBIAS   (common MOREPAR) illumination level about which noise 
;                    fluctuations 
;                    should have a symetrical relaxation times for positive
;                    and negative fluctuations. This is normally the estimated
;                    background level on the map - the minimum real sky
;                    illumination encountered
;         SIGNALPREV (single number in V/s): actual detector response at T(0)=0
;                     immediately prior to the initial flux step from 
;                     SILLUMPREV to SILLUM(0). Note this number is not normally
;                     actually measured due to the destructive read at T(0)=0 
;                     so has to be a model estimate. SIGNALPREV must be 
;                     positive.
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at T(0)=0. Must be positive.
;         SILLUM      (array in V/s): array of new 
;                      illuminations at time T. There are no restrictions
;                      on the values SILLUM can be take, so long as they are
;                      positive. Thus, SILLUM(0) need not differ from 
;                      SILLUMPREV, and any adjacent elements can be identical.
;         T           (array in s) time since flux step
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be the first read 
;                      interval.
;
; output: SIGNAL      (array in V/s) detector response at times in T since 
;                      initial illumination step from SILLUMPREV to SILLUM(0).
;                       
;
;print,'EXPFUN INPUT PARAMETERS beta,tau,signalprev,sillumprev'
;print,beta,tau,signalprev,sillumprev
;help,sillum
;print,sillum(0:10)
;help,t
;print,t(0:10)
;
@common_morepar
;common morepar,gammaexp,zerobias,betashoot,taushoot,gammaexpshoot,$
;betacoeffshoot,betaexpshoot,tauoffsetshoot,tauoffset,betaexp,betacoeff
;
n=n_elements(t)
if(n_elements(sillum) ne n)then $
print,'EXPFUN warning: n_elements of sillum,t = ',n_elements(sillum),n 
;print,'n=',n
signal=dblarr(n)
sf=signalprev ; detector response just prior to flux step at time zero
sillump=sillumprev ; illumination just prior to flux step at time zero
;
if(min(sillum) eq max(sillum) and n gt 1)then goto, c555 ; vector calculation
;
; point by point calculation (for arbitrary illumination sequences and for
; treatment of singularity
;
for i=0L,n-1 do begin ; i=0 corresponds to first signal
;
if(sillum(i) ne sillump or i eq 0)then begin ; we have changed illumination
;
if(i gt 0)then sf=signal(i-1)
s0=beta*(sillum(i)-sillump)+sf
sillump=sillum(i) ; update
b=s0-sillum(i)
em=abs(sillum(i)-zerobias)+zerobias
;e=(sillum(i)^gammaexp)/tau          
e=(em^gammaexp)/tau 
;           
ii=i-1 ; reference read for when illumination actually changed
if(i gt 0)then tchange=t(ii) ; assumes time zero for first flux change
if(i eq 0)then tchange=0.0
;
endif ; sillum(i) ne sillump
;
;print,'i,ii',i,ii
ee=(t(i)-tchange)*e
;
signal(i)=sillum(i)+b*exp(-ee)
;
endfor ; i
;
goto, c999
;
c555: ; FOUKESFAST STARTS (no loops)
;
s0=beta*(sillum(0)-sillump)+sf 
b=s0-sillum(0) ; scalar constants for plateau
em=abs(sillum(0)-zerobias)+zerobias          
e=(em^gammaexp)/tau 
;e=(sillum(0)^gammaexp)/tau 
tchange=0.0
ee=(t-tchange)*e ; vector
;
signal=sillum+b*exp(-ee)
;
c999:
;
return
end






