pro findsigillum0,beta,tau,imodel,keyp,vigp,from,t,illhist,$
bestsky,nbestsky,signal0,sillum0,ifound
;
; finds starting state of detector by simulating a single chopper sweep
; at first spacecraft raster pointing direction, using estimate 
; of illumination history in ILLHIST. Note that SIGNAL0
; is not an input parameter as this is set to BESTSKY at the end of the first
; chopper sweep.
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         IMODEL       = 1 for Foukes-Schubert
;         SILLUM0     (single number in V/s): input guess at illumination
;                     prior to the first data point. Must be positive.
;         DTIME       (common  TIMER) = non-destructive read interval
;         ARRAY       (common  STEERIN) = 'c100' or 'c200'
;         FROM        (integer array dimension no. of plateaus) 
;                     identifier of last point on a plateau
;         VIGP        (real array dimension no. of plateaus) 
;                     value of vignetting factor (normally <=1) on a plateau
;         KEYP        (integer array dimension no. of plateaus) 
;                     value of natural grid identifier on a plateau
;         T           (array in s) This array contains times for each read
;                     (including times where data is read on board but not
;                      transmitted since time=0 at start of observation.
;         ILLHIST      (1D array in V/s dimensioned as the whole of the 
;                      primitive expanded read stream). This is the
;                      illumination history (sky illumination * VIG).
;         BESTSKY      (1D array in V/s dimension number of directions on
;                      P32 natural grid). This is the illumination that each 
;                      point on the natural sky grid would receive in the
;                      absence of vignetting. 
;                      BESTSKY is left zero where there are no contributing 
;                      data points. 
;         NBESTSKY     (ID integer array) Total number of elementary 
;                      data samples contributing to the value for BESTSKY.
;
; output:
;         SIGNAL0     (single number in V/s): estimate of 
;                     (noiseless) detector response at time zero prior to the  
;                     1st data sample, calculated by following the detector
;                     response to ILLHIST through the first chopper sweep.
;         SILLUM0     (single number in V/s): illumination
;                     prior to the first data point, set to value of BESTSKY 
;                     at the end of the first chopper sweep.
;         IFOUND      = 1 if new values for SIGNAL0 SILLUM0 could be found
;                     = 0 otherwise. 
;
;common timer,time,dtime,complete
@common_timer
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;
ifound=0
;
kstart=keyp(0) ; pointing identifier of first data sample 
;                (start of chopper sweep)
if(array eq 'c100')then begin
nchop=13 
det=1
endif ; array eq 'c100'
;
if(array eq 'c200')then begin
nchop=7 
det=2
endif ; array eq 'c200'
;
kstop=kstart+nchop-1 ; pointing identifier of last plateau on sweep
;                      k always increases on first raster
;vigpix,det,xvig ; this is no good (detector pixel dependent)
;
if(bestsky(kstop) gt 0.0)then sillum0=vigp(kstop)*bestsky(kstop) 
; new estimate
if(bestsky(kstop) le 0.0)then begin
print,'FINDSIGILLUM0: aborting - no SKY information at end of chopper sweep'
goto, c999
endif ; 
;
; find identifier of last elementary read before returning to start of sweep
; we assume that plateau 0 is at start of sweep and plateau 1 is next
; chopper position.
;
if(keyp(0) eq keyp(1))then begin
print,'FINDSIGILLUM0: aborting - plateaus not in expected sequence'
goto, c999
endif ; keyp(0) eq keyp(1)
;
k2=from(nchop)-1 ; last sample of first chopper sweep
;
; simulate first chopper sweep using revised SILLLUM0 from above to find
; new SIGNAL0
;
illhtrial=illhist(0:k2)
tcalc=t(0:k2)-t(0)+dtime
;
; unless we start detector in equilibrium we risk
;                     detector being in a forbidden state
signal0=sillum0
;
if(imodel eq 1)then foukes,beta,tau,signal0,sillum0,illhtrial,tcalc,resphist
if(imodel eq 2)then expfun,beta,tau,signal0,sillum0,illhtrial,tcalc,resphist
signal0=resphist(k2)
sillum0=illhtrial(k2) 
;
ifound=1
;
print,'FINDSIGILLUM0: revised SIGNAL0,SILLUM0 = ',signal0,sillum0
;
c999:
;
return
end






