pro findtau,beta,tauout,signalprev,signalprevshoot,sillumprev,sillum,t,signal,$
rms,tauprec,imodel,taumin,taumax,isquare,fitresultout,modelfitout
;
;print,'FINDTAU: calling with beta,signalprev,sillumprev, ...'
;print,'... tauprec,imodel,taumin,taumax,isquare = '
;print,beta,signalprev,sillumprev,$
;tauprec,imodel,taumin,taumax,isquare
;
; find tauout giving best fit (with chi-squared-like 
; value FITRESULT and fit MODELFIT) to data following an fcs illumination step
; held in SIGNAL array for fixed input BETA, SIGNALPREV and SILLUMPREV
; and SILLUM.
; Routine is normally called from FINDBETATAUILLUM. FINDTAU uses linear arrays
; (ie for just one detector pixel data prepared in FINDBETATAUILLUM).
; 
; The analysis proceeds by finding best fit for fcs illumination for trial 
; values of tau, running SOLVEILLUM1 for each trial to produce a best
; value for SILLUM.
;
; input: 
;
;    SILLUM     (single real in V/s ): true illumination from FCS
;    SILLUMPREV (single real in V/s ): illumination immediately
;                                      prior to fcs illumination.
;    SIGNALPREV (single real in V/s ): signal immediately
;                                      prior to fcs illumination.
;    BETA       (single real)        : first detector constant
;    T          (array in s)         : time since fcs illumination at time 0
;    SIGNAL     (array in V/s) measured detector signal at times in T 
;                      since illumination step at time zero. All points must be
;                      positive 
;    RMS        (array in V/s) RMS uncertainty in SIGNAL
;    TAUPREC    (single real in V)  Precision to which TAUOUT is to be found
;                                   (set in FINDBETATAUILLUM)
;    TAUMIN,TAUMAX              min and max possible values for TAUOUT
;                               found by chop procedure here 
;                               (set by FINDBETATAUILLUM)    
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;
; output:
;
;         TAUOUT    (single real in V): best fit for second 
;                     detector model constant for the input
;                     combination of BETA, SIGNALPREV and SILLUMPREV.
;         FITRESULTOUT (Single real in V/s) goodness of fit 
;                     - lower is better - according to
;                     value of input parameter ISQUARE - see description
;                     of isquare, above)
;         MODELFITOUT (array in V/s): best fit model
;                      for the fcs plateau. 
;
;
if(t(0) le 0.0)then begin
print,'FINDTAU: aborting - time array does not start at small positive value' 
goto, c999
endif ; t(0) le 0.0
if(min(signal) le 0.0)then begin
print,'FINDTAU: aborting - SIGNAL has zero or negative value'
goto, c999
endif ; t(0) le 0.0
;
;
sillumarray=t
sillumarray(*)=sillum
;
; control parameters for the cut in tau in this  routine:
niter=1 ; number of iterations in this routine 
;         for diagnostics - set to a large number to control convergence
;         by TAUPREC. In fact, though iter is built in to the logic, we
;         ignore it to avoid all possibility of getting a too crude
;         sampling of the minimum.
ncut=5 ; must be odd and at least 5
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
;
; process each pixel
;
; start cut scheme in tau
;
; derived controlling parameters for chop in tau
;
xncut=float(ncut)
top=taumax
bot=taumin
dcut=(top-bot)/(xncut-1.0)
tautrial=bot+dcut*findgen(ncut)
;
;print,'bot,top,dcut,tautrial = ',bot,top,dcut,tautrial
;
tau=tautrial(0)
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumarray,t,$
signalmodel
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumarray,t,$
signalmodel
if(imodel eq 3)then expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,$
sillumarray,t,signalmodel

findfitresult,signal,rms,signalmodel,fitresult,isquare
;print,'FINDTAU: tau,fitresult = ',tau,fitresult
;
icutmin=0
fitresultmin=fitresult
fitresultout=fitresult
fitresultbegin=fitresult
tauout=tau
modelfitout=signalmodel ; best model fit to plateau
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
tau=tautrial(icut)
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumarray,t,$
signalmodel
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumarray,t,$
signalmodel
if(imodel eq 3)then expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,$
sillumarray,t,signalmodel
findfitresult,signal,rms,signalmodel,fitresult,isquare
;
;print,'FINDTAU: tau,fitresult = ',tau,fitresult
;
if(fitresult lt fitresultmin)then begin ; found new minimum
icutmin=icut
fitresultmin=fitresult
fitresultout=fitresultmin
tauout=tau
modelfitout=signalmodel ; best model fit
;print,'FINDTAU: beta,tauout,fitresultout = ',beta,tauout,fitresultout
endif ; fitresult lt fitresultmin
;
endfor ; icut
;
if(dcut le tauprec)then goto, c200
d=top-bot
d=d/reduce
d2=d/2.0
bot=tauout-d2
top=tauout+d2
bottmp=bot
toptmp=top
if(bot lt taumin)then begin
bottmp=taumin
toptmp=bottmp+d
endif ; bot
if(top gt taumax)then begin
toptmp=taumax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
tautrial=bot+dcut*findgen(ncut)
;
iter=iter+1
;print,'FINDTAU: iter,bot,top,tautrial ',iter,bot,top,tautrial
;
goto, c100
;
c200:
;
c999:
;
return
end











