pro findtauillum,beta,tauout,signalprev,sillumprev,sillumout,t,signal,rms,$
tauprec,fitprec,imodel,silmin,silmax,taumin,taumax,isquare,fitresultout,$
modelfitout
;
;print,'FINDTAUILLUM: calling with beta,signalprev,sillumprev, ...'
;print,'... tauprec,fitprec,imodel,silmin,silmax,taumin,taumax,isquare = '
;print,beta,signalprev,sillumprev,$
;tauprec,fitprec,imodel,silmin,silmax,taumin,taumax,isquare
;
; find sillumout,tauout combination giving best fit (with chi-squared-like 
; value FITRESULT and fit MODELFIT) to data following an fcs illumination step
; held in SIGNAL array for fixed input BETA, SIGNALPREV and SILLUMPREV.
; Routine is normally called from FINDBETATAUILLUM. FINDTAUILLUM uses linear 
; arrays (ie for just one detector pixel data prepared in FINDBETATAUILLUM).
; 
; The analysis proceeds by finding best fit for fcs illumination for trial 
; values of tau, running SOLVEILLUM1 for each trial to produce a best
; value for SILLUM.
;
; input: 
;
;    SILLUMPREV (single real in V/s ): illumination immediately
;                                      prior to fcs illumination.
;    SIGNALPREV (single real in V/s ): signal immediately
;                                      prior to fcs illumination.
;    BETA       (single real)        : first detector constant
;    T          (array in s)         : time since fcs illumination at time 0
;    SIGNAL     (array in V/s) measured detector signal at times in T 
;                      since illumination step at time zero. All points must be
;                      positive 
;    RMS        (array in V/s) RMS uncertainty in SIGNAL
;    TAUPREC    (single real in V)  Precision to which TAUOUT is to be found
;                                   (set in FINDBETATAUILLUM)
;    FITPREC    (dimensionless) gives the precision to which the best fit to
;                               SILLUMOUT is found by SOLVEILLUM1.
;                               The interval of search is reduced
;                               until it is below the mean of RMS*FITPREC.
;    SILMIN,SILMAX              min and max possible values for SILLUMOUT
;                               found by SOLVEILLUM1 (set by FINDBETATAUILLUM)
;    TAUMIN,TAUMAX              min and max possible values for TAUOUT
;                               found by chop procedure here 
;                               (set by FINDBETATAUILLUM)    
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;
; output:
;
;         TAUOUT    (single real in V): best fit for second 
;                     detector model constant for the input
;                     combination of BETA, SIGNALPREV and SILLUMPREV.
;         SILLUMOUT (single real in V/s): best 
;                     fit constant illumination from FCS for the input 
;                     combination of BETA, SIGNALPREV and SILLUMPREV.
;         FITRESULTOUT (Single real in V/s) goodness of fit 
;                     - lower is better - according to
;                     value of input parameter ISQUARE - see description
;                     of isquare, above)
;         MODELFITOUT (array in V/s): best fit model
;                      for the fcs plateau. 
;
;
if(t(0) le 0.0)then begin
print,'FINDTAUILLUM: aborting...
print,'... time array does not start at small positive value' 
goto, c999
endif ; t(0) le 0.0
if(min(signal) le 0.0)then begin
print,'FINDTAUILLUM: aborting - SIGNAL has zero or negative values'
goto, c999
endif ; t(0) le 0.0
;
; control parameters for the cut in tau in this  routine:
niter=1 ; number of iterations in this routine 
;         for diagnostics - set to a large number to control convergence
;         by TAUPREC. In fact, though iter is built in to the logic, we
;         ignore it to avoid all possibility of getting a too crude
;         sampling of the minimum.
ncut=5 ; must be odd and at least 5
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
irmswk=1 ; = 1 to tune fitprec in SOLVEILLUM1 call to the search interval
;              in tau (for speed)
;
;
; process each pixel
;
; start cut scheme in tau
; for each trial tau in cut scheme find best fit for illumination 
; SILLUM of fcs plateau by calling SOLVEILLUM1
;
; derived controlling parameters for chop in tau
;
xncut=float(ncut)
top=taumax
bot=taumin
dcut=(top-bot)/(xncut-1.0)
tautrial=bot+dcut*findgen(ncut)
if(irmswk eq 0)then fitprecwk=fitprec
if(irmswk eq 1)then begin
fitprecwk=fitprec*dcut/tauprec ;        it is unnecessry to have high precision
if(fitprecwk lt fitprec)then fitprecwk=fitprec
if(fitprecwk gt 0.5)then fitprecwk=0.5 ; fits where the model parameters are
;                                        far from optimum
if(fitprecwk lt 0.1)then fitprecwk=0.1
endif ; irmswk eq 1
;
;print,'bot,top,dcut,tautrial = ',bot,top,dcut,tautrial
;
; set up initial values for goodness of fit in search. First solveillum1
; call must use FITPREC rather than FITPRECWK
;
tau=tautrial(0)
;
solveillum1,beta,tau,signalprev,sillumprev,sillum,t,signal,rms,$
fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;print,'FINDTAUILLUM: tau,fitresult = ',tau,fitresult
;
icutmin=0
fitresultmin=fitresult
fitresultout=fitresult
fitresultbegin=fitresult
tauout=tau
sillumout=sillum
modelfitout=modelfit ; best model fit to plateau
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
tau=tautrial(icut)
;
solveillum1,beta,tau,signalprev,sillumprev,sillum,t,signal,rms,$
fitprecwk,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;print,'FINDTAUILLUM: tau,fitresult = ',tau,fitresult
;
if(fitresult lt fitresultmin)then begin ; found new minimum
icutmin=icut
fitresultmin=fitresult
fitresultout=fitresultmin
tauout=tau
sillumout=sillum
modelfitout=modelfit ; best model fit
;print,'FINDTAUILLUM: beta,tauout,fitresultout = ',beta,tauout,fitresultout
endif ; fitresult lt fitresultmin
;
endfor ; icut
;
if(dcut le tauprec)then goto, c200
d=top-bot
d=d/reduce
d2=d/2.0
bot=tauout-d2
top=tauout+d2
bottmp=bot
toptmp=top
if(bot lt taumin)then begin
bottmp=taumin
toptmp=bottmp+d
endif ; bot
if(top gt taumax)then begin
toptmp=taumax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
tautrial=bot+dcut*findgen(ncut)
if(irmswk eq 0)then fitprecwk=fitprec
if(irmswk eq 1)then begin
fitprecwk=fitprec*dcut/tauprec ;        it is unnecessry to have high precision
if(fitprecwk lt fitprec)then fitprecwk=fitprec
if(fitprecwk gt 0.5)then fitprecwk=0.5 ; fits where the model parameters are
;                                        far from optimum
if(fitprecwk lt 0.1)then fitprecwk=0.1
endif ; irmswk eq 1
;
; always find a new minimum for each value of dcut. Because the fit result is
; decreased not only due to the approach to the correct tau, but also by
; the decrease in FITPRECWK in each iteration:
if(irmswk eq 1)then fitresultmin=fitresultbegin 
;
iter=iter+1
;print,'FINDTAUILLUM: iter,bot,top,tautrial ',iter,bot,top,tautrial
;
goto, c100
;
c200:
;
c999:
;
return
end











