;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GIVE_DARKCUR_ORB
;
; PURPOSE:
;	Returns orbital dependent dark current value
;
; CATEGORY:
;	PIA - algorithms
;
; CALLING SEQUENCE:
;	GIVE_DARKCUR_ORB, tmpdata, DARKCUR, DARKU 
;	
;INPUTS
;	A PIA data structure (ERD, SRD or SCP)
;	
;OUTPUTS 
;	DARKCUR:		The dark current array
;	DARKU:			DARKCUR uncertainty
;
; COMMON BLOCKS:
;	phtdarkorb
;
; MODIFICATION HISTORY:
;	Written by  Carlos GABRIEL (ESA/VILSPA)			November 1999
;-
PRO give_darkcur_orb, tmpdata,  DARKCUR, DARKU 

@phtdarkorb

;check if dark current cal-G files have been read 
IF N_ELEMENTS(C1DARKORB) EQ 0 THEN read_darkorb

IF N_ELEMENTS(TIMEDEP) EQ 0 THEN timedep = DARKTDEP
; if mode is not selected via keyword then use current default values
; as stored in common phtdarkorb

PRO_MEAS=1   ; here always pro measurement


; Obtain the seconds from begin of observation by subtracting from the ITKs
; the START ITK of the first measurement and multiplying with the conversion 
; factor - After this normalize to orbital position using the orbit offset 
; portion at the begin 
GIVE_CS, tmpdata, cst, record, cst_conv
fct = FXPAR(tmpdata.hdr, 'TREFITKU', count=count)
IF count EQ 0 THEN BEGIN
    PRINT, 'Warning:  TREFITKU not found on header'
    fct = 6.1035156e-05
ENDIF


;determine orbital position offset at reference time from header if available 
; or alternatively using info from ALLPERIGEE file
;First read the revolution number from ADMN (needed in any case)
  rev = STRMID(tmpdata.admn,4,3)

; determine the orbital position (check the keyword TREFPHA1 in the header)
 orb_off = FXPAR(tmpdata.hdr, 'TREFPHA1', count=count)

IF count EQ 0 THEN BEGIN 
    orb_off = set_orbpos(rev, tmpdata.unit)
    isnot_olpv7 = 1 
  IF KEYWORD_SET(VERBOSE) THEN $
      print,'Computing orbital position from ALLPERIGEE internal file'
ENDIF ELSE BEGIN 
  orb_off = orb_off - rev
  isnot_olpv7 = 0
  IF KEYWORD_SET(VERBOSE) THEN print,'Orbital position read from header '
ENDELSE  


;now compute the orbital position for any time depending on the mode
; 
   timall = tmpdata.tkey 	
   times = timall

; just for the case in the future the other possibilities are introduced
pro_raster=0
pro_aot=0
   CASE 1 OF 
     PRO_RASTER: BEGIN 
        dorb_mode = 'PER_RASTER'
        get_rastersteps, tmpdata.rpid, raststeps
        FOR irast = 1,max(raststeps) DO BEGIN
          ind= WHERE(raststeps EQ irast)
          times(ind) = AVG([timall(ind)]) - cst.ikst(0)
        ENDFOR
     END   
     PRO_MEAS: BEGIN 
        dorb_mode = 'PER_MEASUREMENT'
        times = times*0.+AVG([timall]) - cst.ikst(0) 
     END
     PRO_AOT: BEGIN
        time_mid = (cst.ikst(0)*1.+ cst.ikst(N_ELEMENTS(cst.ikst)-1)*1.)/2. 
        times = times*0. + (time_mid - cst.ikst(record)) * isnot_olpv7
        dorb_mode = 'PER_AOT'
     END
     ELSE: BEGIN 
        IF KEYWORD_SET(scp) THEN dorb_mode = 'PER_CHOPPERPL.' ELSE $
        dorb_mode = 'PER_SIGNAL'
        times = times - cst.ikst(0)
     END
   ENDCASE  
     


;IF PRO_AOT THEN orbpos=orb_off  $
; ELSE BEGIN
    times = times * fct
    iso_orbit = 23.+56./60    ; duration of ISO orbit in hours  
    orbpos = orb_off + times / 3600. / iso_orbit 
;ENDELSE
;	print,orbpos

;determine detector and number of pixels
det = tmpdata.det
npix = pixel_det(tmpdata.det)

darkcur = FLTARR(npix)

CASE det of 
     'P1': dk = ppdarkorb(*,0,*)
     'P2': dk = ppdarkorb(*,1,*)
     'P3': dk = ppdarkorb(*,2,*)
     'C1': dk = c1darkorb
     'C2': dk = c2darkorb
     'SL': dk = sldarkorb
     ELSE: RETURN
ENDCASE

darku = darkcur
FOR ipix=0, npix-1 DO BEGIN
    darkcur(ipix) = interpol(dk(*,ipix,0),ORBPOS_DARK,orbpos(0))
    darku(ipix) = interpol(dk(*,ipix,1),ORBPOS_DARK,orbpos(0))
ENDFOR




RETURN
END
 
