pro mapstack,nmappix,mappix,mapbright,maprms,mapdisp,$
mapoffy,mapoffz,mapra,mapdec,mapnum,iweight1,iweight2,iotf,$
ibeam,beamflux,beamback
;
; makes 2D maps on the P32 natural grid in a two stage process:
;
; (1) running MAKEMAPMULTI to make one map for each detector pixel,
;     with IWEIGHT1 = 0 to give equal weight for all data from each det pixel 
;     and  IWEIGHT1 = 1 to weight according to (1/rms)^2 for each data sample.
; (2) averaging data from different detector pixels in each map pixel 
;     to give a single map, using:
;     IWEIGHT2 = 0 to give equal weight to each detector pixel
;     IWEIGHT2 = 1 to weight each detector pixel according to (1/rms)^2 
;                  (where rms is the predicted error in mean from propagation 
;                  of the rms array in each pixel from MAKEMAPMULTI) for the 
;                  detector pixel map exposures in a given map pixel.
;     IWEIGHT2 = 2 to weight each detector pixel according to (1/drms)^2 
;                  (where drms is the error in the mean from observed spread
;                   in data for a given map pixel and a given detector pixel
;                   from MAPMULTI) for the detector pixel map exposures in a 
;                   given map pixel.
;     IWEIGHT2 = 3 As for IWEIGHT2 = 2 but take median of each detector
;                  pixel for the brightness map.
;
; the parameters are:
;
; NMAPPIX,MAPPIX specify the detector pixels to be used in the map making 
; MAPSTACKMASK(npix,nygrid,nzgrid) specifies map-pixel/detector pixel 
;              combinations to be censored.  (common mapstk) 
; MMAPBRIGHT,MMAPRMS,MMAPDISP,MMAPOFFY,MMAPOFFZ,MMAPNUM are output maps
; (as defined in MAKEMAP)
;
; if IOTF = 1 only data on target are used
; if IOTF = 0 only data off target are used
; if IOTF = -1 all data are used
;
; if IBEAM = 1 then a beam map is made (of flux BEAMFLUX (Jy) and 
;              and background BEAMBACK (MJy/sterad) 
;
; For maps with high dynamic range, for which the drift correction accuracy
; in a given map pixel may vary with the s/c pointing used (eg if for one s/c
; pointing a bright source is viewed at the end of the chopper sweep, and a 
; faint background is observed at the start of the sweep), IWEIGHT1=1
; IWEIGHT2=0 or IWEIGHT2=2 is advisable, following a drift correction with 
; IRMSCORR=2. IWEIGHT1=IWEIGHT2=1 will give the best sensitivity for faint 
; sources on bright backgrounds. As a guide to which combination is best, first
; run MAKEMAP with IWEIGHT=0 and look at the difference between
; MAPRMS (the error in the mean derived from propagation of the original rms
; estimates) and MAPDISP (the error in the mean derived from the spread
; in the input data). If the two are close, then MAPSTACK with 
; IWEIGHT1=IWEIGHT2=1 will work best.
;
; In addition to the quantities mapped as for MAKEMAP and MAKEMAPMULTI,
; MAPSTACK also outputs double precision maps of RA and DEC in units of 
; degrees (MAPRA, MAPDEC). Finally, inlike MAKEMAP and MAKEMAPMULTI,all maps 
; are reflected about the Z axis so as to give the correct right handed
; coordinate system with positive Z = positive declination increasing to top
; of screen and positive Y=positive RA increasing to left of screen.
;
; Note that none of the FLAT field routines call MAPSTACK, so there
; is no real need to make the flat calculaation dependent on the combination
; of IWEIGHT1 and IWEIGHT2 used to make the map products. In all the flat
; routines, each detector pixel is given equal weight as IWEIGHT1=0 in the
; MAKEMAPMULTI calls.
; The only possible effect here is the use as weights of the RMSs in the 
; MAPFLAT1 routine and in the weighted polynomial fit in SKYFLAT. However
; these effects are very minor.
;
;common mapper,sgridy,sgridz,nygrid,nzgrid,ygrid,zgrid,errflag
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz
@common_mapstk
;common mapstk,mapstackmask
@common_pointcen
;common pointcen,racenc,deccenc,papointiny
@common_mapper
@common_steer1 
; steer1 to define npix
@common_signal
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract 
;
if(n_elements(nmappix) le 0)then begin
print,'mapstack,nmappix,mappix,mapbright,maprms,mapdisp,mapoffy,mapoffz,$'
print,'mapra,mapdec,mapnum,iweight1,iweight2,iotf,ibeam,beamflux,beamback'
print,'eg conservative target map with no weighting and only fine pointings: '
print,'mapstack,npix,lindgen(npix),map,mapr,mapd,mapy,mapz,$'
print,'mapra,mapdec,mapn,0,0,1,0,1.,0.'
print,'eg deep target map using fine pointings and slews: '
print,'mapstack,npix,lindgen(npix),map,mapr,mapd,mapy,mapz,$'
print,'mapra,mapdec,mapn,1,1,-1,0,1.,0.'
print,'eg beam map from same pointing data and same weighting as target map:'
print,'mapstack,npix,lindgen(npix),mapbeam,mapr,mapd,mapy,mapz,$'
print,'mapra,mapdec,mapn,0,0,1,1,1.0,0.0' 
goto,c333
endif
;
if(ibeam eq 1)then begin
ssav=s
simul32,1 ; simulated point source of unity integrated flux in units of Jy/
          ; detector pixel
; conversion factors between brightness and flux on P32 natural grid. Multiply
; data in MJy/sterad by RATIO to convert to Jy/map pixel and by RATIOD
; to convert to Jy/detector pixel
;
fluxbright,ratio,ratiod
;
s=beamback+s*beamflux/ratiod
;
endif ; ibeam eq 1
;
if(nmappix ne n_elements(mappix))then begin
print,'MAPSTACK: aborting - error in specification of detector pixels'
goto,c999
endif ; nmappix ne n_elements(mappix)
;
mapbright=fltarr(nygrid,nzgrid)
maprms=mapbright
mapdisp=mapbright
mapoffy=mapbright
mapoffz=mapbright
mapra=double(mapbright)
mapdec=mapra
mapnum=lonarr(nygrid,nzgrid)
;
makemapmulti,nmappix,mappix,mmapbright,mmaprms,mmapdisp,$
mmapoffy,mmapoffz,mmapnum,iweight1,iotf
;
ntar=npix*nygrid*nzgrid
if(n_elements(mapstackmask) eq ntar)then begin
; mapstackmask has been defined
  itest=where(mapstackmask(mappix,*,*) eq 0,nitest)
  if(nitest eq nmappix*nygrid*nzgrid)then print,'MAPSTACK: no censoring used'
  if(nitest ne nmappix*nygrid*nzgrid)then print,'MAPSTACK: censoring used'
  ii=where(mapstackmask(mappix,*,*) ne 0,nii)
  if(nii gt 0)then mmapnum(ii)=0
endif ; n_elements(mapstackmask) eq ntar
if(n_elements(mapstackmask) ne ntar)then begin
; mapstackmask has not been defined
  print,'MAPSTACK: censor mask undefined'
endif ; n_elements(mapstackmask) ne ntar
;
parad=papointiny*!pi/180.0D
sinpa=double(sin(parad))
cospa=double(cos(parad))
;
for iz=0,nzgrid-1 do begin
for iy=0,nygrid-1 do begin
;
ii=where(mmapnum(*,iy,iz) gt 0,nii)
if(nii gt 0)then begin
b=mmapbright(ii,iy,iz)
r=mmaprms(ii,iy,iz)
d=mmapdisp(ii,iy,iz)
n=mmapnum(ii,iy,iz)
y=mmapoffy(ii,iy,iz)
z=mmapoffz(ii,iy,iz)
;
if(iweight2 eq 0)then begin ; equal weights
rmsprop,b,r,bmean,bsig,bsigmean,0 ; photometry
rmsprop,y,r,ymean,ysig,ysigmean,0 ; y astrometry
rmsprop,z,r,zmean,zsig,zsigmean,0 ; z astrometry
endif ; iweight2 eq 0
;
if(iweight2 eq 1)then begin ; weight as (1/r)^2
rmsprop,b,r,bmean,bsig,bsigmean,1 ; photometry
rmsprop,y,r,ymean,ysig,ysigmean,1 ; y astrometry
rmsprop,z,r,zmean,zsig,zsigmean,1 ; z astrometry
endif ; iweight2 eq 1
;
if(iweight2 eq 2 or iweight2 eq 3)then begin ; weight as (1/d)^2
rmsprop,b,d,bmean,bsig,bsigmean,1 ; photometry
rmsprop,y,d,ymean,ysig,ysigmean,1 ; y astrometry
rmsprop,z,d,zmean,zsig,zsigmean,1 ; z astrometry
endif ; iweight2 eq 2
;
mapbright(iy,iz)=bmean
maprms(iy,iz)=bsigmean
mapnum(iy,iz)=total(n)
mapdisp(iy,iz)=bsig  
mapoffy(iy,iz)=ymean
mapoffz(iy,iz)=zmean
mapra(iy,iz)=ymean*sinpa-zmean*cospa ; offset in arcsec in RA from map centre
mapdec(iy,iz)=zmean*sinpa+ymean*cospa ; offset in arcsec in DEC from map centre
;
if(iweight2 eq 3)then begin
mediantrue,b,bmedtrue
mapbright(iy,iz)=bmedtrue
endif ; iweight2 eq 3
;
endif ; nii gt 0

endfor ; iy=0,nygrid-1
endfor ; iz=0,nzgrid-1
;
;mapra=(mapra/3600.0D)+racenc ; absolute RA of pixel in degrees
;mapdec=(mapdec/3600.0D)+deccenc
;
; assign mean pixel value of input maps to get the dummy value where
; for directions with no data, to preserve definitions in MAKEMAP:
;
ii=where(mapnum le 0,nii)
if(nii gt 0)then begin
;
mapbright(ii)=total(mmapbright(ii))/float(nii)
maprms(ii)=total(mmaprms(ii))/float(nii)
mapdisp(ii)=total(mmapdisp(ii))/float(nii)
mapoffy(ii)=total(mmapoffy(ii))/float(nii)
mapoffz(ii)=total(mmapoffz(ii))/float(nii)
;
endif ; nii gt 0
;
; MIRROR maps about the z axis
;
irotate=5
mapbright=rotate(mapbright,irotate)
maprms=rotate(maprms,irotate)
mapdisp=rotate(mapdisp,irotate)
mapoffy=rotate(mapoffy,irotate)
mapoffz=rotate(mapoffz,irotate)
mapra=rotate(mapra,irotate)
mapdec=rotate(mapdec,irotate)
mapnum=rotate(mapnum,irotate)
;
c999:
;
if(ibeam eq 1)then begin
s=ssav
endif ; ibeam eq 1
;
c333:
;
return
end









