pro model3searchrange,ipix
;
; For given monotonic illumination dependence of shooting parameters 
; (BETAEXPSHOOTDEFAULT and GAMMAEXPSHOOTDEFAULT) and given ranges in the offset
; terms (BETASHOOTDOWN to BETASHOOTUP and 
; TAUOFFSETSHOOTDOWN to TAUOFFSETSHOOTUP) the routine MODEL3SEARCHRANGE finds 
; the search range in the illumination coefficient of the shooting beta 
; parameter (BETACOEFFSHOOTDOWN to BETACOEFFSHOOTUP) and search range in the 
; illumination coefficient shooting time parameter (TAUSHOOTDOWN to TAUSHOOTUP)
; such that within a specified illumination range ILLCON1 to ILLCON2 the
; shooting beta parameter is always within a range BETASHORTCONMIN to 
; BETASHORTCONMAX and the shooting time parameter is always in a range 
; TAUSHORTCONMIN to TAUSHORTCONMAX. The search precisions BETACOEFFSHOOTPREC 
; TAUSHOOTPREC are set to a certain fraction (internal program constant)
; of the search ranges.
;
; The routine operates on element IPIX of the arrays.
;
; This routine is needed when calibrating the fast response of the detector 
; model by self calibration, in order that the search ranges for the 
; model parameters are tailored to the actual range of illuminations 
; encountered for each source/background/pixel/pointing combination. It is
; normally called from the routine MODEL3SETUP, using the region of 
; MODEL3SETUP reserved for composite self calibrations.
;
; Note: the default parameters BETASHOOTDEFAULT BETACOEFFSHOOTDEFAULT
; TAUOFFSETSHOOTDEFAULT, and TAUSHOOTDEFAULT are not set by MODEL3SEARCHRANGE
; 
; *****************************************************************************
; Note: in principle we should use an inverse of setshootbeta_model3 and 
; setshoottime_model3. Changes in these routines should be reflected by 
; changes in model3searchrange. The use of setshootbeta_model3 and 
; setshoottime_model3 would be mandatory if the scope of model3searchrange 
; were to be increase to cover the long beta and tau parameters as well as the
; shooting parameters, due to the decreasing in the beta jump by the betashoot
; value currently implemented in the setshootbeta_model3
; *****************************************************************************
;
; Input:
;  common DETDEFAULT:
;   BETAEXPSHOOTDEFAULT GAMMAEXPSHOOTDEFAULT BETASHOOTDOWN BETASHOOTUP
;   TAUOFFSETSHOOTDOWN TAUOFFSETSHOOTUP (all element IPIX)
;  common SCCONSTRAIN:
;   BETASHORTCONMIN BETASHORTCONMAX TAUSHORTCONMIN TAUSHORTCONMAX
;   ILLCON1 ILLCON2 (V/s)
;
; Output:
;  common DETDEFAULT:
;   BETACOEFFSHOOTDOWN BETACOEFFSHOOTUP TAUSHOOTDOWN 
;   TAUSHOOTUP (all element IPIX)
;   BETACOEFFSHOOTPREC TAUSHOOTPREC
;
@common_scconstrain
;common scconstrain,illcon1,illcon2,betalongconmin,betalongconmax,$
;betashortconmin,betashortconmax,taulongconmin,taulongconmax,$
;taushortconmin,taushortconmax,idriftrancon
@common_detdefault
;common detdefault,betadefault,taudefault,gammaexpdefault,$
;betashootdefault,taushootdefault,gammaexpshootdefault,$
;betacoeffdefault,betaexpdefault,tauoffsetdefault,$
;betacoeffshootdefault,betaexpshootdefault,tauoffsetshootdefault,$
;betaup,betadown,tauup,taudown,gammaexpup,gammaexpdown,$
;betashootup,betashootdown,taushootup,taushootdown,$
;gammaexpshootup,gammaexpshootdown,$
;betacoeffup,betacoeffdown,betaexpup,betaexpdown,$
;tauoffsetup,tauoffsetdown,betacoeffshootup,betacoeffshootdown,$
;betaexpshootup,betaexpshootdown,tauoffsetshootup,tauoffsetshootdown,$
;betaprec,tauprec,gammaexpprec,betashootprec,taushootprec,gammaexpshootprec,$
;betacoeffprec,betaexpprec,tauoffsetprec,$
;betacoeffshootprec,betaexpshootprec,tauoffsetshootprec
;
xfactor=0.01 ; fractional precision of search in coefficients
;
; SHOOTING BETA PARAMETER = betashoot + betacoeffshoot * ILL^(betaexpshoot)
;
; for ILL = ILLCON1:
;  test1 is betacoeffshoot giving betashortconmin for illcon1 and betashootdown
;  test2 is betacoeffshoot giving betashortconmax for illcon1 and betashootdown
; ... betacoeffshoot can be in range testmin1=min([test1,test2]) 
;                             ... to testmax1=max([test1,test2])
;  test3 is betacoeffshoot giving betashortconmin for illcon1 and betashootup
;  test4 is betacoeffshoot giving betashortconmax for illcon1 and betashootup
; ... betacoeffshoot can be in range testmin2=min([test3,test4]) 
;                             ... to testmax2=max([test3,test4])
; Thus for ILL = ILLCON1 betacoeffshoot must be in range 
; min([testmin1,testmin2])=x1 <= betacoeffshoot <= x2=max([testmax1,testmax2])
;
; for ILL = ILLCON2:
;  test5 is betacoeffshoot giving betashortconmin for illcon2 and betashootdown
;  test6 is betacoeffshoot giving betashortconmax for illcon2 and betashootdown
; ... betacoeffshoot can be in range testmin3=min([test5,test6]) 
;                             ... to testmax3=max([test5,test6])
;  test7 is betacoeffshoot giving betashortconmin for illcon2 and betashootup
;  test8 is betacoeffshoot giving betashortconmax for illcon2 and betashootup
; ... betacoeffshoot can be in range testmin4=min([test7,test8]) 
;                             ... to testmax4=max([test7,test8])
; Thus for ILL = ILLCON2 betacoeffshoot must be in range 
; min([testmin3,testmin4])=x3 <= betacoeffshoot <= x4=max([testmax3,testmax4])
; 
; The overall required range is thus from max([x1,x3]) to min([x2,x4])
;
test1=(betashortconmin-betashootdown(ipix))/(illcon1^betaexpshootdefault(ipix))
test2=(betashortconmax-betashootdown(ipix))/(illcon1^betaexpshootdefault(ipix))
test3=(betashortconmin-betashootup(ipix))/(illcon1^betaexpshootdefault(ipix))
test4=(betashortconmax-betashootup(ipix))/(illcon1^betaexpshootdefault(ipix))
test5=(betashortconmin-betashootdown(ipix))/(illcon2^betaexpshootdefault(ipix))
test6=(betashortconmax-betashootdown(ipix))/(illcon2^betaexpshootdefault(ipix))
test7=(betashortconmin-betashootup(ipix))/(illcon2^betaexpshootdefault(ipix))
test8=(betashortconmax-betashootup(ipix))/(illcon2^betaexpshootdefault(ipix))
testmin1=min([test1,test2]) 
testmax1=max([test1,test2])
testmin2=min([test3,test4]) 
testmax2=max([test3,test4])
testmin3=min([test5,test6])
testmax3=max([test5,test6])
testmin4=min([test7,test8]) 
testmax4=max([test7,test8])
x1=min([testmin1,testmin2])
x2=max([testmax1,testmax2])
x3=min([testmin3,testmin4])
x4=max([testmax3,testmax4])
betacoeffshootdown(ipix)=max([x1,x3]) ; condition for range to apply at 
betacoeffshootup(ipix)=min([x2,x4])   ; both ILLCON1 or ILLCON2
;betacoeffshootdown(ipix)=min([x1,x3]) ;  condition for range to apply at 
;betacoeffshootup(ipix)=max([x2,x4])   ; either ILLCON1 or ILLCON2
betacoeffshootprec=xfactor*(betacoeffshootup(ipix)-betacoeffshootdown(ipix))
;
; SHOOTING TIME PARAMETER = tauoffsetshoot + taushoot * ILL^(-gammaexpshoot)
;
; for ILL = ILLCON1:
;  - test1 is taushoot giving taushortconmin for illcon1 and tauoffsetshootdown
;  - test2 is taushoot giving taushortconmax for illcon1 and tauoffsetshootdown
; ... taushoot can be in range testmin1=min([test1,test2]) 
;                              ... to testmax1=max([test1,test2])
;  - test3 is taushoot giving taushortconmin for illcon1 and tauoffsetshootup
;  - test4 is taushoot giving taushortconmax for illcon1 and tauoffsetshootup
; ... taushoot can be in range testmin2=min([test3,test4]) 
;                              ... to testmax2=max([test3,test4])
; Thus for ILL = ILLCON1 taushoot must be in range 
; min([testmin1,testmin2])=x1 <= taushoot <= x2=max([testmax1,testmax2])
;
; for ILL = ILLCON2:
;  - test5 is taushoot giving taushortconmin for illcon2 and tauoffsetshootdown
;  - test6 is taushoot giving taushortconmax for illcon2 and tauoffsetshootdown
; ... taushoot can be in range testmin3=min([test5,test6]) 
;                              ... to testmax3=max([test5,test6])
;  - test7 is taushoot giving taushortconmin for illcon2 and tauoffsetshootup
;  - test8 is taushoot giving taushortconmax for illcon2 and tauoffsetshootup
; ... taushoot can be in range testmin4=min([test7,test8]) 
;                              ... to testmax4=max([test7,test8])
; Thus for ILL = ILLCON2 taushoot must be in range 
; min([testmin3,testmin4])=x3 <= taushoot <= x4=max([testmax3,testmax4])
; 
; The overall required range is thus from max([x1,x3]) to min([x2,x4])
;
;print,'illcon1,illcon2,taushortconmin,taushortconmax,gammaexpshootdefault'
test1=$
(taushortconmin-tauoffsetshootdown(ipix))*(illcon1^gammaexpshootdefault(ipix))
test2=$
(taushortconmax-tauoffsetshootdown(ipix))*(illcon1^gammaexpshootdefault(ipix))
test3=$
(taushortconmin-tauoffsetshootup(ipix))*(illcon1^gammaexpshootdefault(ipix))
test4=$
(taushortconmax-tauoffsetshootup(ipix))*(illcon1^gammaexpshootdefault(ipix))
test5=$
(taushortconmin-tauoffsetshootdown(ipix))*(illcon2^gammaexpshootdefault(ipix))
test6=$
(taushortconmax-tauoffsetshootdown(ipix))*(illcon2^gammaexpshootdefault(ipix))
test7=$
(taushortconmin-tauoffsetshootup(ipix))*(illcon2^gammaexpshootdefault(ipix))
test8=$
(taushortconmax-tauoffsetshootup(ipix))*(illcon2^gammaexpshootdefault(ipix))
testmin1=min([test1,test2]) 
testmax1=max([test1,test2])
testmin2=min([test3,test4]) 
testmax2=max([test3,test4])
testmin3=min([test5,test6])
testmax3=max([test5,test6])
testmin4=min([test7,test8]) 
testmax4=max([test7,test8])
x1=min([testmin1,testmin2])
x2=max([testmax1,testmax2])
x3=min([testmin3,testmin4])
x4=max([testmax3,testmax4])
taushootdown(ipix)=max([x1,x3])
taushootup(ipix)=min([x2,x4])
taushootdown(ipix)=max([x1,x3]) ; condition for range to apply at 
taushootup(ipix)=min([x2,x4])   ; both ILLCON1 or ILLCON2
;taushootdown(ipix)=min([x1,x3]) ;  condition for range to apply at 
;taushootup(ipix)=max([x2,x4])   ; either ILLCON1 or ILLCON2
taushootprec=xfactor*(taushootup(ipix)-taushootdown(ipix))

;
c999:
;
return
end








