pro optimize_shell,iparsolve,parmin,parmax,parstart,parprec,niter,par,chimin
;
; general purpose shell for multidimensional cut optimization routine to
; find minimum value CHIMIN of a function FUN(X1,X2,...XN,chi) 
; where X1,X2,...XN are constant dependent arrays or single values and 
; not changed in program. X1,X2,...XN must be added to the called parameters 
; according to each application, and the nested do loop updated according
; to the number of parameters. 
;
; input
;  PARMIN,PARMAX (arrays dimension number of variables): best estimate for
;    value of parameters on input
;  PARSTART (arrays dimension number of variables): search range
;    for parameters
;  PARPREC (arrays dimension number of variables): precision for 
;    solution for parameters
;  IPARSOLVE (integer array dimension number of variables): set element to 1 
;    for variables to be varied in optimisation; set element to 0 for variables
;    not to be varied (these variables are fixed with the value of the
;    corresponding element of PARSTART during the optimisation).
;  NITER maximum number of iterations
;
; output
;  PAR (array dimension number of variables): best fit values. Variables not
;    searched for have the corresponding element of PAR set to the value in 
;    PARSTART.
;  PARPREC (arrays dimension number of variables): precision for 
;    solution for parameters beta tau gammaexp. Set to actual fineness
;    of search on output
;  CHIMIN (single real) minimum vaalue of chi-squared for fit
;
parmin_in=parmin
parmax_in=parmax
;
npar=n_elements(parstart)
par=dblarr(npar)
;
; internal constants controlling trade off between speed, precision and 
; robustness of fit
;
ncut=lonarr(npar)
ncut(*)=5 ; must be odd and at least 5, for parameters included in optimisation
          ; (ie with IPARSOLVE=1). For parameters not included in optimisation
;           (ie with IPARSOLVE=0) the corresponding element of NCUT should be 
;           set to 1. 
iicut0=where(iparsolve le 0,niicut0)
iicut1=where(iparsolve gt 0,niicut1)
if(niicut0 gt 0)then ncut(iicut0)=1
icut=lonarr(npar)
reduce=dblarr(npar)
reduce(*)=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
;xncut=dblarr(npar)
xncut=float(ncut)
top=parmax ; array dimension npar
bot=parmin
dcut=fltarr(npar)
dcut(iicut1)=(top(iicut1)-bot(iicut1))/(xncut(iicut1)-1.0) ; array dim. ncut
if(niicut0 gt 0)then $
dcut(iicut0)=(top(iicut0)-bot(iicut0))/(max(xncut(iicut1))-1.0)
partrial=dblarr(npar) ; one trial value for each parameter
;
for ipar=0,npar-1 do begin
if(ncut(ipar) eq 1)then begin
bot(ipar)=parstart(ipar)
top(ipar)=bot(ipar)
endif ; ncut(ipar) eq 1
partrial(ipar)=bot(ipar)
;
print,'ipar,ncut(ipar),bot(ipar),top(ipar),dcut(ipar),partrial(ipar) = '
print,ipar,ncut(ipar),bot(ipar),top(ipar),dcut(ipar),partrial(ipar)
;
; diagnostic prior to loop
;print,'ipar,ncut(ipar),dcut(ipar),bot(ipar),top(ipar) = '
;print,ipar,ncut(ipar),dcut(ipar),bot(ipar),top(ipar)
;
endfor ; ipar=0,npar-1
;
; first trial to initialise par,chimin
;fun,partrial(0),partrial(1)...partrial(npar-1),chi
chi=1.0+randomu(seed) ; test control
;
par(*)=partrial(*)
chimin=chi
;
iter=0 ; counter of number of iterations
;
c100:
;
; example of three dimensions
;
for i0=0,ncut(0)-1 do begin
icut(0)=i0
partrial(0)=bot(0)+dcut(0)*float(icut(0))
for i1=0,ncut(1)-1 do begin
icut(1)=i1
partrial(1)=bot(1)+dcut(1)*float(icut(1))
for i2=0,ncut(2)-1 do begin
icut(2)=i2
partrial(2)=bot(2)+dcut(2)*float(icut(2))
;
;fun,partrial(0),partrial(1)...partrial(npar-1),chi
chi=1.0+randomu(seed) ; test control
print,'icut,partrial,chi = ',icut,partrial,chi
;
if(chi lt chimin)then begin ; found new minimum
chimin=chi ; single value
par=partrial ; array
icutmin=icut ; array
print,'*** par,chimin = ',par,chimin
endif ; chi lt chimin
;
endfor ; i2=0,ncut(1)-1
endfor ; i1=0,ncut(1)-1
endfor ; i0=0,ncut(0)-1
;
; update search ranges bot top dcut ncut xncut
; 
for ipar=0,npar-1 do begin
;
if(ncut(ipar) gt 1)then begin
;
d=top(ipar)-bot(ipar)
d=d/reduce(ipar)
d2=d/2.0
bot(ipar)=par(ipar)-d2
top(ipar)=par(ipar)+d2
bottmp=bot(ipar)
toptmp=top(ipar)
if(bot(ipar) lt parmin(ipar))then begin
bottmp=parmin(ipar)
toptmp=bottmp+d
endif ; bot(ipar)
if(top(ipar) gt parmax(ipar))then begin
toptmp=parmax(ipar)
bottmp=toptmp-d
endif ; top(ipar)
bot(ipar)=bottmp
top(ipar)=toptmp
dcut(ipar)=(top(ipar)-bot(ipar))/(xncut(ipar)-1.0)
;
; if requested precision has been attained for ipar suppress further search 
; in var(ipar):
;
if(dcut(ipar) le parprec(ipar))then begin
;
ncut(ipar)=1
xncut(ipar)=float(ncut(ipar))
top(ipar)=par(ipar) ; array dimension npar
bot(ipar)=par(ipar)
;
endif ; dcut(ipar) le parprec(ipar)
;
endif ; ncut(ipar) gt 1
;
; diagnostic
print,'iter,ipar,ncut(ipar),dcut(ipar),bot(ipar),top(ipar) = '
print,iter,ipar,ncut(ipar),dcut(ipar),bot(ipar),top(ipar)
;
endfor ; ipar=0,npar-1
;
iter=iter+1
;
if(iter ge niter or max(ncut) le 1)then goto,c200
;
goto, c100
;
c200:
;
c999:
;
parprec=dcut
parmin=parmin_in
parmax=parmax_in
;
return
end


