;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_DISP_MAP
;
; PURPOSE:
;       This routine starts a graphical user interface to define the options
;       for map display
;
; DESCRIPTION:
;       The GUI consists of the following major sections.
;
;       1. Calibration input:                       
;          toggle button to define 'Linearize signal' on/off
;          radio buttons to select the units on displayed map                
;            (MJy/sr, Jy/beam, V/s)
;       2. Radio buttons to select the flux calibration method:               
;          Default: calculates the expected default responsivity according to
;                   the orbital position.
;          Avg. FCS's: responsivity calculated with the average of fcs1 and fcs2
;                      measurements.
;          FCS#1: responsivity calculated with the fcs1 measurement.
;          FCS#2: responsivity calculated with the fcs2 measurement.
;
;       3. Data Selection
;          buttons to select/deselect slew data.
;
;       4. Detector pixels selection:
;          buttons to select/deselect pixels on the output map.
;
;       5. toggle button to turn on/off the flatfielding option applied before
;          the map is displayed.
;
;       6. Processing buttons
;          Close: close the widget after saving the current values of all the  
;                 options and the pixels.                             
;          Cancel: close the widget discarding the current selections.       
;
;          Help: help on the processing 
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_disp_map, pid, GROUP=group
;
; INPUTS:
;       pid:      pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       GROUP : The group leader of the P32 Tools widget. 
;               If none defined then PIA top widget is taken as the
;               group leader.
;
; CALLED ROUTINES:
;       PIA_DISP_MAP_EVENT (Internal procedure of Event handler)
;
; CALLED FROM:    
;       PIA_ERD2MAP_MAIN      
;
; COMMON BLOCKS:
;       PIXPTR
;
; PROCEDURE:
;       @common_steer1
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;-
;-----------------------------------------------------------------------
;
PRO PIA_disp_map_EVENT,event

COMMON e2m_map_locids, lid
@common_steer1
common pixptr, map_all, map_pix

	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=pid
	
        debug = 0
        IF debug THEN help,action,event,/str
        nps = n_elements(map_pix)
        IF debug THEN print, STRMID(action, 0, 4)
	CASE STRMID(action, 0, 4) of 
	
         'FCAL': BEGIN
                 indx = event.value
                 lid.cal = indx
                 IF debug THEN print,'lid cal',lid.cal
         END

	 'DATA': BEGIN
	 	CASE event.value OF
		  0: lid.otf= -1 
		  1: lid.otf= 1
		ENDCASE
                  IF debug THEN print,'otf',lid.otf
	 END

	 'SELA': BEGIN
		  map_all = 1
		  map_pix(*) = 1     
                  IF debug THEN print,'map pix', map_all  
                  for i = 0, nps-1 do $
                    WIDGET_CONTROL, lid.pixsel(i), set_button=1
	 END
	 'SELN': BEGIN
		  map_all = 0
		  map_pix(*) = 0     
                  IF debug THEN print,'map pix', map_all  
                  for i = 0, nps-1 do $
                    WIDGET_CONTROL, lid.pixsel(i), set_button=0
	 END
	 'SELP': BEGIN
                   map_all = 0
                   npv = fix(strmid(action,4,1))
                   IF debug THEN print,npv
                   map_pix(npv) = event.select
                   IF debug THEN print,map_pix
	 END

         'SLIN': BEGIN
                (*pid).lin_sig = event.select
                IF debug THEN print,'lin sig', (*pid).lin_sig
         END


	 'CAL': BEGIN
                lid.map_unit = event.value
               IF debug THEN  print,'unit=',lid.map_unit
	 END

         'MAP': BEGIN
                lid.flatf = event.select
               IF debug THEN  print,'ff=',lid.flatf        
	 END

	 "CLOS": BEGIN
                 IF debug THEN print,'clos',map_pix
                 (*pid).otf = lid.otf
                 (*pid).erd2map_ff = lid.flatf
                 (*pid).map_unit = lid.map_unit
                 (*pid).cal_type = lid.cal 
                 IF debug THEN print,'caltype',(*pid).cal_type
                 WIDGET_CONTROL, event.top, /DESTROY
         END

	 "QUIT": BEGIN
                 map_pix = lid.save_pix
                 IF debug THEN print,'quit map_pix',map_pix
                 WIDGET_CONTROL, event.top, /DESTROY
         END


         "HELP": BEGIN
           IF !VERSION.OS NE 'vms' THEN $
              spawn, "$w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_map.html)' &"  $
           ELSE $
              spawn, "w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_map.html)'",/nowait
         END


	 ELSE:

	ENDCASE

 	RETURN
	END
	

	
PRO PIA_disp_map, pid, GROUP=group

@common_steer1
COMMON e2m_map_locids, lid
COMMON pixptr, map_all, map_pix


        debug = 0
        otf =  (*pid).otf 
        IF otf LT 0 THEN otf = 0
        flatf = (*pid).erd2map_ff
        nps = n_elements(map_pix)
        IF debug THEN print,'in',map_pix
        save_pix=map_pix
 	lid ={map_unit:(*pid).map_unit,otf:otf, cal:(*pid).cal_type, $
              flatf:flatf, pixbase:0L, $
              save_pix:save_pix, pixsel:lonarr(nps)}

	IF not KEYWORD_SET(group) THEN group=!PIA_TOP_WID
	id_base= WIDGET_BASE(TITLE="Map DISPLAY options ", /COLUMN)
	
	id_0 = WIDGET_BASE(id_base, /COL, /FRAME)
        lab = WIDGET_LABEL(id_0, VAL='Calibration:', /align_left, $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')


        id_1 = WIDGET_BASE(id_0, /ROW, /NONEXCLUSIVE)
        slin = WIDGET_BUTTON(id_1, VALUE='Linearize Signal', UVALUE='SLIN')
        IF (*pid).lin_sig EQ 1 THEN $
           WIDGET_CONTROL, slin, /set_button

        
	stdbut = cw_bgroup(id_0,   $ 
			/EXCLUSIVE, ROW=3, $
			SET_VALUE = (*pid).map_unit,  $
			UVALUE = 'CAL', $
			["Surface brightness units (MJy/sr)  ", $
			 "Point source units (Jy/beam)  ", $
			 "Signals(V/s)"])

        lab = WIDGET_LABEL(id_0 ,VAL='Flux Calibration using:', /align_left,  $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')

        id_1 = WIDGET_BASE(id_0, /COL)


        stdbut = cw_bgroup(id_1,   $
                        /EXCLUSIVE, COLUMN=4, $
                        SET_VALUE = lid.cal, $
                        UVALUE = 'FCAL', $
                        ["Default", $
                         "Avg. FCS's", $
                         "FCS#1","FCS#2" ])


	id_0 = WIDGET_BASE(id_base, /COL, /FRAME)
        lab = WIDGET_LABEL(id_0, VAL='Data Selection:', /align_left, $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	stdbut = cw_bgroup(id_0,   $ 
			/EXCLUSIVE, ROW=3, $
			SET_VALUE = otf, $
			UVALUE = 'DATA', $
			["Include slew data  ", $
			 "Exclude Slew data "])   

	id_0 = WIDGET_BASE(id_base, /COL, /FRAME)
        lab = WIDGET_LABEL(id_0, VAL='Select detector pixels to be displayed:', /align_left, $
                           FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
        id_1 =  WIDGET_BASE( id_0, /ROW)
        junk = WIDGET_BUTTON( id_1, VAL="Select All", UVA="SELA" )
        junk = WIDGET_BUTTON( id_1, VAL="Deselect All", UVA="SELN" )

        if nps eq 4 then rown = 2 else rown=3
        lid.pixbase = WIDGET_BASE( id_0, /NONEXCLUSIVE,  UVAL='PIXBASE', ROW=rown)      

        nps = n_elements(map_pix)
        for i=0, nps-1 DO BEGIN
          pstr = 'Pixel # ' + STRTRIM(i+1, 2)
                    uv = 'SELP'+STRTRIM(i,2)
          lid.pixsel(i) = WIDGET_BUTTON( lid.pixbase, VAL=pstr, UVAL=uv )
          IF map_pix(i) EQ 1 THEN $
            WIDGET_CONTROL, lid.pixsel(i), set_button=1
        endfor

;	id_0 = WIDGET_BASE(id_base, /COL, /NONEXCLUSIVE)
        ;ff_id = WIDGET_BUTTON(id_0, VAL = "Apply Flatfielding Algorithm", $
                             ; UVAL="MAP", SET_VALUE = '1')
        ;derp_id =  WIDGET_BUTTON(id_0, VAL = "Automatic pop up display after drift processing", $
                                 ;UVAL="MAP", SET_VALUE = '1')
 	id_0 = WIDGET_BASE(id_base, /ROW)
 	stdbut = cw_bgroup(id_0,   $ 
 			/NONEXCLUSIVE, ROW=2, $
 			SET_VALUE = flatf,  $
 			UVALUE = 'MAP', $
 			"Apply flatfielding algorithm")  

	id_0 = WIDGET_BASE(id_base, /ROW)
	id_1 = WIDGET_BUTTON( id_0, VAL="  Close ", UV="CLOSE" )
	id_1 = WIDGET_BUTTON( id_0, VAL="  Cancel  ", UV="QUIT" )
	id_1 = WIDGET_BUTTON( id_0, VAL="  Help ", UV="HELP" )

	
	WIDGET_CONTROL, id_base, /realize, set_UVALUE=pid	
	Xmanager, /NO_BLOCK, "PIA_DISP_MAP", id_base, GROUP=group

	RETURN
	END
	
	
