;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PIA P32 Processing Tools Software.              !
;! Designed by: Nanyao Lu at IPAC (lu@ipac.caltech.edu)                        !
;!              Bernhard Schulz at Vilspa (bschulz@iso.vilspa.esa.es)          !
;! Developed by: Iffat Khan at IPAC (irk@ipac.caltech.edu)                     !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_ERD2MAP_DG
;       Adapted from VIEWSIG
;       Widget Utility to rapidly inspect data
;       Author: Bernhard Schulz, Version:  Mar 29,2001
;
;
;
; PURPOSE:
;       This routine starts a P32 data deglitching graphical
;       user interface. 
;
; DESCRIPTION:
;       The GUI consists of the following major sections.
;
;       1. Data display in plotting window. The valid data is plotted
;          in red color, slew in blue, and bad data in yellow color. 
;          The positions and strengths of the detected  glitches are       
;          overplotted as vertical white lines above a horizontal white
;          line and the exponential spike trails are shown in violet. 
;
;       2. Data inspection widget buttons under 'Navigate' and 'Display'.
;          once activated, data is selected using the left mouse button.
;
;       3. Manual Deglitching:
;          Select: Deglitch the selected data
;          Deselect: discards the deglitches in the selected data
;
;       4. Control buttons:              
;          Quit: discards all the changes made to bad and signal arrays
;                during processings in this window. resets to state at
;                which they entered into this widget.
;          Exit: keeps all changes. Bad and signal arrays are altered on exit.
;
;       5. Richard Tuffs's deglitching tool:
;          4 text buttons are shown to enter the values of deglitching parameters
;          'Fraction bad readouts (%)', 'Time constant spike tails (s)',
;          'Spike inclusion threshold (V/s)', 'Spike clipping threshold (V/s)'.
;
;          Toggle button for Patch flag on/off.
;
;          'Deglitch' button to deglitch using the above parameters and to obtain    
;           the modified values of BAD and SIGNAL arrays for the selected pixel.
;
; CATEGORY:
;       PIA - P32     
;
; CALLING SEQUENCE:
;       pia_erd2map_dg, n1, n2, pix, pid  
;
; INPUTS:
;       n1  : start of the desired fine pointing range
;       n2  : end of the desired fine pointing range
;       pix : IDL convention for desired pixel number (i.e pixel # - 1)
;       pid : pointer variable to global parameters structure
;
; OPTIONAL INPUTS:
;       None.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;
; CALLED ROUTINES:
;       ERD2MAP_DG2
;       Internal Routines:
;       MAKE_DG_BASE
;       PIA_ERD2MAP_DG_EVENT (Event handler)
;       VT_COOR2DATA 
;       VT_FLAGRECT
;       VT_PLRECT 
;       VT_REDRAW 
;       VT_ZOOMIN 
;
; CALLED FROM:    
;       PIA_ERD2MAP_SPIX      
;
; COMMON BLOCKS:
;       DG_OUT
;
; PROCEDURE:
;       @common_signal
;       @common_timer  
;       @common_all
;       @common_driftpar
;       @common_e32
;       @common_plotter
;       @common_steerin
;       @colour_indices
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;	original design, Version 1.1 IRK        Jan 08, 2001
;       Version 1.35                            Mar 08, 2001
;       Version 1.40                            Mar 16, 2001
;       Version 1.50                            Mar 30, 2001
;-
;-----------------------------------------------------------------------
;



pro pia_erd2map_dg_event, event

@common_signal
@common_driftpar

debug = 0
widget_control, get_uvalue = cm, event.top

ctrl    = cm(0)
sigbuf  = cm(1)
widg    = cm(2)
pid  = cm(5)
;help,pid,bad

widget_control, get_uvalue = uval, event.id
widget_control, get_value = val, event.id
pix = (*ctrl).cur_pix
;print,'pix',pix
ip1 = (*ctrl).sind
ip2 = (*ctrl).eind

if n_elements(uval) GT 0 then begin

 CASE uval OF
  'EXIT': begin
            bad(pix,ip1:ip2) = (*sigbuf).flag
            s(pix,ip1:ip2)   = (*sigbuf).slop 
            dgp = (*pid).dgq

            WIDGET_CONTROL, (*widg).fracbad, GET_VALUE=val
            dgp.fracb = FLOAT(val(0))

            WIDGET_CONTROL, (*widg).tglitch, GET_VALUE=val
            dgp.tglch = FLOAT(val(0))

            WIDGET_CONTROL, (*widg).thresh1, GET_VALUE=val
            dgp.thres1 = FLOAT(val(0))

            WIDGET_CONTROL, (*widg).thresh2, GET_VALUE=val
            dgp.thres2 = FLOAT(val(0))
            dgp.irep = (*widg).irep
            (*pid).dgq = dgp
            (*pid).imple = 1
            ndriftsolve(pix) = 0

  	    widget_control, event.top, /destroy
	    ptr_free, cm(0), cm(1), cm(2), cm(3), cm(4)
	  end
  'QUIT': begin
            bad(pix,ip1:ip2) = *cm(3)
            s(pix,ip1:ip2) = *cm(4)
            widget_control, event.top, /destroy
	    ptr_free, cm(0), cm(1), cm(2), cm(3), cm(4)
          end

  'HELP': begin
           IF !VERSION.OS NE 'vms' THEN $
             spawn, "$w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_dg.html)' &"  $
           ELSE $
             spawn, "w3net -remote 'openURL('$PIA_ERD2MAP_HTML'/p32_dg.html)'",/nowait
          end

   'IRE': begin
             (*widg).irep = event.select
             IF debug THEN print,'ire=',(*widg).irep
           end

         'IMPLE': begin
; implement solution?
                WIDGET_CONTROL, event.top, /hourglass

                (*ctrl).dgflag = 0
                bad(pix,ip1:ip2) = (*sigbuf).flag
                s(pix,ip1:ip2)   = (*sigbuf).slop 
                dgp = (*pid).dgq     
                WIDGET_CONTROL, (*widg).fracbad, GET_VALUE=val
                fractbadscp = FLOAT(val(0))/100.

                WIDGET_CONTROL, (*widg).tglitch, GET_VALUE=val
                tglitch = FLOAT(val(0))

                WIDGET_CONTROL, (*widg).thresh1, GET_VALUE=val
                thresherd1 = FLOAT(val(0))

                WIDGET_CONTROL, (*widg).thresh2, GET_VALUE=val
                thresherd2 = FLOAT(val(0))

                pixarray = pix
                IF debug THEN help,pixarray
                ifine1 = (*ctrl).n1
                ifine2 = (*ctrl).n2
                irep = (*widg).irep

                IF ndriftsolve(pix) EQ 0 THEN iviguse = 1 $
                ELSE iviguse = 0

                ibias = 0
                iusefine = 0
                implementerd = 1
                implementscp = 1

                erd2map_dg2, pixarray, iviguse, irep, ibias, iusefine, fractbadscp, $
                     tglitch, thresherd1, thresherd2, implementerd, implementscp, $
                     ifine1, ifine2
                (*sigbuf).flag = bad(pix,ip1:ip2)
                (*sigbuf).slop = s(pix,ip1:ip2)
                (*ctrl).dgflag = 1
                vt_redraw, ctrl, sigbuf
                IF debug THEN print,pix, ndriftsolve(pix)

                IF debug THEN help,dgp,/str

         END

         'DEGLITCH': begin
                (*ctrl).dgflag = 0
                WIDGET_CONTROL, event.top, /hourglass

                WIDGET_CONTROL, (*widg).fracbad, GET_VALUE=val
                fractbadscp = FLOAT(val(0))/100.

                WIDGET_CONTROL, (*widg).tglitch, GET_VALUE=val
                tglitch = FLOAT(val(0))

                WIDGET_CONTROL, (*widg).thresh1, GET_VALUE=val
                thresherd1 = FLOAT(val(0))

                WIDGET_CONTROL, (*widg).thresh2, GET_VALUE=val
                thresherd2 = FLOAT(val(0))

                pixarray = pix
                IF debug THEN help,pixarray
                ifine1 = (*ctrl).n1
                ifine2 = (*ctrl).n2
                irep = (*widg).irep

                IF ndriftsolve(pix) EQ 0 THEN iviguse = 1 $
                ELSE iviguse = 0

                ibias = 0
                iusefine = 0
                implementerd = 1
                implementscp = 1
                bad(pix,ip1:ip2) = *cm(3)
                s(pix,ip1:ip2) = *cm(4)
                if debug then begin
                  bindx = where(bad(pix,ip1:ip2) gt 0, badcnt)
                  bindx = where(bad(pix,ip1:ip2) eq 0, gdcnt)
                  print,"bad good", badcnt, gdcnt
                endif
 
                erd2map_dg2, pixarray, iviguse, irep, ibias, iusefine, fractbadscp, $
                     tglitch, thresherd1, thresherd2, implementerd, implementscp, $
                     ifine1, ifine2
                (*sigbuf).flag = bad(pix,ip1:ip2)
                (*sigbuf).slop = s(pix,ip1:ip2)
                if debug then begin
                  bindx = where(bad(pix,ip1:ip2) gt 0, badcnt)
                  bindx = where(bad(pix,ip1:ip2) eq 0, gdcnt)
                  print,"Deglich: bad good", badcnt, gdcnt
                endif
 
                (*ctrl).dgflag = 1
                vt_redraw, ctrl, sigbuf


         END


  'FREE':   begin
             (*ctrl).scale = 0
             vt_redraw, ctrl, sigbuf
  	    end
  'FIX':    begin
             (*ctrl).scale = 1
  	    end
  'UP':     begin
	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             len = ((*ctrl).ymax - (*ctrl).ymin)	     
             (*ctrl).ymin = (*ctrl).ymin + len / 3
             (*ctrl).ymax = (*ctrl).ymax + len / 3
             vt_redraw, ctrl, sigbuf
  	    end
  'DOWN': begin
	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             len = ((*ctrl).ymax - (*ctrl).ymin)	     
             (*ctrl).ymin = (*ctrl).ymin - len / 3
             (*ctrl).ymax = (*ctrl).ymax - len / 3
             vt_redraw, ctrl, sigbuf
  	    end
  'RIGHT':     begin
	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             len = ((*ctrl).xmax - (*ctrl).xmin)	     
             (*ctrl).xmin = (*ctrl).xmin + len / 3
             (*ctrl).xmax = (*ctrl).xmax + len / 3
             vt_redraw, ctrl, sigbuf
  	    end
  'LEFT': begin
	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             len = ((*ctrl).xmax - (*ctrl).xmin)	     
             (*ctrl).xmin = (*ctrl).xmin - len / 3
             (*ctrl).xmax = (*ctrl).xmax - len / 3
             vt_redraw, ctrl, sigbuf
  	    end
  'ZOOM_IN': begin
	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             (*ctrl).zoom = 1		;activate zoom
             (*ctrl).flagging = 0	;deactivate flagging

  	    end
  'ZOOM_OUT': begin
            (*ctrl).zoom = 0		;dactivate zoom in
 	     widget_control, set_button=1, (*cm(2)).fix
             (*ctrl).scale = 1
             len = (*ctrl).xmax - (*ctrl).xmin
	     center = (*ctrl).xmin + len*0.5
             (*ctrl).xmin = center - len * 1.05
             (*ctrl).xmax = center + len * 1.05
             len = ((*ctrl).ymax - (*ctrl).ymin)	     
	     center = (*ctrl).ymin + len*0.5
             (*ctrl).ymin = center - len * 1.05
             (*ctrl).ymax = center + len * 1.05
             vt_redraw, ctrl, sigbuf
  	    end
  'ALL':   begin
             (*ctrl).disp = 0
             vt_redraw, ctrl, sigbuf
  	    end
  'VALID':    begin
             (*ctrl).disp = 1
             vt_redraw, ctrl, sigbuf
  	    end
  'HIDE': begin
  	     (*ctrl).zoom = 0
  	     CASE (*ctrl).flagging OF
	       0: (*ctrl).flagging = 1
	       else:
	     ENDCASE
  	  end
  'UNHIDE': begin
  	     CASE (*ctrl).flagging OF
	       0: (*ctrl).flagging = 2
	       else:
	     ENDCASE
  	    end
  'DOTS': begin
  	     (*ctrl).symb = 3 
             vt_redraw, ctrl, sigbuf
	  end
  'SYMBOLS': begin
  	       (*ctrl).symb = 6 
               vt_redraw, ctrl, sigbuf
	     end 
  'NOERR': begin
  	     (*ctrl).plerr = 0 
             vt_redraw, ctrl, sigbuf
	  end
  'PLERR': begin
  	       (*ctrl).plerr = 1 
               vt_redraw, ctrl, sigbuf
	     end 
  'DRAW': begin

	      if event.type EQ 0 AND $
		 (event.press AND 04b) EQ 4  THEN begin  ;shortcut for hide with
  	     	(*ctrl).zoom = 0			;right mouse button
  	     	(*ctrl).flagging = 1
	     endif


  	     IF (*ctrl).flagging GT 0 OR (*ctrl).zoom GT 0 THEN BEGIN
               CASE event.type OF
;-------------------------------------------
;Button Press
	        0: begin
		     (*ctrl).cx1 = event.x	;save first coord.
		     (*ctrl).cy1 = event.y
		     (*ctrl).cx2 = event.x	;initialize previous coord.
		     (*ctrl).cy2 = event.y
		     (*ctrl).rect = 1		;activate rectangle
		   end

;-------------------------------------------
;Mouse Motion
		2: begin
		     IF (*ctrl).rect GT 0 THEN BEGIN
		       vt_coor2data, [(*ctrl).cx1,(*ctrl).cx2], $
		       		     [(*ctrl).cy1,(*ctrl).cy2], x, y
		       vt_plrect, x, y, ctrl, color=0, linestyle=1		;erase last
		       (*ctrl).cx2 = event.x	;store coord. for next event
		       (*ctrl).cy2 = event.y
		       vt_coor2data, [(*ctrl).cx1,(*ctrl).cx2], $
		       		     [(*ctrl).cy1,(*ctrl).cy2], x, y
		       vt_plrect, x, y, ctrl, color=255, linestyle=1	;draw new
		     ENDIF
		   end

;-------------------------------------------
;Button Release
		1: begin
		     vt_coor2data, [(*ctrl).cx1,(*ctrl).cx2], $
		       		   [(*ctrl).cy1,(*ctrl).cy2], x, y
		     vt_plrect, x, y, ctrl, color=0, linestyle=1 	;erase last

		     vt_coor2data, [(*ctrl).cx1,event.x], $
		      	 	   [(*ctrl).cy1,event.y], x, y

                     if debug then print,'select',x,y

		     IF (*ctrl).flagging GT 0 THEN BEGIN
		       if (*ctrl).flagging EQ 2 then unhide = 1 else unhide = 0
		       vt_flagrect, x, y, sigbuf, unhide=unhide
		       (*ctrl).flagging = 0	;deactivate flagging
		     ENDIF ELSE BEGIN
		       vt_zoomin, x, y, ctrl
		       (*ctrl).zoom = 0		;deactivate zooming
		     ENDELSE
		     
	             vt_redraw, ctrl, sigbuf
		     (*ctrl).rect = 0		;deactivate rectangle
		   end


		else:
	       ENDCASE
;-------------------------------------------

  	     ENDIF 
	     

          end
  else:
 ENDCASE

endif

end

PRO make_dg_base, basid, dgc, cm
;---------------------------------------------------------
; Draw data according to control parameters
;
; Input:
;	ctrl	Control data structure
;	sigbuf	SRD data structure
;
;---------------------------------------------------------


        id_base =  WIDGET_BASE(basid, ROW=3, /FRAME)
        id_1 = WIDGET_BASE(id_base,/align_center,/ROW)
	junk = WIDGET_LABEL(id_1, VAL='Deglitch using following parameters:',  $  
               FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
	junk = WIDGET_BUTTON( id_1, VAL=" Deglitch ", UV="DEGLITCH" )
;junk = WIDGET_BUTTON( id_1, VAL=" PATCHING  ", UV="IMPLE")   
	junk = WIDGET_BUTTON( id_1, VAL="  Help    ", UV="HELP" )

        base1 = WIDGET_BASE(id_base, /ROW)

	id_0 = WIDGET_BASE(base1, /COL)
	junk = WIDGET_LABEL(id_0, VAL='Fraction bad readouts')     
        id_1 = WIDGET_BASE(id_0, /ROW)
        (*cm(2)).fracbad = WIDGET_TEXT(id_1, VAL=strtrim(dgc.fracb,2), UVAL="JUNK", /editable, ysiz=1)
        junk = WIDGET_LABEL(id_1, VAL='%')
 
        id_0 = WIDGET_BASE(base1, /COL)
	junk = WIDGET_LABEL(id_0, VAL='Time const. spike tails')    
        id_1 = WIDGET_BASE(id_0, /ROW)
        (*cm(2)).tglitch = WIDGET_TEXT(id_1, VAL=strtrim(dgc.tglch), UVAL="JUNK", /editable, ysiz=1)
        junk = WIDGET_LABEL(id_1, VAL='s')

	id_0 = WIDGET_BASE(base1, /COL)
	junk = WIDGET_LABEL(id_0, VAL='Spike inclusion threshold')    
        id_1 = WIDGET_BASE(id_0, /ROW)
        (*cm(2)).thresh1 = WIDGET_TEXT(id_1, VAL=strtrim(dgc.thres1,2), UVAL="JUNK", /editable, ysiz=1)
        junk = WIDGET_LABEL(id_1, VAL='V/s')

 
	id_0 = WIDGET_BASE(base1, /COL)
	junk = WIDGET_LABEL(id_0, VAL='Spike clipping threshold')    
        id_1 = WIDGET_BASE(id_0, /ROW)
        (*cm(2)).thresh2 = WIDGET_TEXT(id_1, VAL=strtrim(dgc.thres2,2), UVAL="JUNK", /editable,ysiz=1)
        junk = WIDGET_LABEL(id_1, VAL='V/s')

        (*cm(2)).irep =  dgc.irep
 
 	id_2 = WIDGET_BASE(base1, /COL) 
 	junk = WIDGET_LABEL(id_2, VAL='Patch')
 	id_3 = WIDGET_BASE(id_2, /COL ,/NONEXCLUSIVE)
 	irep = WIDGET_BUTTON( id_3, VAL=" ",  UV="IRE")
         IF dgc.irep EQ 1 THEN $       
           WIDGET_CONTROL, irep, /set_button
			 
 return
end

;---------------------------------------------------------
; Draw data according to control parameters
;
; Input:
;	ctrl	Control data structure
;	sigbuf	SRD data structure
;
;---------------------------------------------------------

pro vt_redraw, ctrl, sigbuf

common dg_out, xgl, gl_erd, gl_spikes, xpatch, spatch
@common_e32
@common_plotter
@common_steerin
@colour_indices

print,'dgflag',(*ctrl).dgflag
debug = 0
set_colours
help, xgl, gl_erd, gl_spikes

;----------------------------
; autoscaling

  ix = where((*sigbuf).flag EQ 0,cnt)
  if debug then print,'cnt=',cnt
  ymin = min((*sigbuf).slop(ix))
  ymin = min([ymin,0.0])
  ymax = max((*sigbuf).slop(ix))
  deltay = ymax - ymin
  if debug then print,ymin,ymax


if (*ctrl).scale EQ 0 AND (*ctrl).disp EQ 0 then begin
  (*ctrl).xmin = min((*sigbuf).stim)
  (*ctrl).xmax = max((*sigbuf).stim)
  (*ctrl).ymin = ymin - 0.3*deltay
  (*ctrl).ymax = ymax + 1.5*deltay
endif

if (*ctrl).scale EQ 0 AND (*ctrl).disp EQ 1 then begin
  (*ctrl).xmin = min((*sigbuf).stim(ix))
  (*ctrl).xmax = max((*sigbuf).stim(ix))
  (*ctrl).ymin = ymin - 0.3*deltay
  (*ctrl).ymax = ymax + 0.3*deltay
endif


;----------------------------
;plot all valid ones

ix = where((*sigbuf).flag EQ 0,cnt)
if debug then print,'plot',cnt
if cnt GT 0 then begin

  stime  = (*sigbuf).stim(ix) 
  slope = (*sigbuf).slop(ix)
  err   = (*sigbuf).slun(ix)
  if cnt EQ 1 THEN BEGIN
    stime  = [stime,stime]
    slope = [slope,slope]
    err   = [err,err]
  endif

  wset, (*ctrl).wdraw
  if debug then print,'plot valid data', (*ctrl).symb
  if debug then print,(*ctrl).xmin,(*ctrl).xmax
  plot, xrange=[(*ctrl).xmin,(*ctrl).xmax], $
	yrange=[(*ctrl).ymin,(*ctrl).ymax], $
	xstyle=3, ystyle=3, $
        [1,1], [1,1], $
;;	stime, slope, $
	psym=(*ctrl).symb, $
;	title='Pixel # '+string((*ctrl).cur_pix, format='(I3)'), $
	xtitle='time [s]', ytitle='signal [V/s]', symsize=0.5, /nodata
  oplot, stime, slope, color=red, psym=(*ctrl).symb

  if (*ctrl).plerr EQ 1 and cnt GT 0 then begin
    errplot, stime, slope-err, slope+err  
  endif

 
; mn1 = avg(slope) & sg1 = sigma(slope) & md1 = median(slope)
; fromto = [stime(0), stime(n_elements(stime)-1)]
; oplot, fromto, [mn1,mn1], linestyle=2
; oplot, fromto, [mn1-sg1,mn1-sg1], linestyle=1
; oplot, fromto, [mn1+sg1,mn1+sg1], linestyle=1
; oplot, fromto, [md1,md1], linestyle=3
  

endif

;---------------------------------
; plot all invalid ones

if (*ctrl).disp EQ 0 then begin

 ix = where((*sigbuf).flag GT 0,cnt)

 if cnt GT 0 then begin
  stime  = (*sigbuf).stim(ix) 
  slope = (*sigbuf).slop(ix)
  err   = (*sigbuf).slun(ix)

  if cnt EQ 1 THEN BEGIN
    stime = [stime,stime]
    slope=[slope,slope]
    err   = [err,err]
  ENDIF

  if (*ctrl).symb EQ 3 then psym=3 $
  else psym = 7

  if debug then print,'plot invalid data', psym
  oplot, stime, slope, psym=psym, color=yellow
  if (*ctrl).plerr EQ 1 then begin
    errplot, stime, slope-err, slope+err  
  endif

 endif

endif 

;---------------------------------
; plot slew data


help,(*sigbuf), /str
ix = where((*sigbuf).fpts EQ 0 AND (*sigbuf).flag EQ 0,cnt)
 print,'slew valid',cnt
if debug then print,'slew valid',cnt

if cnt GT 0 then begin
 stime  = (*sigbuf).stim(ix)
 slope = (*sigbuf).slop(ix)
 err   = (*sigbuf).slun(ix)

 if cnt EQ 1 THEN BEGIN
   stime = [stime,stime]
   slope=[slope,slope]
   err   = [err,err]
 ENDIF

  if debug then print,'plot slew  data', (*ctrl).symb
 oplot, stime, slope, psym=(*ctrl).symb, color=blue, symsize=0.5
endif

if debug then print,'disp',(*ctrl).disp
if (*ctrl).disp EQ 0 then begin
 ix = where(fpoint EQ 0 AND (*sigbuf).flag GT 0 ,cnt)
 if debug then print,'slew invalid',cnt
  print,'slew invalid',cnt

 if cnt GT 0 then begin
  stime  = (*sigbuf).stim(ix) 
  slope = (*sigbuf).slop(ix)
  err   = (*sigbuf).slun(ix)

  if cnt EQ 1 THEN BEGIN
    stime = [time,time]
    slope=[slope,slope]
    err   = [err,err]
  endif


  if (*ctrl).symb NE 3 then $
  	oplot, stime, slope, psym=psym, color=blue, symsize=0.5

 endif
 
endif 

;plot degliches and spikes
if (*ctrl).dgflag GT 0 then begin
   ymaxp = ymax + 0.3*(ymax-ymin)
   print,'max',ymax,ymaxp
   oplot,xgl,  $
         ymaxp-1.0*(ymaxp-ymax)+(ymaxp-ymax)*gl_spikes,color=cyan  
   oplot,xgl,  $
         ymaxp-1.0*(ymaxp-ymax)+(ymaxp-ymax)*gl_erd
   help,xpatch,spatch
;  print,xpatch,spatch
   if n_elements(xpatch) gt 1 then begin
     oplot, xpatch, spatch, color=green, psym=(*ctrl).symb
   endif
endif 

;---------------------------------
; plot chopper sweep boundaries

  if debug then print,'plot choppers '
  stime = (*sigbuf).stim
  nii = n_elements(stime)
  itest=lonarr(nii)
  itest(1:nii-1)=step(1:nii-1)-step(0:nii-2)
  if(array eq 'c100')then chswlim=12
  if(array eq 'c200')then chswlim=6
  iii=where(abs(itest) ge chswlim,niii)
  if debug then print,'nchop',niii
  if(niii gt 0)then begin
    xpc=stime(iii(0:niii-1)) ; plot versus time
    lin =  (*ctrl).ymax
    linarr = FLTARR(niii) + lin
    for i=0, n_elements(xpc)-1 do $
      oplot, [xpc(i), xpc(i)], [(*ctrl).ymin, linarr(i)], linestyle=1
  endif


;---------------------------------
; plot initially invalid ones

;if (*ctrl).disp EQ 0 then begin
;
; ix = where((*sigbuf).flag GT 0,cnt)
; if cnt GT 0 then begin
;  time  = (*sigbuf).stim(ix) 
;  slope = (*sigbuf).slop(ix)
;  err   = (*sigbuf).slun(ix)
;
;  if cnt EQ 1 THEN BEGIN
;    time = [time,time]
;    slope=[slope,slope]
;    err   = [err,err]
;  ENDIF
;
; oplot, stime, slope, psym=5, color=green, symsize=0.8
; endif
; 
;endif 

;if (*ctrl).plerr EQ 1 and cnt GT 0 then begin
;  oploterr, time, slope, err, 3, color = 118
;endif


;if (*ctrl).s_flag((*ctrl).cur_pix) EQ 1 THEN $		;overplot smooth line
;  oplot,(*sigbuf).stim , (*ctrl).s_smooth(((*ctrl).cur_pix),*)



end



;---------------------------------------------------------
;Convert diagonal corner coordinates of rectangle from
;device to data coordinates and exchange 
;so that x1 < x2 and y1 < Y2
;
; Input: 
;	 a	x 2-el.-Vector in device coord.
;        b	y 2-el.-Vector in device coord.
;
; Output:
;	 x	x 2-el.-Vector in data coord.
;        y	y 2-el.-Vector in data coord.
;
;---------------------------------------------------------


pro vt_coor2data, a, b, x, y

dc = convert_coord([a(0),a(1)], $
		   [b(0),b(1)], $
		/device, /to_data)
	
;reverse order of cornerpoints if necessary

if dc(0,0) GT dc(0,1) THEN dc(0,*) = dc(0,[1,0])
if dc(1,0) GT dc(1,1) THEN dc(1,*) = dc(1,[1,0])

x = dc(0,*)
y = dc(1,*)

end

;---------------------------------------------------------
;Overplot rectangle
;
; Input: 
;	 x	x 2-el.-Vector
;        y	y 2-el.-Vector
; Keywords:
;	linestyle	same as for plot
;	color		same as for plot
;---------------------------------------------------------


pro vt_plrect, x, y, ctrl, color=color, linestyle=linestyle

wset, (*ctrl).wdraw
oplot, x, [y(0),y(0)], color=color, linestyle=linestyle
oplot, [x(1),x(1)], y, color=color, linestyle=linestyle
oplot, x, [y(1),y(1)], color=color, linestyle=linestyle
oplot, [x(0),x(0)], y, color=color, linestyle=linestyle

end


;---------------------------------------------------------
; Flagging of Datapoints within rectangle
;
; Input: 
;	x	x 2-el.-Vector
;       y	y 2-el.-Vector
;	sigbuf	signal data structure
; Keywords:
;	/unhide	if set pixels are flagged valid
;---------------------------------------------------------

pro vt_flagrect, x, y, sigbuf, unhide=unhide


x1 = x(0) & y1 = y(0)
nx = x(1) - x1 & ny = y(1) - y1
stime = (*sigbuf).stim(*)
slope = (*sigbuf).slop(*)
flag  = (*sigbuf).flag(*)

if NOT keyword_set(unhide) THEN BEGIN
 
 bdum = where((*sigbuf).flag gt 0, fcnt)
 ix = where((*sigbuf).stim(*) GE x1 $		;hide
  AND  stime LE x1+nx  AND  slope GE y1 $
  AND  slope LE y1+ny, cnt)	     

 
 if cnt GT 0 then $
  (*sigbuf).flag(ix) = (*sigbuf).flag(ix) OR 01b

ENDIF ELSE BEGIN

 ix = where((*sigbuf).stim(*) GE x1 $		;unhide
  AND  stime LE x1+nx  AND  slope GE y1 $
  AND  slope LE y1+ny AND  $
  (flag  GT 0), cnt)	     

 if cnt GT 0 then $
  (*sigbuf).flag(ix) = (*sigbuf).flag(ix) AND 00b

ENDELSE
			
END


;---------------------------------------------------------
; Zoom into rectangle
;
; Input: 
;	x	x 2-el.-Vector
;       y	y 2-el.-Vector
;	ctrl	control data structure
;---------------------------------------------------------

pro vt_zoomin, x, y, ctrl

(*ctrl).xmin = x(0)
(*ctrl).xmax = x(1)
(*ctrl).ymin = y(0)
(*ctrl).ymax = y(1)

END


;---------------------------------------------------------
; Main routine
;
;
;
; Input: 
;	sigbufptr
;		in pointer to signal buffer
;
;---------------------------------
pro pia_erd2map_dg, n1, n2, pix, pid  
@common_signal
@common_timer  
@common_steerin
@common_e32    

orig = where(ireduce eq 0, norig) ;list of transmitted points in EXPAND32

print,'norig',norig
if(norig ne nptin)then begin
  print,'DEGLITCHER aborting:  number of good points in expanded data not NPTIN'
  return      
endif ; norig ne nptin

fpt = fpoint(orig)
help,fpt,fpoint


startendpoint,n1,n2,$
  irasterstart,irasterstop,ip1,ip2,$
  irasterstarte,irasterstope,ieplot1,ieplot2

print,'pix n1 n2',pix,n1,n2,ip1,ip2
dgc = (*pid).dgq

sigbuf = {stim:time(ip1:ip2),  $
          slop:reform(s(pix,ip1:ip2)),     $
          slun:reform(rms(pix,ip1:ip2)),    $
          flag: reform(bad(pix,ip1:ip2)),  $   
          fpts: fpt(ip1:ip2) }

help, sigbuf, /str

help,sigbuf,/str
help,bad
print,min(s(pix,*),max=smax), smax
print,min(time,max=maxt),maxt

sigbufptr = ptr_new(sigbuf)
;---------------------------------
; define control structures
xmin = min((*sigbufptr).stim, max=xmax)
ymin = min((*sigbufptr).slop,max=ymax)
ymin = min([ymin,0.0])


ctrl = {	 $
      cur_pix:pix, $	      ;currently drawn measurement
      n1:n1,    $
      n2:n2,    $
      sind: ip1, $
      eind:ip2, $
      xmin:xmin,  $                    ;min. of xrange
      xmax:xmax,  $                    ;max. of xrange
      ymin:ymin,  $                  ;min. of yrange
      ymax:ymax,  $                   ;max. of yrange
      scale:0,  	       $  ;scaling: 0=free, 1=fixed
      disp:0,		       $  ;display: 0=all, 1=valid only
      symb:0,		       $  ;data display: 0=dots, 1=symbols
      plerr:0,  	       $  ;data display: 0=no errorbars, 1=plot errorbars
      flagging:0, $  ;flagging state: 0=inactive, 1=hide, 2=unhide
      dgflag:0, $
      cx1:0,  $  ;cursor x position at button press
      cy1:0,  $  ;cursor y position at button press
      cx2:0,  $  ;previous cursor x position
      cy2:0,  $  ;previous cursor y position
      rect:0, $  ;rectangle state: 0=inactive, 1= activated
      zoom:0, $  ;zoom state: 0=inactive, 1=zoom activated
      param:dgc,   $ ;deglitching parameters
      wdraw:0  } ;draw window number for wset()

widg = {	$
      base:0L,  $     ;structure ifor all widget id's
      lcol:0L,  $
      rcol:0L,  $
      labl:0L,  $
      draw:0L,  $
      slidb:0L, $
      back:0L,  $
      slid:0L,  $
      next:0L,  $
      fracbad:0L,$
      tglitch:0L,$
      thresh1 :0L,$
      thresh2:0L,$
      irep:0,$
      nsig1:0L, $
      delta_t:0L,$
      nsig2:0L, $
      niter:0L, $
      navi:0L,  $
      exit:0L,  $
      quit:0L,  $
      up:0L,	$
      lrbut:0L, $
      left:0L,  $
      right:0L, $
      down:0L,  $
      scal:0L,  $
      xscal:0L, $
      free:0L,  $
      fix:0L ,  $
      bzoom:0L, $
      zin:0L,	$
      zout:0L,  $
      disp:0L,  $
      xdisp:0L, $
      all:0L,	$
      valid:0L, $
      bflag:0L, $
      dasel:0L, $
      hide:0L,  $
      unhide:0L,$
      xsymb:0L, $
      dots:0L,  $
      symb:0L,  $
      xperr:0L, $
      noerr:0L, $
      plerr:0L, $
      clip:0L   }



cm = ptrarr(6)
cm(0)= ptr_new(ctrl)
cm(1)= sigbufptr
cm(2)= ptr_new(widg)
cm(3)= ptr_new((*sigbufptr).flag)
cm(4)= ptr_new((*sigbufptr).slop)
cm(5)= pid


;---------------------------------
; construct window

group=!PIA_TOP_WID
(*cm(2)).base  = WIDGET_BASE(TITLE='Deglitcher',row=2, uvalue=cm , $
                             /MODAL, GROUP=group)
help,dgc,/str
upbase = WIDGET_BASE((*cm(2)).base, /row)
(*cm(2)).lcol  = WIDGET_BASE(upbase,/column)
(*cm(2)).rcol  = WIDGET_BASE(upbase,/column)

		
(*cm(2)).draw  = WIDGET_DRAW((*cm(2)).lcol,xsize=600,ysize=512, $
		uvalue='DRAW', /BUTTON_EVENTS, /MOTION_EVENTS)

dnbase = WIDGET_BASE((*cm(2)).base, /col, /align_center)
make_dg_base , dnbase, dgc, cm

;---------------------------------
 
;(*cm(2)).slidb = WIDGET_BASE((*cm(2)).lcol,/row, /align_center)
;
;if n_elements((*sigbufptr).slop) GT 1 THEN BEGIN
;(*cm(2)).back  = WIDGET_BUTTON((*cm(2)).slidb, value='Back', uvalue='BACK')
;(*cm(2)).slid  = WIDGET_SLIDER((*cm(2)).slidb, title='Pixel',uvalue='SLIDER', $
;			    value=0,min=0,max=n_elements((*sigbufptr).slop)-1)
;(*cm(2)).next  = WIDGET_BUTTON((*cm(2)).slidb, value='Next', uvalue='NEXT')
;endif




;---------------------------------


(*cm(2)).navi  = WIDGET_BASE((*cm(2)).rcol,/column, /frame, /align_center)
w1             = WIDGET_LABEL((*cm(2)).navi, VALUE = 'Navigate',  $
               FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')

(*cm(2)).up    = WIDGET_BUTTON((*cm(2)).navi, VALUE = 'Up',    UVALUE = 'UP')

(*cm(2)).lrbut = widget_base((*cm(2)).navi,/row, /align_center)
(*cm(2)).left  = WIDGET_BUTTON((*cm(2)).lrbut, VALUE = 'Left',  UVALUE = 'LEFT')
(*cm(2)).right = WIDGET_BUTTON((*cm(2)).lrbut, VALUE = 'Right', UVALUE = 'RIGHT')

(*cm(2)).down  = WIDGET_BUTTON((*cm(2)).navi, VALUE = 'Down',  UVALUE = 'DOWN')

(*cm(2)).bzoom  = WIDGET_BASE((*cm(2)).navi, /row, /align_center)
(*cm(2)).zin    = WIDGET_BUTTON((*cm(2)).bzoom, VALUE = 'Zoom In',  UVALUE = 'ZOOM_IN')
(*cm(2)).zout   = WIDGET_BUTTON((*cm(2)).bzoom, VALUE = 'Zoom Out',  UVALUE = 'ZOOM_OUT')


(*cm(2)).scal  = WIDGET_BASE((*cm(2)).rcol, /col,/frame,/align_center)
w1              = WIDGET_LABEL((*cm(2)).scal, VALUE = 'Scaling')
(*cm(2)).xscal  = WIDGET_BASE((*cm(2)).scal, /row, /exclusive,/align_center)
(*cm(2)).free   = WIDGET_BUTTON((*cm(2)).xscal, VALUE = 'Free',  UVALUE = 'FREE')
(*cm(2)).fix    = WIDGET_BUTTON((*cm(2)).xscal, VALUE = 'Fix',  UVALUE = 'FIX')


(*cm(2)).disp  = WIDGET_BASE((*cm(2)).rcol, /col,/frame)
w1              = WIDGET_LABEL((*cm(2)).disp, VALUE = 'Display',  $
               FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
(*cm(2)).xdisp  = WIDGET_BASE((*cm(2)).disp, /row, /exclusive)
(*cm(2)).all    = WIDGET_BUTTON((*cm(2)).xdisp, VALUE = 'All',  UVALUE = 'ALL')
(*cm(2)).valid   = WIDGET_BUTTON((*cm(2)).xdisp, VALUE = 'Valid Only',  UVALUE = 'VALID')
(*cm(2)).xsymb  = WIDGET_BASE((*cm(2)).disp, /row, /exclusive)
(*cm(2)).dots   = WIDGET_BUTTON((*cm(2)).xsymb, VALUE = 'Dots'   ,  UVALUE = 'DOTS')
(*cm(2)).symb   = WIDGET_BUTTON((*cm(2)).xsymb, VALUE = 'Symbols',  UVALUE = 'SYMBOLS')
(*cm(2)).xperr  = WIDGET_BASE((*cm(2)).disp, /row, /exclusive)
(*cm(2)).noerr  = WIDGET_BUTTON((*cm(2)).xperr, VALUE = 'No Errorbars'   ,  UVALUE = 'NOERR')
(*cm(2)).plerr  = WIDGET_BUTTON((*cm(2)).xperr, VALUE = 'Errorbars',  UVALUE = 'PLERR')

(*cm(2)).bflag  = WIDGET_BASE((*cm(2)).rcol, /col, /align_center,/frame)
w1              = WIDGET_LABEL((*cm(2)).bflag, VALUE = 'Manual Deglitching', $
               FONT='-adobe-courier-bold-r-normal--12-120-75-75-m-70-iso8859-1')
(*cm(2)).dasel  = WIDGET_BASE((*cm(2)).bflag, /row, /align_center)
(*cm(2)).hide    = WIDGET_BUTTON((*cm(2)).dasel, VALUE = 'Select',UVALUE = 'HIDE')
(*cm(2)).unhide  = WIDGET_BUTTON((*cm(2)).dasel, VALUE = 'Deselect',  UVALUE = 'UNHIDE')


w1              = WIDGET_LABEL((*cm(2)).rcol, VALUE = '  ')
w1              = WIDGET_LABEL((*cm(2)).rcol, VALUE = '  ')
         quit  = WIDGET_BUTTON((*cm(2)).rcol, VALUE = 'Quit (Discard Changes)',  $
                                /MENU,  UVALUE = 'JUNK')
(*cm(2)).quit  = WIDGET_BUTTON( quit, VALUE ="        Are You Sure?        ", UV="QUIT" )
(*cm(2)).exit  = WIDGET_BUTTON((*cm(2)).rcol, VALUE = 'Exit', UVALUE = 'EXIT')



;-----------------------------------------
;Initialize radio buttons

widget_control, set_button=1, (*cm(2)).free
widget_control, set_button=1, (*cm(2)).all
widget_control, set_button=1, (*cm(2)).symb
widget_control, set_button=1, (*cm(2)).noerr

;-----------------------------------------
; Initialize flags

(*cm(0)).zoom = 0
(*cm(0)).rect = 0
(*cm(0)).symb = 6
(*cm(0)).plerr = 0
(*cm(0)).disp  = 0

;for i=0L, n_elements((*cm(0)).s_flag)-1L do (*cm(0)).s_flag = 0
;-----------------------------------------
; Draw window


widget_control, /realize, (*cm(2)).base

widget_control, (*cm(2)).draw, get_value = tmp
(*cm(0)).wdraw = tmp

help,bad
vt_redraw, cm(0), cm(1)
help,bad


xmanager, 'pia_erd2map_dg', (*cm(2)).base, /NO_BLOCK, GROUP=group



END

