pro preparedrift,idriftflag,$
driftincrem,isdriftsave,sdriftsave,idefcal,backminv,backpredv,pixarray
;
; to do: DRIFT32 add a routine compress32 and execute it in DRIFT32
;
; this routine ensures that only positive data, at a level roughly compatible
; with what is known about the source, are used for the drift correction 
; routines.
;
; for each pixel either makes an addition, DRIFTINCREM, such that the 
; background level in input data is BACKPRED 
; translated into V/s (this is case for INEGMETHOD=1). Or (INEGMETHOD=0)
; retain the input data without change, but perform checks.
; 
; Then any remaining points which are not set bad in the input dataset and 
; are less than the (internal parameter) minimum threshold brightness in 
; MJy/st BACKMIN are set to 1 in the temporary mask IDRIFTFLAG. The same points
; are set to a low positive value (same for all pixels) which are stored in
; the 1D array SDRIFTSAVE (exactly the right length to accommodate the values)
; and the corresponding indices of S are stored in the 1D long array
; ISDRIFTSAVE. IDRIFTFLAG is not added to BAD, as we wish to use the points
; marked 1 in IDRIFTFLAG in the drift correction.
;
; In the calling routine (normally DRIFT32), after the drift corrections 
; have been found (but before implementation in S), the points marked by 
; ISDRIFTSAVE should be restored to S, followed by subtraction of DRIFTINCREM.
; This is the exact reverse of the procedure here in PREPAREDIFT.
; 
; NB this routine has a safety feature to prevent it being called more
; than once as it alters S and saves
; the changes in SDRIFTSAVE. Multiple calls will result in information in S
; being lost.
;
; input (common NAMES):
;    TDTNAM: string denoting name of TDT eg '53801001'
;    ISEQ:   string denoting sequence number of target source map
;            (normally '01' or '03' or '05' for multifilter sequence separated
;             by FCS measurements) 
;    DIRNAM: dstring denoting irectory name for internal PIA files
;            eg dirnam='/d13/vlk/rjt/pia/p32p22p25/v7_3/'
; input:
;    IDEFCAL = 1 for translation between V/s and MJy/st 
;                with default responsivities
;            = 0 for translation between V/s and MJy/st 
;                with actual (fcs) responsivities
;    BACKPRED (common DRIFTPAR) Predicted background in MJy/St
;    PIXARRAY = (integer*NPIX) array of pixels to process  
;
; input from common block DRIFTPAR (set in routine or read in)
;
;       INEGMETHOD = 1 to scale mean level to BACKPRED in V/S in
;                      routine PREPAREDRIFT.
;                      (should be used in underexposed or high dynamic range
;                       observations if hook response makes level negative
;                       or wrong
;                  = 0 to avoid scaling to BACKPRED.
;                  Note that either all pixels are scaled or none. 
;                  Routine should if possible be run with INEGMETHOD=0 
;       BACKPRED predicted background level in MJy/sterad. Should be found from
;                IRSKY taken at position of observations (ie within 6 x 6
;                arcmin area, or from previous
;  input from common block SIGNAL
;       S(npix,nptin)
;       BAD(npix,nptin) 
;  input from common block PLOTTER
;       YREL,ZREL (double, nptin) offsets in Y, Z from map centre in arcsec
;  input from common block STEERIN
;       NPTIN
;  output
;       DRIFTINCREM(npix) temporary DC additions made to S in case INEGMETHOD=1
;       S(npix,nptin)     S with (i) temporary addition of DRIFTINCREM and
;                               (ii) low values even after addition of
;                                    DRIFTINCREM substituted by a low 
;                                    positive value. 
;       IDRIFTFLAG(npix,nptin) mask array set to 1 for points where 
;                         input BAD was zero but 
;                         S values zero or smaller even after addition of 
;                         DRIFTINCREM
;       ISDRIFTSAVE       1D array containing indices of S for which 
;                         input BAD was zero but 
;                         S values zero or too small even after addition of 
;                         DRIFTINCREM
;       SDRIFTSAVE        1D array containing input values of S for which
;                         input BAD was zero but 
;                         S values zero or too small even after addition of
;                         DRIFTINCREM
;       BACKMINV          (real array dimension NPIX)
;                         Minimum allowed (positive) illumination in
;                         V/s (found from internal parameter BACKMIN which is
;                         minimum brightness in MJy/sterad) for addmittance of
;                         data and whether a shift in the background is 
;                         required.
;       BACKPREDV         (real array dimension NPIX). Best prediction of
;                         background in V/s. Needed by DRIFTCORR to 
;                         fix search region in the optimisation. 
;
;common names,tdtnam,iseq,dirnam,caldir,cfilter
@common_names
;common driftpar,beta,tau,imodel,ndriftsolve,islewmethod,backpred,inegmethod
@common_driftpar
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_signal
;common plotter,otfstart,otfstop,rdstat,chop,step,otf,rpid,yrel,zrel,$
;ytrue,ztrue,yzrpidcen,nyraster,nzraster
@common_plotter
;common steerin,array,nptin,npt,ninramp,imeas,osamp
@common_steerin
@common_map3dref.pro
;common map3dref,map3dref,map3drefrms,map3drefd,map3drefy,map3drefz,map3drefn
@common_dg2block
;common dg2block,idg2badvalue1,idg2badvalue2,dg2bad
;
@phterd
;
; prevent successive calls corrupting S
;
if(n_elements(isdriftsave) gt 0)then begin
if(max(s(isdriftsave)) eq min(s(isdriftsave)))then begin
; S has already been altered by a previous call so inhibit processing
print,'PREPAREDRIFT aborting as successive changes to S not allowed...'
print,'... to rerun PREPAREDRIFT first restore S with ...'
print,'... s(isdriftsave)=sdriftsave ...
print,'... followed by a loop s(ipix,*)=s(ipix,*)-driftincrem(ipix)'
goto, c999
endif ; max(s(isdriftsave)) eq min(s(isdriftsave))
endif ; n_elements(isdriftsave) gt 0
;
;
; the following are internal steering parameters
;backmin=backpred/4.0
;
backmin=0.1 ; minimum allowed background in MJy/sterad. Note that this
 ; must be set low as an expected characteristic of the drift-corrected
 ; solutions are sharp falls to low signals when coming off source,
 ; in contrast to the signal which follows a slow quasi exponential decline
 ; also we must not bias the signal when signal to noise is low on a low
 ; background
backfract=0.25 ; determines statistical level of background -
;               0.1 is too small as switch on drifts are admitted and
;               0.5 is too large as source is admitted. 0.25 is normally OK.
;               Will bias background to lower
;               than real
;               than rreality if there is no sky structure and maxfractneg is
;               < 0.5. 
maxfractneg=0.25
;               fraction of points at whose level the
;               background should be positive. This couold occur eg due
;               to the hook effect. 
;               RJT NOV99 - changed from 0.1 to 0.25. Also, only a warning
;               is issued, no abort.
;
;print,'FINDBACK: backmin,backfract,maxfractneg = ',$
;backmin,backfract,maxfractneg
;
c555:
;
npix=n_elements(s(*,0))
;help,s,npix

driftincrem=fltarr(npix) ; must always be initialised
idriftflag=bad
idriftflag(*,*)=0
sav=fltarr(npix)
smed=sav
newbad=lonarr(npix)
;
; First determine backgrounds BACK(ipix) in MJy/s
;
findback,backfract,backraw,0 ; method from map
back=backraw ; in V/s
;help,back
;if(idefcal eq 1)then cal32,result1,result2,1 ; default
;if(idefcal eq 0)then cal32,result1,result2,0 ; actual
;
;CG - take now conversion routine - take result2 = result1 in this case

		get_conv_s2b, imeas, idefcal, result1
result2=result1

for ipix=0,npix-1 do begin ; correct for non-linearity (do loop for all pixels)
;
bkx=back(ipix) 
nonlincorr,ipix,bkx,bkxcorr
fcorr1=bkxcorr/bkx ; non-linearity correction for background
bkx=result2(ipix)
nonlincorr,ipix,bkx,bkxcorr
fcorr2=bkxcorr/bkx ; non-linearity correction for reference input V/s level
back(ipix)=back(ipix)*result1(ipix) ; now in MJy/sterad using non-linearity 
;                                correction at input V/s level given by RESULT2
back(ipix)=back(ipix)*fcorr1/fcorr2 ;MJy/sterad using non-linearity correction 
;                           appropriate for uncorrected V/s level from FINDBACK
;
endfor ; ipix=0,npix-1
;
if(idefcal eq 0)then print,$
'PREPAREDRIFT: predicted background/MJy/St = ',backpred
if(idefcal eq 0)then print,$
'PREPAREDRIFT: measured backd/MJy/St(act),/V/s for backfract',$
backfract 
if(idefcal eq 1)then print,$
'PREPAREDRIFT: measured backd/MJy/St(def),/V/s for backfract',$
backfract 
;
npixloop=n_elements(pixarray)
backadd=fltarr(npix) ; initialisation
;
for pix=0,npixloop-1 do begin ; ipix loop 1
ipix=pixarray(pix)
print,'ipix, (backgd (MJy/st,V/s) = ',ipix,back(ipix),backraw(ipix)
endfor ; pix
;
if(inegmethod eq 0)then begin ; check that no pixels have too many negative
;                               points
;print,'FINDBACK: running with INEGMETHOD = ',inegmethod
findback,maxfractneg,back,0 ; from map method
;back=back*result1 ; rjt added 051099 this converts back to MJy/sterad
; above commented Apr20/00 as back already exists as MJy/sterad 
; following two CAL32 statements not necessary since this has been done above
;if(idefcal eq 1)then cal32,result1,result2,1 ; default
;if(idefcal eq 0)then cal32,result1,result2,0 ; actual
;
for pix=0,npixloop-1 do begin ; ipix loop 2
ipix=pixarray(pix)
igood=where(bad(ipix,*) eq 0,nigood)
if(back(ipix) le backmin)then begin
print,'PREPAREDRIFT: warning - for pixel IPIX more than a fraction...'
print,'...MAXFRACTNEG of good data points are fainter than BACKMIN MJY/St'
print,'...IPIX, MAXFRACTNEG, BACK(IPIX), BACKMIN = ',$
ipix,maxfractneg,back(ipix),backmin
print,'...consider rerunning with INEGMETHOD=1'
;goto, c999 commented by RJT 29/11/99. We must allow negative data for 
;           overshooting.
endif
endfor ; pix
;
endif ; inegmethod eq 0
;
if(inegmethod eq 1)then begin
;
;print,'PREPAREDRIFT: running with INEGMETHOD = ',inegmethod
print,'PREPAREDRIFT - adding increments onto data in (V/s) ... '
;
for pix=0,npixloop-1 do begin ; ipix loop 3
ipix=pixarray(pix)
;
backadd(ipix)=backpred-back(ipix) ; in MJy/sterad
;driftincrem(ipix)=backadd(ipix)/result1(ipix)
mjtovs,ipix,idefcal,backpred,backdummy
driftincrem(ipix)=backdummy-backraw(ipix)
print,'... ipix,backadd (MYy/St),(V/s) = ',ipix,backadd(ipix),driftincrem(ipix)
s(ipix,*)=s(ipix,*)+driftincrem(ipix)
; added JUL00 RJT
map3dref(ipix,*,*)=map3dref(ipix,*,*)+driftincrem(ipix)
;
endfor ; pix loop 3
;
endif ; inegmethod eq 1
;
; find points lower than BACKMINV
;
;print,'PREPAREDRIFT: statistics of low points:'
;backminvall=backmin/(total(result1)/float(npix))
;
backminv=fltarr(npix)
backpredv=backminv
;
for pix=0,npixloop-1 do begin ; ipix loop 4
ipix=pixarray(pix)
;
mjtovs,ipix,idefcal,backmin,backmindummy
mjtovs,ipix,idefcal,backpred,backpreddummy
;backminv(ipix)=backmin/result1(ipix)
;backpredv(ipix)=backpred/result1(ipix)
backminv(ipix)=backmindummy
backpredv(ipix)=backpreddummy
;inewbad=where(bad(ipix,*) eq 0 and $
;s(ipix,*)*result1(ipix) le BACKMIN,ninewbad)
inewbad=where(bad(ipix,*) eq 0 and $
s(ipix,*) le backminv(ipix),ninewbad)
newbad(ipix)=ninewbad
if(ninewbad gt 0)then begin
;idriftflag(ipix,inewbad)=1 ; commented RJT NOV99 to preserve negative data
endif ; ninewbad gt 0
test=float(ninewbad)/float(nptin)
print,'PREPAREDRIFT: ipix,fraction low = ',ipix,test
;
endfor ; pix loop 4
;
; the following IF loop (no comments) was commented by RJT NOV 99 to preserve
; negative data for the overshooting
;if(total(idriftflag) gt 0)then begin
;isdriftsave=where(idriftflag eq 1,nisdriftflag)
;sdriftsave=s(isdriftsave) ; note this is done after addition of driftincrem
;print,'PREPAREDRIFT: setting NISDRIFTFLAG low points in S to BACKMINVALL ...'
;print,'... NISDRIFTFLAG,BACKMINVALL (V/s) = ',nisdriftflag,backminvall
;s(isdriftsave)=backminvall  ; set to positive value roughly equal to 
;                          BACKMIN in V/s
;endif ; total(idriftflag) gt 0
;
if(total(idriftflag) le 0)then begin
sdriftsave=0.0
isdriftsave=-1
;print,'PREPAREDRIFT: no points in S reset'
endif ; total(idriftflag) le 0
;
; the following two lines added RJT NOV99
;
sdriftsave=0.0
isdriftsave=-1
;
;bad=bad+idriftflag ; don't do this since most low points are due to noise
;                     and should be included in the drift reconstruction
;                     as values reset to low positive value BACKMINV (above) 
;
for pix=0,npixloop-1 do begin ; print loop ipix loop 5
ipix=pixarray(pix)
;if(pix eq 0)then print,'PREPAREDRIFT....'
print,'... ipix,driftinc(V/s, MJy/st),n =',$
byte(ipix),float(driftincrem(ipix)),float(backadd(ipix)),newbad(ipix)
endfor ; pix print loop ipix loop 5
;
; Now make patched SCP plateaus temporarily visible to DRIFT32 by subtracting
; DG2BAD from bad
bad=bad-dg2bad
;
c999:
;
print,'PREPAREDRIFT: backminv  for MJy/st',backmin,' =',backminv(pixarray)
print,'PREPAREDRIFT: backpredv for MJy/st',backpred,' =',backpredv(pixarray)
;
return
end












