pro psfvs,idefcal
;
; creates map of beam in volts/second on P32 standard grid without background,
; for all detector pixels for a source of 1Jy and stores conversion factors 
; between MJy/st and V/s in common PSSIM. This routine should be called by 
; ERDTOMAP prior to the self calibrations. The beam map is used as 
; a template in self calibrations using ICOMPOSITECORR=1 and 
; COMPOSITEWEIGHT(1) non zero (in which case the predicted illumination
; for a given point source Jy and background is used as a constrain in the 
; parameter determination for the model describing the transient behaviour.
; The conversion Jy to V/s is found by a call to CAL32 - ie finds a constant
; conversion factor which takes into account signal non-linearity over an
; average of the illuminations for the TDT considered.
; The V/s beam map is used in the self cal procedures to compare signal with
; predicted signal. It must be further corrected for vignetting at that stage,
; but not for dark, ramp non-linearity, as these corrections have been included
; in the data signal processing prior to the self calibration. The background
; (and possibly the source flux in Jy) should be added in MJy/sterad by 
; inspection of off source points at the self cal stage using 
; DISPLAYCOMPOSITE, which displays the source and background currently
; contained by PSFLUX (common PSSIM) and BACKPRED (common DRIFTPAR).
;
; Note1: the beam map in V/s is found from the beam map in MJy/st by
;        division by a single scaling factor found from CAL32, stored
;        in the PSCONV1 array in common PSSIM. The reference V/s (uncorrected 
;        for signal non-linearity) for which PSCONV1 was calculated) are
;        stored in the PSCONV2 array. Thus, the distortions due to signal 
;        non-linearity are not present in the output V/s beam map, as they
;        depend on the actual background and source flux used in each
;        application. These are calculated using the PSCONV1 and PSCONV2 
;        arrays witin the calling routines 
;        such as DISPLAYCOMPOSITE, COMPOSITEILLCHI TIMELINECOMPOSITE and 
;        DETSELFCAL where the V/s template of a point source of a certain flux
;        on a certain background is to be calculated.
;
; Note2: this routine should be called before creation of ES arrays by DRIFT32
; as the ES arrays in common E32 are overwritten by the call to SIMUL32.
; A save and restore of ES is not done (as for S) as in common use PSFVS
; would be called internally by ERDTOMAP before first creation of the ES 
; arrays. 
;
; input:
;          IDEFCAL = 0 for actual (interpolated FCS calibration) responsivites
;                  = 1 for default
;          S      (common SIGNAL) written to disk at start of call
;          PIXSIZE (common STEER1) detector pixel size in arcsec 
;
; output:  
;          PSMAP (common PSSIM) simulated map of point source on natural
;                               P32 grid without background in units V/s
;                               (NPIX*NYGRID*NZGRID)
;          S    (common SIGNAL) read from disk at end of call, to
;                               nullify effect of overwriting by SIMUL32 call
;          PSCONV1 (common PSSIM) MJy/sterad * PSCONV1 = V/s for this TDT and 
;                                filter
;          PSCONV2 (common PSSIM) reference V/s (uncorrected for signal 
;                                 non-linearity) for which PSCONV1 was 
;                                 calculated)
; 
@common_steer1
;common steer1,npix,pixsize,pixpitch,pixoff,nchop,dcy,dry,drz 
@common_signal
;common signal,rs,cs,nch,s_in,s,badin,bad,rel,rms,glitch,jf,njf,ninchmax,rd,$
;nkf,kf,nkfmax,irelstart,nchopmed,gbad,npolyrms,nrmsmed,sigmaxfract
@common_pssim
;common pssim,psflux,psmap,vsmap,psconv1,psconv2,compositeweight,illsim
@common_steerin
;common steerin,array,nptin,npt,ninramp,imeas,osamp
;
;CG: DON't:     save,/xdr,s,rms,file='stmp.xdr'
s_before = s
rms_before = rms
;
simul32,1 ; overwrites s, es with 1.0 Jy flux source with no vignetting, 
          ; noise, etc. Units are Jy per detector pixel area
;s=s*psflux
;print,min(s(0,*)),max(s(0,*))
;
; convert to MJy/sterad from Jy per detector pixel area:
;
factor=pixsize*3.14159/(3600.0*180.0)
factor=1.0/(factor*factor*1000000.)
s=s*factor
;print,min(s(0,*)),max(s(0,*))
;
; convert from MJy/sterad to V/s per detector pixel area and find conversion
; factors
;
;cal32,result1,result2,0 
;print,result1
;CG - take now conversion routine

		get_conv_s2b, imeas, idefcal, result1

psconv1=result1
;psconv2=result2

psconv2 = psconv1   ; after checking that psconv2 is nowhere used

for ipix=0,npix-1 do s(ipix,*)=s(ipix,*)/psconv1(ipix)
for ipix=0,npix-1 do rms(ipix,*)=rms(ipix,*)/psconv1(ipix)
;
; make maps using only data with otf on
;
makemapmulti,npix,lindgen(npix),$
mapb,mapr,mapd,mapy,mapz,mapn,0,1
psmap=mapb
;
;CG: don't use:  restore,file='stmp.xdr' ; restore original S and RMS
s = s_before
rms = rms_before
;
return
end 


