pro restore,beta,tau,signal0,sillum0,sky,keypoint,fpoint,itrans,mask,$
vig,t,signal,rms,imodel,isquare,islewmethod,fitresult,modelfit,$
respcorr,niter,ierr
;
; solves for a detector responsivity drift model (at moment only foukes 
; schubert is catered for) on a full series of chopper plateaus
; seen through a P32 observation by a single detector pixel.
;
; RJT notes: before this program can be run expand32 must give reliable
; chopper because of the vignetting. Also include fpoint in expand32 if it
; is not already there.
;
; To add for first chopper plateau work backwards to find what SIGNAL0 SILLUM0
; give the observed trend on the first plateau, given the ILLUM derived from
; the second observations on that plateau, for which the memory effect from the
; initial should have died out. Otherwise we will tend to get innacurate first
; plateaus. Alternatively, always work in pairs with optimisation starting at
; last point and proceeding back in time after going forward in time, keeping
; going back and forth until the solution does not change.
;
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         SIGNAL0     (single number in V/s): input guess for actual 
;                     (noiseless) detector response at time zero prior to the  
;                     1st data sample. Note this number would normally not be 
;                     numerically identical to the first measured point due to
;                     (1) noise on the measured point (2) due to any 
;                     residual drifts between time zero and the time of the
;                     first measured point for the case that the detector
;                     was not stabilized before the observation started and
;                     (3) because the vignetting may have been different 
;                     if the chopper was moving at time zero. Setting SIGNAL0
;                     equal to SILLUM0 means the detector would be stabilized.
;                     SIGNAL0 must be positive.
;         SILLUM0     (single number in V/s): input guess at illumination
;                     prior to the first data point. Must be positive.
;                     One would expect SILLUM0 to be close (within the
;                     vignetting variations) to the illumination on the first
;                     plateau.
;         NITER       number of passes through the data. Should be at least 2,
;                     as early determinations of plateau illuminations
;                     in the first pass are not constrained by the information
;                     on these plateaus gathered later in the dataset when
;                     the detector pixel passes again over the same sky
;                     position. 3 iterations are desirable as the second
;                     uses improved estimates in place of the initial
;                     user guess for SILLUM0 and SIGNAL0
;         KEYPOINT   (1D array INTEGER). For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains the integer key K
;                     to the nearest direction on the P32 natural grid.
;                     K= iy + iz * (nygrid -1),  where iy,iz are the 2D
;                     pointing positions and the grid has dimensions NYGRID,
;                     NZGRID, though this information is not needed, and not
;                     used by this routine.
;         FPOINT     (1D array INTEGER) For each time sample 
;                    (including data taken on board but not transmitted 
;                     in fast readout modes) this contains 1 if the 
;                     on target flag (OTF) was set (as defined by ERDTOMAP)
;                     and zero if the OTF is not set - ie when the data
;                     is taken on the fly.
;         ITRANS     (1D array INTEGER) This is set to 1 for data which is
;                     transmitted to the ground and zero for data not
;                     transmitted (in the fast readout modes).  
;         MASK       (1D array integer) = 1 for data to be ignored. We have
;                     to import all time sampling points in the time array
;                     T in order to follow the illumination history
;                     so for ease of programming we import all data
;                     too, necessitating a mask.
;         VIG        (1D floating array) This is set to the attenuation
;                     due to vignetting (VIG <= 1) as a function of chopper
;                     position in the sense detector signal = vig * sky
;                     As the same sky direction is viewed by the same pixel
;                     in different chopper offsets, the detector pixel
;                     illumination can vary even if the same sky direction is
;                     viewed.
;         T           (array in s) This array contains times for each read
;                     (including times where data is read on board but not
;                      transmitted since time=0 at start of observation.
;                      T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be a small positive
;                      number, normally the non-destructive read interval. 
;                      *** BEWARE that
;                      if T(0) is zero spurious results can be obtained.
;         SIGNAL       (real 1D aray) measured detector signal at times in T 
;                      since time zero. All points must be
;                      positive 
;         RMS          (array in V/s) RMS uncertainty in SIGNAL
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
; ISLEWMETHOD          Integer flag.
; Note that on slews where there is no transmission there is in principle
; no way to get information about illumination. The user should try various
; methods to get the best result. In this case we can either
; use the nearest pixel information to define the illumination (ISLEWMETHOD
; = 1). Alternatively we can do a spline interpolation over the slew for
; the measured signals/vig, (ie in sky) on option ISLEWMETHOD=2). This is
; also very reasonable due to the very rapid spatial sampling at high 
; brightness and high s/n.
;                       
;
;
; output: SKY         (1D array in V/s) This is the illumination that each 
;                      point on the natural sky grid would receive in the
;                      absence of vignetting. The dimension of sky is the
;                      total number of points on the natural P32 grid. 
;                      SKY is constrained to be positive.
;         FITRESULT    (1D real array) an output parameter normalised to RMS 
;                      indicating how good the fit is for each elementary 
;                      sample points (ie all individual points on plateau 
;                      calculated according to 
;                      the value of the input parameter ISQUARE (see above). 
;                      The lower the better. It can be used to identify 
;                      plateaus not fitted by the data. This enables plateaus 
;                      to be masked out by the calling program if required
;                      by the application eg mapping. Note that for plateaus 
;                      with only 
;                      one data point (eg points taken on the fly) FITRESULT
;                      will tend to zero, rather than RMS, as the number of 
;                      search iterations increases. Because there is no
;                      statistical information on the goodness of fit for
;                      single points, FITRESULT is set to 0.0 for single points
;         MODELFIT     (array in V/s): best fit model
;                      for the data. 
;         RESPCORR     (array, dimensioness) illumination/response calculated 
;                      from the vignetting and the model. This is the basic
;                      transient correction result.
;         IERR         =1 if an error occurred
;


Note the callin program finds illumination

;common timer,time,dtime,complete
@common_timer

; first find SKYMAX (single REAL in V/s) which indicates maximum possible
; brightness which could be encountered. SKYMAX is used to define search
; space in the solution for the illumination for each plateau.
;
; internal parameters
;
smax=max(signal)
skymax=4.0*smax ; factor of 4 absolute maximum for single step and beta 0.25
skymax=min([skymax,1.0/dtime]) ; otherwise saturation
print,'RESTORE: upper limit for maximum signal set to ',skymax
if(smax*dtime ge 1.0)then print,'RESTORE: warning: looks like saturated data')


; for each chopper plateau optimisation
; SILMIN,SILMAX (used minimum and maximum possible values for SILLUM 
; (used to control the cutting algorithm)


silmax=skymax
silmin=0.00001

fitprec=0.1

;         fitprec is internal constant giving gives the precision to 
;         which the best fit is found. The interval of search for 
;         ILLUM=SKYSOL*VIG for each plateau used by SOLVEILLUM1
;         ILLUM is RMS*FITPREC. Where SKYSOL is the solution for the
;         plateau, VIG is the vignetting for the plateau (<=1) and
;         RMS is the detector noise for the plateau.


if(min(rms) le 0.0)then begin
print,'RESTORE: rms array zero or negative '
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(isquare lt 0 or isquare gt 2)then begin
print,'RESTORE: isquare out of range - isquare = ',isquare
ierr=1
goto,c999
endif
;
if(imodel ne 1)then begin
print,'RESTORE: model must be Foukes-Schubert'
ierr=1
goto,c999
endif
;
if(min(signal) le 0.0)then begin
print,'RESTORE: aborting: detector data must be positive - '
print,'min(signal) = ',min(signal)
ierr=1
goto,c999
endif
;
if(beta le 0.0 or beta gt 1.0)then begin
print,'RESTORE: aborting: beta out of range - beta = ',beta
ierr=1
goto,c999
endif

if(tau le 0.0)then begin
print,'RESTORE: aborting: tau negative - tau = ',tau
ierr=1
goto,c999
endif
;
if(signal0 le 0.0)then begin
print,'RESTORE: aborting: SIGNAL0 must be positive - signalprev = ',$
signalprev
ierr=1
goto,c999
endif
;
if(sillum0 le 0.0)then begin
print,'RESTORE: '
print,'aborting: SILLUM0 must be positive - sillumprev = ',sillumprev
ierr=1
goto,c999
endif
;
if(min(t) le 0.0)then begin
print,'RESTORE: aborting: time array t must be a positive interval since'
print,'the last destructive read - min(t) = ',min(t)
ierr=1
goto,c999
endif

if(niter le 1)then begin
print,'RESTORE: aborting: need at least two iterations: niter = ',niter
ierr=1
goto,c999
endif

; find plateaus from FPOINT array. When slewing only have one point 
; per plateau.

npt=n_elements(t)

i=0
ninplateau=1
istart=0

c100:
i=i+1
ii=i-1
if(i ge npt)then begin
if(ninplateau gt 0)then nplateau=[nplateau,ninplateau]
goto,c200:
endif ; i ge npt
if(fpoint(i) eq fpoint(ii))then begin ; i is still on plateau already started
ninplateau=ninplateau+1
endif ; fpoint and keypoint
if(fpoint(i) ne fpoint(ii)) or fpoint(i) eq 0)then begin ; i is first point 
;                       on next plateau or we are slewing
if(istart gt 1)then nplateau=[nplateau,ninplateau
if(istart lt 0)then nplateau=[ninplateau]
ninplateau=1
istart=1
endif ; fpoint ne fpoint
goto,c100:
c200:

nplat=n_elements(nplateau)

; solve for the plateau illumination
; and sky brightness. In initial iteration update sky brightness stack as 
; go along

for iplat=0,nplat-1 do begin


endfor ; iplat

;
c999:
;
return
end
