;+
;---------------------------------------------------
; Responsivity Linearisation
;
; Input:
;	sig	(float)	input signal [V/s]
;	filt	(string) filtername (PIA convention)
;	pixel	(int) pixel number starting at 1
;
; Keywords:
;	calpath	(string) path to calibration files to be used  
;
;	silent	suppresses warnings when limits 
;		exceeded
;
;	fload	force loading from file
;
; Output:
;		(float) linearised signal [V/s]
;
; Example:
;	print, rlin_fct(1.0, 'C1_60', 1, calpath='./cal_rlin/',/fload)
;
;
; Edition History
;
; 10/10/98	initial test version	C. Gabriel
; 18/01/99	symmetric zero crossing	B. Schulz
; 19/01/99	new keywords added	B. Schulz
; 03/01/99	buffer for tables 	B. Schulz
; 04/01/99	array length buffered 	B. Schulz
;	rlin_tables under !CALG (CG)				Jul 99
;	V7.9
; keyword det added for check (CG) July 1999
;	V8.0
;
;---------------------------------------------------
FUNCTION rlin_fct, sig, filt, pixel, det=det, calpath=calpath, $
	silent=silent, fload=fload, REVERSE=reverse
;-		

  COMMON RLINTABLES, p_lin, c1_lin, c2_lin, $
  		     p_lm,  c1_lm,  c2_lm, $
		     p_flg, c1_flg, c2_flg, $
		     p_nel, c1_nel, c2_nel


  npnts = 51	;max. number of points in transformation table

  pix = pixel - 1	;start counting from 0
  
;--------------------------------------
; find position of filter

  ix = filt_no(filt)-1

  if ix LT 0 THEN begin 
    message, /info, 'Unknown filtername!'
    RETURN, -99
  endif

;--------------------------------------
; initialise common blocks if necessary

  IF n_elements(p_lin) EQ 0 OR keyword_set(fload) THEN BEGIN
; linearisation curve
    p_lin  = fltarr(14,npnts,2)		; [filter, data, (0=sigin,1=sigout)]
    c1_lin = fltarr(6,9,npnts,2)	; [filter, pix, data, (0=sigin,1=sigout)]
    c2_lin = fltarr(5,4,npnts,2)	; [filter, pix, data, (0=sigin,1=sigout)]
; lower and upper limits
    p_lm   = fltarr(14,2)		; [filter, (0=lower limit, 1=upper limit)]
    c1_lm  = fltarr(6,9,2)		; [filter, pix, (0=lower limit, 1=upper limit)]
    c2_lm  = fltarr(5,4,2)		; [filter, pix, (0=lower limit, 1=upper limit)]
; valid flag
    p_flg  = intarr(14)			; [filter]
    c1_flg = intarr(6,9)		; [filter, pix]
    c2_flg = intarr(5,4)		; [filter, pix]
; number of elements
    p_nel  = intarr(14)			; [filter]
    c1_nel = intarr(6,9)		; [filter, pix]
    c2_nel = intarr(5,4)		; [filter, pix]

    if NOT keyword_set(silent) THEN message, /info, 'Buffer cleared!'

  ENDIF


;--------------------------------------
; check if data already available
; and check detector-filter compatibility

  CASE 1 OF				
    ix LT 14: BEGIN
        IF not KEYWORD_SET(det) THEN det = 'P'
        IF STRMID(det,0,1) NE 'P' THEN BEGIN
           MESSAGE, /INFO, 'Filter-det not corresponding, no linearisation'
           RETURN, sig
        ENDIF
    	flag = p_flg(ix)
    END       
    ix LT 20: BEGIN
        IF not KEYWORD_SET(det) THEN det = 'C1'
        IF STRMID(det,0,2) NE 'C1' THEN BEGIN
           MESSAGE, /INFO, 'Filter-det not corresponding, no linearisation'
           RETURN, sig
        ENDIF
    	flag = c1_flg(ix-14,pix)
    END
    ELSE: BEGIN
        IF not KEYWORD_SET(det) THEN det = 'C2'
        IF STRMID(det,0,2) NE 'C2' THEN BEGIN
           MESSAGE, /INFO, 'Filter-det not corresponding, no linearisation'
           RETURN, sig
        ENDIF
        flag = c2_flg(ix-20,pix)
    END
  ENDCASE


;--------------------------------------
; load table from file if necessary

  IF flag EQ 0 THEN BEGIN


; set directory containing the tables

    if keyword_set(calpath) then tabdir = calpath $
    else tabdir = !CALG + 'rlin_tables/'

    scor = -99.	;default error value 
	
    asc_fil = filt + '_' + STRTRIM(pixel,2) + '_sig.txt'
	
    table = tabdir+asc_fil

    IF is_file_exist(table) THEN BEGIN
    
      read_rlin_table, table, sig_in, sig_out, limits		;read file
      len = n_elements(sig_in)

      CASE 1 OF				;store tables and set flag
        ix LT 14: begin
          p_lin(ix,0:len-1,0) = sig_in
          p_lin(ix,0:len-1,1) = sig_out
          p_lm(ix,*)=limits
	  p_nel(ix)=len
          p_flg(ix)=1
	  end
        ix LT 20: begin
          c1_lin(ix-14,pix,0:len-1,0) = sig_in
          c1_lin(ix-14,pix,0:len-1,1) = sig_out
          c1_lm(ix-14,pix,*)=limits
	  c1_nel(ix-14,pix)=len
          c1_flg(ix-14,pix)=1
	  end
        ELSE:     begin
          c2_lin(ix-20,pix,0:len-1,0) = sig_in
          c2_lin(ix-20,pix,0:len-1,1) = sig_out
	  c2_lm(ix-20,pix,*)=limits
	  c2_nel(ix-20,pix)=len
          c2_flg(ix-20,pix)=1
	  end
      ENDCASE
   
    ENDIF ELSE BEGIN

      PRINT, 'File ' + table + ' not found. ' 

      CASE 1 OF				;set flag to invalid in file not found
        ix LT 14: begin
          p_flg(ix)=0
	  end
        ix LT 20: begin
          c1_flg(ix-14,pix)=0
	  end
        ELSE:     begin
          c2_flg(ix-20,pix)=0
	  end
      ENDCASE

      RETURN, -99
      
    ENDELSE

  ENDIF

;--------------------------------------
; pull data from common block


  CASE 1 OF				;store tables and set flag
    ix LT 14: begin
      sig_in  = p_lin(ix,0:p_nel(ix)-1,0)
      sig_out = p_lin(ix,0:p_nel(ix)-1,1)
      limits  = p_lm(ix,*)
      end
    ix LT 20: begin
      sig_in  = c1_lin(ix-14,pix,0:c1_nel(ix-14,pix)-1,0)
      sig_out = c1_lin(ix-14,pix,0:c1_nel(ix-14,pix)-1,1)
      limits  = c1_lm(ix-14,pix,*)
      end
    ELSE:     begin
      sig_in  = c2_lin(ix-20,pix,0:c2_nel(ix-20,pix)-1,0)
      sig_out = c2_lin(ix-20,pix,0:c2_nel(ix-20,pix)-1,1)
      limits  = c2_lm(ix-20,pix,*)
      end
  ENDCASE

  sig_in  = reform(sig_in)
  sig_out = reform(sig_out)
 
;--------------------------------------
; check limits

  indmin = WHERE(sig LT limits(0), nmin)
  indmax = WHERE(sig GT limits(1), nmax)


  IF NOT keyword_set(silent) THEN BEGIN
    IF nmin GT 0 THEN $
      PRINT, 'RLIN: '+ STRTRIM(nmin,2) + $
               ' Signal(s) below lower limit of table'

    IF nmax GT 0 THEN $
      PRINT, 'RLIN: '+ STRTRIM(nmax,2) + $
               ' Signal(s) above upper limit of table'
  ENDIF

;--------------------------------------
; interpolation

  sig_in = [-rotate(sig_in,2), 0, sig_in]	;make zero symmetric
  sig_out = [-rotate(sig_out,2), 0,sig_out]
  IF not KEYWORD_SET(REVERSE) THEN $
   scor = interpol(sig_out, sig_in, sig)	 ELSE $	;linear interpolation
   scor = interpol(sig_in, sig_out, sig)	; for getting signal UNcorrected
  

  RETURN, scor

END
		
	
