pro solvebackillum1,beta,tau,signalprev,sillumprev,sillum,t,signal,rms,$
fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;
; solves for either FOUKES or EXPFUN detector responsivity drift models to 
; define a unique detector state defined by the combination
; (SILLUMPREV, SIGNALPREV) at the end of the previous plateau,
; where the illumination array SILLUM is known and fixed and SIGNAL is the
; measured signal. As SILLUMPREV and SIGNALPREV are linearly constrained
; for both models via the internal parameter 
; s0=signalprev+beta*(sillum-sillumprev), a search is made in 1D in S0 for the
; value of s0 which best fits the data, and SILLUMPREV is constrained to be 
; SILLUM(0). Thus the output values for SILLUMPREV and SIGNALPREV have
; no individual unique meaning. As a pair of parameters, however, they
; uniquely define the initial detector state at the end of the previous
; plateau prior to T(0). The user should be aware that in general the
; assumption SILLUMPREV=SILLUM(0) will be poor, since the routine is normally 
; called at the start of a fine pointing plateau, immediately after a chopper
; step. S0 is found by direct search in the range SILMIN, SILMAX. 
;
; Note that SIGNAL T and SILLUM can encompass any number of pointing plateaus.
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         SILLUM      (array in V/s): best fit illumination for the 
;                      detector at the times in T
;         SIGNAL      (array in V/s) measured detector signal at times in T 
;                      since illumination step at time zero. All points must be
;                      positive (no mask is input here - it is assumed a mask 
;                      a mask or other manipulation has been applied before 
;                      the call. For this reason T will normally be 
;                      monotonically increasing, but irregularly sampled.
;         T           (array in s) time since flux step at start of plateau
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be a small positive
;                      number, normally the non-destructive read interval if
;                      the first point has not been masked. *** BEWARE that
;                      if T(0) is zero spurious results can be obtained.
;         RMS          (array in V/s) RMS uncertainty in SIGNAL
;         SILMIN,SILMAX minimum and maximum possible values for SILLUMPREV 
;                      (used to control the cutting algorithm)
;         FITPREC      gives the precision to which the best fit is found.
;                      The interval of search for SILLUM is reduced
;                      until it is below the mean of RMS*FITPREC. 
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;         IERR         error flag. should be 0 on input
;                       
; output: 
;         SIGNALPREV (single number in V/s): actual detector response at time
;                     zero immediately prior to the initial flux step from 
;                     SILLUMPREV to the (to be determined) SILLUM. SIGNALPREV 
;                     should be positive. A warning is issued if a negative
;                     value is found. Only meaningful when combined with
;                     SILLUMPREV to define a unique detector starting state.
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at time zero, constrained to
;                     be SILLUM(0). Only meaningful when combined with
;                     SIGNALPREV to define a unique detector starting state.
;         FITRESULT    an output parameter normalised to RMS indicating how 
;                      good the fit is, calculated according to the value of 
;                      the input parameter ISQUARE (see above). 
;                      It can be used to identify
;                      plateaus not fitted by the data. This enables plateaus 
;                      to be masked out by the calling program if required
;                      by the application. Note that for plateaus with only 
;                      one data point (eg points taken on the fly) FITRESULT
;                      will tend to zero, rather than RMS, as the number of 
;                      search iterations increases. Because there is no
;                      statistical information on the goodness of fit for
;                      single points, FITRESULT is set to 0.0 for single points
;         MODELFIT     (array in V/s): best fit model
;                      for the plateau. 
;         IERR         =1 if an error occurred
;
;
xnpt=float(n_elements(t))
rmsmean=total(rms)/xnpt
;print,'xnpt,rmsmean = ',xnpt,rmsmean
;
if(min(rms) le 0.0)then begin
print,'SOLVEBACKILLUM1: rms array zero or negative '
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(fitprec lt 0.1 or fitprec gt 1.0)then begin
print,'SOLVEBACKILLUM1: fitprec set too low for fast convergence or too high'
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(isquare lt 0 or isquare gt 2)then begin
print,'SOLVEBACKILLUM1: isquare out of range - isquare = ',isquare
ierr=1
goto,c999
endif
;
if(imodel ne 1 and imodel ne 2)then begin
print,'SOLVEBACKILLUM1: model must be Foukes-Schubert or exponential'
ierr=1
goto,c999
endif
;
; commented out rjt nov99
;
;if(min(signal) le 0.0)then begin
;print,'SOLVEBACKILLUM1: aborting: detector data must be positive - '
;print,'min(signal) = ',min(signal)
;ierr=1
;goto,c999
;endif
;
if(imodel eq 1 or imodel eq 2)then begin
if(beta le 0.0 or beta ge 1.0)then begin
print,'SOLVEBACKILLUM1: aborting: beta out of range - beta = ',beta
ierr=1
goto,c999
endif
endif

if(tau le 0.0)then begin
print,'SOLVEBACKILLUM1: aborting: tau negative - tau = ',tau
ierr=1
goto,c999
endif
;
if(min(sillum) lt silmin or max(sillum) gt silmax)then begin
print,'SOLVEBACKILLUM1: '
print,'warning: SILLUM out of range silmin,silmax = ',silmin,silmax,' ...'
print,'... min(sillum), max(sillum) = ',min(sillum),max(sillum)
ierr=1
goto,c999
endif
;
if(min(t) le 0.0)then begin
print,'SOLVEBACKILLUM1: aborting: time array t must be a positive ...'
print,'... interval since the last destructive read - min(t) = ',min(t)
ierr=1
goto,c999
endif
;
if(silmin le 0.0 or silmax le silmin)then begin
print,'SOLVEBACKILLUM1: aborting: SILMIN must be positive ...'
print,'... and less than SILMAX - silmin,silmax = ',silmin,silmax
ierr=1
goto,c999
endif
;
; internal constants controlling trade off between speed, precision and 
; robustness of fit
;
niter=1 ; for diagnostics - set to a large number to control convergence
        ; by FITPREC
ncut=5 ; must be odd and at least 5
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
chiresult=fltarr(ncut)
;
xncut=float(ncut)
top=silmax
bot=silmin
reducefactor=2.0
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
;
;print,'bot,top,dcut,sigtrial = ',bot,top,dcut,sigtrial
;
sillumprev=sillum(0) ; constraint
; because of this constraint (effectively the search is in S0) the trial 
; value for signalprev is simply:
signalprevtrial=sigtrial(0)
;signalprevtrial=sigtrial(0)-beta*(sillum(0)-sillumprev)
;
if(imodel eq 1)then foukes,beta,tau,signalprevtrial,sillumprev,sillum,t,model
if(imodel eq 2)then expfun,beta,tau,signalprevtrial,sillumprev,sillum,t,model
;
findfitresult,signal,rms,model,sumsq,isquare
;
icutmin=0
sumsqmin=sumsq
signalprev=signalprevtrial
modelfit=model
;print,'sumsqmin: ',sumsqmin
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
signalprevtrial=sigtrial(icut)
;
if(imodel eq 1)then foukes,beta,tau,signalprevtrial,sillumprev,sillum,t,model
if(imodel eq 2)then expfun,beta,tau,signalprevtrial,sillumprev,sillum,t,model
;
findfitresult,signal,rms,model,sumsq,isquare
chiresult(icut)=sumsq
;
; rjt sep 10 1999 icutmin=0
;
if(sumsq lt sumsqmin)then begin ; found new minimum
icutmin=icut
sumsqmin=sumsq
signalprev=signalprevtrial
modelfit=model
;help,modelfit
;print,'maxmin model fit = ',max(modelfit),min(modelfit)
;print,'icut,sillum,sumsqmin',icut,sillum,sumsqmin
endif ; sumsq lt sumsqmin
;
endfor ; icut
;
if(dcut le fitprec*rmsmean)then goto, c200
;if(ncut ge 5)then begin
;cutoptimise,chiresult,ncut,reduce,ncutout,reduceout,ioptimize
;if(dcut le rmsmean)then begin ; speed up towards solution
;ncut=3
;reduce=2.0
;endif ; dcut le rmsmean
;endif ; ncut ge 5
d=top-bot
d=d/reduce
d2=d/2.0
bot=signalprev-d2
top=signalprev+d2
bottmp=bot
toptmp=top
if(bot lt silmin)then begin
bottmp=silmin
toptmp=bottmp+d
endif ; bot
if(top gt silmax)then begin
toptmp=silmax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
;
iter=iter+1
;print,'iter,bot,top,sigtrial ',iter,bot,top,sigtrial
;
goto, c100
;
c200:
;
if(xnpt gt 1.0)then fitresult=sumsqmin*xnpt/(xnpt-1.0)
if(xnpt le 1.0)then fitresult=0.0 ; no contribution from single point
;                                   plateaus to fit
;
;
c999:
;
return
end



