pro solveillum1,beta,tau,signalprev,signalprevshoot,sillumprev,sillum,$
t,signal,rms,fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;
; solves for a detector responsivity drift model on a single chopper plateau to
; find best fit illumination SILLUM on the plateau. The plateau must be at 
; least one point long (ie we can use this for drift correction on the fly as
; well as on fixed pointings). SILLUM is found by direct search in the 
; range SILMIN, SILMAX.
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         SIGNALPREV (single number in V/s): actual detector response at time
;                     zero immediately prior to the initial flux step from 
;                     SILLUMPREV to the (to be determined) SILLUM. Note this 
;                     number is not normally actually measured
;                     due to the destructive read at time zero so has to be a 
;                     model estimate. SIGNALPREV must be positive.
;         SIGNALPREVSHOOT fast detector response  at time
;                     zero immediately prior to the initial flux step.
;                     (only used for imodel gt 2).
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at time zero. Must be positive.
;         SIGNAL      (array in V/s) measured detector signal at times in T 
;                      since illumination step at time zero. All points must be
;                      positive (no mask is input here - it is assumed a mask 
;                      a mask or other manipulation has been applied before 
;                      the call. For this reason T will normally be 
;                      monotonically increasing, but irregularly sampled.
;         T           (array in s) time since flux step at start of plateau
;                      at time 0. T refers to the time at which the detector
;                      is actually read. Note this is a non-PIA standard, and
;                      follows the time assignment in the SIGNALCREATE.PRO 
;                      routine - ie T(0) should actually be a small positive
;                      number, normally the non-destructive read interval if
;                      the first point has not been masked. *** BEWARE that
;                      if T(0) is zero spurious results can be obtained.
;         RMS          (array in V/s) RMS uncertainty in SIGNAL
;         SILMIN,SILMAX minimum and maximum possible values for SILLUM 
;                      (used to control the cutting algorithm)
;         FITPREC      gives the precision to which the best fit is found.
;                      The interval of search for SILLUM is reduced
;                      until it is below the mean of RMS*FITPREC. 
;         IMODEL       = 1 for Foukes-Schubert
;         ISQUARE      parameter specifying which quantity the optimisation
;                      seeks to minimise. The minimum value of the quantity
;                      found is output as FITRESULT.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;         SPEEDFACTOR (common SPEED) between 1.5 and 2.0 - controls
;                      speed of convergence
;         IERR         error flag. should be 0 on input
;                       
; output: SILLUM      (single number in V/s): best fit constant illumination 
;                      over the plateau. 
;         FITRESULT    an output parameter normalised to RMS indicating how 
;                      good the fit is, calculated according to the value of 
;                      the input parameter ISQUARE (see above). 
;                      It can be used to identify
;                      plateaus not fitted by the data. This enables plateaus 
;                      to be masked out by the calling program if required
;                      by the application. Note that for plateaus with only 
;                      one data point (eg points taken on the fly) FITRESULT
;                      will tend to zero, rather than RMS, as the number of 
;                      search iterations increases. Because there is no
;                      statistical information on the goodness of fit for
;                      single points, FITRESULT is set to 0.0 for single points
;         MODELFIT     (array in V/s): best fit model
;                      for the plateau. 
;         IERR         =1 if an error occurred
;
;
@common_speed
;common speed,speedfactor
;
;save,beta,tau,signalprev,signalprevshoot,sillumprev,$
;t,signal,rms,fitprec,imodel,silmin,silmax,isquare,file='solveillum1.xdr'
;
xnpt=float(n_elements(t))
rmsmean=total(rms)/xnpt
;print,'xnpt,rmsmean = ',xnpt,rmsmean
;
if(min(rms) le 0.0)then begin
print,'SOLVEILLUM1: rms array zero or negative '
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(fitprec lt 0.1 or fitprec gt 1.0)then begin
print,'SOLVEILLUM1: fitprec set too low for fast convergence or too high'
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(isquare lt 0 or isquare gt 2)then begin
print,'SOLVEILLUM1: isquare out of range - isquare = ',isquare
ierr=1
goto,c999
endif
;
if(imodel ne 1 and imodel ne 2 and imodel ne 3)then begin
print,'SOLVEILLUM1: model must be Foukes-Schubert or exponential or expshoot'
ierr=1
goto,c999
endif
; commented out NOV99 RJT
;if(min(signal) le 0.0)then begin
;print,'SOLVEILLUM1: aborting: detector data must be positive - '
;print,'min(signal) = ',min(signal)
;ierr=1
;goto,c999
;endif
;
if(imodel eq 1 or imodel eq 2)then begin
if(beta le 0.0 or beta ge 1.0)then begin
print,'SOLVEILLUM1: aborting: beta out of range - beta = ',beta
ierr=1
goto,c999
endif
endif ; imodel eq 1 or imodel eq 2

if(imodel lt 3 and tau le 0.0)then begin
print,'SOLVEILLUM1: aborting: tau negative - tau = ',tau
ierr=1
goto,c999
endif
;
;Commented out RJT
;
;if(signalprev le 0.0)then begin
;print,'SOLVEILLUM1: aborting: SIGNALPREV must be positive - signalprev = ',$
;signalprev
;ierr=1
;goto,c999
;endif
;
if(sillumprev le 0.0)then begin
print,'SOLVEILLUM1: '
print,'aborting: SILLUMPREV must be positive - sillumprev = ',sillumprev
ierr=1
goto,c999
endif
;
if(min(t) le 0.0)then begin
print,'SOLVEILLUM1: aborting: time array t must be a positive interval since'
print,'the last destructive read - min(t) = ',min(t)
ierr=1
goto,c999
endif
;
if(silmin le 0.0 or silmax le silmin)then begin
print,'SOLVEILLUM1: aborting: SILMIN must be positive and less than SILMAX - '
print,'silmin,silmax = ',silmin,silmax
ierr=1
goto,c999
endif
;
; internal constants controlling trade off between speed, precision and 
; robustness of fit
;
niter=1 ; for diagnostics - set to a large number to control convergence
        ; by FITPREC
;ncut=5 ; must be odd and at least 3
ncut=3 ; safe for a 1D optimisation
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
reduce=speedfactor
chiresult=fltarr(ncut)
;
xncut=float(ncut)
top=silmax
bot=silmin
reducefactor=2.0
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
template=t
template(*)=1.0
;
;print,'bot,top,dcut,sigtrial = ',bot,top,dcut,sigtrial
;
sillumtrial=template*sigtrial(0)
;print,'imodel,sillumtrial,beta,tau = ',imodel,sillumtrial,beta,tau
;print,'signalprev,sillumprev = ',signalprev,sillumprev
;print,'t ',t
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 3)then expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,$
sillumtrial,t,model,modelslow,modelshoot
;
findfitresult,signal,rms,model,sumsq,isquare
;
icutmin=0
sumsqmin=sumsq
sillum=sigtrial(0)
modelfit=model
;print,'sumsqmin: ',sumsqmin
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
sillumtrial=template*sigtrial(icut)
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 3)then expshoot,beta,tau,signalprev,signalprevshoot,sillumprev,$
sillumtrial,t,model,modelslow,modelshoot
;
findfitresult,signal,rms,model,sumsq,isquare
chiresult(icut)=sumsq
;
; rjt sep 10 1999 icutmin=0
;
if(sumsq lt sumsqmin)then begin ; found new minimum
icutmin=icut
sumsqmin=sumsq
sillum=sigtrial(icut)
modelfit=model
endif ; sumsq lt sumsqmin
;
endfor ; icut
;
if(dcut le fitprec*rmsmean)then goto, c200
;if(ncut ge 5)then begin
;cutoptimise,chiresult,ncut,reduce,ncutout,reduceout,ioptimize
;if(dcut le rmsmean)then begin ; speed up towards solution
;ncut=3
;reduce=2.0
;endif ; dcut le rmsmean
;endif ; ncut ge 5
d=top-bot
d=d/reduce
d2=d/2.0
bot=sillum-d2
top=sillum+d2
bottmp=bot
toptmp=top
if(bot lt silmin)then begin
bottmp=silmin
toptmp=bottmp+d
endif ; bot
if(top gt silmax)then begin
toptmp=silmax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
;
iter=iter+1
;print,'iter,bot,top,sigtrial ',iter,bot,top,sigtrial
;
goto, c100
;
c200:
;
if(xnpt gt 1.0)then fitresult=sumsqmin*xnpt/(xnpt-1.0)
if(xnpt le 1.0)then fitresult=0.0 ; no contribution from single point
;                                   plateaus to fit
;
c999:
;
return
end



