pro solveslew1,beta,tau,signalprev,sillumprev,echprev,sillumout,t,ech,mask,$
signal,rms,fitprec,imodel,silmin,silmax,isquare,fitresult,modelfit,ierr
;
;
; solves for a detector responsivity drift model for a series of consecutive
; time samplings on a slew, for which only the last point 
; has good transmitted signal. It is needed since the use of SOLVEILLUM1
; on slews with missing data requires extrapolations (or guesses) of the 
; illumination history which can sometimes produce cumulative divergence of 
; the solution for the illumination from the real solution. 
;
; SOLVESLEW1 is called both for data sets with 
; full transmission and for data sets with data reduction (only a sub-multiple
; of the data transsmitted). For data sets with full transmission, the call is
; always with just one time sampling point except where there has been a 
; glitch between encompassing points with good signal. If called with just one
; point, the solution is the same as that for SOLVEILLUM1 irrespective of
; whether or not the chopper changed (see next paragraphs). If there is
; incomplete transmission, the illumination is calculated for the
; time samples with missing data while stll restricting the number of 
; variables to unity as follows:
;
; If the input time sequence is for a portion of a slew with constant chopper 
; phase the illumination is calculated as a linear variation in time between 
; the first and last points of T, output in the array SILLUMOUT. The first
; element of SILLUMOUT corresponds to the time sample immediately following
; that corresponding to the parameters specifying the detector state on input
; SILLUMPREV and SIGNALPREV. The last element of SILLUMOUT has illumination 
; SILLUM (internal parameter) corresponding to the measured signal on the last
; point, SIGNAL (input data; single value). The values in SILLUMOUT are thus
; calculated assuming a linear variation between SILLUMPREV at time 0 and 
; SILLUM at the final element of T. The first element of T correspnds to the 
; time interval between the sample corresponding to SILLUMPREV and SIGNALPREV 
; and the first sample thereafter - it must never be zero otherwise the model
; subroutines ail. SIGNALPREV (input single value, is normally 
; calculated in the calling routine as the response of the detector model to 
; the previously calculated illumination history with last value SILLUMPREV.
; In this formulation, SOLVESLEW1 is a generalisation of the response
; of the SOLVEILLUM1 routine on slews. However, we still require the two
; routines with separate identity due to the different action of
; SOLVEILLUM1 on fine pointing plateaus.  
;
; If the chopper moves during the time sequence (that is, the
; chopper phase for any time sample differs from the phase for the 
; sample corresponding to SIGNALPREV,SILLUMPREV), then the
; assumption of linear variation of illumination with time is no longer 
; justified due to the abrupt jump in position due to the chopper. 
; This situation may occur routinely at the end of a chopper plateau in 
; readout modes with a data reduction in transmission or in any sequence
; with data masked due to glitches. In this case a step brightness profile is
; assumed, with SILLUMOUT set to SILLUMPREV for all points preceding
; the chopper motion and to the internal parameter SILLUM for points at,
; and subsequent to, the chopper step. This type of solution is normally
; sufficiently representative of the true illumination history where
; the chopper changes at the final sample, which is the normal situation
; without glitches - the solution is then exactly that for SOLVEILLUM1 for
; fully transmitted datasets; although some inaccuracy will occur in not fully
; transmitted data as the actual illumination may vary over the non transmited
; points prior to the final point where the chopper changed, this variation is
; relatively small due to the fast readouts, and smooth. The only exception
; to this would be where a whole chopper phase was omitted and by accident the
; good data started at the start of a chopper plateau. Here we set the ouput
; MASK array to 1 for all but the first points so that the calling program 
; knows the points are unreliable for photometry. The solution could, by 
; contrast, be only rather approximate where the chopper changed at an 
; intermediate part of the slew sequence between the reference (SILLUMPREV,
; SIGNALPREV) point and the last point, which would only occur when part of 
; the data was lost due to the presence of glitches removing data. Here, again,
; we set the MASK array to 1 for all but the first point. Even where the MASK 
; array is set, the illumination is still output to allow the calling program 
; to get some estimate for the illumination history to proceed further with the
; solution.
;
; Notes for the calling routine: 
; (i) when finishing a slew the calling program must in principle
; still call SOLVESLEW1 when the previous good transmitted point is on a slew 
; and the current good transmitted point is on a fine pointing, to fill the 
; points of the illumination history up to the start of the fine pointing.
; In practise we can just use the extrapolation in SOLVEILLHIST in the
; ISLEWMETHOD=2 mode to fill the points with little error. Thus, SOLVESLEW1
; need only be called when the current good transmitted plateau is on a slew.
; (ii) In the ISLEWMETHOD=2 mode SOLVESLEW1 always uses existing values of 
; the SKYPASS array where they are completely filled in. The output
; from SOLVESLEW1 in SILLUMOUT must be transferred to SKYPASS (and not 
; overwritten), irrespective of the values output in MASK.
;
; The routine works by a direct search for the internal parameter SILLUM in 
; the range SILMIN, SILMAX.
;
; input:  BETA detector model constant
;         TAU (VOLTS) detector model constant
;         SIGNALPREV (single number in V/s): actual detector response at time
;                     zero immediately prior to the initial flux step from 
;                     SILLUMPREV to the (to be determined) SILLUM. Note this 
;                     number is normally a model estimate. SIGNALPREV must be 
;                     positive.
;         SILLUMPREV  (single number in V/s): illumination immediately prior
;                     to illumination step at time zero. Must be positive.
;         SIGNAL      (single number in V/s) measured detector signal at 
;                     time of last element in T. Must be positive.
;         T           (array in s) time since detector was in initial state 
;                     SIGNALPREV,SILLUMPREV. The elements of T up to but not
;                     including the last element correspond to reads for
;                     which (due to non-transmitted data or data with glitches)
;                     have no good data. All elements of T, including the last,
;                     are on the fly. The last element of T refers to the 
;                     time at which the detector produces the next good data,
;                     of signal SIGNAL.
;         ECH         (integer array) chopper plateau identifier at times in T
;                     (1 to 13 for c100, 1 to 7 for c200) 
;         ECHPREV     (single integer) chopper plateau identifier of 
;                      sample corresponding to (SILLUMPREV, SIGNALPREV)
;         RMS         (single element in V/s) RMS uncertainty in SIGNAL
;         SILMIN,SILMAX minimum and maximum possible values for internal 
;                     parameter SILLUM (used to control the cutting algorithm)
;         FITPREC     gives the precision to which the best fit is found.
;                     The interval of search for SILLUM is reduced
;                     until it is below the mean of RMS*FITPREC. 
;         IMODEL      = 1 for Foukes-Schubert
;                     = 2 for Exponential
;         ISQUARE     parameter specifying which quantity the optimisation
;                     seeks to minimise. The optimisation is with respect to
;                     a comparison of SIGNAL with the last element of 
;                     MODEFIT out, which is the model response to the
;                     illumination along the slew SILLUMOUT, starting with
;                     detector in the state (SIGNALPREV, SILLUMPREV).
;                     The minimum value of the quantity (defined below)
;                     optimised is output as FITRESULT. Since there is only
;                     one datapoint FITRESULT tends to zero, and we force it
;                     to be zero to show it is in efect meaningless in terms
;                     of describing the goodness of fit.
;                      ISQUARE=1: square root of AVERAGE of sum of squares
;                                 of residuals between model and data, after
;                                 dividing each residual by RMS.
;                                 Best for clean data without glitches.
;                      ISQUARE=2: modulus of median of residuals between 
;                                 model and data. After taking the median the
;                                 median is divided by the mean RMS. 
;                                 Best for data still containing glitches.
;                      ISQUARE=0: modulus of mean of residuals between
;                                 model and data. After taking the mean the
;                                 mean is divided by the mean RMS. 
;                                 Best for clean data where the model is
;                                 not giving a good representation of the
;                                 detector response to illumination steps.
;         IERR         error flag. should be 0 on input
;                       
; output: SILLUMOUT    (array in V/s): best fit illumination history 
;                      over the slew, at the times in T since the detector
;                      was set at it's initial state (SILLUMPREV, SIGNALPREV). 
;         MASK         (integer array in V/s): Set to 1 if chopper changed
;                      from first position at a point before the last point.
;         FITRESULT    an output parameter normalised to RMS indicating how 
;                      good the fit is, calculated according to the value of 
;                      the input parameter ISQUARE. It is always output as
;                      zero in this routine as there is only one data point
;                      to be fitted (see descriptino of ISQUARE above), and
;                      there is no statistical information on the goodness of 
;                      fit in this case.
;         MODELFIT     (array in V/s): best fit model detector response to 
;                      illumination history SILLUMOUT over the slew at the 
;                      times in T since the detector was set at it's initial 
;                      state (SILLUMPREV, SIGNALPREV). for the plateau. 
;         IERR         =1 if an error occurred
;
; internal parameter: 
;         SILLUM      (single number in V/s): trial illumination 
;                      for the last time sample of T.
;
;
n=n_elements(t)
sillumout=fltarr(n)
mask=lonarr(n) ; default is that output fit to illumination history good
xnpt=float(n)
rmsmean=rms
;print,'xnpt,rmsmean = ',xnpt,rmsmean
;
if(rms le 0.0)then begin
print,'SOLVESLEW1: rms zero or negative '
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(fitprec lt 0.1 or fitprec gt 1.0)then begin
print,'SOLVESLEW1: fitprec set too low for fast convergence or too high'
print,' for precision within the noise - fitprec = ',fitprec
ierr=1
goto,c999
endif
;
if(isquare lt 0 or isquare gt 2)then begin
print,'SOLVESLEW1: isquare out of range - isquare = ',isquare
ierr=1
goto,c999
endif
;
if(imodel ne 1 and imodel ne 2)then begin
print,'SOLVESLEW1: model must be Foukes-Schubert or exponential'
ierr=1
goto,c999
endif
;
if(signal le 0.0)then begin
print,'SOLVESLEW1: aborting: detector data must be positive - '
print,'min(signal) = ',min(signal)
ierr=1
goto,c999
endif
;
if(beta le 0.0 or beta ge 1.0)then begin
print,'SOLVESLEW1: aborting: beta out of range - beta = ',beta
ierr=1
goto,c999
endif

if(tau le 0.0)then begin
print,'SOLVESLEW1: aborting: tau negative - tau = ',tau
ierr=1
goto,c999
endif
;
if(signalprev le 0.0)then begin
print,'SOLVESLEW1: aborting: SIGNALPREV must be positive - signalprev = ',$
signalprev
ierr=1
goto,c999
endif
;
if(sillumprev le 0.0)then begin
print,'SOLVESLEW1: '
print,'aborting: SILLUMPREV must be positive - sillumprev = ',sillumprev
ierr=1
goto,c999
endif
;
if(min(t) le 0.0)then begin
print,'SOLVESLEW1: aborting: time array t must be a positive interval since'
print,'the last destructive read - min(t) = ',min(t)
ierr=1
goto,c999
endif
;
if(silmin le 0.0 or silmax le silmin)then begin
print,'SOLVESLEW1: aborting: SILMIN must be positive and less than SILMAX - '
print,'silmin,silmax = ',silmin,silmax
ierr=1
goto,c999
endif
;
; internal constants controlling trade off between speed, precision and 
; robustness of fit
;
niter=1 ; for diagnostics - set to a large number to control convergence
        ; by FITPREC
ncut=5 ; must be odd and at least 5
reduce=1.5 ; must be in range > 1.0 (slow  to 2.0 (fast)
chiresult=fltarr(ncut)
;
xncut=float(ncut)
top=silmax
bot=silmin
reducefactor=2.0
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
;
; set up template array according to whether or not the chopper changed
; position at a mid point in array. each trial illumination on the slew is set
; up for step mode (cmod=1) as:    sillumtrial(i1:i2)=sillumprev 
;                          and     sillumtrial(i2:i3)=sillum
; and with signal to compare model prediction:
;                                  sigcompare(i1:i2)=signalprev
;                                  sigcompare(i2:i3)=signal
; and for ramp mode (cmode= 0) as: 
;     sillumtrial(*)=sillumprev+(sillum-sillumprev)*template
; and with signal to compare model prediction:
;            sigcompare(*)=signalprev+(signal-signalprev)*template
;
template=fltarr(n)
sillumtrial=template
if(n eq 1)then begin
cmod=0 ; linear interpolation mode
template=fltarr(1)
template(0)=1.0D
; i1 to i2 is level of SILLUMPREV (0 to 0)
; i2 to i3 is (overwritten) linear ramp from element 0
; SILLUMPREV+(SILLUM-SILLUMPREV)*template
i1=0
i2=0
i3=0
endif ; n eq 1
;
if(n gt 1)then begin
;test=abs(ech-echprev)
tester=abs(ech-echprev)
test=tester
test(1:n-1)=tester(1:n-1)-tester(0:n-2)
test(0)=tester(0)
itest=where(test gt 0,nitest) ; number of changes in illumination
template(*)=1.0D
if(nitest le 0)then begin
cmod=0 ; no chopper change - linear interpolation mode
;template=(1.0+findgen(n))/xnpt
template=t/max(t)
; i1 to i2 is level of SILLUMPREV (0 to 0)
; i2 to i3 is (overwritten) linear ramp from element 0
; SILLUMPREV+(SILLUM-SILLUMPREV)*template
i1=0
i2=0
i3=n-1
endif ; nitest le 0
;
if(nitest gt 0)then begin
cmod=1 ; chopper changes - step mode
if(n ge 128 or max(test(0:n-2)) gt 0)then mask(*)=1 ; chopper changes in  
;                                             mid slew so solution unreliable
ichange=0
if(nitest gt 1)then ichange=nitest/2 ; if several chopper changes change in
;                                        middle
; i1 to i2 is level of SILLUMPREV
; i2 to i3 is (overwritten) level of internal parameter SILLUM
i1=0
i2=itest(ichange) 
i3=n-1
endif ; nitest gt 0
;
endif ; n gt 1
;
; set up comparison signal and correspnding rms. We do not attempt to
; interpolate rms
;
sigcompare=t
rmscompare=t
rmscompare(*)=rmsmean
;
if(cmod eq 0)then begin
sigcompare(*)=signalprev+(signal-signalprev)*template
endif ; cmod eq 0
if(cmod eq 1)then begin
sigcompare(i1:i2)=signalprev
sigcompare(i2:i3)=signal ; (overwrites)
endif ; cmod eq 1
;
;print,'bot,top,dcut,sigtrial = ',bot,top,dcut,sigtrial
;
if(cmod eq 0)then sillumtrial(*)=sillumprev+(sigtrial(0)-sillumprev)*template
if(cmod eq 1)then begin
sillumtrial(i1:i2)=sillumprev ; this need from here on not be changed
sillumtrial(i2:i3)=sigtrial(0) ; must be changed for each trial
endif ; cmod eq 1
;
;print,'imodel,sillumtrial,beta,tau = ',imodel,sillumtrial,beta,tau
;print,'signalprev,sillumprev = ',signalprev,sillumprev
;print,'t ',t
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumtrial,t,model
;
;help,model
;print,'maxmin of model = ',max(model),min(model)
;help,sillumtrial
;print,'maxmin of sillumtrial',sillumtrial
;res=signal-model
;resrms=res/rms
;help,resrms
;plot,t,model
;if(isquare eq 1)then sumsq=sqrt(total(resrms*resrms)/xnpt)
; rjt sep11_99 if(isquare eq 0)then sumsq=abs(total(res)/xnpt)/rmsmean
;if(isquare eq 0)then sumsq=(total(abs(res))/xnpt) / rmsmean
;if(isquare eq 2)then begin
; rjt sep11_99 mediantrue,res,resmedtrue
; rjt sep11_99 sumsq=abs(median(resmedtrue))/rmsmean
;mediantrue,abs(res),resmedtrue
;sumsq=resmedtrue/rmsmean
;endif ; isquare eq 2
;
;model1=model(n-1)
;findfitresult,signal,rmsmean,model1,sumsq,isquare
findfitresult,sigcompare,rmscompare,model,sumsq,isquare
;
icutmin=0
sumsqmin=sumsq
sillum=sigtrial(0)
modelfit=model
;print,'sumsqmin: ',sumsqmin
;
iter=0
;
c100:
;
for icut=0,ncut-1 do begin
;
if(cmod eq 0)then $
sillumtrial(*)=sillumprev+(sigtrial(icut)-sillumprev)*template
if(cmod eq 1)then sillumtrial(i2:i3)=sigtrial(icut) 
;
if(imodel eq 1)then foukes,beta,tau,signalprev,sillumprev,sillumtrial,t,model
if(imodel eq 2)then expfun,beta,tau,signalprev,sillumprev,sillumtrial,t,model
;
;res=signal-model
;resrms=res/rms
;if(isquare eq 1)then sumsq=sqrt(total(resrms*resrms)/xnpt)
; rjt sep11_99 if(isquare eq 0)then sumsq=abs(total(res)/xnpt)/rmsmean
;if(isquare eq 0)then sumsq=(total(abs(res))/xnpt) / rmsmean
;if(isquare eq 2)then begin
; rjt sep11_99 mediantrue,res,resmedtrue
; rjt sep11_99 sumsq=abs(median(resmedtrue))/rmsmean
;mediantrue,abs(res),resmedtrue
;sumsq=resmedtrue/rmsmean
;endif ; isquare eq 2
;
;model1=model(n-1)
;findfitresult,signal,rmsmean,model1,sumsq,isquare
findfitresult,sigcompare,rmscompare,model,sumsq,isquare
chiresult(icut)=sumsq
;
; rjt sep 10 1999 icutmin=0
;
if(sumsq lt sumsqmin)then begin ; found new minimum
icutmin=icut
sumsqmin=sumsq
sillum=sigtrial(icut)
modelfit=model
;help,modelfit
;print,'maxmin model fit = ',max(modelfit),min(modelfit)
;print,'icut,sillum,sumsqmin',icut,sillum,sumsqmin
endif ; sumsq lt sumsqmin
;
endfor ; icut
;
if(dcut le fitprec*rmsmean)then goto, c200
;if(ncut ge 5)then begin
;cutoptimise,chiresult,ncut,reduce,ncutout,reduceout,ioptimize
;if(dcut le rmsmean)then begin ; speed up towards solution
;ncut=3
;reduce=2.0
;endif ; dcut le rmsmean
;endif ; ncut ge 5
d=top-bot
d=d/reduce
d2=d/2.0
bot=sillum-d2
top=sillum+d2
bottmp=bot
toptmp=top
if(bot lt silmin)then begin
bottmp=silmin
toptmp=bottmp+d
endif ; bot
if(top gt silmax)then begin
toptmp=silmax
bottmp=toptmp-d
endif ; top
bot=bottmp
top=toptmp
dcut=(top-bot)/(xncut-1.0)
sigtrial=bot+dcut*findgen(ncut)
;
iter=iter+1
;print,'iter,bot,top,sigtrial ',iter,bot,top,sigtrial
;
goto, c100
;
c200:
;
;if(xnpt gt 1.0)then fitresult=sumsqmin*xnpt/(xnpt-1.0)
;if(xnpt le 1.0)then fitresult=0.0 ; no contribution from single point
;                                   plateaus to fit
fitresult=0.0
;
;
if(cmod eq 0)then sillumout(*)=sillumprev+(sillum-sillumprev)*template
if(cmod eq 1)then begin
sillumout(i1:i2)=sillumprev ; this need from here on not be changed
sillumout(i2:i3)=sillum ; must be changed for each trial
endif ; cmod eq 1
;
c999:
;
return
end



