;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO ACC_VOLT, ICONP, STFLAG, DR, VFLAG
;+
; NAME:
;	ACC_VOLT
;
; PURPOSE:
;	To return a bad flag array 
;	taking into account voltage limits (a minimum 
;	and a maximum accepted, e.g. saturation voltage) 
;	according to the CB PHTACC  
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	ACC_VOLT, ICONP, VFlag, DR
;
; INPUTS:
;	IConp	: Read-out voltages array
;	STFlag	: Status Flag before call
;	DR	: Array indicating Destructive Readouts
;
; OUTPUTS:
;	STFlag	: Flag array (Input Vflag + 1:out of voltage limits)
;	VFlag   : Flag array (1= out of voltage limits)	
;
; COMMON BLOCKS:
;               phtacc
;
; PROCEDURE:
;	Using the CB PHTACC, flag the values which are
;	not conform to the minimum and maximum voltages
;	In case that a value is above the allowed maximum voltage, all
;	values up to the end of the ramp are flagged as bad 
;
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel
;	Modified by CG - (last: Nov. 96)
;	V5.x
;	An additional output (VFLAG) containing pure voltage flag, to
;	be consistent with FLAG code (PIA >7.3) 	      JAcosta (May 1999)
;	V7.4
;-

@phtacc

;
; On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 3 THEN MESSAGE, $
	   'Syntax: ACC_VOLT, ICONP, STFLAG, DR, VFLAG '
	   
	nele = N_ELEMENTS(iconp)   
	   
	vflag = stflag * 0  

; discard voltages below lower limit
	IND = WHERE (ICONP LT MINVOLT, N)
	IF (N GT 0) THEN BEGIN
;		MESSAGE, /INFO, ' discarding voltages below limit'
		vflag(ind) = 1
	ENDIF

; discard voltages above upper limit 
	IND = WHERE (ICONP GT MAXVOLT, N)
	IF (N GT 0) THEN BEGIN
;		MESSAGE, /INFO, ' discarding voltages above limit'
		vflag(ind) = 1
	ENDIF

; where are the DRs
	inddr = WHERE(DR EQ 1, ndr)
	IF ndr EQ 0 THEN BEGIN 
		  MESSAGE, /INFO, 'No Destructive Reads ? , Returning'
		  RETURN
	ENDIF

; discard all ramps with a reset above volt=0.3

; if last read-out is a DR ignore it - (nothing coming after)
	IF nele-1 EQ max(inddr) THEN BEGIN
		inddr = inddr(0:N_ELEMENTS(inddr)-2)
		ndr = ndr - 1
	ENDIF
	lastdr = max(inddr)
; first ramp	
	IF iconp(0) GT 0.3 THEN BEGIN 
		vflag(0:inddr(0)) = 1
;		MESSAGE, /INFO, ' discarding first ramp because of high reset'
        ENDIF

        ind0v = WHERE(iconp(inddr+1) GT 0.3, n0v)

	IF n0v GT 0 THEN BEGIN
		FOR I=0L, n0v-2 DO BEGIN
			VFLAG(inddr(ind0v(I))+1:inddr(ind0v(I)+1)) = 1
;		   IF I LT 5 THEN $
;		     MESSAGE, /INFO, ' discarding a ramp because of high reset'
		   IF i EQ 50 THEN $  
		     MESSAGE, /INFO, ' > 50 x discard because of high reset'
                ENDFOR
; last ramp
	        IF inddr(ind0v(n0v-1))+1 GT max(inddr) THEN $
			vflag(inddr(ind0v(n0v-1))+1:*) = 1 $
		ELSE $   
			vflag(inddr(ind0v(n0v-1))+1:inddr(ind0v(n0v-1)+1)) = 1 
        ENDIF


; now discard all ramps having a negative gradient above 0.6 V - 

	delta = fltarr(nele-1)
	delta = iconp(1:nele-1) - iconp(0:nele-2)

; first ramp
	IF inddr(0) EQ 0 THEN BEGIN
	  inddr = inddr(1:*)
	  ndr = ndr-1
	ENDIF
	
	deltramp = delta(0:inddr(0)-1)
	indneg = WHERE(deltramp LT 0. AND iconp(1:inddr(0)) GT 0.6, nneg)
	IF nneg NE 0 THEN BEGIN
		vflag(indneg(0):inddr(0)) = 1 
;		MESSAGE, /INFO, $
;		 ' discarding ramp partly because of negative diff for V > 0.6 '
        ENDIF
; all ramps
	count = 0l
	FOR II=0l, ndr-2 DO BEGIN
; prevent for telemetry drop-outs from DR to DR
	 IF inddr(II)+1 GE inddr(II+1)-1 THEN GOTO, jump_over	 

	 deltramp = delta(inddr(II)+1:inddr(II+1)-1)
	 indneg = $
	   WHERE(deltramp LT 0. AND iconp(inddr(II)+2:inddr(II+1)) GT .6, nneg)
	   IF nneg NE 0 THEN BEGIN
		count = count + 1
		vflag(inddr(II)+1+indneg(0):inddr(II+1)) = 1 

;		IF count LT 5 THEN $
;		 MESSAGE, /INFO, $
;		  ' discarding ramp partly because of negative diff for V > 0.6'
		IF count EQ 50 THEN $
		 MESSAGE, /INFO, $
		  ' > 50 x discard partly because of negative diff for V > 0.6'
           ENDIF
jump_over:
	ENDFOR
; last ramp - be careful if only one point in the last ramp no delta
        IF N_ELEMENTS(delta) GT (inddr(ndr-1)+1) THEN BEGIN
	 deltramp = delta(inddr(ndr-1)+1:*) 	  
	  indneg = $
	   WHERE(deltramp LT 0. AND iconp(inddr(ndr-1)+2:*) GT 0.6, nneg)
	   IF nneg NE 0 THEN BEGIN
		vflag(inddr(ndr-1)+1+indneg(0):*) = 1 
;		MESSAGE, /INFO, $
;		  ' discarding ramp partly because of negative diff for V > 0.6'
           ENDIF
        ENDIF
	
	;update stflag to be consistent with previous versions (PIA <V7.4)
	indv = WHERE(vflag GT 0, nv)
	IF nv GT 0 THEN stflag(indv) = 1
	
	RETURN
	END
