;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;! This routine belongs to the PHT-Interactive Analysis package, PIA.        !
;!                                                                           !
;! PIA was developed by the European Space Agency (ESA/ESTEC)                !
;! in collaboration with                                                     !
;!   Max-Planck-Institute for Nuclear Physics (MPIK), Heidelberg, Germany,   !
;!   Dublin Institute for Advanced Studies (DIAS), Dublin, Ireland,          !
;!   Rutherford Appleton Laboratory, (DRAL), Chilton, UK, and                !
;!   Astronomy Institute Potsdam, (AIP), Potsdam, Germany,                   !
;! with support from Max-Planck-Institute for Astronomy (MPIA), Heidelberg.  !
;!                                                                           !
;! It shall be used for scientific analysis and calibration of ISOPHOT data. !
;!                                                                           !
;! Its use and distribution are free, as long as this header text is not     !
;! removed, or changed. When publishing data reduced with this analysis      !
;! package, please mention this in the acknowledgement.                      !
;!                                                                           !
;! No warranties for installation / support/ maintenance can be given.       !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO anal_fpgc, Infile, Text, ERROR=error 

;+
; NAME:
;     ANAL_FPGC
;
; PURPOSE:
;       Analyse a Focal Plane Geometry Calibration File and and compare
;	it to theoretical raster.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	ANAL_FPGC, Infile, ERROR=error
;
; INPUTS:
; 	Infile: name of FPGC file
;
; KEYWORD PARAMETERS:
;	ERROR: Returns error status:
;		0 = no error
;		1 = error: output is not valid
;
; OUTPUTS:
;	Text: print-out contents of FPGC file
;
; PROCEDURES USED:
;	PIA LIB: read_fpgc
;       PIA LIB: pia_xplot
;
; MODIFICATION HISTORY:
;       Written by:	I. Heinrichsen
;	Modified by:	C. Gabriel
;
;	Oct 95 Version 3.5
;
;	Modified by:	I. Heinrichsen	Rotation angle, coordinate  Oct. 95
;					display corrected
;			IH		Change axes label
;					invert theoretical coordinates Nov. 95
;-
;
  on_error, 2
;
; initialize error status
	error = 1
; initialize debug
; 		debug=1 give detailed print-out
;		debug=0 give no print-out
	debug = 1
;
; define raster orientation
; zsign = +1 start with the negative z-coordinate
; zsign = -1 start with positive z-coordinate
zsign = +1
; ysign = +1 start with the negative y-coordinate
; ysign = -1 start with positive y-coordinate
ysign = -1
;
; define text array for print-out into window
text=strarr(1)
	
; read FPGC file
read_fpgc, infile, npts, nlin, dst_pts, dst_lin, rota, $
                  rline, rcolumn, ycoor, zcoor, TEXTOUT=text, ERROR=read_error 

if (read_error ne 0) then goto, readerror

; generate theoretical coordinates
ytheo=fltarr(nlin*npts)
ztheo=fltarr(nlin*npts)

for j=0,nlin-1 do begin
  for i=0,npts-1 do begin
    case (nlin mod 2) of
  	0: ztheo(i+j*npts)=zsign*((j-nlin/2)*dst_lin+float(dst_lin)/2.)
  	1: ztheo(i+j*npts)=zsign*(j-nlin/2)*dst_lin
    endcase
    case (npts mod 2) of 
    1: begin
	case (j mod 2) of  
	  0: ytheo(i+j*npts)=ysign*(i-npts/2)*dst_pts
	  1: ytheo(i+j*npts)=-ysign*(i-npts/2)*dst_pts
	endcase
	end
    0: begin
	case (j mod 2) of  
	  0: ytheo(i+j*npts)=ysign*((i-npts/2)*dst_pts+float(dst_pts)/2.)
	  1: ytheo(i+j*npts)=-ysign*((i-npts/2)*dst_pts+float(dst_pts)/2.)
	endcase
        end
    endcase
  endfor 
endfor

; rotate theoretical coordinates
; switch off rotation !!!
 if ((npts eq 2) or (nlin eq 2)) then begin
 	if (abs(ycoor(4)-ycoor(1)) gt abs(zcoor(4)-zcoor(1))) then begin
 		rota=0.
 		print, 'case 1 rota = 0'
 		print, abs(ycoor(4)-ycoor(1)), abs(zcoor(4)-zcoor(1))
 	endif else begin
 		rota=90.
 		print, 'case 2 rota = 90'
 		print, abs(ycoor(4)-ycoor(1)), abs(zcoor(4)-zcoor(1))
 	endelse
 endif else begin
 	rota=0.
 endelse
 rota_rad=-!pi*rota/180.
 ytmp=ytheo*cos(rota_rad)+ztheo*sin(rota_rad)
 ztmp=ztheo*cos(rota_rad)-ytheo*sin(rota_rad)
 ytheo=ytmp
 ztheo=ztmp

; don't ask me why
ytheo=-ytheo
ztheo=-ztheo
print, ytheo
print, ztheo


if (debug eq 1) then begin
	text = [text, ' ']
	text = [text, ' ']
	text = [text,  "Printing theoretical data contents and differences:"]
	text = [text, ' ']
	text = [text,  "Asumption nsteps count in Y-coordinate"] 
	text=  [text, "(small increment)"]
	case zsign of
	1: text = [text,  "Asumption start with the negative Z-coordinate"]
	-1: text = [text,  "Asumption start with the positive Z-coordinate"]
	else: text = [text, " Warning zsign NOT set correctly !!!"]
	endcase
	case ysign of
	1: text = [text,  "Asumption start with the negative Y-coordinate"]
	-1: text= [text, "Asumption start with the positive Y-coordinate"]
	else: text = [text, " Warning ysign NOT set correctly !!!"]
	endcase
 	text = [text,  ' '] 
        text = [text,  'Theoretical Y Coordinates: ']
        text = [text,  string(ytheo)]
        text = [text,  'Theoretical Z Coordinates: ']
        text = [text,  string(ztheo)]
endif

;calculate distances
diff=sqrt((ycoor-ytheo)*(ycoor-ytheo)+(zcoor-ztheo)*(zcoor-ztheo))
if (debug eq 1) then begin
	text = [text,  'Euclidic Distances: ']
	text = [text,  string(diff)]
endif


;
; plot theoretical versus FPGC coordinates
;
pia_xplot, ztheo, ytheo, $
	   title='Comparison of nominal and FPGC raster coordinates', $
	   xtitle='S/C Z-coordinate [arcsec]',$
	   ytitle='S/C Y-coordinate [arcsec]', $
	   legend='Theoretical raster coordinates', psym=1
pia_xplot, ztheo, ytheo, $
           legend='Theoretical raster coordinates', linestyle=1, /overplot	   
pia_xplot, zcoor, ycoor, $
           legend='Raster coordinates from FPGC file', psym=2, /overplot
pia_xplot, zcoor, ycoor, $
           legend='Raster coordinates from FPGC file', linestyle=2, /overplot
;
; Plot euclidic distances theoretical/FPGC coordinates
; 
pia_xplot, diff(0:npts-1), $
   	   title='Difference between nominal and FPGC raster coordinates', $
	   xtitle='Raster point id - 1',$
	   ytitle='Euclidic distance nominal/FPGC'
pia_xplot, diff(0:npts-1), psym=7, /overplot
for j=1, nlin-1 do begin
	   pia_xplot, diff(j*npts:(j+1)*npts-1), linestyle=(j mod 5), /overplot
	   pia_xplot, diff(j*npts:(j+1)*npts-1), psym=(j mod 7), /overplot
endfor
;
; everything ok.
	error = 0
	return
	
readerror:
	print, "ANAL_FPGC: Unable to read file:", infile
	return

end
