;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro anal_sig, ind, admn, time, signal, noise, flags, st_time, TYPE=type, $ 
    ERROR=error 
;+
; NAME:
;     ANAL_SIG
;
; PURPOSE:
;       Analyse several SRD/SCP measurements
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	ANAL_SIG, ind, orbstr
;
; INPUTS:
;	Ind: index array containing indexes for extraction from SRD structure
;
;
; KEYWORD PARAMETERS:
;
;	TYPE 	type of PIA structure to be analysed (SRD of SCP), default=SRD
;
;	ERROR		Returns an error status:
;						0 = no error
;						1 = error: output is not valid
;
; OUTPUTS:
;	ADMN    administration name
;	TIME	array with absolute times for SIGNAL and NOISE
;	SIGNAL	array of signals (SLOP, AVER) per detector for all 
;		selected measurements from ind
;	NOISE	array of  uncertainties per detector (SLUN, AVUN) for all 
;		selected measuremenst from ind
;	FLAGS  	array of flags per detector (FLAG) for all 
;		selected measuremenst from ind
;	ST_TIME	Absolute start time (STRING)
;
; COMMON BLOCKS:
;	phtsrd
;	phtscp
;
; PROCEDURES CALLED:
;
; RESTRICTIONS
;	Due to the nature of the timing information for PHT the absolute 
;	start time of the contained measuremnets are not as accurate as 
;	as the relative times within a measurement 
;
; MODIFICATION HISTORY:
;       Written by:	Ingolf Heinrichsen		December 1995
;	Bug fixed (CG)	(phtsrd instead phtscp)		January 1996
;	time and other arrays now of same dims (CG)	February 1996
;	Bug for 1 pixel & 1 signal fixed (CG)		February 1996
;	Change call of date_conv to unit2juldate +
;	    adding admn as output                (JAc)  January 1998
;
;-
;
; initialize common blocks
@phtsrd
@phtscp
;
;  on_error, 2
;
; initalize error state
;
	error=1
;
; check for parameters
;
 	npar = N_params()
 	if npar EQ 0 then goto, nopar
 	if n_elements(ind) eq 0 then goto, noind
 	if n_elements(type) eq 0 then type='SRD'

; 
; define arrays
;
 	signal=fltarr(2,2)
	noise=fltarr(2,2)
	flags=fltarr(2,2)
	time=fltarr(1)


CASE type OF
'SRD':BEGIN


;
; get/convert times for selected measurements
;
	n=n_elements(ind)
	timestr = PHTSRD(ind).UNIT
	timearr=dblarr(n)
	for i=0, n-1 do begin
	 	timearr(i) = unit2juldate(timestr(i))
	endfor
;
; normalise to time of first measurement
; and convert to seconds
;	
	timearr=timearr-min(timearr)
	timearr=timearr*60*60*24
	tind=bsort(timearr)	
;
; check for all data from same detector
;
	olddet=PHTSRD(ind(0)).det
	for i=0, n-1 do begin
		det=PHTSRD(ind(i)).det
		if olddet ne det then goto, notsame
		olddet=det
	endfor
;
; get signal and noise data, flags and times within measurements
; 	
; initialize for first iteration
;
	handle_value, phtsrd(ind(tind(0))).slop, slop
	slop = [slop]
	s=size(slop)
	if s(0) eq 1 then s(2)=1
	sigold=fltarr(s(1), s(2))
	signew=fltarr(s(1), s(2))
	noinew=fltarr(s(1), s(2))
	noiold=fltarr(s(1), s(2))
	flgnew=fltarr(s(1), s(2))
	flgold=fltarr(s(1), s(2))
;
; for all selected measurements
;
	for i=0, n-1 do begin
; write st_time and admn
	        IF i EQ 0 THEN BEGIN
	           st_time = phtsrd(ind(tind(i))).unit
	           admn = phtsrd(ind(tind(i))).admn
	        ENDIF ELSE BEGIN 
	           st_time = [st_time, phtsrd(ind(tind(i))).unit] 
	           admn = [admn, phtsrd(ind(tind(i))).admn] 
	        ENDELSE   
	        
		handle_value, phtsrd(ind(tind(i))).slop, slop
		handle_value, phtsrd(ind(tind(i))).slun, slun
		handle_value, phtsrd(ind(tind(i))).stim, stim
		handle_value, phtsrd(ind(tind(i))).flag, flag
; make times
		stim=stim+timearr(tind(i))
		IF i EQ 0 THEN time=stim ELSE $
			time=[time,stim]

; signals for all pixels
		slop = [slop]
		s_slop=size(slop)
		if s_slop(0) eq 1 then s_slop(2)=1
		s_sigold=size(sigold)
		if s_sigold(0) eq 1 then s_sigold(2)=1
		if i gt 0 then signew=fltarr(s_slop(1),s_slop(2)+s_sigold(2))
		for j=0,s_slop(1)-1 do begin
			if i eq 0 then begin
				signew=slop
			endif else begin
				signew(j,*)=[reform(sigold(j,*)),reform(slop(j,*))]
			endelse
		endfor
		sigold=signew

; noises for all pixels
		slun = [slun]
		s_noi=size(slun)
		if s_noi(0) eq 1 then s_noi(2)=1
		s_noiold=size(noiold)
		if s_noiold(0) eq 1 then s_noiold(2)=1
		if i gt 0 then noinew=fltarr(s_noi(1),s_noi(2)+s_noiold(2))
		for j=0,s_noi(1)-1 do begin
			if i eq 0 then begin
				noinew=slun
			endif else begin
				noinew(j,*)=[reform(noiold(j,*)),reform(slun(j,*))]
			endelse
		endfor
		noiold=noinew

; flags for all pixels
		flag = [flag]
		s_flg=size(flag)
		if s_flg(0) eq 1 then s_flg(2)=1
		s_flgold=size(flgold)
		if s_flgold(0) eq 1 then s_flgold(2)=1
		if i gt 0 then flgnew=fltarr(s_flg(1),s_flg(2)+s_flgold(2))
		for j=0,s_flg(1)-1 do begin
			if i eq 0 then begin
				flgnew=flag
			endif else begin
				flgnew(j,*)=[reform(flgold(j,*)),reform(flag(j,*))]
			endelse
		endfor
		flgold=flgnew
	endfor
; set it !
	flags=flgnew
	signal=signew
	noise =noinew
;
; everything ok.
;
	error = 0
	return

END
'SCP':BEGIN
;
; get/convert times for selected measurements
;
	n=n_elements(ind)
	timestr = PHTSCP(ind).UNIT
	timearr=dblarr(n)
	for i=0, n-1 do begin
	 	timearr(i) = unit2juldate(timestr(i))
	endfor
;
; normalise to time of first measurement
; and convert days to seconds
;	
	timearr=timearr-min(timearr)
	timearr=timearr*60*60*24
	tind=bsort(timearr)	
;
; check for all data from same detector
;
	olddet=PHTSCP(ind(0)).det
	for i=0, n-1 do begin
		det=PHTSCP(ind(i)).det
		if olddet ne det then goto, notsame
		olddet=det
	endfor
;
; get signal and noise data and times within measurements
;
; initialize for first measurement
; 	
	handle_value, phtscp(ind(tind(0))).aver, slop
; if just a scalar then s(0)=0 -> convert to pseudo-vector
	slop = [slop]
	s=size(slop)
	if s(0) eq 1 then s(2)=1
	sigold=fltarr(s(1), s(2))
	signew=fltarr(s(1), s(2))
	noinew=fltarr(s(1), s(2))
	noiold=fltarr(s(1), s(2))
	flgnew=fltarr(s(1), s(2))
	flgold=fltarr(s(1), s(2))

	for i=0, n-1 do begin
; write st_time and admn
	        IF i EQ 0 THEN BEGIN
	           st_time = phtscp(ind(tind(i))).unit
	           admn = phtscp(ind(tind(i))).admn
	        ENDIF ELSE BEGIN 
	           st_time = [st_time, phtscp(ind(tind(i))).unit] 
	           admn = [admn, phtscp(ind(tind(i))).admn] 
	        ENDELSE   
	        
		handle_value, phtscp(ind(tind(i))).aver, slop
		handle_value, phtscp(ind(tind(i))).avun, slun
		handle_value, phtscp(ind(tind(i))).time, stim
		handle_value, phtscp(ind(tind(i))).flag, flag

		stim=stim+timearr(tind(i))
		IF i EQ 0 THEN time = stim ELSE $
			time=[time,stim]
	
; signals for all pixels
		slop = [slop]
		s_slop=size(slop)
		if s_slop(0) eq 1 then s_slop(2)=1
		s_sigold=size(sigold)
		if s_sigold(0) eq 1 then s_sigold(2)=1
		if i gt 0 then signew=fltarr(s_slop(1),s_slop(2)+s_sigold(2))
		for j=0,s_slop(1)-1 do begin
			if i eq 0 then begin
				signew=slop
			endif else begin
				signew(j,*)=[reform(sigold(j,*)),reform(slop(j,*))]
			endelse
		endfor
		sigold=signew

; noises for all pixels
		slun = [slun]
		s_noi=size(slun)
		if s_noi(0) eq 1 then s_noi(2)=1
		s_noiold=size(noiold)
		if s_noiold(0) eq 1 then s_noiold(2)=1		
		if i gt 0 then noinew=fltarr(s_noi(1),s_noi(2)+s_noiold(2))
		for j=0,s_noi(1)-1 do begin
			if i eq 0 then begin
				noinew=slun
			endif else begin
				noinew(j,*)=[reform(noiold(j,*)),reform(slun(j,*))]
			endelse
		endfor
		noiold=noinew

; flags for all pixels
		flag = [flag]
		s_flg=size(flag)
		if s_flg(0) eq 1 then s_flg(2)=1
		s_flgold=size(flgold)
		if s_flgold(0) eq 1 then s_flgold(2)=1
		if i gt 0 then flgnew=fltarr(s_flg(1),s_flg(2)+s_flgold(2))
		for j=0,s_flg(1)-1 do begin
			if i eq 0 then begin
				flgnew=flag
			endif else begin
				flgnew(j,*)=[reform(flgold(j,*)),reform(flag(j,*))]
			endelse
		endfor
		flgold=flgnew
	endfor
	signal=signew
	noise =noinew
	flags=flgnew

;
; everything ok.
;
	error = 0
	return

END
ELSE: goto, wrongtype
ENDCASE
;
; error handling
;
nopar:
	print,"anal_srd, ind, admn, signal, noise, PIXEL=pixel, ERROR=error"
	return

notsame:
	message, "Not all selected SRD/SCP data from same detector"
	return
noind:
	message, "No valid index array specified"
	return
wrongtype:
	message, "Wrong PIA structure type specified, (valid: SRD/SCP)"
end










