;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Calc_Aper, SubInst, Pixel, Point, Line, Chop, x, y
;+
; NAME:
;	CALC_APER
;
; PURPOSE:
;	This procedure calculates the image co-ordinates of each readout
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	CALC_APER, SunInst, Pixel, Point, Line, Chop, X, Y
;
; INPUTS:
;	SubInst: PHT Subinstrument - 'C1', 'C2', 'P', 'S'
;	Pixel: Pixel number - Only for PHT-C (anything otherwise)
;	Point: Raster Point - Only for raster scans (anything otherwise)
;	Line: Raster Line   - Only for raster scans (anything otherwise)
;	Chop: Array of chopper positions
;
; OUTPUTS:
;	X: x co-ordinate of aperture centre
;	Y: y co-ordinate of aperture centre
;
; COMMON BLOCKS:
;	raster_info
;
; PROCEDURES CALLED:
; 	None
;
; MODIFICATION HISTORY:
;	Written by:	Huw Morris
;	Oct 95 Version 3.5
;-


@raster_info

	ON_ERROR, 2            ; Return to calling procedure

; Define the pixel sizes for PHT-C
	C1Pix = 43.5
	C2Pix = 89.4

; If a raster scan then calculate the offset of each point from the centre
; field of view (defined as the centre of the raster scan)
	IF (Att_Type EQ 'R') THEN BEGIN

; Calculate the position of the centre of the aperture in angular
; co-ordinates for the raster position (point, line)
  	  xr = (Point - ((NPoints - 1.0)/2.0) - 1.0) * SPoints
	  yr = (Line - ((NLines - 1.0)/2.0) -1.0) * SLines

	ENDIF ELSE BEGIN

	  xr = 0.0    ; else looking at the image centre
	  yr = 0.0

	ENDELSE

; Initialise offset due to the pixels of PHT-C
	xk = 0.0
	yk = 0.0

; For PHT-C, there is a correction due to the pixel number
	IF (SubInst EQ 'C1') THEN BEGIN
	  PixX = (Pixel MOD 3)
	  PixY = FIX(Pixel / 3)

	  xk = (PixX - 1.0) * C1Pix
	  yk = (1.0 - PixY) * C1Pix
	ENDIF

        IF (SubInst EQ 'C2') THEN BEGIN
          PixX = (Pixel MOD 2)
          PixY = FIX(Pixel / 2)

          xk = (PixX - 0.5) * C2Pix
          yk = (0.5 - PixY) * C2Pix
        ENDIF

; Now take into account the chopper position
	xc = Chop * SIN(MChop)
	yc = Chop * COS(MChop)

; From all these factors, work out the centre of the aperture in image 
; co-ordinates (in degrees)
	xa = (xr + xk + xc) / 3.6d3
	ya = (yr + yk + yc) / 3.6d3

; Take the roll angle (in radians) into consideration
	R_Ang = (Roll + MRaster) * !DTOR    ; Raster angle = instrument rotation
                                            ; + raster misalignment
    	x = (xa * COS(R_Ang)) + (ya * SIN(R_Ang))
       	y = (ya * COS(R_Ang)) - (xa * SIN(R_Ang))
	
	END

