;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_DARKCUR_PROC
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to the Dark Current File production
;
; CATEGORY:
;          PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_DARKCUR_PROC, det, newdarkcurr, cal_darkc_str, newnep
;
; INPUT:
;       det: String indicating the detector 
;
; OUTPUT:
;	newdarkcurr: computed dark current
;	cal_darkc_str: computed dark currents structure
;	newnep: computed minimum NEP
;
; COMMON BLOCKS:
;	deglitch_param
;	phtcs
;	phtacc
;	phtcap
;	phterd
;	phtresdef
;	phtscp
;	phtsrd
;	gencho (internal)
;
; PROCEDURES USED:
;	PIA LIB:
;		deglitch_param_init
;		lin_voltages
;		phtcap_init
;		pia_cal_darkcur
;		pia_gencho
;		pia_load_ierd
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pickmeas
;		process_erd
;		process_srd
;		read_erd
;		read_resp
;		set_orbpos
;		weight_mean
;       ASTRO LIB:
;		boost_array
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/ESTEC)                     	January 1995
;	Modified (last) by: CG					July 1995
;	Oct 95 Version 3.5
;	Ramp deglitching added (CG)				December 1995
;	NEP calculation modified (JACOSTA)			March 1996
;	The final value of dark current and min NEP is taken
;	as the median value, not as weighted mean (J.Acosta)	March 1996 
;	From_SRD option included (CG)				June 1996
;	Change way of computing orbital pos. (JAcosta)		October 1996
;	Bug for first SCP element + add Proc_date to 
;	cumulated structure    (JAcosta)			October 1997
;	V6.5
;
;-
; *********************************************************************
; *
; *********************************************************************
PRO PIA_CAL_DARKCUR_PROC, det, newdarkcurr, cal_darkc_str, newnep, $
		FROM_SRD=from_srd, FROM_SCP=from_scp 

@phtacc
@phtcs
@phterd
@phtsrd
@phtscp
@phtcap
@phtresdef
@phtw2jy
@deglitch_param
@gen_choices
COMMON widget_log_cmn, wd_text, wd_title

	wd_title = 'CFDP 52201/52511: Dark Current + NEP proc.'
        WIDGET_LOG, wd_base, ACTION='CREATE'

;
;  Get the necessary information to start
;
	
	IF N_ELEMENTS(c1cap) EQ 0 THEN phtcap_init
	IF N_ELEMENTS(c1resp) EQ 0 THEN read_resp
	IF N_ELEMENTS(v2jy_s) EQ 0 THEN phtw2jy_init

        pia_gencho, DET=det, group=wd_base

        IF gen_stop EQ 1 THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
		WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
        ENDIF


;
; ERD Selection criteria 
; 
	IF (gen_select.erdsel) THEN pia_rodisc, nndr = -1

;
; SRD Selection criteria 
; 
	IF (gen_select.srdsel) THEN pia_sigdisc, nsig = -1

;
; Select ERD Files or individual measurements using multiselectors
;


; FITS files
	IF gen_fits.fits EQ 0 THEN BEGIN

	  fillist = pia_pickfile(/multi, filter=gen_filt, path= !FITDIR,$
	  		group=wd_base)

          IF fillist(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		  ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
          ENDIF


;
; Compute first measurement to be used
; 
;          firstscp = N_ELEMENTS(phtscp)
;

; Read ERD files	
;	 					
	  FOR I=0,N_ELEMENTS(fillist)-1 DO BEGIN 

        	filename = STRUPCASE(STRTRIM(fillist(i),2))
        	filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)
		help, filename
         	WIDGET_LOG, wd_base, TEXT ='Reading ERD '+filename, $ 
         	            ACTION='APPEND'

		read_erd, filename,hdr,ilunr
          	
; Compute first measurement on the ERD level
		IF I EQ 0 THEN firsterd = ilunr(0)

; If first measurement of each file to be displayed		
;		pia_disperd, phterd(ilunr(0))

	  ENDFOR		

;
; Compute last measurement to be used
; 
          lasterd = max(ilunr)

;
; Compute all measurements to be processed to SRD 
;

;  --- > Compute which of those measurements belong to the desired detector 
; (P1, P2 and P3) can be present within the same file
;
	 IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phterd(firsterd:lasterd).admn,0,2) EQ det, n)
	  meas = firsterd + inddet
         ENDIF ELSE $
	  meas = indgen(lasterd-firsterd+1) + firsterd

; choose corresponding capacities and responses

	  CASE STRMID(phterd(firsterd).admn,0,2) OF
		'C1': BEGIN
			capac = c1cap
			resp = c1resp
		END
		'C2': BEGIN
			capac = c2cap
			resp = c2resp
	        END
		'SL': BEGIN
			capac = [sscap,slcap]
			resp = fltarr(128)
			ind1 = indgen(64)
			resp(ind1) = v2jy_s(ind1) / w2jy_s(ind1) * capac(0)
			resp(ind1+64) = v2jy_s(ind1+64) / w2jy_s(ind1+64) $
			                            * capac(1)  
			help,resp                            
		END
		'P1': BEGIN
			capac = p1cap 
			resp = p1resp
		END
		'P2': BEGIN
			capac = p2cap 
			resp = p2resp
		END
		'P3': BEGIN
			capac = p3cap 
			resp = p3resp
		END
	  ENDCASE

	ENDIF ELSE BEGIN

; individual measurements

	  CASE det OF
		'P1': BEGIN
			capac = p1cap
			resp = p1resp
		END
		'P2': BEGIN
			capac = p2cap
			resp = p2resp
		END
		'P3': BEGIN
			capac = p3cap
			resp = p3resp
		END
		'C1': BEGIN
			capac = c1cap
			resp = c1resp
		END
		'C2': BEGIN
			capac = c2cap
			resp = c2resp
	        END
		'SL': BEGIN
			capac = [sscap,slcap]
			resp = fltarr(128)
			ind1 = indgen(64)
			resp(ind1) = v2jy_s(ind1) / w2jy_s(ind1) * capac(0)
			resp(ind1+64) = v2jy_s(ind1+64) / w2jy_s(ind1+64) $
			                            * capac(1)  
		END
	  ENDCASE

          start_proc = 0   ; default value assume that processing will start from ERD
          IF KEYWORD_SET(from_scp) THEN start_proc = 2
          IF KEYWORD_SET(from_srd) THEN start_proc = 1
       

	  case start_proc of
	   
	  2: BEGIN
	  	measnames = pickmeas(FTYPE="SCP", DTYPE=det, /multi, POSI=posi)

	   	IF measnames(0) EQ '' THEN BEGIN
                  MESSAGE, /INFO, $
		   ' No Data selected, Calibration Procedure Sequence stopped'
		  WIDGET_LOG, wd_base, ACTION='DESTROY'
                  RETURN
          	ENDIF
 
                WIDGET_LOG, wd_base, TEXT ='Starting with measurements at SCP ',$ 
                   ACTION='APPEND'

		meas = 0l
           	  FOR I=0,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	  	measadd = WHERE(phtscp.admn EQ measnames(I),n)
			meas = [meas,measadd(posi(i))]
	    	  ENDFOR
	        scpps = meas(1:*)
	        goto, SCP_START
	      END

	  1: BEGIN
	  	measnames = pickmeas(FTYPE="SRD", DTYPE=det, /multi, POSI=posi)

	   	IF measnames(0) EQ '' THEN BEGIN
                  MESSAGE, /INFO, $
		   ' No Data selected, Calibration Procedure Sequence stopped'
		  WIDGET_LOG, wd_base, ACTION='DESTROY'
                  RETURN
          	ENDIF
 
                WIDGET_LOG, wd_base, TEXT ='Starting with measurements at SRD',$ 
                   ACTION='APPEND'


	  	meas = 0l
           	  FOR I=1,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	  	measadd = WHERE(phtsrd.admn EQ measnames(I),n)
			meas = [meas,measadd(posi(i))]
	    	  ENDFOR
	        srdps = meas(1:*)
	        goto, SRD_START
	      END
		

	  0: BEGIN
	  	measnames = pickmeas(FTYPE="ERD", DTYPE=det, /multi, POSI=posi)

	   	IF measnames(0) EQ '' THEN BEGIN
                  MESSAGE, /INFO, $
		   ' No Data selected, Calibration Procedure Sequence stopped'
		  WIDGET_LOG, wd_base, ACTION='DESTROY'
                  RETURN
          	ENDIF
 
                WIDGET_LOG, wd_base, TEXT ='Starting with measurements at ERD ',$ 
                   ACTION='APPEND'


	  	meas = 0l
           	  FOR I=1,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	  	measadd = WHERE(phterd.admn EQ measnames(I),n)
			meas = [meas,measadd(posi(i))]
	    	  ENDFOR
	  	meas = meas(1:*)
	      END
	ENDCASE	

	ENDELSE


;
; Linearization
;
	IF (gen_correct.linear) THEN BEGIN

          WIDGET_LOG, wd_base, TEXT ='Linearizing ramps ...  ', ACTION='APPEND'

	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		newhdr = tmpdata.hdr
		lin_voltages, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set,/no_copy
		handle_value, phterd(meas(i)).hdr, newhdr,/set,/no_copy
           ENDFOR

	ENDIF
;
; ERD-deglitching
;
	IF (gen_correct.deglit2) THEN BEGIN

          WIDGET_LOG, wd_base, TEXT ='Deglitching ramps ...  ', ACTION='APPEND'

	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		ramp_deglitch, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set
		handle_value, phterd(meas(i)).hdr, newhdr,/set
           ENDFOR

	ENDIF
;
; Process all data to srd 
;

        WIDGET_LOG, wd_base, TEXT =['Processing all measurements  ERD->SRD', $ 
		'(This will take some time, take it easy) '], $
                      ACTION='APPEND'
 
	process_erd, meas, 1, hdr, srdps


; 
; Compute effective integration time for minimum NEP calculation
;
	handle_value, phterd(meas(0)).time, time
	ind = where(time gt phterd(meas(0)).tint)
	reads_per_ramp = ind(0) 
	not_acc = N_ELEMENTS(ndrnacc)
	if not_acc EQ 1 and ndrnacc(0) EQ 0 THEN not_acc = 0
	not_acc = not_acc + ndr1nac + drnacc
	eff = (reads_per_ramp * 1.0 - not_acc) / reads_per_ramp   
	tint = phterd(meas).tint * eff

;
;
; Loop over SRD structures

SRD_START:
;
; Set a dummy value to index of SCP
        scpps = 0
; Loop over SRD structures

	FOR i=0,N_ELEMENTS(srdps)-1 DO BEGIN

	  pia_load_isrd, phtsrd(srdps(i)), tmpdata
	  
	  newheader = tmpdata.hdr
;
; Subtract dark current 
;
	  IF (gen_correct.darkc) THEN $ 
            WIDGET_LOG, wd_base, TEXT ='No Dark Current Subtraction possible',$
                 ACTION='APPEND'

;
;  SRD Deglitching
;

	  IF (gen_correct.deglit) THEN BEGIN

         	WIDGET_LOG, wd_base, TEXT ='Deglitching ...', ACTION='APPEND'

                degl_acc = tmpdata.slop * 0.
                deglitch_param_init
                sizdim1 = n_elements(degl_acc(*,0))
                FOR II = 0,sizdim1 - 1 DO BEGIN
                  deglitch_all, tmpdata.slop(II,*), $
                        tmpdata.step, tmpdata.rpid, tmpdata.slun(II,*), degacc
                  degl_acc(II,*) = degacc
                  indnacc = WHERE(degacc NE 0, count)
                  IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
                ENDFOR
	  ; include information in header
                hdrlen = N_ELEMENTS(newheader)
                hdrend = newheader(hdrlen-1)
                JUNK = 'PRS_DEGL=          '+$
                                 '          1 / Deglitching corr. applied'
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
	  	handle_value, phtsrd(srdps(i)).flag, tmpdata.flag, /set
	  ENDIF

          WIDGET_LOG, wd_base, TEXT =' Processing measurements  SRD->SCP '  $
           	+ tmpdata.admn, ACTION='APPEND'

	; Write SRD measurements into PIA internal files
 
        IF (gen_save.savsrd) THEN BEGIN
            filename = tmpdata.admn+'.ISRD'
            phtisrd = tmpdata
            save, phtisrd, FILE=!INTDIR+filename, /XDR
            WIDGET_LOG, wd_base,  $
               TEXT = 'Saving SRD measurement into '+!INTDIR+filename, $
                ACTION='APPEND'
                
            indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), count)
            IF count GT 0 THEN BEGIN
                      PIA_LOAD_ICS, phtcs(indcs(0)), phtics
                      FILE='CS'+phtics.UNIT+'.ICS'
                      junk = FINDFILE(!intdir+FILE, COUNT=count)
                      IF count GT 0 THEN $
                        MESSAGE, /INFO, $
                          'Compact Status file not written, already present' $
                      ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
            ENDIF ELSE MESSAGE, /INFO, $
                        'Compact Status not found in common, not saved' 
            
        ENDIF
 
	
	; processing to SCP level
	   
	if gen_process.drft eq 1 then   $
	WIDGET_LOG, wd_base,    $ 
	     TEXT = ' Applying Drift recognition ', ACTION='APPEND'
	
	WIDGET_LOG, wd_base, $
	     TEXT = ' Processing measurements  SRD->SCP', ACTION='APPEND'
	
	  process_tmpsrd, tmpdata, newheader, scppos, driftcorr=gen_process.drft
	  
	  scpps = [scpps, scppos(0)]		

   	ENDFOR

	scpps = scpps(1:*)


; Write SCP data into PIA internal files
 
          IF (gen_save.savscp) THEN BEGIN
	   FOR I=0, n_elements(scpps)-1 DO BEGIN
            pia_load_iscp, phtscp(scpps(i)), tmpscp
            filename = tmpscp.admn+'.ISCP'
            phtiscp = tmpscp
            save, phtiscp, FILE=!INTDIR+filename, /XDR
            WIDGET_LOG, wd_base, $
                TEXT ='Saving SCP measurement into '+!INTDIR+filename, $
                ACTION = 'APPEND'
            indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), count)
            IF count GT 0 THEN BEGIN
                      PIA_LOAD_ICS, phtcs(indcs(0)), phtics
                      FILE='CS'+phtics.UNIT+'.ICS'
                      junk = FINDFILE(!intdir+FILE, COUNT=count)
                      IF count GT 0 THEN $
                        MESSAGE, /INFO, $
                          'Compact Status file not written, already present' $
                      ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
            ENDIF ELSE MESSAGE, /INFO, $
                        'Compact Status not found in common, not saved' 
           ENDFOR             
          ENDIF


; Here start processing from SCP
SCP_START: 

; now compute the DARK CURRENTS

        WIDGET_LOG, wd_base, TEXT ='Now computing the Dark Currents', $
              ACTION='APPEND'

	
	detpix = pixel_det(det)

        darkcurav = fltarr(detpix)
        darkcurun = darkcurav
        noiseav = darkcurav
        noiseun = darkcurav

	alldarkc = FLTARR(detpix,n_elements(scpps)) 
        alldarku = alldarkc
        allnoise = alldarkc
        allflag = INTARR(detpix,n_elements(scpps)) 


	WIDGET_LOG, wd_base, TEXT = ['','Dark signals '], ACTION='APPEND'

 	FOR ipixl = 0, detpix-1 DO BEGIN

	  counter = 0

	  FOR imeas=0, n_elements(scpps)-1 DO BEGIN


	  handle_value, phtscp(scpps(imeas)).aver, scpave
	  handle_value, phtscp(scpps(imeas)).avun, scpavu
	  handle_value, phtscp(scpps(imeas)).avsd, scpavsd
	  handle_value, phtscp(scpps(imeas)).arsi, scparsi
	  handle_value, phtscp(scpps(imeas)).flag, scpflg

	        IF N_ELEMENTS(scpave(ipixl,*)) GT 1 THEN BEGIN
			MESSAGE, /INFO, $
			  'Computing dark currents with chopped measurements ??'

			ind = WHERE(scpflg(ipixl,*) EQ 0, n)  
			IF n GT 1 THEN BEGIN
                         weight_mean, $
			    scpave(ipixl,ind),scpavu(ipixl,ind),mean,std,sigma
			 weight_mean, $ 
			    scparsi(ipixl, ind), scpavu(ipixl,ind), tmean
			 scpave(ipixl,0)  = mean
			 scpavu(ipixl,0)  = std
			 scpavsd(ipixl,0) = sigma
			 scparsi(ipixl,0) = tmean   
			 scpflg(ipixl,0)  = 0
			ENDIF ELSE IF n EQ 1 THEN BEGIN
			 scpave(ipixl,0)  = scpave(ipixl,ind(0))
			 scpavu(ipixl,0)  = scpavu(ipixl,ind(0))
			 scpavsd(ipixl,0) = scpavsd(ipixl, ind(0))
			 scparsi(ipixl,0) = scparsi(ipixl, ind(0))
			 scpflg(ipixl,0)  = 1
			ENDIF ELSE BEGIN
			 ind = WHERE(scpflg(ipixl,*) EQ 1, n)
			 IF n GT 1 THEN BEGIN   
                          weight_mean, $
				scpave(ipixl,ind),scpavu(ipixl,ind),mean,std,sigma
			  weight_mean, $ 
			    scparsi(ipixl, ind), scpavu(ipixl,ind), tmean
			  scpave(ipixl,0)  = mean
			  scpavu(ipixl,0)  = std
			  scpavsd(ipixl,0) = sigma
			  scparsi(ipixl,0) = tmean   
			  scpflg(ipixl,0)  = 1
			 ENDIF ELSE IF n EQ 1 THEN BEGIN   
			  scpave(ipixl,0)  = scpave(ipixl,ind(0))
			  scpavu(ipixl,0)  = scpavu(ipixl,ind(0))
			  scpavsd(ipixl,0) = scpavsd(ipixl, ind(0))
			  scparsi(ipixl,0) = scparsi(ipixl, ind(0))
			  scpflg(ipixl,0)  = 1
			 ENDIF ELSE BEGIN
			  scpflg(ipixl,0) = 2
			 ENDELSE
			ENDELSE
		ENDIF	
		
		
		alldarkc(ipixl,imeas) = scpave(ipixl,0)
		alldarku(ipixl,imeas) = scpavu(ipixl,0)
		allnoise(ipixl,imeas) = scpavsd(ipixl,0) $ 
		                                  * sqrt(2.*scparsi(ipixl,0))
		allflag(ipixl,imeas) = scpflg(ipixl,0)

          	IF counter EQ 0 THEN BEGIN 
			allave = [scpave(ipixl,0)] 
			allavu = [scpavu(ipixl,0)]
			allnoi = [scpavsd(ipixl,0)*sqrt(2.*scparsi(ipixl,0))]
			allflg = [scpflg(ipixl,0)]
			counter = 1 
	  	ENDIF ELSE BEGIN
		 	allave = [allave,scpave(ipixl,0)] 	
			allavu = [allavu,scpavu(ipixl,0)]
			allnoi = [allnoi,scpavsd(ipixl,0)*sqrt(2.*scparsi(ipixl,0))]
			allflg = [allflg,scpflg(ipixl,0)]
	  	ENDELSE 	
          ENDFOR

	  indok = WHERE(allflg EQ 0, n) 
	  
	  CASE 1 of
	     n GT 1: BEGIN
		  med_dkc = median (allave(indok))
		  indmed = where(allave(indok) LT med_dkc)
		  q1me_dkc = median(allave(indok(indmed)))
	  	  weight_mean, allave(indok), allavu(indok), mean, std, noise 
	  	  weight_mean, allnoi(indok), allavu(indok), noi_mean, noi_std
	  	END
	     n eq 1: BEGIN
		  med_dkc = allave(indok(0))
		  q1me_dkc = 0.0 * med_dkc
		  mean = med_dkc
		  std = 0.0 * mean & noise = std
		  noi_mean = allnoi(indok(0)) & noi_std = noi_mean*0.0
	  	END
	     ELSE: BEGIN	       
	          indok1 = WHERE(allflg EQ 1, nn)
	          CASE 1 of
	              nn GT 1: BEGIN
		           med_dkc = median (allave(indok1))
		           indmed = where(allave(indok1) LT med_dkc)
		           q1me_dkc = median(allave(indok1(indmed)))
	  	           weight_mean, allave(indok1), allavu(indok1), mean, $
	  	              std, noise  
	  	           weight_mean, allnoi(indok1), allavu(indok1), $ 
	  	              noi_mean, noi_std 
	  	         END     
	              nn EQ 1: BEGIN
		           med_dkc = allave(indok1(0))
		           q1me_dkc = 0.0 * med_dkc
	  	           mean	= med_dkc
		           std = 0.0 * mean & noise = std
		           noi_mean = allnoi(indok1(0)) & noi_std = noi_mean*0.0
	  	         END     
	              ELSE: BEGIN
		           med_dkc = 0. 
		           q1me_dkc = 0.
		           noi_mean = 0.
		           noi_std  = 0.
	  	         END     
	          ENDCASE

	  	END
	  ENDCASE 
		

;	  darkcurav(ipixl) = med_dkc
;	  darkcurun(ipixl) = q1me_dkc
	  darkcurav(ipixl) = mean
	  darkcurun(ipixl) = noise
	  noiseav(ipixl) = noi_mean
	  noiseun(ipixl) = noi_std
	  counter = 0
	  WIDGET_LOG, wd_base, TEXT = 'Pixel('+strtrim(ipixl+1,2) + ')= '+ $
		strtrim(mean,2), ACTION='APPEND'

        ENDFOR

; Fill the structures

	nele_str = n_elements(scpps)

	CASE det of 
 	 'SL': cal_darkc_str = {SLDARKSTR_typ}
 	 'C2': cal_darkc_str = {C2DARKSTR_typ}
 	 'C1': cal_darkc_str = {C1DARKSTR_typ}
 	 ELSE: cal_darkc_str = {PDARKSTR_typ}
        ENDCASE

	cal_darkc_str = REPLICATE(cal_darkc_str,nele_str)

	cal_darkc_str.proc_date = !STIME
	
	FOR I = 0, nele_str-1 DO BEGIN

	;write the reset interval
	 give_cs, phtscp(scpps(i)), cs, record, conv_cst 
	 inttmp = conv_cst.intt(record)
	 cal_darkc_str(I).treset = STRMID(inttmp,0,STRLEN(inttmp)-2)
	
	;check if reset interval correction has been applied
	 handle_value,phtscp(scpps(i)).hdr,tmphdr
	 rin0 = FXPAR(tmphdr, 'PRS_RIN0',count=ipar)
	 IF ipar GT 0 THEN cal_darkc_str(I).rescor = 1
	 
        ; Calculate orbit position just as time difference to perigee pass
         norb = strmid(phtscp(scpps(i)).admn,4,3)
         orbit_pos = set_orbpos (norb, phtscp(scpps(i)).unit)
         
        ; compute julian date
            unittmp = phtscp(scpps(i)).unit
            jdate = unit2juldate(unittmp)

	  cal_darkc_str(I).FLAG = FIX(MEDIAN(allflag(0:detpix-1,I)))
	  cal_darkc_str(I).ADMN = phtscp(scpps(I)).admn	
	  cal_darkc_str(I).DATE = phtscp(scpps(I)).unit
	  cal_darkc_str(I).ORBPOS = orbit_pos
	  handle_value, phtscp(scpps(I)).temp, temp
	  cal_darkc_str(I).TEMP = temp(0)
	  cal_darkc_str(I).DARKC = alldarkc(0:detpix-1,I)	
	  cal_darkc_str(I).DARKU = alldarku(0:detpix-1,I)
	  cal_darkc_str(I).RESP = resp
	  cal_darkc_str(I).JDATE = jdate
	  cal_darkc_str(I).REV = norb

	  IF STRMID(phtscp(scpps(I)).det,0,1) NE 'S' THEN  $
	   nep = allnoise(0:detpix-1,I) * capac / resp       $
	  ELSE BEGIN 
	   nep = allnoise(0:63,I) * capac(0) / resp(0:63)  
	   nepl= allnoise(64:127,I)*capac(1) / resp(64:127)  
	   nep = [nep,nepl]
	  ENDELSE

	  cal_darkc_str(I).NEP = nep       ; nep is an array of 1-D of dim = detpix
	  cal_darkc_str(I).SFLG = allflag(0:detpix-1,I)	

	ENDFOR	
	

	x = indgen(detpix) + 1 

		
	newdarkcurr = darkcurav
	boost_array, newdarkcurr, darkcurun 
	
	WIDGET_LOG, wd_base, TEXT = ['','Min NEP '], ACTION='APPEND'

	newnep = fltarr(detpix)
	weight = intarr(nele_str) + 1
 	FOR ipixl = 0, detpix-1 DO BEGIN
		;weight_mean, cal_darkc_str.nep(ipixl), weight, mean, std
		;newnep(ipixl) = mean 
		if nele_str gt 1 then $ 
		    newnep(ipixl) = median(cal_darkc_str.nep(ipixl))  $
		else newnep(ipixl) = cal_darkc_str.nep(ipixl)
 	        WIDGET_LOG, wd_base, TEXT = 'Pixel('+strtrim(ipixl+1,2) + ')= '+$
		   strtrim(newnep(ipixl),2), ACTION='APPEND'


	ENDFOR


        WIDGET_LOG, wd_base, TEXT =['','Ready READY Ready READY Ready'], $
		 ACTION='APPEND'

        XMANAGER, No_BLOCK=1, "WIDGET_LOG", wd_base, GROUP= group
	
	
	
	RETURN
	END
