;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_FCSPOW_CROS
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to the FCS power tables File production -
;	FCS1-FCS2 cross calibration part
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_FCSPOW_CROS, det, fcstab
;
; INPUT:
;       det: String indicating the detector
;	fcstab: FCS Power table to be used
;
; KEYWORD PARAMETERS:
;	fcs_nr:	Number of FCS measurements performed in the cob - Def=4
;
; OUTPUT:
;	none
;
; COMMON BLOCKS:
;	deglitch_param
;	phtapertures
;	phtcs
;	phterd
;	phtscp
;	phtsrd
;	phtfilter
;	gencho 
;	pia_cal_fcspow_proc_cmn (internal)
;	pia_cal_fcspow_proc_res (containing the results):
;		heatpow: Flt(fcs_nr) Array:  FCS heating powers used in the  
;				calibration sequence
;	 	fcspow: Flt(pixels,fcs_nr) Array: computed optical in-band powers 
;	 	fcspun: uncertainties of fcspow
;	 	cal_fcspow_str: computed fcs-in-band-powers structure
;	 	cal_fcspre_str: computed pre_reduced signal table
;
; PROCEDURES USED:
;	PIA LIB:
;		aperture_size_init
;		deglitch_param_init
;		get_wheels
;		lin_voltages
;		phtfilter_init
;		pia_cal_fcspow
;		pia_gencho
;		pia_load_ierd
;		pia_load_iscp
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pickmeas
;		process_erd
;		process_tmpsrd
;		read_cs
;		read_erd
;		weight_mean
;       ASTRO LIB:
;		date_conv
;	INTERNAL:
;		pia_cal_fcspow_proc_event
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel  (ESA/ESTEC-SAI)             November 1995
;	Ramp deglitching + stability analysis added (CG)    February 1996
;	Bug in structure & overplot fixed (CG)		    February 1996
;	Bug with identification of C100 filter fixed (CG)   April 1996
;	Change calculation of orbital position	(JAcosta)   October 1996
;			
;-

;
PRO PIA_CAL_FCSPOW_CROS_EVENT, event

	COMMON pia_cal_fcspow_cros_cmn, text1
 	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue

        CASE UVALUE OF
        
		'PRINT': BEGIN
				WIDGET_CONTROL, text1, GET_VALUE=text
				pia_disp_print_file, "FCS_Powtab_crosscal", $
			 		GROUP = event.top, TEXT=text, $
			 	  TITLE = "PIA FCS1-FCS2 cross calibration"
			 END
			 	
        	'QUIT': WIDGET_CONTROL, event.top, /destroy
		ELSE:

	END
	
RETURN
END

; *********************************************************************
; *
; *********************************************************************
PRO PIA_CAL_FCSPOW_CROS, det, fcstab, fcs_nr=fcs_nr

@phtcs
@phterd
@phtsrd
@phtscp
@deglitch_param
@phtfilter
@phtapertures
@phtoptpow
@gencho

	COMMON pia_cal_fcspow_cros_cmn, text1
	COMMON pia_cal_fcspow_cros_res, heatpow, fcspow, fcspun, $
		cal_fcspow_str, proc_status
; at start
	proc_status = 2


	IF KEYWORD_SET(fcs_nr) THEN fcs_nr=fcs_nr(0) ELSE fcs_nr = 13

	source='CROSS_CAL'

	
        elpow = fltarr(fcs_nr)

	IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init
	allcfilters = [c1filters, c2filters]
	allpfilters = [p1filters, p2filters, p3filters]
	  
	IF N_ELEMENTS(p_as) EQ 0 THEN aperture_size_init
	
        base = WIDGET_BASE(TITLE='CFDP-52401b: FCS1-FCS2 Cross calibration ', $
			/COLUMN)

	control = WIDGET_BASE(base, /ROW)
	quit = WIDGET_BUTTON(control, VALUE=" QUIT ", UVALUE= "QUIT")
	prtsave = WIDGET_BUTTON(control, VALUE=" PRINT & SAVE ", UVALUE= "PRINT")
	
        text1 = WIDGET_TEXT(base, XSIZE=50, YSIZE=30, /SCROLL)

        WIDGET_CONTROL, base, /REALIZE

        WIDGET_CONTROL, base, /hourglass

;
;  Get the necessary information to start
;
	
        pia_gencho, DET=det

        IF gen_stop EQ 1 THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
                RETURN
        ENDIF

        IF N_ELEMENTS(gen_fits) EQ 0 THEN gen_fits = 0
        IF N_ELEMENTS(gen_erd) EQ 0 THEN gen_erd = 0
        IF N_ELEMENTS(gen_srd) EQ 0 THEN gen_srd = 0
        IF N_ELEMENTS(gen_scp) EQ 0 THEN gen_scp = 0
        IF N_ELEMENTS(gen_lin) EQ 0 THEN gen_lin = 0
        IF N_ELEMENTS(gen_dar) EQ 0 THEN gen_dar = 0
        IF N_ELEMENTS(gen_deg) EQ 0 THEN gen_deg = 0
        IF N_ELEMENTS(gen_deg2) EQ 0 THEN gen_deg2 = 0
        IF N_ELEMENTS(gen_drft) EQ 0 THEN gen_drft = 0


;
; ERD Selection criteria 
; 
	IF (gen_erd) THEN pia_rodisc, nndr = -1

;
; SRD Selection criteria 
; 
	IF (gen_srd) THEN pia_sigdisc, nsig = -1

;
; Select ERD Files or individual measurements using multiselectors
;


; FITS files
	IF gen_fits EQ 0 THEN BEGIN

	  fillist = pia_pickfile(/multi, filter=gen_filt, path=!FITDIR)

          IF fillist(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		  ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
          ENDIF


;
; Compute first measurement to be used
; 
          firstscp = N_ELEMENTS(phtscp)
;

; Read ERD files	
;	 					
	  FOR I=0,N_ELEMENTS(fillist)-1 DO BEGIN 

        	filename = STRUPCASE(STRTRIM(fillist(i),2))
        	filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)

        	WIDGET_CONTROL, text1, SET_VALUE='Reading ERD '+filename, $
			/append
		WIDGET_CONTROL, base, /hourglass

		read_erd, filename,hdr,ilunr
          	
; Compute first measurement on the ERD level
		IF I EQ 0 THEN firsterd = ilunr(0)

; If first measurement of each file to be displayed		
;		pia_disperd, phterd(ilunr(0))

	  ENDFOR		

;
; Compute last measurement to be used
; 
          lasterd = max(ilunr)

;
; Compute all measurements to be processed to SRD 
;

;  --- > Compute which of those measurements belong to the desired detector 
; (P1, P2 and P3) can be present within the same file
;
	 IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phterd(firsterd:lasterd).admn,0,2) EQ det, n)
	  meas = firsterd + inddet
         ENDIF ELSE $
	  meas = indgen(lasterd-firsterd+1) + firsterd

	ENDIF ELSE BEGIN

; individual measurements

	  CASE STRMID(gen_filt,1,1) OF
		'1': dtyp = 'C1'
		'2': dtyp = 'C2'
		'S': dtyp = 'SL'
		'P': dtyp = 'P#'
	  ENDCASE

	  measnames = pickmeas(FTYPE="ERD", DTYPE=dtyp, /multi)

          IF measnames(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, ' No Data selected, CAL procedure stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
          ENDIF

          WIDGET_CONTROL, text1, SET_VALUE='Starting with measurements  ', $
			/append
	  WIDGET_CONTROL, base, /hourglass

	  meas = WHERE(phterd.admn EQ measnames(0),n)
	  IF N_ELEMENTS(measnames) GT 1 THEN BEGIN
            FOR I=1,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	measadd = WHERE(phterd.admn EQ measnames(I),n)
		IF n GT 1 THEN BEGIN
		   measlist = phterd(measadd).admn
		   pia_multmeas, measlist
		   meas = [meas,measadd(measlist)]
		ENDIF ELSE $
		   meas = [meas,measadd(0)]
	    ENDFOR
	  ENDIF
		
; 
          firstscp = N_ELEMENTS(phtscp)

	ENDELSE


;
; Linearization
;
	IF (gen_lin) THEN BEGIN

          WIDGET_CONTROL, text1, SET_VALUE='Linearizing ramps ...  ', $
			/append
          WIDGET_CONTROL, base, /hourglass
	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		newhdr = tmpdata.hdr
		lin_voltages, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set,/no_copy
		handle_value, phterd(meas(i)).hdr, newhdr,/set,/no_copy
           ENDFOR

	ENDIF
;
; ERD-deglitching
;
	IF (gen_deg2) THEN BEGIN

          WIDGET_CONTROL, text1, SET_VALUE='Deglitching ramps ...  ', $
			/append
          WIDGET_CONTROL, base, /hourglass
	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		newhdr = tmpdata.hdr
		ramp_deglitch, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set,/no_copy
		handle_value, phterd(meas(i)).hdr, newhdr,/set,/no_copy
           ENDFOR

	ENDIF
;
; Process all data to srd 
;

        WIDGET_CONTROL, text1, $
                SET_VALUE='Processing all measurements  ERD->SRD', $
                /append
        WIDGET_CONTROL, text1, $
                SET_VALUE='(This will take some time, take it easy) ', $
                /append
        WIDGET_CONTROL, base, /hourglass
 

	process_erd, meas, 1, hdr, srdps

;
;
; Loop over SRD structures

	FOR i=srdps(0),srdps(N_ELEMENTS(srdps)-1) DO BEGIN

	  pia_load_isrd, phtsrd(i), tmpdata
	  
	  newheader = tmpdata.hdr
;
; Subtract dark current 
;
          IF (gen_dar) THEN BEGIN

                CASE tmpdata.det of
                        'C1': detpix = 9
                        'C2': detpix = 4
                        'SL': detpix = 128
                        ELSE: detpix = 1
                ENDCASE

        WIDGET_CONTROL, text1, SET_VALUE='Subtracting Dark Current', $
                /append
        WIDGET_CONTROL, base, /hourglass

                darkcur, tmpdata.slop(0:detpix-1,*), tmpdata.det, outslop


; Replace slop contents, log it into the newheader

                IF (N_ELEMENTS(tmpdata.slop(0:detpix-1,*)) EQ    $
                        N_ELEMENTS(outslop)) THEN BEGIN
                          tmpdata.slop(0:detpix-1,*) = outslop
                          hdrlen = N_ELEMENTS(newheader)
                          hdrend = newheader(hdrlen-1)
                          JUNK = 'PRS_DARK=          '+$
                                 '          1 / Dark Current subtracted  '
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
                ENDIF
          ENDIF

;
;  Deglitching
;

	  IF (gen_deg) THEN BEGIN

        	WIDGET_CONTROL, text1, SET_VALUE='Deglitching ...', $
                /append

        	WIDGET_CONTROL, base, /hourglass

                degl_acc = tmpdata.slop * 0.
                deglitch_param_init
                sizdim1 = n_elements(degl_acc(*,0))
                FOR II = 0,sizdim1 - 1 DO BEGIN
                  deglitch_all, tmpdata.slop(II,*), $
                        tmpdata.step, tmpdata.slun(II,*), degacc
                  degl_acc(II,*) = degacc
                  indnacc = WHERE(degacc NE 0, count)
                  IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
                ENDFOR
; include information in header
                hdrlen = N_ELEMENTS(newheader)
                hdrend = newheader(hdrlen-1)
                JUNK = 'PRS_DEGL=          '+$
                                 '          1 / Deglitching corr. applied'
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
	  ENDIF

          WIDGET_CONTROL, text1, $
		SET_VALUE=' Processing measurements  SRD->SCP', $
                /append
          WIDGET_CONTROL, base, /hourglass

	  process_tmpsrd, tmpdata, newheader, scppos, driftcorr=gen_drft		

   	ENDFOR

; last measurement in SCP
	lastscp = N_ELEMENTS(phtscp) -1

; Compute which of those measurements belong to the desired detector 
; (P1, P2 and P3) can be present within the same file
;
	IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phtscp(firstscp:lastscp).admn,0,2) EQ det, n)
	  firstscp = firstscp + min(inddet)
	  lastscp = firstscp + max(inddet)
	ENDIF

; now compute the FCS power tables

        WIDGET_CONTROL, text1, $
		SET_VALUE='Now computing the FCS power tables', $
                /append
        WIDGET_CONTROL, base, /hourglass

	CASE phtscp(firstscp).det OF
                        'C1': detpix = 9
                        'C2': detpix = 4
                        'SL': detpix = 128
                        ELSE: detpix = 1
	ENDCASE

; 
; get first general information
;
        get_wheels, phtscp(firstscp).admn, wheels
	begdate = phtscp(firstscp).unit

;
; Accomodate all values into the COB given scheme
;
	CASE STRMID(det,0,1) OF
                        'C': BEGIN
			   categ = 0
			   aperture = 15
			   filter = wheels(1)
			   wheels2id, 'C',wheels,filter_id,apert_id,lambda,area
			   IF STRMID(det,1,1) EQ '2' THEN $
			   	filtpos = filter - 9 ELSE $
			   	filtpos = filter - 2
                        END
						  
                        'P': BEGIN

			 aperture = wheels(1)
			 filter = wheels(2)
			 wheels2id, 'P', wheels, filter_id,apert_id,lambda,area
			   CASE STRMID(det,1,1) OF
			   	'1': filtpos = filter - 1
			   	'2': filtpos = filter - 11
			   	'3': filtpos = filter - 13
			   ENDCASE
			END
			
                        ELSE: BEGIN
				MESSAGE, /INFO, 'wrong input, returning ...'
				RETURN
			END
	ENDCASE

	fcs1_sig = fltarr(detpix,fcs_nr)
	fcs1_unc = fcs1_sig
	fcs1_heatpow = fltarr(fcs_nr)
	fcs1_flag = intarr(detpix,fcs_nr)
	fcs2_sig = fcs1_sig
	fcs2_unc = fcs1_sig
	fcs2_heatpow = fltarr(fcs_nr)
	fcs2_flag = fcs1_flag
	
	tmptxt = ['FCS values: ', '===========']
	
	FOR i=0,fcs_nr-1 DO BEGIN
		handle_value, phtscp(firstscp+i).aver, aver
		handle_value, phtscp(firstscp+i).avun, avun
		handle_value, phtscp(firstscp+i).fcsn, fcsn
		handle_value, phtscp(firstscp+i).flag, flag
		indf1 = WHERE(fcsn EQ 1, c1)
		indf2 = WHERE(fcsn EQ 2, c2)

		IF c1 GT 0 AND c2 GT 0 THEN BEGIN
		   FOR ipix = 0, detpix - 1 DO BEGIN 
		     indok = WHERE(flag(ipix,indf1) EQ 0, count)
		     IF count GT 0 THEN BEGIN
		      WEIGHT_MEAN, $
		   	aver(ipix,indf1(indok)),avun(ipix,indf1(indok)),av,sg 
		      fcs1_sig(ipix,i) = av
		      fcs1_unc(ipix,i) = sg
		     ENDIF ELSE BEGIN
		      MESSAGE, /info, $
	'Warning: meas '+STRTRIM(STRING(i+1),2)+ ' is not OK - Pixel '+ $
			           STRTRIM(STRING(ipix+1),2) 
		      fcs1_flag(ipix,i)=2
		     ENDELSE
		     
		     indok = WHERE(flag(ipix,indf2) EQ 0, count)
		     IF count GT 0 THEN BEGIN
		      WEIGHT_MEAN, $
		   	aver(ipix,indf2(indok)),avun(ipix,indf2(indok)),av,sg 
			fcs2_sig(ipix,i) = av
			fcs2_unc(ipix,i) = sg
		     ENDIF ELSE BEGIN
		      MESSAGE, /info, $
	'Warning: meas '+STRTRIM(STRING(i+1),2)+ ' is not OK - Pixel '+ $
			           STRTRIM(STRING(ipix+1),2) 
		      fcs2_flag(ipix,i)=2
		     ENDELSE 
		   ENDFOR

		ENDIF ELSE BEGIN
			MESSAGE, /info, $
		  'Warning: measurement '+STRTRIM(STRING(i+1),2)+ ' is not OK'  		 	 
			fcs1_flag(*,i) = 2
			fcs2_flag(*,i) = 2
		ENDELSE
		handle_value, phtscp(firstscp+i).fcs1, fcs1
		handle_value, phtscp(firstscp+i).fcs2, fcs2
		fcs1_heatpow(i) = fcs1(0)
		fcs2_heatpow(i) = fcs2(0)
		
		tmptxt = [tmptxt, $
		  'FCS1 Heatpower = '+ STRTRIM(STRING(fcs1_heatpow(i)),2), $
		  'Pixel signal: '+ STRTRIM(STRING(fcs1_sig(*,i)),2) + $
		  ' +/- ' + STRTRIM(STRING(fcs1_unc(*,i)),2), '  ', $  
		  'FCS2 Heatpower = '+ STRTRIM(STRING(fcs2_heatpow(i)),2), $
		  'Pixel signal: '+ STRTRIM(STRING(fcs2_sig(*,i)),2) + $
		  ' +/- ' + STRTRIM(STRING(fcs2_unc(*,i)),2), '  ', ' ----- ']  
	ENDFOR
		
	WIDGET_CONTROL, text1, $
			SET_VALUE=tmptxt, /append

; Read CS records to obtain TRS used 

	psta_file='PSTA'+strmid(phtscp(firstscp).admn,4,8)+'.FITS'
	read_cs, psta_file, cshdr, csun
	handle_value, phtcs(csun).handle, cst
	trsa = cst.f1ts(0) - 1
	fcsa = 0
	
; Interpolate in-band powers 

	theor_flux = interpol(longpow,fcstab(*,filtpos,trsa,fcsa),fcs1_heatpow)
	
; Derive now the FCS2 fluxes 
	fcspow = fcs2_sig / fcs1_sig 	
	fcspun = fcspow * SQRT( (fcs2_unc/fcs2_sig)^2 + (fcs1_unc/fcs1_sig)^2 )
	FOR i=0,detpix-1 DO BEGIN
		fcspow(i,*) = theor_flux * fcspow(i,*)
		fcspun(i,*) = theor_flux * fcspun(i,*)

	ENDFOR
	
	tmptxt = ['  ', '**********************************************', $
		'FCS powers calculations: ', '====================== ']

	FOR I = 0, fcs_nr-1 DO BEGIN

	 tmptxt = [tmptxt, 'FCS2 position '+STRTRIM(STRING(I+1),2)+$
	 	  ', Heating power: '+STRTRIM(STRING(fcs2_heatpow(I)),2), $
	 		' FCS power: '+ STRTRIM(STRING(fcspow(*,I)),2) + $
			  ' +/- ' + STRTRIM(STRING(fcspun(*,I)),2), ' ']

	ENDFOR
	
	WIDGET_CONTROL, text1, $
			SET_VALUE=tmptxt, /append

	

;  Set provisorily all flags to 0	
	fcsflg = FIX(fcspow*0)


; Calculate orbit position just as time difference to perigee pass
        norb = strmid(phtscp(firstscp).admn,4,3)
        orbit_pos = set_orbpos (norb, date_conv(phtscp(firstscp).unit))
	 

; Fill the structures

	nele_str = lastscp - firstscp + 1

	CASE det of 
 	 'SL': 
 	 'C2': BEGIN  

 	        cal_fcspre_str = {C2fcspre_typ, Type_ID: '', $
		  Filt_id:'', Aper_ID:'', Orb_pos: 0., Date: '',$
		  TDTnr:0L, $
		  Source:'', FCS:0, TRS:0, Heatpow:0., t_int:0., $
		  Signal:fltarr(detpix), Sigunc:fltarr(detpix), $
		  Red_sig:fltarr(detpix), Red_sun:fltarr(detpix), $
		  Sig_flg: intarr(detpix), Flag:0, Proc_time:''}

 	        cal_fcspow_str = {C2_fcspow_typ}
	       END
 	 'C1': BEGIN  
 	        cal_fcspre_str = {C1fcspre_typ, Type_ID: '', $
		  Filt_id:'', Aper_ID:'', Orb_pos: 0., Date: '',$
		  TDTnr:0L, $
		  Source:'', FCS:0, TRS:0, Heatpow:0., t_int:0., $
		  Signal:fltarr(detpix), Sigunc:fltarr(detpix), $
		  Red_sig:fltarr(detpix), Red_sun:fltarr(detpix), $
		  Sig_flg: intarr(detpix), Flag:0, Proc_time:''}

 	        cal_fcspow_str = {C1_fcspow_typ}

;	  	cal_fcspow_str = {C1_fcspow_typ, FLAG:0, DATE:'', SOURCE:'', $
;old one	  FILTER:0, IBFLUX: 0., HEATPOW: 0., FCS:0, TRS:0,$
;	    	  fcspow:fltarr(detpix), fcspun:fltarr(detpix), $
;		  SFLG: intarr(detpix)}
	       END
 	 ELSE: BEGIN
 	        cal_fcspre_str = {Pfcspre_typ, Type_ID: '', $
		  Filt_id:'', Aper_ID:'', Orb_pos: 0., Date: '', TDTnr: 0L, $
		  Source:'', FCS:0, TRS:0, Heatpow:0., t_int:0., $
		  Signal:fltarr(detpix), Sigunc:fltarr(detpix), $
		  Red_sig:fltarr(detpix), Red_sun:fltarr(detpix), $
		  Sig_flg: intarr(detpix), Flag:0, Proc_time:''}

 	        cal_fcspow_str = {P_fcspow_typ}

;	  	cal_fcspow_str = {P_fcspow_typ, FLAG:0, DATE:'', SOURCE:'', $
;old one	  FILTER:0, IBFLUX: 0., HEATPOW: 0., FCS:0, TRS:0,$
;	    	  fcspow:fltarr(detpix), fcspun:fltarr(detpix), $
;		  SFLG: intarr(detpix)}

	       END
        ENDCASE

	newcal_fcspow_str = cal_fcspow_str
	
; Fill in the accumulated pre-reduced signal table - bit by bit (late requs)
; all into CROSS_CORR category
	cal_fcspre_str = REPLICATE(cal_fcspre_str, fcs_nr *2)

	cal_fcspre_str(*).filt_id = filter_id
	cal_fcspre_str(*).aper_id = apert_id
	cal_fcspre_str(*).orb_pos = orbit_pos
	cal_fcspre_str(*).tdtnr = LONG(STRMID(phtscp(firstscp).admn, 4,6))
	cal_fcspre_str(*).source = 'CROSS_CORR'		
	cal_fcspre_str(*).proc_time = systime()		

	cal_fcspre_str(0:fcs_nr-1).fcs = 1
	cal_fcspre_str(0:fcs_nr-1).trs = trsa+1
	cal_fcspre_str(0:fcs_nr-1).heatpow = fcs1_heatpow
	cal_fcspre_str(fcs_nr:2*fcs_nr-1).fcs = 2
	cal_fcspre_str(fcs_nr:2*fcs_nr-1).trs = trsa+1
	cal_fcspre_str(fcs_nr:2*fcs_nr-1).heatpow = fcs2_heatpow

	cal_fcspre_str(*).type_id = 'FCS'

	cal_fcspre_str(0:fcs_nr-1).date = $
		phtscp(firstscp:firstscp+fcs_nr-1).unit
	cal_fcspre_str(fcs_nr:2*fcs_nr-1).date = $
		phtscp(firstscp:firstscp+fcs_nr-1).unit

	FOR I=0,fcs_nr-1 DO BEGIN
	 ind = WHERE(phterd.admn EQ phtscp(firstscp+i).admn) 
	 cal_fcspre_str(i).t_int = phterd(ind(0)).tint
	 cal_fcspre_str(i+fcs_nr).t_int = phterd(ind(0)).tint
	 cal_fcspre_str(i).signal = fcs1_sig(*,i)
	 cal_fcspre_str(i).sigunc = fcs1_sig(*,i)
	 cal_fcspre_str(i+fcs_nr).signal = fcs2_sig(*,i)
	 cal_fcspre_str(i+fcs_nr).sigunc = fcs2_sig(*,i)
	 cal_fcspre_str(i).sig_flg = fcs1_flag(*,i)
	 cal_fcspre_str(i+fcs_nr).sig_flg = fcs2_flag(*,i)
	ENDFOR
	
	cal_fcspre_str(*).red_sig = 0.
	cal_fcspre_str(*).red_sun = 0.


; Fill in the accumulated FCS Data Table 

	FOR I=0,fcs_nr-1 DO BEGIN

	 newcal_fcspow_str.FLAG = 0	
	 newcal_fcspow_str.DATE = phtscp(firstscp).unit
	 newcal_fcspow_str.Filt_id = filter_id
	 newcal_fcspow_str.Aper_id = apert_id
	 newcal_fcspow_str.Source = 'CROSS_CORR'
	 newcal_fcspow_str.F_star = 0.
	 newcal_fcspow_str.Qb_star = 0.
	 newcal_fcspow_str.Heatpow = fcs2_heatpow(I)
	 newcal_fcspow_str.FCS = 2
	 newcal_fcspow_str.TRS = trsa + 1
	 newcal_fcspow_str.FCSPOW = fcspow(*,I)
	 newcal_fcspow_str.FCSPUN = fcspun(*,I)
	 newcal_fcspow_str.SFLG = fcs2_flag(*,i)	
	 newcal_fcspow_str.proc_time = systime()	
	 newcal_fcspow_str.SED_file = ['','']		

	 IF I EQ 0 THEN cal_fcspow_str = newcal_fcspow_str ELSE $
		cal_fcspow_str = [cal_fcspow_str,newcal_fcspow_str]
        ENDFOR

	heatpow = fcs2_heatpow
	proc_status = 0

	
	pia_xplot, heatpow, fcspow(0,*), fcspun(0,*), $
		psym=1, title='FCS2 optical powers',$
		xtitle='Electrical Power  [mW]', ytitle='Optical power  [W]',$
		annotation='Cross Calibration '+phtscp(firstscp).unit,$
		legend='Pixel 1'

	IF detpix GT 1 THEN BEGIN 
	 FOR i=1,detpix-1 DO BEGIN
	  psym = i+1
	  IF psym GE 8 THEN psym=psym-6
	  pia_xplot, heatpow, fcspow(i,*), fcspun(i,*), psym=psym, color=i, $
		annotation='Cross Calibration '+phtscp(firstscp).unit,$
		legend='Pixel '+STRTRIM(STRING(i+1),2), overplot=99
	 ENDFOR
	ENDIF

        WIDGET_CONTROL, text1, SET_VALUE='Ready READY Ready READY Ready', $
                /append

	xMANAGER,No_BLOCK=1, "PIA_CAL_FCSPOW_CROS", base, GROUP=group 

	RETURN
	END
