;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_FCSPOW_PROC
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to the FCS power tables File production
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_FCSPOW_PROC, det, heatpow, fcspow, fcspun, cal_fcspow_str
;
; INPUT:
;       det: String indicating the detector
;
; KEYWORD PARAMETERS:
;	source: Name of Calibration Source used
;	inbflux: In-band flux of Calibration Source used [W/pixel] 
;	qb_star: Number of photons / second on detector for the filter used
;	fcs_nr:	Number of FCS measurements performed in the cob - Def=4
;	orbit_pos: Orbital position at time of cob begin
;
; OUTPUT:
;	none
;
; COMMON BLOCKS:
;	deglitch_param
;	phtapertures
;	phtcs
;	phterd
;	phtppsf
;	phtscp
;	phtsrd
;	phtfilter
;	gencho 
;	pia_get_inbp_res
;	pia_cal_fcspow_proc_cmn (internal)
;	pia_cal_fcspow_proc_res (containing the results):
;		heatpow: Flt(fcs_nr) Array:  FCS heating powers used in the  
;				calibration sequence
;	 	fcspow: Flt(pixels,fcs_nr) Array: computed optical in-band powers 
;	 	fcspun: uncertainties of fcspow
;	 	cal_fcspow_str: computed fcs-in-band-powers structure
;	 	cal_fcspre_str: computed pre_reduced signal table
;
; PROCEDURES USED:
;	PIA LIB:
;		aperture_size_init
;		deglitch_param_init
;		get_wheels
;		lin_voltages
;		phtfilter_init
;		pia_cal_fcspow
;		pia_gencho
;		pia_get_inbpower
;		pia_load_ierd
;		pia_load_iscp
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pickmeas
;		process_erd
;		process_srd
;		read_cs
;		read_erd
;		read_ppsf
;		set_orbpos
;		subtr_bckg
;		weight_mean
;       ASTRO LIB:
;		date_conv
;	INTERNAL:
;		pia_cal_fcspow_proc_event
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel  (ESA/ESTEC)                    	May 1995
;	Modified (last) by: cg					July 1995
;	Oct 95 Version 3.5
;	Bug counting for P1 fixed (CG)				December 1995
;	Ramp deglitching added (CG)				December 1995
;	P1 and P2 are not anymore a special case (CG)		December 1995
;	Orbit position from PSF file, extension of FCSPOW
;		structure (CG)					December 1995
;	Processing from SRD level now possible (CG) + 
;		effective int times used 
;	IDL !stime instead of systime() for proc time (CG)	January 1996
;	/From_SRD works also with discontinuous set (CG)	February 1996
;	Drift stability analysis choice implemented (CG)	April 1996
;	Processing from SCP level implemented (JA)		May 1996
;	Index mistake fixed (CG)				June 1996
;	Change procedure for data input	(JAcosta)		June 1996
;	Change calculation of orbital position	(JAcosta)	October 1996
;	Using posi from pickmeas, and new gen_choices (JAcosta) June 1997
;	V6.3
;			
;-

;
PRO PIA_CAL_FCSPOW_PROC_EVENT, event

	COMMON pia_cal_fcspow_proc_cmn, text1
 	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue

        CASE UVALUE OF
        
		'PRINT': BEGIN
				WIDGET_CONTROL, text1, GET_VALUE=text
				pia_disp_print_file, "FCS_Powtab_proc", $
			 		GROUP = event.top, TEXT=text, $
			 		TITLE = "PIA FCS power tables processing"
			 END
			 	
        	'QUIT': WIDGET_CONTROL, event.top, /destroy
		ELSE:

	END
	
RETURN
END

; *********************************************************************
; *
; *********************************************************************
PRO PIA_CAL_FCSPOW_PROC, det, FROM_SRD=from_srd, FROM_SCP=from_scp

@phtcs
@phterd
@phtsrd
@phtscp
@deglitch_param
@phtfilter
@phtapertures
@phtppsf
@gen_choices
COMMON widget_log_cmn, wd_text, wd_title

COMMON pia_get_inbp_res, c_inbp
COMMON pia_cal_fcspow_proc_cmn, text1
COMMON pia_cal_fcspow_proc_res, heatpow, fcspow, fcspun, cal_fcspow_str,$
		 cal_fcspre_str, proc_status


ON_ERROR, 2
; at start
	proc_status = 2
	det = STRMID(det,0,2)

	
	; extract the necessary info relative to measurement sequence
	

	data_names = ['No. of FCS ','Pointing (P) or raster map (M) for bck', $
	   'No. of Background ', 'No. of SRC ', 'No. of Stray']
	vars = [strtrim(1,2), 'P', strtrim(1,2), strtrim(1,2), strtrim(1,2)] 
	data_exa   = vars
	vartype = [1,1,1,1,1]
	pia_select_var, data_exa, vars, VARNAMES=data_names, VARTYP= vartype, $
	    STOP = sel_stop 
	
	if sel_stop then return  
	
	fcs_nr = FIX(vars(0))

	fcs_nr_type = size(fcs_nr)
	IF fcs_nr_type(1) NE 2 AND fcs_nr_type(1) NE 4 THEN BEGIN
		MESSAGE, /INFO, 'Not valid number, returning'
		RETURN
	ENDIF

	if strmid(det,0,1) eq 'P' then bck_mapp='P' $ 
	     else bck_mapp = STRUPCASE(vars(0,1))

	if bck_mapp eq 'M' then bck_nr = 1 else bck_nr = FIX(vars(0,2))
	
	if bck_mapp eq 'M' then src_nr = 1 else src_nr = FIX(vars(0,3))
	
	stray_nr = FIX(vars(0,4))

	bck_nr_type = size(bck_nr)

	IF bck_nr_type(1) GT 2 OR bck_nr_type(1) LT 0 THEN BEGIN
		MESSAGE, /INFO, 'Not valid number, returning'
		RETURN
	ENDIF
 
        elpow = fltarr(fcs_nr)

	IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init
	allcfilters = [c1filters, c2filters]
	allpfilters = [p1filters, p2filters, p3filters]
	  
	IF N_ELEMENTS(p_as) EQ 0 THEN aperture_size_init
	IF N_ELEMENTS(phtppsf) EQ 0 THEN read_ppsf
	
        wd_title = 'CFDP-52401: FCS Power Tables Processing '
        WIDGET_LOG, wd_base, ACTION='CREATE'

;
;  Get the necessary information to start
;
	
        pia_gencho, DET=STRMID(det,0,2)

        IF gen_stop EQ 1 THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
                WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
        ENDIF


; write to the screen the sequence how measurements should be especified
	if bck_mapp eq 'P' then $
	info_text = ['The following sequence of measurements ',            $
		     'is expected in this procedure:',			   $
		     'all Straylight (maximum 2)',			   $
		     'all FCS (no limits)',			           $
		     'all Background (maximum 2)',			   $
		     'all source Measurements'	]    			   $
	else $
	info_text = ['The following sequence of measurements is expected', $
		     'is expected in this procedure:',			   $
		     'all Straylight (maximum 2)',			   $
		     'all FCS (no limits)',			           $
		     'Raster of Source + Background'  ]
		     
	widget_log, wd_base, TEXT= info_text, ACTION='APPEND'

	;xdisplayfile, '', text=info_text, $ 
	;   title='Input of FCS Sequence Measurements'	     		      


;
; ERD Selection criteria 
; 
	IF (gen_select.erdsel) THEN pia_rodisc, nndr = -1

;
; SRD Selection criteria 
; 
	IF (gen_select.srdsel) THEN pia_sigdisc, nsig = -1

;
; General Info	
;	 					
	        WIDGET_LOG, wd_base, $ 
	          TEXT = 'Starting '+det+' FCS Power Tables processing ', ACTION='APPEND'
	        WIDGET_LOG, wd_base, $ 
	          TEXT = '======================================= ', ACTION='APPEND'
	        WIDGET_LOG, wd_base, TEXT = ' ', ACTION='APPEND'

;
; Select ERD Files or individual measurements using multiselectors
;


; FITS files
	IF gen_fits.fits EQ 0 THEN BEGIN

	  fillist = pia_pickfile(/multi, filter=gen_filt, path=!FITDIR)

          IF fillist(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		  ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
          ENDIF


;
; Compute first measurement to be used
; 
          firstscp = N_ELEMENTS(phtscp)

;
; Read ERD files	
;	 					
	  FOR I=0,N_ELEMENTS(fillist)-1 DO BEGIN 

        	filename = STRUPCASE(STRTRIM(fillist(i),2))
        	filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)

	        WIDGET_LOG, wd_base, TEXT = 'Reading ERD '+filename, ACTION='APPEND'

		read_erd, filename,hdr,ilunr
          	
; Compute first measurement on the ERD level
		IF I EQ 0 THEN firsterd = ilunr(0)

; If first measurement of each file to be displayed		
;		pia_disperd, phterd(ilunr(0))

	  ENDFOR		

;
; Compute last measurement to be used
; 
          lasterd = max(ilunr)

;
; Compute all measurements to be processed to SRD 
;

;  --- > Compute which of those measurements belong to the desired detector 
; (P1, P2 and P3) can be present within the same file
;
	 IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phterd(firsterd:lasterd).admn,0,2) EQ det, n)
	  meas = firsterd + inddet
         ENDIF ELSE $
	  meas = indgen(lasterd-firsterd+1) + firsterd

	ENDIF ELSE BEGIN

; individual measurements

	  CASE STRMID(gen_filt,1,1) OF
		'1': dtyp = 'C1'
		'2': dtyp = 'C2'
		'S': dtyp = 'SL'
		'P': dtyp = 'P#'
	  ENDCASE

          start_proc = 0   ; default value assume that processing will start from ERD
          IF KEYWORD_SET (from_scp) THEN start_proc = 2
          IF KEYWORD_SET (from_srd) THEN start_proc = 1
       
	  case start_proc of
	   
	    2 : BEGIN
	    	 measnames = pickmeas(FTYPE="SCP", DTYPE=dtyp, /multi, POSI=posi)
	    	
                 IF measnames(0) EQ '' THEN BEGIN
                   MESSAGE, /INFO, ' No Data selected, AOT procedure stopped'
		   WIDGET_LOG, wd_base, ACTION='DESTROY'
                   RETURN
                 ENDIF

	         WIDGET_LOG, wd_base, TEXT = 'Starting from SCP level   ', $ 
	             ACTION='APPEND'

	         meas = 0l
                  FOR I=0,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	   measadd = WHERE(phtscp.admn EQ measnames(I),n)
		   meas = [meas,measadd(posi(i))]
	          ENDFOR
	         scpps = meas(1:*)
	         goto, SCP_START
	        END
	        
	     1: BEGIN     
	         measnames = pickmeas(FTYPE="SRD", DTYPE=dtyp, /multi, POSI=posi)

                 IF measnames(0) EQ '' THEN BEGIN
                  MESSAGE, /INFO, ' No Data selected, AOT procedure stopped'
		  WIDGET_LOG, wd_base, ACTION='DESTROY'
                  RETURN
                 ENDIF

	         WIDGET_LOG, wd_base, TEXT = 'Starting from SRD level   ', ACTION='APPEND'

	         meas = 0l
                   FOR I=0,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	     measadd = WHERE(phtsrd.admn EQ measnames(I),n)
		     meas = [meas,measadd(posi(i))]
	           ENDFOR
	         srdps = meas(1:*)
	         goto, SRD_START
	        END
	       
	      else: BEGIN    
	             measnames = pickmeas(FTYPE="ERD", DTYPE=dtyp, /multi, POSI=posi)

                     IF measnames(0) EQ '' THEN BEGIN
                       MESSAGE, /INFO, ' No Data selected, CAL procedure stopped'
		       WIDGET_LOG, wd_base, ACTION='DESTROY'
                       RETURN
                     ENDIF

	             WIDGET_LOG, wd_base, TEXT = 'Starting with measurements  ', $
	                 ACTION='APPEND'

	             meas = 0l
                       FOR I=0,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	         measadd = WHERE(phterd.admn EQ measnames(I),n)
		         meas = [meas,measadd(posi(i))]
	               ENDFOR
	             meas = meas(1:*)
	           END  

	ENDCASE

; 

	ENDELSE
        
;        firstscp = N_ELEMENTS(phtscp)

;
; Linearization
;
	IF (gen_correct.linear) THEN BEGIN

	   WIDGET_LOG, wd_base, TEXT = 'Linearizing ramps ...  ', ACTION='APPEND'

	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		newhdr = tmpdata.hdr
		lin_voltages, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set,/no_copy
		handle_value, phterd(meas(i)).hdr, newhdr,/set,/no_copy
           ENDFOR

	ENDIF
;
; ERD-deglitching
;
	IF (gen_correct.deglit2) THEN BEGIN

	WIDGET_LOG, wd_base, TEXT = 'Deglitching ramps ...  ', ACTION='APPEND'

	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		ramp_deglitch, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set
		handle_value, phterd(meas(i)).hdr, newhdr,/set
           ENDFOR

	ENDIF
;
; Process all data to srd 
;

	WIDGET_LOG, wd_base, TEXT = 'Processing all measurements  ERD->SRD', $
	 	ACTION='APPEND'
	
	WIDGET_LOG, wd_base, TEXT = '(This will take some time, take it easy) ', $
	 	ACTION='APPEND'

	process_erd, meas, 1, hdr, srdps

SRD_START:
;
;
; Set a dummy value to index of SCP
        scpps = 0
; Loop over SRD structures

	FOR i=0,N_ELEMENTS(srdps)-1 DO BEGIN

	  pia_load_isrd, phtsrd(srdps(i)), tmpdata
	  
	  newheader = tmpdata.hdr
;
; Subtract dark current 
;
          IF (gen_correct.darkc) THEN BEGIN

                CASE tmpdata.det of
                        'C1': detpix = 9
                        'C2': detpix = 4
                        'SL': detpix = 128
                        ELSE: detpix = 1
                ENDCASE

	WIDGET_LOG, wd_base, TEXT = 'Subtracting Dark Current',  ACTION='APPEND'

                darkcur, tmpdata.slop(0:detpix-1,*), tmpdata.det, outslop


; Replace slop contents, log it into the newheader

                IF (N_ELEMENTS(tmpdata.slop(0:detpix-1,*)) EQ    $
                        N_ELEMENTS(outslop)) THEN BEGIN
                          tmpdata.slop(0:detpix-1,*) = outslop
                          hdrlen = N_ELEMENTS(newheader)
                          hdrend = newheader(hdrlen-1)
                          JUNK = 'PRS_DARK=          '+$
                                 '          1 / Dark Current subtracted  '
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
                ENDIF
          ENDIF

;
;  Deglitching
;

	  IF (gen_correct.deglit) THEN BEGIN

		WIDGET_LOG, wd_base, TEXT= 'Deglitching ...', ACTION='APPEND'

                degl_acc = tmpdata.slop * 0.
                deglitch_param_init
                sizdim1 = n_elements(degl_acc(*,0))
                FOR II = 0,sizdim1 - 1 DO BEGIN
                  deglitch_all, tmpdata.slop(II,*), $
                        tmpdata.step, tmpdata.slun(II,*), degacc
                  degl_acc(II,*) = degacc
                  indnacc = WHERE(degacc NE 0, count)
                  IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
                ENDFOR
; include information in header
                hdrlen = N_ELEMENTS(newheader)
                hdrend = newheader(hdrlen-1)
                JUNK = 'PRS_DEGL=          '+$
                                 '          1 / Deglitching corr. applied'
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
	  ENDIF

 
        ; Write SRD measurements into PIA internal files
 
        IF (gen_save.savsrd) THEN BEGIN
            filename = tmpdata.admn+'.ISRD'
            phtisrd = tmpdata
            save, phtisrd, FILE=!INTDIR+filename, /XDR
            WIDGET_LOG, wd_base,  $
               TEXT = 'Saving SRD measurement into '+!INTDIR+filename, $
                ACTION='APPEND'
                
            indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), count)
            IF count GT 0 THEN BEGIN
                      PIA_LOAD_ICS, phtcs(indcs(0)), phtics
                      FILE='CS'+phtics.UNIT+'.ICS'
                      junk = FINDFILE(!intdir+FILE, COUNT=count)
                      IF count GT 0 THEN $
                        MESSAGE, /INFO, $
                          'Compact Status file not written, already present' $
                      ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
            ENDIF ELSE MESSAGE, /INFO, $
                        'Compact Status not found in common, not saved' 
            
        ENDIF
 

	; processing to SCP level
	   
	if gen_process.drft eq 1 then   $
	WIDGET_LOG, wd_base,    $ 
	     TEXT = ' Applying Drift recognition ', ACTION='APPEND'
	
	WIDGET_LOG, wd_base, $
	     TEXT = ' Processing measurements  SRD->SCP', ACTION='APPEND'

	  process_tmpsrd, tmpdata, newheader, scppos, driftcorr=gen_process.drft	

          scpps = [scpps,scppos(0)]
   	ENDFOR

	scpps = scpps(1:*)

SCP_START: 

	
; Write SCP data into PIA internal files
 
          IF (gen_save.savscp) THEN BEGIN
           FOR I=0, n_elements(scpps)-1 DO BEGIN
            pia_load_iscp, phtscp(scpps(i)), tmpscp
            filename = tmpscp.admn+'.ISCP'
            phtiscp = tmpscp
            save, phtiscp, FILE=!INTDIR+filename, /XDR
            WIDGET_LOG, wd_base, $
                TEXT ='Saving SCP measurement into '+!INTDIR+filename, $
                ACTION = 'APPEND'
            indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), count)
            IF count GT 0 THEN BEGIN
                      PIA_LOAD_ICS, phtcs(indcs(0)), phtics
                      FILE='CS'+phtics.UNIT+'.ICS'
                      junk = FINDFILE(!intdir+FILE, COUNT=count)
                      IF count GT 0 THEN $
                        MESSAGE, /INFO, $
                          'Compact Status file not written, already present' $
                      ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
            ENDIF ELSE MESSAGE, /INFO, $
                        'Compact Status not found in common, not saved' 
           ENDFOR             
          ENDIF
 
	

; now compute the FCS power tables

        WIDGET_LOG, wd_base, TEXT ='Now computing the FCS power tables', ACTION = 'APPEND'

	CASE phtscp(scpps(0)).det OF
                        'C1': detpix = 9
                        'C2': detpix = 4
                        'SL': detpix = 128
                        ELSE: detpix = 1
	ENDCASE

; 
; get first general information
;
        get_wheels, phtscp(scpps(0)).admn, wheels
	begdate = phtscp(scpps(0)).unit

;
; Accomodate all values into the COB given scheme
;
; The new scheme is that the same sequence is applied for all detectors
; The only difference is given by the pointing for the C detectors (Dec.20-95)
;

; Selection of source and background depends on the pointing or raster map mode



; straylight 1
			   str_pos = scpps(0)
			   handle_value, phtscp(str_pos(0)).aver, s_st1sig
			   handle_value, phtscp(str_pos(0)).avun, s_st1unc
			   handle_value, phtscp(str_pos(0)).flag, s_st1flg
			   t_st1 = date_conv(phtscp(str_pos(0)).unit)

; straylight 2			
			   if stray_nr eq 1 then str_pos = [str_pos, str_pos] $
				else str_pos = [str_pos, scpps(1)]
			   handle_value, phtscp(str_pos(1)).aver, s_st2sig
			   handle_value, phtscp(str_pos(1)).avun, s_st2unc
			   handle_value, phtscp(str_pos(1)).flag, s_st2flg
			   t_st2 = date_conv(phtscp(str_pos(1)).unit)
			   if stray_nr eq 1 then t_st2 = t_st2 + 5.  ;if only one stray

; FCSs	
			   s_fcsig = fltarr(detpix,fcs_nr)
			   s_fcsun = fltarr(detpix,fcs_nr)
			   s_fcsflg = fltarr(detpix,fcs_nr)
			   t_fcs = dblarr(fcs_nr)
			   tint_fcs = fltarr(fcs_nr)
			   heatpow = fltarr(fcs_nr)

			   fcs_pos = scpps(stray_nr+indgen(fcs_nr))
			   handle_value, phtscp(fcs_pos(0)).fcsn, fcsn
			   fcs = fcsn(0)
			   admn = phtscp(fcs_pos(0)).admn
			   psta_file='PSTA'+strmid(admn,4,8)+'.FITS'
			   read_cs,psta_file,header,csun
                           handle_value, phtcs(csun).handle, cst

			   IF fcs EQ 1 THEN trs = cst.f1ts(0) $
			   ELSE IF fcs EQ 2 THEN trs = cst.f2ts(0) ELSE BEGIN
				MESSAGE, /INFO, 'FCS number wrong, returning'
				RETURN
			   ENDELSE

			   FOR ifcs = 0,fcs_nr-1 DO BEGIN
			    handle_value, $
				phtscp(fcs_pos(ifcs)).fcs1, fcs1
			    heatpow(ifcs) = fcs1(0)
			    handle_value, $
				phtscp(fcs_pos(ifcs)).aver, junk
			    s_fcsig(*,ifcs) = junk(0:detpix-1)
			    handle_value, $
				phtscp(fcs_pos(ifcs)).avun, junk
			    s_fcsun(*,ifcs) = junk(0:detpix-1)
			    t_fcs(ifcs) = $
				date_conv(phtscp(fcs_pos(ifcs)).unit)
			    handle_value, phtscp(fcs_pos(ifcs)).arsi, arsi
			    tint_fcs(ifcs) = arsi(0)
			    handle_value, $
				phtscp(fcs_pos(ifcs)).flag, junk
			    s_fcsflg(*,ifcs) = junk(0:detpix-1)
			   ENDFOR


; distinguish between P and M modes

case bck_mapp of

	'P': BEGIN       ; this mode is valid for P and C detectors
	; background 1
			   bg_pos = scpps(stray_nr+fcs_nr)
			   handle_value, phtscp(bg_pos).aver, s_bg1sig
			   handle_value, phtscp(bg_pos).avun, s_bg1unc
			   handle_value, phtscp(bg_pos).flag, s_bg1flg
			   t_bg1 = date_conv(phtscp(bg_pos).unit)

	; Accomodate s_bg1 info into new arrays for the accumulated structure
			   str_bg1s = s_bg1sig
			   str_bg1u = s_bg1unc
			   str_bg1f = s_bg1flg
			   str_bg1t = t_bg1
	; background 2
			   if bck_nr eq 1 then bg_pos = [bg_pos, bg_pos]  $
 			      else  bg_pos = [bg_pos,scpps(stray_nr+fcs_nr+1)]
			   handle_value, phtscp(bg_pos(1)).aver, s_bg2sig
			   handle_value, phtscp(bg_pos(1)).avun, s_bg2unc
			   handle_value, phtscp(bg_pos(1)).flag, s_bg2flg
			   t_bg2 = date_conv(phtscp(bg_pos(1)).unit)
			   if bck_nr eq 1 then t_bg2 = t_bg2 + 5.


	; source 1  
			   src_pos = scpps(stray_nr+fcs_nr+bck_nr)
			   s_sr1sig = fltarr(detpix)
			   s_sr1unc = fltarr(detpix)
			   s_sr1flg = fltarr(detpix)
			   t_sr1 = dblarr(detpix)
			   handle_value, phtscp(src_pos).aver, s_srsig
			   handle_value, phtscp(src_pos).avun, s_srunc
			   handle_value, phtscp(src_pos).flag, s_srflg
			   handle_value, phtscp(src_pos).time, s_time
			   t_sr = date_conv(phtscp(src_pos).unit)

			   CASE 1 OF 
			   
			   detpix eq 1: BEGIN
			   	s_sr1sig = s_srsig
			   	s_sr1unc = s_srunc
			   	s_sr1flg = s_srflg
			   	t_sr1 = t_sr 
			   	END
			   		
			   detpix gt 1: BEGIN  ; - separate into pixels
				CASE det OF
					'C2': irp_src = [1,2,3,4]
					'C1': irp_src = [3,4,9,2,5,8,1,6,7]
					else: BEGIN
			   			print,' Wrong detector. Returning'
			   			return
			   		      END 
			        ENDCASE  
			        		      
			    irp_src = irp_src - 1
			    
			   	FOR I=0,detpix-1 DO BEGIN
					s_sr1sig(I) = s_srsig(I,irp_src(I))			   
                                	s_sr1unc(I) = s_srunc(I,irp_src(I))
                                	s_sr1flg(I) = s_srflg(I,irp_src(I))
					t_sr1(I) = t_sr + s_time(irp_src(I))/24./60/60
			   	ENDFOR 
			   	END
			   	
			   ELSE: BEGIN
			   	print,' Wrong detector. Returning'
			   	return
			   	END	

			   ENDCASE
			   

	; source 2 - separate into pixels 
			   if src_nr eq 1 then src_pos = [src_pos,src_pos] $ 
			      else src_pos =				   $
			         [src_pos,scpps(stray_nr+fcs_nr+bck_nr+1)]
			   s_sr2sig = fltarr(detpix)
			   s_sr2unc = fltarr(detpix)
			   s_sr2flg = fltarr(detpix)
			   t_sr2 = dblarr(detpix)
			   handle_value, phtscp(src_pos(1)).aver, s_srsig
			   handle_value, phtscp(src_pos(1)).avun, s_srunc
			   handle_value, phtscp(src_pos(1)).flag, s_srflg
			   handle_value, phtscp(src_pos(1)).time, s_time
			   t_sr = date_conv(phtscp(src_pos(1)).unit)
			   if src_nr eq 1 then t_sr = t_sr+5.  ; if only one src

			   CASE 1 OF 
			   
			   detpix eq 1: BEGIN
			   	s_sr2sig = s_srsig
			   	s_sr2unc = s_srunc
			   	s_sr2flg = s_srflg
			   	t_sr2 = t_sr 
			   	END
			   		
			   detpix gt 1: BEGIN  ; - separate into pixels
				CASE det OF
					'C2': irp_src = [1,2,3,4]
					'C1': irp_src = [3,4,9,2,5,8,1,6,7]
					else: BEGIN
			   			print,' Wrong detector. Returning'
			   			return
			   		      END 
			        ENDCASE  
			        		      
			    irp_src = irp_src - 1
			    
			   	FOR I=0,detpix-1 DO BEGIN
					s_sr2sig(I) = s_srsig(I,irp_src(I))			   
                                	s_sr2unc(I) = s_srunc(I,irp_src(I))
                                	s_sr2flg(I) = s_srflg(I,irp_src(I))
					t_sr2(I) = t_sr + $ 
					      s_time(irp_src(I))/24./60/60
			   	ENDFOR 
			   	END
			   	
			   ELSE: BEGIN
			   	print,' Wrong detector. Returning'
			   	return
			   	END	

			   ENDCASE

			   if src_nr eq 1 then t_sr2 = t_sr2 + 5.  ; if only one src

	END
	
	'M': BEGIN
	
		CASE det OF
			'C2': BEGIN
			       irp_src = [5,6,7,8]
			       irp_bck = intarr(4,4)
			       irp_bck(0,*) = [2,10,11,12]
			       irp_bck(1,*) = [1,9,11,12]
			       irp_bck(2,*) = [1,2,4,12]
			       irp_bck(3,*) = [1,2,3,11]
			      END
			     
			 'C1': BEGIN
			 	irp_src = [4,9,10,5,8,11,6,7,12]
			 	irp_bck = intarr(9,4)
			 	irp_bck(0,*) = [13,14,15,-4]
			 	irp_bck(1,*) = [1,2,13,14]
			 	irp_bck(2,*) = [1,2,3,-4]
			 	irp_bck(3,*) = [13,14,15,-4] 
			 	irp_bck(4,*) = [1,3,13,15]
			 	irp_bck(5,*) = [1,2,3,-4]
			 	irp_bck(6,*) = [13,14,15,-4] 
			 	irp_bck(7,*) = [2,3,14,15]
			 	irp_bck(8,*) = [1,2,3,-4]
			       END
			       
			  ELSE: BEGIN
			  	 print,'Wrong detector for this mode '
			  	 return
			  	END
		ENDCASE
		
		irp_src = irp_src - 1
		irp_bck = irp_bck - 1 
			  	      	
		; there is only one source measurement and several Raster points
		; for background
		src_pos = scpps(stray_nr+fcs_nr)
		src_pos = [src_pos, src_pos]
		bg_pos = src_pos  
		s_sr1sig = fltarr(detpix)
		s_sr1unc = fltarr(detpix)
		s_sr1flg = intarr(detpix)
		t_sr1 = dblarr(detpix)
		s_bg1sig = fltarr(detpix)
		s_bg1unc = fltarr(detpix)
		s_bg1flg = intarr(detpix)
		handle_value, phtscp(src_pos(0)).aver, s_srsig
		handle_value, phtscp(src_pos(0)).avun, s_srunc
		handle_value, phtscp(src_pos(0)).flag, s_srflg
		handle_value, phtscp(src_pos(0)).time, s_time
		t_sr = date_conv(phtscp(src_pos(0)).unit)

		t_bg1 = t_sr   ; assign the time of measurement to background
		
		FOR I=0,detpix-1 DO BEGIN
			s_sr1sig(I) = s_srsig(I,irp_src(i))			   
                        s_sr1unc(I) = s_srunc(I,irp_src(i))
                        s_sr1flg(I) = s_srflg(I,irp_src(i))
			t_sr1(I) = t_sr + s_time(irp_src(i))/24./60/60

			ind_val_bck = where(irp_bck(i,*) ge 0,nbck)
			IF nbck gt 0 THEN BEGIN 
			  weight_mean, s_srsig(I,irp_bck(i,ind_val_bck)), $
			      s_srunc(I,irp_bck(i,ind_val_bck)), bck_mn, bck_sg
			  s_bg1sig(I) = bck_mn
			  s_bg1unc(I) = bck_sg
			  s_bg1flg(I) = s_srflg(I,irp_bck(i,ind_val_bck(0)))
			ENDIF ELSE BEGIN
			  s_bg1sig(I) = s_srflg(I,irp_bck(i,0))
			  s_bg1unc(I) = s_srunc(I,irp_bck(i,0))
			  s_bg1flg(I) = 2
			ENDELSE        

		ENDFOR 
		
		s_bg2sig = s_bg1sig
		s_bg2unc = s_bg1unc
		s_bg2flg = s_bg1flg
		t_bg2 = t_bg1 + 5.
		
		s_sr2sig = s_sr1sig
		s_sr2unc = s_sr1unc
		s_sr2flg = s_sr1flg
		t_sr2 = t_sr1 + 5.
	

	END
	
endcase

	det_type = STRMID(phtscp(scpps(0)).det,0,1)
	CASE det_type OF
                        'C': BEGIN
			   categ = 0
			   aperture = 15
			   filter = wheels(1)
			   psf_correction = phtppsf(aperture-1, filter-2)
			   END

                        'P': BEGIN
			   categ = 1
			   aperture = wheels(1)
			   filter = wheels(2)
			   psf_correction = phtppsf(aperture-1, filter-1)
			   END

	ENDCASE

		wheels2id, det_type,wheels,filter_id,apert_id,lambda,area
		handle_value, phtscp(src_pos(0)).hdr, hdr
		source = fxpar(hdr,'OBJECT')
		pia_get_inbpower, source, filter_id, group=wd_base, $ 
		        date_conv(phtscp(src_pos(0)).unit, 'STRING')
		inbflux = c_inbp.inbpow
		inbflunc = c_inbp.inbpun / inbflux
		source = c_inbp.src_id
		qb_star = c_inbp.inbpht
	
		IF inbflux(0) EQ -1. THEN BEGIN
			MESSAGE, /INFO, 'Processing interrupted, returning'
			RETURN
		ENDIF

                WIDGET_LOG, wd_base, TEXT ='Source used: '+source, ACTION = 'APPEND'
                WIDGET_LOG, wd_base, TEXT ='FILTER used: '+ c_inbp.filter, $ 
                     ACTION = 'APPEND'

; compute in-band power and number of photons for actual filter and aperture
; assumption here is that input inbflux is in W/pixel for all detectors
	   	  inbflux = inbflux * psf_correction
	   	  qb_star = qb_star * psf_correction


; Interpolate and subtract background if necessary (C200, C100, P3)

	IF (categ LT 2) THEN BEGIN

; Reduce all data to real pixels dimension
; -> data 
	s_bg1sig = s_bg1sig(0:detpix-1)
	s_bg2sig = s_bg2sig(0:detpix-1)
	s_st1sig = s_st1sig(0:detpix-1)
	s_st2sig = s_st2sig(0:detpix-1)
	s_sr1sig = s_sr1sig(0:detpix-1)
	s_sr2sig = s_sr2sig(0:detpix-1)
	s_bg1unc = s_bg1unc(0:detpix-1)
	s_bg2unc = s_bg2unc(0:detpix-1)
	s_st1unc = s_st1unc(0:detpix-1)
	s_st2unc = s_st2unc(0:detpix-1)
	s_sr1unc = s_sr1unc(0:detpix-1)
	s_sr2unc = s_sr2unc(0:detpix-1)

; -> flags
	s_bg1flg = s_bg1flg(0:detpix-1)
	s_bg2flg = s_bg2flg(0:detpix-1)
	s_st1flg = s_st1flg(0:detpix-1)
	s_st2flg = s_st2flg(0:detpix-1)
	s_sr1flg = s_sr1flg(0:detpix-1)
	s_sr2flg = s_sr2flg(0:detpix-1)

		indfl2 = WHERE(s_bg2flg EQ 2, nflg2)
		indfl1 = WHERE(s_bg1flg EQ 2, nflg1)


; Flag as bad (=2) those pixels which are in both bckgr bad, as
; semi-bad (=1) those which could not be interpolated, taking the accepted 
; ones without interpolation
 
		s_bgflg = intarr(detpix)
		IF nflg1 NE 0 and nflg2 NE 0 THEN BEGIN
			match, indfl1, indfl2, suba, subb, count=count
			s_bgflg(indfl1) = 1                        
			s_bgflg(indfl2) = 1                        
			IF count GT 0 THEN $ 
				s_bgflg(suba) = 2 
		ENDIF ELSE IF nflg1 NE 0 THEN BEGIN  
			s_bgflg(indfl1) = 1                        
		ENDIF ELSE IF nflg2 NE 0 THEN BEGIN			 
			s_bgflg(indfl2) = 1                        
		ENDIF

		srbg1 = (s_bg2sig - s_bg1sig) / (t_bg2 - t_bg1) * $
				(t_sr1 - t_bg1) +  s_bg1sig  
		srbgu = (s_bg2sig+s_bg2unc - s_bg1sig-s_bg1unc) / $
			(t_bg2 - t_bg1) * (t_sr1 - t_bg1) +  s_bg1sig+s_bg1unc  
		srbg1un = srbgu - srbg1

			     
		IF nflg1 NE 0 THEN BEGIN
		  srbg1(indfl1) = s_bg2sig
		  srbg1un(indfl1) = s_bg2unc
		ENDIF
  
		IF nflg2 NE 0 THEN BEGIN
		  srbg1(indfl2) = s_bg1sig  
		  srbg1un(indfl2) = s_bg1unc
		ENDIF
		

		srbg2 = (s_bg2sig - s_bg1sig) / (t_bg2 - t_bg1) * $
				(t_sr2 - t_bg1) +  s_bg1sig
		srbgu = (s_bg2sig+s_bg2unc - s_bg1sig-s_bg1unc) / $
			(t_bg2 - t_bg1) * (t_sr2 - t_bg1) +  s_bg1sig+s_bg1unc  
		srbg2un = srbgu - srbg2

		IF nflg1 NE 0 THEN BEGIN
		  srbg2(indfl1) = s_bg2sig
		  srbg2un(indfl1) = s_bg2unc
		ENDIF
  
		IF nflg2 NE 0 THEN BEGIN
		  srbg2(indfl2) = s_bg1sig  
		  srbg2un(indfl2) = s_bg1unc
		ENDIF
		
		tmptxt = ['Background calculations: ', $
			  'Bckgr. 1: '+STRTRIM(STRING(s_bg1sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_bg1unc),2), $  
			  'Bckgr. 2: '+STRTRIM(STRING(s_bg2sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_bg2unc),2), ' ', $
			  'Bckgr. at Pos. 1: '+ STRTRIM(STRING(srbg1),2)+ $
			  ' +/- ' + STRTRIM(STRING(srbg1un),2), $  
			  'Bckgr. at Pos. 2: '+ STRTRIM(STRING(srbg2),2)+ $
			  ' +/- ' + STRTRIM(STRING(srbg2un),2), ' ', ' ']  
			  
                WIDGET_LOG, wd_base, TEXT =tmptxt, ACTION = 'APPEND'

		tmptxt = ['Source calculations: ', $
			  'Source+Bckgr. 1: '+STRTRIM(STRING(s_sr1sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_sr1unc),2), $  
			  'Source+Bckgr. 2: '+STRTRIM(STRING(s_sr2sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_sr2unc),2), ' ']

                s_sr1subt = s_sr1sig - srbg1 
                s_sr2subt = s_sr2sig - srbg2 
                s_sr1sunc = sqrt(s_sr1unc^2 + srbg1un^2)
                s_sr2sunc = sqrt(s_sr2unc^2 + srbg2un^2)

		s_sr1flg = s_bgflg
		s_sr2flg = s_bgflg

		indflg = WHERE(s_sr1flg EQ 2, nflg)
		IF nflg GT 0 THEN s_sr1flg(indflg) = 2
		indflg = WHERE(s_sr2flg EQ 2, nflg)
		IF nflg GT 0 THEN s_sr2flg(indflg) = 2
 
		tmptxt = [tmptxt, $
			  'Source 1: '+ STRTRIM(STRING(s_sr1subt),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_sr1sunc),2), $  
			  'Source 2: '+ STRTRIM(STRING(s_sr2subt),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_sr2sunc),2), ' ', ' ']  
			  
                WIDGET_LOG, wd_base, TEXT =tmptxt, ACTION = 'APPEND'

        ENDIF


; Interpolate signals of stellar calibrator for the times
; of the FCS exposures
	S_cal = fltarr(detpix,fcs_nr)
	S_calunc = S_cal
	tmptxt = ['Source at FCS positions:  ']
	FOR I=0,fcs_nr-1 DO BEGIN
		S_cal(*,I) = FLOAT((s_sr2subt - s_sr1subt) / (t_sr2 - t_sr1) * $
				(t_fcs(I) - t_sr1) +  s_sr1subt)  
		S_calunc(*,I) = FLOAT((s_sr2subt+s_sr2sunc - s_sr1subt-s_sr1sunc) /$
			(t_sr2 - t_sr1) * (t_fcs(I) - t_sr1) +  $
				s_sr1subt+s_sr1sunc ) - S_cal(*,I)

		tmptxt = [tmptxt, 'FCS pos '+STRTRIM(STRING(I+1),2)+': '+ $
			  STRTRIM(STRING(s_cal(*,I)),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_calunc(*,I)),2), ' ']
	ENDFOR 
	
        WIDGET_LOG, wd_base, TEXT =tmptxt, ACTION = 'APPEND'


;  Reduce FCS measurements signals with Interpolated straylight signals

	tmptxt = ['', 'Straylight calculations: ']
	tmptxt = [tmptxt, $
			  'Straylight 1: '+ STRTRIM(STRING(s_st1sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_st1unc),2), $  
			  'Straylight 2: '+ STRTRIM(STRING(s_st2sig),2)+ $
			  ' +/- ' + STRTRIM(STRING(s_st2unc),2), ' ', ' ']  
			  
		
	tmptxt = [tmptxt, 'FCS before and after straylight subtraction:  ']

	s_fcsred = s_fcsig
	s_fcsrun = s_fcsun
	
	FOR I=0,fcs_nr-1 DO BEGIN
		st_int = (s_st2sig - s_st1sig) / (t_st2 - t_st1) * $
				(t_fcs(I) - t_st1) +  s_st1sig 

		tmptxt = [tmptxt, 'FCS pos '+STRTRIM(STRING(I+1),2)+': '+ $
			  'Int. Strayl. :'+STRTRIM(STRING(st_int),2), $
			  'FCS crude: '+STRTRIM(STRING(s_fcsig(*,I)),2) + $
			  ' +/- ' + STRTRIM(STRING(s_fcsun(*,I)),2), ' ']

		S_fcsred(*,I) = S_fcsig(*,I) - st_int        

		S_fcsrun(*,I) = SQRT(S_fcsun(*,I)^2 + $        
			( (s_st2sig+s_st2unc - s_st1sig-s_st1unc) / $
				(t_st2 - t_st1) * (t_fcs(I) - t_st1) +  $
				s_st1sig+s_st1unc - st_int ) ^2 )

		tmptxt = [tmptxt, $
			  'FCS clean: '+STRTRIM(STRING(s_fcsred(*,I)),2) + $
			  ' +/- ' + STRTRIM(STRING(s_fcsrun(*,I)),2), ' ']

	ENDFOR
  
        WIDGET_LOG, wd_base, TEXT =tmptxt, ACTION = 'APPEND'


;  Calibrate FCS powers - FCSPOW/PIXEL for C - FCSPOW/mm^2 for P's
	IF categ EQ 0 THEN ap_corr = 1 ELSE ap_corr = 1. / area
	fcspow = fltarr(n_elements(s_fcsred),fcs_nr)
	fcspow = s_fcsred * inbflux / s_cal * ap_corr  	
        fcspun = fcspow * SQRT((s_fcsrun/s_fcsred)^2 + (s_calunc/s_cal)^2 +$
        		inbflunc^2 )

	tmptxt = ['  ', '**********************************************', $
		'FCS powers calculations: ', '====================== ']

	FOR I = 0, fcs_nr-1 DO BEGIN

	 tmptxt = [tmptxt, 'FCS position '+STRTRIM(STRING(I+1),2)+$
	 		', Heating power: '+STRTRIM(STRING(heatpow(I)),2), $
	 		' FCS power: '+ STRTRIM(STRING(fcspow(*,I)),2) + $
			  ' +/- ' + STRTRIM(STRING(fcspun(*,I)),2), ' ']

	ENDFOR
	
        WIDGET_LOG, wd_base, TEXT =tmptxt, ACTION = 'APPEND'

	

;  Set provisorily all flags to 0	
	fcsflg = FIX(fcspow*0)

; Fill the structures

	nele_str = n_elements(scpps)

	CASE det of 
 	 'SL': 
 	 'C2': BEGIN  
 	        cal_fcspre_str = {C2fcspre_typ}
 	        cal_fcspow_str = {C2_fcspow_typ}
	       END
 	 'C1': BEGIN  
 	        cal_fcspre_str = {C1fcspre_typ}
 	        cal_fcspow_str = {C1_fcspow_typ}
	       END
 	 ELSE: BEGIN
 	        cal_fcspre_str = {Pfcspre_typ}
 	        cal_fcspow_str = {P_fcspow_typ}
	       END
        ENDCASE

	
; Fill in the accumulated pre-reduced signal table - bit by bit (late requs)
; Straylight -> 1,2  -  Bckgr -> 3,4  -  Source -> 5,6  - FCS  -> all others

; Calculate orbit position just as time difference to perigee pass
         norb = strmid(phtscp(fcs_pos(0)).admn,4,3)
         orbit_pos = set_orbpos (norb, date_conv(phtscp(fcs_pos(0)).unit))
	 
	cal_fcspre_str = REPLICATE(cal_fcspre_str, stray_nr+fcs_nr+bck_nr+src_nr)
	cal_fcspow_str = REPLICATE(cal_fcspow_str, fcs_nr)

	cal_fcspre_str(*).filt_id = filter_id
	cal_fcspre_str(*).aper_id = apert_id
	cal_fcspre_str(*).orb_pos = orbit_pos
	cal_fcspre_str(*).source = source		
	cal_fcspre_str(*).proc_time = !stime	

	ind_str = indgen(stray_nr)
	ind_bck = indgen(bck_nr) + stray_nr
	ind_src = indgen(src_nr) + stray_nr + bck_nr
	ind_fcs = indgen(fcs_nr) + stray_nr + bck_nr + src_nr
	
	; Straylight
	t_str = date_conv(t_st1,'string')
	if stray_nr gt 1 then t_str = [t_str, date_conv(t_st2,'string')]
	cal_fcspre_str(ind_str).fcs = fcs
	cal_fcspre_str(ind_str).trs = trs
	cal_fcspre_str(ind_str).heatpow = 0.
	cal_fcspre_str(ind_str).type_id = 'Straylight'	
	cal_fcspre_str(ind_str).date = t_str

	s_stsig = fltarr(detpix, stray_nr)
	s_stunc = fltarr(detpix, stray_nr)
	s_stflg = intarr(detpix, stray_nr)
	s_stsig(*,0) = s_st1sig
	if stray_nr gt 1 then s_stsig(*,1) = s_st2sig
	s_stunc(*,0) = s_st1unc
	if stray_nr gt 1 then s_stunc(*,1) = s_st2unc
	s_stflg(*,0) = s_st1flg
	if stray_nr gt 1 then begin
	  s_stsig(*,1) = s_st2sig
	  s_stunc(*,1) = s_st2unc
	  s_stflg(*,1) = s_st2flg
	endif
	
	for i=0, stray_nr-1 do begin
		handle_value, phtscp(str_pos(i)).arsi, arsi
		cal_fcspre_str(ind_str(i)).tdtnr = phtscp(str_pos(i)).admn
		cal_fcspre_str(ind_str(i)).t_int = arsi(0) 
		cal_fcspre_str(ind_str(i)).signal = s_stsig(*,i)
		cal_fcspre_str(ind_str(i)).sigunc = s_stunc(*,i)
		cal_fcspre_str(ind_str(i)).red_sig = 0.
		cal_fcspre_str(ind_str(i)).red_sun = 0.
		cal_fcspre_str(ind_str(i)).sig_flg = s_stflg(*,i)
	endfor	

	; Background
	t_bck = date_conv(t_bg1, 'string')
	if bck_nr gt 1 then t_bck = [t_bck, date_conv(t_bg2, 'string')] 
	cal_fcspre_str(ind_bck).fcs = 0
	cal_fcspre_str(ind_bck).trs = 0
	cal_fcspre_str(ind_bck).heatpow = 0.
	cal_fcspre_str(ind_bck).type_id = 'Background'
	cal_fcspre_str(ind_bck).date = t_bck

	s_bgsig = fltarr(detpix, bck_nr)
	s_bgunc = fltarr(detpix, bck_nr)
	s_bgflg = intarr(detpix, bck_nr)
	s_bgsig(*,0) = s_bg1sig
	s_bgunc(*,0) = s_bg1unc
	s_bgflg(*,0) = s_bg1flg
	if bck_nr gt 1 then begin
	  s_bgsig(*,1) = s_bg2sig
	  s_bgunc(*,1) = s_bg2unc
	  s_bgflg(*,1) = s_bg2flg
	endif  
	
	for i=0, bck_nr-1 do begin
		handle_value, phtscp(bg_pos(i)).arsi, arsi
                cal_fcspre_str(ind_bck(i)).tdtnr = phtscp(bg_pos(i)).admn
		cal_fcspre_str(ind_bck(i)).t_int = arsi(0)
		cal_fcspre_str(ind_bck(i)).signal = s_bgsig(*,i)
		cal_fcspre_str(ind_bck(i)).sigunc = s_bgunc(*,i)
		cal_fcspre_str(ind_bck(i)).red_sig = 0.
		cal_fcspre_str(ind_bck(i)).red_sun = 0.
		cal_fcspre_str(ind_bck(i)).sig_flg = s_bgflg(*,i)
	endfor	
	
	; Source
	; write source time = first pixel in case of C100 and C200
	t_src = date_conv(t_sr1(0), 'string')
	if src_nr gt 1 then t_src = [t_src, date_conv(t_sr2(0), 'string')]
	cal_fcspre_str(ind_src).fcs = 0
	cal_fcspre_str(ind_src).trs = 0
	cal_fcspre_str(ind_src).heatpow = 0.
	cal_fcspre_str(ind_src).type_id = 'Source'
	cal_fcspre_str(ind_src).date = t_src

	s_srsig = fltarr(detpix, src_nr)
	s_srunc = fltarr(detpix, src_nr)
	s_srsubt = fltarr(detpix, src_nr)
	s_srsunc = fltarr(detpix, src_nr)
	s_srflg = intarr(detpix, src_nr)
	
	s_srsig(*,0) = s_sr1sig
	s_srunc(*,0) = s_sr1unc
	s_srsubt(*,0) = s_sr1subt
	s_srsunc(*,0) = s_sr1sunc
	s_srflg(*,0) = s_sr1flg
	if src_nr gt 1 then begin
	  s_srsig(*,1) = s_sr2sig
	  s_srunc(*,1) = s_sr2unc
	  s_srsubt(*,1) = s_sr2subt
	  s_srsunc(*,1) = s_sr2sunc
	  s_srflg(*,1) = s_sr2flg
	endif  

 	for i=0, src_nr -1 do begin
 		handle_value, phtscp(src_pos(i)).arsi, arsi
 		cal_fcspre_str(ind_src(i)).t_int = arsi(0)
	        cal_fcspre_str(ind_src(i)).tdtnr = phtscp(src_pos(i)).admn
		cal_fcspre_str(ind_src(i)).signal = s_srsig(*,i)
		cal_fcspre_str(ind_src(i)).sigunc = s_srunc(*,i)
		cal_fcspre_str(ind_src(i)).red_sig = s_srsubt(*,i)
		cal_fcspre_str(ind_src(i)).red_sun = s_srsunc(*,i)
		cal_fcspre_str(ind_src(i)).sig_flg = s_srflg(*,i)
	endfor	 
	
	
	; FCS
	
	cal_fcspre_str(ind_fcs).fcs = fcs
	cal_fcspre_str(ind_fcs).trs = trs
	cal_fcspre_str(ind_fcs).type_id = 'FCS'
	cal_fcspre_str(ind_fcs).date = phtscp(fcs_pos).unit
	cal_fcspre_str(ind_fcs).tdtnr = phtscp(fcs_pos).admn

	 cal_fcspow_str.FLAG = 0	
	 cal_fcspow_str.DATE = phtscp(fcs_pos).unit
	 cal_fcspow_str.TDTNR = phtscp(fcs_pos).admn
	 cal_fcspow_str.Filt_id = filter_id
	 cal_fcspow_str.Aper_id = apert_id
	 cal_fcspow_str.PSF_COR = psf_correction
	 cal_fcspow_str.Source = source
	 cal_fcspow_str.F_star = inbflux
	 cal_fcspow_str.Qb_star = Qb_star
	 cal_fcspow_str.FCS = fcs
	 cal_fcspow_str.TRS = trs
	 cal_fcspow_str.proc_time = !stime	
	 cal_fcspow_str.sed_file = c_inbp.sed_id	
	
	 for i=0, fcs_nr-1 do begin
	        cal_fcspre_str(ind_fcs(i)).heatpow = heatpow(i)
		cal_fcspre_str(ind_fcs(i)).t_int = tint_fcs(i)
		cal_fcspre_str(ind_fcs(i)).signal = s_fcsig(*,i)
	 	cal_fcspre_str(ind_fcs(i)).sigunc = s_fcsun(*,i)
	 	cal_fcspre_str(ind_fcs(i)).red_sig = s_fcsred(*,i)
	 	cal_fcspre_str(ind_fcs(i)).red_sun = s_fcsrun(*,i)
	 	cal_fcspre_str(ind_fcs(i)).sig_flg = s_fcsflg(*,i)

	 	cal_fcspow_str(i).FCSPOW = fcspow(*,I)
	 	cal_fcspow_str(i).FCSPUN = fcspun(*,I)
	 	cal_fcspow_str(i).SFLG = fcsflg(*,I)
	 	cal_fcspow_str(i).heatpow = heatpow(i)	
	 endfor
		

	proc_status = 0
        WIDGET_LOG, wd_base, TEXT='Ready READY Ready READY Ready', ACTION='APPEND'

	xMANAGER,No_BLOCK=1, "WIDGET_LOG", wd_base, GROUP=group 

	RETURN
	END
