;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_RESPCHK_PROC
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to responsivity check analysis
;
; CATEGORY:
;          PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_RESPCHK_PROC, det, respchkstr
;
; INPUT:
;       det: String indicating the detector 
;
; OUTPUT:
;	respchkstr: computed responsivity check structure
;
; COMMON BLOCKS:
;	deglitch_param
;	phtcs
;	phtacc
;	phterd
;	phtresdef
;	phtscp
;	phtsrd
;	pickmeas_results
;	gencho (internal)
;
; PROCEDURES USED:
;	PIA LIB:
;		deglitch_param_init
;		lin_voltages
;		phtcap_init
;		pia_cal_darkcur
;		pia_gencho
;		pia_load_ierd
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pickmeas
;		process_erd
;		process_srd
;		read_erd
;		read_resp
;		set_orbpos
;		weight_mean
;       ASTRO LIB:
;		boost_array
;
; MODIFICATION HISTORY:
;       Written by: Jose Acosta                     		June 1997
;	V6.3
;
;-
; *********************************************************************
; *
; *********************************************************************
PRO PIA_CAL_RESPCHK_PROC, det, respchkstr, FROM_SRD=from_srd, FROM_SCP=from_scp 

@phtacc
@phtscp
@gen_choices
COMMON widget_log_cmn, wd_text, wd_title

months = ['   ','JAN','FEB','MAR','APR','MAY','JUN','JUL','AUG','SEP','OCT',$
	'NOV','DEC']

	wd_title = 'Responsivity check Processing '
        WIDGET_LOG, wd_base, ACTION='CREATE'

;
;  Get the necessary information to start
;
	;check if SS detector the file name selection should be done using SL
	IF det EQ 'SS' THEN detid = 'SL' ELSE detid = det	
  
	IF NOT KEYWORD_SET(FROM_SRD) THEN from_srd = 0
	IF NOT KEYWORD_SET(FROM_SCP) THEN from_scp = 0  


;any step will produce new phtscp, therefore the index of phtscp can be 
; easilty obtained from difference of indexes between post and pre-processing 
firstscp = N_ELEMENTS(phtscp)


; call general processing, if starting from SRD or ERD, but always stop at SCP 
IF from_scp THEN BEGIN  ; no processing needed
	measnames = pickmeas(FTYPE="SCP", DTYPE=detid, /multi, POSI=posi)
	IF measnames(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		   ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
        ENDIF
 
        WIDGET_LOG, wd_base, TEXT ='Starting with measurements at SCP ',$ 
                   ACTION='APPEND'

	meas = 0L
	
        FOR I=0,N_ELEMENTS(measnames) - 1 DO BEGIN
	     measadd = WHERE(phtscp.admn EQ measnames(I),n)
             meas = [meas,measadd(posi(i))]
	ENDFOR

	scpps = meas(1:*)

ENDIF ELSE BEGIN   ; call general processing, if starting from SRD or ERD, 
	     ; but always stop at SCP  
	     pia_proc_gen, /SCP, FROM_SRD = from_srd
  	IF gen_stop THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
		WIDGET_LOG, wd_base, ACTION='DESTROY'
                RETURN
  	ENDIF
  	scpps = indgen(N_ELEMENTS(phtscp) - firstscp) + firstscp
  
ENDELSE
  

; Here starts computing new respchk structure from scp 

; now compute the DARK CURRENTS

        WIDGET_LOG, wd_base, TEXT =['Now constructing the structure',''], $
              ACTION='APPEND'

	det = phtscp(scpps(0)).det
	detpix = pixel_det(det,/ALLPX)

	allsg_resp = fltarr(detpix, N_ELEMENTS(scpps))
	allsgu_resp = fltarr(detpix, N_ELEMENTS(scpps))
	allsgflg_resp = intarr(detpix, N_ELEMENTS(scpps))
	
	CASE 1 OF 
	  strmid(det,0,1) eq 'P': respchkstr = {PRESPCHKSTR_TYP}
	  det eq 'C1':	respchkstr = {C1RESPCHKSTR_TYP}
	  det eq 'C2':	respchkstr = {C2RESPCHKSTR_TYP}
	  strmid(det,0,1) eq 'S':    respchkstr = {SLRESPCHKSTR_TYP}
	  ELSE:
	ENDCASE
	
	respchkstr = replicate(respchkstr,N_ELEMENTS(scpps))
        respchkstr.proc_date = !STIME

	; start loop over SCP measurements
	FOR imeas = 0, N_ELEMENTS(scpps)-1 DO BEGIN 
	  
; Fill the structures

	    pia_load_iscp,phtscp(scpps(imeas)), scptmp

            ;write the reset interval
            give_cs, scptmp, cs, record, conv_cst 
            inttmp = conv_cst.intt(record)
            respchkstr(imeas).treset = STRMID(inttmp,0,STRLEN(inttmp)-2)
            
            ;check if reset interval correction has been applied
            rin0 = FXPAR(scptmp.hdr, 'PRS_RIN0',count=ipar)
            IF ipar GT 0 THEN respchkstr(imeas).rescor = 1
         
	    flag = 0
	    ; compute orbital position, just as the time after perigee passagge
	    norb = strmid(scptmp.admn,4,3)
	    ; compute orbital position, just as the time after perigee passagge
	    orbit_pos = set_orbpos(norb, scptmp.unit)
	    
	    ; compute julian date
	    temp = scptmp.unit
	    dayom = fix(gettok(temp,'-'))
	    month_name = gettok(temp,'-') 
	    year = fix(gettok(temp,' '))
	    hour = fix(gettok(temp,':'))
	    minute = fix(gettok(temp,':'))
	    sec = float(strtrim(strmid(temp,0,5)))
	    
	    nmonth = where(months eq month_name)
	    nmonth = nmonth(0)	
	    rday = dayom + (hour+ minute/60. + sec/3600.)/24.    

	    juldate,[year,nmonth,rday],jdate

	    respchkstr(imeas).admn = scptmp.admn
	    respchkstr(imeas).date = scptmp.unit
	    respchkstr(imeas).rev = norb
	    respchkstr(imeas).jdate = jdate
	    respchkstr(imeas).orbpos = orbit_pos
	    CASE 1 of 
	      scptmp.fcs1 gt 0.0: BEGIN
	           fcsn = 1
	           fcspow = scptmp.fcs1
	          END
	      scptmp.fcs2 gt 0.0: BEGIN
	           fcsn = 2
	           fcspow = scptmp.fcs2
	          END
	      ELSE: flag = 1
	    ENDCASE                
	    respchkstr(imeas).fcsn = fcsn
	    respchkstr(imeas).fcspow = fcspow
	    respchkstr(imeas).flag = flag
	    respchkstr(imeas).temp = scptmp.temp
		
	    respchkstr(imeas).signal  = scptmp.aver
	    respchkstr(imeas).sgunc = scptmp.avun
	    respchkstr(imeas).sgflg = scptmp.flag
	

	   ; compute orbital position, just as the time after perigee passagge
	   norb = strmid(scptmp.admn,4,3)

	   WIDGET_LOG, wd_base, TEXT = ['Including Measurement '+scptmp.admn], $
	      ACTION='APPEND'
	      
	ENDFOR      

        WIDGET_LOG, wd_base, TEXT =['','Ready READY Ready READY Ready'], $
		 ACTION='APPEND'

        xMANAGER,No_BLOCK=1, "WIDGET_LOG", wd_base, GROUP= group
	
	
	
	RETURN
	END
