;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_RESPONS_PROC
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to the Responsivity tables File production
;
; CATEGORY:
;          PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_RESPONS_PROC, det
;
; INPUT:
;       det: 		String indicating the detector
;	source: 	Name of Calibration Source used
;	flux: 		Expected flux of Calibration Source used [Jy], 
;			where flux is a two elements array, and flux(0)
;			is the flux and flux(1) is the uncertainty
;	inbpower: 	Expected power on detector [W], where inbpower
;             		is a two elements array, and inbpower(0) is the 
;			power and inbpower(1) is the uncertainty
;
; COMMON BLOCKS:
;	aperture_info
;	deglitch_param
;	phtcap
;	phterd
;	phtscp
;	phtsrd
;	gencho 
;	pia_get_inbp_res
;	pickmeas_results
;	pia_cal_respons_proc_cmn (internal)
;	pia_cal_respons_proc_res_cmn (for passing results)
;
; PROCEDURES USED:
;	PIA LIB:
;		deglitch_param_init
;		get_wheels
;		lin_voltages
;		pia_cal_respons
;		pia_gencho
;		pia_get_inbpower
;		pia_load_ierd
;		pia_load_iscp
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pickmeas
;		process_erd
;		process_srd
;		read_erd
;		subtr_bckg
;		weight_mean
;		pia_fcspre_struct_val_choice
;		get_fcspre_str
;       ASTRO LIB:
;		date_conv
;	INTERNAL:
;		pia_cal_respons_proc_event
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/ESETC)                     	May 1995
;	Modified (last) by: cg, wmt				September 1995
;	Oct 95 Version 3.5
;	Pre-reduced signal tables handling +
;	print + file included (CG)				November 1995
;	In-band power from FITS file, extension of resp 
;	  accumulated structure, only /from_table allowed (CG)	December 1995
;	The orbital position was added to the responsivity 
;	  accumulated structure			      (Jacosta) April 1996
;	Using new pickmeas and pickmeas_results-CB (CG)		May 1997
;	V6.3
;			
;-
;
; *********************************************************************
; * Event routine for PIA_CAL_RESPONS_PROC
; *********************************************************************
PRO PIA_CAL_RESPONS_PROC_EVENT, event

COMMON pia_cal_respons_proc_cmn, text1

 	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue

        CASE UVALUE OF
        
		'PRINT': BEGIN
				WIDGET_CONTROL, text1, GET_VALUE=text
				pia_disp_print_file, "Respons_proc", $
			 		GROUP = event.top, TEXT=text, $
			 		TITLE = "PIA Responses (P+C) processing"
			 END
			 	
        	'QUIT': WIDGET_CONTROL, event.top, /destroy
		ELSE:

	END
	
RETURN
END
; *********************************************************************
; * Main widget and processing routine for PIA_CAL_RESPONS_PROC
; *********************************************************************
PRO PIA_CAL_RESPONS_PROC, det, source=source, $
                 ibpower=ibpower, ibpowun=ibpowun, $
                 from_table=from_table, nr_of_fcs=nr_of_fcs
@phterd
@phtsrd
@phtscp
@phtcap
@phtppsf
@aperture_info
@deglitch_param
@gencho

COMMON pia_cal_respons_proc_cmn, text1
COMMON pia_get_inbp_res, c_inbp

COMMON pia_cal_respons_proc_res_cmn, resp, dresp, allres, alldres, $
	resp_str, proc_status 
COMMON pickmeas_results, pickmeas_res



proc_status = -1
; just for testing purposes
	IF KEYWORD_SET(source) THEN source=source ELSE source='UNKNOWN'
	IF KEYWORD_SET(ibpower) THEN inbpower=ibpower ELSE inbpower=1.
	IF KEYWORD_SET(ibpowun) THEN inbpowun=ibpowun ELSE inbpowun=1.
	
	IF STRMID(det,0,1) NE 'P' AND STRMID(det,0,1) NE 'C' THEN BEGIN
		XDISPLAYFILE, "", $
			        TITLE = "Input not accepted", $
                                WIDTH = 50, $
                                HEIGHT = 4, $
                          TEXT = [" Detector not accepted  "]
                        RETURN
	ENDIF

; get the infos needed for all the different modes
	IF N_ELEMENTS(c1cap) EQ 0 THEN phtcap_init
	IF N_ELEMENTS(phtppsf) EQ 0 THEN read_ppsf
	
	CASE STRMID(det,0,2) OF
                        'C1': begin
                                detpix = 9
                                capac = C1CAP
                              end
                        'C2': begin
                                detpix = 4
                                capac = C2CAP
                              end
                        'P1': begin
                                detpix = 1
                                capac = P1CAP
                              end
                        'P2': begin
                                detpix = 1
                                capac = P2CAP
                              end
                        'P3': begin
                                detpix = 1
                                capac = P3CAP
                              end
	ENDCASE

		

        base = WIDGET_BASE(TITLE='CFDP 52516: Responsivity Tables Processing ', $
			/COLUMN)

	control = WIDGET_BASE(base, /ROW)
	quit = WIDGET_BUTTON(control, VALUE=" QUIT ", UVALUE= "QUIT")
	prtsave = WIDGET_BUTTON(control, VALUE=" PRINT & SAVE ", UVALUE= "PRINT")
	
        text1 = WIDGET_TEXT(base, XSIZE=40, YSIZE=30, /SCROLL)

        WIDGET_CONTROL, base, /REALIZE

        WIDGET_CONTROL, base, /hourglass


;  Choice of measurements or FCS Pre-reduced Signal table 

CASE 1 OF 
	 KEYWORD_SET(from_table): BEGIN    ; USING CUMULATED PRE-REDUCED TABLE
          WIDGET_CONTROL, text1, SET_VALUE='Using pre-reduced tables ', $
			/append
	  WIDGET_CONTROL, base, /hourglass

; get the whole pre-reduced signal structure

	  get_fcspre_str, det, prestr

; take only the Source measurements from it
	  indsrc = WHERE(prestr.type_id EQ 'Source', nsrc)
	  IF nsrc EQ 0 THEN BEGIN
	  	Message, /info, 'No Source found, returning'
	  	RETURN
	  ENDIF
	  substr = prestr(indsrc)
	  
; call the choice menu
	  pia_fcspre_struct_val_choice, substr, acc_ind
	  
	  IF acc_ind(0) EQ -1 THEN BEGIN
	  	Message, /info, 'nothing to be used, returning'
	  	RETURN
	  ENDIF
	  
	  red_proc_time = substr(acc_ind).proc_time
	  red_dates = substr(acc_ind).date
	  red_orbpos = substr(acc_ind).orb_pos
	  red_tdtnr = substr(acc_ind).tdtnr
	  red_sig = substr(acc_ind).red_sig
	  red_sun = substr(acc_ind).red_sun
	  sig_flg = substr(acc_ind).sig_flg
	  red_src = substr(acc_ind).source
	  filt_id = substr(acc_ind).filt_id
	  t_int = substr(acc_ind).t_int
	  aper_id = substr(acc_ind).aper_id	  	  	

	  IF STRMID(det,0,1) EQ 'P' THEN BEGIN
		detpix = 1
		nelem = N_ELEMENTS(red_sig)
	  ENDIF ELSE BEGIN
		detpix = N_ELEMENTS(red_sig(*,0))
		nelem = N_ELEMENTS(red_sig(0,*))
	  ENDELSE
	  
          i_aver = FLTARR(detpix)
          i_avun = i_aver

; accomodate in responses / source 
	  FOR IELE = 0, nelem - 1 DO BEGIN
	  
	  	  if detpix gt 1 then begin
	  	  	i_aver = red_sig(*,IELE) 
	  	  	i_avun = red_sun(*,IELE)
	  	  endif else begin 
	  	        i_aver = red_sig(IELE)
	  	        i_avun = red_sun(IELE)
	  	  endelse      
		  i_src = red_src(IELE)
		  i_filt = filt_id(IELE)
		  i_dates = red_dates(IELE)
		  
		  pia_get_inbpower, i_src, i_filt, i_dates, group=base
		  inbpower = c_inbp.inbpow
		  inbpowun = c_inbp.inbpun
		  photons = c_inbp.inbpht
		  err_photons = c_inbp.inphun
		  
		  IF inbpower EQ -1. THEN BEGIN
		    tmpcom = 'In-band-flux for '+ i_src + ': '+ i_filt 
		    pia_write_comment, instcom=tmpcom, inbpower
		    inbpower = inbpower(0)
		    inbpowun = inbpower * 0.2
		    tmpcom = 'Nr. of Photons for '+ i_src + ': '+ i_filt 
		    pia_write_comment, instcom=tmpcom, photons
		    photons = photons(0)
		    err_photons = photons * 0.2
		  ENDIF

		  id2wheels, det, i_filt, aper_id(IELE), wheels, lambda, area
; if P, DO NOT multiply with aperture area in addition to psf
	 IF STRMID(det,0,1) EQ 'P' THEN BEGIN 
	    inbpower = inbpower * phtppsf(wheels(1)-1, wheels(2)-1) 
	    inbpowun = inbpowun * phtppsf(wheels(1)-1, wheels(2)-1) 
	    photons = photons * phtppsf(wheels(1)-1, wheels(2)-1)
	    err_photons = err_photons * phtppsf(wheels(1)-1, wheels(2)-1)
	 ENDIF ELSE BEGIN
	    inbpower = inbpower * phtppsf(14, wheels(1)-2)
	    inbpowun = inbpowun * phtppsf(14, wheels(1)-2)
	    photons = photons * phtppsf(14, wheels(1)-2)
	    err_photons = err_photons * phtppsf(14, wheels(1)-2)
	 ENDELSE

        	  respons = capac * (i_aver / inbpower)

        	  drespons = respons * sqrt((i_avun / i_aver)^2 + $
        	  	(inbpowun/inbpower)^2)
		  nep = i_avun * capac / respons * sqrt(2*t_int(IELE))
		  err_nep = nep * (drespons/respons)


		  lambda = FLOAT(STRMID(i_filt,3,4)) * 1.e-6


		  eta =  (SQRT(photons)/NEP)^2 * (4.d-25 / lambda)^2 
		  err_eta = eta * SQRT( (err_photons/photons)^2 + $
		  			4*(err_nep/nep)^2)  

		  IF N_ELEMENTS(iall_proc_time) EQ 0 THEN BEGIN
			iall_proc_time = red_proc_time(IELE)
			iall_dates = red_dates(IELE)
			iall_orbpos = red_orbpos(IELE)
			iall_tdtnr = red_tdtnr(IELE)
		  	iall_filt = i_filt
		  	iall_photons = photons
		  	iall_inbpow = inbpower
		  	iall_inbpun = inbpowun
		  	iall_src = i_src
		  ENDIF ELSE BEGIN
		  	iall_proc_time  = [iall_proc_time, red_proc_time(IELE)]
		  	iall_dates	= [iall_dates, red_dates(IELE)]
			iall_orbpos     = [iall_orbpos, red_orbpos(IELE)]
			iall_tdtnr      = [iall_tdtnr, red_tdtnr(IELE)]
		  	iall_filt	= [iall_filt, i_filt]
		  	iall_photons	= [iall_photons, photons]
		  	iall_inbpow	= [iall_inbpow, inbpower]
		  	iall_inbpun	= [iall_inbpun, inbpowun]
		  	iall_src	= [iall_src, i_src]
		  ENDELSE
		  
		  boost_array, iall_resp, respons
		  boost_array, iall_dresp, drespons
		  boost_array, iall_nep, nep
		  boost_array, iall_errnep, err_nep
		  boost_array, iall_eta, eta
		  boost_array, iall_erreta, err_eta

		ENDFOR

		  
	END    ; END OF READING FROM PREREDUCED TABLES
	

	ELSE: BEGIN   ; Process directly from measurements
	  MESSAGE, /INFO, 'This is bad implemented, use of tables mandatory'
	  RETURN
	;
	;  Get the necessary information to start
	;
	
          pia_gencho, DET=det

          IF gen_stop EQ 1 THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
                RETURN
          ENDIF

          IF N_ELEMENTS(gen_fits) EQ 0 THEN gen_fits = 0
          IF N_ELEMENTS(gen_erd) EQ 0 THEN gen_erd = 0
          IF N_ELEMENTS(gen_srd) EQ 0 THEN gen_srd = 0
          IF N_ELEMENTS(gen_scp) EQ 0 THEN gen_scp = 0
          IF N_ELEMENTS(gen_lin) EQ 0 THEN gen_lin = 0
          IF N_ELEMENTS(gen_dar) EQ 0 THEN gen_dar = 0
          IF N_ELEMENTS(gen_deg) EQ 0 THEN gen_deg = 0


	;
	; ERD Selection criteria 
	; 
	  IF (gen_erd) THEN pia_rodisc, nndr = -1

	;
	; SRD Selection criteria 
	; 
	  IF (gen_srd) THEN pia_sigdisc, nsig = -1

	;
	; Select ERD Files or individual measurements using multiselectors
	;


	; FITS files
	  IF gen_fits EQ 0 THEN BEGIN

	; the /multi keyword is used here, pia_pickfile without /multi would 
	; return an empty string
	    fillist = pia_pickfile(/multi, filter=gen_filt)
            IF fillist(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		  ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
            ENDIF


	;
	; Compute first measurement to be used
	; 
            firstscp = N_ELEMENTS(phtscp)
	;

	; Read ERD files	
	;	 					
        	filename = STRUPCASE(STRTRIM(fillist(0),2))
        	filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)

        	WIDGET_CONTROL, text1, SET_VALUE='Reading ERD '+filename, $
			/append
		WIDGET_CONTROL, base, /hourglass

		read_erd, filename,hdr,ilunr
          	
	; Compute first measurement on the ERD level
	;		IF I EQ 0 THEN firsterd = ilunr(0)
                firsterd = ilunr(0)
	; If first measurement of each file to be displayed		
	;		pia_disperd, phterd(ilunr(0))

	;
	; Compute last measurement to be used
	; 
          lasterd = max(ilunr)

	;
	; Compute all measurements to be processed to SRD 
	;

	;  --- > Compute which of those measurements belong to the desired  
	; detector (P1, P2 and P3) can be present within the same file
	;
	 IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phterd(firsterd:lasterd).admn,0,2) EQ det, n)
	  meas = firsterd + inddet
         ENDIF ELSE $
	  meas = indgen(lasterd-firsterd+1) + firsterd
	;
	; background1, source1, source2, background2 for C100, C200 and P3 
          meas = firsterd + [0, 2, 10, 12]
	  if det eq 'P1' or det eq 'P2' then $
            meas = firsterd + [1, 9]

	; ######   end of FITS files reading  ######

	ENDIF ELSE BEGIN

	; ######   begin of individual measurements reading #######

	; individual measurements

	  CASE STRMID(gen_filt,1,1) OF
		'1': dtyp = 'C1'
		'2': dtyp = 'C2'
		'S': dtyp = 'SL'
		'P': dtyp = 'P#'
	  ENDCASE

	  measnames = pickmeas(FTYPE="ERD", DTYPE=dtyp, /multi)

          IF measnames(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, ' No Data selected, CAL procedure stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
          ENDIF

          WIDGET_CONTROL, text1, SET_VALUE='Starting with measurements  ', $
			/append
	  WIDGET_CONTROL, base, /hourglass

	  meas = WHERE(phterd.admn EQ measnames(0),n)
	  IF n GT 1 THEN meas = meas(pickmeas_res.position(0))

	  IF N_ELEMENTS(measnames) GT 1 THEN BEGIN
            FOR I=1,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	measadd = WHERE(phterd.admn EQ measnames(I),n)
	        IF n GT 1 THEN measadd = measadd(pickmeas_res.position(i))
		meas = [meas,measadd(0)]
	    ENDFOR
	  ENDIF
          
		
	; 
          firstscp = N_ELEMENTS(phtscp)

	ENDELSE


	;
	; Linearization
	;
	IF (gen_lin) THEN BEGIN

          WIDGET_CONTROL, text1, SET_VALUE='Linearizing ramps ...  ', $
			/append
          WIDGET_CONTROL, base, /hourglass
	   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN
		pia_load_ierd, phterd(meas(i)), tmpdata
		newhdr = tmpdata.hdr
		lin_voltages, tmpdata, newhdr
		handle_value, phterd(meas(i)).conp, tmpdata.conp,/set,/no_copy
		handle_value, phterd(meas(i)).hdr, newhdr,/set,/no_copy
           ENDFOR

	ENDIF
	;
	; Process all data to srd 
	;

        WIDGET_CONTROL, text1, $
                SET_VALUE='Processing all measurements  ERD->SRD', $
                /append
        WIDGET_CONTROL, text1, $
                SET_VALUE='(This will take some time, take it easy) ', $
                /append
        WIDGET_CONTROL, base, /hourglass
 

	process_erd, meas, 1, hdr, srdps

;
;
; Loop over SRD structures

	FOR i=0,N_ELEMENTS(srdps)-1 DO BEGIN

	  pia_load_isrd, phtsrd(srdps(i)), tmpdata
	  
	  newheader = tmpdata.hdr
;
; Subtract dark current 
;
	  IF (gen_dar) THEN BEGIN
            CASE tmpdata.det of
                    'C1' : detpix = 9
                    'C2' : detpix = 4
                    'SL' : detpix = 128
                    ELSE : detpix = 1
            ENDCASE

	    WIDGET_CONTROL, text1, $
              SET_VALUE='Subtracting Dark Current', $
                /append
            WIDGET_CONTROL, base, /hourglass
            darkcur, tmpdata.slop(0:detpix-1, *), tmpdata.det, outslop

; Replace slop contents, log it into the newheader

                IF (N_ELEMENTS(tmpdata.slop(0:detpix-1,*)) EQ    $
                        N_ELEMENTS(outslop)) THEN BEGIN
                          tmpdata.slop(0:detpix-1,*) = outslop
                          hdrlen = N_ELEMENTS(newheader)
                          hdrend = newheader(hdrlen-1)
                          JUNK = 'PRS_DARK=          '+$
                                 '          1 / Dark Current subtracted  '
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
                ENDIF
          ENDIF
;
;  Deglitching
;

	  IF (gen_deg) THEN BEGIN

        	WIDGET_CONTROL, text1, SET_VALUE='Deglitching ...', $
                /append

        	WIDGET_CONTROL, base, /hourglass

                degl_acc = tmpdata.slop * 0.
                deglitch_param_init
                sizdim1 = n_elements(degl_acc(*,0))
                FOR II = 0,sizdim1 - 1 DO BEGIN
                  deglitch_all, tmpdata.slop(II,*), $
                        tmpdata.step, tmpdata.slun(II,*), degacc
                  degl_acc(II,*) = degacc
                  indnacc = WHERE(degacc NE 0, count)
                  IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
                ENDFOR
; include information in header
                hdrlen = N_ELEMENTS(newheader)
                hdrend = newheader(hdrlen-1)
                JUNK = 'PRS_DEGL=          '+$
                                 '          1 / Deglitching corr. applied'
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
	  ENDIF

          WIDGET_CONTROL, text1, $
		SET_VALUE=' Processing measurements  SRD->SCP', $
                /append
          WIDGET_CONTROL, base, /hourglass

	  process_srd, tmpdata.admn, newheader, scppos		

   	ENDFOR

	; last measurement in SCP
	lastscp = N_ELEMENTS(phtscp) -1

	; Compute which of those measurements belong to the desired detector 
	; (P1, P2 and P3) can be present within the same file
	;
	IF STRMID(det,0,1) EQ 'P' THEN BEGIN
	  inddet = WHERE(STRMID(phtscp(firstscp:lastscp).admn,0,2) EQ det, n)
	  firstscp = firstscp + min(inddet)
	  lastscp = firstscp + max(inddet)
	ENDIF

	; now compute the responsivity tables

        WIDGET_CONTROL, text1, $
		SET_VALUE='Now computing responsivity tables', $
                /append
        WIDGET_CONTROL, base, /hourglass


	; 
	; get first general information
	;
        get_wheels, phtscp(firstscp).admn, wheels
	begdate = phtscp(firstscp).unit

	;
	; Accomodate all values into the COB given scheme
	;
	; PHT-P3, C1 and C2
	;   0 background
	;   1 cold_fcs
	;   2 src (staring)
	;   3 fcs
	;   4 short source
	;   5 fcs
	;   6 short source
	;   7 short fcs
	;   8 short source
	;   9 fcs
	;  10 src
	;  11 cold_fcs
	;  12 background
	;
	CASE STRMID(phtscp(firstscp).det,0,1) OF
                        'C': BEGIN
			   categ = 0
			   filter = wheels(1)
	; background 1
			   handle_value, phtscp(firstscp).aver, s_bg1sig
			   handle_value, phtscp(firstscp).avun, s_bg1unc
			   handle_value, phtscp(firstscp).flag, s_bg1flg
			   t_bg1 = date_conv(phtscp(firstscp).unit)
	; background 2
			   handle_value, phtscp(firstscp+3).aver, s_bg2sig
			   handle_value, phtscp(firstscp+3).avun, s_bg2unc
			   handle_value, phtscp(firstscp+3).flag, s_bg2flg
			   t_bg2 = date_conv(phtscp(firstscp+3).unit)
	; source 1 - separate into pixels 
			   s_sr1sig = fltarr(detpix)
			   s_sr1unc = fltarr(detpix)
			   s_sr1flg = fltarr(detpix)
			   t_sr1 = dblarr(detpix)
			   handle_value, phtscp(firstscp+1).aver, s_srsig
			   handle_value, phtscp(firstscp+1).avun, s_srunc
			   handle_value, phtscp(firstscp+1).flag, s_srflg
			   handle_value, phtscp(firstscp+1).time, s_time
			   t_sr = date_conv(phtscp(firstscp+1).unit)
			   FOR I=0,detpix-1 DO BEGIN
	; we want the center of the map which should be the max of all other signals
				irp = WHERE(s_srsig(I,*) EQ max(s_srsig(I,*)))
				s_sr1sig(I) = s_srsig(I,irp)			   
                                s_sr1unc(I) = s_srunc(I,irp)
                                s_sr1flg(I) = s_srflg(I,irp)
				t_sr1(I) = t_sr + s_time(irp)/24./60/60
			   ENDFOR 

	; source 2 - separate into pixels 
			   s_sr2sig = fltarr(detpix)
			   s_sr2unc = fltarr(detpix)
			   s_sr2flg = fltarr(detpix)
			   t_sr2 = dblarr(detpix)
			   handle_value, phtscp(firstscp+2).aver, s_srsig
			   handle_value, phtscp(firstscp+2).avun, s_srunc
			   handle_value, phtscp(firstscp+2).flag, s_srflg
			   handle_value, phtscp(firstscp+2).time, s_time
			   t_sr = date_conv(phtscp(firstscp+2).unit)
			   FOR I=0,detpix-1 DO BEGIN
				irp = WHERE(s_srsig(I,*) EQ max(s_srsig(I,*)))
				s_sr2sig(I) = s_srsig(I,irp)			   
                                s_sr2unc(I) = s_srunc(I,irp)
                                s_sr2flg(I) = s_srflg(I,irp)
				t_sr2(I) = t_sr + s_time(irp)/24./60/60
			   ENDFOR 

                        END
						  
                        'P': BEGIN

			 filter = wheels(2)
	; P1 and P2 are different from P3
			 IF STRMID(phtscp(firstscp).det,0,2) EQ 'P3' THEN BEGIN
			   categ = 1

	; background 1
			   handle_value, phtscp(firstscp).aver, s_bg1sig
			   handle_value, phtscp(firstscp).avun, s_bg1unc
			   handle_value, phtscp(firstscp).flag, s_bg1flg
			   t_bg1 = date_conv(phtscp(firstscp).unit)
	; background 2
			   handle_value, phtscp(firstscp+3).aver, s_bg2sig
			   handle_value, phtscp(firstscp+3).avun, s_bg2unc
			   handle_value, phtscp(firstscp+3).flag, s_bg2flg
			   t_bg2 = date_conv(phtscp(firstscp+3).unit)
	; source 1  
			   handle_value, phtscp(firstscp+1).aver, s_sr1sig
			   handle_value, phtscp(firstscp+1).avun, s_sr1unc
			   handle_value, phtscp(firstscp+1).flag, s_sr1flg
			   t_sr1 = date_conv(phtscp(firstscp+1).unit)

	; source 2  
			   handle_value, phtscp(firstscp+2).aver, s_sr2sig
			   handle_value, phtscp(firstscp+2).avun, s_sr2unc
			   handle_value, phtscp(firstscp+2).flag, s_sr2flg
			   t_sr2 = date_conv(phtscp(firstscp+2).unit)

			 ENDIF ELSE BEGIN
			   categ = 2
	; P1, P2
; PHT-P1 and P2
;   0 cold_fcs
;   1 src (Chopped), src -> bkg1 -> src -> bkg2 -> src
;   2 fcs
;   3 short source
;   4 fcs
;   5 short source
;   6 short fcs
;   7 short source
;   8 fcs
;   9 src (Chopped), src -> bkg1 -> src -> bkg2 -> src
;  10 cold_fcs
; source 1 + bckgr (chopped)  
			   pia_load_iscp, phtscp(firstscp), tmpdata
			   subtr_bckg, tmpdata, 2, srcsubt 
			   t_sr1 = date_conv(phtscp(firstscp).unit)
			   WEIGHT_MEAN, srcsubt.aver, srcsubt.avun, $
					s_sr1sig, s_sr1unc
                           s_sr1flag = srcsubt.flag
; source 2 + bckgr (chopped) 
			   pia_load_iscp, phtscp(firstscp+1), tmpdata
			   subtr_bckg, tmpdata, 2, srcsubt 
			   t_sr2 = date_conv(phtscp(firstscp+1).unit)
			   WEIGHT_MEAN, srcsubt.aver, srcsubt.avun, $
					s_sr2sig, s_sr2unc
                           s_sr2flag = srcsubt.flag
        		 ENDELSE   
			END

                        ELSE: BEGIN
				MESSAGE, /INFO, 'wrong input, returning ...'
				RETURN
			END
	ENDCASE

; Interpolate and subtract background if necessary (C200, C100, P3)

	IF (categ LT 2) THEN BEGIN

; Reduce all data to real pixels dimension
; -> data 
		s_bg1sig = s_bg1sig(0:detpix-1)
		s_bg2sig = s_bg2sig(0:detpix-1)
		s_sr1sig = s_sr1sig(0:detpix-1)
		s_sr2sig = s_sr2sig(0:detpix-1)
		s_bg1unc = s_bg1unc(0:detpix-1)
		s_bg2unc = s_bg2unc(0:detpix-1)
		s_sr1unc = s_sr1unc(0:detpix-1)
		s_sr2unc = s_sr2unc(0:detpix-1)

; -> flags
		s_bg1flg = s_bg1flg(0:detpix-1)
		s_bg2flg = s_bg2flg(0:detpix-1)
		s_sr1flg = s_sr1flg(0:detpix-1)
		s_sr2flg = s_sr2flg(0:detpix-1)


		indfl2 = WHERE(s_bg2flg EQ 2, nflg2)
		indfl1 = WHERE(s_bg1flg EQ 2, nflg1)

; here it should follow a similar algorithm as in pia_cal_fcspow_proc to get
; rid of those bad flagged measurements - "future music" - since this routine
; has to be upgraded soon.
		

                s_sr1sig = s_sr1sig - s_bg1sig 
                s_sr2sig = s_sr2sig - s_bg2sig 
                s_sr1unc = sqrt(s_sr1unc^2 + s_bg1unc^2)
                s_sr2unc = sqrt(s_sr2unc^2 + s_bg2unc^2)

;		s_sr1flg = s_bgflg
;		s_sr2flg = s_bgflg

;		indflg = WHERE(s_sr1flg EQ 2, nflg)
;		IF nflg GT 0 THEN s_sr1flg(indflg) = 2
;		indflg = WHERE(s_sr2flg EQ 2, nflg)
;		IF nflg GT 0 THEN s_sr2flg(indflg) = 2
 
        ENDIF
 
; Output table 1 according to cfdp52516.txi
        tmp_tdt = strmid(phtscp(firstscp).admn, 4, 8)

; Level 3, measurement processing
        
         if strmid(det, 0, 1) eq 'C' then begin
                     
          i_aver = fltarr(detpix)
          i_avun = i_aver

	  FOR I=0,detpix-1 DO BEGIN
           weight_mean, [s_sr1sig(i), s_sr2sig(i)], $
                        [s_sr1unc(i),s_sr2unc(i)], i_ave, i_avu
	   i_aver(i) = i_ave
	   i_avun(i) = i_avu
	  ENDFOR 

         endif else weight_mean, [s_sr1sig, s_sr2sig], $
                          [s_sr1unc, s_sr2unc], i_aver, i_avun


; get the id's, lambda and aperture areas
	 i_src = source
	 wheels2id, det, wheels, i_filt, i_aper, lambda, area
	 t_int = phterd(meas(1)).tint


; read in-band powers from calibration files
	 table = STRMID(det,0,1) + 'inbpow_table'
	 i_calfilt = STRMID(i_filt,0,1) + STRMID(i_filt,2,5)
	 get_inbpower, !CALC + table, source, i_calfilt, inbpower

; if P, multiply with aperture area in addition to psf
	 IF STRMID(det,0,1) EQ 'P' THEN $ 
	    inbpower = inbpower * area * phtppsf(wheels(1)-1, wheels(2)-1) $
	    ELSE inbpower = inbpower * phtppsf(15, wheels(1)-1)
	 
	 inbpowun = 0.2 * inbpower

         respons = capac * (i_aver / inbpower)
         drespons = respons * sqrt((i_avun / i_aver)^2 + $
        	  	(inbpowun/inbpower)^2)
	 nep = i_avun * capac / respons * sqrt(2*t_int)
	 err_nep = nep * (drespons/respons)

;		  table = STRMID(det,0,1) + 'photon_table'
;		  get_photons, !CALC + table, source, i_filt, photons

		  lambda = FLOAT(STRMID(i_filt,3,4)) * 1.e-6
; provisory until table is created


	photons = inbpower * 1.d0 / 6.63e-34 / lambda
	err_photons = 0.2 * photons

		  eta =  (SQRT(photons)/NEP)^2 * (4.d-25 / lambda)^2 
		  err_eta = eta * SQRT( (err_photons/photons)^2 + $
		  			4*(err_nep/nep)^2)  
	
; fill elements in the arrays for structure
	 iall_date = phtscp(firstscp+1).unit
	 iall_src = source
	 iall_filt = i_filt
	 iall_inbpow = inbpower
	 iall_inbpun = inbpowun
	 iall_photons = photons
	 iall_respons = respons
	 iall_drespons = drespons
	 iall_nep = nep
	 iall_errnep = err_nep
	 iall_eta = eta
	 iall_erreta = err_eta  
	 iall_proc_time = systime()

; **************************
; end of measurements choice
; **************************

	END
	
	
ENDCASE
	  
	

        CASE STRMID(det,0,2) of
         'SL':
         'C2': BEGIN
		cal_respons_str = {C2respstr_typ}
	 END
         'C1': BEGIN
		cal_respons_str = {C1respstr_typ}
	 END
         ELSE: BEGIN
		cal_respons_str = {Prespstr_typ}
	 END
        ENDCASE

	allelem = N_ELEMENTS(iall_dates)
	flag = INTARR(allelem)

	cal_respons_str = REPLICATE(cal_respons_str,allelem)

	peles = 0
	celes = 0
	FOR eles = 0, allelem-1 DO BEGIN
         cal_respons_str(eles).flag = flag(eles)
         cal_respons_str(eles).date = iall_dates(eles)
	 cal_respons_str(eles).orbpos = iall_orbpos(eles)
	 cal_respons_str(eles).tdtnr = iall_tdtnr(eles)
         cal_respons_str(eles).source = iall_src(eles)
         cal_respons_str(eles).filter_id = iall_filt(eles)
         cal_respons_str(eles).f_star = iall_inbpow(eles)
         cal_respons_str(eles).f_star_unc = iall_inbpun(eles)
         cal_respons_str(eles).qb_star = iall_photons(eles)
         cal_respons_str(eles).resps = iall_resp(*,eles)
         cal_respons_str(eles).respu = iall_dresp(*,eles)
         cal_respons_str(eles).eta = iall_eta(*,eles)
         cal_respons_str(eles).err_eta = iall_erreta(*,eles)
         cal_respons_str(eles).nep = iall_nep(*,eles)
         cal_respons_str(eles).err_nep = iall_errnep(*,eles)
         cal_respons_str(eles).proc_time = iall_proc_time(eles)
         cal_respons_str(eles).sed_file = c_inbp.sed_id

	 xpixl = findgen(N_ELEMENTS(iall_resp(*,0))) + 1 + eles*0.03

	 peles = peles + 1
	 IF peles EQ 8 THEN peles=1
	 celes = celes + 1
	 IF celes EQ 10 THEN celes = 1
	 IF eles EQ 0 THEN overp = 0 ELSE overp = 1
	 anntmp = iall_src(eles) + ' on ' + iall_filt(eles) + $
	 	' -  Meas. Date: '+ iall_dates(eles)
	 pia_xplot, xpixl, iall_resp(*,eles), iall_dresp(*,eles),$
	 	Title= det+ ' Computed Responses', xtitle='Pixel #', $
	 	ytitle='Response    [A/W]', overplot=overp, $
	 	psym=peles, color=celes, $
	 	annotation= anntmp, $
	 	legend='Response #'+STRTRIM(STRING(eles+1),2) 


	ENDFOR

	peles = 0
	celes = 0
	FOR pix = 0, N_ELEMENTS(iall_resp(*,0))-1 DO BEGIN

	 peles = peles + 1
	 IF peles EQ 8 THEN peles=1
	 celes = celes + 1
	 IF celes EQ 10 THEN celes = 1
	 IF pix EQ 0 THEN overp = 0 ELSE overp = 1
	 anntmp = 'Responses for pixel '+ STRTRIM(STRING(pix+1),2)

	 PIA_XPLOT, iall_inbpow, iall_resp(pix,*), $
	 	iall_dresp(pix,*), Title= det+ ' Computed Responses', $
	 	xtitle='Expected in-band power', $
	 	ytitle='Response    [A/W]', overplot=overp, $
	 	psym=peles, color=celes, $
	 	annotation= anntmp, $
	 	legend='Response Pix '+STRTRIM(STRING(pix+1),2) 
	ENDFOR

	
	resp = fltarr(detpix)
	dresp = resp 
	eta = fltarr(detpix)
	deta = eta 
	nep = fltarr(detpix)
	dnep = nep 
	text = ['Calculations done: ', '========================', ' ']
           WIDGET_CONTROL, text1, SET_VALUE=text, $
                /append

	FOR I=0,detpix-1 DO BEGIN
	   weight_resp = iall_dresp(i,*)*0.+1.
           weight_mean, iall_resp(i,*), weight_resp, avr, sig
	   resp(i) = avr
	   dresp(i) = sig

           weight_mean, iall_eta(i,*), iall_erreta(i,*), avr, sig
	   eta(i) = avr
	   deta(i) = sig

           weight_mean, iall_nep(i,*), iall_errnep(i,*), avr, sig
	   nep(i) = avr
	   dnep(i) = sig

	   text = [' ', 'Responsivity Values found for pixel ' + $
	   	STRTRIM(STRING(i+1),2), $
		'---------------------------------------------', $
		'Source: '+ iall_src + ' --> ' + $
		STRTRIM(STRING(iall_resp(i,*)),2) + '+/-' + $
		STRTRIM(STRING(iall_dresp(i,*)),2), ' ', $
		'Average:  ' + STRTRIM(STRING(resp(i)),2) + '+/-' + $
		STRTRIM(STRING(dresp(i)),2) ]
		
           WIDGET_CONTROL, text1, SET_VALUE=text, $
                /append

	   text = [' ', 'Quantum efficiency Values found for pixel ' + $
	   	STRTRIM(STRING(i+1),2), $
		'---------------------------------------------', $
		'Source: '+ iall_src + ' --> ' + $
		STRTRIM(STRING(iall_eta(i,*)),2) + '+/-' + $
		STRTRIM(STRING(iall_erreta(i,*)),2), ' ', $
		'Average:  ' + STRTRIM(STRING(eta(i)),2) + '+/-' + $
		STRTRIM(STRING(deta(i)),2) ]
		
           WIDGET_CONTROL, text1, SET_VALUE=text, $
                /append

	   text = [' ', 'NEP Values found for pixel ' + $
	   	STRTRIM(STRING(i+1),2), $
		'---------------------------------------------', $
		'Source: '+ iall_src + ' --> ' + $
		STRTRIM(STRING(iall_nep(i,*)),2) + '+/-' + $
		STRTRIM(STRING(iall_errnep(i,*)),2), ' ', $
		'Average:  ' + STRTRIM(STRING(nep(i)),2) + '+/-' + $
		STRTRIM(STRING(dnep(i)),2) ]
		
           WIDGET_CONTROL, text1, SET_VALUE=text, $
                /append
	ENDFOR

	proc_status = 0
	allres = iall_resp
	alldres = iall_dresp
	resp_str = cal_respons_str
		
        WIDGET_CONTROL, text1, SET_VALUE='Ready READY Ready READY Ready', $
                /append

	xMANAGER,No_BLOCK=1, "PIA_CAL_RESPONS_PROC", base, GROUP=group


	RETURN
	END

