;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_CAL_SWAVL_PROC
;
; PURPOSE:
;	Calibration Files Derivation Procedures (CFDP)  
;	corresponding to the PHT-S Wavelengths tables File production
;
; CATEGORY:
;          PIA - processing
;
; CALLING SEQUENCE:
;       PIA_CAL_SWAVL_PROC
;
; OUTPUT:
;	MNPW: FLTARR(128,n) = mean powers of SS/SL obtained for all n raster 
;		position  
;	MNPU: FLTARR(128,n) = MNPW uncertainties
;
; COMMON BLOCKS:
;	colour_indices
;	deglitch_param
;	phterd
;	phtscp
;	phtslwavlen
;	phtsrd
;	pickmeas_results
;	gencho (internal)
;
; PROCEDURES USED:
;	PIA LIB:
;		deglitch_param_init
;		get_rastersteps
;		pia_gencho
;		pia_load_iscp
;		pia_load_isrd
;		pia_pickfile
;		pia_rodisc
;		pia_sigdisc
;		pia_xplot
;		pickmeas
;		process_erd
;		process_scp
;		process_srd
;		read_erd
;		weight_mean
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA)                   	June 1995
;	Modified (last) by: cg					August 1995
;	Oct 95 Version 3.5
;	Using new pickmeas and pickmeas_results-CB (CG)		May 1997
;	V6.3
;			
;-
; *********************************************************************
; *
; *********************************************************************
PRO PIA_CAL_SWAVL_PROC, MNPW, MNPU

@phtslwavlen
@deglitch_param
@colour_indices
@phterd
@phtsrd
@phtscp
@gencho
COMMON pickmeas_results, pickmeas_res


	NGC7027_lines = [2.6252, 2.826, 2.873, 3.039, 3.093, 3.297, 3.740, $
			4.0512, 4.485, 4.654, 4.765, 5.608, 6.949, 6.985, $
			7.457, 8.992, 9.715, 10.524, 12.814] 

        base = WIDGET_BASE(TITLE='CFDP 53201: S-Wavelengths Tables Processing ', $
			/COLUMN)

        text1 = WIDGET_TEXT(base, XSIZE=50, YSIZE=30, /SCROLL)

        WIDGET_CONTROL, base, /REALIZE

        WIDGET_CONTROL, base, /hourglass

;
;  Get the necessary information to start
;
	
        pia_gencho, DET='SL'

        IF gen_stop EQ 1 THEN BEGIN
                MESSAGE, /INFO, ' CAL Procedure stopped'
                RETURN
        ENDIF

        IF N_ELEMENTS(gen_fits) EQ 0 THEN gen_fits = 0
        IF N_ELEMENTS(gen_erd) EQ 0 THEN gen_erd = 0
        IF N_ELEMENTS(gen_srd) EQ 0 THEN gen_srd = 0
        IF N_ELEMENTS(gen_scp) EQ 0 THEN gen_scp = 0
        IF N_ELEMENTS(gen_lin) EQ 0 THEN gen_lin = 0
        IF N_ELEMENTS(gen_dar) EQ 0 THEN gen_dar = 0
        IF N_ELEMENTS(gen_deg) EQ 0 THEN gen_deg = 0


;
; ERD Selection criteria 
; 
	IF (gen_erd) THEN pia_rodisc, nndr = -1

;
; SRD Selection criteria 
; 
	IF (gen_srd) THEN pia_sigdisc, nsig = -1

;
; Select ERD Files or individual measurements using multiselectors
;


; FITS files
	IF gen_fits EQ 0 THEN BEGIN

; just one file picking is here allowed
	  fillist = pia_pickfile(PATH=!FITDIR, filter=gen_filt, /multi)
          IF fillist(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, $
		  ' No Data selected, Calibration Procedure Sequence stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
          ENDIF


;
; Compute first measurement to be used
; 
          firstscp = N_ELEMENTS(phtscp)
;

; Read ERD files	
;	 					
	  FOR I=0,N_ELEMENTS(fillist)-1 DO BEGIN 

        	filename = STRUPCASE(STRTRIM(fillist(i),2))
          	filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)

          	WIDGET_CONTROL, text1, SET_VALUE='Reading ERD '+filename, $
			/append
	  	WIDGET_CONTROL, base, /hourglass

	  	read_erd, filename,hdr,ilunr
          	
; Compute first measurement on the ERD level
	  	IF I EQ 0 THEN firsterd = ilunr(0)

; If first measurement of each file to be displayed		
;	  	pia_disperd, phterd(ilunr(0))

	  ENDFOR
;
; Compute last measurement to be used
; 
          lasterd = max(ilunr)

;
; Compute all measurements to be processed to SRD 
;

	  meas = indgen(lasterd-firsterd+1) + firsterd
;

	ENDIF ELSE BEGIN

; individual measurements
	  dtyp = 'SL'

	  measnames = pickmeas(FTYPE="ERD", DTYPE=dtyp, /multi)

          IF measnames(0) EQ '' THEN BEGIN
                MESSAGE, /INFO, ' No Data selected, CAL procedure stopped'
		WIDGET_CONTROL, base, /DESTROY
                RETURN
          ENDIF

          WIDGET_CONTROL, text1, SET_VALUE='Starting with measurements  ', $
			/append
	  WIDGET_CONTROL, base, /hourglass

	  meas = WHERE(phterd.admn EQ measnames(0),n)
	  IF n GT 1 THEN meas = meas(pickmeas_res.position(0))

	  IF N_ELEMENTS(measnames) GT 1 THEN BEGIN
            FOR I=1,N_ELEMENTS(measnames) - 1 DO BEGIN
	  	measadd = WHERE(phterd.admn EQ measnames(I),n)
	        IF n GT 1 THEN measadd = measadd(pickmeas_res.position(i))
		meas = [meas,measadd(0)]
	    ENDFOR
	  ENDIF
          
		
; 
          firstscp = N_ELEMENTS(phtscp)

	ENDELSE


;
; Process all data to srd 
;

        WIDGET_CONTROL, text1, $
                SET_VALUE='Processing all measurements  ERD->SRD', $
                /append
        WIDGET_CONTROL, text1, $
                SET_VALUE='(This will take some time, take it easy) ', $
                /append
        WIDGET_CONTROL, base, /hourglass
 

	process_erd, meas, 1, hdr, srdps

;
;
; Loop over SRD structures

	FOR i=0,N_ELEMENTS(srdps)-1 DO BEGIN

	  pia_load_isrd, phtsrd(srdps(i)), tmpdata
	  
	  newheader = tmpdata.hdr
;
; Subtract dark current 
;
	  detpix = 128

	  IF (gen_dar) THEN BEGIN
	   
	    WIDGET_CONTROL, text1, $
              SET_VALUE='Subtracting Dark Current', $
                /append
            WIDGET_CONTROL, base, /hourglass
            darkcur, tmpdata.slop(0:detpix-1, *), tmpdata.det, outslop

; Replace slop contents, log it into the newheader

                IF (N_ELEMENTS(tmpdata.slop(0:detpix-1,*)) EQ    $
                        N_ELEMENTS(outslop)) THEN BEGIN
                          tmpdata.slop(0:detpix-1,*) = outslop
                          hdrlen = N_ELEMENTS(newheader)
                          hdrend = newheader(hdrlen-1)
                          JUNK = 'PRS_DARK=          '+$
                                 '          1 / Dark Current subtracted  '
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
                ENDIF
	  ENDIF
;
;  Deglitching
;

	  IF (gen_deg) THEN BEGIN

        	WIDGET_CONTROL, text1, SET_VALUE='Deglitching ...', $
                /append

        	WIDGET_CONTROL, base, /hourglass

                degl_acc = tmpdata.slop * 0.
                deglitch_param_init
                sizdim1 = n_elements(degl_acc(*,0))
                FOR II = 0,sizdim1 - 1 DO BEGIN
                  deglitch_all, tmpdata.slop(II,*), $
                        tmpdata.step, tmpdata.slun(II,*), degacc
                  degl_acc(II,*) = degacc
                  indnacc = WHERE(degacc NE 0, count)
                  IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
                ENDFOR
; include information in header
                hdrlen = N_ELEMENTS(newheader)
                hdrend = newheader(hdrlen-1)
                JUNK = 'PRS_DEGL=          '+$
                                 '          1 / Deglitching corr. applied'
                          newheader = [newheader(0:hdrlen-2),JUNK]
                          newheader = [newheader,hdrend]
	  ENDIF

          WIDGET_CONTROL, text1, $
		SET_VALUE=' Processing measurements  SRD->SCP', $
                /append
          WIDGET_CONTROL, base, /hourglass

	  process_srd, tmpdata.admn, newheader, scppos		

   	ENDFOR

; last measurement in SCP
	lastscp = N_ELEMENTS(phtscp) -1

; now compute the Spectrae

        WIDGET_CONTROL, text1, $
		SET_VALUE='Now computing the spectrae', $
                /append
        WIDGET_CONTROL, base, /hourglass

; average the background spectra

; background 1
	handle_value, phtscp(firstscp).aver, s_bg1sig
	handle_value, phtscp(firstscp).avun, s_bg1unc
	handle_value, phtscp(firstscp).flag, s_bg1flg

; background 2
	handle_value, phtscp(firstscp+2).aver, s_bg2sig
	handle_value, phtscp(firstscp+2).avun, s_bg2unc
	handle_value, phtscp(firstscp+2).flag, s_bg2flg

        i_aver = fltarr(detpix)
        i_avun = i_aver

	FOR I=0,detpix-1 DO BEGIN
          weight_mean, [s_bg1sig(i), s_bg2sig(i)], $
                        [s_bg1unc(i),s_bg2unc(i)], i_ave, i_avu
	  i_aver(i) = i_ave
	  i_avun(i) = i_avu
	ENDFOR 
	
; source   
	handle_value, phtscp(firstscp+1).aver, s_sr1sig
	handle_value, phtscp(firstscp+1).avun, s_sr1unc
	handle_value, phtscp(firstscp+1).flag, s_sr1flg
 	handle_value, phtscp(firstscp+1).rpid, rpid

; perform background subtraction
	get_rastersteps, rpid, step
	FOR I=0,max(step)-1 DO BEGIN
	 s_sr1sig(0:detpix-1,i) = s_sr1sig(0:detpix-1,i) - i_aver 
	 s_sr1unc(0:detpix-1,i) = sqrt(s_sr1unc(0:detpix-1,i)^2 + i_avun^2)
	ENDFOR
	
; now convert the subtracted values into a pseudo SPD (not into phtspd)
	pia_load_iscp, phtscp(firstscp+1), tmpscp
	tmpscp.aver = s_sr1sig
	tmpscp.avun = s_sr1unc
	process_scp, tmpscp, tmpscp.hdr, 0, spdps, spdstr, /no_SPD 
	
	handle_value, spdstr(1).mnpw, mnss
	handle_value, spdstr(0).mnpw, mnsl
	handle_value, spdstr(1).mnpu, mnus
	handle_value, spdstr(0).mnpu, mnul

	FOR I=0,max(step)-1 DO BEGIN
	  mnpw = [mnss(*,I),mnsl(*,I)]
	  mnpu = [mnus(*,I),mnul(*,I)]
	  
	  pia_xplot, slwavelen, mnpw, mnpu, $
	 	psym = 4, xtitle='Pixel #', $
	 	ytitle = 'In-band Power Pos '+ STRTRIM(STRING(I+1),2), $
	 	title='Bckgr. subtracted Spectrum', $
	 	annotation = tmpscp(0).unit, legend=ytitle  
	  npoints = N_ELEMENTS(NGC7027_lines)
	  points = fltarr(npoints) + max(mnpw)
	  FOR ipoint=0, npoints-1 DO BEGIN 
	   pia_xplot, [NGC7027_lines(ipoint), NGC7027_lines(ipoint)], $
	   	[0, points(ipoint)], psym=0, color=red, overplot=99,$
	  	legend = 'NGC7027 emission line '+STRTRIM(STRING(ipoint+1),2)
	  ENDFOR	  	 
	  
	ENDFOR 

; accomodate mean powers and uncertainties to the output variables

	mnpw = [mnss,mnsl]
	mnpu = [mnus,mnul]

        
        WIDGET_CONTROL, text1, SET_VALUE='Ready READY Ready READY Ready', $
                /append
        WAIT, 3
        WIDGET_CONTROL, base, /destroy

	RETURN
	END

