;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;     WRITE_CHOPCOR
; PURPOSE:
;       Write signal losses correction factor to a FITS file
;
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       WRITE_CHOPCOR, DET, CHOPCOR_STR, FILENAME = FILENAME
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       det: character string of 'C1', 'C2', 'P'
;       CHOPCOR_STR: structure containing the correction factor per 
;		     detector 
;
; OPTIONAL INPUT:
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;       PIA library:
;               c2p_IA2pipe: changing order of C2 pixels 
;
; MODIFICATION HISTORY:
;       Written by: Jose Acosta  - Jan 98
;       V7.0
;;
;-
pro write_chopcor, det, chopcor_str, COMMENT=comment, VERSION=version, $ 
       FILENAME = filename, OVERWRITE = overwrite

  IF N_PARAMS() NE 2 THEN BEGIN
    print,'Syntax: write_chopcor, det, chopcor_str [, COMMENT=comment]' + $
        '[, FILENAME = filename] [,VERSION=version]'
    RETURN
  ENDIF
  
  IF det NE chopcor_str.det THEN BEGIN 
     print,'Error: detector and  input structure are incompatible'
     RETURN
  ENDIF      
 
  CASE 1 OF
    strmid(det,0,2) eq 'C1' : BEGIN
      pixels = 9
      indpix = INDGEN(pixels)
      tcomment = 'C100 signal losses correction per chopper freq.'
      pfilename = 'PC1CHOPCOR'
      END
    strmid(det,0,2) eq 'C2' : BEGIN
      pixels = 4
      indpix = INDGEN(pixels)
      tcomment = 'C200 signal losses correction per chopper freq.'
      pfilename = 'PC2CHOPCOR'
      END
    strmid(det,0,1) eq 'P'  : BEGIN
      pixels = 3
      CASE det OF 
         'P1': indpix = 0
         'P2': indpix = 1
         'P3': indpix = 2
      ENDCASE   
      tcomment = 'PP signal losses correction per chopper freq.'
      pfilename = 'PPCHOPCOR'
      END
    ELSE: BEGIN
      message, 'DETECTOR not valid. Accepted detectors are C1, C2 and P#'
      RETURN
    END
  ENDCASE

NFREQ = N_ELEMENTS(chopcor_str.inchop_time)

IF NOT KEYWORD_SET(comment) THEN comment = tcomment

IF KEYWORD_SET(filename) THEN filetmp = filename ELSE filetmp = pfilename 

IF strpos(filetmp,'.FITS') EQ -1 THEN filetmp = filetmp+ '.FITS'
fitsfile = !CALSAVE + filetmp  

res = FINDFILE(fitsfile, count=count)

IF count NE 0 THEN newfile = 0 ELSE newfile = 1

; if newfile then define array dark_calg, if not read from fitsfile
IF newfile EQ 1 THEN chopcor_calg = FLTARR(pixels, NFREQ, 2) $
  ELSE BEGIN 
     FXREAD, fitsfile, chopcor_calg 
     SXHREAD, fitsfile, newhdr
  ENDELSE    

chopcor_calg(indpix,*,0) = chopcor_str.corr
chopcor_calg(indpix,*,1) = chopcor_str.corr_unc

; change pixel ordering according to PIPELINE convention
IF det eq 'C2' THEN BEGIN 
      c2p_ia2pipe, chopcor_calg, transchopcor
      chopcor_calg = transchopcor      
ENDIF    

  ; if file does not exist then create hdr
  IF newfile THEN BEGIN 
      print,' Creating new header '
      mkhdr, newhdr, chopcor_calg
      fxaddpar, newhdr, 'ORIGIN', 'ESA', 'European Space Agency'
      fxaddpar, newhdr, 'TELESCOP', 'ISO', 'Infrared Space Observatory'
      fxaddpar, newhdr, 'INSTRUME', 'PHT', 'Instrument used'
      fxaddpar, newhdr, 'FILENAME', pfilename, $
         'File name assigned in ISO archive'
      fxaddpar, newhdr, 'FILEVERS', '', 'Version ID assigned in ISO archive'
  ENDIF 

      fxaddpar, newhdr, 'DATE', STRMID(!STIME,0,11), 'Date of creating file 
      fxaddpar, newhdr, 'COMMENT', comment
      fxaddpar, newhdr, 'PIA_VERS',!PIA_VERSION, $ 
         'Internal (user) PIA-CAL Version'
      
  ; write FITS file  
      IF newfile EQ 0 THEN $
         IF NOT KEYWORD_SET(OVERWRITE) THEN fitsfile = fitsfile + '.NEW' $
             ELSE  spawn,'/bin/rm '+EXPAND_PATH(fitsfile)        
      writefits, fitsfile, chopcor_calg, newhdr  

RETURN
END
