;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_darkorb, det, darkorb_str, COMMENT=comment, VERSION=version, $ 
       FILENAME = filename, OVERWRITE = overwrite

;+
; NAME:
;     WRITE_DARKORB
; PURPOSE:
;	Write a dark current values depending on orbital position to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_DARKORB, FILENAME, DET, DARKORB_STR
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       det: character string of 'C1', 'C2', 'P'
;       DARKORB_STR: 
;
; OPTIONAL INPUT:
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;       PIA library:
;        	c2p_IA2pipe: changing order of C2 pixels 
;
; MODIFICATION HISTORY:
;       Written by: Jose Acosta
;       V6.6
;;
;-
  on_error, 2
  
  IF N_PARAMS() NE 2 THEN BEGIN
    print,'Syntax: write_darkorb, det, darkc_orb_str'
    RETURN
  ENDIF
  
  IF det NE darkorb_str.det THEN BEGIN 
     print,'Error: detector and  input structure are incompatible'
     RETURN
  ENDIF      

  CASE 1 OF
    strmid(det,0,2) eq 'SL' : BEGIN
      pixels = 128
      indpix = INDGEN(pixels)
      tcomment = 'SS/SL dark current per orb. pos'
      pfilename = 'PSDARK'
      END
    strmid(det,0,2) eq 'C1' : BEGIN
      pixels = 9
      indpix = INDGEN(pixels)
      tcomment = 'C100 dark current per orb. pos'
      pfilename = 'PC1DARK'
      END
    strmid(det,0,2) eq 'C2' : BEGIN
      pixels = 4
      indpix = INDGEN(pixels)
      tcomment = 'C200 dark current per orb. pos'
      pfilename = 'PC2DARK'
      END
    strmid(det,0,1) eq 'P'  : BEGIN
      pixels = 3
      CASE det OF 
         'P1': indpix = 0
         'P2': indpix = 1
         'P3': indpix = 2
      ENDCASE   
      tcomment = 'PP dark current per orb. pos'
      pfilename = 'PPDARK'
      END
    ELSE: BEGIN
      message, 'DETECTOR not valid. Accepted detectors are SL, C1, C2 and P#'
      RETURN
    END
  ENDCASE
  
  IF NOT KEYWORD_SET(comment) THEN comment = det+ $ 
       ' - Dark current per orbital pos.'

  IF KEYWORD_SET(filename) THEN filetmp = filename ELSE filetmp = pfilename 
  
  IF strpos(filetmp,'.FITS') EQ -1 THEN filetmp = filetmp+ '.FITS'
  fitsfile = !CALSAVE + filetmp  
  
  res = FINDFILE(fitsfile, count=count)

  IF count NE 0 THEN newfile = 0 ELSE newfile = 1

  ;construct the grid in orbital position
  ; the orbit will be divided into 100 bits
  nbits = 100
  orbpos_dark = INDGEN(nbits)*0.01 + 1./nbits/2
  
  ; if newfile then define array dark_calg, if not read from fitsfile
  IF newfile EQ 1 THEN dark_calg = FLTARR(nbits, pixels,2) $
  ELSE BEGIN 
     FXREAD, fitsfile, dark_calg 
     SXHREAD, fitsfile, newhdr
  ENDELSE    
  
  
  ;interpolate the dark values from the tables
     xorb = darkorb_str.orbpos
  FOR ipix = 0, N_ELEMENTS(indpix)-1 DO BEGIN 
     ydark = REFORM(darkorb_str.sig(ipix,*))
     yint = INTERPOL(ydark, xorb, orbpos_dark) 
     edark = REFORM(darkorb_str.unc(ipix,*))
     eint = INTERPOL(edark, xorb, orbpos_dark) 
     dark_calg(*,indpix(ipix),0) = yint
     dark_calg(*,indpix(ipix),1) = eint
  ENDFOR   
  
; change pixel ordering according to PIPELINE convention
  IF det eq 'C2' THEN BEGIN 
      tmpdark = TRANSPOSE(dark_calg,[1,0,2])
      c2p_ia2pipe, tmpdark, transdark
      dark_calg = TRANSPOSE(transdark,[1,0,2])
  ENDIF    
  
  ; if file does not exist then create hdr
  IF newfile THEN BEGIN 
      print,' Creating new header '
      mkhdr, newhdr, dark_calg
      fxaddpar, newhdr, 'ORIGIN', 'ESA', 'European Space Agency'
      fxaddpar, newhdr, 'TELESCOP', 'ISO', 'Infrared Space Observatory'
      fxaddpar, newhdr, 'INSTRUME', 'PHT', 'Instrument used'
      fxaddpar, newhdr, 'FILENAME', pfilename,'File name assigned in ISO archive'
      fxaddpar, newhdr, 'FILEVERS', '', 'Version ID assigned in ISO archive'        
  ENDIF 

      fxaddpar, newhdr, 'DATE', STRMID(!STIME,0,11), 'Date of creating file 
      fxaddpar, newhdr, 'COMMENT', comment
      fxaddpar, newhdr, 'PIA_VERS',!PIA_VERSION, $ 
         'Internal (user) PIA-CAL Version'
      
  ; write FITS file  
      IF newfile EQ 0 THEN $
         IF NOT KEYWORD_SET(OVERWRITE) THEN fitsfile = fitsfile + '.NEW' $
             ELSE  spawn,'/bin/rm '+EXPAND_PATH(fitsfile)        
      writefits, fitsfile, dark_calg, newhdr  

END
