;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	CHECK_STEP
;
; PURPOSE:	
;	Function to check the chopper steps vs chopper positions 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	result = CHECK_STEP(chop, step, time, mode, cflag)
;
; INPUT PARAMETERS:
;	chop:	The chopper positions of the ramps (SRD level)
;	step:	The step positions of the ramps
;	time:	The times of the ramps
;	mode:	Structure containing:
;			cste(nr of steps), cmod(chopper mode),cinc(increment)
;			camp(amplitude), fcs(nr of active FCS)
;	cflag:	Chopper position flag (same dimension as chop)
; 
; KEYWORD INPUT:
;	admn:	The admn name for more precise message in case of suspected
;		time dropout
;
; OUTPUT:
;	result:	the step mode calculated according to the chopper positions 
;
; RESTRICTIONS:  
;
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel (ESA/VILSPA)		June 1997
;	V6.4
;	Use exact expected positions for each mode as
;	mask - Do not use bad flagged positions (CG+JA) September 1997
;	V6.5 
;	Avoiding hundreds of reassigning messages (CG)  June 1998
;	V7.2
;-
FUNCTION check_step, chop, step, time, cmode, cflag, admn=admn
	ON_ERROR, 2

	IF NOT KEYWORD_SET(admn) THEN name = 'No Name' ELSE $
	   decode_admn, admn, name

	step_corr = step
;	cflag = INTARR(N_ELEMENTS(step))
	
	nele = N_ELEMENTS(chop)
	IF N_ELEMENTS(step) NE nele THEN MESSAGE, /INFO, 'No correspondence'
	
	diff = chop(1:*) - chop(0:nele-2)
	
	nmod = WHERE(step NE step(0), nn)
	IF nn EQ 0 THEN RETURN, step_corr
	
; find all chopper jumps - If no jumps no problem -> return

	ind0 = WHERE(abs(diff) GT 0., ndif0)
	IF ndif0 EQ 0 THEN RETURN, step_corr


;Check offset

	jump_allowed = 1
	CASE 1 OF 
	 STRPOS(STRUPCASE(cmode.cmod),'TRIANGULAR') NE -1: BEGIN 
	       mode = 'TRIA'
	       ntstep = cmode.cste*2 + 1
	       chopjump = FLOAT(STRMID(cmode.cinc,0, $ 
	                   STRPOS(cmode.cinc,'arcsec')))
	       choptol = 0.1*chopjump            
	       chopzero =  - chopjump * cmode.cste           
	    END   
	 STRPOS(STRUPCASE(cmode.cmod),'SAWTOOTH') NE -1: BEGIN 
	       mode = 'SAWT'
	       ntstep = cmode.cste*2 + 1
	       chopjump = FLOAT(STRMID(cmode.cinc,0, $ 
	                   STRPOS(cmode.cinc,'arcsec')))
	       choptol = 0.1*chopjump            
	       chopzero =  - chopjump * cmode.cste           
	       jump_allowed = ntstep - 1
	    END   
	 STRPOS(STRUPCASE(cmode.cmod),'RECTANGULAR') NE -1: BEGIN
	       mode = 'RECT'
	       ntstep = 2
	       chopjump = 2*FLOAT(STRMID(cmode.camp,0, $ 
	                   STRPOS(cmode.camp,'arcsec')))
	       choptol = 0.1*chopjump            
	       chopzero =  - chopjump / 2.           
	    END   
	 STRPOS(STRUPCASE(cmode.cmod),'STARING') NE -1: BEGIN
	 	step_corr = step_corr * 0 + 1
	 	RETURN, step_corr
	    END
	 ELSE: BEGIN 
	       mode = 'CALRECT' 
	       indnf = WHERE(cmode.fcs EQ 0, ncount)
	       IF ncount GT 0 THEN BEGIN
	          step_corr = step_corr *0 + 2
	          step_corr(indnf) = 1
	       ENDIF ELSE BEGIN
	          indf2 = WHERE(cmode.fcs EQ 2, nc)
	          IF nc EQ 0 THEN MESSAGE, /info, 'Something is wrong, no FCS2'	                   
		  step_corr = step_corr * 0 + 2
		  step_corr(indf2) = 1
	       ENDELSE
	       RETURN, STEP_corr 
	    END   
        ENDCASE

	indgood = -1
	indOK = WHERE(cflag EQ 0, nOK)
	IF nOK EQ 0 THEN BEGIN
	  PRINT, 'nothing to be checked, all chopper flags bad'
	  RETURN, STEP_CORR
	ENDIF
	
	FOR istep=0,ntstep-1 DO BEGIN
	   choppos = chopzero + istep*chopjump 
	   indices = WHERE(ABS(chop(indok)-choppos) LT choptol, count)
	   IF count GT 0 THEN BEGIN
	     step_corr(indok(indices)) = istep+1
	     indgood = [indgood,indok(indices)]
	   ENDIF
	ENDFOR
; sort the defined points sequentially
	IF N_ELEMENTS(indgood) EQ 1 THEN MESSAGE, /info, $
	   'Step assignment impossible - Chopper positions unexpected, returning'
	indgood = indgood(1:*)
	res = bsort(indgood)
	indgood = indgood(res)
	
	indbad = WHERE(step_corr(indok) EQ 0, badcount)
	IF badcount GT 0 THEN indbad = indok(indbad)

; This should give always the time difference between signals from same
; chopper plateau
	med_tdif = median(time(1:*)-time(0:N_ELEMENTS(time)-2))
		
; Loop over not assigned steps because chopper position out of tolerated region

	times = FLTARR(2)

	FOR ibad=0,badcount-1 DO BEGIN

	 indg1 = WHERE(indgood GT indbad(ibad), nc)

	 IF nc GT 0 THEN BEGIN

	  st1 = step_corr(indgood(indg1(0)))
	  times(1) = time(indgood(indg1(0)))

	  IF indg1(0) NE 0 THEN BEGIN
	    st0 = step_corr(indgood(indg1(0)-1)) 
	    times(0) = time(indgood(indg1(0)-1))
	  ENDIF ELSE st0=st1

	  IF st0 EQ st1 THEN step_corr(indbad(ibad)) = st0 ELSE BEGIN
	     diff = ABS(times - time(indbad(ibad)))
	     jump = ABS(ST1 - ST0)

	     IF jump NE 1 or jump NE jump_allowed THEN cflag(indbad(ibad)) = 1 $
	     ELSE BEGIN	         
	        IF diff(0) GT med_tdif THEN $
	        	step_corr(indbad(ibad)) = st1 ELSE $
	        	step_corr(indbad(ibad)) = st0
	     ENDELSE

	  ENDELSE

	 ENDIF ELSE step_corr(indbad(ibad)) = step_corr(max(indgood)) 

	ENDFOR 	

; Look for steps with only one element between steps with same number
;	last_ele = N_ELEMENTS(step_corr)- 2
;        step_dif = step_corr(1:*) - step_corr(0:last_ele)
;	ind = WHERE(step_dif NE 0, ndif)
;
;	IF ndif GE 2 THEN BEGIN
;	 ind_dif = ind(1:*) - ind(0:ndif-2)
;	
;	 anomaly = WHERE(ind_dif EQ 1, nanom)
;	
;	 IF nanom GT 0 THEN BEGIN
;	   FOR iano = 0,nanom-1 DO BEGIN
;	          step_corr(ind(anomaly)+1) = step_corr(ind(anomaly)) 	
;	   ENDFOR
;	 ENDIF
;	ENDIF

	pseudo_step = step
	
	IF mode EQ 'TRIA' THEN BEGIN

	  FOR i=1,MAX(step)-ntstep DO BEGIN
	    ind=WHERE(pseudo_step EQ ntstep+i,n)
	    pseudo_step(ind) = ntstep - i
	  ENDFOR
	ENDIF
	
	indflag = WHERE(cflag NE 0, nflag)
	   
	IF nflag GT 0 THEN BEGIN
	    PRINT, STRTRIM(nflag,2) + $
	     ' signals without chopper assignment'
	ENDIF
	    
; do not warn by chopper signal bad flagged and by not assigned steps (=-1)
	gflag = WHERE(cflag EQ 0, fl)
	inddif = WHERE(step_corr(gflag) NE pseudo_step(gflag), ndif)
	  IF ndif NE 0 THEN BEGIN
	    inddif_ass = WHERE(pseudo_step(gflag(inddif)) NE -1, ndif_ass)
	    IF ndif_ass GT 0 THEN BEGIN
	     PRINT, 'Chopper step number vs chopper position check shows problems'
	     IF N_ELEMENTS(inddif) GT 20 THEN indred = inddif(0:19) ELSE $
	        indred = inddif
	     PRINT, 'Reassigning chopper steps starting at step: ' + $
	     STRTRIM(gflag(indred)+1,2) + ' at time:' + $
	     STRTrIM(TIME(gflag(indred)),2)
	     IF N_ELEMENTS(inddif) GT 20 THEN $
	        PRINT, ' ... and same for ' + STRTRIM(N_ELEMENTS(inddif),2) +$
		                            ' times'
	     PRINT, 'Measurement '+ name + ' has probably a dropout'
	    ENDIF
	    ystep = step_corr
	  ENDIF 	  	     	       

	
RETURN, step_corr
END
