;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	CHECK_UNC
;
; PURPOSE:
;	Checks an uncertainty array returning it setting positive values for
;	uncertainties equal 0.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	check_unc, values, unc
;
; INPUT:
;	values:  Values array
;	unc:	 Uncertainties array (same dimension as values)
;
; KEYWORD PARAMETERS:
;	Verbose: For printing information
;	Abs_min: Absolute minimum accepted for a distribution
;	
; OUTPUT:
;	(none)
;	
; SIDE EFFECTS:
;	Change the unc values, which were 0. by entry
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC) + C. Gabriel(ESA/SAI)	 September 1996
;	Version 5.0 - Sept.96
;	Abs_min Keyword added (CG)				 December 1997
;	V6.6
;	Assign stdev/SQRT(NPTS) as uncertainty in case all 
;	unc are zero (JAc) 					August 1998
;			
;-

PRO check_unc, val, unc, verbose=verbose, abs_min=abs_min

	IF not KEYWORD_SET(abs_min) THEN abs_min = 0.
	dims = size(val)
; 2 dimensional case
	IF dims(0) EQ 2 THEN BEGIN
	 val_or = val
	 unc_or = unc
	 FOR I=0,dims(1)-1 DO BEGIN
	   val = reform(val_or(i,*))
	   unc = reform(unc_or(i,*))

	   indval = WHERE(ABS(val) GT 0., nindval)


	   IF nindval GT 0 THEN BEGIN
	    indu = WHERE(unc(indval)/abs(val(indval)) LT .001, nu) 
	    IF abs_min GT 0. THEN $
	      indu1 = WHERE(unc LT abs_min, nu1) ELSE nu1 = 0
	   ENDIF ELSE BEGIN
	     nu=0 
	     nu1=0
	   ENDELSE
	   
	   IF nu GT 0 THEN unc(indval(indu)) = 0. 
	   IF nu1 GT 0 THEN unc(indu1) = 0. 

	  ind = WHERE(unc EQ 0., nind)
	  indinv = WHERE(unc GT 0., nindinv)

IF KEYWORD_SET(verbose) THEN $
print, 'Pixel ', STRTRIM(I+1,2), ' - ', STRTRIM(nind,2), ' vs ', $
  STRTRIM(nindinv,2)
  
	  IF nind GT 0 THEN BEGIN
	   ; if all uncertainties are 0 then assign STDEV of val
	   IF nind EQ N_ELEMENTS(unc) THEN                       $
	          unc = unc + stdev(val)/SQRT(N_ELEMENTS(val))   $
	     ELSE unc(ind) = 4. * median(unc(indinv))
	  		
	   IF KEYWORD_SET(VERBOSE) THEN $
	     PRINT, 'Pixel ', STRTRIM(I+1,2),'-Uncertainties added: ', $
	        STRTRIM(nind,2), $
	     	' out of ', STRTRIM(N_ELEMENTS(val),2)
	   unc_or(i,*) = unc
	  ENDIF
	 ENDFOR
	 unc = unc_or
	 val = val_or

	ENDIF ELSE BEGIN
; 1 dimensional case
	 
	 indval = WHERE(val GT 0., nindval)
	 IF nindval GT 0 THEN BEGIN
	    indu = WHERE(unc(indval)/val(indval) LT .001, nu) 
	    IF abs_min GT 0. THEN $
	      indu1 = WHERE(unc LT abs_min, nu1) ELSE nu1 = 0
	 ENDIF ELSE BEGIN
	     nu=0 
	     nu1=0
	 ENDELSE
	 IF nu GT 0 THEN unc(indval(indu)) = 0. 
	 IF nu1 GT 0 THEN unc(indu1) = 0. 

	 ind = WHERE(unc EQ 0., nind)
	 indinv = WHERE(unc GT 0., nindinv)
	 IF nind GT 1 THEN BEGIN
	
	   IF nind EQ N_ELEMENTS(unc) THEN                           $
	   		unc = unc + stdev(val)/SQRT(N_ELEMENTS(val)) $
	       ELSE unc(ind) = 4. * median([unc(indinv)])
	  		
	   IF KEYWORD_SET(VERBOSE) THEN $
	     PRINT, 'Uncertainties added: ', STRTRIM(nind,2), $
	     	' out of ', STRTRIM(N_ELEMENTS(val),2)
	 ENDIF ELSE IF nind EQ 1 THEN BEGIN
	   IF nind EQ N_ELEMENTS(unc) THEN $
	   		unc = val ELSE $
	  		unc(ind) = 4. * median([unc(indinv)])
	 ENDIF   
	ENDELSE
	
	RETURN
	END
