;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       CHOP_FREQ_CORR
;
; PURPOSE:
;       Perform the correction for signal losses in a chopped measurement to
;	the subtracted values
;       
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       CHOP_FREQ_CORR, tmpdata
;
; INPUT PARAMETERS:
;	tmpdata: Background subtracted measurement 
;        
; OUTPUT PARAMETERS:
;       
;
; MODIFICATION HISTORY:
;       Written by: J. Acosta / C. Gabriel		January 1998
;       Version 7.0
;	Problem with AAP case solved + systematic 
;	 unc. not propagated but into hdr   (CG)	March 1998
;	Bug correction dealing with the header (Jac)    April 1998
;	V7.1
;-
;
 
PRO CHOP_FREQ_CORR, tmpdata, SPD=SPD, AAP=AAP, CORRVAL=CORRVAL, $
	CORRUNC=CORRUNC, NEWHEADER=NEWHEADER, INFO= info

@phtchopcor

; Check that IT is a subtracted meas.

	IF not KEYWORD_SET(NEWHEADER) THEN newheader=tmpdata.hdr
	
	res = fxpar(newheader, 'PRC_SUBT', count=issub_scp)
	res = fxpar(newheader, 'PRD_SUBT', count=issub_spd)
	res = fxpar(newheader, 'PRA_SUBT', count=issub_aap)
	
	IF (issub_scp + issub_spd + issub_aap) EQ 0 THEN BEGIN
	   INFO = 'It is only possible to apply ' + $
	   'the signal losses correction in chopped meas. after background' + $ 
	   'subtraction, returning'
	   RETURN
	ENDIF    
	
	;Check if correction was already applied
	res = fxpar(newheader, 'PRC_CFCF', count=iscor_scp)
	res = fxpar(newheader, 'PRD_CFCF', count=iscor_spd)
	res = fxpar(newheader, 'PRA_CFCF', count=iscor_aap)

	IF (iscor_scp + iscor_spd + iscor_aap) GT 0 THEN BEGIN 
	  INFO ='Signal losses correction in chopped m. already applied, skipped'
	  RETURN
	ENDIF    
	 

;compute the time per chopper plateau: product of t_reset * Number of DRs 
	give_cs, tmpdata, cs, record, conv_cs
	intt = 2.^(7-cs.intt(record))
	ndr = FLOAT(conv_cs.ndr(record))
	tinchop = ROUND(intt*ndr)

;select the corresponding chopper frequency 
	val = CHOP_FRCORR_FCT(tmpdata.det, tinchop)
	size_val = size(val)
	IF size_val(0) EQ 1 THEN BEGIN
	 corrval = val(0)
	 corrunc = val(1)
	ENDIF ELSE BEGIN
	 corrval = val(*,0)
	 corrunc = val(*,1)
	ENDELSE
	
	IF corrval(0) EQ -1 THEN BEGIN 
		INFO = 'Signal losses correction in chopped m. not done, skipped'
	ENDIF

	npix = pixel_det(tmpdata.det)

	CASE 1 OF 
	  KEYWORD_SET(SPD): BEGIN 
		 tmpdata.mnpw(0:npix-1,*) = tmpdata.mnpw(0:npix-1,*) / corrval
	         tmpdata.mnpu(0:npix-1,*) = tmpdata.mnpu(0:npix-1,*) / corrval
;	         prev_unc = tmpdata.mnpu(0:npix-1,*) * 0.
;		 ind = where(tmpdata.mnpw ne 0., n)	         
;	         prev_unc(ind) = tmpdata.mnpu(ind) / tmpdata.mnpw(ind)
; including	 tmpdata.mnpu(0:npix-1,*) = tmpdata.mnpw(0:npix-1,*) * $
; propagation	 	SQRT(prev_unc^2+(corrunc/corrval)^2)
	         tmpdata.mdpw(0:npix-1,*) = tmpdata.mdpw(0:npix-1,*) / corrval
	         tmpdata.q1pw(0:npix-1,*) = tmpdata.q1pw(0:npix-1,*) / corrval
	         tmpdata.q3pw(0:npix-1,*) = tmpdata.q3pw(0:npix-1,*) / corrval
	         fxaddpar, newheader, 'PRD_CFCF', avg([corrval]), $
	 	    ' Signal loss by chopping - Corr. factors average', BEFORE='XTENSION' 
	         fxaddpar, newheader, 'PRD_CF_S', avg([corrunc/corrval]), $
	 	    ' Systematic rel. unc. - signal loss by chopping ', BEFORE='XTENSION' 
	  END
	  KEYWORD_SET(AAP): BEGIN 
	  	 corrunc_rel = corrunc/corrval
	         ; average values of flux 
	         tmpdata.mnfl(0:npix-1,*) = tmpdata.mnfl(0:npix-1,*) / corrval
	         tmpdata.mnfu(0:npix-1,*) = tmpdata.mnfu(0:npix-1,*) / corrval
;	         prev_unc = tmpdata.mnfu(0:npix-1,*) / tmpdata.mnfl(0:npix-1,*)
; propag	 tmpdata.mnfu(0:npix-1,*) = tmpdata.mnfl(0:npix-1,*) * $
;	 				SQRT(prev_unc^2+corrunc_rel^2)
	  ; average values of brightness
 	         tmpdata.mnbr(0:npix-1,*) = tmpdata.mnbr(0:npix-1,*) / corrval
 	         tmpdata.mnbu(0:npix-1,*) = tmpdata.mnbu(0:npix-1,*) / corrval
 	         
;	         prev_unc = tmpdata.mnbu(0:npix-1,*) / tmpdata.mnbr(0:npix-1,*)
; pr.	         tmpdata.mnbu(0:npix-1,*) = tmpdata.mnbr(0:npix-1,*) * $
;	 				SQRT(prev_unc^2+corrunc_rel^2)
	 	 ; median values of flux			
	         tmpdata.mdfl(0:npix-1,*) = tmpdata.mdfl(0:npix-1,*) / corrval
	         tmpdata.q1fl(0:npix-1,*) = tmpdata.q1fl(0:npix-1,*) / corrval
	         tmpdata.q3fl(0:npix-1,*) = tmpdata.q3fl(0:npix-1,*) / corrval
	 	 ; median values of brightness			
	         tmpdata.mdbr(0:npix-1,*) = tmpdata.mdbr(0:npix-1,*) / corrval
	         tmpdata.q1br(0:npix-1,*) = tmpdata.q1br(0:npix-1,*) / corrval
	         tmpdata.q3br(0:npix-1,*) = tmpdata.q3br(0:npix-1,*) / corrval
	         fxaddpar, newheader, 'PRA_CFCF', avg([corrval]), $
	 	  ' Signal loss by chopping - Corr. factors average', BEFORE='XTENSION' 
	         fxaddpar, newheader, 'PRA_CF_S', avg([corrunc_rel]), $
	 	  ' Systematic rel. unc. - signal loss by chopping ', BEFORE='XTENSION' 
	  END
	  ELSE: BEGIN
	         tmpdata.aver(0:npix-1,*) = tmpdata.aver(0:npix-1,*) / corrval
	         tmpdata.avun(0:npix-1,*) = tmpdata.avun(0:npix-1,*) / corrval

;	         prev_unc = tmpdata.avun(0:npix-1,*) / tmpdata.aver(0:npix-1,*)
; prop.	         tmpdata.avun(0:npix-1,*) = tmpdata.aver(0:npix-1,*) * $
;	 				SQRT(prev_unc^2+(corrunc/corrval)^2)
	         tmpdata.medi(0:npix-1,*) = tmpdata.medi(0:npix-1,*) / corrval
	         tmpdata.q1me(0:npix-1,*) = tmpdata.q1me(0:npix-1,*) / corrval
	         tmpdata.q3me(0:npix-1,*) = tmpdata.q3me(0:npix-1,*) / corrval
	         fxaddpar, newheader, 'PRC_CFCF', avg([corrval]), $
	 	  ' Signal loss by chopping - Corr. factors average', BEFORE='XTENSION' 
	         fxaddpar, newheader, 'PRC_CF_S', avg([corrunc/corrval]), $
	 	  ' Systematic rel. unc. - signal loss by chopping ', BEFORE='XTENSION' 
	  END
	ENDCASE

RETURN
END
