;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO Chop_Info_Init_spd, Name, Verbose = V, phtspdt = phtspdt, $
                                csunit = csun
;+
; NAME:
;       CHOP_INFO_INIT_SPD
;
; PURPOSE:
;       This routine reads the SPD and CS to initialise the chop_info 
;	common block
;
; CATEGORY:
;	PIA - initialisation
;
; CALLING SEQUENCE:
;       CHOP_INFO_INIT_SPD, Name
;
; INPUTS:
;       Name: SPD admname as in the PHTSPD structure
;
; OPTIONAL INPUTS:
;       Verbose: If set, print the parameters as they are read in.
;       phtspdt: temporary phtspd structure
;       csunit: Position of the corresponding Compact Status record
;
; OUTPUTS:
;       None
;
; COMMON BLOCKS:
;	adm
;	phtcs
;	phtspd
;	chop_info
;
; PROCEDURES USED:
;       PIA LIB: read_cs
;
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel (adopted from CHOP_INFO_INIT)
;	Oct 95 Version 3.5
;
;-


@adm
@phtcs
@phtspd
@chop_info

        ON_ERROR, 2                    ; Return to calling program

        IF (NOT KEYWORD_SET (V)) THEN V=0
        IF (KEYWORD_SET(phtspdt)) THEN spdt = 1 ELSE spdt = 0
        IF (NOT KEYWORD_SET(csunit)) THEN csunit = 0


; Set the chopper transition times
        Trans_Time_Meas = 224
        Trans_Time_Cal = 832

; Extract the part of the name which gives the subinstrument
        SubInst = STRMID(Name,0,2)


; If structure not passed search for it

        IF spdt EQ 0 THEN BEGIN
          Index = WHERE(phtspd.admn eq Name, n)
          Index = Index(0)

          IF n EQ 0 THEN BEGIN
            MESSAGE, /INFO, 'Measurement not found'
            RETURN
          ENDIF

          phtspdt = phtspd(Index)
        ENDIF

; Get the TDT from the filename (5th to 12th characters)
        TDT = STRMID(Name, 4, 8)

; Extract the raw chopper position
        HANDLE_VALUE, phtspdt.Cpos, Raw_Chop

; Extract the time key
        HANDLE_VALUE, phtspdt.TKey, Current_Time

; Get the CS unit. If necessary, read the CS file
        IF CSUnit eq 0 THEN BEGIN
; - change -> shape4
           CSName = 'PSTA'+TDT+'.FITS'
           read_cs, CSName, cshdr, CSUnit
           Admind = WHERE(adm.name eq Name)
           Admind = Admind(0)
           adm(admind).phtcs = CSUnit
           IF CSUnit le 0 THEN BEGIN
                MESSAGE, /INFO, 'CS file not found'
                RETURN
           ENDIF
        ENDIF


; Access the structure pointed to from the unit
        HANDLE_VALUE, phtcs(CSUnit).handle, cs

; Find the CS Index of the required measurement by comparing start times
        CSIndex = WHERE(cs.ikst eq current_time(0))

; Convert CSIndex from a one dimensional array to a scalar.
        CSIndex = CSIndex(0)

; At this point, the required variables in the compact status common block
; can be accessed with: CS.xxxx(CSIndex)


; Get the number of chopper steps
	Steps = CS.cste(CSIndex)

; Get the chopper increment
	Increment = CS.cinc(CSIndex)

; Get the data reduction
	Data_Red = CS.DRS(CSIndex)

; Get the chopper mode and submode, from the raw mode
	Mode = (CS.cmod(CSIndex) AND 12) / 4

        CASE Mode OF

          0:    BEGIN
	    SubMode = (CS.CMod(CSIndex) AND 2) / 2
	    Chop_Amp = Steps * Increment
	    IF (Submode eq 0) THEN BEGIN
	      Plat = 2 * Steps + 1 
	      Mode_type = 'Sawtooth'
	    ENDIF ELSE BEGIN 
	      Plat = 4 * Steps
	      Mode_type = 'Triangular'
	    ENDELSE
          END

          1:    BEGIN
	    SubMode = 0
	    Plat = 2
	    Chop_Amp = CS.camp(CSIndex)
	    Mode_type = 'Chopping'
	  END

	  2: BEGIN
	     SubMode = CS.CMod(CSIndex) AND 3
	     Plat = 2               ; To distinguish between FCS1 and FCS2
	     Chop_Amp = CS.camp(CSIndex)
	     Mode_type = 'Calibration'
	  END

	  3: BEGIN
	     SubMode = CS.CMod(CSIndex) AND 3
	     Plat = 1
	     Chop_Amp = CS.camp(CSIndex)
	     Mode_type = 'Staring'
	  END

        ENDCASE

; Number of destructive readouts
	Dest_Read = CS.NDR(CSIndex)
	Dest_Read = 2^Dest_Read

; Integration time
	Int_Time = CS.IntT(CSIndex)
	Int_Time = 2^(14-Int_Time)

;  Not strictly needed as Current_Time(0) = Start_Time 
	Start_Time = CS.IKSt(CSIndex)

; If verbosity is required then print out the details
	IF V THEN BEGIN

	  PRINT, '----'
	  PRINT, 'Chopper Mode:          ', Mode_type
	  PRINT, 'Destructive Read-Outs: ', Dest_Read
	  PRINT, 'Integration Time:      ', Int_Time
	  PRINT, 'Chopper Steps:         ', Steps
	  PRINT, 'Chopper Plateaux:      ', Plat
	  PRINT, 'Chopper Increment:     ', Increment
	  PRINT, 'Data Reduction:        ', Data_Red
	  PRINT, '----'
	
	ENDIF

	END
