;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO chop_levels, tmpdata, chplevl, ISPD = ispd                               
;+
; NAME:
;	CHOP_LEVELS
;
; PURPOSE:
;	This procedure returns averages and medians of chopper plateaux 
;	per chopper level
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	CHOP_LEVELS, Tmpdata, Chplevl 
;
; INPUTS:
;	Tmpdata: A structure containing data (SCP or SPD or AAP level)
;	
; KEYWORD PARAMETERS:
;	ISPD: (0:SCP; 1:SPD level; 2: AAP level)
; 
; OUTPUTS:
;	Chplevl: Structure containing
;	      	name: measurement name
;		level: 'SCP' or 'SPD' or 'AAP'
;		aver: array (pixel #, step #) of averages 
;		avun: array (pixel #, step #) of averages' uncertainties
;		medi: array (pixel #, step #) of medians
;		m1qu: array (pixel #, step #) of first quartiles
;		m3qu: array (pixel #, step #) of third quartiles
;		flag: flag array (pixel #, step #)
;
; PROCEDURES USED:
;	PIA LIB: cp_disc
;	         spd_disc
;	         weight_mean
;	
; MODIFICATION HISTORY:
;	Written by:	C.Gabriel
;	Oct 95 Version 3.5
;	Modified to include subtraction at AAP level	  J.Acosta (June 1999)
;	
;-

; set variables for  SCP (spd=0), SPD (spd=1) and AAP (spd=2) level
	IF NOT KEYWORD_SET(ispd) THEN ispd = 0

	CASE ispd OF   
	0: BEGIN    ; SCP case
	        level = 'SCP'
		step = tmpdata.step
		aver = tmpdata.aver
                avun = tmpdata.avun
		medi = tmpdata.medi
		m1qu = tmpdata.q1me
		m3qu = tmpdata.q3me
                ; flag
		cp_disc, tmpdata, flagarr 
	   END	
	1: BEGIN    ; SPD case
	        level = 'SPD'
		step = tmpdata.cstp
		aver = tmpdata.mnpw
                avun = tmpdata.mnpu
		medi = tmpdata.mdpw
		m1qu = tmpdata.q1pw
		m3qu = tmpdata.q3pw
                ; flag
		spd_disc, tmpdata, flagarr 
	   END
	2: BEGIN    ; AAP case
	        level = 'AAP'
		step = tmpdata.cstp
		aver = tmpdata.mnfl
                avun = tmpdata.mnfu
		medi = tmpdata.mdfl
		m1qu = tmpdata.q1fl
		m3qu = tmpdata.q3fl
                ; flag
		aap_disc, tmpdata, flagarr 
	   END
	ENDCASE       

	dim1 = N_ELEMENTS(flagarr(*,0))
	dim2 = max(step)


	aver = aver(0:dim1-1,*) 
        avun = avun(0:dim1-1,*) 
	medi = medi(0:dim1-1,*) 
	m1qu = m1qu(0:dim1-1,*) 
	m3qu = m3qu(0:dim1-1,*) 

	chpave = fltarr(dim1,dim2)
	chpavu = chpave
	chpmed = chpave
	chpm1q = chpave
	chpm3q = chpave
	chpflg = FIX(chpave) + 2

	
	FOR istep = 1, max(step) DO BEGIN
	   ind = WHERE(step EQ istep, n)

; if one chopper plateau on step, selection is not possible
	   IF n EQ 1 THEN BEGIN
		chpave(*,istep-1) = aver(*,ind) 
		chpavu(*,istep-1) = avun(*,ind)
		chpmed(*,istep-1) = medi(*,ind)
		chpm1q(*,istep-1) = m1qu(*,ind)
		chpm3q(*,istep-1) = m3qu(*,ind)
		chpflg(*,istep-1) = flagarr(*,ind) > 1
		mess = 'Warning -> just one chopper plateau on step '
		Message, /INFO, mess
	   ENDIF ELSE BEGIN

; if several chopper plateaus on step, look for flags
		FOR i=0,dim1-1 DO BEGIN
		   indacc = WHERE(flagarr(i,ind) EQ 0, n)
		   IF n GT 1 THEN BEGIN
			weight_mean, aver(i,ind(indacc)), $
				avun(i,ind(indacc)), yact, yacterr
			chpave(i,istep-1) = yact
			chpavu(i,istep-1) = yacterr
			chpmed(i,istep-1) = median(medi(i,ind(indacc)))
			mini = WHERE(medi(i,ind(indacc)) LT chpmed(i,istep-1))
			maxi = WHERE(medi(i,ind(indacc)) GE chpmed(i,istep-1))
			chpm1q(i,istep-1) = median(medi(i,mini(ind(indacc))))
			chpm3q(i,istep-1) = median(medi(i,maxi(ind(indacc))))
			chpflg(i,istep-1) = 0
		   ENDIF ELSE IF n EQ 1 THEN BEGIN
			chpave(i,istep-1) = aver(i,indacc) 
			chpavu(i,istep-1) = avun(i,indacc)
			chpmed(i,istep-1) = medi(i,indacc)
			chpm1q(i,istep-1) = m1qu(i,indacc)
			chpm3q(i,istep-1) = m3qu(i,indacc)
			chpflg(i,istep-1) = 1
			mess = 'Warning -> just one chopper plateaux accepted: '
			mess = mess+'Pixel'+string(i+1) 
			MESSAGE, /INFO, mess 
		  ENDIF ELSE BEGIN
			mess = 'Warning -> no chopper plateau accepted: '
			mess = mess+'Pixel'+string(i+1) 
			MESSAGE, /INFO, mess 
		  ENDELSE
        	ENDFOR
	   ENDELSE
        ENDFOR

; Fill structure
	chplevl = {NAME:tmpdata.admn, $
			level:level, $
			aver:chpave, $
			avun:chpavu, $
			medi:chpmed, $
			m1qu:chpm1q, $
			m3qu:chpm3q, $
			flag:chpflg }



RETURN
END
