;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Chop_Pos, Conv_Chop, Step_No, FCS_No
;+
; NAME:
;	CHOP_POS
;
; PURPOSE:
;	This procedure calculates the chopper postion from the telemetry data.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	CHOP_POS, Conv_Chop, Step_No, FCS_No
; 
; INPUTS:
;	None
;
; OUTPUTS:
;	Conv_Chop: Converted Chopper Position (Float Array)
;	  For calibration measurements,
;	  -180: FCS 1
;	     0: COFOV
;	   180: FCS 2
;	Step_No: Step number within the chopper cycle (Int Array)
;          From 1 - Plat
;	FCS_No: Which calibration source is being pointed at (Int array)
;	  -1: error
;	   0: not a calibration measurement
;	   1: FCS 1
;	   2: FCS 2
;
; COMMON BLOCKS:
;	chop_info
;	chop_param
;
; PROCEDURES USED:
;	PIA LIB: calc_step
;
; MODIFICATION HISTORY:
;	Written by:	Huw Morris
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;
;-

@chop_info
@chop_param

	ON_ERROR, 2           ; Return to calling procedure


;  Constants for calibration
	FCS1_Pos = 180.0
	CFOV_Pos = 0.0
	FCS2_Pos = -180.0


;  Calculate the input voltage from the TMDATA word
	In_Volt = 1000 * (Raw_Chop * Q/P - R) * C

;  Calculate the step number of every readout
	Calc_Step, Step_No

;  Get the number of raw chopper values
	Dim = N_Elements(Raw_Chop)

;  Use the chopper mode to decide how to convert the raw chopper position
	CASE Mode OF

	  0:  BEGIN               ; Sawtooth / Triangular modes

;  Flag as not being a calibration measurement
	    FCS_No = INTARR(Dim)

;  First approximation
	    Ang_Line = -(In_Volt + B) / A
	    Angle = INTERPOL(DU_DC, Ang_Dev, -Chop_Amp)
	    Angle = Angle(0)      ; Convert to a scalar
	    Conv_Chop = Ang_Line - (Angle / A)

;  Correct for hysteresis effects
	    IF (SubMode EQ 0) THEN BEGIN
;  Sawtooth mode

	      Angle = INTERPOL(DU_ACI, Norm_Ang, -ABS(Conv_Chop)/Chop_Amp)
	      Conv_Chop = Conv_Chop - Chop_Amp/(150*A) * Angle

	    ENDIF ELSE BEGIN
;  Triangular mode

;  Increasing profile
              Inc_profile = WHERE(Step_no le Plat/2, count)

	      IF count GT 0 THEN BEGIN

	       Angle = INTERPOL(DU_ACI, Norm_Ang, $
	         -ABS(Conv_Chop(Inc_Profile))/Chop_Amp)

	       Conv_Chop(Inc_Profile) = Conv_Chop(Inc_Profile) - $
	         Chop_Amp/(150*A) * Angle
              ENDIF
;  Decreasing profile
              Dec_profile = WHERE(Step_no gt Plat/2, count)

	      IF count GT 0 THEN BEGIN

	       Angle = INTERPOL(DU_ACD, Norm_Ang, $
	         -ABS(Conv_Chop(Dec_Profile))/Chop_Amp)

	       Conv_Chop(Dec_Profile) = Conv_Chop(Dec_Profile) - $
	         Chop_Amp/(150*A) * Angle
              ENDIF

	    ENDELSE
                          

          END

	  1: BEGIN
;  Chopper mode 1: Rectangular mode

;  Flag as not being a calibration measurement
	    FCS_No = INTARR(Dim)

;  First approximation
	    Ang_Line = -(In_Volt + B) / A
	    Angle = INTERPOL(DU_DC, Ang_Dev, -ABS(Ang_Line))
	    Conv_Chop = Ang_Line - (Angle / A)

;  Now correct for hysteresis effects

;  Calculate where the Chopper position is positive and where it is negative
	    NCP_Pos = WHERE(Conv_Chop GT 0, CPos)
	    NCP_Neg = WHERE(Conv_Chop LT 0, CNeg)

;  Positive chopper positions
	    IF (CPos GT 0) THEN BEGIN
	      Table_Size = N_Elements(DU_ACI) - 1
	      Conv_Chop(NCP_Pos) = Conv_Chop(NCP_Pos) - $
	         DU_ACI(Table_Size)*ABS(Conv_Chop(NCP_Pos)) / (150*A)
	    ENDIF

;  Negative chopper positions
	    IF (CNeg GT 0) THEN BEGIN
	      Conv_Chop(NCP_Neg) = Conv_Chop(NCP_Neg) - $
	         DU_ACD(0)*ABS(Conv_Chop(NCP_Neg)) / (150*A)
	    ENDIF

	  END

	  2: BEGIN
;  Chopper mode 2: Calibration

;  Initially, set the FCS to flag errors: nonerror readouts will
;  overwrite this value.
 	    FCS_No = INTARR(Dim) - 1         ; fill the array with -1s
	    Conv_Chop = FLTARR(Dim)          ; initialize

	    IF (SubMode EQ 0) THEN BEGIN
;  FCS1 - FCS2
;  Change to original routine (CG) - FCS2 = step 1, FCS1 = step 2 

	      FCS1 = WHERE(Step_No EQ 2, Count1)   ; FCS1 measurements
	      FCS2 = WHERE(Step_No EQ 1, Count2)   ; FCS2 measurements

;  Mark those readouts corresponding to the FCS1
	      IF (Count1 GT 0) THEN BEGIN
	        FCS_No(FCS1) = 1
	        Conv_Chop(FCS1) = FCS1_Pos
	      ENDIF

;  Mark those readouts corresponding to the FCS2
	      IF (Count2 GT 0) THEN BEGIN
	        FCS_No(FCS2) = 2
	        Conv_Chop(FCS2) = FCS2_Pos
	      ENDIF

	    ENDIF
	
	    IF (SubMode EQ 1) THEN BEGIN
;  FCS1 - COFOV

	      FCS1 = WHERE(Step_No EQ 1, Count1)   ; FCS1 measurements
	      CFOV = WHERE(Step_No EQ 2, CountC)   ; CFOV measurements
	 
;  Mark those readouts corresponding the FCS1
	      IF (Count1 GT 0) THEN BEGIN
	        Conv_Chop(FCS1) = FCS1_Pos
	        FCS_No(FCS1) = 1
	      ENDIF

;  Mark those readouts corresponding to the centre field of view
	      IF (CountC GT 0) THEN BEGIN
	        Conv_Chop(CFOV) = CFOV_Pos
	        FCS_No(CFOV) = 0
	      ENDIF

	    ENDIF

	    IF (SubMode EQ 2) THEN BEGIN
;  FCS2 - COFOV

	      FCS2 = WHERE(Step_No EQ 1, Count2)   ; FCS1 measurements
	      CFOV = WHERE(Step_No EQ 2, CountC)   ; CFOV measurements

;  Mark those readouts corresponding to the centre field of view
	      IF (CountC GT 0) THEN BEGIN
	        Conv_Chop(CFOV) = CFOV_Pos
	        FCS_No(CFOV) = 0
	      ENDIF

;  Mark those readouts corresponding to the FCS2
	      IF (Count2 GT 0) THEN BEGIN
	        Conv_Chop(FCS2) = FCS2_Pos
	        FCS_No(FCS2) = 2
	      ENDIF

	    ENDIF

	  END

	  3: BEGIN
;  Chopper mode 3: Staring mode

	    IF (SubMode EQ 0 OR SubMode EQ 1) THEN BEGIN
;  Modes 31n and 31p

; Flag as not being a calibration measurement
	      FCS_No = INTARR(Dim)

	      Ang_Line = -(In_Volt + B) / A
	      Angle = INTERPOL(DU_DC, Ang_Dev, ang_line)

  	      Conv_Chop = Ang_Line - (Angle / A)

	    ENDIF

;  Mode 32 using FCS 1
	    IF (Submode EQ 2) THEN BEGIN
	      Conv_Chop = FLTARR(Dim) + FCS1_Pos
	      FCS_No = INTARR(Dim) + 1
	    ENDIF

;  Mode 32 using FCS 2
	    IF (Submode EQ 3) THEN BEGIN
	      Conv_Chop = FLTARR(Dim) + FCS2_Pos
	      FCS_No = INTARR(Dim) + 2
	    ENDIF

          END

	ENDCASE

                   
	END                     

