;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        FUNCTION CONV_CRE, RAWDATA, DET, ORDR, CSUN
;+
; NAME:
;       CONV_CRE
;
; PURPOSE:
;       This function converts the CRE Digitized Numbers DNs 
;	(as from the telemetry) returning 
;       the Output Voltages (in Volts) using the informations already gained 
;	from the CRE Conversion Tables
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       Result = CONV_CRE(Rawdata, Det, Ordr, Csun)
;
; INPUTS:
;       Rawdata	: Array containing the Digitized Numbers from the CRE
;	Det	: Detector Identifier (two characters string)
;       Ordr	: Measurement Order Number within 1 detector subsystem 
;		  in the Compact Status
;       Csun	: Compact Status Unit 
;
; OUTPUTS:
;       Returns an array containing the CRE Voltages
;
; COMMON BLOCKS:
;	phtcs
;	phtdietrans
;
; PROCEDURE:
;	Use the common PHTDIETRANS to compute the conversion together 
;	with the selected Gain and Offset obtained from the CS
; 
; PROCEDURES USED:
;       None
;
; MODIFICATION HISTORY:
;       Written by:	C. Gabriel
;	Oct 95 Version 3.5
;	U_off bug fixed - CG		February 1997
;	V6.1
;
;-
;
@phtcs
@phtdietrans
        ON_ERROR, 2

;
;  Check
	HANDLE_VALUE, PHTCS(CSUN).HANDLE, PHTCST
;  Look which positions in the compact status correspond to the detector 
	subs = PHTCST.SUBS
	subs = conv_subs(subs)
	IF STRMID(det,0,1) EQ 'S' THEN detgen = 'SL' ELSE detgen = det 
	ind = WHERE(subs EQ detgen, N)
	measnr = ind(ordr) 
        X1 = PHTCST.MUX1
        X1 = X1(MEASNR)
        X2 = PHTCST.MUX2
        X2 = X2(MEASNR)

;
;  Now look the cross status and compute mux, pos and di according to 
;  DIE1 or DIE2 used 
;  In order to use a similar array structure for the CRE conv tables as in the
;  Derive_SPD we turn the POS in the telemetry word PSTAD#GA:
;  (0=Gain40,1=Gain8,2=Gain1) to (2,1,0) 
;
        CROSS = PHTCST.XSTA   
        CROSS = CROSS(MEASNR)  
;  For MUX1 (or MUX2 in case of CROSS=1)
	MUX1 = CROSS         	        	; 0 for MUX1
	POS1 = PHTCST.D1GA 
	POS1 = 2 - POS1(MEASNR) 
	DI1 = PHTCST.D1OF
	DI1 = DI1(MEASNR)
;  For MUX2 (or MUX1 in case of CROSS=1)
        MUX2 = 1 - CROSS   		        ; 1 for MUX2
	POS2 = PHTCST.D2GA 
	POS2 = 2 - POS2(MEASNR) 
	DI2 = PHTCST.D2OF
	DI2 = DI2(MEASNR)

;  Handle the different detectors separately
	CASE DET of
	  'P1': LABEL = 1
	  'P2': LABEL = 1
          'P3': LABEL = 2
	  'C1': LABEL = 2
	  'C2': LABEL = 1
	  'S1': LABEL = 1
	  'S2': LABEL = 2
	  ELSE: MESSAGE, /INFO, 'Not recognized DET keyword'
	ENDCASE
	CASE LABEL OF
	  1: BEGIN 
;
;  Take the variables from PHTDIETRANS
;
        	GAIN1 = PDIEGN1(POS1,MUX1)
        	GAIN2 = PDIEGN2(POS1,MUX1)
        	D0 = PDIEFIX(POS1,MUX1)
        	UOFF = PDIEUOF(POS1,MUX1)    
		DI = DI1
;
;  Calculate M and C like in Derive_SPD and convert 
;
        	M_CONVCRE = -20. / (4096 * GAIN1)
        	C_CONVCRE = (D0 - GAIN2 * (DI - 2048) ) * 20 / $
				(4096 * GAIN1)  + UOFF
        	CONV = M_CONVCRE * RAWDATA + C_CONVCRE
	  END
	  2: BEGIN 
;
;  Take the variables from PHTDIETRANS
;
        	GAIN1 = PDIEGN1(POS2,MUX2)
        	GAIN2 = PDIEGN2(POS2,MUX2)
        	D0 = PDIEFIX(POS2,MUX2)
        	UOFF = PDIEUOF(POS2,MUX2)    
		DI = DI2
;
;  Calculate M and C like in Derive_SPD and convert 
;
        	M_CONVCRE = -20. / (4096 * GAIN1)
        	C_CONVCRE = (D0 - GAIN2 * (DI - 2048) ) * 20 / $
				(4096 * GAIN1)  + UOFF
        	CONV = M_CONVCRE * RAWDATA + C_CONVCRE
	  END
	  ELSE: MESSAGE, /INFO, 'Not recognized LABEL'
	ENDCASE 

        RETURN, CONV
        END
