;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
pro conv_cs,file,convcs

;+
; NAME:
;       CONV_CS
;
; PURPOSE:
;       This procedure converts extended housekeeping data from rawvalues 
;	to physical values.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       CONV_CS, File, Convcs 
;
; INPUTS:
;       File: PHTCS_TYPE Array containing rawvalues
;
; OUTPUTS:
;       Convcs: PHTCS_TYPE Array containing converted value
;
; PROCEDURE:
;       Convert numbers to text strings with the help of 
;       conversion arrays, interpreting the rawvalue as an
;       array index.
;       Convert all others with the formulas described in the
;       'Converting Algorithms For Telemetry Data'        
;
; PROCEDURES USED:
;       None
;
; MODIFICATION HISTORY:
;       Written by:	Andreas Karch
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;	New table values for change wheels (according to TR-PHT-0000 MP/25 
;	 from November 27, 1995) + Meas.time=LONG INT	(CG)	January 1996
;-

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  CONV_CS, FILE, CONVCS '

;initialize convcs as same struct as file
;copy all values
convcs=file

;get datas in struct old
handle_value,convcs.handle,old

;define tables for conversion

 trs_of_fcs=['no TRS','TRS 1','TRS 2']
 chopper_mode=['Scan sawtooth','Scan sawtooth','Scan triangular',$
		'Scan triangular',$
               'Rectangular','Rectangular','Rectangular','Rectangular',$
               'FCS1<->FCS2','COFOV<->FCS1','COFOV<->FCS2','NOTUSED    ',$
               'Staring negative Offset','Staring positive Offset',$
		'Staring FCS1','Staring FCS2']
 frequencies=[' 128 HZ','1024 Hz','4096 Hz','8192 Hz']
 line_of_mux1=['?','S1','?','C200','P1','P2']
 line_of_mux2=['?','S2','?','C100','P3','?','Ref.DIE1','Ref.DIE2']
 cross_status=['1/1,2/2','1/2,2/1']
 gain=[40,8,1]
 subsystems=['P1     ','P2     ','P3     ','SL  ','NOTUSED',$
             'NOTUSED','C100   ','C200   ']

 ;polarisers, apertures and filters on the wheels
 wheel1_position=['UNKNOWN     ','PHT_P_POL000','PHT_P_POL120','PHT_P_POL240',$
                  'PHT_P_NO_POL','PHT_S       ','PHT_C_RETICL','PHT_C_FREE1 ',$
                  'PHT_C_FREE2 ','PHT_C_FREE3 ','PHT_S_REDUND','PHT_C_POL000',$
                  'PHT_C_POL120','PHT_C_POL240','PHT_C_NO_POL','NOTUSED     ']
 wheel2_positi_s=['UNKNOWN     ','52_SEC      ','79_SEC      ','99_SEC      ',$
                  '120_SEC     ','127X127_SEC2','180_SEC     ','180_SEC_XBLK',$
                  '5_SEC       ','7P6_SEC     ','10_SEC      ','13P8_SEC    ',$
                  '18_SEC      ','20X32_SEC2  ','23_SEC      ','NOTUSED     ']
 wheel2_positi_c=['UNKNOWN     ','C100_OPEN   ','C100_90_RED ','C100_50_UM  ',$
                  'C100_60_UM  ','C100_70_UM  ','C100_100_UM ','C100_105_UM ',$
                  'C100_90_UM  ','C200_160_UM ','C200_200_UM ','C200_180_UM ',$
                  'C200_135_UM ','C200_120_UM ','C200_RETICLE','NOTUSED     ']
 wheel3_position=['UNKNOWN     ',$
                  'P1_3P29_UM  ','P1_3P6_UM   ',$
                  'P1_4P85_UM  ','P1_7P3_UM   ',$
                  'P1_7P7_UM   ','P1_10_UM    ',$
                  'P1_11P3_UM  ','P1_12P8_UM  ',$
                  'P1_16P0_UM  ','P1_11P5_UM  ',$
                  'P2_20_UM    ','P2_25_UM    ',$
                  'P3_60_UM    ','P3_100_UM   ',$
                  'NOTUSED     ']
 wheel2_position=[wheel2_positi_s,wheel2_positi_c]

 ;change wheel position converting parameters
 volt_pos1_fm=[[-3617,-3697],[-195,-269],[-2773,-2852],[465,385],[-2284,-2359],$
              [660,575],[-1845,-1925],[1495,1405],[-1513,-1602],[2784,2699],$
              [-1210,-1319],[-863,-968],[-4828,-4913],[-600,-675]]
 volt_pos2_fm=[[-717,-831],[2960,2851],[-414,-538],[4254,4140],[-219,-313],$
              [181,87],[-3583,-3697],[504,365],[-2714,-2808],[855,727],$
              [-1650,-1754],[1397,1288],[-1152,-1256],[2076,1967]]
 volt_pos3_fm=[[225,131],[-3896,-3985],[523,444],[-2499,-2594],[982,902],$
              [-1840,-1930],[1417,1337],[-1220,-1309],[2403,2319],[-805,-904],$
              [3912,3818],[-448,-553],[-58,-162],[-5419,-5504]]

 ;voltage converting parameters for all three wheels
 c_volt=1.0
 p_volt=4096.
 q_volt=20.
 r_volt=10.

 cre_status=['','CRE1 ','CRE2 ']
 detector_assy=['','P1  ','P2  ','P3  ','S1  ',$
                   'S2  ','C100','C200']
 
;convert those, which must be converted
;copy all the others

;UKST

UKST =old.UKST

;UKEN

UKEN =old.UKEN

;IKST

IKST =old.IKST

;IKEN

IKEN =old.IKEN

;UTST

UTST =old.UTST

;UTEN

UTEN =old.UTEN

;AOTN

AOTN =old.AOTN

;SUBS

SUBS=subsystems(old.subs)

;OPFO

OPFO =strcompress(old.OPFO,/remove_all)

;F1TS

F1TS =trs_of_fcs(old.F1TS)

;F2TS

F2TS =trs_of_fcs(old.F2TS)

;F1PS

F1PS =strcompress(string(float(old.F1PS)*50/4095),/remove_all)+' mW'

;F2PS

F2PS =strcompress(string(float(old.F2PS)*50/4095),/remove_all)+' mW'

;C1PS

C1PS =wheel1_position(old.C1PS)

;C2PS

C2PS =wheel2_position(old.C2PS+16*((old.subs eq 6) or (old.subs eq 7)))

;C3PS

C3PS =wheel3_position(old.C3PS)

;CMOD

CMOD =chopper_mode(old.CMOD)

;CAMP

CAMP =strcompress(old.CAMP,/remove_all)+' arcsec'

;CSTE

CSTE =strcompress(old.CSTE,/remove_all)

;CINC

CINC =strcompress(old.CINC,/remove_all)+' arcsec'

;CRES

CRES=cre_status(1*((old.cres and 1) gt 0))+$
     cre_status(2*((old.cres and 2) gt 0))

;DETA

DETA =detector_assy(1*((old.deta and 1) gt 0))+$
      detector_assy(2*((old.deta and 2) gt 0))+$
      detector_assy(3*((old.deta and 4) gt 0))+$
      detector_assy(4*((old.deta and 8) gt 0))+$
      detector_assy(5*((old.deta and 16) gt 0))+$
      detector_assy(6*((old.deta and 64) gt 0))+$
      detector_assy(7*((old.deta and 128) gt 0))

;DRS

DRS=strcompress(old.drs,/remove_all)

;MUX1

MUX1 =line_of_mux1(old.MUX1)

;MUX2

MUX2 =line_of_mux2(old.MUX2)

;XSTA

XSTA ='MUX/Diff.Amp: '+cross_status(old.XSTA)

;D1OF

D1OF =strcompress(-float(old.D1OF)*10/4095+5,/remove_all)+' V'

;D1GA

D1GA =strcompress(gain(old.D1GA),/remove_all)

;D2OF

D2OF=strcompress(-float(old.D2OF)*10/4095+5,/remove_all)+' V'

;D2GA

D2GA =strcompress(gain(old.D2GA),/remove_all)

;FREQ

FREQ =frequencies(old.FREQ)

;NNDR

NNDR =strcompress(2^old.NNDR-1,/remove_all)

;NDR

NDR=strcompress(2^old.NDR,/remove_all)

;INTT

INTT=strarr(n_elements(old.INTT))
for i=0,n_elements(old.INTT)-1 do begin
  t=7-old.INTT(i)
  if t lt 0 then begin
    INTT(i) = '1/'+strcompress(string(2^(-t)),/remove_all)+' s'
  endif else begin
    INTT(i) = strcompress(string(2^t),/remove_all)+' s'
  endelse
endfor

;MEAT

MEAT =strcompress(2L^old.MEAT,/remove_all)+' s'

;MPC1

help=intarr(file.dims)
mpc1=strarr(file.dims)
adc_input=old.MPC1*q_volt/p_volt-r_volt
input_voltage=adc_input*c_volt*1000
for i=0,file.dims-1 do begin
 help(i)=where(volt_pos1_fm(0,*) ge input_voltage(i) $
               and input_voltage(i) ge volt_pos1_fm(1,*))+1
 mpc1(i)=wheel1_position(help(i))
endfor

;MPC2

mpc2=strarr(file.dims)
adc_input=old.MPC2*q_volt/p_volt-r_volt
input_voltage=adc_input*c_volt*1000
for i=0,file.dims-1 do begin
 help(i)=where(volt_pos2_fm(0,*) ge input_voltage(i) $
               and input_voltage(i) ge volt_pos2_fm(1,*))+1
 mpc2(i)=wheel2_position(help(i)+16*((old.subs(i) eq 6) $
                                     or (old.subs(i) eq 7)))
endfor

;MPC3

mpc3=strarr(file.dims)
adc_input=old.MPC3*q_volt/p_volt-r_volt
input_voltage=adc_input*c_volt*1000
for i=0,file.dims-1 do begin
 help(i)=where(volt_pos3_fm(0,*) ge input_voltage(i) $
               and input_voltage(i) ge volt_pos3_fm(1,*))+1
 mpc3(i)=wheel3_position(help(i))
endfor

;MET

Resistance=(float(old.MET)-2048.)/(4096.-float(old.MET))*7500.
MET=strcompress(3442./(3.9489+alog(Resistance)),/remove_all)+' K'

;initialize similar struct for converted values )
                new = { $
           UKST: [UKST], $
           UKEN: [UKEN], $
           IKST: [IKST], $
           IKEN: [IKEN], $
           UTST: [UTST], $
           UTEN: [UTEN], $
           AOTN: [AOTN], $
           SUBS: [SUBS], $
           OPFO: [OPFO], $
           F1TS: [F1TS], $
           F2TS: [F2TS], $
           F1PS: [F1PS], $
           F2PS: [F2PS], $
           C1PS: [C1PS], $
           C2PS: [C2PS], $
           C3PS: [C3PS], $
           CMOD: [CMOD], $
           CAMP: [CAMP], $
           CSTE: [CSTE], $
           CINC: [CINC], $
           CRES: [CRES], $
           DETA: [DETA], $
           DRS: [DRS], $
           MUX1: [MUX1], $
           MUX2: [MUX2], $
           XSTA: [XSTA], $
           D1OF: [D1OF], $
           D1GA: [D1GA], $
           D2OF: [D2OF], $
           D2GA: [D2GA], $
           FREQ: [FREQ], $
           NNDR: [NNDR], $
           NDR: [NDR], $
           INTT: [INTT], $
           MEAT: [MEAT], $
           MPC1: [MPC1], $
           MPC2: [MPC2], $
           MPC3: [MPC3], $
           MET: [MET] }

;insert struct in convcs
convcs.handle=handle_create(value=new)

end
