;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	DARKCUR_ORB
;
; PURPOSE:
;	Performs dark current subtraction on a measurement at SRD or SCP levels.
;
; CATEGORY:
;	PIA - algorithms
;
; CALLING SEQUENCE:
;	DARKCUR_ORB, tmpdata, DARK = dark, SCP = scp, NEWHEADER = newheader 
;	
;INPUTS
;
; KEYWORD PARAMETERS:
;	DARK:		The dark current value subtracted
;       SCP:    	If selected tmpdata is expected to be a SCP temporary 
;			structure; if not it will be SRD
;	NEWHEADER: Header of the measurement where info will be added. 
;	PRO_RASTER:	Perform subtraction per raster point (overrides
;			    PRO_MEAS and PRO_AOT)
;	PRO_MEAS:	Perform subtraction per whole measurement (overrides 
;			    PRO_AOT) 
;	PRO_AOT:	Perform subtraction per AOT (default choice)
;			   
;
; COMMON BLOCKS:
;	phtdarkorb
;
; MODIFICATION HISTORY:
;	Written by Jose ACOSTA (IAC/PIDT) and 
;		   Carlos GABRIEL (ESA/VILSPA)			January 1998
;	Bug correction for rasters	(JAc)			February 1998		
;	V7.0
;	No use of syst. unc. but -> header (CG)			March 1998
;	V7.1
;	Bug for extreme ITK cases fixed (CG)			May 1998
;	V7.2
;	Included Exposure Time dependency for SL dark current  
;	  and better handling of KEYWORDS PRO_* (JAc)		August 1998
;-
PRO darkcur_orb, tmpdata,  DARK = dark, SCP=scp, NEWHEADER = newheader, $ 
    ERROR=error, PRO_RASTER=pro_raster, PRO_MEAS=pro_meas, PRO_AOT=pro_aot, $
    verbose=verbose, TIMEDEP = timedep

@phtdarkorb

error=""
IF N_ELEMENTS(newheader) EQ 0 THEN newheader = tmpdata.hdr

dark_done = fxpar(newheader,'PRS_DARK',count= count)

IF count GT 0 THEN BEGIN
  error = 'Dark Current subtraction already performed, skipped'
  message, error,/continue  
  RETURN
ENDIF   


;check if dark current cal-G files have been read 
IF N_ELEMENTS(C1DARKORB) EQ 0 THEN read_darkorb

IF N_ELEMENTS(TIMEDEP) EQ 0 THEN timedep = DARKTDEP
; if mode is not selected via keyword then use current default values
; as stored in common phtdarkorb
IF N_ELEMENTS(PRO_MEAS) EQ 0 AND N_ELEMENTS(PRO_RASTER) EQ 0 AND $ 
   N_ELEMENTS(PRO_AOT) EQ 0 THEN BEGIN 
     pro_meas = DARKPROM 
     pro_raster = DARKPRORP
     pro_aot = DARKPROAOT
ENDIF     

; Obtain the seconds from begin of observation by subtracting from the ITKs
; the START ITK of the first measurement and multiplying with the conversion 
; factor - After this normalize to orbital position using the orbit offset 
; portion at the begin 
GIVE_CS, tmpdata, cst, record, cst_conv
fct = FXPAR(tmpdata.hdr, 'TREFITKU', count=count)
IF count EQ 0 THEN BEGIN
    PRINT, 'Warning:  TREFITKU not found on header'
    fct = 6.1035156e-05
ENDIF


;determine orbital position offset at reference time from header if available 
; or alternatively using info from ALLPERIGEE file
;First read the revolution number from ADMN (needed in any case)
  rev = STRMID(tmpdata.admn,4,3)

; determine the orbital position (check the keyword TREFPHA1 in the header)
IF PRO_AOT THEN orb_off = FXPAR(newheader, 'TREFPHA2', count=count) $
 ELSE orb_off = FXPAR(newheader, 'TREFPHA1', count=count)

IF count EQ 0 THEN BEGIN 
    orb_off = set_orbpos(rev, tmpdata.unit)
    isnot_olpv7 = 1 
  IF KEYWORD_SET(VERBOSE) THEN $
      print,'Computing orbital position from ALLPERIGEE internal file'
ENDIF ELSE BEGIN 
  orb_off = orb_off - rev
  isnot_olpv7 = 0
  IF KEYWORD_SET(VERBOSE) THEN print,'Orbital position read from header '
ENDELSE  


;now compute the orbital position for any time depending on the mode
; 
   timall = tmpdata.tkey 	
   times = timall

   CASE 1 OF 
     PRO_RASTER: BEGIN 
        dorb_mode = 'PER_RASTER'
        get_rastersteps, tmpdata.rpid, raststeps
        FOR irast = 1,max(raststeps) DO BEGIN
          ind= WHERE(raststeps EQ irast)
          times(ind) = AVG([timall(ind)]) - cst.ikst(0)
        ENDFOR
     END   
     PRO_MEAS: BEGIN 
        dorb_mode = 'PER_MEASUREMENT'
        times = times*0.+AVG([timall]) - cst.ikst(0) 
     END
     PRO_AOT: BEGIN
        time_mid = (cst.ikst(0)*1.+ cst.ikst(N_ELEMENTS(cst.ikst)-1)*1.)/2. 
        times = times*0. + (time_mid - cst.ikst(record)) * isnot_olpv7
        dorb_mode = 'PER_AOT'
     END
     ELSE: BEGIN 
        IF KEYWORD_SET(scp) THEN dorb_mode = 'PER_CHOPPERPL.' ELSE $
        dorb_mode = 'PER_SIGNAL'
        times = times - cst.ikst(0)
     END
   ENDCASE  
     


;IF PRO_AOT THEN orbpos=orb_off  $
; ELSE BEGIN
    times = times * fct
    iso_orbit = 23.+56./60    ; duration of ISO orbit in hours  
    orbpos = orb_off + times / 3600. / iso_orbit 
;ENDELSE
;	print,orbpos

;determine detector and number of pixels
det = tmpdata.det
npix = pixel_det(tmpdata.det)
IF KEYWORD_SET(SCP) THEN $ 
darkc = FLTARR(npix, tmpdata.dims) ELSE $
darkc = FLTARR(npix, tmpdata.sldi)

CASE det of 
     'P1': dk = ppdarkorb(*,0,*)
     'P2': dk = ppdarkorb(*,1,*)
     'P3': dk = ppdarkorb(*,2,*)
     'C1': dk = c1darkorb
     'C2': dk = c2darkorb
     'SL': dk = sldarkorb
     ELSE: RETURN
ENDCASE

darku = darkc
FOR ipix=0, npix-1 DO BEGIN
    darkc(ipix,*) = interpol(dk(*,ipix,0),ORBPOS_DARK,orbpos)
    darku(ipix,*) = interpol(dk(*,ipix,1),ORBPOS_DARK,orbpos)
ENDFOR

IF KEYWORD_SET(scp) THEN BEGIN
  tmpdata.aver(0:npix-1,*) = tmpdata.aver(0:npix-1,*) - darkc
  ;;tmpdata.avun(0:npix-1,*) = SQRT(tmpdata.avun(0:npix-1,*)^2 + darku^2)
  rel_unc = avg([darku/tmpdata.aver(0:npix-1,*)])
  tmpdata.medi(0:npix-1,*) = tmpdata.medi(0:npix-1,*) - darkc
  tmpdata.q1me(0:npix-1,*) = tmpdata.q1me(0:npix-1,*) - darkc
  tmpdata.q3me(0:npix-1,*) = tmpdata.q3me(0:npix-1,*) - darkc
ENDIF ELSE BEGIN
  ;correct for time dependency if detector is SL
  ; determine observing time, it may not work for raster maps 
  IF det EQ 'SL' AND TIMEDEP THEN BEGIN
    exptime = FIX(GETTOK(cst_conv.meat(record),' '))
    darkcur_time, det, darkc, darkc, exptime
  ENDIF 
  tmpdata.slop(0:npix-1,*) = tmpdata.slop(0:npix-1,*) - darkc
  ;tmpdata.slun(0:npix-1,*) = SQRT(tmpdata.slun(0:npix-1,*)^2 + darku^2)
  rel_unc = avg([darku/tmpdata.slop(0:npix-1,*)])    
ENDELSE

;write output dark current into keyword dark
dark = darkc

;write info into the header
fxaddpar, newheader, 'PRS_DARK',1,' Dark Current subtracted (orbital dep)', $
       BEFORE='XTENSION'
fxaddpar, newheader, 'PRS_DC_S',rel_unc,' <Rel.Syst.Unc.> from Dark Curr. subtr.', $
       BEFORE='XTENSION'
fxaddpar, newheader, 'PRS_DMOD',dorb_mode,' DC orbital dep. mode used', $
       BEFORE='XTENSION'


RETURN
END
 
