;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO DECODE_PIXF, PIXF, SUSPECT, CHOPON, CHOPS, RDSTAT, OTF, AUTDR
;+
; NAME:
;       DECODE_PIXF
;
; PURPOSE:
;       Thie procedure extracts the information in the word --PIXF
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       DECODE_PIXF, Pixf
;
; INPUTS:
;       Pixf - A word in the ERD files containing information in 5 bits
;	ERD PIXF is defined in the following manner (see IDPD): 
;	  1 bit - chopper on position (most significant bit)
;	  1 bit - chopper sign
;	  1 bit - readout status
;	  1 bit - on-target flag
;	  1 bit - automatic data reduction flag
;	  10 bits - spare     ;;;;;;   don't ask me why?
;	  1 bit - suspected data corruption flag (least significant bit)
;	  
;
; OUTPUTS:
;	None
;
; OPTIONAL OUTPUTS:
;       Suspect:	Suspicious 		0:no	1:yes	-1:bad
;       Chopon: 	Chopper on-position	0:no	1:yes	-1:bad       
;       Chops: 		Chopper sign		0:-	1:+	-1:bad
;       Rdstat:		Read-out status 	0:NDR	1:DR    -1:bad
;       Otf:		On-target flag 		0:no	1:yes   -1:bad
;       Autdr:		Auto data reduction 	0:no	1:yes   -1:bad
;
; PROCEDURES USED:
;       None
;
; METHOD:
;	Swap first to get all bits together and count nicely afterwards 
;
; MODIFICATION HISTORY:
;       Written by:	Carlos Gabriel
;	Oct 95 Version 3.5
;-
 On_ERROR,2                                          ;Return to caller
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE,    $
'Syntax:  DECODE_PIXF, PIXF, [SUSP],[CHPON], [CHOPS], [RDSTAT], [OTF], [AUTDR]'
;
; Swap first the bytes in order to simplify things
;
	X = SWAP_ENDIAN(PIXF)
;
; Define the arrays according to the number of dimensions of pixf
;
	S = SIZE(X)
	NX = S(1)
	IF S(0) EQ 2 THEN BEGIN 
 		NY = S(2)
        	SUSPECT = INTARR(NX,NY)
	ENDIF ELSE IF S(0) EQ 3 THEN BEGIN
		NY = S(2)
		NZ = S(3)
		SUSPECT = INTARR(NX,NY,NZ)
	ENDIF ELSE BEGIN
                SUSPECT = INTARR(NX)
	ENDELSE
	CHOPON = SUSPECT
	CHOPS = SUSPECT
	RDSTAT = SUSPECT
	OTF = SUSPECT
	AUTDR = SUSPECT

; Less significant bit  <-- swapped --> bit 8 
	P = WHERE(X GT 255, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
       		SUSPECT(P) = 1
		X(P) = X(P) - 256
	ENDIF
; Most significant bit  <-- swapped --> bit 7 
	P = WHERE(X GT 127, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
		CHOPON(P) = 1
		X(P) = X(P) - 128
	ENDIF
; Bit 6
	P = WHERE(X GT 63, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
		CHOPS(P) = 1
		X(P) = X(P) - 64
	ENDIF
; Bit 5
	P = WHERE(X GT 31, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
		RDSTAT(P) = 1
		X(P) = X(P) - 32
	ENDIF
; Bit 4
	P = WHERE(X GT 15, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
		OTF(P) = 1
		X(P) = X(P) - 16
	ENDIF
; Bit 3
	P = WHERE(X GT 7, NCOUNT)
	IF (NCOUNT GT 0) THEN BEGIN
		AUTDR(P) = 1
		X(P) = X(P) - 8
	ENDIF
; Check - In case there are words which cannot be decomposed into the
; bits expected tell it and set them in all variables to -1 
	CHECK = WHERE (X NE 0, NCOUNT)
	IF (NCOUNT GT 0)  THEN BEGIN 
           PRINT,NCOUNT, '  times something wrong with PIXF, values set to -1'
	   SUSPECT(CHECK) = -1
           CHOPON(CHECK) = -1
           CHOPS(CHECK) = -1
           RDSTAT(CHECK) = -1
           OTF(CHECK) = -1
           AUTDR(CHECK) = -1
	ENDIF
	RETURN
	END
