;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       DEFAULT_RESPONSE
;
; PURPOSE:
;       Calculate the expected default responsivity according to the orbital
;	position
;       
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       res = DEFAULT_RESPONSE(tmpdata)
;
; INPUT PARAMETERS:
;	tmpdata: temp structure corresponding to SRD or SCP level 
;        
; OUTPUT PARAMETERS:
;       res: 		array of responsivities corresponding to each element in 
;	     		tmpdata.tkey
;
; KEYWORD PARAMETERS:
;	NEWHEADER: 	a changed header of the measurement
;	PRO_AOT:	if set, central AOT position is taken for orbital
;			position. Default is per measurement
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)			January 1998
;       Version 7.0
;	Bug for Resp pro AOT caused by dimensions fixed (CG)	September 1999
; V8.1
;-
FUNCTION DEFAULT_RESPONSE, tmpdata, NEWHEADER=newheader, $
	 PRO_AOT=pro_aot 

@phtresporb

	detsh = tmpdata.det
	IF STRMID(detsh,0,1) EQ 'P' THEN detsh='P'
	IF N_ELEMENTS(newheader) EQ 0 THEN newheader = tmpdata.hdr
	
	orig_name = 'P'+detsh+'RESP'
	det_calfile, orig_name, tmpdata.unit, tmp_name, error=error
	
	READ_RESPORB, filename=tmp_name
	
	timall = tmpdata.tkey
	times = timall
	
	IF KEYWORD_SET(PRO_AOT) OR RESPPROAOT THEN BEGIN
      		times = times*0.
      		dorb_mode = 'PER_AOT'
   	ENDIF ELSE BEGIN
		dorb_mode = 'PER_MEASUREMENT'
      		times = times*0.+AVG([timall]) 
      	ENDELSE
      
	rev = STRMID(tmpdata.admn,4,3)
; determine the orbital position (check the keyword TREFPHA1 in the header)
	IF KEYWORD_SET(PRO_AOT) OR RESPPROAOT THEN $
		orb_off = FXPAR(newheader, 'TREFPHA2', count=count) ELSE $
		orb_off = FXPAR(newheader, 'TREFPHA1', count=count)


	IF count EQ 0 THEN BEGIN 
    		orb_off = set_orbpos(rev, tmpdata.unit) 
  		IF KEYWORD_SET(VERBOSE) THEN $
      		print,'Computing orbital position from ALLPERIGEE internal file'
	ENDIF ELSE BEGIN 
  		orb_off = orb_off - rev
  		IF KEYWORD_SET(VERBOSE) THEN print,'Orbital position read from header '
	ENDELSE  

; Obtain the seconds from begin of observation by subtracting from the ITKs
; the START ITK of the first measurement and multiplying with the conversion 
; factor - After this normalize to orbital position using the orbit offset 
; portion at the begin 

	GIVE_CS, tmpdata, cst
	fct = FXPAR(tmpdata.hdr, 'TREFITKU', count=count)
	IF count EQ 0 THEN BEGIN
    		PRINT, 'Warning:  TREFITKU not found on header'
    		fct = 6.1035156e-05
	ENDIF

	IF KEYWORD_SET(PRO_AOT) or RESPPROAOT THEN BEGIN
	    orbpos=FLTARR(N_ELEMENTS(times))
	    orbpos(*)=orb_off 	    
	ENDIF ELSE BEGIN

 	 	times = (times - cst.ikst(0)) * fct

 		iso_orbit = 23.+56./60    ; duration of ISO orbit in hours  
 		orbpos = orb_off + times / 3600. / iso_orbit 
	ENDELSE

;determine detector and number of pixels
	det = tmpdata.det
	npix = pixel_det(tmpdata.det)

	CASE det of 
     	'P1': rp = ppresporb(*,0,*)
     	'P2': rp = ppresporb(*,1,*)
     	'P3': rp = ppresporb(*,2,*)
     	'C1': rp = c1resporb
     	'C2': rp = c2resporb
     	ELSE: RETURN, -1
	ENDCASE

	respons = FLTARR(npix, N_ELEMENTS(orbpos), 2)
	
	FOR IPIX=0,npix-1 DO BEGIN
	 respons(ipix,*,0) = INTERPOL(rp(*,ipix,0),orbpos_resp, orbpos)
	 respons(ipix,*,1) = INTERPOL(rp(*,ipix,1),orbpos_resp, orbpos)
	ENDFOR
	
	RETURN, respons
	END
	
		
	
	 
