;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Deglitch, Slope, Error, Accept
;+
;  NAME:
;	DEGLITCH
;
;  PURPOSE:
;	This procedure flags 'bad' ramps - those that are a specified 
;	distance away from their neighbours.
;	The ramps should be part of the same chopper position.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	DEGLITCH, Slope, Error, Accept
;
; INPUTS:
;	Slope: Array of ramp slopes within one chopper position
;	Error: Error associated with each slope
;
; OUTPUTS:
;	Accept: Array of flags corresponding to each slope
;	  0: Accept
;	  1: Not Accept
;
; COMMON BLOCKS:
;	deglitch_param
;
; PROCEDURE:
;	Take a local distribution and flag all points a certain number
;	of standard deviations from the median. Advance the local 
;	distribution. Reject all points that have been flagged a set
;	number of times. 
;
; PROCEDURES USED
;	None
;
; MODIFICATION HISTORY:
;	Written by:	Huw Morris (adapted from Steve Guest's Fortran)
;	Oct 95 Version 3.5
;	Use for calculating the standard deviation all but the most
;	away point (CG)			December 1995
;	Problem derived from last change fixed 	December 1995
;	Change taken away to have exactly same algorithm as OLP 	Feb 97
;	V6.1 
;-

@deglitch_param

	ON_ERROR, 0                    ; Stay within this routine 

	Num_Slopes = N_ELEMENTS(Slope)  

;  Flag all slopes as accepted to start with
	Accept = INTARR(Num_slopes)

;  If the number of points is below a certain minimum, reject all slopes
;  with an error greater than the preset threshold.
	IF (Num_Slopes LT Min_Deglitch) THEN BEGIN

	  Flag = WHERE (Error GT Max_Error, Num_Errors)

; Check there were any errors
	  IF (Num_Errors GT 0) THEN Accept(Flag) = Accept(Flag) + 1

	  RETURN

	ENDIF

;  Otherwise, apply the filtering algorithm

	Cur_Iter = 0                ; Number of iteration currently performed

	WHILE (Cur_Iter LT Max_Iter) DO BEGIN

	  Cur_Iter = Cur_Iter + 1
	  First_Slope = 0
	  Terminate = 0             ; False - reached end of the array yet?

;  Number of times each slope has been identified as bad.
	  Bad_count = INTARR(Num_slopes)

;  If there are less slopes than in the local distribution, a slope only
;  needs to be flagged once to be rejected
	  IF (Num_Slopes LE Num_Local) THEN BEGIN
	    Num_Bad_Used = 1
	  ENDIF ELSE BEGIN
	    Num_Bad_Used = Num_Bad
	  ENDELSE

	  WHILE (Terminate EQ 0) DO BEGIN     ; While not done all the slopes

;  Calculate the bounds of the local distribution.
	    Last_Slope = First_Slope + Num_Local - 1

;  Check does not go over the end of the array
	    IF (Last_Slope GE Num_Slopes) THEN BEGIN
	      Last_Slope = Num_Slopes - 1
	      Terminate = 1         ; True - reached the end of the array
	    ENDIF

;  Get slopes that have not already been rejected
	    Good = WHERE (Accept(First_Slope:Last_Slope) EQ 0, Num_Used)
	    Good = Good + First_Slope

;  Can't do anything useful with less than 2 slopes
	    IF (Num_Used GT 2) THEN BEGIN
	      Temp_Slope = Slope(Good)

;  Scale the data to the first slope
	      Est = Temp_Slope(0)
	      Temp_Slope = Temp_Slope - Est

;  Take for the standard deviation all but the "most away" point (CG)
	      IF degl_clip THEN BEGIN
	       maxi = MAX(Temp_Slope, min=mini)
	       ind = WHERE(Temp_Slope NE maxi AND Temp_Slope NE mini, ncount) 
	       IF ncount GT 1 THEN $
	        Std_Dev = STDEV(Temp_Slope(ind)) ELSE IF ncount EQ 1 THEN $
	        Std_Dev = Temp_Slope(ind) ELSE  Std_Dev=0.
	       Med = MEDIAN(Temp_Slope)
	      ENDIF ELSE BEGIN

	       ncount = N_ELEMENTS(temp_slope)
	       ind = indgen(ncount)
	       IF ncount GT 1 THEN $
	        Std_Dev = STDEV(Temp_Slope(ind)) ELSE IF ncount EQ 1 THEN $
	        Std_Dev = Temp_Slope(ind) ELSE  Std_Dev=0.
	        Med = MEDIAN(Temp_Slope)
	      ENDELSE

;  Flag everything more than a given amount from the median
	      Dif = ABS(Temp_Slope - Med)
	      Bad =  WHERE(Dif GT (Num_Sigma * Std_Dev), Count)

	      IF (Count GT 0) THEN BEGIN
	        Bad_count(Good(Bad)) = Bad_count(Good(Bad)) + 1
	      ENDIF

	    ENDIF

;  Next local distribution
	    First_Slope = First_Slope + Num_Step

	    IF (First_Slope GE Num_Slopes) THEN Terminate = 1

	  ENDWHILE

;  Reject points flagged at least Num_Bad_Used times
	  Rejects = WHERE(Bad_count GE Num_Bad_Used, Num_Reject)

;  If nothing was rejected this iteration return to the calling procedure
	  IF (Num_Reject EQ 0) THEN RETURN

	  Accept(Rejects) = 1

	ENDWHILE

	END
