;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Deglitch_All, Slope, Step, RPID, Error, Sgdisc, Accept, $ 
	       SLICE_CHOPP= slice_chopp, NOSLICE_RASTER = noslice_raster
;+
; NAME:
;	DEGLITCH_ALL
;
; PURPOSE:
;	This routine takes an array of ramps, and flags those which appear
;	to be glitches in the data. Chopper plateaux are taken into account.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	DEGLITCH_ALL, Slope, Step, RPID, Error, Sgdisc, Accept, [/SLICE_CHOPP], $
;			[,/NOSLICE_RASTER]
;
; INPUTS:
;	Slope: Array of ramp slopes
;	Step:  Array of chopper steps (1-n)
;	RPID:  2 BYTE-array of Raster Points ID
;	Error: Error associated with each ramp
;	SGDISC: Acceptance flag array (0=acc, all others discarded) 
;
; OUTPUTS:
;	Accept: Array of flags corresponding to each slope
;	  0:  Accept
;	  1:  Not Accept
;
; KEYWORDS:
;	slice_chopp: if set then perform deglitch on individual chopper
;		     plateau. By default all chopper plateaus of the same 
;		     step are used.
;
; RESTRICTIONS:
;	The input arrays must be of the same dimension
; 
; PROCEDURE:
;	The ramps are separated into chopper/raster steps, then each step is
;	deglitched separately using only the accepted points.
;
; PROCEDURES USED:
;	PIA LIB: deglitch
;
; MODIFICATION HISTORY:
;	Written by:	Huw Morris
;	Modified by JA			 		November 96
;	V5.4
;	File header changes (CG)			February 1998
;	V7.0
;	
;-

; This procedure is an extension to Deglitch that takes into account
; the chopper position. Deglitch is run for each chopper position in
; turn, rather than for the whole measurement at a time.

	ON_ERROR, 2                  ; Return to calling procedure

; Check that the Slope, Step and Error arrays are the same size
	Num_Slope=N_ELEMENTS(Slope)
	Num_Step=N_ELEMENTS(Step)
	Num_Error=N_ELEMENTS(Error)

	Max_Arr=MAX([Num_Slope,Num_Step,Num_Error])
	Min_Arr=MIN([Num_Slope,Num_Step,Num_Error])

	If (Max_Arr NE Min_Arr) THEN BEGIN
	  MESSAGE, 'Input arrays not the same size.'
	  RETURN
	ENDIF


; define slice_chopp
  IF NOT KEYWORD_SET(slice_chopp) then slice_chopp = 0 
    
  
; Initially flag all slopes as accepted
	Accept = INTARR(Max_Arr)

; Find the number of chopper steps
	Max_Steps = MAX(Step)

; Loop over each raster point ID
	IF KEYWORD_SET(noslice_raster) THEN BEGIN 
	   N_RASTER = 1 
	   raststeps = make_array(Num_slope, value=1)
	   slice_chopp = 0  
	ENDIF ELSE BEGIN
	   get_rastersteps, rpid, raststeps
	   N_RASTER = max(raststeps)
	ENDELSE    

	FOR r_point = 1, N_RASTER DO BEGIN

	 ;ind_rpid = WHERE(RPID(0,*) EQ rast_tmp(0) AND $
	 ; 		RPID(1,*) EQ rast_tmp(1), ncount)
	 ind_rpid = WHERE(raststeps eq r_point, ncount)
	 IF ncount NE 0 THEN tmp_rpid = rpid(*,ind_rpid(0):*)
	  
	 IF ncount GT 0 THEN BEGIN 
	 
	  CASE slice_chopp of 
	 
	   0: BEGIN  ; Loop over each chopper STEP
	      FOR Cur_Step = 1, Max_Steps DO BEGIN
              ; For each chopper step, extract the slopes corresponding to 
              ; that step and deglitch those slopes in isolation.
	         Step_Indices = WHERE(Step(ind_rpid) EQ Cur_Step, Count0)
	 	 IF (Count0 NE 0) THEN BEGIN 
	 	  Disc_Ind = WHERE(Sgdisc(ind_rpid(Step_Indices)) EQ 0, Count)
	 	  

	  	  IF (Count NE 0) THEN BEGIN
  	   	   Temp_Slope = Slope(ind_rpid(Step_Indices(Disc_Ind)))
	   	   Temp_Error = Error(ind_rpid(Step_Indices(Disc_Ind)))

	   	   Deglitch, Temp_Slope, Temp_Error, Temp_Accept
 
		   ; Translate the flagged errors of the temporary slope array 
		   ; into indices of the 'proper' array.
	   	   Accept(ind_rpid(Step_Indices(Disc_Ind))) = Temp_Accept

		  ENDIF
		 ENDIF 

		ENDFOR
	      END  
	      
	   1: BEGIN ; Loop over each chopper PLATEAU
	      get_nchoppsweep, step(ind_rpid), nchopp, ind_platt
	      ind_platt = [-1,ind_platt]
	      
	      FOR cur_platt = 1, nchopp DO BEGIN
	         np_platt = ind_platt(cur_platt)-ind_platt(cur_platt-1)
	         IF np_platt gt 0 THEN BEGIN
	           platt_indices =  $ 
	               ind_rpid(ind_platt(cur_platt-1)+1:ind_platt(cur_platt))
	           Disc_Ind = WHERE(Sgdisc(platt_indices) EQ 0, Count) 
	           IF (Count NE 0) THEN BEGIN     
	            temp_slope = slope(platt_indices(Disc_ind)) 
	            temp_error = error(platt_indices(Disc_ind))
	          
	            deglitch, temp_slope, temp_error, temp_accept
	          
		    ; Translate the flagged errors of the temporary slope array 
		    ; into indices of the 'proper' array.
	   	    Accept(platt_indices(Disc_ind)) = Temp_Accept
	   	   ENDIF 
	            
	         ENDIF
	      ENDFOR   
	      
	      END
	  ELSE: MESSAGE, /INFO, 'Wrong case, returning ...'
	  
	  ENDCASE       
	    
	 ENDIF

	 ;ind_nrpid = WHERE(tmp_RPID(0,*) NE rast_tmp(0) OR $
	 ; 		tmp_RPID(1,*) NE rast_tmp(1), ncount)
	 ;IF ncount GT 0 THEN rast_tmp = tmp_rpid(*,ind_nrpid(0))
	ENDFOR

	END
