;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       DO_CORR_RESETINT
;
; PURPOSE:
;       Perform the reset interval correction for a measurement, at
;	SRD or SCP level.
;
; CATEGORY:
;       Signal corrections
;
; CALLING SEQUENCE:
;       DO_CORR_RESETINT, tmpdata, newheader, SCP= scp, INFO= info
;
; INPUTS:
;       tmpdata: Temporary data structure at SRD or SCP level
; KEYWORD PARAMETERS:
;       SCP:  If set then the measurement is
;       INFO:   A text containing info about the success or not of the correction
;
; COMMON BLOCKS:
;
; RESTRICTIONS:
;       TMPDATA must be a valid element of PHTSRD/PHTSCP temporary structure
;
; PROCEDURES USED:
;       PIA LIB:
;               give_cs
;		corr_reset_int
;       ASTRO LIB:
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: J.Acosta (ISO Soc)               November 1997
;       V6.6
;	Systematic unc. not propagated but into hdr   (CG)	March 1998
;	V7.1
;	Writing [-1.,-1.] for undefined uncertainties (CG)
;	 and avoiding division by 0.				July 1998
;	V7.2
;	Changed according to bitflag system (CG)
;       V7.4
;-
;

PRO do_corr_resetint, tmpdata, newheader, SCP = scp, INFO= info


IF N_ELEMENTS(newheader) EQ 0  THEN newheader = tmpdata.hdr 


; check: if correction already applied return
                done = fxpar(newheader,'PRS_RIN0')
                IF done THEN BEGIN 
                   info = ' Reset interval correction was already applied'
                   RETURN
                ENDIF   
                
        	  INFO =' Applying reset interval correction'
; check: if dark current already subtracted warn
                  done = fxpar(newheader,'PRS_DARK')
                  IF done THEN INFO =[INFO,  $
        	   ' dark current already subtracted! should not happen ']
                
		  IF KEYWORD_SET(scp) THEN BEGIN 
		     tmpsig = tmpdata.aver 
		     uncert = tmpdata.avun  
		  ENDIF ELSE BEGIN 
		     tmpsig = tmpdata.slop
		     uncert = tmpdata.slun
		  ENDELSE   
		    
		  GIVE_CS, tmpdata, cst, in, error=error
		  IF error THEN BEGIN 
		     INFO = [INFO,'ERROR reading CS ' + STRTRIM(tmpdata.admn,2)]
		     RETURN 
		  ENDIF     

		  corr_reset_int, tmpsig, tmpdata.det, $
			cst.intt(in), cst.drs(in), $
			out_sig, corr, UNCERT = uncert 
			
		  IF corr.a1 NE 0. THEN BEGIN
		   CASE 1 OF
		     KEYWORD_SET(scp): BEGIN 
		       tmpdata.aver = out_sig
		       tmpdata.avun = corr.a1 * tmpdata.avun
		       tmpdata.medi = corr.a0 + corr.a1*tmpdata.medi
		       tmpdata.q1me = corr.a0 + corr.a1*tmpdata.q1me
		       tmpdata.q3me = corr.a0 + corr.a1*tmpdata.q3me
		       ppix = pixel_det(tmpdata.det)
; use bitflag system
		       get_validflag, tmpdata.flag, 'SCP', $
		          redflag, det=tmpdata.det
		         
		       flags = redflag(0:ppix-1,*)
		       
		       avers = tmpdata.aver(0:ppix-1,*)
		       
		       indok = WHERE(flags EQ 0 AND $
		       		     avers NE 0., nok)
		       IF nok EQ 0 THEN $
		        indok = WHERE(avers NE 0., nok)
		       IF nok GT 0 THEN BEGIN
		        rel_unc = FLTARR(2)
		        rel_unc(0) = ABS(avg([uncert(0)/avers(indok)]))
		        rel_unc(1) = ABS(avg([uncert(1)/avers(indok)]))
		       ENDIF ELSE rel_unc = [-1.,-1.]
		      END
		    ELSE:  BEGIN  
		       tmpdata.slop = out_sig
		       tmpdata.slun = corr.a1 * tmpdata.slun
		       ppix = pixel_det(tmpdata.det)
; use bitflag system
		       get_validflag, tmpdata.flag, 'SRD', $
		          redflag, det=tmpdata.det
		         
		       flags = redflag(0:ppix-1,*)

		       slops = tmpdata.slop(0:ppix-1,*)
		       indok = WHERE(flags EQ 0 AND $
		       		     slops NE 0., nok)
		       IF nok EQ 0 THEN $
		        indok = WHERE(slops NE 0., nok)
		       IF nok GT 0 THEN BEGIN
		        rel_unc = FLTARR(2)
		        rel_unc(0) = ABS(avg([uncert(0)/slops(indok)]))
		        rel_unc(1) = ABS(avg([uncert(1)/slops(indok)]))
		       ENDIF ELSE rel_unc = [-1.,-1.]
		      END
		   ENDCASE   

		    fxaddpar, newheader,'PRS_RIN0',$
		        corr.a0,' Reset interval correction: A0', $
              		BEFORE='XTENSION' 
		    fxaddpar, newheader,'PRS_RIN1',$
		        corr.a1,' Reset interval correction: A1', $
              		BEFORE='XTENSION'
	            fxaddpar, newheader, 'PRS_R0_S', rel_unc(0), $
	 	     ' <Systematic rel. unc.> - Reset Int. corr. A0', $
	 	       BEFORE='XTENSION' 
	            fxaddpar, newheader, 'PRS_R1_S', rel_unc(1), $
	 	     ' <Systematic rel. unc.> - Reset Int. corr. A1', $
	 	       BEFORE='XTENSION' 
		  ENDIF ELSE  $
        	    INFO =[INFO,' Reset interval correction not possible']
 




RETURN
END
