;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	DRFT1.PRO
;
; PURPOSE:
;	Function corresponding to the numerical differential drift model
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	fct = DRFT1(x,a)
;
; INPUT:
;	x:	independent variable 
;	a:	parameter set:
;		a(0)	F  ideal signal during measurement
;		a(1)	p0 ideal signal before measurement
;		a(2)	a  coupling parameter
;
;	
; OUTPUT:
;	fct:	function by time x	
;			
; METHOD:
;
; Numerical solution of differential equation
;
;    d(fct)
;   ------- = F * a - fct * a = v
;      dt
;
; MODIFICATION HISTORY:
;	Written by: B. Schulz (ESA/VILSPA-SAI)		August 1996 
;	Modified by: CG					August 1996
;	Version 5.0 - Aug.96
;			
;-
function drft1, t, a

;print,"---> ",a

F = a(0)	; initial signal
;dt = 0.01	; integration time interval
dt = (t(1) - t(0)) / 4 > 0.5 
n = max(t) / dt + 1	; number of steps
nterms = 3	; number of variables
noutp = n_elements(t) ;number of output values
p  = fltarr(n)
t0 = fltarr(n)

t0(0) = 0.		;set starting conditions
p(0) = a(1)


; numerical intergration -------------------------------
t0 = indgen(n) * dt

for i=1, n-1 do begin
  IF p(i-1) GT 1.e10 THEN BEGIN
    p(i:*) = p(i-1) 
    GOTO, escape
  ENDIF
  p(i) = p(i-1) + (F * a(2) - p(i-1)*a(2))*dt
endfor

escape:
; interpolate ------------------------------------------


ix = where(finite(p))

if ix(0) EQ -1 then begin
	print, 'ERROR: No valid number found in function!!'
	return, 0
	end

pp = p(ix)
tt = t0(ix)

if n_elements(ix) LT n_elements(p) then print, "NaN found!"

s = interpol(pp, tt, t)		;interpolate main function

return, s
end
