;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	DRIFT_MOD_PROC
;
; PURPOSE:
;	Call a drift modelling routine returning the modelled signal
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	DRIFT_MOD_PROC, det, time, slop, slop_unc, slop_flag, rpid, step,$
;	 tempera, fit_par_str, newhdr, CHOPPED=chopped, MULTI_PIX= multi_pix, $
; 	 PIXCH= pixch
;
; INPUT:
;	det:		Detector name
;	time:		Array containing the signals' times
;	slop:		Slopes array
;	slop_unc:	Slopes' uncertainties array
;	slop_flag:	Slopes' flags array
;	rpid:		Raster points array
;	step:		Chopper step positions array
;	tempera;	Detector temperature array
;
; KEYWORD PARAMETERS:
;	MULTI_PIX:	For non-interactive use. Performing loop over different
;			pixels.
;	
; OUTPUT:
;	fit_par_str:	Structure containing the fitting information:
;			 mod_func:	function used
;			 nparams:  	number of parameters used 
;			 flag:		output flag
;                	 param:         parameters array 
;                	 		(3-dim: pixel, step, parameter)
;                	 sg_param:	uncertainties of param
;                	 slop: 		fitted slopes
;                	 slop_flag:	flag of the fitted slopes
;                 	 signal:	final signals
;			 time_sg:	time for final signals
;                	 err:		uncertainties of 'signal'
;                	 chisq:		chi-squares of the fit
;			
;	
; COMMON BLOCKS:
;	TEMPERATURE
;		
; RESTRICTIONS:
;	The structure fit_par_str has to be defined prior to access this routine
;
; PROCEDURES USED:
;	PIA LIB:
;		check_unc
;		fit_function
;		fouks_start_par
;		get_rastersteps
;		get_nchoppsweep
;		gauss_start_par
;		gaussinv_start_par
;		pixel_det
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC)		   		June 1996 
;	Modified by: C.Gabriel (ESA/VILSPA-SAI)	   		August 1996
;	Temperature is included as input to be
;	used in thermal Fouks, by JAcosta(ESA/IAC) 		September 1996
;	Version 5.0 - Aug.96
;	Chi-square calculations improved (JA+CG)   		February 1997
;	Bug corrected when not enough points (JA)  		February 1997
;	V6.1
;	Possibility to use Fouks rectangular chopping mode	
;	  for meas. performde in triangular mode (JA)		December 1998	 
;			
;-
PRO DRIFT_MOD_PROC, det, time, slop, slop_unc, slop_flag, rpid, step, tempera, $
	 fit_par_str, newhdr, MULTI_PIX= multi_pix, PIXCH = pixch, $
	 WEIGHTM = weightm, Verbose=verbose, CMODE = cmode
	 

COMMON TEMPERATURE, tmptime_temp, tmptemp
COMMON PIA_DRIFT_PAR, pia_drift_par


; define the number of pixels depending 
IF KEYWORD_SET(multi_pix) THEN npix = pixel_det(det) ELSE npix = 1
IF NOT KEYWORD_SET(pixch) THEN pixch = 0
IF NOT KEYWORD_SET(weightm) THEN weightm = 'NONE'

ffunct = fit_par_str.mod_func

chopped = 0 
allstep = 0
allrst = 0
; set chopped to 1 if fitting chopper plateau by pairs
CASE 1 OF
        strpos(ffunct,'_RST_') gt -1: allrst = 1 
	strpos(ffunct,'TCHO') gt -1: allstep = 1
	strpos(ffunct,'CHOP') gt -1: chopped = 1
	ELSE: 
ENDCASE	

;

cstep = 0

get_rastersteps, rpid, raststeps

get_rpchopstep, raststeps, step, rpchopsteps

CASE 1 OF 		; start case depending on fitting mode
; fitting is done using all raster positions
allrst:  BEGIN
 indrst = INDGEN(N_ELEMENTS(fit_par_str.slop(0,*)))
 set_common_drift, ffunct, raststeps, fit_par_str.t_zero
 
 ; Start loop for all pixels      
 FOR ipix = 0, npix-1 DO BEGIN
      IF KEYWORD_SET(verbose) THEN $
      print,'DRIFT_MOD_PROC: Doing pixel ',ipix+1 

      ; check if there is initialization error
      IF fit_par_str.flag(ipix,0) eq 0 $ 
           THEN BEGIN  
        ;if no initialization error then make fitting   
        set_driftpar_srd, fit_par_str, drift_par, ipix, 0
        drift_par_ini = drift_par
       
        ind = where (fit_par_str.slop_flag(ipix, indrst) le 1, nc) 
        ind_val = indrst(ind) 
        ind_all = indrst

	tmpx = time(ind_val)
	tmpy = reform(slop(ipix,ind_val))
	tmpflg = reform(slop_flag(ipix,ind_val))
	tmperry = reform(slop_unc(ipix,ind_val))
	cstep = step(ind_val)

	; special case for temperature of PHT-S (2-D array for SS and SL)
	IF det eq 'SL' THEN BEGIN
	  IF keyword_set(multi_pix) THEN                             	$
	         tmptemp = reform(tempera(fix(ipix/64),indrst)) 	$
	  ELSE tmptemp = reform(tempera(fix(pixch/64), indrst))
	ENDIF ELSE tmptemp = tempera(indrst)
	tmptime_temp = time(indrst)

        ;check that errors are not zero, if yes then set a suitable value
	;check_unc, tmpy, tmperry	
	
	invar = WHERE(drift_par_ini.a_var EQ 1, nvar)
	IF nvar GT 0 THEN $
	   fit_function, tmpx, tmpy, tmperry, tmpflg, drift_par, $ 
	       /NODERIVATIVE, WEIGHT = weightm,/NOT_MARQUAD
	
	;check for limits in the fitting parameters
	check_limits_dfpar, drift_par_ini.p_field, drift_par, lpar_flg
			
	funct = drift_par.ffunct
	
	pia_drift_par = drift_par    ; save the recently found parameters 
	 	
	call_procedure, funct, time(indrst), drift_par.p_field, fct, tmppder,  $
	          ff_err, /FINAL
        fit_par_str.slop(ipix, indrst) = fct
        
; Noise calculation
	noise = fltarr(max(cstep))
	FOR istep=1,max(cstep) DO BEGIN
	  ind = WHERE(cstep EQ istep,nst)
	  noise(istep-1) = SQRT(TOTAL((fct(ind)-tmpy(ind))^2) / (nst-1))
	ENDFOR
	


      IF KEYWORD_SET(verbose) THEN BEGIN
	print,'Abs_tstep, Fin_sig ', iistep+1,drift_par.p_field(0)
	print,'Fit_error ' ,ff_err
	print,'Boundary error', lpar_flg
      ENDIF	
	        
        save_drift_str, drift_par, fit_par_str, PIXEL=ipix, NOISE=noise, $ 
                 FF_ERROR = ff_err, LPAR_ERROR = lpar_flg 
                
              
      ENDIF ELSE BEGIN    ;Initialization error, do not make any fitting
      
	   PRINT, 'Not enough valid points to be used for fitting'
	   
           ;; !!!!!It should be revised, only provisional solution!!!!!!!!
           ;fit_par_str.signal(ipix, irpchop) = avg(slop(indrpchop))
           ;fit_par_str.err(ipix, irpchop) = stdev(slop(indrpchop))
           fit_par_str.flag(ipix, iistep) = 1 + 2 + 4 ; set 3 digit flag
	   
      ENDELSE       

  

 ENDFOR ;close loop per pixel

END

; fitting is done using all chopper plateaux per raster position
allstep: BEGIN 
 ind_tmp = fit_par_str.ind_steps - 1
 
 newstep = step
 
 IF STRPOS(ffunct,'RTCHO') $ 
       NE -1 AND STRPOS(STRUPCASE(cmode),'TRIANGULAR') NE -1 THEN BEGIN  
          ind_step3 = where(newstep eq 3, nstep3)
	  IF nstep3 GT 0 THEN newstep(ind_step3) = 1
          ind_step4 = where(newstep eq 4, nstep4)
	  IF nstep4 GT 0 THEN newstep(ind_step4) = 2
 ENDIF
 ; start loop per Raster position
 FOR iistep = 0, max(raststeps)-1 DO BEGIN
 
  indrst = WHERE(raststeps EQ iistep+1, nc)
  
  
   ; Start loop for all pixels      
   FOR ipix = 0, npix-1 DO BEGIN
   
      IF KEYWORD_SET(verbose) THEN $
      print,'DRIFT_MOD_PROC: Doing pixel ',ipix+1 

      ; check if there is initialization error
      IF TOTAL(fit_par_str.flag(ipix,ind_tmp(iistep)+1:ind_tmp(iistep+1))) eq 0 $ 
           THEN BEGIN  
        ;if no initialization error then make fitting   
        
        set_driftpar_srd, fit_par_str, drift_par, ipix, iistep
        drift_par_ini = drift_par

        ind = where (fit_par_str.slop_flag(ipix, indrst) le 1, nc) 
        ind_val = indrst(ind) 
        ind_all = indrst

	tmpx = time(ind_val)
	tmpy = reform(slop(ipix,ind_val))
	tmpflg = reform(slop_flag(ipix,ind_val))
	tmperry = reform(slop_unc(ipix,ind_val))
	cstep = newstep(ind_val)
	set_common_drift, ffunct, cstep, drift_par.t_zero


	; special case for temperature of PHT-S (2-D array for SS and SL)
	IF det eq 'SL' THEN BEGIN
	  IF keyword_set(multi_pix) THEN                             	$
	         tmptemp = reform(tempera(fix(ipix/64),ind_all)) 	$
	  ELSE tmptemp = reform(tempera(fix(pixch/64), ind_all))
	ENDIF ELSE tmptemp = tempera(ind_all)
	tmptime_temp = time(ind_all)
  
        ;check that errors are not zero, if yes then set a suitable value
	check_unc, tmpy, tmperry	
	
	invar = WHERE(drift_par_ini.a_var EQ 1, nvar)
	IF nvar GT 0 THEN $
	   fit_function, tmpx, tmpy, tmperry, tmpflg, drift_par, $ 
	       WEIGHT = weightm,STEP=cstep
	
	;check for limits in the fitting parameters
	check_limits_dfpar, drift_par_ini.p_field, drift_par, lpar_flg
			
	funct = drift_par.ffunct
	
	pia_drift_par = drift_par    ; save the recently found parameters 
	 	
	set_common_drift, ffunct, newstep(indrst), drift_par.t_zero
	call_procedure, funct, time(indrst), drift_par.p_field, fct, tmppder,  $
	          ff_err, /FINAL
        fit_par_str.slop(ipix, indrst) = fct
        
; Noise calculation
	noise = fltarr(max(cstep))
	FOR istep=1,max(cstep) DO BEGIN
	  ind = WHERE(cstep EQ istep,nst)
	  noise(istep-1) = SQRT(TOTAL((fct(ind)-tmpy(ind))^2) / (nst-1))
	ENDFOR
	


      IF KEYWORD_SET(verbose) THEN BEGIN
	print,'Abs_tstep, Fin_sig ', iistep+1,drift_par.p_field(0)
	print,'Fit_error ' ,ff_err
	print,'Boundary error', lpar_flg
      ENDIF	
	        
        save_drift_str, drift_par, fit_par_str, PIXEL=ipix, NOISE=noise, $ 
                ABS_TSTEP = iistep, FF_ERROR = ff_err, LPAR_ERROR = lpar_flg 
                
              
      ENDIF ELSE BEGIN    ;Initialization error, do not make any fitting
      
	   PRINT, 'Not enough valid points to be used for fitting'
	   
           ;; !!!!!It should be revised, only provisional solution!!!!!!!!
           ;fit_par_str.signal(ipix, irpchop) = avg(slop(indrpchop))
           ;fit_par_str.err(ipix, irpchop) = stdev(slop(indrpchop))
           fit_par_str.flag(ipix, iistep) = 1 + 2 + 4 ; set 3 digit flag
	   
      ENDELSE       


   ENDFOR ;close loop per pixel

 ENDFOR ; close loop per raster position

END


ELSE: BEGIN	
; fitting is done on chopper plateau basis or at maximum by pairs 
; start loop per RPID/chopper plateau
 FOR irpchop=0, max(rpchopsteps)-1, (chopped+1) DO BEGIN   
 
 indrpchop = WHERE(rpchopsteps EQ irpchop+1, nc)
 ; check if min_npts 
 
 
 IF chopped THEN BEGIN 
       	indrpchop2 = WHERE(rpchopsteps EQ irpchop+2, nc2)
 	;check wether the next step is a valid one in the case of chopped 
	; fit the last irpchop can be an odd number 
 	IF nc2 eq 0 THEN goto, NO_SIG_RASTER 
 ENDIF	

 iistep = irpchop/(chopped+1) 
    
 ; Start loop for all pixels      
 FOR ipix = 0, npix-1 do begin
     
      IF KEYWORD_SET(verbose) THEN $
      print,'DRIFT_MOD_PROC: Doing pixel ',ipix+1 
     
      ; check if there is initialization error
      IF TOTAL(fit_par_str.flag(ipix,irpchop:irpchop+chopped)) eq 0 $ 
           THEN BEGIN  
        ;if no initialization error then make fitting   
        
        set_driftpar_srd, fit_par_str, drift_par, ipix, irpchop
        drift_par_ini = drift_par

        ind = where (fit_par_str.slop_flag(ipix, indrpchop) le 1, nc) 
        ind_val = indrpchop(ind) 
        ind_all = indrpchop
      
        IF chopped THEN BEGIN
           ind2 = where(fit_par_str.slop_flag(ipix, indrpchop2) le 1, nc2) 
           ind_val = [ind_val, indrpchop2(ind2)]
           ind_all = [ind_all, indrpchop2]
        ENDIF          
 	
	tmpx = time(ind_val)
	tmpy = reform(slop(ipix,ind_val))
	tmpflg = reform(slop_flag(ipix,ind_val))
	tmperry = reform(slop_unc(ipix,ind_val))
	IF chopped THEN cstep = step(ind_val)
	
	; special case for temperature of PHT-S (2-D array for SS and SL)
	IF det eq 'SL' THEN BEGIN
	  IF keyword_set(multi_pix) THEN                             	$
	         tmptemp = reform(tempera(fix(ipix/64),ind_all)) 	$
	  ELSE tmptemp = reform(tempera(fix(pixch/64), ind_all))
	ENDIF ELSE tmptemp = tempera(ind_all)
	tmptime_temp = time(ind_all)
	cstep_full = step(ind_all)	
	
	check_unc, tmpy, tmperry
	
	
	invar = WHERE(drift_par_ini.a_var EQ 1, nvar)
	IF nvar GT 0 THEN $
	   fit_function, tmpx, tmpy, tmperry, tmpflg, drift_par, STEP=cstep,$ 
	       WEIGHT = weightm
	
	;check for limits in the fitting parameters
	check_limits_dfpar, drift_par_ini.p_field, drift_par, lpar_flg
			
	;print,'a_var at drift_mod_proc ', drift_par.a_var

	funct = drift_par.ffunct
	
	tmpx_fin = time(ind_all)
	
	pia_drift_par = drift_par    ; save the recently found parameters  
	IF chopped THEN set_common_drift, ffunct, cstep_full, drift_par.t_zero
	call_procedure, funct, tmpx_fin, drift_par.p_field,  $
	          fct, tmppder, ff_err, /FINAL
	
;	print,'Fitting error ',ff_err
	
        fit_par_str.slop(ipix, ind_all) = fct
        
; Noise calculation
	IF chopped THEN cst = cstep ELSE cst = INTARR(nc) + 1
	noise = fltarr(max(cst))
	FOR istep=1,max(cst) DO BEGIN
	  ind = WHERE(cst EQ istep,nst)
	  noise(istep-1) = SQRT(TOTAL((fct(ind)-tmpy(ind))^2) / (nst-1) )
	ENDFOR

	        
      IF KEYWORD_SET(verbose) THEN BEGIN
	print,'Abs_tstep, Fin_sig ', irpchop+1,drift_par.p_field(0)
	print,'Fit_error ' ,ff_err
	print,'Boundary error', lpar_flg
      ENDIF	
	        
        save_drift_str, drift_par, fit_par_str, PIXEL=ipix, NOISE=noise, $ 
                ABS_TSTEP = irpchop, FF_ERROR = ff_err, LPAR_ERROR = lpar_flg 
                
              
      ENDIF ELSE BEGIN    ;Initialization error, do not make any fitting
      
	   PRINT, 'Not enough valid points to be used for fitting'
	   
           ;; !!!!!It should be revised, only provisional solution!!!!!!!!
           ;;fit_par_str.signal(ipix, irpchop) = avg(slop(indrpchop))
           ;;fit_par_str.err(ipix, irpchop) = stdev(slop(indrpchop))
           fit_par_str.flag(ipix, irpchop) = 1 + 2 + 4  ; set 3 digit flag
	   
	   IF chopped THEN BEGIN
              ;;fit_par_str.signal(ipix, irpchop+1) = avg(slop(indrpchop2))
              ;;fit_par_str.err(ipix, irpchop+1) = stdev(slop(indrpchop2))
              fit_par_str.flag(ipix, irpchop+1) = 1 + 2 + 4  ; set 3 digit flag
	   ENDIF
	    
      ENDELSE       
    

    ENDFOR ; close loop per pixel

   NO_SIG_RASTER:
  
 ENDFOR       ; close loop per Raster Step/chopper position
 
 END		; end else case 

ENDCASE



; Log into header the relevant parameters used

	IF N_ELEMENTS(newhdr) GT 2 THEN BEGIN
		PRSPIX = 'PRS_DMFF' 
		FXADDPAR, newhdr, PRSPIX, drift_par_ini.ffunct, $
			' Drift modelling function used ', BEFORE='XTENSION'	 
		
		PRSPIX = 'PRS_DMPN' 
		FXADDPAR, newhdr, PRSPIX, drift_par_ini.minp_ext, $
			' Minimum number of points for drift modelling ',$
			BEFORE='XTENSION'	 

		PRSPIX = 'PRS_DMFL' 
		FXADDPAR, newhdr, PRSPIX, drift_par_ini.fract_left, $
			' Meas.Fraction left out for drift modelling  ', $
			BEFORE='XTENSION'	 

		PRSPIX = 'PRS_DMCH' 
		FXADDPAR, newhdr, PRSPIX, drift_par_ini.chisqt, $
			' Chi square tolerance for accepting fit result ', $
			BEFORE='XTENSION'	 

		PRSPIX = 'PRS_DMWE' 
		FXADDPAR, newhdr, PRSPIX, drift_par_ini.weightm, $
			' Weigthin method used in the fitting procedure ', $
			BEFORE='XTENSION'	 

		fixpar = WHERE(drift_par_ini.a_var(0:fit_par_str.nparams-1) $
				EQ 0, nfix)
		IF nfix GT 0 THEN BEGIN
			FOR ifix=0,nfix-1 DO BEGIN
			  PRSPIX = 'PRS_DMF'+STRTRIM(STRING(fixpar(ifix)+1),2) 
			   FXADDPAR, newhdr, PRSPIX, fixpar(ifix)+1, $
			   ' Parameter fixed in fitting procedure ', $
			  BEFORE='XTENSION'	 
			ENDFOR
		ENDIF
		
	ENDIF

;;


RETURN

END
