;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FITPAR_STR_INIT
;
; PURPOSE:
;	Initializes the drift fitting parameters for a given function
;
; CATEGORY:
;	PIA - algorithms
;
; CALLING SEQUENCE:
;	FITPAR_STR_INIT, tmpsrd, fitpar_str, init_error, ffunct, 
;	 	[,/ONLY_MEASPAR] [,/DEFAULT] [,PIXEL=pixel]  
;
; INPUT:
;	tmpsrd:		A phtsrd like temporary structure
;	ffunct:		Optional parameter to specify a particular function. 
;			If not specified a default function is used depending
;			on the detector: 
;			     Fouks function for P1, P2 & SL
;			     Exponential function for P3, C100 & C200
;
; OUTPUT:
;	fit_par_str:	Structure containing the initial parameters for fitting
;	init_error:	Initialization error (1-D array  of NPIX elements)
;
; KEYWORD PARAMETERS:
;	ONLY_MEASPAR:	Change only parameters related to the measurement
;	DEFAULT:	Assing values stored in common block DFFUNCT_INFO to
;			parameters independent of measurements
;	PIXEL:		Use only the specified pixel 
;	
; RESTRICTIONS:
;	TMPDATA must be a valid PHTSRD temporary structure
;	
; COMMON BLOCKS:
;	dffunct_info
;	pia_drift_par 
;		
; PROCEDURES USED:
;	PIA LIB:
;		decode_admn
;		drift_mod_proc
;		pia_disp_print_file
;		pia_load_driftpar
;		pia_xplot
;       INTERNAL:
;		pia_drift_mod_event
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC-La Laguna)		September 1996
;	Added flag as input parameter to parfunc call
;					    (J.Acosta)  October 1996
;	include all treatment with flag at beggining of 
;	each chopper/raster plateau. Consistency with
;	fraction of the given measurement (J.Acosta)	November 1996
;	Adapted to use mininmum time to discard values
;	(JAcosta)					June 1997
;	Version 6.1 - Oct.96
;			
;-
PRO fitpar_str_init, tmpdata, fit_par_str, init_error, ffunct, $
   ONLY_MEASPAR = only_measpar, PIXEL= pixel, DEFAULT= DEFAULT, CMODE = cmode
   
COMMON pia_drift_par, pia_drift_par

COMMON DFFUNCT_INFO, funct_info

IF KEYWORD_SET(only_measpar) THEN only_measpar=1 ELSE only_measpar=0

; check if common dffunct_info is not defined
if n_elements(funct_info) eq 0 then dffunct_info

det = tmpdata.det
ind_det = WHERE(funct_info.det_def EQ det)


if keyword_set(PIXEL) then begin
  npix = 1
  indpix = pixel-1
endif else begin
  npix = pixel_det(det)
  indpix = indgen(npix)
endelse  
  
stim = tmpdata.stim  
slop = tmpdata.slop(indpix,*)
slop_flg = tmpdata.flag(indpix,*)
temp = tmpdata.temp
step = tmpdata.step

get_rastersteps, tmpdata.rpid, raststeps, rpid_flag, TIME = stim
get_nchoppsweep, step, tot_nchopp

get_rpchopstep, raststeps, step, rpchopsteps
ntsteps = max(rpchopsteps)

; check if Function does exist
; check if a function was specified 
;if pia_drift_par is defined then use this function, otherwise use first in
; dffunct
IF n_elements(ffunct) eq 0 THEN BEGIN 
   if n_elements(pia_drift_par) gt 0 then ffunct = pia_drift_par.ffunct $
       else ffunct= funct_info.det_deffunct(ind_det)
ENDIF       

; check for staring measurements with 'chopped models'

IF tot_nchopp LT 2 AND STRPOS(ffunct,'CHO') NE -1 THEN BEGIN
	ffunct=funct_info.det_deffunct(ind_det)
	only_measpar = 0
ENDIF  


  ind_funct = where(funct_info.lst(1,*) eq ffunct,nf)
  IF nf eq 0 THEN BEGIN
    print, string(7b),'RETURNING: Function not valid'
    init_error = 2
    n_funct = n_elements(funct_info.lst(0,*))
    print, 'VALID FUNCTIONS:'
    for ifu=0, n_funct-1 do print,'                 '+funct_info.lst(1,ifu)
    RETURN
  ENDIF

; check if function using different chopper steps are consistently used
   CASE 1 of 
    strpos(ffunct,'RTCHO') NE -1: BEGIN 
     IF STRPOS(STRUPCASE(cmode),'RECTANGULAR') EQ -1 AND $ 
	STRPOS(STRUPCASE(cmode),'TRIANGULAR') EQ -1  AND $
        STRPOS(STRUPCASE(cmode),'FCS1<->FCS2') EQ -1 AND $
        STRPOS(STRUPCASE(cmode),'COFOV<->FCS') EQ -1 THEN BEGIN
        print, string(7b), $ 
         'ERROR: Measurement was not performed in rectangular chopped mode'
        init_error = 2
        RETURN
     ENDIF
     IF STRPOS(STRUPCASE(cmode),'TRIANGULAR') NE -1 THEN BEGIN 
          ind_step3 = where(step eq 3, nstep3)
	  IF nstep3 GT 0 THEN step(ind_step3) = 1
          ind_step4 = where(step eq 4, nstep4)
	  IF nstep4 GT 0 THEN step(ind_step4) = 2
     ENDIF
    END      
    strpos(ffunct,'STCHO') gt -1: BEGIN 
     IF  STRPOS(STRUPCASE(cmode),'SAWTOOTH') EQ -1 THEN BEGIN
        print, string(7b), $ 
         'ERROR: Measurement was not performed in sawtooth chopped mode'
        init_error = 2
        RETURN
     ENDIF
    END      
    strpos(ffunct,'TTCHO') gt -1: BEGIN 
     IF STRPOS(STRUPCASE(cmode),'TRIANGULAR') EQ -1 THEN BEGIN
        print, string(7b), $ 
         'ERROR: Measurement was not performed in triangular chopped mode'
        init_error = 2
        RETURN
     ENDIF
    END  
    ELSE:
    ENDCASE    
	
;
; Initializes parameter which do not depend on function
IF n_elements(pia_drift_par) eq 0 OR KEYWORD_SET(DEFAULT) THEN BEGIN 

  fract_left = funct_info.fractleft(ind_funct(0))
  minp_ext = funct_info.minpext
  mintime = funct_info.mintime(ind_funct(0))
  use_mintime =  funct_info.use_mintime
  chisqtol = funct_info.chisqtol
  weightm = funct_info.weightm(0)
  only_measpar = 0

ENDIF ELSE BEGIN 

  fract_left = pia_drift_par.fract_left
  minp_ext = pia_drift_par.minp_ext
  mintime = pia_drift_par.mintime
  use_mintime =  pia_drift_par.use_mintime
  chisqtol = pia_drift_par.chisqt
  weightm = pia_drift_par.weightm
 	
ENDELSE




;;;;;;;;;;;;;;;;;;;;;;;;;;

IF STRPOS(ffunct,'CHOP') GT -1 THEN fit_chopped = 1 ELSE fit_chopped = 0

IF STRPOS(ffunct,'TCHO') GT -1 THEN fit_allstep = 1 ELSE fit_allstep = 0

IF STRPOS(ffunct,'_RST_') GT -1 THEN fit_allrst = 1 ELSE fit_allrst =0 

nparfunc = funct_info.npar(ind_funct(0))
parfunc = funct_info.lst(3,ind_funct(0))

neff_steps = ntsteps
if fit_chopped THEN neff_steps = ntsteps/(fit_chopped+1)
if fit_allstep THEN neff_steps = max(raststeps) 
if fit_allrst THEN BEGIN 
  neff_steps = 1
  nparfunc0 = nparfunc 
  nparfunc = nparfunc0 + ntsteps - 1  
ENDIF


; if all Raster pos. are used then change the flag corresponding to NOTF 
IF fit_allrst THEN BEGIN 
    IF npix EQ 1 THEN dettmp = 0 ELSE dettmp = det 
    encode_piaflag, 'SRD', 'NOTF',slop_flg, slop_flg, DET=dettmp,/RESET
ENDIF

a_var_exp = INTARR(npix, neff_steps, nparfunc)
prange_pix = FLTARR(npix, neff_steps, 3, nparfunc)

; Initializes parameter which do not depend on measurements, but preserve
; default values if keyword ONLY_MEASPAR is set 

IF NOT only_measpar THEN BEGIN 
    a_var = funct_info.avar.(ind_funct(0))
    p_range = funct_info.prange.(ind_funct(0))
ENDIF ELSE BEGIN 
    a_var = pia_drift_par.a_var
    p_range = pia_drift_par.p_range
ENDELSE    
    
    
 IF fit_allrst AND NOT only_measpar THEN BEGIN
  istp=0
    FOR ipix=0, npix-1 DO BEGIN  
      prange_pix(ipix,istp,*,0:nparfunc0-1) = p_range(*,*)
      a_var_exp(ipix,istp,0:nparfunc0-1) = a_var
    ENDFOR
    FOR ist = 1, ntsteps-1 DO BEGIN 
      FOR ipix=0, npix-1 DO BEGIN  
        prange_pix(ipix,istp,*,nparfunc0-1+ist) = p_range(*,0)
        a_var_exp(ipix,istp,nparfunc0-1+ist) = a_var(0)   
    ENDFOR
  ENDFOR    
 ENDIF ELSE BEGIN
  FOR istp=0,neff_steps-1 DO BEGIN 
    FOR ipix=0,npix-1 DO BEGIN
     prange_pix(ipix,istp,*,*) = p_range 
     a_var_exp(ipix,istp,*) = a_var
    ENDFOR
  ENDFOR    
 ENDELSE

start_par = fltarr(neff_steps,nparfunc)


fit_par_str = { TDTNR: tmpdata.admn, det:det,			$
		mod_func:ffunct, nparams:nparfunc, 		$	
                chisqtol:chisqtol, minp_ext: minp_ext, 		$
                a_var:a_var_exp,	 			$
                fract_left:make_array(npix,value=fract_left),	$
		weightm:make_array(npix,value=weightm),		$
		use_mintime:use_mintime,			$
		mintime:make_array(npix,value=mintime),		$
		neff_steps:neff_steps,				$
		ind_steps:intarr(neff_steps+1),			$
                param:fltarr(npix,neff_steps,nparfunc),  	$
                sg_param:fltarr(npix,neff_steps,nparfunc),	$
		prange:prange_pix,			 	$
		t_ini:fltarr(npix,ntsteps),			$
		temp_ini:fltarr(npix,ntsteps),			$
		t_zero:fltarr(ntsteps),				$
		flag:intarr(npix, ntsteps),			$
                signal:fltarr(npix, ntsteps), 			$
                time_sg:fltarr(ntsteps), 			$
                err:fltarr(npix,ntsteps), 			$
                jump:fltarr(npix,ntsteps), 			$
                err_jump:fltarr(npix,ntsteps),			$
		time: float(tmpdata.stim),			$
                slop: slop*0., 					$
                slop_flag:slop_flg, 				$
                chisq:fltarr(npix, neff_steps),date_proc:''}
                

init_error = intarr(npix, neff_steps)
; check if number of points exceeds the minimum required for fitting
; the model
IF tmpdata.sldi lt minp_ext THEN BEGIN
   init_error = init_error*0 + 2
   fit_par_str.flag = make_array(npix,ntsteps,value=2)
   RETURN
ENDIF
; define time interval to be used later
delta_stim = median(tmpdata.stim(1:tmpdata.sldi-1) - $ 
       tmpdata.stim(0:tmpdata.sldi-2))




fit_par_str.ind_steps(0) = 0

; Now make estimate of initial values

CASE 1 OF 
fit_allrst: BEGIN    
; all chopper plateaux are used together, but separate per raster points
  indrst = raststeps 
  ntrst = max(raststeps)  
  ; in this case t_zero and t_ini are equivalent, except for the first
  ; step position. t_zero should be the time when the flux has changed
  FOR irpstep=1, ntrst DO BEGIN
        indtmp = WHERE(raststeps EQ irpstep, nc)
	fit_par_str.time_sg(irpstep-1) = stim(indtmp(nc-1))
	t_zero =  max([stim(indtmp(0)) - delta_stim/2.,0.])
	fit_par_str.t_ini(*,irpstep-1) = t_zero
	fit_par_str.t_zero(irpstep-1) = t_zero
	; Interpolate the temperature to t_zero
	IF nc GT 1 THEN      $  
	  temp_ini = temp(indtmp(0)) - (temp(indtmp(0)+1) - temp(indtmp(0)))/2. $
	ELSE temp_ini =  temp(indtmp(0))      
	fit_par_str.temp_ini(*,irpstep-1) = temp_ini 
  ENDFOR
    
  
  ; At the moment it discard the whole sequence. The discard per chopper plateau
  ; may be done by sigdisc
   CASE 1 OF 
      use_mintime: BEGIN
                mm_all = $ 
                 WHERE((stim - fit_par_str.t_zero(0)) GE mintime)
                mm  = mm_all(0)
               END
      ELSE: BEGIN
                mm_all = fix(fract_left/100.*nc)
                mm = mm_all(0)
               END
   ENDCASE 
     
  
  	FOR ipix=0, npix-1 DO BEGIN
  	  
  	 ;fit_par_str.prange(ipix, *, *, *) = p_range
	
	 ;check if the number of valid signal in whole raster pos. is >= minp_ext
	 ind = WHERE(slop_flg(ipix, *) le 1, nind)
	 IF nind lt minp_ext THEN BEGIN
	   fit_par_str.slop_flag(ipix,*) = 8
	   GOTO, ENDLOOP_PIX_ALL_RST  	  
         ENDIF
         
         ; now make deselection for all sequence
         ind = where(slop_flg(ipix,*) le 1, nind)
         IF nind GT 0 THEN BEGIN  
              ind0 = max([ind(0),mm]) 
              ind0 = min([ind0,nc])
              IF ind0 GT 0 THEN    $  
                   fit_par_str.slop_flag(ipix,0:ind0-1) = 8
              t_ini = stim(ind0)
	 ENDIF ELSE BEGIN
	      t_ini = stim(0)
  	 ENDELSE  
         
	 CALL_PROCEDURE, parfunc, det, stim, slop(ipix,*), slop_flg(ipix,*),  $ 
	            indpix(ipix), start_par, RASTERSTEP=raststeps, 	    $
	            ERROR=error, T_ZERO = t_ini
	            	            
  
	 init_error(ipix,0) = error
	         	 
	 fit_par_str.t_ini(ipix,0) = t_ini 
	 fit_par_str.flag(ipix, 0) = fix(init_error(ipix)/2)

	 ENDLOOP_PIX_ALL_RST:
	 CASE 1 OF 
	 init_error(ipix, 0) eq 0: BEGIN
	                fit_par_str.param(ipix, *, *) = start_par
	                ;;fit_par_str.jump(ipix, *) = jump(0,*)
	 	       END
	 init_error(ipix, 0) eq 1:  $ 
	       print,'Warning: Not proper initialization at pixel ', $	 
	           strtrim(indpix(ipix)+1,2)
	 ELSE: BEGIN
	           print,'Error: Initialization error at pixel ',             $   
	             strtrim(indpix(ipix)+1,2)
	           return
	       END    
	 ENDCASE      
	     
	 fit_par_str.param(ipix, *, *) = start_par
	 ;;fit_par_str.jump(ipix, *) = jump(0,*)
	           	        
	ENDFOR ; endloop for pixels


END
 
fit_allstep: BEGIN    
; all chopper plateaux are used together, but separate per raster points
 FOR iistep=0, max(raststeps)-1 DO BEGIN
  indrst = WHERE (raststeps EQ (iistep+1), nrst)
  minrpch = min(rpchopsteps(indrst),max=maxrpch)
  ntchop = maxrpch- minrpch + 1
  fit_par_str.ind_steps(iistep+1) = ntchop
  indrpchop = indgen(ntchop) + minrpch -1
  
  ; in this case t_zero and t_ini are equivalent, except for the first
  ; step position. t_zero should be the time when the flux has changed
  FOR irpchop=0, ntchop-1 DO BEGIN
        indtmp = WHERE(rpchopsteps EQ minrpch+irpchop, nc)
	fit_par_str.time_sg(minrpch+irpchop-1) = stim(indtmp(nc-1))
	t_zero =  max([stim(indtmp(0)) - delta_stim/2.,0.])
	fit_par_str.t_ini(*,minrpch+irpchop-1) = t_zero
	fit_par_str.t_zero(minrpch+irpchop-1) = t_zero
	; Interpolate the temperature to t_zero
	IF nc GT 1 THEN      $  
	  temp_ini = temp(indtmp(0)) - (temp(indtmp(0)+1) - temp(indtmp(0)))/2. $
	ELSE temp_ini =  temp(indtmp(0))      
	fit_par_str.temp_ini(*,minrpch+irpchop-1) = temp_ini 
  ENDFOR
    
  
  ; At the moment it discard the whole sequence. The discard per chopper plateau
  ; may be done by sigdisc
   CASE 1 OF 
      use_mintime: BEGIN
                mm_all = $ 
                 WHERE((stim(indrst)- fit_par_str.t_zero(minrpch-1)) GE mintime)
                mm  = mm_all(0)
               END
      ELSE: BEGIN
                mm_all = fix(fract_left/100.*nc)
                mm = mm_all(0)
               END
   ENDCASE 
     
	indrpchop_all = indrst 	
  
  	FOR ipix=0, npix-1 DO BEGIN
  	  
  	 ;fit_par_str.prange(ipix, iistep, *, *) = p_range
	
	 ;check if the number of valid signal in whole raster pos. is >= minp_ext
	 ind = WHERE(slop_flg(ipix, indrst) le 1, nind)
	 IF nind lt minp_ext THEN BEGIN
	   fit_par_str.slop_flag(ipix,indrst) = 8
	   GOTO, ENDLOOP_PIX_ALL  	  
         ENDIF
         
         ; now make deselection for all sequence
         ind = where(slop_flg(ipix,indrst) le 1, nind)
         IF nind GT 0 THEN BEGIN  
              ind0 = max([ind(0),mm]) 
              ind0 = min([ind0,nc])
              IF ind0 GT 0 THEN    $  
                   fit_par_str.slop_flag(ipix,indrst(0:ind0-1)) = 8
              t_ini = stim(indrst(ind0))
	 ENDIF ELSE BEGIN
	      t_ini = stim(indrst(0))
  	 ENDELSE  
         
	 CALL_PROCEDURE, parfunc, det, stim(indrpchop_all), 		    $ 
	   	    slop(ipix,indrpchop_all), slop_flg(ipix,indrpchop_all), $ 
	            indpix(ipix), start_par, step=step(indrpchop_all), 	    $
	            ERROR=error, T_ZERO = t_ini, SG_JUMP = jump
	            	            
  
	 init_error(ipix, iistep) = error(0)
	         	 
	 fit_par_str.t_ini(ipix,minrpch-1) = t_ini 
	 fit_par_str.flag(ipix, indrpchop) = fix(init_error(ipix,iistep)/2)

	 ENDLOOP_PIX_ALL:
	 CASE 1 OF 
	 init_error(ipix, iistep) eq 0: BEGIN
	                fit_par_str.param(ipix, iistep, *) = start_par
	                fit_par_str.jump(ipix, indrpchop) = jump(0,*)
	 	       END
	 init_error(ipix, iistep) eq 1:  $ 
	       print,'Warning: Not proper initialization at pixel ', $	 
	           strtrim(indpix(ipix)+1,2)+' - Step ',strtrim(iistep+1,2)
	 ELSE: BEGIN
	           print,'Error: Initialization error at pixel ',             $   
	             strtrim(indpix(ipix)+1,2)+' - Step ',strtrim(iistep+1,2)
	           return
	       END    
	 ENDCASE      
	     
	 fit_par_str.param(ipix, iistep, *) = start_par
	 fit_par_str.jump(ipix, indrpchop) = jump(0,*)
	           	        
	ENDFOR ; endloop for pixels


 ENDFOR ; ENDLOOP for raster positions

END 

; single chopper plateau or at maximum pairs of them
ELSE: BEGIN   
 FOR irpchop=0, ntsteps-1, (1+fit_chopped) DO BEGIN
  
   iistep = irpchop/(fit_chopped+1)

   indrpchop = WHERE (rpchopsteps EQ (irpchop+1), nc)
   
   fit_par_str.ind_steps(iistep) = irpchop+1
   
   IF fit_chopped THEN BEGIN 
      indrpchop2 = WHERE(rpchopsteps EQ (irpchop+2), nc2) 
      ;check wether the next step is a valid one in the case of chopped 
      ; fit the last irpchop can be an odd number 
      fit_par_str.ind_steps(iistep) = irpchop+2
      if nc2 eq 0 then goto, NO_SIG_RASTER 
   ENDIF   
                
   ; set time for the extrapolated  signal  as the ending time of the 
   ; chopper plateau 
   fit_par_str.time_sg(irpchop) = stim(indrpchop(nc-1)) 

   ; set initial time as the beginning of the chopper plateau
   ; check for ON-target flag (it has to be checked when flag system is used)
   ;indflg = where(slop_flg(0,indrpchop) lt 2)
   ;tonflg = stim(indrpchop(indflg(0)))
   ;t_zero = max([stim(indrpchop(0)) - Delta_stim/2.,0.,tonflg])
   t_zero = max([stim(indrpchop(0)) - Delta_stim/2.,0.])
   fit_par_str.t_zero(irpchop) = t_zero
   
   CASE 1 OF 
      use_mintime: BEGIN
                mm_all = WHERE((stim(indrpchop)- t_zero) GE mintime)
                mm  = mm_all(0)
               END
      ELSE: BEGIN
                mm_all = fix(fract_left/100.*nc)
                mm = mm_all(0)
               END
   ENDCASE   
   
      
   IF fit_chopped THEN BEGIN 
     CASE 1 OF 
        use_mintime: BEGIN
                t_zero2 = stim(indrpchop2(0)) - Delta_stim/2.
                mm_all = WHERE((stim(indrpchop2)- t_zero2) GE mintime)
                mm2 = mm_all(0)
               END
        ELSE: BEGIN
                mm_all = fix(fract_left/100.*nc2)
                mm2 = mm_all(0)
               END
     ENDCASE   
      fit_par_str.time_sg(irpchop) = stim(indrpchop2(nc2-1)) 
   ENDIF
	 

	FOR ipix=0, npix-1 DO BEGIN
		
	 ;fit_par_str.prange(ipix,iistep, *,*) = p_range

         ind = where(slop_flg(ipix,indrpchop) le 1, nind) 
         IF nind GT 0 THEN BEGIN
           ind0 = max([ind(0),mm]) 
           ind0 = min([ind0,nc])
           IF ind0 ge 1 THEN fit_par_str.slop_flag(*,indrpchop(0:ind0-1)) = 8
         ENDIF ELSE ind0 = 0 
             
         nup = nc - ind0 
         IF nup ge minp_ext THEN BEGIN
             t_ini = stim(indrpchop(ind0))
             temp_ini = temp(indrpchop(ind0))
	     indrpchop_all = indrpchop(ind0:*) 
	     cstep = step(indrpchop(ind0:*))
	 ENDIF ELSE BEGIN
	     t_ini = stim(indrpchop(0))
	     temp_ini = temp(indrpchop(0))
	     init_error(ipix, iistep) = 1
	     GOTO, ENDLOOP_PIX
	 ENDELSE  
	   
	
	 IF fit_chopped THEN BEGIN 
	 
           ind2 = where(slop_flg(ipix,indrpchop2) le 1, nind2) 
           ind20 = max([ind2(0),mm2]) 
           ind20 = min([ind20,nc2])
           if ind20 gt 1 then $ 
               fit_par_str.slop_flag(*,indrpchop2(0:ind20-1)) = 8
           nup2 = nc2 - ind20
	   IF nup2 ge minp_ext THEN BEGIN
               t_ini = [t_ini, stim(indrpchop2(ind20))]
               temp_ini = [temp_ini, temp(indrpchop2(ind20))]
	       indrpchop_all = [indrpchop_all,indrpchop2(ind20:*)]
	       cstep = [cstep, step(indrpchop2(ind20:*))] 
	   ENDIF ELSE BEGIN 
               t_ini = [t_ini, stim(indrpchop2(0))]
               temp_ini = [temp_ini, temp(indrpchop2(0))]
	       init_error(ipix, iistep) = init_error(ipix, iistep) + 1
	       GOTO, ENDLOOP_PIX
	   ENDELSE      

	 ENDIF  ; end of chopped 
          
         
	 fit_par_str.t_ini(ipix,irpchop:irpchop+fit_chopped) = 	$
	       t_ini
	 fit_par_str.temp_ini(ipix,irpchop:irpchop+fit_chopped) =  $
	       temp_ini
         
         
	 IF fit_chopped THEN  $	  
	   CALL_PROCEDURE, parfunc, det, stim(indrpchop_all), 		 $ 
	   	    slop(ipix,indrpchop_all), slop_flg(ipix,indrpchop_all),$ 
	            indpix(ipix), start_par, step=step(indrpchop_all), 	 $
	            ERROR=error, T_ZERO = t_zero			$ 
	   ELSE CALL_PROCEDURE, parfunc, det, stim(indrpchop_all),        $
	            slop(ipix,indrpchop_all), slop_flg(ipix,indrpchop_all), $ 
	            indpix(ipix), start_par, ERROR=error, T_ZERO = t_zero
	            
	 init_error(ipix, iistep) = error(0)        
	
	 ENDLOOP_PIX:
	 IF init_error(ipix, iistep) eq 0 THEN 				$
	          fit_par_str.param(ipix, iistep, *) = start_par  	$
	    ELSE print,' Initialization error at pixel ', 		$ 
	           strtrim(indpix(ipix)+1,2)+' - Step ',strtrim(iistep+1,2)
	 
	 fit_par_str.flag(ipix, irpchop:irpchop+fit_chopped) = $ 
	        fix((init_error(ipix,iistep)+1)/2)*2
	 fit_par_str.t_ini(ipix,irpchop:irpchop+fit_chopped) = t_ini
	 fit_par_str.temp_ini(ipix,irpchop:irpchop+fit_chopped) = temp_ini
	
	 
	ENDFOR 		
		   
   NO_SIG_RASTER:
 
 ENDFOR
 
 END
 
ENDCASE 

RETURN
END
