;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKSC_CHOP_DRIFT
;
; PURPOSE:
;	Calculate the temperature independent Fouks function for rectangular
;		chopped case and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKSC_CHOP_DRIFT, t, a, step, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;	step:	Chopper step number 
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Perform operation in double precision to
;	avoid underflows  (J.Acosta)		  October 1996
;	Version 5.0 - Aug.96
;			
;-
PRO FOUKSC_CHOP_DRIFT, t, a, fct, pder, error, FINAL=final, STEP=step

COMMON PIA_DRIFT_PAR, drift_par
COMMON TEMPERATURE, time_Temp, Temp

error =0

	p_field = drift_par.p_field
	s_field = drift_par.s_field
	a_var = drift_par.a_var
	t_ini = drift_par.t_ini
	  
; A(0) = S_inf1 
; A(1) = S_ini1
; A(2) = Beta
; A(3) = E_j / Zeta
; A(4) = S_inf2
; A(5) = S_ini2


; Define aa from variable and fixed parameters a
	aa = p_field
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF 

	xele = N_ELEMENTS(t)
	nvar = N_ELEMENTS(a)
	
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	s_inf = [aa(0),aa(4)]
	s_ini = [aa(1),aa(5)]
	beta = aa(2)
	emod_j = aa(3)
	
	step_dif = step(1:xele-1) - step(0:xele-2)
	indst = WHERE(step_dif NE 0, ndif)
	IF ndif EQ 0 THEN BEGIN
		MESSAGE, /INFO, 'Not valid chopped sample'
		RETURN
	ENDIF

	indst = [0,indst+1]
	pos_sinf = s_inf
	
	bsinf = fct
	sini = fct
	sinf = fct
	deltat = fct
	
	indst1 = WHERE(step EQ 1,n1)
	deltat(indst1) = t(indst1)-t_ini(0)
	bsinf(indst1) = beta * s_inf(0)
	sini(indst1) = s_ini(0)
	sinf(indst1) = s_inf(0)
	indst2 = WHERE(step EQ 2,n2)
	deltat(indst2) = t(indst2)-t_ini(1)
	bsinf(indst2) = beta * s_inf(1)
	sini(indst2) = s_ini(1)
	sinf(indst2) = s_inf(1)
	
	teta = emod_j / bsinf



; Avoid underflows
	quot = deltat/teta
	;ind = WHERE(quot LT 80., nq)
	;Za = quot * 0.  
	;IF nq GT 0 THEN Za(ind) = exp(-quot(ind))
	Za = exp(-double(quot))

	Zc = Sini - beta*Sinf
	Zb = Zc + (Sinf - Sini) * Za

	ddfct = beta * Sinf +   $
	     ( (1.- beta) * Zc * sinf / Zb) 
	     
	fct = float(ddfct)
	
; do check of beta parameter, avoid pole [S_0 - Beta * S >0]
        beta_check = S_ini(0) - beta*fct(indst1)
        beta_check = [beta_check, S_ini(1) - beta*fct(indst2)]
        ind_beta = where(beta_check lt 0., nbeta)
        if nbeta gt 0 then error = 1 else error = 0
 
  ; do check of Temperature 
        err_temp = check_temp(time_Temp, Temp, S_inf(0), emod_j, beta, 0, 0)/2
        err_temp1 = check_temp(time_Temp, Temp, S_inf(1), emod_j, beta, 0, 0)/2
        error = error + max([err_temp,err_temp1])*2
 
  ;print,'Error in Fouksc ',error
		
		 	
; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

	IF N_PARAMS() LT 4 THEN RETURN
; S_inf
	k = 0
	IF a_var(0) THEN BEGIN
	 pder(indst1,k) = float(beta + (1.-beta) * Zc(indst1)/Zb(indst1) * $
		   (1.-beta/Zc(indst1)*S_inf(0)-S_inf(0)/Zb(indst1)*$
		   (Za(indst1)-beta-(S_inf(0)-S_ini(0))*deltat(indst1)*$
		   Za(indst1)/S_inf(0)/teta(indst1))))
	 pder(indst2,k) = 0.
	 k = k+1
	ENDIF

; S_ini
	IF a_var(1) THEN BEGIN
	 pder(indst1,k) = float((1.-beta) * S_inf(0)/Zb(indst1) * $
	 	(1.-Zc(indst1)/Zb(indst1)*(1.-Za(indst1))))
	 pder(indst2,k) = 0.
	 k = k+1
	ENDIF
; beta
	IF a_var(2) THEN BEGIN
	 pder(indst1,k) = float(S_inf(0) / Zb(indst1) * $
		(Zb(indst1) - Zc(indst1) - $
		(1.-beta) * (S_inf(0)-Zc(indst1)/Zb(indst1)* $
		(S_inf(0) + (S_inf(0)-S_ini(0))* deltat(indst1)/teta(indst1)/$
		beta*Za(indst1)))))
	 pder(indst2,k) = float(S_inf(1) / Zb(indst2) * $
		(Zb(indst2) - Zc(indst2) - $
		(1.-beta) * (S_inf(1)-Zc(indst2)/Zb(indst2)* $
		(S_inf(1) + (S_inf(1)-S_ini(1))* deltat(indst2)/teta(indst2)/$
		beta*Za(indst2)))))
	 k = k+1
	ENDIF	
	
; E_j 
	IF a_var(3) THEN BEGIN
	
	 pder(*,k) = float(-(1.-beta)*Sinf*Zc/Zb*(Sinf-Sini) * Za/Zb  $
	              *deltat/Emod_j/teta)
	 k = k+1

	ENDIF	    
; S_inf
	IF a_var(4) THEN BEGIN
	 pder(indst2,k) = float(beta + (1.-beta) * Zc(indst2)/Zb(indst2) * $
		   (1.-beta/Zc(indst2)*S_inf(1)-S_inf(1)/Zb(indst2)*$
		   (Za(indst2)-beta-(S_inf(1)-S_ini(1))*deltat(indst2)*$
		   Za(indst2)/S_inf(1)/teta(indst2))))
	 pder(indst1,k) = 0.
	 k = k+1
	ENDIF

; S_ini
	IF a_var(5) THEN BEGIN
	 pder(indst2,k) = float((1.-beta) * S_inf(1)/Zb(indst2) * $
	 	(1.-Zc(indst2)/Zb(indst2)*(1.-Za(indst2))))
	 pder(indst1,k) = 0.
	 k = k+1
	ENDIF

        RETURN
        END	 	
	 
