;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKSC_DRIFT
;
; PURPOSE:
;	Calculate the temperature independent Fouks function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKSC_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	ERROR:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	E_j is taken normalized to Temp = 3.K
;	Version 5.0 - Aug.96
;			
;-
PRO FOUKSC_DRIFT, t, a, fct, pder, error, FINAL=final

COMMON PIA_DRIFT_PAR, drift_par
COMMON TEMPERATURE, time_Temp, Temp

	p_field = drift_par.p_field
	s_field = drift_par.s_field
	a_var = drift_par.a_var
	t_ini = drift_par.t_ini
	  
; A(0) = S_inf 
; A(1) = S_ini
; A(2) = Beta
; A(3) = E_j / Zeta * sqrt(Temp_ini / 3.K)



; Define aa from variable and fixed parameters a
	aa = p_field
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF 

	xele = N_ELEMENTS(t)
	nvar = N_ELEMENTS(a)
	
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	

	s_inf = aa(0)
	s_ini = aa(1)
	beta = aa(2) 
	emod_j = aa(3) * sqrt(drift_par.Temp_ini/ 3.)
	
;	IF (beta * s_inf) EQ 0. THEN bsinf = 1.e-10 ELSE 
	bsinf=beta * s_inf
	teta = emod_j / bsinf 
	deltat = t - t_ini


; Avoid underflows
	quot = deltat/teta
	Za = exp(-double(quot))
	Zc = S_ini - beta*S_inf
	Zb = Zc + (S_inf - S_ini) * Za

	ddfct = beta * S_inf +   $
	     ( (1.- beta) * Zc * s_inf / Zb) 
	     
	fct = float(ddfct)     
	

  ; do check of beta parameter, avoid pole [S_0 - Beta * S >0]
	beta_check = S_ini - beta*fct
	ind_beta = where(beta_check lt 0., nbeta)
	if nbeta gt 0 then error = 1 else error = 0
	
  ; do check of Temperature 
        err_temp = check_temp(time_Temp, Temp, S_inf, emod_j, beta, 0, 0)/2	
        error = error + err_temp*2
	
  ;print,'Error in Fouksc ',error
	
		 	
; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

	IF N_PARAMS() LT 4 THEN RETURN
; S_inf
	k = 0
	IF a_var(0) THEN BEGIN
	 pder(*,k) = float(beta + (1.-beta) * Zc/Zb * $
		   (1.-beta/Zc*S_inf-S_inf/Zb*$
		   (Za-beta-(S_inf-S_ini)*deltat*Za/S_inf/teta)))
	 k = k+1
	ENDIF

; S_ini
	IF a_var(1) THEN BEGIN
	 pder(*,k) = float((1.-beta) * S_inf/Zb * (1.-Zc/Zb*(1.-Za)))
	 k = k+1
	ENDIF
; beta
	IF a_var(2) THEN BEGIN
	 pder(*,k) = float(S_inf / Zb * $
		(Zb - Zc - $
		(1.-beta) * (S_inf-Zc/Zb* $
		(S_inf + (S_inf-S_ini)* deltat/teta/beta*Za))))
	 k = k+1
	ENDIF	
	
; E_j 
	IF a_var(3) THEN $
	 pder(*,k) = float(-(1.-beta)*S_inf*Zc/Zb*(S_inf-S_ini)*Za/Zb $
		    *deltat/Emod_j/teta)
		    

        RETURN
        END	 	
	 
