;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKSC_RTCHO_DRIFT
;
; PURPOSE:
;	Calculate the temperature independent Fouks function for rectangular
;		chopped case and derivatives. It includes all chopper 
;		steps and compute the signal jump based on final values.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKSC_RTCHO_DRIFT, t, a, step, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;	step:	Chopper step number 
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Perform operation in double precision to
;	avoid underflows  (J.Acosta)		  October 1996
;	Version 5.0 - Aug.96
;			
;-
PRO FOUKSC_RTCHO_DRIFT, t, a, fct, pder, error, FINAL=final

ON_ERROR, 2

COMMON PIA_DRIFT_PAR, drift_par
COMMON TEMPERATURE, time_Temp, Temp
COMMON TCHO_CMN, step, t_zero

error =0
	  
; A(0) = S_inf1 
; A(1) = S_inf2
; A(2) = Beta
; A(3) = E_j1 / Zeta
; A(4) = E_j2 / Zeta  
; A(5) = S_ini0

	IF N_ELEMENTS(t_zero) EQ 0 THEN $ 
	     MESSAGE,'T_ZERO has to be specified in this mode'
	
	IF NOT KEYWORD_SET(FINAL) THEN BEGIN
	 aa = drift_par.p_field
	 a_var = drift_par.a_var
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF ELSE BEGIN 
	  aa = a
	ENDELSE  

	xele = N_ELEMENTS(t)
	nvar = N_ELEMENTS(a)
	
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	s_inf = [aa(0), aa(1)]
	beta = aa(2)
	emod_j = [aa(3), aa(4)]
	s_ini0 = aa(5)
	
	;indst = [0,indst+1]
	;pos_sinf = s_inf
	
	bsinf = fct
	sini = fct
	sinf = fct
	deltat = fct
	teta = fct
	
	get_nchoppsweep,step, nchopp, ind_sweep, nchopstep
	ind_sweep = [-1,ind_sweep]
	IF nchopp LT 2 THEN BEGIN
		MESSAGE, /INFO, 'Not valid chopped sample'
		RETURN
	ENDIF
	
	
	stepref = [step(ind_sweep(1)),step(ind_sweep(2))]
	
	
	FOR ichop = 0, nchopp-1 DO BEGIN
	        ind_ichop = WHERE(nchopstep eq ichop+1)
	        istep = step(ind_ichop(0))
	        
		; isteppar is the index of sinf values
		isteppar = WHERE(stepref EQ istep)
		isteppar = isteppar(0)

		; assing Sinf to part of the complete Sinf array
		Sinf(ind_ichop) = s_inf(isteppar)

		; compute deltat for all time within chopper step
		deltat(ind_ichop) = t(ind_ichop) - t_zero(ichop)

		; To compute initial value take first value for first step,
		; otherwise apply jump condition, using beta
		IF ichop eq 0 THEN Sini(ind_ichop) = S_ini0 	$
		  ELSE Sini(ind_ichop) = Sfincur + 	$
		        beta*(S_inf(isteppar)-S_infpre)
		        
		bsinf(ind_ichop) = beta * s_inf(isteppar)

		; assing time constant to part of the complete teta array        
		teta(ind_ichop) = emod_j(isteppar) / bsinf(ind_ichop)
		
		;compute final value for current chopper plateau, except for 
		; the last chopper step
		; t_zero should be used here, instead of tini
		IF ichop lt nchopp-1 THEN BEGIN 
		  quoti = (t_zero(ichop+1)-t_zero(ichop))/teta(ind_ichop(0))
		  sfincur = float(bsinf(ind_ichop(0)) + 		$ 
		     (1-beta)*(sini(ind_ichop(0))-bsinf(ind_ichop(0)))* 	$
		     sinf(ind_ichop(0))/(sini(ind_ichop(0))-bsinf(ind_ichop(0))+$
		     (sinf(ind_ichop(0))-sini(ind_ichop(0)))*exp(-quoti)))
		ENDIF     
		
		; reassing S_inf[n] to S_inf[n-1] for next chopper step
		S_infpre = S_inf(isteppar)
		
	ENDFOR	
	
	indst1 = WHERE(step EQ stepref(0),n1)
	indst2 = WHERE(step EQ stepref(1),n2)
	

; Avoid underflows
	quot = deltat/teta
	;ind = WHERE(quot LT 80., nq)
	;Za = quot * 0.  
	;IF nq GT 0 THEN Za(ind) = exp(-quot(ind))
	Za = exp(-double(quot))

	Zc = Sini - beta*Sinf
	Zb = Zc + (Sinf - Sini) * Za

	ddfct = beta * Sinf +   $
	     ( (1.- beta) * Zc * sinf / Zb) 
	     
	fct = float(ddfct)
	
; do check of beta parameter, avoid pole [S_0 - Beta * S >0]
;        beta_check = S_ini(0) - beta*fct(indst1)
;        beta_check = [beta_check, S_ini(1) - beta*fct(indst2)]
;        ind_beta = where(beta_check lt 0., nbeta)
;        if nbeta gt 0 then error = 1 else error = 0
	error = 0
 
  ; do check of Temperature 
;        err_temp = check_temp(time_Temp, Temp, S_inf(0), emod_j, beta, 0, 0)/2
;        err_temp1 = check_temp(time_Temp, Temp, S_inf(1), emod_j, beta, 0, 0)/2
;        error = error + max([err_temp,err_temp1])*2
 
  ;print,'Error in Fouksc ',error
		
		 	
; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

	IF N_PARAMS() LT 4 THEN RETURN

	k = 0
; S_infty1
	IF a_var(0) THEN BEGIN
	 pder(indst1,k) = float(beta + (1.-beta) * Zc(indst1)/Zb(indst1) * $
		   (1.-beta/Zc(indst1)*S_inf(0)-S_inf(0)/Zb(indst1)*$
		   (Za(indst1)-beta-(S_inf(0)-Sini(indst1))*deltat(indst1)*$
		   Za(indst1)/S_inf(0)/teta(indst1))))
	 pder(indst2,k) = 0.
	 k = k+1
	ENDIF

; S_infty2
	IF a_var(1) THEN BEGIN
	 pder(indst2,k) = float(beta + (1.-beta) * Zc(indst2)/Zb(indst2) * $
		   (1.-beta/Zc(indst2)*S_inf(1)-S_inf(1)/Zb(indst2)*$
		   (Za(indst2)-beta-(S_inf(1)-Sini(indst2))*deltat(indst2)*$
		   Za(indst2)/S_inf(1)/teta(indst2))))
	 pder(indst1,k) = 0.
	 k = k+1
	ENDIF

; beta
	IF a_var(2) THEN BEGIN
	 pder(indst1,k) = float(S_inf(0) / Zb(indst1) * $
		(Zb(indst1) - Zc(indst1) - $
		(1.-beta) * (S_inf(0)-Zc(indst1)/Zb(indst1)* $
		(S_inf(0) + (S_inf(0)-Sini(indst1))* deltat(indst1)/teta(indst1)/$
		beta*Za(indst1)))))
	 pder(indst2,k) = float(S_inf(1) / Zb(indst2) * $
		(Zb(indst2) - Zc(indst2) - $
		(1.-beta) * (S_inf(1)-Zc(indst2)/Zb(indst2)* $
		(S_inf(1) + (S_inf(1)-Sini(indst2))* deltat(indst2)/teta(indst2)/$
		beta*Za(indst2)))))
	 k = k+1
	ENDIF	
	
; E_j1 
	IF a_var(3) THEN BEGIN
	
	 pder(indst1,k) = float(-(1.-beta)*S_inf(0)*Zc(indst1)/Zb(indst1)* $ 
	             (S_inf(0)-Sini(indst1)) * Za(indst1)/Zb(indst1)  $
	              *deltat(indst1)/Emod_j(0)/teta(indst1))
	 pder(indst2,k) = 0.
	 k = k+1

	ENDIF	    

; E_j2 
	IF a_var(4) THEN BEGIN
	
	 pder(indst2,k) = float(-(1.-beta)*S_inf(1)*Zc(indst2)/Zb(indst2)* $ 
	             (S_inf(1)-Sini(indst2)) * Za(indst2)/Zb(indst2)  $
	              *deltat(indst2)/Emod_j(1)/teta(indst2))
	 pder(indst1,k) = 0.
	 k = k+1

	ENDIF	    

; S_ini0
	IF a_var(5) THEN BEGIN
	 pder(*,k) = 0.
	 IF ind_sweep(1) GT 0 THEN indst0 = indgen(ind_sweep(1))   $
	   ELSE indst0 = ind_sweep(1)
	 pder(indst0,k) = float((1.-beta) * S_inf(0)/Zb(indst0) *  $
	 	(1.-Zc(indst0)/Zb(indst0)*(1.-Za(indst0))))
	ENDIF

        RETURN
        END	 	
	 
