;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKSC_STCHO_start_par
;
; PURPOSE:
;	Calculate start parameters for the fouks model for sawtooth 
;	chopped measurements. It includes all chopper steps and 
;	compute the signal jump based on final values.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKSC_STCHO_start_par, det, time, slop, flag, pixel, start_par, $
;		ERROR=error, STEP=step, SG_JUMP = sg_jump
;
; INPUT:
;	det:	PHT detector ('P1', 'C2', ...)
;	slop:	Signal slopes array
;	pixel:	Pixel to be used
;
; KEYWORD PARAMETERS:
;	ERROR:	Results of computation
;	STEP:	Chopper step array
;	
; OUTPUT:
;	start_par: The start parameters
;	
; COMMON BLOCKS:
;	fouks_start_par 
;		
; PROCEDURES USED:
;       PIA LIB:
;               s_ini_final
;	Astrolib:
;		readcol
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (VILSPA)	  	  May 1997
;	V6.2
;	Starting values for final fluxes come from
;	all chopper plateaux corresponding to a given 
;	step. Jumps are determined in a rough way (JACOSTA)  August 1997
;	v6.4
;			
;-
PRO fouksc_stcho_start_par, det, time, slop, flag, pixel, start_par,  $
    error=error, step=step, T_ZERO = t_zero, SG_JUMP = sg_jump 

COMMON fouks_start_par, s_beta, s_C, s_Nd, det_par

; For simplicity we assume that steps 2 and 4, which are on the centre
; of the FOV will be the same final value
; A(0) = S_inf1 
; A(1) = S_inf2
; A(2) = S_inf3
; A(3) = Beta
; A(4) = E_j1 / Zeta
; A(5) = E_j2 / Zeta  
; A(6) = E_j3 / Zeta  
; A(7) = S_ini0
	
	error = 1	      

	get_nchoppsweep,FIX(step), nchopp, ind_sweep
	ind_sweep = [-1,ind_sweep]
	IF nchopp EQ 0 or max(step) ne 3 THEN BEGIN
		MESSAGE, /INFO, 'Not valid chopped sample'
		RETURN
	ENDIF
	
	tstep = step
	
	; signal 1 will the corresponding to step at first chopper plateau
	step1 = FIX(tstep(0))


	; Initialize  parameter depending on detector
	start_par = fltarr(8)
	
	IF N_ELEMENTS(det_par) EQ 0 THEN det_par = det
; read data from driftpar file in case is not in
	IF N_ELEMENTS(s_beta) EQ 0 OR (det_par NE det) THEN BEGIN
	   CASE det OF 
	    'P2': filepar= !CALG+'fouks_P2_par.txt'
	    'SL': filepar = !CALG+'fouks_SL_par.txt'
	    ELSE: filepar= !CALG+'fouks_P2_par.txt'
	   ENDCASE        
	   det_par = det  
	   readcol, filepar,pix, s_beta, s_C, s_ND, f='i,f,d,d'       
	ENDIF

	; Set parameters according to selected detector
	CASE det of
	   'SL': BEGIN
	     tmp_ND = s_ND(pixel)
	     tmp_C = s_C(pixel)
	     tmp_beta = s_beta(pixel)
	     IF pixel LT 64 THEN  tmp_Z = 8.052  ELSE tmp_Z = 9.644
	    END
	    ELSE: BEGIN
	     tmp_ND = s_ND(0)
	     tmp_C = s_C(0)
	     tmp_beta = s_beta(0)
	     tmp_Z = 10.
	    END
	ENDCASE
	      A = 1.949e-11
	


	;estimate initial signal of all sequence   
	nele_chop0 = ind_sweep(1) + 1 
	ind_ichop0 = indgen(nele_chop0)  
	s_ini_final, time(ind_ichop0), slop(ind_ichop0), flag(ind_ichop0), $ 
	    S_ini0, S_fin0, error=err_ini0  ;;, t_zero = t_zero(0)
	    
	;estimate final signal for beams 1, 2 and 3 using all chopper plateaux
	;corresponding to each step

	err_ini1 = 0
	s_fin = fltarr(3)
	FOR jstep = 0, 2 DO BEGIN
	  indpar = where(tstep eq step1+jstep mod 3)
	  ind_valj = where(flag(indpar) lt 2, nvalj)
	  CASE 1 of 
	    nvalj GT 3: s_fin(jstep) = MEDIAN(slop(indpar(ind_valj)))
	    nvalj GT 0: s_fin(jstep) = AVG(slop(indpar(ind_valj)))
	    ELSE: BEGIN 
	                s_fin(jstep) = 1.e-4
	                err_ini1 = 1
	          END
	  ENDCASE
	ENDFOR	     
 
	error = err_ini0 +err_ini1 		
		
	; compute signal jumps
	indval0  = where(flag(ind_sweep(0)+1:ind_sweep(1)) LT 2,nval0)
	IF nval0 GT 0 THEN s_cho0 = avg(slop(indval0))  $
	      ELSE s_cho0 = 1.e-5  
	sig_jump = fltarr(nchopp)
	sig_jump(0) = s_cho0       
	FOR ichop = 1, nchopp-1 DO BEGIN
	  indvali = where(flag(ind_sweep(ichop-1)+1:ind_sweep(ichop)) LT 2,nvali)
	  IF nvali GT 0 THEN s_choi = avg(slop(indvali))  $
	     ELSE s_choi =  1.e-5
	  sig_jump(ichop) = (s_choi - s_cho0) / tmp_beta  + sig_jump(ichop-1)
	ENDFOR  
	     	
	        
	S_ini0 = max([S_ini0,S_Fin(0)/tmp_beta*1.1,1.e-3])
	      	      
	Ej = s_fin * 0.      
	FOR jstep = 0, 2 DO BEGIN 
	  ; avoid negative values
	  S_fin(jstep) = max([1.e-5, S_fin(jstep)])
	  logarg = alog(tmp_Nd/s_fin(jstep)/tmp_C)      
	      IF logarg GT 0. THEN Ej(jstep) = sqrt(A*tmp_Nd/logarg)   $
	         ELSE BEGIN  ; set  tmp_C= 1.013e13
	           Ej(jstep) = sqrt(A*tmp_Nd/alog(tmp_Nd/s_infty1/1.013e13))
	           print,'Using a dummy value for Ej'+strtrim(jstep+1,2)
	         ENDELSE
	ENDFOR           
	      
                                            
	start_par(0:2) = S_fin

	start_par(3) = tmp_beta
	      
	start_par(4:6) = Ej / tmp_Z
	    
	start_par(7) = S_ini0 
	sg_jump = fltarr(2,nchopp)
	sg_jump(0,*) = sig_jump 
	
	
	 
	 RETURN
	 END
