;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKST_CHOP_DRIFT
;
; PURPOSE:
;	Calculate the temperature dependent Fouks function and its derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKST_CHOP_DRIFT, t, a, fct, pder, error, FINAL=final, STEP=step
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;	therm_int
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Correction of bugs and include a new way to compute 
;	integrals  (J.Acosta)					  October 1996
;	Version 5.0 - Aug.96
;			
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO THERM_INT, t, mod_par, Zattemp, Zat, Zatemp,  $ 
     Zattemp_st, Zat_st, Zatemp_st, time_st
; This routine calculate the integral A_INT, B_INT, and C_INT, in a much
; simpler way to save computing time. 
; Steps:
;    1- Divide into several intervals, using a log time scale
;    2- Compute the integral for each interval. It uses the result of previous 
; 	steps. 
;    3- Interpolate the result to the required input time (t) and returns.
;
;	Notice that the time scale time_TEMP includes all points per chopper
;	plateau, while t is only of the valid signal data points. 
;
; Tauj= mod_par(0)
; Tj = mod_par(1)
;
  
COMMON TEMPERATURE, time_Temp, Temp

t0_norm = time_Temp(0) / mod_par(0) 
time_Temp_norm = time_Temp / mod_par(0) - t0_norm  ; offset time axis to t_(n-1)
Temp0_norm = Temp(0) / mod_par(1)
Temp_norm = Temp / mod_par(1)

 mfactor = 1.5
 
; Compute the number of steps
	; First step =  4 * time_reset
 	t1 = (time_Temp_norm(1)-time_Temp_norm(0))*4. 
	tf = max(time_Temp_norm)
	Nsteps = ROUND(alog10(tf/t1)/alog10(mfactor)) + 1

; Now perform integrals for each step
Zattemp_st = fltarr(Nsteps)
Zat_st = fltarr(Nsteps)
Zatemp_st = fltarr(Nsteps)
time_st = time_Temp_norm(0)

Nint_tab = 20  ;number of points to subdivide the i-step to compute integral
MIN_NPTS = 5   ; minimum number of points which are in the subinterval
time_i1 = time_Temp_norm(0)

	FOR istep = 0, Nsteps-1 DO BEGIN
	  time_i0 = time_i1 
	  time_i1 = t1*mfactor^istep 
	  ; divide the interval into small pieces
	  time_i = time_i0 + indgen(Nint_tab)*(time_i1-time_i0)/(Nint_tab-1)
	  ; select time_temp in the range [time_i0,time_i1]
	  ind_i =  $
	    where(time_Temp_norm ge time_i0 and time_Temp_norm le time_i1,nind)
	  CASE 1 OF 
	    nind gt MIN_NPTS: BEGIN   ; fit Temp_norm to a polynom of degree 3
	                   coef3 = poly_fit(time_Temp_norm, Temp_norm, 3)
	                   temp_i = poly(time_i, coef3)
	    		END
	    nind gt 0: temp_i = interpol(Temp_norm, time_Temp_norm, time_i)
	    else: BEGIN 
	    		print, 'Not enough points in the interval '$ 
	    		   +strtrim(istep+1,2)
	    		goto, not_pts 
	    	  END	  
	  ENDCASE     
	
	  ; calculate the integrals (only the portion for time_i0 to time_i1
	  dTemp_i = Temp_i - Temp0_norm
	  exp_ttemp = exp(double(time_i - time_i1 + dTemp_i))
	  Zattemp_st(istep) = int_tabulated(time_i, exp_ttemp)
	  Zat_st(istep) = int_tabulated(time_i, time_i*exp_ttemp)
	  Zatemp_st(istep) = int_tabulated(time_i, dTemp_i*exp_ttemp)
	  ; if istep gt 0 then add the contribution from previous portions
	  if istep gt 0 then begin
	     expti = exp(time_i0-time_i1)
	     Zattemp_st(istep) = Zattemp_st(istep) + $
	          Zattemp_st(istep-1) * expti 
	     Zat_st(istep) = Zat_st(istep) + $
	          Zat_st(istep-1) * expti 
	     Zatemp_st(istep) = Zatemp_st(istep) + $
	          Zatemp_st(istep-1) * expti 
	  endif        	  
	  time_st = [time_st,time_i1] 	
	ENDFOR         ; close the loop over integration steps
	
; Take into account that we have used normalized time and Temp

time_st = time_st * mod_par(0)
Zattemp_st = [0., Zattemp_st]
Zat_st = [0.,Zat_st * mod_par(0)]
Zatemp_st = [0.,Zatemp_st * mod_par(1)]	
	
; interpolate to obtain the integrals at the time t

Zattemp = interpol(Zattemp_st, time_st, t)  
Zat = interpol(Zat_st, time_st, t)
Zatemp = interpol(Zatemp_st, time_st, t)
	  
RETURN

not_pts:

RETURN

END	  


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO THERM_FULINT, t, mod_par, Zattemp, Zat, Zatemp

COMMON TEMPERATURE, time_Temp_all, Temp_all

match, time_Temp_all, t, suball, subt, COUNT=Imatch
if imatch eq 0 then begin 
  print, 'Time scales do not match '
  return
endif

Temp = Temp_all(suball)

teta = mod_par(0)
T_j = mod_par(1)
Temp_ini = Temp_all(0)
t_ini = time_Temp_all(0)

; Compute integrals (if not more than 15 points then interpolate)

	xele = n_elements(t)

	Zab = fltarr(xele) 
	Zat = fltarr(xele) 
	Zate = fltarr(xele) 

	for iele = 0, xele-1 do begin
	
	ind_iele = where(time_Temp_all le t(iele), niele)

	  case 1 of
	  (niele gt 14): begin
	         quotprime = (time_Temp_all(0:niele-1) - t(iele)) / teta
	         quottempprime = (Temp_all(0:niele-1) - Temp_ini) / T_j 
	         Zaprime = quotprime * 0. + 1.e36
	         Zatprime = Zaprime
	         Zateprime = Zaprime
	         tprime = time_Temp_all(0:niele-1) - t_ini
	         ind = WHERE (quotprime + quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zaprime(ind) = exp(quotprime+quottempprime)
	         ind = WHERE (quotprime LT 80., nq)
	         IF nq GT 0 THEN Zatprime(ind) = Zaprime(ind) * tprime(ind)
	         ind = WHERE (quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zateprime(ind)= Zaprime(ind)*quottempprime(ind)$
	                  *T_j
	    end
	  else: begin ; interpolate
	         t_tmp = indgen(15)/14.*time_Temp_all(niele-1)
	         tprime = t_tmp - t_ini
	         Temp_tmp = interpol(Temp_all, time_Temp_all, t_tmp)  
	         quotprime = (t_tmp - t(iele))/ teta
	         quottempprime = (Temp_tmp - Temp_ini) / T_j
	         Zaprime = quotprime * 0. + 1.e36
	         Zatprime = Zaprime
	         Zateprime = Zaprime
	         ind = WHERE (quotprime + quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zaprime(ind) = exp(quotprime+quottempprime)
	         ind = WHERE (quotprime LT 80., nq)
	         IF nq GT 0 THEN Zatprime(ind) = Zaprime(ind) * tprime(ind)
	         ind = WHERE (quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zateprime(ind)= Zaprime(ind)*quottempprime(ind)$
	                  *T_j
	    end
	  endcase  
	  Zab(iele) = int_tabulated(tprime, Zaprime)
	  Zat(iele) = int_tabulated(tprime, Zatprime)
	  Zate(iele)= int_tabulated(tprime, Zateprime)
	
	endfor
	 	
	Zattemp = Zab / teta
	Zat = Zat / teta
	Zatemp = Zate / teta
	    
RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO FOUKST_CHOP_DRIFT, t, a, fct, pder, error, FINAL=final, STEP=step

COMMON TEMPERATURE, time_Temp, Temp

COMMON PIA_DRIFT_PAR, drift_par

error = 0 

	p_field = drift_par.p_field
	s_field = drift_par.s_field
	a_var = drift_par.a_var
	t_ini = drift_par.t_ini
	Temp_ini = drift_par.Temp_ini
	  
; A(0) = S_inf_1
; A(1) = S_ini_1
; A(2) = Beta
; A(3) = E_j / Zeta
; A(4) = T_j
; A(5) = S_inf_2
; A(6) = S_ini_2



; Define aa from variable and fixed parameters a
	aa = p_field
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF 

	xele = N_ELEMENTS(t)
	nvar = N_ELEMENTS(a)
	
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	step_dif = step(1:xele-1) - step(0:xele-2)
	indst = WHERE(step_dif NE 0, ndif)
	IF ndif EQ 0 THEN BEGIN
		MESSAGE, /INFO, 'Not valid chopped sample'
		RETURN
	ENDIF

	bsinf = fct
	emod_j= fct
	T_j = fct
	sini  = fct
	sinf  = fct
	deltat = fct
	deltaTemp = fct
	Zab = fct
	Zat = fct
	Zate = fct
	

	s_inf = [aa(0),aa(5)]
	s_ini = [aa(1),aa(6)]
	beta = aa(2)
	NTemp_ini = Temp_ini/3.
	emod_j_norm = aa(3) * sqrt(NTemp_ini)
	T_j_norm = aa(4)* NTemp_ini
		
	indst1 = WHERE(step EQ 1,n1)
	deltat(indst1) = t(indst1)-t_ini(0)
	deltaTemp(indst1) = Temp(indst1)-Temp_ini(0)
	bsinf(indst1) = beta * s_inf(0)
	emod_j(indst1) = emod_j_norm(0) 
	T_j(indst1) =  T_j_norm(0) 
	sini(indst1) = s_ini(0)
	sinf(indst1) = s_inf(0)
	indst2 = WHERE(step EQ 2,n2)
	deltat(indst2) = t(indst2)-t_ini(1)
	deltaTemp(indst2) = Temp(indst2)-Temp_ini(1)
	bsinf(indst2) = beta * s_inf(1)
	emod_j(indst2) = emod_j_norm(1) 
	T_j(indst2) = T_j_norm(1) 
	sini(indst2) = s_ini(1)
	sinf(indst2) = s_inf(1)
		
	beta1 = 1. - beta
	teta = emod_j / bsinf
	

; Avoid underflows
	quot = double(deltat/teta)
	exp_t = exp(-quot)

	quottemp = double(deltaTemp/T_j)
	exp_Te = exp(quottemp)	
	
; Compute integrals 
	; do first chopper step
	teta_tmp = teta(indst1(0))
	T_j_tmp = T_j(indst1(0))
	therm_int, t(indst1), [teta_tmp, T_j_tmp], Zab_tmp, Zat_tmp, Zate_tmp
	Zab(indst1) = Zab_tmp
	Zat(indst1) = Zat_tmp
	Zate(indst1) = Zate_tmp
	; do second chopper step
	teta_tmp = teta(indst2(0))
	T_j_tmp = T_j(indst2(0))
	therm_int, t(indst2), [teta_tmp, T_j_tmp], Zab_tmp, Zat_tmp, Zate_tmp
	Zab(indst2) = Zab_tmp
	Zat(indst2) = Zat_tmp
	Zate(indst2) = Zate_tmp
		 		
	Zc = Sini - bsinf
	Zb = beta1 * Sinf*exp_t + Zc * Zab

	fct = float(bsinf + beta1 * Zc * sinf * exp_Te / Zb)
		 	
	
; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

	IF N_PARAMS() LT 4 THEN RETURN
; S_inf_1
	k = 0
	IF a_var(0) THEN BEGIN
	 pder(indst1,k) = float(beta + beta1*exp_Te(indst1)/Zb(indst1)*       $
	   (Zc(indst1)*(1+t(indst1)/teta(indst1))-bsinf(indst1)- Zc(indst1)/  $
	   Zb(indst1)*(beta1*exp_t(indst1)*Sinf(indst1) + Zab(indst1)*       $
	   (Zc(indst1)-bsinf(indst1)) + Zc(indst1)/teta(indst1)*Zat(indst1)))) 
	 pder(indst2,k) = 0.
	 k = k+1
	ENDIF

; S_ini_1
	IF a_var(1) THEN BEGIN
	 pder(indst1,k) = float(beta1 * Sinf(indst1)/Zb(indst1)* exp_Te(indst1)$
	  * (1.-Zc(indst1)*Zab(indst1)/Zb(indst1)))
	 pder(indst2,k) = 0.
	 k = k+1
	ENDIF
; beta
	IF a_var(2) THEN BEGIN
	 pder(*,k) = float(Sinf * (1. + exp_Te/Zb *  $
	      (quot/teta*beta1/beta*Zc-Zc-Sinf*beta1-beta1*Zc/Zb* $
	        (Zc/beta*(Zab+Zat/teta)-Sinf*(exp_t+Zab)))))
	 k = k+1
	ENDIF	
	
; E_j 
	IF a_var(3) THEN BEGIN
	 pder(*,k) = float(beta1*Sinf*Zc*exp_Te/Zb/Emod_j*  $
	      (Zc/Zb*(Zab+Zat/teta)-quot))
	 k = k+1
	ENDIF	
		    
; T_j 
	IF a_var(4) THEN BEGIN
	 pder(*,k) = -float(beta1*Sinf*Zc/Zb*exp_Te/T_j^2* $ 
	       (deltaTemp-Zate/Zb*Zc))
	 k = k+1
	ENDIF	
		    
; S_inf_2
	IF a_var(5) THEN BEGIN
	 pder(indst2,k) = float(beta + beta1*exp_Te(indst2)/Zb(indst2)*        $
	 (Zc(indst2)*(1+t(indst2)/teta(indst2))-bsinf(indst2)-Zc(indst2)/      $
	 Zb(indst2)*(beta1*exp_t(indst2)*Sinf(indst2) + Zab(indst2)*           $
	 (Zc(indst2)-bsinf(indst2)) + Zc(indst2)/teta(indst2)*Zat(indst2)))) 
	 k = k+1
	ENDIF

; S_ini_2
	IF a_var(6) THEN $
	 pder(indst2,k) = float(beta1 * Sinf(indst2)/Zb(indst2) * $
	 exp_Te(indst2) * (1.-Zc(indst2)*Zab(indst2)/Zb(indst2)))


        RETURN
        END	 	
	 
