;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKST_CHOP_start_par
;
; PURPOSE:
;	Calculate start parameters for the fouks model for chopped measurements,
;	 with temperature influence
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKST_CHOP_start_par, det, time, slop, flag, pixel, start_par, $
;		error=error, step=step
;
; INPUT:
;	det:	PHT detector ('P1', 'C2', ...)
;	slop:	Signal slopes array
;	pixel:	Pixel to be used
;
; KEYWORD PARAMETERS:
;	ERROR:	Results of computation
;	STEP:	Chopper step array
;	
; OUTPUT:
;	start_par: The start parameters
;	
; COMMON BLOCKS:
;	fouks_start_par 
;		
; PROCEDURES USED:
;       PIA LIB:
;               s_ini_final
;	Astrolib:
;		readcol
;
; MODIFICATION HISTORY:
;	Written by: C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Added flag as input parameter (J.Acosta)  October 1996
;	Added time as input parameter J.Acosta	  October 1996
;	Added t_zero as input keyword (J.Acosta)  November 1996
;	Small change in initial pars (jacg)	  Dec 1996
;	Set a dummy value for S_C(pixel) when 
;	argument of sqrt is negative  (J.Acosta)  January 1997	 
;	Version 5.0 - Aug.96
;			
;-
PRO foukst_chop_start_par, det, time, slop, flag, pixel, start_par,  $
    ERROR=error, STEP=step, T_ZERO = t_zero

COMMON fouks_start_par, s_beta, s_C, s_Nd, det_par
	error = 1	      
	start_par = fltarr(7)
; read data from driftpar file in case is not in
	IF N_ELEMENTS(s_beta) EQ 0 THEN $
	   readcol, !CALU+'fouks_dpar.txt', pix, s_beta, s_C, s_ND, f='i,f,d,d'

	indst1 = WHERE(step EQ 1, nc1)
	indst2 = WHERE(step EQ 2, nc2)
	IF nc1 EQ 0 or nc2 EQ 0 THEN BEGIN
	   MESSAGE, /info, 'step array not acceptable'
	   RETURN
	ENDIF
	
	;;if not keyword_set(t_zero) then $
	;;      t_zero = [time(indst1(0)),time(indst2(0))]
	
	s_ini_final, time(indst1), slop(indst1), flag(indst1), S_ini1, S_fin1, $
	    error=err_ini1   ;;, t_zero = t_zero(0)
	s_ini_final, time(indst2), slop(indst2), flag(indst2), S_ini2, S_fin2, $
	    error=err_ini2   ;;, t_zero = t_zero(1)

	error = err_ini1 + err_ini2
	if error gt 0 then return

	start_par(0) = S_fin1
	start_par(1) = max([S_ini1,S_fin1*s_beta(pixel)*1.2])
	start_par(5) = S_fin2
	start_par(6) = max([S_ini2,S_fin2*s_beta(pixel)*1.2])

	CASE det of
	
	   'SL': BEGIN
	      
	      error = 0	    
	        
	      start_par(2) = s_beta(pixel)
	      a = 1.949e-11
	      logarg = alog(s_Nd(pixel)/start_par(0)/s_C(pixel))
	      IF logarg GT 0. THEN ej = sqrt(A*s_Nd(pixel)/logarg)   $
	         ELSE BEGIN 
	             logarg = alog(s_Nd(pixel)/start_par(0)/1.013e13)
	             ej = sqrt(A*s_Nd(pixel)/logarg)
	             print,'Using a dummy value for Ej '
	         ENDELSE
	
	      IF pixel LT 64 THEN $
	            start_par(3) = ej / 8.052   $ 
	        ELSE $ 
	            start_par(3) = ej / 9.644

	      start_par(4) = 0.25

	    END
; other detectors just taking some numbers	 
	    ELSE: BEGIN
	      error = 0	      
; take s_beta as for pixel 30 PHT-S
	      start_par(2) = s_beta(30)
	      start_par(3) = 3.	        
	      start_par(4) = 0.25        
	    END
	 ENDCASE
	 
	 RETURN
	 END
