;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FOUKST_DRIFT
;
; PURPOSE:
;	Calculate the temperature dependent Fouks function and its derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FOUKSC_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;	therm_int
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Correction of bugs and include a new way to compute 
;	integrals  (J.Acosta)					  October 1996
;	Version 5.0 - Aug.96
;			
;-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO THERM_INT, t, mod_par, Zattemp, Zat, Zatemp,  $ 
     Zattemp_st, Zat_st, Zatemp_st, T_INI = t_ini, TEMP_INI = Temp_ini
; This routine calculate the integral A_INT, B_INT, and C_INT, in a much
; simpler way to save computing time. 
; Steps:
;    1- Divide into several intervals, using a log time scale
;    2- Compute the integral for each interval. It uses the result of previous 
; 	steps. 
;    3- Interpolate the result to the required input time (t) and returns.
;
;	Notice that the time scale time_TEMP includes all points per chopper
;	plateau, while t is only of the valid signal data points. 
;
; Tauj= mod_par(0)
; Tj = mod_par(1)
;
  
COMMON TEMPERATURE, time_Temp, Temp


IF NOT KEYWORD_SET (T_INI) THEN time0 = time_temp(0) $ 
   ELSE time0 = t_ini
IF NOT KEYWORD_SET (TEMP_INI) THEN Temp0 = Temp(0) $ 
   ELSE Temp0 = Temp_ini
   
; Compute the number of steps
mfactor = 1.5 
	; First step =  4 * time_reset
 	Deltt = (time_Temp(1)-time_Temp(0)) 
 	t1 = Deltt * 4.
	tf = max(time_Temp)-time0
	Nsteps = FIX(alog10(tf/t1)/alog10(mfactor))+2
	

; Now perform integrals for each step
Zattemp_st = fltarr(Nsteps)
Zat_st = fltarr(Nsteps)
Zatemp_st = fltarr(Nsteps)
time_st = time0

Nint_tab = 20  ;number of points to subdivide the i-step to compute integral
MIN_NPTS = 5   ; minimum number of points which are in the subinterval
time_i1 = time0

	FOR istep = 0, Nsteps-1 DO BEGIN
	  time_i0 = time_i1 
	  time_i1 = min([t1*mfactor^istep,tf]) + time0
	  ; divide the interval into small pieces
	  time_i = time_i0 + indgen(Nint_tab)*(time_i1-time_i0)/(Nint_tab-1)
	  ; select time_temp in the range [time_i0,time_i1]
	  ind_i =  $
	    where(time_Temp ge time_i0 and time_Temp le time_i1,nind)
	  ; add boundaries 
	  IF ind_i(0) gt 0 THEN BEGIN 
	      ind_i = [ind_i(0)-1,ind_i]
	      nind = nind + 1
	  ENDIF    
	  IF ind_i(nind-1) LT n_elements(time_temp)-1 THEN BEGIN  
	      ind_i = [ind_i,ind_i(nind-1)+1]
	      nind = nind + 1
	  ENDIF    
	  CASE 1 OF 
	    nind ge MIN_NPTS: BEGIN   ; fit Temp_norm to a polynom of degree 3
	                   ;coef3 = poly_fit(time_Temp(ind_i)-time0, $ 
	                   ;     Temp(ind_i), 3)
	                   ;Temp_i = poly(time_i-time0, coef3)
	                   Temp_i = spline(time_Temp(ind_i)-time0, Temp(ind_i), $
	                        time_i-time0, 0.1)
	    		END
	    nind gt 1: BEGIN   ; fit Temp_norm to a polynom of degree 2
	                   coef2 = poly_fit(time_Temp(ind_i), $
	                        Temp(ind_i), 2)
	                   Temp_i = poly(time_i, coef2)
	    		END
	    else: BEGIN 
	    		print, 'Not enough points in the interval '$ 
	    		   +strtrim(istep+1,2)
	    		goto, not_pts 
	    	  END	  
	  ENDCASE     
	
	  ; calculate the integrals (only the portion for time_i0 to time_i1
	  dTemp_i = Temp_i - Temp0
	  exp_ttemp = exp(double((time_i - time_i1)/mod_par(0) + $ 
	         dTemp_i/ mod_par(1))) / mod_par(0)
	  Zattemp_st(istep) = int_tabulated(time_i, exp_ttemp)
	  Zat_st(istep) = int_tabulated(time_i, time_i*exp_ttemp)
	  Zatemp_st(istep) = int_tabulated(time_i, dTemp_i*exp_ttemp)
	  ; if istep gt 0 then add the contribution from previous portions
	  if istep gt 0 then begin
	     expti = exp(double(time_i0-time_i1)/mod_par(0))
	     Zattemp_st(istep) = Zattemp_st(istep) + $
	          Zattemp_st(istep-1) * expti 
	     Zat_st(istep) = Zat_st(istep) + $
	          Zat_st(istep-1) * expti 
	     Zatemp_st(istep) = Zatemp_st(istep) + $
	          Zatemp_st(istep-1) * expti 
	  endif        	  
	  time_st = [time_st,time_i1] 	
	ENDFOR         ; close the loop over integration steps
	

time_st = time_st 
Zattemp_st = [0., Zattemp_st]
Zat_st = [0.,Zat_st ]
Zatemp_st = [0.,Zatemp_st ]	
	
; interpolate to obtain the integrals at the time t

;print,'Time_st, Zattemp_st ', Time_st, Zattemp_st

Zattemp = interpol(Zattemp_st, time_st, t)  
Zat = interpol(Zat_st, time_st, t)
Zatemp = interpol(Zatemp_st, time_st, t)

;for i=0, n_elements(t)/5 do print,'t, Zattemp ',t(i),Zattemp(i)
	  
RETURN

not_pts:

RETURN

END	  


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO THERM_FULINT, t, mod_par, Zattemp, Zat, Zatemp

COMMON TEMPERATURE, time_Temp_all, Temp_all

match, time_Temp_all, t, suball, subt, COUNT=Imatch
if imatch eq 0 then begin 
  print, 'Time scales do not match '
  return
endif

Temp = Temp_all(suball)

teta = mod_par(0)
T_j = mod_par(1)
Temp_ini = Temp_all(0)
t_ini = time_Temp_all(0)

; Compute integrals (if not more than 15 points then interpolate)

	xele = n_elements(t)

	Zab = fltarr(xele) 
	Zat = fltarr(xele) 
	Zate = fltarr(xele) 

	for iele = 0, xele-1 do begin
	
	ind_iele = where(time_Temp_all le t(iele), niele)

	  case 1 of
	  (niele gt 14): begin
	         quotprime = (time_Temp_all(0:niele-1) - t(iele)) / teta
	         quottempprime = (Temp_all(0:niele-1) - Temp_ini) / T_j 
	         Zaprime = quotprime * 0. + 1.e36
	         Zatprime = Zaprime
	         Zateprime = Zaprime
	         tprime = time_Temp_all(0:niele-1) - t_ini
	         ind = WHERE (quotprime + quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zaprime(ind) = exp(quotprime+quottempprime)
	         ind = WHERE (quotprime LT 80., nq)
	         IF nq GT 0 THEN Zatprime(ind) = Zaprime(ind) * tprime(ind)
	         ind = WHERE (quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zateprime(ind)= Zaprime(ind)*quottempprime(ind)$
	                  *T_j
	    end
	  else: begin ; interpolate
	         t_tmp = indgen(15)/14.*time_Temp_all(niele-1)
	         tprime = t_tmp - t_ini
	         Temp_tmp = interpol(Temp_all, time_Temp_all, t_tmp)  
	         quotprime = (t_tmp - t(iele))/ teta
	         quottempprime = (Temp_tmp - Temp_ini) / T_j
	         Zaprime = quotprime * 0. + 1.e36
	         Zatprime = Zaprime
	         Zateprime = Zaprime
	         ind = WHERE (quotprime + quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zaprime(ind) = exp(quotprime+quottempprime)
	         ind = WHERE (quotprime LT 80., nq)
	         IF nq GT 0 THEN Zatprime(ind) = Zaprime(ind) * tprime(ind)
	         ind = WHERE (quottempprime LT 80., nq)
	         IF nq GT 0 THEN Zateprime(ind)= Zaprime(ind)*quottempprime(ind)$
	                  *T_j
	    end
	  endcase  
	  Zab(iele) = int_tabulated(tprime, Zaprime)
	  Zat(iele) = int_tabulated(tprime, Zatprime)
	  Zate(iele)= int_tabulated(tprime, Zateprime)
	
	endfor
	 	
	Zattemp = Zab / teta
	Zat = Zat / teta
	Zatemp = Zate / teta
	    
RETURN
END


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO FOUKST_DRIFT, t, a, fct, pder, error, FINAL=final

COMMON TEMPERATURE, time_Temp, Temp

COMMON PIA_DRIFT_PAR, drift_par


	p_field = drift_par.p_field
	s_field = drift_par.s_field
	a_var = drift_par.a_var
	t_ini = drift_par.t_ini
	Temp_ini = drift_par.Temp_ini
	  
; A(0) = S_inf 
; A(1) = S_ini
; A(2) = Beta
; A(3) = E_j / Zeta
; A(4) = T_j



; Define aa from variable and fixed parameters a
	aa = p_field
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF 

	xele = N_ELEMENTS(t)
	nvar = N_ELEMENTS(a)
	
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	s_inf = aa(0)
	s_ini = aa(1)
	beta = aa(2)
	NTemp_ini = Temp_ini/3. 
	emod_j = aa(3) * sqrt(NTemp_ini)
	T_j = aa(4) * NTemp_ini
	
;	IF (beta * s_inf) EQ 0. THEN bsinf = 1.e-10 ELSE 
	bsinf=beta * s_inf
	beta1 = 1. - beta
	teta = emod_j / bsinf
	deltat = t - t_ini
	deltaTemp = Temp - Temp_ini


; Avoid underflows
	quot = double(deltat/teta)
	exp_t = exp(-quot)

	quottemp = double(deltaTemp/T_j)
	exp_Te = exp(quottemp)	
	
; Compute integrals 

	therm_int, t, [teta, T_j], Zab, Zat, Zate, T_INI = t_ini, $ 
	 TEMP_INI = Temp_ini 
	;therm_fulint, t, [teta, T_j], Zab, Zat, Zate
		 		
	Zc = S_ini - bsinf
	Zb = beta1 * S_inf*exp_t + Zc * Zab   ; Denominator

; compute value of function
	fct = float(bsinf + beta1 * Zc * s_inf * exp_Te / Zb)

  ; do check of beta parameter, avoid pole [S_0 - Beta * S >0]
        beta_check = S_ini - beta*fct
        ind_beta = where(beta_check lt 0., nbeta)
        if nbeta gt 0 then error = 1 else error = 0
 
  ; do check of Temperature 
        err_temp = check_temp(time_Temp, Temp, S_inf, emod_j, beta, 0, 0)
        error = error + err_temp*2
		 	
	
; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

	IF N_PARAMS() LT 4 THEN RETURN
; S_inf
	k = 0
	IF a_var(0) THEN BEGIN
	 pder(*,k) = float(beta + beta1*exp_Te/Zb*(Zc*(1+t/teta)-bsinf-Zc/Zb*   $
	         (beta1*exp_t*S_inf + Zab*(Zc-bsinf) + Zc/teta*Zat))) 
	 k = k+1
	ENDIF

; S_ini
	IF a_var(1) THEN BEGIN
	 pder(*,k) = float(beta1 * S_inf/Zb * exp_Te * (1.-Zc*Zab/Zb))
	 k = k+1
	ENDIF
; beta
	IF a_var(2) THEN BEGIN
	 pder(*,k) = float(S_inf * (1. + exp_Te/Zb *  $
	      (quot/teta*beta1/beta*Zc-Zc-S_inf*beta1-beta1*Zc/Zb* $
	        (Zc/beta*(Zab+Zat/teta)-S_inf*(exp_t+Zab)))))
	 k = k+1
	ENDIF	
	
; E_j 
	IF a_var(3) THEN BEGIN
	 pder(*,k) = float(beta1*S_inf*Zc*exp_Te/Zb/Emod_j*  $
	      (Zc/Zb*(Zab+Zat/teta)-quot))
	 k = k+1
	ENDIF	
		    
; T_j 
	IF a_var(4) THEN $
	 pder(*,k) = -float(beta1*S_inf*Zc/Zb*exp_Te/T_j^2* $ 
	       (deltaTemp-Zate/Zb*Zc))
		    

        RETURN
        END	 	
	 
