;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	INVEXP_DRIFT
;
; PURPOSE:
;	Calculate an offset inverse exponential function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	INVEXP_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Version 5.0 - Aug.96
;			
;-
pro invexp_drift, time, param, fct, pder, error, final=final

COMMON PIA_DRIFT_PAR, drift_par

error = 0

; Define aa from variable and fixed parameters a
	aa = drift_par.p_field
	a_var = drift_par.a_var
	t0 = drift_par.t_zero

	xele = N_ELEMENTS(time)
	nvar = N_ELEMENTS(param)
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = param
	ENDIF 

	A0 = aa(0)
	beta = aa(1)
	alpha = aa(2)
	tau = aa(3)

; Function is defined as 
; gy = A0*(1.+ beta/(1.+alpha*exp(-(t-t_0)/tau)))
; param(2) may be fixed 

	tt_sh = time-t0
	fct = A0 * (1.+ beta/(1.+alpha*exp(-tt_sh/tau)))

; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

        IF N_PARAMS() LT 4 then RETURN
	
	k=0
	IF a_var(0) THEN BEGIN
		pder(*,k) = 1.+beta/(1.+alpha*exp(-tt_sh/tau))
		k = k+1
	ENDIF
	
	IF a_var(1) THEN BEGIN
		pder(*,k) = A0/(1.+alpha*exp(-tt_sh/tau))
		k = k+1
	ENDIF
	IF a_var(2) THEN BEGIN
		pder(*,k) = -A0*beta/(1.+exp(-tt_sh/tau))^2 * exp(-tt_sh/tau)
		k = k+1
	ENDIF
	IF a_var(3) THEN BEGIN
		pder(*,k) = -A0*beta/(1.+exp(-tt_sh/tau))^2 * alpha * $
		   exp(-tt_sh/tau) * tt_sh /tau^2     
	ENDIF

end

