;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	NUMDIF_DRIFT
;
; PURPOSE:
;	Calculate an offset exponential function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	NUMDIF_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: B. Schulz (ESA/VILSPA-SAI) + CG	  August 1996
;	Version 5.0 - Aug.96
;			
;-
pro numdif_drift, x,a,f,pder, error, FINAL=final

COMMON PIA_DRIFT_PAR, drift_par


error = 0 

; Define aa from variable and fixed parameters a
	aa = drift_par.p_field
	a_var = drift_par.a_var

	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = a
	ENDIF 

npoints = n_elements(x)		; number of x values
nterms = n_elements(aa)		; number of parameters

f = drft1(x,aa)			; calculate function

;-------------------------------------------
; calculate partial derivatives if requested

if n_params() EQ 4 then begin

  pder = fltarr(npoints,N_ELEMENTS(a))		;create matrix

  delta = (max(x) - min(x))/n_elements(x)/10.	;get shift distance

  k = 0

  for i=0,nterms-1 do begin			; for each parameter
   	    
   IF a_var(i) THEN BEGIN
    b = aa
    b(i) = b(i)+delta			; shift parameter by delta
    bb = drft1(x,b)			; calculate function there
    pder(*,k) = (bb-f)/delta		; calculate approx. derivative
    k = k+1
   ENDIF
  endfor
endif

return

end

