;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	OFFEXP2_DRIFT
;
; PURPOSE:
;	Calculate an offset exponential function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	OFFEXP2_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  November 1996
;	Version 5.3 - Nov.96
;			
;-
pro offexp2_drift, time, param, fct, pder, error, FINAL=final

COMMON PIA_DRIFT_PAR, drift_par

error = 0


; set a = 1
 
; Define aa from variable and fixed parameters a
	aa = drift_par.p_field
	a_var = drift_par.a_var
	t0 = drift_par.t_zero

	xele = N_ELEMENTS(time)
	nvar = N_ELEMENTS(param)
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = param
	ENDIF 

	S_inf = aa(0)
	S_0 = aa(1)
	Sprime_0 = aa(2)
	Taui = aa(3)
	E_l = aa(4)
	lambda = aa(5)

; Function is defined as 
; S(t) = C1*exp(-(t-t_zero)/Taui) + C2*(1-exp(-(t-t_zero)/Taul)) + C3
;  C1  = Taui/(Taul-Taui)*(S_inf - S_0 - Taul*Sprime_0) 
;  C2  = Taul/(Taul-Taui)*(S_inf-S_0- Taui*Sprime_0)
;  C3  = 1/(Taul-Taui)*(Taul*Taui*Sprime_0 + S_0*Taul - S_inf*Taui)
; the function can be simplified as: 
; S(t) = C1*exp(-(t-t_zero)/Taui) - C2*exp(-(t-t_zero)/Taul)) + S_inf

	; set aind = 1./2
	aind = 1.
	tt_sh = double(time-t0)
	quoti = double (tt_sh/taui)
	if abs(S_inf) gt 1.e-5 then taul = E_l/S_inf^aind $
	  else taul = 1.e6 
	quotl = double(tt_sh/taul)
	
	c1 = -lambda*Taui*Sprime_0
	c2 = -S_inf + S_0 - c1
	
	
	print,'C1, c2 ',c1,c2
	
	fct = float(c1*exp(-quoti)+c2*exp(-quotl) + S_inf)

; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

        IF N_PARAMS() LT 4 then RETURN
	
	k=0
	IF a_var(0) THEN BEGIN
		pder(*,k) =  $ 
		   float(1.-(1.-c1*aind*S_inf^(aind-1.)*tt_sh/E_l)*$
		   exp(-quotl))
		k = k+1
	ENDIF
	
	IF a_var(1) THEN BEGIN
		pder(*,k) = float(exp(-quotl))
		k = k+1
	ENDIF
	
	IF a_var(2) THEN BEGIN
		pder(*,k) = float(lambda*Taui*(-exp(-quoti)+exp(-quotl)))
		k = k+1
	ENDIF

	IF a_var(3) THEN BEGIN
		pder(*,k) = $ 
		   float(((1.+tt_sh/Taui)*exp(-quoti)-exp(-quotl))*c1/Taui)
		k = k+1
	ENDIF

	IF a_var(4) THEN BEGIN
		pder(*,k) = float(c2*quotl/E_l*exp(-quotl))
		k = k+1
	ENDIF

	IF a_var(5) THEN BEGIN
		pder(*,k) = float(Taui*Sprime_0*(-exp(-quoti)+exp(-quotl)))
	ENDIF

end

