;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	OFFEXP3_DRIFT
;
; PURPOSE:
;	Calculate an offset exponential function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	OFFEXP3_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  November 1996
;	Version 5.3 - Nov.96
;			
;-
pro offexp3_drift, time, param, fct, pder, error, FINAL=final

COMMON PIA_DRIFT_PAR, drift_par

error = 0


; set a = 1
 
; Define aa from variable and fixed parameters a
	aa = drift_par.p_field
	a_var = drift_par.a_var
	t0 = drift_par.t_zero

	xele = N_ELEMENTS(time)
	nvar = N_ELEMENTS(param)
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = param
	ENDIF 

	S_inf = aa(0)
	S_0 = aa(1)
	lambda = aa(2)      
	beta = aa(3)
	beta1 = 1.-beta

	E1 = aa(4)
	E2 = aa(5)
	E3 = aa(6)
	 	
	p1 = aa(7)
	p2 = aa(8)
	p3 = aa(9)
	
	
	Tau1 = E1 / S_inf^p1
	Tau2 = E2 / S_inf^p2
	Tau3 = E3 / S_inf^p3

; Function is defined as 
; S(t) = S_inf*(1+lambda*(exp(-(t-t_zero)/Tau1)-exp(-(t-t_zero)/Tau3))) + $
;	 (S_inf-S_0)*(beta*exp(-(t-t_zero)/Tau2)+(1-beta)*exp(-(t-t_zero)/Tau3))
;       

	; set aind = 1./2
	tt_sh = double(time-t0)
	quot1 = double (tt_sh/Tau1)
	quot2 = double (tt_sh/Tau2)
	quot3 = double (tt_sh/Tau3)
	
	
	C1 = exp(-quot1)
	C2 = exp(-quot2)
	C3 = exp(-quot3)
	
	fct = float(S_inf*(1.+ lambda*(C1 - C3)) - $ 
	    (S_inf-S_0)*(beta*C2+beta1*C3))

; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

        IF N_PARAMS() LT 4 then RETURN
	
	k=0
	
	IF a_var(0) THEN BEGIN    ; S_inf
		pder(*,k) =  $ 
		   float(1.+lambda*(C1-C3)-beta*C2-beta1*C3 + $
		   lambda*(-p1*quot1*C1+p3*quot3*C3)  -       $
		   (S_inf-S_0)/S_inf*(-p2*beta*quot2*C2-p3*beta1*quot3*C3))
		k = k+1
	ENDIF
	
	IF a_var(1) THEN BEGIN   ; S_ini
		pder(*,k) = float(beta*C2+beta1*C3)
		k = k+1
	ENDIF
	
	IF a_var(2) THEN BEGIN   ; lambda
		pder(*,k) = float(S_inf*(C1-C3))
		k = k+1
	ENDIF

	IF a_var(3) THEN BEGIN   ; beta
		pder(*,k) = float(-(S_inf-S_0)*(C2-C3))
		k = k+1
	ENDIF

	IF a_var(4) THEN BEGIN   ; E1
		pder(*,k) = float(lambda*S_inf/E1*quot1*C1)
		k = k+1
	ENDIF

	IF a_var(5) THEN BEGIN   ; E2
		pder(*,k) = float(-(S_inf-S_0)/E2*quot2*C2)
		k = k+1
	ENDIF

	IF a_var(6) THEN BEGIN   ; E3
		pder(*,k) = float(-(lambda*S_inf+beta1*(S_inf-S_0))/E3*quot3*C3)
		k = k+1
	ENDIF

	IF a_var(7) THEN BEGIN   ; p1
		pder(*,k) = 0.0
		k = k+1
	ENDIF

	IF a_var(8) THEN BEGIN   ; p2
		pder(*,k) = 0.0
		k = k+1
	ENDIF

	IF a_var(9) THEN BEGIN   ; p3
		pder(*,k) = 0.0
		k = k+1
	ENDIF

end

