;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	OFFEXP_DRIFT
;
; PURPOSE:
;	Calculate an offset exponential function and derivatives
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	OFFEXP_DRIFT, t, a, fct, pder, error, FINAL=final
;
; INPUT:
;	t:	The independent variable
;	a:	The parameter set (variable and fixed parameters)
;
; KEYWORD PARAMETERS:
;	FINAL:	For a final calculation, without looking which parameters
;		are variable. No calculation of derivatives.
;	
; OUTPUT:
;	fct: 	The function values for t
;	pder:	The derivatives
;	error:	Error flag checking for the consistency of the function params.
;	
; COMMON BLOCKS:
;	pia_drift_par 
;		
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: J.Acosta (IAC) + C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	Change the functional form to have tau=E/S_final^p        Feb    1997
;	Using drift_par.t_ini instead of drift_par.t_zero, as
;	definition has changed	(JAcosta)			  June   1997
;	Version 5.0 - Aug.96
;			
;-
pro offexp_drift, time, param, fct, pder, error, FINAL=final

COMMON PIA_DRIFT_PAR, drift_par

error = 0

; Define aa from variable and fixed parameters a
	aa = drift_par.p_field
	a_var = drift_par.a_var
	t0 = drift_par.t_ini
	

	xele = N_ELEMENTS(time)
	nvar = N_ELEMENTS(param)
	fct = FLTARR(xele)
	pder = FLTARR(xele,nvar)
	
	IF not KEYWORD_SET(FINAL) THEN BEGIN
	 ind = WHERE(a_var EQ 1, nvar)
	 aa(ind) = param
	ENDIF 

	a0 = aa(0)
	a1 = aa(1)
	a2 = aa(2)

	p = aa(3) ; power law index  of time constant

; Function is defined as 
; gy = param(0)*(1.+ param(1)*exp(-(t-t_0)/param(2))
; param(2) may be fixed 

	tt_sh = double(time-t0)
	quot = double (tt_sh/a2*a0^p)
	fct = float(a0 + (a1-a0)*exp(-quot))

; Calculation of derivatives: perform if not final
	IF KEYWORD_SET(final) THEN RETURN

        IF N_PARAMS() LT 4 then RETURN
	
	k=0
	IF a_var(0) THEN BEGIN
	        quot1 = double (tt_sh/a2*a0^(p-1)*p)
		pder(*,k) = float(1.+exp(-quot)*(quot1*(a0-a1)-1.))
		k = k+1
	ENDIF
	
	IF a_var(1) THEN BEGIN
		pder(*,k) = float(exp(-quot))
		k = k+1
	ENDIF
	IF a_var(2) THEN BEGIN
		pder(*,k) = float((a1-a0)*quot/a2*exp(-quot))
	ENDIF

end

