;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	S_ini_final
;
; PURPOSE:
;	Estimate initial and final value for a stream of signals  
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	S_ini_final, time, slop, flag, s_ini, s_final, ERROR=error, 
;	     T_ZERO=t_zero, DETECTOR = detector
;
; INPUT:
;	slop:	Signal slopes array (it can be 2-dimensional)
;	flag;   Flag signal array (it can be 2-dimensional, same dims as slop)
;
; KEYWORD PARAMETERS:
;	ERROR:	Results of computation
;	INI_EXTPOL: extrapolate to obtain initial value
;	FIN_EXTPOL: extrapolate to obtain final value
;	
; OUTPUT:
;	s_ini: Initial signal
;	s_final: Final signal
;	DerS_ini: estimate of derivative at t_zero
;	
; COMMON BLOCKS:
;	(none) 
;		
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC)	  October 1996
;	Version 5.2 - Oct.96
;			
;-

PRO S_ini_final, time, slop, flag, S_ini, S_final, DerS_ini, S_ini_unc, $
		S_final_unc, ERROR=error, DETECTOR = det, T_ZERO=t_zero, $ 
		T_FINAL= t_final, MINNPTS  = minnpts
		 

        if not keyword_set(det) then begin
        	;print, 'Warning: Assuming 1-pixel detector [S_ini_final]'
        	det = 'P1'
        endif	
        
        if n_elements(t_zero) eq 0 then t_zero = time(0)
        if not keyword_set(t_final) then t_final = time(n_elements(time)-1)
        
        ; take the number of points to estimate depending on the time-scale
        ; it will tend to comprise all signals within 8 second
	IF NOT KEYWORD_SET (minnpts) then MIN_NPTS = 4 ELSE MIN_NPTS = minnpts
        IF t_zero gt 0 THEN time_sh = time - t_zero ELSE  time_sh = time
        ;ind_shift = where(time_sh lt 8,nshift) 
        ;SHIFT_ELE = max([nshift, MIN_NPTS])
        SHIFT_ELE = MIN_NPTS
        
	

	; check if slop is a 2-D array, then computes values for all pixels
	sdim = size(slop)
	IF sdim(0) eq 2 THEN BEGIN   ; multi pixel case
	  npix = pixel_det(det)
	  error = intarr(npix)
	  S_ini = fltarr(npix)
	  S_final = fltarr(npix)
	  DerS_ini = fltarr(npix)
	  S_ini_unc = fltarr(npix)
	  S_final_unc = fltarr(npix)
	  
	  
	  FOR ipix=0, npix-1 DO BEGIN
	     ind_val = WHERE(flag(ipix,*) lt 2, nval)
	     IF nval gt SHIFT_ELE THEN BEGIN 
	        nlast = N_ELEMENTS(ind_val) - 1
	        ind0 = nlast-SHIFT_ELE 
	      ; final value
	        degree = 1
	        tmpx = time(ind_val(ind0:nlast)) - t_final
	        tmpy = slop(ipix,ind_val(ind0:nlast))
	        coef_ext = poly_fit(tmpx, tmpy, degree,tmpfit,tmpyb,sig,aa)
	        S_final(ipix) = coef_ext(0)
	        S_final_unc(ipix) = sqrt(aa(0,0))

	      ; initial value  
	        if SHIFT_ELE ge 6 then degree = 2 else degree = 1
	        tmpx = time(ind_val(0:SHIFT_ELE)) - t_zero
	        tmpy = slop(ipix,ind_val(0:SHIFT_ELE))
	        coef_ext = poly_fit(tmpx, tmpy, degree,tmpfit,tmpyb,sig,aa)
	        S_ini(ipix) = coef_ext(0)
	        S_ini_unc(ipix) = sqrt(aa(0,0))
		DerS_ini(ipix) = coef_ext(1)
	      
	    ENDIF ELSE error(ipix) =  1
	     
          ENDFOR


	ENDIF ELSE BEGIN      ; 1 pixel case
	 ;print,'Doing 1 pixel case '
	
	  error = 0 

	  S_ini = 0.
	  S_final = 0.
	  
	  ind_val = WHERE(flag(*) lt 2, nval)
	  IF nval gt SHIFT_ELE THEN BEGIN 
	    nlast = N_ELEMENTS(ind_val) - 1
	    ind0 = nlast - SHIFT_ELE

	    ; final value
	       degree = 1
	       tmpx = time(ind_val(ind0:nlast)) - t_final
	       tmpy = slop(ind_val(ind0:nlast))
	       coef_ext = poly_fit(tmpx, tmpy, degree,tmpfit,tmpyb,sig,aa)
	       S_final = coef_ext(0)
	       S_final_unc = sqrt(aa(0,0))
	  
	    ; initial value  
	       if SHIFT_ELE ge 6 then degree = 2 else degree = 1
	       tmpx = time(ind_val(0:SHIFT_ELE)) - t_zero
	       tmpy = slop(ind_val(0:SHIFT_ELE))
	       coef_ext = poly_fit(tmpx, tmpy, degree,tmpfit,tmpyb,sig,aa)
	       S_ini = coef_ext(0)
	       S_ini_unc = sqrt(aa(0,0))
	       DerS_ini = coef_ext(1)
	  ENDIF  ELSE error = 1

	ENDELSE
	

RETURN
END
