;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	SAVE_DRIFT_STR
;
; PURPOSE:
;	Save a given drift fit parameter set into a structure per pixel drift 
;	parameter fit 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	SAVE_DRIFT_STR, 
;
; INPUT:
;	drift_par: 	Structure containing the last fit parameters
;
; KEYWORD PARAMETERS:
;	PIXEL:		The pixel number of the data to be used
;	ABS_TSTEP:	The absolute chopper/raster position 
;	FF_ERROR:	Error of the fitting function (third and leftmost bits)
;	LPAR_ERROR:	Fitting parameters out of limits (second bit)
;	
; OUTPUT:
;			
; SIDE EFFECT: 
;	fit_par_str:	Structure containing the fitting information. 
;			It should be defined before input.
;	
; COMMON BLOCKS:
;	none 
;		
; PROCEDURES USED:
;	PIA LIB:
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC)		September 1996 
;	Inclusion of new flag code		December 1996
;	Version 5.0 - Aug.96
;			
;-

PRO save_drift_str, drift_par, fit_par_str, PIXEL= ipix, ABS_TSTEP = abs_tstep, $
  FF_ERROR = ff_error, LPAR_ERROR = lpar_error, Verbose=verbose, NOISE=noise

if not keyword_set(ipix) then ipix=0

if not keyword_set(ABS_TSTEP) then abs_tstep = 0

if not keyword_set(ff_error) then ff_error = 0

if not keyword_set(lpar_error) then lpar_error = 0*drift_par.p_field

if strpos(drift_par.ffunct,'CHOP') gt -1 then chopp = 1 else chopp = 0

npar = n_elements(drift_par.p_field)

iff_error = 8*ff_error ; Fitting function checking errors start at 4th pixel

; check for goodness of the fit 
ichisq_error = 0
if drift_par.chisq gt drift_par.chisqt then ichisq_error = 1
ichisq_error = 4*ichisq_error   ; Goodness of fit by limiting chi-square

 

 	case  drift_par.ffunct of
 	'INVEXP_DRIFT': BEGIN 
 	  		A0 = drift_par.p_field(0) 
 	  		Err_A0 = drift_par.s_field(0)
 	  		beta = drift_par.p_field(1)
 	  		Err_beta = drift_par.s_field(1)
 			fit_par_str.signal(ipix,abs_tstep) = A0*(1.+beta)
			IF not KEYWORD_SET(noise) THEN $
 			 noise = sqrt((1.+beta)^2*Err_A0^2+ A0^2*Err_beta^2)
 			fit_par_str.err(ipix, abs_tstep) = noise 
 	                fit_par_str.flag(ipix,abs_tstep)    =         $
 	                  2*fix((total(lpar_error(*))+npar-1)/npar) + $
 	                  drift_par.flag + iff_error + ichisq_error
  			 
 		    END
 	'FOUKSC_CHOP_DRIFT': BEGIN
 	        cc_err = fix((1+total(lpar_error(2:3)))/2) 
 		fit_par_str.signal(ipix,abs_tstep)   = drift_par.p_field(0)
		IF not KEYWORD_SET(noise) THEN $
                  noise = [drift_par.s_field(0),drift_par.s_field(4)] $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(4)])
		  ENDELSE                    
                fit_par_str.err(ipix, abs_tstep)     = noise(0)
                fit_par_str.flag(ipix,abs_tstep)     = 			$
                       2*fix((2+lpar_error(0)+lpar_error(1)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix,abs_tstep+1) = drift_par.p_field(4)
                fit_par_str.err(ipix,abs_tstep+1)    = noise(1)
                fit_par_str.flag(ipix,abs_tstep+1)   = 			$
                       2*fix((2+lpar_error(4)+lpar_error(5)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
		    END
 	'FOUKSC_RTCHO_DRIFT': BEGIN
 	        cc_err = fix((1+total(lpar_error(2)+lpar_error(5)))/2) 
 	        ntmp = fit_par_str.ind_steps(abs_tstep+1) - $ 
 	             fit_par_str.ind_steps(abs_tstep) 
 	        indtmp1 = fit_par_str.ind_steps(abs_tstep) +  $
 	             indgen(fix(ntmp+1)/2)*2
 	        indtmp2 = fit_par_str.ind_steps(abs_tstep) + 1 + $
 	             indgen(fix(ntmp)/2)*2
 		fit_par_str.signal(ipix, indtmp1)  = drift_par.p_field(0)
		IF not KEYWORD_SET(noise) THEN noise = drift_par.s_field(0:1) $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(1)])
		  ENDELSE                    
		
                fit_par_str.err(ipix, indtmp1)     = noise(0)
                fit_par_str.flag(ipix, indtmp1)     = 			$
                       2*fix((2+lpar_error(0)+lpar_error(3)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix, indtmp2) = drift_par.p_field(1)
                fit_par_str.err(ipix, indtmp2)    = noise(1)
                fit_par_str.flag(ipix, indtmp2)   = 			$
                       2*fix((2+lpar_error(1)+lpar_error(4)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
		    END
 	'FOUKSC_TTCHO_DRIFT': BEGIN   ; Fouks for triangular chopping
 	        cc_err = fix((1+total(lpar_error(3)+lpar_error(7)))/2) 
 	        ntmp = fit_par_str.ind_steps(abs_tstep+1) - $ 
 	             fit_par_str.ind_steps(abs_tstep) 
 	        indtmp1 = fit_par_str.ind_steps(abs_tstep) +  $
 	             indgen((ntmp-1)/4+1)*4
 	        indtmp2 = fit_par_str.ind_steps(abs_tstep) + 1 + $
 	             indgen((ntmp-2)/4+1)*4
 	        indtmp3 = fit_par_str.ind_steps(abs_tstep) + 2 + $
 	             indgen((ntmp-3)/4+1)*4
 	        indtmp4 = fit_par_str.ind_steps(abs_tstep) + 3 + $
 	             indgen((ntmp-4)/4+1)*4

		IF not KEYWORD_SET(noise) THEN noise = drift_par.s_field(0:2) $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(1)])
                    noise(2) = max([noise(2), drift_par.s_field(2)])
		  ENDELSE                    
		
 		fit_par_str.signal(ipix, indtmp1)  = drift_par.p_field(0)
                fit_par_str.err(ipix, indtmp1)     = noise(0)
                fit_par_str.flag(ipix, indtmp1)     = 			$
                       2*fix((2+lpar_error(0)+lpar_error(4)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix, indtmp2) = drift_par.p_field(1)
                fit_par_str.err(ipix, indtmp2)    = noise(1)
                fit_par_str.flag(ipix, indtmp2)   = 			$
                       2*fix((2+lpar_error(1)+lpar_error(5)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
 		fit_par_str.signal(ipix, indtmp3) = drift_par.p_field(2)
                fit_par_str.err(ipix, indtmp3)    = noise(2)
                fit_par_str.flag(ipix, indtmp3)   = 			$
                       2*fix((2+lpar_error(2)+lpar_error(6)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
 		fit_par_str.signal(ipix, indtmp4) = drift_par.p_field(1)
                fit_par_str.err(ipix, indtmp4)    = noise(1)
                fit_par_str.flag(ipix, indtmp4)   = 			$
                       2*fix((2+lpar_error(1)+lpar_error(5)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
		    END
 	'FOUKSC_STCHO_DRIFT': BEGIN   ; Fouks for sawtooth chopping
 	        cc_err = fix((1+total(lpar_error(3)+lpar_error(7)))/2) 
 	        ntmp = fit_par_str.ind_steps(abs_tstep+1) - $ 
 	             fit_par_str.ind_steps(abs_tstep) 
 	        indtmp1 = fit_par_str.ind_steps(abs_tstep) +  $
 	             indgen((ntmp-1)/3+1)*3
 	        indtmp2 = fit_par_str.ind_steps(abs_tstep) + 1 + $
 	             indgen((ntmp-2)/3+1)*3
 	        indtmp3 = fit_par_str.ind_steps(abs_tstep) + 2 + $
 	             indgen((ntmp-3)/3+1)*3

		IF not KEYWORD_SET(noise) THEN noise = drift_par.s_field(0:2) $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(1)])
                    noise(2) = max([noise(2), drift_par.s_field(2)])
		  ENDELSE                    
		
 		fit_par_str.signal(ipix, indtmp1)  = drift_par.p_field(0)
                fit_par_str.err(ipix, indtmp1)     = noise(0)
                fit_par_str.flag(ipix, indtmp1)     = 			$
                       2*fix((2+lpar_error(0)+lpar_error(4)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix, indtmp2) = drift_par.p_field(1)
                fit_par_str.err(ipix, indtmp2)    = noise(1)
                fit_par_str.flag(ipix, indtmp2)   = 			$
                       2*fix((2+lpar_error(1)+lpar_error(5)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
 		fit_par_str.signal(ipix, indtmp3) = drift_par.p_field(2)
                fit_par_str.err(ipix, indtmp3)    = noise(2)
                fit_par_str.flag(ipix, indtmp3)   = 			$
                       2*fix((2+lpar_error(2)+lpar_error(6)+cc_err)/3)+ $
 	               drift_par.flag + iff_error + ichisq_error               
		    END
 	'FOUKST_CHOP_DRIFT': BEGIN 
 	        cc_err = fix((2+total(lpar_error(2:4)))/3) 
 		fit_par_str.signal(ipix,abs_tstep)   = drift_par.p_field(0)
		IF not KEYWORD_SET(noise) THEN $
                  noise = [drift_par.s_field(0),drift_par.s_field(5)] $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(5)])
		  ENDELSE                     
                fit_par_str.err(ipix, abs_tstep)     = noise(0)
                fit_par_str.flag(ipix,abs_tstep)     =                   $
                       2*fix((2+lpar_error(0)+lpar_error(1)+cc_err)/3)+  $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix,abs_tstep+1) = drift_par.p_field(5)
                fit_par_str.err(ipix,abs_tstep+1)    = noise(1)
                fit_par_str.flag(ipix,abs_tstep+1)   = 			 $
                       2*fix((2+lpar_error(5)+lpar_error(6)+cc_err)/3) + $
 	               drift_par.flag + iff_error + ichisq_error              
		    END
 	'OFFEXP_RTCHO_DRIFT': BEGIN
 	        cc_err = fix((1+total(lpar_error(2)+lpar_error(5)))/2) 
 	        ntmp = fit_par_str.ind_steps(abs_tstep+1) - $ 
 	             fit_par_str.ind_steps(abs_tstep) 
 	        indtmp1 = fit_par_str.ind_steps(abs_tstep) +  $
 	             indgen(fix(ntmp+1)/2)*2
 	        indtmp2 = fit_par_str.ind_steps(abs_tstep) + 1 + $
 	             indgen(fix(ntmp)/2)*2
 		fit_par_str.signal(ipix, indtmp1)  = drift_par.p_field(0)
		IF not KEYWORD_SET(noise) THEN noise = drift_par.s_field(0:1) $
                  ELSE BEGIN 
                    noise(0) = max([noise(0), drift_par.s_field(0)])
                    noise(1) = max([noise(1), drift_par.s_field(1)])
		  ENDELSE                    
		
                fit_par_str.err(ipix, indtmp1)     = noise(0)
                fit_par_str.flag(ipix, indtmp1)     = 			$
                       2*fix((1+lpar_error(0)+cc_err)/2)+ $
 	               drift_par.flag + iff_error + ichisq_error
 		fit_par_str.signal(ipix, indtmp2) = drift_par.p_field(1)
                fit_par_str.err(ipix, indtmp2)    = noise(1)
                fit_par_str.flag(ipix, indtmp2)   = 			$
                       2*fix((1+lpar_error(1)+cc_err)/2)+ $
 	               drift_par.flag + iff_error + ichisq_error               
		    END
 	else: BEGIN
		IF not KEYWORD_SET(noise) THEN noise=drift_par.s_field(0) $
		  ELSE noise = max([noise, drift_par.s_field(0)])
                fit_par_str.signal(ipix, abs_tstep) = drift_par.p_field(0)
                fit_par_str.err(ipix, abs_tstep)    = noise
 	        fit_par_str.flag(ipix,abs_tstep)    = 			$
 	              2*fix((total(lpar_error(*))+npar-1)/npar) + 	$
 	              drift_par.flag + iff_error + ichisq_error             
 
              END  
        endcase   
        

        istep = abs_tstep/(chopp+1)
        fit_par_str.chisq(ipix, istep) = drift_par.chisq
        fit_par_str.param(ipix, istep,*) = drift_par.p_field
        fit_par_str.sg_param(ipix, istep,*) = drift_par.s_field


      IF KEYWORD_SET(verbose) THEN BEGIN
        print,'Abs step ' , abs_tstep
        print,'Istep ',istep
        print,'Final Signal ',fit_par_str.signal(ipix,abs_tstep)
      ENDIF
      
        
RETURN
END        
   
