;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	SET_DRIFTPAR_SRD
;
; PURPOSE:
;	Call a drift modelling routine returning the modelled signal
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	SET_DRIFTPAR_SRD, fitpar_srd, drift_par, pix, abs_tstep
;
; INPUT:
;	fitpar_srd:	Initial values coming from a particular measurement
;	abs_tstep:	Absolute chopper/raster position for which the output
;			parameters will be derived
;	pix:		Pixel number for which initial params. are required,
;			if not specified it will take 1 (eg. 1-pix detectors)
;
; KEYWORD PARAMETERS:
;	DEFAULT:	It returns a dummy structure based on the common block
;			DFFUNCT_INFO. The keyword DET has to be set simultaneously.
;	DET:		It must be used in combination with the keyword DEFAULT 
;			to specify a detector. It is mandatory when /DEFAULT is set.
;	FFUNCT:		The name of the drift modelling function. If not used the
;			function which is default for the detector specified. 
;	
; OUTPUT:
;	drift_par:	Structure containing the fitting information:
;			 p_label:	Label of parameters
;			 a_var:		flag for variation [0:fixed, 1:free]
;			 ffunc:		function used
;			 minp_ext:	Minimum number of points used for fit
;			 fract_left:  	fraction of measurement not used 
;			 p_field:	parameter to be fitted
;			 s_field:	error of fitted parameters
;		  	 p_range:	limits of variation for fitting params.
;			 t_zero:	Initial time of meas. If chopped keyword
;					is set then it contains two values
;			 temp_zero:	Initial temp. of meas. If chopped keyword
;					is set then it contains two values
;			 flag:		output fit flag
;                	 chisq:		chi-squares of the fit
;			 chisqt:	limiting chi-square to accept the fit
;			
;	
; COMMON BLOCKS:
;	dffunct_info 
;		
; RESTRICTIONS:
;	The structure fitpar_srd has to be defined prior to access this routine
;
; PROCEDURES USED:
;	PIA LIB:
;
; MODIFICATION HISTORY:
;	Written by: J. Acosta (IAC)		   October 1996 
;	keyword long_sTR included and order of
;	input parameters			   January 1997
;       Added keywords DEFAULT, FFUNCT and DET	   June 1997
;	Bug correction			(JAcosta)  June 1997	
;	Version 5.x - Oct.96
;			
;-
PRO set_driftpar_srd, fitpar_srd, drift_par, pix, abs_tstep, LONG_STR= long_str,$
    DEFAULT = default, FFUNCT = ffunct, DET = det

COMMON DFFUNCT_INFO, funct_info

IF n_params() lt 2 THEN  BEGIN
  print,'Syntax: set_driftpar_srd, fitpar_str,drift_par, [pix, abs_tstep] '+ $ 
     ' [,/LONG_STR] [,/DEFAULT] [,FFUNCT=ffunct] [,DET = det] '
  RETURN
ENDIF   
  
 
IF KEYWORD_SET(default) THEN BEGIN
     IF KEYWORD_SET(ffunct) THEN 				$ 
        ind_ff = WHERE(funct_info.lst(1,*) eq ffunct,nff)  	$
       ELSE BEGIN 
        IF NOT KEYWORD_SET(DET) then BEGIN
  		print,'Detector must be specified in this case '
  		RETURN
	ENDIF               
        ind_det = WHERE(funct_info.det_def EQ det)  
        ind_ff = $ 
            WHERE(funct_info.lst(1,*) eq funct_info.det_deffunct(ind_det),nff)      
       ENDELSE
ENDIF ELSE BEGIN 
    ind_ff = where(funct_info.lst(1,*) eq fitpar_srd.mod_func,nff)
    default = 0
ENDELSE    

IF nff eq 0 THEN BEGIN
  print,'FITTING FUNCTION NOT FOUND in DFFUNCT_INFO '
  RETURN
ENDIF   

IF n_elements(pix) eq 0 THEN pix = 0
IF n_elements(abs_tstep) eq 0 THEN abs_tstep = 0

CASE 1 of

keyword_set(LONG_STR): BEGIN 

drift_par = {TDTNR: fitpar_srd.TDTNR, DET: fitpar_srd.DET, 		$
             mod_func: fitpar_srd.mod_func,nparams:fitpar_srd.nparams, 	$
             chisqtol:fitpar_srd.chisqtol, minp_ext:fitpar_srd.minp_ext,$ 
             a_var:fitpar_srd.a_var(pix,*), 				$
             fract_left:fitpar_srd.fract_left(pix), 			$
	     use_mintime:fitpar_srd.use_mintime,			$
	     mintime:fitpar_srd.mintime(pix),				$
	     weightm:fitpar_srd.weightm(pix),				$
             neff_steps:fitpar_srd.neff_steps, 				$
             ind_steps:fitpar_srd.ind_steps,				$
             param:fitpar_srd.param(pix,*,*),				$
             sg_param:fitpar_srd.sg_param(pix,*,*),			$
	     prange:fitpar_srd.prange(pix,*,*,*),		 	$
	     t_ini:fitpar_srd.t_ini(pix,*),				$
	     temp_ini:fitpar_srd.temp_ini(pix,*),			$
	     t_zero:fitpar_srd.t_zero,					$
	     flag:fitpar_srd.flag(pix, *),				$
             signal:fitpar_srd.signal(pix, *), 				$
             time_sg:fitpar_srd.time_sg,	 			$
             err:fitpar_srd.err(pix,*), 				$
             jump:fitpar_srd.jump(pix,*), 				$
             err_jump:fitpar_srd.err_jump(pix,*),			$
	     time: fitpar_srd.time,					$
             slop: fitpar_srd.slop(pix,*), 				$
             slop_flag:fitpar_srd.slop_flag(pix,*), 			$
             chisq:fitpar_srd.chisq(pix, *),date_proc:''}

 END

ELSE: BEGIN    ; define the internal structure for fitting algorithm

  p_label = STRING(funct_info.parlabel.(ind_ff(0)),format="(A10)")

  CASE 1 of 
     DEFAULT: BEGIN 
      		ffunc = funct_info.lst(1,ind_ff(0))
      		a_var = funct_info.avar.(ind_ff(0))
      		minp_ext = funct_info.minpext
      		use_mintime = funct_info.use_mintime
      		mintime = funct_info.mintime(ind_ff(0))
      		fract_left = funct_info.fractleft(ind_ff(0))
      		weightm = funct_info.weightm(0)
      		p_field = fltarr(funct_info.npar(ind_ff(0)))
      		s_field = p_field
      		p_range = funct_info.prange.(ind_ff(0))
      		t_zero = 0.  
      		t_ini = 0.
      		temp_ini = 0.   
       		chisqtol = funct_info.chisqtol
              END
              
     ELSE:    BEGIN
  		chopped = 0 
  		allstep = 0
  		allrst = 0
  		CASE 1 OF 
			strpos(fitpar_srd.mod_func,'_RST_') gt -1: allrst = 1
			strpos(fitpar_srd.mod_func,'TCHO') gt -1: allstep = 1
			strpos(fitpar_srd.mod_func,'CHOP') gt -1: chopped = 1
		ELSE: 
  		ENDCASE	      	
                iistep = abs_tstep/(chopped+1)
                ind_isteps = fitpar_srd.ind_steps 
     		ffunc = fitpar_srd.mod_func
     	  	a_var = REFORM(fitpar_srd.a_var(pix,iistep,*))
     	  	minp_ext = fitpar_srd.minp_ext
     	  	use_mintime = fitpar_srd.use_mintime
     	  	mintime = fitpar_srd.mintime(pix)
     	  	fract_left = fitpar_srd.fract_left(pix)
     		weightm = fitpar_srd.weightm(pix) 
       		p_field = REFORM(fitpar_srd.param(pix,iistep,*))
       		s_field = REFORM(fitpar_srd.param(pix,iistep,*)*0.0)
       		p_range = REFORM(fitpar_srd.prange(pix,iistep,*,*))
       		CASE 1 OF 
       		   allrst: BEGIN 
	             t_ini = fitpar_srd.t_ini
	             t_zero = fitpar_srd.t_zero
	             temp_ini = fitpar_srd.temp_ini 
       		   END
       		   allstep: BEGIN 
       		      ind_tmp = indgen(ind_isteps(iistep+1)-ind_isteps(iistep)) $
                            + ind_isteps(iistep) 
	              t_ini = fitpar_srd.t_ini(pix,ind_tmp)
	              t_zero = fitpar_srd.t_zero(ind_tmp)
	              temp_ini = fitpar_srd.temp_ini(pix,ind_tmp)
                   END 
                   ELSE: BEGIN	
	              t_ini = fitpar_srd.t_ini(pix,abs_tstep:abs_tstep+chopped)
	              temp_ini = fitpar_srd.temp_ini(pix,abs_tstep:abs_tstep    $
	                          +chopped)
	              t_zero = fitpar_srd.t_zero(abs_tstep:abs_tstep+chopped)
	              sz = size(t_ini) 
	              IF sz(0) eq 2 then BEGIN
	                 t_ini = reform(t_ini)
	                 temp_ini = reform(temp_ini)
	              ENDIF 
	           END      
                ENDCASE

       		chisqtol = fitpar_srd.chisqtol
       		
     
     	      END
  ENDCASE   	                      
 

  drift_par = {p_label:p_label,						$ 
             a_var:a_var, 						$
             ffunct:ffunc,						$
	     minp_ext:minp_ext,						$
             use_mintime:use_mintime,					$
             mintime:mintime,						$
             fract_left:fract_left,					$
             weightm:weightm,						$
             p_field:p_field, 						$
             s_field:s_field, 						$
             p_range:p_range, 						$
             t_ini:t_ini, 						$
             temp_ini:temp_ini,	 					$
             t_zero:t_zero,						$
             flag:0, chisq:0., chisqt:chisqtol} 
             
  END             

ENDCASE             
             
RETURN
END             
 
