;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	CALIB_CHOPPMEAS_SL
;
; PURPOSE:
;	Direct calibration of chopped ISOPHOT-S measurements, includes
;	chopped SPRF plus first order correction. 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	CALIB_CHOPPMEAS_SL, sig_subt, flux_subt, flag, sprf_act  $
;          [,/FIRST_CORR] [,/EXTRAPOLATE]
;
; INPUT PARAMETERS:
;       sig_subt: array containing the signal to be calibrated
;
; OUTPUT PARAMETERS:
;	flux_subt: signal calibrated in Jy
;	flag: flag containing 1 if signal are outside the calibrated range
;	sprf_act: Actual SPRF (optional)
;	
; KEYWORD PARAMETERS:
;	FIRST_CORR: Includes first order correction
;       EXTRAPOLATE: extrapolate first order correction if outside the 
;		     calibrated range. No effect if first_corr is not selected
;
; COMMON BLOCKS:
; 	phtslchopp
;
; MODIFICATION HISTORY:
;       Written by: J. Acosta (IAC)                             Feb. 1999
;       V7.3
;-



PRO calib_choppmeas_SL, sig_subt, flux_subt, flag, sprf_act, t_onchop, $
  FIRST_CORR = first_corr, EXTRAPOLATE=extrapolate, INFO=info

@phtslchopp

;check that calibration file has been loaded
IF N_ELEMENTS(sl_chop_sprf) EQ 0 THEN read_specal_chop

;check that measurement t_onchop is calibrated 
IF t_onchop LE 15. THEN $
  info = 'Chopper plateau length is outside the calibrated range '


sprf_avg = sl_chop_sprf.avg
a0 = sl_chop_sprf.a0
a1 = sl_chop_sprf.a1
max_sig = sl_chop_sprf.max_sig
min_sig = sl_chop_sprf.min_sig


npix = 128
nele = N_ELEMENTS(sig_subt(0,*))
flux_subt = sig_subt
FOR Ipix=0, npix-1 DO flux_subt(ipix,*) = sig_subt(ipix,*) / sprf_avg(ipix)

corr_fact = flux_subt*0.+1.

flag = BYTARR(npix,nele)

IF KEYWORD_SET(first_corr) THEN BEGIN
   ;check limits of calibrated signals
   ind_outb = WHERE(ABS(sig_subt) GT max_sig OR ABS(sig_subt) LT min_sig,nout)
   IF nout GT 0 THEN flag(ind_outb) = 1 
    FOR ipix=0, npix-1 DO BEGIN
      ; use absolute values to 
      xx = ABS(sig_subt(ipix,*))
      IF MIN(xx) LT min_sig(ipix) OR MAX(xx) GT max_sig(ipix) THEN BEGIN
        text = 'Signal difference outside calib. range for pixel '+ $
	   STRTRIM(ipix+1,2) 
	IF N_ELEMENTS(info) EQ 0 THEN info = text ELSE info = [info,text]   
      ENDIF 
      IF NOT KEYWORD_SET(EXTRAPOLATE) THEN BEGIN
        ;check that signals do not exceed the limits of calibrated range
        FOR iele=0,nele-1 DO xx(iele) = MIN([xx(iele),max_sig(ipix)])
        FOR iele=0,nele-1 DO xx(iele) = MAX([xx(iele),min_sig(ipix)])
      ENDIF 
      corr_fact(ipix,*) = A0(ipix) + A1(ipix) * ALOG10(xx)
      flux_subt(ipix,*) = flux_subt(ipix,*) / corr_fact(ipix,*)	     
    ENDFOR  
ENDIF

sprf_act = sig_subt
FOR ipix=0, npix-1 DO sprf_act(ipix,*) = sprf_avg(ipix) * corr_fact(ipix,*)


RETURN
END 
