;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       CALIB_DYNSRD
;
; PURPOSE:
;       Calibrate an input srd structure, given a list of calibrators per
;	pixels. It produces a temporary dyncal structure. It checks that 
;	dark current was subtracted and the detector temperature is within  
;       the calibrated range.
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       CALIB_DYNSRD, SRD, SELCAL_ADMN
;
; INPUT:
;       SRD         : PHTSRD temporary structure
;	SELCAL_ADMN : Array containing  ADMNs for NCAL calibrators and each 
;			pixel [NPIX,NCAL].
;
; OUTPUTS:
;       DYNCAL      : A PHTDYNCAL temporary structure.
;
; KEYWORDS:
;	NEWHEADER   : Modified header 
;	VERBOSE     : Set to obtain information about dyn. calibration details.
;
; MODIFICATION HISTORY:
;       Written by: J.Acosta (ISO-SOC, VILSPA)               August 1998
;	V7.x
;-

PRO calib_dynsrd, srd, selcal_admn, dyncalps, tmpdyncal,  $
  VERBOSE = verbose, NEWHEADER = newheader   
  ; ,RESTORE_GLITCH = restore_glitch ,FIT_TARGET = fit_target, 
  ; WEIGHT_MEAN = WEIGHT_MEAN, 

; INPUTS:
; srd: structure to calibrate
; selcal_admn: array with the selected entries from the database (npix,ncalused)
;      

ON_ERROR,2

@phtdyncal

path_db = !CALSAVE + '/dyncal/'

det = srd.det
npix = pixel_det(det)
IF NOT KEYWORD_SET(newheader) THEN newheader = srd.hdr

;check that dark current has been subtracted
dark_done = fxpar(newheader,'PRS_DARK')
IF dark_done EQ 0 THEN print,STRING(7b), $
  'Warning: Dark current should be subtracted before dynamic calibration'

;check that detector temperature is within calbirated range
CASE srd.det OF 
 'SL': tempera_range = [2.8,3.1]
 ELSE: tempera_range = [2.6,3.5]
ENDCASE 
ind_tempera = WHERE(AVG(srd.temp) LT tempera_range(0) OR $
  srd.temp GT tempera_range(1),ntempout)
IF ntempout GT 0 THEN print,STRING(7b), $
  'Warning: Detector temperature is out of bounds ['+STRTRIM(tempera_range(0),2) $
     + ', ' + STRTRIM(tempera_range(1),2) +']'

;GET number of calibrators per pixel to be used
siz_selcal_admn = SIZE(selcal_admn) 
IF siz_selcal_admn(0) EQ 2 THEN ncal = siz_selcal_admn(2) ELSE ncal = 1 

; first look for all needed calibrators 
;admn = selcal_admn
load_calib_inuse, selcal_admn, cal_info, DET=det

;IF IFAIL THEN BEGIN
;   MESSAGE,'ERROR: Cannot access to the required calibrators ',/CONTINUE  
;ENDIF 

cal_admn = cal_info.admn
cal_time1 = cal_info.time1
cal_time2 = cal_info.time2
cal_flux = cal_info.flux
cal_grado = cal_info.grado
cal_coefcheby = cal_info.coefcheby

tempera = [AVG(srd.temp(0,*)),0.]
IF det EQ 'SL' THEN tempera(1) = AVG(srd.temp(1,*))

nele = N_ELEMENTS(srd.stim)

slun = srd.slun(0:npix-1,*) 
slop = srd.slop(0:npix-1,*)           

; compute time interval between subsequent signals and number of signals, these
;  quantities are needed for the discarding process
dims = srd.sldi
IF dims LT 5 THEN tint = AVG(srd.stim(1:dims-1) - srd.stim(0:dims-2))   $ 
 ELSE tint = MEDIAN(srd.stim(1:dims-1) - srd.stim(0:dims-2))


;define dynamic calibration temporary structure 
tmpdyncal = {admn: srd.admn,					$
	     meas_name: srd.admn,				$
	     det: srd.det,					$
	     temperature: tempera,				$
	     sldi: dims,					$
	     tinter: tint,					$
	     proc_time: !STIME,					$
	     hdr: newheader,					$
	     time: srd.stim,					$
	     sig_tar: slop,					$
	     unc_tar: slun,					$
	     sig_cal: FLTARR(nele,ncal,npix),			$
	     flmed: FLTARR(ncal,npix),				$
	     flave: FLTARR(ncal,npix),				$
	     flunc: FLTARR(ncal,npix),				$
	     calid: STRARR(ncal,npix),				$
	     rat: FLTARR(nele,ncal,npix),			$
	     urat: FLTARR(nele,ncal,npix),			$
	     mask: BYTARR(nele,ncal+1,npix),			$
	     flux: FLTARR(ncal,npix),				$
	     flag: BYTARR(ncal,npix) } 


; the first mask contains the flag of the srd, note that is a flag
; not a mask like for the subsequent indexes (different calibrators) 
tmpdyncal.mask(*,0,*) = TRANSPOSE(srd.flag(0:npix-1,*))

;CASE 1 OF
;   KEYWORD_SET(FIT_TARGET): slop = $ 
;          aprox_cheby(srd.stim,srd.slop(0:npix-1,*),srd.flag(0:npix-1,*))
;   KEYWORD_SET(RESTORE_GLITCH): BEGIN
;            restore_glitch, srd.stim, srd.slop,srd.flag,flag_glitch,$
;                 sigrestore, DET = srd.det
;            slop = sigrestore     
;          END    
;   ELSE: slop = srd.slop(0:npix-1,*)
;ENDCASE 

; stabi_det decide whether to take all signals or perform stability analysis to
; the signal ratio 
stabi_det = 0


FOR ipix=0, npix-1 DO BEGIN
  ;print,'Calibrating pixel '+STRTRIM(ipix+1,2)
  FOR ical=0,ncal-1 DO BEGIN
    admn_tmp = selcal_admn(ipix,ical)
    ind_iuniq = WHERE(cal_admn eq admn_tmp,niuniq)
    IF niuniq GT 0 THEN BEGIN 
	ind_iuniq = ind_iuniq(0)
	ind_valtime_tar = WHERE(srd.stim LE cal_time2(ind_iuniq)*1.1)
	grado_tmp = cal_grado(ind_iuniq,ipix)
	cal_sig = poly_cheby(srd.stim(ind_valtime_tar),         $
	  cal_coefcheby(0:grado_tmp,ind_iuniq,ipix),               $
	   RANGE=[cal_time1(ind_iuniq),cal_time2(ind_iuniq)])

	tmpdyncal.mask(ind_valtime_tar,ical+1,ipix) = 1B 
	tmpdyncal.sig_cal(ind_valtime_tar,ical,ipix) = cal_sig
	tmpdyncal.calid(ical,ipix) = admn_tmp
	tmpdyncal.flux(ical,ipix) = cal_flux(ind_iuniq,ipix) 
    ENDIF ELSE tmpdyncal.flag(ical,*) = 2

  ENDFOR
ENDFOR

    ;compute the ratio of target to calibrator signals and update flux calib.
    get_ratsig, slop, tmpdyncal, newheader, NPIX = npix, NCAL =ncal, SLUN=slun
    		      

fxaddpar, newheader , 'PR_DYNCA',ncal,      $ 
    'Dynamic cal: Number of calib. per pixel'
fxaddpar, newheader, 'PR_DYSTA',stabi_det,  $
    'Dynamic cal: Stability applied to sg. ratio'  

dyncalps = N_ELEMENTS(phtdyncal)
     
;tmpdyncal = {admn: srd.admn,					$
;	     meas_name: srd.admn,				$
;	     det: srd.det,					$
;	     temperature: tempera,				$
;	     hdr: newheader,					$
;	     time: srd.stim,					$
;	     sig_tar: slop,					$
;	     unc_tar: slun,					$
;	     sig_cal: target_sigcal,				$
;	     flmed: target_mdcalib,				$
;	     flave: target_avgcalib,				$
;	     flunc: target_avuncalib,				$
;	     calid: target_calid,				$
;	     rat: target_rat,					$
;	     mask: target_rat_mask,				$
;	     flux: target_flux,					$
;	     flag: target_flag } 

fill_dyncal, tmpdyncal, newheader

RETURN

END
