;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	COMBINE_DYNSRD
;
; PURPOSE:
;	Combine different calibrators by different weightings 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       COMBINE_DYNSRD, tmpdyncal, avgcomb_flux, medcomb_flux, avgcomb_uflux, $ 
;      	q1comb_flux, q3comb_flux, WEIGHT=weight, OFLAG= oflag
; 
;
; INPUT:
;       tmpdyncal 	: temporary dynamic calibration structure, as produce
;			  by calib_dynsrd
;
; KEYWORDS:
;	WEIGHT		: Weighting mode [0: no weighting, 1: mean ratio, 
;	OFLAG		: Output flag indicating the quality of calibration
;
; OUTPUTS:
;	avgcomb_flux	: Combined average fluxes
;	medcomb_flux	: Combined median fluxes
;	avgcomb_uflux	: Uncertainty of combined average fluxes
;	q1comb_flux	: First quartile of median fluxes
;	q3comb_flux	: Third quartile of median fluxes
;
; RESTRICTIONS:
;       
;
; MODIFICATION HISTORY:
;       Written by:	J.Acosta (ESA/MPIA/IAC)          	     July 1998
;	Version 7.x
;-
PRO combine_dynsrd, tmpdyncal, avgcomb_flux, medcomb_flux, avgcomb_uflux, $
     q1comb_flux, q3comb_flux, WEIGHT = weight, OFLAG= oflag

npix = pixel_det(tmpdyncal.det)


; take into account that mask is an array of dimensions (nsignal,ncal+1,npix)
; the matrix mask(*,0,*) is the 'flag' (0-1 is valid, >2 is not) of the target 
; signals. However mask(*,1:*,*) is really a mask (1 is valid, 0 is not valid)

ncal_use = N_ELEMENTS(tmpdyncal.flave(*,0))


avgcomb_flux = FLTARR(npix)
avgcomb_uflux = FLTARR(npix)
medcomb_flux = FLTARR(npix)
medcomb_uflux = FLTARR(npix)
oflag = BYTARR(npix)

;IF ncal_use EQ 1 THEN BEGIN
;   avgcomb_flux(*) = tmpdyncal.flave
;   medcomb_flux(*) = tmpdyncal.flmed
;   avgcomb_uflux(*) = tmpdyncal.flunc
;   ; computation of quartiles is temporary
;   q1comb_flux = medcomb_flux - avgcomb_uflux
;   q3comb_flux = medcomb_flux + avgcomb_uflux
;   
;   combine_dynsrd_pix,tmpdyncal.rat
;   RETURN
;ENDIF

avgcomb_flux = FLTARR(npix)
avgcomb_uflux = FLTARR(npix)
medcomb_flux = FLTARR(npix)
medcomb_uflux = FLTARR(npix)
oflag = BYTARR(npix)

FOR ipix=0, npix-1 DO BEGIN
    ind_valcal = WHERE(tmpdyncal.flag(*,ipix) LT 2,nvalcal)
    IF nvalcal GT 0 THEN BEGIN 
	  tmp_mask = REFORM(tmpdyncal.mask(*,1:*,ipix))
	  tmp_rat = REFORM(tmpdyncal.rat(*,*,ipix))
	  tmp_urat = REFORM(tmpdyncal.urat(*,*,ipix))
	  IF ncal_use GT 1 THEN tmp_flave = REFORM(tmpdyncal.flave(*,ipix)) $
	     ELSE tmp_flave = tmpdyncal.flave(*,ipix)
	  IF ncal_use GT 1 THEN tmp_flunc = REFORM(tmpdyncal.flunc(*,ipix)) $
	     ELSE tmp_flunc = tmpdyncal.flunc(*,ipix)
	  IF ncal_use GT 1 THEN tmp_flmed = REFORM(tmpdyncal.flmed(*,ipix)) $
	     ELSE tmp_flmed = tmpdyncal.flmed(*,ipix)
	  combine_dynsrd_pix, tmp_rat, weight, tmp_flave, tmp_flunc, $
	     tmp_combflave, tmp_combflunc, tmp_flmed, tmp_combflmed, $
	     tmp_combflmed_unc, UNC_RATIO=tmp_urat, NCAL_USE = ncal_use, $ 
	     MASK_RAT  = tmp_mask, COMBFLAG = combflag

	  avgcomb_flux(ipix) = tmp_combflave
	  avgcomb_uflux(ipix) = tmp_combflunc
	  medcomb_flux(ipix) = tmp_combflmed
	  medcomb_uflux(ipix) = tmp_combflmed_unc
	  oflag(ipix) = combflag
    ENDIF ELSE oflag(ipix) = 4

ENDFOR

q1comb_flux = medcomb_flux - medcomb_uflux
q3comb_flux = medcomb_flux + medcomb_uflux

;write information into header
newhdr = tmpdyncal.hdr

FXADDPAR, newhdr,'PRA_DYNO',ncal_use,$
   'Nr. of calibrators used in dynamic cal. method',BEFORE='XTENSION'

CASE weight OF 
   0: BEGIN  ; no weighting
   	comm = 'No weighting used for combination of calibrators'
	kvalcomb = 'NO_WEIGHT'
      END
   1: BEGIN  ; use the signals ratio as weighting
   	comm = $
	 'Signal ratio weight in combining calibrators'
	kvalcomb = 'WEIGHT_SIGNAL_RATIO'
      END
    2: BEGIN  ; use the STDEV from signal ratio as weighting
   	comm = $
	  'Sigma of signal ratio weight in combining calibrators'
	kvalcomb = 'WEIGHT_ERROR_RATIO'
      END
    ELSE:
ENDCASE
        	
FXADDPAR,newhdr,'PRA_DYWE',kvalcomb,comm,BEFORE='XTENSION'

;write info about flags
ind_ngt2 = WHERE(OFLAG EQ 2,ngt2)
ind_ngt10 = WHERE(OFLAG EQ 3,ngt10)

IF ngt2 GT 0 THEN $
  FOR i=0,N_ELEMENTS(ind_ngt2)-1 DO $ 
    FXADDPAR,newhdr,'COMMENT','Dynamic cal.: Signal ratio Pixel '+$
    STRTRIM(ind_ngt2(i)+1,2)+ ' is a factor >2, but <10',BEFORE='XTENSION'

IF ngt10 GT 0 THEN $
  FOR i=0,N_ELEMENTS(ind_ngt10)-1 DO $ 
    FXADDPAR,newhdr,'COMMENT','Dynamic cal.: Signal ratio Pixel '+$
    STRTRIM(ind_ngt10(i)+1,2)+ ' is a factor >10',BEFORE='XTENSION'
    
change_tmphdr, tmpdyncal, 'DYNCAL', newhdr, tmpdyncal 

RETURN
END
