;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	COMBINE_DYNSRD_PIX
;
; PURPOSE:
;	Combine different calibrators by different weightings, for a single pixel
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       COMBINE_DYNSRD_PIX, sig_ratio, weight, flave, flunc, comb_flave, $ 
;      	comb_flunc, flmed, comb_flmed, comb_flmed_unc, UNC_RATIO = unc_ratio, $
;       NCAL_USE = ncal_use, MASK_RAT = mask_rat, COMBFLAG = combflag
; 
;
; INPUT:
;       sig_ratio 	: ratio of target to calibrator signals. It is usually
;			  an array of dimension (NSIGNALS, NCAL_USE)
;	weight		: Weighting mode [0: no weighting, 1: mean ratio, 
;			  2: uncertainty of calibrated flux]  
;	flave		: Calibrated mean flux
;	flunc		: Uncertainty of calibrated mean flux
;	flmed		: Calibrated median flux [optional]
;
; KEYWORDS:
;	UNC_RATIO	: The uncertainty of sig_ratio, dimensions should be 
;			  compatible
;	NCAL_USE	: number of calibrators used, if not specified assume 1
;	MASK_RAT	: Mask of sig_ratio, dimensions should be compatible
;       COMBFLAG	: Output combination flag 
;                                  weighted ratio	 abs(log(ratio))  flag
;                                ---------------------------------------------
;                                   r > 10, <0.1	   >1		  3
;                                2<r<10 or 0.1<r<0.5	   0.3<lr<1	  2
;                                1.5<r<2 or 0.5<r<0.67     0.18<lr<0.3    1
;                                1<r<1.5 or 0.5<r<1	   0<lr<0.18	  0
;				
;
; OUTPUTS:
;	comb_flave	: Combined flave
;	comb_flunc	: uncertainty of Combined flave
;	comb_flmed	: Combined flmed
;	comb_flmed_unc	: uncertainty of combined flmed
;
; RESTRICTIONS:
;       
;
; MODIFICATION HISTORY:
;       Written by:	J.Acosta (ESA/MPIA/IAC)          		July 1998
;	Version 7.x
;-
PRO combine_dynsrd_pix, sig_ratio,  weight, flave, flunc, comb_flave, $ 
      comb_flunc, flmed, comb_flmed, comb_flmed_unc, UNC_RATIO = unc_ratio, $
      NCAL_USE = ncal_use, MASK_RAT = mask_rat, COMBFLAG = combflag
      
      
IF N_ELEMENTS(unc_ratio) EQ 0 THEN unc_ratio = sig_ratio * 0.	
IF N_ELEMENTS(mask_rat) EQ 0 THEN mask_rat = FIX(sig_ratio * 0. + 1)	
IF NOT KEYWORD_SET(scal_we) THEN scal_we = 2.
; use median if more than 15 points
IF N_ELEMENTS(sig_ratio) GT 15 THEN median = 1 ELSE median = 0
IF NOT KEYWORD_SET(ncal_use) THEN ncal_use = 1

;compute a mean/median value of the ratio, used for weighting according to 
; ratio and also for flagging
tmp_rat = FLTARR(ncal_use)
FOR ical=0, ncal_use-1 DO BEGIN
   ind_val = WHERE(mask_rat(*,ical) EQ 1,nval)
   IF NVAL GT 0 THEN WEIGHT_MEAN, REFORM(sig_ratio(ind_val,ical)),  $
       REFORM(unc_ratio(ind_val,ical)), medi, MEDIAN=median  $
      ELSE medi= 0.
   tmp_rat(ical) = ABS(medi)
ENDFOR



;compute weighting factors as needed

CASE weight OF 
   1: BEGIN  ; use the ratio as weighting factor 
    	  we = exp(-ABS(ALOG10(tmp_rat))*SCAL_WE*1.D0)
      END
   2: BEGIN  ; use the uncertainty as weighting factor
          ; Avoid unc = 0.
	  unc = flunc
	  ind = where(unc EQ 0., nzero)
          IF nzero GT 0 THEN check_unc, flave, unc
          we = 1./unc^2
      END
   ELSE: we = make_array(ncal_use,value=1.)     
ENDCASE

; compute weight mean for flave
weight_mean, flave, flunc, mean, stdev, SET_WEIGHT = we
comb_flave = mean
comb_flunc = stdev

;assign a flag depending on the weighted ratio
; combflag is a flag formed by two bits
; IF weighted ratio      ABS(log(ratio)) --> flag
;     r > 10, <0.1           >1             3
;  2<r<10 or 0.1<r<0.5       0.3<lr<1       2
;  1.5<r<2 or 0.5<r<0.67     0.18<lr<0.3    1
;  1<r<1.5 or 0.5<r<1        0<lr<0.18      0
rat_weight= TOTAL(we*ABS(ALOG10(tmp_rat)))/TOTAL(we)
CASE 1 OF
   rat_weight GT 1.: combflag = 3
   rat_weight GT 0.3: combflag = 2
   rat_weight GT 0.18: combflag = 1
   ELSE: combflag = 0
ENDCASE    

IF N_ELEMENTS(flmed) NE 0 THEN BEGIN 
	weight_mean, flmed, flunc, mean, stdev, SET_WEIGHT = we
	comb_flmed = mean
	comb_flmed_unc = stdev
ENDIF  


RETURN
END
