;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GET_RATSIG_PIX
;
; PURPOSE:
;	Get the ratio of the signal of the target to a given calibrator,
;	including the flag for the target and the mask for the calibrator.
;	  	
;
; CATEGORY:
;	PIA - process
;
; CALLING SEQUENCE:
;	GET_RATSIG, target_signal, target_flag, calib_sig, calib_mask, $
;	    
;	
; KEYWORD PARAMETERS:
;	NPIX:	Number of pixels
;       NCAL:   Number of calibrators to be used
;	
; COMMON BLOCKS:
;	phtacc_ratsig
;
; RESTRICTIONS:
;       target_signal must be a vector [npix,nsignal] and tmpdyncal a 
;	 valid temporary dynamic cal structure
;		
; PROCEDURES USED:
;	PIA LIB:
;		decode_admn
;       ASTRO LIB:
;		avg
;       INTERNAL:
;		pixbutevent2
; MODIFICATION HISTORY:
;       Written by: J.Acosta (ISO-SOC, VILSPA)               August 1998
;	V7.x
;-
PRO get_ratsig, slop, tmpdyncal, newheader, NPIX = npix, NCAL = ncal, SLUN=slun

COMMON phtacc_ratsig, phtacc_ratsig

IF N_ELEMENTS(phtacc_ratsig) EQ 0 THEN phtacc_ratsig_init

IF NOT KEYWORD_SET(NPIX) THEN npix = PIXEL_DET(tmpdyncal.det)
IF NOT KEYWORD_SET(ncal) THEN ncal = 1 

IF N_ELEMENTS(slun) EQ 0 THEN slun = slop*0. 
   

IF N_ELEMENTS(newheader) EQ 0 THEN newheader = tmpdyncal.hdr

nsignal = tmpdyncal.sldi
treset = tmpdyncal.tinter

;selcrit = phtacc_ratsig.selcrit
;    CASE selcrit OF
;          0: sigupto = phtacc_ratsig.firstsig
;          1: sigupto = ROUND(phtacc_ratsig.mfrac * nsignal)
;          2: sigupto = phtacc_ratsig.sigfrom
;          3: sigupto = ROUND(phtacc_ratsig.timefrom/treset)
;          ELSE: sigupto = 0
;    ENDCASE 
sigupto = GET_SIGUPTO(phtacc_ratsig, nsignal, treset)
         
fxaddpar, newheader, 'PR_DYDIS',sigupto,  $
    'Dynamic cal: First Signal ratios discarded'  

slop_flag = REFORM(tmpdyncal.mask(*,0,*)) 

cal_mask = tmpdyncal.mask(*,1:*,*)  
cal_sig = tmpdyncal.sig_cal


FOR ipix=0, npix-1 DO BEGIN 
  ;print,'Calibrating pixel '+STRTRIM(ipix+1,2)
  FOR ical=0,ncal-1 DO BEGIN
  
    ratio_time  = FLTARR(nsignal)
    uratio_time  = FLTARR(nsignal)

    ind_valtime = WHERE(cal_mask(*,ical,ipix) EQ 1, ncal_sig)
    
    tmp_slopflag = MAKE_ARRAY(N_ELEMENTS(slop(ipix,*)),VALUE=1B)
    IF sigupto GT 0 THEN tmp_slopflag(0:sigupto-1) = 0B 
    
    tmp_slopflag = (slop_flag(*,ipix) LE 1) AND tmp_slopflag 
    
    IF ncal_sig GT 0 THEN BEGIN 
       ind_val_sig = WHERE(tmp_slopflag(ind_valtime) EQ 1,nvalsig)
       rat_tmp = REFORM(slop(ipix,ind_valtime)/cal_sig(ind_valtime,ical,ipix))
       urat_tmp = REFORM(slun(ipix,ind_valtime)/cal_sig(ind_valtime,ical,ipix))
       ratio_time(ind_valtime) = rat_tmp       
       uratio_time(ind_valtime) = urat_tmp
    ENDIF
    
    ratio_flag = 0
    
    CASE 1 OF 
       ncal_sig EQ 0: BEGIN
                        ratio_flag =  2
                        mome = 0.
			unc = 0.
	                med = mome
		      END	
       nvalsig GT 4:  BEGIN
			weight_mean, rat_tmp(ind_val_sig), $ 
			  urat_tmp(ind_val_sig), mome, unc
			med = MEDIAN(rat_tmp)
		      END
       nvalsig GT 1:  BEGIN 
                        weight_mean, rat_tmp(ind_val_sig), $ 
			  urat_tmp(ind_val_sig), mome, unc
	                med = mome
                      END   
       nvalsig EQ 1:  BEGIN 
                        mome = rat_tmp(ind_val_sig)
			unc = urat_tmp(ind_val_sig)
	                med = mome
			ratio_flag = 1
       		      END
       ELSE:          BEGIN 
                        mome = 0.
			unc = 0.
	                med = mome
                        ratio_flag = 2
		      END		
    ENDCASE     	       	

    tmpdyncal.rat(*,ical,ipix) = ratio_time
    tmpdyncal.urat(*,ical,ipix) = uratio_time
    tmpdyncal.flmed(ical,ipix) = tmpdyncal.flux(ical,ipix) * med 
    tmpdyncal.flave(ical,ipix) = tmpdyncal.flux(ical,ipix) * mome
    tmpdyncal.flunc(ical,ipix) = tmpdyncal.flux(ical,ipix) * unc
    tmpdyncal.flag(ical,ipix) = ratio_flag

  ENDFOR
ENDFOR

RETURN
END
