;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GETFLUXDB_TIME
; PURPOSE:
;       Converts a given signal [V/s] to flux [Jy], for a given time (fixed 
;	value) and pixel, by interpolating along the signal axis. 
; CATEGORY:
;       PIA - I/O
;
; CALLING SEQUENCE:
;       FLUX=GETFLUXDB_TIME
;
; INPUT PARAMETERS:
;       (none)
;
; OPTIONAL INPUT PARAMETERS:
;       FILENAME: The file to be read
;       PATH: The directory to find the file to be read
;       TABLE: To read a savefile instead of cal-G file
;
; OUTPUT PARAMETERS:
;       NONE
;
; RESTRICTIONS:
;       FILENAME must be a valid PHT PSPECAL fits file
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               phtslchopp
;       Astronomical Library:
;               fxhread
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: J. Acosta (IAC)                             Feb. 1999
;       V8.1
;-
FUNCTION getfluxdb_time_blin, s, ltime_meas, ipix, WEIGHT=weight
; this function returns the assigned flux for a given signal, 
; by interpolating the calibration tables for a given time. 

@phtdyncaltab

IF N_ELEMENTS(sl_dyncal_tab) EQ 0 THEN read_specal_dyncal

lf = s*0.
weight = s*0.

;check for negative signals 
ind_neg = WHERE(s LE (-1.*sl_dyncal_tab.signal_zero),nneg)
nsig_neg = SL_DYNCAL_TAB.nsig_neg
IF nneg GT 0 THEN BEGIN 
  ff = sl_dyncal_tab.lflux(0:nsig_neg-1,*,ipix)
  zz = sl_dyncal_tab.weight(0:nsig_neg-1,*,ipix)
  ; interpol along signal logarithmic axis to get gridding 
  xx = ALOG10(-sl_dyncal_tab.lsig(0:nsig_neg-1))
  ixx = INDGEN(nsig_neg)
  ix = INTERPOL(ixx, xx, ALOG10(-s(ind_neg)))
  ; interpol along time logarithmic axis to get gridding for signals
  yy = sl_dyncal_tab.ltime
  iyy = INDGEN(N_ELEMENTS(yy))
  iy = INTERPOL(iyy, yy, ltime_meas(ind_neg))
  lf(ind_neg) = INTERPOLATE(ff,ix,iy)
  weight(ind_neg) = INTERPOLATE(zz,ix,iy)
ENDIF

;select zero signals
ind_zero = WHERE(ABS(s) LT sl_dyncal_tab.signal_zero,nzero)
IF nzero GT 0 THEN BEGIN 
  ff = REFORM(sl_dyncal_tab.lflux(nsig_neg,*,ipix))
  zz = REFORM(sl_dyncal_tab.weight(nsig_neg,*,ipix))
  ; no need to interpol along signal axis
  ; interpol along time logarithmic axis to get gridding for signals
  yy = sl_dyncal_tab.ltime
   lf(ind_zero) = INTERPOL(ff,yy,ltime_meas(ind_zero))
   weight(ind_zero) = INTERPOL(zz,yy,ltime_meas(ind_zero))
ENDIF


;select positive signals 
ind_pos =  WHERE(s GE sl_dyncal_tab.signal_zero, npos)
nsig_pos= N_ELEMENTS(sl_dyncal_tab.lsig) - nsig_neg -1 
IF npos GT 0 THEN BEGIN
  ff = sl_dyncal_tab.lflux(nsig_neg+1:*,*,ipix)
  zz = sl_dyncal_tab.weight(nsig_neg+1:*,*,ipix)
  ; interpol along signal logarithmic axis to get gridding 
  xx = ALOG10(sl_dyncal_tab.lsig(nsig_neg+1:*))
  ixx = INDGEN(nsig_pos)
  ix = INTERPOL(ixx, xx, ALOG10(s(ind_pos)))
  ; interpol along time logarithmic axis to get gridding for signals
  yy = sl_dyncal_tab.ltime
  iyy = INDGEN(N_ELEMENTS(yy))
  iy = INTERPOL(iyy, yy, ltime_meas(ind_pos))
  lf(ind_pos) = INTERPOLATE(ff,ix,iy)
  weight(ind_pos) = INTERPOLATE(zz,ix,iy)
ENDIF
 

RETURN, lf
END

