;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;     WRITE_DYNCALRESP
; PURPOSE:
;       Write responsivity values depending on orbital position to a FITS file
;
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       WRITE_DYNCALRESP, DET, FLUX, WEIGHT, FLUX_MAX, FLUX_MIN, 
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       det: character string of 'C1', 'C2', 'P'
;       RESPORB_STR: 
;
; OPTIONAL INPUT:
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;       PIA library:
;               c2p_IA2pipe: changing order of C2 pixels 
;
; MODIFICATION HISTORY:
;       Written by: Jose Acosta
;       V6.6
;;
;-
pro write_dyncalresp, det, flux, weight, flux_max, flux_min, $
       COMMENT=comment, VERSION=version, PATH = path, $ 
       OVERWRITE = overwrite, CALSAVE = calsave 
       
  on_error, 2
  
  IF N_PARAMS() LT 1 THEN BEGIN
    print,'Syntax: write_dyncalresp, det'
    RETURN
  ENDIF
  
  CASE det OF
    'SL': BEGIN 
            npix = 128
	    pfilename = 'PSDYNAMIC'
	    tcomment = 'PHT-S Dynamic Calibration - corrected flux'
	    tcomment1 = 'PHT-S Dynamic Calibration - weighting function'
	    pwfilename = 'PSDYNWT'
	    IF KEYWORD_SET(calsave) THEN BEGIN 
	        calsave = 'SL_DYNCAL_CALG.SAV'
		restore,!CALSAVE+'dyncal_olp/'+ calsave
	    ENDIF	
	  END 
    ELSE: BEGIN
      message, 'DETECTOR not valid. Accepted detector is only SL'
      RETURN
    END
  ENDCASE

  filetmp = pfilename + '.FITS'
  filetmp1 = pwfilename + '.FITS'
  
  IF NOT KEYWORD_SET(path) THEN path = !CALSAVE +'dyncal_olp/'
  fitsfile = path + filetmp  
  fitsfile1 = path + filetmp1  
  
;check if PSDYNAMIC file exists
  res = FINDFILE(fitsfile, count=count)
  IF count NE 0 THEN newfile = 0 ELSE newfile = 1
; if newfile then read header, if not read from fitsfile
  IF newfile EQ 0 THEN SXHREAD, fitsfile, newhdr
     
  cflux = flux
  
;check if PSDYNWT file exists
  res = FINDFILE(fitsfile1, count=count) 
  IF count NE 0 THEN newfile1 = 0 ELSE newfile1 = 1
; if newfile then define array dark_calg, if not read from fitsfile
  IF newfile EQ 0 THEN SXHREAD, fitsfile1, newhdr1
  
  cweight = weight     
  
  ; if file does not exist then create hdr
  IF newfile EQ 1 THEN BEGIN 
      print,pfilename+': Creating new header '
      mkhdr, newhdr, cflux
      fxaddpar, newhdr, 'ORIGIN', 'ESA', 'European Space Agency'
      fxaddpar, newhdr, 'TELESCOP', 'ISO', 'Infrared Space Observatory'
      fxaddpar, newhdr, 'INSTRUME', 'PHT', 'Instrument used'
      fxaddpar, newhdr, 'FILENAME', pfilename, $   
                            'File name assigned in ISO archive'
      fxaddpar, newhdr, 'FILEVERS', '', 'Version ID assigned in ISO archive'    
  ENDIF 
 
; write the limits of calibrated range
FOR ipix=0,npix-1 DO BEGIN 
              fxaddpar,newhdr,'FLX'+STRTRIM(ipix+1,2)+'U', flux_max(ipix),$
                    'Upper limit(Jy) of flux calibration for pixel ' +    $
		    STRTRIM(ipix+1,2)
              fxaddpar,newhdr,'FLX'+STRTRIM(ipix+1,2)+'L', flux_min(ipix),$
                    'lower limit(Jy) of flux calibration for pixel ' +    $
		    STRTRIM(ipix+1,2)
ENDFOR		       
			  
      fxaddpar, newhdr, 'DATE', STRMID(!STIME,0,11), 'Date of creating file 
      fxaddpar, newhdr, 'COMMENT', tcomment
      fxaddpar, newhdr, 'PIA_VERS',!PIA_VERSION, $ 
         'Internal (user) PIA-CAL Version'
      
  
  ; if file does not exist then create hdr
  IF newfile1  EQ 1 THEN BEGIN 
      print,pwfilename+': Creating new header '
      mkhdr, newhdr1, cweight
      fxaddpar, newhdr1, 'ORIGIN', 'ESA', 'European Space Agency'
      fxaddpar, newhdr1, 'TELESCOP', 'ISO', 'Infrared Space Observatory'
      fxaddpar, newhdr1, 'INSTRUME', 'PHT', 'Instrument used'
      fxaddpar, newhdr1, 'FILENAME', pwfilename, $   
                            'File name assigned in ISO archive'
      fxaddpar, newhdr1, 'FILEVERS', '', 'Version ID assigned in ISO archive'    
  ENDIF 
 
      fxaddpar, newhdr1, 'DATE', STRMID(!STIME,0,11), 'Date of creating file 
      fxaddpar, newhdr1, 'COMMENT', tcomment1
      fxaddpar, newhdr1, 'PIA_VERS',!PIA_VERSION, $ 
         'Internal (user) PIA-CAL Version'
      
  ; write FITS file  
      IF newfile EQ 0 THEN $
         IF NOT KEYWORD_SET(OVERWRITE) THEN fitsfile = fitsfile + '.NEW' $
             ELSE  spawn,'/bin/rm '+EXPAND_PATH(fitsfile)
      print,'WRITING FLUX CAL-G FILE -> ',fitsfile	             
      writefits, fitsfile, cflux, newhdr  
 
      IF newfile1 EQ 0 THEN $
         IF NOT KEYWORD_SET(OVERWRITE) THEN fitsfile1 = fitsfile1 + '.NEW' $
             ELSE  spawn,'/bin/rm '+EXPAND_PATH(fitsfile1)        
      print,'WRITING WEIGHT CAL-G FILE -> ',fitsfile1	             
      writefits, fitsfile1, cweight, newhdr1  
 
END
