;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FFMAP
;
; PURPOSE:
;	Compute flat fielding parameters of C100, C200 maps
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FFMAP, gridsize, ndist, x_coord, y_coord, dwel_xpd, bright, uncert, $
;		flag, ff, ERROR=ERROR, p_map_expose=p_map_expose 
;
; INPUTS:
;	GRIDSIZE	The map gridsize [FLTARR(2)]
;
;	NDIST		Distance of map pixel centre to detector pixel centre
;				in units of map pixel dimensions [FLTARR(2)]
;
;	X_COORD		Map x-coordinates of input data
;			as calculated by PIA_MAPSET [FLTARR(npixels,n)]
;
;	Y_COORD		Map y-coordinates of input data
;			as calculated by PIA_MAPSET
;
;	DWEL_XPD	Dwell time over detector area [FLTARR(npixels,n)]
;
;	BRIGHT		Brightness density array [FLTARR(npixels,n)]
;
;	UNCERT		Brightnes density uncertainty array [FLTARR(npixels,n)]
;
;	FLAG		Flag array [FLTARR(npixels,n)]
; 
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output arguments are not correct
;
; OUTPUTS:
;	FF		The flat field correction factors normalized 
;				to the average of used pixels
;
;
; MODIFICATION HISTORY:
;	Written by: 	C. Gabriel, ESA/VILSPA-SAI	March 1996
;	Modified by CG
;	V5.x - November 96
;-
PRO ffmap, gridsize, ndist, x_coord, y_coord, dwel_xpd, bright, uncert, $
	flag, ff, ERROR=ERROR

	ndata = N_ELEMENTS(flag(0,*))
	npix = N_ELEMENTS(flag(*,0))

	det_size = ndist * 2. - 1.
	max_overlap_x = MIN([det_size(0),1.])
	max_overlap_y = MIN([det_size(1),1.])
	
	chdim = gridsize - [8,8]
	dim0st = 3
	dim1st = 3
	IF min(chdim) LT 1 THEN  BEGIN
	   PRINT, 'Gridsize too small for flat fielding'  
	   GOTO, no_ffpos
	ENDIF
	
	IF gridsize(0) GT 20 THEN BEGIN
		chdim(0) = 12
		dim0st = gridsize(0) / 2 - 10
	ENDIF
	IF gridsize(1) GT 20 THEN BEGIN
		chdim(1) = 12
		dim1st = gridsize(1) / 2 - 10
	ENDIF

	p_map_expose = fltarr(npix,chdim(0),chdim(1))
	p_map_bright = p_map_expose

	FOR ipix=0,npix-1 DO BEGIN $
	 FOR x = dim0st,dim0st+chdim(0)-1 DO $
	  FOR y = dim1st,dim1st+chdim(1)-1 DO BEGIN
	    xred = x-dim0st
	    yred = y-dim1st
	    index = WHERE( ABS(x_coord(ipix,*)-x) LT ndist(0) AND $
	                   ABS(y_coord(ipix,*)-y) LT ndist(1) AND $
	                   (flag(ipix,*) EQ 0)                   $
	                   , nfound )

	    IF nfound GT 0 THEN BEGIN
	      ; overlapping area in units of map pixel dimensions
	      overlapx = ndist(0)-ABS(REFORM(x_coord(ipix,index)-x))
	      indx = where(overlapx GT max_overlap_x, novx)
	      IF novx GT 0 THEN overlapx(indx) = max_overlap_x
	      
	      overlapy = ndist(1)-ABS(REFORM(y_coord(ipix,index)-y))
	      indy = where(overlapy GT max_overlap_y, novy)
	      IF novy GT 0 THEN overlapy(indy) = max_overlap_y
	      
;	      overlap = ( MIN([ndist(0)-$
;	      		ABS(REFORM(x_coord(ipix,index)-x)),max_overlap_x]) ) $
;			* $
;	                ( MIN([ndist(1)-$
;	                ABS(REFORM(y_coord(ipix,index)-y)),max_overlap_y]) )

	      overlap = overlapx * overlapy

	      weight = dwel_xpd(ipix,index) * overlap
	      p_map_expose(ipix,xred,yred) = FLOAT( TOTAL(weight) )
	      p_map_bright(ipix,xred,yred) = FLOAT( TOTAL(weight*bright(ipix,index)) $
	                               / p_map_expose(ipix,xred,yred) )
	    ENDIF
	  ENDFOR
	ENDFOR

	indall = indgen(N_ELEMENTS(p_map_expose(0,*,*)))
	npix_used = intarr(npix) + 1
	FOR ipix=0,npix-1 DO BEGIN
		ind = WHERE(p_map_expose(ipix,*,*) NE 0., n)
		IF n GT 0 THEN BEGIN
			match, indall, ind, suba, subb, count=count
			IF count EQ 0 THEN GOTO, no_ffpos
			indall = indall(suba)
		ENDIF ELSE npix_used(ipix) = 0
	ENDFOR

	ff = fltarr(npix)
	

	indx = indall mod chdim(0)
	indy = indall / chdim(0)

	FOR ipix=0,npix-1 DO BEGIN
		ff(ipix) = median(p_map_bright(ipix,indx,indy))
	ENDFOR

	ind_used = WHERE(npix_used NE 0, nus)
	IF nus EQ 0 THEN BEGIN
		MESSAGE, /INFO, 'No pixel usable for flat field, returning'
		help, p_map_expose
		GOTO, no_ffpos
	ENDIF 
	
	ff = ff / avg(ff(ind_used))
	FOR ipix=0,npix-1 DO BEGIN
	  IF npix_used(ipix) THEN BEGIN
	  	bright(ipix,*) = bright(ipix,*) / ff(ipix)
	  	uncert(ipix,*) = uncert(ipix,*) / ff(ipix)
	  ENDIF  
	ENDFOR

	ERROR = 0
	
	RETURN

no_ffpos:
	ERROR = 1
	message,/info, 'Flat Fielding not possible'
	RETURN


	END
