;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FFMAP_SIMPLE
;
; PURPOSE:
;	Compute flat fielding parameters of C100, C200 maps assuming perfect
;		P32-AOT performance (no use of astronomical coordinates)
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FFMAP_SIMPLE, dim, y_low, z_low, cover, dwel, ratio, $
;			bright, uncert, flag, ff, ERROR=ERROR 
; INPUTS:
;	DIM		The map gridsize [FLTARR(2)]
;
;	Y_LOW		First map pixel (in Y-direction) covered by detector 
;			pixel [FLTARR(npixels,n)]
;
;	Z_LOW		First map pixel (in Z-direction) covered by detector 
;			pixel [FLTARR(npixels,n)]
;
;	COVER		Number of grid points covered by one detector pixel
;		
;	DWEL		Dwell time [FLTARR(npixels,n)]
;
;	RATIO		Ratio of map pixel over detector pixel area
;
;	BRIGHT		Brightness density array [FLTARR(npixels,n)]
;
;	UNCERT		Brightnes density uncertainty array [FLTARR(npixels,n)]
;
;	FLAG		Flag array [FLTARR(npixels,n)]
; 
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output arguments are not correct
;
; OUTPUTS:
;	FF		The flat field correction factors normalized 
;				to the average of the pixels used
;
;
; MODIFICATION HISTORY:
;	Written by: 	C. Gabriel, ESA/VILSPA-SAI	March 1996
;	Modified by CG	(last: November 1996)
;	V5.x
;-
PRO ffmap_simple, dim, y_low, z_low, cover, dwel, ratio, $
	bright, uncert, flag, ff, ERROR=ERROR 

	n_plat = N_ELEMENTS(flag(0,*))
	npix = N_ELEMENTS(flag(*,0))

	chdim = dim
	dim0st = 0
	dim1st = 0
	
	IF dim(0) GT 20 THEN BEGIN
		chdim(0) = 20
		dim0st = dim(0) / 2 - 10
	ENDIF
	IF dim(1) GT 20 THEN BEGIN
		chdim(1) = 20
		dim1st = dim(1) / 2 - 10
	ENDIF
	
	p_map_expose = fltarr(npix,chdim(0),chdim(1))
	p_map_bright = p_map_expose
	weight = TRANSPOSE(REBIN( dwel * ratio, $
				n_plat, npix, /SAMPLE ))

	FOR ipix=0,npix-1 DO BEGIN 
	 FOR x = dim0st, dim0st+chdim(0)-1 DO $
	  FOR y = dim1st, dim1st+chdim(1)-1 DO BEGIN
	    xred = x-dim0st
	    yred = y - dim1st
	    index = WHERE( (y_low(ipix,*) LE x AND y_low(ipix,*) GT x-cover(0)) AND $
		         (z_low(ipix,*) LE y AND z_low(ipix,*) GT y-cover(1)) AND $
	                 (flag(ipix,*) EQ 0) $
	                , nfound )
	    IF nfound GT 0 THEN BEGIN

	      p_map_expose(ipix,xred,yred) = FLOAT( TOTAL(weight(index) ))
	      p_map_bright(ipix,xred,yred) = $
	      	FLOAT( TOTAL(weight(index)*bright(ipix,index)) $
	                               / p_map_expose(ipix,xred,yred) )
	    ENDIF
	  ENDFOR
	ENDFOR

	indall = indgen(N_ELEMENTS(p_map_expose(0,*,*)))
	npix_used = intarr(npix) + 1

	FOR ipix=0,npix-1 DO BEGIN
		ind = WHERE(p_map_expose(ipix,*,*) NE 0., n)
		IF n GT 0 THEN BEGIN
			match, indall, ind, suba, subb, count=count
			IF count EQ 0 THEN GOTO, no_ffpos
			indall = indall(suba)
		ENDIF ELSE npix_used(ipix) = 0
	ENDFOR

	ff = fltarr(npix)
	indx = indall mod chdim(0)
	indy = indall / chdim(0)
	FOR ipix=0,npix-1 DO BEGIN
		ff(ipix) = median(p_map_bright(ipix,indx,indy))
	ENDFOR

	ind_used = WHERE(npix_used EQ 1, nus)
	IF nus EQ 0 THEN BEGIN
		MESSAGE, /INFO, 'No pixel usable for flat field, returning'
		GOTO, no_ffpos
	ENDIF 
	
	ff = ff / avg(ff(ind_used))
	FOR ipix=0,npix-1 DO BEGIN
	  IF npix_used(ipix) THEN BEGIN
	  	bright(ipix,*) = bright(ipix,*) / ff(ipix)
	  	uncert(ipix,*) = uncert(ipix,*) / ff(ipix)
	  ENDIF  
	ENDFOR

	ERROR = 0
	
	RETURN

no_ffpos:
	ERROR = 1
	message,/info, 'Flat Fielding not possible'
	RETURN


	END
