;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FILT_NO
;
; PURPOSE:
;	Determine filter sequence number. This number is defined
;	first by the sequence of the detector subsystem in the
;	order: P1, P2, P3, C100, C200, and second by the sequence
;	of rising nominal central wavelengths of the filters
;	used. The numbers are assigned only for practically used
;	filterpositions, e.g. backup filters etc. are left out.
;	The filter name can be any string that has been used so
;	far to unambiguous distinguish the filter. The 90micron
;	backup filter is not recognised and will be taken as 
;	90micron standard filter.
;	The first character must be either 'P' or 'C', regardless
;	of case!
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FILT_NO(filter)
;
; INPUT:
;	filter:		(string) 
;
;KEYWORD PARAMETERS:	
;	none
;
; OUTPUT:
;	function:	(integer) filter number as described
;			above
;
; COMMON BLOCKS:
;
; RESTRICTIONS:
;
; PROCEDURES USED:	
;	EXTR_NUM
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;	Written by:	Bernhard Schulz
;	25/10/96	first version		B. Schulz
;	19/01/99	bugfix in 2. num. part	B. Schulz
;-
function filt_no, filter


case strupcase(filter) of
   'P_3P29':	filno=1
   'P_3P3':	filno=1
   'P_3P6': 	filno=2
   'P_4P85':	filno=3
   'P_4P9':	filno=3
   'P_7P3': 	filno=4
   'P_7P7': 	filno=5
   'P_10': 	filno=6
   'P_10P0': 	filno=6
   'P_11P3':	filno=7
   'P_11P5':	filno=8
   'P_12P0':	filno=8
   'P_12':	filno=8
   'P_12P8':	filno=9
   'P_16': 	filno=10
   'P_16P0': 	filno=10
   'P_20': 	filno=11
   'P_25': 	filno=12
   'P_60': 	filno=13
   'P_100':	filno=14
   'C_50': 	filno=15
   'C_60': 	filno=16
   'C_70': 	filno=17
   'C_90': 	filno=18
   'C_100': 	filno=19 
   'C_105': 	filno=20
   'C_120': 	filno=21
   'C_135': 	filno=22
   'C_160': 	filno=23
   'C_175': 	filno=23
   'C_180': 	filno=24
   'C_200': 	filno=25
   'P1_3P29':	filno=1
   'P1_3P3':	filno=1
   'P1_3P6': 	filno=2
   'P1_4P85':	filno=3
   'P1_4P9':	filno=3
   'P1_7P3': 	filno=4
   'P1_7P7': 	filno=5
   'P1_10': 	filno=6
   'P1_10P0': 	filno=6
   'P1_11P3':	filno=7
   'P1_11P5':	filno=8
   'P1_12P0':	filno=8
   'P1_12':	filno=8
   'P1_12P8':	filno=9
   'P1_16': 	filno=10
   'P1_16P0': 	filno=10
   'P2_20': 	filno=11
   'P2_25': 	filno=12
   'P3_60': 	filno=13
   'P3_100':	filno=14
   'C1_50': 	filno=15
   'C1_60': 	filno=16
   'C1_70': 	filno=17
   'C1_90': 	filno=18
   'C1_100': 	filno=19 
   'C1_105': 	filno=20
   'C2_120': 	filno=21
   'C2_135': 	filno=22
   'C2_160': 	filno=23
   'C2_175': 	filno=23
   'C2_180': 	filno=24
   'C2_200': 	filno=25
   'C100_50': 	filno=15
   'C100_60': 	filno=16
   'C100_70': 	filno=17
   'C100_90': 	filno=18
   'C100_100': 	filno=19 
   'C100_105': 	filno=20
   'C200_120': 	filno=21
   'C200_135': 	filno=22
   'C200_160': 	filno=23
   'C200_175': 	filno=23
   'C200_180': 	filno=24
   'C200_200': 	filno=25

   else: begin 

;----------------------------------------------------------------
; In case no known code is found
; Extract numerical substring 
; if there are two, take second

    f = strupcase(filter)

    i = strpos(f,'P',1)			;look for 'P' as decimal 
    					;point
    if i GE 0 then strput, f, '.', i	;replace 'P'

    a = extr_num(f)			;extract first numerical 
    					;part

    x = ''				;generate non numerical 
    					;replace string
    for i=0, strlen(a)-1 do x=x+'a'

    strput,f,x,strpos(f,a) 		;replace first num part

    b = extr_num(f)			;extract second numerical
    					;part

    if b EQ 'E' OR b EQ 'D' then b=''	;catch appearance of d or e
    					;in nonnumerical string 


    if strlen(b) EQ 0 then f=a else f=b


	if strlen(f) GT 0 then begin
	
	
	  
	  flt = float(f)

	  case strmid(strupcase(filter),0,1) of
	  'P': begin
	    case flt of
	      3.29: filno=1
	      3.3: filno=1
	      3.6:  filno=2
	      4.85: filno=3
	      4.9: filno=3
	      7.3:  filno=4
	      7.7:  filno=5
	      10.0: filno=6
	      11.3: filno=7
	      11.5: filno=8
	      12.0: filno=8
	      12.8: filno=9
	      16.0: filno=10
	      20.0: filno=11
	      25.0: filno=12
	      60.0: filno=13
	      100.0:filno=14
	      else: filno=-1
	    endcase    
	       end
		
	  'C': begin
	    case flt of
	      50.0: filno=15
	      60.0: filno=16	
	      70.0: filno=17
	      90.0: filno=18
	      100.0: filno=19
	      105.0: filno=20
	      120.0: filno=21
	      135.0: filno=22
	      160.0: filno=23
	      180.0: filno=24
	      200.0: filno=25
	      else: filno = -1 
	    endcase    
	       end
	       
	  else: filno=-1
         endcase	; (case strmid..)
	endif $		; (if strlen..)
	   else filno = -1
    end			; (else..)
endcase  		; (case strupcase..)  



return, filno

end
