;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FIT_FUNCTION
;
; PURPOSE:
;	Interface routine between measurement, signal drift parameters and 
;	fitting routine
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FIT_Function, xx, yy, yyerr, yyflag, drift_par, step=STEP, $ 
;		RASTERSTEP = rasterstep
;
; INPUT:
;	xx:	The independent variable
;	yy:	The dependant variable
;	yyerr:	Uncertainty array of yy
;	yyflag:	Flag array corresponding to yy
;	
;
; KEYWORD PARAMETERS:
;	Step:	Chopper step (not set for non chopped measurements)
;	WEIGHT: indicate which weighting system is used among the
;		following possibilities:
;			sigma 	=  1/sigma^2   (DEFAULT)
;			none	= 1
;			time    = exp((t-t_zero)/512.)
;			signal	= 1/abs(signal)	
;	          
;	
; OUTPUT:
;	drift_par:	A PIA_drift_par likely structure, containing e.g. the 
;			fitted parameters, sigmas, etc
;	
; COMMON BLOCKS:
;	(none) 
;		
; PROCEDURES USED:
;	PIA lib:
;		pia_curfit
;
; MODIFICATION HISTORY:
;	Written by: C.Gabriel (ESA/VILSPA-SAI)	  August 1996
;	New way to estimate the weights of points
;	for the fit. A normalization based on stdev
;	of the difference between consecutive signals
;	J. Acosta				  January 1997
;	Version 5.0 - Aug.96
;	Bug in the flag treatment for chopped meas., 
;	+ change call of get_nchoppsweep - JAcosta,  Aug 1997  
;			
;-
PRO FIT_Function, xx, yy, yyerr, yyflag, drift_par, STEP=step, WEIGHT= weight,$
     Verbose=verbose, NOT_AMOEBA=not_amoeba, NODERIVATIVE= noderivative, $
     NOT_MARQUAD = not_marquad
;
; This routine perform a fitting to a specific function
;
; INPUTS:
; xx: abscisae array
; yy: values to be fitted
; yyerr: error in the values to be fitted
; yyflag: flag set to >1 if non valid point
;
COMMON PIA_DRIFT_PAR, pia_drift_par 
COMMON F_2B_FIT, f_2b_f_x, f_2b_f_y, f_2b_f_wei

IF NOT KEYWORD_SET(NODERIVATIVE) THEN noderivative=0
 	pia_drift_par = drift_par 
	
	p_field = drift_par.p_field
	s_field = drift_par.s_field
	a_var = drift_par.a_var
	p_range = drift_par.p_range
	ffunct = drift_par.ffunct
	  
	indfl = WHERE(yyflag LE 1, n)
	IF n EQ 0 THEN BEGIN
	  drift_par.flag = 2
	  RETURN
	ENDIF
	t = xx(indfl)*1.D0
	s = yy(indfl)*1.D0
	if not keyword_set(weight) then	weight = 'NONE'
	
      IF KEYWORD_SET(verbose) THEN $
	print,'weight =',weight 

	case STRUPCASE(weight) of 
	   'SIGMA': w = 1./yyerr(indfl)^2 *1.D0
	   'NONE': w=s*0.+1.d0
	   'TIME': w = exp(-(t-xx(0))/512.D0)/512.D0
	   'SIGNAL': w = 1.D0/abs(s)
	   else: BEGIN 
	           message, 'WEIGHT FUNCTION not specified. NONE used', $
	             /informational
	           w = s*0.+1.d0
	       END 
	endcase 
	
	IF NOT KEYWORD_SET(step)  THEN  step = 0
	get_nchoppsweep, step, nchopp, ind_sweep
	
	nele = N_ELEMENTS(yy)

	; normalize weigth using the stdev of the difference between 
	; consecutive signals / Sqrt(2) [=1.4142]
	ind_nnval = where(yyflag lt 2,nnval)


	IF nnval gt 1 THEN BEGIN
	   diff_yy = yy(1:*) - yy(0:nele-2)
	   ; assign flags to diff_yy: 0 if both are valid
	   diff_flg = yyflag(1:*)/2 + yyflag(0:nele-2)/2
	   ind_ndval = where(diff_flg eq 0,ndval)
	   IF nchopp gt 1 THEN BEGIN 
	       ; avoid signal jumps due to chopping
	       diff_step = step(1:*) - step(0:nele-2)
	       ind_nsweep = where(diff_step(ind_ndval) eq 0, nnsweep)
	       mom_diffyy = moment(diff_yy(ind_ndval(ind_nsweep)))
	       aver_wei = total(w(ind_ndval(ind_nsweep))) / nnsweep
	   ENDIF ELSE BEGIN
	       IF ndval GT 1 THEN BEGIN           
	        mom_diffyy = moment(diff_yy(ind_ndval))
	        aver_wei = total(w(ind_ndval))/ ndval
	       ENDIF ELSE BEGIN
	        mom_diffyy = [0.,1.,0.,0.]
		aver_wei = diff_yy(0)
	        MESSAGE, /INFO, 'Warning: no valid elements '
	       ENDELSE
	   ENDELSE  
	   ; new weigth = norm_weigth / [stdev(diff_yy)/sqrt(2)]^2 
;	   f_2b_f_wei = w	
	   w = w / aver_wei / mom_diffyy(1) * 2
	   f_2b_f_wei = w	
	ENDIF  
	
	
	ind = WHERE(a_var EQ 1, nvar)
	par = FLTARR(nvar)
	par= p_field(ind) *1.D0
	range = p_range(*,ind)
	
; set down the acceptance for NUMDIF_DRIFT
	acc = 1.e-6
	itmax = 50
	IF ffunct EQ 'NUMDIF_DRIFT' THEN acc = .001
	;yfit = pia_curfit(t, s, w, par, sigma, range, chiq, flag=flag, $
	;	funct=ffunct, quiet=0, accept=acc, max_its=maxit, STEP=step)


	IF not KEYWORD_SET(NOT_AMOEBA) THEN BEGIN
	  f_2b_f_x = t
	  f_2b_f_y = s
	  am_res = amoeba(1.e-5, scale=1.e-2, p0=par*1.D0, NMAX=200, $
	      function_name='FUNCTION_2B_FITTED', function_value=fct)
	  IF N_ELEMENTS(am_res) EQ 1 AND am_res(0) EQ -1 THEN $
	     PRINT, ' Amoeba failed to converge' ELSE BEGIN
	       par = am_res * 1.D0
               IF KEYWORD_SET(verbose) THEN $
	         print, 'Amoeba parameters', par
	     					 ENDELSE
	ENDIF
	    
	IF NOT KEYWORD_SET(NOT_MARQUAD) THEN BEGIN      
	   yfit = pia_curvefit(t, s, w, par, sigma, FUNCTION_NAME = ffunct, $
	         CHI2=chiq, itmax=itmax, FLAG= flag, TOL = acc, QUIET=1, $
	         NODERIVATIVE=noderivative)
           drift_par.flag=flag
	   n_dof = N_ELEMENTS(t) - nvar + 1
	   norm_fct = SQRT(chisqr_cvf(0.32,n_dof))
	   sigma = sigma * norm_fct
	   drift_par.s_field(ind) = sigma ;;;* sqrt(mom_diffyy(1)/2.)
	   drift_par.chisq = chiq ;;;;/ mom_diffyy(1) * 2.
        ENDIF   

	drift_par.p_field(ind) = par
	
;	print, norm_fct, n_dof
	
	

		
	RETURN
	END
