;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	FIT_RAMPS
;
; PURPOSE:
;	Fit a polynomial to the voltage ramps of a measurement returning 
;	the slopes, their uncertainties and the fitted lines.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;    FIT_RAMPS, Xall, Yall, Ndeg, Rampend, Ylin, Yslop, Yslopr, $
;	        Yflag, Ytime, Yresi, Ynros
;
; INPUTS:
;	Yall	: Array containing values of read-outs from a measurement [V]
;	Xall	: Array containing the independent variable [t]  (same size)
;	Ndeg	: Polynomial degree
;
; OUTPUTS:
;	Ylin	: Array containing the fitted lines
;	Yslop	: Array of found slopes 
;	Yslopr	: Array of uncertainties in slopes
;	Yflag	: Array of flags (0:ok; 1:2 read-outs used; 2:bad)
;	Ytime	: Slopes times   
;	Yresi	: Array with effective reset intervals used
;	Ynros	: Array with effective number of read-outs used
;
; RESTRICTIONS:
;	Yall, Xall, Dr and Tint must have the same number of elements
;
; PROCEDURES USED:
;	None
;
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel		
;	Modified (last) by CG:	October 1996
;	V5.x
;	Flagging with 1 pseudo_ramps with less than !subd_ramp/2. 
;	 to avoid problems with too small uncertainties
;	Bug fixed: start time of pseudo-ramp was shifted one read-out (CG)
;	V6.3 
;	Bug fixed: double use of indices (CG)	September 1997
;	V6.5
;	Use of glitched flagged elements (CG)	November 1997
;	V6.6
;	Upgrade (larger efficiency) by Jose Acosta (IAC) using
;	bitflag system				May 1999
;	Data reduction info added (CG)		May 1999
;	Mask bug by all points valid fixed (CG) June 1999	
;	V7.4			
;-
PRO FIT_RAMPS, XALL, YALL, NDEG, ST, RAMPEND, DR, YLIN, YSLOP, YSLOPR, YFLAG, $
	               YTIME, YRESI, YNROS, CODEFLAG=codeflag 

; Common blockss		
COMMON piaflag_info, piaflag_str

;
;On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 6 THEN MESSAGE, $
        'Syntax: FIT_RAMPS, XALL, YALL, NDEG, ST, RAMPEND, DR, YLIN, YSLOP, '+$
          'YSLOPR, YFLAG, YTIME, YRESI, YNROS'
;
	nsum = N_ELEMENTS(xall)
	IF nsum NE N_ELEMENTS(yall) THEN $
		MESSAGE, 'Xall and Yall must have same number of elements'

; Now proceed ramp per ramp
nramp = N_ELEMENTS(rampend)

newyall = yall

ylin = FLTARR(nsum)
yslop = FLTARR(nramp)
yslopr = FLTARR(nramp)
yflag = FLTARR(nramp)
yresi = FLTARR(nramp)
ynros = FLTARR(nramp)
ytime = FLTARR(nramp)

iramp0 = 0
FOR iramp=0L, nramp-1 DO BEGIN   ;starts loop over ramps
  
  iramp1 = rampend(iramp)
  npt_ramp = iramp1 - iramp0 + 1
  ind_ramp = INDGEN(npt_ramp) + iramp0
  xxt = xall(ind_ramp)
  yyt = yall(ind_ramp)
  fyt = st(ind_ramp)
  indok = WHERE(fyt EQ 0, nok) 
  indmask = WHERE(fyt NE 0, nmask)

  IF nok GT 1 THEN BEGIN 
    IF nmask GT 0 THEN BEGIN 
     ; check that there is at least one good readout before last mask readout,
     ; except for the last readout in the ramp
     indmask_nolast = WHERE(indmask NE (npt_ramp-1),nnolast)
     IF nnolast GT 0 THEN BEGIN 
       indmask = indmask(indmask_nolast)
       indreb = WHERE(indmask GT MIN(indok),nreb)
       ;reconstruct ramps only if there are valid reads in between masked 
       ; points, which are not the last readout
       IF nreb GT 0 THEN BEGIN 
          indreb = indmask(indreb)
	  yyt = rebuild_ramps(xxt, yyt, indok, indreb)
       ENDIF	    
     ENDIF
    ENDIF  
    newyall(ind_ramp) = yyt
    xfit = xxt(indok)-xxt(indok(0))
    coef = POLY_FIT(xfit,yyt(indok), ndeg, yff, yband, sigma, covm)
    yslop(iramp) = coef(1)
    yslopr(iramp) = SQRT(covm(1,1))
    yresi(iramp) = MAX(xfit)
    ynros(iramp) = nok
    ytime(iramp) = AVG(xxt(indok))
    IF nok EQ 2 THEN BEGIN 
       yflag(iramp) = 1
       yslopr(iramp) = 0.
    ENDIF  
    ylin(ind_ramp) = POLY(xxt,coef)
  ENDIF ELSE BEGIN
    yflag(iramp) = 2
    ytime(iramp) = AVG(xxt) 
  ENDELSE
  
  iramp0 = iramp1+(1*dr(iramp1))

ENDFOR	   ;close loop over ramps	


; set code flag for the recently created SRD 
IF KEYWORD_SET(codeflag) THEN BEGIN 
   codeflag = yflag*0
   ;encode flag for NDR = 2 
   ind = WHERE(yflag EQ 2, nind)
   tmpflag = yflag*0 
   IF nind GT 0 THEN BEGIN 
     tmpflag(ind) = 1
     encode_piaflag, 'SRD', 'RAMPNDR_2', codeflag, codeflag, tmpflag
   ENDIF
   ;encode flag for NDR = 1
   ind = WHERE(yflag EQ 1, nind) 
   IF nind GT 0 THEN BEGIN 
     tmpflag(ind) = 1
     encode_piaflag, 'SRD', 'RAMPNDR_1', codeflag, codeflag, tmpflag
   ENDIF
   
   yflag = codeflag  
   
ENDIF

RETURN
END
