;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       decode_piaflag
; PURPOSE:
;	Decode a PIA coded flag, returning a flag array corresponding to the
;	given element in the given processing level (ERD, SRD, ...).
;       It can be called with a bidimensional array.
;
; CATEGORY:
;       PIA - flag system
;
; CALLING SEQUENCE:
;       DECODE_PIAFLAG, dtype, lex_flag, codeflag, unitflag  $
;                [,DET = det] [,/ALLPIX]
;
; INPUT PARAMETERS:
;       codeflag: Flag using the coding of PIA (Normal case: tmpdata.flag)
;       dtype:    Data type  [ERD,SRD,SCP,SPD,AAP]     
;       lex_flag: Name of the flag to decode
;
; OUTPUT PARAMETERS: 
;       unitflag: Flag array corresponding to lex_flag (0 or 1)
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               piaflag_info
;
; MODIFICATION HISTORY:
;       Written by: J.Acosta
;       Feb 98 Version 7.x
;	Updated by JA+CG 		May 99
;	Adding initialization (CG)	May 99
;	Simplified code (JA+CG)		June 99
;	V7.4
;	DYNR included (JA)		February 2000
;	V8.2
;	Setting all variables to INT before boolean operations (Jacosta) Jul2000
;-
PRO decode_piaflag, dtype, lex_flag, codeflag, unitflag,   $ 
    DET=det, ALLPIX=allpix  

ON_ERROR, 2

COMMON piaflag_info, piaflag_str

IF N_PARAMS() LE 3 THEN MESSAGE, $ 
    'Syntax: decode_piaflag, dtype, lex_flag, codeflag, unitflag'+ $
     ' [,DET=det] [,/ALLPIX]'
is_spdaap = 0 

IF N_ELEMENTS(piaflag_str) EQ 0 THEN piaflag_info
   
CASE dtype OF
	'ERD' : piaflag_tmp = piaflag_str.erd
	'SRD' : piaflag_tmp = piaflag_str.srd
	'DYNR': piaflag_tmp = piaflag_str.dynr
	'SCP' : piaflag_tmp = piaflag_str.scp
	'SPD' : BEGIN
		piaflag_tmp = piaflag_str.spd
		is_spdaap = 1
	       END
	'AAP' : BEGIN
		piaflag_tmp = piaflag_str.aap
		is_spdaap = 1
	      END
	ELSE: MESSAGE,'Not valid DATA TYPE'
ENDCASE	


IF NOT KEYWORD_SET(DET) THEN det='P1'  ;if not detector set then assume 1-pix
IF NOT KEYWORD_SET(allpix) THEN allpix = 0 ELSE allpix =1


npix = PIXEL_DET(det,ALLPX=allpix, IS_SPD=is_spdaap)
;check if the dimension of the array corresponds to the specified detector
siz_flag = SIZE(codeflag) 
nele = N_ELEMENTS(codeflag(0,*))

IF siz_flag(0) EQ 2 THEN tmpdim = siz_flag(1) ELSE tmpdim  = 1 
; trouble could come from an array for all pixels with one element
IF tmpdim NE npix THEN $
   IF siz_flag(1) NE npix THEN $
    MESSAGE,$
       'Not valid flag vector for specified detector'    

ind_flag = WHERE(piaflag_tmp.lex EQ STRTRIM(STRUPCASE(lex_flag),2),nlex)

IF nlex EQ 0 THEN BEGIN 
   print,'Not valid FLAG identifier'
   RETURN
ENDIF    

nflag = piaflag_tmp.nflag

val_flag = piaflag_tmp.valflag(ind_flag(0))
unitflag = (FIX(codeflag) AND FIX(val_flag)) / val_flag


RETURN
END      
