;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       encode_piaflag
; PURPOSE:
; 	Encode a coded flag from a reduced flag, which depends on the data type,
;	(ERD, SRD, ...). It can be called with a bidimensional array.
;
; CATEGORY:
;       PIA - flag system
;
; CALLING SEQUENCE:
;       ENCODE_PIAFLAG, dtype, lex_flag, codeflag, newflag, unitflag, [,/RESET] $
;	         [,DET = det] [,/ALLPIX]
;
; INPUT PARAMETERS:
;       unitflag: Flag from any processing system
;	dtype:	  Data type  [ERD,SRD,SCP,SPD,AAP]     
;	lex_flag: Name of the flag to encode
;       codeflag: Previous absolute flag value 
;
; OUTPUT PARAMETERS: 
;       newflag: New absolute Flag value 
;
; KEYWORD INPUT:
;	RESET:	setting lex_flag to 0 in newflag (unitflag irrelevant)
;	DET:	detector used
;	ALLPIX:	update also flags corresponding to open+resistor pixels
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               piaflag_info
;
; MODIFICATION HISTORY:
;       Written by: J.Acosta
;       Feb 98 Version 7.x
;	Update by CG+JA 			May 1999
;	Logic changed (CG)			June 1999
;	V7.4
;	DYNR included (JA)		February 2000
;	V8.2
;	Bug corrected: Changing to FIX all arrays before doing 
;         logical operations, to avoid floating arrays. (JAcosta) July 2000
;-
PRO encode_piaflag, dtype, lex_flag, codeflag, newflag, unitflag,   $ 
    RESET = reset, det=DET, allpix=ALLPIX
    

ON_ERROR, 2

COMMON piaflag_info, piaflag_str

IF N_PARAMS() LT 4 THEN MESSAGE, $ 
    'Syntax: encode_piaflag, dtype, lex_flag, codeflag, newflag [, unitflag]'+ $
     '  [,/RESET]'
    

CASE dtype OF
	'ERD': piaflag_tmp = piaflag_str.erd
	'SRD': piaflag_tmp = piaflag_str.srd
	'DYNR': piaflag_tmp = piaflag_str.dynr
	'SCP': piaflag_tmp = piaflag_str.scp
	'SPD': BEGIN
		piaflag_tmp = piaflag_str.spd
		is_spdaap = 1
	       END
	'AAP': BEGIN
		piaflag_tmp = piaflag_str.aap
		is_spdaap = 1
	      END
	ELSE: MESSAGE,'Not valid DATA TYPE'
ENDCASE	

ind_flag = WHERE(piaflag_tmp.lex EQ STRTRIM(STRUPCASE(lex_flag),2),nlex)

IF nlex EQ 0 THEN BEGIN 
   print,'Not valid FLAG identifier'
   RETURN
ENDIF    

valflag = piaflag_tmp.valflag(ind_flag(0))

IF N_ELEMENTS(unitflag) NE 0 AND NOT KEYWORD_SET(RESET) THEN BEGIN
	
	ncode = N_ELEMENTS(codeflag)
	nunit = N_ELEMENTS(unitflag)
	unitsfl = codeflag * 0 
	
	IF ncode NE nunit THEN BEGIN
	   npix = N_ELEMENTS(codeflag(*,0))
	   IF N_ELEMENTS(codeflag(0,*)) EQ nunit THEN BEGIN
	     FOR ipix=0,npix-1 DO unitsfl(ipix,*) = unitflag 
	   ENDIF ELSE BEGIN		   
	    PRINT, 'Warning: flag dimensions do not correspond'
	    STOP
	   ENDELSE
	ENDIF ELSE unitsfl = unitflag

; Reset first the value 
	newflag = FIX(codeflag) AND NOT FIX(valflag)
; Use the new values
	newflag = FIX(newflag) OR FIX(unitsfl*valflag)
	
ENDIF ELSE IF KEYWORD_SET(RESET) THEN BEGIN

	newflag = FIX(codeflag) AND NOT FIX(valflag)
	
ENDIF ELSE BEGIN

	MESSAGE, /INFO, 'Provide a flag vector or set RESET keyword'
	
ENDELSE




RETURN
END      
