;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       get_validflag
; PURPOSE:
; 	Get a reduced flag from a coded flag, which depends on the data type,
;	(ERD, SRD, ...). It can be called with a bidimensional array.
;
; CATEGORY:
;       PIA - flag
;
; CALLING SEQUENCE:
;       GET_VALIDFLAG, codeflag, dtype, redflag [,INDSEVERE=indsevere], $
;	         [,DET = det] [,/ALLPIX]
;
; INPUT PARAMETERS:
;       codeflag: Flag using the coding of PIA
;	dtype:	  Data type  [ERD,SRD,SCP,SPD,AAP]     
;
; OUTPUT PARAMETERS: 
;       redflag: flag set to 1 when a point is discarded and 0 otherwise.
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               piaflag_info
;
; KEYWORDS: 
;       DET: detector
;	ALLPIX: set for all pixels, including resistor and open pixels
;	INDSEVERE: index of flags which are to be rejected
;
; MODIFICATION HISTORY:
;       Written by: J.Acosta
;       Feb 98 Version 7.x
;	Updated by JA + CG - May 99
;	Using boolean algebra (CG) - Jul 99
;	V8.0
;	Setting all variables to INT before boolean operations (Jacosta) Jul2000
;-
PRO get_validflag, codeflag, dtype, redflag, INDSEVERE=indsevere, DET = det, $
    ALLPIX= ALLPIX
ON_ERROR,2

COMMON piaflag_info, piaflag_str

IF N_PARAMS() LT 3 THEN MESSAGE, $
   'Syntax: get_validflag, codeflag, dtype, redflag [,INDSEVERE=indsevere], '+ $
   '[,DET=det] [,/ALLPIX]'

IS_SPD = 0  & IS_AAP = 0

CASE dtype OF
	'ERD': piaflag_tmp = piaflag_str.erd
	'SRD': piaflag_tmp = piaflag_str.srd
	'DYNR': piaflag_tmp = piaflag_str.dynr
	'SCP': piaflag_tmp = piaflag_str.scp
	'SPD': BEGIN 
	         piaflag_tmp = piaflag_str.spd
		 IS_SPD =1 
	       END
	'AAP': BEGIN 
	         piaflag_tmp = piaflag_str.aap
		 IS_AAP = 1
	       END	
	ELSE: MESSAGE,'Not valid DATA TYPE'
ENDCASE	

nsev = N_ELEMENTS(indsevere)
IF nsev EQ 0 THEN $
  indsevere = WHERE(piaflag_tmp.severe EQ 1,nsev) 

redflag = codeflag*0
; return if there is no severe flags
IF nsev EQ 0 THEN RETURN

IF NOT KEYWORD_SET(DET) THEN det='P1'  ;if not detector set then assume 1-pix
IF NOT KEYWORD_SET(allpix) THEN allpix = 0 ELSE allpix =1



npix = PIXEL_DET(det,ALLPX=allpix,IS_SPD=is_spd,IS_AAP=is_aap)
;check if the dimension of the array corresponds to the specified detector
;siz_flag = SIZE(codeflag) 
;IF siz_flag(0) EQ 2 THEN tmpdim = siz_flag(1) ELSE tmpdim  = siz_flag(0) 
;IF tmpdim NE npix THEN MESSAGE, $
;   'Not valid flag vector for specified detector' 


nflag = piaflag_tmp.nflag
nele = N_ELEMENTS(redflag(0,*))

;decode_flag, codeflag, nflag, tmpflag

redflag = codeflag * 0

FOR isev=0,nsev-1 DO BEGIN
 val_flag = piaflag_tmp.valflag(indsevere(isev))
 redflag = redflag + (FIX(codeflag) AND FIX(val_flag)) / val_flag 
ENDFOR

ind = WHERE(redflag GT 0, n)
IF n GT 0 THEN redflag(ind) = 1
  
RETURN
END
