;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      PRO Flux, Power, Detector, Filter, Aperture, Type, Out

;+
; NAME:
;	FLUX
;
; PURPOSE:
;	Converts input Power to Flux Density or Surface Brightness
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FLUX, Power, Detector, Filter, Aperture, Type, Out
;
; INPUTS:
;	Power    : Doubleprecision array of Power in W
;	Detector : Subsystem - 'P', 'C1', 'C2', 'SS', 'SL'
;	Filter   : Filter number            }  as
;	Aperture : Aperture or pixel number } follows:
;	      PHT-P : Filter number (1-14), aperture number (1-14)
;	      PHT-C1: Filter number (1-8), pixel number (1-9)
;	      PHT-C2: Filter number (9-14), pixel number (1-4)
;	      PHT-SS: <anything>, Pixel number (1-64)
;	      PHT-SL: <anything>, Pixel number (1-64)
;	Type: Conversion type
;	      0: Power -> Flux Density
;	      1: Power -> Surface Brightness
;
; OUTPUTS:
;	Out: Doubleprecision array, units depending on the conversion type
;
; COMMON BLOCKS:
;	aperture_info
;	pht_s
;	phtapertures
;	phtppsf
;
; RESTRICTIONS:
;	For PHT-P, C, the aperture_info common block must be initialised first
;	For PHT-S, the pht-s common block must be initialised first
;
; PROCEDURES USED:
;	PIA LIB: flux_PC
;	         flux_S
;	         aperture_info_init
;	         aperture_size_init
;
; MODIFICATION HISTORY:
;	Written by:	Huw Morris
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;	According to new flux_s routine (CG)		April 1996
;	phtapertures not further use (CG)		February 1998
;	V7.0
;
;-

@pht_s
@aperture_info
@phtppsf

	ON_ERROR, 0              ; Stay with this procedure

;  Initialise an empty output array in case of error
	Out = Power * 0.

;  If aperture_info or phtapertures empty initialize 
	IF N_ELEMENTS(phtppsf) EQ 0 THEN read_ppsf
	IF N_ELEMENTS(N_FILT) EQ 0 THEN aperture_info_init

;  Different actions depending on the detector. In particular PHT-S
;  requires a different routine to PHT-P and PHT-C

	IF STRMID(Detector,0,1) EQ 'P' THEN Detector = 'P' 

	CASE Detector OF

	  'P' : BEGIN
	    Flux_PC, Power, Detector, Filter, Aperture, Type, Out
	  END
	  'C1' : BEGIN
	    Flux_PC, Power, Detector, Filter, Aperture, Type, Out
	  END

	  'C2' : BEGIN
	    Flux_PC, Power, Detector, Filter, Aperture, Type, Out
	  END

 	  'SS' : BEGIN
	    Flux_S, Power, Detector, Type, Out
	  END

	  'SL' : BEGIN
	    Flux_S, Power, Detector, Type, Out
	  END

	  ELSE : BEGIN
	    MESSAGE, 'Invalid detector specified'
	    RETURN
	  ENDELSE

	ENDCASE

	END
