;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      PRO Flux_PC, Power, Detector, Filter, Aperture, Type, Out

;+
; NAME:
;	Flux_PC
;
; PURPOSE:
;	Converts input Power into Flux Density or Surface Brightness for
;	PHT-P and PHT-C
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FLUX_PC, Power, Detector, Filter, Aperture, Type, Out
;
; INPUTS:
;	Power    : Doubleprecision two-dim array of Power [pixel,npoints] (in W)
;	Detector : 'P', 'C1', 'C2' 
;	Filter   : Filter number
;	Aperture : PHT-P aperture, or PHT-C pixel number
;	Type: Conversion type
;	      0: Power -> Flux Density
;	      1: Power -> Surface Brightness
;
; OUTPUTS:
;	Out: Doubleprecision 2dim-array (as Power), units depending on the 
;	conversion type
;
; COMMON BLOCKS:
;	phtppsf
;	phtw2jy
;
; RESTRICTIONS:
;     The aperture common block must be initialised first
;
; PROCEDURES USED:
;     None
;
; Modification History:
;	Written by:	Huw Morris
;	Oct 95 Version 3.5
;	Derivation changed, calibration numbers taken from phtw2jy common
;	containing the C1 factors, telescope_area multiplication taken out
;	for Jy -> MJy/sr (CG)					March 1996
;	C100/C200 are not PSF corrected pixelwise now (CG)	January 1997
;	V6.0
;	Common aperture_info deleted, psf info by phtppsf,
;	 filter and apert./pixel dependant omega used 
;	 for surface brightness calculation (CG)		January 1998
;	V7.0
;-
;@aperture_info
@phtppsf
@phtw2jy

	ON_ERROR, 0              ; Stay with this procedure


        C = 299792500D0                 ; Speed of light
        Epsilon = 0.3                   ; Correction for the secondary mirror
	Tel_Area = 0.28274334           ; Area of the ISO main aperture
	detpix = pixel_det(detector)


;  Check the inputs before we start

;  Check the conversion type is valid
	IF (Type LT 0) OR (type GT 1) THEN BEGIN       
	  MESSAGE, 'Invalid conversion type supplied'
	  RETURN
	ENDIF


;  Check the detector is valid
	IF (Detector NE 'P')  AND (Detector NE 'C1') AND $
	   (Detector NE 'C2')  THEN BEGIN
	  MESSAGE, 'Invalid detector supplied'
	  RETURN
	ENDIF



;  Calculate the ratio of the detector reponse at the central wavelength
;  the central wavelength in m, and the point spread function
	IF (Detector EQ 'P') THEN BEGIN
;  PHT-P
	  PSF = Double(phtppsf(Aperture-1, Filter-1))
	  C1const = W2JY_P(Filter-1)
	  IF type EQ 1 THEN BEGIN
	   Omega = FLTARR(1,2)
	   om = GET_OMEGA(detector, aperture-1, filter-1)
	   Omega(0,*) = om
	  ENDIF
	ENDIF ELSE BEGIN
;  PHT-C100/C200
;    C100/C200 are not corrected pixelwise but for the full array
	  PSF = 1.D0
	  C1const = W2JY_C(Filter-1)
	  IF type EQ 1 THEN BEGIN
	   If STRMID(detector,0,2) EQ 'C2' THEN $
	     filt_red=filter-9 ELSE $
	     filt_red=filter-2 
	     Omega = get_omega(detector, indgen(detpix), filt_red)
	  ENDIF
	ENDELSE

;  Convert Power (W) to Flux Density (Jy)

;  First calculate flux per unit wavelength

	FluxD = Power / (PSF * C1const)

	IF (Type EQ 0) THEN BEGIN        ; W -> Jy

; We have the required output units
	  Out = FluxD

	ENDIF ELSE BEGIN
		            
;  Convert Jy to Jy/sr
	   Brightness = Fluxd
	   FOR ipix=0,detpix-1 DO $
	    Brightness(ipix,*) = $
	     ((FluxD(ipix,*) * PSF) / (Omega(ipix,0)*(1D0 - Epsilon^2)))


 	    Out = Brightness

	ENDELSE

	END
