;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro fpg_cross, iscpfile, fpgcfile, centre, SCAN=scan, Inter=inter, $
  						  PIXEL=pixel, ERROR=error 

;+
; NAME:
;     FPG_CROSS
;
; PURPOSE:
;       Analyse Focal Plane Geometry Cross Scans
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	FPG_CROSS, Iscpfile, Fpgcfile, Centre
;
; INPUTS:
;	Iscpfile: name of ISCP product file, containing cross scan data
;	Fpgcfile: name of Focal Plane Geometry Calibration File, related
;				 to cross scan
;
;
; KEYWORD PARAMETERS:
;	SCAN		Defines whether to use Y or Z coordinate from FPGC file
;				'Y': use Y coordinate
;				'Z': use Z coordinate
;				if not specified default is use Y coordinate
;
;	INTER		Decides, whether s/w shall run in:
;						'Y' = interactive mode
;						'N' = automatic mode
;						defualt is 'N'
;
;	PIXEL 	decide which PHT-S pixel shall be analysed
;						default=1
;
;	ERROR		Returns an error status:
;						0 = no error
;						1 = error: output is not valid
;
; OUTPUTS:
;	Centre: coordinate of scan centre, in FPGC coordinates
;
; COMMON BLOCKS:
;	phtscp
;
; PROCEDURES CALLED:
;	PIA LIB: read_fpgc	
;	         read_ispd
;	         pia_xplot
;	         multi_fit
;
; MODIFICATION HISTORY:
;       Written by:	Ingolf Heinrichsen
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;	Call to pia_multi_fit using weights and flags (CG)	October 1995
;
;-
;
; initialize common blocks
@phtscp
COMMON pia_multi_fit_results, 	xmf1, ymf1, parmf1, sigmf1, $
				xmf2, ymf2, parmf2, sigmf2

  on_error, 2
; initalize error state
	error=1
; initalize debugger, 0 no output, 1 print-out
	debug=0
; check for parameters
 npar   = N_params()
 if npar EQ 0 then goto, nopar
; initialize keywords
	if (n_elements(inter) eq 0) then inter="N"
	if (n_elements(pixel) eq 0) then pixel=1
;	
; define text array for print-out into window
;
	text=strarr(1)
;	
; read FPGC file
;
	read_fpgc, fpgcfile, npts, nlin, dst_pts, dst_lin, rota, $
              rline, rcolumn, ycoor, zcoor, TEXTOUT=text, ERROR=read_error 
	if (read_error ne 0) then goto, readerror
;
; read ISCP file
;
	read_iscp, iscpfile
;
; copy handles out to arrays
;
	UNIT = PHTSCP.UNIT
   meas_num=n_elements(unit)-1
   DET  = PHTSCP(meas_num).DET
   ADMN = PHTSCP(meas_num).ADMN
   ORDR = PHTSCP(meas_num).ORDR
   DIMS = PHTSCP(meas_num).DIMS
;
;  copy the handles
;
	HANDLE_VALUE, PHTSCP(meas_num).HDR, hdr
   HANDLE_VALUE, PHTSCP(meas_num).TIME, time  
   HANDLE_VALUE, PHTSCP(meas_num).AVER, aver  
   HANDLE_VALUE, PHTSCP(meas_num).AVUN, avun     
   HANDLE_VALUE, PHTSCP(meas_num).FLAG, flag  
   HANDLE_VALUE, PHTSCP(meas_num).RPID, rpid 

; take only "good" points for first pixel (OK for PHT-P, dubious for PHT-S)

   indok = WHERE(flag(0,*) EQ 0, ncount) 

   IF ncount GE 14 THEN BEGIN
   	 time = time(indok)
   	 aver = aver(*,indok)
   	 avun = avun(*,indok)
   	 flag = flag(*,indok)		
   	 rpid = rpid(*,indok)
   ENDIF ELSE BEGIN
 	indok = WHERE(flag LE 1, ncount)
 	IF ncount GE 14 THEN BEGIN     	
   	 	time = time(indok)
   	 	aver = aver(*,indok)
   	 	avun = avun(*,indok)
   	 	flag = flag(*,indok)		
   	 	rpid = rpid(*,indok)
   	 ENDIF ELSE BEGIN
   	 	MESSAGE, /info, "not enough good points, returning"
   	 	RETURN
   	 ENDELSE
   ENDELSE
;
; check for correct detector
;
	if (debug eq 1) then print, 'Detector: ', det
;	if (det ne 'P1') then begin
;		if (det ne 'SS') then, goto, deterror
;	endif
;
; decide which is line and which is column from rpid
;
	if (debug eq 1) then begin
		print, "rpid dim 0: ", max(rpid(0,*))
		print, "rpid dim 1: ", max(rpid(1,*))
	endif
	if (max(rpid(0,*)) eq 2) then begin
		line_dim=0
		scan_dim=1
	endif else if (max(rpid(1,*)) eq 2) then begin 
	   line_dim=1
	   scan_dim=0
	endif else begin
;		goto, rpiderror
	endelse
	
			; set for testing puposes ONLY !!!!!!!!!!!!!!
			;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			if (det eq "P2") then begin
   	   		line_dim=1
	   			scan_dim=0
	   			det="P1"
	   	endif
	   	;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;
;	define dimensions
;
	nline=2
	nscan=max(rpid(scan_dim,*))
;
;====================================================
;  for P1 detector
;
if (det eq "P1") then begin
;
;		extract partial scans
;
		aver1=aver(0:nscan-1)
		avun1=avun(0:nscan-1)
		flag1=flag(0:nscan-1)
		aver2=aver(nscan:2*nscan-1)
		avun2=avun(nscan:2*nscan-1)
		flag2=flag(nscan:2*nscan-1)

		if (debug eq 1) then begin
			print, "Pixel: ",pixel," Scan data 1st Scan"
			print, aver1(pixel,*)
			print,"Pixel: ",pixel,"Scan data 2nd Scan "
			print, aver2(pixel,*)
		endif
;
; 		check FPGC data for conformance with RPID
;
		if (debug eq 1) then begin
			print, "NPOINTS: ", npts
			print, "NLINES: ", nlin
			print, "FPGC Y-coordiante"
			print, ycoor
			print, "FPGC Z-coordiante"
			print, zcoor
		endif
		ntmp=[nlin, npts]
		if (min(ntmp) ne 2) then goto, fpgcerror
		if (max(ntmp) ne nscan)	then goto, fpgcincons
; 
;		decide on FPGC scan direction and assign X-coordinates
;
		if (n_elements(scan) eq 0) then scan = 'Y'
		if (scan eq 'Z') then begin
			x1=zcoor(0:nscan-1)
			x2=zcoor(nscan:2*nscan-1)
		endif else begin
			x1=ycoor(0:nscan-1)
			x2=ycoor(nscan:2*nscan-1)
		endelse
			if (debug eq 1) then begin
			print, "X1-coordiante"
			print, x1
			print, "X2-coordiante"
			print, x2
		endif
;
; 		define constant parameters for fit
;
		lambda=3.29
		width=(lambda*0.84)/2.
;
; 		do not read CSH, but make asumptions on aperture used	
;
		if ((dst_pts lt 3) and (dst_lin lt 3)) then begin
			if ((inter eq "Y") or (debug eq 1)) then begin
				print, "Asumption Aperture 10 arcsec"
			endif
			aperture=10.
		endif else begin
			if ((inter eq "Y") or (debug eq 1)) then begin
				print, ' '
				print, "Asumption Aperture 23 arcsec"
				print, ' '
			endif
			aperture=23.
		endelse
		plateau=aperture-width
;
; 		perform centre moment method for automatic processing
;
		centre=fltarr(2)
		centre(0)= total(x1*aver1)/(total(aver1))
		centre(1)= total(x2*aver2)/(total(aver2))
		if(debug eq 1) then print, "Centre moment:" ,centre
		if (inter eq 'Y') then begin
			Print, "Start Parameters for fit as asumed from scan parameter"
			Print, "======================================================"
			print, " "
			print, "Aperture: ", aperture
			print, "Wavelength: ", lambda
			print, "Width: ", width
			print, "Plateau: ", plateau
			print, "Centre: ", centre
		endif
;
; 		perform fit of profile
;
; 		manually
;
		if (inter eq 'Y') then begin
			weight = 1. / avun1^2
			pia_multi_fit, x1, aver1, a=a, sigmaa=sigmaa, psym=2,$
				weight=weight
			xmf1 = x1
			ymf1 = aver1
			parmf1 = a(0:N_ELEMENTS(sigmaa)-1)
			sigmf1 = sigmaa
			weight = 1. / avun2^2
			pia_multi_fit, x2, aver2, a=a, sigmaa=sigmaa, psym=2,$
				weight=weight
			xmf2 = x2
			ymf2 = aver2
			parmf2 = a(0:N_ELEMENTS(sigmaa)-1)
			sigmf2 = sigmaa
		endif
endif
;
;====================================================
;  for PHT-S detector
;
if (det eq "SL") then begin
		;
		; initalize arrays
		;
		aver1=fltarr(128,nscan)
		avun1=fltarr(128,nscan)
		flag1=fltarr(128,nscan)
		aver2=fltarr(128,nscan)
		avun2=fltarr(128,nscan)
		flag2=fltarr(128,nscan)
		aver1=aver(*,0:nscan-1)
		avun1=avun(*,0:nscan-1)
		flag1=flag(*,0:nscan-1)
		aver2=aver(*,nscan:2*nscan-1)
		avun1=avun(*,nscan:2*nscan-1)
		flag1=flag(*,nscan:2*nscan-1)
;
; 		check FPGC data for conformance with RPID
;
		if (debug eq 1) then begin
			print, "NPOINTS: ", npts
			print, "NLINES: ", nlin
			print, "FPGC Y-coordiante"
			print, ycoor
			print, "FPGC Z-coordiante"
			print, zcoor
		endif
		ntmp=[nlin, npts]
		if (min(ntmp) ne 2) then goto, fpgcerror
		if (max(ntmp) ne nscan)	then goto, fpgcincons
; 
;		decide on FPGC scan direction and assign X-coordinates
;
		if (n_elements(scan) eq 0) then scan = 'Y'
		if (scan eq 'Z') then begin
			x1=zcoor(0:nscan-1)
			x2=zcoor(nscan:2*nscan-1)
		endif else begin
			x1=ycoor(0:nscan-1)
			x2=ycoor(nscan:2*nscan-1)
		endelse
			if (debug eq 1) then begin
			print, "X1-coordiante"
			print, x1
			print, "X2-coordiante"
			print, x2
		endif
;
; 		define constant parameters for fit
;
		lambda=fltarr(128)
		lambda(0:63)=((5-2.5)/64*findgen(64)+2.5)
		lambda(64:127)=((12.0-6.0)/64*findgen(64)+6.0)
		width=(lambda*0.84)/2.
;
; 		perform centre moment method for automatic processing
;		for all pixels
;
		centre=fltarr(2,128)
		for i=0,127 do begin
			centre(0,i)= total(x1*aver1(i,*))/(total(aver1(i,*)))
			centre(1,i)= total(x2*aver2(i,*))/(total(aver2(i,*)))
		endfor
		if(debug eq 1) then begin
			print, "Centre moment:" &
			print, string(findgen(128))+string(centre(0,*))+$
			string(centre(1,*))
			pia_xplot, centre(0,*)
			pia_xplot, centre(1,*)
		endif		
;
; 		do not read CSH, but make asumptions on aperture used	
;
		aperture=24.
		plateau=aperture-width
		if (inter eq 'Y') then begin
			Print, "Start Parameters for fit as asumed from scan parameters"
			Print, "======================================================="
			print, " "
			print, "Pixel Number:", pixel
			print, "Aperture: ", aperture
			print, "Wavelength: ", lambda(pixel)
			print, "Width: ", width(pixel)
			print, "Plateau: ", plateau(pixel)
			print, "Centre: ", centre(*, pixel)
		endif
;
; 		perform fit of profile and plot scans
;
; 		manually for one pixel
;
		if (inter eq 'Y') then begin
			y=fltarr(132)
			y(0:127)=lambda
			surface, aver1, y, x1, $
	   				title='Cross Scan No. 1', $
	   				ztitle='Signal [V/s]',$
	   				xtitle='S/C FPGC coordinates [arcsec]', $
			ytitle='PHT-S pixel number'
			weight = 1. / avun1^2
			pia_multi_fit, x1, a=a, weight=weight, sigmaa=sigmaa, $
				aver1(pixel,*), psym=2
			xmf1 = x1
			ymf1 = aver1(pixel,*)
			parmf1 = a
			sigmf1 = sigmaa

			surface, aver2, y, x2, $
	   				title='Cross Scan No. 2', $
	   				ztitle='Signal [V/s]',$
	   				xtitle='S/C FPGC coordinates [arcsec]', $
						ytitle='PHT-S pixel number'

			weight = 1. / avun2^2
			pia_multi_fit, x2, aver2(pixel,*), a=a, sigmaa=sigmaa,$
					psym=2, weight=weight			
			xmf2 = x2
			ymf2 = aver2(pixel,*)
			parmf2 = a
			sigmf2 = sigmaa
		endif
	
endif
;
; everything ok.
;
	error = 0
	return
;
; error handling
;
nopar:
	print,"fpg_cross, iscpfile, fpgcfile, centre, SCAN=scan, INTER=inter, "
	print,"			   PIXEL=pixel, ERROR=error"
	return
fpgcerror:
	print, "FPG_CROSS: Dimensions of FPGC not consistent with cross scan"
	print, "Dimensions :", nlin, npts
	return  	
rpiderror:
	print, "FPG_CROSS: Dimensions of RPID not consistent with cross scan"
	print, "Dimensions: ",max(rpid(0,*)), max(rpid(1,*))
	return
fpgcincons:
	print, "FPG_CROSS: Dimensions of FPGC and RPID not consistent"
	print, "FPGC Dimensions :", nlin, npts
	print, "RPID Dimensions: ",max(rpid(0,*)), max(rpid(1,*))
	return  	
deterror:
	print, "FPG_CROSS: Detector not P1 or PHT-S, but:", det
	return
readerror:
	print, "ANAL_FPGC: Unable to read file:", infile
	return
end
