;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GEN_AAP
;
; PURPOSE:
;       Fills in DECP and RAP fields of PHTIAAP structure which contains the 
;	central RA and DEC position for every point in the AAP, including chopper 
;	deflections and detector pixel offsets.  This makes the data more 
;	transparent and mapping easier.
;
; CATEGORY:
;       PIA - processing
;
; CALLING SEQUENCE:
;       GEN_AAP, phtiaap
;
; INPUTS:
;       PHTIAAP		The AAP structure
;
; KEYWORD PARAMETERS:
;	(none)
;
; OUTPUTS:
;       PHTIAAP		The AAP structure with correct DECP and RAP
;
; COMMON BLOCKS:
;       (none)
;
; PROCEDURES USED:
;       PIA Procedure(s):	 
;
;       Astronomical Library:	FXPAR
;
; FUNCTIONS USED:	pixel_det (function) 
;
; MODIFICATION HISTORY:
;       Written by:             Min Hur, IPAC/Caltech       	Jan 2000
;	Bug for staring case fixed (CG)				Mar 2000
;	Data for P and S derived and
;	 bug for PHT-S fixed (CG)				Mar 2000
;	Forgotten COS(DEC) used for calculation (CG)		Apr 2000
;	V8.2
;-

PRO gen_aap, phtiaap

det = phtiaap.det
dims = phtiaap.dims

cstp = phtiaap.cstp
cpos = phtiaap.cpos
rpid = phtiaap.rpid
roll = AVG([phtiaap.roll])
rota = FXPAR(phtiaap.hdr,'ATTRROTA')

; IF ATTRORIE = 1 THEN ROTA keyword is invalid and pattern
; determined by roll angle 

rotaflag = FXPAR(phtiaap.hdr,'ATTRORIE')
IF rotaflag THEN rota = roll

dec = phtiaap.dec
ra = phtiaap.ra

pn = PIXEL_DET(det, /IS_AAP)
decp = dblarr(pn,dims)
rap = dblarr(pn,dims)

; First, define the ra and dec of the dectector center at each position

rrota = rota * !pi / 180.0
theta = avg([dec]) * !PI / 180.
cintpos = round(cpos)

c_dec= dec- (cintpos * SIN(rrota))/3600
c_ra = ra + (cintpos * COS(rrota))/3600/cos(theta)

; determine the ra and dec of each pixel at each position

angle = roll * !pi / 180.0

IF det eq 'C1' THEN BEGIN
   pixnum = 9		; 9 pixels each with dimensions 43.5" x 43.5"
   delta = 46.0		; distance between the pixel centers
      for i = 0, 2 do begin
         for j = 0, 2 do begin
            p = 3 * j + i
            rap(p,*) = c_ra + $
	    	( delta*(j - 1)*COS(angle) + delta*(i - 1)*SIN(angle) ) / 3600 $
		  / cos(theta)
            decp(p,*)= c_dec+ ( delta*(i - 1)*COS(angle) - delta*(j - 1)*SIN(angle) ) / 3600
         endfor
      endfor
ENDIF ELSE IF det eq 'C2' THEN BEGIN
   pixnum = 4		; 4 pixels each with dimensions 89.4" x 89.4"
   delta = 46.0		; equals half the pixel size + half the gap size   
      for i = 0, 1 do begin
         for j = 0, 1 do begin
            p = abs(3 * i - j)
            rap(p,*) = c_ra + $
	    ( delta*(2*i - 1)*SIN(angle) + delta*(1 - 2*j)*COS(angle) ) / 3600 $
	    / cos(theta)
            decp(p,*)= c_dec+ ( delta*(2*i - 1)*COS(angle) - delta*(1 - 2*j)*SIN(angle) ) / 3600
         endfor 
      endfor
ENDIF ELSE BEGIN
   FOR ipix=0,pn-1 DO BEGIN
    rap(ipix,*) = ra
    decp(ipix,*) = dec
   ENDFOR
ENDELSE

phtiaap.rap = rap 
phtiaap.decp = decp

RETURN
END
