;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GET_CC.PRO
;
; PURPOSE:
;	Provide colour correction factor depending on filter and spectral
;	type for a black-body spectrum of a given temperature or a
;	power-law spectrum. Linear interpolation of CC table is
;	performed for a given temperature. 
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	GET_CC, filter, type, parameter, cc_val
;
; INPUTS:
;       filter :        string containing filter name as defined in
;                       COMMON block phtfilter
;       type   :        'bb'     for Black-body
;                       'power'  for Power-law
;                       'mbbnu1' for modified BB with emissivity
;                                proportinal nu^1
;                       'mbbnu2' for modified BB with emissivity
;                                proportinal nu^2
;       parameter:      Temperature; if type='bb', 'mbbnu1' or 'mbbnu2'
;                       Power-law index; if type='power' 
;
; OUTPUTS:
;      cc_val    :        colour correction factor
;
; KEYWORD PARAMETERS:
;	ERROR : 	returns an error condition:
;			0 = no error
;			1 = could not successfully read the necessary table
;	ASCII :         if set read ASCII file, otherwise read FITS file
;
; PROCEDURES USED:
;       Internal:
;               read_ccbb
;
; MODIFICATION HISTORY:
;	Written by: 	I. Heinrichsen, MPIK/Heidelberg, Mai 1997
;	                V6.3
;                       IH, changed format of cc_bb.txt
;                           added modified BBs
;	                V6.4
;                       IH, read FITS files added
;                       V6.6
;	Different names for CALG (without _ and numbers) added (CG)	Apr 98
; V7.1
;	Using also -1. as invalid temperature (CG)			Dec. 99
;	V8.2
;-
PRO READ_CCBB, filename, filter, parameter, fits, cc_val, ERROR=error
;
; Subroutine internal to GET_CC
;
error=1
; check valid temperature range
if ((parameter gt 10000.) or (parameter lt 10.)) then begin
    goto, temp_range
endif
;ascii first
if fits eq "false" then begin  
    fmt="a,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f,f"
    readcol, filename, fil, t1, t2, t3, t4, t5, t6, t7, t8, t9, $
      t10, t11, t12, t13, t14, t15, t16, t17, t18, t19,  $
      t20, t21, t22, t23, t24, t25,t26, t27,  $
      FORMAT=fmt, SILENT=1
    cc=[[t1],[t2], [t3], [t4], [t5], [t6], [t7], [t8], [t9],  $
        [t10], [t11], [t12], [t13], [t14], [t15], [t16],  $
        [t17], [t18], [t19], [t20], [t21], [t22], $
        [t23], [t24], [t25], [t26], [t27]]
    ind_filter=where(fil eq filter)
    if ind_filter(0) ne -1 then begin
; get the nearest two temperatures
        tmax=max(where(cc(0,*) ge parameter))
        tmin=min(where(cc(0,*) le parameter))
        if (cc(ind_filter(0),tmax) eq 10.) or  $
          (cc(ind_filter(0),tmin) eq 10.) or $
	  (cc(ind_filter(0),tmax) eq -1.) or  $
          (cc(ind_filter(0),tmin) eq -1.) then begin
            goto, temp_range
        endif
        if tmax ne tmin then begin
; linear interpolate cc factor  
            cc_val=cc(ind_filter(0),tmin)+  $
              (cc(ind_filter(0),tmax) - cc(ind_filter(0),tmin)) *  $
              (1-(cc(0,tmax)-parameter)/(cc(0,tmax)-cc(0,tmin)))
        endif else begin
            cc_val=cc(ind_filter(0),tmax)
        endelse
    endif else begin
        goto, wrong_filter
    endelse
endif else begin
; now the the fits
    fxbopen, unit, filename, 1, hdr
    fxbread, unit, temperature, 1
junk='TEMPERATURE'
    fxbread, unit, cc, filter
    fxbclose, unit
    free_lun, unit
    tmax=max(where(temperature(*) ge parameter))
    tmin=min(where(temperature(*) le parameter))
    if (cc(tmax) eq 10.) or (cc(tmin) eq 10.) then goto, temp_range
    if tmax ne tmin then begin
; linear interpolate cc factor  
        cc_val=cc(tmin)+  $
          (cc(tmax) - cc(tmin)) *  $
          (1-(temperature(tmax)-parameter)/ $
           (temperature(tmax)-temperature(tmin)))
    endif else begin
        cc_val=cc(tmax)
    endelse
endelse

error=0 
RETURN
;
; Error Handling
;

wrong_filter: 
MESSAGE, /INFO, "Wrong filter ID: "+filter
RETURN

temp_range: 
MESSAGE, /INFO, "Temperature out of range for Black-body colour correction: " $
  +string(parameter)
RETURN

END

PRO GET_CC, filter, type, parameter, cc_val, ASCII=ascii, ERROR=error 

; initialize error condition
error = 1
; initialize return value
cc_val = 0.

; check for correct calling sequence
if (n_params() lt 4) then begin
    goto, wrong_params
endif

case type of
;
; Black-body Correction
;
    'bb': begin
        fits="false"
; ensure the necessary GAL-G files are availbale
        errfits=0
        if not(keyword_set(ascii)) then begin
            filename=strcompress(!CALG+'PCC_BB.FITS', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then begin
             filename=strcompress(!CALG+'PCCBB.FITS', /REMOVE_ALL)
             test=findfile(filename)
            ENDIF

            if test(0) eq '' then begin
                errfits=1
                MESSAGE, /INFO, "FITS CAL-G file "+filename+" not found, reading ASCII file"
            endif else fits="true"
        endif
        if ((errfits eq 1) or  keyword_set(ascii)) then begin    
            filename=strcompress(!CALG+'cc_bb.txt', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then goto, no_file_bb
        endif

;file exists go ahead read it!
;
        read_ccbb, filename, filter, parameter, fits, cc_val, error=error
    end
;
; Modified black-body nu^1
;
    'mbbnu1': begin
        fits="false"
        errfits=0
        if not(keyword_set(ascii)) then begin
            filename=strcompress(!CALG+'PCC_MBBNU1.FITS', /REMOVE_ALL)
            test=findfile(filename)

            if test(0) eq '' then begin
             filename=strcompress(!CALG+'PCCMBBONE.FITS', /REMOVE_ALL)
             test=findfile(filename)
            ENDIF

            if test(0) eq '' then begin
                errfits=1
                MESSAGE, /INFO, "FITS CAL-G file "+filename+" not found, reading ASCII file"
            endif else fits="true"
        endif
        if ((errfits eq 1) or  keyword_set(ascii)) then begin    
            filename=strcompress(!CALG+'cc_mbbnu1.txt', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then goto, no_file_bb
        endif

;file exists go ahead read it!
;
        read_ccbb, filename, filter, parameter, fits, cc_val, error=error
    end
; Modified black-body nu^1
;
    'mbbnu2': begin
        fits="false"
; ensure the necessary GAL-G files are availbale
        errfits=0
        if not(keyword_set(ascii)) then begin
            filename=strcompress(!CALG+'PCC_MBBNU2.FITS', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then begin
             filename=strcompress(!CALG+'PCCMBBTWO.FITS', /REMOVE_ALL)
             test=findfile(filename)
            ENDIF

            if test(0) eq '' then begin
                errfits=1
                MESSAGE, /INFO, "FITS CAL-G file "+filename+" not found, reading ASCII file"
            endif else fits="true"
        endif
        if ((errfits eq 1) or  keyword_set(ascii)) then begin    
            filename=strcompress(!CALG+'cc_mbbnu2.txt', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then goto, no_file_bb
        endif

;file exists go ahead read it!
        read_ccbb, filename, filter, parameter, fits, cc_val, error=error

    end
;
; Power law correction
;
    'power': begin

        fits="false"
; check of allowed power-law
        powers=[-3., -2., -1., 0., 1.,2., 3.] 
        ind=where(powers eq parameter)
        if ind(0) eq -1. then goto, power_range
; ensure the necessary GAL-G files are available
        errfits=0
        if not(keyword_set(ascii)) then begin
            filename=strcompress(!CALG+'PCC_POWER.FITS', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then begin
             filename=strcompress(!CALG+'PCCPOWER.FITS', /REMOVE_ALL)
             test=findfile(filename)
            ENDIF

            if test(0) eq '' then begin
                errfits=1
                MESSAGE, /INFO, "FITS CAL-G file "+filename+" not found, reading ASCII file"
            endif else fits="true"
        endif
        if ((errfits eq 1) or  keyword_set(ascii)) then begin    
            filename=strcompress(!CALG+'cc_power.txt', /REMOVE_ALL)
            test=findfile(filename)
            if test(0) eq '' then goto, no_file_bb
        endif
;file exists go ahead read it!
;ascii first
        if fits eq "false" then begin  
            fmt="a,f,f,f,f,f,f,f"
            readcol, filename, fil, t1, t2, t3, t4, t5, t6, t7,  $
              FORMAT=fmt, SILENT=1
            cc=[[t1],[t2], [t3], [t4], [t5], [t6], [t7]]
            ind_filter=where(fil eq filter)
            if ind_filter(0) ne -1 then begin
; pick correct cc factor for filter and power-law
                ind_power=where(cc(0,*) eq parameter)
                cc_val=cc(ind_filter(0),ind_power(0))
            endif else begin
                goto, wrong_filter
            endelse
        endif else begin
; now the the fits
            fxbopen, unit, filename, 1, hdr
            fxbread, unit, power, 'POWER'
            fxbread, unit, cc, filter
            cc_val=cc(where(power eq parameter))
        endelse
        error=0 
    end
;
    else: goto, wrong_type
endcase

RETURN
;
; Error Handling
;
wrong_params: 
MESSAGE, /INFO, "Syntax: GET_CC, filter, type, parameter, cc_val, ASCII=ascii, ERROR=error"
RETURN

wrong_type: 
MESSAGE, /INFO, "Wrong spectral type for colour correction: "+type
RETURN

wrong_filter: 
MESSAGE, /INFO, "Wrong filter ID: "+filter
RETURN

power_range: 
MESSAGE, /INFO, "Power out of range for Power-law colour correction: " $
  +string(parameter)
RETURN

no_file_bb: 
MESSAGE, /INFO, "Colour correction calibration file not available: "+filename
RETURN

END
;
;
;
