;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GET_COORDINATES
;
; PURPOSE:
;       Read a PHT IRPH file filling the coordinates data into the SPD 
;       temporary structure given as input
; 
; CATEGORY:
;       PIA - I/O
;
; CALLING SEQUENCE:
;       get_coordinates, admn, rpid, ra, dec, roll
;
; INPUT:
;       admn : Identification of a valid measurement (12 character string).
;	rpid :  Raster point Id for the same measurement (BYTARR[2,npts])
;
; OUTPUT:
;	ra :  Right Ascension (FLTARR[npts])
;	dec:  Declination (FLTARR[npts])
;	roll: Roll Angle (FLTARR[npts])
;	
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               phtcs
;       PIA Procedure(s):
;               read_irph
;               read_cs
;
; MODIFICATION HISTORY:
;       Written by: Jose Acosta (ISO-Soc/VILSPA)     		July 1998
;       V7.x
;	Some text added (CG)					March 1999
;	V7.4
;	Reading internal PIA IRPH files added (for simulations)	September 1999
;	V8.1
;-
PRO get_coordinates, admn, rpid, ra, dec, roll, ierr, INFO = INFO

@phtcs

	ierr = 0

        filename = !FITDIR + 'IRPH' + $
                        STRMID(admn,4,8) + '.FITS'
 
        resu = pcont_getfiles(filename, /NOSORT)
        IF resu(0) EQ '' THEN BEGIN 
		ierr = 1
                INFO = ["Warning: The file " + filename + " was not found", $
                        "No computation of astronomical coordinates"]
		IF is_file_exist(!INTDIR + admn + '.IRPH') THEN BEGIN
			RESTORE, !INTDIR + admn + '.IRPH'
			rpid_irph = irph.rpid
			ra_irph = irph.ra
			dec_irph = irph.dec
			roll_irph = irph.roll
			ierr = 0
			GOTO, after_read
		ENDIF			
		
                RETURN
        ENDIF

; Read the IRPH file
        read_irph, filename, odec, ora, oroll, utc, rpid_irph, oRPQ, HDR, $
                  DEC_irph, RA_irph, ROLL_irph, RPQ, OTFT  

; Replace by uncorrected if wished
        IF !CORRCOOR THEN BEGIN
          dec_irph = odec
          ra_irph = ora
          roll_irph = oroll
          rpq = orpq
        ENDIF  

after_read:

; Take the right measurement 
        measpos = FIX(STRMID(admn,2,2))
        ind11 = WHERE(RPID_irph(0,*) EQ 1b and RPID_irph(1,*) EQ 1b, count)
 
; Prevent for RP separation of measurements - look into repetition of [1,1] 
; if not take simply everything, if yes, separation won't work 

   	IF count EQ 1 THEN BEGIN
	  indfirst = 0
	  indlast = N_ELEMENTS(ra_irph) -1 
	ENDIF ELSE BEGIN
         csname = 'PSTA'+STRMID(admn,4,8)+'.FITS'
         read_cs, csname, cshdr, indcs 
         handle_value, phtcs(indcs(0)).handle, cst
         utst = cst.utst(0,measpos)
         indfirst = WHERE(utc(ind11) LT utst, count)
         IF count GT 0 THEN BEGIN
                        indfirst = ind11(max(indfirst))
                        indlast = indfirst + $
                FIX(MAX(rpid(1,*)))* FIX(MAX(rpid(0,*))) - 1
         ENDIF ELSE BEGIN
		ierr = 1
                info = 'Cannot find the way through the IRPH'
                RETURN
         ENDELSE
	ENDELSE

; Prevent for separation of measurements - look into repetition of [1,1] - 
	IF STRMID(admn,12,2) EQ "_y" AND N_ELEMENTS(ra) EQ 1 THEN BEGIN
		ind = WHERE(rpid_irph(0,*) EQ rpid(0) AND $
			     rpid_irph(1,*) EQ rpid(1), count)
		IF count EQ 1 THEN BEGIN
		   ra = ra_irph(ind)
		   dec = dec_irph(ind)
		   roll = roll_irph(ind)
		ENDIF ELSE MESSAGE, /INFO, $ 
		'Meas. '+ dec_admn(admn) + ': Data Raster point not found in IRPH' 
	ENDIF ELSE BEGIN
 
	   FOR I=indfirst, indlast DO BEGIN
		ind = WHERE(rpid(0,*) EQ rpid_irph(0,I) AND $
				rpid(1,*) EQ rpid_irph(1,I), nfound) 
; fill structure
		IF nfound GT 0 THEN BEGIN
 			  dec(ind) = dec_irph(I) 
			  ra(ind) = ra_irph(I) 
			  roll(ind) = roll_irph(I)
		ENDIF ELSE MESSAGE, /INFO, $ 
		'Meas. '+ dec_admn(admn) + ': Raster point not found in data'
	   ENDFOR
	ENDELSE
	                                                                  
RETURN
END 
