;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
pro get_pht_hk,meas_name,phtcs_redu, FULL_TDT=FULL_TDT

;+
; NAME:
;	GET_PHT_HK
;
; PURPOSE:
;	This routine needs an specific measurement-name as an input. It
;       then loads the corresponding PSTA and GEHK fits files and
;       extracts the specific data for this measurement.
;       All values, including the time, are converted to strings or to
;       physical values. The converted time is in seconds with t=0s
;       at the start of the PSTA .
;       Output is stored in the phthk common block.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;       GET_PHT_HK, Meas_name, Phtcs_redu
;
; INPUTS:
;       Meas_name : Name of measurement
;
; KEYWORDS:
;	FULL_TDT: IF set, full TDT info is extracted rather than the one
;		corresponding to the measurement
; OUTPUTS:
;       Phtcs_redu: Structure with the reduced and converted ext. hk 
;
; COMMON BLOCKS:
;	phtcs
;	phthk
; 
; PROCEDURE:
;       Use the uk-time for extracting, then convert to seconds.
;       Use conv_hk and conv_cs to convert the rawvalues. 
;
; PROCEDURES USED:
;       PIA LIB:   read_idb_def
;                  read_cs
;	           read_phthk
;                  conv_hk
;                  conv_cs
;	ASTRO LIB: fxbclose
;	           fxbopen
;	           sxpar	
;
; MODIFICATION HISTORY:
;       Written by:	Andreas Karch
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;	Bug with CS name (fxbopen) fixed  (CG)	Nov 95
;-

@phthk
@phtcs

;cut meas_name into it's three parts

detector=strmid(meas_name,0,2)
position=strmid(meas_name,2,2)
tdtnum=strmid(meas_name,4,8)

;check, if idb_def is already loaded
;if not,load it

if n_elements(idb) eq 0 then begin
read_idb_def,hdr_idb,idb,idb_ar_table,idb_ar_image
endif

;check, if hkout is already from this measurement
;else load and convert data from the GEHK.FITS file
;with the tdtnum


; add FULL_TDT to meas_name 
IF KEYWORD_SET(FULL_TDT) THEN meas_name = meas_name +'F_TDT'


if n_elements(hkout) eq 0 then hkout='not defined'
if hkout(n_elements(hkout(*,0))-1,0) ne meas_name then begin

;read fits file in arrays hkdat (housekeeping data)
;                               timestep
;                                headerf (header of fitsfile)
;if it is not already there
 
 if n_elements(hkdat) eq 0 then begin
  read_phthk,tdtnum,hkdat,timestep,headerf
 endif else begin
  handle_value,hkdat(n_elements(hkdat(*,0))-1,0),test_t_num
  if test_t_num ne tdtnum then begin
  read_phthk,tdtnum,hkdat,timestep,headerf
  endif 
 endelse

;load PSTA file

; - change -> shape4
; filename=!FITDIR + 'PSTA'+tdtnum+'.FITS'
 filename= 'PSTA'+tdtnum+'.FITS'
;  print, 'filename', filename
 read_cs,filename,hdr_psta,index_phtcs

;if PSTA file was read in in former times, re-read the header

 if n_elements(hdr_psta) eq 0 then begin
  fxbopen,unit,!FITDIR+filename,1,hdr_psta
  fxbclose,unit
  free_lun, unit
 endif

;read time conversion factor

 trefitku=sxpar(hdr_psta,'trefitku')

;for test purposes while header not right use the common value
 IF trefitku EQ 0 THEN trefitku = 0.00006103515630

;convert phtcs
 conv_cs,phtcs(index_phtcs),convcs

;extract named measurement

 handle_value,phtcs(index_phtcs).handle,info

 case detector of
  'P1': index=where(info.subs eq 0)
  'P2': index=where(info.subs eq 1)
  'P3': index=where(info.subs eq 2)
  'SL': index=where(info.subs eq 3)
  'C1': index=where(info.subs eq 6)
  'C2': index=where(info.subs eq 7)
  ELSE: print,'this detector does not exist'
 endcase

 if index(0) eq -1 then print,'this measurement does not exist'

 index=index(position)

 handle_value,convcs.handle,convcs

;initialize phtcs_redu struct

 phtcs_redu={                $
              UKST : convcs.UKST(index), $
              UKEN : convcs.UKEN(index), $
              IKST : convcs.IKST(index), $
              IKEN : convcs.IKEN(index), $
              UTST : convcs.UTST(index), $
              UTEN : convcs.UTEN(index), $
              AOTN : convcs.AOTN(index), $
              SUBS : convcs.SUBS(index), $
              OPFO : convcs.OPFO(index), $
              F1TS : convcs.F1TS(index), $
              F2TS : convcs.F2TS(index), $
              F1PS : convcs.F1PS(index), $
              F2PS : convcs.F2PS(index), $
              C1PS : convcs.C1PS(index), $
              C2PS : convcs.C2PS(index), $
              C3PS : convcs.C3PS(index), $
              CMOD : convcs.CMOD(index), $
              CAMP : convcs.CAMP(index), $
              CSTE : convcs.CSTE(index), $
              CINC : convcs.CINC(index), $
              CRES : convcs.CRES(index), $
              DETA : convcs.DETA(index), $
              DRS  : convcs.DRS(index),$
              MUX1 : convcs.MUX1(index), $
              MUX2 : convcs.MUX2(index), $
              XSTA : convcs.XSTA(index), $
              D1OF : convcs.D1OF(index), $
              D1GA : convcs.D1GA(index), $
              D2OF : convcs.D2OF(index), $
              D2GA : convcs.D2GA(index), $
              FREQ : convcs.FREQ(index), $
              NNDR : convcs.NNDR(index), $
               NDR : convcs.NDR(index), $
              INTT : convcs.INTT(index), $
              MEAT : convcs.MEAT(index), $
              MPC1 : convcs.MPC1(index), $
              MPC2 : convcs.MPC2(index), $
              MPC3 : convcs.MPC3(index), $
              MET : convcs.MET(index) }   


;calculate conversion factor from uk-time in seconds
;t=0s equals uk-start

 all_time_sec=(info.iken(index)-info.ikst(index))*trefitku
 sec_per_ukstep=all_time_sec/(info.uken(index)-info.ukst(index))

;extract that data from hkdat taken during or short before and
;after the given measurement

 IF KEYWORD_SET(FULL_TDT) THEN $
   index_time=indgen(N_ELEMENTS(timestep)) $
  ELSE BEGIN
     help=where(timestep ge info.ukst(index) and timestep le info.uken(index))
     if help(0) eq -1 then print,'hk times and ext.hk times not matching'
     if help(0) ne 0 then help=[help(0)-1,help]
     if help(n_elements(help)-1) ne n_elements(timestep) then $
            help=[help,help(n_elements(help)-1)+1]
     index_time=help
  ENDELSE

 hkdat_redu=hkdat(*,index_time)
 timestep_redu=timestep(index_time)
 timestep_redu=(timestep_redu-info.ukst(index))*sec_per_ukstep

;now call conversion routine for hkdat
 
 conv_hk,idb,hkout,hkdat_redu,timestep,idb_ar_table,idb_ar_image
 
;describe this generated hkout array with meas_name
 hkout(n_elements(hkout(*,0))-1,*)=meas_name

endif 

end
