;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GET_RAMPEND
;
; PURPOSE:
;	determines the limits of all ramps at ERD level, using DR and 
;	taking into account possible telemetry drops. It also works
;       when ramp subdivision is selected. 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;    GET_RAMPEND, Xall, Dr, Tint, Rampflag, Rampend, Dimsrd, SUB_RAMP =sub_ramp 
;
; INPUTS:
;	Xall	: Array containing the independent variable [t]  (same size)
;	Ndeg	: Polynomial degree
;	Dr	: Read-out status (0 = NDR, 1 = DR) 
;	Tint	: On chip integration time applied in the measurement
;
; OUTPUTS:
;	Rampflag: Array containing a flag specifying the actual end of a 
;		  ramp.
;	Rampend : Array containing the index where ramps end.
;	Dimsrd	: Dimension of SRD
;	
;
; KEYWORD INPUT:
;	SUB_RAMP: Specifies the number of sub-ramps
;	DATRED:	  Data reduction factor
;
; RESTRICTIONS:
;	Xall, Dr and Tint must have the same number of elements
;
; PROCEDURES USED:
;	None
;
; MODIFICATION HISTORY:
;	Written by:	Jose Acosta 				May 1999
;	Data reduction keyword added (CG)			May 1999	
;	Rampends in the right sequence if time drop 
;	 and tolerance larger cause of chopper pl. (CG)		May 1999
;	V7.4
;	Check for avoiding rampends twice (CG)			July 1999
;	V8.0
;-
;
PRO GET_RAMPEND, ERDTIME, DR, TINT, RAMPFLAG, RAMPEND, DIMSRD, $ 
	  SUB_RAMP = sub_ramp, DATRED=datred
;	  
;On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
IF N_PARAMS() LT 4 THEN BEGIN 
       print,'Syntax: GET_RAMPEND, ERDTIME, DR, TINT, RAMPFLAG  ' + $
          '[,RAMPEND] [,DIMSRD] [,SUB_RAMP=sub_ramp]'
	RETURN
ENDIF	  
;
	nele = N_ELEMENTS(erdtime)
	IF NELE NE N_ELEMENTS(dr) THEN $
		MESSAGE, 'ERDTIME and DR must have same number of elements'
		
	IF NOT KEYWORD_SET(DATRED) THEN BEGIN
		MESSAGE, /INFO, 'Data reduction factor not given, assuming 1'
		datred=1
	ENDIF

	rampflag = dr	

; check that time elapsed between consecutive DR is of same order than 
; integration time
        ind_dr = WHERE(DR eq 1, n_dr)
	IF n_dr EQ 0 THEN BEGIN 
	   rampflag(nele-1) = 1
	   rampend = nele-1
	   dimsrd = 1
	   print,'WARNING: ERD Measurement contains no DR'
	   RETURN
	ENDIF   
	
; complete ind_dr to have the end and beginning of the measurement
	IF ind_dr(n_dr-1) NE (nele-1) THEN ind_dr = [ind_dr,nele-1]
	   
	n_dr = N_ELEMENTS(ind_dr) - 1  
	time_rend = erdtime(ind_dr(0:n_dr))
	time_sh = time_rend - [erdtime(0),time_rend(0:n_dr-1)]
	
	;define a tolerance time interval as the minimum between consecutive 
	; readouts * nr_routs [because of different time elapsed bet. chop-plats]
	ntol = 5 < n_dr
	tolnumber = min(ind_dr(1:ntol)-ind_dr(0:ntol-1))
	toltime = tolnumber * MIN(erdtime(1:nele-1)-erdtime(0:nele-2))
	; check that 
	ind_drop = WHERE(time_sh GT (tint*datred+toltime), ndrop)
 
        IF ndrop GT 0 THEN BEGIN
	  FOR i=0, ndrop-1 DO BEGIN
	     time_ref = time_rend(ind_drop(i)) - (tint*datred+toltime)
	     ind_newdr = WHERE(erdtime LT time_ref,nnewdr)
	     IF nnewdr GT 0 THEN ind_dr = [ind_dr,ind_newdr(nnewdr-1)]
	     bsor = bsort(ind_dr, sorted)
	     ind_dr = sorted
	  ENDFOR
	ENDIF
	
	rampflag(ind_dr) = 1
	rampend = ind_dr
	dimsrd = N_ELEMENTS(ind_dr)
	
	IF KEYWORD_SET(SUB_RAMP) THEN BEGIN 
	   newrampend = get_rampsplit(rampend,sub_ramp)
	   dimsrd = N_ELEMENTS(newrampend)
	   rampend = newrampend
	   rampflag(rampend) = 1
	ENDIF 
	
	diff = rampend(1:dimsrd-1) - rampend(0:dimsrd-2)
	ind = WHERE(diff GT 0, n)
	nr = dimsrd - n - 1
	IF nr GT 0 THEN BEGIN
	 rampend = [rampend(ind), rampend(dimsrd-1)]
	 MESSAGE, /INFO, 'Fixed wrong assigned rampends found: ' + $
	 		STRTRIM(nr)
	 dimsrd = dimsrd - nr
	ENDIF 
	
RETURN
END
