;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GET_RESPONS
;
; PURPOSE:
;	Responsivity calculation.
;
; CATEGORY:
;	PIA -  processing
;
; CALLING SEQUENCE:
;	GET_respons, tmpdata, header
;
; INPUTS:
;	tmpdata
;		A temporary SCP structure
;	header
;		The corresponding header
;	
; KEYWORD PARAMETERS:
;	GROUP
;		The group leader of the widget hirarchy
;	NO_WIDGET
;		To work without widget use - [=1: avg FCS1, =2: medians FCS1]
;	
; OUTPUTS:
;	header
;		Modified header
;	
; KEYWORD OUTPUT:
;	out_of_limits	If out_of_limits check positive, is set to 1
;	err_resp	If no responsivity can be calculated, return with 1	
;	
; SIDE EFFECTS:
;	Variables in the common block for actual responsivities are changed
;	
; RESTRICTIONS:
;	tmpdata must be a temporary PIA SCP structure, corresponding to an
;	FCS measurement
;
; METHOD:
;	Use of an FCS measurement and the corresponding calibration tables
;	for the responsivity determination	
;
; COMMON BLOCKS:
;	respon_cmn	local common
;	phtcs		compact status dynamical structure
;	phtresdef	default responsivities common
;	phtresact	actual responsivities common
;	phtfcs		FCS calibration tables
;	phtillum	FCS illumination matrices common
;	phtcap		capacitances common
;	phtoptpow	FCS optical power range
;	phtacc		actual selection criteria
;	
; PROCEDURES USED:
;	PIA Procedure(s):
;		conv_subs
;		cp_disc
;		phtcap_init
;		read_cs
;		read_fcsill
;		read_fcspow
;		read_resp
;	Internal:
;		pia_respons_event
;	Astrolib:
;		fxaddpar
;
; MODIFICATION HISTORY:
;	Written by:	Jose Acosta (ESA/VILSPA)		June 1998
;	V7.2
;	ERR_RESP keyword added (CG)				November 1998
;	V7.3
;	Filter to Filter for P added (CG)			July 1999
;	V7.9
;	OUT OF LIMIT == previous soft limit (CG) 		May 2002
;	V10.0
;-
PRO ASSIGN_RESPONS, respons, det, choice, URESP= uresp  

@phtresact

IF N_PARAMS() NE 3 THEN BEGIN 
   print,'Syntax: assign_respons, respons, det, choice
   RETURN
ENDIF

   r_origin = respons.resp_origin

   CASE choice OF 
       0: BEGIN 
             tmpresp = respons.defv
	     tmprespunc = respons.defunc
	     r_origin(0) = 'DEFAULT'
          END
       1: BEGIN  ; FCS1, average FCS signals 
             tmpresp = respons.aver(*,0)
	     tmprespunc = respons.avun(*,0) 
	     r_origin(2) = 'FCS1 - aver'
          END 
       2: BEGIN
             tmpresp = respons.med(*,0)
	     tmprespunc = respons.medu(*,0) 
	     r_origin(2) = 'FCS1 - medi'
       	  END
       3: BEGIN
             tmpresp = respons.aver(*,1)
	     tmprespunc = respons.avun(*,1) 
	     r_origin(2) = 'FCS2 - aver'
       	  END
       4: BEGIN
             tmpresp = respons.med(*,1)
	     tmprespunc = respons.medu(*,1) 
	     r_origin(2) = 'FCS2 - medi'
       	  END
       5: BEGIN
             tmpresp = (respons.aver(*,0) + respons.aver(*,1))/2.
	     tmprespunc = SQRT(respons.avun(*,0)^2+respons.avun(*,1)^2)/2. 
	     r_origin(2) = '<FCS1,FCS2>-aver'
       	  END
       6: BEGIN
             tmpresp = (respons.med(*,0) + respons.med(*,1))/2.
	     tmprespunc = SQRT(respons.medu(*,0)^2+respons.medu(*,1)^2)/2.
	     r_origin(2) = '<FCS1,FCS2>-medi'
       	  END
       ELSE:     
   ENDCASE
   
   	
   CASE det of 
	 'P1': BEGIN
		 p1respact = tmpresp 
		 p1respunc = tmprespunc 
		 respact_origin.p1n = r_origin(0)
		 respact_origin.p1f = r_origin(1)
		 respact_origin.p1m = r_origin(2)
	       END   
	 'P2': BEGIN
		 p2respact = tmpresp 
		 p2respunc = tmprespunc 
		 respact_origin.p2n = r_origin(0)
		 respact_origin.p2f = r_origin(1)
		 respact_origin.p2m = r_origin(2)
	       END    
	 'P3': BEGIN
		 p3respact = tmpresp 
		 p3respunc = tmprespunc 
		 respact_origin.p3n = r_origin(0)
		 respact_origin.p3f = r_origin(1)
		 respact_origin.p3m = r_origin(2)
	       END    
	 'C1': BEGIN
		 c1respact = tmpresp 
		 c1respunc = tmprespunc
		 respact_origin.c1n = r_origin(0)
		 respact_origin.c1f = r_origin(1)
		 respact_origin.c1m = r_origin(2)
	       END    
	 'C2': BEGIN
		 c2respact = tmpresp 
		 c2respunc = tmprespunc 
		 respact_origin.c2n = r_origin(0)
		 respact_origin.c2f = r_origin(1)
		 respact_origin.c2m = r_origin(2)
	       END    
   ENDCASE

   uresp = tmpresp  

RETURN
END


PRO get_respons, tmpdata, header, respons, fcsill, nfcs, LABEL = label, $ 
  OUT_OF_LIMITS=out_of_limits, INFO = info, ERR_RESP=err_resp

@phtcs
@phtacc
@phtfcs
@phtillum
@phtcap
@phtoptpow
@phtffield
@phtftof
@phtfilter
@phtapert

	info = ''
	out_of_limits = 0
	err_resp = 0
	
; FS!!   aperture = [4.0828, 8.9738, 14.522, 21.237, 30.581, 48.029, 48.029, $
;                    .03142, .1018, .173, .322, .454, 1.129, .849]
		
;        aperture = [4.1548, 9.0792, 14.522, 21.237, 30.250, 48.398, 48.398, $
;                    .03140, .1001, .1735,.2827,.528, 1.129, .7854]

	IF N_ELEMENTS(p_area) EQ 0 THEN read_omega
	 aperture = p_area(*,0)

; If phtresp not yet filled, read it from the FITS file
	def_resp = default_response(tmpdata)
	npix = pixel_det(tmpdata.det)
	thresp = FLTARR(npix)
	thresu = thresp
	FOR ipix=0,npix-1 DO BEGIN
	 thresp(ipix)=AVG(def_resp(ipix,*,0))	
	 thresu(ipix)=AVG(def_resp(ipix,*,1))
	ENDFOR	

; If phtffield not yet filled, read it from the FITS file
	IF n_elements(c1flatf) EQ 0 THEN read_flatfield
        det = tmpdata.det

; If phtfilter not yet filled, initialize
	IF n_elements(p1filters) EQ 0 THEN phtfilter_init

; Which FCS measurement type have we got

	indfcs = WHERE(tmpdata.fcsn NE 0, nfc)
	junk = WHERE(tmpdata.fcsn EQ 0, nofc)
	maxpos = max(tmpdata.fcsn(indfcs))
	minpos = min(tmpdata.fcsn(indfcs))

	CASE 1 OF
	   nfc EQ 0: BEGIN
	                MESSAGE,/INFO, STRTRIM(tmpdata.admn,2) + $ 
			      ' is not a FCS measurement, returning'
			err_resp = 1 
			RETURN
	   	     END 
	   nofc GT 0: labtxt = 'Chopped measurement: Source vs FCS'+$
			strtrim(string(maxpos),2) 
           maxpos EQ minpos: labtxt = 'Staring measurement: FCS'+   $
	                strtrim(string(maxpos),2)
	   ELSE: labtxt = 'Chopped measurement: FCS1 vs FCS2'
	ENDCASE
	     
; Call cp_disc to flag all discarded chopper plateaux
	cp_disc, tmpdata, cp_disc

	avch = fltarr(npix,maxpos-minpos+1)
	avcu = avch
	mech = avch
	mecu = avch	   


; Check if dark current subtraction already performed, and if not, perform it 
	junk =  FXPAR(header, 'PRS_DARK', count=count)

	IF count EQ 0 THEN BEGIN
		PRINT, 'Performing dark current subtraction on FCS measurement'
		darkcur_orb, tmpdata, /SCP
	ENDIF


; Loop over the present FCSs
	count = -1

	FOR fcsno=minpos, maxpos DO BEGIN

	   count = count + 1
; Reduce the arrays to the elements corresponding to the FCS
	   
	   indfcs = WHERE(tmpdata.fcsn EQ fcsno, n)
	   datav = tmpdata.aver(*,indfcs)
	   datau = tmpdata.avun(*,indfcs)

	   datme = tmpdata.medi(*,indfcs)
	   
           cpdf = cp_disc(*,indfcs)

; Now we must loop over the pixels due to the individual flags

	   FOR PIX=0,npix-1 DO BEGIN

		indacc = WHERE(cpdf(pix,*) EQ 0, nele)

		IF nele GT 1 THEN BEGIN
		  weight_mean, datav(pix,indacc), datau(pix,indacc), av, std 
		  avcu(pix,count) = std
		  avch(pix,count) = av 
		  mech(pix,count) = median(datme(pix,indacc)) 
		ENDIF ELSE IF nele EQ 1 THEN BEGIN  
		  avch(pix,count) = datav(pix,indacc) 
		  avcu(pix,count) = datau(pix,indacc) 
		  mech(pix,count) = datme(pix,indacc) 
                ENDIF ELSE BEGIN
                  MESSAGE, /INFO, $
		   'No uncertainty values available for FCS'+$
		    strtrim(string(fcsno),2)+', Pixel '+strtrim(string(pix+1),2)  
                  MESSAGE, /INFO, 'Continuing, but uncertainties cannot be trusted'
		  indacc = WHERE(cpdf(pix,*) EQ 1, nele)
		  IF nele GT 0 THEN BEGIN 
		   weight_mean, datav(pix,indacc), datau(pix,indacc), av, std 
		   avcu(pix,count) = std
		   avch(pix,count) = av 
		   mech(pix,count) = median(datme(pix,indacc)) 
		  ENDIF ELSE BEGIN 
                   MESSAGE, /INFO, $
		   'No values available for FCS'+$
		    strtrim(string(fcsno),2)+', Pixel '+strtrim(string(pix+1),2)  
                   MESSAGE, /INFO, 'Giving up, returning'
		   err_resp = 1
                   RETURN
                  ENDELSE
                 ENDELSE
		   

     	   ENDFOR		

	ENDFOR
	
; Median uncertainty = standard deviation
	mecu = avcu  	

; Read Capacities and/or FCS powers if necessary 
	IF N_ELEMENTS(slcap) EQ 0 THEN phtcap_init

; Access the right fcspow calibration values
	orig_name = 'P' + STRMID(tmpdata.admn,0,2) + 'FCSPOW'
	det_calfile, orig_name, tmpdata.unit, tmp_name, error=error	
	read_fcspow, tmp_name

; Read FCS illumination matrix if necessary
	IF STRMID(tmpdata.admn,0,1) EQ 'C' THEN BEGIN
	 orig_name = 'P' + STRMID(tmpdata.admn,0,2) + 'ILLUM'
	 det_calfile, orig_name, tmpdata.unit, tmp_name, error=error	
	 read_fcsill, tmp_name
	ENDIF

	CASE det of
		'P1': BEGIN
			capac = p1cap	
			fcspow = p1fcspow
			val_limits = p1fcsp_lim
			filpos = 3
			filoff = 1
			optpow = longpow
			txtsize = 2
			fcsill = intarr(2,2) + 1
			fcsilun = intarr(2,2)
; take default responsivities in the begin
			;p1respact = thresp 
			;p1respunc = thresu 
			IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof
			 flatu = ppftofu
			ENDIF ELSE BEGIN
			 flatf = ppftof * 0. + 1.
			 flatu = ppftof * 0.
			ENDELSE
		END   
		'P2': BEGIN
			capac = p2cap	
			fcspow = p2fcspow
			val_limits = p2fcsp_lim
			filpos = 3
			filoff = 11
			optpow = longpow
			txtsize = 2
			fcsill = intarr(2,2) + 1
			fcsilun = intarr(2,2)
; take default responsivities in the begin
			;p2respact = thresp 
			;p2respunc = thresu 
			IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof
			 flatu = ppftofu
			ENDIF ELSE BEGIN
			 flatf = ppftof * 0. + 1.
			 flatu = ppftof * 0.
			ENDELSE
		END   
		'P3': BEGIN
			capac = p3cap	
			fcspow = p3fcspow
			val_limits = p3fcsp_lim
			filpos = 3
			filoff = 13
			optpow = longpow
			txtsize = 2
			fcsill = intarr(2,2) + 1
			fcsilun = intarr(2,2)
; take default responsivities in the begin
			;p3respact = thresp 
			;p3respunc = thresu 
			IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof
			 flatu = ppftofu
			ENDIF ELSE BEGIN
			 flatf = ppftof * 0. + 1.
			 flatu = ppftof * 0.
			ENDELSE
		END   
		'C1': BEGIN
			capac = c1cap	
			fcspow = c1fcspow
			val_limits = c1fcsp_lim
			filpos = 2
			filoff = 2
			optpow = longpow
			txtsize = 9
			fcsill = c1ilcor
			fcsilun = c1illun
; take default responsivities in the begin
			;c1respact = thresp 
			;c1respunc = thresu 
			IF !PIA_FFUSE THEN BEGIN
			 flatf = c1flatf
			 flatu = c1flatu
			ENDIF ELSE BEGIN
			 flatf = c1flatf*0.+ 1.
			 flatu = c1flatu*0.
			ENDELSE
			 
		END   
		'C2': BEGIN
			capac = c2cap	
			fcspow = c2fcspow
			val_limits = c2fcsp_lim
			filpos = 2
			filoff = 9
			optpow = longpow
			txtsize = 4
			fcsill = c2ilcor
			fcsilun = c2illun
; take default responsivities in the begin
			;c2respact = thresp 
			;c2respunc = thresu 
			IF !PIA_FFUSE THEN BEGIN
			 flatf = c2flatf
			 flatu = c2flatu
			ENDIF ELSE BEGIN
			 flatf = c2flatf*0. + 1.
			 flatu = c2flatu*0.
			ENDELSE
		END   
		ELSE: BEGIN
			MESSAGE, /INFO, 'Detector not valid'
			err_resp = 1
			RETURN
		ENDELSE
	ENDCASE

; Calculate In-band-Powers

	avch = avch * capac
	avcu = avcu * capac
	mech = mech * capac
	mecu = mecu * capac

; Read the Compact Status if necessary or get the unit - change -> shape4
	give_cs, tmpdata, cst, record, cst_conv, ERROR = isnot_cs
	IF isnot_cs THEN BEGIN 
	  csfile = 'PSTA'+strmid(tmpdata.admn,4,8)+'.FITS'
	  read_cs, csfile
	  give_cs, tmpdata, cst, record, cst_conv
	ENDIF  

; Read TRSnr, FCS selected powers, FILTERnr --> convert to index begin = 0
; + aperture (to be multiplied to IBP if P-Detector [W/mm^2])
	trs = [cst.f1ts(record),cst.f2ts(record)] - 1 	
	fcscho = [cst.f1ps(record),cst.f2ps(record)] 	
	CASE filpos OF
		2: BEGIN
			filtnr = cst.c2ps(record) - filoff		
			apert = 1
			fcsill = reform(fcsill(*,filtnr,*))
			fcsilun = reform(fcsilun(*,filtnr,*))
			flatf = reform(flatf(*,filtnr))
			flatu = reform(flatu(*,filtnr))
		END
		3: BEGIN
			filtnr = cst.c3ps(record) - filoff		
			apert = aperture(cst.c2ps(record)-1)
			flatf = flatf(cst.c3ps(record)-1)
			flatu = flatu(cst.c3ps(record)-1)
		END
		ELSE:
        ENDCASE

	fcscho = float(fcscho) * 50 / 4095


; Calculate expected in-band-power -> responses

	theo_in_band = fltarr(maxpos-minpos+1)
	respon_av = avch * 0
	respon_me = respon_av
	respon_au = respon_av	
	respon_mu = respon_av
	respon_sy = respon_av	
	count = -1
	
	IF maxpos GT minpos THEN out_of_limits = INTARR(2)
	FOR FCSNO=minpos-1,maxpos-1 DO BEGIN

	   count = count + 1
	   pow_cs = fcscho(fcsno)
; Check out of soft (val_lim) and 
; hard limits (< min(fcspow(*,...)) or > max(fcspow(<99.,...)) )
	   IF pow_cs EQ 0. THEN BEGIN
;		out_of_limits = 1
		exprange = [0.,99.]
	   ENDIF ELSE BEGIN

 	    expind  = WHERE(fcspow[*,filtnr,trs(fcsno),fcsno] GE pow_cs)
	    expind = expind(0)
	    exprange = [fcspow[expind-1,filtnr,trs(fcsno),fcsno], $
				fcspow[expind,filtnr,trs(fcsno),fcsno]]
            inbrange = [optpow(expind-1),optpow(expind)]
	   ENDELSE
	   
	   val_lim = $
		 [val_limits(0,filtnr,fcsno),val_limits(1,filtnr,fcsno)]

	   IF pow_cs LT val_lim(0) OR pow_cs GT val_lim(1) THEN BEGIN
   	          out_of_limits(fcsno) = 1
		  decode_admn, tmpdata.admn, dec_str
; Default responsivity to be taken if one of the points completely invalid
; CHANGED for PIA 10.0 : soft limit == LIMIT
;		  IF exprange(0) LE 0. OR exprange(1) GE 99. THEN $
;		    out_of_limits(fcsno) = 1
;		  IF out_of_limits(fcsno) THEN $
;		   special_text = $
;			['FCS el.power ('+STRTRIM(pow_cs,2)+'mW) is '+$
;			  'outside the "hard" limits of the FCS table.', $
;			 'Value used is invalid.'] ELSE $
		   special_text = $
			['FCS el.power ('+STRTRIM(pow_cs,2)+'mW) is '+$
			 'outside the limits of the FCS table.', $
 			 'Value derived is an extrapolation of measured points']
 
		  text_warn = [ $
 		   'WARNING:  using '+dec_str+ ' (FCS ' + STRTRIM(FCSNO+1,2) + $
		   ') by Responsivity calculation', $
 		   ' ', special_text, $
 		  'Actual limits for FCS '+STRTRIM(fcsno+1,2) +  $
		  ', filter '+STRTRIM(filtnr+1,2) + ' are ['+$
		  STRTRIM(val_lim(0),2)+','+STRTRIM(val_lim(1),2)+']mW']
		  INFO = text_warn
		  

	   ENDIF   ; end of out of limits checking

	   exprange = ALOG10(exprange)
	   inbrange = ALOG10(inbrange)
	   
	   exp_in_band = 10.^(INTERPOL(inbrange, exprange, alog10(pow_cs)))
				
; Multiply by aperture (dummy if not P)
	   theo_in_band(count) = exp_in_band * apert
	
; Divide by illumination matrix to correct for non-flat FCS illumination
; and divide by external flatfield
	   oldch = avch(*,count)
	   avch(*,count) = avch(*,count) / fcsill(*,fcsno) / flatf 	   
           avcu(*,count) = avch(*,count) * $
			   sqrt ( (avcu(*,count)   / oldch         )^2 )
			   
;			   		+$
;				  (fcsilun(*,fcsno)/fcsill(*,fcsno))^2 +$
;				  (flatu / flatf)^2 )

	   oldch = mech(*,count)
	   mech(*,count) = mech(*,count) / fcsill(*,fcsno) / flatf 	   
           mecu(*,count) = mech(*,count) * $
			   sqrt ( (mecu(*,count)   / oldch         )^2 )
			   
;			   		+$
;				  (fcsilun(*,fcsno)/fcsill(*,fcsno))^2 +$
;				  (flatu / flatf)^2 )

; Calculate responses

	   respon_av(*,count) = avch(*,count) / theo_in_band(count)  
	   respon_me(*,count) = mech(*,count) / theo_in_band(count)  

	   respon_au(*,count) = avcu(*,count) / theo_in_band(count) 
	   respon_mu(*,count) = mecu(*,count) / theo_in_band(count)

	   respon_sy(*,count) = respon_sy(*,count) + $
	   		SQRT ( (fcsilun(*,fcsno)/fcsill(*,fcsno))^2 +$
				  (flatu / flatf)^2 )
	ENDFOR
	
; Get the identification of calibrator measurement

	CASE tmpdata.det of 
		'P1': BEGIN
				r_origin = [tmpdata.admn,p1filters(filtnr),'']
		END   
		'P2': BEGIN
				r_origin = [tmpdata.admn,p2filters(filtnr),'']
		END    
		'P3': BEGIN
				r_origin = [tmpdata.admn,p3filters(filtnr),'']
		END    
		'C1': BEGIN
				r_origin = [tmpdata.admn,c1filters(filtnr),'']
		END    
		'C2': BEGIN
				r_origin = [tmpdata.admn,c2filters(filtnr),'']
		END 
		ELSE:   
         ENDCASE


	label = labtxt
	nfcs = maxpos - minpos + 1 
	
	respons = {aver:respon_av, avun:respon_au, med:respon_me, $
	            medu:respon_mu, sysun:respon_sy, resp_origin:r_origin,          $
		    defv:thresp, defunc:thresu}

	RETURN
END
