;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
PRO GET_RODISC, TMPDATA, RO_DISC, ENCODE_FLAG = ENCODE_FLAG, DR=dr, OTF=otf

;+
; NAME:
;	GET_RODISC
;
; PURPOSE:
;	Get a flag array corresponding to the read-outs which are discarded
;	according to the actual variables in PHTACC 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       GET_RODISC, Tmpdata, Ro_disc
;
; INPUTS:
;       Tmpdata: A PHT-ERD temporary structure
;
; OUTPUTS:
;       Ro_disc: A flag array (same dimensions as phterd.conp) with flag=1
;		 for all discarded signals
;
; KEYWORDS:
;	ENCODE_FLAG: select if tmpdata.flag is changed with the flag encoded
;
; COMMON BLOCKS:
;	phtacc
;	phtselndr
;
; PROCEDURES USED:
;	None
;
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel
;	Oct 95 Version 3.5
;	Last ramp bug fixed (CG)		December 1995
;	allowing for gen criteria and 1st NDR deselected (CG)	May 1996
;	Changed according to bitflag system (CG)		May 1999
;	V7.4
;
;-
@phtacc
@phtselndr

; Check number of parameters


        ON_ERROR, 2

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  GET_RODISC, tmpdata, rodisc'

	nndr = tmpdata.nndr + 1

;  Use number of read-outs dependent selection (PHTSELNDR)

	IF nselndr THEN BEGIN 

	   IF N_ELEMENTS(pp1sndr) EQ 0 THEN read_selndr

	   ind = FIX( alog(nndr) / alog(2) ) - 1

	   CASE STRMID(tmpdata.admn,0,2) of
	     'P1': var = PP1SNDR 
	     'P2': var = PP2SNDR 
	     'P3': var = PP3SNDR 
	     'C1': var = PC1SNDR 
	     'C2': var = PC2SNDR 
	     'SS': var = PSSSNDR 
	     'SL': var = PSLSNDR 
	   ENDCASE

           IF var(ind) GT 0 THEN BEGIN
                ndrnacc = INDGEN(var(ind)) + 1
           ENDIF ELSE BEGIN
                ndrnacc = 0
           ENDELSE

	ENDIF

	ro_disc=fix(tmpdata.conp*0.)
	dim2 = N_ELEMENTS(ro_disc(0,*))

;  Write string for not accepted selection criteria             
        restxt = ''

	IF nndr LT N_ELEMENTS(ndrnacc) THEN BEGIN
	   restxt = 'Read-outs per ramp < discarded:  selection canceled'
	   ndrnacc = 0
	ENDIF  	

	minr = minr(0)
	maxr = maxr(0)

	IF minr GE ((dim2-1)/nndr) THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	     'Last ramp < first ramp to be taken:  selection canceled'
	   minr = 0
	ENDIF  	

	IF maxr GE ((dim2-1)/nndr) THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	     'Last ramp < last ramp to be taken:  selection canceled'
	   maxr = 0
	ENDIF  	

	indwr = where (anyr GE dim2, nwr)

	IF nwr NE 0 THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	'>= 1 individual read-out to be discarded out of range:  selection canceled'
	   anyr = 0
	ENDIF  	


;check  that DR and OTF are defined
        IF N_ELEMENTS(dr) EQ 0 OR N_ELEMENTS(otf) EQ 0 THEN $
	   DECODE_PIXF, tmpdata.pixf, suspect, chopon, chops, dr, otf

;Fill ro_disc with discarded signals

        drist = WHERE(dr EQ 1, n)
        IF drnacc EQ 1 AND n GT 0 THEN ro_disc(*,drist) = 1

	IF ndr1nac eq 1 AND n GT 0 then begin
		dummy = WHERE(drist - nndr+1 ge 0,count)
		IF count NE 0 THEN ro_disc(*,drist(dummy)-nndr+1) = 1
		IF N_ELEMENTS(ro_disc(0,*)) GT max(drist)+1 THEN $
				ro_disc(*,max(drist)+1) = 1
	ENDIF

	IF ndrnacc(0) ne 0 AND n GT 0 THEN BEGIN
		FOR j=0,n_elements(ndrnacc)-1 DO BEGIN
			dummy = WHERE(drist -nndr+1+ndrnacc(j) ge 0,count)
			IF count NE 0 THEN $
				ro_disc(*,drist(dummy)-nndr+ndrnacc(j)) = 1
			IF N_ELEMENTS(ro_disc(0,*)) GT max(drist)+ndrnacc(j) $
				THEN ro_disc(*,max(drist)+ndrnacc(j)) = 1 
		ENDFOR
	ENDIF

	if minr gt 0 then ro_disc(*,0:minr*nndr)=1 

	if maxr gt 0 then ro_disc(*,maxr*nndr:dim2-1)=1 

	IF anyr(0) NE 0 THEN ro_disc(*,anyr-1)=1  
	
	IF KEYWORD_SET(ENCODE_FLAG) THEN BEGIN
	   encode_piaflag, 'ERD', 'GENDISC', tmpdata.flag, tmpflag, ro_disc, $ 
	     DET=tmpdata.det,/ALLPIX
	   tmpdata.flag = tmpflag  
	ENDIF 	
	
	otf_disc = ro_disc * 0	
        otflow = WHERE(otf EQ !OTFLOW, n)
        IF n GT 0 THEN BEGIN
	   ro_disc(*,otflow) = 1
	   otf_disc(*,otflow) = 1
	ENDIF

	IF KEYWORD_SET(ENCODE_FLAG) THEN BEGIN
	   encode_piaflag, 'ERD', 'NOTF', tmpdata.flag, tmpflag, otf_disc, $ 
	     DET=tmpdata.det,/ALLPIX
	   tmpdata.flag = tmpflag  
	ENDIF 	
		
	IF restxt NE '' THEN $
		XDISPLAYFILE, "", $
		TITLE = "Warning ", $
		WIDTH = 60, $
		HEIGHT = 6, $
		TEXT = [restxt]

	RETURN
	END

