;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
PRO GET_SPDDISC, TMPDATA, SPD_DISC, ENCODE_FLAG=encode_flag

;+
; NAME:
;	GET_SPDDISC
;
; PURPOSE:
;	Get a flag array corresponding to the chopper plateaux which are 
;	discarded according to the actual variables in the common block
;	phtacc. 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       GET_SPDDISC, Tmpdata, Spd_disc
;
; INPUTS:
;       Tmpdata: A PHT-SPD or PHT-AAP temporary structure
;
; OUTPUTS:
;       Spd_disc: A flag array (same dimensions as tmpdata.flag) with flag=1
;		  for all discarded signals
;
; COMMON BLOCKS:
;	phtacc
;
; KEYWORD INPUT:
;	Encode_flag:	Put info on bitflag system
;
; PROCEDURES USED:
;	None
;
; MODIFICATION HISTORY:
;       Written by:	Carlos Gabriel (ESA/ESTEC-SAI)
;	Oct 95 Version 3.5
;	Bad flagged if tmpdata.FLAG > 1 (CG)		January 1996 
;	Bug for individual selection fixed (CG)		September 1997
;	V6.5
;	Extended for being used by AAP also (CG)	November 1997
;	V6.6
;	Bitflag system change (CG)			May 1999
;	V7.4
;-
@phtacc

; Check number of parameters


        ON_ERROR, 2

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  GET_SPDDISC, tmpdata, spddisc'

;  Copy (pseudo)-arrays into variables to avoid crash because of just one 
;  element in them
	flag = tmpdata.flag


; Fill spd_disc with discarded chopper plateaux

	nchp = tmpdata.dims
	dim1 = n_elements(flag(*,0))
	spd_disc = intarr(dim1,nchp)

; Check deselected chopper plateaux 
        restxt = ''
	
	IF spdnacc(0) GT 0 AND nchp LE N_ELEMENTS(spdnacc) THEN BEGIN
	   restxt = '# Chopper plateaux <= discarded:  selection canceled'
	   spdnacc = 0
	ENDIF  	

	minscp = minscp(0)
	maxscp = maxscp(0)

	IF minscp GE nchp THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	 'First Chopper Plateau to be taken > Last found:  selection canceled'
	   minscp = 0
	ENDIF  	

	IF maxscp GT nchp THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	 'Last Chopper Plateau to be taken > Last found:  selection canceled'
	   maxscp = 0
	ENDIF  	

	indwhr = WHERE(anyscp GT nchp, nl)

	IF nl NE 0 THEN BEGIN
	   restxt = string([byte(restxt),10b,13b])+$
	 '>= 1 chopper plateau to be discarded > Last found:  selection canceled'
	   anyscp = 0
	ENDIF  	
	

; now compute spd_disc
; Taken out corresponding to introduction of bitflag system
;	FOR i=0,dim1-1 DO BEGIN
;	  spddisc = WHERE(flag(i,*) GT 1, ncount)
;	  IF ncount gt 0 THEN spd_disc(i,spddisc) = 1
;	ENDFOR

	IF spdnacc(0) NE 0 THEN $
	   spd_disc(*,spdnacc-1) = 1


	IF minscp gt 0 THEN spd_disc(*,0:minscp-1) = 1
	IF maxscp gt 0 THEN spd_disc(*,maxscp-1:nchp-1) = 1
	IF anyscp(0) gt 0 THEN spd_disc(*,anyscp-1) = 1

	IF restxt NE '' THEN $
		XDISPLAYFILE, "", $
		TITLE = "Warning ", $
		WIDTH = 60, $
		HEIGHT = 6, $
		TEXT = [restxt]

; Decide if SPD or AAP
	names = tag_names(tmpdata)
	i = WHERE(names EQ 'MNBR',n)
	IF n GT 0 THEN BEGIN
	 IF KEYWORD_SET(encode_flag) THEN BEGIN
	  encode_piaflag, 'AAP', 'GENDISC', tmpdata.flag, tmpflag, $
	 	spd_disc, det=tmpdata.det
	  tmpdata.flag = tmpflag
	 ENDIF 
	ENDIF ELSE BEGIN
	 IF KEYWORD_SET(encode_flag) THEN BEGIN
	  encode_piaflag, 'SPD', 'GENDISC', tmpdata.flag, tmpflag, $
	 	spd_disc, det=tmpdata.det
	  tmpdata.flag = tmpflag
	 ENDIF 
	ENDELSE
	RETURN
	END



