;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	GLUE_AAP
;
; PURPOSE:
;	Concatenate two measurements on AAP level, accomodating the new
;	produced pseudo-measurement into the phtaap buffer
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	GLUE_AAP, aap1, aap2, aapps, mednorm=mednorm, error=error	
;
; INPUTS:
;	aap1		The first temporary AAP structure
;	
;	aap2		The second temporary AAP structure
;
; KEYWORD PARAMETERS:
;	MEDNORM		An array (same dimension as detector pixels in the
;			AAP structures) containing normalization values
;
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output arguments are not correct
;
; OUTPUTS:
;	AAPPS		The position within the AAP buffer of the concatenated
;			pseudo-measurement
;
; PROCEDURES USED: 
;	(none)
;
; COMMON BLOCKS:
;	(none)
;
; ASSUMPTIONS:
;	AAP1 and AAP2 are aap temporary structures corresponding to the
;	same detector
;
; MODIFICATION HISTORY:
;	Written by: 		
;	C. Gabriel (ESA-VILSPA SAI)			December 1997
;	V6.6
;	Including concatenation information on header,
;	 & avoiding long measurement names (CG)		July 1998
;	V7.2 
;	Extension for new elements DECP and RAP (CG)	March 2000
;	V8.2
;-
PRO GLUE_AAP, aap1, aap2, aapps, mednorm=mednorm, error=error

@phtaap

	error = 0
	
	PHTAAPT = {PHTAAP_TYPE}
        PHTAAPT.HDR = HANDLE_CREATE()
	PHTAAPT.TIME = HANDLE_CREATE()
	PHTAAPT.CSTP = HANDLE_CREATE()
	PHTAAPT.CPOS = HANDLE_CREATE()
	PHTAAPT.DWEL = HANDLE_CREATE()
	PHTAAPT.MNFL = HANDLE_CREATE()
	PHTAAPT.MNFU = HANDLE_CREATE()
	PHTAAPT.MNBR = HANDLE_CREATE()
	PHTAAPT.MNBU = HANDLE_CREATE()
	PHTAAPT.MDFL = HANDLE_CREATE()
	PHTAAPT.Q1FL = HANDLE_CREATE()
	PHTAAPT.Q3FL = HANDLE_CREATE()
	PHTAAPT.MDBR = HANDLE_CREATE()
	PHTAAPT.Q1BR = HANDLE_CREATE()
	PHTAAPT.Q3BR = HANDLE_CREATE()
	PHTAAPT.RPID = HANDLE_CREATE()
	PHTAAPT.FLAG = HANDLE_CREATE()
	PHTAAPT.DEC = HANDLE_CREATE()
	PHTAAPT.RA = HANDLE_CREATE()
	PHTAAPT.DECP = HANDLE_CREATE()
	PHTAAPT.RAP = HANDLE_CREATE()
	PHTAAPT.ROLL = HANDLE_CREATE()

	COMMENT = "Measurements concatenated: "+aap1.admn+" & "+aap2.admn 
	hdr = [COMMENT,aap1.hdr]
	res = FXPAR(hdr, 'PRA_CN*', count=count)
	FXADDPAR, hdr, 'PRA_CN' + STRTRIM(count+1,2), aap2.admn, $
	  ' Measurement concatenated', BEFORE='XTENSION'
	IF STRPOS(aap1.ADMN,'G') EQ -1 THEN $
	  PHTAAPT.ADMN = aap1.ADMN + 'G' ELSE $
	  PHTAAPT.ADMN = aap1.ADMN
	FXADDPAR, hdr, 'PIA_MNAM', phtaapt.admn, $
          "Name chosen for measurement by PIA user", BEFORE='XTENSION'

	HANDLE_VALUE, PHTAAPT.HDR, hdr, /set
	PHTAAPT.DET  = aap1.DET
	PHTAAPT.FILT = aap1.filt
	PHTAAPT.APER = aap1.aper
	PHTAAPT.POLZ = aap1.polz
	PHTAAPT.DIMS = aap1.dims + aap2.dims	
	PHTAAPT.CWVL = aap1.cwvl

	nele = N_ELEMENTS(aap1.time)
;	IF nele GT 1 THEN $
;	 diff_time = aap1.time(nele-1) - aap1.time(nele-2) ELSE diff_time=1.
	diff_time = 100.
	
	HANDLE_VALUE, PHTAAPT.TIME, $
	    [aap1.TIME,aap2.TIME + aap1.time(nele-1) + diff_time], /SET
	HANDLE_VALUE, PHTAAPT.CPOS, [aap1.cpos,aap2.cpos], /SET
	HANDLE_VALUE, PHTAAPT.CSTP, [aap1.cstp,aap2.cstp], /SET
	HANDLE_VALUE, PHTAAPT.DWEL, [aap1.dwel,aap2.dwel], /SET
	HANDLE_VALUE, PHTAAPT.DEC, [aap1.DEC,aap2.DEC], /SET
	HANDLE_VALUE, PHTAAPT.RA, [aap1.RA,aap2.RA], /SET
	HANDLE_VALUE, PHTAAPT.ROLL, [aap1.ROLL,aap2.ROLL], /SET
	
	dim1a = N_ELEMENTS(aap1.mnfl(*,0))
	dim2a = N_ELEMENTS(aap1.mnfl(0,*))
	dim1b = N_ELEMENTS(aap2.mnfl(*,0))
	dim2b = N_ELEMENTS(aap2.mnfl(0,*))
	IF dim1a NE dim1b THEN GOTO, errdim
	
	
	
	mnfl = FLTARR(dim1a,dim2a+dim2b)
	mnfu = mnfl
	mnbr = mnfl
	mnbu = mnfl
	mdfl = mnfl
	q1fl = mnfl
	q3fl = mnfl
	mdbr = mnfl
	q1br = mnfl
	q3br = mnfl
	decp = mnfl
	rap = mnfl
	flag = INTARR(dim1a,dim2a+dim2b)
	rpid = BYTARR(2,dim2a+dim2b)

	IF NOT KEYWORD_SET(mednorm) THEN mednorm=fltarr(dim1a)+1.	
	rpid(*,0:dim2a-1) = aap1.rpid 
	mnfl(*,0:dim2a-1) = aap1.mnfl
	mnfu(*,0:dim2a-1) = aap1.mnfu
	mnbr(*,0:dim2a-1) = aap1.mnbr
	mnbu(*,0:dim2a-1) = aap1.mnbu
	mdfl(*,0:dim2a-1) = aap1.mdfl
	q1fl(*,0:dim2a-1) = aap1.q1fl
	q3fl(*,0:dim2a-1) = aap1.q3fl
	mdbr(*,0:dim2a-1) = aap1.mdbr
	q1br(*,0:dim2a-1) = aap1.q1br
	q3br(*,0:dim2a-1) = aap1.q3br
	decp(*,0:dim2a-1) = aap1.decp
	rap(*,0:dim2a-1) = aap1.rap
	
	rpid(*,dim2a:*) = aap2.rpid
	FOR i = 0,dim1a-1 DO BEGIN
 		mnfl(i,dim2a:*) = aap2.mnfl(i,*)*mednorm(i)
		mnfu(i,dim2a:*) = aap2.mnfu(i,*)*mednorm(i)
		mnbr(i,dim2a:*) = aap2.mnbr(i,*)*mednorm(i)
		mnbu(i,dim2a:*) = aap2.mnbu(i,*)*mednorm(i)
		mdfl(i,dim2a:*) = aap2.mdfl(i,*)*mednorm(i)
		q1fl(i,dim2a:*) = aap2.q1fl(i,*)*mednorm(i)
		q3fl(i,dim2a:*) = aap2.q3fl(i,*)*mednorm(i)
		mdbr(i,dim2a:*) = aap2.mdbr(i,*)*mednorm(i)
		q1br(i,dim2a:*) = aap2.q1br(i,*)*mednorm(i)
		q3br(i,dim2a:*) = aap2.q3br(i,*)*mednorm(i)
		decp(i,dim2a:*) = aap2.decp(i,*)*mednorm(i)
		rap(i,dim2a:*) = aap2.rap(i,*)*mednorm(i)
	ENDFOR
	
	HANDLE_VALUE, PHTAAPT.RPID, RPID, /SET
        HANDLE_VALUE, PHTAAPT.MNFL, mnfl, /SET 
	HANDLE_VALUE, PHTAAPT.MNFU, mnfu, /SET 
	HANDLE_VALUE, PHTAAPT.MNBR, mnbr, /SET 
	HANDLE_VALUE, PHTAAPT.MNBU, mnbu, /SET
	HANDLE_VALUE, PHTAAPT.MDFL, mdfl, /SET 
	HANDLE_VALUE, PHTAAPT.Q1FL, q1fl, /SET 
	HANDLE_VALUE, PHTAAPT.Q3FL, q3fl, /SET 
	HANDLE_VALUE, PHTAAPT.MDBR, mdbr, /SET 
	HANDLE_VALUE, PHTAAPT.Q1BR, q1br, /SET 
	HANDLE_VALUE, PHTAAPT.Q3BR, q3br, /SET 
	HANDLE_VALUE, PHTAAPT.FLAG, flag, /SET
	HANDLE_VALUE, PHTAAPT.DECP, decp, /SET
	HANDLE_VALUE, PHTAAPT.RAP, rap, /SET
	
; Update the PHTAAP structure using the temporary structure
	PHTAAP = [PHTAAP, PHTAAPT]
; Update the PHTSPD value in the administration file using the index
	AAPPS = N_ELEMENTS(PHTAAP) - 1
	RETURN

ERRDIM: ERROR=1
	PRINT, STRING(7B) + ' concatenation is not possible '
	RETURN
	END
	
		
	
	

	 	
