;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_BASE_DRIFT
; PURPOSE:
;	Widget routine for graphical establishment of baseline drift factors,
;	as observed from long term responsivity changes.
; CATEGORY:
;	PIA - graphics
;                
; CALLING SEQUENCE: 
;	PIA_base_drift, tmpdata, bas_drift, pixel=pixel
;
; INPUT PARAMETERS:
;	TMPDATA	: Structure containing the SCP data to be processed
;
; OPTIONAL KEYWORD :
;	PIXEL	: indicating the detector pixel to start with
;
; OUTPUT:
;	bas_drift: Baseline drift factors, normalized to the last point, with
;		   same dimensions as tmpdata.aver
;
; RESTRICTIONS:
;	tmpdata must be a valid phtscp temporary data structure
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;		colour_indices
;		pia_dspchz_cmn
;       PIA Procedure(s):
;		pia_disp_print_file
;		pia_graph_extract
;		pia_xplot
;		
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel (ESA/VILSPA-SAI)			July 1997
;	V6.4
;-
PRO PIA_base_drift_event, event
@colour_indices
COMMON pia_dspchz_cmn, zoom_psymch, zoom_linech, zoom_color

	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=idsa_handle
	handle_value, idsa_handle, idsa 
	CASE action OF
	  'ALLPIX': BEGIN
	     npix = N_ELEMENTS(idsa.interp(*,0))
	     FOR I=0,npix-1 DO $
	      idsa.interp(I,*) = idsa.interp(idsa.pix,*)
	     handle_value, idsa_handle, idsa, /SET
	     WIDGET_CONTROL, event.top, SET_UVALUE=idsa_handle
	  END

	  'EXT': BEGIN
		WIDGET_CONTROL, idsa.id_pix, GET_VALUE=pix
		idsa.pix = pix(0)-1
		
		PIA_GRAPH_EXTRACT, idsa.x, idsa.data(idsa.pix,*), interp, $
			error=error, GROUP_LEADER=event.top
		IF error EQ 1 THEN BEGIN
		   PRINT, 'Graph extraction failed, returning'
		   RETURN
		ENDIF
	     
	        nlast = N_ELEMENTS(interp)-1
		idsa.interp(idsa.pix,*) = interp / interp(nlast)
		WIDGET_CONTROL, idsa.id_plot, /sensitive
		WIDGET_CONTROL, idsa.id_accept, /sensitive
		WIDGET_CONTROL, idsa.id_all, /sensitive
	     handle_value, idsa_handle, idsa, /SET
	     WIDGET_CONTROL, event.top, SET_UVALUE=idsa_handle
	  END
	  
	  'SHOW': BEGIN
	  	text = ['Results of graphical baseline drift handling', ' ']
	        npix = N_ELEMENTS(idsa.interp(*,0))
	        FOR I=0,npix-1 DO $
	          text = [text, 'Pixel '+STRTRIM(i+1,2), $
	            STRING(idsa.interp(i,*))]
	        PIA_DISP_PRINT_FILE, '',Title='Baseline drift handling', $
	        	TEXT=text
	  END
	            
	  	 
	  'PLOT': BEGIN
		WIDGET_CONTROL, idsa.id_pix, GET_VALUE=pix
		idsa.pix = pix(0)-1
		
	  	text = 'Results of graphical baseline drift handling'
		zoom_psymch = 1
		zoom_color = -1
		PIA_XPLOT, idsa.x, idsa.interp(idsa.pix,*), PSYM=zoom_psymch, $
		   color = white, title=text, /MODAL, GROUP=event.top 
		   
	  END
	            
	  'OPLOT': BEGIN
		WIDGET_CONTROL, idsa.id_pix, GET_VALUE=pix
		idsa.pix = pix(0)-1
		
		IF zoom_color EQ -1 THEN zoom_color = green ELSE $
		  zoom_color = zoom_color + 1
		IF zoom_color EQ 9 then zoom_color = 4
		  zoom_psymch = zoom_psymch + 1
		IF zoom_psymch GE 8 then zoom_psymch = 1

		PIA_XPLOT, idsa.x, idsa.interp(idsa.pix,*), PSYM=zoom_psymch, $
		     color=zoom_color, overplot=1, /MODAL
		   
	  END

	  'CANCEL': BEGIN
	  	idsa.interp = idsa.interp * 0. + 1.
	        handle_value, idsa_handle, idsa, /SET
	  	WIDGET_CONTROL, event.top, /DESTROY
	  END
	  
	  'ACCEPT': WIDGET_CONTROL, event.top, /DESTROY
	  
	  ELSE:
	  
	ENDCASE
	
	
	RETURN
	END
	

PRO PIA_base_drift, tmpdata, bas_drift, pixel=pixel, GROUP_LEADER=group_leader

	IF not KEYWORD_SET(pixel) THEN pixel = 0
	det_pix = N_ELEMENTS(tmpdata.aver(*,0)) 
	IF det_pix GT 1 THEN det_pix = det_pix-2
	IF det_pix GT 40 THEN det_pix = det_pix-2
	
	data = tmpdata.aver(0:det_pix-1,*)
	unc = tmpdata.avun(0:det_pix-1,*)
	x = tmpdata.time
	
	IF not KEYWORD_SET(group_leader) THEN group_leader = !pia_top_wid

	base = WIDGET_BASE( TITLE = 'PIA graphical baseline drift correction', $
			/COLUMN, /MODAL, GROUP_LEADER=group_leader )

	rbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(rbase, VALUE='Pixel to be used:     ')
	id_pix = WIDGET_TEXT(rbase, VALUE=STRTRIM(pixel+1,2), UVALUE='JUNK', $
				XSIZE = 6, /EDITABLE)
	
	rbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(rbase, VALUE='Extend to all pixels: ')
	id_all = WIDGET_BUTTON(rbase, VALUE= ' Now ', UVALUE='ALLPIX')
	
	rbase = WIDGET_BASE(base, /ROW, /FRAME)
	id_ext = WIDGET_BUTTON(rbase,    VALUE='   Extract   ', UVALUE='EXT')
	id_print = 0
;	id_print = WIDGET_BUTTON(rbase,  VALUE='Print Results', UVALUE='SHOW')
	id_plot =  WIDGET_BUTTON(rbase,  VALUE='Plot Results ', /MENU)
	id_plnew = WIDGET_BUTTON(id_plot,VALUE='Plot ', UVALUE='PLOT')
	id_oplot = WIDGET_BUTTON(id_plot,VALUE='Overplot ', UVALUE='OPLOT')
;	rbase = WIDGET_BASE(base, /ROW, /FRAME)
	id_cancel = WIDGET_BUTTON(rbase, VALUE='   Cancel    ', UVALUE='CANCEL')
	id_accept = WIDGET_BUTTON(rbase, VALUE='Accept & Quit', UVALUE='ACCEPT')

;	WIDGET_CONTROL, id_print, sensitive=0
	WIDGET_CONTROL, id_plot, sensitive=0
	WIDGET_CONTROL, id_accept, sensitive=0
	WIDGET_CONTROL, id_all, sensitive=0
	
	interp = data * 0. + 1.
	
	idsa = {x:x, data:data, unc:unc, interp:interp, id_pix:id_pix, $
		pix:pixel, id_show:id_print, id_plot:id_plot, id_all:id_all, $
		id_accept:id_accept}
	 
	idsa_handle = handle_create()
	handle_value, idsa_handle, idsa, /SET, /NO_COPY 
	
	WIDGET_CONTROL, base, /realize, SET_UVALUE=idsa_handle
	
	xMANAGER,No_BLOCK=1, "pia_base_drift", base

	handle_value, idsa_handle, idsa, /NO_COPY 
	bas_drift = idsa.interp
	
	RETURN
	END	
	
