;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_CONVOLVE
;
; PURPOSE:
;	Convolves a map (using a gaussian) to a different resolution
;
; CATEGORY:
;	PIA - graphics/processing
;
; CALLING SEQUENCE:
;	PIA_CONVOLVE, imap, imappar, imaphdr, scan=scan
;
; INPUTS:
;	IMAP		The two dimensional map of the input image
;
;	IMAPPAR		The map parameter structure  as used in PIA
;
;	IMAPHDR		The header of the measurement
;
; KEYWORD PARAMETERS:
;	GROUP		The widget group leader
;
; OUTPUTS:
;	(none)
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_mapdraw
;		phtfilter_init
;	Astronomical Library:
;		psf_gaussian
;	Internal:
;		pia_convolve_event
;
; COMMON BLOCKS:
;	PIA_convolv_cmn		internal
;	phtfilter		filter id's
;	phtfilter_prop		filter properties
;
; MODIFICATION HISTORY:
;	Written by:		C. Gabriel, ESA/VILSPA		June 1996
;	Convolve (astrolib) replaced by convol (IDL) because of
;	 edges problems +
;	 taking maximal dimensions for psf up to 15 (CG)	October 1997
;	V6.6
;-
PRO PIA_CONVOLVE_EVENT, event

@phtfilter
@phtfilter_prop
COMMON pia_convol_cmn, filts, airyd, filtin_id, filtout_id, thetac_id, $
			use_scan, scan_id


	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue

	i = WHERE(filts EQ uvalue, n)
	i = i(0)
	IF n NE 0 THEN uvalue = 'FILT'

	CASE uvalue of

	   'FILT': BEGIN
	        airyd_out = airyd(i)
	        WIDGET_CONTROL, filtin_id, GET_VALUE=filtin
	        iin = WHERE(filts EQ filtin(0),n)
	        IF n EQ 0 THEN RETURN
	        airyd_in = airyd(iin(0))
		IF airyd_out LE airyd_in THEN BEGIN
			XDISPLAYFILE, "", $
                    		TITLE = "Input not accepted", $
                    		WIDTH = 50, $
                    		HEIGHT = 4, $
                    	  TEXT = [" Can only convolve to larger wavelengths", $]
                          	  "     ", $ ]
                          	  " Fix it and try again, please "]
                        RETURN
                ENDIF
	        thetac = SQRT((airyd_out/2)^2 - (airyd_in/2)^2)
	        WIDGET_CONTROL, filtout_id, SET_VALUE=filts(i)
		WIDGET_CONTROL, thetac_id, SET_VALUE=STRING(thetac)
	   END
	   
	   'OK': BEGIN
		WIDGET_CONTROL, event.top, /hourglass
	   	WIDGET_CONTROL, event.top, GET_UVALUE=hsave
		HANDLE_VALUE, hsave, save
		WIDGET_CONTROL, thetac_id, GET_VALUE=thetac
		thetac = FLOAT(thetac(0))
		WIDGET_CONTROL, filtout_id, GET_VALUE=filtout

		
		IF use_scan EQ 0 THEN BEGIN
		 fwhm = thetac / save.imappar.pixsize
		 npixel = [N_ELEMENTS(save.imap(*,0)),N_ELEMENTS(save.imap(0,*))]
		 npixel = npixel + (npixel mod 2) - 3  < [11,11]
		 IF min(npixel) LT 3 THEN BEGIN
		    PRINT, STRING(7B), 'Dimension too low, returning'
		    RETURN
		 ENDIF 
		 
 		 psf = psf_gaussian(npixel=npixel,FWHM=fwhm, /NORMAL)
  		 
 		 imconv = convol(save.imap, psf, /EDGE_WRAP)
 		 posi = strpos(save.imappar.admn_decoded, 'micron')
 		 totlen = STRLEN(save.imappar.admn_decoded)
 		 headline = $
 		  STRMID(save.imappar.admn_decoded, 0, posi(0)+7) + $
 		  ' Convolved to ' + filtout
 		 headline = headline(0)
 		 save.imappar.admn_decoded = headline
 		 pia_mapdraw, imconv, save.imappar.trans, save.imappar, $
 			save.imaphdr, 0, headline=headline

 		ENDIF ELSE BEGIN

		 fwhm = thetac / save.imappar.pixsize(0)
		 npixel = N_ELEMENTS(save.imap(*,0))
		 IF npixel LT 5 THEN BEGIN
		    PRINT, STRING(7B), 'Dimension too low, returning'
		    RETURN
		 ENDIF 
		 npixel = npixel + (npixel mod 2) - 3 < 11
 		 psf = psf_gaussian(npixel=npixel,NDIMEN=1,FWHM=fwhm, /NORMAL)
		 WIDGET_CONTROL, scan_id, GET_VALUE=row
		 row = FIX(row) - 1
		 scanconv = convol(save.imap(*,row),psf, /edge_wrap)
		 imconv = save.imap * 0.
		 imconv(*,row) = scanconv
 		 posi = strpos(save.imappar.admn_decoded, 'micron')
 		 totlen = STRLEN(save.imappar.admn_decoded)
 		 headline = $
 		  STRMID(save.imappar.admn_decoded, 0, posi(0)+7) + $
 		  ' Scan convolved to ' + filtout
 		 headline = headline(0)
 		 save.imappar.admn_decoded = headline
 		 pia_mapdraw, imconv, save.imappar.trans, save.imappar, $
 			save.imaphdr, 0, headline=headline
 		ENDELSE
		 		 
 		 
 	  END
 	  
 	  'QUIT': WIDGET_CONTROL, event.top, /DESTROY
 	  
 	  ELSE:
 	  
 	ENDCASE
 	  
 	RETURN
 	END	
		
;;;;;;;;;;;;;;;               The menu routine      ;;;;;;;;;;;;;;;;;;;;;;	
	     

PRO PIA_CONVOLVE, imap, imappar, imaphdr, scan=scan

@phtfilter
@phtfilter_prop
COMMON pia_convol_cmn, filts, airyd, filtin_id, filtout_id, thetac_id, $
			use_scan, scan_id

	IF N_ELEMENTS(c1filters) EQ 0 THEN PHTFILTER_INIT
	IF KEYWORD_SET(scan) THEN use_scan=scan ELSE use_scan=0

	filts = [p1filters, p2filters, p3filters, c1filters, c2filters]
	airyd = [filtprop.p1airyd, filtprop.p2airyd, filtprop.p3airyd, $
		 filtprop.c1airyd, filtprop.c2airyd]
	
	base = WIDGET_BASE(TITLE="PIA map/scan convolution menu", /COLUMN)
	
	parbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(parbase, VALUE='PHT-filter of origin:')
	filtin_id = WIDGET_TEXT(parbase, VALUE=imappar.filter, $
		UVALUE='JUNK')
		
	parbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(parbase, VALUE='PHT-filter to be convolved to:')
	filtout_id = WIDGET_BUTTON(parbase, VALUE = ' Choose ', /MENU)
	FOR I=0,25 DO id_0 = WIDGET_BUTTON(filtout_id, $
		VALUE=filts(i),UVALUE=filts(i))
		
	parbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(parbase, $
		VALUE='Theta convolution:  ')
	thetac_id = WIDGET_TEXT(parbase, /editable, VALUE='    ', $
		UVALUE='JUNK')
	
	parbase = WIDGET_BASE(base, /ROW)
	label = WIDGET_LABEL(parbase, $
		VALUE='Scan of row:      ')
	IF KEYWORD_SET(scan) THEN $
		scan_val=STRING((N_ELEMENTS(imap(0,*))+1)/2) ELSE $
				  scan_val='     '
	scan_id = WIDGET_TEXT(parbase, /editable, VALUE=scan_val, $
		UVALUE='JUNK', XSIZE=15)
	IF use_scan EQ 0 THEN WIDGET_CONTROL, scan_id, SENSITIVE=0
	
	cont_base = WIDGET_BASE(base, /ROW)
	
	test_id = WIDGET_BUTTON(cont_base, VALUE='   OK  ', UVALUE='OK')
	
	quit_id = WIDGET_BUTTON(cont_base, VALUE= '   Quit  ', $
		UVALUE= 'QUIT')


	save = {imap:imap, imappar:imappar, imaphdr:imaphdr}
	
	hsave = HANDLE_CREATE()
	HANDLE_VALUE, hsave, save, /SET, /NO_COPY

        WIDGET_CONTROL, base, /REALIZE, TLB_SET_XOFFSET=150, $
                        TLB_SET_YOFFSET=150, SET_UVALUE=hsave

        XManager, "pia_convolve", base, GROUP=group
	
        RETURN
END
	
