;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_DEFAPER
;
; PURPOSE:
;	Define aperture constants (ISO,PHT-P) and pixel sizes (PHT-C/PHT-S)
;	and store them in a common block.
;
; CATEGORY:
;          PIA - initialization
;
; CALLING SEQUENCE:
;	PIA_DEFAPER
;
; COMMON BLOCKS:
;	piacmn_pow2fb	To hold the aperture values
;
; MODIFICATION HISTORY:
;	Written by: D. Skaley, MPIK/Heidelberg		Feb-95
;		     Took values from SPG procedure 'AAP_APER_DIM' (S.Guest)
;	Modified by: cg					Jul-95
;	Oct 95 Version 3.5
;			
;-
;*********************************************************************
;*
;*********************************************************************
PRO PIA_DEFAPER

; common blocks:

@piacmn_pow2fb

; define the aperture arrays (members of common block PIACMN_POW2FB)
; aper_m units are meters (meter^2), aper_r units are radians (sr)
; aper_m/_r(0,*) are the circular or rectangular 1st dim values
; aper_m/_r(1,*) are 0.0 for circular or 2nd dim rectangualar values
; aper_m/_r(2,*) are the area values

	aper_m = fltarr(3,18)
	aper_r = fltarr(3,18)

; Size of main ISO aperture (diameter in mm and arcsec)

	aper_m(*,0) = 	[ 600.0,     0.0, 0.0 ]
	aper_r(*,0) = 	[ 2231.5497, 0.0, 0.0 ]

; PHT-P aperture dimensions (in mm)

	aper_m(*,1:14) = [	[ 2.30, 0.0 , 0.0 ], $
				[ 3.40, 0.0 , 0.0 ], $
				[ 4.30, 0.0 , 0.0 ], $
				[ 5.20, 0.0 , 0.0 ], $
				[ 5.50, 5.50, 0.0 ], $ ; rectangular
				[ 7.85, 0.0 , 0.0 ], $
				[ 7.85, 0.0 , 0.0 ], $
				[ 0.22, 0.0 , 0.0 ], $
				[ 0.33, 0.0 , 0.0 ], $
				[ 0.44, 0.0 , 0.0 ], $
				[ 0.60, 0.0 , 0.0 ], $
				[ 0.77, 0.0 , 0.0 ], $
				[ 0.87, 1.40, 0.0 ], $ ; rectangular
				[ 1.00, 0.0 , 0.0 ] ]

; PHT-P aperture dimensions (in arcsec)

	aper_r(*,1:14) = [	[ 52.0,  0.0  , 0.0 ], $
				[ 79.0,  0.0  , 0.0 ], $
				[ 99.0,  0.0  , 0.0 ], $
				[ 120.0, 0.0  , 0.0 ], $
				[ 127.0, 127.0, 0.0 ], $ ; rectangular
				[ 180.0, 0.0  , 0.0 ], $
				[ 180.0, 0.0  , 0.0 ], $
				[ 5.0,   0.0  , 0.0 ], $
				[ 7.6,   0.0  , 0.0 ], $
				[ 10.0,  0.0  , 0.0 ], $
				[ 13.8,  0.0  , 0.0 ], $
				[ 18.0,  0.0  , 0.0 ], $
				[ 20.0,  32.0 , 0.0 ], $ ; rectangular
				[ 23.0,  0.0  , 0.0 ] ]

; Size of (square) pixels for PHT-C (in mm and arcsec)

	aper_m(*,15:16) = [	[ 1.9, 1.9, 0.0 ], $   ; C100
				[ 3.9, 3.9, 0.0 ] ]    ; C200

	aper_r(*,15:16) = [	[ 43.5, 43.5, 0.0 ], $  ; C100
				[ 89.4, 89.4, 0.0 ] ]   ; C200

; Size of (square) aperture for PHT-S (in mm and arcsec)

	aper_m(*,17) = 	[  1.043,  1.043, 0.0 ]	; S
	aper_r(*,17) = 	[ 24.0,   24.0,   0.0 ]	; S

; transform mm -> m and arcsec -> rad

	aper_m = aper_m * 1.E-3
	aper_r = aper_r * (!pi/(180.*3600.))

; calculate the area...
; ...first the rectangualar:

	ix = WHERE( aper_m(1,*) NE 0. )
	aper_m(2,ix) = aper_m(0,ix)*aper_m(1,ix)
	aper_r(2,ix) = aper_r(0,ix)*aper_r(1,ix)

; ...then the circular:

	ix = WHERE( aper_m(1,*) EQ 0. )
	aper_m(2,ix) = (aper_m(0,ix)/2.0)^2 * !pi
	aper_r(2,ix) = (aper_r(0,ix)/2.0)^2 * !pi

	RETURN
	END
