;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_DISP_EOHA
;
; PURPOSE:
;	Read the Edited Observation History (EOHA) from FITSDIR and 
;	display/print contents for a given TDT
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	PIA_DISP_EOHA, tdtnr
;
; INPUTS:
;	tdtnr:		TDT number for which OLP report shall be displayed
;
; KEYWORD PARAMETERS:
;	ERROR : 	returns an error condition:
;			0 = no error
;			1 = could not successfully read the EOHA file
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_disp_print_file
;
; MODIFICATION HISTORY:
;	Written by: 	I. Heinrichsen, MPIK/Heidelberg, Mai 1997
;-

PRO PIA_DISP_EOHA, tdtnr, ERROR=error

; initialize error condition

error = 1

; define the filename
IF STRLEN(tdtnr) EQ 8 THEN tdtnr = STRMID(tdtnr,0,6)

if strnumber(tdtnr,val) then begin
    tdt=string(fix(val/1000))
    filename = strcompress(!fitdir+'EOHA'+tdt+'.FITS', /REMOVE_ALL)
endif else begin
    goto, no_eoha
endelse


; open the EOHA file

OPENR, lun, /GET_LUN, filename, /BLOCK, ERROR=err
IF err NE 0 THEN goto, no_eoha
FREE_LUN, lun
fxbopen, unit, filename, 1, header

; read the FITS data

fxbread, unit, EOHATDTN, 'EOHATDTN'

row=where(EOHATDTN eq tdtnr)
if row(0) eq -1 then begin
    goto, no_tdt
endif

row=row(0)+1

fxbread, unit,EOHAUTCS,'EOHAUTCS', row
fxbread, unit,EOHAUTCE,'EOHAUTCE', row
fxbread, unit,EOHAINST,'EOHAINST', row
fxbread, unit,EOHANCPU,'EOHANCPU', row
fxbread, unit,EOHARES,'EOHARES', row
fxbread, unit,EOHAAOTN,'EOHAAOTN', row
fxbread, unit,EOHAPRID,'EOHAPRID', row
fxbread, unit,EOHAPLID,'EOHAPLID', row
fxbread, unit,EOHAOSN,'EOHAOSN', row
fxbread, unit,EOHAPSN,'EOHAPSN', row
fxbread, unit,EOHAPCAT,'EOHAPCAT', row
fxbread, unit,EOHACIND,'EOHACIND', row
fxbread, unit,EOHATNAM,'EOHATNAM', row
fxbread, unit,EOHASCAT,'EOHASCAT', row
fxbread, unit,EOHALIND,'EOHALIND', row
fxbread, unit,EOHASPAR,'EOHASPAR', row
fxbread, unit,EOHAAOTV,'EOHAAOTV', row

; Display the result

yname="EOHA Information"+tdtnr

text="EOHAUTCS: "+EOHAUTCS+ $
  "      / UTC start of observation (yydddhhmmss)"
text=[text,"EOHAUTCE: "+EOHAUTCE+ $
      "      / UTC end of observation (yydddhhmmss)"]
text=[text,"EOHAINST: "+EOHAINST+ $
      "                / Prime instrument ID"]
text=[text,"EOHAAOTN: "+EOHAAOTN+ $
      "             / AOT name"]
text=[text,"EOHAPRID: "+EOHAPRID $
      +"         / Proposer ID "]
text=[text,"EOHAPLID: "+EOHAPLID $
      +"         / Proposal ID "]
text=[text,"EOHAOSN : "+EOHAOSN+ $
      "               / Observation Sequence Number (OSN)"]
text=[text,"EOHAPCAT: "+EOHAPCAT+ $
      "                / Proposal category"]
text=[text,"EOHACIND: "+EOHACIND+ $
      "                / Calibration indicator"]
text=[text,"EOHATNAM: "+EOHATNAM+ $
      " / Target name"]
text=[text,"EOHASCAT: "+EOHASCAT+ $
      "              / Scientific category"]
text=[text,"EOHALIND: "+EOHALIND+ $
      "               / Link indicator"]

text=[text," "]
text=[text,"EOHAAOTV: "]

; decode AOTVAR entries, only valid for PHT

if EOHAINST eq 'P' then begin
    text=[text," "]
    text=[text,"   Number of Points per Raster Line  : "+strmid(EOHAAOTV,0,2)]
    text=[text,"   Number of Raster Lines            : "+strmid(EOHAAOTV,2,2)]
    text=[text,"   AOT Duration (s)                  : "+strmid(EOHAAOTV,4,5)]
    text=[text,"   AOT Processing Time (yydddhhmmss) : "+strmid(EOHAAOTV,9,11)]
    text=[text,"   Number of measurements in AOT     : "+strmid(EOHAAOTV,32,5)]
    text=[text,"   Number of sub-observations in AOT : "+strmid(EOHAAOTV,37,1)]
    text=[text,"   Calibration Sequence of AOT       : "+strmid(EOHAAOTV,38,1)]
endif else begin
    text=[text,EOHAAOTV] 
endelse

pia_disp_print_file, 'EOHA'+tdtnr, title=yname, text=text

; normal exit
error = 0
fxbclose, unit
RETURN

; error conditions:
no_eoha: 
MESSAGE, /INFO, "EOHA file not available: "+filename
RETURN

no_tdt: 
MESSAGE, /INFO, "TDT number: "+tdtnr+" does not exist in EOHA file: "+filename
RETURN
END




