;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_DISP_EOHI
;
; PURPOSE:
;	Read the Edited Observation History (EOHI) from FITSDIR and 
;	display/print contents for a given TDT
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	PIA_DISP_EOHI, tdtnr
;
; INPUTS:
;	tdtnr:		TDT number for which OLP report shall be displayed
;
; KEYWORD PARAMETERS:
;	ERROR : 	returns an error condition:
;			0 = no error
;			1 = could not successfully read the EOHA file
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_disp_print_file
;
; MODIFICATION HISTORY:
;	Written by: 	I. Heinrichsen, MPIK/Heidelberg, Mai 1997
;-

PRO PIA_DISP_EOHI, tdtnr, ERROR=error

; initialize error condition

error = 1

; define the filename
IF STRLEN(tdtnr) EQ 8 THEN tdtnr = STRMID(tdtnr,0,6)

if strnumber(tdtnr,val) then begin
    tdt=string(fix(val/1000))
    filename = strcompress(!fitdir+'EOHI'+tdt+'.FITS', /REMOVE_ALL)
endif else begin
    goto, no_eohi
endelse


; open the EOHI file

OPENR, lun, /GET_LUN, filename, /BLOCK, ERROR=err
IF err NE 0 THEN goto, no_eohi
FREE_LUN, lun
fxbopen, unit, filename, 1, header

; read the FITS data

fxbread, unit, EOHITDTN, 'EOHITDTN'

row=where(EOHITDTN eq tdtnr)
if row(0) eq -1 then begin
    goto, no_tdt
endif

text=" "

for i=0,n_elements(row)-1 do begin
    ro=row(i)+1
    fxbread, unit, EOHIOSN,'EOHIOSN', ro
    fxbread, unit, EOHIUTC, 'EOHIUTC', ro
    fxbread, unit, EOHIINST, 'EOHIINST', ro
    fxbread, unit, EOHITDTC, 'EOHITDTC', ro
    fxbread, unit, EOHIMSG1, 'EOHIMSG1', ro

; Display the result

    yname="EOHI Information"+tdtnr

    text=[text, "EOHIOSN : "+EOHIOSN+ $
          "              / Observation Sequence Number (OSN)"]
    text=[text,"EOHIUTC : "+EOHIUTC+ $
          "     / UTC of sending associated ICS (yydddhhmms)"]
    text=[text,"EOHIINST: "+EOHIINST+ $
          "              / Prime instrument ID"]
    text=[text,"EOHITDTC: "+EOHITDTC+ $
          "             / TDT Component part"]

    text=[text," "]
    text=[text,"EOHIMSG1: "]

; decode AOTVAR entries, only valid for PHT

    if strmid(EOHIINST,0,1) eq 'P' then begin
        text=[text," "]
        if strnumber(strmid(EOHIMSG1,0,4),val) then begin
            sn=strcompress(string(round(100*10^val)/100.))
        endif else begin
            sn=" "
        endelse
        text=[text,"   Expected S/N                    : " $
              +sn]
        text=[text,"   Point or Extended Source        : " $
              +strmid(EOHIMSG1,4,1)]
        if strnumber(strmid(EOHIMSG1,16,8),val) then begin
            sf=strcompress(string(round(100*10^val)/100.))
        endif else begin
            sf=" "
        endelse
        text=[text,"   Expected Source Flux            : "  $
              +sf]
        if strnumber(strmid(EOHIMSG1,24,8),val) then begin
            unc=strcompress(string(round(100*10^val))/100.)
        endif else begin
            unc=" "
        endelse
        text=[text,"   Uncertaintiy in Source Flux     : " $
              +unc]
        if strnumber(strmid(EOHIMSG1,32,8),val) then begin
            bg=strcompress(string(round(100*10^val))/100.)
        endif else begin
            bg=" "
        endelse
        text=[text,"   Maximum Background Flux         : " $
              +bg]
        text=[text,"   Subsystem ID                    : " $
              +strmid(EOHIMSG1,40,1)]
        text=[text,"   Measurements in Sub-observation : " $
              +strmid(EOHIMSG1,41,3)]        
    endif else begin
        text=[text,EOHIMSG1] 
    endelse
    if i ne  n_elements(row)-1 then begin
        text=[text,""]
        text=[text,"---------- New Record ---------------"]
        text=[text,""]
    endif
endfor

pia_disp_print_file, 'EOHI'+tdtnr, title=yname, text=text

; normal exit
error = 0
fxbclose, unit
RETURN

; error conditions:
no_eohi: 
MESSAGE, /INFO, "EOHI file not available: "+filename
RETURN

no_tdt: 
MESSAGE, /INFO, "TDT number: "+tdtnr+" does not exist in EOHI file: "+filename
RETURN
END




