;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME: 
;	PIA_Disp_Print_File	
;
; PURPOSE:
;	Display an ASCII text file using widgets and the widget manager, giving
;	the possibility of printing it or saving it as a file
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_disp_print_file, Filename
;
; INPUTS:
;     Filename:	A scalar string that contains the filename of the file
;		to display.  The filename can include a path to that file.
;		It is also used as the first choice for the output filename.
;
; KEYWORD PARAMETERS:
;	FONT:   The name of the font to use.  If omitted use the default
;		font.
;
;	GROUP:	The widget ID of the group leader of the widget.  If this 
;		keyword is specified, the death of the group leader results in
;		the death of PIA_disp_print_file.
;
;	HEIGHT:	The number of text lines that the widget should display at one
;		time.  If this keyword is not specified, 24 lines is the 
;		default.
;
;	TEXT:	A string or string array to be displayed in the widget
;		instead of the contents of a file.  This keyword supercedes
;		the FILENAME input parameter.
;
;	TITLE:	A string to use as the widget title rather than the file name 
;		or "PIA_disp_print_file".
;
;	WIDTH:	The number of characters wide the widget should be.  If this
;		keyword is not specified, 80 characters is the default.
;
;	UNIQUE:	If set, a special name for Xregistering is used, which can be
;		used for avoiding multiple warnings 
;		"PIA_DISP_PRINT_FILE_UNIQUE"
;
; OUTPUTS:
;	No explicit outputs.  A file viewing widget is created.
;
; COMMON BLOCKS:
;	pia_disp_print_file_cmn
;
; SIDE EFFECTS:
;	Triggers the xMANAGER,No_BLOCK=1 if it is not already in use.
;
; PROCEDURE:
;	Open a file and create a widget to display its contents.
;
; PROCEDURES USED:
;       INTERNAL:
;		PIA_Disp_Print_File_evt
;
; MODIFICATION HISTORY:
;	Written by: Carlos GABRIEL (ESA/VILSPA-SAI)  		July 1995
;	Oct 95 Version 3.5
;	Savename in common replaced by widget_id (CG)		Christmas 1995
;	Free_lun instead of close - get_lun used (CG)		May 1996
;	Savefiles now by default to PIA_SAVEDIR (CG)		March 1997
;	V6.2
;	Using the system variable !PIA_PRINTDIR instead of
;	 the environment variable for SPAWN (CG)		June 1997
;	V6.3 
;	Editing text accepted (CG)				June 1997
;	V6.4
;	optional keyword SAVENAME added (IH)			August 1997
;	Optional keyword "unique" added (CG)			January 1998
;	Bug for text > 1000 lines fixed (CG)			February 1998
;	V7.0
;	Adding Adobe prefix to a2ps output (CG)			March 1998
;	V7.1
;			
;-
; *********************************************************************
; *
; *********************************************************************
PRO PIA_Disp_Print_File_evt, event


WIDGET_CONTROL, GET_UVALUE = retval, event.id
WIDGET_CONTROL, event.top, GET_UVALUE=IDS

filetext_id = ids.filetext_id
saven_id = ids.saven_id

CASE retval OF

	"EXIT": WIDGET_CONTROL, event.top, /DESTROY
	
	"PRINT": BEGIN
		WIDGET_CONTROL, filetext_id, GET_VALUE=a
		lines = N_ELEMENTS(a) 	
		filename = !PIA_PRINTDIR + 'tmp.txt'
	        OPENW,unit,filename, /get_lun
	        PRINTF,unit
	        FOR i=0,lines-1 DO $
	           PRINTF,unit,a(i)
;	  	PRINTF,unit,string(12B)
	  	FREE_LUN, unit
	  	
; for UNIX use ascii to PS conversion 
	    	IF !VERSION.OS EQ 'sunos' THEN BEGIN
		  spawn, '$PIA_INCL/a2ps -1 -p '+filename+ $
		                          ' > '+!PIA_PRINTDIR+'PIA_txt.ps'
		  spawn, 'cat $PIA_INCL/adobe_prefix.ps '$
		  	+!PIA_PRINTDIR+'PIA_txt.ps > ' $
		  	+!PIA_PRINTDIR+'PIA_txtc.ps'
	      	  spawn,'lpr -P$pslaser ' + !PIA_PRINTDIR + 'PIA_txtc.ps'
	      	  spawn,'rm ' + !PIA_PRINTDIR + 'PIA_txt.ps'
	      	  spawn,'rm ' + !PIA_PRINTDIR + 'PIA_txtc.ps'
	    	ENDIF ELSE IF !version.OS EQ 'vms' THEN $
	      		spawn,'print/queue=pslaser/notify/delete '+filename $
	      	      ELSE spawn,'lpr -P$pslaser '+filename
	  	
	  END

	 "SAVE": BEGIN
	  	WIDGET_CONTROL, saven_id, get_value=savename
	  	savename = savename(0)
		WIDGET_CONTROL, filetext_id, GET_VALUE=a
		lines = N_ELEMENTS(a)	
	        OPENW,1,savename
	        PRINTF,1
	        FOR i=0L,lines-1 DO $
	           PRINTF,1,a(i)
	  	PRINTF,1,string(12B)
	  	CLOSE,1
	  END	

	  ELSE:
	  
ENDCASE
END


; *********************************************************************
; *
; *
; *
; *********************************************************************
PRO PIA_Disp_print_File, FILENAME, TITLE = TITLE, GROUP = GROUP, MODAL=modal,$
		WIDTH = WIDTH, $
		HEIGHT = HEIGHT, TEXT = TEXT, FONT = font, $
		SAVENAME = SAVENAME, UNIQUE=UNIQUE

IF not KEYWORD_SET(modal) THEN modal=0

IF KEYWORD_SET(unique) THEN $
  IF XREGISTERED(unique) THEN RETURN

IF N_PARAMS() EQ 0 THEN filename = ""
IF n_elements(savename) EQ 0 THEN BEGIN
	IF filename EQ "" THEN savename = !INTDIR+"tmp.txt" ELSE $
				savename = !INTDIR+filename+".txt"
ENDIF

IF(NOT(KEYWORD_SET(TITLE))) THEN TITLE = FILENAME	;use the defaults if
IF(NOT(KEYWORD_SET(HEIGHT))) THEN HEIGHT = 24		;the keywords were not
IF(NOT(KEYWORD_SET(WIDTH))) THEN WIDTH = 80		;passed in

IF(NOT(KEYWORD_SET(TEXT)))THEN BEGIN
  OPENR, unit, FILENAME, /GET_LUN, ERROR=i		;open the file and then
  if i lt 0 then begin		;OK?
	a = [ !err_string, ' Can not display ' + filename]  ;No
  endif else begin
	  a = strarr(1000)				;Maximum # of lines
	  i = 0
	  c = ''
	  while not eof(unit) AND i LT 1000 do begin
		readf,unit,c
		a(i) = c
		i = i + 1
		endwhile
	  a = a(0:(i-1)>0)  ;Added empty file check -KDB
	  FREE_LUN, unit				;free the file unit.
  endelse
ENDIF ELSE a = TEXT

IF modal THEN BEGIN
 IF not KEYWORD_SET(group) THEN group = !pia_top_wid
ENDIF ELSE $
 IF not KEYWORD_SET(group) THEN group = ''
 
filebase = WIDGET_BASE(TITLE = TITLE, $			;create the base
		/COLUMN, $
		SPACE = 20, $
		XPAD = 20, $
		YPAD = 20, modal=modal, GROUP_LEADER=group)

act_base = WIDGET_BASE(filebase, /ROW)

filequit = WIDGET_BUTTON(act_base, $			;create a Done Button
		VALUE = " Quit ", $
		UVALUE = "EXIT")

filep_id = WIDGET_BUTTON(act_base, $			;create a Done Button
		VALUE = " Print ", $
		UVALUE = "PRINT")

files_id = WIDGET_BUTTON(act_base, $			;create a Done Button
		VALUE = " Save to File ", $
		UVALUE = "SAVE")

saven_id = WIDGET_TEXT(act_base, $			;filename under which
		VALUE = savename, xsize=50, /scroll, $	;file will be saved
		UVALUE = "JUNK", /EDITABLE)		

IF n_elements(font) gt 0 then $
 filetext_id = WIDGET_TEXT(filebase, /editable, $		;create a text widget
		XSIZE = WIDTH, $			;to display the file's
		YSIZE = HEIGHT, $			;contents
		/SCROLL, FONT = font, $
		VALUE = a, UVALUE='Junk') $
ELSE filetext_id = WIDGET_TEXT(filebase, /EDITABLE,$	;create a text widget
		XSIZE = WIDTH, $			;to display the file's
		YSIZE = HEIGHT, $			;contents
		/SCROLL, $
		VALUE = a, UVALUE='Junk')


IDS = {filetext_id:filetext_id, saven_id:saven_id}

WIDGET_CONTROL, filebase, SET_UVALUE = IDS

WIDGET_CONTROL, filebase, /REALIZE			;instantiate the widget

IF KEYWORD_SET(unique) THEN xreg_name=unique ELSE $
			    xreg_name="PIA_disp_print_File"

xMANAGER,No_BLOCK=1, xreg_name, $					;register it with the
		filebase, $				;widget manager
		GROUP_LEADER = GROUP, $
		EVENT_HANDLER = "PIA_disp_print_file_evt" 

END  ;--------------------- procedure PIA_disp_print_File ---------------------
