;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_DY2X
;
; PURPOSE:
;	Calculate map X-coordinates for given declination and given map
;	Y-coordinate values. Arrays can be used in two forms:
;	(1) a set of Y pixel values, and a scalar declination, or
;	(2) a set of declination values, and a scalar Y value.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	x = PIA_DY2X( dec, y, trans, ra )
;
; INPUTS:
;	DEC:	Declination value(s) in degrees.
;		If Y is an array, then DEC must be a scalar.
;
;	Y:	Map Y-coordinate value(s).
;		If DEC is an array, then Y must be a scalar.
;
;	TRANS:	Transformation parameters as provided by PIA_MKTRANS.
;
; OUTPUTS:
;	X:	Map X-coordinate value(s).
;
; OPTIONAL OUTPUTS:
;	RA:	Corresponding right ascension value(s) in degrees.
;
; CALLED ROUTINES:
;	PIA_RD2XY
;
; RESTRICTIONS:
;	Program will have difficulty converging for declination values near 90.
;
;	The transformation structure TRANS is not compatible with the
;	ASTR structure of ASTROLIB routines!
;
; PROCEDURES USED:
;	PIA LIB:
;		pia_rd2xy
;
; MODIFICATION HISTORY:
;	Written by: D.Skaley, MPIK Mar-95 
;	Oct 95 Version 3.5
;			
;-
; *********************************************************************
; *
; *********************************************************************
FUNCTION PIA_DY2X, dec, y, trans, ra

	ON_ERROR,2

	dtor = !DPI/180.0D0

	crval = trans.crval * dtor
	delta_y = y - trans.crpix(1)
	sin_d0 = sin(crval(1))
	cos_d0 = cos(crval(1))

	a = trans.dc(1,0)
	b = delta_y*cos_d0 + sin_d0*trans.dc(1,1)
	c = tan(dec*dtor)*(delta_y*sin_d0-cos_d0*trans.dc(1,1))
	sign = 2*( a GT 0 ) - 1 

	ra = crval(0) + atan(b/a) + sign*asin( c/sqrt(a^2+b^2) )
	ra = ra / dtor

	PIA_RD2XY, ra, dec, trans, x

	RETURN, x
	END
