;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_FILEFILTER
;	
; PURPOSE:
;	Return a proper file filter corresponding to PIA product type
;	
; CATEGORY:
;	PIA - support
;	
; CALLING SEQUENCE:
;	filter = PIA_FILEFILTER( producttype, detectortype )
;	
; INPUTS:
;	producttype :	Type of PIA product. Valid types are:
;			'CS'	= compact status file
;			'ERD'	= Edited Raw Data file
;			'SPD'	= Standard Processed Data file
;			'CALA'	= cal-A file
;			'AAR'	= Auto Analysis Results file
;			'*'	= any file
;
;	detectortype :	Subtype of file corrsponding to PHT detector.
;			(are not for all producttypes meaningful)
;			'P'	= P1, P2, or P3
;			'P1'	= P1
;			'P2'	= P2
;			'P3'	= P3
;			'C'	= C100 or C200
;			'C1'	= C100
;			'C2'	= C200
;			'S'	= SS or SL
;			'SS'	= SS
;			'SL'	= SL
;			'*'	= all types
;
; OPTIONAL INPUTS:
;	(none)
;	
; KEYWORD PARAMETERS:
;	(none)
;	
; OUTPUTS:
;	(none)
;
; RETURN VALUE:
;	filter :	The file filter string corresponding to product type
;			and eventually detectortype.
;	
; CALLED ROUTINES:
;	(none)
;	
; COMMON BLOCKS:
;	(none)
;	
; SIDE EFFECTS:
;	(none)
;	
; RESTRICTIONS:
;	Not all non-product files can be excluded with the returned filter.
;	
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK/Heidelberg
;	Oct 95 Version 3.5
;-

FUNCTION PIA_FILEFILTER, producttype, detectortype

	IF N_ELEMENTS(producttype) EQ 0 THEN producttype = "*"
	IF N_ELEMENTS(detectortype) EQ 0 THEN detectortype = "*"
	producttype = STRUPCASE(producttype)
	detectortype = STRUPCASE(detectortype)

	fits_extension = ".FITS"

	CASE !version.os OF
	'vms':	wildcard = "%"
	ELSE:	wildcard = "?"
	ENDCASE

	CASE producttype OF

	'CS':	filter = "PSTA*" + fits_extension

	'ERD':	BEGIN
		filter = "P%ER*" + fits_extension
		CASE detectortype OF
		'P':	STRPUT, filter, 'P', 1
		'P1':	STRPUT, filter, 'P', 1
		'P2':	STRPUT, filter, 'P', 1
		'P3':	STRPUT, filter, 'P', 1
		'C1':	STRPUT, filter, '1', 1
		'C2':	STRPUT, filter, '2E%', 1
		'S':	STRPUT, filter, 'S', 1
		'SS':	STRPUT, filter, 'S', 1
		'SL':	STRPUT, filter, 'S', 1
		ELSE:	STRPUT, filter, wildcard, 1
		ENDCASE
		END

	'SPD':	BEGIN
		filter = "P%%S*" + fits_extension
		STRPUT, filter, wildcard, 2
		CASE detectortype OF
		'*':	STRPUT, filter, wildcard, 1
		ELSE:	STRPUT, filter, detectortype, 1
		ENDCASE
		END

	'CALA':	BEGIN
		filter = "P%%A*" + fits_extension
		STRPUT, filter, wildcard, 2
		CASE detectortype OF
		'*':	STRPUT, filter, wildcard, 1
		ELSE:	STRPUT, filter, detectortype, 1
		ENDCASE
		END

	'AAR':	BEGIN
		filter = "P%A*" + fits_extension
		CASE STRMID(detectortype,0,1) OF
		'P':	STRPUT, filter, 'P', 1
		'C':	STRPUT, filter, 'C', 1
		'S':	STRPUT, filter, 'S', 1
		ELSE:	STRPUT, filter, wildcard, 1
		ENDCASE
		END

	ELSE:	filter = "*.*"
	ENDCASE

	RETURN, filter
END
