;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_FILETYPE
;	
; PURPOSE:
;	Determine the product type and the detector type of a product file
;	from the file name.
;	
; CATEGORY:
;	PIA - support
;	
; CALLING SEQUENCE:
;	PIA_FILETYPE( filename, producttype, detectortype )
;	
; INPUTS:
;	filename :	The name of the product file.
;			May contain the directory path.
;
;
; OPTIONAL INPUTS:
;	(none)
;	
; KEYWORD PARAMETERS:
;	(none)
;	
; OUTPUTS:
;	producttype :	Type of PIA product:
;			'CS'	= compact status file
;			'ERD'	= Edited Raw Data file
;			'SPD'	= Standard Processed Data file
;			'CALA'	= cal-A file
;			'AAR'	= Auto Analysis Results file
;			'*'	= unknown file type
;
;	detectortype :	Detector type of file:
;			'P'	= P1, P2, or P3
;			'P1'	= P1
;			'P2'	= P2
;			'P3'	= P3
;			'C'	= C100 or C200
;			'C1'	= C100
;			'C2'	= C200
;			'S'	= SS or SL
;			'SS'	= SS
;			'SL'	= SL
;			'G'	= general (for AAR)
;			'*'	= any detector or unknown
;
; CALLED ROUTINES:
;	(none)
;	
; COMMON BLOCKS:
;	(none)
;	
; SIDE EFFECTS:
;	(none)
;	
; RESTRICTIONS:
;	(none)
;	
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK/Heidelberg
;	Oct 95 Version 3.5
;-

PRO PIA_FILETYPE, filename, producttype, detectortype

	IF N_ELEMENTS(filename) EQ 0 THEN BEGIN
		producttype = "*"
		detectortype = "*"
		RETURN
	ENDIF

; eliminate the directory name
	file = filename
	CASE !version.os OF
	'vms':	file = STRMID(file,STRPOS(file,"]")+1,STRLEN(file))
	'unix':	REPEAT BEGIN
			pos = STRPOS(file,"/")
			file = STRMID(file,pos+1,STRLEN(file))
		END UNTIL pos EQ -1
	ELSE:
	ENDCASE


	CASE 1 OF

	STRMID(file,0,1) NE "P": BEGIN
		producttype = "*"
		detectortype = "*"
	END

	STRMID(file,1,3) EQ "STA": BEGIN
		producttype = "CS"
		detectortype = "*"
	END

	STRMID(file,2,2) EQ "ER": BEGIN
		producttype = "ERD"
		CASE STRMID(file,1,1) OF
		'P': detectortype = "P"
		'1': detectortype = "C1"
		'2': detectortype = "C2"
		'S': detectortype = "S"
		ELSE: detectortype = "*"
		ENDCASE
	END

	STRMID(file,3,1) EQ "S": BEGIN
		producttype = "SPD"
		detectortype = STRMID(file,1,2)
	END

	STRMID(file,3,1) EQ "A": BEGIN
		producttype = "CALA"
		detectortype = STRMID(file,1,2)
	END

	STRMID(file,2,1) EQ "A": BEGIN
		producttype = "AAR"
		detectortype = STRMID(file,1,1)
	END

	ELSE: BEGIN
		producttype = "*"
		detectortype = "*"
	END

	ENDCASE
	RETURN
END
