;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_GET_INBPOWER
;
; PURPOSE:
;	Graphical I/F to read records from the source calibration files
;	accomodating values into a structure returned in a common block
;
; CATEGORY:
;       PIA - graphics-I/O
;
; CALLING SEQUENCE:
;       PIA_GET_INBPOWER, source_id, filter_id
;
; INPUT:
;	source_id: 	Name of Calibration Source to start with. 
;	filter_id:	Filter name to start with. 
;
; COMMON BLOCKS:
;	pia_get_inbp_res (returning results)
;	pia_get_inbp_cmn (internally used)
;
; PROCEDURES USED:
;	PIA LIB:
;       ASTRO LIB:
;		FXBOPEN
;		FXBREAD
;		FXBCLOSE
;	INTERNAL:
;		pia_get_inbpower_event
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)			December 1995
;	sed_id shown together with source_id (CG)		January 1996
;	Bug fixed (IBP uncert. was initialized as INT !) (CG)	February 1996
;	Filter names changed in Source files (CG)		February 1996
;	Modal defined as prescribed for IDL>=5.0 (CG)		January 1999
;	V7.3
;			
;-

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;	Read the FITS file containing the inband power and photon number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO read_inbpow_file, filename

COMMON pia_get_inbpower_cmn, file_wd, filt_wd, srcid_wd, srcch_wd, sedid_wd, $ 
        pow_wd, u_pow_wd, pht_wd, u_pht_wd, file_vers, group
COMMON read_inbpo_cmn, ib_pow, u_ib_pow, ib_pht, u_phot, sources, sed_id, $
        sed_date, filts		

	file_tmp = findfile(filename, count=count)
	IF count EQ 0 THEN BEGIN
		TEXT=['Source Calibration File ' + filename, $
			     'was not found. Returning ']
		PIA_DISP_print_file, '', Title='Source Cal File not found',$
			TEXT=text, Width=50, Height=4, /modal, group=group
	 	RETURN		
	ENDIF ELSE filename=file_tmp(0)
		
	FXBOPEN, unit, filename, 1, hdr
	
	FXBREAD, unit, ib_pow, 'IB_POW'
	FXBREAD, unit, u_ib_pow, 'U_IB_POW'
	FXBREAD, unit, ib_pht, 'PHOT'
	FXBREAD, unit, u_phot, 'U_PHOT'
	FXBREAD, unit, sources, 'SOURCE_ID'
	FXBREAD, unit, sed_id, 'SED_FILE'
	FXBREAD, unit, sed_date, 'SED_DATE'
	FXBCLOSE, unit
	FREE_LUN, unit

	filts = STRTRIM(FXPAR(hdr,'FILTER*'),2)

RETURN	

END

;;;;;;;;;;;;;;;;;;;
;
; The event routine
;
;;;;;;;;;;;;;;;;;;;
PRO pia_get_inbpower_event, event

COMMON pia_get_inbpower_cmn, file_wd, filt_wd, srcid_wd, srcch_wd, sedid_wd, $ 
        pow_wd, u_pow_wd, pht_wd, u_pht_wd, file_vers, group
COMMON pia_get_inbp_res, c_inbp
COMMON read_inbpo_cmn, ib_pow, u_ib_pow, ib_pht, u_phot, sources, sed_id, $
        sed_date, filts		

	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
	c_inbp ={c_inbp_typ}

	CASE uvalue of
	
	  'SEARCH': BEGIN
 	     	WIDGET_CONTROL, file_wd, GET_VALUE=filename
	     	WIDGET_CONTROL, filt_wd, GET_VALUE=filter_id

		read_inbpow_file, filename(0)
		filt_pos = WHERE(filts EQ strtrim(filter_id(0),2), n)
		IF n NE 1 THEN BEGIN
	  	 MESSAGE, /INFO, 'Cannot find the filter position in FITS file'

		ENDIF ELSE BEGIN
	
		PIA_MULTCHOICE, index, elemnames=sources+sed_id, $
			group=group, /modal 

		power = ib_pow(filt_pos,index)
		unc_tmp = u_ib_pow(filt_pos, index)
		if unc_tmp(0) gt 0.0 then  $
		     uncer = u_ib_pow(filt_pos, index)    $
		  else   uncer = power * 0.1   
		photons = ib_pht(filt_pos, index)
		unc_tmp = u_phot(filt_pos, index)
		if unc_tmp(0) gt 0.0 then  $
		     unphots = u_phot(filt_pos, index)  $
		  else unphots = photons * 0.1 
		csed_id = sed_id(index)
		csed_date = sed_date(index)
		file_vers = [csed_id, csed_date]
		 
		WIDGET_CONTROL, srcch_wd, SET_VALUE=sources(index)
		WIDGET_CONTROL, sedid_wd, SET_VALUE=csed_id
		WIDGET_CONTROL, pow_wd, SET_VALUE=STRTRIM(STRING(power),2)
		WIDGET_CONTROL, u_pow_wd, SET_VALUE=STRTRIM(STRING(uncer),2)
		WIDGET_CONTROL, pht_wd, SET_VALUE=STRTRIM(STRING(photons),2)
		WIDGET_CONTROL, u_pht_wd, SET_VALUE=STRTRIM(STRING(unphots),2)
		
		ENDELSE
		
	    END
	    
	    'OK': BEGIN
		WIDGET_CONTROL, srcch_wd, GET_VALUE=source_id
	    	WIDGET_CONTROL, pow_wd, GET_VALUE=c_inbpow
	    	WIDGET_CONTROL, u_pow_wd, GET_VALUE=c_inbpun
	    	WIDGET_CONTROL, pht_wd, GET_VALUE=c_inbpht
	    	WIDGET_CONTROL, u_pht_wd, GET_VALUE=c_inphun
	     	WIDGET_CONTROL, filt_wd, GET_VALUE=filter_id
		WIDGET_CONTROL, sedid_wd, GET_VALUE=csed_id
		filter_id = filter_id(0)
		
	    	c_inbp.inbpow=FLOAT(c_inbpow(0))
		IF c_inbp.inbpow GT 0. THEN BEGIN 
	    	  c_inbp.inbpun=FLOAT(c_inbpun(0))
	    	  c_inbp.inbpht=FLOAT(c_inbpht(0))
	    	  c_inbp.inphun=FLOAT(c_inphun(0))
	    	  c_inbp.filter=filter_id(0)
		  c_inbp.src_id=source_id(0)
		  c_inbp.sed_id=file_vers
		ENDIF ELSE BEGIN
	    	  c_inbp.inbpow=-1.
	    	  c_inbp.inbpun=-1.
	    	  c_inbp.inbpht=-1.
	    	  c_inbp.inphun=-1.
	    	  c_inbp.filter=''
		  c_inbp.src_id=''
		  c_inbp.sed_id=['',''] 
	 	ENDELSE
	    	WIDGET_CONTROL, event.top, /destroy

	    END
	    
	    'CANCEL': BEGIN
	    	c_inbp.inbpow=-1.
	    	c_inbp.inbpun=-1.
	    	c_inbp.inbpht=-1.
	    	c_inbp.inphun=-1.
	    	c_inbp.filter=''
		c_inbp.src_id=''
		c_inbp.sed_id=['',''] 
	    	WIDGET_CONTROL, event.top, /destroy
	    END
	    
	    ELSE:
	   ENDCASE
RETURN
END	

;;;;;;;;;;;;;;;;;;;
;
; The main routine
;
;;;;;;;;;;;;;;;;;;;
				
PRO PIA_get_inbpower, source_id, filter_id, date_id, group=groupw

COMMON pia_get_inbpower_cmn, file_wd, filt_wd, srcid_wd, srcch_wd, sedid_wd, $ 
        pow_wd, u_pow_wd, pht_wd, u_pht_wd, file_vers, group 
COMMON read_inbpo_cmn, ib_pow, u_ib_pow, ib_pht, u_phot, sources, sed_id, $
        sed_date,filts		

	group = groupw
	ON_ERROR, 2
	
	IF not KEYWORD_set(group) THEN group=''
	
	c_inbp = {c_inbp_typ, inbpow:-1., inbpun:-1., inbpht:-1.,$
		inphun:-1.,src_id:'',filter:'', sed_id:['','']}
;
; Check the number of parameters
;
	IF N_PARAMS() LT 2 THEN MESSAGE, $
'Syntax: PIA_GET_INBPOW, SOURCE_ID, FILTER_ID '


	CASE STRMID(FILTER_ID, 0, 2) OF
	
	  'P1': filename = !CALC + 'SW_CAL.FITS'
	  'P2': filename = !CALC + 'SW_CAL.FITS'	  
	  'P3': filename = !CALC + 'LW_CAL.FITS'
	  'C1': filename = !CALC + 'LW_CAL.FITS'
	  'C2': filename = !CALC + 'LW_CAL.FITS'
	  ELSE: MESSAGE, /INFO, 'Detector not known'
	ENDCASE
	
	read_inbpow_file, filename
	
	filt_pos = WHERE(filts EQ STRTRIM(filter_id,2), nfil)

        IF nfil eq 0 THEN BEGIN
	  	  MESSAGE, /INFO, 'Cannot find the filter position in FITS file'
	 	  RETURN
	ENDIF 	  
	
	src_pos = WHERE(STRTRIM(STRUPCASE(sources),2) EQ $
			STRTRIM(STRUPCASE(source_id),2), nsrc)
	        	
	CASE 1 OF 
	   nsrc eq 1: noentry_found = 0
	   
	   nsrc gt 1: BEGIN 
	   	  date_info = 'Observation done: '+date_id+' Source: '
	  	  PIA_MULTCHOICE, index, elemnames=date_info+  $ 
	  	              sources(src_pos)+sed_id(src_pos),xsize=800
		   src_pos = src_pos(index(0))
		  noentry_found = 0
		END
	   nsrc eq 0: BEGIN 
	  	  MESSAGE, /INFO, 'Cannot find the given source in FITS file'
	  	  MESSAGE, /INFO, string(7b)+'<<<<<Please do search>>>>>'
	  	  noentry_found  = 1    
		END
		
	ENDCASE				

	
	IF noentry_found ne 1 THEN BEGIN
		power = ib_pow(filt_pos, src_pos)
		; if the error is undefined take a conservative value (10%)
		unc_tmp = u_ib_pow(filt_pos, src_pos)
		if unc_tmp(0) gt 0.0 then  $
		     uncer = u_ib_pow(filt_pos, src_pos)    $
		  else   uncer = power * 0.1   
		photons = ib_pht(filt_pos, src_pos)
		unc_tmp = u_phot(filt_pos, src_pos)
		if unc_tmp(0) gt 0.0 then  $
		     unphots = u_phot(filt_pos, src_pos)  $
		  else unphots = photons * 0.1 
		csed_src = sources(src_pos)
		csed_id = sed_id(src_pos)
		csed_date = sed_date(src_pos)
		file_vers = [csed_id, csed_date]
	ENDIF ELSE BEGIN
		power = ""
		uncer = ""
		photons = "" 
		unphots = "" 
		csed_src = ""
		csed_id = "" 
		csed_date = "" 
		file_vers = ["",""]
	ENDELSE	
		

	inbpbase = WIDGET_BASE(TITLE = 'PIA In-band power CAL tables', $
                                /COLUMN, /modal, group=group)

	filebase = WIDGET_BASE(inbpbase, /ROW)
	filelab = WIDGET_LABEL(filebase, VALUE='File: ')
	file_wd = WIDGET_TEXT(filebase, VALUE=filename, UVALUE='JUNK', $
			/editable, XSIZE=60)
	
	filtbase = WIDGET_BASE(inbpbase, /ROW)
	filtlab = WIDGET_LABEL(filtbase, VALUE='Filter chosen:    ')
	filt_wd = WIDGET_TEXT(filtbase, VALUE=filter_id, UVALUE='JUNK', $
			/editable)
	
	junklab = WIDGET_LABEL(inbpbase, VALUE='')
	
	srcbase_wd = WIDGET_BASE(inbpbase, /ROW)
	srclab_wd = WIDGET_LABEL(srcbase_wd, VALUE='Source Identifier: ')
	srcid_wd = WIDGET_TEXT(srcbase_wd, VALUE=source_id, UVALUE='JUNK')
	srcbase_wd = WIDGET_BASE(inbpbase, /ROW)
	srclab_wd = WIDGET_LABEL(srcbase_wd, VALUE='Observation Date: ')
	srcid_wd = WIDGET_TEXT(srcbase_wd, VALUE=date_id, UVALUE='JUNK')

	srcbase_wd = WIDGET_BASE(inbpbase, /ROW)
	srclab_wd = WIDGET_LABEL(srcbase_wd, VALUE='Source chosen: ')
	srcch_wd = WIDGET_TEXT(srcbase_wd, /editable, VALUE=csed_src, $ 
	    UVALUE='JUNK')
	srcbase_wd = WIDGET_BASE(inbpbase, /ROW)
	srclab_wd = WIDGET_LABEL(srcbase_wd, VALUE='Actual SED Filename: ')
	sedid_wd = WIDGET_TEXT(srcbase_wd, /editable, VALUE=csed_id, $
	    UVALUE='JUNK')


	junklab = WIDGET_LABEL(inbpbase, VALUE='')
	
	powbase = WIDGET_BASE(inbpbase, /ROW)
	powlab = WIDGET_LABEL(powbase, VALUE='In-Band Power: ')
	pow_wd = WIDGET_TEXT(powbase, /editable, XSIZE=12, $
		VALUE=string(power,format='(E10.3)'), UVALUE='JUNK')	
	powlab = WIDGET_LABEL(powbase, VALUE='Uncertainty: ')
	u_pow_wd = WIDGET_TEXT(powbase, /editable, XSIZE=12, $
		VALUE=string(uncer,format='(E10.3)'), UVALUE='JUNK')	
	
	junklab = WIDGET_LABEL(inbpbase, VALUE='')
	

	phtbase = WIDGET_BASE(inbpbase, /ROW)
	phtlab = WIDGET_LABEL(phtbase, VALUE='Inc. Photons:  ')
	pht_wd = WIDGET_TEXT(phtbase, /editable, XSIZE=12,  $
	       VALUE=string(photons,format='(E10.3)'), UVALUE='JUNK')	
	powlab = WIDGET_LABEL(phtbase, VALUE='Uncertainty: ')
	u_pht_wd = WIDGET_TEXT(phtbase, /editable, XSIZE=12, $
		VALUE=string(unphots,format='(E10.3)'), UVALUE='JUNK')	
	
	junklab = WIDGET_LABEL(inbpbase, VALUE='')

	contbase = WIDGET_BASE(inbpbase, /ROW)
	search = WIDGET_BUTTON(contbase, VALUE= '    Search   ', UVALUE='SEARCH')
	ok = WIDGET_BUTTON(contbase, VALUE='     OK     ', UVALUE='OK')
	cancel = WIDGET_BUTTON(contbase, VALUE='    Cancel    ', UVALUE='CANCEL')
	
        WIDGET_CONTROL, inbpbase, /REALIZE, TLB_SET_XOFFSET=300, $
                        TLB_SET_YOFFSET=50

        xMANAGER,No_BLOCK=1, "pia_get_inbpower", inbpbase, GROUP=group

RETURN
END
		
	
	
