;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_MAP_OPERATIONS
;
; PURPOSE:
;	Perform math. operations with maps and extract correlations
;
; CATEGORY:
;	PIA - graphics/processing
;
; CALLING SEQUENCE:
;       PIA_MAP_OPERATIONS, map, mappar, maphdr, par=par, dmap_type=dmap_type, $
;	div=div, subtr=subtr, norm_peak=norm_peak, $
;	subt_1quartil=subt_1quartil, corr_hist=corr_hist 
;
; INPUT:
;       (none)
;
; INPUT KEYWORDS:
;	div:		If set, maps chosen by the GUI will be divided (1/2)
;	subtr:		If set, maps chosen by the GUI will be subtracted (1-2)
;	norm_peak:	If set, normalization to the peak value prior operation
;	subt_1quartil:	If set, "background" subtraction performed "    "
;	corr_hist:	A diplot correlating the positions will be produced 
;	
; OUTPUT:
;	map:		A resulting map from the applied operation
;	mappar:		The PIA map parameter structure corresponding to "map"
;	maphdr:		Its header
;	dmap_type:	Its type
;	 
;
; OUTPUT KEYWORDS:
;	par:	
;
; MODIFICATION HISTORY:
;       Written by:	Carlos GABRIEL (ESA/VILSPA)		March 1999
;	V7.4
;-
PRO pia_map_operations, map, mappar, maphdr, par=par, dmap_type=dmap_type, $
	div=div, subtr=subtr, norm_peak=norm_peak, $
	subt_1quartil=subt_1quartil, corr_hist=corr_hist
	
	filenames = pia_pickfile(path=!xdrdir, filter="*.IMAP*", $
        	  /read, /must_exist, get_path = newpath, /multi)
		  
	IF not KEYWORD_SET(subt_1quartil) THEN subt_1quartil=0
	IF not KEYWORD_SET(norm_peak) THEN norm_peak=0
		  
	map =-1
	mappar=-1
	maphdr=-1
	
; make a dummy trans structure
	trans = PIA_MKTRANS([0,0],[0,0],[1,1],0)

	IF not KEYWORD_SET(par) THEN $
	par = { $
		mapmethod:0,	$	; Mapping method type
		dropfct:0.3,	$	; Mapping method type
		shx:0.,	$
		shy:0.,	$
		aapin:0,	$	; AAP data present flag
		mapin:0,	$	; maps produced flag
		detector:'',	$	; PHTAAP detector code
		filter:'',	$	; Filter_id
		aper:'',	$	; Aperture_id
		admn:'',	$	; PHTAAP administration string
		admn_decoded:'', $	; admn, decoded string
		aotn:'',	$	; AOT name
		rota:0.,	$	; rotation angle of map
		rota_orig:0.,	$	; rotation angle of map as in AAP
		ra:0.,		$	; right ascension of center of map
		dec:0.,		$	; declination of center of map
		npts:0,		$	; # points per raster line
		nlns:0,		$	; # raster lines
		dpts:0.,	$	; distance of raster points
		dlns:0.,	$	; distance of raster lines
		trans:trans,	$	; transformation structure
		trans_ra:0.,	$	; ra from pia_mapset
		trans_dec:0.,	$	; dec from pia_mapset
		datatype:0,	$	; mean, median, 1st, 3rd quartil
		pixsize:[0.,0.], $	; chosen map pixel size
		npixel:1,	$	; the number of pixels of the detector
		pixselect:[1,1,1,1,1,1,1,1,1], $ ; detector pixel selection
		ffield:0	$	; automatic flat fielding
	}
	IF N_ELEMENTS(filenames) NE 2 THEN BEGIN
		   PRINT, STRING(7B), 'Please, select TWO files'
		   RETURN
	ENDIF 
	IF newpath NE '' THEN BEGIN
;		   files = newpath + filenames
		   files = filenames
		   !XDRDIR = newpath 
	ENDIF ELSE $
		   files = filenames
	restore, files(0)
; be careful with old maps
	parnames = TAG_NAMES(imappar)
	par1 = par

	IF parnames(0) NE 'MAPMETHOD' THEN BEGIN
		   impnames = TAG_NAMES(par) 
		   FOR inam=0, N_ELEMENTS(parnames)-1 DO BEGIN
		      indtag = WHERE(impnames EQ parnames(inam),namcount)
		      IF namcount GT 0 THEN BEGIN
			siz = size(imappar.(inam))
			IF siz(2) EQ 8 THEN BEGIN
			   FOR itag=0,N_ELEMENTS(tag_names(imappar.(inam)))-1 DO $
				par1.(indtag(0)).(itag) = imappar.(inam).(itag) 
			 
			ENDIF ELSE $
		           par1.(indtag(0)) = imappar.(inam)
		      ENDIF
		   ENDFOR
		  
		   PRINT, 'Changing restored structure due to old (<7.3) format' 
	ENDIF ELSE $
		   par1 = imappar

	map1 = imap
	hdr1 = imaphdr
	siz1 = size(imap)
	type1 = imap_type
	dmap_type = type1
	
	restore, files(1)
; be careful with old maps
	parnames = TAG_NAMES(imappar)
	par2 = par

	IF parnames(0) NE 'MAPMETHOD' THEN BEGIN
		   impnames = TAG_NAMES(par) 
		   FOR inam=0, N_ELEMENTS(parnames)-1 DO BEGIN
		      indtag = WHERE(impnames EQ parnames(inam),namcount)
		      IF namcount GT 0 THEN BEGIN
			siz = size(imappar.(inam))
			IF siz(2) EQ 8 THEN BEGIN
			   FOR itag=0,N_ELEMENTS(tag_names(imappar.(inam)))-1 DO $
				par2.(indtag(0)).(itag) = imappar.(inam).(itag) 
			 
			ENDIF ELSE $
		           par2.(indtag(0)) = imappar.(inam)
		      ENDIF
		   ENDFOR
		  
		   PRINT, 'Changing restored structure due to old (<7.3) format' 
	ENDIF ELSE $
		   par2 = imappar

	map2 = imap
	hdr2 = imaphdr
	siz2 = size(imap)
	type2 = imap_type
	IF siz1(1) NE siz2(1) OR siz1(2) NE siz2(2) THEN BEGIN
		   PRINT, STRING(7B), 'Maps have different sizes, trying my best'
		   IF TOTAL(par1.pixsize - par2.pixsize) NE 0. THEN BEGIN
		    PRINT, STRING(7B), 'Maps have different pixel sizes, returning'
		    RETURN
		   ENDIF  
		   
		   IF abs(par1.rota - par2.rota) GT 1. THEN BEGIN
		    PRINT, STRING(7B), $
		     'Maps have too different rotation angles (>1.degree), returning'
		    RETURN
		   ENDIF  
		      
		   difra = par1.ra - par2.ra
		   difdec = par1.dec - par2.dec
		   cc = cos(par1.rota * 2. * !PI / 360.)
		   difpix = SQRT((difra*15.*cc)^2. + difdec^2.)
		   
		   IF difpix GT par1.pixsize(0) OR difpix GT par1.pixsize(1) $
		     THEN BEGIN
		    PRINT, STRING(7B), $
		     'Central positions must be within same pixel (sorry), returning'
		    RETURN
		   ENDIF  
		     
		   dif = fix(par2.trans.crpix - par1.trans.crpix)
		    
		   IF dif(0) LT 0 THEN BEGIN
		      nele = N_ELEMENTS(map2(*,0))
		      map1 = map1(-dif(0):-dif(0)+nele-1,*) 
		      par1.trans.crpix(0) = par2.trans.crpix(0)
		   ENDIF ELSE BEGIN
		      nele = N_ELEMENTS(map1(*,0))
		      map2 = map2(dif(0):dif(0)+nele-1,*)
		      par2.trans.crpix(0) = par1.trans.crpix(0)
		   ENDELSE
		   IF dif(1) LT 0 THEN BEGIN
		      nele = N_ELEMENTS(map2(0,*))
		      map1 = map1(*,-dif(1):-dif(1)+nele-1) 
		      par1.trans.crpix(1) = par2.trans.crpix(1)
		   ENDIF ELSE BEGIN
		      nele = N_ELEMENTS(map1(0,*))
		      map2 = map2(*,dif(1):dif(1)+nele-1)
		      par2.trans.crpix(1) = par1.trans.crpix(1)
		   ENDELSE
	ENDIF

	IF type1 NE type2 THEN PRINT, STRING(7B), $
		  'Warning, you are operating with different kinds of maps'		
	
	imappar = par1
	
	ftit = 'Flux '

	IF KEYWORD_SET(norm_peak) THEN BEGIN
		  map1 = map1/max(map1)
		  map2 = map2/max(map2)
		  ftit = ftit + '-normalized-'
	ENDIF 
		
	IF KEYWORD_SET(subt_1quartil) THEN BEGIN
		  med1 = median(map1)
		  ind = WHERE(map1 LT med1)
		  q1 = median(map1(ind))
		  map1 = map1 - q1
		  med2 = median(map2)
		  ind = WHERE(map2 LT med2)
		  q1 = median(map2(ind))
		  map2 = map2 - q1
		  ftit = ftit + '-bck_subtr-'
	ENDIF
		
	IF KEYWORD_SET(div) THEN BEGIN 
		imap = map1 / map2
		imaphdr = hdr1
		FXADDPAR, imaphdr, 'PRA_DIVI', 1, 'Map division performed', $
			BEFORE='XTENSION'
		imappar.filter = par1.filter + '/' + par2.filter
		par = imappar
		par.admn = "Colour map - " + STRMID(par2.admn,4,8)
		IF norm_peak THEN par.admn = par.admn + ' (norm.)'
		IF subt_1quartil THEN par.admn = par.admn + " (subtr.)"
		OBJ = fxpar(imaphdr,"OBJECT")
		par.admn_decoded = par.admn + '  - ' + obj + '  ' + $
		    imappar.filter
		imappar = par 
		map = imap
		mappar = imappar
		maphdr = imaphdr

	ENDIF ELSE IF KEYWORD_SET(subtr) THEN BEGIN
	 	imap = map1 - map2
		imaphdr = hdr1
		FXADDPAR, imaphdr, 'PRA_SUBI', 1, 'Map subtraction performed', $
			BEFORE='XTENSION'
		imappar.filter = par1.filter + '-' + par2.filter
		par = imappar
		par.admn = "Subtraction map - " + STRMID(par2.admn,4,8)
		IF subt_1quartil THEN par.admn = par.admn + " (subtr.)"
		IF norm_peak THEN par.admn = par.admn + ' (norm.)'
		OBJ = fxpar(imaphdr,"OBJECT")
		par.admn_decoded = par.admn + '  - ' + obj + '  ' + $
		    imappar.filter
		imappar = par 
		map = imap
		mappar = imappar
		maphdr = imaphdr
	ENDIF
	
	IF KEYWORD_SET(corr_hist) THEN BEGIN
	
		arr = map1
		boost_array, arr, map2
		
		PIA_XPLOT, arr(*,*,0), arr(*,*,1), psym=4, $
			Title="Map Correlation", $
			Xtitle=ftit + ' ' +dec_admn(par1.admn), $ 
			Ytitle=ftit + ' ' +dec_admn(par2.admn)
			
	ENDIF
	RETURN
	END 
		 	
