;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPAAP
;
; PURPOSE:
;	Produce a map of AAP data.
;	This routine operates on a PHTAAP structure containing one measurement.
;	Unpacks the arrays from handles and passes them to the mapping routines.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_MAPAAP, aap, hbright, pixselect, pixsize, rota, $
;			map_bright, map_uncert, map_expose, trans
;
; INPUTS:
;	AAP		The PHT structure (arrays are stored with handles)
;
;	HBRIGHT		Handle containing the brightness array
;
;       PIXSELECT       A 4 or 9 element array of logical values (0 or 1)
;                       indicating whether an array pixel should be selected
;                       for processing (not used for PHT-P).
;
;	PIXSIZE		The requested map pixel size [x,y]
;
;	ROTA		The requested rotation angle of the map
;
; KEYWORD PARAMETERS:
;
;	FFIELD		Flat field method (0:no, 1:central FF, 2:med.filter, 
;			   3: chosen background, 4: first percentile norm)
;
;	PERCENTILE	Defining the percentile to be taken for FlatField 4
;	
;	MMETHOD		Map method (0:full cov.,1:dist.wgt,2:trigrid,3:tr.quint)
;
;	FRACTION	Fraction of pixel size to be used for methods 1-3	
;
;	FILTER		Filter name
;
;	EVENT		Event ID calling this routine
;
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output is not valid
;
; OUTPUTS:
;	MAP_BRIGHT	The brightness map
;
;	MAP_UNCERT	The uncertainty map
;
;	MAP_EXPOSE	The exposure map
;
;	TRANS		Transformation structure from PIA_MKTRANS
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_mapbin
;		pia_mapset
;
; COMMON BLOCKS:
;	(none)
;
; MODIFICATION HISTORY:
;	Written by: 			D. Skaley, MPIK/Heidelberg
;	Oct 95 Version 3.5
;	Flat fielding option added	(CG)	                 February 1996
;	Pivoting added and different interpolation  (JA & CG)	 April 1997
;	Putting roll as rota only for chopped meas. (CG)	 December 1997
;	V6.6
;	Passing parameter FRACTION in call to map binning (CG)	 January 1999
;	V7.3
;	Shift_x & Shift_y keywords added 			February 1999
;	Use of coded bitflag system (CG)			May 1999
;	V7.4 
;	Percentile keyword added (CG)				May 2002
;	V10.0
;-

PRO PIA_MAPAAP, aap, hbright, pixselect, pixsize, rota,  $
		map_bright, map_uncert, map_expose, trans, $
		trans_ra, trans_dec, FFIELD=ffield, fraction=fraction, $
		MMETHOD=mmethod, filt=filt, flux=flux, percentile=percentile, $
		ERROR=error, event=event, shift_x=shift_x, shift_y=shift_y

; initialize error status

	error = 1


	IF NOT KEYWORD_SET(flux) THEN flux=0
	IF NOT KEYWORD_SET(mmethod) THEN mmethod=0
	IF NOT KEYWORD_SET(FILT) THEN filt=''
	IF NOT KEYWORD_SET(shift_x) THEN shift_x=0.
	IF NOT KEYWORD_SET(shift_y) THEN shift_y=0.
; get the arrays from the PHTAAP structure

	detector = aap.det
	HANDLE_VALUE, aap.rpid, rpid
	HANDLE_VALUE, aap.cpos, cpos 
	HANDLE_VALUE, aap.dwel, dwel 
	HANDLE_VALUE, aap.ra, ra 
	HANDLE_VALUE, aap.dec, dec 
	HANDLE_VALUE, aap.roll, roll 
	HANDLE_VALUE, aap.mnbu, uncert 
	HANDLE_VALUE, aap.cstp, step 

; get the brightness array

	HANDLE_VALUE, hbright, bright, /NO_COPY

; get a copy of the FLAG array (will be modified)

	HANDLE_VALUE, aap.flag, flag
	get_validflag, flag, 'AAP', redflag, det=aap.det
	flag = redflag

; disable flags for deselected array pixels

	IF STRMID(detector,0,1) EQ 'C' THEN BEGIN
                index = WHERE( pixselect EQ 0, count )
                IF count GT 0 THEN flag(index,*) = flag(index,*) OR 2
	ENDIF

; get the aperture ID for PHT-P

	aperture = aap.aper 

; set up map

	IF max(step) GT 1 THEN BEGIN
	IF ABS(rota-avg([roll])) GT 0.01 THEN BEGIN
		oldrota = rota
		rota = avg([roll])
		print, 'Rotation RA-DEC to map coords taken from roll angle'
		print, STRING(oldrota) + ' --> ' + STRING(rota)
	ENDIF
	ENDIF

;IF N_ELEMENTS(cpos) EQ 1 THEN BEGIN
;  cpos = 0.
;  PRINT, 'Warning, I am zeroing the chopper position'
;ENDIF
	PIA_MAPSET, detector, pixsize, aperture, rota, rpid, cpos, ra, dec, $
		roll, gridsize, x_coord, y_coord, trans, trans_ra, $
		trans_dec, ERROR=err, shift_x=shift_x, shift_y=shift_y
	IF err THEN GOTO, leave

; produce the maps

	IF not KEYWORD_SET(event) THEN event=0
IF N_ELEMENTS(cpos) EQ 1 AND min(bright) LT 0. THEN BEGIN
  minbr = min(bright)
  bright = bright - minbr - 0.01*minbr
  PRINT, 'Warning, I am adding an offset to the signals because some negative'
  PRINT, 'OFFSET= ', STRTRIM(minbr+0.01*minbr)
ENDIF

	IF not KEYWORD_SET(fraction) THEN fraction = 0.3
	IF not KEYWORD_SET(percentile) THEN percentile=25

	PIA_MAPBIN, detector, aperture, pixsize, gridsize, x_coord, y_coord, $
		dwel, flag, bright, uncert, $
		map_bright, map_uncert, map_expose, $
		FFIELD=ffield, ERROR=err, event=event, percentile=percentile,$
		mmethod=mmethod, fraction=fraction, filt=filt, flux=flux 
	IF err THEN GOTO, leave

; everything ok.
	
	error = 0

leave:

; move data arrays back into PHTAAP structure

;	HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
;	HANDLE_VALUE, aap.cpos, cpos, /SET, /NO_COPY
;	HANDLE_VALUE, aap.dwel, dwel, /SET, /NO_COPY
;	HANDLE_VALUE, aap.ra, ra, /SET, /NO_COPY
;	HANDLE_VALUE, aap.dec, dec, /SET, /NO_COPY
;	HANDLE_VALUE, aap.roll, roll, /SET, /NO_COPY
;	HANDLE_VALUE, aap.mnbu, uncert, /SET, /NO_COPY

; move brightness array back into handle

	HANDLE_VALUE, hbright, bright, /SET, /NO_COPY

	RETURN
	END
