;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPBIN
;
; PURPOSE:
;	Bin detector brightness density values into map grid.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPBIN, detector, aperture, pixsize, gridsize, x_coord, y_coord, $
;			dwel, flag, bright, uncert, $
;			map_bright, map_uncert, map_expose
;
; INPUTS:
;	DETECTOR	'C1', 'C2', 'P1', 'P2', 'P3'
;
;	APERTURE	PHT-P aperture ID.
;			For PHT-C the value of this argument is ignored.
;
;	PIXSIZE		Grid pixel size of map [x,y] in arcsec
;
;	GRIDSIZE	Total size of grid points [nx,ny]
;			as calculated by PIA_MAPSET
;
;	X_COORD		Map x-coordinates of input data
;			as calculated by PIA_MAPSET
;
;	Y_COORD		Map y-coordinates of input data
;			as calculated by PIA_MAPSET
;
;	DWEL		Dwell time of input data
;
;	FLAG		Flags array
;
;	BRIGHT		Brightness density values [MJy/sr]
;
;	UNCERT		Uncertainty of brightness density [MJy/sr]
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output arguments are not correct
;
; OUTPUTS:
;	MAP_BRIGHT	Brightness density map [MJy/sr]
;
;	MAP_UNCERT	Uncertainty map of brightness density [MJy/sr]
;
;	MAP_EXPOSE	Exposure map [sec]
;
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_defaper
;
; COMMON BLOCKS:
;	piacmn_pow2fb
;	
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	March 1995
;	Modified by:	C. Gabriel, ESA/VILSPA		July 1995
;	Oct 95 Version 3.5
;	Flat fielding call added (CG)			February 1996
;	Bug with dwell time units & map exposure times
;		fixed (CG)				March 1996
;	Forgotten gap_factor included  (CG)		March 1996
;	Single uncertainties used as weight for the 
;	 flux computation (IH + CG)			June 1997
;	V6.3
;	Call to median filtering, smoothing and 
;	 background flat-fielding added (CG)		October 1997
;	V6.5
;	Changes for treating correctly P map data,
;	shifts in semi-pixels corrected	(CG)		May 1998
;	Bug with overlap fixed following (CG)		June 1998
;	V7.2
;	Different mapping methods implemented (CG)	September 1998
;	Trigrid map converted to floating point (CG)	January 1999
;	FF_med_smooth call using modal keyword (CG)	January 1999
;	V7.3
;	Limit for triangulation to grid > [4,4] +
;	calculation of map uncertainty using weighted
;	mean derivation (CG)				June 1999
;	V7.4
;	1st Quartile Normalization FField included (CG)	August 1999
;	Separation of median filtering and bcgr. (CG)	August 1999
;	V8.0
;	COMMON PIA_FF_INFO  added (CG)			December 1999
;	V8.2
;-

PRO PIA_MAPBIN, detector, aperture, pixsize, gridsize, x_coord, y_coord, $
		dwel, flag, in_bright, in_uncert, $
		map_bright, map_uncert, map_expose, fraction=fraction, $
		FFIELD=FFIELD, ERROR=error, event=event, percentile=percentile,$
		MMETHOD=mmethod, FILT=filt, FLUX=flux, verbose=verbose

; common block for aperture constants
@piacmn_pow2fb
@colour_indices

COMMON pia_ffield_info, ff, no_widget

	IF N_ELEMENTS(no_widget) EQ 0 THEN no_widget=0
; load the aperture constants into common block (if not yet done)

	IF N_ELEMENTS( aper_r ) EQ 0 THEN PIA_DEFAPER
	IF KEYWORD_SET(event) THEN WIDGET_CONTROL, event.top, /hourglass

	bright = in_bright
	uncert = in_uncert
	
; error status
	start_time = systime(1)

	error = 1

; check detector input
; and select the detector pixel size (PHT-C) or aperture size (PHT-P)
; and define number of detector pixels per array: n_pixel

	detector = STRING(detector)
	CASE detector OF
	'C1':	BEGIN
		n_pixel = 9
		det_size = aper_r(*,15)
		gap_factor = 1. / 1.0575
		END
	'C2':	BEGIN
		n_pixel = 4
		det_size = aper_r(*,16)
		gap_factor = 1. / 1.035
		END
	'P1':	BEGIN
		n_pixel = 1
		det_size = aper_r(*,aperture)
		gap_factor = 1.
		END
	'P2':	BEGIN
		n_pixel = 1
		det_size = aper_r(*,aperture)
		gap_factor = 1.
		END
	'P3':	BEGIN
		n_pixel = 1
		det_size = aper_r(*,aperture)
		gap_factor = 1.
		END
	'SS':	BEGIN
		n_pixel = 1
		det_size = aper_r(*,17)
		gap_factor = 1. / 1.03
		END
	'SL':	BEGIN
		n_pixel = 1
		det_size = aper_r(*,17)
		gap_factor = 1. / 1.03
		END
	ELSE:	GOTO, err_detector
	ENDCASE


; check the other arguments
; and define number of plateaux: n_plat

	IF N_ELEMENTS(pixsize) NE 2 THEN GOTO, err_pixsize
	IF N_ELEMENTS(gridsize) NE 2 THEN GOTO, err_gridsize

	n_plat = N_ELEMENTS(dwel)
;help, n_plat
	IF n_plat EQ 0 THEN GOTO, err_array

	n_data = n_plat*n_pixel
;help, n_data,flag,x_coord, y_coord, bright, uncert
	IF N_ELEMENTS(flag) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS(x_coord) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS(y_coord) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS(bright) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS(uncert) NE n_data THEN GOTO, err_array


; detector size: (0),(1) = width,height, (2) = area
; convert from rad -> arcsec

	det_size = det_size * !RADEG * 3600.
	det_size(2) = det_size(2) * !RADEG * 3600.

	; for PHT-P: set x and y-dimensions to diameter and change cond. 
	circular = 0
	IF det_size(1) EQ 0. THEN BEGIN
	         det_size(1) = det_size(0) 
	         circular = 1
	         IF ABS(pixsize(1)-pixsize(0)) GT 0.3*pixsize(0) THEN BEGIN
	           MESSAGE, /INFO, $
	            ' image pixel sizes should be in this case similar '   
	           MESSAGE, /INFO, $
	            ' because of the circular aperture !!! '   
	         ENDIF
	ENDIF

; convert detector size from acrsec -> map pixel dimension units

	det_size(0:1) = det_size(0:1) / pixsize
	det_size(2) = det_size(2) / (pixsize(0)*pixsize(1))


; set up the grid arrays

	map_bright = FLTARR(gridsize(0),gridsize(1))
	map_uncert = FLTARR(gridsize(0),gridsize(1))
	map_expose = FLTARR(gridsize(0),gridsize(1))

; temporary variables:
; dwell time over detector area (area in units of map pixel size)
; expanded to data array size

	dwel_xpd = TRANSPOSE(REBIN([dwel]/det_size(2), $
			n_plat,n_pixel, /SAMPLE))


	distnc = ( det_size(0:1) * gap_factor + 1. ) / 2.
	 	 
	; the maximum overlap of map pixels and detector pixels
	; in units of map pixel dimensions
	max_overlap_x = MIN([ det_size(0), 1. ])
	max_overlap_y = MIN([ det_size(1), 1. ])

	tmp_bright = bright
	tmp_uncert = uncert
	 
; flag as bad points with 0. or negative brightness
	ind = WHERE(tmp_bright LE 0., nbad)
	IF nbad GT 0 THEN flag(ind) = 2
;	IF nbad GT 0 THEN PRINT, 'Warning, signals below 0'
; check uncertainties
	ind = WHERE(tmp_uncert EQ 0. AND flag EQ 0, nbad)
	IF nbad GT 0 THEN $
	   check_unc, tmp_bright, tmp_uncert, /verbose
	
	
	IF KEYWORD_SET(ffield) THEN $
	   IF ffield EQ 1 THEN BEGIN
		ffmap, gridsize, distnc, x_coord, y_coord, $
		  dwel_xpd, tmp_bright, tmp_uncert, flag, ff, error=error
		IF error EQ 0 AND NOT no_widget THEN $
			PIA_DISP_PRINT_FILE, '',TITLE='Flat field calculation',$
				TEXT=['The calculated flat field parameters:',$
				'Pixel '+$
				STRTRIM(STRING(indgen(n_pixel)+1),2)+':' +  $
				STRING(ff)]
	   ENDIF ELSE IF ffield EQ 2 THEN BEGIN
	      pia_ff_med_smooth, tmp_bright, tmp_uncert, tmp_bright, $
	      	flag=flag, /modal, group=event.top
	   ENDIF ELSE IF ffield EQ 3 THEN BEGIN
	      pia_ff_backgr, tmp_bright, tmp_uncert, tmp_bright, flag=flag, $
	      	/modal, group=event.top
	   ENDIF ELSE IF ffield EQ 4 THEN BEGIN
	        IF not KEYWORD_SET(percentile) THEN percentile=25
	   	FIRSTQ_FF_TMP, tmp_bright, tmp_uncert, ff, error=error,$
		 percentile=percentile
		IF error EQ 0 AND NOT no_widget THEN $
			PIA_DISP_PRINT_FILE, '',TITLE='Flat field calculation',$
				TEXT=['The calculated flat field parameters:',$
				'Pixel '+$
				STRTRIM(STRING(indgen(n_pixel)+1),2)+':' +  $
				STRING(ff)]
	   ENDIF
	        

	IF KEYWORD_SET(event) THEN WIDGET_CONTROL, event.top, /hourglass

	IF not KEYWORD_SET(MMETHOD) THEN mmethod=0
	IF mmethod GE 2 THEN BEGIN
		
	  IF min(gridsize) LT 5 THEN BEGIN
	    PRINT, STRING(7B), $
	     ' Triangulation not possible, too few image pixels, returning'
	    RETURN
	  ENDIF
	   
;	  IND = WHERE(flag EQ 0, count)
	  
	  IF NOT KEYWORD_SET(FRACTION) THEN fraction=0.25
	  
	  reduce_map_pos, x_coord, y_coord, tmp_bright, tmp_uncert, flag, $
	  	red_x, red_y, red_b, red_u, tol_dist=fraction, ERROR=error
	
	  IF ERROR NE 0 THEN BEGIN
	    print, 'no valid points, returning'
	    return
	  ENDIF
	  
	  sh = ((det_size(0:1))-1 ) / (-2)
;	  sh = [-1.,-0.5]
	  red_x = red_x - sh(0)
	  red_y = red_y - sh(1)
	  IF mmethod EQ 3 THEN quintic=1 ELSE quintic=0
	  triangulate, red_x, red_y, tr, b
	  trmap = trigrid(red_x, red_y, red_b, $
	    tr, [1.,1.], [0,0,gridsize(0)-1, gridsize(1)-1], quintic=quintic)
	  trmap = float(trmap)

; fill the map to the edges
	  imax = MIN(gridsize)-1 < 3
	  for i=0,imax do begin
	    ind0 = WHERE(trmap(i,*) EQ 0., nind)
	    IF nind GT 0 THEN FOR ii=0,i DO trmap(ii,ind0) = trmap(i+1,ind0)
	    ind0 = WHERE(trmap(gridsize(0)-1-i,*) EQ 0., nind)
	    IF nind GT 0 THEN FOR ii=0,i DO trmap(gridsize(0)-1-ii,ind0) = $
	            trmap(gridsize(0)-2-i,ind0)
	    ind0 = WHERE(trmap(*,i) EQ 0., nind)
	    IF nind GT 0 THEN FOR ii=0,i DO trmap(ind0,i) = trmap(ind0,i+1)
	    ind0 = WHERE(trmap(*,gridsize(1)-1-i) EQ 0, nind)
	    IF nind GT 0 THEN FOR ii=0,i DO trmap(ind0,gridsize(1)-1-i) = $
	        trmap(ind0,gridsize(1)-2-i)
	   endfor



	  map_bright=trmap	  

; mapping method using the distance weighted interpolation

	ENDIF ELSE IF mmethod EQ 1 THEN BEGIN
	 ; the distance of map pixel center to detector pixel center
	 ; in units of map pixel dimensions

	sh = ((det_size(0:1))-1 ) / (-2)

	IF not KEYWORD_SET(fraction) THEN fraction=0.3
	
	FOR x = 0,gridsize(0)-1 DO $
	 FOR y = 0,gridsize(1)-1 DO BEGIN

	    xsh = x + sh(0)
	    ysh = y + sh(1)

; take a good number of contributors 
	    xxs = x_coord-xsh
	    yys = y_coord-ysh


	    fctnr = fraction
	    index = WHERE( ABS(xxs) LT distnc(0)*fctnr AND $
	                   ABS(yys) LT distnc(1)*fctnr AND $
	                   (flag EQ 0), nfound ) 
	    IF nfound EQ 0 THEN BEGIN
	     fctnr = 2*fctnr
	     index = WHERE( ABS(xxs) LT distnc(0)*fctnr AND $
	                   ABS(yys) LT distnc(1)*fctnr AND $
	                   (flag EQ 0), nfound ) 
	    ENDIF
	    IF nfound EQ 0 THEN BEGIN
	     fctnr = 3*fctnr
	     index = WHERE( ABS(xxs) LT distnc(0)*fctnr AND $
	                   ABS(yys) LT distnc(1)*fctnr AND $
	                   (flag EQ 0), nfound ) 
	    ENDIF
	    
	    IF nfound GT 0 THEN BEGIN
	        distance = SQRT( (abs(xxs(index))/distnc(0))^2+$
				 (abs(yys(index))/distnc(1))^2 )
		
		 
	      
	     weight = (SQRT(2.*fctnr^2) - distance)^3. 
	     cont = weight / (SQRT(2.*fctnr^2))^3.

	      
;	      tot_weight = FLOAT( TOTAL(weight) )
	      tot_weight = FLOAT( TOTAL(cont) )

	      map_expose(x,y) = FLOAT( TOTAL(dwel_xpd(index) * cont) )

;	      map_bright(x,y) = FLOAT( TOTAL(weight*tmp_bright(index)) $
;	                               / tot_weight )
	      map_bright(x,y) = FLOAT( TOTAL(cont*tmp_bright(index)) $
	                               / tot_weight )

; use general uncertainty derivation
	      
	      IF nfound GT 1 THEN BEGIN 
	       dif = map_bright(x,y) - tmp_bright(index)	      
	       map_uncert(x,y) = FLOAT( SQRT(TOTAL(cont*dif*dif) $
	                               / tot_weight )) / SQRT(nfound-1)
	      ENDIF ELSE map_uncert(x,y) = tmp_uncert(index(0))
		  	    
	       
	 
	    ENDIF
	    
	  ENDFOR  
	
	ENDIF ELSE BEGIN
	
; simple trapezoidal gridding function

	distance_avg = (distnc(0)+distnc(1))/2
	
	sh = ((det_size(0:1)/gap_factor)-1 ) / (-2)

	FOR x = 0,gridsize(0)-1 DO $
	 FOR y = 0,gridsize(1)-1 DO BEGIN
	  IF not circular THEN BEGIN
	    xsh = x + sh(0)
	    ysh = y + sh(1)
	    index = WHERE( ABS(x_coord-xsh)/gap_factor LT distnc(0) AND $
	                   ABS(y_coord-ysh)/gap_factor LT distnc(1) AND $
	                   (flag EQ 0), nfound ) 

 
	  ENDIF ELSE BEGIN
	    xsh = x + sh(0)
	    ysh = y + sh(1)
	    index = WHERE( SQRT((x_coord-xsh)^2 + $
	    		         (y_coord-ysh)^2) LT distance_avg AND $
	                   (flag EQ 0), nfound )

	  ENDELSE                      
	  
	  IF nfound GT 0 THEN BEGIN

	      ; overlapping area in units of map pixel dimensions
	      ; a mere approximation for the circular case
	      
	     IF not circular THEN BEGIN
	       overlapx = distnc(0)- ABS(x_coord(index)-xsh)/gap_factor 
	       indx = where(overlapx GT max_overlap_x, novx)
	       IF novx GT 0 THEN overlapx(indx) = max_overlap_x
	      
	       overlapy = distnc(1)-ABS(y_coord(index)-ysh)/gap_factor
	       indy = where(overlapy GT max_overlap_y, novy)
	       IF novy GT 0 THEN overlapy(indy) = max_overlap_y
	     
	        overlap = overlapx * overlapy

	      ENDIF ELSE BEGIN
	      
		max_overlap = (max_overlap_x + max_overlap_y) / 2
	        overlap = distance_avg - $
	        	SQRT((x_coord(index)-xsh)^2+(y_coord(index)-ysh)^2)
	        indd = WHERE(overlap GT max_overlap, nov)
	        IF nov GT 0 THEN overlap(indd) = max_overlap 

	      ENDELSE

	      weight = dwel_xpd(index) * overlap / $
	      (tmp_uncert(index)/tmp_bright(index))^2

	      tot_weight = FLOAT( TOTAL(weight) )

	      map_expose(x,y) = FLOAT( TOTAL(dwel_xpd(index) * overlap)) 

	      map_bright(x,y) = FLOAT( TOTAL(weight*tmp_bright(index)) $
	                               / tot_weight )

;	      map_uncert(x,y) = FLOAT( SQRT(TOTAL(weight*tmp_uncert(index)^2)) $
;	                               / tot_weight )
; use general uncertainty derivation
	      
	      IF nfound GT 1 THEN BEGIN 
	       dif = map_bright(x,y) - tmp_bright(index)	      
	       map_uncert(x,y) = FLOAT( SQRT(TOTAL(weight*dif*dif) $
	                               / tot_weight )) / SQRT(nfound-1)
	      ENDIF ELSE map_uncert(x,y) = tmp_uncert(index(0))

	    ENDIF
	 ENDFOR
	ENDELSE


; This would be used if Jy per image pixel instead of Jy/beam 
;	IF KEYWORD_SET(flux) THEN BEGIN
;		flux_factor = det_size(0) * det_size(1) / gap_factor / gap_factor
;		map_bright = map_bright / flux_factor
;		map_uncert = map_uncert / flux_factor
;	ENDIF
		
	IF KEYWORD_SET(verbose) THEN $
	 PRINT, 'Time used: ', STRTRIM(systime(1)-start_time,2), ' seconds'
	error = 0
	RETURN

err_detector:
	MESSAGE, /INFO, "Wrong detector code: "+detector
	RETURN

err_gridsize:
	MESSAGE, /INFO, "Wrong input: GRIDSIZE"
	RETURN

err_pixsize:
	MESSAGE, /INFO, "Wrong input: PIXSIZE"
	RETURN

err_array:
	MESSAGE, /INFO, "Inconsistent input array dimensions"
	RETURN

	END
