;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPDRIFT_P32
;
; PURPOSE:
;	Perform a drift correction for P32 raster data.
;	The 'simple map' method is used for the map definition.
;	(See procedure PIA_P32SMAP.PRO for 'simple' assumptions.)
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPDRIFT_P32, detector, n_raster, dist_raster, $
;			time, rpid, cpos, dwel, flag, bright, $
;			corr, n_fit, dirft_rate
;
; INPUTS:
;	DETECTOR	'C1' or 'C2'
;
;	N_RASTER	Number of spacecraft raster points
;			(0) = Y-direction, (1) = Z-direction
;
;	DIST_RASTER	The distance of the raster points/lines [arcsec]
;			(0) = Y-direction, (1) = Z-direction
;
;	TIME		Time values for all measurements (any units)
;
;	RPID		Raster point IDs
;			rpid(0,*) = sequence number in +Y direction
;			rpid(1,*) = sequence number in -Z direction
;
;	CPOS		Chopper positions [arcsec]
;
;	DWEL		Chopper dwell times [2^-7 seconds]
;
;	FLAG		Flags array
;
;	BRIGHT		Brightness density array [MJy/sr]
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no errors
;			1 = error: maps were not produced
;
; OUTPUTS:
;	CORR		Brightness density array [MJy/sr] containing
;			drift corrected values.
;
; OPTIONAL OUTPUTS:
;	N_FIT		Number of fits used for calculating the drift
;			correction. A value of 0 means that the corresponding
;			value DRIFT_RATE is not valid.
;			The array size is the number of detector pixels.
;
;	DRIFT_RATE	Drift rates [MJy/sr/sec].
;			The array size is the number of detector pixels.
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		ggt
;
; COMMON BLOCKS:
;	(none)
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	March 1995
;	Oct 95 Version 3.5
;-

PRO PIA_MAPDRIFT_P32, detector, n_raster, dist_raster, $
		time, rpid, cpos, dwel, flag, bright, $
		corr, n_fit, drift_rate, $
		ERROR=error

; initialize error status

	error = 1

;*****************************************************************************
; the pixel numbering scheme in SPD/AAP products is as follows:
;
;    detector index for             frame of reference
;    C100:        C200:             of spacecraft:
;
;    3 6 9        3 4               +z
;    2 5 8        2 1               ^
;    1 4 7                          |
;                                   0--> +y (chopper)
;
;*****************************************************************************

; check 'detector' and define detector constants

	detector = STRING(detector)
	CASE detector OF
	'C1':	BEGIN
		n_pixel = 9			; num of pixels in array
		array_size = [ 3, 3 ]		; num of pixels in Y/Z dir
		detector_size = [ 45., 45. ]	; detector pixel size in arcsec
		pixel_pos_y = [ 0, 0, 0, 1, 1, 1, 2, 2, 2 ]	; relative pos
		pixel_pos_z = [ 0, 1, 2, 0, 1, 2, 0, 1, 2 ]	; to lower-left
		chopper_increment = 15.0	; expected ch incr in arcsec
		END
	'C2':	BEGIN
		n_pixel = 4			; num of pixels in array
		array_size = [ 2, 2 ]		; num of pixels in Y/Z dir
		detector_size = [ 90., 90. ]	; detector pixel size in arcsec
		pixel_pos_y = [ 1, 0, 0, 1 ]	; relative position
		pixel_pos_z = [ 0, 0, 1, 1 ]	; to lower-left pixel
		chopper_increment = 30.0	; expected ch incr in arcsec
		END
	ELSE:	GOTO, err_detector
	ENDCASE

; check raster parameters

	IF N_ELEMENTS(n_raster) NE 2 OR MIN(n_raster) LE 0 THEN GOTO,err_nrast
	IF N_ELEMENTS(dist_raster) NE 2 OR MIN(dist_raster) LE 0 THEN $
		GOTO,err_drast

; check consistency of array sizes

	n_plat = N_ELEMENTS( cpos )
	IF N_ELEMENTS( rpid ) NE 2*n_plat THEN GOTO, err_array
	IF N_ELEMENTS( time ) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS( dwel ) NE n_plat THEN GOTO, err_array
	n_data = n_pixel*n_plat
	IF N_ELEMENTS( flag ) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS( bright ) NE n_data THEN GOTO, err_array

; the chopper step size determins the map grid resolution in s/c Y-direction

	pixsize_y = chopper_increment
	chopper_shift = ROUND(cpos/pixsize_y)
	ch_min = MIN(chopper_shift, MAX=ch_max)
	chopper_shift = chopper_shift - ch_min	; shift so that min = 0

; the raster distance in s/c Z-direction determins the map in s/c Z-direction
; in units of the detector pixel size it can be: 1/3, 1/2, 1, 3/2, 2
; i.e a multiple of 1/6.

	detector_sixth = detector_size(1)/6.
	rounded_dist = ROUND(dist_raster(1)/detector_sixth)*detector_sixth
	pixsize_z = GGT(detector_size(1),rounded_dist)

; the grid parameters

	pixsize = [ pixsize_y, pixsize_z ]
	cover = ROUND(detector_size/pixsize)
	raster_shift = ROUND(dist_raster/pixsize)
	chopper_steps = [ ch_max-ch_min, 0 ]	; excluding 0 position
	dim = array_size*cover + raster_shift*(n_raster-1) + chopper_steps
	ratio = 1./(cover(0)*cover(1))
	pixel_shift_y = pixel_pos_y * cover(0)
	pixel_shift_z = pixel_pos_z * cover(1)

; initialize drift rate arrays

	n_fit = INTARR(n_pixel)			; number of slopes found
	drift_rate = FLTARR(n_pixel)		; initialized with 0.

; calculate the drift for all detector pixels individually

	FOR i_pixel = 0,n_pixel-1 DO BEGIN

		; initialize slopes array with dummy
		slopes = [0.]

; the raster IDs mapped to grid

		raster_y = rpid(0,*)-1
		raster_z = MAX(rpid(1,*))-rpid(1,*)

; The following index arrays have the same dimension as the input arrays
; for one detector pixel.
; The values are entries into the resulting map array.
; [y_low,z_low] is the map pixel covered by the lower left corner of the
; detector pixel.

		y_low = raster_y*raster_shift(0) + chopper_shift $
			+ pixel_shift_y(i_pixel)

		z_low = raster_z*raster_shift(1) $
			+ pixel_shift_z(i_pixel)

; calculate a slope for each map-position if possible

		FOR y = 0, dim(0)-1 DO $
		 FOR z = 0, dim(1)-1 DO BEGIN
		  index = WHERE( (y_low EQ y AND z_low EQ z) AND $
		                 (flag(i_pixel,*) EQ 0) $
		                , nfound )
		  IF nfound GT 1 THEN BEGIN
			xval = time(index)
			yval = bright(i_pixel,index)
			fit = POLY_FIT(xval,yval,1)
			slopes = [ slopes, fit(1) ]
		  ENDIF
		 ENDFOR

; if there was at least one map position with more than one brightness value
; then calulate the median of all slopes

		n_slopes = N_ELEMENTS(slopes) - 1
		IF n_slopes GT 0 THEN BEGIN
			slopes = slopes(1:*)
			n_fit(i_pixel) = n_slopes
			drift_rate(i_pixel) = MEDIAN(slopes)
		ENDIF

	ENDFOR

; calculate corrected brightness values

	corr = bright
	endtime = time(n_plat-1)
	FOR i_pixel = 0,n_pixel-1 DO BEGIN
		corr(i_pixel,*) = $
			corr(i_pixel,*) - (endtime-time)*drift_rate(i_pixel)
	ENDFOR

	error = 0
	RETURN

err_detector:
	MESSAGE, /INFO, "Wrong detector code: "+detector
	RETURN

err_nrast:
	MESSAGE, /INFO, "Wrong input argument: N_RASTER"
	RETURN

err_drast:
	MESSAGE, /INFO, "Wrong input argument: DIST_RASTER"
	RETURN

err_array:
	MESSAGE, /INFO, "Inconsistent input array dimensions"
	RETURN

	END
