;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPGRID
;
; PURPOSE:
;	Draw a grid over a map in gnomonic coordinates.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_MAPGRID, trans, area, ngrid
;
; INPUTS:
;	TRANS		Structure of transformation values
;
;	AREA		Image area in device coordinates: [X0,Y0,X1,Y1]
;			TRANS.CRPIX should refer to [X0,Y0]
;
;	NGRID		Approximate maximum number of required grid lines
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_dx2y
;		pia_dy2x
;		pia_rd2xy
;		pia_rx2y
;		pia_ry2x
;		pia_xy2rd
;		pia_xy2rd
;	Astronomical Library:
;		sixty
;	Internal:
;		pia_mapgrid_tics (function)
;
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK/Heidelberg	March 1995
;-

; ****************************************************************************
; NAME:
;	PIA_MAPGRID_TICS
;
; PURPOSE:
;	Calculate optimum increment of tic steps in degrees
;
; CALLING SEQUENCE:
;	delta = PIA_MAPGRID_TICS( degmin, degmax, ntics, /ra)
;
; INPUTS:
; 	DEGMIN		Minimum coordinate value in degrees
;	DEGMAX		Maximum coordinate value in degrees
;	NTICS		Approximate maximum number of required tics
;
; KEYWORDS:
;	/RA	If set, round to minutes/seconds of hours,
;		else to minutes/seconds of degrees.
;
; OUTPUTS:
;	DELTA		Stepsize of tics in degrees
; ****************************************************************************

FUNCTION PIA_MAPGRID_TICS, degmin, degmax, ntics, RA=ra

; conversion to minutes of degrees or to minutes of hours

	IF KEYWORD_SET(ra) THEN mult = 4.0 ELSE mult = 60.0

; approx. increment in minutes per tic step

	incr = (degmax-degmin)*mult/FLOAT(ntics)

; if still too small, calculate seconds

	IF incr LT 1.0 THEN BEGIN
		mult = mult * 60.0
		incr = incr * 60.0
	ENDIF

; determine optimum increment value

	CASE 1 OF
	incr GT 600.0  : incr = 1200.0      ; 20 degrees/hours
	incr GT 240.0  : incr = 600.0       ; 10 degrees/hours
	incr GT 120.0  : incr = 300.0       ;  5 degrees/hours
	incr GT  60.0  : incr = 120.0       ;  2 degrees/hours
	incr GT  30.0  : incr =  60.0       ;  1 degree/hour/minute
	incr GT  15.0  : incr =  30.0       ; 30 minutes/seconds
	incr GT  10.0  : incr =  15.0       ; 15 minutes/seconds
	incr GT   5.0  : incr =  10.0       ; 10 minutes/seconds
	incr GT   2.0  : incr =   5.0       ;  5 minutes/seconds
	incr GT   1.0  : incr =   2.0       ;  2 minutes/seconds
	ELSE           : incr =   1.0       ;  1 minute/second
	ENDCASE

	RETURN, incr/mult	; return increment in degrees
	END

; ****************************************************************************
; PIA_MAPGRID -- main routine
; ****************************************************************************

PRO PIA_MAPGRID, trans, area, ngrid, simple=simple

; some constants

	IF !D.NAME EQ "PS" THEN pscale=24 ELSE pscale=1	; PostScript scaling
	eps = 0.0001			; a bit less than half a second

; the image size -1 in device coordinates

	xsize = area(2)-area(0)
	ysize = area(3)-area(1)

; find approximate min/max values of image

	x = [ 0, 0, 0, xsize/2, xsize, xsize, xsize, xsize/2 ]
;	y = [ ysize, xsize/2, 0, 0, 0, ysize/2, ysize, ysize ]
	y = [ ysize, ysize/2, 0, 0, 0, ysize/2, ysize, ysize ]
	PIA_XY2RD, x, y, trans, lon, lat
	lonmin = MIN(lon)
	lonmax = MAX(lon)
	latmin = MIN(lat)
	latmax = MAX(lat)

; min/max of declination at left axis for label limits

	dmin = lat(0)<lat(1)<lat(2)
	dmax = lat(0)>lat(1)>lat(2)

; the hemishere

	IF trans.crval(1) GE 0.0 THEN hemi = 1 ELSE hemi = -1

; check if image includes pole

	pole = 0
	lon  = 0.0
	lat  = hemi * 90.0
	PIA_RD2XY, lon, lat, trans, x, y
	IF x GE 0 AND x LE xsize AND y GE 0 AND y LE ysize THEN BEGIN
		IF hemi EQ 1 THEN latmax = 90.0 ELSE latmin = -90.0
		lonmin = 0.0
		lonmax = 359.99
		pole   = 1
	ENDIF

; check the orientation of the coordinate lines

	lat0 = trans.crval(0)
	lon0 = trans.crval(1)
	PIA_RD2XY, lat0, lon0, trans, x, y, xr, xd, yr, yd
	orientation = (ABS(xr) GE ABS(yr))

; find grid increment values

	IF pole THEN londel = 30.0 $
		ELSE londel = PIA_MAPGRID_TICS(lonmin,lonmax,ngrid,/RA)
	latdel = PIA_MAPGRID_TICS(latmin,latmax,ngrid)

; draw grid lines of constant RA

	; the number of points per grid line
	step = 4 < (latmax-latmin)/10.	; limit step size to 4 degrees
	len  = FIX((latmax-latmin)/step+1)
	lati  = FINDGEN(len)*step+latmin
	IF lati(len-1) LT latmax THEN BEGIN
		lati = [lati,latmax]
		len = len+1
	ENDIF

	; formatting RA labels
	d = londel*4.0
	CASE 1 OF
		d MOD 60. EQ 0. : nl = 0
		d MOD 1.  EQ 0. : nl = 1
		ELSE            : nl = 2
	ENDCASE

	; loop through RA values
	first = lonmin-(lonmin MOD londel)
	FOR lon = first,lonmax,londel DO BEGIN

		; plot one grid line for constant RA
		loni = REPLICATE(lon,len)
		PIA_RD2XY, loni, lati, trans, x, y
		x = x+area(0)
		y = y+area(1)
		PLOTS, x, y, /DEVICE, CLIP=area, NOCLIP=0

		; attach a RA label
		d = lon/15.0
		IF d LT 0.0 THEN d = d+24.0
		label = STRING((SIXTY(d+eps))(0:nl), $
			FORMAT='(i3.2,"!Ah!N",i2.2,"!Am!N",i2.2,"!As!N")')
; simple map case
		IF KEYWORD_SET(simple) THEN label = ''
		
		IF pole THEN BEGIN
			; in this case we want to have labels only
			; at top and bottom of image
			IF hemi EQ 1 THEN lat=latmin ELSE lat=latmax
			PIA_RD2XY, lon, lat, trans, x, y
			CASE 1 OF
			y LT 0:		ylabel = -20*pscale
			y GT ysize:	ylabel = ysize+10*pscale
			ELSE:		ylabel = 0
			ENDCASE
			IF ylabel NE 0 THEN BEGIN
				xlabel = FIX(PIA_RY2X(lon,ylabel,trans))
				XYOUTS, xlabel+area(0), ylabel+area(1), $
					label, /DEVICE, ALIGN=0.5, $
					CHARSIZE=!P.CHARSIZE
			ENDIF
		ENDIF ELSE BEGIN
		  IF orientation THEN BEGIN
			; RA labels at top or bottom
			IF hemi EQ 1 THEN ylabel = ysize+10*pscale $
			             ELSE ylabel = -20*pscale
			xlabel = FIX(PIA_RY2X(lon,ylabel,trans))
			IF xlabel GE 0 AND xlabel LT xsize THEN $
				XYOUTS, xlabel+area(0), ylabel+area(1), $
					label, /DEVICE, ALIGN=0.5, $
					CHARSIZE=!P.CHARSIZE
		  ENDIF ELSE BEGIN
			; RA labels at left
			xlabel = -10*pscale
			ylabel = FIX(PIA_RX2Y(lon,xlabel,trans))
			IF ylabel GE 0 AND ylabel LT ysize THEN $
				XYOUTS, xlabel+area(0), ylabel+area(1), $
					label, /DEVICE, ALIGN=1.0, $
					CHARSIZE=!P.CHARSIZE
		  ENDELSE
		ENDELSE
	ENDFOR

; draw grid lines of constant DEC

	; the number of points per grid line
	step = 4 < (lonmax-lonmin)/10.	; limit step size to 4 degrees
	len  = FIX((lonmax-lonmin)/step+1)
	loni = FINDGEN(len)*step+lonmin
	IF loni(len-1) LT lonmax THEN BEGIN
		loni = [loni,lonmax]
		len = len+1
	ENDIF

	; formatting DEC labels
	d = latdel*60.0
	CASE 1 OF
		d MOD 60. EQ 0. : nl = 0
		d MOD 1.  EQ 0. : nl = 1
		ELSE            : nl = 2
	ENDCASE

	; in case of pole limit label plotting
	IF pole THEN BEGIN
		PIA_RD2XY, 0, 90*hemi, trans, x, y
		PIA_XY2RD, 0, y, trans, lon, lat_limit
	ENDIF

	; loop through DEC values
	first = latmin-(latmin MOD latdel)
	FOR lat = first,latmax,latdel DO BEGIN

		; plot one grid line for constant RA
		lati = REPLICATE(lat,len)
		PIA_RD2XY, loni, lati, trans, x, y
		x = x+area(0)
		y = y+area(1)
		PLOTS, x, y, /DEVICE, CLIP=area, NOCLIP=0

		; attach a DEC label
		IF lat LT 0.0 THEN sign = -1 ELSE sign = 1
		label = STRING( (SIXTY(ABS(lat)+eps))(0:nl), $
		 FORMAT='(i3.2,"!Ao!N",i2.2,"!A''!N",i2.2,"!A''''!N")')
; simple map case
		IF KEYWORD_SET(simple) THEN label = ''

		IF sign LT 0 THEN STRPUT, LABEL, '-', 0
		IF pole THEN BEGIN
		   ; limit labels to circles which cut the border
		   IF ABS(lat) LE ABS(lat_limit) THEN BEGIN
			; DEC labels at left
			xlabel = -10*pscale
			ylabel = FIX(PIA_DX2Y(lat,xlabel,trans))
			IF ylabel GE 0 AND ylabel LT ysize THEN $
				XYOUTS, xlabel+area(0), ylabel+area(1),  $
					label, /DEVICE, ALIGN=1.0, $
					CHARSIZE=!P.CHARSIZE
		   ENDIF
		ENDIF ELSE BEGIN
		   IF orientation THEN BEGIN
			; DEC labels at left
			xlabel = -10*pscale
			ylabel = FIX(PIA_DX2Y(lat,xlabel,trans))
			IF ylabel GE 0 AND ylabel LT ysize THEN $
				XYOUTS, xlabel+area(0), ylabel+area(1),  $
					label, /DEVICE, ALIGN=1.0, $
					CHARSIZE=!P.CHARSIZE
		   ENDIF ELSE BEGIN
			; DEC labels at bottom
			ylabel = -20*pscale
			xlabel = FIX(PIA_DY2X(lat,ylabel,trans))
			IF xlabel GE 0 AND xlabel LT xsize THEN $
				XYOUTS, xlabel+area(0), ylabel+area(1),  $
					label, /DEVICE, ALIGN=0.5, $
					CHARSIZE=!P.CHARSIZE
		   ENDELSE
		ENDELSE
	ENDFOR

; add a border around the image

	PLOTS, /DEVICE, $
		[area(0),area(2),area(2),area(0),area(0)], $
		[area(1),area(1),area(3),area(3),area(1)]

	RETURN 
	END
