;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPSET
;
; PURPOSE:
;	Define parameters for mapping of raster data
;	for given grid pixel size.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPSET, detector, pixsize, rota, rpid, cpos, ra, dec, roll, $
;			gridsize, x_coord, y_coord, trans
;
; INPUTS:
;	DETECTOR	'C1', 'C2', 'P1', 'P2', 'P3'
;
;	PIXSIZE		Pixel size [x,y] of the resulting grid in arcsec
;
;	ROTA		Rotation of the map axes [x,y] relative to sky
;			coordinates [-RA,DEC]. Rota increases counterclockwise.
;			Supply the overall roll angle of the raster to achieve
;			a map which is parallel to detector [y,z] axes.
;
;	RPID		The raster IDs of all chopper plateaux
;
;	CPOS		The chopper positions of all chopper plateaux
;
;	RA		The right ascension values of all chopper plateaux
;
;	DEC		The declination values of all chopper plateaux
;
;	ROLL		The roll angles of all chopper plateaux
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output arguments are not correct
;
; OUTPUTS:
;	GRIDSIZE	The total size [nx,ny] of the resulting grid points
;
;	X_COORD		The map x-coordinates of the detector pixels
;			The array dimension is [ n_pixel, n_plat ]
;			where n_pixel is the number of detector pixels in the
;			array and n_plat is the number of chopper plateax
;			in the data arrays.
;
;	Y_COORD		The map y-coordinates of the detector pixels
;			The array dimension is [ n_pixel, n_plat ]
;
; OPTIONAL OUTPUTS:
;	TRANS		The transformation parameters RA, DEC -> X, Y
;
; PROCEDURES USED: 
;	PIA Procedure(s):
;		pia_mktrans
;		pia_rd2xy
;
; COMMON BLOCKS:
;	(none)
;
; ASSUMPTIONS:
;	The exact definition of the roll angle and the orientation of
;	detector pixels is still uncertain to me!
;	Here I make the following assumptions:
;	(1) The detector coordinates [y,z] map directly onto the sky without
;	    having to be reflected to achieve a true image of the source.
;	    Ref.: TN 2069-81000DS/01
;	(2) The roll angle is the rotation angle of the detector axes [y,z]
;	    relative to the sky coordinates [-RA,DEC] and increases
;           counterclockwise.
;
; MODIFICATION HISTORY:
;	Written by: 		D. Skaley, MPIK/Heidelberg	  March 1995
;	Oct 95 Version 3.5
;	Provisory change. Only valid for P32 and rota=360.-avg(roll) 
;		until derivatives problem fixed (CG)		  March 1996
;	ROTA=avg(roll) ! probably after NEW definition change	  May 1996
;	No border size & gridsize adjusted for small jitter(CG)   June 1996		
;	CRVAL is the centre of the map, as given by (RA,Dec) + 
;	border size are computed from pixel size/aperture (JA)    April 1997
;	phtapert used now (CG)					  February 1998
;	V7.0
;	Changes for treating correctly P map data,
;	semi-pixels correctly treated (CG)			  May 1998
;	V7.2	
;	Shift_x & Shift_y keywords added 			February 1999
;	V7.4 
;-

PRO PIA_MAPSET, detector, pixsize, aper, rota, rpid, cpos, ra, dec, roll, $
		gridsize, x_coord, y_coord, trans, trans_ra, trans_dec, $
		ERROR=error, shift_x=shift_x, shift_y=shift_y

; info for apertures and check if defined
;@phtapert
;IF n_elements(p_area) eq 0 THEN READ_OMEGA
@piacmn_pow2fb
	IF N_ELEMENTS( aper_r ) EQ 0 THEN PIA_DEFAPER

; initialize error status

	error = 1

	IF NOT KEYWORD_SET(shift_x) THEN shift_x=0.
	IF NOT KEYWORD_SET(shift_y) THEN shift_y=0.
; check the input arguments

	; check detector ID
	; and define position offsets in the ISO frame of reference
	; relative to the detector center.
	; pixel numbering must be defined exactly as in the SPD product.
	detector = STRING(detector)
	CASE detector OF
	'C1':	BEGIN
		n_pixel = 9
		pixoff_y = [-1,-1,-1,0,0,0,+1,+1,+1] * 46.0	; arcsec
		pixoff_z = [-1,0,+1,-1,0,+1,-1,0,+1] * 46.0	; arcsec
		border_size = 0.			; additional border size
		detsiz = [46.,46.]
		END
	'C2':	BEGIN
		n_pixel = 4
		pixoff_y = [+1,-1,-1,+1] * 46.0			; arcsec
		pixoff_z = [-1,-1,+1,+1] * 46.0			; arcsec
		border_size = 0.			; additional border size
		detsiz = [92.,92.]
		END
	'P1':	BEGIN
		n_pixel = 1
		pixoff_y = [ 0.0 ]
		pixoff_z = [ 0.0 ]
		border_size = aper_r(0,aper)/(!pi/(180.*3600.))	; additional border size
		border_size=border_size
		detsiz = [border_size, border_size]
		END
	'P2':	BEGIN
		n_pixel = 1
		pixoff_y = [ 0.0 ]
		pixoff_z = [ 0.0 ]
		border_size = aper_r(0,aper)/(!pi/(180.*3600.)) ; additional border size
		border_size=border_size
		detsiz = [border_size, border_size]
		END
	'P3':	BEGIN
		n_pixel = 1
		pixoff_y = [ 0.0 ]
		pixoff_z = [ 0.0 ]
		border_size = aper_r(0,aper)/(!pi/(180.*3600.))	; additional border size
		border_size=border_size
		detsiz = [border_size, border_size]
		END
	'SS':	BEGIN
		n_pixel = 1
		pixoff_y = [ 0.0 ]
		pixoff_z = [ 0.0 ]
		border_size = 0.		; additional border size
		detsiz = [24.,24.]
		END
	'SL':	BEGIN
		n_pixel = 1
		pixoff_y = [ 0.0 ]
		pixoff_z = [ 0.0 ]
		border_size = 0.		; additional border size
		detsiz = [24.,24.]
		END	
	ELSE: GOTO, err_detector
	ENDCASE

	; check grid parameters
	IF N_ELEMENTS(pixsize) NE 2 THEN GOTO, err_pixsize
	IF N_ELEMENTS(rota) NE 1 THEN GOTO, err_rota

	; check data arrays
	n_plat = N_ELEMENTS(cpos)
	IF n_plat LE 0 THEN GOTO, err_array
	IF N_ELEMENTS(ra) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS(dec) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS(roll) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS(rpid) NE n_plat*2 THEN GOTO, err_array

; select a sky reference position (somewhere in the middle of the s/c raster)
; preliminary map reference position is [0,0]

;	imin_ra = MIN( rpid(0,*), MAX=imax_ra )
;	imin_dec = MIN( rpid(1,*), MAX=imax_dec )
;	ih_ra = (imin_ra+imax_ra)/2
;	ih_dec = (imin_dec+imax_dec)/2
;	index = WHERE( rpid(0,*) EQ ih_ra AND $
;	               rpid(1,*) EQ ih_dec, n)
;	IF n EQ 0 THEN MESSAGE, "Wrong raster IDs"
	
	min_ra = MIN(ra, max=max_ra)
	min_dec = MIN(dec, max=max_dec) 
	
	ra_cen = (max_ra + min_ra ) / 2.
	dec_cen = (max_dec + min_dec) / 2.

; define the transformation parameters RA/DEC -> map x/y

;	trans = PIA_MKTRANS([ra(index(0)),dec(index(0))],[0,0],pixsize,rota)
;	trans_ra = ra(index(0))
;	trans_dec = dec(index(0))
	trans = PIA_MKTRANS([ra_cen, dec_cen], [0.,0.],pixsize, rota)

; for mapping just one position change (like in chopped mode on vs off) 
; take into account that central map position is changed by chopper

	IF N_ELEMENTS(cpos) EQ 1. THEN begin
	   diff = cpos / pixsize(0)
	   pia_xy2rd, -diff, 0., trans, ra_cen,dec_cen
	   trans.crval = [ra_cen, dec_cen]
	ENDIF  

	trans_ra = ra_cen
	trans_dec = dec_cen
;	print,ra_cen, dec_cen

; calculate for all s/c raster points the map coordinates and partial deriv.
; (here we have for each chopper plateau the RA,DEC values, which is a bit
;  too much calculation. but anyway...)

	PIA_RD2XY, ra, dec, trans, x_raster, y_raster, xr, xd, yr, yd

; convert partial derivatives to unit [pixel_of_map/arcsec]

        xr = xr / 3600.
        xd = xd / 3600.
        yr = yr / 3600.
        yd = yd / 3600.
 
; the chopper direction in RA/DEC coordinates for all s/c raster points

; +cos replaces -cos here (CG) - roll used was in degrees, changed to radians!!!
	arcroll = roll * !dpi / 180.
	d_ra  = +cos(arcroll)
	d_dec = -sin(arcroll)

; the map x/y components of one arcsec chopper throw for all s/c raster points
; (in our case for all chopper plateaux; again a bit too much calculation...)

; original    	d_x = xr*d_ra + xd*d_dec 
; and 		d_y = yr*d_ra + yd*d_dec
; changed after testing, although I do not know where the factor 2 is missed!

	d_x = xr*d_ra + 2*xd*d_dec
	d_y = yr*d_ra + 2*yd*d_dec
	
; provisory change, only valid for P32 and rota = roll
	d_x = d_x * 0. - 1. / pixsize(0)
	d_y = d_y * 0. 

 ; the x/y map coordinates of all chopper plateaux, center detector array

; Here I assume negative direction for the chopper to be consistent with
; mirroring the pixels in x direction (CG) - changed again after testing 
	x_chopper = x_raster + cpos*d_x    
	y_chopper = y_raster + cpos*d_y    

; the x/y offsets in the map of all pixels for each s/c raster point
; for rota == roll: s/c [Y,Z] coords. correspond to the map [x,y] coords.
; generally:
; the s/c Y-direction points to map direction [dx,dy] = [+d_x,+d_y]
; the s/c Z-direction points to map direction [dx,dy] = [-d_y,+d_x]

	x_pixoff = pixoff_y # [d_x] + pixoff_z # [d_y]
; I do not understand this bit, then it could be zero for chopper but mus be
; something for offset
;	y_pixoff = pixoff_y # d_y - pixoff_z # d_x
	d_dy = [d_y] * 0. - 1. / pixsize(1)
	y_pixoff = -pixoff_z # [d_dy]

; the x/y map coordinates of all detector pixels

	x_coord = REPLICATE(1,n_pixel) # [x_chopper] + x_pixoff
	y_coord = REPLICATE(1,n_pixel) # [y_chopper] + y_pixoff

; the extent of the grid

	x_min = MIN( x_coord, MAX = x_max ) 
	y_min = MIN( y_coord, MAX = y_max ) 
; add a few pixels to the left/right/bottom/top of map

;	x_extra = FIX(.5*border_size/pixsize(0))
;	y_extra = FIX(.5*border_size/pixsize(1))
	x_extra = (.5*border_size/pixsize(0)) + shift_x
	y_extra = (.5*border_size/pixsize(1)) + shift_y
	x_min = x_min - x_extra
	x_max = x_max + x_extra
	y_min = y_min - y_extra
	y_max = y_max + y_extra

; shift the grid so that the lower left corner is [0,0]

	trans.crpix = [ -x_min, -y_min ]
	x_coord = x_coord - x_min
	y_coord = y_coord - y_min

; try to get crpix exactly in the middle of the map
	x_med = (x_min+x_max) / 2.
	y_med = (y_min+y_max) / 2.
;	trans.crpix = trans.crpix + [x_med,y_med]

	add = (detsiz/pixsize) > [1,1]
	trans.crpix = trans.crpix + [x_med,y_med] + (add-1.) / 2.
	x_coord = x_coord + x_med
	y_coord = y_coord + y_med
; the grid size
	;gridsize = CEIL( [ x_max-x_min+0.5, y_max-y_min+0.5 ] )
	gridsize = $
ROUND( [ x_max-x_min+detsiz(0)/pixsize(0),y_max-y_min+detsiz(1)/pixsize(1)] )
	

; normal return

	error = 0
	RETURN
err_detector:
	MESSAGE, /INFO, "Wrong detector code: "+detector
	RETURN

err_pixsize:
	MESSAGE, /INFO, "Wrong input argument: PIXSIZE"
	RETURN

err_rota:
	MESSAGE, /INFO, "Wrong input argument: ROTA"
	RETURN

err_array:
	MESSAGE, /INFO, "Inconsistent input array dimensions"
	RETURN

	END
