;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPSIMPLE_AAP
;
; PURPOSE:
;	Produce a simple map of AAP data for PHT-C or for PHT-P
;	raster data ignoring true pointing.
;
;	Be sure to call this procedure not for any other data types.
;	No checks are performed on this.
;
;	This routine operates on a PHTAAP structure containing one measurement.
;	Unpacks the arrays from handles and passes them to the mapping routine.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPSIMPLE_AAP, aap, hbright, pixselect, $
;			map_bright, map_uncert, map_expose, pixsize, trans
;
; INPUTS:
;	AAP		The PHT structure (arrays are stored with handles)
;
;       HBRIGHT         Handle containing the brightness array
;
;	PIXSELECT	A 4 or 9 element array of logical values (0 or 1)
;			indicating whether an array pixel should be selected
;			for processing (not used for PHT-P).
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output is not valid
;
;	PCHOP		For chopped P crude mapping 
;
; OUTPUTS:
;	MAP_BRIGHT	The brightness map
;
;	MAP_UNCERT	The uncertainty map
;
;	MAP_EXPOSE	The exposure map
;
;	PIXSIZE		The map pixel size [x,y]
;
;	TRANS		Transformation structure from PIA_MKTRANS
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_mapsimple_p32
;		pia_mapsimple_phtp
;		pia_mktrans
;	Astronomical Library:
;		fxpar
;
; COMMON BLOCKS:
;	(none)
;
; MODIFICATION HISTORY:
;	Written by: 		D. Skaley, MPIK/Heidelberg	March 1995
;	Oct 95 Version 3.5
;	Extension for P chopped maps (CG)			January 1995
;	Flat fielding added (CG)				March 1996
;	Allowing for ffield=2 (CG)				October 1997
;	V6.5
;-

PRO PIA_MAPSIMPLE_AAP, aap, hbright, pixselect, $
		map_bright, map_uncert, map_expose, $
		pixsize, trans, FFIELD=ffield, ERROR=error, PCHOP=pchop

; initialize error status

	error = 1

	IF not KEYWORD_SET(ffield) THEN ffield = 0
; get some scalar parameters from the PHTAAP structure

	detector = aap.det
	dettype = STRMID(detector,0,1)
	aperture = aap.aper

; get some parameters from the header

	HANDLE_VALUE, aap.hdr, hdr, /NO_COPY

	keyword = "ATTRA"
	ra = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTDEC"
	dec = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTRROTA"
	rota = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTRNPTS"
	npts = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTRNLNS"
	nlns = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTRDPTS"
	dpts = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

	keyword = "ATTRDLNS"
	dlns = FXPAR( hdr, keyword, COUNT=count )
	IF count LE 0 THEN BEGIN
		MESSAGE, /INFO, 'Error: Missing keyword '+keyword
		GOTO, leave1
	ENDIF

; get the arrays from the PHTAAP structure

	HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
	HANDLE_VALUE, aap.cpos, cpos, /NO_COPY
	HANDLE_VALUE, aap.dwel, dwel, /NO_COPY
	HANDLE_VALUE, aap.mnbu, uncert, /NO_COPY

; get a copy of the flags array (will be modified)

	HANDLE_VALUE, aap.flag, flag

; get the brightness array

        HANDLE_VALUE, hbright, bright, /NO_COPY

; produce the simple maps ignoring true pointing

	IF KEYWORD_SET(PCHOP) THEN BEGIN

		PIA_MAPSIMPLE_CHOP, detector, [npts,nlns], [dpts,dlns], $
			rpid, cpos, dwel, flag, bright, uncert, $
			map_bright, map_uncert, map_expose, pixsize, $
			ERROR=err, aperture=aap.aper

	ENDIF ELSE BEGIN

	 CASE dettype OF
	 'C':	BEGIN
		index = WHERE( pixselect EQ 0, count )
		IF count GT 0 THEN flag(index,*) = flag(index,*) OR 2
		PIA_MAPSIMPLE_P32, detector, [npts,nlns], [dpts,dlns], $
			rpid, cpos, dwel, flag, bright, uncert, $
			map_bright, map_uncert, map_expose, pixsize, $
			FFIELD=ffield, ERROR=err
		END
	 'P':	PIA_MAPSIMPLE_PHTP, detector, [npts,nlns], [dpts,dlns], $
			aperture, rpid, dwel, flag, bright, uncert, $
			map_bright, map_uncert, map_expose, pixsize, $
			ERROR=err
	 ELSE:	err = 1
	 ENDCASE
	ENDELSE
	
	IF err THEN GOTO, leave2

; determine coordinate transformation parameters

	msize = SIZE(map_bright)
	middle = msize(1:2)/2. - 0.5
	trans = PIA_MKTRANS([ra,dec],middle,pixsize,rota)

; everything ok.
	
	error = 0

leave2:

; move data arrays back into PHTAAP structure

	HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
	HANDLE_VALUE, aap.cpos, cpos, /SET, /NO_COPY
	HANDLE_VALUE, aap.dwel, dwel, /SET, /NO_COPY
	HANDLE_VALUE, aap.mnbu, uncert, /SET, /NO_COPY

; move brightness array back into handle

        HANDLE_VALUE, hbright, bright, /SET, /NO_COPY

leave1:

; move the header back into PHTAAP structure

	HANDLE_VALUE, aap.hdr, hdr, /SET, /NO_COPY

	RETURN
	END
