;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPSIMPLE_P32
;
; PURPOSE:
;	Produce a simple map from P32 SPD data (PHT-C).
;	The simplifications are:
;
;	(1) True spacecraft pointing is ignored: It is assumed that the
;	    spacecraft raster is a perfect regular grid.
;
;	(2) The spacecraft raster is assumed to coincide with multiples of
;	    1 or 1/2 or 1/3 of the detector pixel size. In y-direction it is
;	    also assumed to coincide with the chopper amplitude.
;
;	The resulting maps are produced in the frame of reference [x,y] which
;	coincides with the spacecraft axes [Y,Z].
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPSIMPLE_P32, detector, n_raster, dist_raster, $
;			rpid, cpos, dwel, flag, bright, uncert, $
;			map_bright, map_uncert, map_expose, pixsize
;
; INPUTS:
;	DETECTOR	'C1' or 'C2'
;
;	N_RASTER	Number of spacecraft raster points
;			(0) = Y-direction, (1) = Z-direction
;
;	DIST_RASTER	The distance of the raster points/lines [arcsec]
;			(0) = Y-direction, (1) = Z-direction
;
;	RPID		Raster point IDs
;			rpid(0,*) = sequence number in +Y direction
;			rpid(1,*) = sequence number in -Z direction
;
;	CPOS		Chopper positions [arcsec]
;
;	DWEL		Chopper dwell times
;
;	FLAG		Flags array
;
;	BRIGHT		Brightness density array [MJy/sr]
;
;	UNCERT		Uncertainties array of brightness density [MJy/sr]
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no errors
;			1 = error: maps were not produced
;
; OUTPUTS:
;	MAP_BRIGHT	Brightness map
;
;	MAP_UNCERT	Uncertainty map
;
;	MAP_EXPOSE	Exposure map
;
;	PIXSIZE		The pixel size of the maps in [x,y] directions
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		ggt
;
; COMMON BLOCKS:
;	(none)
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg		March 1995
;	Modified by:	C. Gabriel, ESA/ESTEC			April 1995
;	Oct 95 Version 3.5
;	Raster distance = 0 allowed (case Nx1 raster) (CG)	December 1995
;	y-direction changed to have righthand coordinates (CG)	January 1996 
;	Flat fielding option added (CG)				February 1996
;	Dwell time units bug fixed (CG)				March 1996
;	Back to 45 and 90 arcsec  (CG)				May 1996
;	Call to median filtering, smoothing and 
;	 background flat-fielding added (CG)			October 1997
;	Points weighted with uncertainties (CG)			October 1997
;	V6.5
;-

PRO PIA_MAPSIMPLE_P32, detector, n_raster, dist_raster, $
		rpid, cpos, dwel, flag, bright, uncert, $
		map_bright, map_uncert, map_expose, pixsize, $
		FFIELD=FFIELD, ERROR=error

; initialize error status

	error = 1

;*****************************************************************************
; the pixel numbering scheme in SPD products is as follows:
;
;    detector index for             frame of reference
;    C100:        C200:             of spacecraft:
;
;    3 6 9        3 4               +z
;    2 5 8        2 1               ^
;    1 4 7                          |
;                                   0--> +y (chopper)
;
;*****************************************************************************

; check 'detector' and define detector constants

	detector = STRING(detector)
	CASE detector OF
	'C1':	BEGIN
		n_pixel = 9			; num of pixels in array
		array_size = [ 3, 3 ]		; num of pixels in Y/Z dir
		detector_size = [ 45., 45. ]	; detector pixel size in arcsec
		pixel_pos_y = [ 2, 2, 2, 1, 1, 1, 0, 0, 0 ]	; relative pos
		pixel_pos_z = [ 0, 1, 2, 0, 1, 2, 0, 1, 2 ]	; to lower-left
		chopper_increment = 15.0	; expected ch incr in arcsec
		END
	'C2':	BEGIN
		n_pixel = 4			; num of pixels in array
		array_size = [ 2, 2 ]		; num of pixels in Y/Z dir
		detector_size = [ 90., 90. ]	; detector pixel size in arcsec
		pixel_pos_y = [ 0, 1, 1, 0 ]	; relative position
		pixel_pos_z = [ 0, 0, 1, 1 ]	; to lower-left pixel
		chopper_increment = 30.0	; expected ch incr in arcsec
		END
	ELSE:	GOTO, err_detector
	ENDCASE

; check raster parameters

	IF N_ELEMENTS(n_raster) NE 2 OR MIN(n_raster) LE 0 THEN GOTO,err_nrast

; N x 1 raster are possible, in that case the dist_raster is 0! 
	IF N_ELEMENTS(dist_raster) NE 2 OR MIN(dist_raster) LT 0 THEN $
		GOTO,err_drast

; check consistency of array sizes

	n_plat = N_ELEMENTS( cpos )
	IF N_ELEMENTS( rpid ) NE 2*n_plat THEN GOTO, err_array
	IF N_ELEMENTS( dwel ) NE n_plat THEN GOTO, err_array
	n_data = n_pixel*n_plat
	IF N_ELEMENTS( flag ) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS( bright ) NE n_data THEN GOTO, err_array
	IF N_ELEMENTS( uncert ) NE n_data THEN GOTO, err_array

; the chopper step size determins the map grid resolution in s/c Y-direction

	pixsize_y = chopper_increment		; the map grid size in Y
	chopper_shift = ROUND(-cpos/pixsize_y)	; integers of chopper posns
	ch_min = MIN(chopper_shift, MAX=ch_max)	; the chopper range
	chopper_shift = chopper_shift - ch_min	; shift ch.range, so that min=0

; the raster distance in s/c Z-direction determins the map in s/c Z-direction
; in units of the detector pixel size, it can be: 1/3, 1/2, 1, 3/2, 2
; i.e a multiple of 1/6.

	detector_sixth = detector_size(1)/6.
	rounded_dist = ROUND(dist_raster(1)/detector_sixth)*detector_sixth
	IF rounded_dist NE 0. THEN $
	 pixsize_z = GGT(detector_size(1),rounded_dist) ELSE $
	 pixsize_z = detector_size(1)

; the grid parameters

	; the map pixel size in arcsec
	pixsize = [ pixsize_y, pixsize_z ]

	; the number of grid points covered by one detector pixel
	cover = ROUND(detector_size/pixsize)

	; the distance for adjacent raster points in units of map points
	raster_shift = ROUND(dist_raster/pixsize)

	;the number of chopper positions for one raster point - 1
	chopper_steps = [ ch_max-ch_min, 0 ]

	; the dimensions of the resulting map
	dim = array_size*cover + raster_shift*(n_raster-1) + chopper_steps

	; the ratio of map pixel (map point area) over detector pixel area
	ratio = 1./(cover(0)*cover(1))

	; the distance of adjacent detector pixels in units of map points
	pixel_shift_y = pixel_pos_y * cover(0)
	pixel_shift_z = pixel_pos_z * cover(1)

; the weights [seconds] (expand array to data arrays dimension)

	weight = TRANSPOSE(REBIN( dwel*ratio, $
				n_plat, n_pixel, /SAMPLE ))
	ind = where(uncert NE 0., nind)
	IF nind GT 0 THEN $
	weight(ind) = weight(ind) / (uncert(ind) / bright(ind))^2

; initialize the resulting arrays

	map_bright = FLTARR(dim(0),dim(1))		; initialized with 0.
	map_uncert = FLTARR(dim(0),dim(1))		; initialized with 0.
	map_expose = FLTARR(dim(0),dim(1))		; initialized with 0.

; the S/C raster point numbers in map orientation

	raster_y = MAX(rpid(0,*))-rpid(0,*)
	raster_z = MAX(rpid(1,*))-rpid(1,*)

; The following index arrays have the same dimension as the input arrays,
; i.e. for each input datum there is an index pair.
; The values are entries into the resulting map array.
; [y_low,z_low] is the map pixel covered by the lower left corner of the
; detector pixel.
; [y_low,z_low]+cover is the map pixel just not covered by the upper right
; corner of the detector pixel.

	y_low = REPLICATE(1,n_pixel) # $
		(raster_y*raster_shift(0) + chopper_shift) $
		+ pixel_shift_y # REPLICATE(1,n_plat)

	z_low = REPLICATE(1,n_pixel) # raster_z*raster_shift(1) $
		+ pixel_shift_z # REPLICATE(1,n_plat)

	tmp_bright = bright
	tmp_uncert = uncert
	
	IF KEYWORD_SET(ffield) THEN $
	   IF ffield EQ 1 THEN BEGIN
		ffmap_simple, dim, y_low, z_low, cover, dwel, $
			ratio, tmp_bright, tmp_uncert, flag, ff, error=error
		IF error EQ 0 THEN $
			PIA_DISP_PRINT_FILE, '',TITLE='Flat field calculation',$
				TEXT=['The calculated flat field parameters:',$
				'Pixel '+$
				STRTRIM(STRING(indgen(n_pixel)+1),2)+':' +  $
				STRING(ff)] $
		ELSE PRINT, 'FF error found' 
	   ENDIF ELSE IF ffield EQ 2 THEN BEGIN
	      set_colours
	      pia_ff_med_smooth, bright, uncert, tmp_bright, flag=flag
	   ENDIF 
				
	
; build the maps of brightness, uncertainty, and exposure
	FOR y = 0, dim(0)-1 DO $
	 FOR z = 0, dim(1)-1 DO BEGIN
	  index = WHERE( (y_low LE y AND y_low GT y-cover(0)) AND $
		         (z_low LE z AND z_low GT z-cover(1)) AND $
	                 (flag EQ 0) $
	                , nfound )
	  IF nfound GT 0 THEN BEGIN
	   map_expose(y,z) = FLOAT( TOTAL(weight(index)))
	   map_bright(y,z) = FLOAT( TOTAL(weight(index)*tmp_bright(index)) $
	                            / map_expose(y,z) )
	   map_uncert(y,z) = FLOAT( SQRT(TOTAL(weight(index)*tmp_uncert(index)^2)) $
	                           / map_expose(y,z) )
	  ENDIF
	 ENDFOR

; no need to transpose maps, since s/c [Y,Z] coords are identical to [x,y]

	error = 0
	RETURN

err_detector:
	MESSAGE, /INFO, "Wrong detector code: "+detector
	RETURN

err_nrast:
	MESSAGE, /INFO, "Wrong input argument: N_RASTER"
	RETURN

err_drast:
	MESSAGE, /INFO, "Wrong input argument: DIST_RASTER"
	RETURN

err_array:
	MESSAGE, /INFO, "Inconsistent input array dimensions"
	RETURN

	END
