;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPSIMPLE_PHTP
;
; PURPOSE:
;	Produce a simple map from PHT-P SPD raster data (PHT-P).
;	Chopper positions (other than 0) are not considered, since raster
;	data are always taken in staring mode. For AOT P32 an individual
;	routine exists for making a 'simple' map.
;
;	The simplification is:
;
;	(1) True spacecraft pointing is ignored: It is assumed that the
;	    spacecraft raster is a perfect regular grid.
;
;	The resulting maps are produced in the frame of reference [x,y] which
;	coincides with the spacecraft axes [Y,Z].
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_MAPSIMPLE_PHTP, detector, n_raster, dist_raster, aperture, $
;			rpid, dwel, flag, bright, uncert, $
;			map_bright, map_uncert, map_expose, pixsize
;
; INPUTS:
;	DETECTOR	'P1', 'P2' or 'P3'
;
;	N_RASTER	Number of spacecraft raster points
;			(0) = Y-direction, (1) = Z-direction
;
;	DIST_RASTER	The distance of the raster points/lines [arcsec]
;			(0) = Y-direction, (1) = Z-direction
;
;	APERTURE	Aperture ID for the PHT-P measurement
;
;	RPID		Raster point IDs
;			rpid(0,*) = sequence number in +Y direction
;			rpid(1,*) = sequence number in -Z direction
;
;	DWEL		Chopper dwell times [2^-7 seconds]
;
;	FLAG		Flags array
;
;	BRIGHT		Brightness density array [MJy/sr]
;
;	UNCERT		Uncertainties array of brightness density [MJy/sr]
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no errors
;			1 = error: maps were not produced
;
; OUTPUTS:
;	MAP_BRIGHT	Brightness map
;
;	MAP_UNCERT	Uncertainty map
;
;	MAP_EXPOSE	Exposure map
;
;	PIXSIZE		The pixel size of the maps in [x,y] directions
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_defaper
;
; COMMON BLOCKS:
;		piacmn_pow2fb
;
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK/Heidelberg	March 1995
;	Modified by:	C. Gabriel, ESA/VILSPA		July 1995
;	Oct 95 Version 3.5
;	Y-Raster direction turned around to get right hand coordinates
;							February 1996
;	Points weighted with uncertainties (CG)			October 1997
;	V6.5
;-

PRO PIA_MAPSIMPLE_PHTP, detector, n_raster, dist_raster, aperture, $
		rpid, dwel, flag, bright, uncert, $
		map_bright, map_uncert, map_expose, pixsize, $
		ERROR=error

; common block holding aperture constants for PHT-P

@piacmn_pow2fb

; load aperture constants into common block (first call only)

	IF N_ELEMENTS( aper_r ) EQ 0 THEN PIA_DEFAPER

; initialize error status

	error = 1

; check 'detector'

	detector = STRING(detector)
	CASE detector OF
	'P1':
	'P2':
	'P3':
	ELSE: GOTO, err_detector
	ENDCASE

; check raster parameters

	IF N_ELEMENTS(n_raster) NE 2 OR MIN(n_raster) LE 0 THEN GOTO,err_nrast
	IF N_ELEMENTS(dist_raster) NE 2 OR MIN(dist_raster) LE 0 THEN $
		GOTO,err_drast

; check consistency of array sizes

	n_plat = N_ELEMENTS( dwel )
	IF N_ELEMENTS( rpid ) NE 2*n_plat THEN GOTO, err_array
	IF N_ELEMENTS( flag ) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS( bright ) NE n_plat THEN GOTO, err_array
	IF N_ELEMENTS( uncert ) NE n_plat THEN GOTO, err_array

; the grid parameters

	; the dimensions of the resulting map
	dim = n_raster

	; the map pixel size in arcsec
	pixsize = dist_raster

	; aperture area size (arcsec^2)
	aperture_area = aper_r(2,aperture) * (!RADEG * 3600.)^2

	; the ratio of map pixel (map point area) over detector pixel area
	ratio = (pixsize(0)*pixsize(1))/aperture_area

; the weights [seconds]

	weight = dwel*(ratio*2.0D0^(-7))
	ind = where(uncert NE 0., nind)
	IF nind GT 0 THEN $
	weight(ind) = weight(ind) / (uncert(ind) / bright(ind))^2

; initialize the resulting arrays

	map_bright = FLTARR(dim(0),dim(1))		; initialized with 0.
	map_uncert = FLTARR(dim(0),dim(1))		; initialized with 0.
	map_expose = FLTARR(dim(0),dim(1))		; initialized with 0.

; the indexes into the data arrays

	y_index = MAX(rpid(0,*))-rpid(0,*)
	z_index = MAX(rpid(1,*))-rpid(1,*)

; build the maps of brightness, uncertainty, and exposure

	FOR y = 0, dim(0)-1 DO $
	 FOR z = 0, dim(1)-1 DO BEGIN
	  index = WHERE( ((y_index EQ y) AND (z_index EQ z )) AND $
	                 (flag AND 1) EQ 0 $
	                , nfound )
	  IF nfound GT 0 THEN BEGIN
	   map_expose(y,z) = FLOAT( TOTAL(weight(index)))
	   map_bright(y,z) = FLOAT( TOTAL(weight(index)*bright(index)) $
	                            / map_expose(y,z) )
	   map_uncert(y,z) = FLOAT( SQRT(TOTAL(weight(index)*uncert(index)^2)) $
	                           / map_expose(y,z) )
	  ENDIF
	 ENDFOR

; no need to transpose maps, since s/c [Y,Z] coords are identical to [x,y]

	error = 0
	RETURN

err_detector:
	MESSAGE, /INFO, "Wrong detector code: "+detector
	RETURN

err_nrast:
	MESSAGE, /INFO, "Wrong input argument: N_RASTER"
	RETURN

err_drast:
	MESSAGE, /INFO, "Wrong input argument: DIST_RASTER"
	RETURN

err_array:
	MESSAGE, /INFO, "Inconsistent input array dimensions"
	RETURN

	END
