;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MAPW
;
; PURPOSE:
;	Widget user interface to process maps for rastered images.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_MAPW
;
; INPUTS:
;	(none)
;
; KEYWORD PARAMETERS:
;	GROUP		The group leader of this widget hirarchy.
;
; OUTPUTS:
;	(none)
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		decode_admn
;		pia_aapview
;		pia_disp_hdr
;		pia_mapaap
;		pia_mapdef
;		pia_mapdraw
;		pia_mapdrift_p32
;		pia_mapsimple_aap
;		pia_mktrans
;		pia_open_iaap
;	Astronomical Library:
;		fxpar
;	Internal:
;		pia_mapw_drift_event
;		pia_mapw_drift
;		pia_mapw_defmap_event
;		pia_mapw_defmap
;		pia_mapw_cleanup
;		pia_mapw_event
;
; COMMON BLOCKS:
;	(none)
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	March 1995
;	Modified by:	C. Gabriel, ESA/SAI		April 1995
;	Oct 95 Version 3.5
;	
;	setting colour table automatically (CG)		December 1996
;	V5.5
;	Using chosen directory as default for further 
;	 searches (both by FITS & internal) (CG)	June 1997
;	V6.4
;	Call to median filtering, smoothing and 
;	 background flat field added (CG)		September 1997
;	V6.5
;	Map division and map subtraction added (CG)	
;	(also ROTA from header)				November 1997
;	V6.6
;	Array rotation for explicit given raster rotation
;	(ATTRORIE=0) computed correctly	
;	Tag APER added to structure PAR (CG)		June 1998
;	V7.2  
;	V7.3
;	Possibility for Shift in x and y added (CG)	February 1999
;	Replacing pia_pickfile1 (CG)			February 1999
;	Use of coded bitflag system (CG)		May 1999
;	V7.4 
;	Intrinsic noise calculation added for undefined
;	 map noise (CG)					August 1999
;	V8.0
;	Percentile configurable (CG)			May 2002
;	V10.0
;		
;-


; ***************************************************************************
; PIA_MAPW_DRIFT_EVENT  -  event handling routine for PIA_MAPW_DRIFT
; ***************************************************************************

PRO PIA_MAPW_DRIFT_EVENT, event

	WIDGET_CONTROL, event.id, GET_UVALUE=action

	CASE action OF

	; accept correection: store corrected array to original handle
	'ACC':	BEGIN
		WIDGET_CONTROL, event.top, GET_UVALUE=save
		HANDLE_VALUE, save.hcorr, corr, /NO_COPY
		HANDLE_VALUE, save.hbright, corr, /SET, /NO_COPY
		WIDGET_CONTROL, event.top, /DESTROY
		END

	; discard correction
	'DIS':	WIDGET_CONTROL, event.top, /DESTROY

	ELSE:
	ENDCASE

	RETURN
	END

PRO PIA_MAPW_PICKFILT, hdr, list

	n_ele = fxpar(hdr, 'NAXIS3')
	if n_ele gt 0 then begin
	filt_list = [' ']
	for i =0, n_ele - 1 do begin
	keyword = 'FILTER' + strtrim(i+1, 2)
	temp = fxpar(hdr, keyword)
	filt_list = [filt_list, temp]
	endfor
	filt_list = filt_list(1:n_ele)
	end else begin
	temp = fxpar(hdr, 'FILTER1')
        filt_list = [ temp ]
	endelse
	junkbase = widget_base(title='select the filter',/column,xsize = 300)
	junklist = widget_list(junkbase, value=filt_list, uvalue='list', $
	ysize=n_ele)
	widget_control, junkbase, /realize
	event = widget_event(junkbase)
	widget_control, event.id, get_uvalue=uvalue
	list = event.index
	widget_control, junkbase, /destroy
END

; ***************************************************************************
; PIA_MAPW_DRIFT  -  drift correction for P32
; ***************************************************************************

PRO PIA_MAPW_DRIFT, hpar, haap, hbright, GROUP=group

	; get program parameters
	HANDLE_VALUE, hpar, par

	id = { $
		det:0L,		$
		drift:0L,	$
		nfit:0L		$
	}

	id_base = WIDGET_BASE( TITLE="Drift processing for AOT P32", /COLUMN,$
		/modal, group=group )

	id_menu = WIDGET_BASE( id_base, /ROW )
	id_0 = WIDGET_BUTTON( id_menu, VAL="     Accept    ", UVA="ACC" )
	id_0 = WIDGET_BUTTON( id_menu, VAL="     Discard    ", UVA="DIS" )

	id_panel = WIDGET_BASE( id_base, /ROW )

	id_0 = WIDGET_BASE( id_panel, /COLUMN )
	id_1 = WIDGET_LABEL( id_0, VALUE="Detector:")

	CASE par.detector OF
	'C1': detpix = ['C100.1', 'C100.2', 'C100.3', $
			'C100.4', 'C100.5', 'C100.6', $
			'C100.7', 'C100.8', 'C100.9' ]
	'C2': detpix = ['C200.1', 'C200.2', 'C200.3','C200.4']
	ELSE: detpix = [ 'Error', ' - ', 'unknown','detector' ]
	ENDCASE
	id.det = WIDGET_LIST( id_0, YSIZE=9, value=detpix )

	id_0 = WIDGET_BASE( id_panel, /COLUMN )
	id_1 = WIDGET_LABEL( id_0, VALUE="Drift rates:           ")
	id.drift = WIDGET_LIST( id_0, YSIZE=9 )

	id_0 = WIDGET_BASE( id_panel, /COLUMN )
	id_1 = WIDGET_LABEL( id_0, VALUE="# points:")
	id.nfit = WIDGET_LIST( id_0, YSIZE=9 )

	; create a handle for storing the corrected array
	hcorr = HANDLE_CREATE()

	; realize the widget
	WIDGET_CONTROL, id_base, /REALIZE, /HOURGLASS, $
		SET_UVALUE={ hbright:hbright, hcorr:hcorr }


	; calculate the drifts

	; get arrays from the aap structure
	HANDLE_VALUE, haap, aap
	HANDLE_VALUE, aap.time, time, /NO_COPY
	HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
	HANDLE_VALUE, aap.cpos, cpos, /NO_COPY
	HANDLE_VALUE, aap.dwel, dwel, /NO_COPY
	HANDLE_VALUE, aap.flag, flag, /NO_COPY

	; get the locally stored brightness array
	HANDLE_VALUE, hbright, bright, /NO_COPY

	; calculate corrected brightness array
	PIA_MAPDRIFT_P32, par.detector, $
		[ par.npts, par.nlns ], [ par.dpts, par.dlns ], $
		time, rpid, cpos, dwel, flag, bright, $
		corr, n_fit, drift_rate, ERROR=err

	; put back arrays into aap
	HANDLE_VALUE, aap.time, time, /SET, /NO_COPY
	HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
	HANDLE_VALUE, aap.cpos, cpos, /SET, /NO_COPY
	HANDLE_VALUE, aap.dwel, dwel, /SET, /NO_COPY
	HANDLE_VALUE, aap.flag, flag, /SET, /NO_COPY

	; store back original and corrected brightness arrays
	HANDLE_VALUE, hbright, bright, /SET, /NO_COPY
	HANDLE_VALUE, hcorr, corr, /SET, /NO_COPY

	; show drift rates in widget
	WIDGET_CONTROL, id.drift, SET_VALUE=STRING(drift_rate)
	WIDGET_CONTROL, id.nfit, SET_VALUE=STRING(n_fit)


	; wait for pressing a button
	xMANAGER,No_BLOCK=1, "PIA_MAPW_DRIFT", id_base, GROUP=group

	; remove the corrected array handle
	HANDLE_FREE, hcorr

	RETURN
	END

; ***************************************************************************
; PIA_MAPW_DEFMAP_EVENT  -  event handling routine for PIA_MAPW_DEFMAP
; ***************************************************************************

PRO PIA_MAPW_DEFMAP_EVENT, event

	WIDGET_CONTROL, event.id, GET_UVALUE=action
	WIDGET_CONTROL, event.top, GET_UVALUE=hsave
	HANDLE_VALUE, hsave, save

	; get the widget IDs and program parameters
	HANDLE_VALUE, save.hid, id
	HANDLE_VALUE, save.hpar, par

	CASE action OF

	'GO':   BEGIN
		save.hgo = 1
		from_go = 1
		GOTO, OK
		END

	'OK':	BEGIN
		from_go = 0
OK:
		; get and check entries in widgets
		WIDGET_CONTROL, id.mmet, GET_VALUE=val
		par.mapmethod = val(0)
		WIDGET_CONTROL, id.pixx, GET_VALUE=val
		pixx = val(0)
		WIDGET_CONTROL, id.pixy, GET_VALUE=val
		pixy = val(0)
		WIDGET_CONTROL, id.drop, GET_VALUE=val
		par.dropfct = val(0)
		WIDGET_CONTROL, id.shx, GET_VALUE=val
		par.shx = val(0)
		WIDGET_CONTROL, id.shy, GET_VALUE=val
		par.shy = val(0)

;		WIDGET_CONTROL, id.rota, GET_VALUE=val
;		rota = val(0)
;		IF pixx LT 0 OR pixy LT 0 OR rota LT 0 OR rota GT 360 THEN BEGIN
;			PRINT, STRING('7'XB)
;			RETURN
;		ENDIF
		IF pixx LT 0 OR pixy LT 0 THEN BEGIN
			PRINT, STRING('7'XB)
			RETURN
		ENDIF
;		WIDGET_CONTROL, id.datatype, GET_UVALUE=datatype

		IF STRMID(par.detector,0,1) EQ 'C' THEN BEGIN 
		  WIDGET_CONTROL, id.pixbase, GET_UVALUE=pixselect 
		  WIDGET_CONTROL, id.pixff, GET_VALUE=fff
		  par.ffield = fff
		  WIDGET_CONTROL, id.perc, GET_VALUE=val
		  par.percentile = val(0)
		ENDIF ELSE pixselect = par.pixselect

		; store back parameters
		par.pixsize = [ pixx, pixy ]
;		par.rota = rota
;		par.datatype = datatype
		par.pixselect = pixselect
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		
		; save the selected brightness array locally (for drift corr.)
;		HANDLE_VALUE, save.haap, aap
;		CASE datatype OF
;		1:	HANDLE_VALUE, aap.q1br, bright
;		2:	HANDLE_VALUE, aap.mdbr, bright
;		3:	HANDLE_VALUE, aap.q3br, bright
;		ELSE:	HANDLE_VALUE, aap.mnbr, bright
;		ENDCASE
;		HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY

	        HANDLE_VALUE, hsave, save, /SET
		WIDGET_CONTROL, event.top, /DESTROY
		END

	'CANC':	WIDGET_CONTROL, event.top, /DESTROY

	'MAPMET': par.mapmethod = event.value

	'MEAN':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.mnbr, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 0
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END
		 		 
	'QUA1':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.q1br, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 1
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END

	'MEDI':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.mdbr, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 2
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END

	'QUA3':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.q3br, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 3
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END

	'FLUX':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.mnfl, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 4
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END

	'FLMD':	BEGIN
		 HANDLE_VALUE, save.haap, aap
		 HANDLE_VALUE, aap.mdfl, bright
		 HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY
		 par.datatype = 5
		 HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		END

	'DEFA':	BEGIN
		; calculate default pixsize
		PIA_MAPDEF, par, pixsize
		WIDGET_CONTROL, id.pixx, SET_VALUE=STRING(pixsize(0))
		WIDGET_CONTROL, id.pixy, SET_VALUE=STRING(pixsize(1))
		WIDGET_CONTROL, id.drop, SET_VALUE=STRTRIM(0.3,2)
		WIDGET_CONTROL, id.shx, SET_VALUE=STRTRIM(0.,2)
		WIDGET_CONTROL, id.shy, SET_VALUE=STRTRIM(0.,2)
		WIDGET_CONTROL, id.rota, SET_VALUE=STRING(par.rota_orig)
		WIDGET_CONTROL, id.datatype, SET_BUTTON=0, SET_UVALUE=0
		WIDGET_CONTROL, id.mean, SET_BUTTON=1
		WIDGET_CONTROL, id.mmet, SET_VALUE=0
		par.mapmethod = 0
		END

	'FFF' : BEGIN
		par.ffield = event.value
		END

	ELSE:	CASE STRMID(action,0,3) OF

		'SEL':	BEGIN
			CASE STRMID(action,3,1) OF

			'A':	BEGIN
				FOR i = 0,par.npixel-1 DO $
					WIDGET_CONTROL, id.pixselect(i), $
						SET_BUTTON=1
				WIDGET_CONTROL, id.pixbase, $
					SET_UVALUE=[1,1,1,1,1,1,1,1,1]
				END

			'N':	BEGIN
				FOR i = 0,par.npixel-1 DO $
					WIDGET_CONTROL, id.pixselect(i), $
						SET_BUTTON=0
				WIDGET_CONTROL, id.pixbase, $
					SET_UVALUE=[0,0,0,0,0,0,0,0,0]
				END

			ELSE:	BEGIN
				i = FIX(STRMID(action,3,1)) - 1
				WIDGET_CONTROL, id.pixbase, $
					GET_UVALUE=pixselect
				pixselect(i) = event.select
				WIDGET_CONTROL, id.pixbase, $
					SET_UVALUE=pixselect
				END

			ENDCASE
			END

		ELSE:
		ENDCASE

	ENDCASE

	RETURN
	END


; ***************************************************************************
; PIA_MAPW_DEFMAP  -  define map pixel size and rotation of map
; INPUT:	detector
; IN/OUT:	pixsize, rota
; ***************************************************************************

PRO PIA_MAPW_DEFMAP, hpar, haap, hbright, GO, GROUP=group

; get a copy of program parameters

	HANDLE_VALUE, hpar, par

; set detector name

	CASE par.detector OF
	'C1':	detector_name = "C100"
	'C2':	detector_name = "C200"
	'P1':	detector_name = "P1"
	'P2':	detector_name = "P2"
	'P3':	detector_name = "P3"
	'SS':	detector_name = "SS"
	'SL':	detector_name = "SL"
	ELSE:	MESSAGE, "Wrong detector"
	ENDCASE

; define widget IDs

	id = { $
		mmet:0L, 	$
		pixx:0L, 	$
		pixy:0L, 	$
		drop:0L, 	$
		shx:0L, 	$
		shy:0L, 	$
		rota:0L, 	$
		datatype:0L, 	$
		mean:0L, 	$
		qua1:0L, 	$
		medi:0L, 	$
		qua3:0L, 	$
		flux:0L, 	$
		flmd:0L, 	$
		defa:0L, 	$
		pixbase:0L,	$
		pixselect:LONARR(9),	$
		perc:0L, 	$
		pixff:0L	$
	}

; set up widgets

	id_base = WIDGET_BASE( TITLE="Define Map Parameters", /COLUMN,$
			/modal, group=group )

	id_menu = WIDGET_BASE( id_base, /ROW )
	id_0 = WIDGET_BUTTON( id_menu, VAL="    OK    ", UVA="OK" )
	id_0 = WIDGET_BUTTON( id_menu, VAL="  Cancel  ", UVA="CANC" )
	id_go = WIDGET_BUTTON( id_menu,VAL=" Do the map ", UVAL="GO" )	
	id_0 = WIDGET_LABEL( id_menu, VAL="    Det: " )
	id_0 = WIDGET_LABEL( id_menu, VAL=detector_name )

	id_panel = WIDGET_BASE( id_base, /ROW )

	id_left = WIDGET_BASE( id_panel, /COLUMN )

;	id_0 = WIDGET_BASE( id_left, /ROW )
;	id_1 = WIDGET_LABEL( id_0, VAL="Detector: " )
;	id_1 = WIDGET_LABEL( id_0, VAL=detector_name )

	mapmet = ['Full coverage', 'Distance weighting', 'Trigrid Interp.', $
			'Trigrid / quintic interp.']
	id_0 = WIDGET_BASE( id_left, /COLUMN, /FRAME )
	id.mmet = CW_BGROUP(id_0, /EXCLUSIVE, SET_VALUE=par.mapmethod, $
	 mapmet, UVALUE='MAPMET', LABEL_LEFT="Mapping method:") 

	id_0 = WIDGET_BASE( id_left, /COLUMN, /FRAME )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, $
	   VAL="Pixel size of map in arcsec (configurable):" )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, VAL="      X: " )
	id.pixx = WIDGET_TEXT( id_1, VAL='', UVA='PIXX', /EDITABLE, XSIZE=12 )
	id_2 = WIDGET_LABEL( id_1, VAL="      Y: " )
	id.pixy = WIDGET_TEXT( id_1, VAL='', UVA='PIXY', /EDITABLE, XSIZE=12 )

	id_0 = WIDGET_BASE( id_left, /COLUMN, /FRAME )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, $
	   VAL="Drop factor for Dist.Weight. and Trigrid:" )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, VAL="Portion of image pixel:" )
	id.drop = WIDGET_TEXT( id_1, VAL='', UVA='DROP', /EDITABLE, XSIZE=6 )

	id_0 = WIDGET_BASE( id_left, /COLUMN, /FRAME )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, $
	   VAL="Shift (image pixsize) [Full Cov.& Dist.W.] :" )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, VAL="X: " )
	id.shx = WIDGET_TEXT( id_1, VAL='', UVA='SHIFTX', /EDITABLE, XSIZE=5 )
	id_2 = WIDGET_LABEL( id_1, $
	   VAL="    ")
	id_2 = WIDGET_LABEL( id_1, VAL="Y: " )
	id.shy = WIDGET_TEXT( id_1, VAL='', UVA='SHIFTY', /EDITABLE, XSIZE=5 )

	id_0 = WIDGET_BASE( id_left, /COLUMN, /FRAME )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, $
		VAL="Rotation angle of map relative to RA,DEC (info):" )
	id_1 = WIDGET_BASE( id_0, /ROW )
	id_2 = WIDGET_LABEL( id_1, VAL="ROTA: " )
	id.rota = WIDGET_TEXT( id_1, VAL='', UVA='ROTA', XSIZE=12 )

	id_0 = WIDGET_BASE( id_left, /ROW, /FRAME )
	id_1 = WIDGET_LABEL( id_0, VAL="Produce map of:" )
	id.datatype = WIDGET_BASE( id_0, /EXCLUSIVE )
	id.mean = WIDGET_BUTTON( id.datatype, $
		VAL="Mean brightness values", UVA="MEAN", $
				/NO_RELEASE )
	id.qua1 = WIDGET_BUTTON( id.datatype, VAL='1st quartile', UVA="QUA1", $
				/NO_RELEASE )
	id.medi = WIDGET_BUTTON( id.datatype, $
		VAL="Median brightness values", UVA="MEDI", $
				/NO_RELEASE )
	id.qua3 = WIDGET_BUTTON( id.datatype, VAL='3rd quartile', UVA="QUA3", $
				/NO_RELEASE )
	id.flux = WIDGET_BUTTON( id.datatype, VAL='Mean fluxes', UVA="FLUX", $
				/NO_RELEASE )
	id.flmd = WIDGET_BUTTON( id.datatype, VAL='Median fluxes', UVA="FLMD", $
				/NO_RELEASE )

	id.defa = WIDGET_BUTTON( id_left, VAL="Set default values",UVA='DEFA')

	IF par.npixel GT 1 THEN BEGIN

		id_right = WIDGET_BASE( id_panel, /COLUMN )

		id_0 = WIDGET_LABEL( id_right, VAL="Select detector pixels:" )
		id_0 = WIDGET_BASE( id_right, /ROW )
		id_1 = WIDGET_BUTTON( id_0, VAL="Select all", UVA="SELA" )
		id_1 = WIDGET_BUTTON( id_0, VAL="Deselect all", UVA="SELN" )

		id.pixbase = WIDGET_BASE( id_right, /NONEXCLUSIVE, $
						UVAL=par.pixselect )
		FOR i = 1,par.npixel DO BEGIN
			pstr = 'Pixel # ' + STRING(i)
			uv = 'SEL'+STRTRIM(i,2)
			id.pixselect(i-1) = WIDGET_BUTTON( id.pixbase, $
							VAL=pstr, UVAL=uv )
		ENDFOR

		id.pixff = CW_BGROUP(id_right, LABEL_TOP='Flat Fielding:   ',$
		 /FRAME, /EXCLUSIVE, SET_VALUE=par.ffield, $
		 UVALUE='FFF', ["Disabled", $
		 	"Central overlapping Flat Fielding", $
		 	"Median filter method", $
			"Chosen Background normalization", $
			"First Percentile normalization"])

		id_1 = WIDGET_BASE( id_right, /ROW )
		id_2 = WIDGET_LABEL( id_1, VAL="Percentile value " )
		id.perc = WIDGET_TEXT( id_1, /EDITABLE, $
			VAL=STRTRIM(par.percentile,2), $
		        UVA='PERC', XSIZE=6 )


;		id_go = WIDGET_BUTTON( id_right, VAL="Do the map", UVAL="GO" )
	ENDIF

; store program parameters

	hsave = HANDLE_CREATE()
	hid = HANDLE_CREATE( hsave,  VALUE=id )
	HANDLE_VALUE, hsave, $
		{ hid:hid, hpar:hpar, haap:haap, hbright:hbright, hgo:0 }, /SET

; realize widget, set values

	WIDGET_CONTROL, id_base, /REALIZE, SET_UVALUE=hsave
	WIDGET_CONTROL, id.pixx, SET_VALUE=STRING(par.pixsize(0))
	WIDGET_CONTROL, id.pixy, SET_VALUE=STRING(par.pixsize(1))
	WIDGET_CONTROL, id.drop, SET_VALUE=STRTRIM(par.dropfct,2)
	WIDGET_CONTROL, id.shx, SET_VALUE=STRTRIM(par.shx,2)
	WIDGET_CONTROL, id.shy, SET_VALUE=STRTRIM(par.shy,2)
	WIDGET_CONTROL, id.rota, SET_VALUE=STRING(par.rota_orig)
	CASE par.datatype OF
	1:	id_0 = id.qua1
	2:	id_0 = id.medi
	3:	id_0 = id.qua3
	4:	id_0 = id.flux
	5:	id_0 = id.flmd
	ELSE:	id_0 = id.mean
	ENDCASE
	WIDGET_CONTROL, id.datatype, SET_UVALUE=par.datatype
	WIDGET_CONTROL, id_0, /SET_BUTTON
	IF par.npixel GT 1 THEN FOR i = 0,par.npixel-1 DO $
		WIDGET_CONTROL, id.pixselect(i), SET_BUTTON = par.pixselect(i)

	xMANAGER,No_BLOCK=1, "PIA_MAPW_DEFMAP", id_base, GROUP=group

	HANDLE_VALUE, hsave, save
	IF save.hgo THEN GO = 1 ELSE GO = 0

	HANDLE_FREE, hsave
	RETURN
	END

 
	


; ***************************************************************************
; PIA_MAPW_CLEANUP  -  clean-up routine for PIA_MAPW
; ***************************************************************************

PRO PIA_MAPW_CLEANUP, id_top

	WIDGET_CONTROL, id_top, GET_UVALUE=hsave
	HANDLE_FREE, hsave
	RETURN
	END 
 
; ***************************************************************************
; PIA_MAPW_EVENT  -  event handling routine for PIA_MAPW
; ***************************************************************************

PRO PIA_MAPW_EVENT, event 

@phtfilter
COMMON test_inter, test_all_map
@phtslwavlen
@phtaap

	WIDGET_CONTROL, event.id, GET_UVALUE=action 
	WIDGET_CONTROL, event.top, GET_UVALUE=hsave
	HANDLE_VALUE, hsave, save
	HANDLE_VALUE, save.hpar, par

	CASE action OF 

; select an AAP measurement from the PHTAAP common block

	'FOMA':	BEGIN
		; select a PHTAAP structure
		PIA_OPEN_IAAP, aap, /map, GROUP=event.top, ERROR=err,$
			SR_STR=sr_str

		IF err THEN RETURN

		; get some parameters from the PHTAAP structure
		par.detector	= aap.det
		par.admn	= aap.admn
		par.aper	= aap.aper
		DECODE_ADMN, par.admn, dadmn, decstr_sh=tmp

		HANDLE_VALUE, aap.hdr, hdr
		object     = FXPAR( hdr,"OBJECT"  , COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init

		
		; define the number of pixels of the detector
		CASE par.detector OF
		'C1':	BEGIN
			 par.npixel = 9
			 filt_id = c1filters(aap.filt - 2)
			END
		'C2':	BEGIN
			 par.npixel = 4
			 filt_id = c2filters(aap.filt - 9)
			END
		'P1':	BEGIN
			 par.npixel = 1
			 filt_id = p1filters(aap.filt - 1)
			END
		'P2':	BEGIN
			 par.npixel = 1
			 filt_id = p2filters(aap.filt - 11)
			END
		'P3':	BEGIN
			 par.npixel = 1
			 filt_id = p3filters(aap.filt - 13)
			END
		ELSE:	BEGIN
			 par.npixel = 1
			 filt_id = 'FFF' + STRTRIM(aap.cwvl(0),2)
			 par.pixselect = par.pixselect*0 - 1
			 FOR Isel=0,(N_ELEMENTS(sr_str.pixel)-1)<8 DO $
			     par.pixselect(isel) = sr_str.pixel(isel)
			END
		ENDCASE
		lambda = STRMID(filt_id,3,4)

		par.filter = filt_id
		par.admn_decoded = object + ' - ' + lambda + '!4l!3m - '+tmp


		; get some parameters from the AAP header
		HANDLE_VALUE, aap.hdr, hdr

		keyword = "EOHAPLID"
		plid     = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "EOHAOSN"
		osn      = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "EOHAAOTN"
		par.aotn = FXPAR( hdr, keyword, COUNT=count)
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
		par.aotn = STRTRIM(par.aotn,2)

		keyword = "ATTRA"
		par.ra   = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "ATTDEC"
		par.dec  = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "ATTRROTA"
		par.rota = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
		par.rota_orig = par.rota

		keyword = "ATTRORIE"
		orie = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			orie= 1
		ENDIF

		keyword = "ATTRNPTS"
		par.npts = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
			par.npts = MAX(rpid(0,*))
			HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
		ENDIF

		keyword = "ATTRNLNS"
		par.nlns = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
			par.nlns = MAX(rpid(1,*))
			HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
		ENDIF

		keyword = "ATTRDPTS"
		par.dpts = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			par.dpts = 90.
		ENDIF

		keyword = "ATTRDLNS"
		par.dlns = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			par.dlns = 90.
		ENDIF

		; define a default pixel size
		PIA_MAPDEF, par, pixsize
		par.pixsize = pixsize
		par.mapmethod = 0
		par.dropfct = 0.3

		; fill the info panel
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.aotn, SET_VALUE=STRING(par.aotn)
		sdet = par.detector
		IF STRMID(sdet,0,1) EQ 'C' THEN sdet = sdet + '00'
		WIDGET_CONTROL, id.det, SET_VALUE=STRING(sdet)
		measname = dec_admn(par.admn)
		WIDGET_CONTROL, id.admn, SET_VALUE=measname
		WIDGET_CONTROL, id.plid, SET_VALUE=STRING(plid)
		WIDGET_CONTROL, id.osn, SET_VALUE=par.filter
		WIDGET_CONTROL, id.raster, SET_VALUE= $
			STRING(par.npts,par.nlns,FORMAT="(i4,' x ',i4)")

		; get a local copy of the brightness array
		; default is MNBR (mean brightness) - use par.datatype
		CASE par.datatype of
		 0: HANDLE_VALUE, aap.mnbr, bright
		 1: HANDLE_VALUE, aap.q1br, bright
		 2: HANDLE_VALUE, aap.mdbr, bright
		 3: HANDLE_VALUE, aap.q3br, bright
		 4: HANDLE_VALUE, aap.mnfl, bright
		 5: HANDLE_VALUE, aap.mdfl, bright
		ENDCASE
		HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY

		handle_value, aap.roll, roll
		handle_value, aap.ra, ra

		IF roll(0) EQ -99. AND ra(0) EQ -99 THEN $
                        XDISPLAYFILE, "", $
                          TITLE = "Warning ", $
                          WIDTH = 50, $
                          HEIGHT = 12, $
                          TEXT = [" Measurement does not contain ", 	$
                                  " coordinates - Only simple mapping",	$
                                  " without pointing is meaningful "]
		  

		; save the AAP data structure locally
		HANDLE_VALUE, save.haap, aap, /SET, /NO_COPY

		; enable/disable menu items and set control flags
		; Change the buffer_state variable to 0
		id.buffer_state = 0
		WIDGET_CONTROL, id.mpdef, SENSITIVE=1
		WIDGET_CONTROL, id.mpmap, SENSITIVE=1
		WIDGET_CONTROL, id.mpmesm, SENSITIVE=1
		sens1 = (par.aotn EQ "P32")
;		WIDGET_CONTROL, id.mpdrc, SENSITIVE=sens1
		sens2 = sens1 OR (STRMID(par.detector,0,1) EQ 'P')
;		WIDGET_CONTROL, id.mpsim, SENSITIVE=sens2
		WIDGET_CONTROL, id.mpsim, SENSITIVE=1
		WIDGET_CONTROL, id.mpsep, SENSITIVE=1
		WIDGET_CONTROL, id.mpsepc, SENSITIVE=1
		WIDGET_CONTROL, id.mshh, SENSITIVE=1
		WIDGET_CONTROL, id.msha, SENSITIVE=1
		WIDGET_CONTROL, id.mshm, SENSITIVE=0
;		WIDGET_CONTROL, id.mshs, SENSITIVE=0
		WIDGET_CONTROL, id.shbr, SENSITIVE=1
		WIDGET_CONTROL, id.shun, SENSITIVE=1
		WIDGET_CONTROL, id.shex, SENSITIVE=1
		WIDGET_CONTROL, id.shan, SENSITIVE=1
		par.aapin = 1
		par.mapin = 0
; CG: calculate default ROTATION for chopped measurements (valid always for P32)
		IF sens1 OR orie THEN BEGIN
		    par.rota = avg([roll])
		    par.rota_orig = par.rota 
		ENDIF ELSE $
		  par.rota = par.rota_orig + avg([roll])
		IF STRMID(par.detector,0,1) NE 'C' THEN $
			par.rota = par.rota_orig
;print, sens1, orie, par.rota, par.rota_orig, avg([roll])
		IF par.rota GE 360. THEN par.rota = par.rota - 360.  

		; store the local parameters
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY
		

		END

	'XINT': BEGIN

		PIA_CONVERT_IAAP, aap, all_aap, error=err	
		IF err THEN RETURN

		; get some parameters from the PHTAAP structure
		par.detector	= aap.det
		par.admn	= aap.admn
		par.aper	= aap.aper
		DECODE_ADMN, par.admn, dadmn, decstr_sh=tmp

		HANDLE_VALUE, aap.hdr, hdr
		object     = FXPAR( hdr,"OBJECT"  , COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init

		
		; define the number of pixels of the detector
		CASE par.detector OF
		'C1':	BEGIN
			 par.npixel = 9
			 filt_id = c1filters(aap.filt - 2)
			END
		'C2':	BEGIN
			 par.npixel = 4
			 filt_id = c2filters(aap.filt - 9)
			END
		'P1':	BEGIN
			 par.npixel = 1
			 filt_id = p1filters(aap.filt - 1)
			END
		'P2':	BEGIN
			 par.npixel = 1
			 filt_id = p2filters(aap.filt - 11)
			END
		'P3':	BEGIN
			 par.npixel = 1
			 filt_id = p3filters(aap.filt - 13)
			END
		ELSE:	BEGIN
			 par.npixel = 1
			 filt_id = 'FFF' + STRTRIM(aap.cwvl(0),2)
			 par.pixselect = par.pixselect*0 - 1
			END
		ENDCASE
		lambda = STRMID(filt_id,3,4)

		par.filter = filt_id
		par.admn_decoded = object + ' - ' + lambda + '!4l!3m - '+tmp


		; get some parameters from the AAP header
		HANDLE_VALUE, aap.hdr, hdr

		keyword = "EOHAPLID"
		plid     = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "EOHAOSN"
		osn      = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "EOHAAOTN"
		par.aotn = FXPAR( hdr, keyword, COUNT=count)
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
		par.aotn = STRTRIM(par.aotn,2)

		keyword = "ATTRA"
		par.ra   = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "ATTDEC"
		par.dec  = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "ATTRROTA"
		par.rota = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
		par.rota_orig = par.rota

		keyword = "ATTRORIE"
		orie = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			orie= 1
		ENDIF

		keyword = "ATTRNPTS"
		par.npts = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
			par.npts = MAX(rpid(0,*))
			HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
		ENDIF

		keyword = "ATTRNLNS"
		par.nlns = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			HANDLE_VALUE, aap.rpid, rpid, /NO_COPY
			par.nlns = MAX(rpid(1,*))
			HANDLE_VALUE, aap.rpid, rpid, /SET, /NO_COPY
		ENDIF

		keyword = "ATTRDPTS"
		par.dpts = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			par.dpts = 90.
		ENDIF

		keyword = "ATTRDLNS"
		par.dlns = FXPAR( hdr, keyword, COUNT=count )
		IF count LE 0 THEN BEGIN
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword
			par.dlns = 90.
		ENDIF

		; define a default pixel size
		PIA_MAPDEF, par, pixsize
		par.pixsize = pixsize
		par.mapmethod = 0
		par.dropfct = 0.3

		; fill the info panel
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.aotn, SET_VALUE=STRING(par.aotn)
		sdet = par.detector
		IF STRMID(sdet,0,1) EQ 'C' THEN sdet = sdet + '00'
		WIDGET_CONTROL, id.det, SET_VALUE=STRING(sdet)
		measname = dec_admn(par.admn)
		WIDGET_CONTROL, id.admn, SET_VALUE=measname
		WIDGET_CONTROL, id.plid, SET_VALUE=STRING(plid)
		WIDGET_CONTROL, id.osn, SET_VALUE=par.filter
		WIDGET_CONTROL, id.raster, SET_VALUE= $
			STRING(par.npts,par.nlns,FORMAT="(i4,' x ',i4)")

		; get a local copy of the brightness array
		; default is MNBR (mean brightness) - use par.datatype
		HANDLE_VALUE, aap(0).mdbr, bright
		HANDLE_VALUE, save.hbright, bright, /SET, /NO_COPY

		handle_value, aap.roll, roll
		handle_value, aap.ra, ra

		IF roll(0) EQ -99. AND ra(0) EQ -99 THEN $
                        XDISPLAYFILE, "", $
                          TITLE = "Warning ", $
                          WIDTH = 50, $
                          HEIGHT = 12, $
                          TEXT = [" Measurement does not contain ", 	$
                                  " coordinates - Only simple mapping",	$
                                  " without pointing is meaningful "]
		  

		; save the AAP data structure locally
		HANDLE_VALUE, save.haap, aap, /SET

		; enable/disable menu items and set control flags
		; Change the buffer_state variable to 0
		id.buffer_state = 0
		WIDGET_CONTROL, id.mpdef, SENSITIVE=1
		WIDGET_CONTROL, id.mpmap, SENSITIVE=1
		WIDGET_CONTROL, id.mpmesm, SENSITIVE=1
		sens1 = (par.aotn EQ "P32")
;		WIDGET_CONTROL, id.mpdrc, SENSITIVE=sens1
		sens2 = sens1 OR (STRMID(par.detector,0,1) EQ 'P')
;		WIDGET_CONTROL, id.mpsim, SENSITIVE=sens2
		WIDGET_CONTROL, id.mpsim, SENSITIVE=1
		WIDGET_CONTROL, id.mpsep, SENSITIVE=1
		WIDGET_CONTROL, id.mpsepc, SENSITIVE=1
		WIDGET_CONTROL, id.mshh, SENSITIVE=1
		WIDGET_CONTROL, id.msha, SENSITIVE=1
		WIDGET_CONTROL, id.mshm, SENSITIVE=0
;		WIDGET_CONTROL, id.mshs, SENSITIVE=0
		WIDGET_CONTROL, id.shbr, SENSITIVE=1
		WIDGET_CONTROL, id.shun, SENSITIVE=1
		WIDGET_CONTROL, id.shex, SENSITIVE=1
		WIDGET_CONTROL, id.shan, SENSITIVE=1
		par.aapin = 1
		par.mapin = 0
; CG: calculate default ROTATION for chopped measurements (valid always for P32)
		IF sens1 OR orie THEN BEGIN
		    par.rota = avg([roll])
		    par.rota_orig = par.rota 
		ENDIF ELSE $
		  par.rota = par.rota_orig + avg([roll])
		IF STRMID(par.detector,0,1) NE 'C' THEN $
			par.rota = par.rota_orig
;print, sens1, orie, par.rota, par.rota_orig, avg([roll])
		IF par.rota GE 360. THEN par.rota = par.rota - 360.  

		; store the local parameters
;		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY

		i=0
		 bright = all_aap(i).mdbr
		 PIA_MAPAAP, all_aap(i), bright, par.pixselect(0:par.npixel-1), $
			par.pixsize, par.rota, event=event, $
			shift_x=par.shx, shift_y=par.shy, $
			map_bright, map_uncert, map_expose, trans, $
			mmethod = par.mapmethod, fraction=par.dropfct, $
			trans_ra, trans_dec, FFIELD=par.ffield, ERROR=err
		siz = size(map_bright)
		all_map = FLTARR(siz(1),siz(2),64)
		all_mappar = REPLICATE(par,64)
 
		IF par.detector EQ "SL" THEN addfilt = 64 ELSE addfilt=0
		all_map(*,*,0) = map_bright
		all_mappar(0).pixselect = par.pixselect*0
		all_mappar(0).pixselect(0) = 0
		all_mappar(0).filter = "FWL"+STRTRIM(slwavelen(0+addfilt),2)		
		FOR I=1,63 DO BEGIN
		 
		 bright = all_aap(i).mdbr
		 PIA_MAPAAP, all_aap(i), bright, par.pixselect(0:par.npixel-1), $
			par.pixsize, par.rota, event=event, $
			shift_x=par.shx, shift_y=par.shy, $
			map_bright, map_uncert, map_expose, trans, $
			mmethod = par.mapmethod, fraction=par.dropfct, $
			trans_ra, trans_dec, FFIELD=par.ffield, ERROR=err
		all_map(*,*,i) = map_bright
		all_mappar.pixselect = par.pixselect*0
		all_mappar(i).pixselect(0) = i		
		all_mappar(i).filter = "FWL"+STRTRIM(slwavelen(i+addfilt),2)		
		ENDFOR
		test_all_map = {map:all_map, par:all_mappar}
		
		sizx = siz(1) * 20
		sizy = siz(2) * 20
		mm = rebin(test_all_map.map, sizx, sizy,64)	  
		XINTERANIMATE, SET=[sizx,sizy,64], /showload
		for i=0,63 do xinteranimate, frame=i, image=mm[*,*,i]
		XINTERANIMATE, /KEEP_PIXMAPS
		END
; IRAF'S Drizzle in batch mode
	'DRIZZLE': BEGIN
		IF is_file_exist(!IRAF_LOGIN+'login.cl') THEN BEGIN

		   pia_drizzle_if, drizpar
		   IF drizpar.error THEN RETURN
		   		   
		   data = pickmeas(FTYPE="AAP", POSI=posi)
		   
		   here = GETENV('PWD')
		   IF data NE '' THEN BEGIN
		     index = WHERE(phtaap.admn EQ data, count) 
		     cd, !IRAF_LOGIN   
		     mapaap2drizzle, index(posi(0)), /execute, $
		     	imagename=drizpar.name, $
			scale=drizpar.scale, $
			mappixelsize=drizpar.pixsize,$
		        positangle=drizpar.rota, $
			drizzlefraction=drizpar.dropf, $
			mapracenter=drizpar.rac, $
			mapdeccenter=drizpar.decc, $
			increasesize=drizpar.imext, $
			surfacebrightness=drizpar.type, $
			medianvalues=drizpar.val, $
			everypixel=drizpar.evpix, $
			gnomonicprojection=drizpar.proj, $
			spacecraftcoords=drizpar.sccoor, $
			fits=drizpar.fits
		     cd, here  
		     MESSAGE, /INFO, $
		       'Drizzle job sent in batch mode, you will get'  
		     MESSAGE, /INFO, $
		       'an IRAF info line telling you it is finished '  
		   ENDIF
		   
		ENDIF ELSE BEGIN 
		   MESSAGE, /INFO, $
		     'I do not find the file '+!IRAF_LOGIN+'login.cl'
		   MESSAGE, /INFO, $
		     ' so I cannot handle it further'

		ENDELSE

	END

; IRAF'S Drizzle in batch mode
	'DRIZZLE_IAAP': BEGIN
		IF is_file_exist(!IRAF_LOGIN+'login.cl') THEN BEGIN
		   
		   pia_drizzle_if, drizpar
		   IF drizpar.error THEN RETURN

		  filename = pia_pickfile(path=!fitdir, filter="*.IAAP", $
		  /read, get_path = newpath, group=event.top, /MULTI) 
		   
		   IF filename(0) NE '' THEN BEGIN 
		   		     
		     cd, !IRAF_LOGIN   
		     mapaap2drizzle, filename, /execute, $
		     	imagename=drizpar.name, $
			scale=drizpar.scale, $
			mappixelsize=drizpar.pixsize,$
		        positangle=drizpar.rota, $
			drizzlefraction=drizpar.dropf, $
			mapracenter=drizpar.rac, $
			mapdeccenter=drizpar.decc, $
			increasesize=drizpar.imext, $
			surfacebrightness=drizpar.type, $
			medianvalues=drizpar.val, $
			everypixel=drizpar.evpix, $
			gnomonicprojection=drizpar.proj, $
			spacecraftcoords=drizpar.sccoor, $
			fits=drizpar.fits
			
		   
		     MESSAGE, /INFO, $
		       'Drizzle job sent in batch mode, you will get'  
		     MESSAGE, /INFO, $
		       'an IRAF info line telling you it is finished '  
		   ENDIF
		   
		ENDIF ELSE BEGIN 
		   MESSAGE, /INFO, $
		     'I do not find the file '+!IRAF_LOGIN+'login.cl'
		   MESSAGE, /INFO, $
		     ' so I cannot handle it further'

		ENDELSE

	END

; WMT: Read in the FITS files
	'RFIT': BEGIN
		HANDLE_VALUE, save.hid, id
; Change the buffer_state variable to 1
		id.buffer_state = 1
		filename = pia_pickfile(path=!fitdir, filter="PGA*.FITS", $
		  /read, get_path = newpath, group=event.top) 
		if filename NE '' then begin 
		if newpath NE '' then BEGIN
;		  filename = newpath + filename
		  !FITDIR = newpath
		ENDIF
		fxread, filename, fmap, hdr
		keyword = strmid(hdr, 0, 8)
		handle_value, save.hpar, par
; change the mapin to 2 for fits file only if map is 3 dimensional
		siz_map = size(fmap)
		par.mapin = siz_map(0) - 1
		filename = fxpar(hdr, 'FILENAME')
		admn = fxpar(hdr, 'PIA_MNAM', count=count)
		IF count NE 0 THEN par.admn = admn ELSE $
		 par.admn = filename

		CASE strmid(filename, 0, 4) of
		'PGAI': imap_type = 0 
		'PGAU': imap_type = 1
		'PGAT': imap_type = 2
		  ELSE: imap_type = 0
		ENDCASE


		filter = FXPAR(hdr,"FILTER*", count=count)
		IF count EQ 0 THEN BEGIN
		 filter = '' 
		 filtnr = 0
		ENDIF ELSE BEGIN
		 IF count GT 1 THEN $
		  pia_multchoice, nch, elemnames=filter $
		 ELSE IF count EQ 1 THEN nch = 0
		 filter = filter(nch(0))
		 filtnr = nch(0)
		ENDELSE		    
		
		detector = strmid(filter, 0, 2)
		par.detector = detector
		par.filter = STRTRIM(filter,2)
		object = FXPAR( hdr,"OBJECT" ) 
		par.admn_decoded = object + ' ' + detector

		; get some parameters from the header

		keyword = "EOHAPLID"
		plid     = FXPAR( hdr, keyword)

		keyword = "EOHAOSN"
		osn      = FXPAR( hdr, keyword)

		keyword = "EOHAAOTN"
		par.aotn = FXPAR( hdr, keyword)
		par.aotn = STRTRIM(par.aotn,2)

		keyword = "CRVAL1"
		par.ra   = FXPAR( hdr, keyword)
		par.trans_ra = par.ra

		keyword = "CRVAL2"
		par.dec  = FXPAR( hdr, keyword)
		par.trans_dec = par.dec

; the fits file standard start with pixel 1, 1 and we ofcourse
; use the idl array format which is 0, 0
		keyword = "CRPIX1"
		refpix1 = FXPAR( hdr, keyword) - 1

		keyword = "CRPIX2"
		refpix2 = FXPAR( hdr, keyword) - 1

; according to the fits standard, only crota2 is used
		keyword = "CROTA2"
		par.rota = FXPAR( hdr, keyword)
		par.rota = 360. - par.rota
		par.rota_orig = par.rota

                keyword = "ATTRNPTS"
                par.npts = FXPAR( hdr, keyword, count=count)
                IF count LE 0 THEN BEGIN
                        MESSAGE, /INFO, 'Warning: missing keyword '+keyword
                        par.npts = 0
                ENDIF

		keyword = "ATTRNLNS"
		par.nlns = FXPAR( hdr, keyword, count=count)
                IF count LE 0 THEN BEGIN
                        MESSAGE, /INFO, 'Warning: missing keyword '+keyword
                        par.nlns = 0
                ENDIF

; what the pia_mktrans is expecting arcsec
		pix1 =  -( FXPAR( hdr, 'CDELT1') * 3600. )
		pix2 = FXPAR( hdr, 'CDELT2') * 3600.
		IF pix1 LT 0 THEN BEGIN
		  PRINT, 'Wrong CDELT1 Keyword: Changing sign'
		  pix1 = -pix1
		ENDIF 

		par.pixsize = [pix1, pix2 ]

		par.trans = PIA_MKTRANS([par.ra, par.dec], $
			[refpix1, refpix2], $
			par.pixsize, par.rota)


		CASE strmid(filename, 0, 4) of
		'PGAI': handle_value, save.hmapb, $
		  fmap(*,*,filtnr), /set, /no_copy
		'PGAU': handle_value, save.hmapu, $
		  fmap(*,*,filtnr), /set, /no_copy
		'PGAT': handle_value, save.hmape, $
		  fmap(*,*,filtnr), /set, /no_copy
		  ELSE: handle_value, save.hmapb, $
		  fmap(*,*,filtnr), /set, /no_copy
		ENDCASE

		; fill the info panel
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.aotn, SET_VALUE=STRING(par.aotn)
		sdet = par.detector
		IF STRMID(sdet,0,1) EQ 'C' THEN sdet = sdet + '00'
		WIDGET_CONTROL, id.det, SET_VALUE=STRING(sdet)
		WIDGET_CONTROL, id.admn, SET_VALUE=par.admn_decoded
		WIDGET_CONTROL, id.plid, SET_VALUE=STRING(plid)
		WIDGET_CONTROL, id.osn, SET_VALUE=par.filter
		WIDGET_CONTROL, id.raster, SET_VALUE= $
			STRING(par.npts,par.nlns,FORMAT="(i4,' x ',i4)")

		handle_value, save.hpar, par, /set, /no_copy

		aap = {phtaap_type}
		aap.hdr = handle_create()
		HANDLE_VALUE, aap.hdr, hdr, /set
		handle_value, save.haap, aap, /set, /no_copy

		WIDGET_CONTROL, id.mpdef, SENSITIVE=0
		WIDGET_CONTROL, id.mpmap, SENSITIVE=0
;		WIDGET_CONTROL, id.mpdrc, SENSITIVE=0
		WIDGET_CONTROL, id.mpsim, SENSITIVE=0
		WIDGET_CONTROL, id.mpsep, SENSITIVE=0
		WIDGET_CONTROL, id.mpsepc, SENSITIVE=0
		WIDGET_CONTROL, id.mshh, SENSITIVE=1
		WIDGET_CONTROL, id.msha, SENSITIVE=0
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=0
		CASE imap_type OF
		0: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		1: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=1
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		2: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=1
		   END
		ELSE: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		ENDCASE
		endif
		END

; Read CAM FITS files
	'RFIT_CAM': BEGIN
		HANDLE_VALUE, save.hid, id
; Change the buffer_state variable to 1
		id.buffer_state = 1
		filename = pia_pickfile(path=!fitdir, filter="CMAP*.FITS", $
		  /read, get_path = newpath, group=event.top)
		if filename NE '' then begin 
;		if newpath NE '' then filename = newpath + filename
		ehdr = headfits(filename)
		FXBOPEN, unit, filename, 1, hdr
		keyword = strmid(hdr, 0, 8)
		handle_value, save.hpar, par

		imap_type = 0 
; now follow CAM advice to read it in
		FXBREAD, unit, bscale, 'BSCALE'
		FXBREAD, unit, bzero, 'BZERO'
		FXBREAD, unit, channel, 'CHANNEL'
		FXBREAD, unit, wavelen, 'WAVELENG'
		channel = channel + ' ' + STRTRIM(wavelen,2)
		nmaps = N_ELEMENTS(bscale)
		nchoice = 0
		IF nmaps GT 1 THEN BEGIN
		  PIA_MULTCHOICE, nchoice, elemnames=channel 		
		ENDIF  		
		FXBREAD, unit, array, 'ARRAY'
		array = reform(array(*,nchoice))
		array= float(array) * bscale(nchoice) + bzero(nchoice)
		FXBREAD, unit, naxis1, 'NAXIS1'
		FXBREAD, unit, naxis2, 'NAXIS2'
		naxis1 = naxis1(nchoice)
		naxis2 = naxis2(nchoice)
		nb = naxis1*naxis2 - 1
		fmap = fltarr(naxis1,naxis2)
		FOR I=0,naxis2-1 DO $
		  fmap(*,i) = array(i*naxis1:i*naxis1+naxis1-1)
		  

		
; change the mapin to 2 for fits file only if map is 3 dimensional
		siz_map = size(fmap)
		par.mapin = siz_map(0) - 1
		filename = fxpar(ehdr, 'FILENAME')
		par.admn = filename

		imap_type = 0 
		detector = 'CAM'
		par.detector = detector
		object = FXPAR( ehdr,"OBJECT" ) 
		par.admn_decoded = object + ' ' + detector

		; get some parameters from the header

;		keyword = " "
		plid     = object

;		keyword = "EOHAOSN"
		osn      = '00'

;		keyword = "EOHAAOTN"
;		par.aotn = FXPAR( hdr, keyword)
		par.aotn = 'CAM-AOT'

		keyword = "CRVAL1"
		FXBREAD, unit, keyw, keyword
		par.ra   = keyw(nchoice)
		par.trans_ra = par.ra

		keyword = "CRVAL2"
		FXBREAD, unit, keyw, keyword
		par.dec  = keyw(nchoice)
		par.trans_dec = par.dec

; the fits file standard start with pixel 1, 1 and we ofcourse
; use the idl array format which is 0, 0
		keyword = "CRPIX1"
		FXBREAD, unit, keyw, keyword
		refpix1 = keyw(nchoice) - 1

		keyword = "CRPIX2"
		FXBREAD, unit, keyw, keyword
		refpix2 = keyw(nchoice) - 1

; according to the fits standard, onlly crota2 is used
		keyword = "CROTA2"
		FXBREAD, unit, keyw, keyword
		par.rota = keyw(nchoice)
;		par.rota = 360. - par.rota
		par.rota_orig = par.rota

;                keyword = "NAXIS1"
                par.npts = naxis1

;		keyword = "NAXIS2"
		par.nlns = naxis2

; what the pia_mktrans is expecting arcsec
		FXBREAD, unit, cdelt1, 'CDELT1'
		FXBREAD, unit, cdelt2, 'CDELT2'
		pix1 =  -( cdelt1(nchoice) * 3600. )
		pix2 = cdelt2(nchoice) * 3600.
		IF pix1 LT 0 THEN BEGIN
		  PRINT, 'Wrong CDELT1 Keyword: Should Change sign'
;		  pix1 = -pix1
		ENDIF 

		par.pixsize = [pix1, pix2 ]

		par.trans = PIA_MKTRANS([par.ra, par.dec], $
			[refpix1, refpix2], $
			par.pixsize, par.rota)

		handle_value, save.hmapb, fmap, /set, /no_copy

		; fill the info panel
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.aotn, SET_VALUE=STRING(par.aotn)
		sdet = par.detector
		IF STRMID(sdet,0,1) EQ 'C' THEN sdet = sdet + '00'
		WIDGET_CONTROL, id.det, SET_VALUE=STRING(sdet)
		WIDGET_CONTROL, id.admn, SET_VALUE=par.admn_decoded
		WIDGET_CONTROL, id.plid, SET_VALUE=STRING(plid)
		WIDGET_CONTROL, id.osn, SET_VALUE=''
		WIDGET_CONTROL, id.raster, SET_VALUE= $
			STRING(par.npts,par.nlns,FORMAT="(i4,' x ',i4)")

		handle_value, save.hpar, par, /set, /no_copy

		aap = {phtaap_type}
		aap.hdr = handle_create()
		nehdr = N_ELEMENTS(ehdr)
		nhdr = N_ELEMENTS(hdr)
		fullhdr = STRARR(nehdr + nhdr)
		fullhdr(0:nehdr-1) = ehdr
		fullhdr(nehdr:*) = hdr
		HANDLE_VALUE, aap.hdr, fullhdr, /set
		handle_value, save.haap, aap, /set, /no_copy

		WIDGET_CONTROL, id.mpdef, SENSITIVE=0
		WIDGET_CONTROL, id.mpmesm, SENSITIVE=0
		WIDGET_CONTROL, id.mpmap, SENSITIVE=0
;		WIDGET_CONTROL, id.mpdrc, SENSITIVE=0
		WIDGET_CONTROL, id.mpsim, SENSITIVE=0
		WIDGET_CONTROL, id.mpsep, SENSITIVE=0
		WIDGET_CONTROL, id.mpsepc, SENSITIVE=0
		WIDGET_CONTROL, id.mshh, SENSITIVE=1
		WIDGET_CONTROL, id.msha, SENSITIVE=0
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=0
		CASE imap_type OF
		0: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		1: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=1
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		2: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=1
		   END
		ELSE: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		ENDCASE
		FXBCLOSE, unit
		FREE_lun, unit

		endif
	END

	'CORR_MAP_NORM': BEGIN
		norm_peak = 1
		subt_1quartil = 0
		GOTO, corr_map
	END

	'CORR_MAP_SUBTR': BEGIN
		norm_peak = 0
		subt_1quartil = 1
		GOTO, corr_map
	END

	'CORR_MAP': BEGIN
		norm_peak = 0
		subt_1quartil = 0
corr_map:
		PIA_MAP_operations, dmap, dmappar, dmaphdr, par=par, $
			norm_peak=norm_peak, subt_1quartil=subt_1quartil,$
			/corr_hist
		
	END

	'RINT_DIV_NORM': BEGIN
		norm_peak = 1
		GOTO, rint_div
	END

	'RINT_DIV': BEGIN
		norm_peak = 0
rint_div:
		PIA_MAP_operations, dmap, dmappar, dmaphdr, par=par, $
			/div, norm_peak=norm_peak, dmap_type=dmap_type
			
		IF N_ELEMENTS(dmap) EQ 1 THEN RETURN
		
		imap = dmap
		imappar = dmappar
		imaphdr = dmaphdr 
		imap_type = dmap_type

		handle_value, save.hpar, imappar, /set, /no_copy
		CASE imap_type OF
		  0: handle_value, save.hmapb, imap, /set, /no_copy
		  1: handle_value, save.hmapu, imap, /set, /no_copy
		  2: handle_value, save.hmape, imap, /set, /no_copy
		  ELSE: handle_value, save.hmapb, imap, /set, /no_copy
		ENDCASE
 		
		GOTO, jump_in

		END

	'RINT_SUB_SUBTR': BEGIN
		 subt_1quartil = 1
		 GOTO, rint_sub
		END

	'RINT_SUB': BEGIN
		subt_1quartil = 0
rint_sub:
		PIA_MAP_operations, dmap, dmappar, dmaphdr, par=par, $
		   /subtr, subt_1quartil=subt_1quartil, dmap_type=dmap_type
		IF N_ELEMENTS(dmap) EQ 1 THEN RETURN
		
		imap = dmap
		imappar = dmappar
		imaphdr = dmaphdr 
		imap_type = dmap_type
		

		handle_value, save.hpar, imappar, /set, /no_copy
		CASE imap_type OF
		  0: handle_value, save.hmapb, imap, /set, /no_copy
		  1: handle_value, save.hmapu, imap, /set, /no_copy
		  2: handle_value, save.hmape, imap, /set, /no_copy
		  ELSE: handle_value, save.hmapb, imap, /set, /no_copy
		ENDCASE
 		
		GOTO, jump_in

		END


; WMT: Read in the Internal files
	'RINT': BEGIN
        	filename = pia_pickfile(path=!xdrdir, filter="*.IMAP*", $
        	  /read, /must_exist, get_path = newpath, group=event.top)
        
		if filename NE '' then $
		begin
; Change the buffer_state variable to 1
		if newpath NE '' then BEGIN
		   file = filename
;		   file = newpath + filename
		   !XDRDIR = newpath
		ENDIF else $
		  file = filename
		restore, file
		imappar.mapin = 1
; be careful with old maps
		parnames = TAG_NAMES(imappar)
		IF parnames(0) NE 'MAPMETHOD' THEN BEGIN
		   impnames = TAG_NAMES(par) 
		   FOR inam=0, N_ELEMENTS(parnames)-1 DO BEGIN
		      indtag = WHERE(impnames EQ parnames(inam),namcount)
		      IF namcount GT 0 THEN BEGIN
			siz = size(imappar.(inam))
			IF siz(2) EQ 8 THEN BEGIN
			   FOR itag=0,N_ELEMENTS(tag_names(imappar.(inam)))-1 DO $
				par.(indtag(0)).(itag) = imappar.(inam).(itag) 
			 
			ENDIF ELSE $
		           par.(indtag(0)) = imappar.(inam)
		      ENDIF
		   ENDFOR
		  
		   PRINT, 'Changing restored structure due to old (<7.3) format' 
		ENDIF ELSE $
		   par = imappar

		handle_value, save.hpar, par, /set
		CASE imap_type OF
		  0: BEGIN
		  	handle_value, save.hmapb, imap, /set
			IF is_file_exist(file+"_UNC") THEN BEGIN
; save files contain data with same names
			   real_imap = imap
			   real_par = imappar
			   real_imaphdr = imaphdr
			   real_imap_type = imap_type
			   
			   restore, file+"_UNC"
			   MESSAGE, /INFO, $
			    'File ' + file+"_UNC" + ' found.'
			   MESSAGE, /INFO, $
			    'Map uncertainties taken from it'
			   mapunc = imap
			   
			   imap = real_imap 
			   imappar = real_par
			   imaphdr = real_imaphdr
			   imap_type = real_imap_type
			 
			ENDIF ELSE BEGIN
	  		 int_noise = get_map_noise(imap)
			 mapunc = imap * 0. + int_noise
			 MESSAGE, /INFO, $
			 'Warning: Map uncertainties calculated from "intrinsic noise"'
		        ENDELSE
			HANDLE_VALUE, save.hmapu, mapunc, /SET
		  END
		  1: handle_value, save.hmapu, imap, /set, /no_copy
		  2: handle_value, save.hmape, imap, /set, /no_copy
		  ELSE: handle_value, save.hmapb, imap, /set, /no_copy
		ENDCASE
 
JUMP_in:
;print, 'imap_type:', imap_type	
	aap = {phtaap_type}
		aap.hdr = handle_create()
		HANDLE_VALUE, aap.hdr, imaphdr, /set
		handle_value, save.haap, aap, /set, /no_copy

; setup various keywords and display the panel information
		keyword = "EOHAPLID"
		plid     = FXPAR( imaphdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		keyword = "EOHAOSN"
		osn      = FXPAR( imaphdr, keyword, COUNT=count )
		IF count LE 0 THEN $
			MESSAGE, /INFO, 'Warning: missing keyword '+keyword

		; fill the info panel
		HANDLE_VALUE, save.hid, id

		; change the buffer state to read external files
		id.buffer_state = 1

		; really fill the info panel now
		WIDGET_CONTROL, id.aotn, SET_VALUE=STRING(par.aotn)
		sdet = par.detector
		IF STRMID(sdet,0,1) EQ 'C' THEN sdet = sdet + '00'
		WIDGET_CONTROL, id.det, SET_VALUE=STRING(sdet)
		measname = dec_admn(par.admn) 
		WIDGET_CONTROL, id.admn, SET_VALUE=measname
		WIDGET_CONTROL, id.plid, SET_VALUE=STRING(plid)
		WIDGET_CONTROL, id.osn, SET_VALUE=par.filter
		WIDGET_CONTROL, id.raster, SET_VALUE= $
			STRING(par.npts,par.nlns,FORMAT="(i4,' x ',i4)")

		WIDGET_CONTROL, id.mpdef, SENSITIVE=0
		WIDGET_CONTROL, id.mpmesm, SENSITIVE=0
		WIDGET_CONTROL, id.mpmap, SENSITIVE=0
;		WIDGET_CONTROL, id.mpdrc, SENSITIVE=0
		WIDGET_CONTROL, id.mpsim, SENSITIVE=0
		WIDGET_CONTROL, id.mpsep, SENSITIVE=0
		WIDGET_CONTROL, id.mpsepc, SENSITIVE=0
		WIDGET_CONTROL, id.mshh, SENSITIVE=1
		WIDGET_CONTROL, id.msha, SENSITIVE=0
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=0
		CASE imap_type OF
		0: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=1
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		1: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=1
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		2: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=0
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=0
		   WIDGET_CONTROL, id.shex, SENSITIVE=1
		   END
		ELSE: BEGIN
		   WIDGET_CONTROL, id.shbr, SENSITIVE=1
		   WIDGET_CONTROL, id.shan, SENSITIVE=1
		   WIDGET_CONTROL, id.shun, SENSITIVE=1
		   WIDGET_CONTROL, id.shex, SENSITIVE=0
		   END
		ENDCASE
		endif

	END

; quit the mapping application
; quit the mapping application

	'QUIT': BEGIN
	  WIDGET_CONTROL, event.top, /DESTROY
;	  set_colours
	 END

; convert power values to brightness (and flux density)
;
; !!! this was planned for processing based on SPD
; !!! commented out during change to AAP structure

;	'PCNV':	BEGIN
;		IF NOT par.aapin THEN RETURN
;		HANDLE_VALUE, save.haap, aap
;		PIA_POW2FB, aap, ERROR=err
;		IF err THEN RETURN
;		END

; define map parameters

	'PDEF':	BEGIN
		IF NOT par.aapin THEN RETURN
		PIA_MAPW_DEFMAP, save.hpar, save.haap, save.hbright, $
				GO, GROUP=event.top
		IF GO THEN GOTO, MAPPING
		END

; filter and smoothing background
	'MPMESM': BEGIN

		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.hbright, bright
		HANDLE_VALUE, save.haap, aap
		HANDLE_VALUE, aap.mnbu, mnbu
		HANDLE_VALUE, aap.flag, flag
		get_validflag, flag, 'AAP', redflag, det=aap.det
		flag = redflag
		
;		set_colours
	        pia_ff_med_smooth, bright, mnbu, tmp_bright, flag=flag, $
		   GROUP=event.top
;	        loadct, 3
		HANDLE_VALUE, save.hbright, tmp_bright, /set, /NO_COPY
		HANDLE_VALUE, hsave, save, /SET
	END
		


; produce maps

	'PMAP':	BEGIN
MAPPING:
	HANDLE_VALUE, save.hpar, par
		; check input data
		IF NOT par.aapin THEN RETURN
		IF par.ffield GT 0 THEN ffield=par.ffield ELSE ffield=0

		; produce the maps
		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		IF par.datatype GE 4 THEN flux=1 ELSE flux=0
		PIA_MAPAAP, aap, save.hbright, par.pixselect(0:par.npixel-1), $
			par.pixsize, par.rota, event=event, $
			shift_x=par.shx, shift_y=par.shy, $
			map_bright, map_uncert, map_expose, trans, $
			mmethod = par.mapmethod, fraction=par.dropfct, $
			trans_ra, trans_dec, FFIELD=par.ffield, ERROR=err, $
			percentile=par.percentile
		IF err THEN print, 'error found during map processing'
		IF err THEN RETURN
		par.trans = trans
		par.trans_ra = trans_ra
		par.trans_dec = trans_dec
		par.mapin = 1

		; draw the maps
		handle_value, aap.hdr, thdr
		; By WMT
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map

		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
;		CASE par.mapmethod OF
;		 0: mm = 'Full_cov'
;		 1: mm = 'Dist_wgt'
;		 2: mm = 'Trigrid '
;		 3: mm = 'Trig_qnt'
;		ENDCASE
		
		FXADDPAR, thdr, 'PRM_METH', mm, ' Method used to produce the map', $
			BEFORE='XTENSION'
		IF par.mapmethod GE 1 THEN $
		 FXADDPAR, thdr, 'PRM_DROP', par.dropfct, ' Pixel drop factor used', $
			BEFORE='XTENSION'
		IF par.ffield EQ 4 THEN $
		 FXADDPAR, thdr, 'PRM_PERC', par.percentile, ' Percentile used ', $
			BEFORE='XTENSION'
		 
;		IF par.datatype GE 4 THEN title="Flux Density" ELSE $
;			title="Brightness"
		PIA_MAPDRAW, map_bright, trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" map", $
			HEADLINE=headline, mapunc=map_uncert
;		PIA_MAPDRAW, map_uncert, trans, par.pixsize, GROUP=event.top, $
;			TITLE="Uncertainty Map", $
;			HEADLINE="Uncertainty Map: "+par.admn_decoded
;		PIA_MAPDRAW, map_expose, trans, par.pixsize, GROUP=event.top, $
;			TITLE="Exposure Map", $
;			HEADLINE="Exposure Map: "+par.admn_decoded

		; for usage of 'Show' buttons we store the maps locally
		HANDLE_VALUE, save.hmapb, map_bright, /SET, /NO_COPY
		HANDLE_VALUE, save.hmapu, map_uncert, /SET, /NO_COPY
		HANDLE_VALUE, save.hmape, map_expose, /SET, /NO_COPY
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=1
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY

		END

; process P32 drift correction

	'PDRC':	BEGIN
		IF NOT par.aapin THEN RETURN
		IF par.aotn NE "P32" THEN RETURN
		PIA_MAPW_DRIFT, save.hpar, save.haap, save.hbright, $
				GROUP=event.top
		END

; produce simple maps for P32 or PHT-P

	'PSIM':	BEGIN

		; check input data
		IF NOT par.aapin THEN RETURN
		simple = (par.aotn EQ "P32") OR $
				(STRMID(par.detector,0,1) EQ 'P')
;		IF NOT simple THEN RETURN

		; produce the simple maps for P32 or PHT-P
		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		PIA_MAPSIMPLE_AAP, aap, save.hbright, $
			par.pixselect(0:par.npixel-1), $
			map_bright, map_uncert, map_expose, pixsize, trans, $
			FFIELD=par.ffield, ERROR=err
		IF err THEN RETURN
		par.pixsize = pixsize

		; draw the maps
		handle_value, aap.hdr, thdr
		; By WMT
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
		PIA_MAPDRAW, map_bright, trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" map", /NO_ROT, $
			HEADLINE=headline, /SIMPLE, mapunc=map_uncert
;		PIA_MAPDRAW, map_uncert, trans, pixsize, GROUP=event.top, $
;			TITLE="Uncertainty Map", $
;			HEADLINE="Uncertainty Map: "+par.admn_decoded
;		PIA_MAPDRAW, map_expose, trans, pixsize, GROUP=event.top, $
;			TITLE="Exposure Map", $
;			HEADLINE="Exposure Map: "+par.admn_decoded

		; for usage of 'Show' buttons we store the maps locally
		HANDLE_VALUE, save.hmapb, map_bright, /SET, /NO_COPY
		HANDLE_VALUE, save.hmapu, map_uncert, /SET, /NO_COPY
		HANDLE_VALUE, save.hmape, map_expose, /SET, /NO_COPY
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=1
		par.pixsize = pixsize
		par.trans = trans
		par.mapin = 1
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY

		END

	'PSIMPS': BEGIN

		; check input data
		IF NOT par.aapin THEN RETURN

		; produce the simple maps for P32 or PHT-P
		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		PIA_MAPSIMPLE_AAP, aap, save.hbright, $
			par.pixselect(0:par.npixel-1), $
			map_bright, map_uncert, map_expose, pixsize, trans, $
			ERROR=err
		IF err THEN RETURN

		; draw the maps
		handle_value, aap.hdr, thdr
		; By WMT
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
		PIA_MAPDRAW, map_bright, trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" Map", /NO_ROT, /SIMPLE, $
			HEADLINE=headline, mapunc=map_uncert

		; for usage of 'Show' buttons we store the maps locally
		HANDLE_VALUE, save.hmapb, map_bright, /SET, /NO_COPY
		HANDLE_VALUE, save.hmapu, map_uncert, /SET, /NO_COPY
		HANDLE_VALUE, save.hmape, map_expose, /SET, /NO_COPY
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=1
		par.pixsize = pixsize
		par.trans = trans
		par.mapin = 1
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY

		END

	'PSIMPC':	BEGIN

		; check input data
		IF NOT par.aapin THEN RETURN

		; produce the simple maps for P32 or PHT-P
		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		PIA_MAPSIMPLE_AAP, aap, save.hbright, $
			par.pixselect(0:par.npixel-1), $
			map_bright, map_uncert, map_expose, pixsize, trans, $
			ERROR=err, /pchop
		IF err THEN RETURN

		; draw the maps
		handle_value, aap.hdr, thdr
		; By WMT
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
		PIA_MAPDRAW, map_bright, trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" Map", /NO_ROT, /SIMPLE, $
			HEADLINE=headline, mapunc=map_uncert

		; for usage of 'Show' buttons we store the maps locally
		HANDLE_VALUE, save.hmapb, map_bright, /SET, /NO_COPY
		HANDLE_VALUE, save.hmapu, map_uncert, /SET, /NO_COPY
		HANDLE_VALUE, save.hmape, map_expose, /SET, /NO_COPY
		HANDLE_VALUE, save.hid, id
		WIDGET_CONTROL, id.mshm, SENSITIVE=1
;		WIDGET_CONTROL, id.mshs, SENSITIVE=1
		par.pixsize = pixsize
		par.trans = trans
		par.mapin = 1
		HANDLE_VALUE, save.hpar, par, /SET, /NO_COPY

		END

	'MPSEP':BEGIN

		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		pia_write_anything, 'Raster leg ', leg
		leg = FIX(leg(0))
		WIDGET_CONTROL, event.top, /HOURGLASS
		pia_sep_aap, aap, leg
		
		END


	'MPSEP_CSTEP':BEGIN

		WIDGET_CONTROL, event.top, /HOURGLASS
		HANDLE_VALUE, save.haap, aap
		pia_write_anything, 'Chopper step 1', chop
		cstep = FIX(chop(0))
		pia_write_anything, 'Chopper step 2 (if not desired:cancel)', $
			chop2
		cstep = FIX(chop)
		IF chop2(0) NE '' THEN cstep = [cstep, FIX(chop2)]
		WIDGET_CONTROL, event.top, /HOURGLASS
		pia_sep_aap_chop, aap, cstep
		
		END


	'SHH':	BEGIN
		CASE par.mapin of
		0: BEGIN
		  HANDLE_VALUE, save.haap, aap
		  HANDLE_VALUE, aap.hdr, hdr, /NO_COPY
		  PIA_DISP_HDR, hdr, par.admn, 'PHTAAP', GROUP=event.top
;		  PIA_VIEW_TXT, hdr, TITLE="AAP FITS header", $
;			NAME=par.admn, GROUP=event.top
		  HANDLE_VALUE, aap.hdr, hdr, /SET, /NO_COPY
		END
		1: BEGIN
		  HANDLE_VALUE, save.haap, aap
		  HANDLE_VALUE, aap.hdr, hdr, /NO_COPY
		  PIA_DISP_HDR, hdr, par.admn, 'PHTAAP', GROUP=event.top
;		  PIA_VIEW_TXT, hdr, TITLE="AAP FITS header", $
;			NAME=par.admn, GROUP=event.top
		  HANDLE_VALUE, aap.hdr, hdr, /SET, /NO_COPY
		END
		2: BEGIN
		  HANDLE_VALUE, save.haap, aap
		  HANDLE_VALUE, aap.hdr, hdr, /NO_COPY
		  PIA_DISP_HDR, hdr, par.admn, 'PHTMAP', GROUP=event.top
;		  PIA_VIEW_TXT, hdr, TITLE="MAP FITS header", $
;			NAME=par.admn, GROUP=event.top
		  HANDLE_VALUE, aap.hdr, hdr, /SET, /NO_COPY
		END
		ELSE:
		ENDCASE
		END

; show data arrays

	'SHA':	BEGIN
		IF NOT par.aapin THEN RETURN
		HANDLE_VALUE, save.haap, aap
		PIA_AAPVIEW, aap, GROUP=event.top
		END

; show brightness map

	'SHMB':	BEGIN
		CASE par.mapin of
		1: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapb, map
		HANDLE_VALUE, save.hmapu, mapunc
		IF N_ELEMENTS(mapunc) EQ 0 THEN BEGIN
	  		int_noise = get_map_noise(map)
			mapunc = map * 0. + int_noise
			MESSAGE, /INFO, $
			 'Warning: Map uncertainties calculated from "intrinsic noise"'
		 HANDLE_VALUE, save.hmapu, mapunc, /SET
		ENDIF 
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline, mapunc=mapunc
		
		END
		2: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapb, map
		HANDLE_VALUE, save.hmapu, mapunc
		IF N_ELEMENTS(mapunc) EQ 0 THEN BEGIN
	  		int_noise = get_map_noise(map)
			mapunc = map * 0. + int_noise
			MESSAGE, /INFO, $
			 'Warning: Map uncertainties calculated from "intrinsic noise"'
		 HANDLE_VALUE, save.hmapu, mapunc, /SET
		ENDIF 
;		pia_mapw_pickfilt, thdr, filt_ind
;		map = map(*,*,filt_ind)
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 0, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline, mapunc=mapunc
		END
		ELSE:
		ENDCASE
		END

; show brightness map

	'SHAN':	BEGIN
		CASE par.mapin of
		1: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapb, map
		HANDLE_VALUE, save.hmapu, mapunc
		IF N_ELEMENTS(mapunc) EQ 0 THEN BEGIN
	  		int_noise = get_map_noise(map)
			mapunc = map * 0. + int_noise
			MESSAGE, /INFO, $
			 'Warning: Map uncertainties calculated from "intrinsic noise"'
		 HANDLE_VALUE, save.hmapu, mapunc, /SET
		ENDIF 
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, -1, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, -1, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline, mapunc=mapunc
		END
		2: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapb, map
		IF N_ELEMENTS(mapunc) EQ 0 THEN BEGIN
	  		int_noise = get_map_noise(map)
			mapunc = map * 0. + int_noise
			MESSAGE, /INFO, $
			 'Warning: Map uncertainties calculated from "intrinsic noise"'
		 HANDLE_VALUE, save.hmapu, mapunc, /SET
		ENDIF 
;		pia_mapw_pickfilt, thdr, filt_ind
;		map = map(*,*,filt_ind)
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, -1, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 0, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline, mapunc=mapunc
		END
		ELSE:
		ENDCASE
		END

; show uncertainty map

	'SHMU':	BEGIN
		CASE par.mapin OF
		1: BEGIN
		HANDLE_VALUE, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapu, map
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 1, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 1, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline
		END
		2: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmapu, map
;		pia_mapw_pickfilt, thdr, filt_ind
;		map = map(*,*,filt_ind)
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 1, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 1, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline
		END
		ELSE:
		ENDCASE

		END

; show exposure map

	'SHME':	BEGIN
		CASE par.mapin OF
		1: BEGIN
		HANDLE_VALUE, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmape, map
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 2, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 2, GROUP=event.top, $
			TITLE=title + " Map", $
			HEADLINE=headline
		END
		2: BEGIN
		handle_value, save.haap, aap
		handle_value, aap.hdr, thdr
		HANDLE_VALUE, save.hmape, map
;		pia_mapw_pickfilt, thdr, filt_ind
;		map = map(*,*,filt_ind)
		; we parse the map type to pia_mapdraw, where 
		;   0: brightness map
		;   1: uncertainty map
		;   2: exposure map
		headline = GET_HEADLINE(par, 2, mm=mm, title=title)
		PIA_MAPDRAW, map, par.trans, par, thdr, 2, GROUP=event.top, $
			TITLE=title+" Map", $
			HEADLINE=headline
		END
		ELSE:
		ENDCASE
		END

; show statistics

	'SHS':	BEGIN
		Message, /INFO, 'Not yet implemented'
		END

; load color table

	'OCLT': XLOADCT, ncolors=!D.N_COLORS-12, bottom=1

; hypertext help
	'HELP': BEGIN
	   IF !VERSION.OS NE 'vms' THEN $
		spawn, '$w3net $PIA_HTML/um/mapping.html &' ELSE $
		spawn, 'w3net /pia_dsk/pia/html/um/mapping.html'
	END

	ELSE: 
	ENDCASE 
	RETURN 
	END 
 
; ***************************************************************************
; PIA_MAPW  -  PIA_MAPW main routine
; ***************************************************************************

PRO PIA_MAPW, GROUP=group 


; Start a colour table 
	loadct, 13, ncolors=!d.n_colors-12, bottom=1

; define program parameters

; Buffer_state added to the id structure to distingush between the
; measurement buffer or fits/internal buffer. (WMT, Feb 96)
; where the value 0 is the measurement and 1 for the fits/internal
; buffer state
	id = { $
		mpdef:0L,	$
		mpmesm:0L,	$
		mpdrc:0L,	$
		mpmap:0L,	$
		mpsim:0L,	$
		mpsep:0L,	$
		mpsepc:0L,	$
		mshh:0L,	$
		msha:0L,	$
		mshm:0L,	$
		mshs:0L,	$
		shbr:0L,	$
		shun:0L,	$
		shex:0L,	$
		shan:0L,	$
		admn:0L,	$
		plid:0L,	$
		osn:0L,		$
		aotn:0L,	$
		det:0L,		$
		raster:0L,	$
		buffer_state:0  $
	}

	; make a dummy trans structure
	trans = PIA_MKTRANS([0,0],[0,0],[1,1],0)

	par = { $
		mapmethod:0,	$	; Mapping method type
		dropfct:0.3,	$	; Mapping method type
		shx:0.,	$
		shy:0.,	$
		aapin:0,	$	; AAP data present flag
		mapin:0,	$	; maps produced flag
		detector:'',	$	; PHTAAP detector code
		filter:'',	$	; Filter_id
		aper:'',	$	; Aperture_id
		admn:'',	$	; PHTAAP administration string
		admn_decoded:'', $	; admn, decoded string
		aotn:'',	$	; AOT name
		rota:0.,	$	; rotation angle of map
		rota_orig:0.,	$	; rotation angle of map as in AAP
		ra:0.,		$	; right ascension of center of map
		dec:0.,		$	; declination of center of map
		npts:0,		$	; # points per raster line
		nlns:0,		$	; # raster lines
		dpts:0.,	$	; distance of raster points
		dlns:0.,	$	; distance of raster lines
		trans:trans,	$	; transformation structure
		trans_ra:0.,	$	; ra from pia_mapset
		trans_dec:0.,	$	; dec from pia_mapset
		datatype:0,	$	; mean, median, 1st, 3rd quartil
		pixsize:[0.,0.], $	; chosen map pixel size
		npixel:1,	$	; the number of pixels of the detector
		pixselect:[1,1,1,1,1,1,1,1,1], $ ; detector pixel selection
		percentile: 25, $ 	; percentile to be used for FF
		ffield:0	$	; automatic flat fielding
	}

; set up widget hierarchy

	id_base = WIDGET_BASE( TITLE="PIA Mapping", /COLUMN ) 

	id_menu = WIDGET_BASE( id_base, /ROW ) 

	id_0 = WIDGET_BUTTON( id_menu, VAL="   File   ", /MENU )
	id_1 = WIDGET_BUTTON( id_0, VAL="Select measurement", UV="FOMA" )
	id_1 = WIDGET_BUTTON( id_0, VAL="IRAF's Drizzle batch", /MENU )
	id_2 = WIDGET_BUTTON( id_1, VAL="Using buffer", UVA="DRIZZLE" )
	id_2 = WIDGET_BUTTON( id_1, VAL="Using Internal files", $
	   			UVA="DRIZZLE_IAAP" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Read FITS: PHT Type", UVA="RFIT" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Read FITS: CAM Type", UVA="RFIT_CAM" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Read Internal files", UVA="RINT" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Internal maps operations", /MENU )
	id_2 = WIDGET_BUTTON( id_1, VAL="Divide Internal files", /MENU)
	id_3 = WIDGET_BUTTON( id_2, VAL=" as given", UVA="RINT_DIV" )
	id_3 = WIDGET_BUTTON( id_2, VAL=" norm to peak", UVA="RINT_DIV_NORM" )
	id_2 = WIDGET_BUTTON( id_1, VAL="Subtract Internal files", /MENU )
	id_3 = WIDGET_BUTTON( id_2, VAL=" as given", UVA="RINT_SUB" )
	id_3 = WIDGET_BUTTON( id_2, VAL=" 'Bckgr.' subtracted", $
			UVA="RINT_SUB_SUBTR" )
	id_2 = WIDGET_BUTTON( id_1, VAL="Correlate maps", /MENU )
	id_3 = WIDGET_BUTTON( id_2, VAL=" as given", UVA="CORR_MAP" )
	id_3 = WIDGET_BUTTON( id_2, VAL=" 'Bckgr.' subtracted", $
			UVA="CORR_MAP_SUBTR" )
	id_3 = WIDGET_BUTTON( id_2, VAL=" norm to peak", $
			UVA="CORR_MAP_NORM" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Quit", UVA="QUIT" )

	id_0 = WIDGET_BUTTON( id_menu, VAL=" Process ", /MENU )
	id.mpdef = WIDGET_BUTTON( id_0, VAL="Define map parameters",UVA="PDEF")
	id.mpmesm = WIDGET_BUTTON( id_0, $
	 VAL= "Median filtering method application", $
				UVA='MPMESM')
	id.mpmap = WIDGET_BUTTON( id_0, VAL="Produce map", UVA="PMAP" )
;	id.mpdrc = WIDGET_BUTTON( id_0,VAL="Perform P32 drift correction", $
;					UV="PDRC")
	id.mpsim = WIDGET_BUTTON( id_0, VAL= "Produce map ignoring pointing...", $
					/MENU)
	id_mpsim = WIDGET_BUTTON( id.mpsim, VAL="PHT-C simple map",$
					UVA="PSIM" )
	id_mpsimps = WIDGET_BUTTON( id.mpsim, VAL="PHT-P simple map (staring)",$
					UVA="PSIMPS" )
	id_mpsimpc = WIDGET_BUTTON( id.mpsim, VAL="PHT-P simple map (chopped)",$
					UVA="PSIMPC" )
	id.mpsep = WIDGET_BUTTON( id_0, VAL= "Produce scan with one raster leg",$
				UVA='MPSEP')
	id.mpsepc = WIDGET_BUTTON( id_0, VAL= "Do map choosing chopper step(s)", $
				UVA='MPSEP_CSTEP')
	idinter = WIDGET_BUTTON(id_0, VAL="Animate PHT-S map", UVA="XINT")

	id_0 = WIDGET_BUTTON( id_menu, VAL="   Show   ", /MENU )
	id.mshh = WIDGET_BUTTON( id_0, VAL="Header", UVA="SHH" )
	id.msha = WIDGET_BUTTON( id_0, VAL="Data arrays", UVA="SHA" )
	id_1 = WIDGET_BUTTON( id_0, VAL="Map", /MENU )
	id.mshm = id_1
	id.shbr = WIDGET_BUTTON( id_1, VAL="Brightness", UVA="SHMB" )
	id.shun = WIDGET_BUTTON( id_1, VAL="Uncertainty", UVA="SHMU" )
	id.shex = WIDGET_BUTTON( id_1, VAL="Exposure", UVA="SHME" )
	id.shan = WIDGET_BUTTON( id_1, VAL="Undefined", UVA="SHAN" )
;	id.mshs = WIDGET_BUTTON( id_0, VAL="Statistics", UVA='SHS' )

	id_0 = WIDGET_BUTTON( id_menu, VAL=" Options ", /MENU ) 
	id_1 = WIDGET_BUTTON( id_0, VAL="Colors", /MENU ) 
	id_2 = WIDGET_BUTTON( id_1, VAL="Load Color Table...", UVA="OCLT" ) 

	id_0 = WIDGET_BUTTON( id_menu, VAL="   Help   ", UVALUE="HELP") 

	id_info = WIDGET_BASE( id_base, /COLUMN )

	id_0 = WIDGET_BASE( id_info, /ROW )
	id_1 = WIDGET_LABEL( id_0, $
		VALUE="Measurement:  " )
	id.admn = WIDGET_LABEL( id_0, $
	   VALUE="                                                 " )

	id_0 = WIDGET_BASE( id_info, /ROW )
	id_1 = WIDGET_LABEL( id_0, $
		VALUE="Proposal ID:             " )
	id.plid = WIDGET_LABEL( id_0, VALUE="                          " )

;	id_0 = WIDGET_BASE( id_info, /ROW )
;	id_1 = WIDGET_LABEL( id_0, $
;		VALUE="Observation sequence number:      " )
;	id.osn = WIDGET_LABEL( id_0, VALUE="     " )

	id_0 = WIDGET_BASE( id_info, /ROW )
	id_1 = WIDGET_LABEL( id_0, $
		VALUE="AOT:                                   " )
	id.aotn = WIDGET_LABEL( id_0, VALUE="        " )

	id_0 = WIDGET_BASE( id_info, /ROW )
	id_1 = WIDGET_LABEL( id_0, $
		VALUE="Detector:    " )
	id.det = WIDGET_LABEL( id_0, VALUE="     " )
        id_1 = WIDGET_LABEL( id_0, $
               VALUE="   Filter: " )
        id.osn = WIDGET_LABEL( id_0, VALUE="                    " )

	id_0 = WIDGET_BASE( id_info, /ROW )
	id_1 = WIDGET_LABEL( id_0, $
		VALUE="Raster (points x lines):           " )
	id.raster = WIDGET_LABEL( id_0, VALUE="          " )

; realize widget

	hsave = HANDLE_CREATE()
	hid = HANDLE_CREATE( hsave, VALUE=id )
	hpar = HANDLE_CREATE( hsave, VALUE=par )
	haap = HANDLE_CREATE( hsave )
	hbright = HANDLE_CREATE( hsave )
	hmapb = HANDLE_CREATE( hsave )
	hmapu = HANDLE_CREATE( hsave )
	hmape = HANDLE_CREATE( hsave )
	save = { hid:hid, hpar:hpar, haap:haap, hbright:hbright, $
		hmapb:hmapb, hmapu:hmapu, hmape:hmape }
	HANDLE_VALUE, hsave, save, /SET, /NO_COPY

	WIDGET_CONTROL, id_base, /REALIZE, SET_UVALUE=hsave
	WIDGET_CONTROL, id.mpdef, SENSITIVE=0
	WIDGET_CONTROL, id.mpmesm, SENSITIVE=0
	WIDGET_CONTROL, id.mpmap, SENSITIVE=0
;	WIDGET_CONTROL, id.mpdrc, SENSITIVE=0
	WIDGET_CONTROL, id.mpsim, SENSITIVE=0
	WIDGET_CONTROL, id.mpsep, SENSITIVE=0
	WIDGET_CONTROL, id.mpsepc, SENSITIVE=0
	WIDGET_CONTROL, id.mshh, SENSITIVE=0
	WIDGET_CONTROL, id.msha, SENSITIVE=0
	WIDGET_CONTROL, id.mshm, SENSITIVE=0
;	WIDGET_CONTROL, id.mshs, SENSITIVE=0

	xMANAGER,No_BLOCK=1, "PIA_MAPW", id_base, GROUP=group, CLEANUP="PIA_MAPW_CLEANUP" 

	RETURN 
	END 
