;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_MKTRANS
;
; PURPOSE:
;	Make a transformation structure appropriate for usage with
;	routines PIA_XY2RD and PIA_RD2XY.
;
;	No checks of input parameters are made.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	trans = PIA_MKTRANS( crval, crpix, pixsize, theta )
;
; INPUTS:
;	CRVAL		Reference sky position [ RA, DEC ] in degrees
;
;	CRPIX		Reference map point [ X0, Y0 ]
;
;	PIXSIZE		Pixel size of map [ Xsize, Ysize ] in arcsec
;
;	THETA		Rotation of map [x,y] axes relative to sky [-RA,DEC]
;			coordinates at the reference point in degrees.
;			Theta increases counterclockwise.
;
; KEYWORD PARAMETERS:
;	(none)
;
; OUTPUTS:
;	TRANS		Transformation structure:
;	.CRVAL		The reference sky position in degrees
;	.CRPIX		The reference map position
;	.CD		The forward transformation matrix X,Y -> RA,DEC in rad
;	.DC		The back transformation matrix RA,DEC -> X,Y in rad
;
; PROCEDURES USED:
;	none
;
; NOTES:
;	Note that this definition of the CD matrix is different
;	from the ASTROLIB structure ASTR. Here the usage is:
;
;	[ delta_xi  ]   [ CD(0,0) CD(0,1) ]   [ delta_x   ]
;	|           | = |                 | * |           |
;	[ delta_eta ]   [ CD(1,0) CD(1,1) ]   [ delta_y   ]
;
;	[ delta_x   ]   [ DC(0,0) DC(0,1) ]   [ delta_xi  ]
;	|           | = |                 | * |           |
;	[ delta_y   ]   [ DC(1,0) DC(1,1) ]   [ delta_eta ]
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	March 1995
;	CRVAL and CRPIX -> double precision (CG)	March 1998
;	Forcing cd and dc to single precision to
;	avoid problems with maps with different 
;	formats in a declared structure (CG)		May 1999
;	V7.4
;-

	FUNCTION PIA_MKTRANS, crval, crpix, pixsize, theta

	cd = fltarr(2,2)
	t = theta * !DTOR
	cd(0,0) = -cos(t)
	cd(0,1) = sin(t)
	cd(1,0) = sin(t)
	cd(1,1) = cos(t)
	cdelt = [ [pixsize(0),0], [0,pixsize(1)] ] *!DTOR/3600.
	cd = cd # cdelt

	dc = INVERT(cd)

	trans = {trans_gnomonic2, $
		crval:DOUBLE(crval), crpix:DOUBLE(crpix), $
		cd:FLOAT(cd), dc:FLOAT(dc) }

	RETURN, trans
	END
