;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:     
;	pia_multi_fit_nomodal
;
; PURPOSE:
;	        Perform different fits in a widget environment 
;		(Same as pia_multi_fit, but in widget /nomodal mode)
;
; CATEGORY:
;	PIA - graphics (+ processing)
;
; CALLING SEQUENCE:
;       pia_multi_fit_nomodal, xin, yin ,xout=xout, yout=yout, weight=weight, $
;		a=a, sigmaa=sigmaa, xrange=xrange, yrange=yrange, psym=psym
;                                    
; INPUTS:        Name      I/O   Type       Description
;                ---------------------------------------------------------
;                xin        I    fltarr     x data
;                yin        I    fltarr     y data
;
; KEYWORD PARAMETERS:
;                xout       O    fltarr     x data    
;                yout       O    fltarr     y data    
;                weight     I    fltarr     weighting 
;                a          O    fltarr     parameter data
;                sigmaa     O    flt/fltarr stand.dev.    
;                xrange     I    fltarr     xrange        
;                yrange     I    fltarr     yrange        
;                psym       I    integer    plot-symbol   
;
; OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	pia_multi_fit_nm_cmn
;	colour_indices
;	multi_fit_help
;	dfermi
;
; PROCEDURES USED:
;	Astronomical Library:
;		match
;	Internal:
;		nfit_event
;		nfct_sum
;		nproposal
;		nplus_event
;		npar_in_ev
;		npar_cur_val_ev
;		ncursor_ev
;		npar_fix
;		npeak_del
;		nsel_desen
;		nsel_box
;		nselection_ev
;		ndegree_event
;		ndesi
;		nresi
;		nmess
;		nmess_ev
;		ntext_event
;		ncancel_event
;		nnext_event
;		nnew_range
;		nfile_event
;		nprint_view
;		nhelp_event
;		nexit_event
;
; MODIFICATION HISTORY:
;	Written by:	 Carlos GABRIEL (ESA/VILSPA) 	July 1995
;		[Adapted from a ISO-SWS (K. Seidenschwang)
;		 routine (multi_fit.pro) for its use within the PHT-IA]
;	Oct 95 Version 3.5
;	Common pia_multi_fit_res added  CG		October 1995
;-
 
; common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
;                weight_sel, p_field, p_field_vf, var_ind, sig  , $
;                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
;                funct_str, cur_par, peak
;
; pia_multi_fit_nm_cmn = common-block with global variables
;
; x            fltarr  input x-data in increasing order
; y            fltarr  input y-data 
; yfit         fltarr  selection area (computed by fit)
; gewicht      fltarr  weight of input data (vector), 0. < gewicht < 1.
; sig          fltarr  standard-deviation
;                      for polyfitw: standard-deviation in y units
; p_field      fltarr  field of var_parameter for the fit-function
; p_field_vf   fltarr  fix_var_values of parameters
; var_ind      intarr  indices of variable parameter
; win_ids      struct  window-ids
; xra          fltarr  plot-area
; yra          fltarr  plot-area
; ps           int     plot-symbol  (default=3)
; x_sel        fltarr  selected x_range
; y_sel        fltarr  selected y_range
; gewicht_sel  fltarr  weight for selected part
; ind_min      int     index of selected part y(ind_min) = y_sel(0)
; y_n          int     n_elements of y_sel
; ndegree      int     degree of basic-polynom
; nfunct       int     number of applied functions
; funct_str    strarr  names of applied functions
; par_n        int     number of required parameter (< 50)
; cur_par      string  name of parameter determined by cursor
; peak         struct  structure containing number and position of peaks

;-----------------------------------------------------------------------
@multi_fit_help.pro

  PRO HTMLHELP_EVENT, event
  
  IF !VERSION.OS NE 'vms' THEN $
    spawn, '$w3net $PIA_HTML/um/multifit.html &' ELSE $
    spawn, 'w3net /pia_dsk/pia/html/um/multifit.html',/nowait

 END

;-----------------------------------------------------------------------
;execute fit, write results to assigned windows 
;----------------------------------------------------------------------
 pro nfit_event, event

@colour_indices
 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak
COMMON pia_multi_fit_resu, xres, yres, parres, parsigres, parres_nam


 widget_control, win_ids.fit, hourglass = 1

 if ( max( abs(x_sel) ) ge 100. ) then $
   MESSAGE,/info, $
    'polynom-fit might go wrong for higher order and large x-values'

 if ( nfunct eq 1 ) then begin          ; only polynom
    widget_control, win_ids.poly_degree, get_value=h_str

;if degree-input was not confirmed:
    if ( fix( h_str(0) ) ne ndegree ) then begin
      nmess,'poly_degree !!'
      widget_control, win_ids.poly_degree, set_value=string(ndegree) 
    endif

    p_field_vf(0:ndegree) = $
                polyfitw(x_sel,y_sel,weight_sel,ndegree,yfit,yband,sig) 

    ndesi, win_ids.selection

 endif else begin  ; polynom + additional_function
   p_field = p_field * 1.d0
   yfit = curfit(x_sel,y_sel,weight_sel,p_field,sig_v,funct='nfct_sum',/quiet)
   p_field_vf(var_ind) = p_field
   sig                 = fltarr(par_n)
   sig(var_ind)        = sig_v
 endelse

 xxplfit = indgen(N_ELEMENTS(x_sel)*10) * (max(x_sel) - min(x_sel)) / $
 		(N_ELEMENTS(x_sel)*10) + x_sel(0) 
 xxplfit = [xxplfit, max(x_sel)]
 nfct_sum, xxplfit, p_field, yyplfit
 widget_control, win_ids.file_io, set_uvalue=1
 oplot, x_sel, yfit, color = red, psym=5 
 oplot, xxplfit, yyplfit, color=red
; pia_xplot, x_sel, yfit, overplot=99
;write parameter to display field
 for i=0, par_n - 1 do $
   widget_control, win_ids.p_id(i), set_value=string(p_field_vf(i)) 

 npeak_del
 fct_num = 0
 for i = ndegree+2, par_n-1, 3  do begin
   fct_num              = fct_num + 1
   widget_control, win_ids.p_id(i), get_value = h_par
   peak.number(fct_num) = strcompress( fct_num,/remove_all )
   peak.pos(fct_num)    = float( h_par(0) )
 endfor

 xyouts, peak.pos, peak.y, peak.number, align=0.5

;residuen_plot:
 widget_control, win_ids.res_bild, get_value=window
 wset,window
 resid   = ( y_sel - yfit ) 
 resid_g = resid * weight_sel

 plot, x_sel, resid, xst=1, xrange=xra, title='residuum'
; , background=white, color=black
 oplot,x_sel, resid_g, line=1, color=red

 x0    = !x.crange(0) 
 xn    = !x.crange(1)
 y0    = !y.crange(0)
 yn    = !y.crange(1)
 y_r   = y0 - (yn-y0) * 0.18
 y_r_g = y0 - (yn-y0) * 0.25

 plots,  [x0 + (xn-x0)*0.05 ,x0+(xn-x0)*0.15], [y_r,y_r]
 plots,  [x0 + (xn-x0)*0.05 ,x0+(xn-x0)*0.15], [y_r_g,y_r_g], line=1
 xyouts,  x0 + (xn-x0)*0.2  ,y_r, 'standard deviation'+ $
                             strcompress(stdev(resid)),charsize=0.8
 xyouts,  x0 + (xn-x0)*0.2  ,y_r_g, 'st. deviation, weighted '+ $
                             strcompress(stdev(resid_g)),charsize=0.8
 text= ['standard deviation'+ $
                             strcompress(stdev(resid))]
 text = [text, 'st. deviation, weighted '+ $
                             strcompress(stdev(resid_g))]
 plots, [xra(0),xra(1)], [0,0], line=2

; pia_xplot, x_sel, resid, xst=1, ytitle="Residuum", overplot=-1, annot=text, lin=1
; pia_xplot,x_sel, resid_g, line=1, overplot=99

;pia_disp_print_file, "", title='Standard Deviation', text=text

 widget_control, win_ids.res_bild, map=1
 ndesi, win_ids.fit
 nresi, win_ids.plus
 widget_control, win_ids.fit_bild, get_value=f
 wset, f

; fill the common used by other programs to get the fit results 
 xres = x_sel
 yres = yfit
 parres = p_field_vf
 parsigres = sig
 parres_nam = strarr(max([N_ELEMENTS(parsigres),2]))
 
 II = 0
 FOR I=0,nfunct-1 DO BEGIN
   CASE STRMID(funct_str(I),5,1) OF
     'p': BEGIN
      	parres_nam(II) = "Pol. 0th"
	II = II+1
	parres_nam(II) = "Pol. 1st"
	II= II+1
     END
     
     'g': BEGIN
      	parres_nam(II) = "Gauss: Height"
	II = II+1
	parres_nam(II) = "Gauss: Center"
	II= II+1
	parres_nam(II) = "Gauss: Width"
	II= II+1
     END
     
     '(': BEGIN
      	parres_nam(II) = "(sin(x)/x)^2): Height"
	II = II+1
	parres_nam(II) = "(sin(x)/x)^2): Center"
	II= II+1
	parres_nam(II) = "(sin(x)/x)^2): Phase Width"
	II= II+1
     END
     
     'e': BEGIN
      	parres_nam(II) = "Expon.: Height"
	II = II+1
	parres_nam(II) = "Expon.: Center"
	II= II+1
	parres_nam(II) = "Expon.: Width"
	II= II+1
     END
     
     'c': BEGIN
      	parres_nam(II) = "Cauchy: Height"
	II = II+1
	parres_nam(II) = "Cauchy: Center"
	II= II+1
	parres_nam(II) = "Cauchy: FWHM"
	II= II+1
     END
     
     'd': BEGIN
      	parres_nam(II) = "Double Fermi: Height"
	II = II+1
	parres_nam(II) = "Double Fermi: Center"
	II= II+1
	parres_nam(II) = "Double Fermi: Width"
	II= II+1
     END
   ENDCASE  
  ENDFOR
 end ; nfit_event

;----------------------------------------------------------------------
;collect all required fit-functions in one function to call curvefit
;----------------------------------------------------------------------
 pro nfct_sum, xx, a, f, pder

 common dfermi, dfwid
 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 f                   = fltarr(N_ELEMENTS(xx))        ; function values
; added to avoid bug when coming from outside into this routine (CG)
 IF N_ELEMENTS(a) NE N_ELEMENTS(var_ind) THEN $
   p_field_vf(var_ind) = a(var_ind) ELSE $
 p_field_vf(var_ind) = a                  ; turn arround of var_parameter
 n_var               = n_elements(var_ind)
 pder                = fltarr(y_n,n_var)
 index               = ndegree-5
 k                   = 0                  ; index of pder
 aa                  = p_field_vf         ; parameterfield (fix and var)

;instead of aa introduce p_field_vf 

 for i = 0,nfunct-1 do begin              ; funktion-data
   index = index + 3                      ; position of 'a(0)' in
                                          ; parameterfield aa
   fct   = strmid(funct_str(i),5,1)

   case fct of
     'p' : begin                              ; **** poly ****
           for j = 0, ndegree do begin
             f = f*xx + aa(ndegree-j)
             if (n_params(0) gt 3) then begin
               if (var_ind(k) eq j) then begin
                 pder(0,k) = xx^j
                 k = (k+1) < (n_var-1)
               endif
             endif
           endfor
           end

     'g' : begin                              ; **** gauss ****
           if aa(index+2) ne 0.0 then $
             z = (xx-aa(index+1)) / aa(index+2) $
           else $
             z = 10.
           ez = exp(-z^2/2.) * (abs(z) le 7.)
           f  = f + aa(index)*ez
           if (n_params(0) gt 3) then begin
             for j = 0,2 do begin
               if ( var_ind(k) eq index+j ) then begin
                 case j of
                   0 : pder(0,k) = ez
                   1 : if aa(index+2) ne 0.0 then $
                       pder(0,k) = aa(index)*ez*z/aa(index+2) 
                   2 : if aa(index+2) ne 0.0 then $
                       pder(0,k) = (aa(index)*ez*z/aa(index+2))*z 
                 endcase
                 k = (k+1) < (n_var-1)
               endif
             endfor
           endif
           end

     '(' : begin                              ; **** sin.. ****
           xa = aa(index+2) * (xx - aa(index+1))
           z  = sin(xa)/xa
           f  = f + aa(index)*z^2
           if (n_params(0) gt 3) then begin
              for j = 0,2 do begin
                if (var_ind(k) eq index+j) then begin
                   case j of
                     0 : pder(0,k) = z^2
                     1 : pder(0,k) = -(2.*aa(index)*z/aa(index+2) * $
                                    (cos(xa)-z))* aa(index+2)/(xx-aa(index+1))
                     2 : pder(0,k) = 2.*aa(index)*z/aa(index+2)*(cos(xa)-z)
                   endcase
                 k = (k+1) < (n_var-1)
               endif
             endfor
           endif
           end

     'e' : begin                              ; **** expo ****
           z = aa(index) * exp(-abs(xx-aa(index+1)) / aa(index+2))
           f = f + z
           if (n_params(0) gt 3) then begin
             for j = 0,2 do begin
               if ( var_ind(k) eq index+j ) then begin
                  case j of
                    0 : pder(0,k) = z  / aa(index)
                    1 : pder(0,k) = z  / aa(index+2)*(2.*(xx lt aa(index+1))-1)
                    2 : pder(0,k) = 1. / aa(index+2) / aa(index+2)* $
                                         abs( xx-aa(index+1) )*z
                  endcase
                 k = (k+1) < (n_var-1)
               endif
             endfor
           endif
           end

     'c' : begin                              ; **** cauchy ****
           f = f + aa(index) / (1.+(2.*(xx-aa(index+1)) / aa(index+2))^2)
           if (n_params(0) gt 3) then begin
             for j = 0,2 do begin
               if (var_ind(k) eq index+j) then begin
                  case j of
                    0 : pder(0,k) = 1. / ( 1.+(2.*(xx-aa(index+1) )  / $
                                           aa(index+2))^2 )
                    1 : pder(0,k) = 8.* aa(index) * (xx-aa(index+1) )/$
                                    aa(index+2)^2 / $
                                   (1.+ (2.*(xx-aa(index+1))/aa(index+2))^2)^2
                    2 : pder(0,k) = 8. * aa(index) * (xx-aa(index+1))^2   / $
                                    aa(index+2)^3/(1.+(2.*(xx-aa(index+1))/ $
                                    aa(index+2))^2)^2
                  endcase
                  k = (k+1) < (n_var-1)
               endif
             endfor
           endif
           end
      'd' : begin                              ; **** double fermi ****
	   IF N_ELEMENTS(dfwid) EQ 0 THEN dfwid = 0.7 
           f = f + aa(index) * exp(aa(index+2)/2/dfwid) / $
           	(2*(cosh((xx-aa(index+1))/dfwid) + $
           	cosh(aa(index+2)/2/dfwid))) 

           if (n_params(0) gt 3) then begin
             for j = 0,2 do begin
               if (var_ind(k) eq index+j) then begin
                  case j of

                    0 : pder(0,k) = exp(aa(index+2)/2/dfwid) / $
                    		(2*(cosh((xx-aa(index+1))/dfwid) + $
                    		 cosh(aa(index+2)/2/dfwid)))
                    
                    1 : pder(0,k) = 2/dfwid * aa(index) * $
                    	exp(aa(index+2)/2/dfwid) * $
                    		sinh((xx-aa(index+1))/dfwid) / $
	 		(4 * (cosh((xx-aa(index+1))/dfwid) + $
	 		cosh(aa(index+2)/2/dfwid))^2)

                    2 : pder(0,k) = 1 / 2*dfwid *aa(index)* $
                    	exp(aa(index+2)/2/dfwid) / $
                    		(2*cosh((xx-aa(index+1)/dfwid) + $
                    		 2*cosh(aa(index+2)/2/dfwid))) - $
                    		aa(index) * exp(aa(index+2)/2/dfwid) * $
                    		sinh(aa(index+2)/2/dfwid) / $
                    		(2*cosh((xx-aa(index+1))/dfwid) + $
                    		 2*cosh(aa(index+2)/2/dfwid))^2 / dfwid  
                    
                  endcase
                  k = (k+1) < (n_var-1)
               endif
             endfor
           endif
           end
   endcase
 endfor

 end ; nfct_sum

;----------------------------------------------------------------------
; only called if nfunct = 2 (polynom + 1 additional function)
; and only called for first fit,
; estimated start values for parameter: background and startparameter
;----------------------------------------------------------------------
 pro nproposal, fct_name

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 a      = fltarr(4)
 fct    = strmid(fct_name,5,1)
 c      = polyfitw(x_sel,y_sel,weight_sel,1,yf)
 backgr = c(0)
 yd     = y_sel-yf

 if (fct eq '(' or fct eq 'c' or fct eq 'd') then begin  ; sin.. ,cauchy or d.fermi
   ymax = max(yd)
   roi  = where(yd gt .5*ymax)
   i0   = roi(0)
   i1   = roi(n_elements(roi)-1)
   a(1) = ymax                          ; height 
   if (fct eq '(') then begin
     a(2) = x_sel(!c)*1.01              ; center 
     a(3) = !pi/(x_sel(i1)-x_sel(i0))   ; width  
   endif else if (fct eq 'd') then begin
     a(2) = x_sel(!c)*1.01              ; center 
     a(3) = abs(x_sel(i1)-x_sel(i0))   	; distr. width   
   endif else begin
     a(2) = x_sel(where(yd eq ymax))    ; center 
     a(3) = abs(x_sel(i1)-x_sel(i0))    ; width  
   endelse

 endif else begin
   ymax = max(yd)                       ; x,y and subscript of extrema 
   xmax = x_sel(!c) 
   imax = !c 
   ymin = min(yd) 
   xmin = x_sel(!c) 
   imin = !c

   if abs(ymax) gt abs(ymin) then $     ; emiss or absorp?
     i0 = imax $
   else $
     i0 = imin 

   i0  = i0 > 1 < (y_n-2)	        ;never take edges
   dy  = yd(i0)		                ;diff between extreme and mean
   del = dy / exp(1.)	                ;1/e value
   i   = 0
   while ( (i0+i+1) lt y_n ) and ((i0-i) gt 0) and $    ;guess at 1/2 width.
   (abs(yd(i0+i)) gt abs(del)) and (abs(yd(i0-i)) gt abs(del)) do $
      i=i+1

   !c   =0		             ; reset cursor for plotting
   a(1) = yd(i0)                     ; height 
   a(2) = x_sel(i0)                  ; center 
   a(3) = abs(x_sel(i0)-x_sel(i0+i)) ; width  

 endelse

 widget_control,win_ids.p_id(0),set_value = string(backgr)

 for i = 1,3 do $
   widget_control,win_ids.p_id(ndegree+i),set_value = string(a(i))

 end ; end nproposal

;----------------------------------------------------------------------
;show selected fit function
;fix the parameter definition
;fill funct_str, nfunct, par_n
;----------------------------------------------------------------------
 pro nplus_event,event                ; event.value = funktion-name

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 if ( par_n+3 le 50 ) then begin

   mark              = '('+string(nfunct,format='(i2)')+') '
   funct_str(nfunct) = mark + event.value ; first additional funct. on pos. 1
   ndesi, win_ids.poly_degree
   ndesi, win_ids.fit
   widget_control, win_ids.poly_but_base, map=0
;if the degree-input is not confirmed :
   widget_control, win_ids.poly_degree, get_value=h_str

   if (fix(h_str(0)) ne ndegree) then begin
      nmess,'poly_degree !!'
      widget_control, win_ids.poly_degree, set_value=string(ndegree) 
   endif

   widget_control, win_ids.formel, set_value = funct_str
                  
   str    = strarr(3)
   str(0) = 'height'
   str(1) = 'center'
   str(2) = '1/e width'

   case event.value of
     '(sin(x)/x)^2' : begin
                        for i = 0,1 do $
                          widget_control, win_ids.par_def(par_n+i), $
                                          set_value = mark + str(i)

                          widget_control, win_ids.par_def(par_n+2), $
                                          set_value = mark + 'phase width'
                      end 
    'cauchy'        : begin
                       for i = 0,1 do $
                          widget_control, win_ids.par_def(par_n+i), $
                                          set_value = mark + str(i)

                          widget_control, win_ids.par_def(par_n+2), $
                                          set_value = mark + 'fwhm'
                      end 
    'double fermi'   : begin
                       for i = 0,1 do $
                          widget_control, win_ids.par_def(par_n+i), $
                                          set_value = mark + str(i)
                          widget_control, win_ids.par_def(par_n+2), $
                                          set_value = mark + 'width'

                      end 
    else            :  for i = 0,2 do $
                          widget_control, win_ids.par_def(par_n+i), $
                                          set_value = mark + str(i)
   endcase

   nfunct = nfunct + 1 
   par_n  = par_n + 3

 endif else $
   nmess, ['no additional functions','possible']

 end ; nplus_event

;----------------------------------------------------------------------
;Examination of <OK> and <RESET> in window for parameter input using 
;cursor
; ok    : fulfill data handling
;         not or incomplete determined data will not be passed
;         write peak-nr. into graphic
; reset : remove (wrong) input 
;----------------------------------------------------------------------
 function npar_cur_ev, event 

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 case event.value of
  'ok'     : begin
             widget_control, win_ids.par_cur_fct, get_value=h_fct

             peak_num = strmid(funct_str(h_fct),1,2)
             peak_num = strcompress(peak_num,/remove_all)
             fct_num  = fix(peak_num)                  ; fkt-nr

             if ( fct_num eq 0 ) then begin            ;basis-polyn., background
               widget_control,win_ids.par_cur_anz(0),get_val=h_val

               if (h_val(0) ne 'not def.')then $
                  widget_control,win_ids.p_id(0),set_value=h_val

             endif else begin                          ; additional funktionen
               fct_str = strmid(funct_str(h_fct),5,1)  ; fkt_name
               pos     = ndegree + 3*(fct_num - 1)

               ; height:
               widget_control, win_ids.par_cur_anz(1), get_val=h_val
               pos_to = strpos(h_val,' to ?')
               if ( pos_to(0) ge 0 ) then begin        ; incomplete input
                 widget_control, win_ids.par_cur_anz(1), set_val='not def.'
                 nmess, ['incomplete','input of', '<height>']
               endif else if (h_val(0) ne 'not def.') then $
                 widget_control, win_ids.p_id(1+pos), set_value=h_val

               ; center:
               widget_control, win_ids.par_cur_anz(2), get_val=h_val
               if (h_val(0) ne 'not def.') then begin
                 peak.number(fct_num) = peak_num
                 widget_control, win_ids.p_id(2+pos), set_value=h_val
                 peak.pos(fct_num)    = float(h_val(0))
                 npeak_del
                 xyouts, peak.pos, peak.y, peak.number, align=0.5
               endif

               ; width:
               widget_control, win_ids.par_cur_anz(3), get_val=h_val
               pos_to = strpos(h_val,' to ?')
               if (pos_to(0) ge 0) then begin            ; incomplete input
                 widget_control, win_ids.par_cur_anz(3), set_val='not def.'
                 nmess, ['incomplete','input of', '<height> or <width>']
               endif else if (h_val(0) ne 'not def.') then begin
                 if (fct_str eq '(') then $
                   h_val = string(!pi/float(h_val))

                 widget_control, win_ids.p_id(3+pos), set_value=h_val
               endif

             endelse

             end

  'reset' : begin
             for i=0,3 do begin
               widget_control, win_ids.par_cur_anz(i), set_value='not def.'
               ndesi, win_ids.par_cur_val(i)
             endfor

             widget_control, win_ids.par_cur_fct, set_value=0
             widget_control, win_ids.par_cur_val(0), input_focus=1
             nresi, win_ids.par_cur_val(0)
             cur_par = 'backgr'

            end
 endcase

 return,event.value
 end   ; par_cur_ev end

;----------------------------------------------------------------------
;Evaluation of <cursor> , <ok>, ,reset> in window parameter definition
;
; cursor : Open window for cursor-input and activate cursor in graphic
; ok     : Take parameter for fit calculation, not filled data will are set
;          to zero
;          Evaluation of  <fix> and <var>,
;          Write peak-nr. to graphic
; reset  : Set (by edit or cursor) modified data back
;----------------------------------------------------------------------
 function npar_ein_ev, event 

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 test = widget_info(win_ids.par_cur_fct,/valid_id)

 case event.value of
   'cursor' : begin
              widget_control, win_ids.par_cur, map=1
              widget_control, win_ids.fit_bild, draw_button_events=1
              if not test then $
                win_ids.par_cur_fct = cw_bselector(win_ids.par_cur_ein, $
                                      funct_str(0:nfunct-1),/frame,     $
                                      event_funct='npar_cur_fct_ev',     $
                                      set_value=0,/return_name)         $
              else $
                widget_control, win_ids.par_cur_fct, set_value=0

              for i = 0,3 do begin
                widget_control, win_ids.par_cur_anz(i), set_value='not def.'
                ndesi, win_ids.par_cur_val(i)
              endfor

              widget_control, win_ids.par_cur_val(0), input_focus=1
              nresi, win_ids.par_cur_val(0)
              cur_par = 'backgr'
              end

   'ok'     : begin
              if test then widget_control, win_ids.par_cur_fct, /destroy
              widget_control, win_ids.fit_bild, draw_button_events=0
              widget_control, win_ids.par_cur, map=0
              widget_control, win_ids.input, map=0
              nresi, win_ids.fit
              nresi, win_ids.plus

              ; note peak-numbers in graphik
              npeak_del
              fct_num = 0

              for i = ndegree+2,par_n-1, 3 do begin
                fct_num = fct_num + 1
                widget_control, win_ids.p_id(i), get_value = h_par
                peak.number(fct_num) = strcompress(fct_num,/remove_all)
                if (h_par(0) ne 'not def.') then peak.pos(fct_num) = $
                   float(strcompress(h_par(0),/remove_all)) $
                else $
                   peak.pos(fct_num) = 0.

              endfor

              xyouts, peak.pos,peak.y, peak.number, align=0.5
   
;separate var u. fix parameter :
              j = -1
              for i = 0, par_n -1 do begin
                widget_control, win_ids.p_id(i), edit = 0
                widget_control, win_ids.p_id(i), get_value = h_par
                h_par = h_par(0)

                if (strpos(h_par,'not') ge 0) then begin
                  h_par = 0.
                  widget_control, win_ids.p_id(i) , set_value = '0.0'
                  widget_control, win_ids.p_fix(i), set_value = 'var'
                endif else $
                  h_par = float(strcompress(h_par,/remove_all))

                widget_control, win_ids.p_fix(i), get_value = h_fix

                if (h_fix(0) ne 'fix') then begin
                  j = j + 1
                  var_ind(j) = i                   ; index der variablen params
                  p_field(j) = h_par               ; array der variablen params
                endif

                p_field_vf(i) = h_par              ; array var und fix par.
              endfor

              var_ind = var_ind(0:j)
              p_field = p_field(0:j)
              end

 'reset'   :  begin
              widget_control, win_ids.par_cur , map=0
              widget_control, win_ids.fit_bild, draw_button_events=0
              test = where(p_field_vf,first)    ; first=0 fuer 1.lauf
              if (first eq 0) then begin
                for i = 0,par_n-1 do $
                   widget_control, win_ids.p_id(i), set_value='not def.'

                if (nfunct eq 2) then nproposal, funct_str(1)

              endif else $
                for i = 0,par_n-1 do $
                  widget_control,win_ids.p_id(i), $
                  set_value=string(p_field_vf(i))

              npeak_del

              end
 endcase

 return, event.value
 end

;----------------------------------------------------------------------
;allow input or modification of fit parameter by edite (in window parameter 
;definition) or cursor.
;Call proposal for determination of start parameter in first run for 
;first peak (polynom + 1 fct)
;----------------------------------------------------------------------
 pro npar_in_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 if (nfunct gt 1) then begin
   p_field = fltarr(par_n)
   var_ind = intarr(par_n)
   for i = 0, par_n-1 do widget_control,win_ids.p_id(i),/edit

   ndesi,win_ids.plus
   ndesi,win_ids.selection

   widget_control,win_ids.file_io,set_uvalue=0
   widget_control,win_ids.input,map=1    ;cursor,ok,reset

   test = where(p_field_vf,first)        ; first=0 for first run

   if (first ne 0) then begin
     nsel_box,x_sel(0),x_sel(y_n-1),''
     peak.number = ''
     fct_num     = 0

     for i = ndegree+2, par_n-1,3 do begin
       fct_num = fct_num + 1
       widget_control, win_ids.p_id(i), get_value = h_par
       if (h_par(0) ne 'not def.') then begin
         peak.number(fct_num) = strcompress(fct_num,/remove_all)
         peak.pos(fct_num)    = float(h_par(0))
       endif
     endfor

     xyouts,peak.pos,peak.y,peak.number,align=0.5

   endif else if (nfunct eq 2) then nproposal, funct_str(1)

 endif else $
   nmess,['no par_input on ','< polynom only >','possible']

 end ; par_in_ev

;----------------------------------------------------------------------
;Set cur_par,
;cur_par (pia_multi_fit_nm_cmn) gives the parameter which is assigned with the 
;(by cursor) selected value 
;----------------------------------------------------------------------
 pro npar_cur_val_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,event.id,get_value=cur_par ; backgr,height,center,width

 end

;----------------------------------------------------------------------
;controls parameter-evaluation using cursor depending from function
;for polynom only : backgr
;for others  only : height, center, width
;----------------------------------------------------------------------
 function npar_cur_fct_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 for i = 0,3 do begin
   widget_control,win_ids.par_cur_anz(i),set_value='not def.'
   ndesi,win_ids.par_cur_val(i)
 endfor

 if (event.value eq funct_str(0)) then begin
   widget_control,win_ids.par_cur_val(0),input_focus=1
   nresi,win_ids.par_cur_val(0)
   cur_par = 'backgr'
 endif else begin
   widget_control,win_ids.par_cur_val(1),input_focus=1
   for i = 1,3 do nresi,win_ids.par_cur_val(i)
   cur_par = 'height'
 endelse

 return,event.value
 end

;----------------------------------------------------------------------
; cursor-events for parameter and selection
;----------------------------------------------------------------------
 pro ncursor_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak
 x_cur = event.x
 y_cur = event.y
 xy    = convert_coord(x_cur,y_cur,/device,/to_data)
 x_str = string(xy(0))
 y_str = string(xy(1))

; event for parameter selection
 case cur_par of
   'backgr' : widget_control,win_ids.par_cur_anz(0),set_val=y_str

   'height' : begin
              if not event.press then begin   ;reaction only for button release
                widget_control, win_ids.par_cur_anz(1), get_val=h
                pos = strpos(h,' to ?')
                if (pos(0) eq -1) then begin  ; 1.wert
                  h = y_str + ' to ?'
                  widget_control,win_ids.par_cur_anz(1),set_val=h
                endif else begin              ; 2.wert
                  len = strlen(h)-5
                  h   = strmid(h,0,len(0))
                  h   = xy(1) - float(h)
                  h   = string(h)
                  widget_control,win_ids.par_cur_anz(1),set_val=h
                endelse
              endif
              end

   'center' : widget_control,win_ids.par_cur_anz(2),set_val=x_str

   'width ' : begin
              if not event.press then begin   ;reaction only for button release
                widget_control,win_ids.par_cur_anz(3),get_val=h
                pos = strpos(h,' to ?')
                if (pos(0) eq -1) then begin  ; 1.wert
                  h = x_str + ' to ?'
                  widget_control,win_ids.par_cur_anz(3),set_val=h
                endif else begin              ; 2.wert
                  len = strlen(h)-5
                  h   = strmid(h,0,len(0))
                  h   = abs(float(h) - xy(0))
                  h   = string(h)
                  widget_control,win_ids.par_cur_anz(3),set_val=h
                endelse
              endif
            end

;event for selection:
   'x1'     : begin
              widget_control,win_ids.x1,set_val=x_str
              widget_control,win_ids.fit_bild,draw_button_events=0
              widget_control,win_ids.x2,get_value=x_max
              nsel_box,xy(0),float(strcompress(x_max(0),/remove_all)),''
              end

   'x2'     : begin
              widget_control,win_ids.x2,set_val=x_str
              widget_control,win_ids.fit_bild,draw_button_events=0
              widget_control,win_ids.x1,get_value=x_min
              nsel_box,float(strcompress(x_min(0),/remove_all)),xy(0),''
              end
 endcase
 end

;----------------------------------------------------------------------
; change button_value : fix <--> var
; auslese in window parameter_definition, button <ok>
;----------------------------------------------------------------------
 pro npar_fix,event

 widget_control,event.id,get_value=h
 if (h eq 'fix') then $
   widget_control,event.id,set_value='var' $
 else $
   widget_control,event.id,set_value='fix'
 end 

;----------------------------------------------------------------------
; remove notification of peaks in graphic
;----------------------------------------------------------------------
 pro npeak_del

@colour_indices
 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 !p.color = black
 xyouts,peak.pos,peak.y,peak.number,align=0.5
 !p.color = white

 end

;----------------------------------------------------------------------
; rebuild windows after selection
;----------------------------------------------------------------------
 pro nsel_desen

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control, win_ids.sel_in, map  = 0
 widget_control, win_ids.x1    , edit = 0
 widget_control, win_ids.x2    , edit = 0

 nresi, win_ids.plus
 nresi, win_ids.par_in
 nresi, win_ids.poly_degree

 if (nfunct eq 1) then nresi, win_ids.fit
 end 

;----------------------------------------------------------------------
; mark  not selected parts in graphic
;----------------------------------------------------------------------
 pro nsel_box , x_min, x_max, titel

@colour_indices
 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

; COMMON PIA_XPLOT, active_base

; pia_xplot, x, y, psym=ps, title=titel, xstyle=1

 plot, x, y, psym=ps, title=titel, xst=1, xrange=xra, yrange=yra
;  background=white, color=black
 oploterr, x, y, 1./sqrt(gewicht), 3

 x0     = !x.crange(0) & xn = !x.crange(1)
 y0     = !y.crange(0) & yn = !y.crange(1)
 xmin   = x0 > x_min
 xmax   = xn < x_max
 box1_x = [x0,xmin,xmin,x0]
 box2_x = [xmax,xn,xn,xmax]
 box_y  = [y0,y0,yn,yn]

 polyfill,box1_x,box_y,orientation=45
 polyfill,box2_x,box_y,orientation=-45

 end 

;----------------------------------------------------------------------
;Get information from selected range of window
; all        : the whole graphic area (xrange)
; reset      : reset data to value at call of selection
; ok         : accept selected area
;              (--> x_sel, y_sel, gewicht_sel),
;              show graphic
; cancel     : leave selection without data modification
; cursor_min : activate cursor to determine x_min
; cursor_max : activate cursor to determine x_max
;----------------------------------------------------------------------
 function nsel_in_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 case event.value of
   'all'    : begin
              ind        = where(x ge xra(0) and x le xra(1),x_n)
              ind_min    = ind(0)
              x_sel      = x(ind(0):ind(x_n-1))
              y_sel      = y(ind(0):ind(x_n-1))
              weight_sel = gewicht(ind(0):ind(x_n-1))
 
              widget_control,win_ids.x1,set_value=string(x_sel(0))
              widget_control,win_ids.x2,set_value=string(x_sel(x_n-1))
 
              nsel_desen                              ; rebuild widgets
              nsel_box,x_sel(0),x_sel(x_n-1) ,''
              y_n = x_n 
              end

   'ok'     : begin
              widget_control,win_ids.x1,get_value=h1
              widget_control,win_ids.x2,get_value=h2

              x_min  = float(strcompress(h1(0),/remove_all))
              x_max  = float(strcompress(h2(0),/remove_all))
              x_ind1 = where(x ge x_min)
              x_ind1 = x_ind1(0)
              x_ind2 = where(x gt x_max,count)

              if (count eq 0) then $
                x_ind2 = n_elements(y)-1 $
              else $
                x_ind2 = x_ind2(0)-1

              if (x_ind1 lt x_ind2) then begin
                ind_min    = x_ind1
                x_sel      = x(x_ind1:x_ind2)
                y_sel      = y(x_ind1:x_ind2)
                weight_sel = gewicht(x_ind1:x_ind2)
                y_n        = n_elements(y_sel)
              endif

;set selected part on real x values
              widget_control,win_ids.x1,set_value=string(x_sel(0))
              widget_control,win_ids.x2,set_value=string(x_sel(y_n-1))
              nsel_desen                              ; rebuild widgets
              nsel_box,x_sel(0),x_sel(y_n-1) ,''
              end

   'reset'  : begin
              widget_control,win_ids.x1,set_value=string(x_sel(0))
              widget_control,win_ids.x2,set_value=string(x_sel(y_n-1))
              nsel_box,x_sel(0),x_sel(y_n-1),''
              end

   'cancel' : begin
              widget_control,win_ids.x1,set_value=string(x_sel(0))
              widget_control,win_ids.x2,set_value=string(x_sel(y_n-1))
              nsel_box,x_sel(0),x_sel(y_n-1),''
              nsel_desen
              end
; event-procedure for win_ids.fit_bild = ncursor_ev 
  'cursor_min' : begin
                 cur_par = 'x1'
                 widget_control,win_ids.fit_bild,draw_button_events=1
                 end

  'cursor_max' : begin
                 cur_par = 'x2'
                 widget_control,win_ids.fit_bild,draw_button_events=1
                 end

  endcase

 return,event.value

 end ; end sel_in_ev

;----------------------------------------------------------------------
;activate window <select range> to modify the selected area 
;----------------------------------------------------------------------
 pro nselection_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 nsel_box, x_sel(0),x_sel(y_n-1),''

 ndesi, win_ids.plus
 ndesi, win_ids.par_in
 ndesi, win_ids.fit
 ndesi, win_ids.poly_degree

 widget_control, win_ids.sel_in, map=1
 widget_control, win_ids.x1, edit=1
 widget_control, win_ids.x2, edit=1

 end 

;----------------------------------------------------------------------
;take degree of basic-polynom
;----------------------------------------------------------------------
 pro ndegree_event,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,win_ids.poly_degree,get_value = h

 ndegree = fix(h(0))
 ndegree = 0 > ndegree

 widget_control,win_ids.poly_degree,set_value = string(ndegree)

 par_n = ndegree + 1

 widget_control,win_ids.par_def(0),set_value='polynom, backgr.'

 for i = 1,ndegree do $
   widget_control,win_ids.par_def(i),set_value='polynom'

 for i = ndegree+1,49 do $
   widget_control,win_ids.par_def(i),set_value=''

 end 

;----------------------------------------------------------------------
;----------------------------------------------------------------------
 pro ndesi, id

 widget_control,id,sensitive=0

 end

;----------------------------------------------------------------------
;----------------------------------------------------------------------
 pro nresi, id

 widget_control,id,sensitive=1

 end

;----------------------------------------------------------------------
;output of warnings and error messages
;no influence on program
;----------------------------------------------------------------------
 pro nmess,text

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,win_ids.message,set_value=text
 widget_control,win_ids.mess_but,map=1
 widget_control,win_ids.mess_but,input_focus=1

 print,string(7b),string(7b)

 end 

;----------------------------------------------------------------------
;delete message-window
;----------------------------------------------------------------------
 pro nmess_ev,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,win_ids.message,set_value=''
 widget_control,win_ids.mess_but,map=0
 end

;----------------------------------------------------------------------
;Prevent reaction on pressing of return-key in text-widgets
;----------------------------------------------------------------------
 pro ntext_event,event      ; dummy
 end

;----------------------------------------------------------------------
;Reset to start conditions,
;Attetion: xrange will not be resetted
;----------------------------------------------------------------------
 pro ncancel_event,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,win_ids.cancel,hourglass=1

 ndegree         = 1
 par_n           = 2
 nfunct          = 1
 var_ind         = 0
 p_field_vf      = fltarr(50)
 funct_str(1:16) = ''

 widget_control,win_ids.res_bild,map=0

 ind        = where(x ge xra(0) and x le xra(1),x_n)
 ind_min    = ind(0)
 x_sel      = x(ind(0):ind(x_n-1))
 y_sel      = y(ind(0):ind(x_n-1))
 weight_sel = gewicht(ind(0):ind(x_n-1))

 widget_control,win_ids.x1,set_value=string(x_sel(0)) 
 widget_control,win_ids.x2,set_value=string(x_sel(x_n-1))

 nsel_desen ; rebuild widget
 nsel_box,x_sel(0),x_sel(x_n-1) ,''
 y_n = x_n 
 npeak_del
 peak.number = ''

 test = widget_info(win_ids.par_cur_fct,/valid_id)
 if test then widget_control,win_ids.par_cur_fct,/destroy

 test = widget_info(win_ids.top_next,/valid_id)
 if test then widget_control,win_ids.top_next,/destroy

 for i = 0,49 do begin
   widget_control,win_ids.par_def(i),set_value=''
   widget_control,win_ids.p_id(i),edit=0
   widget_control,win_ids.p_id(i),set_value='not def.'
   widget_control,win_ids.p_fix(i),set_value='var'
 endfor

 widget_control,win_ids.par_def(0),set_value='polynom, backgr.'
 widget_control,win_ids.par_def(1),set_value='polynom'
 widget_control,win_ids.fit_bild,draw_button_events=0
 widget_control,win_ids.par_cur,map=0
 widget_control,win_ids.sel_in,map=0
 widget_control,win_ids.input,map=0

 nresi,win_ids.next
 nresi,win_ids.plus
 nresi,win_ids.selection
 nresi,win_ids.par_in
 nresi,win_ids.fit
 nresi,win_ids.poly_degree

 widget_control,win_ids.poly_but_base,map=1
 widget_control,win_ids.poly_degree,set_value='   1'
 widget_control,win_ids.formel,set_value=''
 widget_control,win_ids.file_io,set_uvalue=0

 end 

;----------------------------------------------------------------------
; activate window for input of xrange
;----------------------------------------------------------------------
 pro nnext_event,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 test = widget_info(win_ids.top_next,/valid_id)
 if not test then begin
   win_ids.top_next = widget_base(/column,/frame,group_leader=event.top)
   titel          = widget_label(win_ids.top_next,value='new range')
   zeile          = widget_base(win_ids.top_next,/row)
   label          = widget_label(zeile,value='xrange_min = ')
   range_min      = widget_text(zeile,/edit,value=string(x(0)), $
                                event_pro='ntext_event')
   zeile          = widget_base(win_ids.top_next,/row)
   label          = widget_label(zeile,value='xrange_max = ')
   range_max      = widget_text(zeile,/edit,value=string(x(n_elements(x)-1)), $
                                event_pro='ntext_event')
   range_but      = widget_button(win_ids.top_next,value='ok',$
                                event_pro='nnew_range')

   widget_control,win_ids.top_next,/realize
   win_ids.range_min = range_min
   win_ids.range_max = range_max
 endif else $
   widget_control,win_ids.top_next,show = 1

 widget_control,win_ids.file_io,set_uvalue = 0
 
 end

;----------------------------------------------------------------------
;take new xrange and set to start conditions (calls cancel)
;----------------------------------------------------------------------
 pro nnew_range,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,win_ids.range_min,get_value=ramin
 widget_control,win_ids.range_max,get_value=ramax

 xra(0) = float(strcompress(ramin(0),/remove_all))
 xra(1) = float(strcompress(ramax(0),/remove_all))

 ncancel_event

 end
;----------------------------------------------------------------------
;Write/read fit-data from/to file
; restore    : Load fit data from file (stored by save command)
;              Check data compability with start data
; save       : Store fit-data in file 
; display    : Output of fit-data on screen
; print_file : Store fit-data in file (easy to edit) 
;----------------------------------------------------------------------
 pro nfile_event,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 case event.value of
   'restore' : begin 
               file_name = pickfile(file='multi_fit_sav.mfs',  $
                           group=event.top, /read, /must_exist, $
                           filter='*.mfs')
               if (file_name ne '') then begin 
; inbetween storage of data , which might be destroyed from wrong file
                 yy = y_n       
                 xx = xra    
                 pp = p_field_vf
                 ff = funct_str  
                 nn = nfunct 
                 dd = ndegree

                 restore,file_name
                 widget_control,event.id,hourglass=1

                 ind_max = ind_min + y_n-1
                 ind_y   = n_elements(y)-1

                 if ((ind_min ge ind_y) or (ind_max gt ind_y)) then begin
                   nmess, ['restore-data',' not compatible', $
                        'with start-data'] 
                   y_n        = yy       
                   xra        = xx    
                   p_field_vf = pp
                   funct_str  = ff 
                   nfunct     = nn 
                   ndegree    = dd
                 endif else begin
                   if ((y(ind_min) ne y_sel_min) or $
                       (y(ind_max) ne y_sel_max) or $
                       (x(ind_min) ne x_sel_min) or $
                       (x(ind_max) ne x_sel_max)) then begin
                      nmess, ['restore-data',' not compatible', $
                           'with start-data'] 
                      y_n        = yy       
                      xra        = xx    
                      p_field_vf = pp
                      funct_str  = ff 
                      nfunct     = nn 
                      ndegree    = dd
                   endif else begin
                      y_sel      = y(ind_min:ind_max)
                      x_sel      = x(ind_min:ind_max)
                      weight_sel = gewicht(ind_min:ind_max)

                      nsel_box,x_sel(0),x_sel(y_n-1),''
                      widget_control,win_ids.x1,set_value=string(x_sel(0))
                      widget_control,win_ids.x2,set_value=string(x_sel(y_n-1))
                      widget_control,win_ids.formel,set_value = funct_str

                      par_n = ndegree -2 + 3*nfunct
                      str    = strarr(3)
                      str(0) = 'height' 
                      str(1) = 'center' 
                      str(2) = '1/e width' 

                      widget_control,win_ids.par_def(0), $
                                   set_value='polynom, backgr.'

                      for i = 1,ndegree do $
                        widget_control,win_ids.par_def(i),set_value='polynom'

                      for i = 1,nfunct do begin  ; nfunct = number of functions 
                        pos    = ndegree - 2 + 3*i    ; pos. of height in 
                                                      ; param.field
                        fct    = strmid(funct_str(i),5,1)
                        fct_nr = strmid(funct_str(i),0,5)
                        case fct of
                          '(' : begin                 ; **** '(sin...' ****
                                for j=0,1 do $
                                  widget_control,win_ids.par_def(pos+j), $
                                             set_value = fct_nr + str(j)

                                widget_control,win_ids.par_def(pos+2), $
                                             set_value = fct_nr + 'phase width'
                                end

                           'c' : begin                 ; **** 'cauchy' ****
                                for j = 0,1 do $
                                   widget_control,win_ids.par_def(pos+j), $
                                           set_value = fct_nr + str(j)
                                widget_control,win_ids.par_def(pos+2), $
                                         set_value = fct_nr + 'fwhm'
                                end 

                           'd' : begin                 ; **** 'double fermi' ****
                                for j = 0,1 do $
                                   widget_control,win_ids.par_def(pos+j), $
                                           set_value = fct_nr + str(j)
                                widget_control,win_ids.par_def(pos+2), $
                                         set_value = fct_nr + 'width'
                                end 

                          else:  for j = 0,2 do widget_control, $
                                    win_ids.par_def(pos+j),$
                                    set_value = fct_nr + str(j)
                        endcase
                      endfor

                      for i = 0,par_n-1 do begin 
                        widget_control,win_ids.p_id(i),$
                                     set_value=string(p_field_vf(i))
                        widget_control,win_ids.p_fix(i),set_value='var'
                        widget_control,win_ids.p_id(i),edit=0
                      endfor

                      for i = par_n,49 do begin 
                        widget_control,win_ids.p_id(i),set_value='not def.'
                        widget_control,win_ids.p_id(i),edit=0
                        widget_control,win_ids.p_fix(i),set_value='var'
                        widget_control,win_ids.par_def(i),set_value = ''
                      endfor

                      var_ind              = indgen(par_n)
                      p_field_vf(par_n:49) = 0.0
                      p_field              = p_field_vf(0:par_n-1)
;peaks notification :
                      fct_num     = 0
                      peak.number = ''
                      for i = ndegree+2, par_n-1,3 do begin 
                        fct_num = fct_num+1
                        widget_control,win_ids.p_id(i),get_value = h_par
                        peak.number(fct_num) = strcompress(fct_num,/remove_all)
                        peak.pos(fct_num)    = float(h_par(0))
                      endfor

                      xyouts,peak.pos,peak.y,peak.number,align=0.5
; tune windows
;                      widget_control,win_ids.res_bild,map=0
                      widget_control,win_ids.fit_bild,draw_button_events=0
                      widget_control,win_ids.par_cur,map=0
                      widget_control,win_ids.sel_in,map=0
                      widget_control,win_ids.input,map=0
                      widget_control,win_ids.poly_but_base,map=0
                      widget_control,win_ids.poly_degree, $
                                     set_value=string(ndegree)

                      ndesi,win_ids.selection
                      ndesi,win_ids.poly_degree

                      nresi,win_ids.fit
                      nresi,win_ids.par_in
                      nresi,win_ids.next

                      test = widget_info(win_ids.par_cur_fct,/valid_id)
                      if test then widget_control,win_ids.par_cur_fct,/destroy
                      test = widget_info(win_ids.top_next,/valid_id)
                      if test then widget_control,win_ids.top_next,/destroy

                    endelse ; matching restore-data, index

                 endelse ; matching restore-data, werte

               endif else $
                 nmess,['null filename','no restore']

          end ; restore

   else : begin  
          widget_control,win_ids.file_io,get_uvalue=test
          if (test eq 0) then $
             nmess,['keine parameter-daten','berechnet,',$
                   'option nicht sinnvoll'] $
          else begin 

            case event.value of
              'save' :  begin
                        file_name = pickfile(file='multi_fit_sav.mfs',  $
                                    group=event.top, /write, filter='*.mfs')

                        if (file_name ne '') then begin
                          y_sel_min = y_sel(0) 
                          y_sel_max = y_sel(y_n-1)
                          x_sel_min = x_sel(0) 
                          x_sel_max = x_sel(y_n-1)

                          save, file=file_name, y_sel_min, y_sel_max, $
                                x_sel_min, x_sel_max,y_n, ind_min,    $
                                xra, p_field_vf, funct_str, nfunct, ndegree
 
                        endif else $
                          nmess,['null filename','no save']
                        end

            'display' : begin
                        openw, lun_display, 'display.dat', /get_lun
                        nprint_view, lun_display
                        close, lun_display
                        xdisplayfile, 'display.dat', group=event.top,$
                                      title='fit_view'
                        openu, lun_display, 'display.dat', /delete
                        close, lun_display
                        free_lun,lun_display
                        end

         'print_file' : begin
                        file_name = pickfile(file='multi_fit_print.mfp',  $
                                    group=event.top, /write, filter='*.mfp')

                        if (file_name ne '') then begin
                          openw,lun_print,file_name,/get_lun
                          nprint_view,lun_print
                          close,lun_print
                          free_lun,lun_print
                        endif else $
                          nmess,['null filename','no print_file']
                        end

            endcase ; save,display,print_fille
          endelse

   end     

   endcase ; restore,else

 end       ; file_event

;----------------------------------------------------------------------
;write fit-data (capable to edit)
;called from <display> and <print_file>
;----------------------------------------------------------------------
 pro nprint_view,lun_fit

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 printf,lun_fit,'functions: ',funct_str
 printf,lun_fit

 if (nfunct ne 1) then begin             ;curvefit
   printf,lun_fit,'parameters and standard deviations for the parameters:'
   k = 0

   for i=0, par_n-1 do begin
      a_string = 'a'+string(i,format='(i2)')+', '
      widget_control,win_ids.par_def(i),get_value=def_string
      def_string = def_string(0) + string(9b)  ;string(9b)=ascii von tab

      if strlen(def_string) le 16 then $
         def_string = def_string + string(9b)

      if (var_ind(k) eq i) then begin
         k = k + 1
         vf = 'var :'
      endif else vf = 'fix :'

      printf,lun_fit,a_string,def_string,vf, $
                     string(9b),p_field_vf(i),string(9b),sig(i)

   endfor

 endif else begin               ;polyfitw
   printf,lun_fit,'correlation matrix of coefficients:'
   printf,lun_fit,p_field_vf(0:ndegree),format='(2x,g)'
   printf,lun_fit
   printf,lun_fit,'standard deviation in y-units : ',sig

 endelse

 printf,lun_fit
 printf,lun_fit,'selected x, y, yfit'
 for i = 0,y_n-1 do printf,lun_fit,x_sel(i),string(9b), $
                           y_sel(i),string(9b),yfit(i)

 end

;----------------------------------------------------------------------
; activate help window
;----------------------------------------------------------------------
 pro nhelp_event,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 test = widget_info(win_ids.top_help,/valid_id)
 if test then begin
   widget_control,win_ids.top_help,show=1
   widget_control,win_ids.top_help,get_uvalue=help_ident
   help_ident(1) = event.value
   widget_control,win_ids.top_help,set_uvalue=help_ident
 endif else begin
   win_ids.top_help = widget_base( title='help text', /column, $
                                  group_leader=event.top )
   but_base         = cw_bgroup( win_ids.top_help, ['close','quit','next'], $
                                  row=1, event_funct='nhelp_but', /return_name)
   help_text        = widget_text(win_ids.top_help, xsize=55,$
                                  ysize=30, val='', /scroll)

   widget_control,win_ids.top_help,/realize
   help_ident = [help_text,event.value] ; text-id and text-place, int-arr
   widget_control,win_ids.top_help,set_uvalue=help_ident
 endelse

 multi_fit_help,help_ident ;procedure with help-text

 end ; help

;----------------------------------------------------------------------
;Read buttons of help window
; close : remove help window
; quit  : put help window behind work window
; next  : jump to next help-point
;----------------------------------------------------------------------
 function nhelp_but,event

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 case event.value of
    'close': widget_control,event.top,/destroy
    'quit' : widget_control,event.top,show=0
    'next' : begin
             widget_control,event.top,get_uvalue=help_ident
             nr = help_ident(1)

             if (nr le 27) then $
               nr=nr+1 $
             else $
               nr=1

             help_ident(1) = nr
             widget_control,event.top,set_uvalue=help_ident
             multi_fit_help,help_ident ; procedure with help-text
             end 
          
 endcase

 return,event

 end

;----------------------------------------------------------------------
; 'standard' exit
;----------------------------------------------------------------------
 pro nexit_event,event      

 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 widget_control,event.id,hourglass=1
; test = widget_info(win_ids.par_cur_fct,/valid_id)
; if test then widget_control,win_ids.par_cur_fct,/destroy
 widget_control,event.top, /destroy
 
 end

;----------------------------------------------------------------------
; the input 'xrange' has inpact on "plot" and(!) calculation.
; the input 'yrange' is valid only(!)
; for  "plot" and has no inpact on calculation
;----------------------------------------------------------------------
 pro pia_multi_fit_nomodal, xin, yin, xout=xout, yout=yout, weight=weight, $
                a=a, sigmaa=sigmaa, $
                xrange=xrange, yrange=yrange, psym=psym

@colour_indices
 common pia_multi_fit_nm_cmn, x, y, x_sel, y_sel, ind_min, y_n, yfit, gewicht, $
                weight_sel, p_field, p_field_vf, var_ind, sig  , $
                win_ids, xra, yra, ps, ndegree, nfunct, par_n  , $ 
                funct_str, cur_par, peak

 x_n = n_elements(xin)
 y_n = n_elements(yin)
 if (x_n eq 0) then BEGIN
   message,/info,'missing input parameter'
   RETURN
 END
 if (x_n ne y_n) then BEGIN
   message,/INFO,'x and y array must have same size ! '
   RETURN
 ENDIF
 x = float(xin)
 y = float(yin)

 if (n_elements(yrange) eq 0) then $
   yra = [min(y),max(y)] else  yra = yrange

 if (n_elements(xrange) eq 0) then $
   xra = [min(x),max(x)] else  xra = xrange

 if (n_elements(psym) eq 0) then $
   ps = 3 else ps = psym

;sort x arrays in increasing order
 ind = sort(x)
 x   = x(ind) 
 y   = y(ind)

;default weight is 1.
 w_n = n_elements(weight)
 if (w_n eq 0) then $
   gewicht = replicate(1.,y_n) $
 else $
   if (w_n eq y_n) then gewicht = weight(ind) else begin
      MESSAGE,/INFO,'Weight-array and y-array must have same size !'
      RETURN
   endelse

 ind = where(x ge xra(0) and x le xra(1),y_n)

 if (y_n gt 2) then begin
   ind_min    = ind(0)
   x_sel      = x(ind)
   y_sel      = y(ind)
   weight_sel = gewicht(ind)
 endif else begin
   MESSAGE,/INFO, 'xrange less than 3 inputs'
   RETURN
 endelse

 yfit         = y
 p_field      = 0.
 p_field_vf   = fltarr(50) ; if this array will be modified it is mandatory
                           ; to modify the array size of 'peak' as well
 var_ind      = 0
 sig          = 0.

 ndegree      = 1
 par_n        = 2
 nfunct       = 1
 funct_str    = strarr(17)
 funct_str(0) = '( 0) poly'
 peak         = {number:strarr(17), pos:fltarr(17), y:fltarr(17)}

 if (xregistered('pia_multi_fit_nomodal') ne 0) then return

 !p.background=black
 !p.color=white

 win_ids = {top_next:0l   , top_save:0l    , top_help:0l, $
            fit_bild:0l   , res_bild:0l    , $
            par_cur:0l    , par_cur_ein:0l , par_cur_anz:lonarr(4), $
            par_cur_fct:0l, par_cur_val:lonarr(4), $
            plus:0l       , selection:0l   , par_in:0l , fit:0l, $
            cancel:0l     , $
            file_io:0l    , next:0l        , $
            range_min:0l  , range_max:0l   , $
            poly_degree:0l, poly_but_base:0l, $
            formel:0l     , $
            sel_in:0l     , x1:0l          , x2:0l, input_b:0l  , $
            input:0l      , $
            message:0l    , mess_but:0l    , $
            par_def:lonarr(50), p_id:lonarr(50), p_fix:lonarr(50) }

 desc_1       = replicate({flags:0, name:''},6)
 desc_1.flags = [1,0,0,0,0,2]
 desc_1.name  = [' + ', 'gauss'          , $
                        '(sin(x)/x)^2'   , $
                        'expo'           , $
                        'cauchy'	 , $
                        'double fermi' ]

 desc_f       = replicate({flags:0, name:''},6)
 desc_f.flags = [1,0,0,1,0,2]
 desc_f.name  = ['file_i/o', 'restore'   , $
                             'save'      , $
                             'view'      , $
                             'display'   , $
                             'print_file']

 desc_h       = replicate({flags:0,name:''},29)
 desc_h.flags = [1,0,0,0,1,0,0,0,0,0,0,0,2,1,0,1,0,0,0,0,2,0,0,2,0,0,0,0,2]
 desc_h.name  = ['help', $               ; 0
                   'allgemein'      ,$  ; 1
                   'base_function'  ,$  ; 2
                   ' + '            ,$  ; 3
                   'selection'      ,$  ; 4
                      'allgemein'   ,$  ; 5
                      'all'         ,$  ; 6
                      'reset'       ,$  ; 7
                      'ok'          ,$  ; 8
                      'cancel'      ,$  ; 9
                      'cursor_min'  ,$  ; 10
                      'cursor_max'  ,$  ; 11
                      'min_max'     ,$  ; 12
                   'par_input'      ,$  ; 13
                      'allgemein'   ,$  ; 14
                      'cursor'      ,$  ; 15
                         'allgemein',$  ; 16
                         'ok'       ,$  ; 17
                         'reset'    ,$  ; 18
                         '( )fct'   ,$  ; 19
                         'parameter',$  ; 20
                      'var'         ,$  ; 21
                      'ok'          ,$  ; 22
                      'reset'       ,$  ; 23
                   'fit'            ,$  ; 24
                   'cancel'         ,$  ; 25
                   'file_i/o'       ,$  ; 26
                   'new range'      ,$  ; 27
                   'exit']              ; 28

 top     = widget_base(title='pia_multi_fit')

;display design:
 work    = widget_base(top,/row)
;work    = widget_base(top,xsize=900,ysize=800,/row)

 work_l  = widget_base(work,/column) ; linke seite
 fit_bild= widget_draw(work_l,event_pro='ncursor_ev',xsize=370,ysize=280,/frame)

; fit_bild= widget_draw(work_l,event_pro='ncursor_ev',xsize=400,ysize=300,/frame)
 parameter     = ['backgr', 'height', 'center', 'width ']
 par_cur       = widget_base(work_l, /column, event_func='npar_cur_ev', /frame)
 par_cur_end   = cw_bgroup(par_cur, ['ok','reset'], row=1, /return_name, $
                 label_left='<ok> to confirm each set of parameters')
 par_cur_ein   = widget_base(par_cur,/row)
 par_cur_ein_l = widget_base(par_cur_ein,/column, $
                            event_pro='npar_cur_val_ev')
 for i=0,3 do begin
   zeile = widget_base(par_cur_ein_l,/row)
   win_ids.par_cur_val(i) = widget_button(zeile,val=parameter(i))
   win_ids.par_cur_anz(i) = widget_text(zeile,val='not def.')
 endfor

 res_base = widget_base(work_l)
 res_bild = widget_draw(res_base,xsize=370,ysize=200,/frame)
; res_bild = widget_draw(res_base,xsize=400,ysize=250,/frame)
 work_r   = widget_base(work, /column)                 ; right side
 auswahl  = widget_base(work_r, /row)                  ; choice-buttons:
; base_hlp = widget_base(auswahl, event_pro='nhelp_event')
; hlp      = cw_pdmenu(base_hlp, desc_h)
 hlp  = WIDGET_BUTTON(auswahl, val = ' help ', event_pro='htmlhelp_event', $
 /frame )
 base_file= widget_base(auswahl, event_pro='nfile_event')
 file_io  = cw_pdmenu(base_file, desc_f, /return_name, uvalue=0)
WIDGET_CONTROL, file_io, SENSITIVE=0
 base_plus= widget_base(auswahl, event_pro='nplus_event')
 plus     = cw_pdmenu(base_plus, desc_1, /return_name) ; function choice
 selection= widget_button(auswahl,val = 'selection', $
                     event_pro = 'nselection_ev', /frame)
 par_in   = widget_button(auswahl,val='par_in', $
                          event_pro='npar_in_ev',/frame) 
 fit      = widget_button(auswahl,val='fit', $
                          event_pro = 'nfit_event',/frame)
 cancel   = widget_button(auswahl,val='cancel', $
                          event_pro = 'ncancel_event',/frame)
 next     = widget_button(auswahl,val='new range', $
                          event_pro = 'nnext_event',/frame)
 exit     = widget_button(auswahl,val = 'exit', $
                          event_pro = 'nexit_event',/frame)

 work_rm1     = widget_base(work_r, /row)
 polynom      = widget_base(work_rm1, /frame, /column)
 poly_anz     = widget_label(polynom, val='base_function = polynom')
 poly_n       = widget_base(polynom, /row, event_pro='ndegree_event')
 poly_n1      = widget_label(poly_n, val='degree = ')
 poly_degree  = widget_text(poly_n, /edit, xsize=15, value='   1')
 poly_but_base= widget_base(poly_n, /frame)
 poly_but     = widget_button(poly_but_base, val='ok')

 formel_base    = widget_base(work_rm1, /frame, /column)
 formel_lab     = widget_label(formel_base, value=' + function')
 win_ids.formel = widget_text(formel_base, value='', $
                             xsize=21, ysize=4, /scroll)
 work_rm2 = widget_base(work_r,/row)

;build widget for range-selection:
 select     = widget_base(work_rm2,/column,/frame)
 label      = widget_label(select,val='selected range')
 sel_in     = widget_base(select,/column,event_func='nsel_in_ev')
 input_b    = cw_bgroup(sel_in,['all','reset','ok','cancel'], $
                        label_left='input:',/return_name,row=1)
 cursor_b   = cw_bgroup(sel_in,['cursor_min','cursor_max'], $
                        label_left='      ',/return_name,row=1)
 zeile      = widget_base(select,/row)
 label      = widget_label(zeile,val='min = ')
 win_ids.x1 = widget_text(zeile,event_pro='ntext_event', $
                          value=string(x_sel(0)))
 zeile      = widget_base(select,/row)
 label      = widget_label(zeile,val='max = ')
 win_ids.x2 = widget_text(zeile,event_pro='ntext_event', $
                          value=string(x_sel(y_n-1)))                          

 mess_win         = widget_base(work_rm2,/column,/frame)
 mess_titel       = widget_label(mess_win,value='warnings -  dismiss with <ok>')
 win_ids.mess_but = widget_button(mess_win,value='ok',/frame, $
                                  event_pro='nmess_ev')
 win_ids.message  = widget_text(mess_win,xsize=21,ysize=8,/scroll) 

 par   = widget_base(work_r,/column,/frame) ;parameter input
 head  = widget_base(par,/row)
 label = widget_label(head,val='parameter_definition')
 input = cw_bgroup(head,['cursor','ok','reset'], $
                   /return_name,row=1,event_funct='npar_ein_ev')
 par_field = widget_base(par,/column,y_scroll_size=300)

 for i = 0,49 do begin
    zeile = widget_base(par_field,/row)
    label = widget_label(zeile,val='a'+string(i,format='(i2)')+' = ')
    win_ids.p_id(i)    = widget_text(zeile,event_pro='ntext_event', $
                                     val='not def.')
    win_ids.par_def(i) = widget_text(zeile,event_pro='ntext_event', $
                                     xsize=18,value='')
    win_ids.p_fix(i)   = widget_button(zeile,value='var',event_pro='npar_fix')
 endfor

 widget_control,win_ids.par_def(0),set_value='polynom, backgr.'
 widget_control,win_ids.par_def(1),set_value='polynom'
 widget_control,par_cur,map=0
 widget_control,sel_in,map=0
 widget_control,input,map=0
 widget_control,win_ids.mess_but,map=0
 widget_control,res_bild,map=0

 win_ids.plus          = plus
 win_ids.selection     = selection
 win_ids.par_in        = par_in
 win_ids.fit           = fit
 win_ids.cancel        = cancel
 win_ids.file_io       = file_io
 win_ids.next          = next
 win_ids.par_cur       = par_cur
 win_ids.par_cur_ein   = par_cur_ein
 win_ids.poly_degree   = poly_degree
 win_ids.poly_but_base = poly_but
 win_ids.input_b       = input_b
 win_ids.sel_in        = sel_in
 win_ids.input         = input
 win_ids.fit_bild      = fit_bild
 win_ids.res_bild      = res_bild

 widget_control, top, /realize

;draw given points to fit_bild :
 widget_control,win_ids.fit_bild,get_value=window
 wset, window
 plot,x,y,psym=ps,xst=1,xrange=xra,yrange=yra
; ,background=white, color=black 
 oploterr, x, y, 1./sqrt(gewicht), 3
; pia_xplot,x,y,psym=ps,xstyle=1

 widget_control,win_ids.fit_bild,get_value=window
 wset, window


 y0     = !y.crange(0) & yn = !y.crange(1)
 peak.y = peak.y + yn + (yn-y0)*0.04

 xMANAGER,No_BLOCK=1,'pia_multi_fit_nomodal',top,event='nfit_event'

 xout   = x_sel
 yout   = yfit
 a      = p_field_vf
 sigmaa = sig
 
 !p.background = black
 !p.color      = white

 end
