;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_PLOT
;
; PURPOSE:
;	PIA PLOT routine. It allows plotting one point, not possible with
;	the normal PLOT routine
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_PLOT, X, Y
;
; INPUTS:
;	X: Abscisse data to be plotted. It can be a scalar
;	Y: Ordinate data to be plotted. It can be a scalar
;	
; KEYWORD PARAMETERS:
;	XERR: X uncertainties
;	YERR: Y uncertainties
;	+ some original plot keywords: 
;		psym, title, xtitle, ytitle, subtitle, charsize, 
;		color, symsize, xrange, yrange, xmargin
;	thick keyword added
; OUTPUTS:
;	none
;	
; PROCEDURES USED:
;	Astronomical Library:
;		oploterr
;	
; COMMON BLOCKS:
;	none
;	
; MODIFICATION HISTORY:
;	Written by C. Gabriel (ESA/ESTEC)		October 1994
;	Oct 95 Version 3.5
;	thick keyword added (CG)			August 1997
;	NODATA keyword added (JAcosta)			August 1997
;	V6.5
;	Limiting output possible through 
;		!PIA_PLOT_LIMIT (CG)			January 1998
;	V7.0
;	Correct color handling for reverse table (CG)	November 1998
;	V7.3
;	
;-

PRO PIA_PLOT, XPL, YPL, XERR=xerr, YERR=yerr, PSYM=psym, TITLE=title, $
	XTITLE=xtitle, YTITLE=ytitle, SUBTITLE=subtitle, CHARSIZE=charsize,$
	COLOR=color, SYMSIZE=symsize, XRANGE = xrange, YRANGE = yrange, $
        XMARGIN = xmargin, THICK=thick, LINESTYLE=linestyle, NODATA = nodata

	IF max(!PIA_PLOT_LIMIT) NE 0 AND $
	   max(!PIA_PLOT_LIMIT) LT N_ELEMENTS(xpl)THEN BEGIN
	  limits = !PIA_PLOT_LIMIT
	  PIA_DISP_PRINT_FILE, '', TITLE='PIA_PLOT Warning (unique)', $
	  	TEXT=['The displayed Plot(s) is/are partially limited to', $
	  	     'the elements: ['+STRTRIM(limits(0),2)+','+$
	  	     		      STRTRIM(limits(1),2)+']'], $
	  	UNIQUE='PIA_PLOT_WARNING',WIDTH=10, HEIGHT=4

	  x = xpl(limits(0):limits(1))
	  y = ypl(limits(0):limits(1))
	ENDIF ELSE BEGIN
	  x = xpl
	  y = ypl
	ENDELSE


	xerrset = keyword_set(xerr)
	yerrset = keyword_set(yerr)
	IF KEYWORD_SET(psym) THEN psym=psym ELSE psym=0 
	IF KEYWORD_SET(title) THEN title=title ELSE title='' 
	IF KEYWORD_SET(xtitle) THEN xtitle=xtitle ELSE xtitle='' 
	IF KEYWORD_SET(ytitle) THEN ytitle=ytitle ELSE ytitle='' 
	IF KEYWORD_SET(subtitle) THEN subtitle=subtitle ELSE subtitle='' 
	IF KEYWORD_SET(charsize) THEN charsize=charsize ELSE charsize=1.
	IF KEYWORD_SET(symsize) THEN symsize=symsize ELSE symsize=1
	IF KEYWORD_SET(thick) THEN thick=thick ELSE thick=1
	IF KEYWORD_SET(color) THEN color=color ELSE color=39
	IF !P.background EQ 0 THEN color = !d.n_colors - 1 ELSE $
	   IF !P.background EQ !d.n_colors - 1 THEN color = 0
	IF KEYWORD_SET(linestyle) THEN linestyle=linestyle ELSE linestyle=0
	IF KEYWORD_SET(xmargin) THEN xmargin=xmargin ELSE xmargin=[10,3]
	IF NOT KEYWORD_SET(nodata) THEN nodata = 0

;Duplicate the values for the case if there is just one point per axis
        xdiff = 0
        ydiff = 0
        IF N_ELEMENTS(x) EQ 1 THEN BEGIN
		IF x(0) eq 0 THEN xdiff = 0.001
                x = [x,x]
                if (XerrSet) then xerr = [xerr, xerr]
        ENDIF ELSE xdiff = x(1) - x(0)
        IF N_ELEMENTS(y) EQ 1 THEN BEGIN
		IF y(0) eq 0 THEN ydiff = 0.001
                y = [y,y]
                if (YerrSet) then yerr = [yerr, yerr]
        ENDIF ELSE BEGIN
		IF max(y) EQ 0 AND min(y) EQ 0 THEN ydiff = 0.001
	ENDELSE 

; Change xrange and yrange
	IF KEYWORD_SET(xrange) THEN xrange=xrange ELSE BEGIN
	  xrange = $
	    float([min(x)-abs(min(x))*0.2-xdiff,max(x)+abs(max(x))*0.2+xdiff])
	  IF xerrset THEN $
		xrange = [xrange(0) - max(xerr)*1.4, xrange(1) + max(xerr)*1.4]
        ENDELSE

	IF KEYWORD_SET(yrange) THEN yrange=yrange ELSE BEGIN
	  yrange = $
	    float([min(y)-abs(min(y))*0.2-ydiff,max(y)+abs(max(y))*0.2+ydiff])
	  IF yerrset THEN $
		yrange = [yrange(0) - max(yerr)*1.4, yrange(1) + max(yerr)*1.4]
	ENDELSE 

	; if nodata keyword is selected then 
	IF KEYWORD_SET(nodata) AND N_ELEMENTS(x) EQ 2 AND N_ELEMENTS(y) EQ 2 $
	      THEN BEGIN 
	   xrange = x
	   yrange = y
	ENDIF   

        PLOT, X, Y, psym=psym, title=title, subtitle=subtitle, xtitle=xtitle,$
	  ytitle=ytitle, charsize=charsize, xrange=xrange, yrange=yrange, $
	  color=color, symsize=symsize, xmargin=xmargin, thick=thick, $ 
	  nodata=nodata 

	IF yerrset THEN OPLOTERR, X, Y, yerr, 3		 

        RETURN
END

