;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_RD2XY
;
; PURPOSE:
;	Transform right_ascension/declination to map x/y coordinates
;	and optionally the derivatives.
;	No input parameter checks are performed.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PIA_RD2XY, r, d, trans, x, y, xr, xd, yr, yd
;
; INPUTS:
;	R		The right_ascension value(s) in degrees.
;			May be an array.
;
;	D		The declination value(s) in degrees.
;			May be an array.
;
;	TRANS		The transformation parameters structure:
;	.CD		forward transformation matrix
;	.DC		back transformation matrix
;	.CRVAL		reference position (ra,dec)
;	.CRPIX		reference pixel (x,y)
;
; KEYWORD PARAMETERS:
;	(none)
;
; OUTPUTS:
;	X		The x coordinate values in the map
;
;	Y		The y coordinate values in the map
;
; OPTIONAL OUTPUTS:
;	XR		The partial derivative dX/dR (R in degrees)
;
;	XD		The partial derivative dX/dD (D in degrees)
;
;	YR		The partial derivative dY/dR (R in degrees)
;
;	YD		The partial derivative dY/dD (D in degrees)
;
; NOTES:
;	This routine differs from the similar ASTROLIB routine AD2XY
;	in several ways, mainly:
;	(1) The input parameter TRANS is expected to contain the
;	    inverse transformation matrix DC.
;	(2) The definitions of the matrixes CD and DC are not consistent
;	    with the definition of the ASTROLIB matrix CD or its inverse.
;	(3) CRPIX values correspond to IDL indexing scheme (starting at 0).
;	(4) Derivatives are computed.
;
; PROCEDURES USED:
;	none
; MODIFICATION HISTORY:
;	Written by:	Detlef Skaley, MPIK, Heidelberg		March 1995
;			[Adapted from ASTROLIB routine AD2XY]
;	Oct 95 Version 3.5
;	Derivative parameter changed after testing (CG)		March 1996
;-

	PRO PIA_RD2XY, r, d, trans, x, y, xr, xd, yr, yd

	dtor = !DPI/180.0D
	dec = d * dtor
	delta_ra = (r - trans.crval(0))*dtor

	sin_d = sin(dec)
	cos_d = cos(dec)
	sin_d0 = sin(trans.crval(1)*dtor)
	cos_d0 = cos(trans.crval(1)*dtor)
	sin_rr0 = sin(delta_ra)
	cos_rr0 = cos(delta_ra)

	h = sin_d*sin_d0 + cos_d*cos_d0*cos_rr0
	xsi = cos_d*sin_rr0/h
	eta = (sin_d*cos_d0 - cos_d*sin_d0*cos_rr0)/h
 
	x = trans.dc(0,0)*xsi + trans.dc(0,1)*eta + trans.crpix(0)
	y = trans.dc(1,0)*xsi + trans.dc(1,1)*eta + trans.crpix(1)

	IF N_PARAMS() GT 5 THEN BEGIN
		hr = -cos_d*cos_d0*sin_rr0
		hd = cos_d*sin_d0-sin_d*cos_d0*cos_rr0
		xsir = ( cos_d*cos_rr0 - xsi*hr)/h * dtor
		xsid = (-sin_d*sin_rr0 - xsi*hd)/h * dtor
		etar = (               cos_d*sin_d0*sin_rr0 - eta*hr)/h * dtor
		etad = (-cos_d*cos_d0 + sin_d*sin_d0*cos_rr0 - eta*hd)/h * dtor
; replacing
; old 		etad = (cos_d*cos_d0 + sin_d*sin_d0*cos_rr0 - eta*hd)/h * dtor

		xr = trans.dc(0,0)*xsir + trans.dc(0,1)*etar
		xd = trans.dc(0,0)*xsid + trans.dc(0,1)*etad
		yr = trans.dc(1,0)*xsir + trans.dc(1,1)*etar
		yd = trans.dc(1,0)*xsid + trans.dc(1,1)*etad
	ENDIF

	RETURN
	END
