;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_READ_CS
;
; PURPOSE:
;	Read in the binary table of a PHT CS (Compact Status) file
;	whis has been opened already with PIA_OPEN_FXB.
;	The CS file is in FITS binary table format.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;       PIA_READ_CS, unit, table, rows
;
; INPUTS:
;       unit :	The unit number of the opened FXB file.
;
; OPTIONAL INPUTS:
;	rows :	If 'rows' is a single value, only this row is read.
;		If 'rows' ia a 2-dimensional array, it gives the range
;		of rows to be read. Row numbers start with 1.
;
; KEYWORDS:
;	(none)
;
; OUTPUTS:
;	table :	Structure of binary values read from the CS file table.
;
;	rows :	A 2-dimensional array giving the range of rows which have
;		been read.
;
; RESTRICTIONS:
;	unit must have been opened with FXBOPEN
;
; COMMON BLOCKS:
;	fxbintable	from ASTRON library
;
; PROCEDURES USED:
;	Astronomical Library:
;		fxbread
;
; MODIFICATION HISTORY:
;       Written by:	C. Gabriel, ESA/ESTEC		February 1994
;	Modified by:	D. Skaley, MPIK			August 1994
;	Oct 95 Version 3.5
;-

PRO PIA_READ_CS, unit, table, rows

; include the FXBINTABLE common block

@fxbintable

;  Find the actual dimension of the table

	i = (WHERE(lun EQ unit))(0)	; lun is defined in FXBINTABLE
	ncol = tfields(i)
	nrow = naxis2(i)

;  Determine which rows to read

	argr = N_ELEMENTS(rows)
	IF argr EQ 0 OR rows(0) EQ 0 THEN BEGIN
		row_from = 1
		row_to = nrow
	ENDIF ELSE BEGIN
		row_from = (rows(0) > 1) < nrow
		IF argr EQ 1 THEN $
			row_to = row_from $
		ELSE $
			row_to = (rows(1) > row_from) < nrow
	ENDELSE
	rows = [ row_from, row_to ]

;  Read in columns first into single variables...

;  Read the Compact Status General Prefix  
	FXBREAD, unit, ukst, 'CSGPUKST', rows
	FXBREAD, unit, uken, 'CSGPUKEN', rows
	FXBREAD, unit, ikst, 'CSGPIKST', rows
	FXBREAD, unit, iken, 'CSGPIKEN', rows
	FXBREAD, unit, utst, 'CSGPUTST', rows
	FXBREAD, unit, uten, 'CSGPUTEN', rows
	FXBREAD, unit, osn,  'CSGPOSN',  rows
;  Read the specific PHT part
	FXBREAD, unit, subs, 'PSTASUBS', rows
	FXBREAD, unit, opfo, 'PSTAOPFO', rows
	FXBREAD, unit, f1ts, 'PSTAF1TS', rows
	FXBREAD, unit, f2ts, 'PSTAF2TS', rows
	FXBREAD, unit, f1ps, 'PSTAF1PS', rows
	FXBREAD, unit, f2ps, 'PSTAF2PS', rows
	FXBREAD, unit, c1ps, 'PSTAC1PS', rows
	FXBREAD, unit, c2ps, 'PSTAC2PS', rows
	FXBREAD, unit, c3ps, 'PSTAC3PS', rows
	FXBREAD, unit, cmod, 'PSTACMOD', rows
	FXBREAD, unit, camp, 'PSTACAMP', rows
	FXBREAD, unit, cste, 'PSTACSTE', rows
	FXBREAD, unit, cinc, 'PSTACINC', rows
	FXBREAD, unit, cres, 'PSTACRES', rows
	FXBREAD, unit, deta, 'PSTADETA', rows
	FXBREAD, unit, drs,  'PSTADRS', rows
	FXBREAD, unit, mux1, 'PSTAMUX1', rows
	FXBREAD, unit, mux2, 'PSTAMUX2', rows
	FXBREAD, unit, xsta, 'PSTAXSTA', rows
	FXBREAD, unit, d1of, 'PSTAD1OF', rows
	FXBREAD, unit, d1ga, 'PSTAD1GA', rows
	FXBREAD, unit, d2ga, 'PSTAD2GA', rows
	FXBREAD, unit, d2of, 'PSTAD2OF', rows
	FXBREAD, unit, freq, 'PSTAFREQ', rows
	FXBREAD, unit, nndr, 'PSTANNDR', rows
	FXBREAD, unit, ndr,  'PSTANDR', rows
	FXBREAD, unit, intt, 'PSTAINTT', rows
	FXBREAD, unit, meat, 'PSTAMEAT', rows
	FXBREAD, unit, mpc1, 'PSTAMPC1', rows
	FXBREAD, unit, mpc2, 'PSTAMPC2', rows
	FXBREAD, unit, mpc3, 'PSTAMPC3', rows
	FXBREAD, unit, met,  'PSTAMET', rows

;  ...then put it into an anonymous structure to be returned

	table = { $
		ukst: [ ukst ], $
		uken: [ uken ], $
		ikst: [ ikst ], $
		iken: [ iken ], $
		utst: [ utst ], $
		uten: [ uten ], $
		osn:  [ osn ],  $
		subs: [ subs ], $
		opfo: [ opfo ], $
		f1ts: [ f1ts ], $
		f2ts: [ f2ts ], $
		f1ps: [ f1ps ], $
		f2ps: [ f2ps ], $
		c1ps: [ c1ps ], $
		c2ps: [ c2ps ], $
		c3ps: [ c3ps ], $
		cmod: [ cmod ], $
		camp: [ camp ], $
		cste: [ cste ], $
		cinc: [ cinc ], $
		cres: [ cres ], $
		deta: [ deta ], $
		drs:  [  drs ], $
		mux1: [ mux1 ], $
		mux2: [ mux2 ], $
		xsta: [ xsta ], $
		d1of: [ d1of ], $
		d1ga: [ d1ga ], $
		d2of: [ d2of ], $
		d2ga: [ d2ga ], $
		freq: [ freq ], $
		nndr: [ nndr ], $
		ndr:  [  ndr ], $
		intt: [ intt ], $
		meat: [ meat ], $
		mpc1: [ mpc1 ], $
		mpc2: [ mpc2 ], $
		mpc3: [ mpc3 ], $
		met:  [  met ] $
	}

END
