;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_READ_ERD
;
; PURPOSE:
;	Read in the binary table of a PHT ERD (Edited Raw Data) file
;	whis has been opened already with PIA_OPEN_FXB.
;	The ERD file is in FITS binary table format.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;       PIA_READ_ERD, unit, erdtype, table, rows
;
; INPUTS:
;       unit :	The unit number of the opened FXB file.
;
;	erdtype :
;		The ERD type: 'C1' = C100, 'C2' = C200, 'P' = Px, 'S' = Sx
;
; OPTIONAL INPUTS:
;	rows :	If 'rows' is a single value, only this row is read.
;		If 'rows' ia a 2-dimensional array, it gives the range
;		of rows to be read. Row numbers start with 1.
;
; KEYWORDS:
;	(none)
;
; OUTPUTS:
;	table :	Structure of binary values read from the ERD file table.
;
;	rows :	A 2-dimensional array giving the range of rows which have
;		been read.
;
; RESTRICTIONS:
;	unit must have been opened with FXBOPEN
;
; COMMON BLOCKS:
;	fxbintable	from ASTRON library
;
; PROCEDURES USED:
;	Astronomical Library:
;		fxbread
;
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK		August 1994	
;	Oct 95 Version 3.5
;-

PRO PIA_READ_ERD, unit, erdtype, table, rows

; include the FXBINTABLE common block

@fxbintable

;  Find the actual dimension of the table

	i = (WHERE(lun EQ unit))(0)	; lun is defined in FXBINTABLE
	ncol = tfields(i)
	nrow = naxis2(i)

;  Determine which rows to read

	argr = N_ELEMENTS(rows)
	IF argr EQ 0 OR rows(0) EQ 0 THEN BEGIN
		row_from = 1
		row_to = nrow
	ENDIF ELSE BEGIN
		row_from = (rows(0) > 1) < nrow
		IF argr EQ 1 THEN $
			row_to = row_from $
		ELSE $
			row_to = (rows(1) > row_from) < nrow
	ENDELSE
	rows = [ row_from, row_to ]

;  Read in columns first into single variables...

;  Read the columns of the FITS table
	FXBREAD, unit, tkey, 'GPSCTKEY', rows
	FXBREAD, unit, rpid, 'GPSCRPID', rows
	FXBREAD, unit, pixf, 'P1ERPIXF', rows
	FXBREAD, unit, pcs1, 'P1ERPCS1', rows
	FXBREAD, unit, pcs2, 'P1ERPCS2', rows
	FXBREAD, unit, cpos, 'P1ERCPOS', rows

	CASE erdtype OF
	'C1' :	BEGIN					; C100
		FXBREAD, unit, crev, 'P1ERCREV', rows
		FXBREAD, unit, temp, 'P1ERTEMP', rows
		FXBREAD, unit, mbv , 'P1ERMBV ', rows
		FXBREAD, unit, pixr, 'P1ERPIXR', rows
		FXBREAD, unit, pixo, 'P1ERPIXO', rows
		FXBREAD, unit, pix , 'P1ERPIX ', rows
		END
	'C2' :	BEGIN					; C200
		FXBREAD, unit, crev, 'P1ERCREV', rows
		FXBREAD, unit, temp, 'P1ERTEMP', rows
		FXBREAD, unit, mbv1, 'P1ERMBV1', rows
		FXBREAD, unit, mbv2, 'P1ERMBV2', rows
		FXBREAD, unit, mbv3, 'P1ERMBV3', rows
		FXBREAD, unit, mbv4, 'P1ERMBV4', rows
		FXBREAD, unit, pix1, 'P1ERPIX1', rows
		FXBREAD, unit, pix2, 'P1ERPIX2', rows
		FXBREAD, unit, pix3, 'P1ERPIX3', rows
		FXBREAD, unit, pix4, 'P1ERPIX4', rows
		FXBREAD, unit, pix5, 'P1ERPIX5', rows
		FXBREAD, unit, pix6, 'P1ERPIX6', rows
		END
	'P' :	BEGIN					; P1/P2/P3
		FXBREAD, unit, temp, 'P1ERTEMP', rows
		FXBREAD, unit, mbv , 'P1ERMBV ', rows
		FXBREAD, unit, pix , 'P1ERPIX ', rows
		END
	'S' :	BEGIN					; S1/S2
		FXBREAD, unit, crev, 'P1ERCREV', rows
		FXBREAD, unit, tem1, 'P1ERTEM1', rows
		FXBREAD, unit, tem2, 'P1ERTEM2', rows
		FXBREAD, unit, mbv1, 'P1ERMBV1', rows
		FXBREAD, unit, mbv2, 'P1ERMBV2', rows
		FXBREAD, unit, pix1, 'P1ERPIX1', rows
		FXBREAD, unit, pix2, 'P1ERPIX2', rows
		END
	ELSE:
	ENDCASE

;  ...then put it into an anonymous structure to be returned

	CASE erdtype OF
	'C1' :	BEGIN					; C100
		table = { $
			tkey: [ tkey ], $
			rpid: [ rpid ], $
			pixf: [ pixf ], $
			pcs1: [ pcs1 ], $
			pcs2: [ pcs2 ], $
			crev: [ crev ], $
			temp: [ temp ], $
			cpos: [ cpos ], $
			mbv : [ mbv  ], $
			pixr: [ pixr ], $
			pixo: [ pixo ], $
			pix : [ pix  ] $
		}
		END
	'C2' :	BEGIN					; C200
		table = { $
			tkey: [ tkey ], $
			rpid: [ rpid ], $
			pixf: [ pixf ], $
			pcs1: [ pcs1 ], $
			pcs2: [ pcs2 ], $
			crev: [ crev ], $
			temp: [ temp ], $
			cpos: [ cpos ], $
			mbv1: [ mbv1 ], $
			mbv2: [ mbv2 ], $
			mbv3: [ mbv3 ], $
			mbv4: [ mbv4 ], $
			pix1: [ pix1 ], $
			pix2: [ pix2 ], $
			pix3: [ pix3 ], $
			pix4: [ pix4 ], $
			pix5: [ pix5 ], $
			pix6: [ pix6 ] $
		}
		END
	'P' :	BEGIN					; P1/P2/P3
		table = { $
			tkey: [ tkey ], $
			rpid: [ rpid ], $
			pixf: [ pixf ], $
			pcs1: [ pcs1 ], $
			pcs2: [ pcs2 ], $
			temp: [ temp ], $
			cpos: [ cpos ], $
			mbv : [ mbv  ], $
			pix : [ pix  ] $
		}
		END
	'S' :	BEGIN					; S1/S2
		table = { $
			tkey: [ tkey ], $
			rpid: [ rpid ], $
			pixf: [ pixf ], $
			pcs1: [ pcs1 ], $
			pcs2: [ pcs2 ], $
			crev: [ crev ], $
			tem1: [ tem1 ], $
			tem2: [ tem2 ], $
			cpos: [ cpos ], $
			mbv1: [ mbv1 ], $
			mbv2: [ mbv2 ], $
			pix1: [ pix1 ], $
			pix2: [ pix2 ] $
		}
		END
	ENDCASE
	RETURN

END
