;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_READ_FITS
;
; PURPOSE:
;	Recognize several ISOPHOT FITS files and pass it over to 
;	the corresponding I/O routine
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	PIA_READ_FITS, filename [, header], [path = path]
;
; INPUTS:
;	filename	The name of the file (PATH not necessary)
;	
; OPTIONAL INPUT:
;       path:  path for the filename
; OUTPUTS:
;	Header
;	
; SIDE EFFECTS:
;	None
;	
; RESTRICTIONS:
;	filename must correspond to a real PHT-FITS file
;	
; METHOD:
;	Use the name to distinguish among the different PHT FITS files
; 	
; COMMON BLOCKS:
;	None
;	
; PROCEDURES USED:
;	PIA Procedure(s):
;		read_cala
;		read_cs
;		read_erd
;		read_scp
;		read_spd
;		read_spd_sl
;		read_srd
;
; MODIFICATION HISTORY:
;	Written by:	Carlos GABRIEL (ESA/ESTEC)		July 1994
;	Modified(last):	SB 					October 1996
;	V5.x
;	Upgraded avoiding the search for suffix 'FITS' (CG)	May 1997
;	V6.3
;	Bug for IA produced FITS files corrected (CG)		October 1997
;	V6.5
;-
;

PRO pia_read_fits, filename, header, path = path


        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  PIA_READ_FITS, FILENAME [, HEADER], [PATH = PATh]'

	filename = STRUPCASE(STRTRIM(filename,2))

; Change to avoid problem with directories containing suffix FITS
;	posfits = STRPOS(filename,'.FITS')
;	posfits = posfits+1
; Reverse the name, look for last directory element, get the right name
; but keep the old way of deriving 'posfits' to avoid errors 
 
	revers_name = STR_REVERSE(filename)
        IF !version.os EQ "VMS" THEN $
            ind = STRPOS(revers_name,']') ELSE $
            ind = STRPOS(revers_name,'/') 
        IF ind eq -1 THEN tmpname = revers_name ELSE $            
          tmpname = STRMID(revers_name,0,ind)
        tmpname = STR_REVERSE(tmpname)

	poscs = STRPOS(filename,tmpname)
	posfits = poscs + 13	

;	poscs = posfits - 13

; Is it a Compact Status file?
; - change -> shape4
	if (STRMID(filename,poscs,4) eq 'PSTA') then BEGIN
		filename = STRMID(filename,poscs,17)
		if keyword_set(path) then $
                  read_cs, filename, header, path = path $
                else read_cs, filename, header
		RETURN
	endif

; Is it an IA produced FITS file
	IF STRMID(filename,posfits-1,1) EQ '_' THEN BEGIN

		type = STRMID(filename, posfits,3)
		filename = STRMID(filename, posfits-13, 21)

		CASE type of 

			'ERD': begin
				 if keyword_set(path) then $
				   read_erd, filename,  path = path $
				 else $
			           read_erd, filename
			       end
			'SRD': begin
                                 if keyword_set(path) then $
                                   read_srd, filename, path = path $
                                 else $
                                   read_srd, filename
                               end
			'SCP': begin
                                 if keyword_set(path) then $
                                   read_scp, filename, path = path $
                                 else $
                                   read_scp, filename
                               end
			'SPD': begin
				 magic_l = STRMID(filename,1,1)
				 IF magic_l EQ 'S' or magic_l EQ 'L' THEN sl=1 $
				 ELSE sl = 0
                                 if keyword_set(path) then begin
                                  IF sl THEN $
				    read_spd_sl, filename, path = path ELSE $
				  read_spd, filename, path = path 
                                 endif else begin
                                  IF sl THEN $
				    read_spd_sl, filename ELSE $
				  read_spd, filename 
				endelse
                               end
			'AAR': begin
                                 if keyword_set(path) then $
				   if strmid(filename, 0, 3) EQ 'PGA' then $
				     read_map, filename, hdr, buf, $
				     path=path else $
                                   read_aar, filename, hdr, buf, path = path $
                                 else $
				   if strmid(filename, 0, 3) EQ 'PGA' then $
				     read_map, filename, hdr, buf $
				     else $
                                   read_aar, filename, hdr, buf
;                                 pia_showstr, buf, GROUP=event.top  
                               end
                        ELSE:

		ENDCASE

		RETURN

	ENDIF

; IF it is a Serendipity FITS File

	if STRMID(filename,posfits-13,4) EQ 'P2SM' THEN BEGIN
	   filename = STRMID(filename, posfits-13, 17)
	   if keyword_set(path) then $
              read_smc, filename, path = path $
           else $
              read_smc, filename
	   RETURN
	endif

; IF not it is a real external fits file

	posall = posfits - 13

	if (STRMID(filename,posall+2,1) eq 'A') then BEGIN
		filename = STRMID(filename, posall, 17)
                if keyword_set(path) then $
		  if strmid(filename, 0, 3) EQ 'PGA' then $
		    read_map, filename, hdr, buf, $
		    path=path else $
                    read_aar, filename, hdr, buf, path = path $
                else $
		  if strmid(filename, 0, 3) EQ 'PGA' then $
		    read_map, filename, hdr, buf $
		    else $
                  read_aar, filename, hdr, buf
;                pia_disp_print_file, '', TEXT=hdr, title='AAR'
;                pia_showstr, buf 
		RETURN
	endif

	filename = STRMID(filename, posall, 17)

	l4 = STRUPCASE(STRMID(filename,3,1))

	CASE l4 of

		'R': begin
		       if keyword_set(path) then $
		         read_erd, filename, header, path = path $
		       else read_erd, filename, header
		     end
		'S': begin
		    IF STRUPCASE(STRMID(filename,0,4)) EQ 'P2ES' THEN BEGIN
		       if keyword_set(path) then $
		         read_erd, filename, header, path = path $
		       else read_erd, filename, header
		    ENDIF ELSE BEGIN
		       if keyword_set(path) then begin
		         IF STRMID(filename,0,2) EQ 'PS' THEN $
		           read_spd_sl, filename, path = path ELSE $
			 read_spd, filename, header, path = path
		       endif else begin
		         IF STRMID(filename,0,2) EQ 'PS' THEN $
		           read_spd_sl, filename ELSE $
			 read_spd, filename, header
		       endelse
		     ENDELSE
		     
		     end

		'A': begin
		       if keyword_set(path) then $
		         read_cala, filename, header, path = path $
		       else read_cala, filename, header
		     end
		
		ELSE: MESSAGE, /INFO, 'No file match   '

	ENDCASE

	RETURN
END
