;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_READ_PPSF
;
; PURPOSE:
;	Read the cal-G file PPSF and store the values in a common block.
;	PPSF contains the 'Point Spread Function' correction values.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	PIA_READ_PPSF, filename
;
; INPUTS:
;	(none)
;
; OPTIONAL INPUTS:
;	filename :	An alternative PSF cal-G file.
;			The default file is read if this argument is not given.
;
; KEYWORD PARAMETERS:
;	ERROR : 	returns an error condition:
;			0 = no error
;			1 = could not successfully read the cal-G file
;
; OUTPUTS:
;	(none)
;
; SIDE EFFECTS:
;	The PSF values are stored in the common block
;
; COMMON BLOCKS:
;	PIACMN_GBLDEF	global defaults
;	PIACMN_POW2FB	to hold the PPSF values
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_gbldef_init
;		read_ppsf
;		readhdr
;	Astronomical Library:
;		fxpar
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	February 1995
;	Modified by:	C. Gabriel, ESA/VILSPA		July 1995	
;	Oct 95 Version 3.5
;-

	PRO PIA_READ_PPSF, filename, ERROR=error

; common blocks

; global defaults: need the default directory for cal-G files

@piacmn_gbldef

; to store the PPSF values

@piacmn_pow2fb

; initialize error condition

	error = 1

; define the default cal-G file

	IF N_ELEMENTS(filename) EQ 0 THEN BEGIN
		IF N_ELEMENTS(gbldef) EQ 0 THEN PIA_GBLDEF_INIT
		filename = gbldef.calgdir + "PPSF" + gbldef.fitsext
	ENDIF

; open the PPSF file

	OPENR, lun, /GET_LUN, filename, /BLOCK, ERROR=err
	IF err NE 0 THEN BEGIN
		MESSAGE, /INFO, !ERR_STRING
		RETURN
	ENDIF

; read the FITS header and do some checks

	READHDR, lun, header, ERROR=err
	IF err THEN GOTO, not_ppsf
	IF FXPAR( header, "NAXIS" ) NE 2 THEN GOTO, not_ppsf
	IF FXPAR( header, "BITPIX" ) NE -32 THEN GOTO, not_ppsf
	naxis1 = FXPAR( header, "NAXIS1" )
	IF naxis1 NE 15 THEN GOTO, not_ppsf
	naxis2 = FXPAR( header, "NAXIS2" )
	IF naxis2 NE 14 THEN GOTO, not_ppsf

; read the values and transform to host bit representation
; variable 'ppsf' is stored in PIACMN_POW2FB

	tmp = fltarr( naxis1, naxis2 )
	ON_IOERROR, not_ppsf
	READU, lun, tmp
	BYTEORDER, tmp, /XDRTOF

; waste index 0 to be compatible with aperture/filter numbering

	ppsf = fltarr( naxis1+1, naxis2+1 )
	ppsf(1,1) = tmp

; normal exit

	error = 0
	FREE_LUN, lun
	RETURN

; error conditions:

not_ppsf:
	MESSAGE, /INFO, "Not a valid PPSF file: "+filename
	FREE_LUN, lun
	RETURN

	END
