;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_READ_SPD
;
; PURPOSE:
;	Read in a ISOPHOT SPD FITS file.
;	The data are stored in arrays which can be accessed by handles.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;	PIA_READ_SPD, filename, data, ERROR=error
;
; INPUTS:
;	filename :	The full name of the SPD file to be read
;			(may include a directory specification).
;
; KEYWORD PARAMETERS:
;	ERROR :		Returns an error status:
;			0 = no error: file has been read
;			1 = error: file could not be read
;
; OUTPUTS:
;	data :		A handle with child handles containing the data.
;			The first child handle containes the FITS header
;			(primary+extension header combined).
;			The sibling childs contain the data columns from the
;			FITS binary table.
; COMMON BLOCKS:
;	none
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		mergehdr
;		readhdr
;	Astronomical Library:
;		fxpar
;
; MODIFICATION HISTORY:
;	Written by: 	D. Skaley, MPIK/Heidelberg	February 1995
;	Oct 95 Version 3.5
;-

PRO PIA_READ_SPD, filename, data, ERROR = error

; initialize error status

	error = 1

; open the SPD FITS file

	OPENR, lun, /GET_LUN, filename, /BLOCK, ERROR=err
	IF err NE 0 THEN BEGIN
		MESSAGE, /INFO, !ERR_STRING
		RETURN
	ENDIF

; read the primary header
; check whether it contains an extension

	READHDR, lun, primary_header, ERROR=err
	IF err THEN GOTO, not_spd
	IF NOT FXPAR(primary_header,"EXTEND") THEN GOTO, not_spd

; read the extended header

	READHDR, lun, extended_header, ERROR=err
	IF err THEN GOTO, not_spd

; merge the two headers

	MERGEHDR, primary_header, extended_header, header
	primary_header = 0
	extended_header = 0

; determine the size of the binary table

	naxis = FXPAR( header, "NAXIS" )
	IF naxis NE 2 THEN GOTO, not_spd
	naxis1 = FXPAR( header, "NAXIS1" )
	naxis2 = FXPAR( header, "NAXIS2" )

; determine the detector-dependant SPD product type

	tfields = FXPAR( header, "TFIELDS" )
	IF tfields LT 4 THEN GOTO, not_spd
	ttype4 = FXPAR( header, "TTYPE4" )
	product = STRMID(ttype4,0,4)

; define the table columns

	CASE product OF
	'PC1S':	BEGIN					; C100 SPD
		IF naxis1 NE 232 THEN GOTO, not_spd
		IF tfields NE 16 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpscfill:0, $
			kyid:0, mnum:0, spar:0, filt:0, $
			aper:0, polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(9,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(9,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(9,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(9,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(9,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(9,/BYTE,/NOZERO), $
			fill:[0B,0B,0B] $
		}
		END
	'PC2S':	BEGIN					; C200 SPD
		IF naxis1 NE 124 THEN GOTO, not_spd
		IF tfields NE 15 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:0, filt:0, $
			aper:0, polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(4,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(4,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(4,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(4,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(4,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(4,/BYTE,/NOZERO) $
		}
		END
	'PP1S':	BEGIN					; P1 SPD
		IF naxis1 NE 64 THEN GOTO, not_spd
		IF tfields NE 16 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:0, filt:0, $
			aper:0, polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(1,/BYTE,/NOZERO), $
			fill:[0B,0B,0B] $
		}
		END
	'PP2S':	BEGIN					; P2 SPD
		IF naxis1 NE 64 THEN GOTO, not_spd
		IF tfields NE 16 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:0, filt:0, $
			aper:0, polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(1,/BYTE,/NOZERO), $
			fill:[0B,0B,0B] $
		}
		END
	'PP3S':	BEGIN					; P3 SPD
		IF naxis1 NE 64 THEN GOTO, not_spd
		IF tfields NE 16 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:0, filt:0, $
			aper:0, polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(1,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(1,/BYTE,/NOZERO), $
			fill:[0B,0B,0B] $
		}
		END
	'PSLS':	BEGIN					; SL SPD
		IF naxis1 NE 1384 THEN GOTO, not_spd
		IF tfields NE 15 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:[0,0,0], $
			polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(64,/BYTE,/NOZERO) $
		}
		END
	'PSSS':	BEGIN					; SS SPD
		IF naxis1 NE 1384 THEN GOTO, not_spd
		IF tfields NE 15 THEN GOTO, not_spd
		row = { tkey:0L, rpid:[0B,0B], gpcsfill:0, $
			kyid:0, mnum:0, spar:[0,0,0], $
			polz:0, ndrs:0, cstp:0, $
			dwel:0L, plen:0L, meas:0L, cpos:0L, $
			mnpw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			mnpu:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			mdpw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			q1pw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			q3pw:MAKE_ARRAY(64,/FLOAT,/NOZERO), $
			flag:MAKE_ARRAY(64,/BYTE,/NOZERO) $
		}
		END
	ELSE:	GOTO, not_spd
	ENDCASE

; detector type is: 'C', 'P' or 'S' (for later usage)

	dettype = STRMID(product,1,1)

; read the binary table

	buffer = REPLICATE( row, naxis2 )
	ON_IOERROR, not_spd
	READU, lun, buffer
	FREE_LUN, lun

; create the handles to hold the data

	data = HANDLE_CREATE()
	hid = { spd_data, $
		product: product, $		; product type
		hdr:	HANDLE_CREATE(data), $	; FITS header
		tkey:	HANDLE_CREATE(data), $	; Instrument time key
		rpid:	HANDLE_CREATE(data), $	; Raster point ID
		kyid:	HANDLE_CREATE(data), $	; Keyword ID
		mnum:	HANDLE_CREATE(data), $	; Measurement number
		filt:	HANDLE_CREATE(data), $	; Filter ID
		aper:	HANDLE_CREATE(data), $	; Aperture ID
		polz:	HANDLE_CREATE(data), $	; Polarizer ID
		ndrs:	HANDLE_CREATE(data), $	; Number of destructive readouts
		cspt:	HANDLE_CREATE(data), $	; Chopper step number
		dwel:	HANDLE_CREATE(data), $	; Chopper dwell time
		plen:	HANDLE_CREATE(data), $	; Plateau length
		meas:	HANDLE_CREATE(data), $	; Measurement time
		cpos:	HANDLE_CREATE(data), $	; Chopper position
		mnpw:	HANDLE_CREATE(data), $	; Mean or fitted power
		mnpu:	HANDLE_CREATE(data), $	; Uncertainty in mean power
		mdpw:	HANDLE_CREATE(data), $	; Median power
		q1pw:	HANDLE_CREATE(data), $	; 1st quartile of power
		q3pw:	HANDLE_CREATE(data), $	; 3rd quartile of power
		flag:	HANDLE_CREATE(data) $	; Status flag
	}

; transform table columns to host bit representation
; transpose 2-dim arrays where appropriate
; and move arrays to handles

	HANDLE_VALUE, hid.hdr, header, /SET, /NO_COPY

	tmp = buffer.tkey
	BYTEORDER, tmp, /NTOHL
	HANDLE_VALUE, hid.tkey, tmp, /SET, /NO_COPY

	tmp = buffer.rpid
	; rpid array is NOT transposed!
	HANDLE_VALUE, hid.rpid, tmp, /SET, /NO_COPY

	tmp = buffer.kyid
	BYTEORDER, tmp, /NTOHS
	HANDLE_VALUE, hid.kyid, tmp, /SET, /NO_COPY

	tmp = buffer.mnum
	BYTEORDER, tmp, /NTOHS
	HANDLE_VALUE, hid.mnum, tmp, /SET, /NO_COPY

	; 'filt' and 'aper' do not exist for PHT-S
	IF dettype NE 'S' THEN BEGIN

		tmp = buffer.filt
		BYTEORDER, tmp, /NTOHS
		HANDLE_VALUE, hid.filt, tmp, /SET, /NO_COPY

		tmp = buffer.aper
		BYTEORDER, tmp, /NTOHS
		HANDLE_VALUE, hid.aper, tmp, /SET, /NO_COPY

	ENDIF

	tmp = buffer.polz
	BYTEORDER, tmp, /NTOHS
	HANDLE_VALUE, hid.polz, tmp, /SET, /NO_COPY

	tmp = buffer.ndrs
	BYTEORDER, tmp, /NTOHS
	HANDLE_VALUE, hid.ndrs, tmp, /SET, /NO_COPY

	tmp = buffer.cstp
	BYTEORDER, tmp, /NTOHS
	HANDLE_VALUE, hid.cstp, tmp, /SET, /NO_COPY

	tmp = buffer.dwel
	BYTEORDER, tmp, /NTOHL
	HANDLE_VALUE, hid.dwel, tmp, /SET, /NO_COPY

	tmp = buffer.plen
	BYTEORDER, tmp, /NTOHL
	HANDLE_VALUE, hid.plen, tmp, /SET, /NO_COPY

	tmp = buffer.meas
	BYTEORDER, tmp, /NTOHL
	HANDLE_VALUE, hid.meas, tmp, /SET, /NO_COPY

	tmp = buffer.cpos
	BYTEORDER, tmp, /NTOHL
	HANDLE_VALUE, hid.cpos, tmp, /SET, /NO_COPY


	; transpose power arrays for PHT-C and PHT-S, not for PHT-P
	IF dettype EQ 'P' THEN trans=0 ELSE trans=1

	tmp = buffer.mnpw
	BYTEORDER, tmp, /XDRTOF
	IF trans THEN tmp = TRANSPOSE(TEMPORARY(tmp))
	HANDLE_VALUE, hid.mnpw, tmp, /SET, /NO_COPY

	tmp = buffer.mnpu
	BYTEORDER, tmp, /XDRTOF
	IF trans THEN tmp = TRANSPOSE(TEMPORARY(tmp))
	HANDLE_VALUE, hid.mnpu, tmp, /SET, /NO_COPY

	tmp = buffer.mdpw
	BYTEORDER, tmp, /XDRTOF
	IF trans THEN tmp = TRANSPOSE(TEMPORARY(tmp))
	HANDLE_VALUE, hid.mdpw, tmp, /SET, /NO_COPY

	tmp = buffer.q1pw
	BYTEORDER, tmp, /XDRTOF
	IF trans THEN tmp = TRANSPOSE(TEMPORARY(tmp))
	HANDLE_VALUE, hid.q1pw, tmp, /SET, /NO_COPY

	tmp = buffer.q3pw
	BYTEORDER, tmp, /XDRTOF
	IF trans THEN tmp = TRANSPOSE(TEMPORARY(tmp))
	HANDLE_VALUE, hid.q3pw, tmp, /SET, /NO_COPY


	tmp = buffer.flag
	; flag array is NOT transposed!
	HANDLE_VALUE, hid.flag, tmp, /SET, /NO_COPY

; store handle IDs in data VALUE

	HANDLE_VALUE, data, hid, /SET, /NO_COPY

; regular return

	error = 0
	RETURN


; error condition: the specified file is not a valid SPD file

not_spd:
	MESSAGE, /INFO, "Not an valid SPD file: "+STRING(filename)
	FREE_LUN, lun
	RETURN

	END
