;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_READ_UNKN
;
; PURPOSE:
;	Read in the table of a general FITS binary table file
;	which has already been opened with FXBOPEN.
;
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;       PIA_READ_UNKN, unit, table, rows, columns
;
; INPUTS:
;       unit :	The unit number of the opened FXB file.
;
; OPTIONAL INPUTS:
;	rows :	If single value, the row to be read,
;		if 2-dimensional array, the range of rows to be read.
;		Row numbers start with 1.
;
;	columns : If single value, the column to be read,
;		If 2-dimensional array, the range of columns to be read.
;		Column numbers start with 1.
;
; OUTPUTS:
;	table :	Handle pointing to FITS table data.
;		First child points to array of keywords (table column names),
;		the remaining childs point to arrays of table column values.
;
;	rows :	The actual range of rows which has been read in.
;
;	columns : The actual range of columns which has been read in.
;
; RESTRICTIONS:
;	(none)
;
; COMMON BLOCKS:
;	fxbintable  (from ASTRO lib)
;
; PROCEDURES USED:
;	Astronomical Library:
;		fxbfind
;		fxbread
;
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK/Heidelberg	August 1994
;	Oct 95 Version 3.5
;-

PRO PIA_READ_UNKN, unit, table, rows, columns

; include the FXBINTABLE common block

@fxbintable

;  Create parent handle to hold FITS data

	table = HANDLE_CREATE()

;  Find the actual dimensions of the table

	i = (WHERE( lun EQ unit ))(0)	; lun is defined in fxbintable
	ncol = tfields(i)
	nrow = naxis2(i)

;  Determine which columns / rows to read in

	argc = N_ELEMENTS(columns)
	IF argc EQ 0 OR columns(0) EQ 0 THEN BEGIN
		col_from = 1
		col_to = ncol
	ENDIF ELSE BEGIN
		col_from = (columns(0) > 1) < ncol
		IF argc EQ 1 THEN $
			col_to = col_from $
		ELSE $
			col_to = (columns(1) > col_from) < ncol
	ENDELSE
	columns = [ col_from, col_to ]

	argr = N_ELEMENTS(rows)
	IF argr EQ 0 OR rows(0) EQ 0 THEN BEGIN
		row_from = 1
		row_to = nrow
	ENDIF ELSE BEGIN
		row_from = (rows(0) > 1) < nrow
		IF argr EQ 1 THEN $
			row_to = row_from $
		ELSE $
			row_to = (rows(1) > row_from) < nrow
	ENDELSE
	rows = [ row_from, row_to ]

;  Read the keywords from the header and move them to first child of 'table'

	FXBFIND, unit, 'TTYPE', c, values, n_found, '????????'
	indx = INDGEN(col_to-col_from+1)+col_from-1
	values = values(indx)
	id = HANDLE_CREATE( table, VALUE=values, /NO_COPY )

;  Read in all columns and make them childs of 'table'

	FOR i = col_from,col_to DO BEGIN
		FXBREAD, unit, values, i, rows
		id = HANDLE_CREATE( table, VALUE=values, /NO_COPY )
	ENDFOR

	RETURN
END
