;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_RESPONS
;
; PURPOSE:
;	Menu for PIA Response calculation.
;
; CATEGORY:
;	PIA - graphics / processing
;
; CALLING SEQUENCE:
;	PIA_respons, tmpdata, header
;
; INPUTS:
;	tmpdata
;		A temporary SCP structure
;	header
;		The corresponding header
;	
; KEYWORD PARAMETERS:
;	GROUP
;		The group leader of the widget hirarchy
;	NO_WIDGET
;		To work without widget use - [=1: avg FCS1, =2: medians FCS1]
;	
; OUTPUTS:
;	header
;		Modified header
;	
; KEYWORD OUTPUT:
;	out_of_limits	If out_of_limits check positive, is set to 1	
;	
; SIDE EFFECTS:
;	Variables in the common block for actual responsivities are changed
;	
; RESTRICTIONS:
;	tmpdata must be a temporary PIA SCP structure, corresponding to an
;	FCS measurement
;
; METHOD:
;	Use of an FCS measurement and the corresponding calibration tables
;	for the responsivity determination	
;
; COMMON BLOCKS:
;	respon_cmn	local common
;	phtcs		compact status dynamical structure
;	phtresdef	default responsivities common
;	phtresact	actual responsivities common
;	phtfcs		FCS calibration tables
;	phtillum	FCS illumination matrices common
;	phtcap		capacitances common
;	phtoptpow	FCS optical power range
;	phtacc		actual selection criteria
;	
; PROCEDURES USED:
;	PIA Procedure(s):
;		conv_subs
;		cp_disc
;		phtcap_init
;		read_cs
;		read_fcsill
;		read_fcspow
;		read_resp
;	Internal:
;		pia_respons_event
;	Astrolib:
;		fxaddpar
;
; MODIFICATION HISTORY:
;	Written by:	Carlos GABRIEL (ESA/ESTEC)	October 1994
;	Modified(last):	CG					November 1996
;	V5.x
;	Adapted to new cal file FCSILL - CG		January 1997
;	V6.0
;	Adapted to use external flatfield factors - CG	May 1997
;	V6.3
;	Fills into phtresact name and filter used (CG)	July 1997
;	Widget appearance modified (Jacosta)		July 1997
;	V6.4
;	Using FCSPOW validity limits  
;	 and taking apertures from phtapert (CG + IH)	January 1998
;	Old default responsivity deleted (CG)		
;	KEYWORD out_of_limits added			February 1998
;	V7.0
;	Out_of_limits only set if "hard" limits 
;	 violation (0. or 99. mW)			March 1998
;	V7.1
;	Default choice is now FCS1 instead of 
;	 "default response" (CG)			June 1998
;	V7.2
;	Checking error from get_respons (CG)		November 1998
;	Modal use according to IDL >= 5.0 (CG)		January 1999
;	V7.3
;	Out_of_limits set if "soft" limits 
;	 violation - as in OLP 10.0 (CG) 		May 2002
;	 (done in routine get_respons.pro)
;-

; ****************************************************************************
;  Event handling routine for PIA Response Calculation
; ****************************************************************************

PRO pia_respons_event, event

COMMON respon_cmn1, respons, det, all_text, wids 

	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
	WIDGET_CONTROL, event.top, /HOURGLASS
	

	CASE 1 OF 
	   uvalue EQ 97: WIDGET_CONTROL, event.top, /DESTROY
	   uvalue GE 98: BEGIN
	            	   WIDGET_CONTROL, wids.id_fcschoi, GET_VALUE=fcschoi
	                   WIDGET_CONTROL, wids.id_valchoi, GET_VALUE=valchoi
	                   resp_choice = $
			     (2*fcschoi + valchoi * (fcschoi GE 1) - 1) > 0
	                   assign_respons, respons, det, resp_choice
 			   IF uvalue EQ 98 THEN BEGIN 
	                    add_text = 'Selected Responsivity is '
	                    CASE fcschoi OF 
	                        0: add_text = add_text + 'default value'
	                        1: add_text = add_text + 'from FCS1'
	                        2: add_text = add_text + 'from FCS2'
	                        3: add_text = add_text + 'from average <FCS1,2>'
	                    ENDCASE
	                    add_text = ['',add_text]
	                    IF fcschoi GT 0 THEN BEGIN
	                      IF valchoi EQ 0 THEN add_text = [add_text,    $
	                         'Using signal average'] 		    $
	                      ELSE add_text = [add_text,  		    $
	                         'Using signal median'] 
                            ENDIF
                            pia_disp_print_file, /MODAL, group=event.top, $
	                        "Resp_Res_"+STRTRIM(all_text(0),2),            $
                                TITLE = "Results of responsivity calculation", $
                                WIDTH = 30, HEIGHT = 40, $
                                TEXT = [add_text,'',all_text(1:*)]
 			   ENDIF
                           WIDGET_CONTROL, event.top, /DESTROY
	                 END
	   
	   ELSE:
	ENDCASE   
                
	RETURN
END

; ****************************************************************************
;  PIA Respons - menu definition procedure
; ****************************************************************************

PRO pia_respons, tmpdata, header, GROUP=group, NO_WIDGET=no_widget, $
	OUT_OF_LIMITS=out_of_limits, RCHOICE_BATCH = rchoice_batch 

COMMON respon_cmn1, respons_tmp, det, all_text, wids 

; If already registered, exit

	IF Xregistered("pia_respons") NE 0 THEN BEGIN
		RETURN
	END

	IF not KEYWORD_SET(no_widget) THEN $
	 IF not KEYWORD_SET(group) THEN group = !pia_top_wid	
	
	det = tmpdata.det
	npix = pixel_det(det)
	
	IF N_ELEMENTS(header) EQ 0 THEN header = tmpdata.hdr
	
	get_respons, tmpdata, header, respons, fcsill, nfcs, $
	   OUT_OF_LIMITS = out_of_limits, INFO = info_grespons, LABEL = labtxt,$
	   err_resp = err_resp
	
	IF info_grespons(0) NE '' THEN BEGIN
	 PRINT, STRING(7B)
	 PRINT, info_grespons
	ENDIF
	
	IF err_resp THEN BEGIN
	 MESSAGE, /INFO, ' stopping processing and returning'
	 RETURN
	END 
	 	
	respon_av = respons.aver
	respon_au = respons.avun
	respon_me = respons.med
	respon_mu = respons.medu
	theresp = respons.defv
	theresu = respons.defunc
		
	  
; Decide whether to widget or not
	 IF N_ELEMENTS(NO_WIDGET) EQ 0 THEN no_widget = 0 
	 ; check for old use of NO_WIDGET EQ 2 for default response and 
	 ; NO_WIDGET EQ 1 for FCS1
	 ; IF  rchoice is set then use this value 
	 IF N_ELEMENTS(rchoice_batch) EQ 0 THEN BEGIN 
	    CASE no_widget OF 
	       0: rchoice_batch = 1
	       1: rchoice_batch = 1
	       2: rchoice_batch = 2
	       ELSE: rchoice_batch = 1
            ENDCASE
	 ENDIF
	 	      
	 IF rchoice_batch LT 0 THEN rchoice_batch = 0
	 
         assign_respons, respons, det, rchoice_batch, uresp=uresp
	
; IF NO_WIDGET selected then stop here
	IF KEYWORD_SET(no_widget) THEN RETURN

; Define the widgets

	respons_tmp = respons
	
	txtsize = npix > 2
	decode_admn, tmpdata.admn, title
	titl0 = tmpdata.admn
	title = title + '   '+ tmpdata.unit

        respbase = WIDGET_BASE(TITLE = 'PIA Response Menu', $
                                /COLUMN, /MODAL, GROUP=group)

	junklabel= WIDGET_LABEL(respbase, VALUE=title)
	junklabel= WIDGET_LABEL(respbase, VALUE=labtxt)
	
	all_text = [titl0, title, labtxt]
	
 	id_0 = WIDGET_BASE(respbase, /ROW )

	id_1 = WIDGET_BASE(id_0, /COLUMN )
	text = 'Default Responses '
	all_text = [all_text,' ',text]
	
	junklabel= WIDGET_LABEL(id_1, VALUE=text)
	strresp = STRTRIM(STRING(theresp,format='(f6.2)'),2)+'+/-'+$
		   STRTRIM(STRING(theresu,format='(f6.2)'),2)+ ' [A/W]'
        junklist = WIDGET_TEXT(id_1, VALUE=strresp, YSIZE=txtsize)
	all_text = [all_text,strresp]

	id_1 = WIDGET_BASE(id_0, /COLUMN )
	text = 'Used FCS illum matrices '
	all_text = [all_text,' ',text]
	junklabel= WIDGET_LABEL(id_1, VALUE=text)

	id_2 = WIDGET_BASE(id_1, /ROW )
	text = STRTRIM(string(fcsill(*,0)),2)
	all_text = [all_text,text]
        junklist = WIDGET_TEXT(id_2, VALUE=text, $
			YSIZE=txtsize, XSIZE=8, no_newline=0)

	text = STRTRIM(string(fcsill(*,1)),2)
	all_text = [all_text,' ',text]
        junklist = WIDGET_TEXT(id_2, VALUE=text, $
			 YSIZE=txtsize, XSIZE=8, no_newline=0)

	 
 	id_0 = WIDGET_BASE(respbase, /ROW )

	count = -1
	bcount = 0
	resp_choice = 'Default'


	FOR FCSNO = 1, nfcs DO BEGIN
         count = count + 1
	
 	 id_1 = WIDGET_BASE( id_0, /COLUMN )
	 text = 'FCS '+strtrim(string(FCSNO),2)
	 all_text = [all_text,' ',text]
	 all_text = [all_text, 'using Averages:']
	 junklabel = WIDGET_LABEL(id_1, VALUE=text)

 	 id_2 = WIDGET_LABEL( id_1, VALUE='Responses using Averages:' )
	 strresp = STRTRIM(STRING(respon_av(*,count),format='(f7.3)'),2)+$
			'  +/- '+$
		   STRTRIM(STRING(respon_au(*,count),format='(f7.3)'),2)+$
		        '  [A/W]  '
	 all_text = [all_text,strresp]
         id_2 = WIDGET_TEXT(id_1, VALUE=strresp,$ 
				YSIZE=txtsize)
	 
 	 id_2 = WIDGET_LABEL( id_1, VALUE='Responses using Medians:' )
	 all_text = [all_text, ' ','using Medians:']
	 strresp = STRTRIM(STRING(respon_me(*,count),format='(f7.3)'),2)+$
				'  +/- '+$
		   STRTRIM(STRING(respon_mu(*,count),format='(f7.3)'),2)+$
		        '  [A/W]  '
         id_2 = WIDGET_TEXT(id_1, VALUE=strresp,$
				 YSIZE=txtsize)
	 all_text = [all_text,strresp]
	 
	 resp_choice = [resp_choice, 'FCS '+strtrim(string(FCSNO),2)]
	
	ENDFOR	 
	
	IF (nfcs) GT 1 THEN resp_choice = [resp_choice, $
	     'Mean [FCS1,FCS2]']
	;resp_choice = [resp_choice, 'User value']


	id_0 = WIDGET_BASE(respbase,/COLUMN,/FRAME)	
	id_1 = WIDGET_LABEL(id_0, $
		VALUE='Select responsivity value from:')
	id_fcschoi = CW_BGROUP(id_0, /EXCLUSIVE, SET_VALUE=1, $
	   resp_choice,UVALUE=0,COLUMN=N_ELEMENTS(resp_choice))
	   
	id_0 = WIDGET_BASE(respbase,/COLUMN,/FRAME)	
	id_1 = WIDGET_LABEL(id_0, VALUE='Using values from:')
	id_valchoi = CW_BGROUP(id_0, /EXCLUSIVE, SET_VALUE=0, $
	   ['Averages', 'Medians'],UVALUE=1,COLUMN=2)  

	id_0 = WIDGET_BASE(respbase, /ROW )
	id_1 = WIDGET_BUTTON(id_0, VALUE=STRING('ACCEPT',format='(A15)'), $
	     UVALUE=99 )
	id_1 = WIDGET_BUTTON(id_0, VALUE=STRING('ACCEPT PRINT',format='(A15)'), $
             UVALUE=98 )
	id_1 = WIDGET_BUTTON(id_0, VALUE=STRING('NO ACCEPT',format='(A15)'), $
	      UVALUE=97 )
	
	wids = {id_fcschoi:id_fcschoi, id_valchoi:id_valchoi}
	
; Realize widget

	WIDGET_CONTROL, respbase, /REALIZE, TLB_SET_XOFFSET = 600
	Xmanager,No_BLOCK=1, "pia_respons", respbase, GROUP=group


	RETURN
END
