;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_RESPONS_ABS
;
; PURPOSE:
;	Menu for PIA Response calculation for absolute Photometry.
;
; CATEGORY:
;	PIA - graphics / processing
;
; CALLING SEQUENCE:
;	PIA_respons_abs, tmpdata, header
;
; INPUTS:
;	tmpdata
;		A temporary SCP structure
;	header
;		The corresponding header
;	
; KEYWORD PARAMETERS:
;	GROUP
;		The group leader of the widget hirarchy
;	
; OUTPUTS:
;	header
;		Modified header
;
; KEYWORD OUTPUT:
;	out_of_limits	If out_of_limits check positive, is set to 1	
;	
; SIDE EFFECTS:
;	Variables in the common block for actual responsivities (absolute 
;	photometry case) are changed
;	
; RESTRICTIONS:
;	tmpdata must be a temporary PIA SCP structure, corresponding to an
;	absolute photometry measurement
;	
; METHOD:
;	Use of the FCS chopper plateaux within the absolute photometry 
;	measurement and the corresponding calibration tables
;	for responsivity determination per chopper plateau	
;
; COMMON BLOCKS:
;	resp_abs_cmn	local common
;	respabso	common with actual responsivities for abs. phot.
;	colour_indices	PIA color definitions common
;	phtcs		compact status dynamical structure
;	phtresdef	default responsivities common
;	phtresact	actual responsivities common
;	phtfcs		FCS calibration tables
;	phtillum	FCS illumination matrices common
;	phtcap		capacitances common
;	phtoptpow	FCS optical power range
;	phtacc		actual selection criteria
;	
; PROCEDURES USED:
;	PIA Procedure(s):
;		conv_subs
;		cp_disc
;		phtcap_init
;		pia_oplot
;		pia_plot
;		read_cs
;		read_fcsill
;		read_fcspow
;		read_resp
;	Internal:
;		viewabsres
;		resetview_abs
;		pia_respons_abs_event
;	Astrolib:
;		fxaddpar
;
; MODIFICATION HISTORY:
;	Written by:	Carlos GABRIEL (ESA/ESTEC)		November 1994
;	Modified(last):	CG					November 1996
;	V5.x
;	Adapted to new cal file FCSILL - CG		January 1997
;	V6.0
;	Adapted to use external flatfield factors - CG	May 1997
;	V6.3
;	Using FCSPOW validity limits  (CG)              January 1998
;	Old default responsivity deleted (CG)		February 1998
;	V7.0
;	Out_of_limits only set if "hard" limits 
;	 violation (0. or 99. mW)			March 1998
;	V7.1
;	Modal use according to IDL >= 5.0 (CG)		January 1999
;	V7.3
;-

; ****************************************************************************
;  Event routine for plotting
; ****************************************************************************

PRO viewabsres, event

        viewindexid = WIDGET_INFO(event.id, /PARENT)
        WIDGET_CONTROL, viewindexid, GET_UVALUE = viewindex
        WIDGET_CONTROL, event.id, GET_UVALUE = pixel

        resetview_abs, viewindex, pixel
END

; ****************************************************************************
;  Plotting routine
; ****************************************************************************

PRO resetview_abs, viewindex, pixel

@colour_indices
COMMON resp_abs_cmn, respon_av, respon_me, respon_au, respon_mu, $
	pixavcon, pixmecon, avgdraw, meddraw, ctime, flag, respon, respun    

	CASE viewindex of

	  0: BEGIN
		wset, avgdraw
		ytitle = 'Response Pixel '+strtrim(string(pixel+1),2)+'  [A/W]' 
		pia_plot, ctime, respon_av(pixel,*), yerr=respon_au(pixel,*),$
			psym=4, xtitle='Time   [s]',ytitle=ytitle    

		indnacc = WHERE(flag(pixel,*) EQ 2, nnacc)
		IF nnacc GT 0 THEN $
		   pia_oplot, ctime(indnacc), respon_av(pixel,indnacc), $
			color = red, psym = 7

		WIDGET_CONTROL, pixavcon, SET_VALUE='Response Averages Pixel '+$
			strtrim(string(pixel+1),2)

          END

	  1: BEGIN
		wset, meddraw
		ytitle = 'Response Pixel '+strtrim(string(pixel+1),2)+'  [A/W]' 
		pia_plot, ctime, respon_me(pixel,*), yerr=respon_au(pixel,*),$
			psym=4, xtitle='Time   [s]',ytitle=ytitle    

		indnacc = WHERE(flag(pixel,*) EQ 2, nnacc)
		IF nnacc GT 0 THEN $
		   pia_oplot, ctime(indnacc), respon_me(pixel,indnacc), $
			color = red, psym = 7
		WIDGET_CONTROL, pixmecon, SET_VALUE='Response Medians Pixel '+$
			strtrim(string(pixel+1),2)

          END

          ELSE:
	ENDCASE
END


; ****************************************************************************
;  Event handling routine for PIA Response Calculation
; ****************************************************************************

PRO pia_respons_abs_event, event

COMMON resp_abs_cmn, respon_av, respon_me, respon_au, respon_mu, $
	pixavcon, pixmecon, avgdraw, meddraw, ctime, flag, respon, respun    
COMMON respabso, respabs, resuabs, respflag

	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
	WIDGET_CONTROL, event.top, /HOURGLASS

	CASE uvalue OF

	   'ACCAV': BEGIN
		respabs = respon_av
		resuabs = respon_au
		respflag = flag
		WIDGET_CONTROL, event.top, /DESTROY
	   END

	   'ACCME': BEGIN
		respabs = respon_me
		resuabs = respon_mu
		respflag = flag
		WIDGET_CONTROL, event.top, /DESTROY
	   END

	   'QUIT': BEGIN
; set all flags to bad if cancelling
		respflag = flag * 0 + 2
		WIDGET_CONTROL, event.top, /DESTROY
           END
	ENDCASE

END

; ****************************************************************************
;  PIA Respons_abs - menu definition procedure
; ****************************************************************************

PRO pia_respons_abs, tmpdata, header, GROUP=group, OUT_OF_LIMITS=out_of_limits

COMMON resp_abs_cmn, respon_av, respon_me, respon_au, respon_mu, $
	pixavcon, pixmecon, avgdraw, meddraw, ctime, flag, respon, respun  
@phtcs
@phtacc
@phtresact
@phtfcs
@phtillum
@phtcap
@phtoptpow
@phtffield
@phtapert

; If already registered, exit

	IF Xregistered("pia_respons_abs") NE 0 THEN BEGIN
		RETURN
	ENDIF
	
	IF not KEYWORD_SET(group) THEN group = !pia_top_wid	
		
	out_of_limits=0
	
; FS!!   aperture = [4.0828, 8.9738, 14.522, 21.237, 30.581, 48.029, 48.029, $
;                    .03142, .1018, .173, .322, .454, 1.129, .849]
		
;        aperture = [4.1548, 9.0792, 14.522, 21.237, 30.250, 48.398, 48.398, $
;                    .03140, .1001, .1735,.2827,.528, 1.129, .7854]

	IF N_ELEMENTS(p_area) EQ 0 THEN read_omega
	 aperture = p_area(*,0)



; If phtffield not yet filled, read it from the FITS file
	IF n_elements(c1flatf) EQ 0 THEN read_flatfield
; If phtresp not yet filled, read it from the FITS file
	def_resp = default_response(tmpdata)
	npix = pixel_det(tmpdata.det)
	thresp = FLTARR(npix)
	thresu = thresp
	FOR ipix=0,npix-1 DO BEGIN
	 thresp(ipix)=AVG(def_resp(ipix,*,0))	
	 thresu(ipix)=AVG(def_resp(ipix,*,1))
	ENDFOR	

        det = tmpdata.det

; Which FCS measurement type have we got

	indfcs = WHERE(tmpdata.fcsn NE 0, nfc)
	junk = WHERE(tmpdata.fcsn EQ 0, nofc)
	fcspos = max(tmpdata.fcsn(indfcs))

	IF nfc EQ 0 OR nofc EQ 0 THEN BEGIN
		MESSAGE, /INFO, ' No absolute photometry measurement, returning'
		RETURN
	ENDIF

	labtxt = 'Absolute Photometry: Source vs FCS'+$
			strtrim(string(fcspos),2) 
	

; Check if dark current subtraction already performed, and if not, perform it 
	junk =  FXPAR(header, 'PRS_DARK', count=count)
	IF count EQ 0 THEN BEGIN
	
		PRINT, 'Performing dark current subtr. on abs_phot measurement'
		
		darkcur_orb, tmpdata, /SCP		

	ENDIF




; Call cp_disc to flag all discarded chopper plateaux
	cp_disc, tmpdata, cp_disc

	dim1 = N_ELEMENTS(cp_disc(*,0))
        dim2 = N_ELEMENTS(indfcs)

	avch = fltarr(dim1,dim2)
	avcu = avch
	mech = avch
	mecu = avch	   
; Initialize flag as bad
        flag = intarr(dim1,dim2) + 2

	ctime = tmpdata.time(indfcs)
	datav = tmpdata.aver(*,indfcs)
	datau = tmpdata.avun(*,indfcs)

	datme = tmpdata.medi(*,indfcs)
	datmu = (tmpdata.q1me(*,indfcs)+tmpdata.q3me(*,indfcs)) / 2
	   
        cpdf = cp_disc(*,indfcs)

; Loop over the pixels due to the individual flags

	FOR PIX=0,dim1-1 DO BEGIN

; Take only full accepted values
	    indacc = WHERE(cpdf(pix,*) EQ 0, nele)
	    IF nele GE 1 THEN BEGIN
		avch(pix,indacc) = datav(pix,indacc) 
		avcu(pix,indacc) = datau(pix,indacc) 
		mech(pix,indacc) = datme(pix,indacc) 
		mecu(pix,indacc) = datmu(pix,indacc) 
		flag(pix,indacc) = 0
            ENDIF ELSE BEGIN
; If none take only values with FLAG = 1
	       indacc = WHERE(cpdf(pix,*) EQ 1, nele)
	       IF nele GE 1 THEN BEGIN
		   MESSAGE, /INFO, $
			'Warning: Using signal/ch.pl. with FLAG=1 for FCS'+$
		    strtrim(string(fcspos),2)+', Pixel'+strtrim(string(pix+1),2) 
		   avch(pix,indacc) = datav(pix,indacc) 
		   avcu(pix,indacc) = datau(pix,indacc) 
		   mech(pix,indacc) = datme(pix,indacc) 
		   mecu(pix,indacc) = datmu(pix,indacc) 
		   flag(pix,indacc) = 1
	       ENDIF ELSE $
		   MESSAGE, /INFO, $ 	   
		    'No values available for FCS'+$
		    strtrim(string(fcspos),2)+', Pixel'+strtrim(string(pix+1),2) 
	    ENDELSE
	ENDFOR
; Median uncertainty = standard deviation
	mecu = avcu  	

; Read Capacities and/or FCS powers if necessary 
	IF N_ELEMENTS(slcap) EQ 0 THEN phtcap_init

; Access the right fcspow calibration values
	orig_name = 'P' + STRMID(tmpdata.admn,0,2) + 'FCSPOW'
	det_calfile, orig_name, tmpdata.unit, tmp_name, error=error	
	read_fcspow, tmp_name

; Read FCS illumination matrix if necessary
	IF STRMID(tmpdata.admn,0,1) EQ 'C' THEN BEGIN
	 orig_name = 'P' + STRMID(tmpdata.admn,0,2) + 'ILLUM'
	 det_calfile, orig_name, tmpdata.unit, tmp_name, error=error	
	 read_fcsill, tmp_name
	ENDIF


	CASE det of
		'P1': BEGIN
			capac = p1cap	
			fcspow = p1fcspow
			val_limits = p1fcsp_lim
			filpos = 3
			filoff = 1
			optpow = longpow
			txtsize = 2
			fcsill = intarr(1,2) + 1
			fcsilun = intarr(1,2)
			flatf = 1.
			flatu = 0.
		END   
		'P2': BEGIN
			capac = p2cap	
			fcspow = p2fcspow
			val_limits = p2fcsp_lim
			filpos = 3
			filoff = 11
			optpow = longpow
			txtsize = 2
			fcsill = intarr(1,2) + 1
			fcsilun = intarr(1,2)
			flatf = 1.
			flatu = 0.
		END   
		'P3': BEGIN
			capac = p3cap	
			fcspow = p3fcspow
			val_limits = p3fcsp_lim
			filpos = 3
			filoff = 13
			optpow = longpow
			txtsize = 2
			fcsill = intarr(1,2) + 1
			fcsilun = intarr(1,2)
			flatf = 1.
			flatu = 0.
		END   
		'C1': BEGIN
			capac = c1cap	
			fcspow = c1fcspow
			val_limits = c1fcsp_lim
			filpos = 2
			filoff = 2
			optpow = longpow
			txtsize = 9
			fcsill = c1ilcor
			fcsilun = c1illun
			IF !PIA_FFUSE THEN BEGIN
			 flatf = c1flatf
			 flatu = c1flatu
			ENDIF ELSE BEGIN
			 flatf = c1flatf * 0. + 1.
			 flatu = c1flatu*0.
			ENDELSE
		END   
		'C2': BEGIN
			capac = c2cap	
			fcspow = c2fcspow
			val_limits = c2fcsp_lim
			filpos = 2
			filoff = 9
			optpow = longpow
			txtsize = 4
			fcsill = c2ilcor
			fcsilun = c2illun
			IF !PIA_FFUSE THEN BEGIN
			 flatf = c2flatf
			 flatu = c2flatu
			ENDIF ELSE BEGIN
			 flatf = c2flatf*0 + 1.
			 flatu = c2flatu * 0.
			ENDELSE
		END   
		ELSE: BEGIN
			MESSAGE, /INFO, 'Detector not recognized'
			RETURN
		END
	ENDCASE

; Calculate In-band-Powers

	avch = avch * capac
	avcu = avcu * capac
	mech = mech * capac
	mecu = mecu * capac

; Read the Compact Status if necessary or get the unit - change -> shape4
	csfile = 'PSTA'+strmid(tmpdata.admn,4,8)+'.FITS'
	read_cs, csfile, cshdr, cslun
	
; Get out the index within the PHTCS structure
	handle_value, phtcs(cslun).handle, phtcst
	subs = conv_subs(phtcst.subs)
	indcs = WHERE(subs EQ det, n)
	indcs = indcs(tmpdata.ordr)

; Read TRSnr, FCS selected powers, FILTERnr --> convert to index begin = 0
; + aperture (to be multiplied to IBP if P-Detector [W/mm^2])
	CASE fcspos OF
		1: BEGIN 	
			trs = phtcst.f1ts(indcs) - 1 	
			fcscho = phtcst.f1ps(indcs)
		   END 	
		2: BEGIN 	
			trs = phtcst.f2ts(indcs) - 1 	
			fcscho = phtcst.f2ps(indcs)
		   END 		
        ENDCASE

	CASE filpos OF
		2: BEGIN
			filtnr = phtcst.c2ps(indcs) - filoff		
			apert = 1
			fcsill = reform(fcsill(*,filtnr,*))
			fcsilun = reform(fcsilun(*,filtnr,*))
			flatf = reform(flatf(*,filtnr))
			flatu = reform(flatu(*,filtnr))
		END
		3: BEGIN
			filtnr = phtcst.c3ps(indcs) - filoff		
			apert = aperture(phtcst.c2ps(indcs)-1)
		END
		ELSE: 
        ENDCASE

	fcscho = float(fcscho) * 50 / 4095
	IF fcscho EQ 50.0 THEN fcscho = 49.99

; Calculate expected in-band-power -> responses

	respon_av = avch * 0
	respon_me = respon_av
	respon_au = respon_av	
	respon_mu = respon_av	

; Is the electrical power equal to a value in the table? 
	pow_cs = fcscho
	expind = WHERE(fcspow(*,filtnr,trs,fcspos-1) EQ fcscho,n)

; If not, then look for the next positions and interpolate
	IF n EQ 0 THEN BEGIN
		expind = $
		   WHERE(fcspow(*,filtnr,trs,fcspos-1) GT fcscho,n)
		expind = expind(0)
		exprange = [fcspow(expind-1,filtnr,trs,fcspos-1), $
				fcspow(expind,filtnr,trs,fcspos-1)]

; Print warning
		val_lim = $
		 [val_limits(0,filtnr,fcspos-1),val_limits(1,filtnr,fcspos-1)]
		IF pow_cs LT val_lim(0) OR $
		   pow_cs GT val_lim(1) THEN BEGIN
		  decode_admn, tmpdata.admn, dec_str
; Default responsivity to be taken if one of the points completely invalid
		  IF exprange(0) EQ 0. OR exprange(1) EQ 99. THEN $
		    out_of_limits = 1
		  IF out_of_limits THEN $
		   special_text = $
			['FCS el.power applied ('+STRTRIM(pow_cs,2)+'mW) is '+$
			  'outside the "hard" limits of the FCS table.', $
			 'Value used is invalid.'] ELSE $
		   special_text = $
			['FCS el.power applied ('+STRTRIM(pow_cs,2)+'mW) is '+$
			 'outside the "soft" limits of the FCS table.', $
 			 'Value used is an extrapolation of measured points']

		  text_warn = [ $
 		  'WARNING:  using '+dec_str+' by Responsivity calculation', $
 		  ' ', special_text, $
		  'Actual "soft" limits for FCS '+STRTRIM(fcspos,2) + ', filter '+ $
		  STRTRIM(filtnr+1,2) + ' are ['+STRTRIM(val_lim(0),2)+','+$
		  STRTRIM(val_lim(1),2)+']mW']
		  PIA_disp_print_file, '', $
                           TITLE = "WARNING", $
                           TEXT = text_warn, WIDTH = 50, HEIGHT = 5

		ENDIF 

		  inbrange = [optpow(expind-1),optpow(expind)]
		  expnorm = (fcscho - exprange(0)) / $
				(exprange(1) - exprange(0))  		
		
                  exp_in_band = 10^(INTERPOLATE(alog10(inbrange),expnorm))  

        ENDIF ELSE $
	   	exp_in_band = optpow(expind(0)) 

	theo_in_band = exp_in_band * apert
	
; Divide by illumination matrix to correct for non-flat FCS illumination
; be careful with the dimensions: use loop over pixels 
	oldav = avch
 	oldme = mech
	fcsi1dim = fcsill(*,fcspos-1)
	fcsu1dim = fcsilun(*,fcspos-1)


        FOR i=0, dim1-1 DO BEGIN
; Flags are (0 or 2) (in case of "good" signals present) or (1 or 2) (in case of
; just some "not-bad" signals present) 

	  indacc = WHERE(flag(i,*) LT 2, n)
	  IF n GT 0 THEN BEGIN
	    avch(i,indacc) = avch(i,indacc) / fcsi1dim(i) / flatf(i)	   
            avcu(i,indacc) = avch(i,indacc) * $
			    sqrt ( (avcu(i,indacc)   / oldav(i,indacc)) ^2 +$
				      (  fcsu1dim(i) / fcsi1dim(i) ) ^2 + $
				      ( flatu(i) / flatf(i) ) ^2 )

	    mech(i,indacc) = mech(i,indacc) / fcsi1dim(i) / flatf(i) 	   
            mecu(i,indacc) = mech(i,indacc) * $
			    sqrt ( (mecu(i,indacc)  / oldme(i,indacc)) ^2 +$
					(fcsu1dim(i)  / fcsi1dim(i) ) ^2 + $
				      ( flatu(i) / flatf(i) ) ^2 )
; Calculate responses


	    respon_av(i,indacc) = avch(i,indacc) / theo_in_band  
	    respon_me(i,indacc) = mech(i,indacc) / theo_in_band  
	    respon_au(i,indacc) = avcu(i,indacc) / theo_in_band 
	    respon_mu(i,indacc) = mecu(i,indacc) / theo_in_band
          ENDIF
	ENDFOR	

JUMP_CALC: 

; Define the widgets

        respbase = WIDGET_BASE(TITLE = 'PIA Absolute Photometry Response Menu', $
                                /COLUMN, /MODAL, group=group)

	junklabel= WIDGET_LABEL(respbase, VALUE=labtxt)


 	id_0 = WIDGET_BASE(respbase, /ROW )

	id_1 = WIDGET_BASE(id_0, /COLUMN )
	text = 'Default Responses '
	junklabel= WIDGET_LABEL(id_1, VALUE=text)
	strresp = STRTRIM(STRING(thresp,format='(f7.3)'),2)+'  +/- '+$
		   STRTRIM(STRING(thresu,format='(f7.3)'),2)
        junklist = WIDGET_TEXT(id_1, VALUE=strresp, YSIZE=txtsize)

	id_1 = WIDGET_BASE(id_0, /COLUMN )
	text = 'Used FCS illumination matrix - FCS '+strtrim(string(FCSPOS),2)
	junklabel= WIDGET_LABEL(id_1, VALUE=text)

        junklist = WIDGET_TEXT(id_1, $
			VALUE=STRTRIM(string(fcsill(*,fcspos-1)),2), $
			YSIZE=txtsize, XSIZE=8, no_newline=0)

 	id_0 = WIDGET_BASE(respbase, /ROW )

 	id_1 = WIDGET_BASE( id_0, /COLUMN )
 	id_2 = WIDGET_LABEL( id_1, VALUE='Responses using Averages:' )

	pixavcon = WIDGET_BUTTON(id_1, VALUE=' ',MENU=2, UVALUE=0,$
		EVENT_PRO='viewabsres')

	avgdraw = WIDGET_DRAW(id_1, xsize=256, ysize=256)
	
	FOR I=0,dim1-1 DO BEGIN
		viewname = 'Response Averages Pixel '+strtrim(string(i+1),2) 
		dummy = WIDGET_BUTTON(pixavcon, value=viewname, uvalue=i)
	ENDFOR 		

	text = ' Accept and exit' 
	accave = WIDGET_BUTTON(id_1, VALUE=text, UVALUE='ACCAV') 
	 
 	id_1 = WIDGET_BASE( id_0, /COLUMN )
	id_2 = WIDGET_LABEL( id_1, VALUE='Responses using Medians:' )

	pixmecon = WIDGET_BUTTON(id_1, VALUE=' ',MENU=2, UVALUE=1,$
		EVENT_PRO='viewabsres')

	meddraw = WIDGET_DRAW(id_1, xsize=256, ysize=256)

	FOR I=0,dim1-1 DO BEGIN
		viewname = 'Response Medians Pixel '+strtrim(string(i+1),2) 
		dummy = WIDGET_BUTTON(pixmecon, value=viewname, uvalue=i)
	ENDFOR 		
		
	text = ' Accept and exit ' 
	accave = WIDGET_BUTTON(id_1, VALUE=text, UVALUE='ACCME') 

	id_1 = WIDGET_BUTTON(respbase, VALUE='Cancel', UVALUE='QUIT')

; Realize widget

	WIDGET_CONTROL, respbase, /REALIZE, TLB_SET_XOFFSET = 600

	WIDGET_CONTROL, avgdraw, GET_VALUE=avgdraw
        resetview_abs, 0, 0
	WIDGET_CONTROL, pixavcon, SET_VALUE='Response Averages Pixel 1'
	WIDGET_CONTROL, meddraw, GET_VALUE=meddraw
        resetview_abs, 1, 0
	WIDGET_CONTROL, pixmecon, SET_VALUE='Response Medians Pixel 1'

	Xmanager,No_BLOCK=1, "pia_respons_abs", respbase, GROUP=group

	RETURN
END
