;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_RX2Y
;
; PURPOSE:
;	Calculate map Y-coordinates for given right ascension and given map
;	X-coordinate values. Arrays can be used in two forms:
;	(1) a set of X pixel values, and a scalar right ascension, or
;	(2) a set of right ascension values, and a scalar X value.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	y = PIA_RX2Y( ra, x, trans, dec)
;
; INPUTS:
;	RA		Right ascension value(s) in degrees.
;			If X is an array, then RA must be a scalar.
;
;	X		Map X-coordinate value(s).
;			If RA is an array, then X must be a scalar.
;
;	TRANS		Transformation parameters as provided by PIA_MKTRANS.
;
; KEYWORD PARAMETERS:
;	(none)
;
; OUTPUTS:
;	Y		Map Y-coordinate value(s).
;
; OPTIONAL OUTPUTS:
;	DEC		Corresponding declination value(s) in degrees.
;
; COMMON BLOCKS:
;	(none)
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		pia_rd2xy
;
; RESTRICTIONS:
;	Program will have difficulty converging for declination values near 90.
;
; NOTES:
;	The transformation structure TRANS is not compatible with the
;	ASTR structure of ASTROLIB routines!
;
; MODIFICATION HISTORY:
;	Written by:	D. Skaley, MPIK, Heidelberg	March 1995
;			(Adapted from ASTROLIB routine CONS_RA)
;	Oct 95 Version 3.5
;-

FUNCTION PIA_RX2Y, ra, x, trans, dec

	ON_ERROR,2

	dtor = !DPI/180.0D0

	crval = trans.crval * dtor
	delta_x = x - trans.crpix(0)
	delta_ra = ra*dtor - crval(0)
	cos_ra = cos( delta_ra )
	sin_ra = sin( delta_ra )
	cos_d0 = cos( crval(1) )
	sin_d0 = sin( crval(1) )

 	dec = ATAN( sin_d0*cos_ra*trans.dc(0,1) - sin_ra*trans.dc(0,0) $
			+ delta_x*cos_ra*cos_d0, $
			cos_d0*trans.dc(0,1) - delta_x*sin_d0 )
	dec = dec / dtor

	PIA_RD2XY, ra, dec, trans, xdummy, y

	RETURN, y
	END
