;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
;
;  NAME:
;	PIA_source_search
;
;  PURPOSE:
;	Allows the user to interactively find positions and fluxes of stars
; 	in a field and to alter the parameters for searching
;
;  CATEGORY:
;	PIA - processing
;
;  INPUTS:
;	image  : m x n image data
;	trans  : coordinates transformation parameters
;
;  OPTIONAL INPUTS:
;	measname: map measurement name 
;
;  OUTPUTS:
;	x: vector with x-positions of stars
;	y: vector with y-positions of stars
;	flux: vector with estimated fluxes of stars
;
;  COMMON BLOCKS:
;	find_param
;
;  PROCEDURES USED:
;	PIA Procedure(s):
;		acknowledge
;		find_param_init
;		pia_disp_print_file
;		pia_xy2rd
;	Astronomical Library:
;		radec
;	Internal:
;		PIA_source_search_Event
;
;  RESTRICTIONS:
;	none
;
;  Modification History:
;	Written by:	I. Heinrichsen, MPIK, Heidelberg	July 1995	
;	Modified by:	Carlos GABRIEL, ESA/VILSPA		July 1995
;	Oct 95 Version 3.5
;	Pixel index bug fixed	(IH)				October 1995
;-

;;;;;;;   	Event handler for PIA_source_search	;;;;;;;;

PRO PIA_source_search_Event, event


@find_param
; initialize debug, =1 print-out
	debug=0
; Get the information from the widget that caused the event
	Widget_Control, event.id, Get_Value=Value, Get_UValue=UValue

; The user value of the base widget points to another base widget, where
; all the necessary information is stored.
	Degparbase = WIDGET_INFO(event.id, /parent)
	Widget_Control, Degparbase, Get_UValue=realbase
	Widget_Control, realbase, Get_UValue=info

	if (debug EQ 1) then print, Uvalue

	CASE UValue OF

; Threshold for Source
	  'HMIN': BEGIN
	    IF (Value LT 0.0) THEN BEGIN
	      Acknowledge, Text='Threshold must be positive (3-4 sigma > bgr)'
	      Widget_Control, event.id, Set_Value=info.Hmin
	    ENDIF ELSE BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      hmin=Value
       	    ENDELSE
       	    END

; FWHM of Point Source
	  'FWHM': BEGIN
	    IF (Value LT 0.0) THEN BEGIN
	      Acknowledge, Text='   FWHM must be positive   '
	      Widget_Control, event.id, Set_Value=info.Fwhm
	    ENDIF ELSE BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      fwhm=Value
	    ENDELSE
	   END

; Sharpness Statistic lower limit
	  'SHARPLINL': BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      sharplin(0)=Value
	  END

; Sharpness Statistic upper limit
	  'SHARPLINU': BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      sharplin(1)=Value  
	  END
	 
; Roundness Statistic lower limit
	  'ROUNDLINL': BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      roundlin(0)=Value
	  END

; Roundness Statistic upper limit
	  'ROUNDLINU': BEGIN
	      Widget_Control, event.id, Set_Value=Value
	      roundlin(1)=Value
	  END

; Exit
	  'EXIT': BEGIN
	    Widget_Control, realbase, Set_UValue=info 
	    Widget_Control, event.top, /destroy
	  END

; Replace all the widget values with the values they held on entering this
; procedure 
          'ORIG': BEGIN
	    info.Hmin = info.HminDef
	    info.Fwhm = info.FwhmDef
	    info.Sharplin = info.SharplinDef
	    info.Roundlin = info.RoundlinDef
	    Widget_Control, info.HminID, Set_Value=info.HminDef
	    Widget_Control, info.FwhmID, Set_Value=info.FwhmDef
	    Widget_Control, info.SharplinlID, Set_Value=info.SharplinDef(0)
	    Widget_Control, info.SharplinuID, Set_Value=info.SharplinDef(1)
	    Widget_Control, info.RoundlinlID, Set_Value=info.RoundlinDef(0)
	    Widget_Control, info.RoundlinuID, Set_Value=info.RoundlinDef(1)
	    Widget_Control, realbase, Set_UValue=info
	  END
;
; Perform the find stars routine and display the results in a seperate text 
; window
;
	  'FIND': BEGIN
	    Widget_Control, info.HminID, Get_Value=hmin
	    Widget_Control, info.FwhmID, Get_Value=Fwhm
	    Widget_Control, info.SharplinlID, Get_Value=sharptmp
	    Sharplin(0) = sharptmp
	    Widget_Control, info.SharplinuID, Get_Value=sharptmp
	    Sharplin(1) = sharptmp
	    Widget_Control, info.RoundlinlID, Get_Value=roundtmp
	    Roundlin(0) = roundtmp
	    Widget_Control, info.RoundlinuID, Get_Value=roundtmp
	    Roundlin(1) = roundtmp
	    

	  image=info.image
	if (debug EQ 1) then begin 
		print, 'image'
		print, image
		print, 'hmin, fwhm, roundlin, sharplin'
		print, hmin, fwhm, roundlin, sharplin
	endif
; find the stars
  	find, image, x, y, flux, sharp, roundval, hmin, fwhm, roundlin, $
		  sharplin, textout=1, /SILENT
; correct coordinates for additional rows and collumns
	IF N_ELEMENTS(x) GT 0 THEN BEGIN
	   x=x-2.0
	   y=y-2.0
	   if (debug eq 1) then begin
		print, 'find results'
		print, 'X: ',x
		print, 'Y: ',y
		print, 'Flux: ',flux
		print, 'Sharpness: ',sharp
		print, 'Roundness:',roundval
	   endif


; convert coordinates into DEC and RA

	PIA_XY2RD, x-1., y-1., info.trans, ra, dec
;	RADEC, ra, dec, rh, rm, rs, dd, dm, ds
	rh = INTARR(N_ELEMENTS(dec))
	rm = rh
	rs = FLTARR(N_ELEMENTS(dec))
	dd = rh
	dm = rh
	ds = rs
	sra = STRARR(N_ELEMENTS(dec))
	sdec = sra
	
	FOR idec=0,N_ELEMENTS(dec)-1 DO BEGIN
	  RADEC, ra(idec), dec(idec), tmprh, tmprm, tmprs, tmpdd, tmpdm, tmpds
	  IF dec(idec) LT 0 THEN BEGIN
			dd(idec) = -ABS(tmpdd)
			dm(idec) = ABS(tmpdm)
			ds(idec) = ABS(tmpds)
	  ENDIF ELSE BEGIN
			dd(idec) = tmpdd
			dm(idec) = tmpdm
			ds(idec) = tmpds
	  ENDELSE
	  rh(idec) = tmprh
	  rm(idec) = tmprm
	  rs(idec) = tmprs
	  sra(idec) = STRING(rh(idec),rm(idec),rs(idec),$
	  		FORMAT="(I2,'h ',I2,'m ',F5.2,'s')")
	  sdec(idec) = STRING(dd(idec),dm(idec),ds(idec),$
	  		FORMAT="(I3,'d ',I2,'m ',F5.2,'s')")
	ENDFOR
;	sra = STRING(rh,rm,rs,FORMAT="(I2,'h ',I2,'m ',F5.2,'s')")
;	sdec = STRING(dd,dm,ds,FORMAT="(I3,'d ',I2,'m ',F5.2,'s')")

; write results into text
   	   text=strarr(1)
	   text=[text, ' ' ]
	   text = [text, 'Point sources found on ' + info.measid ]
	   text = [text, $
	     '============================================================= ']
	   text = [text,  '']
	   text = [text, 'Parameters used:  '] 
	   text = [text, '  source threshold  = '+ STRTRIM(STRING(hmin),2)]
	   text = [text, '  point source FWHM = ' + STRTRIM(STRING(fwhm),2)]
	   text = [text, '']
	   tmp = '_________________________________________________'
	   tmp = tmp + '_____________________________________________________'  
	   text=[text, tmp]
	   tmp='       |        |                |                  |                  |'
	   tmp = tmp + '               |               ' 
	   text=[text, tmp]
	   tmp='   X   |    Y   |       RA       |       DEC        |  Estimated Flux  |'
	   tmp = tmp + '  Sharpness    |   Roundness   '
	   text=[text, tmp]
	   tmp='       |        |                |                  |                  |'
	   tmp = tmp + '               |               '
	   text=[text, tmp]
	   for i=0,n_elements(y)-1 do begin
		text=[text, string(x(i),FORMAT="(F6.2)") + $
			' | ' + string(y(i),FORMAT="(F6.2)") + $
			' | ' + sra(i) + ' | ' + sdec(i) + $
			'  |'+string(flux(i))+'     |'+strtrim(string(sharp(i))) + $
			'  |'+string(roundval(i))]

	   endfor 
	   tmp='       |        |                |                  |                  |'
	   tmp = tmp + '               |               '
	   text=[text, tmp]
	   tmp = '_________________________________________________'
	   tmp = tmp + '_____________________________________________________'  
	   text=[text, tmp]

	;
	; Print FIND results into text window
	;
        	PIA_disp_print_file, "", $
                  TITLE = "Point sources search results", $
                  WIDTH = 120, $
                  HEIGHT = 20, $
                  TEXT = [text]
	ENDIF ELSE BEGIN 
        	XDISPLAYFILE, "", $
                  TITLE = "Negative point sources search", $
                  WIDTH = 60, $
                  HEIGHT = 5, $
                  TEXT = ["No sources found with given parameters",$
                  	  " Play it again, Sam  " ]

	ENDELSE	   
	   
 	END

	ELSE:
	
	ENDCASE
	RETURN
	END

;-----------------------------------------------------------------------

	PRO PIA_source_search, imageinp, trans, x, y, flux, $
			measname=measname

@find_param
 
; measurement name
	IF KEYWORD_SET(measname) THEN measid = measname ELSE measid = " "

; initialize error status
	error=1.0
; initalize debug parameter =1 for print-out
	debug=0

; initialize default find parameters
	image=imageinp
	find_param_init
	x=fltarr(1)
	y=fltarr(1)
	flux=fltarr(1)

; set hmin threshold to 1.5 times the median of the image, as estimate for
; background 
	med=median(image)
	hmin= 1.5*med

; add 3 additional rows and colomns on both sides to the image and
; fill them with median of image in order to avoid problems with 
; too small images
	type=size(image)

; check dimensions of image
	if ( type(0) NE 2 ) then goto, dimerror
 	n_x  = type(1) 
 	n_y = type(2)
 	if (debug eq 1) then print, 'Image Dimensions:', n_x, n_y
	if ((n_x lt 5) or (n_y lt 5)) then $
		message, /INFO, 'Warning: Small image dimenesions'
	
 	imnew=fltarr(n_x+6,n_y+6)
 	imnew(*,*)=med
 	imnew(3:n_x+2,3:n_y+2)=image(0:n_x-1,0:n_y-1)
 	image=imnew

;
; This is the base widget that will contain the structure passed back from
; the event handler. It will remain unrealized
;
	VBase = Widget_Base()	

;
; This is the apparent base widget. The User Value will point to the
; real base widget. In this way, destroying all the realized widgets does
; not delete the data.
;
	DegparBase = Widget_Base(Title='Source Search Parameters', $
	  Event_Pro='Deglitch_Change_Event', Column=1)


	font='-*-courier-bold-*-*-*-14-*'
;
; A widget that holds hmin: the theshold for source detection
; 			    should be about 3-4 sigma over bgr.
	hmin_id = CW_Field(DegparBase, font=font, $
		Title='Source Threshold:              ', $
		Value=hmin, $
	  UValue='HMIN', /return_events, /floating, xsize=14)
;
; A widget that holds fwhm: the full width half minimum for a point source.
;
	fwhm_id = CW_Field(DegparBase, font=font,$
		Title='Full Width Half Minimum of Point Source: ', $
	  Value=fwhm, UValue='FWHM', /return_events, /floating, xsize=7)
;
; A widget that holds sharplin: Parameteres fo Sharpness Statistics
;				lower limit
;
	sharplinl_id = CW_Field(DegparBase, font=font, $
		Title='Sharpness Statistics, lower limit:       ', $
	  Value=sharplin(0), UValue='SHARPLINL', /return_events, $
	  /floating, xsize=7)
	  
;
; A widget that holds sharplin: Parameteres fo Sharpness Statistics
;				upper limit
;
	sharplinu_id = CW_Field(DegparBase, font=font,$
		Title='Sharpness Statistics, upper limit:       ', $
	  Value=sharplin(1), UValue='SHARPLINU', /return_events, $
	  /floating, xsize=7)
;
; A widget that holds roundlin: Parameteres fo Roundness Statistics
;				lower limit
;
	roundlinl_id = CW_Field(DegparBase, font=font,$
		Title='Roundness Statistics, lower limit:       ', $
	  Value=roundlin(0), UValue='ROUNDLINL', /return_events, $
	  /floating, xsize=7)
;
; A widget that holds roundlin: Parameteres fo Roundness Statistics
;				upper limit
;
	roundlinu_id = CW_Field(DegparBase, font=font,$
		Title='Roundness Statistics, upper limit:       ', $
	  Value=roundlin(1), UValue='ROUNDLINU', /return_events, $
	  /floating, xsize=7)
;
; A button widget to perform the find with the current parameters
; and to display the results in a text widget
	Draw = Widget_Button(DegparBase, Value='Find Stars', $
	  UValue='FIND')

; A button widget to restore the find parameters to the values
; they had on entry to the event handler
	Orig = Widget_Button(DegparBase, Value='Get original parameters', $
	  UValue='ORIG')

; A button widget to accept the current values of the deglitching
; parameters ,and exit
	Exit = Widget_Button(DegparBase, Value='Exit', UValue='EXIT')
;
; Create an anonymous structure containing all the information the event
; handler might need.
; That is, the source search input/output parameters, the common
; block parameters, and default values for the common block parameters
;
	Info = {Image:image, X:x, Y:y, Flux:flux,$
		Hmin:hmin, Fwhm:fwhm,$
		Sharplin:sharplin, Roundlin:roundlin, $
		HminDef:hmin, FwhmDef:fwhm,$
		SharplinDef:sharplin, RoundlinDef:roundlin, $
		HminID:hmin_id, FwhmID:fwhm_id,$
		SharplinlID:sharplinl_id, RoundlinlID:roundlinl_id, $
		SharplinuID:sharplinu_id, RoundlinuID:roundlinu_id, $
		Error:error, Trans:trans, Measid:measid}

; Use the base widget to point to this structure
	Widget_Control, vbase, Set_UValue=info

; Use the realized base to point to the base widget and realize it
	Widget_Control, Degparbase, Set_UValue=VBase
	Widget_Control, Degparbase, /realize

	xMANAGER,No_BLOCK=1, 'PIA_source_search', DegparBase

; Retrive the information from the base widget
; These common variables have not necessarily been set in the event handler,
; and thus must be set here, just to make sure.
	Widget_Control, VBase, Get_UValue=info
	Hmin = info.Hmin
	Fwhm = info.fwhm
	Sharplin = info.Sharplin
	Roundlin = info.Roundlin

;	Accept = info.Accept
;	
; Destroy the base widget - not possible, problems when running from other 
; widgets (CG)
;	Widget_Control, VBase, /destroy
;
;	all OK
	error=0
	return
dimerror:
	 MESSAGE, /INFO, 'Image array must be 2 dimensional'
	 return
 	END
