;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_SPECT_AAR
;
; PURPOSE:
;	Plot PHT-S-Spectra of flux density/surface brightness per chopper level.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_spect_aar, tmpdata, step, mulch, rpid_ch
;
; INPUTS:
;	tmpdata: A structure (AAP type) containing data to be displayed
;	step: Chopper step to be plotted
;	mulch: Print chopper step flag (0: no print, 1: print)
;	rpid_ch: The raster position
;	
; KEYWORD PARAMETERS:
;	overplot
;		to overplot on active PIA_xplot window
;	mjy	
;		to plot brightness surfaces instead of fluxes present in tmpdata
;
; OUTPUTS:
;	none
;	
; RESTRICTIONS:
;	using keyword overplot requires existing plot
;	
; METHOD:
;	average all flux densities/surface brightness of a chopper step per 
;	pixel and plot (PIA_XPLOT) the results versus the pixel central 
;	wavelengths
;
; COMMON BLOCKS:
;	phtslwavlen
;	colour_indices
;	pia_dspchz_cmn
;	
; PROCEDURES USED:
;	PIA Procedure(s):
;		spd_disc
;		decode_admn
;		pia_xplot
;		set_slwavelen
;		weight_mean
;
; MODIFICATION HISTORY:
;	Written by:	C. Gabriel (ESA/VILSPA-SAI)		April 1996
;	Overplot symbol set back if = 8 (CG)			May 1996
;	Extension for combining RPIDs (CG)			December 1997
;	V6.6
;	Use of coded bitflag system (CG)			May 1999
;	V7.4 
;-

PRO pia_spect_aar, tmpdata, step, mulch, rpid_ch, overplot=overplot, MJy=MJy 

; S-wavelengths common
@phtslwavlen
@colour_indices
COMMON pia_dspchz_cmn, zoom_psymch, zoom_linech, zoom_color

	IF KEYWORD_SET(overplot) THEN overplot=overplot ELSE overplot=0
	IF N_ELEMENTS(zoom_color) EQ 0 THEN zoom_color = -1
	IF N_ELEMENTS(zoom_psymch) EQ 0 THEN zoom_psymch = 4

        IF overplot NE 0 THEN BEGIN
                IF zoom_color EQ -1 THEN zoom_color = green ELSE $
                        zoom_color = zoom_color + 1
                IF zoom_color EQ 9 then zoom_color = 4
                zoom_psymch = zoom_psymch + 1
                IF zoom_psymch EQ 8 then zoom_psymch = 1
        ENDIF ELSE BEGIN
                zoom_color = -1
		zoom_psymch = 4
        ENDELSE
 
; SS or SL?
	det = strmid(tmpdata.det,0,2)

	IF strmid(det,0,1) ne 'S' THEN BEGIN
		PRINT, tmpdata.det
		MESSAGE, /INFO, ' Only PHT-S Data accepted'
		RETURN
	ENDIF

	IF N_ELEMENTS(slwavelen) EQ 0 THEN set_slwavelen
	
	IF det EQ 'SS' THEN 	lambda = slwavelen(0:63)     ELSE $
				lambda = slwavelen(64:127) 

	IF KEYWORD_SET(Mjy) THEN ytitle = ' Surface Brightness      [MJy/sr]' $
			    ELSE ytitle = ' Flux Density      		[Jy]'
	decode_admn, tmpdata.admn, title 
 
	xtitle = ' !4k 	   [l!3m]'
	subtitle = ''
	charsize = 1.3	

	x = findgen(64)
	y = fltarr(64)
	yerr = fltarr(64)


	IF mulch EQ 1 THEN $
	     subtitle = 'Spectrum Chop St '+strtrim(string(step),2) ELSE $
	     subtitle = 'Spectrum Staring '


	rpid_copy = tmpdata.rpid
	add0=''	
	add1=''
	
	ch0 = WHERE(tmpdata.rpid(0,*) EQ rpid_ch(0,0) )
	   		
	IF N_ELEMENTS(rpid_ch(0,*)) GT 1 THEN $
	 FOR I=1,N_ELEMENTS(rpid_ch(0,*))-1 DO BEGIN
	   IF rpid_ch(0,i) NE rpid_ch(0,0) THEN BEGIN
	    nch0 = WHERE(tmpdata.rpid(0,*) EQ rpid_ch(0,i) )
	    ch0 = [ch0,nch0]
	    add0=add0+'+'+STRTRIM(rpid_ch(0,i),2)
	   ENDIF
	 ENDFOR  
	 
	ch1 = WHERE(tmpdata.rpid(1,*) EQ rpid_ch(1,0))
	IF N_ELEMENTS(rpid_ch(1,*)) GT 1 THEN $
	 FOR I=1,N_ELEMENTS(rpid_ch(1,*))-1 DO BEGIN
	   IF rpid_ch(1,i) NE rpid_ch(1,0) THEN BEGIN
	    nch1 = WHERE(tmpdata.rpid(1,*) EQ rpid_ch(1,i))
	    add1=add1+'+'+STRTRIM(rpid_ch(1,i),2)
	    ch1 = [ch1,nch1]
	   ENDIF
	 ENDFOR  

	annm = '  RPid [' + $
		STRTRIM(string(FIX(rpid_ch(0,0))),2)+add0+','+$
		STRTRIM(string(FIX(rpid_ch(1,0))),2)+add1+']'

	IF N_ELEMENTS(rpid_ch(0,*)) GT 1 THEN add00='...' ELSE add00=''
	IF N_ELEMENTS(rpid_ch(1,*)) GT 1 THEN add01='...' ELSE add01=''
	subtitle = subtitle + '  RPid [' + $
		STRTRIM(string(FIX(rpid_ch(0,0))),2)+add00+','+$
		STRTRIM(string(FIX(rpid_ch(1,0))),2)+add01+']'
	
	rpid_copy(0,ch0)=-1b
	rpid_copy(1,ch1)=-1b
	        
	ind = WHERE(tmpdata.cstp EQ step AND $
		    rpid_copy(0,*) EQ -1b AND $
		    rpid_copy(1,*) EQ -1b, n)

; if one chopper plateau on step, selection is not possible
	IF n EQ 1 THEN BEGIN
		IF KEYWORD_SET(MJy) THEN BEGIN
		  y = tmpdata.mnbr(*,ind) 
		  yerr = tmpdata.mnbu(*,ind)
		ENDIF ELSE BEGIN
		  y = tmpdata.mnfl(*,ind) 
		  yerr = tmpdata.mnfu(*,ind)
		ENDELSE

;		mess = 'Warning -> just one chopper plateau on step'
;		mess = mess+'- No selection possible'
;		Message, /INFO, mess

	ENDIF ELSE BEGIN

; if several chopper plateaus on step, look for flags
; Call spd_disc to flag all discarded chopper plateaux


		get_validflag, tmpdata.flag, 'AAP', redflag, det=tmpdata.det
		flagarr = redflag

		IF KEYWORD_SET(MJy) THEN BEGIN
			mnpw = tmpdata.mnbr
			mnpu = tmpdata.mnbu
		ENDIF ELSE BEGIN
			mnpw = tmpdata.mnfl
			mnpu = tmpdata.mnfu
		ENDELSE
		
	   	FOR i=0,63 DO BEGIN
		   indacc = WHERE(flagarr(i,ind) EQ 0, n)
		   IF n GT 1 THEN BEGIN
			weight_mean, mnpw(i,ind(indacc)), $
				mnpu(i,ind(indacc)), yact, yerract
			y(i) = yact
			yerr(i) = yerract
		   ENDIF ELSE IF n EQ 1 THEN BEGIN
			y(i) = mnpw(i,ind(indacc)) 
			yerr(i) = mnpu(i,ind(indacc))
			mess = 'Warning -> just one chopper plateaux accepted: '
			mess = mess+'Pixel'+string(i+1) 
			MESSAGE, /INFO, mess 
		  ENDIF ELSE BEGIN
			mess = 'Warning -> no chopper plateau accepted: '
			mess = mess+'Pixel'+string(i+1) 
			MESSAGE, /INFO, mess 
		  ENDELSE
        	ENDFOR
	   ENDELSE


	anntmp = title + '  ' + ytitle + '-' + annm

	IF overplot GE 0 THEN $
	PIA_xplot, lambda, y, yerr , title=title, xtitle=xtitle, $
		ytitle=ytitle, overplot=overplot, $
		ycharsize=1., $
		psym=zoom_psymch, symsize=1.2, color=zoom_color, $
		charsize = charsize, legend = subtitle,  $
		annotation = anntmp $
	ELSE $  
	PIA_xplot, lambda, y, yerr , xtitle=xtitle, $
		ytitle=ytitle, overplot=overplot, $
		ycharsize=1., $
		psym=zoom_psymch, symsize=1.2, color=zoom_color, $
		legend = subtitle, annotation = anntmp 

        RETURN
END

