;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_XPLOT
;
; PURPOSE:
;       This routine provides a graphical user interface to PLOT
;       and OPLOT routines. Overplotting is possible, also the
;       plotting range can be changed interactively.
;
; CATEGORY:
;       PIA - graphics
;
; CALLING SEQUENCE:
;       PIA_XPLOT, [X,] Y [, YERR]
;
; INPUTS:
;       Y:        The ordinate data to be plotted.
;
; OPTIONAL INPUTS:
;       X:        A vector argument. If X is not specified, Y is
;                 plotted as a function of point number (starting at
;                 zero). If both arguments are provided, Y is plotted
;                 as a function of X.
;
;       YERR:     The uncertainties of Y.
;
; OUTPUTS:
;       None.
;
; KEYWORDS:
;       OVERPLOT: Set this keyword to plot over a previously performed
;                 plot. If this keyword is not set or a value of 0 is
;                 supplied a new plot window is opened. A value of -1
;                 draws an additional Y-axis for the new ordinate data.
;                 A value of n greater than 0 uses the n-th previously
;                 drawn Y-axis for scaling the data. If n is larger
;                 than the number of existing Y-axes the last drawn
;                 Y-axis is used.
;
;       FOURIER:  Set this keyword to Fourier-transform the data
;                 before plotting
;
;	DISC_AREA: If this keyword is set, then a selection of an area
;		  can be graphically done. The widget is in this case
;		  of type modal (to avoid problems with overplotting).
;		  The selected region is handled back via the common block
;		  DISC_AREA
;  
;       GROUP:    The group leader of the widget.
;
;       ANNOTATION:
;                The value of type string or string array is printed
;                to the output page. Annotation strings of succeeding
;                calls of PIA_XPLOT will be concatenated.
;
;       LEGEND:  A string describing the data in a legend.
;
;	YERRSTYLE: Linestyle for error bars (default=0)
;
; Graphics Keywords Accepted:
;
;       CHARSIZE COLOR LINESTYLE
;       PSYM SUBTITLE SYMSIZE THICK TITLE
;       XCHARSIZE XSTYLE XTHICK XTITLE XTYPE
;       YCHARSIZE YSTYLE YTHICK YTITLE YTYPE
;
;       See Chapter 2, Graphics Keywords and System Variables,
;       for the description of graphics and plotting keywords.
;
; CALLED ROUTINES:
;       PIA_PRTDEF
;
; COMMON BLOCKS:
;       PIA_XPLOT
;       PIA_PRINT
;	DISC_AREA
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
; PROCEDURE:
;       Common Block(s) or @ procedure(s):
;               colour_indices
;               pia_xplot_bmd
;       PIA Procedure(s):
;               pia_prtdef
;
; MODIFICATION HISTORY:
;	Written by: C Gabriel (adapted from Stefan Bogun)
;	Modified by: Detlef Skaley
;	Oct 95 Version 3.5
;	Hypertext Help added, CG		October 1995
;	Zooming now exact working; FFT only on first 30000 points allowed (CG)
;						December 1995
;	Problem with legend (when psym=10) fixed (CG)
;						December 1995
;	Loop variable now of type LONG (CG)	Christmas 1995	
;	zoomfl now passed over vpar (CG)	February 1996
;	Area selection capability added + 
;	 better zooming +
;	 print values capability  (CG) 		May 1997
;	V6.3
;	Yranges can be individually set + bug in print
;	 values (wrong order) fixed (CG)	July 1997
;	V6.4
;	Allowing for selection of several areas (CG)	September 1997
;	V6.5
;	Number of elements for plotting can be
;	 defined (!PIA_XPLOT_LIMIT system var.) (CG)	January 1998
;	V7.0
;	Xranges can be individually set  (CG)		March 1998
;	Bug when zooming data with one point by x=0 
;	 fixed (CG)					May 1998
;	V7.1
;	Problem with extension of empty space by update
;	(IDL bug!) fixed by taking frames out (CG)	July 1998
;	V7.2
;	Changing according to IDL5.0 modal keyword use
;	(forced by IDL5.1)				January 1999
;	V7.3
;	Better handling of x-y axis keywords (CG)	March 1999
;	V7.4
;	Force exact range and extended range NOT exclusive
;	 by x-y axis keywords (CG)			January 2000
;	Yerrstyle keyword added (CG)			February 2000
;	V8.2
;
;-

;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_ANNPLT, id, PRINTSTYLE=printstyle

; plot the annotation to printer output

        IF N_ELEMENTS(printstyle) EQ 0 THEN printstyle=0

        ; top left corner
        CASE printstyle OF
        1:    origin = [ 0, 0.3 ]     ; printer/portrait
        2:    origin = [ 0.3, 0.4 ]  ; printer/landscape
        ELSE: MESSAGE,'there is something wrong!'
        ENDCASE
        origin = LONG((CONVERT_COORD( origin, /NORMAL, /TO_DEVICE ))(0:1))

        charsize = 0.7

        x_pos    = origin(0) + 0L
        y_decr   = LONG((!d.y_ch_size)*charsize*1.7)
        y_start  = origin(1)

        ; headline
        y_pos = y_start - 2*y_decr
        XYOUTS, x_pos, y_pos, /DEVICE, 'Annotation:', WIDTH=width, $
		charsize=charsize
        PLOTS, /DEVICE, x_pos+[0,width*!d.x_size], $
              LONG(y_pos*[1,1]-!d.y_ch_size*charsize*0.3)

        HANDLE_VALUE, id.annotation, annotation, /NO_COPY
        n_annotation = N_ELEMENTS(annotation)
        IF n_annotation EQ 0 THEN BEGIN
           HANDLE_VALUE, id.annotation, annotation, /SET, /NO_COPY
           RETURN
        ENDIF

        FOR i = 0, n_annotation-1 DO BEGIN
           y_pos = y_pos - y_decr
           XYOUTS, x_pos, y_pos, annotation(i), /DEVICE, charsize=charsize
        ENDFOR
        HANDLE_VALUE, id.annotation, annotation, /SET, /NO_COPY

        RETURN
        END

;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_LEGPLT, id, PRINTSTYLE=printstyle

   	 COMMON pia_print, prt
; plot the legend

        IF N_ELEMENTS(printstyle) EQ 0 THEN printstyle=0

        ; top left corner
        CASE printstyle OF
        0: origin = [ 0, 1 ]        ; screen
        1: origin = [ 0, 0.5 ]      ; printer/portrait
        2: origin = [ 0, 0.4 ]      ; printer/landscape
        ELSE: MESSAGE,'there is something wrong!'
        ENDCASE
        origin = LONG((CONVERT_COORD( origin, /NORMAL, /TO_DEVICE ))(0:1))

        IF printstyle EQ 0 THEN BEGIN
           charsize = 1.3
        ENDIF ELSE BEGIN
           charsize = 0.7
        ENDELSE

        x_pos    = origin(0) + LONG([ 4, 12, 22, 25 ] * !d.x_ch_size*charsize)
        x_mid    = (x_pos(1)+x_pos(2))/2
        y_decr   = LONG((!d.y_ch_size)*charsize*1.7)
        y_start  = origin(1) - y_decr

        ; headlines

        y_pos = y_start
        XYOUTS, x_pos(0), y_pos, /DEVICE, 'Legend:', $
           CHARSIZE=charsize, WIDTH=width
        PLOTS, /DEVICE, x_pos(0)+[0,width*!d.x_size], $
           LONG(y_pos*[1,1]-!d.y_ch_size*charsize*0.3)

        y_pos = y_pos - y_decr
        XYOUTS, x_pos(0), y_pos, /DEVICE, 'Axis:', CHARSIZE=charsize
        XYOUTS, x_pos(1), y_pos, /DEVICE, 'Curve:', CHARSIZE=charsize
        XYOUTS, x_pos(3), y_pos, /DEVICE, 'Description:', CHARSIZE=charsize

        ; loop through curves
        data_id = HANDLE_INFO( id.data, /FIRST_CHILD )
        WHILE data_id NE 0 DO BEGIN
           y_pos   = y_pos - y_decr
           HANDLE_VALUE, data_id, data, /NO_COPY
           color = data.keywords.color

; PROV replacement     IF color EQ -1 THEN color = !p.color
;	   IF color EQ -1 AND printstyle EQ 0 THEN color = !p.color
;           IF printstyle GE 1 THEN color = 0
; PROV
	   IF color EQ -1 THEN color = !p.color
	   IF printstyle GE 1 AND color EQ 10 THEN color = 0
	   IF printstyle GE 1 AND NOT PRT.COLOR THEN color = 0
	   
	   
           yaxis = STRING(ABS(data.yaxis),FORMAT='(I4)')
           XYOUTS, x_pos(0), y_pos, /DEVICE, yaxis, CHARSIZE=charsize
           IF data.keywords.psym LE 0 THEN $
              PLOTS, x_pos([1,2]), y_pos*[1,1], /DEVICE, $
                 LINESTYLE = data.keywords.linestyle, $
                 THICK     = data.keywords.thick, $
                 COLOR     = color

;  PLOTS does not accept psym=10 
	   IF data.keywords.psym NE 10 THEN $
           	PLOTS, x_mid, y_pos, /DEVICE, $
              		PSYM      = data.keywords.psym, $
              		SYMSIZE   = data.keywords.symsize, $
              		THICK     = data.keywords.thick, $
              		COLOR     = color ELSE $
           	PLOTS, x_pos([1,2]), y_pos*[1,1], /DEVICE, $
                 LINESTYLE = 0, $
                 THICK     = data.keywords.thick, $
                 COLOR     = color
              		
           XYOUTS, x_pos(3), y_pos, /DEVICE, data.keywords.legend, $
              CHARSIZE=charsize
           HANDLE_VALUE, data_id, data, /SET, /NO_COPY
           data_id = HANDLE_INFO( data_id, /SIBLING )
        ENDWHILE

        RETURN
        END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_LEGEND_EVENT, event

; destroy legend widget

        WIDGET_CONTROL, event.id, GET_UVALUE=action
        CASE action OF
        'CLOS':  WIDGET_CONTROL, event.top, /DESTROY
        ELSE:
        ENDCASE
        RETURN
        END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_LEGEND, id, group_leader=group_leader, modal=modal

; create a widget to draw the legend

	IF not KEYWORD_SET(modal) THEN modal=0
	
        WIDGET_CONTROL, id.base, TLB_GET_OFFSET=offset, TLB_GET_SIZE=sz
        offset(0) = offset(0)+sz(0)
        id_base = WIDGET_BASE( TITLE='XPLOT Legend', /COLUMN, modal=modal, $
           XOFFSET=offset(0), YOFFSET=offset(1), GROUP_LEADER=group_leader )
        id_menu = WIDGET_BASE( id_base, /ROW )
           id_0 =  WIDGET_BUTTON( id_menu, VALUE='      Close      ', $
                   UVALUE='CLOS' )
        n_curves = HANDLE_INFO( id.data, /NUM_CHILDREN )
        ysize = 300 > (n_curves+3)*!d.y_ch_size*1.3*1.7
; add retain = 2 for IDL to handle backing store, WMT, 2/1996
        id_draw = WIDGET_DRAW( id_base, XSIZE=450, YSIZE=ysize , retain = 2)
        WIDGET_CONTROL, id_base, /REALIZE
        WIDGET_CONTROL, id_draw, GET_VALUE=win
        WSET, win
        HANDLE_VALUE, id.legend, {base:id_base, window:win}, /SET
        PIA_XPLOT_LEGPLT, id
        WSET, id.window
        xMANAGER,No_BLOCK=1, "PIA_XPLOT_LEGEND", id_base, GROUP=id.base

        RETURN
        END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_DRAW, data, xaxis, vpar, ny2axes, yaxis_id
@colour_indices
;      This routine draws one curve and if necessary
;      the corresponding y-axis.
;
        COMMON zoom_para, zoomfl, xpos, ypos, start, xrg, yrg, pos1, pos2
   	COMMON zoom_discareas, disc_flags, all_disc_areas

       HANDLE_VALUE, yaxis_id, yaxis



	IF yaxis.keywords.yrange(0) EQ 0. AND $
	   yaxis.keywords.yrange(1) EQ 0. THEN $
         yrange = yaxis.ym + vpar.ym*yaxis.ywd + $
                yaxis.ywd*vpar.ywd*[-0.5,+0.5]     ELSE $
         yrange = yaxis.keywords.yrange
         
	ypos = yrange
; *[-0.666,+0.666]

       IF data.keywords.color EQ -1 THEN $
           color = !p.color $
       ELSE $
           color = data.keywords.color

       CASE 1 OF
       (data.yaxis EQ 1): BEGIN
               xmargin = [ 15, (ny2axes*10) > 3 ]

	IF xaxis.keywords.xrange(0) EQ 0. AND $
	   xaxis.keywords.xrange(1) EQ 0. THEN $
               xrange = xaxis.xm + vpar.xm*xaxis.xwd + $
                        xaxis.xwd*vpar.xwd*[-0.5,+0.5] ELSE $
           xrange = xaxis.keywords.xrange
               
	       xpos = xrange	
;*[-0.588,+0.588]
               ystyle = yaxis.keywords.ystyle
               IF ny2axes GT 0 THEN ystyle = ystyle OR 8
               PLOT, [0], [0], /NODATA,                      $
                   XMARGIN    = xmargin,                      $
                   XRANGE     = xrange,                       $
                   YRANGE     = yrange,                       $
                   CHARSIZE   = xaxis.keywords.charsize,       $
                   SUBTITLE   = xaxis.keywords.subtitle,       $
                   TITLE      = xaxis.keywords.title,          $
                   XCHARSIZE  = xaxis.keywords.xcharsize,      $
                   XSTYLE     = xaxis.keywords.xstyle,         $
                   XTHICK     = xaxis.keywords.xthick,         $
                   XTITLE     = xaxis.keywords.xtitle,         $
                   XTYPE      = xaxis.keywords.xtype,          $
                   YCHARSIZE  = yaxis.keywords.ycharsize,       $
                   YTHICK     = yaxis.keywords.ythick,          $
                   YTITLE     = yaxis.keywords.ytitle,          $
                   YTYPE      = yaxis.keywords.ytype,           $
                   YSTYLE     = ystyle,                       $
                   COLOR      = color
               yaxis.s = !y.s
               HANDLE_VALUE, yaxis_id, yaxis, /SET, /NO_COPY
           END
       (data.yaxis EQ 2): BEGIN
               AXIS, /YAXIS, /SAVE,                          $
                   YRANGE     = yrange,                       $
                   CHARSIZE   = xaxis.keywords.charsize,       $
                   YCHARSIZE  = yaxis.keywords.ycharsize,       $
                   YSTYLE     = yaxis.keywords.ystyle,          $
                   YTITLE     = yaxis.keywords.ytitle,          $
                   YTHICK     = yaxis.keywords.ythick,          $
                   YTYPE      = yaxis.keywords.ytype,           $
                   COLOR      = color
               yaxis.s = !y.s
               HANDLE_VALUE, yaxis_id, yaxis, /SET, /NO_COPY
           END
       (data.yaxis GT 2): BEGIN
               pos = !x.window(1)*!d.x_size +                $
                    (data.yaxis-2)*10*!d.x_ch_size
               AXIS, pos, /DEVICE, /YAXIS, /SAVE,            $
                   YRANGE     = yrange,                       $
                   CHARSIZE   = xaxis.keywords.charsize,       $
                   YCHARSIZE  = yaxis.keywords.ycharsize,       $
                   YSTYLE     = yaxis.keywords.ystyle,          $
                   YTITLE     = yaxis.keywords.ytitle,          $
                   YTHICK     = yaxis.keywords.ythick,          $
                   YTYPE      = yaxis.keywords.ytype,           $
                   COLOR      = color
               yaxis.s = !y.s
               HANDLE_VALUE, yaxis_id, yaxis, /SET, /NO_COPY
           END
       (data.yaxis LT 0): BEGIN
               !y.s = yaxis.s
           END
       ELSE: MESSAGE, "Wrong yaxis value: check code!"
       ENDCASE

       IF data.keywords.psym EQ 8 THEN data.keywords.psym = 1
       
       OPLOT, [data.x], [data.y],                     $
                COLOR      = color,                     $
                LINESTYLE  = data.keywords.linestyle,   $
                PSYM       = data.keywords.psym,        $
                SYMSIZE    = data.keywords.symsize,     $
                THICK      = data.keywords.thick

      IF N_ELEMENTS(disc_flags) GT 1 THEN BEGIN
        ind = WHERE(disc_flags GE 1, count)
        IF count GT 0 THEN $
        oplot, [data.x(ind)], [data.y(ind)], psym=7, color=red, $
                SYMSIZE    = data.keywords.symsize,     $
                THICK      = data.keywords.thick 
      ENDIF
      	
      IF data.keywords.noerrbar EQ 0 THEN BEGIN	
       yerr = data.yerr
       IF yerr(0) NE -1 AND N_ELEMENTS(yerr) LE N_ELEMENTS(data.y) $
       THEN BEGIN
        xx = data.x
        yy = data.y
           FOR i = 0L, N_ELEMENTS(yerr)-1 DO $
               OPLOT, [xx(i),xx(i)], $
                      [yy(i)+yerr(i),yy(i)-yerr(i)], $
                       COLOR   = color, linestyle=data.keywords.yerrstyle,  $
                       THICK   = data.keywords.thick, psym = 0
       ENDIF
      ENDIF
	
;	IF vpar.zoomfl EQ 3 THEN BEGIN
	   siz= size(all_disc_areas)
	   IF siz(0) EQ 0 THEN RETURN
		 IF vpar.zoomfl EQ 4 THEN BEGIN
		  draw_yrange = [min(data.y), max(data.y)]
		  pos1(1) = -1.e25
		  pos2(1) = 1.e25
		 ENDIF ELSE $
		  draw_yrange = [pos1(1),pos2(1)]
		
	         plots, pos1(0), draw_yrange(0), color=color
	         plots, pos2(0), draw_yrange(0), /continue, color=color
	         plots, pos2(0), draw_yrange(1), /continue, color=color
	         plots, pos1(0), draw_yrange(1), /continue, color=color
	         plots, pos1(0), draw_yrange(0), /continue, color=color
 		FOR I=0,N_ELEMENTS(all_disc_areas)-1 DO BEGIN
		 p1 = all_disc_areas(i).pos1
		 p2 = all_disc_areas(i).pos2
		 color = red

		 IF vpar.zoomfl EQ 4 THEN BEGIN
		  ymin = [min(data.y),all_disc_areas(i).pos1(1)]
		  ymax = [max(data.y),all_disc_areas(i).pos2(1)]
		  draw_yrange = [max(ymin), min(ymax)]
		 ENDIF ELSE $
		  draw_yrange = $
		   [all_disc_areas(i).pos1(1),all_disc_areas(i).pos2(1)]

	         plots, p1(0), draw_yrange(0), color=color
	         plots, p2(0), draw_yrange(0), /continue, color=color
	         plots, p2(0), draw_yrange(1), /continue, color=color
	         plots, p1(0), draw_yrange(1), /continue, color=color
	         plots, p1(0), draw_yrange(0), /continue, color=color
	        ENDFOR
;	ENDIF


       RETURN
       END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_REDRAW, id, vpar, PRINT=print, LANDSCAPE=landscape
         COMMON zoom_para, zoomfl, xpos, ypos, start, xrg, yrg, pos1, pos2
   	 COMMON pia_print, prt
;
;       This routine redraws all previously plotted data
;       (e.g. if you changed the plotting range)
;
        IF (!d.flags AND 256) NE 0 THEN WSET, id.window

        n_axes = HANDLE_INFO( id.axes, /NUM_CHILDREN )
        IF n_axes LT 2 THEN RETURN
        ny2axes = (n_axes-2) > 0

        ; get x-axis parameters
        xaxis_id = HANDLE_INFO( id.axes, /FIRST_CHILD )
        axes_ids = [ xaxis_id ]
        HANDLE_VALUE, xaxis_id, xaxis

        ; get all y-axes ids
        yaxis_id = HANDLE_INFO( xaxis_id, /SIBLING )
        WHILE yaxis_id NE 0 DO BEGIN
           axes_ids = [ axes_ids, yaxis_id ]
           yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
        ENDWHILE

        printstyle = KEYWORD_SET(print)+KEYWORD_SET(landscape)
        CASE printstyle OF
        0: !p.region = [0,0,0,0]
        1: !p.region = [ 0.0, 0.5, 1.0, 1.0 ]
        2: !p.region = [ 0.0, 0.4, 1.0, 1.0 ]
        ELSE: MESSAGE,'there is something wrong!'
        ENDCASE

        ; loop through curves to draw
        data_id = HANDLE_INFO( id.data, /FIRST_CHILD )
        WHILE data_id NE 0 DO BEGIN
           HANDLE_VALUE, data_id, data, /NO_COPY
           yaxis_id = axes_ids(ABS(data.yaxis))

; PROV: provisory change to get rid of laser print
	   oldcolor = data.keywords.color
;	   IF KEYWORD_SET(print) THEN $
;		data.keywords.color = -1  
	   IF KEYWORD_SET(print) and oldcolor EQ 10 THEN $
		data.keywords.color = -1  

; If printer definition for black/white set everything to BW
	   IF KEYWORD_SET(print) AND NOT PRT.COLOR THEN data.keywords.color = -1


           PIA_XPLOT_DRAW, data, xaxis, vpar, ny2axes, yaxis_id

	   data.keywords.color = oldcolor	

           HANDLE_VALUE, data_id, data, /SET, /NO_COPY
           data_id = HANDLE_INFO( data_id, /SIBLING )
        ENDWHILE

        ; reset the region in case of printer output
        !p.region=[0,0,0,0]

        ; activate 'current y-axis' for cursor read-out
        HANDLE_VALUE, axes_ids(vpar.currenty), yaxis
        !y.s = yaxis.s

        ; redraw the legend
        IF KEYWORD_SET(print) THEN BEGIN
           PIA_XPLOT_LEGPLT, id, PRINTSTYLE=printstyle
        PIA_XPLOT_ANNPLT, id, PRINTSTYLE=printstyle
        ENDIF ELSE BEGIN
           HANDLE_VALUE, id.legend, legend
           IF N_ELEMENTS(legend) EQ 1 THEN BEGIN
              IF WIDGET_INFO( legend.base, /VALID_ID ) THEN BEGIN
                 WSET, legend.window
                 ERASE
                 PIA_XPLOT_LEGPLT, id
                 WSET, id.window
              ENDIF
           ENDIF
        ENDELSE

	reg = "PHT-Interactive Analysis Display System"
	XYOUTS, 10, 5, /DEVICE, reg, ALIGNMENT=0., charsize=0.8
        RETURN
        END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_PRINT_VAL, id, vpar
;
;       This routine creates a text window containing the data in ASCII
;

        n_axes = HANDLE_INFO( id.axes, /NUM_CHILDREN )
        IF n_axes LT 2 THEN RETURN
        ny2axes = (n_axes-2) > 0

        ; get x-axis parameters
        xaxis_id = HANDLE_INFO( id.axes, /FIRST_CHILD )
        axes_ids = [ xaxis_id ]
        HANDLE_VALUE, xaxis_id, xaxis

        ; get all y-axes ids
        yaxis_id = HANDLE_INFO( xaxis_id, /SIBLING )
        WHILE yaxis_id NE 0 DO BEGIN
           axes_ids = [ axes_ids, yaxis_id ]
           yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
        ENDWHILE
        ; loop through data to choose
        
	text = ' '
        data_id = HANDLE_INFO( id.data, /FIRST_CHILD )
        WHILE data_id NE 0 DO BEGIN
           HANDLE_VALUE, data_id, data, /NO_COPY
	   text = [text,data.keywords.legend]
              
           HANDLE_VALUE, data_id, data, /SET, /NO_COPY
           data_id = HANDLE_INFO( data_id, /SIBLING )
        ENDWHILE
        text = text(1:*)
        IF N_ELEMENTS(text) GT 1 THEN PIA_MULTCHOICE, ele, elemnames=text $
        	ELSE ele=0
        
        data_id = HANDLE_INFO( id.data, /FIRST_CHILD )
	count = 0
        WHILE data_id NE 0 DO BEGIN
           HANDLE_VALUE, data_id, data, /NO_COPY
	   IF count EQ ele THEN BEGIN 
	    text = ['Text extract: ', $
	           '==============', ' ',$
	           'Plot of: ' + data.keywords.legend, $
	           'Contents: ', '      x               y          +/-       unc   ']
	    IF N_ELEMENTS(data.y) EQ N_ELEMENTS(data.yerr) THEN $
	     FOR Iele = 0,N_ELEMENTS(data.y)-1 DO $
	      text = [text, $
	      STRING(FORMAT="(e14.7,4x,e14.7,4x,e11.4)",data.x(iele), $
	      		data.y(iele), data.yerr(iele))] $
	    ELSE $ 
	     FOR Iele = 0,N_ELEMENTS(data.y)-1 DO $
	     text = [text, $
	      STRING(FORMAT="(e14.7,4x,e14.7)",data.x(iele), data.y(iele))] 
	           
            PIA_disp_print_file, '', Title='PIA_Xplot_extract', $
              TEXT=text, GROUP=vpar.group, modal=vpar.modal
           ENDIF
              
           HANDLE_VALUE, data_id, data, /SET, /NO_COPY
           data_id = HANDLE_INFO( data_id, /SIBLING )
	   count = count+1
        ENDWHILE


        ; activate 'current y-axis' for cursor read-out
        HANDLE_VALUE, axes_ids(vpar.currenty), yaxis
        !y.s = yaxis.s

        ; redraw the legend

           HANDLE_VALUE, id.legend, legend
           IF N_ELEMENTS(legend) EQ 1 THEN BEGIN
              IF WIDGET_INFO( legend.base, /VALID_ID ) THEN BEGIN
		help, legend, /struct
              ENDIF
           ENDIF

        RETURN
        END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_PRINT, id, vpar
;
;      This routine sends the plot to printer output
;
       COMMON PIA_PRINT, prt

       IF N_ELEMENTS(prt) EQ 0 THEN BEGIN
           MESSAGE, /INFO, "Printer is not defined"
           RETURN
       ENDIF
       IF prt.device EQ "" THEN BEGIN
           MESSAGE, /INFO, "Printer is not defined"
           RETURN
       ENDIF

       ; save the screen window parameters
       x_save = !x.s
       y_save = !y.s
       this_device = !d.name

       SET_PLOT, prt.device, COPY=prt.copy, INTERPOLATE=prt.interpolate

       cmd = "DEVICE"
       IF prt.filename NE "" THEN BEGIN
           filename = prt.filename
           cmd = cmd + ", FILENAME=" + "'" + filename + "'"
       ENDIF ELSE BEGIN
           filename = STRLOWCASE("idl."+!d.name)
       ENDELSE
       IF prt.keys NE "" THEN cmd = cmd + prt.keys
       IF cmd NE "DEVICE" THEN BEGIN
           IF NOT EXECUTE(cmd) THEN BEGIN
               MESSAGE, /INFO, "Could not set-up device: check keywords"
               RETURN
           ENDIF
       ENDIF


       PIA_XPLOT_REDRAW, id, vpar, /PRINT, LANDSCAPE=prt.landscape
       plot_id

       DEVICE, /CLOSE
       SET_PLOT, this_device

       IF prt.print AND prt.command NE "" THEN BEGIN
           cmd = prt.command + " " + filename
           SPAWN, cmd
       ENDIF

       ; restore the screen window parameters
       !x.s = x_save
       !y.s = y_save
       WSET, id.window


       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_XKEY_EVENT, event

       WIDGET_CONTROL, event.id, GET_UVALUE=action
       WIDGET_CONTROL, event.top, GET_UVALUE=id
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY

       CASE STRMID(action,0,3) OF

       "LIN":  BEGIN
               keywords.xtype=0
               WIDGET_CONTROL, id.xtype, SET_VALUE='     Linear     '
               END

       "LOG":  BEGIN
               keywords.xtype=1
               WIDGET_CONTROL, id.xtype, SET_VALUE='Logarithmic'
               END

       "STY":  BEGIN
               bit = FIX((STRMID(action,3,1)))
               IF event.select $
                 THEN keywords.xstyle = keywords.xstyle OR 2B^bit $
                 ELSE keywords.xstyle = keywords.xstyle AND NOT 2B^bit
               END

       "QUI":  BEGIN
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       "OK ":  BEGIN
               WIDGET_CONTROL, id.xr1, GET_VALUE=val1
               WIDGET_CONTROL, id.xr2, GET_VALUE=val2
               keywords.xrange = [val1(0),val2(0)]
               WIDGET_CONTROL, id.title, GET_VALUE=value
               keywords.title = value(0)
               WIDGET_CONTROL, id.subtitle, GET_VALUE=value
               keywords.subtitle = value(0)
               WIDGET_CONTROL, id.xtitle, GET_VALUE=value
               keywords.xtitle = value(0)
               WIDGET_CONTROL, id.charsize, GET_VALUE=value
               keywords.charsize = value(0)
               WIDGET_CONTROL, id.xcharsize, GET_VALUE=value
               keywords.xcharsize = value(0)
               WIDGET_CONTROL, id.xthick, GET_VALUE=value
               keywords.xthick = value(0)
               HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       ELSE:
       ENDCASE

       HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_XKEY, keywords, GROUP=group
;
;      Modify X-axis and general keywords
;
       id = { title:0L, subtitle:0L, charsize:0L, $
              xcharsize:0L, xthick:0L, xtitle:0L, xtype:0L, $
              tmp:0L, xr1:0L, xr2:0L $
            }

       id_base = WIDGET_BASE( TITLE="Modify X-Axis and General Keywords", $
                                /COLUMN, /modal, GROUP_LEADER=group )
       XMENU, ['      OK      ','     Quit     '], id_base, /ROW, SPACE=10, $
                                UVALUE=['OK ','QUI']
       id.title     = CW_FIELD( id_base, TITLE="Title:          ", $
                                VALUE=keywords.title, XSIZE=50 )
       id.subtitle  = CW_FIELD( id_base, TITLE="Subtitle:       ", $
                                VALUE=keywords.subtitle, XSIZE=50 )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.charsize  = CW_FIELD( id_1, TITLE="Overall Character Size: ", $
                       VALUE=keywords.charsize )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )
       id.xtitle    = CW_FIELD( id_base, TITLE="Xaxis Title: ", $
                       VALUE=keywords.xtitle, XSIZE=40 )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.xcharsize = CW_FIELD( id_1, TITLE="Xaxis Character Size:   ", $
                       VALUE=keywords.xcharsize )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.xthick    = CW_FIELD( id_1, TITLE="Xaxis Thickness:         ", $
                       VALUE=keywords.xthick )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )

       CASE keywords.xtype OF
       0:     val = '     Linear     '
       1:     val = 'Logarithmic'
       ELSE:  val = ' '
       ENDCASE
       id_1         = WIDGET_BASE( id_base, /ROW )
        id_0        = WIDGET_LABEL( id_1, VALUE='Xaxis Type:                 ' )
        id.xtype    = WIDGET_BUTTON( id_1, /MENU, VALUE=val )
         id_0       = WIDGET_BUTTON( id.xtype, $
                          VALUE='Linear', UVALUE='LIN' )
         id_0       = WIDGET_BUTTON( id.xtype, $
                          VALUE='Logarithmic', UVALUE='LOG' )

       id_1  = WIDGET_BASE( id_base, /ROW )
	id_2  = WIDGET_LABEL(id_1, VALUE='Fixed range:         ' )
	id.xr1  = WIDGET_TEXT(id_1, VALUE=STRTRIM(keywords.xrange(0),2),$
				/EDITABLE, XSIZE=5, UVALUE='X1')
	id.xr2  = WIDGET_TEXT(id_1, VALUE=STRTRIM(keywords.xrange(1),2),$
				/EDITABLE, XSIZE=5, UVALUE='X2')
	id_2  = WIDGET_LABEL(id_1, VALUE='  ([0,0] for default) ')

       id_1   = WIDGET_BASE( id_base, /ROW )
        id_2  = WIDGET_BASE( id_1, /COLUMN )
         id_0  = WIDGET_LABEL( id_2, VALUE='Xaxis Style: ' )
        id_2  = WIDGET_BASE( id_1,  /NONEXCLUSIVE, /COLUMN )
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Force exact range', UVALUE='STY0' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.xstyle AND 2B^0
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Extend axis range', UVALUE='STY1' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.xstyle AND 2B^1
        id_2  = WIDGET_BASE( id_1, /NONEXCLUSIVE, /COLUMN )
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Suppress drawing of axis', UVALUE='STY2' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.xstyle AND 2B^2
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Suppress drawing of box', UVALUE='STY3' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.xstyle AND 2B^3
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Inhibit setting X_min = 0', UVALUE='STY4' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.xstyle AND 2B^4

       id.tmp = HANDLE_CREATE( VALUE=keywords )
       WIDGET_CONTROL, id_base, /REALIZE, SET_UVALUE=id
       xMANAGER,No_BLOCK=1, "PIA_XPLOT_XKEY", id_base, GROUP=group
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY
       HANDLE_FREE, id.tmp

       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_YKEY_EVENT, event

       WIDGET_CONTROL, event.id, GET_UVALUE=action
       WIDGET_CONTROL, event.top, GET_UVALUE=id
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY

       CASE STRMID(action,0,3) OF

       "LIN":  BEGIN
               keywords.ytype=0
               WIDGET_CONTROL, id.ytype, SET_VALUE='     Linear     '
               END

       "LOG":  BEGIN
               keywords.ytype=1
               WIDGET_CONTROL, id.ytype, SET_VALUE='Logarithmic'
               END

       "STY":  BEGIN
               bit = FIX((STRMID(action,3,1)))
               IF event.select $
                 THEN keywords.ystyle = keywords.ystyle OR 2B^bit $
                 ELSE keywords.ystyle = keywords.ystyle AND NOT 2B^bit
               END


       "QUI":  BEGIN
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       "OK ":  BEGIN
               WIDGET_CONTROL, id.yr1, GET_VALUE=val1
               WIDGET_CONTROL, id.yr2, GET_VALUE=val2
               keywords.yrange = [val1(0),val2(0)]
               WIDGET_CONTROL, id.ytitle, GET_VALUE=value
               keywords.ytitle = value(0)
               WIDGET_CONTROL, id.ycharsize, GET_VALUE=value
               keywords.ycharsize = value(0)
               WIDGET_CONTROL, id.ythick, GET_VALUE=value
               keywords.ythick = value(0)
               HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       ELSE:
       ENDCASE

       HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_YKEY, keywords, GROUP=group
;
;      Modify Y-axis keywords
;
       id = { ycharsize:0L, ythick:0L, ytitle:0L, ytype:0L, $
              tmp:0L, yr1:0L, yr2:0L $
            }

       id_base = WIDGET_BASE( TITLE="Modify Y-Axis Keywords", /COLUMN, $
       				/modal, GROUP_LEADER=group )
       XMENU, ['      OK      ','     Quit     '], id_base, /ROW, SPACE=10, $
                                UVALUE=['OK ','QUI']
       id.ytitle    = CW_FIELD( id_base, TITLE="Yaxis Title: ", $
                                VALUE=keywords.ytitle, XSIZE=40 )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.ycharsize = CW_FIELD( id_1, TITLE="Yaxis Character Size: ", $
                                VALUE=keywords.ycharsize )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.ythick    = CW_FIELD( id_1, TITLE="Yaxis Thickness:       ", $
                                VALUE=keywords.ythick )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )

       CASE keywords.ytype OF
       0:     val = '     Linear     '
       1:     val = 'Logarithmic'
       ELSE:  val = ' '
       ENDCASE
       id_1         = WIDGET_BASE( id_base, /ROW )
        id_0        = WIDGET_LABEL( id_1, VALUE='Yaxis Type:              ' )
        id.ytype    = WIDGET_BUTTON( id_1, /MENU, VALUE=val )
         id_0       = WIDGET_BUTTON( id.ytype, $
                          VALUE='Linear', UVALUE='LIN' )
         id_0       = WIDGET_BUTTON( id.ytype, $
                          VALUE='Logarithmic', UVALUE='LOG' )

	id_1  = WIDGET_BASE( id_base, /ROW )
	id_2  = WIDGET_LABEL(id_1, VALUE='Fixed range:         ' )
	id.yr1  = WIDGET_TEXT(id_1, VALUE=STRTRIM(keywords.yrange(0),2),$
				/EDITABLE, XSIZE=5, UVALUE='Y1')
	id.yr2  = WIDGET_TEXT(id_1, VALUE=STRTRIM(keywords.yrange(1),2),$
				/EDITABLE, XSIZE=5, UVALUE='Y2')
	id_2  = WIDGET_LABEL(id_1, VALUE='  ([0,0] for default) ')

       id_1   = WIDGET_BASE( id_base, /ROW )
        id_2  = WIDGET_BASE( id_1, /COLUMN )
         id_0  = WIDGET_LABEL( id_2, VALUE='Yaxis Style: ' )
        id_2  = WIDGET_BASE( id_1, /NONEXCLUSIVE, /COLUMN )
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Force exact range', UVALUE='STY0' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.ystyle AND 2B^0
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Extend axis range', UVALUE='STY1' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.ystyle AND 2B^1
        id_2  = WIDGET_BASE( id_1, /NONEXCLUSIVE, /COLUMN )
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Suppress drawing of axis', UVALUE='STY2' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.ystyle AND 2B^2
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Suppress drawing of box', UVALUE='STY3' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.ystyle AND 2B^3
         id_0 = WIDGET_BUTTON( id_2, $
                 VALUE='Inhibit setting Y_min = 0', UVALUE='STY4' )
         WIDGET_CONTROL, id_0, SET_BUTTON=keywords.ystyle AND 2B^4

       id.tmp = HANDLE_CREATE( VALUE=keywords )
       WIDGET_CONTROL, id_base, /REALIZE, SET_UVALUE=id
       xMANAGER,No_BLOCK=1, "PIA_XPLOT_YKEY", id_base, GROUP=group
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY
       HANDLE_FREE, id.tmp

       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_DKEY_EVENT, event

       WIDGET_CONTROL, event.id, GET_UVALUE=action
       WIDGET_CONTROL, event.top, GET_UVALUE=id
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY

       CASE STRMID(action,0,3) OF

       "LST": BEGIN
              n = STRMID(action,3,1)
              IF n EQ '-' THEN BEGIN
                 val = '          None           '
                 keywords.psym = ABS(keywords.psym)
              ENDIF ELSE BEGIN
                 IF keywords.psym EQ 10 $
                    THEN keywords.psym = ABS(keywords.psym) $
                    ELSE keywords.psym = -ABS(keywords.psym)
                 keywords.linestyle = FIX(n)
                 CASE keywords.linestyle OF
                 0: val = '          Solid          '
                 1: val = '         Dotted        '
                 2: val = '         Dashed        '
                 3: val = '       Dash Dot      '
                 4: val = 'Dash Dot Dot Dot'
                 5: val = '    Long Dashes    '
                 ELSE: val = ' '
                 ENDCASE
              ENDELSE
              WIDGET_CONTROL, id.linestyle, SET_VALUE=val
              END

       "PSY":  BEGIN
               n = FIX(STRMID(action,3,2))
               CASE n OF
               0:    val = '         None         '
               1:    val = '      Plus sign      '
               2:    val = '      Asterisk       '
               3:    val = '       Period         '
               4:    val = '      Diamond       '
               5:    val = '       Triangle       '
               6:    val = '       Square         '
               7:    val = '          X               '
               10:   val = 'Histogram mode '
               ELSE: val = '                     '
               ENDCASE
               WIDGET_CONTROL, id.psym, SET_VALUE=val
               sign = (keywords.psym GT 0)*2 - 1
               IF n EQ 10 THEN sign = 1
               keywords.psym = sign * n
               END

       "ERR":  BEGIN
                n = FIX(STRMID(action,3,2))
               CASE n OF
               0:    val = "   Show error bars      "
               1:    val = "  Don't show error bars "
               ENDCASE      
               WIDGET_CONTROL, id.errbars, SET_VALUE=val
               keywords.noerrbar = n
               END
       		


       "QUI":  BEGIN
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       "OK ":  BEGIN
               WIDGET_CONTROL, id.legend, GET_VALUE=value
               keywords.legend = value(0)
               WIDGET_CONTROL, id.color, GET_VALUE=value
               keywords.color = value(0)
               WIDGET_CONTROL, id.symsize, GET_VALUE=value
               keywords.symsize = value(0)
               WIDGET_CONTROL, id.thick, GET_VALUE=value
               keywords.thick = value(0)
               HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
               WIDGET_CONTROL, event.top, /DESTROY
               RETURN
               END

       ELSE:
       ENDCASE

       HANDLE_VALUE, id.tmp, keywords, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_DKEY, keywords, GROUP=group
;
;      Modify data keywords
;
       id = { legend:0L, color:0L, linestyle:0L, psym:0L, symsize:0L, $
              thick:0L, tmp:0L, errbars:0L $
            }

       id_base = WIDGET_BASE( TITLE="Modify Curve Keywords", /COLUMN, $
       				/modal, GROUP_LEADER=group )
       XMENU, ['      OK      ','     Quit     '], id_base, /ROW, SPACE=10, $
                                UVALUE=['OK ','QUI']
       id.legend    = CW_FIELD( id_base, TITLE="Legend:       ", $
                                VALUE=keywords.legend, XSIZE=30 )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.color     = CW_FIELD( id_1,   TITLE="Color Index:      ", $
                                VALUE=keywords.color )
        id_0 = WIDGET_LABEL( id_1, VALUE='(-1 for black/white)' )
       id_1 = WIDGET_BASE( id_base, /ROW )
        id.thick     = CW_FIELD( id_1,     TITLE="Line Thickness:   ", $
                                VALUE=keywords.thick )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )

       IF keywords.psym GT 0 AND keywords.psym NE 10 THEN BEGIN
           val = '          None           '
       ENDIF ELSE BEGIN
           CASE keywords.linestyle OF
           0:     val = '          Solid          '
           1:     val = '         Dotted        '
           2:     val = '         Dashed        '
           3:     val = '       Dash Dot      '
           4:     val = 'Dash Dot Dot Dot'
           5:     val = '    Long Dashes    '
           ELSE:  val = ' '
           ENDCASE
       ENDELSE
       id_1          = WIDGET_BASE( id_base, /ROW )
        id_0         = WIDGET_LABEL( id_1, VALUE='Line Style:       ' )
        id.linestyle = WIDGET_BUTTON( id_1, /MENU, VALUE=val )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='None', UVALUE='LST-' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Solid', UVALUE='LST0' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Dotted', UVALUE='LST1' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Dashed', UVALUE='LST2' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Dash Dot', UVALUE='LST3' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Dash Dot Dot Dot', $
                                UVALUE='LST4' )
         id_0 = WIDGET_BUTTON( id.linestyle, VALUE='Long Dashes', $
                                UVALUE='LST5' )

        CASE ABS(keywords.psym) OF
               0:    val = '      None         '
               1:    val = '    Plus sign      '
               2:    val = '    Asterisk       '
               3:    val = '     Period        '
               4:    val = '    Diamond        '
               5:    val = '    Triangle       '
               6:    val = '     Square        '
               7:    val = '        X          '
               10:   val = '  Histogram mode   '
               ELSE: val = '                   '
        ENDCASE
       id_1          = WIDGET_BASE( id_base, /ROW )
        id_0         = WIDGET_LABEL( id_1, VALUE='Symbol:           ' )
        id.psym      = WIDGET_BUTTON( id_1, /MENU, VALUE=val )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='None', UVALUE='PSY00' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Plus sign', UVALUE='PSY01' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Asterisk', UVALUE='PSY02' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Period', UVALUE='PSY03' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Diamond', UVALUE='PSY04' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Triangle', UVALUE='PSY05' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Square', UVALUE='PSY06' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='X', UVALUE='PSY07' )
         id_0 = WIDGET_BUTTON( id.psym, VALUE='Histogram mode', UVALUE='PSY10' )

       id_1 = WIDGET_BASE( id_base, /ROW )
        id.symsize   = CW_FIELD( id_1, TITLE="Symbol Size:    ", $
                                VALUE=keywords.symsize )
        id_0 = WIDGET_LABEL( id_1, VALUE='(0 or 1 for default)' )

        CASE ABS(keywords.noerrbar) OF
               0:    val = "   Show error bars      "
               1:    val = " Don't show error bars  "
	ENDCASE
       id_1 = WIDGET_BASE( id_base, /ROW )
        id_0         = WIDGET_LABEL( id_1, VALUE='Error bars:     ' )
        id.errbars      = WIDGET_BUTTON( id_1, /MENU, VALUE=val )
         id_0 = WIDGET_BUTTON( id.errbars, VALUE="Show error bars", $
          UVALUE='ERR0' )
         id_0 = WIDGET_BUTTON( id.errbars, VALUE="Don't show error bars", $
          UVALUE='ERR1' )
          
       id.tmp = HANDLE_CREATE( VALUE=keywords )
       WIDGET_CONTROL, id_base, /REALIZE, SET_UVALUE=id
       xMANAGER,No_BLOCK=1, "PIA_XPLOT_DKEY", id_base, GROUP=group
       HANDLE_VALUE, id.tmp, keywords, /NO_COPY
       HANDLE_FREE, id.tmp

       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_PUTXKEY, xaxis_id, keywords
;
;      Store back (modified) X-axis keywords
;
       HANDLE_VALUE, xaxis_id, xaxis, /NO_COPY
       xaxis.keywords = keywords
       HANDLE_VALUE, xaxis_id, xaxis, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_PUTYKEY, yaxis_id, keywords
;
;      Store back (modified) Y-axis keywords
;
       HANDLE_VALUE, yaxis_id, yaxis, /NO_COPY
       yaxis.keywords = keywords
       HANDLE_VALUE, yaxis_id, yaxis, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       PRO PIA_XPLOT_PUTDKEY, data_id, keywords
;
;      Store back (modified) data keywords
;
       HANDLE_VALUE, data_id, data, /NO_COPY
       data.keywords = keywords
       HANDLE_VALUE, data_id, data, /SET, /NO_COPY
       RETURN
       END
;
;-----------------------------------------------------------------------
;
       FUNCTION PIA_XPLOT_GETXKEY, topid, ID=xaxis_id
;
;      Get keywords saved with x-axis
;
       xaxis_id = HANDLE_INFO( topid, /FIRST_CHILD )
       HANDLE_VALUE, xaxis_id, xaxis
       RETURN, xaxis.keywords
       END
;
;-----------------------------------------------------------------------
;
       FUNCTION PIA_XPLOT_GETYKEY, topid, number, ID=yaxis_id
;
;      Get keywords saved with 'number-th' y-axis (beginning with 1)
;
       nyaxes = HANDLE_INFO( topid, /NUM_CHILDREN ) - 1
       number = number < nyaxes > 1
       yaxis_id = HANDLE_INFO( topid, /FIRST_CHILD )
       FOR i = 1,number DO yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
       HANDLE_VALUE, yaxis_id, yaxis
       RETURN, yaxis.keywords
       END
;
;-----------------------------------------------------------------------
;
       FUNCTION PIA_XPLOT_GETDKEY, topid, number, ID=data_id
;
;      Get keywords saved with 'number-th' data set (beginning with 1)
;
       ndata = HANDLE_INFO( topid, /NUM_CHILDREN )
       number = number < ndata > 1
       data_id = HANDLE_INFO( topid, /FIRST_CHILD )
       FOR i = 2,number DO data_id = HANDLE_INFO( data_id, /SIBLING )
       HANDLE_VALUE, data_id, data
       RETURN, data.keywords
       END
;
;-----------------------------------------------------------------------
;
       FUNCTION PIA_XPLOT_GETYS, topid, number, ID=yaxis_id
;
;      Get scaling saved with 'number-th' y-axis (beginning with 1)
;
       nyaxes = HANDLE_INFO( topid, /NUM_CHILDREN ) - 1
       number = number < nyaxes > 1
       yaxis_id = HANDLE_INFO( topid, /FIRST_CHILD )
       FOR i = 1,number DO yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
       HANDLE_VALUE, yaxis_id, yaxis
       RETURN, yaxis.s
       END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT_SAVE, id, x, y, yerr, vpar, OVERPLOT=overplot, $
                           X_KEYWORDS=x_keywords, $
                           Y_KEYWORDS=y_keywords, $
                           D_KEYWORDS=d_keywords, $
                           ANNOTATION=annotation

;
;       This routine saves data arrays, axis information, keywords,
;       and the annotation
;
        IF NOT KEYWORD_SET(overplot) THEN overplot=0

        n_axes = HANDLE_INFO( id.axes, /NUM_CHILDREN )

        ; x-axis parameters
        IF n_axes EQ 0 THEN BEGIN
           ; create new x-axis
           xaxis_id = HANDLE_CREATE( id.axes )
           xl = MIN(x, MAX=xr)
           IF xl EQ xr THEN BEGIN
               xl = xl*0.9 - (xl EQ 0)
               xr = xr*1.1 + (xr EQ 0)
           ENDIF
        ENDIF ELSE BEGIN
           ; update min/max values of x-axis
           xaxis_id = HANDLE_INFO( id.axes, /FIRST_CHILD )
           HANDLE_VALUE, xaxis_id, xaxis, /NO_COPY
	   equis = reform([x])
           xl = MIN( [ xaxis.xl, equis, xaxis.xr ], MAX=xr)
        ENDELSE
        xm  = (xl + xr) / 2.
        xwd = xr - xl
        HANDLE_VALUE, xaxis_id, /SET, /NO_COPY, $
           { xl:xl, xr:xr, xm:xm, xwd:xwd, keywords:x_keywords }

        ; y-axis parameters
        IF overplot LE 0 THEN BEGIN
           ; create new y-axis
           yaxis_ix = HANDLE_INFO( id.axes, /NUM_CHILDREN )
           yaxis_id = HANDLE_CREATE( id.axes )
           IF yerr(0) EQ -1 THEN BEGIN
               yl = MIN(y, MAX=yr)
           ENDIF ELSE BEGIN
               yl = MIN( y-yerr )
               yr = MAX( y+yerr )
           ENDELSE
           IF yl EQ yr THEN BEGIN
               yl = yl*0.9 - (yl EQ 0)
               yr = yr*1.1 + (yr EQ 0)
           ENDIF

           ; make new menu entries
           value = STRING( yaxis_ix, FORMAT="('# ',A)" )
           uvalue = STRING( yaxis_ix, FORMAT="('KEYY',I4.4)" )
           id_0 = WIDGET_BUTTON( id.keyy, VALUE=value, UVALUE=uvalue )
           uvalue = STRING( yaxis_ix, FORMAT="('CURY',I4.4)" )
           id_0 = WIDGET_BUTTON( id.curs, VALUE=value, UVALUE=uvalue )

        ENDIF ELSE BEGIN
           ; update min/max values of y-axis
           IF n_axes LT 2 THEN MESSAGE, "No y-axis defined: check code"
           yaxis_ix = (n_axes-1) < overplot
           yaxis_id = HANDLE_INFO( id.axes, /FIRST_CHILD )
           FOR i = 1,yaxis_ix DO yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
           HANDLE_VALUE, yaxis_id, yaxis, /NO_COPY
; y can be 2-dimensional or just one value
           IF N_ELEMENTS(y) GT 1 THEN yyy = reform(y) ELSE yyy = y
           yl = MIN( [ yaxis.yl, yyy, yaxis.yr ], MAX=yr)
           yaxis_ix = -yaxis_ix
        ENDELSE
        ym  = (yl + yr) / 2.
        ywd = yr - yl
        HANDLE_VALUE, yaxis_id, /SET, /NO_COPY, $
           { yl:yl, yr:yr, ym:ym, ywd:ywd, keywords:y_keywords, s:!y.s }

        ; define 'current' y-axis as the axis associated with the new data
        vpar.currenty = ABS(yaxis_ix)

        ; create new data storage
        data_id = HANDLE_CREATE( id.data )
        HANDLE_VALUE, data_id, /SET, /NO_COPY, $
           { yaxis:yaxis_ix, x:x, y:y, yerr:yerr, keywords:d_keywords }

        ; make a new menu entry for data curves
        ndata = HANDLE_INFO( id.data, /NUM_CHILDREN )
        value = STRING( ndata, FORMAT="('# ',A)" )
        uvalue = STRING( ndata, FORMAT="('KEYD',I4.4)" )
        id_0 = WIDGET_BUTTON( id.keyd, VALUE=value, UVALUE=uvalue )

        ; store the annotation (string array)
        IF N_ELEMENTS(annotation) GT 1 OR annotation(0) NE '' THEN BEGIN
           HANDLE_VALUE, id.annotation, saved_annotation, /NO_COPY
           IF N_ELEMENTS(saved_annotation) EQ 0 THEN $
              saved_annotation = [ annotation ] $
           ELSE $
              saved_annotation = [ saved_annotation, annotation ]
           HANDLE_VALUE, id.annotation, saved_annotation, /SET, /NO_COPY
        ENDIF

        RETURN
        END
;
; --------------------------------------------------------------------------
;
       PRO PIA_XPLOT_CLEAN, event_id
;
;      clean-up routine called when PIA_XPLOT is destroyed intetionally
;      or unintentionally: deletes all saved data and axes parameters.
;
       WIDGET_CONTROL, event_id, GET_UVALUE=id, /NO_COPY
       HANDLE_FREE, id.vpar
       HANDLE_FREE, id.data
       HANDLE_FREE, id.axes
       HANDLE_FREE, id.annotation
       HANDLE_FREE, id.legend

       RETURN
       END
;
;----------------------------------------------------------------------------
;
   PRO PIA_XPLOT_EVENT, event
;
;  Event handling routine for PIA_XPLOT widget.
;
@colour_indices
   COMMON PIA_XPLOT, active_base
   COMMON zoom_para, zoomfl, xpos, ypos, start, xrg, yrg, pos1, pos2
   COMMON zoom_discareas, disc_flags, all_disc_areas

   WIDGET_CONTROL, event.id, GET_UVALUE=action
   WIDGET_CONTROL, event.top, GET_UVALUE=id
   HANDLE_VALUE, id.vpar, vpar, /NO_COPY
   modal = vpar.modal

   CASE action OF

   'DRAW': BEGIN

	   IF vpar.zoomfl GE 2 THEN BEGIN 
		wset, id.window
	      IF vpar.zoomfl GE 3 THEN color=red ELSE color=white	
	      IF event.type EQ 0 and start EQ 0 THEN BEGIN
   		disc_area =  {disca_typ, pos1:[0.,0.], pos2:[0.,0.]}  
                pos1 = convert_coord(event.x,event.y,/device,/to_data)
		start = 1
	      ENDIF
	      IF event.type EQ 2 and start EQ 1 THEN BEGIN
                pos2 = convert_coord(event.x,event.y,/device,/to_data)
	        plots, pos1(0), pos1(1)
	        plots, pos2(0), pos1(1), /continue , color=color
	        plots, pos2(0), pos2(1), /continue , color=color
	        plots, pos1(0), pos2(1), /continue , color=color
	        plots, pos1(0), pos1(1), /continue , color=color
                WAIT, 0.1
	        plots, pos1(0), pos1(1)
	        plots, pos2(0), pos1(1), /continue , color=black
	        plots, pos2(0), pos2(1), /continue , color=black
	        plots, pos1(0), pos2(1), /continue , color=black
	        plots, pos1(0), pos1(1), /continue , color=black
	      ENDIF
	      IF event.type EQ 1 and start EQ 1 THEN BEGIN
                pos2 = convert_coord(event.x,event.y,/device,/to_data)
                       
	        plots, pos1(0), pos1(1)
	        plots, pos2(0), pos1(1), /continue , color=color
	        plots, pos2(0), pos2(1), /continue , color=color
	        plots, pos1(0), pos2(1), /continue , color=color
	        plots, pos1(0), pos1(1), /continue , color=color

		wait, 0.5
		start = 0
                       
        	; loop through curves to get last xrange and yrange
        	data_id = HANDLE_INFO( id.data, /FIRST_CHILD )
        	xaxis_id = HANDLE_INFO( id.axes, /FIRST_CHILD )
		axes_ids = [xaxis_id]
        	HANDLE_VALUE, xaxis_id, xaxref

; get all y-axes ids
        	yaxis_id = HANDLE_INFO( xaxis_id, /SIBLING )
        	WHILE yaxis_id NE 0 DO BEGIN
        	   axes_ids = [ axes_ids, yaxis_id ]
        	   yaxis_id = HANDLE_INFO( yaxis_id, /SIBLING )
        	ENDWHILE

        	WHILE data_id NE 0 DO BEGIN
           		HANDLE_VALUE, data_id, data
           		yaxis_id = axes_ids(ABS(data.yaxis))

           		data_id = HANDLE_INFO( data_id, /SIBLING )
			
; necessary because error bars are also handled as data with xrg=[0.,0.]
			mini = min(data.x)
			maxi = max(data.x)
;		  IF mini NE 0. OR maxi NE 0. THEN BEGIN 
		 		 xrg = [mini, maxi]
		 		 yrg = [min(data.y), max(data.y)]
       				 HANDLE_VALUE, yaxis_id, yaxref
;		  ENDIF
		ENDWHILE

		IF pos1(0) NE pos2(0) AND pos1(1) NE pos2(1) THEN BEGIN

; put in the 'right' order

		 IF pos1(0) GT pos2(0) THEN BEGIN
		    posi = pos1(0)
		    pos1(0) = pos2(0)
		    pos2(0) = posi
		 ENDIF

		 IF pos1(1) GT pos2(1) THEN BEGIN
		    posi = pos1(1)
		    pos1(1) = pos2(1)
		    pos2(1) = posi
		 ENDIF

; conversion factors from old values 
		 xmed = (pos2(0) + pos1(0)) / 2 
		 ymed = (pos2(1) + pos1(1)) / 2 

; use the display formula (in pia_xplot_redraw) for obtaining the wd's and
; and m's which correspond to xrg and yrg wished

		 IF vpar.zoomfl LT 3 THEN BEGIN
		  xdif = pos2(0) - pos1(0)
		  xsum = pos1(0) + pos2(0)
		  vpar.xwd = xdif / xaxref.xwd
		  vpar.xm = (xsum - 2*xaxref.xm) / (2 *xaxref.xwd)

		  ydif = pos2(1) - pos1(1)
		  ysum = pos1(1) + pos2(1)
		  vpar.ywd = ydif / yaxref.ywd
		  vpar.ym = (ysum - 2*yaxref.ym) / (2 *yaxref.ywd)
		 ENDIF
                ENDIF 

		PIA_XPLOT_REDRAW, id, vpar

		IF vpar.zoomfl GE 3 THEN BEGIN

		 IF vpar.zoomfl EQ 4 THEN BEGIN
		  draw_yrange = [min(data.y), max(data.y)]
		  pos1(1) = -1.e25
		  pos2(1) = 1.e25
		 ENDIF ELSE $
		  draw_yrange = [pos1(1),pos2(1)]
		
	         plots, pos1(0), draw_yrange(0), color=color
	         plots, pos2(0), draw_yrange(0), /continue, color=color
	         plots, pos2(0), draw_yrange(1), /continue, color=color
	         plots, pos1(0), draw_yrange(1), /continue, color=color
	         plots, pos1(0), draw_yrange(0), /continue, color=color
		 p1 = [pos1(0),pos1(1)]
	         p2 = [pos2(0),pos2(1)]
	         disc_area = {disca_typ, pos1:p1, pos2:p2}
	         siz = size(all_disc_areas)
	           IF siz(0) EQ 0 THEN $
	           	all_disc_areas = disc_area ELSE $
	           	all_disc_areas = [all_disc_areas, disc_area]
	        ENDIF
		   
		
	      ENDIF
           ENDIF ELSE IF vpar.zoomfl EQ 1 THEN BEGIN

             pos = convert_coord(event.x,event.y,/device,/to_data)
             WIDGET_CONTROL, id.txtx, SET_VALUE = STRCOMPRESS(STRING(pos(0,0)))
             WIDGET_CONTROL, id.txty, SET_VALUE = STRCOMPRESS(STRING(pos(1,0)))

	   ENDIF 
           END

   'SLDX': BEGIN
           WIDGET_CONTROL, id.sldx, GET_VALUE = xpos
           vpar.xm = xpos / 200.0
           PIA_XPLOT_REDRAW, id, vpar
           END

   'SLDY': BEGIN
           WIDGET_CONTROL, id.sldy, GET_VALUE = ypos
           vpar.ym = ypos / 200.0
           PIA_XPLOT_REDRAW, id, vpar
           END

   'XEXP': BEGIN
           vpar.xwd = vpar.xwd / 2.0
           PIA_XPLOT_REDRAW, id, vpar
           END

   'XCON': BEGIN
           vpar.xwd = vpar.xwd * 2.0
           IF vpar.xwd GT 1.0 THEN BEGIN
               vpar.xm  = 0.0
               vpar.xwd = 1.0
               WIDGET_CONTROL, id.sldx, SET_VALUE = 0
           ENDIF
           PIA_XPLOT_REDRAW, id, vpar
           END

   'XFUL': BEGIN
           vpar.xm  = 0.0
           vpar.xwd = 1.0
           WIDGET_CONTROL, id.sldx, SET_VALUE = 0
           PIA_XPLOT_REDRAW, id, vpar
           END

   'YEXP': BEGIN
           vpar.ywd = vpar.ywd / 2.0
           PIA_XPLOT_REDRAW, id, vpar
           END

   'YCON': BEGIN
           vpar.ywd = vpar.ywd * 2.0
           IF vpar.ywd GT 1.0 THEN BEGIN
               vpar.ym  = 0.0
               vpar.ywd = 1.0
               WIDGET_CONTROL, id.sldy, SET_VALUE = 0
           ENDIF
           PIA_XPLOT_REDRAW, id, vpar
           END

   'YFUL': BEGIN
           vpar.ym  = 0.0
           vpar.ywd = 1.0
           PIA_XPLOT_REDRAW, id, vpar
           WIDGET_CONTROL, id.sldy, SET_VALUE = 0
           END

   'CURB': BEGIN
		vpar.zoomfl = event.value
           END

   'PRNT': BEGIN
           PIA_XPLOT_PRINT, id, vpar
           END

   'PRNT_VAL': BEGIN
           PIA_XPLOT_PRINT_VAL, id, vpar
           END

   'DPRN': BEGIN
           PIA_PRTDEF, GROUP=event.top
           END

   'ACTV': BEGIN
           active_base = id.base
           END

   'QUIT': BEGIN
           WIDGET_CONTROL, event.top, /DESTROY
           RETURN
           END

   'LGND': PIA_XPLOT_LEGEND, id, group_leader=vpar.group, modal=modal 

   'ANNO': BEGIN
           HANDLE_VALUE, id.annotation, annotation
           IF N_ELEMENTS(annotation) EQ 0 THEN $
           annotation = '-- No annotation defined --'
           PIA_DISP_PRINT_FILE, 'dummy', TEXT=annotation, GROUP=event.top, $
                          TITLE='XPLOT Annotation', modal=modal
           END

   'HELP': BEGIN
	   IF !VERSION.OS NE 'vms' THEN $
                         spawn, '$w3net $PIA_HTML/um/xplot.html &' ELSE $
                         spawn, 'w3net /pia_dsk/pia/html/um/xplot.html',/nowait
      	   END
   	   
   ELSE:   BEGIN
           CASE STRMID(action,0,3) OF
           "KEY":  CASE STRMID(action,3,1) OF
                   "X":    BEGIN
                           keywords = PIA_XPLOT_GETXKEY( id.axes, ID=xaxis_id )
                           PIA_XPLOT_XKEY, keywords, GROUP=event.top
                           IF N_ELEMENTS( keywords ) GT 0 THEN BEGIN
                               PIA_XPLOT_PUTXKEY, xaxis_id, keywords
                               PIA_XPLOT_REDRAW, id, vpar
                           ENDIF
                           END
                   "Y":    BEGIN
                           number = FIX(STRMID(action,4,100))
                           keywords = PIA_XPLOT_GETYKEY( id.axes, number, $
                                                         ID=yaxis_id )
                           PIA_XPLOT_YKEY, keywords, GROUP=event.top
                           IF N_ELEMENTS( keywords ) GT 0 THEN BEGIN
                               PIA_XPLOT_PUTYKEY, yaxis_id, keywords
                               PIA_XPLOT_REDRAW, id, vpar
                           ENDIF
                           END
                   "D":    BEGIN
                           number = FIX(STRMID(action,4,100))
                           keywords = PIA_XPLOT_GETDKEY( id.data, number, $
                                                         ID=data_id )
                           PIA_XPLOT_DKEY, keywords, GROUP=event.top
                           IF N_ELEMENTS( keywords ) GT 0 THEN BEGIN
                               PIA_XPLOT_PUTDKEY, data_id, keywords
                               PIA_XPLOT_REDRAW, id, vpar
                           ENDIF
                           END
                   ELSE:
                   ENDCASE

           ELSE:   CASE STRMID(action,0,4) OF
                   "CURY": BEGIN
                           number = FIX(STRMID(action,4,100))
                           !y.s = PIA_XPLOT_GETYS( id.axes, number )
                           vpar.currenty = number
                           END
                   ELSE:
                   ENDCASE

           ENDCASE
           END

   ENDCASE

   HANDLE_VALUE, id.vpar, vpar, /SET, /NO_COPY
   RETURN
   END
;
;-----------------------------------------------------------------------
;
   PRO PIA_XPLOT_WIDGET, id, disc_area=disc_area, modal=modal, $
   	GROUP_LEADER=group_leader
;
;       This routine creates the widget for PIA_XPLOT
;       and defines handles to hold plot data and parameter values
;
         COMMON zoom_para, zoomfl, xpos, ypos, start, xrg, yrg, pos1, pos2

   IF KEYWORD_SET(MODAL) THEN modal=1 ELSE modal=0
   
   id = {  base:0L, draw:0L, sldy:0L, sldx:0L, txxb:0L, txtx:0L, txty:0L, $
           keyy:0L, keyd:0L, curs:0L, zoom:0L, $
           vpar:0L, data:0L, axes:0L, annotation:0L, legend:0L, $
           window:0	$
        }

; Bitmap definitions for 7 buttons: variable BM(*,*,7)

@pia_xplot_bmd

; widget definitions

;   font = '-adobe-helvetica-bold-r-normal--14-140-*'
   font = ''

   id.base = WIDGET_BASE( TITLE = 'PIA_XPLOT', /COLUMN, modal=modal, $
   		GROUP_leader=group_leader)

   id_menu = WIDGET_BASE( id.base, /ROW )
     id_1 = WIDGET_BUTTON( id_menu, VALUE='    Window    ', /MENU )
       id_0 = WIDGET_BUTTON( id_1, VALUE='Activate this', UVALUE='ACTV' )
       id_0 = WIDGET_BUTTON( id_1, VALUE='Print', UVALUE='PRNT' )
       id_0 = WIDGET_BUTTON( id_1, VALUE='Define Printer...', UVALUE='DPRN' )
       id_0 = WIDGET_BUTTON( id_1, VALUE='Print Values ', UVALUE='PRNT_VAL' )
       id_0 = WIDGET_BUTTON( id_1, VALUE='Quit', UVALUE='QUIT' )
     id_2 = WIDGET_BUTTON( id_menu, VALUE='  Customize  ', /MENU )
       id_0    = WIDGET_BUTTON( id_2, VALUE='X-axis and General...', $
                                UVALUE='KEYX' )
       id.keyy = WIDGET_BUTTON( id_2, VALUE='Y-axis', /MENU )
       id.keyd = WIDGET_BUTTON( id_2, VALUE='Curve', /MENU)
       id.curs = WIDGET_BUTTON( id_2, VALUE='Cursor: Active Y-axis', /MENU )
     id_2 = WIDGET_BUTTON( id_menu, VALUE='     Show     ', /MENU )
       id_0 = WIDGET_BUTTON( id_2, VALUE='Legend...', UVALUE='LGND' )
       id_0 = WIDGET_BUTTON( id_2, VALUE='Annotation...', UVALUE='ANNO' )
     id_2 = WIDGET_LABEL( id_menu, VALUE = '                             ')
     id_2 = WIDGET_BUTTON( id_menu, VALUE = '    Help     ', UVALUE='HELP')

     zoomfl = 0
     start = 0
     IF KEYWORD_SET(disc_area) THEN $
      stdbut = cw_bgroup(id.base,  $
                        /EXCLUSIVE, /ROW, $
                        SET_VALUE = zoomfl, $
                        UVALUE = 'CURB', $
                        ["None ",$ 
			 "Cursor position ",$ 
                         "Activate zooming ", $
                         "Select area ", $
                         "Select on X-axis"]) $
     ELSE $
      stdbut = cw_bgroup(id.base,  $
                        /EXCLUSIVE, /ROW, $
                        SET_VALUE = zoomfl, $
                        UVALUE = 'CURB', $
                        ["None ",$ 
			 "Cursor position ",$ 
                         "Activate zooming "])


;   id_plot = WIDGET_BASE( id.base, /COLUMN, /FRAME )  ; no frame - IDL bug
   id_plot = WIDGET_BASE( id.base, /COLUMN )
     id_1 = WIDGET_BASE( id_plot, /ROW, XSIZE = 630 )
       id.draw = WIDGET_DRAW( id_1, XSIZE = 600, $
                 YSIZE = 500, /BUTTON_EVENTS, /MOTION_EVENTS, $
		 UVALUE='DRAW', RETAIN = 2 )
       id.sldy = WIDGET_SLIDER( id_1, MIN = -100, MAX = +100, /DRAG, $
                 /VERTICAL, /SUPPRESS_VALUE, UVALUE='SLDY' )
     id_1 = WIDGET_BASE( id_plot, /ROW, XSIZE = 630 )
       id.sldx = WIDGET_SLIDER( id_1, MIN = -100, MAX = +100, /DRAG, $
                 /SUPPRESS_VALUE, XSIZE=600, UVALUE='SLDX' )


     XMENU, [ BM ], id.base, /ROW, XPAD=20, YPAD=20, SPACE=20, $
          UVALUE=['XEXP','XCON','XFUL','YEXP','YCON','YFUL','QUIT']

;   id_cursor = WIDGET_BASE( id.base, /ROW, /FRAME )  ; no frame - IDL bug
   id_cursor = WIDGET_BASE( id.base, /ROW )
     id.txxb = WIDGET_BASE( id_cursor, /ROW )
       id_0 = WIDGET_LABEL( id.txxb, VALUE = 'X: ', FONT = font )
       id.txtx = WIDGET_TEXT( id.txxb, VALUE = '',  FONT = font, $
                 UVALUE='TXTX' )
       id_0 = WIDGET_LABEL( id.txxb, VALUE = 'Y: ', FONT = font )
       id.txty = WIDGET_TEXT( id.txxb, VALUE = '', FONT = font, $
                 UVALUE='TXTY' )

; set default values

   WIDGET_CONTROL, id.sldx, SET_VALUE = 0
   WIDGET_CONTROL, id.sldy, SET_VALUE = 0
;   WIDGET_CONTROL, id.txxb, SENSITIVE = 0

; realize widget, don't manage yet

   WIDGET_CONTROL, id.base, TLB_SET_XOFFSET=90, TLB_SET_YOFFSET=30, /REALIZE
   WIDGET_CONTROL, id.draw, GET_VALUE = window
   id.window = window

; handles to hold data and parameter values

   id.vpar = HANDLE_CREATE()
   id.data = HANDLE_CREATE()
   id.axes = HANDLE_CREATE()
   id.annotation = HANDLE_CREATE()
   id.legend = HANDLE_CREATE()

   RETURN
   END
;
;-----------------------------------------------------------------------
;
        PRO PIA_XPLOT, x, y, yerr, flagged=flagged, $
           GROUP=group, OVERPLOT=overplot, FOURIER=fourier,            $
           ANNOTATION=annotation, LEGEND=legend,                       $
           CHARSIZE=charsize, SUBTITLE=subtitle, TITLE=title,          $
           XCHARSIZE=xcharsize, XSTYLE=xstyle, XTHICK=xthick,          $
           XTITLE=xtitle, XTYPE=xtype, yerrstyle=yerrstyle,            $
           YCHARSIZE=ycharsize, YSTYLE=ystyle, YTHICK=ythick,          $
           YTITLE=ytitle, YTYPE=ytype,                                 $
           COLOR=color, LINESTYLE=linestyle, PSYM=psym,                $
           SYMSIZE=symsize, THICK=thick, DISC_AREA=disc_area, MODAL=modal
;
;       This is the main procedure.
;
        COMMON PIA_XPLOT, active_base
        COMMON zoom_para, zoomfl, xpos, ypos, start, xrg, yrg, pos1, pos2
   	COMMON zoom_discareas, disc_flags, all_disc_areas

; set modal type in any case 
       IF KEYWORD_SET(modal) THEN modal = 1 ELSE $
        IF KEYWORD_SET(disc_area) THEN modal = 1 ELSE modal = 0

       IF modal THEN BEGIN
	 IF not KEYWORD_SET(group) THEN group = !pia_top_wid 
       ENDIF ELSE IF not KEYWORD_SET(group) THEN group = ''
;
	IF KEYWORD_SET(FLAGGED) THEN disc_flags = flagged ELSE disc_flags=-1
	all_disc_areas = -1
; check data arguments
; change (CG) to question on N_ELEMENTS(x) because X = 0.0 means keyword 
; not set
       IF N_ELEMENTS(x) EQ 0 THEN BEGIN
           MESSAGE, /INFO, "No data specified - can not plot anything"
           RETURN
       ENDIF

       IF N_ELEMENTS(y) GT 0 THEN BEGIN
           yp = y
           xp = x
       ENDIF ELSE BEGIN
           yp = x
           xp = FINDGEN(N_ELEMENTS(x))
       ENDELSE

	IF max(!PIA_XPLOT_LIMIT) NE 0 AND $
	   max(!PIA_XPLOT_LIMIT) LT N_ELEMENTS(xp) THEN BEGIN
	  limits = !PIA_XPLOT_LIMIT
	  PIA_DISP_PRINT_FILE, '', TITLE='PIA_XPLOT Warning (unique)', $
	  	TEXT=['The displayed XPlot(s) is/are partially limited to', $
	  	     'the elements: ['+STRTRIM(limits(0),2)+','+$
	  	     		      STRTRIM(limits(1),2)+']'], $
	  	unique='PIA_XPLOT_WARNING', WIDTH=20, HEIGHT=8, GROUP=group, $
		MODAL=modal
	  	
	  xp = xp(limits(0):limits(1))
	  yp = yp(limits(0):limits(1))
	ENDIF 

       IF N_ELEMENTS(yerr) GT 0 THEN yerrp=yerr ELSE yerrp = [ -1 ]

       IF N_ELEMENTS(psym) EQ 1 THEN $
         IF psym GT 8 AND psym NE 10 THEN psym=1
; fourier transformation

       IF KEYWORD_SET(FOURIER) THEN BEGIN
           ntot = N_ELEMENTS(yp) 

	   IF ntot GT 30000 THEN BEGIN
		xp = xp(0:29999)
		yp = yp(0:29999)
		MESSAGE, /INFO, 'Warning: only first 30000 points taken'
		MESSAGE, /INFO, 'Very (!!) low frequencies taken out '
		ntot = 30000
	   ENDIF	   
	    
           yp = ABS(FFT(yp, -1, /OVERWRITE))
           xp = xp - xp(0)
           tsam = xp(1) - xp(0)
           n21 = ntot/2 + 1
           freq = indgen(ntot)
           freq(n21) = n21 - ntot + findgen(n21-2)
           freq = freq / (ntot * tsam)
           xp = shift(freq, -n21)
           yp = shift(yp, -n21)
; show only the positive frequencies
           xp = xp(n21-2:ntot-1)
           yp = yp(n21-2:ntot-1)
           yerrp = 0
       ENDIF

; create new instance if necessary
; or check whether there is an active widget which can be overplotted

       new_widget = NOT KEYWORD_SET(OVERPLOT)
       IF KEYWORD_SET(disc_area) THEN disc_area=1 ELSE disc_area=0
       IF new_widget THEN BEGIN
           PIA_XPLOT_WIDGET, id, disc_area=disc_area, modal=modal, $
	     GROUP_LEADER=group
           active_base = id.base
           overplot = 0
       ENDIF ELSE BEGIN
           IF WIDGET_INFO( active_base, /VALID_ID) THEN BEGIN
               WIDGET_CONTROL, active_base, GET_UVALUE=id
           ENDIF ELSE BEGIN
               MESSAGE, /INFO, $
                        "There is no XPLOT widget active - can not overplot"
               RETURN
           ENDELSE
       ENDELSE

; save keywords and new data

       IF new_widget THEN BEGIN
           x_keywords = {                                              $
                        charsize:       !p.charsize,                   $
                        subtitle:       !p.subtitle,                   $
                        title:          !p.title,                      $
                        xcharsize:      !x.charsize,                   $
                        xstyle:         !x.style OR 2,                 $
                        xthick:         !x.thick,                      $
                        xtitle:         !x.title,                      $
                        xtype:          !x.type,                       $
                        xrange:         [0.,0.]	                       $                       $
                         }
       ENDIF ELSE BEGIN
           x_keywords = PIA_XPLOT_GETXKEY( id.axes )
       ENDELSE
       IF KEYWORD_SET(charsize)  THEN x_keywords.charsize  = charsize
       IF KEYWORD_SET(subtitle)  THEN x_keywords.subtitle  = subtitle
       IF KEYWORD_SET(title)     THEN x_keywords.title     = title
       IF KEYWORD_SET(xcharsize) THEN x_keywords.xcharsize = xcharsize
       IF KEYWORD_SET(xstyle)    THEN x_keywords.xstyle    = xstyle
       IF KEYWORD_SET(xthick)    THEN x_keywords.xthick    = xthick
       IF KEYWORD_SET(xtitle)    THEN x_keywords.xtitle    = xtitle
       IF KEYWORD_SET(xtype)     THEN x_keywords.xtype     = xtype ELSE $
				      x_keywords.xtype = 0         

       IF overplot LE 0 THEN BEGIN
           y_keywords = {                                              $
                        ycharsize:      !y.charsize,                   $
                        ystyle:         !y.style OR 2,                 $
                        ythick:         !y.thick,                      $
                        ytitle:         !y.title,                      $
                        ytype:          !y.type,                       $
                        yrange:         [0.,0.]	                       $
                         }
       ENDIF ELSE BEGIN
           y_keywords = PIA_XPLOT_GETYKEY( id.axes, overplot )
       ENDELSE
       IF KEYWORD_SET(ycharsize) THEN y_keywords.ycharsize = ycharsize
       IF KEYWORD_SET(ystyle)    THEN y_keywords.ystyle    = ystyle
       IF KEYWORD_SET(ythick)    THEN y_keywords.ythick    = ythick
       IF KEYWORD_SET(ytitle)    THEN y_keywords.ytitle    = ytitle
       IF KEYWORD_SET(ytype)     THEN y_keywords.ytype     = ytype ELSE $
				      y_keywords.ytype = 0         

       d_keywords = {                                                 $
                       legend:         '',                            $
                       color:          -1,                            $
                       linestyle:      !p.linestyle,                  $
                       yerrstyle:      0,                             $
                       psym:           !p.psym,                       $
                       symsize:        !p.symsize,                    $
                       thick:          !p.thick,                      $
                       noerrbar:       !NO_ERRBARS	$
                     }
       IF KEYWORD_SET(legend)    THEN d_keywords.legend    = legend
       IF KEYWORD_SET(color)     THEN d_keywords.color     = color
       IF KEYWORD_SET(linestyle) THEN d_keywords.linestyle = linestyle
       IF KEYWORD_SET(yerrstyle) THEN d_keywords.yerrstyle = yerrstyle
; change(CG) to allow psym=0 call
       IF N_ELEMENTS(psym) EQ 1  THEN d_keywords.psym      = psym
       IF KEYWORD_SET(symsize)   THEN d_keywords.symsize   = symsize
       IF KEYWORD_SET(thick)     THEN d_keywords.thick     = thick

       ; initialize viewing parameters before saving data
       vpar = { xm: 0.0 , ym: 0.0 , xwd: 1.0, ywd: 1.0, currenty:0, zoomfl:0, $
       		modal:modal, group:group }

       IF NOT KEYWORD_SET(annotation) THEN annotation=''

       PIA_XPLOT_SAVE, id, xp, yp, yerrp, vpar, OVERPLOT=overplot, $
                       X_KEYWORDS=x_keywords, $
                       Y_KEYWORDS=y_keywords, $
                       D_KEYWORDS=d_keywords, $
                       ANNOTATION=annotation

; draw the data and save viewing parameters

       PIA_XPLOT_REDRAW, id, vpar
       HANDLE_VALUE, id.vpar, vpar, /SET, /NO_COPY


       IF new_widget THEN BEGIN
           xp    = 0
           yp    = 0
           yerrp = 0
           WIDGET_CONTROL, id.base, SET_UVALUE = id
           xMANAGER,No_BLOCK=1, "PIA_XPLOT", id.base, GROUP=group, $
                     CLEANUP="PIA_XPLOT_CLEAN"
       ENDIF
       RETURN
       END
