;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PIA_XY2RD
;
; PURPOSE:
;	Transform map x/y coordinates to right_ascension/declination.
;	No checks are performed.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_XY2RD, x, y, trans, r, d
;
; INPUTS:
;	X		The x coordinate values in the map
;
;	Y		The y coordinate values in the map
;
;	TRANS		The transformation parameters structure:
;	.CD		forward transformation matrix
;	.DC		back transformation matrix
;	.CRVAL		reference position (ra,dec)
;	.CRPIX		reference pixel (x,y)
;
; KEYWORD PARAMETERS:
;	(none)
;
; OUTPUTS:
;	R		The right_ascension value(s) in degrees.
;			May be an array.
;
;	D		The declination value(s) in degrees.
;			May be an array.
;
; NOTES:
;	This routine differs from the similar ASTROLIB routine XY2AD
;	in several ways, mainly:
;	(1) The input parameter TRANS is expected to contain the
;	    inverse transformation matrix DC.
;	(2) The definitions of the matrixes CD and DC are not consistent
;	    with the definition of the ASTROLIB matrix CD or its inverse.
;	(3) CRPIX values are consistent with IDL indexing (starting at 0).
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by: D Skaley
;	Oct 95 Version 3.5
;-

	PRO PIA_XY2RD, x, y, trans, r, d

	dtor = !DPI/180.0D

	delta_x = x - trans.crpix(0)
	delta_y = y - trans.crpix(1)
	xsi = trans.cd(0,0)*delta_x + trans.cd(0,1)*delta_y
	eta = trans.cd(1,0)*delta_x + trans.cd(1,1)*delta_y

	crval = trans.crval * dtor
	beta = cos(crval(1)) - eta * sin(crval(1))
	r = atan(xsi, beta) + crval(0)
	gamma = sqrt((xsi^2) + (beta^2))
	d = atan(eta*cos(crval(1))+sin(crval(1)) , gamma)

	r = r*!RADEG
	d = d*!RADEG

	RETURN
	END
