  pro interpol_fpgc, infile, x, y, scy, scz, ERROR=error 

;+
; NAME:
;	INTERPOL_FPGC
;
; PURPOSE:
;	Interpolate Focal Plane Geometry Calibration File coordinates form
;	x,y raster coordinates
;	A linear interpolation between the two FPGC coordinates above and 
;	below the given raster coordinates is performed.
;
; CALLING SEQUENCE:
;	interpol_fpgc, infile, x, y, scy, scz, ERROR=error
;
; INPUT:
; 	infile: name of FPGC file
;	x: raster x-coordinate as from PIA
;	y: raster y-coordinate as from PIA
;
; OUTPUT:
;	scy: FPGC coordinate interpolated to correspond to raster x coordinate
;	scy: FPGC coordinate interpolated to correspond to raster x coordinate
;
; OPTIONAL INPUT:
;	none
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output is not valid
;
; PROCEDURES USED:
;	read_fpgc
;
; CALLED ROUTINES:
;	read_fpgc	
;
; MODIFICATION HISTORY:
;       Written by I. Heinrichsen                            Jul. 1995
;
;-
  on_error, 2
;
; initialize error status
	error = 1
; initialize debug
; 		debug=1 give detailed print-out
;		debug=0 give no print-out
	debug = 1

; check for parameters
 npar   = N_params()
 if npar EQ 0 then goto, nopar
; read FPGC file
	read_fpgc, infile, npts, nlin, dst_pts, dst_lin, rota, $
                  rline, rcolumn, ycoor, zcoor, ERROR=read_error 

	if (read_error ne 0) then goto, readerror
;
; check for raster size compatibility
;
	if ((fix(x)+1) gt npts) then goto, xerror
	if ((fix(y)+1) gt nlin) then goto, yerror
;
;	extract coordinates
;

	yl=fix(y)
	yh=fix(y+1)
	yfr=y-fix(y)

	xl=fix(x)
	xh=fix(x+1) 
	xfr=x-fix(x)

	for i=0, npts*nlin-1 do begin
		if ((rline(i) eq xl) and (rcolumn(i) eq yl)) then begin
			scyll=ycoor(i)
			sczll=zcoor(i)
			if (debug eq 1) then print, "found ll:", i, xl, yl, scyll, sczll
		endif
		if ((rline(i) eq xl) and (rcolumn(i) eq yh)) then begin
			scylh=ycoor(i)
			sczlh=zcoor(i)
			if (debug eq 1) then print, "found lh:", i, xl, yh, scylh, sczlh
		endif
		if ((rline(i) eq xh) and (rcolumn(i) eq yl)) then begin
			scyhl=ycoor(i)
			sczhl=zcoor(i)
			if (debug eq 1) then print, "found hl:", i, xh, yl, scyhl, sczhl
		endif
		if ((rline(i) eq xh) and (rcolumn(i) eq yh)) then begin
			scyhh=ycoor(i)
			sczhh=zcoor(i)
			if (debug eq 1) then print, "found hh:", i, xh, yh, scyhh, sczhh
		endif
	endfor
	check=n_elements(scyll)+n_elements(scylh) $
			+n_elements(scyhl)+n_elements(scyhh)
	if (check ne 4) then goto, incomp 
;
; 	select qudrant
;
	if (yfr le 0.5) then begin
		scy=(scyhl-scyll)*xfr+scyll
	endif else begin
		scy=(scyhh-scylh)*xfr+scylh
	endelse
	if (xfr le 0.5) then begin
		scz=(sczlh-sczll)*yfr+sczll
	endif else begin
		scz=(sczhh-sczhl)*yfr+sczhl
	endelse


	if (debug eq 1) then begin	
		print, "Raster coordinates input: X=",x," Y= ",y
		print, "FPGC Y-coordinates:", scyll, scylh, scyhl, scyhh
		print, "FPGC Z-coordinates:", sczll, sczlh, sczhl, sczhh
		print, ycoor, zcoor
		print, "Interpolated FPGC coordinates: SCY=",scy, " SCZ= ",scz
	endif
;
; everything ok.
	error = 0
	return
	
nopar:
	print,"interpol_fpgc, infile, x, y, scy, scz, ERROR=error"
	return
xerror:
	print, "INTERPOL_FPGC: X-coordinate incompatible with FPGC raster size"
	print, "X= ", x, "NPTS= ", npts
	return
yerror:
	print, "INTERPOL_FPGC: Y-coordinate incompatible with FPGC raster size"
	print, "Y= ", y, "NLIN= ", nlin
	return
incomp:
	print, "INTERPOL_FPGC: Coordinate input incompatible with FPGC file"
	return

readerror:
	print, "ANAL_FPGC: Unable to read file:", infile
	return
end
