;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO LIN_VOLTAGES, TMPERD, TMPHDR, SILENT=silent
;+
; NAME:
;	LIN_VOLTAGES
;
; PURPOSE:
;	Apply voltage linearization onto a phterd structure 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	LIN_VOLTAGES, Tmperd, Tmphdr, Silent=silent
;
; INPUTS:
;	Tmperd:	A temporary PHTERD structure
;	Tmphdr: The header to be modified 
;	
; OUTPUTS:
;	Tmperd: The temporary structure after voltage linearization
;	Tmphdr: The modified header
;	
; INPUT KEYWORD:
;	Silent: Avoid window messages if set 
;
; COMMON BLOCKS:
;	phtcs
;	
; PROCEDURES CALLED:
;	PIA LIB: linear_cor
;	         read_cs
;	
;	
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel (ESA-SAI)
;	Oct 95 Version 3.5
;	Keyword silent added (CG)		July 1996
;-
@phtcs
        ON_ERROR, 2

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 2 THEN MESSAGE, $
          'Syntax:  LIN_VOLTAGES, TMPERD, TMPHDR'

	csname = 'PSTA'+strmid(tmperd.admn,4,8)+'.FITS'
	read_cs, csname, cshdr, indcs
	handle_value, phtcs(indcs).HANDLE, phtcst

	CASE tmperd.det of
		'P1': BEGIN 
				subs = 0		    		    
				nrpix = 1
		END
		'P2': BEGIN 
				subs = 1		    		    
				nrpix = 1
		END
		'P3': BEGIN 
				subs = 2		    		    
				nrpix = 1
		END
		'SL': BEGIN 
				IF NOT KEYWORD_SET(SILENT) THEN $
                                 XDISPLAYFILE, "", $
                    		   TITLE = "Linear Correction for S ?? ", $
                    		   WIDTH = 50, $
                    		   HEIGHT = 12, $
                    		TEXT = [" Linear Correction for SS/SL not", $
                            		" yet implemented "]
				RETURN
		END
		'C1': BEGIN 
				subs = 6		    		    
				nrpix = 9
		END
		'C2': BEGIN 
				subs = 7		    		    
				nrpix = 4
		END
		ELSE: BEGIN 	    		    
       				MESSAGE, /INFO, 'Unidentified Detector'
				RETURN
		END
	ENDCASE

	concor = FLTARR(nrpix, tmperd.dims)
	conerr = concor

; Frequency in CRELIN files is phtcst.freq - 1
	indsub = WHERE(phtcst.subs eq subs, n)
	freq = phtcst.freq(indsub(tmperd.ordr)) - 1	

; Now begin with linear correction
	IF n NE 0 THEN BEGIN
	
; Separate P from C because of the table structure
	   IF nrpix EQ 1 THEN BEGIN
			  i = 0
			  linear_cor, tmperd.det, $
				reform(tmperd.conp(i,*)), subs, freq, $
				cor, err
			  concor(i,*) = cor

           ENDIF ELSE BEGIN
	
		FOR I=0,nrpix-1 DO BEGIN
			  linear_cor, tmperd.det, $
				reform(tmperd.conp(i,*)), i, freq, $
				cor, err
			  concor(i,*) = cor
                ENDFOR
           ENDELSE

	ENDIF
; Replace conp contents, log it into the tmphdr
	IF (N_ELEMENTS(tmperd.conp(0:nrpix-1,*)) EQ    $
	    N_ELEMENTS(concor(0:nrpix-1,*))) THEN BEGIN

		tmperd.conp(0:nrpix-1,*) = concor  
			  
		junk = 'PR_LINC =          '+$
			'          1 / Linear correction applied'

		newl = STRING(REPLICATE(32B, 80))
		STRPUT, newl, junk, 0			  
        	KEYWORD = STRMID(tmphdr, 0, 8)
        	XTENSION = WHERE(KEYWORD EQ 'XTENSION')
        	hdrlen = N_ELEMENTS(tmphdr)
        	tmphdr = [ tmphdr( 0 : XTENSION(0) - 1), newl, $
                   tmphdr( XTENSION(0) : HDRLEN - 1 )]

	ENDIF

	RETURN
	END
