;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Linear_Cor, SubInst, In_Volt, Pixel, Freq, Out_Volt, Error
;+
;  NAME:
;	LINEAR_COR
;
;  PURPOSE:
;	Corrects the non-linearities of the integration ramps
;
;  CATEGORY:
;	PIA - processing
;
;  CALLING SEQUENCE:
;	LINEAR_COR, Subinst, In_volt, Pixel, Freq, Out_Volt, Error
; 
;  INPUTS:
;	SubInst: Phot subinstrument identifier string:
;	   C1:PHT-C100  C2:PHT-C200  P:PHT-P    S:PHT-S
;	In_Volt: Input voltages array corresponding to the pixel
;	Pixel: Pixel number to correct for, in the same order as the SPD. 
;	   (For PHT-P, corresponds to the different detectors: 
;	   0: P1  1: P2  2:P3)
;	Freq: The frequency of the measurement:
;	   0: 1KHz   1: 4KHz  2: 8KHz
;
;  OUTPUTS:
;	Out_Volt: Array of corrected voltages per pixel/detector
;	Error: Array of associated uncertainties
;
;  COMMON BLOCKS:
;	phtcrelin
;
;  PROCEDURES USED:
;	PIA LIB: read_crelin
;
;  MODIFICATION HISTORY:
;	Written by:	Huw Morris
;	Modified by:	C. Gabriel
;	Oct 95 Version 3.5
;	Bug on error interpolation fixed (CG)		April 1996
;	
;-
@phtcrelin 	

	On_Error, 2                   ; Return to calling prodecure


; Create Error and Out_Volt arrays of the correct dimension first in 
; case of premature exit from this procedure
	Error = FLTARR(N_ELEMENTS(In_Volt))
	Out_Volt = In_Volt

; Each subinstrument has a seperate Linear Correction file. Determine the
; correct file from the SunInst parameter
	CASE SubInst OF

	  'C1': BEGIN
	    Max_Pix = 9
; Read the Linear Correction File if data not already present
	    IF N_ELEMENTS(pc1crelin) EQ 0 THEN read_crelin
	    Table = pc1crelin
	  END

	  'C2': BEGIN
	    Max_Pix = 4
; Read the Linear Correction File if data not already present
	    IF N_ELEMENTS(pc2crelin) EQ 0 THEN read_crelin
	    Table = pc2crelin
	  END

	  'P1': BEGIN 
	    Max_Pix = 1
; Read the Linear Correction File if data not already present
	    IF N_ELEMENTS(ppcrelin) EQ 0 THEN read_crelin
	    Table = ppcrelin
	  END

	  'P2': BEGIN 
	    Max_Pix = 2
; Read the Linear Correction File if data not already present
	    IF N_ELEMENTS(ppcrelin) EQ 0 THEN read_crelin
	    Table = ppcrelin
	  END

	  'P3': BEGIN 
	    Max_Pix = 3
; Read the Linear Correction File if data not already present
	    IF N_ELEMENTS(ppcrelin) EQ 0 THEN read_crelin
	    Table = ppcrelin
	  END

	  'SL': RETURN                     ; No Linear Correction for PHT-S

; If none of the SubInstruments are recognised, return with an error
	  ELSE: BEGIN
	    MESSAGE, 'Unknown subinstrument.'
	    RETURN
	  ENDELSE

	ENDCASE

; Check the pixel number is valid
	IF (Pixel GE Max_Pix) THEN BEGIN
	  MESSAGE, 'Pixel number out of range.'
	  RETURN
	ENDIF

; Create a table of Voltages (X axis) from -1.2 to +1.2V against correction
; values (Y axis)
	X = (FINDGEN(121) / 50.0) - 1.2
	Y = Table(*, Pixel, Freq, 0)

; Get the correction factors by interpolating the input voltages
	Correction = INTERPOL(Y, X, In_Volt)
                                                                 
; Extract the uncertainty value 
	Error_Table = Table(*, Pixel, Freq, 1)

; Look if (some) uncertainties are filled in
	a = where(Error_Table NE 0., nn)
	IF nn NE 0 THEN $
		Error = INTERPOL(Error_Table, X, In_Volt) ELSE $
		Error = INTARR(N_ELEMENTS(Correction))

; The corrected voltages are the input voltages + the correction factors
	Out_Volt = In_Volt + Correction

	END

