;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;! This routine belongs to the PHT-Interactive Analysis package, PIA.        !
;!                                                                           !
;! PIA was developed by the European Space Agency (ESA/ESTEC)                !
;! in collaboration with                                                     !
;!   Max-Planck-Institute for Nuclear Physics (MPIK), Heidelberg, Germany,   !
;!   Dublin Institute for Advanced Studies (DIAS), Dublin, Ireland,          !
;!   Rutherford Appleton Laboratory, (DRAL), Chilton, UK, and                !
;!   Astronomy Institute Potsdam, (AIP), Potsdam, Germany,                   !
;! with support from Max-Planck-Institute for Astronomy (MPIA), Heidelberg.  !
;!                                                                           !
;! It shall be used for scientific analysis and calibration of ISOPHOT data. !
;!                                                                           !
;! Its use and distribution are free, as long as this header text is not     !
;! removed, or changed. When publishing data reduced with this analysis      !
;! package, please mention this in the acknowledgement.                      !
;!                                                                           !
;! No warranties for installation / support/ maintenance can be given.       !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;     MANN
;
; PURPOSE:
;       Perform Trend-test by Mann
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	mann, y, alpha, test
;
; INPUTS:
;	y: 	array of data to be tested for trend (time sequence)
;	alpha: 	certainty level for test
;
; KEYWORD PARAMETERS:
;
;	Ctest		Test statistics parameter for Y,
;			for use with Kendall statistics
;	Cstar		Criticality parameter for C, alpha
;	Ualpha		Quantile for confidence level alpha
;	ERROR		Returns an error status:
;						0 = no error
;						1 = error: output is not valid
;	optimize	If set and number of elements > 128, reduction by
;			taking averages of neighbour elements
;
; OUTPUTS:
;	tu:	true, if upward trend confirmed with alpha confidence
;		false, otherwise
;	td:	true, if downward trend confirmed with alpha confidence
;		false, otherwise	
;
; PROCEDURES CALLED
;	INTERNAL:	sign
;			hast
;
; RESTRICTIONS
;	The number of elements of the tested array must be grater than 10
;
; MODIFICATION HISTORY:
;       Written by:	Ingolf Heinrichsen (MPIK)		Nov. 95
;	Limit of at least 10 points relaxed to 7 (CG)		January 1996
;
;-
;
; *****************************************************
;
; Sign function
;
; *****************************************************
;
  function sign, x
	if x gt 0. then a=1
	if x eq 0. then a=0
	if x lt 0. then a=-1
	return, a
  end



; ******************************************************
;
; Approximation of the Quantiles of the normal distribution 
; after Hastings
;
; ******************************************************
;
  function hast, alpha
	gamma=1-alpha
; check for validitiy of approximation
	if gamma lt 0.5 then goto, error
	if gamma gt 1.0 then goto, error
	t=sqrt(-2.*alog(1-gamma))
	a0=2.515517
	a1=0.802853
	a2=0.010328
	b1=1.432788
	b2=0.189269
	b3=0.001308
	ugamma=t-(a0+a1*t+a2*t*t)/(1+b1*t+b2*t*t+b3*t*t*t)
return, ugamma
error:
	message, "Hastings approximation only valid for 0.5 < gamma < 1.0"
	ugamma=-1.
return, ugamma
end
	

; ******************************************************
;
; Main programme for Mann statistic test
;
; ******************************************************
;
pro mann, y, alpha, tu, td ,Ctest=Ctest, Ccrit=Ccrit, $	  
Ualpha=Ualpha, error=error, optimize=optimize


	IF KEYWORD_SET(optimize) THEN BEGIN

	  nele = N_ELEMENTS(y)

	  WHILE nele GT 256 DO BEGIN
		nfactor = nele / 128 
		nelem = indgen(nele)
		nmod = nelem mod nfactor
	   	ind = WHERE(nmod EQ 0, nn)
		ynew = FLTARR(nn)
	   	FOR i=0, nn - 2 DO $
	   	  ynew(i) = avg(y(ind(i):ind(i+1)-1))
		  ynew(nn-1) = avg(y(ind(nn-1):*))
		y = ynew
	  	nele = N_ELEMENTS(y)
	  ENDWHILE

	ENDIF

; initialize error
	error=1

; init debug
	debug=0

; check for parameter
	npar = N_params()
	if npar lt 4 then goto, nopar

; check for statistical feasibility
	N=n_elements(y)
	if N lt 7 then goto, nonstat

; initilize
	c=0L
	true=1
	false=0

; calculate C
	for i = 0, N-2 do begin
		for j=i+1,N-1 do begin
			c=c+sign(y(j)-y(i))
		endfor
	endfor

; set output parameter
	if debug eq 1 then print, "C=",c
	ctest=c

; Calculate Cstar
	Cstar=c/sqrt(N*(N-1)*(2*N+5)/18.)
	if debug eq 1 then print, "Cstar=",cstar
	Ccrit=Cstar

; get alpha quantile
	ualpha=hast(alpha)
	if ualpha lt 0. then goto, alphaerror
	if debug eq 1 then print, "Ualpha=",Ualpha

; perform check of trend hypothesis
	tu=false
	td=false
	if Cstar gt ualpha then tu = true
	if Cstar lt -1.*ualpha then td = true 
	if debug eq 1 then begin
		if tu then print, "Trend goes Up"
		if td then print, "Trend goes Down"
		if not (tu + td) then print, "No trend"
	endif
; all OK 
error=0
return

;
; error handling
;
nopar:
	print,"mann, y, alpha, tu, td, Ctest=Ctest ,Ccrit=Ccrit, "
	print,"      Ualpha=Ualpha, ERROR=error"
	return

nonstat:
	message,"Not enough data points to perform statistics"
	return

alphaerror:
	message, "Alpha out of range: 0 < alpha < 0.5"
	return
end















































