
;-------------------------------------------------------------------------
;+
; NAME:  MapAAP2Drizzle
;
; PURPOSE:
;       Make Drizzle Script for IRAF from ISOPHOT / PIA / IAAP - Map Data
;
; VERSION: Jan. 20, 1999
;
; CATEGORY:
;       ISOPHOT, Image Analysis
;
; CALLING SEQUENCE:
;       MapAAP2Drizzle,    FileName_or_FileList
;
; INPUTS:
;       FileName_or_FileList : Name of IAAP DiskFileName (CharacterString !)
;                                                     or
;                              List of IAAP Disk filenames (CharacterArray !)
;
; KEYWORD PARAMETERS:
;      ScriptName         : (List of)Diskfilename(s) for IRAF Script(s)
;      ImageName          : Name of IRAF image (= map) to be created by script
;      IRAFCommandName    : Name of file with IRAF commands
;      Identifier         : Identifier keyword of map
;      Scale              : Ratio of final to original image pixel size
;      MapPixelSize       : final image pixel size (map pixel size) in arcsec
;      PositAngle         : Astron. Position Angle of Y-axis of final image
;                           (degrees ccw, = 0 means RA / Dec coordinate system)
;      DrizzleFraction    : Scalefactor for Drizzle's drop size relative
;                           to the detector pixel size
;      MapRACenter,
;      MapDECCenter       : predefined RA / DEC of map center
;      OutImageExtend     : increase no of pixels of OutputImage
;      NPixX              : size of final map in X direction
;      NPixY              : size of final map in Y direction
;      FillValue          : predefined value for empty pixels in the output map
;      IgnorePixel        : ISOPHOT pixel indeces to be ignored for mapping
;      Parallel           : no of parallel stripes (2, 4, 8, 16, etc...) for
;                           hierarchical processing of non-overlapping stripes
;                           scripts of the same level can be run in parallel
;      DetectorFlatField  : multiplicative (!) flatfield correction factors
;                           for each detector pixel;
;                           to be applied to the detector fluxes/surf.brightnesses
;      AAPGlobalFlatField : multiplicative (!) global flatfield correction factors
;                           for each AAP file;
;      /AddScaleToName    : File/Script names contain image scale
;      /SurfaceBrightness : result is in surf. brightness instead of fluxes
;      /Flux              : result is in units of flux (Default !)
;      /Meanvalues,
;      /Medianvalues      : use the Means or the Medians from PIA AAP structure
;      /SingleAAP         : each input IAAP files gets its own script file
;      /EveryPixel        : one script and map for each detector pixel
;      /SimpleProjection  : output image is made using simple RA/DEC projection
;      /GnomonicProjection: output image is made using gnomonic RA/DEC projection (Default!)
;      /Spacecraftcoords  : align map along ISO spacecraft Y-Z system
;      /IgnoreRollAngle   : pixel rollangle not used for mapping
;      /OutputFrameShifts : Shifts are computed in output frame
;      /FITS              : IRAF scripts automagically write FITS files
;      /EXECUTE           : immediately start IRAF map making from within IDL
;      /DEBUG             : make some debuging output
;
; OUTPUTS:
;       none; IRAF Script(s) is/are written to disk file(s)
;
; EXAMPLES:
;    MapAAP2Drizzle , ['C20144301205pai.IAAP'] , Scale=0.222222, /GNOMONIC
;
;    MapAAP2Drizzle , ['C10144301206paiflt.IAAP'] , /GNOMONIC, MapPixelSize=10., /EXEC, /FITS
;
; NOTES:
;    Requires IRAF V 2.11 and STSDAS 2.0.1
;    Load all of the following from IRAFs login.cl before startup :
;           stsdas
;           toolbox
;           imgtools
;           analysis
;           dither
;
; MODIFICATION HISTORY:
;
;       (C)  M. Stickel    1997-1999    MPIA / IDC  1997-1999
;
;                6. August 97  : intial Version
;               17. August 97  : final image NOT aligned along RA / DEC ;
;                                usage of RollAngle and external Position
;                                Angle in drizzle
;               10. October 97 : usage of PIA  Mean / Median fluxes
;                                surface brightnesses
;               13. October 97 : Single mode : for each datafile a scriptfile
;               17. October 97 : Parallel mode : non-overlapping regions
;                                drizzled in parallel
;                                (basic idea suggested by Sibylle Peschke)
;               16. November 97 : Process results of ParallelMode also in Parallel
;               28. November 97 : A complete map for each pixel
;               29. November 97 : FlatField Correction factors for each pixel
;                2. December 97 : Calling Sequence put into header of script(s)
;                7. December 97 : external definition of MapCenter coordinates
;               02. February 98 : ask IRAF to write out FITS files
;               11. March    98 : use average Rollangle for Image in Y/Z system
;               09. Sept.    98 : artificial increase of output image size
;               03. Nov.     98 : now standalone, no more PIA necessary
;                                 if changes in pixel/aperture size are
;                                 necessary, see subroutine 'MakePixelCoord'
;               03. Nov.     98 : adapted to STSDAS DRIZZLE Version 1.2 (February 1998)
;               16. Nov.     98 : ignore selected pixels for map making
;               25. Nov.     98 : user defined map size
;               07. Dec.     98 : add image scale to names of script/image file
;               17. Dec.     98 : add projection header keywords
;               21. Dec.     98 : add identifier parameter / keyword to header
;               02. Jan.     99 : gnomonic projection now default,ignore rollangle,
;                                 user-defined map pixel size [arcsec] ;
;                                 global aap rescaling fixed; general clean-up
;               18. Jan.     99 : direct IRAF start
;               20. Jan.     99 : fillvalue added
;
;		27. May      99 : wfits command with *.imh to avoid error (CG) 
;		28. May      99 : keywords changed for using discrimination (CG) 
;
;             THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY !!
;
;-
;--------------------------------------------------------------------------

;-------------------------------------------------------------------------------------------


;-------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------





PRO  PrintUsage
;~~~~~~~~~~~~~~

print, '    '
print, ' ... call as  MapAAP2Drizzle    FileName_or_FileList '
print, '                  ScriptName      = Resulting ScriptName '
print, '                  IRAFCommandName = File with IRAF commands '
print, '                  ImageName       = Resulting ImageName '
print, '                  Identifier      = Map Identifier '
print, '                  Scale           = Relative Pixel Scale '
print, '                  MapPixelSize    = Map Pixel Size [arcsec] '
print, '                  DrizzleFraction = Relative Dropsize Scale '
print, '                  PositAngle      = Map Position Angle '
print, '                  Parallel        = No of Scripts to be run in Parallel'
print, '                  DetectorFlatField  = Multipl. Flatfield for each pixel '
print, '                  AAPGlobalFlatField = Multipl. Flatfield for each AAP '
print, '                  IgnorePixel     = unused pixel indeces'
print, '                  MapRACenter     = Reference RightAscension'
print, '                  MapDECCenter    = Reference Declination'
print, '                  IncreaseSize    = no pixel to increase size of OutputImage '
print, '                  NPixX,NPixY     = final X/Y size of maps '
print, '                  /SURFACEBRIGHTNESS, /FLUX, /MEANVALUES, /MEDIANVALUES '
print, '                  /GNOMONICPROJECTION, /SIMPLEPROJECTION, /IGNOREROLLANGLE '
print, '                  /EVERYPIXEL, /SINGLEAAP, /OUTPUTFRAMESHIFTS,             '
print, '                  /ADDSCALETONAME, /FITS, /DEBUG '
print, '    '

Return
END



;-------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------



PRO  PrintVersion, Version
;~~~~~~~~~~~~~~~~~~~~~~~~~

print, '                                                  '
print, ' *************************************************'
print, '                                                 '
print, '    ' + Version + '  '
print, '                                                 '
print, ' *************************************************'
print, '                                                 '

Return
END



;-------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------



PRO  Read_AAP_Data,      FileName_or_FileList, AAP_STRUCTURE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FORWARD_FUNCTION   ISNUMBER

@phtaap

; /* read data files */

If (ISNUMBER (FileName_or_FileList))  $
   then begin
      print, ' <MapAAP2Drizzle> ... using IAAP bufer ' ,  FileName_or_FileList
      pia_load_iaap,  phtaap(FileName_or_FileList),   AAP_STRUCTURE
   endif else begin
      print, ' <MapAAP2Drizzle> ... using file ' , FileName_or_FileList
      restore , FileName_or_FileList
      AAP_STRUCTURE = phtiaap
   endelse

Return
END


;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro  Copy_AAP_Data , AAP_STRUCTURE, AAPName, Detector, Aperture,      $
                     RACenter, DecCenter,                             $
                     RollAngle, SurfBright, Flux, Flag,               $
                     RelChopPos, ChopStepID, RastPntID,               $
                     FlatField,                                       $
                     MEANVALUES=MEANVALUES, MEDIANVALUES=MEDIANVALUES
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



; /* copy data from aap structure and apply flatfield if necessary */

AAPName    = aap_structure.admn

Detector   = aap_structure.det

Aperture   = aap_structure.aper

RACenter   = aap_structure.ra

DecCenter  = aap_structure.dec

RollAngle  = aap_structure.roll

Flag       = aap_structure.flag

RelChopPos = aap_structure.cpos

ChopStepID = aap_structure.cstp

RastPntID  = aap_structure.rpid


IF (NOT Keyword_Set (MEANVALUES))  AND  (NOT Keyword_Set (MEDIANVALUES)) $
   THEN BEGIN
      print, ' <MapAAP2Drizzle> ... using MEAN data '
      SurfBright = aap_structure.mnbr
      Flux       = aap_structure.mnfl
   ENDIF

IF (Keyword_Set (MEANVALUES))  $
   THEN BEGIN
      print, ' <MapAAP2Drizzle> ... using MEAN data '
      SurfBright = aap_structure.mnbr
      Flux       = aap_structure.mnfl
   ENDIF

IF (Keyword_Set (MEDIANVALUES))  $
   THEN BEGIN
      print, ' <MapAAP2Drizzle> ... using MEDIAN data '
      SurfBright = aap_structure.mdbr
      Flux       = aap_structure.mdfl
   ENDIF

If (  (WHERE (FlatField (0:(N_Elements(FlatField)-1)) NE 1.0))(0)  NE  -1  )  $
   then begin
      Dimen  = Size (Flux)
      NPixel = Dimen(1)
      NData  = Dimen(2)

      If (N_Elements(FlatField)  GE  NPixel)  $
         then begin
            FlatFieldVec = FlatField (*)

            print, ' <MapAAP2Drizzle>  ... doing Flatfield Correction with '
            print, ' <MapAAP2Drizzle>      ' , FlatFieldVec (0:NPixel-1)
         endif else begin
            print, ' <MapAAP2Drizzle>  !! WARNING !!  '
            print, ' <MapAAP2Drizzle>  !! WARNING !!  Flatfield Correction Vector too short !'
            print, ' <MapAAP2Drizzle>  !! WARNING !!  No Flatfielding is done ........ '
            print, ' <MapAAP2Drizzle>  !! WARNING !!  '
            FlatFieldVec    = FltArr (NPixel)
            FlatFieldVec (*)= 1.0
         endelse

      For kk = 0L , NPixel-1L , 1  do begin
         SurfBright (kk,*) = SurfBright (kk,*) * FlatFieldVec (kk)
         Flux (kk,*)       = Flux (kk,*) * FlatFieldVec (kk)
      EndFor

;      For k = 0 , NData-1 , 1  do begin
;         For kk = 0 , NPixel-1 , 1  do begin
;            SurfBright (kk,k) = SurfBright (kk,k) * FlatFieldVec (kk)
;            Flux (kk,k)       = Flux (kk,k) * FlatFieldVec (kk)
;         EndFor
;      EndFor
   endif

Return
End



;----------------------------------------------------------------------
;----------------------------------------------------------------------


   ; /* prepare image and script names */

;   If (N_Elements (InIRAFCommandName) GT 0)  $
;      then  IRAFCommandName = InIRAFCommandName        $
;      else  IRAFCommandName = STRCOMPRESS (AAPName + FileExtension + '.IRAF',/remove_all)

;   IF (i EQ 0)  $
;      THEN BEGIN
;         If (N_Elements (InImageName) GT 0)  $
;            then  ImageName = InImageName         $
;            else  ImageName = STRCOMPRESS (AAPName + FileExtension)
;         WeightName = STRCOMPRESS (ImageName + '_wght' , /remove_all)
;      ENDIF

;   IF ((NOT Keyword_Set (SingleAAP))  AND  (i EQ 0))  $
;      THEN BEGIN
;         If (N_Elements (InScriptName) GT 0)  $
;            THEN BEGIN
;               ScriptName = InScriptName
;            ENDIF ELSE BEGIN
;               ScriptName = STRCOMPRESS ('drizzle_' + ImageName + '.cl' , /remove_all)
;            ENDELSE
;      ENDIF

;   IF ((    Keyword_Set (SingleAAP))  AND  (i EQ 0))  $
;      THEN BEGIN
;         ScriptNameList = [STRCOMPRESS ('drizzle_' + AAPName + FileExtension + '_' + String(i+1) + '.cl', /REM)]
;         ImageNameList  = [STRCOMPRESS (AAPName + FileExtension + '_' + String(i+1), /REM)]
;         WeightNameList = [STRCOMPRESS (AAPName + FileExtension + '_wght'  + '_' + String(i+1), /REM)]
;      ENDIF
;   IF ((    Keyword_Set (SingleAAP))  AND  (i GT 0))  $
;      THEN BEGIN
;         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + AAPName + FileExtension + '_' + String(i+1) + '.cl',/REM)]
;         ImageNameList  = [ImageNameList  , STRCOMPRESS (AAPName + FileExtension + '_' + String(i+1), /REM)]
;         WeightNameList = [WeightNameList , STRCOMPRESS (AAPName + FileExtension + '_wght' + '_' + String(i+1), /REM)]
;      ENDIF




Pro   PrepareGeneralFileNames,   InImageName, InScriptName, InIRAFCommandName,  $
                                 AAPNameList, NDataSet, FileExtension,          $
                                 IRAFCommandName,                               $
                                 ScriptName,                                    $
                                 ImageName, WeightName,                         $
                                 ScriptNameList,                                $
                                 ImageNameList, WeightNameList,                 $
                                 SINGLEAAP=SINGLEAAP
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

If (N_Elements (InIRAFCommandName) GT 0)  $
   then  IRAFCommandName = InIRAFCommandName        $
   else  IRAFCommandName = STRCOMPRESS (AAPNameList(0) + FileExtension + '.IRAF', /REMOVE_ALL)

If (N_Elements (InImageName) GT 0)  $
   then  ImageName = InImageName         $
   else  ImageName = STRCOMPRESS (AAPNameList(0) + FileExtension)

WeightName      = STRCOMPRESS (ImageName + '_wght' , /remove_all)

IF (NOT Keyword_Set (SingleAAP))  $
   THEN BEGIN
      If (N_Elements (InScriptName) GT 0)  $
         THEN BEGIN
            ScriptName = InScriptName
        ENDIF ELSE BEGIN
            ScriptName = STRCOMPRESS ('drizzle_' + ImageName + '.cl' , /remove_all)
        ENDELSE
   ENDIF

FOR i = 0L , NDataSet-1L , 1  DO BEGIN

   IF (i EQ 0)  $
      THEN BEGIN
         ScriptNameList = [STRCOMPRESS ('drizzle_' + AAPNameList(i) + FileExtension + '_' + String(i+1) + '.cl', /REM)]
         ImageNameList  = [STRCOMPRESS (AAPNameList(i) + FileExtension + '_' + String(i+1), /REM)]
         WeightNameList = [STRCOMPRESS (AAPNameList(i) + FileExtension + '_wght'  + '_' + String(i+1), /REM)]
      ENDIF

   IF (i GT 0)  $
      THEN BEGIN
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + AAPNameList(i) + FileExtension + '_' + String(i+1) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (AAPNameList(i) + FileExtension + '_' + String(i+1), /REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (AAPNameList(i) + FileExtension + '_wght' + '_' + String(i+1), /REM)]
      ENDIF

ENDFOR

Return
END


;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro   PrepareNamesEveryPixelMode,   Detector, ImageName, WeightName,    $
                                    FileExtension, ScriptNameList,      $
                                    ImageNameList, WeightNameList
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

print, ' <MapAAP2Drizzle> ... preparing the everypixel mode '

Case  Detector  of

  'P1'   : Begin
              ScriptNameList = [ScriptName]
              ImageNameList  = [ImageName]
              WeightNameList = [WeightName]
           end

  'P2'   : Begin
              ScriptNameList = [ScriptName]
              ImageNameList  = [ImageName]
              WeightNameList = [WeightName]
           end

  'P3'   : Begin
              ScriptNameList = [ScriptName]
              ImageNameList  = [ImageName]
              WeightNameList = [WeightName]
           end

  'C1'   : Begin
              For i = 0 , 8 , 1  do begin
                 IF (i EQ 0)  $
                    THEN BEGIN
                       ScriptNameList = [STRCOMPRESS ('drizzle_' + ImageName + '_Pix' + String(i+1) + '.cl',/REM)]
                       ImageNameList  = [STRCOMPRESS (ImageName + '_Pix' + String(i+1),/REM)]
                       WeightNameList = [STRCOMPRESS (WeightName + '_Pix' + String(i+1),/REM)]
                    ENDIF ELSE BEGIN
                       ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + '_Pix' + String(i+1) + '.cl',/REM)]
                       ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + '_Pix' + String(i+1),/REM)]
                       WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + '_Pix' + String(i+1),/REM)]
                    ENDELSE
              EndFor
           end

  'C2'   : Begin
              For i = 0 , 3 , 1  do begin
                IF (i EQ 0)  $
                    THEN BEGIN
                       ScriptNameList = [STRCOMPRESS ('drizzle_' + ImageName + '_Pix' + String(i+1) + '.cl',/REM)]
                       ImageNameList  = [STRCOMPRESS (ImageName + '_Pix' + String(i+1),/REM)]
                       WeightNameList = [STRCOMPRESS (WeightName + '_Pix' + String(i+1),/REM)]
                    ENDIF ELSE BEGIN
                       ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + '_Pix' + String(i+1) + '.cl',/REM)]
                       ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + '_Pix' + String(i+1),/REM)]
                       WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + '_Pix' + String(i+1),/REM)]
                    ENDELSE
              EndFor
           end

  else   : BEGIN
              Print , ' >>>> in PrepareNamesEveryPixelMode : Unknown Detector '
              stop
           End
EndCase

Return
End



;----------------------------------------------------------------------
;----------------------------------------------------------------------




Pro  PrepareNamesParallelMode,  NoDeltaStripes, NoProcLevels,                $
                                ImageName, WeightName, FileExtension,        $
                                DecMin, DecMax, DecStartVec, DecEndVec,      $
                                ScriptNameList, ImageNameList, WeightNameList
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

print, ' <MapAAP2Drizzle> ... preparing the parallel mode '

DeltaGap         = 2.0 / 60.0     ;   > size of C200 detector
DeltaStripeWidth = ((DecMax - DecMin) -  $
                    (Float(NoDeltaStripes-1) * DeltaGap))   $
                    / Float(NoDeltaStripes)


LevelQual = '_L1'
For i = 0 , NoDeltaStripes-1 , 1  do  begin
   If (i eq 0) $
      then begin
         DecStartVec = [DecMin]
         DecEndVec   = [DecMin+DeltaStripeWidth]

         ScriptNameList = [STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [STRCOMPRESS (WeightName + FileExtension + '_P' + String(i),/REM)]

      endif else begin
         DecStartVec = [DecStartVec , DecEndVec(i-1)+DeltaGap]
         DecEndVec   = [DecEndVec   , DecStartVec(i)+DeltaStripeWidth]
         If (i eq (NoDeltaStripes-1))   $
            then begin
               DecEndVec (NoDeltaStripes-1) = DecMax
            endif

         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      endelse
EndFor  ;  i


If (NoProcLevels eq 2)  $
   then begin
      LevelQual = '_L2'
      i = 0
      DecStartVec    = [DecStartVec , DecEndVec(0)]
      DecEndVec      = [DecEndVec   , DecStartVec(1)]
      ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
      ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
      WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
   endif


If (NoProcLevels eq 3)  $
   then begin
      LevelQual = '_L2'
      i = -1
      For ii = 0 , NoDeltaStripes-1 , 2  do begin
         i = i +1
         DecStartVec    = [DecStartVec , DecEndVec(ii)]
         DecEndVec      = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor
      LevelQual = '_L3'
      i = 0
      DecStartVec    = [DecStartVec , DecEndVec(1)]
      DecEndVec      = [DecEndVec   , DecStartVec(2)]
      ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
      ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
      WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
    endif



If (NoProcLevels eq 4)  $
   then begin
      LevelQual = '_L2'
      i = -1
      For ii = 0 , NoDeltaStripes-1 , 2  do begin
         i = i +1
         DecStartVec    = [DecStartVec , DecEndVec(ii)]
         DecEndVec      = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor

      LevelQual = '_L3'
      i = -1
      For ii = 1 , NoDeltaStripes-1 , 4  do begin
         i = i +1
         DecStartVec    = [DecStartVec , DecEndVec(ii)]
         DecEndVec      = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor

      LevelQual = '_L4'
      i = 0
      DecStartVec    = [DecStartVec , DecEndVec(3)]
      DecEndVec      = [DecEndVec   , DecStartVec(4)]
      ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
      ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
      WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
    endif


If (NoProcLevels eq 5)  $
   then begin
      LevelQual = '_L2'
      i = -1
      For ii = 0 , NoDeltaStripes-1 , 2  do begin
         i = i +1
         DecStartVec    = [DecStartVec , DecEndVec(ii)]
         DecEndVec      = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor

      LevelQual = '_L3'
      i = -1
      For ii = 1 , NoDeltaStripes-1 , 4  do begin
         i = i +1
         DecStartVec = [DecStartVec , DecEndVec(ii)]
         DecEndVec   = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor

      LevelQual = '_L4'
      i = -1
      For ii = 3 , NoDeltaStripes-1 , 8  do begin
         i = i +1
         DecStartVec = [DecStartVec , DecEndVec(ii)]
         DecEndVec   = [DecEndVec   , DecStartVec(ii+1)]
         ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
         ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
         WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
      EndFor

      LevelQual = '_L5'
      i = 0
      DecStartVec = [DecStartVec , DecEndVec(7)]
      DecEndVec   = [DecEndVec   , DecStartVec(8)]
      ScriptNameList = [ScriptNameList , STRCOMPRESS ('drizzle_' + ImageName + LevelQual + '_P' + String(i) + '.cl',/REM)]
      ImageNameList  = [ImageNameList  , STRCOMPRESS (ImageName + LevelQual + '_P' + String(i),/REM)]
      WeightNameList = [WeightNameList , STRCOMPRESS (WeightName + LevelQual + '_P' + String(i),/REM)]
    endif

Return
End





;----------------------------------------------------------------------
;----------------------------------------------------------------------





PRO   PrepareImage,  RACenter, DecCenter, RAPixel, DecPixel,            $
                     RollAngle, Rotation, DetPixelSize, DetPixelDist,   $
                     RAMax, RAMin, DecMax, DecMin, ImagePixelSize,      $
                     RelativeScale, NX , NY , NX2 , NY2 , NNXX , NNYY , $
                     EXTEND=EXTEND, NPIXX=NPIXX, NPIXY=NPIXY
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

; /* size of original map */

RasterMapLength = 3600.0 * ABS (RAMax - RAMin) * cos (!DTOR * 0.5 * ABS (DecMax + DecMin))
RasterMapHeight = 3600.0 * ABS (DecMax - DecMin)

RasterMapLength = RasterMapLength + (5. * DetPixelSize)
RasterMapHeight = RasterMapHeight + (5. * DetPixelSize)

; /* size of in-images for drizzling */

NX  = Long (RasterMapLength / DetPixelSize) + 1L
If ((NX mod 2) eq 0)  then  NX = NX + 1L
NX  = NX + 2
NX2 = LONG (NX / 2) + 1L

NY  = Long (RasterMapHeight / DetPixelSize) + 1L
If ((NY mod 2) eq 0)  then  NY = NY + 1L
NY  = NY + 2
NY2 = LONG (NY / 2) + 1L

; /* size of out-image for drizzling */

NNXX = LONG (FLOAT(NX) / RelativeScale) + 4L
NNYY = LONG (FLOAT(NY) / RelativeScale) + 4L

If ((NNXX mod 2) eq 0)  then  NNXX = NNXX + 1L
If ((NNYY mod 2) eq 0)  then  NNYY = NNYY + 1L

; /* make out-image for drizzling user-definded larger */

IF (N_Elements (EXTEND) GT 0)  $
   THEN BEGIN
      NNXX = NNXX + EXTEND
      NNYY = NNYY + EXTEND
   ENDIF

; /* use user-defined out-image size for drizzling  */

IF (N_Elements (NPixX) GT 0)  $
   THEN BEGIN
      NNXX = NPixX
      If ((NNXX mod 2) eq 0)  then  NNXX = NNXX + 1L
   ENDIF

IF (N_Elements (NPixY) GT 0)  $
   THEN BEGIN
      NNYY = NPixY
      If ((NNYY mod 2) eq 0)  then  NNYY = NNYY + 1L
   ENDIF

Return
END




;----------------------------------------------------------------------
;----------------------------------------------------------------------



PRO   PrepareShifts, Detector, RAPixel, DecPixel, RollAngle,         $
                     MapRotation, RefRA, RefDec, RefRot,             $
                     DetPixelSize, DetPixelDist, MapPixelSize,       $
                     XShift , YShift, XYRotation,                    $
                     SIMPLEPROJECTION=SIMPLEPROJECTION,              $
                     GNOMONICPROJECTION=GNOMONICPROJECTION,          $
                     IGNOREROLLANGLE=IGNOREROLLANGLE,                $
                     OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Case  Detector  of

  'P1'   : Begin
              IF (Keyword_Set (SIMPLEPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /DELTACOORDS
                 ENDIF

              IF (Keyword_Set (GNOMONICPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                 ENDIF

              IF (NOT Keyword_Set (GNOMONICPROJECTION))    AND        $
                 (NOT Keyword_Set (SIMPLEPROJECTION))                 $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC

                    Return
                 ENDIF
           end

  'P2'   : Begin
              IF (Keyword_Set (SIMPLEPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /DELTACOORDS
                 ENDIF

              IF (Keyword_Set (GNOMONICPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                 ENDIF

              IF (NOT Keyword_Set (GNOMONICPROJECTION))    AND        $
                 (NOT Keyword_Set (SIMPLEPROJECTION))                 $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC

                    Return
                 ENDIF
           end

  'P3'   : Begin
              IF (Keyword_Set (SIMPLEPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /DELTACOORDS
                    Return
                ENDIF

              IF (Keyword_Set (GNOMONICPROJECTION))  $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC

                    Return
                 ENDIF

              IF (NOT Keyword_Set (GNOMONICPROJECTION))    AND        $
                 (NOT Keyword_Set (SIMPLEPROJECTION))                 $
                 THEN  BEGIN
                    PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC

                    Return
                 ENDIF
             end

  'C1'   : Begin
              IF (Keyword_Set (SIMPLEPROJECTION))  $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /DELTACOORDS
                    Return
                 ENDIF

              IF (Keyword_Set (GNOMONICPROJECTION))  $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                    Return
                 ENDIF

              IF (NOT Keyword_Set (GNOMONICPROJECTION))    AND        $
                 (NOT Keyword_Set (SIMPLEPROJECTION))                 $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                    Return
                 ENDIF
           end

  'C2'   : Begin
              IF (Keyword_Set (SIMPLEPROJECTION))  $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /DELTACOORDS
                    Return
                 ENDIF

              IF (Keyword_Set (GNOMONICPROJECTION))  $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                    Return
                 ENDIF

              IF (NOT Keyword_Set (GNOMONICPROJECTION))    AND        $
                 (NOT Keyword_Set (SIMPLEPROJECTION))                 $
                 THEN  BEGIN
                    PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,    $
                                    MapRotation, RefRA, RefDec, RefRot,           $
                                    DetPixelSize, DetPixelDist, MapPixelSize,  $
                                    XShift, YShift, XYRotation,                $
                                    IGNOREROLLANGLE=IGNOREROLLANGLE,           $
                                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS, /GNOMONIC
                    Return
                 ENDIF
           end

  else   : BEGIN
              Print , ' >>>> in PrepareShifts : Unknown Detector '
              stop
           End
EndCase


Return
END




;----------------------------------------------------------------------
;----------------------------------------------------------------------






PRO   PreparePShifts, Detector, RAPixel, DecPixel, RollAngle,   $
                      MapRotation, RefRA, RefDec, RefRot,       $
                      DetPixelSize, DetPixelDist, MapPixelSize, $
                      XShift, YShift, XYRotation,               $
                      OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,      $
                      IGNOREROLLANGLE=IGNOREROLLANGLE,          $
                      GNOMONIC=GNOMONIC, DELTACOORDS=DELTACOORDS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


Dimen = SIZE (RAPixel)

NPixel = 1
NData  = Dimen(1)

XShift     = FltArr (NPixel , NData)
YShift     = FltArr (NPixel , NData)
XYRotation = FltArr (NPixel , NData)

; /* relative shifts are measured in which frame ? */

RelevantPixelSize = DetPixelSize
IF Keyword_Set (OUTPUTFRAMESHIFTS)  $
   THEN  BEGIN
      RelevantPixelSize = MapPixelSize
   ENDIF

FOR i = 0L , NPixel-1L , 1  DO BEGIN
   TmpRA  =  Transpose (RAPixel (i,*))
   TmpDec =  Transpose (DecPixel (i,*))

   MS_Conv_RADEC_Rect,  TmpRA, TmpDec, RefRA, RefDec,              $
                         DeltaRA, DeltaDec, /ARCSEC, /NORMALXY,     $
                         GNOMONIC=GNOMONIC, DELTACOORDS=DELTACOORDS

   RAShifts  =  DeltaRA (*)  / RelevantPixelSize
   DecShifts =  DeltaDec (*) / RelevantPixelSize

   MS_RotateCoordCCW,   RAShifts, DecShifts, MapRotation, RotXShift, RotYShift, /DEGREE

   XShift (i,*) =  RotXShift (*)
   YShift (i,*) =  RotYShift (*)

   IF (NOT Keyword_Set (IGNOREROLLANGLE))   $
      THEN  BEGIN
         XYRotation (i,*) = RollAngle (*) - MapRotation (*)
      ENDIF ELSE BEGIN
         XYRotation (i,*) = 0.0
      ENDELSE
ENDFOR

Return
END




;----------------------------------------------------------------------
;----------------------------------------------------------------------





PRO   PrepareCShifts, Detector, RAPixel, DecPixel, RollAngle,   $
                      MapRotation, RefRA, RefDec, RefRot,       $
                      DetPixelSize, DetPixelDist, MapPixelSize, $
                      XShift, YShift, XYRotation,               $
                      OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,      $
                      IGNOREROLLANGLE=IGNOREROLLANGLE,          $
                      GNOMONIC=GNOMONIC, DELTACOORDS=DELTACOORDS
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


Dimen = SIZE (RAPixel)

NPixel = Dimen(1)
NData  = Dimen(2)

XShift     = FltArr (NPixel , NData)
YShift     = FltArr (NPixel , NData)
XYRotation = FltArr (NPixel , NData)

; /* relative shifts are measured in which frame ? */

RelevantPixelSize = DetPixelSize
IF Keyword_Set (OUTPUTFRAMESHIFTS)  $
   THEN  BEGIN
      RelevantPixelSize = MapPixelSize
   ENDIF

FOR i = 0L , NPixel-1L , 1  DO BEGIN
   TmpRA  =  Transpose (RAPixel (i,*))
   TmpDec =  Transpose (DecPixel (i,*))

   MS_Conv_RADEC_Rect,  TmpRA, TmpDec, RefRA, RefDec,              $
                        DeltaRA, DeltaDec, /ARCSEC, /NORMALXY,     $
                        GNOMONIC=GNOMONIC, DELTACOORDS=DELTACOORDS

   RAShifts  =  DeltaRA (*)  / RelevantPixelSize
   DecShifts =  DeltaDec (*) / RelevantPixelSize

   MS_RotateCoordCCW,   RAShifts, DecShifts, MapRotation, RotXShift, RotYShift, /DEGREE

   XShift (i,*) =  RotXShift (*)
   YShift (i,*) =  RotYShift (*)

   IF (NOT Keyword_Set (IGNOREROLLANGLE))   $
      THEN  BEGIN
         XYRotation (i,*) = RollAngle (*) - MapRotation (*)
      ENDIF ELSE BEGIN
         XYRotation (i,*) = 0.0
      ENDELSE

ENDFOR

Return
END



;----------------------------------------------------------------------
;----------------------------------------------------------------------





Pro   MakePixelCoord,       Detector, Aperture, RACenter, DecCenter,   $
                            RollAngle, RelChopPos, DetPixelSize,       $
                            DetPixelDist, RAPixel, DecPixel
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


; /* Make PixelCoordinates */

Case  Detector  of

  'P1'   : BEGIN
              P_AS = [52.7120, 77.9220, 98.5490, 119.175, 126.051, 179.909,  $
                      179.909, 4.58400, 8.18200, 10.7710, 13.7510, 18.7940,  $
                      19.0220, 22.9180]

              DetPixelSize = P_AS (aperture)
              DetPixelDist = P_AS (aperture)

              MakePPixelCoord,   RACenter, DecCenter, RollAngle,   $
                                 RelChopPos, DetPixelSize, DetPixelDist,   $
                                 RAPixel, DecPixel
           end

  'P2'   : Begin
              P_AS = [52.7120, 77.9220, 98.5490, 119.175, 126.051, 179.909,  $
                      179.909, 4.58400, 8.18200, 10.7710, 13.7510, 18.7940,  $
                      19.0220, 22.9180]

              DetPixelSize = P_AS (aperture)
              DetPixelDist = P_AS (aperture)

              MakePPixelCoord,   RACenter, DecCenter, RollAngle,   $
                                 RelChopPos, DetPixelSize, DetPixelDist,   $
                                 RAPixel, DecPixel
           end

  'P3'   : Begin
              P_AS = [52.7120, 77.9220, 98.5490, 119.175, 126.051, 179.909,  $
                      179.909, 4.58400, 8.18200, 10.7710, 13.7510, 18.7940,  $
                      19.0220, 22.9180]

              DetPixelSize = P_AS (aperture)
              DetPixelDist = P_AS (aperture)

              MakePPixelCoord,   RACenter, DecCenter, RollAngle,   $
                                 RelChopPos, DetPixelSize, DetPixelDist,   $
                                 RAPixel, DecPixel
           end

  'C1'   : Begin
              DetPixelSize = 43.544
              DetPixelDist = 45.84

              MakeC100PixelCoord,   RACenter, DecCenter, RollAngle,   $
                                    RelChopPos, DetPixelSize, DetPixelDist,   $
                                    RAPixel, DecPixel
           end

  'C2'   : Begin
              DetPixelSize = 89.381
              DetPixelDist = 91.68

              MakeC200PixelCoord,  RACenter, DecCenter, RollAngle,   $
                                   RelChopPos, DetPixelSize, DetPixelDist,   $
                                   RAPixel, DecPixel
           end

  else   : BEGIN
              Print , ' >>>> Unknown Detector '
              stop
           End
EndCase

Return
END



;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro   MakeC100PixelCoord,   RACenter, DecCenter, RollAngle,            $
                            RelChopPos, DetPixelSize, DetPixelDist,    $
                            RAPixel, DecPixel
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
; Note : X and Y defines a normal coordinate system
;



NData  = N_Elements (RACenter)
NPixel = 9L

RAPixel  = FltArr (NPixel , NData)
DecPixel = FltArr (NPixel , NData)

RelRA  = FltArr (NPixel , NData)
RelDec = FltArr (NPixel , NData)

DeltaXCoord =  FltArr (NData)
DeltaYCoord =  FltArr (NData)


; /* Pixel Center Coordinates ! */

LenX = DetPixelDist    ;  [arcsec]
LenY = DetPixelDist    ;  [arcsec]

RelXCoord = [+LenX, +LenX, +LenX, 0.0,   0.0, 0.0,   -LenX, -LenX, -LenX]
RelYCoord = [-LenY ,0.0,   +Leny, -LenY ,0.0, +Leny, -LenY, 0.0,   +Leny]


SinPhi   = Double (sin (RollAngle(*) * !DTOR))
CoSinPhi = Double (cos (RollAngle(*) * !DTOR))

For i = 0L , NPixel-1L  do begin

   DeltaXCoord (*) =  RelXCoord (i)      ;  [arcsec]
   DeltaYCoord (*) =  RelYCoord (i)      ;  [arcsec]

   IF (N_Elements (RelChopPos (*)) GT 0)  $
      THEN BEGIN
         DeltaXCoord (*) = DeltaXCoord (*) - RelChopPos (*)
      ENDIF

   RelRA (i,*)  = - ((DeltaXCoord (*) * CoSinPhi(*))  -  (DeltayCoord (*) *   SinPhi(*)))
   RelRA (i,*)  = RelRA (i,*) / 3600.   ; [degrees]

   RelDec (i,*) = + ((DeltaXCoord (*) * SinPhi(*))    +  (DeltayCoord (*) * CoSinPhi(*)))
   RelDec (i,*) = RelDec (i,*) / 3600.   ; [degrees]

   DecPixel (i,*) = DecCenter (*)  +  RelDec (i,*)
   RAPixel (i,*)  = RACenter (*)   +  (RelRA (i,*) / cos (DecPixel(i,*) * !DTOR))
EndFor

Return
End







Pro   MakeC200PixelCoord,   RACenter, DecCenter, RollAngle,            $
                            RelChopPos, DetPixelSize, DetPixelDist,    $
                            RAPixel, DecPixel
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;
; Note : X and Y defines here a normal coordinate system
;        RelChopPos is defined within ISOs Y/Z coordinate system
;


NData  = N_Elements (RACenter)
NPixel = 4L

RAPixel  = FltArr (NPixel , NData)
DecPixel = FltArr (NPixel , NData)

RelRA  = FltArr (NPixel , NData)
RelDec = FltArr (NPixel , NData)

DeltaXCoord =  FltArr (NData)
DeltaYCoord =  FltArr (NData)

; /* Pixel Center Coordinates ! */

HalfLenX = 0.5 * DetPixelDist      ;  [arcsec]
HalfLenY = 0.5 * DetPixelDist      ;  [arcsec]

RelXCoord = [-HalfLenX , +HalfLenX , +HalfLenX , -HalfLenX]
RelYCoord = [-HalfLenY , -HalfLenY , +HalfLenY , +HalfLenY]

SinPhi   = Double (sin (RollAngle(*) * !DTOR))
CoSinPhi = Double (cos (RollAngle(*) * !DTOR))

For i = 0L , NPixel-1L  do begin

   DeltaXCoord (*) =  RelXCoord (i)      ;  [arcsec]
   DeltaYCoord (*) =  RelYCoord (i)      ;  [arcsec]

   IF (N_Elements (RelChopPos (*)) GT 0)  $
      THEN BEGIN
         DeltaXCoord (*) = DeltaXCoord (*) - RelChopPos (*)
      ENDIF

   RelRA (i,*)  = - ((DeltaXCoord (*) * CoSinPhi(*))  -  (DeltaYCoord (*) *   SinPhi(*)))
   RelRA (i,*)  = RelRA (i,*) / 3600.   ; [degrees]

   RelDec (i,*) = + ((DeltaXCoord (*) * SinPhi(*))    +  (DeltaYCoord (*) * CoSinPhi(*)))
   RelDec (i,*) = RelDec (i,*) / 3600.   ; [degrees]

   DecPixel (i,*) = DecCenter (*)  +  RelDec (i,*)
   RAPixel (i,*)  = RACenter (*)   +  (RelRA (i,*) / cos (DecPixel(i,*) * !DTOR))
EndFor


Return
End









Pro   MakePPixelCoord,   RACenter, DecCenter, RollAngle,            $
                         RelChopPos, DetPixelSize, DetPixelDist,    $
                         RAPixel, DecPixel
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


NData  = N_Elements (RACenter)
NPixel = 1L

RAPixel  = FltArr (NPixel , NData)
DecPixel = FltArr (NPixel , NData)

RAPixel (0,*)  =  RACenter
DecPixel (0,*) =  DecCenter

Return
End



;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro     WriteIRAFCommandFile,  CommandUnit, ScriptName, LogFileName,   $
                               EXECUTE = EXECUTE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

DotPos = STRPOS (ScriptName,".")
LogFileName =  STRCOMPRESS (STRMID (ScriptName,0,DotPos+1) + 'log' , /REMO)

OutString =  '!rm   ' + !IRAF_DATA + LogFileName
PrintF ,  CommandUnit , Format='(A)' , OutString

OutString =  'cd  ' + !IRAF_LOGIN 
PrintF ,  CommandUnit , Format='(A)' , OutString

OutString =  ' '
PrintF ,  CommandUnit , Format='(A)' , OutString

; CG - LOGfile to Data directory
OutString =  'cl  <  ' + !IRAF_DATA + '/' + ScriptName + '   >  ' + !IRAF_DATA + LogFileName + ' & '
PrintF ,  CommandUnit , Format='(A)' , OutString

OutString =  ' '
PrintF ,  CommandUnit , Format='(A)' , OutString

IF (Keyword_Set (EXECUTE)) $
   THEN BEGIN
      OutString =  'logout '
      PrintF ,  CommandUnit , Format='(A)' , OutString

      OutString =  ' '
      PrintF ,  CommandUnit , Format='(A)' , OutString
   ENDIF

OutString =  'cd  ' + !IRAF_DATA 
PrintF ,  CommandUnit , Format='(A)' , OutString

Return
END



;----------------------------------------------------------------------
;----------------------------------------------------------------------




Pro  WriteHeaderDrizzleScript,   Version, OutUnit, Detector, DetPixelSize, $
                          DetPixelDist, RACenter, DecCenter, RAPixel,      $
                          DecPixel, RefData, RefRot,                       $
                          ScriptName, ImageName, WeightName,               $
                          ImagePixelSize, RelativeScale,                   $
                          DrizzleFraction, FillValue,                      $
                          NX, NY, NX2, NY2, NNXX, NNYY,                    $
                          XShift, YShift, CalledAs,                        $
                          OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,             $
                          IGNOREROLLANGLE=IGNOREROLLANGLE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  '#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '# '
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '#  ' + Version
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '#  ... processing done ' + ' ' + !STIME
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '# '
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '# '
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '# ' + CalledAs
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '# '
PrintF , OutUnit , Format='(A)' , OutString
OutString =  '#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
PrintF , OutUnit , Format='(A)' , OutString


OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

; change to data directory to avoid mixing data and cl file

OutString = 'cd ' + !IRAF_DATA
PrintF , OutUnit , Format='(A)' , OutString

; /* commands to create drizzle image and drizzle weight image */

DataImagename = strcompress ('TMP_' + ImageName , /remove_all)
OutString = 'imdelete ' + DataImageName
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'mkgauss ' + DataImageName + '  ' + strcompress(string(nx),/remove_all) + '  ' + strcompress(string(ny),/remove_all) +  $
            '  ' + strcompress(string(nx2),/remove_all) + '  ' + strcompress(string (ny2),/remove_all) + '    0.0  0.0 0.0  0.0 0.0   0.0'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'hedit '  +   DataImageName   + '   exptime   1.0   add+   ver- '
PrintF , OutUnit , Format='(A)' , OutString




OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

DataWeightName = strcompress ('TMP_' + WeightName , /remove_all)

OutString = 'imdelete ' + DataWeightname
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'mkgauss ' + DataWeightname + '  ' + strcompress(string(nx),/remove_all) + '  ' + strcompress(string(ny),/remove_all) +  $
            '  ' + strcompress(string(nx2),/remove_all) + '  ' + strcompress(string (ny2),/remove_all) + '    0.0  0.0 0.0  0.0 0.0   0.0'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'hedit '  +   DataWeightname   + '   exptime   1.0   add+   ver- '
PrintF , OutUnit , Format='(A)' , OutString



OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString



OutString = 'imdele ' + ImageName
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'imdele ' + WeightName
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


StrngNX = string (nx2)
StrngNY = string (ny2)
PositString = '[' + StrngNX + ':' + StrngNX + ',' + StrngNY + ':' + StrngNY + ']'

WeightString = strcompress (DataWeightName + PositString , /remove_all)
OutString = 'imreplace ' + WeightString + '   0.0 '
PrintF , OutUnit , Format='(A)' , OutString

ImageString = strcompress (DataImageName + PositString , /remove_all)
PrepString = strcompress (DataImagename + PositString , /remove_all)
OutString = 'imreplace ' + PrepString + '  ' + string (RefData)
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


; /* create commands to set initial IRAF drizzle parameters */

OutString = 'drizzle.outweig=' + '"' + WeightName + '"'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'drizzle.in_mask=' + '"' + DataWeightName + '"'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'drizzle.wt_scl=1.0'
PrintF , OutUnit , Format='(A)' , OutString

PrepString = 'drizzle.outnx=' + string (nnxx)
OutString = strcompress (PrepString , /remove_all)
PrintF , OutUnit , Format='(A)' , OutString

PrepString = 'drizzle.outny=' + string (nnyy)
OutString = strcompress (PrepString , /remove_all)
PrintF , OutUnit , Format='(A)' , OutString

PrepString = 'drizzle.pixfrac=' + string (DrizzleFraction)
OutString = strcompress (PrepString , /remove_all)
PrintF , OutUnit , Format='(A)' , OutString

PrepString = 'drizzle.scale=' + string (RelativeScale)
OutString = strcompress (PrepString , /remove_all)
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'drizzle.align="center"'
PrintF , OutUnit , Format='(A)' , OutString

IF (NOT  Keyword_Set (OUTPUTFRAMESHIFTS))  $
   THEN BEGIN
      OutString = 'drizzle.shft_un="input"'
      PrintF , OutUnit , Format='(A)' , OutString
   ENDIF ELSE BEGIN
      OutString = 'drizzle.shft_un="output"'
      PrintF , OutUnit , Format='(A)' , OutString
   ENDELSE


OutString = 'drizzle.shft_fr="output"'
PrintF , OutUnit , Format='(A)' , OutString

;------------------------------------------
;IF (NOT  Keyword_Set (IGNOREROLLANGLE))  $
;   THEN BEGIN
;      OutString = 'drizzle.shft_fr="output"'
;      PrintF , OutUnit , Format='(A)' , OutString
;   ENDIF ELSE BEGIN
;;; ---- default since the very beginning ---
;      OutString = 'drizzle.shft_fr="input"'
;      PrintF , OutUnit , Format='(A)' , OutString
;   ENDELSE
;------------------------------------------



;; -- for Drizzle V 1.1 --

;;OutString = 'drizzle.exp_sc=no'
;;PrintF , OutUnit , Format='(A)' , OutString

;; -- for Drizzle V 1.2 --

OutString = 'drizzle.expkey="exptime"'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'drizzle.in_un="cps"'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'drizzle.out_un="cps"'
PrintF , OutUnit , Format='(A)' , OutString

;; --

PrepString = 'drizzle.fillval=' + string (FillValue)
OutString = strcompress (PrepString , /remove_all)
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'lpara   drizzle '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString



; /* write first drizzling commands */

PrepString1 = 'drizzle ' + DataImageName + ' ' + ImageName + ' '
PrepString2 = 'xsh=0.0  ysh=0.0 ' + STRCOMPRESS ('rot=' + STRING(RefRot) , /remove_all)
OutString = PrepString1 + PrepString2
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

WeightString = strcompress (DataWeightName + PositString , /remove_all)
OutString = 'imreplace ' + WeightString + ' 1.0'
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString
OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


Return
END



;----------------------------------------------------------------------
;----------------------------------------------------------------------




Pro     WriteDrizzleScriptSeparator,  OutUnit, AAPName
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

OutString = '#------------------------------------------------------------------------------'
PrintF , OutUnit , Format='(A)' , OutString

OutString = '#----------------------------- ' + AAPName + ' -----------------------------------'
PrintF , OutUnit , Format='(A)' , OutString

OutString = '#------------------------------------------------------------------------------'
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

Return
End



;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro  WriteParallelDrizzleScript,    OutUnitList, Detector,               $
                          DetPixelSize, DetPixelDist,                    $
                          RACenter, DecCenter, RAPixel, DecPixel,        $
                          Data, Flag, IgnorePixelVec,                    $
                          ScriptNameList, ImageNameList, WeightNameList, $
                          ImagePixelSize, RelativeScale,                 $
                          DrizzleFraction, NX2, NY2,                     $
                          XShift, YShift, XYRotation,                    $
                          DecStartVec, DecEndVec, NoDeltaStripes
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Dimen  =  Size (XShift)
NPixel =  Dimen(1)
NData  =  Dimen(2)

StrngNX = string (nx2)
StrngNY = string (ny2)
PositString = '[' + StrngNX + ':' + StrngNX + ',' + StrngNY + ':' + StrngNY + ']'

; /* write all other drizzling commands */

For i = 0L , NData-1L  do begin

   For j = 0L , NPixel-1L do begin

      ; /* ignore some pixels ? */

       FoundIndex =  WHERE (IgnorePixelVec(*) EQ j , NFound)
       IF (NFound GT 0)  THEN  GOTO,  NEXTDATAPOINT

      ; /* find the correct strip for each pixel */

      UseStripeNo = NoDeltaStripes
      For k = 0L , (2*NoDeltaStripes)-2L , 1  DO Begin
         If ( (DecPixel (j,i) ge DecStartVec (k))  AND    $
              (DecPixel (j,i) le DecEndVec (k)) )         $
            then begin
               UseStripeNo = k
               goto, STRIPEFOUND
            endif
      EndFor

STRIPEFOUND:

      ; /* the corresponding file names */

      ImageName  = ImageNameList(UseStripeNo)
      WeightName = WeightNameList(UseStripeNo)
      OutUnit    = OutUnitList(UseStripeNo)


      ; /* check PIA flags */

      If (Flag (j,i) eq 2)  $
         then goto, NEXTDATAPOINT


      ; /* put the stuff into the script */

      DataImageName = STRCOMPRESS ('TMP_' + ImageName , /remove_all)
      DataWeightName = STRCOMPRESS ('TMP_' + WeightName , /remove_all)

      ImageString = STRCOMPRESS (DataImageName + PositString , /remove_all)
      WeightString = STRCOMPRESS (DataWeightName + PositString , /remove_all)
      DataValue = Data (j,i)
      OutString = 'imreplace ' + ImageString + STRING (DataValue)
      PrintF , OutUnit , Format='(A)' , OutString

      PrepString1 = 'drizzle ' + DataImageName + ' ' + ImageName
      PrepString2 = ' ' + STRCOMPRESS ('xsh=' + STRING (xshift(j,i))   , /remove_all)  +   $
                    ' ' + STRCOMPRESS ('ysh=' + STRING (yshift(j,i))   , /remove_all)  +   $
                    ' ' + STRCOMPRESS ('rot=' + STRING (xyrotation(j,i)) , /remove_all)
      OutString = PrepString1 + PrepString2

      PrintF , OutUnit , Format='(A)' , OutString

      WriteHeaderKeyDelete , ImageName, WeightName, OutUnit

NEXTDATAPOINT:

   EndFor

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

EndFor

return
end









Pro  WriteSumUpParallelImages,    OutUnitList, ScriptNameList,     $
                                  ImageNameList, WeightNameList,   $
                                  NoDeltaStripes, NoProcLevels
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


ii = 0L
For i = 0L , (NoDeltaStripes-2L) , 1  do begin

   OutUnit = OutUnitList (NoDeltaStripes+i)

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#------------------------------------------------------------------------------'
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#---------------------- SumUp two Parallel-Made Drizzle Images ----------------'
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#------------------------------------------------------------------------------'
   PrintF , OutUnit , Format='(A)' , OutString

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString


   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

;;;   WriteParallelSumSeparator,  OutUnit

   OutString = 'comp  ' + ImageNameList(NoDeltaStripes+i) + ' = ' + ImageNameList(ii) + ' + ' + ImageNameList(ii+1)
   PrintF , OutUnit , Format='(A)' , OutString
   ii = ii + 2

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#------------------------------------------------------------------------------'
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#---------------------- SumUp two Parallel-Made Drizzle Images ----------------'
   PrintF , OutUnit , Format='(A)' , OutString

   OutString = '#------------------------------------------------------------------------------'
   PrintF , OutUnit , Format='(A)' , OutString
EndFor


Return
End




;----------------------------------------------------------------------
;----------------------------------------------------------------------



Pro  WriteEveryPixelDrizzleScript,    OutUnitList, Detector,             $
                          DetPixelSize, DetPixelDist,                    $
                          RACenter, DecCenter, RAPixel, DecPixel,        $
                          Data, Flag, IgnorePixelVec,                    $
                          ScriptNameList, ImageNameList, WeightNameList, $
                          ImagePixelSize, RelativeScale,                 $
                          DrizzleFraction, NX2, NY2,                     $
                          XShift, YShift, XYRotation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NoDetectorPixel = N_Elements (ScriptNameList)

Dimen  =  Size (XShift)
NPixel =  Dimen(1)
NData  =  Dimen(2)

StrngNX = string (nx2)
StrngNY = string (ny2)
PositString = '[' + StrngNX + ':' + StrngNX + ',' + StrngNY + ':' + StrngNY + ']'

; /* write all other drizzling commands */

For i = 0L , NData-1L  do begin

   For j = 0L , NPixel-1L do begin

      ; /* ignore some pixels ? */

       FoundIndex =  WHERE (IgnorePixelVec(*) EQ j , NFound)
       IF (NFound GT 0)  THEN  GOTO,  NEXTDATAPOINT

      ; /* get the corresponding file names */

      ImageName  = ImageNameList(j)
      WeightName = WeightNameList(j)
      OutUnit    = OutUnitList(j)

      ; /* put the stuff into the script */

      If (Flag (j,i) eq 2)  $
         then goto, NEXTDATAPOINT

      DataImageName = strcompress ('TMP_' + ImageName , /remove_all)
      DataWeightName = strcompress ('TMP_' + WeightName , /remove_all)

      ImageString = strcompress (DataImageName + PositString , /remove_all)
      WeightString = strcompress (DataWeightName + PositString , /remove_all)

      DataValue = Data (j,i)
      OutString = 'imreplace ' + ImageString + string (DataValue)
      PrintF , OutUnit , Format='(A)' , OutString

      PrepString1 = 'drizzle ' + DataImageName + ' ' + ImageName
      PrepString2 = ' ' + strcompress ('xsh=' + string (xshift(j,i))   , /remove_all)  +   $
                    ' ' + strcompress ('ysh=' + string (yshift(j,i))   , /remove_all)  +   $
                    ' ' + strcompress ('rot=' + string (xyrotation(j,i)) , /remove_all)
      OutString = PrepString1 + PrepString2

      PrintF , OutUnit , Format='(A)' , OutString

      WriteHeaderKeyDelete , ImageName, WeightName, OutUnit

NEXTDATAPOINT:

   EndFor

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

EndFor

return
end





;----------------------------------------------------------------------
;----------------------------------------------------------------------




Pro  WriteMainDrizzleScript,  OutUnit, Detector,                         $
                          DetPixelSize, DetPixelDist,                    $
                          RACenter, DecCenter, RAPixel, DecPixel,        $
                          Data, Flag, IgnorePixelVec,                    $
                          ScriptName, ImageName, WeightName,             $
                          ImagePixelSize, RelativeScale,                 $
                          DrizzleFraction, NX2, NY2,                     $
                          XShift, YShift, XYRotation
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Dimen  =  Size (XShift)
NPixel =  Dimen(1)
NData  =  Dimen(2)


DataImageName  = strcompress ('TMP_' + ImageName , /remove_all)
DataWeightName = strcompress ('TMP_' + WeightName , /remove_all)

StrngNX     = string (nx2)
StrngNY     = string (ny2)
PositString = '[' + StrngNX + ':' + StrngNX + ',' + StrngNY + ':' + StrngNY + ']'

ImageString  = strcompress (DataImageName + PositString , /remove_all)
WeightString = strcompress (DataWeightName + PositString , /remove_all)


; /* write all other drizzling commands */

For i = 0L , NData-1L  do begin

   For j = 0L , NPixel-1L do begin

      ; /* ignore some pixels ? */

       FoundIndex =  WHERE (IgnorePixelVec(*) EQ j , NFound)
       IF (NFound GT 0)  THEN  GOTO,  NEXTDATAPOINT

      ; /* check PIA flags ? */

      If (Flag (j,i) eq 2)  $
         then goto, NEXTDATAPOINT


      DataValue = Data (j,i)
      OutString = 'imreplace ' + ImageString + string (DataValue)
      PrintF , OutUnit , Format='(A)' , OutString

      PrepString1 = 'drizzle ' + DataImageName + ' ' + ImageName
      PrepString2 = ' ' + strcompress ('xsh=' + string (xshift(j,i))   , /remove_all)  +   $
                    ' ' + strcompress ('ysh=' + string (yshift(j,i))   , /remove_all)  +   $
                    ' ' + strcompress ('rot=' + string (xyrotation(j,i)) , /remove_all)
      OutString = PrepString1 + PrepString2

      PrintF , OutUnit , Format='(A)' , OutString

      WriteHeaderKeyDelete , ImageName, WeightName, OutUnit

NEXTDATAPOINT:

   EndFor

   OutString =  ' '
   PrintF , OutUnit , Format='(A)' , OutString

EndFor

return
end




;----------------------------------------------------------------------
;----------------------------------------------------------------------


PRO      WriteFITSOutput,   OutUnit, ImageName, Weightname
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

OutString = '#------------------------------------------------------------------------------'
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'wfits  '  +  ImageName  +  '.imh  '  +  ImageName+'.fits  bitpix=-32 '
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'wfits '  +   WeightName  + '.imh  '  +  WeightName+'.fits  bitpix=-32 '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

Return
END


;----------------------------------------------------------------------
;----------------------------------------------------------------------



PRO     WriteHeaderKeyDelete,  ImageName, WeightName, OutUnit
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


OutString = 'hedit '  +    ImageName  + ' D002*   dele+ veri- show-'
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'hedit '  +    WeightName  + ' D002*   dele+ veri- show- '
PrintF , OutUnit , Format='(A)' , OutString

Return
END




;----------------------------------------------------------------------
;----------------------------------------------------------------------



PRO  WriteProjectFITSHeaderKeys,  OutUnit, ImageName, Weightname,        $
                                  MapRACenter,MapDecCenter,              $
                                  ImagePixelSize, NNXX, NNYY, Identifier
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


MapScale = 1.0 / (3600.0 / ImagePixelSize)

XCenterPixel = NNXX / 2L
If ((NNXX mod 2) GT 0)  then  XCenterPixel = FLOAT (XCenterPixel + 1L)
If ((NNXX mod 2) EQ 0)  then  XCenterPixel = FLOAT (XCenterPixel) + 0.5

YCenterPixel = NNYY / 2L
If ((NNYY mod 2) GT 0)  then  YCenterPixel = FLOAT (YCenterPixel + 1L)
If ((NNYY mod 2) EQ 0)  then  YCenterPixel = FLOAT (YCenterPixel) + 0.5


OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

OutString = 'hedit  '  +    ImageName  + '    CRVAL1   ' + STRCOMPRESS (STRING (MapRACenter),/REMO) + '   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString
OutString = 'hedit  '  +    ImageName  + '    CRVAL2   ' + STRCOMPRESS (STRING (MapDecCenter),/REMO) + '   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    CRPIX1   ' + STRCOMPRESS (STRING (XCenterPixel),/REMO) + '   add+  veri-   '
PrintF , OutUnit , Format='(A)' , OutString
OutString = 'hedit  '  +    ImageName  + '    CRPIX2   ' + STRCOMPRESS (STRING (YCenterPixel),/REMO) + '   add+  veri-   '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    CDELT1   ' + STRCOMPRESS (STRING (-MapScale),/REMO) + '   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString
OutString = 'hedit  '  +    ImageName  + '    CDELT2    ' + STRCOMPRESS (STRING (MapScale),/REMO) + '   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    CTYPE1     "RA---TAN"   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString
OutString = 'hedit  '  +    ImageName  + '    CTYPE2     "DEC--TAN"   add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    CROTA1    0.0    add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString
OutString = 'hedit  '  +    ImageName  + '    CROTA2    0.0    add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    EQUINOX   ' + STRCOMPRESS (STRING (2000.0),/REMO) + '    add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString


OutString = 'hedit  '  +    ImageName  + '    IDENT     "'  +  Identifier  +  '"    add+  veri-  '
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString
PrintF , OutUnit , Format='(A)' , OutString

Return
END






;----------------------------------------------------------------------
;----------------------------------------------------------------------



PRO  WriteTrailerDrizzleScript,  OutUnit, ImageName, Weightname
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

OutString = '#------------------------------------------------------------------------------'
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

DataImagename = strcompress ('TMP_' + ImageName , /remove_all)
OutString = 'imdelete ' + DataImageName
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

DataWeightName = strcompress ('TMP_' + WeightName , /remove_all)
OutString = 'imdelete ' + DataWeightName
PrintF , OutUnit , Format='(A)' , OutString

OutString =  ' '
PrintF , OutUnit , Format='(A)' , OutString

Return
END








;================================================================================
;================================================================================






PRO  MS_InitializeVector,  Vector, NData, Init,    $
                               INTEGER=INTEGER,        $
                               DOUBLE=DOUBLE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NInit  = N_Elements (Init)

Vector = FltArr (NData)
IF (Keyword_set (DOUBLE))  THEN  Vector = DblArr (NData)

IF (NInit EQ     1)  THEN  Vector (*) = Init (0)
IF (NInit EQ NData)  THEN  Vector (*) = Init (*)

IF ((NInit LT NData) AND (NInit GT 1))  $
   THEN BEGIN
      Vector (0:NInit-1) = Init (*)
      Vector (NInit:*)   = Init (NInit-1)
   ENDIF

IF (Keyword_Set (INTEGER))  THEN  Vector = LONG (Vector(*))

Return
END







Pro  MS_Vector2QuotedStringList,  InVector, OutStringList
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData = N_Elements (InVector)

For i = 0L , NData-1L , 1  do begin
   IF (NData EQ 1)  $
      then begin
         Tmp = "'" + String (InVector(i)) + "'"
      endif else BEGIN
         IF (i LT (NData-1))  $
            THEN BEGIN
               Tmp = "'" + String (InVector(i)) + "', "
            ENDIF ELSE BEGIN
               Tmp = "'" + String (InVector(i)) + "'"
            ENDELSE
      ENDELSE

   if (i eq 0)  $
      then begin
         OutStringList = '[' + Tmp
      endif else begin
         OutStringList = OutStringList + Tmp
      endelse
EndFor
OutStringList = OutStringList + ']'

Return
End





Pro  MS_Vector2StringList,  InVector, OutStringList
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData = N_Elements (InVector)

For i = 0L , NData-1L , 1  do begin
   IF (NData EQ 1)  $
      then begin
         Tmp = String (InVector(i))
      endif else begin
         Tmp = String (InVector(i)) + ", "
      endelse
   if (i eq 0)  $
      then begin
         OutStringList = '[' + Tmp
      endif else begin
         OutStringList = OutStringList + Tmp
      endelse
EndFor
OutStringList = OutStringList + ']'

Return
End




;================================================================================
;================================================================================








Pro   MS_RotateCoordCCW,   XVec, YVec, RotatAngle, RotXVec, RotYvec, DEGREE=DEGREE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NDataX  = N_Elements (XVec)
NDataY  = N_Elements (YVec)
NData   = min ([NDataX, NDataY])

IF (NDataX  NE  NDataY)  $
   THEN BEGIN
      print, ' X and Y arrays have different length !  '
      return
   ENDIF

If (Keyword_Set (DEGREE))  $
   then begin
      SinPhi   = sin (RotatAngle(0) * !DTOR)
      CoSinPhi = cos (RotatAngle(0) * !DTOR)
   endif else begin
      SinPhi   = sin (RotatAngle(0))
      CoSinPhi = cos (RotatAngle(0))
   endelse

MS_InitializeVector,  SinPhiVec,   NData, SinPhi
MS_InitializeVector,  CoSinPhiVec, NData, CoSinPhi

RotXVec = + (XVec (*) * CoSinPhiVec(*))  +  (YVec (*) *   SinPhiVec(*))
RotYVec = - (XVec (*) *   SinPhiVec(*))  +  (YVec (*) * CoSinPhiVec(*))

Return
End









Pro   MS_RotateCoordCW,   XVec, YVec, RotatAngle, RotXVec, RotYVec, DEGREE=DEGREE
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NDataX  = N_Elements (XVec)
NDataY  = N_Elements (YVec)
NData   = min ([NDataX, NDataY])

IF (NdataX NE NDataY)  $
   THEN BEGIN
      print, ' X and Y arrays have different length !  '
      return
   ENDIF

If (Keyword_Set (DEGREE))  $
   then begin
      SinPhi   = sin (RotatAngle(0) * !DTOR)
      CoSinPhi = cos (RotatAngle(0) * !DTOR)
   endif else begin
      SinPhi   = sin (RotatAngle(0))
      CoSinPhi = cos (RotatAngle(0))
   endelse

MS_InitializeVector,  SinPhiVec,   NData, SinPhi
MS_InitializeVector,  CoSinPhiVec, NData, CoSinPhi

RotXVec = + (XVec (*) * CoSinPhiVec(*))  -  (YVec (*) *   SinPhiVec(*))
RotYVec = + (XVec (*) *   SinPhiVec(*))  +  (YVec (*) * CoSinPhiVec(*))


Return
End




;================================================================================
;================================================================================



Pro  MS_Conv_RADec_Rect,   RAPos, DecPos, RefRA, RefDec, XPos, YPos,  $
                           SPHERICDISTANCE=SPHERICDISTANCE,           $
                           DELTACOORDS=DELTACOORDS,                   $
                           RECTANGCOORDS=RECTANGCOORDS,               $
                           GNOMONIC=GNOMONIC,                         $
                           SERENDGNOMON=SERENDGNOMON,                 $
                           ARCMIN=ARCMIN, ARCSEC=ARCSEC,              $
                           NORMALXY=NORMALXY
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

FORWARD_FUNCTION    MS_SPHDIST


NData  = N_Elements (RAPos)

XPos   = DblArr (NData)
YPos   = DblArr (NData)

CFactor = 1.0
If (Keyword_Set (ArcMin))   then CFactor = 60.0
If (Keyword_Set (ArcSec))   then CFactor = 3600.0


If ( (NOT Keyword_Set (SphericDistance))     AND   $
     (NOT Keyword_Set (DeltaCoords))         AND   $
     (NOT Keyword_Set (RectAngCoords))       AND   $
     (NOT Keyword_Set (Gnomonic))       AND   $
     (NOT Keyword_Set (SerendGnomon)) )            $
   then begin
        Print, ' <Conv_RADec_Rect> NOTE: Doing Gnomonic Transformation from RA/Dec -> X/Y '

        RAPosRad  =  RAPos (*)  * !DTOR
        DecPosRad =  DecPos (*) * !DTOR
        RefRARad  =  RefRA      * !DTOR
        RefDecRad =  RefDec     * !DTOR

        A =  cos (DecPosRad(*)) * cos (RAPosRad(*) - RefRARad)
        F =  !RADEG / ( (sin (RefDecRad) * sin (DecPosRad(*))) + (A(*) * cos (RefDecRad)) )

        XPos (*) =  CFactor * F * cos (DecPosRad(*)) * sin (RAPosRad(*) - RefRARad)
        YPos (*) =  CFactor * F * ( (cos (RefDecRad) * sin (DecPosRad(*)))  -  (A(*) * sin (RefDecRad)) )
   endif


If (Keyword_Set (Gnomonic))  $
   then begin
        RAPosRad  =  RAPos (*)  * !DTOR
        DecPosRad =  DecPos (*) * !DTOR
        RefRARad  =  RefRA      * !DTOR
        RefDecRad =  RefDec     * !DTOR

        A =  cos (DecPosRad(*)) * cos (RAPosRad(*) - RefRARad)
        F =  !RADEG / ( (sin (RefDecRad) * sin (DecPosRad(*))) + (A(*) * cos (RefDecRad)) )

        XPos (*) =  CFactor * F * cos (DecPosRad(*)) * sin (RAPosRad(*) - RefRARad)
        YPos (*) =  CFactor * F * ( (cos (RefDecRad) * sin (DecPosRad(*)))  -  (A(*) * sin (RefDecRad)) )
   endif


If (Keyword_Set (DeltaCoords))  $
   then begin
      DeltaRA  = (RAPos (*) - RefRA)  *  cos (!DTOR * DecPos (*))
      DeltaDec =  DecPos (*) - RefDec

      XPos (*) =  CFactor * DeltaRA (*)
      YPos (*) =  CFactor * DeltaDec (*)
   endif


If (Keyword_Set (SphericDistance))  $
   then begin
      Print, ' <Conv_RADec_Rect> NOTE: Back-Transformation NOT yet supported '

      TempRAPos  = DblArr (NData)
      TempDecPos = DblArr (NData)

      TempRAPos (*)  = RAPos (*)
      TempDecPos (*) = RefDec

      DeltaRA  =  RAPos (*)  - RefRA
      DeltaDec =  DecPos (*) - RefDec

      SphDeltaRA  = MS_SphDist (TempRAPos(*) , TempDecPos(*) , RefRA, RefDec)
      XPos (*) = abs (CFactor * SphDeltaRA (*))

      For i = 0 , NData-1 , 1 do begin
         SphDeltaDec = MS_SphDist (TempRAPos(i) , TempDecPos(i) , RAPos(i) , DecPos(i))
         YPos (i) = abs (CFactor * SphDeltaDec)
      EndFor

      NegIndex = Where (DeltaRA (*) lt 0.0 , NNegativ)
      If (NNegativ gt 0)  then  XPos (NegIndex) = -1.0 * XPos (NegIndex)

      NegIndex = Where (DeltaDec (*) lt 0.0 , NNegativ)
      If (NNegativ gt 0)  then  YPos (NegIndex) = -1.0 * YPos (NegIndex)

   endif


If (Keyword_Set (RectAngCoords))  $
   then begin
      Print, ' <Conv_RADec_Rect> NOTE: Back-Transformation NOT yet supported '

      DeltaRA  = RAPos (*)  - RefRA
      DeltaDec = DecPos (*) - RefDec

      TANDeltaRA  = tan (DeltaRA (*) * !DTOR)
      TANDeltaRA2 = tan (0.5 * DeltaRA (*) * !DTOR)
      SINDeltaDec = sin (DeltaDec (*) * !DTOR)

      SINDec1     = sin (RefDec * !DTOR)
      COSDec1     = cos (RefDec * !DTOR)
      TanDec1     = tan (RefDec * !DTOR)

      K = 206264.8062 /    $
           (1.0 + (SINDec1 * SINDec1 * TANDeltaRA(*) * TANDeltaRA2(*)))

      XPos (*) = K * (1.0 - (TANDec1 * SINDeltaDec(*))) * COSDec1 * TANDeltaRA(*)
      XPos (*) = CFactor / 3600.0 * XPos (*)

      Ypos (*) = ((SINDec1 * COSDec1 * TANDeltaRA * TANDeltaRA2) + SINDeltaDec) * K

      YPos (*) = CFactor / 3600.0 * YPos (*)
   endif


If (Keyword_Set (SerendGnomon))  $
   then begin
      N_RA  = 3
      N_DEC = 3
      MS_DEG2GNO, RAPos, DECPos, RefRA, RefDEC, CFactor, N_RA, N_DEC, XPos, YPos, /FLOAT
      XPos = abs (XPos)
      YPos = abs (YPos)

      DeltaRA  =  RAPos (*)  - RefRA
      DeltaDec =  DecPos (*) - RefDec

      NegIndex = Where (DeltaRA (*) lt 0.0 , NNegativ)
      If (NNegativ gt 0)  then  XPos (NegIndex) = -1.0 * XPos (NegIndex)

      NegIndex = Where (DeltaDec (*) lt 0.0 , NNegativ)
      If (NNegativ gt 0)  then  YPos (NegIndex) = -1.0 * YPos (NegIndex)
   endif


IF (Keyword_Set (NORMALXY))  $
   then begin
      XPos (*) = -1.0 * XPos (*)
   ENDIF

Return
End
















Pro  MS_Conv_Rect_RADec,   XPos, YPos, RefRA, RefDec, RAPos, DecPos,  $
                           SPHERICDISTANCE=SPHERICDISTANCE,           $
                           DELTACOORDS=DELTACOORDS,                   $
                           RECTANGCOORDS=RECTANGCOORDS,               $
                           GNOMONIC=GNOMONIC,                         $
                           SERENDGNOMON=SERENDGNOMON,                 $
                           ARCMIN=ARCMIN, ARCSEC=ARCSEC,              $
                           NORMALXY=NORMALXY
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


NData  = N_Elements (XPos)

RAPos   = DblArr (NData)
DecPos  = DblArr (NData)

XXPos = XPos (*)
IF (Keyword_Set (NORMALXY))  $
   then begin
      XXPos = -1.0 * XPos (*)
   ENDIF

CFactor = 1.0
If (Keyword_Set (ArcMin))   then CFactor = 60.0
If (Keyword_Set (ArcSec))   then CFactor = 3600.0


If ( (NOT Keyword_Set (SphericDistance))     AND   $
     (NOT Keyword_Set (DeltaCoords))         AND   $
     (NOT Keyword_Set (RectAngCoords))       AND   $
     (NOT Keyword_Set (Gnomonic))            AND   $
     (NOT Keyword_Set (SerendGnomon)) )            $
   then begin
        Print, ' <Conv_Rect_RADec> NOTE: Doing Gnomonic Transformation from X/Y -> RA/Dec'

        RefRARad  =  RefRA      * !DTOR
        RefDecRad =  RefDec     * !DTOR

        XPosScale =  XXPos (*) / (CFactor * !RADEG)
        YPosScale =  YPos  (*) / (CFactor * !RADEG)

        D = ATAN (SQRT ((XPosScale(*)^2) + (YPosScale(*)^2)))
        B = ATAN (XPosScale(*) , YPosScale(*))

        XX =  (sin (RefDecRad) * sin (D(*)) * cos (B(*))) + (cos (RefDecRad) * cos (D(*)))
        YY =  sin (D(*)) * sin (B(*))

        RAPos  =  !RADEG * (RefRARad + ATAN (YY , XX))
        DecPos =  !RADEG * ASIN ( (sin (RefDecRad) * cos (D(*)))  -    $
                                  (cos (RefDecRad) * sin (D(*)) * cos (B(*))) )
   endif


If (Keyword_Set (Gnomonic))  $
   then begin
        RefRARad  =  RefRA      * !DTOR
        RefDecRad =  RefDec     * !DTOR

        XPosScale =  XXPos (*) / (CFactor * !RADEG)
        YPosScale =  YPos  (*) / (CFactor * !RADEG)

        D = ATAN (SQRT ((XPosScale(*)^2) + (YPosScale(*)^2)))
        B = ATAN (XPosScale(*) , YPosScale(*))

        XX =  (sin (RefDecRad) * sin (D(*)) * cos (B(*))) + (cos (RefDecRad) * cos (D(*)))
        YY =  sin (D(*)) * sin (B(*))

        RAPos  =  !RADEG * (RefRARad + ATAN (YY , XX))
        DecPos =  !RADEG * ASIN ( (sin (RefDecRad) * cos (D(*)))  -    $
                                  (cos (RefDecRad) * sin (D(*)) * cos (B(*))) )
   endif


If (Keyword_Set (DeltaCoords))  $
   then begin
      DeltaRA  =  XXPos (*) / cos (!DTOR * RefDec) / CFactor
      DeltaDec =  YPos (*) / CFactor

      RAPos (*)  =  RefRA  + DeltaRA (*)
      DecPos (*) =  RefDec + DeltaDec (*)
   endif


If (Keyword_Set (SphericDistance))  $
   then begin
      Print, ' <Conv_Rect_RADec> ... Transformation NOT yet supported '
      Return
   endif


If (Keyword_Set (RectAngCoords))  $
   then begin
      Print, ' <Conv_Rect_RADec> ... Transformation NOT yet supported '
      Return
   endif

If (Keyword_Set (SerendGnomon))  $
   then begin
       XXXPos = -1. * XXPos (*)
       N_RA  = 3
       N_DEC = 3
       MS_GNO2DEG, XXXPos, YPos, RefRA, RefDEC, CFactor, N_RA, N_DEC, RAPos, DECPos
   endif

Return
End



;-------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------




	PRO   MS_DEG2GNO,  RA, DEC, RA0, DEC0, SCALE, N_RA, N_DEC, I_RA, I_DEC, $
                           FLOAT = FLOAT
;+
; NAME:
;	DEG2GNO
;
; PURPOSE:
;	DEG2GNO converts RA and DEC given in degrees into pixel indices I_RA
;	and I_DEC in a gnonomic projection, assuming a projection center
;	[RA0, DEC0]. Optionally, you can choose I_RA and I_DEC returned as
;	floats.
;	Pixel [0,0] is assumed to be in the lower left corner.
;
; CATEGORY:
;       Astronomy.
;
; CALLING SEQUENCE:
;	DEG2GNO, RA, DEC, RA0, DEC0, SCALE, N_RA, N_DEC, I_RA, I_DEC $
;	   [, FLOAT = FLOAT ]
;
; INPUTS:
;	RA:	Float scalar or array; specifying the right ascension(s)
;		[degrees].
;	DEC:	Float scalar or array; specifying the declination(s) [degrees].
;	RA0:	Float scalar; specifying the projection center's right
;		acsension [degrees].
;	DEC0:	Float scalar; specifying the projection center's declination
;		[degrees].
;	SCALE:	Float scalar; specifying the number of pixels per degree.
;		[degrees^-1].
;	N_RA:	Int scalar; specifying the number of pixels in RA-direction.
;	N_DEC:	Int scalar; specifying the number of pixels in DEC-direction.
;
; OPTIONAL INPUT PARAMETERS:
;       None.
;
; OPTIONAL INPUT KEYWORDS:
;	FLOAT:	If specified, I_RA and I_DEC are returned as floats.
;
; OUTPUTS:
;	I_RA:	Int scalar or array; containing the index/indices of the
;		specified position(s) in RA-direction.
;	I_DEC:	Int scalar or array; containing the index/indices of the
;		specified position(s) in DEC-direction.
;
; OPTIONAL OUTPUT PARAMETERS:
;	None.
;
; OPTIONAL OUTPUT KEYWORDS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;       created by: Stefan Bogun            MPIA Heidelberg
;       	    March, 1995
;-

	ON_ERROR, 0

	IF N_PARAMS() LT 9 THEN MESSAGE, $
	   'Syntax: DEG2GNO, RA, DEC, RA0, DEC0, SCALE, N_RA, N_DEC, ' + $
	   'I_RA, I_DEC [, FLOAT = FLOAT ]'

	RAR	= RA / !RADEG
	DECR	= DEC / !RADEG
	RA0R	= RA0 / !RADEG
	DEC0R	= DEC0 / !RADEG

	COSDECR	= COS(DECR)
	SINDECR = SIN(DECR)

	A	= COSDECR * COS(RAR - RA0R)
	F	= SCALE * !RADEG / ( SIN(DEC0R) * SINDECR + A * COS(DEC0R) )

	I_RA	= -F * COSDECR * SIN(RAR - RA0R) + N_RA/2 - 1
	I_DEC	= F * ( COS(DEC0R) * SINDECR - A * SIN(DEC0R) ) + N_DEC/2 - 1

	IF NOT KEYWORD_SET(FLOAT) THEN BEGIN
	   I_RA  = NINT(I_RA < 32767 > (-32768) )
	   I_DEC = NINT(I_DEC < 32767 > (-32768) )
	ENDIF

	RETURN
	END









	PRO   MS_GNO2DEG, I_RA, I_DEC, RA0, DEC0, SCALE, N_RA, N_DEC, RA, DEC
;+
; NAME:
;	GNO2DEG
;
; PURPOSE:
;	GNO2DEG converts pixel indices I_RA and I_DEC in a gnonomic projection
;	into RA and DEC in degrees assuming a projection center [RA0, DEC0].
;	Pixel [0,0] is assumed to be in the lower left corner.
;
; CATEGORY:
;       Astronomy.
;
; CALLING SEQUENCE:
;	GNO2DEG, I_RA, I_DEC, RA0, DEC0, SCALE, N_RA, N_DEC, RA, DEC
;
; INPUTS:
;	I_RA:	Int/Flt scalar or array; specifying the index/indices of the
;		specified position(s) in RA-direction.
;	I_DEC:	Int/Flt scalar or array; specifying the index/indices of the
;		specified position(s) in DEC-direction.
;	RA0:	Float scalar; specifying the projection center's right
;		acsension [degrees].
;	DEC0:	Float scalar; specifying the projection center's declination
;		[degrees].
;	SCALE:	Float scalar; specifying the number of pixels per degree.
;		[degrees^-1].
;	N_RA:	Int scalar; specifying the number of pixels in RA-direction.
;	N_DEC:	Int scalar; specifying the number of pixels in DEC-direction.
;
; OPTIONAL INPUT PARAMETERS:
;       None.
;
; OPTIONAL INPUT KEYWORDS:
;	None.
;
; OUTPUTS:
;	RA:	Float scalar or array; containing the right ascension(s)
;		[degrees].
;	DEC:	Float scalar or array; containing the declination(s) [degrees].
;
; OPTIONAL OUTPUT PARAMETERS:
;	None.
;
; OPTIONAL OUTPUT KEYWORDS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;
; MODIFICATION HISTORY:
;       created by: Stefan Bogun            MPIA Heidelberg
;       	    March, 1995
;-

	ON_ERROR, 0

	IF N_PARAMS() LT 9 THEN MESSAGE, $
	   'Syntax: GNO2DEG, I_RA, I_DEC, RA0, DEC0, SCALE, N_RA, N_DEC, ' + $
	   'RA, DEC'

	RA0R	= RA0 / !RADEG
	DEC0R	= DEC0 / !RADEG

	X	= ( I_RA - N_RA/2 + 1) / ( SCALE * !RADEG )
	Y	= - ( I_DEC - N_DEC/2 + 1) / ( SCALE * !RADEG )

	SIND	= SQRT( ( X^2 + Y^2 ) / ( 1. + X^2 + Y^2 ) )
	COSD	= SQRT( 1. / ( 1. + X^2 + Y^2 ) )

	BETA	= ATAN( -X, Y )
	XX	= SIN(DEC0R) * SIND * COS(BETA) + COS(DEC0R) * COSD
	YY	= SIND * SIN(BETA)

	RA	= ( RA0R + ATAN( YY, XX ) ) * !RADEG
	I0	= WHERE( RA LT 0., N0 )
	IF N0 GT 0 THEN RA(I0) = RA(I0) + 360.
	DEC	= ASIN( SIN(DEC0R) * COSD - COS(DEC0R) * SIND * $
		  COS(BETA) ) * !RADEG
	RETURN
	END






	FUNCTION MS_SPHDIST, RA1, DEC1, RA2, DEC2
;+
; NAME:
;	SPHDIST
;
; PURPOSE:
;	SPHDIST calculates the distance between two points on a sphere.
;	SPHDIST is also able to calculate the distance between a single point
;	and a series of points or between two series of points.
;
; CATEGORY:
;       Astronomy.
;
; CALLING SEQUENCE:
;	RESULT = SPHDIST( RA1, DEC1, RA2, DEC2 )
;
; INPUTS:
;	RA1:	Flt scalar or array; specifying the right ascension of point 1
;		[degrees].
;	DEC1:	Flt scalar or array; specifying the declination of point 1
;		[degrees].
;	RA2:	Flt scalar or array; specifying the right ascension of point 2
;		[degrees].
;	DEC2:	Flt scalar or array; specifying the declination of point 2
;		[degrees].
;
; OPTIONAL INPUT PARAMETERS:
;       None.
;
; OPTIONAL INPUT KEYWORDS:
;	None.
;
; OUTPUTS:
;	SPHDIST returns the distance on the sphere between [RA1,DEC1] and
;	[RA2,DEC2] [degrees]. If the input variables are arrays, SPHDIST
;	returns an array of the same size.
;
; OPTIONAL OUTPUT PARAMETERS:
;	None.
;
; OPTIONAL OUTPUT KEYWORDS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;	RA1 and DEC1 must have the same dimensions, as well as RA2 and DEC2.
;	If RA1, DEC1 are arrays, then RA2, DEC2 must either have the same size
;	or must be scalars. In this case, the distance of [RA2,DEC2] to each
;	of the points specified in [RA1,DEC1] is returned. The same applies to
;	the opposite case.
;
; MODIFICATION HISTORY:
;       created by: Stefan Bogun            MPIA Heidelberg
;       	    March, 1995
;-

	ON_ERROR, 2

	IF N_PARAMS() LT 2 THEN MESSAGE, $
	   'Syntax: RESULT = SPHDIST( RA1, DEC1, RA2, DEC2 )'

	IF ( N_ELEMENTS(RA1) NE N_ELEMENTS(DEC1) ) OR $
	   ( N_ELEMENTS(RA2) NE N_ELEMENTS(DEC2) ) THEN MESSAGE, $
	   'Array dimensions are not compatible.'

	RA11	 = RA1
	DEC11	 = DEC1
	RA21	 = RA2
	DEC21	 = DEC2

	IF N_ELEMENTS(RA11) EQ 1 THEN BEGIN
	   RA11	 = RA21*0 + RA11
	   DEC11 = DEC21*0 + DEC11
	ENDIF

	IF N_ELEMENTS(RA21) EQ 1 THEN BEGIN
	   RA21	 = RA11*0 + RA21
	   DEC21 = DEC11*0 + DEC21
	ENDIF

	RAD	 = RA11 - RA21
	IN	 = WHERE( RAD LT 0, NN )
	IF NN GT 0 THEN BEGIN
	   RAN1	 = RA11(IN)
	   RAN2  = RA21(IN)
	   DECN1 = DEC11(IN)
	   DECN2 = DEC21(IN)

	   RA11(IN)  = RAN2
	   RA21(IN)  = RAN1
	   DEC11(IN) = DECN2
	   DEC21(IN) = DECN1
	ENDIF

	ID	 = WHERE( ABS(RAD) GT 180., ND )
	IF ND GT 0 THEN RA21(ID) = RA21(ID) + 360.

	RESULT	 = ABS( !RADEG*ACOS( SIN(DEC11/!RADEG)*SIN(DEC21/!RADEG) + $
	   COS(DEC11/!RADEG)*COS(DEC21/!RADEG)*COS((RA11-RA21)/!RADEG) ) )

	RETURN, RESULT
	END



;-------------------------------------------------------------------------------------------
;-------------------------------------------------------------------------------------------




;-------------------------------------------------------------
;+
; NAME:
;       DELCHR
; PURPOSE:
;       Delete all occurrences of a character from a text string.
; CATEGORY:
; CALLING SEQUENCE:
;       new = delchr(old, char)
; INPUTS:
;       old = original text string.     in
;       char = character to delete.     in
; KEYWORD PARAMETERS:
; OUTPUTS:
;       new = resulting string.         out
; COMMON BLOCKS:
; NOTES:
; MODIFICATION HISTORY:
;       R. Sterner.  5 Jul, 1988.
;       Johns Hopkins Applied Physics Lab.
;       RES 11 Sep, 1989 --- converted to SUN.
;       R. Sterner, 27 Jan, 1993 --- dropped reference to array.
;
; Copyright (C) 1988, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;-
;-------------------------------------------------------------

        FUNCTION DELCHR, OLD, C, help=hlp

        if (n_params(0) lt 2) or keyword_set(hlp) then begin
          print,' Delete all occurrences of a character from a text string.'
          print,' new = delchr(old, char) '
          print,'   old = original text string.     in'
          print,'   char = character to delete.     in'
          print,'   new = resulting string.         out'
          return, -1
        endif

        B = BYTE(OLD)                      ; convert string to a byte array.
        CB = BYTE(C)                       ; convert char to byte.
        w = where(b ne cb(0))
        if w(0) eq -1 then return, ''      ; Nothing left.
        return, string(b(w))               ; Return new string.
        END



;-------------------------------------------------------------
;+
; NAME:
;       NWRDS
; PURPOSE:
;       Return the number of words in the given text string.
; CATEGORY:
; CALLING SEQUENCE:
;       n = nwrds(txt)
; INPUTS:
;       txt = text string to examine.             in
; KEYWORD PARAMETERS:
;       Keywords:
;         DELIMITER = d.  Set delimiter character (def = space).
; OUTPUTS:
;       n = number of words found.                out
; COMMON BLOCKS:
; NOTES:
;       Notes: See also getwrd.
; MODIFICATION HISTORY:
;       R. Sterner,  7 Feb, 1985.
;       Johns Hopkins University Applied Physics Laboratory.
;       RES 4 Sep, 1989 --- converted to SUN.
;
; Copyright (C) 1985, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;-
;-------------------------------------------------------------


        function nwrds,txtstr, help=hlp, delimiter=delim

        if (n_params(0) lt 1) or keyword_set(hlp) then begin
          print,' Return the number of words in the given text string.'
          print,' n = nwrds(txt)'
          print,'   txt = text string to examine.             in'
          print,'   n = number of words found.                out'
          print,' Keywords:'
          print,'   DELIMITER = d.  Set delimiter character (def = space).'
          print,' Notes: See also getwrd.'
          return, -1
        endif

        if strlen(txtstr) eq 0 then return,0    ; A null string has 0 words.
        ddel = ' '                      ; Default word delimiter is a space.
        if n_elements(delim) ne 0 then ddel = delim ; Use given word delimiter.
        tst = (byte(ddel))(0)                   ; Delimiter as a byte value.
        tb = byte(txtstr)                             ; String to bytes.
        if ddel eq ' ' then begin                     ; Check for tabs?
          w = where(tb eq 9B, cnt)                    ; Yes.
          if cnt gt 0 then tb(w) = 32B                ; Convert any to space.
        endif
        x = tb ne tst                           ; Locate words.
        x = [0,x,0]                             ; Pad ends with delimiters.

        y = (x-shift(x,1)) eq 1                 ; Look for word beginnings.

        n = fix(total(y))                       ; Count word beginnings.

        return, n

        END





;-------------------------------------------------------------
;+
; NAME:
;       ISNUMBER
; PURPOSE:
;       Determine if a text string is a valid number.
; CATEGORY:
; CALLING SEQUENCE:
;       i = isnumber(txt, [x])
; INPUTS:
;       txt = text string to test.                      in
; KEYWORD PARAMETERS:
; OUTPUTS:
;       x = optionaly returned numeric value if valid.  out
;       i = test flag:                                  out
;           0: not a number.
;           1: txt is a long integer.
;           2: txt is a float.
;           -1: first word of txt is a long integer.
;           -2: first word of txt is a float.
; COMMON BLOCKS:
; NOTES:
; MODIFICATION HISTORY:
;       R. Sterner.  15 Oct, 1986.
;       Johns Hopkins Applied Physics Lab.
;       R. Sterner, 12 Mar, 1990 --- upgraded.
;       Richard Garrett, 14 June, 1992 --- fixed bug in returned float value.
;
; Copyright (C) 1986, Johns Hopkins University/Applied Physics Laboratory
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever.  Other limitations apply as described in the file disclaimer.txt.
;-
;-------------------------------------------------------------

        FUNCTION ISNUMBER, TXT0, X, help=hlp

        if (n_params(0) lt 1) or keyword_set(hlp) then begin
          print,' Determine if a text string is a valid number.'
          print,' i = isnumber(txt, [x]) '
          print,'   txt = text string to test.                      in'
          print,'   x = optionaly returned numeric value if valid.  out'
          print,'   i = test flag:                                  out'
          print,'       0: not a number.'
          print,'       1: txt is a long integer.'
          print,'       2: txt is a float.'
          print,'       -1: first word of txt is a long integer.'
          print,'       -2: first word of txt is a float.'
          return, -1
        endif

        TXT = STRTRIM(TXT0,2)   ; trim blanks.
        X = 0                   ; define X.

        IF TXT EQ '' THEN RETURN, 0     ; null string not a number.

        SN = 1
        IF NWRDS(TXT) GT 1 THEN BEGIN   ; get first word if more than one.
          SN = -1
          TXT = GETWRD(TXT,0)
        ENDIF

        f_flag = 0              ; Floating flag.
        b = byte(txt)
        w = where(b eq 43, cnt)
        if cnt gt 1 then return, 0
        t = delchr(txt,'+')
        w = where(b eq 45, cnt)
        if cnt gt 1 then return, 0
        t = delchr(t,'-')
        w = where(b eq 46, cnt)                 ; '.'
        if cnt gt 1 then return, 0              ; May only be 1.
        if cnt eq 1 then f_flag = 1             ; If one then floating.
        t = delchr(t,'.')
        w = where(b eq 101, cnt)                ; 'e'
        if cnt gt 1 then return, 0
        if cnt eq 1 then f_flag = 1
        t = delchr(t,'e')
        w = where(b eq 69, cnt)                 ; 'E'
        if cnt gt 1 then return, 0
        if cnt eq 1 then f_flag = 1
        t = delchr(t,'E')
        w = where(b eq 100, cnt)                ; 'd'
        if cnt gt 1 then return, 0
        if cnt eq 1 then f_flag = 1
        t = delchr(t,'d')
        w = where(b eq 68, cnt)                 ; 'D'
        if cnt gt 1 then return, 0
        if cnt eq 1 then f_flag = 1
        t = delchr(t,'D')
        if total((b eq 101)+(b eq 69)+(b eq 100)+(b eq 68)) gt 1 then return,0
        b = byte(t)
        if total((b ge 65) and (b le 122)) ne 0 then return, 0

        c = strmid(t,0,1)
        if (c lt '0') or (c gt '9') then return, 0  ; First char not a digit.

        x = txt + 0.0                               ; Convert to a float.
        if f_flag eq 1 then return, 2*sn            ; Was floating.
        if x eq long(x) then begin
          x = long(x)
          return, sn
        endif else begin
          return, 2*sn
        endelse

        END

; ---------------------------------------------------------------------------
; Main routine
;

Pro  MapAAP2Drizzle,      FileName_or_FileList,                           $
                          ScriptName=InScriptName,                        $
                          IRAFCommandName=InIRAFCommandName,              $
                          ImageName=ImageName,                    $
                          Identifier=InIdentifier,                        $
                          Scale=Scale,                            $
                          MapPixelSize=MapPixelSize,              $
                          PositAngle=PositAngle,                  $
                          DrizzleFraction=DrizzleFraction,        $
                          FillValue=InFillValue,                          $
                          Parallel=InNoDeltaStripes,                      $
                          DetectorFlatField=DetectorFlatFieldVec,         $
                          AAPGlobalFlatField=AAPGlobalRescaleFacts,       $
                          IgnorePixel=InIgnorePixelVec,                   $
                          MapRACenter=MAPRACenter,                 $
			  MapDECCenter=MapDECCenter,               $
                          IncreaseSize=IncreaseSize,               $
                          NPixX=InNPixX, NPixY=InNPixY,                   $
                          EVERYPIXEL=EVERYPIXEL, SINGLEAAP=SINGLEAAP,     $
                          SURFACEBRIGHTNESS=SURFACEBRIGHTNESS,            $
                          FLUX=FLUX, MEANVALUES=MEANVALUES,               $
                          MEDIANVALUES=MEDIANVALUES,                      $
                          SIMPLEPROJECTION=SIMPLEPROJECTION,              $
                          GNOMONICPROJECTION=GNOMONICPROJECTION,          $
                          ADDSCALETONAME=ADDSCALETONAME,                  $
                          OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,            $
                          AVERROLLCOORDS=AVERROLLCOORDS,                  $
                          SPACECRAFTCOORDS=SPACECRAFTCOORDS,              $
                          IGNOREROLLANGLE=IGNOREROLLANGLE,                $
                          EXECUTE=EXECUTE,                                $
                          FITS=FITS, DEBUG=DEBUG
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
FORWARD_FUNCTION    MS_SPHDIST, ISNUMBER


; /* this is ... */

Version = 'MapAAP2Drizzle  Version 3.7   Jan. 20, 1998'

; /* No of AAP Datasets */

NDataSet =  N_Elements (FileName_or_FileList)
IF (NDataSet LE 0)   $
   then  begin
      print, ' '
      print, ' <MapAAP2Drizzle> ... No AAP file given !'
      PrintUsage
      Return
   endif

; /* check startup params */

If (n_params()  eq  0)   $
   then  BEGIN
      PrintUsage
      Return
   endif

IF ((NOT ISNUMBER (FileName_or_FileList(0)))  AND   $
      (STRLOWCASE (FileName_or_FileList(0))  eq  'help'))     $
   then  begin
      PrintUsage
      return
   endif


; Ask for Keyword_set so that 0's can be used as discriminators (CG)
IF KEYWORD_SET(imagename) THEN InImageName=imagename
IF KEYWORD_SET(scale) THEN InScale=Scale
IF KEYWORD_SET(MapPixelSize) THEN $
   IF MapPixelSize(0) NE 0. OR mappixelsize(1) NE 0. THEN $
             InMapPixelSize=MapPixelSize
IF KEYWORD_SET(PositAngle) THEN InPositAngle = PositAngle
IF KEYWORD_SET(DrizzleFraction) THEN InDrizzleFraction=DrizzleFraction
IF KEYWORD_SET(MapRACenter) THEN InRefRA=MapRACenter
IF KEYWORD_SET(MapDECCenter) THEN InRefDEC=MapDECCenter
IF KEYWORD_SET(IncreaseSize) THEN InIncreaseSize=IncreaseSize

;IF KEYWORD_SET(imagename) THEN print, 'SET imagename:', imagename
;IF KEYWORD_SET(scale) THEN print, 'SET scale:',scale
;IF KEYWORD_SET(MapPixelSize) THEN $
;   IF MapPixelSize(0) NE 0. OR mappixelsize(1) NE 0. THEN $
;             PRINT, 'SET mappixelsize:', mappixelsize
;IF KEYWORD_SET(PositAngle) THEN PRINT, 'SET positangle:', positangle
;IF KEYWORD_SET(DrizzleFraction) THEN PRINT, 'SET drizzlefraction:', $
;drizzlefraction
;IF KEYWORD_SET(MapRACenter) THEN print, 'Set mapracenter:', mapracenter
;IF KEYWORD_SET(MapDECCenter) THEN print, 'Set mapdeccenter:', mapdeccenter
;IF KEYWORD_SET(IncreaseSize) THEN print, 'set increasesize:', increasesize

; /* get rid of all open unit numbers ... damn */

Close , /ALL

; /* StartUp Message */

PrintVersion,  Version

; /* Build up calling sequence */

If (n_params()  gt  0)   $
   then  begin
      CalledAs = ' MapAAP2Drizzle '

      If (ISNUMBER (FileName_or_FileList(0)))  $
         then begin
            MS_Vector2StringList,  FileName_or_FileList , Tmp
            CalledAs = CalledAs + ', ' + StrCompress (String (FileName_or_FileList) , /REM) + ' '
         endif else begin
            MS_Vector2QuotedStringList,  FileName_or_FileList , Tmp
            CalledAs = CalledAs + ', ' + Tmp + ' '
         endelse

      If (N_Elements (InScriptName) GT 0)  $
         then   CalledAs = CalledAs + ", ScriptName='" + StrCompress (InScriptName , /REM) + "' "
      If (N_Elements (InIRAFCommandName) GT 0)  $
         then   CalledAs = CalledAs + ", IRAFCommandName='" + StrCompress (InIRAFCommandName , /REM) + "' "
      If (N_Elements (InImageName) GT 0)  $
         then   CalledAs = CalledAs + ", ImageName='" + StrCompress (InImageName , /REM) + "' "
      If (N_Elements (InIdentifier) GT 0)  $
         then   CalledAs = CalledAs + ", Identifier='" + StrCompress (InIdentifier , /REM) + "' "
      If (N_Elements (InPositAngle) GT 0)  $
         then   CalledAs = CalledAs + ', PositAngle=' + StrCompress (String(InPositAngle) , /REM) + ' '
      If (N_Elements (InDrizzleFraction) GT 0)  $
         then   CalledAs = CalledAs + ', DrizzleFraction=' + StrCompress (String(InDrizzleFraction) , /REM) + ' '
       If (N_Elements (InFillValue) GT 0)  $
         then   CalledAs = CalledAs + ', FillValue=' + StrCompress (String(InFillValue) , /REM) + ' '
      If (N_Elements (InIncreaseSize) GT 0)  $
         then   CalledAs = CalledAs + ', IncreaseSize=' + StrCompress (String(InIncreaseSize) , /REM) + ' '
      If (N_Elements (InNPixX) GT 0)  $
         then   CalledAs = CalledAs + ', NPixX=' + StrCompress (String(InNPixX) , /REM) + ' '
      If (N_Elements (InNPixY) GT 0)  $
         then   CalledAs = CalledAs + ', NPixY=' + StrCompress (String(InNPixY) , /REM) + ' '

      If (N_Elements (DetectorFlatFieldVec) GT 0)  $
         then begin
            MS_Vector2StringList,  DetectorFlatFieldVec , Tmp
            CalledAs = CalledAs + ', DetectorFlatField=' + TMP + ' '
         endif
      If (N_Elements (AAPGlobalRescaleFacts) GT 0)  $
         then begin
            MS_Vector2StringList,  AAPGlobalRescaleFacts , Tmp
            CalledAs = CalledAs + ', AAPGlobalFlatField=' + TMP + ' '
         endif
      If (N_Elements (InIgnorePixelVec) GT 0)  $
         then begin
            MS_Vector2StringList,  InIgnorePixelVec , Tmp
            CalledAs = CalledAs + ', IgnorePixel=' + TMP + ' '
         endif

      If (N_Elements (InRefRA) GT 0)   $
         then   CalledAs = CalledAs + ', MapRACenter=' + StrCompress (String(InRefRA) , /REM) + ' '
      If (N_Elements (InRefDEC) GT 0)  $
         then   CalledAs = CalledAs + ', MapDECCenter=' + StrCompress (String(InRefDEC) , /REM) + ' '

      If (Keyword_Set (FLUX))              then CalledAs = CalledAs + ', /FLUX '
      If (Keyword_Set (SURFACEBRIGHTNESS)) then CalledAs = CalledAs + ', /SURFACEBRIGHTNESS '
      If (Keyword_Set (MEANVALUES))        then CalledAs = CalledAs + ', /MEANVALUES '
      If (Keyword_Set (MEDIANVALUES))      then CalledAs = CalledAs + ', /MEDIANVALUES '

      If (Keyword_Set (GNOMONICPROJECTION))then CalledAs = CalledAs + ', /GNOMONICPROJECTION '
      If (Keyword_Set (SIMPLEPROJECTION))  then CalledAs = CalledAs + ', /SIMPLEPROJECTION '

      If (N_Elements (InNoDeltaStripes) GT 0)  $
         then   CalledAs = CalledAs + ', Parallel=' + StrCompress (String(InNoDeltaStripes) , /REM) + ' '

      If (Keyword_Set (EVERYPIXEL))        then CalledAs = CalledAs + ', /EVERYPIXEL '
      If (Keyword_Set (SINGLEAAP))         then CalledAs = CalledAs + ', /SINGLEAAP '

      If (Keyword_Set (ADDSCALETONAME))    then CalledAs = CalledAs + ', /ADDSCALETONAME '
      If (Keyword_Set (FITS))              then CalledAs = CalledAs + ', /FITS '
   endif

; /* analyze input params */

If (N_Elements (InScale) GT 0.0) $
   THEN BEGIN
      RelativeScale = InScale
   ENDIF ELSE BEGIN
      RelativeScale = 0.50
   ENDELSE

If (N_Elements (InDrizzleFraction) GT 0.0) $
   THEN BEGIN
      DrizzleFraction = InDrizzleFraction
   ENDIF ELSE BEGIN
      DrizzleFraction = 0.65
   ENDELSE

If (N_Elements (InFillValue) GT 0.0) $
   THEN BEGIN
      FillValue = InFillValue
   ENDIF ELSE BEGIN
      FillValue = 0.0
   ENDELSE

If (N_Elements (InPositAngle) GT 0.0) $
   THEN BEGIN
      PositAngle = InPositAngle
   ENDIF ELSE BEGIN
      PositAngle = 0.0
   ENDELSE

If (N_Elements (InIdentifier) GT 0.0) $
   THEN BEGIN
      Identifier = InIdentifier
   ENDIF ELSE BEGIN
      Identifier = 'Undef'
   ENDELSE

If (N_Elements (InIncreaseSize) GT 0.0) $
   THEN BEGIN
      IncreaseSize = InIncreaseSize
   ENDIF ELSE BEGIN
      IncreaseSize = 0L
   ENDELSE

NoDeltaStripes = -1L
If ((Keyword_Set (PARALLEL))  OR  (N_Elements (InNoDeltaStripes) GT 0))   $
   THEN BEGIN
       NoDeltaStripes = 8L
       If (N_Elements (InNoDeltaStripes) GT 0L) $
         THEN BEGIN
            NoDeltaStripes = InNoDeltaStripes
         ENDIF
      If (NoDeltaStripes gt 16L)  $
         THEN BEGIN
            NoDeltaStripes = 16L
            print, ' <MapAAP2Drizzle> ... toooooo many declination - stripes :'
            print, ' <MapAAP2Drizzle>                     set to 16 !!!!!!!!! '
         ENDIF
      TwoPower = [2L , 4L , 8L , 16L]
      If ((Where ((float(TwoPower) / float(NoDeltaStripes)) eq 1.0))(0) eq -1)  $
         then begin
            Tmp = abs ((Float (TwoPower) / Float (NoDeltaStripes)) - 1.0)
            TmpMin = Min (Tmp , TmpMinIndex)
            NoDeltaStripes = Twopower(TmpMinIndex)
            print, ' <MapAAP2Drizzle> ... ' + STRCOMPRESS (String (NoDeltaStripes),/REMO)   $
                     + ' declination - stripes used !!'
         endif
      NoProcLevels = 1L
      If (NoDeltaStripes eq 2L)   then  NoProcLevels = 2L
      If (NoDeltaStripes eq 4L)   then  NoProcLevels = 3L
      If (NoDeltaStripes eq 8L)   then  NoProcLevels = 4L
      If (NoDeltaStripes eq 16L)  then  NoProcLevels = 5L
   ENDIF


If (N_Elements (DetectorFlatFieldVec) GT 0) $
   THEN BEGIN
      PixelFlatField     = DetectorFlatFieldVec (*)
   ENDIF ELSE BEGIN
      PixelFlatField     = FLTARR (9)
      PixelFlatField (*) = 1.0
   ENDELSE

If (N_Elements (AAPGlobalRescaleFacts) GT 0) $
   THEN BEGIN
      AAPGlobalFlatField     =  AAPGlobalRescaleFacts (*)
   ENDIF ELSE BEGIN
      AAPGlobalFlatField     = FLTARR (NDataSet)
      AAPGlobalFlatField (*) = 1.0
   ENDELSE


If (N_Elements (InIgnorePixelVec) GT 0) $
   THEN BEGIN
      IgnorePixelVec = LONG (InIgnorePixelVec (*))
   ENDIF ELSE BEGIN
      IgnorePixelVec     = LONARR (9)
      IgnorePixelVec (*) = -1L
   ENDELSE


; /* define a few magic numbers .... */

InitialPixel      = 0L

; /* determine map center and range in RA and Dec */

print, ' <MapAAP2Drizzle> ... checking input data '

FOR i = 0L , NDataSet-1L , 1  DO BEGIN

   ; /* take also into account global map rescaling factors */

    FlatField = PixelFlatField (*) * AAPGlobalFlatField (i)

   ; /* read data files */

   Read_AAP_Data,    FileName_or_FileList(i), AAP_STRUCTURE

   ; /* select data to be used */

   Copy_AAP_Data , AAP_STRUCTURE, AAPName, Detector, Aperture,        $
                   RACenter, DecCenter,                               $
                   RollAngle, SurfBright, Flux, Flag,                 $
                   RelChopPos, ChopStepID, RastPntID, FlatField,      $
                   MEANVALUES=MEANVALUES, MEDIANVALUES=MEDIANVALUES

   ; /* derive pixel coords etc. */

   MakePixelCoord,    Detector, Aperture, RACenter, DecCenter, RollAngle,  $
                      RelChopPos, DetPixelSize, DetPixelDist,    $
                      RAPixel, DecPixel

   ; /* size of data */

   Dimen  = Size (RAPixel)
   NPixel = Dimen(1)
   NData  = Dimen(2)


   ; /* get range of coordinates */

   IF (i EQ 0)  $
      THEN BEGIN
         RAMin  = Min (RAPixel(*,*)  , Max=RAMax)
         DecMin = Min (DecPixel(*,*) , Max=DecMax)
      ENDIF ELSE BEGIN
         RAMin  = Min ([RAMin , Min(RAPixel(*,*))])
         RAMax  = Max ([RAMax , Max(RAPixel(*,*))])
         DecMin = Min ([DecMin , Min(DecPixel(*,*))])
         DecMax = Max ([DecMax , Max(DecPixel(*,*))])
      ENDELSE

ENDFOR

RAMean  = 0.5 * (RAMin + RAMax)
DecMean = 0.5 * (DecMin + DecMax)

If (N_Elements (InRefRA) GT 0)   $
   then begin
      RAMean  = InRefRA
      DeltaRA = MAX ([abs(RAMin-RAMean) , abs(RAMax-RAMean)])
      RAMin   = RAMean - DeltaRA
      RAMax   = RAMean + DeltaRA
   endif
If (N_Elements (InRefDEC) GT 0)  $
   then begin
      DecMean  = InRefDec
      DeltaDec = MAX ([abs(DecMin-DecMean) , abs(DecMax-DecMean)])
      DecMin   = DecMean - DeltaDec
      DecMax   = DecMean + DeltaDec
   endif



; /* Find reference pixel nearest to map center */

print,  ' <MapAAP2Drizzle> ... finding reference position  '

AAPNameList = STRARR (NDataSet)

FOR i = 0L , NDataSet-1L , 1 DO BEGIN

   ; /* take also into account global map rescaling factors */

    FlatField = PixelFlatField (*) * AAPGlobalFlatField (i)

   ; /* read data files */

   Read_AAP_Data,    FileName_or_FileList(i), AAP_STRUCTURE

   ; /* select data to be used */

   Copy_AAP_Data , AAP_STRUCTURE, AAPName, Detector, Aperture,      $
                   RACenter, DecCenter,                             $
                   RollAngle, SurfBright, Flux, Flag,               $
                   RelChopPos, ChopStepID, RastPntID, FlatField,    $
                   MEANVALUES=MEANVALUES, MEDIANVALUES=MEDIANVALUES

   AAPNameList (i) = AAPName

   ; /* get the pixel RA / Dec coordinates */

   MakePixelCoord,    Detector, Aperture, RACenter, DecCenter, RollAngle,  $
                      RelChopPos, DetPixelSize, DetPixelDist,              $
                      RAPixel, DecPixel

   ; /* determine pixel size output map image */

   IF (i EQ 0)    $
      THEN BEGIN
         ImagePixelSize = RelativeScale * DetPixelSize
         IF (N_Elements (InMapPixelSize) GT 0)  $
            THEN BEGIN
               ImagePixelSize = InMapPixelSize
               RelativeScale  = ImagePixelSize / DetPixelSize
            ENDIF
      ENDIF


   ; /* data to be used */

   If (keyword_set (SurfaceBrightness))  $
      then BEGIN
         Data = SurfBright (*,*)
         IF (I EQ 0)  THEN print, ' <MapAAP2Drizzle> ... using SURFACE BRIGHTNESSes'
      ENDIF ELSE  BEGIN
         Data = Flux (*,*)
         IF (I EQ 0)  THEN print, ' <MapAAP2Drizzle> ... using FLUXes '
      ENDELSE

   Distances    =  MS_SphDist (RAMean , DecMean , RACenter(*) , DecCenter(*))
   SortIndxDist =  SORT (Distances(*))

   ; /* Prepare the reference data */

   IF (i EQ 0)  $
      THEN BEGIN
         DistMin      = Distances (SortIndxDist(0))
         DistMinIndx  = SortIndxDist (0)
         DistMinMNo   = i
         RefRA        = RACenter (DistMinIndx)
         RefDec       = DecCenter (DistMinIndx)
         RefData      = Data (InitialPixel , DistMinIndx)
         RefRot       = RollAngle (DistMinIndx)
      ENDIF ELSE BEGIN
         IF (Distances (SortIndxDist(0)) LT DistMin)  $
             THEN BEGIN
                DistMin      = Distances (SortIndxDist(0))
                DistMinIndx  = SortIndxDist (0)
                DistMinMNo   = i
                RefRA        = RACenter (DistMinIndx)
                RefDec       = DecCenter (DistMinIndx)
                RefData      = Data (InitialPixel , DistMinIndx)
                RefRot       = RollAngle (DistMinIndx)
            ENDIF
      ENDELSE

ENDFOR


; /* prepare the reference rotation angle */

IF (Keyword_Set (IGNOREROLLANGLE)) $
   then BEGIN
      RefRot =  0.0
   ENDIF ELSE  BEGIN
      RefRot =  RefRot
   ENDELSE



; /* use reference position given on command line instead */

If ((N_Elements (InRefRA) GT 0)  AND  (N_Elements (InRefDEC) LE 0))  $
   then begin
      RefRA = InRefRA
   endif

If ((N_Elements (InRefRA) LE 0)  AND  (N_Elements (InRefDEC) GT 0))  $
   then begin
      RefDEC = InRefDEC
   endif

If ((N_Elements (InRefRA) GT 0)  AND  (N_Elements (InRefDEC) GT 0))  $
   then begin
      RefRA  = InRefRA
      RefDEC = InRefDEC
   endif



;  /* Add the actually used RefPos to call */
;  /* ... THAT'S SIMPLY UNKNOWN BEFORE     */

If (N_Elements (InRefRA) LE 0)   $
   then   CalledAs = CalledAs + ', MapRACenter=' + StrCompress (String(RefRA) , /REM) + ' '
If (N_Elements (InRefDEC) LE 0)  $
   then   CalledAs = CalledAs + ', MapDecCenter=' + StrCompress (String(RefDEC) , /REM) + ' '


;  /* Add the actually used pixel size / pixel scale  to call */
;  /* ... THAT'S SIMPLY UNKNOWN BEFORE                        */

If (N_Elements (InScale) GT 0)  $
   then   CalledAs = CalledAs + ', Scale=' + StrCompress (String(RelativeScale) , /REM) + ' '
If (N_Elements (InMapPixelSize) GT 0)  $
   then   CalledAs = CalledAs + ', MapPixelSize=' + StrCompress (String(ImagePixelSize) , /REM) + ' '


; /* set the relative scale as file name extension  */

IF (Keyword_Set (ADDSCALETONAME))  $
   THEN BEGIN
       IF (RelativeScale LT 1.0)  $
          THEN BEGIN
             FileExtension = STRCOMPRESS('_0'+STRING(LONG(+STRING(RelativeScale*1000.,FORMAT='(f10.5)'))),/remo)
          ENDIF ELSE BEGIN
             FileExtension = STRCOMPRESS('_'+STRING(LONG(+STRING(RelativeScale*1000.,FORMAT='(f10.5)'))),/remo)
          ENDELSE
   ENDIF ELSE BEGIN
       FileExtension =  ''
   ENDELSE



; /* prepare image and script names */

PrepareGeneralFileNames,   InImageName, InScriptName, InIRAFCommandName,        $
                           AAPNameList, NDataSet, FileExtension,                $
                           IRAFCommandName,                                     $
                           ScriptName,                                          $
                           ImageName, WeightName,                               $
                           ScriptNameList,                                      $
                           ImageNameList, WeightNameList,                       $
                           SINGLEAAP=SINGLEAAP



;  /* Prepare the parallel mode (Names etc) */

If (NoDeltaStripes gt 0)   OR  (Keyword_Set (PARALLEL))  $
   then begin
      PrepareNamesParallelMode,   NoDeltaStripes, NoProcLevels,                 $
                                  ImageName, WeightName, FileExtension,         $
                                  DecMin, DecMax, DecStartVec, DecEndVec,       $
                                  ScriptNameList, ImageNameList, WeightNameList
   endif


;  /* Prepare the everypixel mode (Names etc) */

If (Keyword_Set (EVERYPIXEL))  $
   then begin
      PrepareNamesEveryPixelMode,    Detector, ImageName, WeightName,   $
                                     FileExtension, ScriptNameList,     $
                                     ImageNameList, WeightNameList
   endif


; /* open IRAF command file */

OpenW , CommandUnit , !IRAF_DATA + IRAFCommandName , /Get_Lun
OutString =  ' '
PrintF ,  CommandUnit , Format='(A)' , OutString


; /* write IRAF command file                       */
; /* open drizzle script file(s) for various modes */

IF ((NOT Keyword_Set (SINGLEAAP))      AND          $
    (NOT Keyword_Set (EVERYPIXEL))     AND          $
    (NoDeltaStripes le 0)              AND          $
    (NOT Keyword_Set (PARALLEL)))                   $
   THEN BEGIN
      print, ' <MapAAP2Drizzle> ... opening ' , ScriptName
      OpenW , OutUnit , !IRAF_DATA + ScriptName , /Get_Lun

      WriteIRAFCommandFile,  CommandUnit, ScriptName, LogFileName, EXECUTE=EXECUTE

;      DotPos = STRPOS (ScriptName,".")
;      LogFileName =  STRCOMPRESS (STRMID (ScriptName,0,DotPos+1) + 'log' , /REMO)

;      OutString =  '!rm   ' + LogFileName
;      PrintF ,  CommandUnit , Format='(A)' , OutString

;      OutString =  ' '
;      PrintF ,  CommandUnit , Format='(A)' , OutString

;      OutString =  'cl  <  ' + ScriptName + '   >  ' + LogFileName + ' & '
;      PrintF ,  CommandUnit , Format='(A)' , OutString

;      OutString =  ' '
;      PrintF ,  CommandUnit , Format='(A)' , OutString
   ENDIF

If (NoDeltaStripes gt 0)   OR  (Keyword_Set (PARALLEL))  $
   then begin
      OutUnitList = IntArr (2*NoDeltaStripes-1)
      For i = 0 , (2*NoDeltaStripes)-2 , 1  do  begin
         ScriptName = ScriptNameList (i)

         print, ' <MapAAP2Drizzle> ... opening ' , ScriptName
         OpenW , OutUnit , !IRAF_DATA + ScriptName , /Get_Lun
         OutUnitList (i) = OutUnit

         WriteIRAFCommandFile,  CommandUnit, ScriptName, LogFileName

;         DotPos = STRPOS (ScriptName,".")
;         LogFileName =  STRCOMPRESS (STRMID (ScriptName,0,DotPos+1) + 'log' , /REMO)

;         OutString =  '!rm   ' + LogFileName
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  'cl  <  ' + ScriptName + '   >  ' + LogFileName + ' & '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString
      EndFor
   endif

If (Keyword_Set (EVERYPIXEL))  $
   then begin
      OutUnitList = IntArr (N_Elements(ScriptNameList))
      For i = 0 , (N_Elements(ScriptNameList))-1 , 1  do  begin
         ScriptName = ScriptNameList (i)

         print, ' <MapAAP2Drizzle> ... opening ' , ScriptName
         OpenW , OutUnit , !IRAF_DATA + ScriptName , /Get_Lun
         OutUnitList (i) = OutUnit

         WriteIRAFCommandFile,  CommandUnit, ScriptName, LogFileName

;         DotPos = STRPOS (ScriptName,".")
;         LogFileName =  STRCOMPRESS (STRMID (ScriptName,0,DotPos+1) + 'log' , /REMO)

;         OutString =  '!rm   ' + LogFileName
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  'cl  <  ' + ScriptName + '   >  ' + LogFileName + ' & '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString
      EndFor
   endif

OutString =  ' '
PrintF ,  CommandUnit , Format='(A)' , OutString



; /* eventually : loop over all AAP input files and write scripts */

print, ' <MapAAP2Drizzle> ... main loop over all AAP files '

FOR i = 0L , NDataSet-1L , 1  DO BEGIN   ; ---------------------------------

   ; /* take also into account global map rescaling factors */

    FlatField = PixelFlatField (*) * AAPGlobalFlatField (i)

   ; /* each AAP get its own script */

   IF (Keyword_Set (SingleAAP))  $
      THEN BEGIN
         ScriptName =  ScriptNameList (i)
         ImageName  =  ImageNameList (i)
         Weightname =  WeightNameList (i)

         print, ' <MapAAP2Drizzle> ... opening ' , ScriptName
         OpenW , OutUnit , !IRAF_DATA + ScriptName , /Get_Lun

         WriteIRAFCommandFile,  CommandUnit, ScriptName, LogFileName

;         DotPos = STRPOS (ScriptName,".")
;         LogFileName =  STRCOMPRESS (STRMID (ScriptName,0,DotPos+1) + 'log' , /REMO)

;         OutString =  '!rm   ' + LogFileName
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  'cl  <  ' + ScriptName + '   >  ' + LogFileName + ' & '
;         PrintF ,  CommandUnit , Format='(A)' , OutString

;         OutString =  ' '
;         PrintF ,  CommandUnit , Format='(A)' , OutString
      ENDIF

   ; /* read data files */

   Read_AAP_Data,    FileName_or_FileList(i), AAP_STRUCTURE

   ; /* copy data from aap_structure */

   print, ' <MapAAP2Drizzle> ... grabbing the data  '

   Copy_AAP_Data , AAP_STRUCTURE, AAPName, Detector, Aperture,      $
                   RACenter, DecCenter,                             $
                   RollAngle, SurfBright, Flux, Flag,               $
                   RelChopPos, ChopStepID, RastPntID, FlatField,    $
                   MEANVALUES=MEANVALUES, MEDIANVALUES=MEDIANVALUES


   ; /* prepare the rotation angle of the final map */

   MapRotation =  0.0 * RollAngle(*)

   IF (Keyword_Set (AVERROLLCOORDS))   $
      THEN BEGIN
         MapRotation (*) = AVG (RollAngle (*))
      ENDIF

   IF (Keyword_Set (SPACECRAFTCOORDS))   $
      THEN BEGIN
         PIA_SM_RADec_PositAngles,  RACenter, DecCenter, CenterPositionAngles
         AVGPositionAngles =  PIA_SM_MEDIAN (CenterPositionAngles(*))

         MapRotation (*) = AVGPositionAngles
      ENDIF

   IF (NOT Keyword_Set (SPACECRAFTCOORDS))  AND    $
      (NOT Keyword_Set (AVERROLLCOORDS))           $
      THEN BEGIN
         If (PositAngle  ne  0.0)  $
            then begin
               MapRotation (*) = MapRotation (*) + PositAngle
            endif
      ENDIF


    ; /* get the pixel RA / Dec coordinates */

    MakePixelCoord,    Detector, Aperture, RACenter, DecCenter, RollAngle,  $
                       RelChopPos, DetPixelSize, DetPixelDist,              $
                       RAPixel, DecPixel

   ; /* ... use fluxes or surf. brightnesses */

   If (Keyword_Set (SURFACEBRIGHTNESS))  $
      then BEGIN
         Data = SurfBright (*,*)
         IF (I EQ 0)  THEN print, ' <MapAAP2Drizzle> ... using SURFACE BRIGHTNESSes'
      ENDIF ELSE  BEGIN
         Data = Flux (*,*)
         IF (I EQ 0)  THEN print, ' <MapAAP2Drizzle> ... using FLUXes '
      ENDELSE

   ; /* determine parameters of output image */

   IF (i EQ 0)   $
      THEN begin
         print, ' <MapAAP2Drizzle> ... deriving image sizes  '
         PrepareImage,  RACenter, DecCenter, RAPixel, DecPixel,           $
                        RollAngle, Rotation, DetPixelSize, DetPixelDist,  $
                        RAMax, RAMin, DecMax, DecMin, ImagePixelSize,     $
                        RelativeScale, NX , NY , NX2 , NY2 , NNXX , NNYY, $
                        EXTEND=IncreaseSize,                              $
                        NPixX=InNPixX, NPixY=InNPixY
      ENDIF

   ; /* derive shifts in units of fractional input/output pixels */

   print, ' <MapAAP2Drizzle> ... deriving image shifts  '

   PrepareShifts,   Detector, RAPixel, DecPixel, RollAngle,     $
                    MapRotation, RefRA, RefDec, RefRot,         $
                    DetPixelSize, DetPixelDist, ImagePixelSize, $
                    XShift, YShift, XYRotation,                 $
                    SIMPLEPROJECTION=SIMPLEPROJECTION,          $
                    GNOMONICPROJECTION=GNOMONICPROJECTION,      $
                    IGNOREROLLANGLE=IGNOREROLLANGLE,            $
                    OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS


   ; /* write the header of the drizzle script(s) : one/several AAP file(s) */

   IF ( (NoDeltaStripes le 0)           AND   (NOT Keyword_Set (PARALLEL))   AND    $
        (NOT Keyword_Set (EVERYPIXEL))  AND                                         $
        ((i EQ 0)  OR  (Keyword_Set (SINGLEAAP))) )                                 $
      THEN BEGIN
         print, ' <MapAAP2Drizzle> ... writing script header '

         WriteHeaderDrizzleScript,   Version, OutUnit, Detector, DetPixelSize,     $
                                  DetPixelDist, RACenter, DecCenter,      $
                                  RAPixel, DecPixel, RefData, RefRot,     $
                                  ScriptName, ImageName, Weightname,      $
                                  ImagePixelSize, RelativeScale,          $
                                  DrizzleFraction, FillValue,             $
                                  NX , NY , NX2 , NY2 , NNXX , NNYY ,     $
                                  XShift, YShift, CalledAs,               $
                                  OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,    $
                                  IGNOREROLLANGLE=IGNOREROLLANGLE

         WriteProjectFITSHeaderKeys,  OutUnit, ImageName, Weightname, RefRA, RefDec,   $
                                      ImagePixelSize, NNXX, NNYY, Identifier
      ENDIF

   ; /* write the header of the drizzle script(s) : parallel mode */

   IF ( ((NoDeltaStripes gt 0) OR (Keyword_Set (PARALLEL)))   AND   (i EQ 0) )  $
      THEN BEGIN
          print, ' <MapAAP2Drizzle> ... writing parallel image sum-up '

          WriteSumUpParallelImages,    OutUnitList, ScriptNameList,     $
                                       ImageNameList, WeightNameList,   $
                                       NoDeltaStripes

         print, ' <MapAAP2Drizzle> ... writing parallel script header '

         For k = 0L , NoDeltaStripes-1L , 1  do  begin
            ScriptName =  ScriptNameList (k)
            ImageName  =  ImageNameList (k)
            Weightname =  WeightNameList (k)
            OutUnit    =  OutUnitList (k)

            WriteHeaderDrizzleScript,   Version, OutUnit, Detector, DetPixelSize,     $
                                  DetPixelDist, RACenter, DecCenter,      $
                                  RAPixel, DecPixel, RefData, RefRot,     $
                                  ScriptName, ImageName, Weightname,      $
                                  ImagePixelSize, RelativeScale,          $
                                  DrizzleFraction, FillValue,             $
                                  NX , NY , NX2 , NY2 , NNXX , NNYY ,     $
                                  XShift, YShift, CalledAs,               $
                                  OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,          $
                                  IGNOREROLLANGLE=IGNOREROLLANGLE

            WriteProjectFITSHeaderKeys,  OutUnit, ImageName, Weightname, RefRA, RefDec,   $
                                         ImagePixelSize, NNXX, NNYY, Identifier
         EndFor

      ENDIF

   ; /* write the header of the drizzle script(s) : every pixel mode */

  IF ( (Keyword_set (EVERYPIXEL))   AND   (i EQ 0) )  $
      THEN BEGIN

         NoDetectorPixel = N_Elements (ImageNameList)
         print, ' <MapAAP2Drizzle> ... writing script header '

         For k = 0L , NoDetectorPixel-1L , 1  do  begin
            ScriptName =  ScriptNameList (k)
            ImageName  =  ImageNameList (k)
            Weightname =  WeightNameList (k)
            OutUnit    =  OutUnitList (k)

            WriteHeaderDrizzleScript,   Version, OutUnit, Detector, DetPixelSize,     $
                                  DetPixelDist, RACenter, DecCenter,      $
                                  RAPixel, DecPixel, RefData, RefRot,     $
                                  ScriptName, ImageName, Weightname,      $
                                  ImagePixelSize, RelativeScale,          $
                                  DrizzleFraction, FillValue,             $
                                  NX , NY , NX2 , NY2 , NNXX , NNYY ,     $
                                  XShift, YShift, CalledAS,               $
                                  OUTPUTFRAMESHIFTS=OUTPUTFRAMESHIFTS,          $
                                  IGNOREROLLANGLE=IGNOREROLLANGLE

            WriteProjectFITSHeaderKeys,  OutUnit, ImageName, Weightname, RefRA, RefDec,   $
                                         ImagePixelSize, NNXX, NNYY, Identifier
         EndFor

     Endif


   ; /* now write drizzle commands to scripts : one/several AAP file(s) */

   IF ((NoDeltaStripes le 0)   AND  (NOT Keyword_Set (PARALLEL))  AND  $
       (NOT Keyword_Set (EVERYPIXEL)))  $
       THEN BEGIN
          print, ' <MapAAP2Drizzle> ... writing AAP separator '

          WriteDrizzleScriptSeparator,  OutUnit, AAPName

          print, ' <MapAAP2Drizzle> ... writing script '

          WriteMainDrizzleScript,   OutUnit, Detector, DetPixelSize,   $
                             DetPixelDist, RACenter, DecCenter,        $
                             RAPixel, DecPixel,                        $
                             Data, Flag, IgnorePixelVec,               $
                             ScriptName, ImageName, Weightname,        $
                             ImagePixelSize, RelativeScale,            $
                             DrizzleFraction, NX2, NY2,                $
                             XShift, YShift, XYRotation
       ENDIF

   ; /* now write drizzle commands to scripts : parallel mode */

   IF (NoDeltaStripes gt 0)  OR  (Keyword_Set (PARALLEL))  $
       THEN BEGIN
          print, ' <MapAAP2Drizzle> ... writing AAP separators '
          For k = 0L , NoDeltaStripes , 1  do  begin
             OutUnit    =  OutUnitList (k)
             WriteDrizzleScriptSeparator,  OutUnit, AAPName
          EndFor

          print, ' <MapAAP2Drizzle> ... writing parallel scripts '

          WriteParallelDrizzleScript,   OutUnitList,                   $
                             Detector, DetPixelSize,                   $
                             DetPixelDist, RACenter, DecCenter,        $
                             RAPixel, DecPixel,                        $
                             Data, Flag, IgnorePixelVec,               $
                             ScriptNameList, ImageNameList,            $
                             WeightnameList,                           $
                             ImagePixelSize, RelativeScale,            $
                             DrizzleFraction, NX2, NY2,                $
                             XShift, YShift, XYRotation,               $
                             DecStartVec, DecEndVec, NoDeltaStripes
       ENDIF

   ; /* now write drizzle commands to scripts : every pixel mode */

  IF (Keyword_Set (EVERYPIXEL))  $
       THEN BEGIN
          print, ' <MapAAP2Drizzle> ... writing AAP separators '
          NoDetectorPixel = N_Elements (ImageNameList)
          For k = 0L , (NoDetectorPixel-1L) , 1  do  begin
             OutUnit = OutUnitList (k)
             WriteDrizzleScriptSeparator,  OutUnit, AAPName
          EndFor

          print, ' <MapAAP2Drizzle> ... writing scripts '

          WriteEveryPixelDrizzleScript,   OutUnitList,                 $
                             Detector, DetPixelSize,                   $
                             DetPixelDist, RACenter, DecCenter,        $
                             RAPixel, DecPixel,                        $
                             Data, Flag, IgnorePixelVec,               $
                             ScriptNameList, ImageNameList,            $
                             WeightnameList,                           $
                             ImagePixelSize, RelativeScale,            $
                             DrizzleFraction, NX2, NY2,                $
                             XShift, YShift, XYRotation
       ENDIF

    ; /* close drizzle script file(s) : single AAP mode */

    IF (Keyword_Set (SingleAAP))  $
      THEN BEGIN
         WriteTrailerDrizzleScript,  OutUnit, ImageName, Weightname

         print, ' <MapAAP2Drizzle> ... closing single AAP output file '
         IF (Keyword_Set (FITS))  THEN   WriteFITSOutput,  OutUnit, ImageName, Weightname
         Close , OutUnit
         Free_LUN , OutUnit
      ENDIF

   IF (i  NE  (NDataSet-1))  THEN  print, ' <MapAAP2Drizzle> ... NEXT AAP - DATASET '

ENDFOR    ; ---------------------- End of Loop over all Datasets



; /* close drizzle script file(s) : one AAP file only */

IF ((NOT Keyword_Set (SingleAAP))  AND  (NOT Keyword_Set (EVERYPIXEL)) AND  $
    (NoDeltaStripes le 0)          AND  (NOT Keyword_Set (PARALLEL)))       $
   THEN BEGIN
      WriteTrailerDrizzleScript,  OutUnit, ImageName, Weightname

      print, ' <MapAAP2Drizzle> ... closing output file '

      IF (Keyword_Set (FITS))  THEN   WriteFITSOutput,  OutUnit, ImageName, Weightname
      Close , OutUnit
      Free_LUN , OutUnit
      Close, CommandUnit
      Free_LUN , CommandUnit
   ENDIF

; /* close drizzle script file(s) : parallel mode */

If (NoDeltaStripes gt 0)   OR   (Keyword_Set (PARALLEL))  $
   then begin
      print, ' <MapAAP2Drizzle> ... closing parallel output files '
      For k = 0L , ((2*NoDeltaStripes)-2L) , 1  do  begin
         OutUnit = OutUnitList (k)

         WriteTrailerDrizzleScript,  OutUnit, ImageNameList(k), WeightnameList(k)

         IF (Keyword_Set (FITS))  AND  (k EQ ((2*NoDeltaStripes)-2L))        $
            THEN   WriteFITSOutput,  OutUnit, ImageNameList(k), WeightnameList(k)

         Close , OutUnit
         Free_LUN , OutUnit
      EndFor
      Close, CommandUnit
      Free_LUN , CommandUnit
   endif

; /* close drizzle script file(s) : every pixel mode */

If (Keyword_Set (EVERYPIXEL))  $
   then begin
      NoDetectorPixel = N_Elements (ImageNameList)
      print, ' <MapAAP2Drizzle> ... closing output file for each pixel '
      For k = 0L , (NoDetectorPixel-1L) , 1  do  begin
         OutUnit = OutUnitList (k)

         WriteTrailerDrizzleScript,  OutUnit, ImageNameList(k), WeightnameList(k)

         IF (Keyword_Set (FITS))  THEN   WriteFITSOutput,  OutUnit, ImageNameList(k), WeightnameList(k)

         Close , OutUnit
         Free_LUN , OutUnit
      EndFor
      Close, CommandUnit
      Free_LUN , CommandUnit
   endif



; /* try to start IRAF directly */

IF (Keyword_Set (EXECUTE))   $
   then BEGIN
     Command = [' cl  < ' + !IRAF_DATA + IRAFCommandName]
     print, ' <MapAAP2Drizzle> ... starting IRAF immediately ! '
     print, ' <MapAAP2Drizzle> ...  '
     print, ' <MapAAP2Drizzle> ...  '
     print, ' <MapAAP2Drizzle> ...  '
     SPAWN,   Command
     print, ' <MapAAP2Drizzle> ...  '
     print, ' <MapAAP2Drizzle> ...  '
     print, ' <MapAAP2Drizzle> ...  '
     print, ' <MapAAP2Drizzle> ... back from IRAF  ! '
     print, ' <MapAAP2Drizzle> ... batch job running ! '
   endif

print, ' '

Return
End





;-------------------------------------------------------------------------------------------



