;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GEN_MAP
;
; PURPOSE:
;       Create a 2-dimensional map of the simulated sky
;
; CATEGORY:
;       PIA - graphics
;
; CALLING SEQUENCE:
;       GEN_MAP, skyc, par=par, mappar=mappar, maphdr=maphdr, skypure=skypure, $
;	graph=graph 
;
; INPUTS:
;       PAR             The structure containing the parameters
;
; KEYWORD PARAMETERS:
;	GRAPH		=1 for an interactive session or
;			=0 for a non-interactive processing of the user-defined 
;			   parameters from using MAPSIM_DEFPARM and PIA_MAPSIM_MPAR
;
; OUTPUTS:
;       PAR		The structure containing the parameters
;
;	MAPPAR		The structure created by using the CREATE_MAPPAR and 
;			then defined with internal variables
;			
;	MAPHDR		The header with some of its KEYWORDs defined appropriately
;
;	SKYC		The simulated sky convolved with the footprint matrix	
;
;	SKYPURE		The simulated sky (a 2-dimensional image)
;
; PROCEDURES USED:
;       PIA Procedure(s):
;		mapsim_defparm (function)
;		colorbar
;		get_omega (function)
;               create_mappar
;		pia_mktrans (function)
;		give_filter_id (function)
;       Astronomical Library:
;		fits_read
;		fxaddpar
;
; MODIFICATION HISTORY:
;       Written by:      Min Hur, IPAC/Caltech and
;                        Carlos Gabriel, ESA/VILSPA     April 1999
;
;	sky map always with odd dimensions 		
;	and central pixel correctly taken 		
;	Skyc is produced in Jy/beam (CG)		December 1999 	
;	
;	Change plot titles of the interactive version
;	to reflect the change to Jy/beam (MH)		Jan 2000					
;-

PRO gen_map, skyc, par=par, mappar=mappar, maphdr=maphdr, skypure=skypure, $
	graph=graph

	IF not KEYWORD_SET(graph) THEN graph=0
	loadct, 13, ncolors=!d.n_colors-12, bottom=1
	
	IF graph THEN BEGIN 
	  
; Allow only square sky for convenience
; Ask for the grid step size in arcseconds (only to the tenth place)
		ss=0.0
		print,' Please enter the grid step size in arcseconds (e.g. 7.5)'
		read,ss

; Ask for the size of the sky in arcseconds
		sd=0.0
		print,' Please enter the size of the square sky in arcseconds (e.g. 1000)'
		print,' Note, it should be > 255 for C100 or > 510 for C200.'
		read,sd
		
		nr_p = ROUND(sd / ss)
		nr_p = nr_p + (nr_p mod 2) - 1
		sd = ss * nr_p

		par = mapsim_defparm()
		par.resol = ss
		par.impix = sd
		
	ENDIF ELSE BEGIN
		ss = par.resol
		sd = par.impix
		nr_p = ROUND(sd / ss)
		nr_p = nr_p + (nr_p mod 2)
		sd = round(ss * nr_p)
		par.impix = sd
	ENDELSE
	
	sky_h=findgen(sd/ss+1)*ss-fix(sd/ss+1)/2*ss
	sky_v=findgen(sd/ss+1)*ss-fix(sd/ss+1)/2*ss

	sky_h_label='South (arcsec)'
	sky_v_label='East (arcsec)'

	
	IF graph THEN BEGIN
		 
		window, /free, xsize=600, ysize=600, $
 			title='Simulated sky'
		!P.MULTI=[0,1,1]
;		gamma_ct,0.6
		ncolors = !D.N_COLORS-12
		position=[0.15,0.10,0.90,0.85]
		xsize=(position(2)-position(0))*!d.x_vsize
		ysize=(position(3)-position(1))*!d.y_vsize
		xstart=position(0)*!d.x_vsize+1
		ystart=position(1)*!d.y_vsize+1

		barposition=[0.15,0.90,0.90,0.95]
		imageposition=[0.15,0.10,0.903,0.853]
		
		cs=1.5
		erase,ncolors/2
		plot,sky_h,sky_v,xstyle=1,ystyle=1,charsize=cs,$
		 xrange=[max(sky_h),min(sky_h)],$
 		 xtitle=sky_h_label,ytitle=sky_v_label,position=imageposition,$
 		 title='Pure sky image - Irrelevant scale',/nodata,/noerase
		tvscl,congrid(fltarr(sd/ss+1,sd/ss+1),xsize,ysize),$
		   xstart,ystart, top=ncolors
		xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
 		 alignment=0.5,orientation=-90.0,charsize=cs

		ecount=0
		print,' How many sources?'
		read,ecount

 		par.nr_src = ecount
		IF ecount NE 1 THEN $
		  par = mapsim_defparm(oldpar=par)
		

	ENDIF ELSE ecount = par.nr_src

		
	if ecount gt 0 then begin
		 sky_set=fltarr(ecount,sd/ss+1,sd/ss+1)
		 skyc_set=fltarr(ecount,sd/ss+1,sd/ss+1)
		 edinfo=strarr(ecount)
		 source_fluxes=fltarr(ecount)
	endif else begin
		 sky_set=fltarr(sd/ss+1,sd/ss+1)
		 skyc_set=sky_set
	endelse
	red=0.0
	bright=0.0

	n=0
	WHILE n lt ecount do begin

	   IF graph THEN BEGIN 
		print,' Click at the desired location of the source.  For an extended'
		print,' source, click at the desired center location.'
		cursor,xed,yed,/data
		print,' '
		print,' Enter the radius of this source in arcseconds or the'
		print,' dimension for the horizontal extension of a rectangular'
		print,' structure. For a circular point source, make the radius'
		print,' small (typically a few arcseconds).'
		read,red 
		print,' If you want a rectangular structure enter now the '
		print,' dimension in vertical direction. If 0, the former '
		print,' value entered will be the radius of a circular source '
		read,zred 
		print,' Please type B if you want to enter the source brightness'
		print,' in MJy/sr or type F for the source flux in Jy'
		print,' (1.0 Jy/arcsec^2 = 4.2545x10^4 MJy/sr)'
		borf='B'
		read,borf
		if STRUPCASE(borf) eq 'B' then begin
	   		print,' What is the source brightness in MJy/sr?'
			read,bright		
			par.f_or_br(n) = 1
		endif
		if STRUPCASE(borf) eq 'F' then begin
	   		print,' What is the source flux in Jy?'
			read,sflux

;			if zred eq 0 then area=!pi*red^2 else area=red*zred
;			bright=sflux/area*4.2545e4

			bright = sflux
			
			print,' source flux: ',sflux,' Jy'
;			print,' source area: ',area,' arcsecond^2'
;			print,' source brightness: ',bright,' MJy/sr'				
			par.f_or_br(n) = 0
		endif
		if STRUPCASE(borf) ne 'B' and STRUPCASE(borf) ne 'F' then begin
			print,' Wrong entry.  Please try again.'
			RETURN
		endif	   	

	   	par.pos_src(0,n) = xed
	   	par.pos_src(1,n) = yed
		par.rad_src(n) = red
		par.rad_zsrc(n) = zred
		par.br_src(n) = bright
		
	   ENDIF ELSE BEGIN
	   	xed = par.pos_src(0,n)
	   	yed = par.pos_src(1,n)
		red = par.rad_src(n)
		zred = par.rad_zsrc(n)
		IF par.f_or_br(n) EQ 0 THEN BEGIN

;			if zred eq 0. then area=!pi*red^2 else area=red*zred
;			bright=sflux/area*4.2545e4

			bright = par.br_src(n)
		ENDIF ELSE BEGIN		  
			bright = par.br_src(n)
		ENDELSE 

	   ENDELSE
	   
	   xind=where(abs(sky_h-xed) eq min(abs(sky_h-xed)))
	   xind=min(xind)
	   yind=where(abs(sky_v-yed) eq min(abs(sky_v-yed)))
	   yind=min(yind)


	   bflux=float(bright)
	   source_fluxes(n)=bflux	
	   
	   
;	   print,' Please wait'

	   IF zred EQ 0 THEN BEGIN
	    for i=0,sd/ss do begin
		   for j=0,sd/ss do begin

		     if (sky_h(i)-sky_h(xind))^2+(sky_v(j)-sky_v(yind))^2 le red^2 $
		     and abs(sky_h(i)-sky_h(xind)) le red $
		     and abs(sky_v(j)-sky_v(yind)) le red then begin
		      sky_set(n,i,j)=sky_set(n,i,j)+bflux
		     endif

		   endfor
	    endfor
	   ENDIF ELSE BEGIN	
	    for i=0,sd/ss do begin
		   for j=0,sd/ss do begin

		     if ABS(sky_h(i)-sky_h(xind)) le red $
		     and abs(sky_v(j)-sky_v(yind)) le zred then begin
		      sky_set(n,i,j)=sky_set(n,i,j)+bflux
		     endif

		   endfor
	    endfor 
	   ENDELSE
	   
	   IF graph THEN BEGIN
	         edinfo(n)=strmid(sky_h(xind),0,30)+'  '+strmid(sky_v(yind),0,30)+$
		           '  '+strmid(red,0,30)+'  '+strmid(bflux,0,30)

		 erase,ncolors/2
		 colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
		  min=0.0,max=max(total(sky_set,1)), ncolors=!D.n_colors-12
		 plot,sky_h,sky_v,ticklen=-0.015,xstyle=1,ystyle=1,$
		  xrange=[max(sky_h),min(sky_h)],$
		  charsize=cs,xtitle=sky_h_label,ytitle=sky_v_label,$
		  position=imageposition,/nodata,/noerase,$
		  title='Pure sky image - Irrelevant scale'
		 tvscl,congrid(rotate(total(sky_set,1),5),xsize,ysize,/interp),$
		 	xstart,ystart, top=ncolors
		 xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
		  alignment=0.5,orientation=-90.0,charsize=cs
	   ENDIF
	   n=n+1
	ENDWHILE




	IF graph THEN BEGIN
; Flat Background
;
		ans='a'
		print, 'Would you like a flat background?  Type Y for yes or N for no'
		read,ans
		flat='yes'
		gb_flux = 0.
		if ecount gt 0 then sky_temp=total(sky_set,1)	else $
		   sky_temp=sky_set
		IF ans eq 'Y' or ans eq 'y' or ans eq 'yes' or ans eq 'Yes' then begin 
		  bkg=0.0
		  print,' Please enter the background flux in MJy/sr'
		  read,bkg
  		  bkgflux=float(bkg)
		  if ecount gt 0 then sky_temp=total(sky_set,1)+bkgflux	else $
		   sky_temp=sky_set+bkgflux

		  bkginfo=strarr(1)
		  bkginfo=' flat background : '+strmid(bkgflux,0,30)

		  while !d.window ne -1 do wdelete

;		   window, /free, $
;		    title='Simulated sky'
;		   !P.MULTI=[0,1,1]
;
;		   shade_surf,sky_temp,sky_h,sky_v,charsize=cs,$
;		    xstyle=1,ystyle=1,xrange=[max(sky_h),min(sky_h)],$
;		    xtitle=sky_h_label,ytitle=sky_v_label,zstyle=2,$
;		    ztitle='Arb. unit',title='Pure sky image - Irrelevant scale'   

		  window, /free, xsize=600, ysize=600, $
		   title='Simulated sky'
		  !P.MULTI=[0,1,1]

		 colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
		  min=0.0,max=max(sky_temp),ncolors=!D.n_colors-12	
		 plot,sky_h,sky_v,ticklen=-0.015,xstyle=1,ystyle=1,$
		  xrange=[max(sky_h),min(sky_h)],$
		  charsize=cs,xtitle=sky_h_label,ytitle=sky_v_label,$
		  position=imageposition,/nodata,/noerase,$
		  title='Pure sky image - Irrelevant scale'
		 tvscl,congrid(rotate(sky_temp,5),xsize,ysize,/interp),$
		 	xstart,ystart, top=ncolors	
		 xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
		  alignment=0.5,orientation=-90.0,charsize=cs

		ENDIF ELSE BEGIN
		   flat='no'
		   bkg = 0.
		ENDELSE
; Gradient Background
;
		ans='a'
		print,' Would you like a gradient background?  Type Y for yes or N for no'
		read,ans

		gback='yes'
		IF ans eq 'Y' or ans eq 'y' or ans eq 'yes' or ans eq 'Yes' then begin
		  print,' You will enter the background flux at two locations, and the'
		  print,' gradient background will be interpolated with these inputs.'
		  gbflux=fltarr(sd/ss+1,sd/ss+1)
		
		  gflux1=0.0
		  print,' Click at the first location for gradient background information.'
		  cursor,xgb1,ygb1,/data
		  xyouts,xgb1,ygb1,/data,'1'
		  x_1=where(abs(sky_h-xgb1) eq min(abs(sky_h-xgb1)))
		  y_1=where(abs(sky_v-ygb1) eq min(abs(sky_v-ygb1)))
		  print,' Please enter the background flux in MJy/sr'
		  read,gflux1
		  gbflux(x_1,y_1)=gflux1

		  gflux2=0.0
		  print,' Click at the second location'
		  cursor,xgb2,ygb2,/data
		  xyouts,xgb2,ygb2,/data,'2'
		  x_2=where(abs(sky_h-xgb2) eq min(abs(sky_h-xgb2)))
		  y_2=where(abs(sky_v-ygb2) eq min(abs(sky_v-ygb2)))
		  print,' Please enter the background flux in MJy/sr'
		  read,gflux2
		  gbflux(x_2,y_2)=gflux2

		  gb_flux=gbflux
		  for i=0,sd/ss do begin
		   for j=0,sd/ss do begin
		    gb_flux(i,j)=((gflux2-gflux1)/(xgb2-xgb1)*(sky_h(i)-xgb1)+2*gflux1+$
		      (gflux2-gflux1)/(ygb2-ygb1)*(sky_v(j)-ygb1))/2.0
		   endfor
		  endfor

;		  print,gb_flux(x_1,y_1),gb_flux(x_2,y_2)

		  neg_flux=where(gb_flux lt 0.0,count)
		  if count gt 0 then gb_flux(neg_flux)=0.0

		  if ecount gt 0 then begin
		  if flat ne 'no' then sky_temp=total(sky_set,1)+bkgflux+gb_flux else $
		    sky_temp=total(sky_set,1)+gb_flux	
		  endif else begin
		  if flat ne 'no' then sky_temp=sky_set+bkgflux+gb_flux else $
		    sky_temp=sky_set+gb_flux	
		  endelse

		  while !d.window ne -1 do wdelete
		window, /free, xsize=600, ysize=600, $
		 title='Simulated sky'
		!P.MULTI=[0,1,1]

		 colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
		  min=0.0,max=max(sky_temp), ncolors=!D.n_colors-12	
		 plot,sky_h,sky_v,ticklen=-0.015,xstyle=1,ystyle=1,$
		  xrange=[max(sky_h),min(sky_h)],$
		  charsize=cs,xtitle=sky_h_label,ytitle=sky_v_label,$
		  position=imageposition,/nodata,/noerase,$
		  title='Pure sky image - Irrelevant scale'
		 tvscl,congrid(rotate(sky_temp,5),xsize,ysize,/interp),$
		 	xstart,ystart, top=ncolors	
		 xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
		  alignment=0.5,orientation=-90.0,charsize=cs

;		   window, /free, $
;		    title='Pure sky image - 3D'
;		   !P.MULTI=[0,1,1]
;
;		   shade_surf,sky_temp,sky_h,sky_v,charsize=cs,$
;		    xstyle=1,ystyle=1,xrange=[max(sky_h),min(sky_h)],$
;		    xtitle=sky_h_label,ytitle=sky_v_label,zstyle=2,$
;		    ztitle='Arb. unit',title='Pure sky image in 3D'   

		ENDIF ELSE gback='no'
		
		if n_elements(edinfo) ne 0 then begin
		   print,' Extended source(s) information :'
		   for n=0,n_elements(edinfo)-1 do print,edinfo(n)
		   print,' '
		endif
		if n_elements(bkginfo) ne 0 then print,bkginfo

	; make filter/detector choice 
	; Let the user choose from the different filter choices.
fd=0
print, 'Please enter the number corresponding to the desired Filter ID.'
print, ' '
print, 'Enter one of these numbers      Filter ID       central wavelength'
print, '----------------------------------------------------------------'
print, '                      1         C100_90_RED     90 um (backup)'
print, '                      2         C100_50_UM      65 um'
print, '                      3         C100_60_UM      60 um'
print, '                      4         C100_70_UM      80 um'
print, '                      5         C100_100_UM     100 um'
print, '                      6         C100_105_UM     105 um'
print, '                      7         C100_90_UM      90 um'
print, '                      8         C200_160_UM     170 um'
print, '                      9         C200_200_UM     200 um'
print, '                      10        C200_180_UM     180 um'
print, '                      11        C200_135_UM     150 um'
print, '                      12        C200_120_UM     120 um'
read,fd	
		par.br_bck = bkg
		par.filt = fd
		IF gback EQ 'yes' THEN BEGIN
		  par.br_gbck(0) = gflux1
		  par.bck_pos1(0) = xgb1
		  par.bck_pos1(1) = ygb1
		  par.br_gbck(1) = gflux2
		  par.bck_pos2(0) = xgb2
		  par.bck_pos2(1) = ygb2
		ENDIF ELSE BEGIN
		  par.br_gbck(0) = 0.
		  par.bck_pos1(0) = 0.
		  par.bck_pos1(1) = 0.
		  par.br_gbck(1) = 0.
		  par.bck_pos2(0) = 0.
		  par.bck_pos2(1) = 0.
		ENDELSE
		  
	ENDIF ELSE BEGIN
		bkg = par.br_bck
  		bkgflux=float(bkg)
		if ecount gt 0 then sky_temp=total(sky_set,1)+bkgflux	else $
		   sky_temp=sky_set+bkgflux
		IF bkg EQ 0. THEN flat='no' ELSE flat='yes'
		fd = par.filt

	  IF par.br_gbck(0) NE 0. or par.br_gbck(1) NE 0. THEN BEGIN
		gback='yes'
		gbflux=fltarr(sd/ss+1,sd/ss+1)
		gflux1=0.0
		xgb1 = par.bck_pos1(0)
		ygb1 = par.bck_pos1(1)
		x_1=where(abs(sky_h-xgb1) eq min(abs(sky_h-xgb1)))
		y_1=where(abs(sky_v-ygb1) eq min(abs(sky_v-ygb1)))
		gflux1 = par.br_gbck(0)
		gbflux(x_1,y_1)=gflux1
	
		gflux2=0.0
		xgb2 = par.bck_pos2(0) 
		ygb2 = par.bck_pos2(1) 
		x_2=where(abs(sky_h-xgb2) eq min(abs(sky_h-xgb2)))
		y_2=where(abs(sky_v-ygb2) eq min(abs(sky_v-ygb2)))
		gflux2 = par.br_gbck(1)
		gbflux(x_2,y_2)=gflux2
	

		gb_flux=gbflux
		for i=0,sd/ss do begin
		   for j=0,sd/ss do begin
			gb_flux(i,j)=((gflux2-gflux1)/(xgb2-xgb1)*(sky_h(i)-xgb1)+2*gflux1+$
			   (gflux2-gflux1)/(ygb2-ygb1)*(sky_v(j)-ygb1))/2.0
		   endfor
		endfor
		neg_flux=where(gb_flux lt 0.0,count)
        	if count gt 0 then gb_flux(neg_flux)=0.0
		
		  if ecount gt 0 then begin
		  if flat ne 'no' then sky_temp=total(sky_set,1)+bkgflux+gb_flux else $
		    sky_temp=total(sky_set,1)+gb_flux	
		  endif else begin
		  if flat ne 'no' then sky_temp=sky_set+bkgflux+gb_flux else $
		    sky_temp=sky_set+gb_flux	
		  skypure = rotate(sky_temp,5)
		  endelse
	     ENDIF ELSE BEGIN
		gb_flux = 0. 
		gback='no'
	     ENDELSE
	ENDELSE	
	skypure = rotate(sky_temp,5)
	filt=fd+1			; For the phtiaap.filt

	if fd eq 1 then cwvl=90.0	; For the phtiaap.cwvl
	if fd eq 2 then cwvl=65.0
	if fd eq 3 then cwvl=60.0
	if fd eq 4 then cwvl=80.0
	if fd eq 5 then cwvl=100.0
	if fd eq 6 then cwvl=105.0
	if fd eq 7 then cwvl=90.0
	if fd eq 8 then cwvl=170.0
	if fd eq 9 then cwvl=200.0
	if fd eq 10 then cwvl=180.0
	if fd eq 11 then cwvl=150.0
	if fd eq 12 then cwvl=120.0
	aper=10				; For the phtiaap.aper
	polz=14				; For the phtiaap.polz

; reading footprint fits files
; () update the directory information for most current calg files

	directory=!CALG
	if (fd ge 1 and fd le 7) then begin
	  pixnum = 9
	  filename='PC1FOOTP.FITS'
	  fits_read,directory+filename,tab,hdr
	  footpr=reform(tab(fd-1,*,*))
	  y1=findgen(35)*7.5-255.0/2.0		; axis label for footpr - C100
	  z1=y1
	  y3=findgen(255.0/ss+1)*ss-fix(255.0/ss)*ss/2	; axis label for ftpt
	  z3=y3						
	  dim=255*10+1
	  y2=findgen(dim)/75.0
	  z2=y2
	  det='C1'	; For the phtiaap.det
	  pix_size=43.5
	  pix_area=pix_size^2
	  delt=46.0    ; each pixel has dimensions 43.5" x 43.5"
	  
; this is between the pixel centers
	  half_dia=46.0+(43.5/2.0)	; 67.7500 = half the detector
	endif
	if (fd ge 8 and fd le 12) then begin
	  pixnum = 4
	  filename='PC2FOOTP.FITS'
	  fits_read,directory+filename,tab,hdr
	  footpr=reform(tab(fd-8,*,*))
	  y1=findgen(35)*15.0-510.0/2.0
	  z1=y1
	  y3=findgen(510.0/ss+1)*ss-fix(510.0/ss)*ss/2
	  z3=y3
	  dim=510*10+1		
	  y2=findgen(dim)/150.0
	  z2=y2
	  det='C2'	; For the phtiaap.det
	  pix_size=89.4
	  pix_area=pix_size^2
	  delt = 46.0       ; each pixel has dimensions 89.4" x 89.4"
	                    ; this equals half the pixel size + half the gap size
	  half_dia=46.0-(89.4/2.0)+89.4		; 90.7000 = half the detector
	endif
	
	now=systime(0)
	if strmid(now,4,3) eq 'Jan' then month='01'
	if strmid(now,4,3) eq 'Feb' then month='02'
	if strmid(now,4,3) eq 'Mar' then month='03'
	if strmid(now,4,3) eq 'Apr' then month='04
	if strmid(now,4,3) eq 'May' then month='05'
	if strmid(now,4,3) eq 'Jun' then month='06'
	if strmid(now,4,3) eq 'Jul' then month='07'
	if strmid(now,4,3) eq 'Aug' then month='08'
	if strmid(now,4,3) eq 'Sep' then month='09'
	if strmid(now,4,3) eq 'Oct' then month='10'
	if strmid(now,4,3) eq 'Nov' then month='11'
	if strmid(now,4,3) eq 'Dec' then month='12'
	day='00'
	if strmid(now,8,1) eq ' ' then day='0'+strmid(now,9,1) $
	 else day=strmid(now,8,2)
	admn=det+'99'+month+day+strmid(now,11,2)+strmid(now,14,2)
	admn_tmp=det+'01'+month+day+strmid(now,11,2)+strmid(now,14,2)
; For the phtiaap.admn
; admn contains det, 99, month, day, hour, and minute in order

IF ecount gt 0 then begin

	IF graph THEN $
	 print,' Please wait.'

	pickno=fix(ss*10)
	sclfootpr=fltarr(2,dim)
	sclfootpr(0,*)=y2
	sclfootpr(1,*)=z2
	xftpt=fltarr(n_elements(y3))
	yftpt=fltarr(n_elements(y3))
	for i=(dim-1)/2,dim-1,pickno do begin
	  for j=(dim-1)/2,dim-1,pickno do xftpt(i/pickno)=sclfootpr(0,i)
	  for j=(dim-1)/2,dim-1,pickno do yftpt(j/pickno)=sclfootpr(1,j)
	endfor
	for i=(dim-1)/2,dim-1,pickno do begin
	  for j=(dim-1)/2,0,-pickno do xftpt(i/pickno)=sclfootpr(0,i)
	  for j=(dim-1)/2,0,-pickno do yftpt(j/pickno)=sclfootpr(1,j)
	endfor
	for i=(dim-1)/2,0,-pickno do begin
	  for j=(dim-1)/2,dim-1,pickno do xftpt(i/pickno)=sclfootpr(0,i)
	  for j=(dim-1)/2,dim-1,pickno do yftpt(j/pickno)=sclfootpr(1,j)
	endfor
	for i=(dim-1)/2,0,-pickno do begin
	  for j=(dim-1)/2,0,-pickno do xftpt(i/pickno)=sclfootpr(0,i)
	  for j=(dim-1)/2,0,-pickno do yftpt(j/pickno)=sclfootpr(1,j)
	endfor
	ftpt=interpolate(footpr,xftpt,yftpt,/grid)
	

;print,'     footpr       ftpt'
;print,max(footpr),max(ftpt)
;print,min(footpr),min(ftpt)

;print,' '	
;print,max(ftpt)*source_fluxes
;if flat ne 'no' then print,max(ftpt)*source_fluxes+bkgflux


	IF graph THEN BEGIN

	  window,/free,xsize=800,ysize=600,title='Footprint matrix in 3D'
	  !p.multi=[0,2,2]
	  shade_surf,footpr,y1,z1,xstyle=1,ystyle=1,charsize=2.0,$
	   zstyle=1,zrange=[0,0.006]
	  shade_surf,ftpt,y3,z3,xstyle=1,ystyle=1,charsize=2.0,$
	   zstyle=1,zrange=[0,0.006]
	  shade_surf,footpr,y1,z1,xstyle=1,ystyle=1,charsize=2.0,title=filename
	  shade_surf,ftpt,y3,z3,xstyle=1,ystyle=1,charsize=2.0,title='scaled'
	  !p.multi=[0,0,0]

	  print,' Processing...  Please wait'

	ENDIF

; We need omega and epsilon for all conversions MJy/sr -> Jy/beam

	IF fd GE 8 THEN ind_om = fd - 8 ELSE ind_om = fd-1
	omega = get_omega(det, 0, ind_om)
	Epsilon = 0.3

	 for i=0,ecount-1 do begin
	  
	  IF par.f_or_br(i) THEN $
	     sky_set(i,*,*) = sky_set(i,*,*) * omega(0) * (1-epsilon^2) * 1.E06	  
	  skyc_set(i,*,*)=convol(reform(sky_set(i,*,*)),ftpt,/edge_truncate)
	  skyc_set(i,*,*)=skyc_set(i,*,*)/max(skyc_set(i,*,*))*max(ftpt)*max(sky_set(i,*,*))
	 endfor
	 
; Convert background in MJy/sr to Jy/beam
	IF flat NE 'no' THEN BEGIN
	 bkgflux = bkgflux * omega(0) * (1-epsilon^2) * 1.E06
	ENDIF
	
	if flat ne 'no' then skyc=total(skyc_set,1)+bkgflux else $
		skyc=total(skyc_set,1)
ENDIF ELSE BEGIN
; begin of only background

; Convert background in MJy/sr to Jy/beam
	IF flat NE 'no' THEN BEGIN
	  IF fd GE 8 THEN ind_om = fd - 8 ELSE ind_om = fd-1
	  omega = get_omega(det, 0, ind_om)
	  Epsilon = 0.3
	  bkgflux = bkgflux * omega(0) * (1-epsilon^2) * 1.E06
	  
	  skyc=skyc_set+bkgflux 
	ENDIF else skyc=skyc_set
ENDELSE

; add gradient background
if gback ne 'no' then BEGIN
	
	  IF fd GE 8 THEN ind_om = fd - 8 ELSE ind_om = fd-1
	  omega = get_omega(det, 0, ind_om)
	  Epsilon = 0.3
	  gb_flux = gb_flux * omega(0) * (1-epsilon^2) * 1.E06
	  
	  skyc=skyc+gb_flux
ENDIF	

	IF graph THEN BEGIN

	 window, /free, $
	  title='The sky convolved with footprint matrix - 3D'
	 !P.MULTI=[0,1,1]
	
	   shade_surf,skyc,sky_h,sky_v,charsize=cs,xstyle=1,ystyle=1,$
	    xrange=[max(sky_h),min(sky_h)],$
	    xtitle=sky_h_label,ytitle=sky_v_label,zstyle=2,$
	    ztitle='Jy/beam',title='Flux/beam Distribution in 3D'   
	
	  window, /free, xsize=600, ysize=600, $
	   title='The sky convolved with footprint matrix - 2D'
	  !P.MULTI=[0,1,1]
	 erase
	 colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
	  min=0.0,max=max(skyc), ncolors=!D.n_colors-12
	 plot,sky_h,sky_v,ticklen=-0.015,xstyle=1,ystyle=1,$
	  xrange=[max(sky_h),min(sky_h)],$
	  charsize=cs,xtitle=sky_h_label,ytitle=sky_v_label,$
	  position=imageposition,/nodata,/noerase,$
	  title='Convolved sky image (Jy/beam)'
	 tvscl,congrid(rotate(skyc,5),xsize,ysize,/interp),$
	  xstart,ystart, top=ncolors
	 xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
	  alignment=0.5,orientation=-90.0,charsize=cs
	
	
	ENDIF
	
	skyc=rotate(skyc,5)
	pixsize = [par.resol,par.resol]
	siz = size(skyc)
	crpix = [siz(1)/2.-0.5, siz(2)/2.-0.5]
	trans = PIA_MKTRANS([0.,0.], crpix, pixsize, 0.)
	
	DEFSYSV, '!CALSAVE', exists=i
	IF i NE 1 THEN DEFSYSV, !CALSAVE, ''
	restore, !CALSAVE + 'AAP_MAP_HDR.SAV'
	FXADDPAR, hdr, 'OBSERVER', 'PIA     ', ' Proposer ID '
	FXADDPAR, hdr, 'OBJECT  ', 'Some    ', ' Target ID as given by proposer '
	FXADDPAR, hdr, 'EOHAAOTN', 'P22     ', ' AOT name '
	FXADDPAR, hdr, 'EOHAPLID', 'LETSSIMU', ' Proposal ID '
	FXADDPAR, hdr, 'BKG_FLAT', flat
	FXADDPAR, hdr, 'GRD_BACK', gback
	FXADDPAR, hdr, 'CWVL_F  ', string(cwvl)+' um'
	FXADDPAR, hdr, 'PIA_VERS', !pia_version

	CREATE_mappar, mappar
	
	mappar.pixsize=pixsize
	mappar.admn = admn 
	mappar.trans=trans
	mappar.datatype = 4 ; map is in Jy/beam
	mappar.mapmethod = -1
	mappar.detector = det
	mappar.filter = GIVE_FILTER_ID(det,filt)
	mappar.admn_decoded = admn_tmp
	mappar.bkg_flat = flat
	mappar.grd_back = gback
	maphdr = hdr
		
	RETURN
	END
	


	

	
