;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GEN_RASTER
;
; PURPOSE:
;       Perform the raster on the simulated sky and creates an AAP structure
;	Enable to deal with three different raster types:
;	  1.  simple raster (P22 AOT)
;	  2.  raster with choppers (P32 AOT)
;	  3.  special case where the raster and detector arrays are mis-aligned
;
; CATEGORY:
;       PIA - graphics
;
; CALLING SEQUENCE:
;       GEN_RASTER, skyc, mappar, rpar, in_buf=in_buf, phtiaap=phtiaap, $
;	raster_coord=raster_coord, raster_pos=raster_pos, GRAPH=graph, $
;	sky_v=sky_v, sky_h=sky_h, zoomd=zoomd, dersrd=dersrd
;
; INPUTS:
;	SKYC            The simulated sky convolved with the footprint matrix 
;			produced by GEN_MAP
;
;	MAPPAR		The structure created by CREATE_MAPPAR and 
;                       then defined by GEN_MAP (contains mapping parameters)
;
;	RPAR            The structure containing the parameters from
;                       MAPSIM_DEFPARR and PIA_MAPSIM_RPAR
;
; KEYWORD PARAMETERS:
;	IN_BUF		Set to load the AAP structure into the PIA buffer automatically
;
;       GRAPH           =1 for an interactive session or
;                       =0 for a non-interactive processing of the user-defined 
;                          parameters 
;
;	DERSRD		Set if the user intends on creating a SRD structure using 
;			the AAP product
;
; OUTPUTS:
;       RPAR            The structure containing the raster parameters
;
;	PHTIAAP		The resulting AAP structure
;
;	RASTER_COORD	The raster coordinates on the simulated sky
;
;	RASTER_POS	The raster coordinates on the zoomed simulated sky in
;			spacecraft coordinate system
;
;	SKY_V		The vertical axis scale (DEC) for the simulated sky
;
;	SKY_H		The horizontal axis scale (RA) 
;
;	ZOOMD		The structure containing the zoomed simulated sky in 
;			spacecraft coordinate system
;
; COMMON BLOCKS:
;	@phtw2jy	
;
; PROCEDURES USED:
;       PIA Procedure(s):
;               colorbar
;               pia_disp_print_file
;               mapsim_defparr (function)
;               fill_aap
;		id_give_filter (function)
;               get_omega (function)
;               unflat
;		phtw2jy_init
;       Astronomical Library:
;               fxaddpar
;               fxread
;
; MODIFICATION HISTORY:
;       Written by:             Min Hur, IPAC/Caltech and
;                               Carlos Gabriel, ESA/VILSPA      April 1999
;
;	Changed positions of sky_h and sky_v for getting 
;	 central pixel positions (CG)				Dec. 1999
;	skyc input is in Jy/beam - Modified accordingly	(CG)	Dec. 1999 
;
;	Plot titles changed to note the Jy/beam scale (MH)	Jan. 2000
;
;	Unflatfielding delayed if the SRD level	will be 
;	produced (MH)						Jan. 2000
;
;	The angles, rota and roll are correctly defined and
;	written to phtiaap structure according to the following
;	definition. (MH)					Jan. 2000
;	-----------------------------------------------------------------
;	phtiaap.roll is the rotation angle of the S/C w.r.t RA-DEC from 
;		north to eastward (always and in all cases)
;	ATTRROTA (rota) is the angle of the raster pattern w.r.t RA-DEC 
;		defined the same way as phtiaap.roll if ATTRORIE = 0
;	* ATTRORIE=1 (aligned) then rota=roll
;	* ATTRORIE=0 (misaligned) then rota and roll are different
;	-----------------------------------------------------------------
;
;	Bug for first raster point = [0.,0.] fixed (CG)		Feb. 2000
;	Bug for noise and unflat for SRD derivation fixed (CG)	Feb. 2000
;
;	Colorbar min. value changed to the min. of the images 
;	instead of 0.0 (MH)					Mar. 2000
;
;-

PRO gen_raster, skyc, mappar, rpar, in_buf=in_buf, phtiaap=phtiaap, $
	raster_coord=raster_coord, raster_pos=raster_pos, GRAPH=graph, $
	sky_v=sky_v, sky_h=sky_h, zoomd=zoomd, dersrd=dersrd
; -------------------------------------------------------------------

	admn = mappar.admn
	   admn_tmp=strmid(admn,0,2)+'01'+strmid(admn,4,8)
	nele = N_ELEMENTS(skyc(*,0))
; CG: changing (findgen(nele) - nele/2. + 1) to (findgen(nele) - nele/2. + 0.5)
	sky_h = (findgen(nele) - nele/2. + 0.5) * mappar.pixsize(0)
	sky_v = sky_h
	sd = nele * mappar.pixsize(0)
	det = mappar.detector
	delt = 46. ; for both C100 and C200
	filt = id_give_filter(det, mappar.filter)
	if (filt-1) eq 1 then cwvl=90.0	; For the phtiaap.cwvl
	if (filt-1) eq 2 then cwvl=65.0
	if (filt-1) eq 3 then cwvl=60.0
	if (filt-1) eq 4 then cwvl=80.0
	if (filt-1) eq 5 then cwvl=100.0
	if (filt-1) eq 6 then cwvl=105.0
	if (filt-1) eq 7 then cwvl=90.0
	if (filt-1) eq 8 then cwvl=170.0
	if (filt-1) eq 9 then cwvl=200.0
	if (filt-1) eq 10 then cwvl=180.0
	if (filt-1) eq 11 then cwvl=150.0
	if (filt-1) eq 12 then cwvl=120.0
	
	aper=10				; For the phtiaap.aper
	polz=14				; For the phtiaap.polz
	
	IF det EQ 'C1' THEN BEGIN
	 half_dia = 46.0+(43.5/2.0) ; 67.7500 = half the detector
	 pix_size=43.5
 	 pix_area=pix_size^2 
	ENDIF ELSE BEGIN     
	 half_dia=46.0-(89.4/2.0)+89.4	      ; 90.7000 = half the detector
 	 pix_size=89.4
 	 pix_area=pix_size^2
	ENDELSE

	 
	IF not KEYWORD_SET(graph) THEN graph=0
raster:
	IF graph THEN BEGIN

	  rpar = mapsim_defparr()
	  
	  ncolors = !D.N_COLORS-12
	  sky_h_label='South (arcsec)'
	  sky_v_label='East (arcsec)'
	  cs = 1.5
	  barposition=[0.15,0.90,0.90,0.95]
	  imageposition=[0.15,0.10,0.903,0.853]
	  det = mappar.detector
	  
	  
	  window, /free, xsize=600, ysize=600, $
	   title='Convolved sky image'
	  !P.MULTI=[0,1,1]
	  erase
	  position=[0.15,0.10,0.90,0.85]
	  xsize=(position(2)-position(0))*!d.x_vsize
	  ysize=(position(3)-position(1))*!d.y_vsize
	  xstart=position(0)*!d.x_vsize+1
	  ystart=position(1)*!d.y_vsize+1
	  colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
	   min=min(skyc),max=max(skyc), ncolors=ncolors
	  plot,sky_h,sky_v,ticklen=-0.015,xstyle=1,ystyle=1,$
	   xrange=[max(sky_h),min(sky_h)],$
	   charsize=cs,xtitle=sky_h_label,ytitle=sky_v_label,$
	   position=imageposition,/nodata,/noerase,$
	   title='Convolved map + raster positions (Jy/beam)'
	  tvscl,congrid(skyc,xsize,ysize,/interp),xstart,ystart,$
	  	top=ncolors
	  xyouts,min(sky_h)*450.0/400.0,0,'West (arcsec)',/data,$
	   alignment=0.5,orientation=-90.0,charsize=cs


	  print,' Click at the center of the raster map'
	  cursor,xcenter,ycenter,/data	  

	  xcenter=double(xcenter)
	  ycenter=double(ycenter)

	  xyouts,xcenter,ycenter,/data,'_',charsize=2.5,alignment=0.5,$
	   color=100,orientation=45
	  xyouts,xcenter,ycenter,/data,'_',charsize=2.5,alignment=0.5,$
	   color=100,orientation=-45

	  mpts=3     ; number of points, M < 32 (along spacecraft Y-direction)
	  nlines=3   ; number of lines, N < 32 (along spacecraft Z-direction)
	  scan_angle=-1.0

	  print,' Enter the number of points (M < 32)'
	  read,mpts
	  print,' Enter the number of legs (N < 32)'
	  read,nlines

	  rpar.pos_rst = [xcenter, ycenter] 
	  rpar.nrpts = mpts
	  rpar.nrlns = nlines
	  rpar.p32 = 0

	  ans='n'
	  print,' Would you like to use the chopper (P32 AOT)?'
	  print,' Type Y for yes or N for no'
	  read,ans

; These mapping simulation mode is different from P32 in that the last 
; chopper position will always have full sweep of the chopper throw 
; (i.e., 13 steps for C100 or 7 for C200).  Using P32 AOT, the observation 
; may end on any chopper throw depending on when the observation time runs out.

; As a default, 3 swings of chopper at each raster position will be assumned.

	  if ans eq 'Y' or ans eq 'y' or ans eq 'yes' or ans eq 'Yes' then begin

		ans = 'Y'
		choice=1
		print,' Please enter the number corresponding to the Y and Z steps '
	        print,' you like to choose.'

		if det eq 'C1' then begin
		   pnum   =9	; there are 9 pixels in C100 detector
		   chopnum=13	; and 13 chopper positions per raster point
		   delt=46.0	; each pixel has dimensions 43.5" x 43.5"
				; this is between the pixel centers
print,' '
print,'  Choice  |  Y step  |  Z step  '
print,'          | (arcsec) | (arcsec) '
print,'--------------------------------'
print,'     1    |   60     |   15     '
print,'     2    |   60     |   23     '
print,'     3    |   60     |   46     '
print,'     4    |   92     |   69     '
print,'     5    |   92     |   92     '
		    read,choice
		    if choice eq 1 then begin
			 m_step=60.0
			 n_step=15.0
		    endif
		    if choice eq 2 then begin
			 m_step=60.0
			 n_step=23.0
		    endif
		    if choice eq 3 then begin
			 m_step=60.0
			 n_step=46.0
		    endif
		    if choice eq 4 then begin
			 m_step=92.0
			 n_step=69.0
		    endif
		    if choice eq 5 then begin
			 m_step=92.0
			 n_step=92.0
		    endif
		    cpos_elements=fltarr(13)
		    for i=0,12 do cpos_elements(i)=-90+i*15.0
		    cstp_elements=intarr(13)
		    for i=0,12 do cstp_elements(i)=i+1
		    start=6
		    step=39
		endif

		if det eq 'C2' then begin
		      pnum=4	; there are 4 pixels in C200 detector
		      chopnum=7	; and 7 chopper positions per raster point
		      delt = 46.0	; each pixel has dimensions 89.4" x 89.4"
					; this equals half the pixel size + half the gap size
		 print,' '
		 print,'  Choice  |  Y step  |  Z step  '
		 print,'--------------------------------'
		 print,'     1    |   92     |   31     '
		 print,'     2    |   92     |   46     '
		 print,'     3    |   92     |   92     '
		 read,choice
		 if choice eq 1 then begin
		  m_step=92.0
		  n_step=31.0
		 endif
		 if choice eq 2 then begin
		  m_step=92.0
		  n_step=46.0
		 endif
		 if choice eq 3 then begin
		  m_step=92.0
		  n_step=92.0
		 endif
		 cpos_elements=fltarr(7)
		 for i=0,6 do cpos_elements(i)=-90+i*30.0
		 cstp_elements=intarr(7)
		 for i=0,6 do cstp_elements(i)=i+1
		 start=3
		 step=21
		endif
		
		rpar.p32 = choice
		
	   ENDIF ELSE ans = 'N'
; end of P32 question 
	
	ENDIF ELSE BEGIN
; end of graph - begin of non-graph

	   xcenter = rpar.pos_rst(0)
	   ycenter = rpar.pos_rst(1)
	   mpts = rpar.nrpts
	   nlines = rpar.nrlns
	   ans = 'N'
	   IF rpar.p32 NE 0 THEN BEGIN
		   ans = 'Y'
		   IF mappar.detector EQ 'C2' THEN BEGIN
		      m_step = 92
		      n_step = [31,46,92]
		      n_step = n_step(rpar.p32 -1)
		      cpos_elements=-90+indgen(7)*30.0
		      cstp_elements=indgen(7)+1
		      start=3
		      step=21
		      pnum   =4	  ; there are 4 pixels in C200 detector
		      chopnum=7   ; and 7 chopper positions per raster point
		      delt=46.0	  ; each pixel has dimensions 43.5" x 43.5"
				  ; this equals half the pixel size + half the gap size
		   ENDIF ELSE IF mappar.detector EQ 'C1' THEN BEGIN
		      m_step = [60,60,60,92,92]
		      n_step = [15,23,46,69,92]
		      m_step = m_step(rpar.p32 -1)
		      n_step = n_step(rpar.p32 -1)
		      cpos_elements=-90+indgen(13)*15.0
		      cstp_elements=indgen(13)+1
		      start=6
		      step=39
		      pnum   =9	  ; there are 9 pixels in C100 detector
		      chopnum=13  ; and 13 chopper positions per raster point
		      delt=46.0	  ; each pixel has dimensions 43.5" x 43.5"
		   ENDIF ELSE BEGIN
		      PRINT, 'Wrong detector name, returning'
		      RETURN
		   ENDELSE

	   ENDIF
; end of non-graphical p32

	ENDELSE
; end of non-graphical
	
	IF ans EQ 'Y' THEN BEGIN
; begin of P32 - both graph and non-graph
		   orie = 1
		   scan_angle = -1.

		   dims=long(mpts*nlines*chopnum*3) 

		   cstp=intarr(dims)
		   for i=0,chopnum-1 do begin
		    for j=0+i,dims-1+i,chopnum do cstp(j)=cstp_elements(i)
		   endfor

		   cpos=fltarr(dims)
		   for i=0,chopnum-1 do begin
		    for j=0+i,dims-1+i,chopnum do cpos(j)=cpos_elements(i)
		   endfor

		   rpts=long(mpts*nlines)
		   rptsid=bytarr(2,rpts)

		   for i=0,mpts-1 do $
		    for j=0+i,rpts-mpts+i,mpts do $
		     rptsid(0,j)=(j-i)/mpts+1

		   if float(nlines)/2 ne fix(nlines/2) then $
		      limit=rpts else $
		      limit=rpts-2*mpts 

		   for i=0,limit,2*mpts do begin
			 for j=0,mpts-1 do rptsid(1,j+i)=j+1
		   endfor
		   for i=mpts,rpts-mpts,2*mpts do begin
			 for j=0,mpts-1 do rptsid(1,j+i)=mpts-j
		   endfor

		   rpid=bytarr(2,dims)
		   for i=0,3*chopnum-1 do for j=0+i,dims-1,3*chopnum do $
			 rpid(*,j)=rptsid(*,(j-i)/(3*chopnum))
			 rpid_tmp=rpid
			 rpid(0,*)=rpid_tmp(1,*)
			 rpid(1,*)=rpid_tmp(0,*)

		   m_size=(mpts-1)*m_step
		   n_size=(nlines-1)*n_step

		   raster_pos=fltarr(2,dims)

		   raster11x=xcenter-m_size/2.0	
		   raster11y=ycenter+n_size/2.0

		   for i=0,dims-1 do begin
		    raster_pos(0,i)=raster11x+(rpid(0,i)-1)*m_step + cpos(i)	
		    raster_pos(1,i)=raster11y-(rpid(1,i)-1)*n_step
		   endfor

		   IF graph THEN BEGIN
		    for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		     /data,'_',charsize=2.5,alignment=0.5,$
		     color=100,orientation=45
		    for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		     /data,'_',charsize=2.5,alignment=0.5,$
		     color=100,orientation=-45

		    for n=start,dims-1,step do xyouts,raster_pos(0,n),raster_pos(1,n),$
		     /data,'_',alignment=0.5,color=170,charsize=2.5,orientation=45
		    for n=start,dims-1,step do xyouts,raster_pos(0,n),raster_pos(1,n),$
		     /data,'_',alignment=0.5,color=170,charsize=2.5,orientation=-45


		    roll_angle=30.0
		    print,' Enter the roll angle in degrees'
		    print,' (counterclockwise direction: North to eastwards)'
		    read,roll_angle
		    roll=roll_angle*!pi/180.0
		    rpar.roll = roll_angle
		    rpar.rota = roll_angle
		    aroll=fltarr(dims)+roll_angle
		   ENDIF ELSE BEGIN
		   	roll_angle = rpar.roll
		   	rpar.rota = roll_angle
			roll = rpar.roll * !pi/180.0
			aroll=fltarr(dims)+roll_angle
		   ENDELSE
		   
		   new_pt=fltarr(2)
		   new_pt(0)=raster_pos(0,0)-xcenter
		   new_pt(1)=raster_pos(1,0)-ycenter
		   length=sqrt(new_pt(0)^2+new_pt(1)^2)

		   if new_pt(0) lt 0.0 and new_pt(1) eq 0.0 then $
		    angle_o=!pi
; CG: adding possibility of new_pt = [0.,0.], since otherwise angle_o not
; defined
		   if new_pt(0) eq 0.0 and new_pt(1) ge 0.0 then $
		    angle_o=!pi/2.0
		   if new_pt(0) lt 0.0 and new_pt(1) gt 0.0 then $
		    angle_o=atan(new_pt(1)/new_pt(0))+!pi

		   if new_pt(0) eq 0.0 and new_pt(1) eq 0.0 then $
		    new_angle=angle_o else new_angle=angle_o-roll

		   new_pt(0)=length*cos(new_angle)
		   new_pt(1)=length*sin(new_angle)

		   raster_coord=dblarr(2,dims)
		   raster11y=new_pt(0)+xcenter
		   raster11z=new_pt(1)+ycenter

		   m_xstep=m_step*cos(roll)
		   m_ystep=m_step*sin(roll)*(-1)
		   n_xstep=n_step*sin(roll)*(-1)	
		   n_ystep=n_step*cos(roll)*(-1)

		   x_cpos=cpos*cos(roll)
		   y_cpos=cpos*sin(roll)*(-1)	
		   raster11y=raster11y-x_cpos(0)
		   raster11z=raster11z-y_cpos(0)


		   raster_coord(0,*)=raster11y+(rpid(0,*)-1)*m_xstep+$
		     (rpid(1,*)-1)*n_xstep + x_cpos
		   raster_coord(1,*)=raster11z+(rpid(0,*)-1)*m_ystep+$
		     (rpid(1,*)-1)*n_ystep + y_cpos

		   corner=where(rpid(1,*) eq nlines and rpid(0,*) eq 1)
		   xposition=raster_coord(0,corner(0))+1.5*n_step*sin(roll)*(-1)-$
		    1.5*m_step*cos(roll) 
		   yposition=raster_coord(1,corner(0))+1.5*n_step*cos(roll)*(-1)-$
		    1.5*m_step*sin(roll)*(-1) 

; end of P32
	ENDIF ELSE BEGIN
; begin of P22

		  dims=long(mpts*nlines)
		  rpid=bytarr(2,dims)

		  for i=0,mpts-1 do $
		   for j=0+i,dims-mpts+i,mpts do $
		    rpid(1,j)=(j-i)/mpts+1

		  if float(nlines)/2 ne fix(nlines/2) then limit=dims $
		  				else limit=dims-2*mpts 

		  for i=0,limit,2*mpts do begin
		   for j=0,mpts-1 do rpid(0,j+i)=j+1
		  endfor
		  for i=mpts,dims-mpts,2*mpts do begin
		   for j=0,mpts-1 do rpid(0,j+i)=mpts-j
		  endfor

		  m_step=60.0	; raster steps in arcseconds
		  n_step=60.0

		  IF graph THEN BEGIN
		    print,' Enter the raster step size in arcseconds between the points'
		    read,m_step
		    print,' Enter the raster step size in arcseconds between the lines'
		    read,n_step
		    rpar.dpts = m_step
		    rpar.dlns = n_step
		  ENDIF ELSE BEGIN
		    m_step = rpar.dpts
		    n_step = rpar.dlns
		  ENDELSE
		  
		  m_size=(mpts-1)*m_step
		  n_size=(nlines-1)*n_step

		  raster_pos=fltarr(2,dims)

		  raster_pos(0,0)=xcenter-m_size/2.0	
		  raster_pos(1,0)=ycenter+n_size/2.0

		  for i=1,dims-1 do begin
		   raster_pos(0,i)=raster_pos(0,0)+(rpid(0,i)-1)*m_step	
		   raster_pos(1,i)=raster_pos(1,0)-(rpid(1,i)-1)*n_step
		  endfor

		  cstp=intarr(dims)+1
		  cpos=fltarr(dims)

		  IF graph THEN BEGIN
		   for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		    /data,'_',charsize=2.5,alignment=0.5,$
		    color=100,orientation=45
		   for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		    /data,'_',charsize=2.5,alignment=0.5,$
		    color=100,orientation=-45

		   for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		    /data,n+1,alignment=0.5,color=70

		   ans='n'
		   print,' '
		   print,' Would you like to do a special case where the detector and '
		   print,' raster pattern are not aligned?  Type Y for yes or N for no.'
		   read,ans

		   if STRUPCASE(STRMID(ans,0,1)) eq 'Y' then begin
		    
		    orie = 0
		    ans = 'Y'
		    scan_angle=30.0
		    print,' Enter the scan angle in degrees'
		    print,' (counterclockwise direction: North to eastwards).'
		    print,' This is for the raster pattern orientation.'
		    read,scan_angle
		    scan_rad=scan_angle*!pi/180.0

		    roll_angle=20.0
		    print,' Enter the angle for the detector array orientation'
		    print,' (counterclockwise direction: North to eastwards).'
		    read,roll_angle
		    roll=roll_angle*!pi/180.0
		    rpar.orie = 0
		    rpar.rota = scan_angle
		    rpar.roll = roll_angle
		    aroll=fltarr(dims)+roll_angle
		    
		   ENDIF ELSE BEGIN
		    ans='N'
		    orie=1
		    rpar.orie=orie		    
		   ENDELSE
; non-graph case
		  ENDIF ELSE BEGIN
		   IF rpar.orie THEN BEGIN
		     ans = 'N' 
		     orie = rpar.orie
		     scan_angle = rpar.roll
		     rpar.rota = rpar.roll
		   ENDIF ELSE BEGIN
		     ans = 'Y'
		     orie = rpar.orie
		     scan_angle = rpar.rota
		   ENDELSE
		   scan_rad=scan_angle*!pi/180.0 
		   roll_angle = rpar.roll
		   roll=roll_angle*!pi/180.0
		   aroll=fltarr(dims)+roll_angle
		  ENDELSE
		   
		  
;		  orie=1
;		  rota = 0.
		   
		  IF ans EQ 'Y' THEN BEGIN
		    orie = 0
		    rota = scan_angle
		     
		    ang_diff=scan_rad-roll

		    new_pt=fltarr(2)
		    new_pt(0)=raster_pos(0,0)-xcenter
		    new_pt(1)=raster_pos(1,0)-ycenter
		    length=sqrt(new_pt(0)^2+new_pt(1)^2)

		    if new_pt(0) lt 0.0 and new_pt(1) eq 0.0 then $
			  angle_o=!pi
; CG: adding possibility of new_pt = [0.,0.], since otherwise angle_o not
; defined
		    if new_pt(0) eq 0.0 and new_pt(1) ge 0.0 then $
			  angle_o=!pi/2.0
		    if new_pt(0) lt 0.0 and new_pt(1) gt 0.0 then $
			  angle_o=atan(new_pt(1)/new_pt(0))+!pi

		    if new_pt(0) eq 0.0 and new_pt(1) eq 0.0 then $
		    new_angle=angle_o else new_angle=angle_o-scan_rad

		    new_pt(0)=length*cos(new_angle)
		    new_pt(1)=length*sin(new_angle)

		    raster_coord=dblarr(2,dims)
		    raster_coord(0,0)=new_pt(0)+xcenter
		    raster_coord(1,0)=new_pt(1)+ycenter

		    m_xstep=m_step*cos(scan_rad)
		    m_ystep=m_step*sin(scan_rad)*(-1)
		    n_xstep=n_step*sin(scan_rad)*(-1)	
		    n_ystep=n_step*cos(scan_rad)*(-1)	

		    for i=1,dims-1 do begin
			 raster_coord(0,i)=raster_coord(0,0)+(rpid(0,i)-1)*m_xstep+$
			  (rpid(1,i)-1)*n_xstep
			 raster_coord(1,i)=raster_coord(1,0)+(rpid(0,i)-1)*m_ystep+$
			  (rpid(1,i)-1)*n_ystep
		    endfor

		    corner=where(rpid(1,*) eq nlines and rpid(0,*) eq 1)
		    if dims gt mpts*nlines then corner=corner(0)
		    xposition=raster_coord(0,corner)+1.5*n_step*sin(roll)*(-1)-$
			 1.5*m_step*cos(roll)
		    yposition=raster_coord(1,corner)+1.5*n_step*cos(roll)*(-1)-$
			 1.5*m_step*sin(roll)*(-1)

		  ENDIF ELSE BEGIN

		    IF graph THEN BEGIN
		     roll_angle=30.0
		     print,' Enter the roll angle in degrees'
		     print,' (counterclockwise direction: North to eastwards)'
		     read,roll_angle
		     roll=roll_angle*!pi/180.0
		     rpar.roll = roll_angle
		     aroll=fltarr(dims)+roll_angle
		     rpar.rota = roll_angle
		     rpar.orie = 1
		    ENDIF 
		    new_pt=fltarr(2)
		    new_pt(0)=raster_pos(0,0)-xcenter
		    new_pt(1)=raster_pos(1,0)-ycenter
		    length=sqrt(new_pt(0)^2+new_pt(1)^2)

		    if new_pt(0) lt 0.0 and new_pt(1) eq 0.0 then $
			  angle_o=!pi
; CG: adding possibility of new_pt = [0.,0.], since otherwise angle_o not
; defined
		    if new_pt(0) eq 0.0 and new_pt(1) ge 0.0 then $
			  angle_o=!pi/2.0
		    if new_pt(0) lt 0.0 and new_pt(1) gt 0.0 then $
			  angle_o=atan(new_pt(1)/new_pt(0))+!pi
			  
		    if new_pt(0) eq 0.0 and new_pt(1) eq 0.0 then $
			  new_angle=angle_o else new_angle=angle_o-roll

		    new_pt(0)=length*cos(new_angle)
		    new_pt(1)=length*sin(new_angle)

		    raster_coord=dblarr(2,dims)
		    raster_coord(0,0)=new_pt(0)+xcenter
		    raster_coord(1,0)=new_pt(1)+ycenter

		    m_xstep=m_step*cos(roll)
		    m_ystep=m_step*sin(roll)*(-1)
		    n_xstep=n_step*sin(roll)*(-1)	
		    n_ystep=n_step*cos(roll)*(-1)	

		    for i=1,dims-1 do begin
			 raster_coord(0,i)=raster_coord(0,0)+(rpid(0,i)-1)*m_xstep+$
			  (rpid(1,i)-1)*n_xstep
			 raster_coord(1,i)=raster_coord(1,0)+(rpid(0,i)-1)*m_ystep+$
			  (rpid(1,i)-1)*n_ystep
		    endfor

		    corner=where(rpid(1,*) eq nlines and rpid(0,*) eq 1)
		    if dims gt mpts*nlines then corner=corner(0)
		    xposition=raster_coord(0,corner)+1.5*n_xstep-1.5*m_xstep
		    yposition=raster_coord(1,corner)+1.5*n_ystep-1.5*m_ystep

		    ang_diff=0.0

		  ENDELSE
; end of P22
		ENDELSE

		if min(raster_coord(0,*))-half_dia lt min(sky_h) or $
		 max(raster_coord(0,*))+half_dia gt max(sky_h) or $
		 min(raster_coord(1,*))-half_dia lt min(sky_v) or $
		 max(raster_coord(1,*))+half_dia gt max(sky_v) then begin
		 IF GRAPH THEN BEGIN
		  print,' The raster spans area outside the sky. '
		  print,' Please choose a smaller map or smaller step size(s).'
		  print,' '
		  goto,raster
		 ENDIF ELSE BEGIN
		  text = [' The raster spans area outside the sky. ',$
		  	  ' Please choose a smaller map or smaller step size(s).']
		  PIA_DISP_PRINT_FILE, '', Text=text, width=50, height=4
		  RETURN		  
		 ENDELSE
		endif

		IF graph THEN BEGIN
		 for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		  /data,'_',charsize=2.5,alignment=0.5,$
		  color=20,orientation=45
		 for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$
		  /data,'_',charsize=2.5,alignment=0.5,$
		  color=20,orientation=-45

		 if dims eq mpts*nlines then $
		 for n=0,dims-1 do xyouts,raster_pos(0,n),raster_pos(1,n),$	
			 /data,n+1,alignment=0.5,color=20 

		 for n=0,dims-1 do xyouts,raster_coord(0,n),raster_coord(1,n),$	
			 /data,'_',charsize=2.5,alignment=0.5,$
			 color=200,orientation=45
		 for n=0,dims-1 do xyouts,raster_coord(0,n),raster_coord(1,n),$
		  /data,'_',charsize=2.5,alignment=0.5,$
		  color=200,orientation=-45

		 if dims eq mpts*nlines then $
		   xyouts,raster_coord(0,*),raster_coord(1,*),$
		    /data,STRTRIM(indgen(dims)+1,2),alignment=0.5,color=200 $
;		   for n=0,dims-1 do xyouts,raster_coord(0,n),raster_coord(1,n),$
;		    /data,n+1,alignment=0.5,color=200 $
		 else begin
		  for n=start,dims-1,step do xyouts,raster_coord(0,n),raster_coord(1,n),$
		         /data,'_',alignment=0.5,color=70,charsize=2.5,orientation=45
		  for n=start,dims-1,step do xyouts,raster_coord(0,n),raster_coord(1,n),$
			 /data,'_',alignment=0.5,color=70,charsize=2.5,orientation=-45
		 endelse

		 xyouts,xposition,yposition,$
			 '-- spacecraft Z direction -->',/data,$
			 charsize=cs,orientation=90.0+roll_angle,color=100

		 xyouts,xposition,yposition,$
			 '<-- spacecraft Y direction --',/data,$
			 charsize=cs,orientation=roll_angle,color=100,alignment=1.0

		ENDIF
		 
; create dummies for the P22 case - otherwise take the chopper throws out
; for calculating ra and dec

		IF N_ELEMENTS(x_cpos) EQ 0 THEN BEGIN
		 x_cpos = 0. & y_cpos = 0.
		ENDIF
		ra=reform(double(raster_coord(0,*)-x_cpos)/60.0/60.0)
		dec=reform(double(raster_coord(1,*)-y_cpos)/60.0/60.0)

; -------------------------------------------------------------------

; interpolate the skyc unto a fine-grid scale?

		xc=n_elements(sky_h(*,0))/sd*(sd/2+xcenter)
		yc=n_elements(sky_v(*,0))/sd*(sd/2+ycenter)

		skyr=rot(rotate(skyc,5),(-1)*roll_angle,1.0,xc,yc,/pivot)
		skyrd=rot(rotate(skyc,5),(-1)*roll_angle,1.0,xc,yc,/pivot,/interp)

		sky_y=(sky_h-xcenter)
		sky_z=(sky_v-ycenter)

		sky_y_label='spacecraft Y axis (arcsec)'
		sky_z_label='spacecraft Z axis (arcsec)'

		r_pos=raster_pos
 		r_pos(0,*)=(raster_pos(0,*)-xcenter)
 		r_pos(1,*)=(raster_pos(1,*)-ycenter)
		raster_pos = r_pos

		if scan_angle ne -1.0 then begin

		 new_pt2=fltarr(2)
		 new_pt2(0)=r_pos(0,0)
		 new_pt2(1)=r_pos(1,0)
		 length=sqrt(new_pt2(0)^2+new_pt2(1)^2)

		 if new_pt2(0) lt 0.0 and new_pt2(1) eq 0.0 then $
		  angle_o2=!pi
; CG: adding case new_pt2 = [0.,0.] otherwise angle_o2 not defined 
		 if new_pt2(0) eq 0.0 and new_pt2(1) ge 0.0 then $
		  angle_o2=!pi/2.0
		 if new_pt2(0) lt 0.0 and new_pt2(1) gt 0.0 then $
		  angle_o2=atan(new_pt2(1)/new_pt2(0))+!pi

		 if new_pt2(0) eq 0.0 and new_pt2(1) eq 0.0 then $
		  new_angle2=angle_o2 else new_angle2=angle_o2-ang_diff

		 new_pt2(0)=length*cos(new_angle2)
		 new_pt2(1)=length*sin(new_angle2)

		 r_pos=dblarr(2,dims)
		 raster11y=new_pt2(0)
		 raster11z=new_pt2(1)

		 m_xstep2=m_step*cos(ang_diff)
		 m_ystep2=m_step*sin(ang_diff)*(-1)
		 n_xstep2=n_step*sin(ang_diff)*(-1)	
		 n_ystep2=n_step*cos(ang_diff)*(-1)

		 for i=0,dims-1 do begin
			 r_pos(0,i)=raster11y+(rpid(0,i)-1)*m_xstep2+$
			  (rpid(1,i)-1)*n_xstep2
			 r_pos(1,i)=raster11z+(rpid(0,i)-1)*m_ystep2+$
			  (rpid(1,i)-1)*n_ystep2
		 endfor
		 raster_pos = r_pos
		endif

; rotate by roll_angle with interp and pivot keywords set for display purpose
 
; -------------------------------------------------------------------

; Using the detector array centers, get the center coordinates of pixels
;  ( ) display the pixel centers with dots

; The center of the detector is noted with X = (y0, z0)
;
; The pixel arrangement for C100 detector.
;
;       3       6       9       +Y to the right
;       2      5=X      8       +Z upwards
;       1       4       7
;
;                       j       i    p=3*j+i    yout    zout 
;                       ---------------------------------------
;                       0       0       0       y0-delt z0-delt
;                       1       0       3       y0      z0-delt
;                       2       0       6       y0+delt z0-delt
;                       0       1       1       y0-delt z0
;                       1       1       4       y0      z0
;                       2       1       7       y0+delt z0
;                       0       2       2       y0-delt z0+delt
;                       1       2       5       y0      z0+delt
;                       2       2       8       y0+delt z0+delt
;
;
; The pixel arrangement for C200 detector.
;
;       3       4               +Y to the right
;           X                   +Z upwards      
;       2       1               
;
;                       j       i  p=abs(3*i-j) yout    zout 
;                       ---------------------------------------
;                       0       0       0       y0+delt z0-delt         
;                       1       0       1       y0-delt z0-delt
;                       0       1       3       y0+delt z0+delt
;                       1       1       2       y0-delt z0+delt
 
		y0=r_pos(0,*)
		z0=r_pos(1,*)
	
		chpl=dims

		if det eq 'C1' then begin
		  pnum=9
		  yout=fltarr(pnum,chpl)
		  zout=fltarr(pnum,chpl)
		  for k=0,chpl-1 do begin
		    for i=0,2 do begin
		      for j=0,2 do begin
		        p=3*j+i
		        yout(p,k)=delt*(j-1)+y0(k)
			zout(p,k)=delt*(i-1)+z0(k)
		      endfor
		    endfor
		  endfor
		endif
		if det eq 'C2' then begin
		  pnum=4
		  yout=fltarr(pnum,chpl)
		  zout=fltarr(pnum,chpl)
		  for k=0,chpl-1 do begin
		    for i=0,1 do begin
		      for j=0,1 do begin
			p=abs(3*i-j)
			yout(p,k)=delt*(1-2*j)+y0(k)
			zout(p,k)=delt*(2*i-1)+z0(k)
		      endfor
		    endfor
		  endfor  
		endif

; zoom in on the raster region
		set_y=where(sky_y ge min(yout)-half_dia $
			 and sky_y le max(yout)+half_dia)
		set_z=where(sky_z ge min(zout)-half_dia $
			 and sky_z le max(zout)+half_dia)

		sky_ys=sky_y(set_y)
		sky_zs=sky_z(set_z)

		skyrds=fltarr(n_elements(set_y),n_elements(set_z))
		skyrs=fltarr(n_elements(set_y),n_elements(set_z))

		for i=0,n_elements(set_y)-1 do begin
		   for j=0,n_elements(set_z)-1 do begin
			  skyrds(i,j)=skyrd(set_y(i),set_z(j))
			  skyrs(i,j)=skyr(set_y(i),set_z(j))
		   endfor
		endfor

		skyrs=rotate(skyrs,5)
		skyrds=rotate(skyrds,5)
 
		zoomd ={sky_ys:sky_ys, sky_zs:sky_zs, yout:yout, zout:zout, $
			skyrds:skyrds}
			
		IF graph THEN BEGIN
		  window, /free, xsize=600, ysize=600, $
		   title='Zoom in on the raster'
		  !P.MULTI=[0,1,1]
 
		  colorbar,charsize=cs,divisions=4,top=1,position=[0.15,0.90,0.90,0.95],$
			  min=min(skyrds),max=max(skyrds), ncolors=ncolors
		  plot,sky_ys,sky_zs,ticklen=-0.015,xstyle=1,ystyle=1,$
		   xrange=[max(yout)+half_dia,min(yout)-half_dia],$
		   yrange=[min(zout)-half_dia,max(zout)+half_dia],$
		   charsize=cs,xtitle=sky_y_label,ytitle=sky_z_label,$
		   position=imageposition,/nodata,/noerase,$
		   title='With spacecraft coordinates (Jy/beam)'
		  tvscl,congrid(skyrds,xsize,ysize,/interp),xstart,ystart,$
		  	top=ncolors
		  if dims eq mpts*nlines then $
		  xyouts,r_pos(0,*),r_pos(1,*),$	
			 /data,STRTRIM(indgen(dims)+1,2),alignment=0.5,color=70
;		  for n=0,dims-1 do xyouts,r_pos(0,n),r_pos(1,n),$	
;			 /data,n+1,alignment=0.5,color=70

		  xyouts,0.0,0.0,/data,'_',charsize=2.5,alignment=0.5,$
			 color=100,orientation=45
		  xyouts,0.0,0.0,/data,'_',charsize=2.5,alignment=0.5,$
		  color=100,orientation=-45

		  for n=0,dims-1 do xyouts,r_pos(0,n),r_pos(1,n),$
		 	/data,'_',charsize=2.5,alignment=0.5,$
			 color=200,orientation=45
		  for n=0,dims-1 do xyouts,r_pos(0,n),r_pos(1,n),$
			 /data,'_',charsize=2.5,alignment=0.5,$
			 color=200,orientation=-45

		  oplot,yout,zout,psym=3
		  for i=0,pnum-1 do xyouts,yout(i,0),zout(i,0),i+1,color=100,alignment=1
		  
; ask if the user wants to continue to the next level which is producing the SRD structure.
		  srd_ans='yes'
		  print,' Do you plan to continue and produce SRD structure for this'
		  print,' simulated data (the signals are in V/s and may contain transients'
		  print,' and/or glitches)? Answer yes or no.'
		     read, srd_ans
		  if STRUPCASE(strmid(srd_ans,0,1)) eq 'N' then begin
		  
		  	dersrd = 0
		  	
		        ; ask for noise level
		        noise=10.
		  	print,' Enter the signal noise level (in %)'
		           read, noise
		  	rpar.noise = noise
		     
		  	noise = noise / 100.
		  	
		  	; introduce unflat fielding
			; ask for unflat level
		  	unflat=10.
		  	print,' Enter the mean responsivity deviation per pixel (in %)'
		  	print,' "Unflatfield"'
		     	   read, unflat
		  	rpar.unflat = unflat
		     
		  	unflat = unflat / 100.

		  endif else begin
		  	
		  	dersrd = 1
		  	noise=0.0
		  	unflat=0.0
		  
		  endelse
		  	rpar.dersrd = dersrd
		  
		ENDIF ELSE BEGIN
		 IF KEYWORD_SET(dersrd) THEN BEGIN
		 	noise = 0. 
		 	unflat = 0.
		 ENDIF ELSE BEGIN
		 	noise = rpar.noise / 100.
		 	unflat = rpar.unflat / 100.
		 ENDELSE
		ENDELSE
; -------------------------------------------------------------------

pixnum=pnum

;CG here we use negative y_indexes because the skyrs should be 
;CG positive to the left (CG)

y_index=(-yout-min(sky_ys))*(n_elements(sky_ys)-1)/(max(sky_ys)-min(sky_ys))
z_index=(zout-min(sky_zs))*(n_elements(sky_zs)-1)/(max(sky_zs)-min(sky_zs))


mnbr=interpolate(double(skyrs),y_index,z_index)

	seed=0L
	ran = randomn(seed, pixnum, dims)
	mnbr = mnbr + mnbr * noise * ran 
	ran = randomn(seed, pixnum, dims)
	mnbu = ABS(mnbr * noise) + mnbr * noise*noise * ran
	
IF unflat GT 0. THEN $
   unflat, mnbr, ffct, unf=unflat
			
; originally mnbr was the brightness distribution
; but we changed it to flux/pixel [Jy/beam]
; change back the variables accordingly

mnfl = mnbr
mnfu = mnbu

fd = filt-1


IF fd GE 8 THEN ind_om = fd - 8 ELSE ind_om = fd-1
	 omega = get_omega(det, 0, ind_om)
	 omega = omega(0)
	 Epsilon = 0.3
	
mnbr = mnfl / omega / (1.-epsilon^2) * 1.0E-06
mnbu = mnfu / omega / (1.-epsilon^2) * 1.0E-06


;mdbr=mnbr

;steradian=double(180.0/!pi*60*60)^2

;mnfl=mnbr*1.0e6*pix_area/steradian	; Jy/beam where beam=pixel area
;mnfu=mnbu*1.0e6*pix_area/steradian	; Jy/beam where beam=pixel area

mdfl = mnfl
mdbr = mnbr
mdbu = mnbu

q1br = mdbr * (1-noise)
q3br = mdbr * (1+noise) 
q1fl = mdfl * (1-noise) 
q3fl = mdfl * (1+noise)

flag=intarr(pnum,dims)
	
; -------------------------------------------------------------------
; convert the MJy/sr into V/s  (refer to the signal2brightness.txt file)
; Note: All these calculations is to determine an appropriate value for the 
;	integration time.

obs=0.91	; an obscuration factor of the secondary mirror of the ISO telescope 
	
@phtw2jy

IF N_ELEMENTS(w2jy_c) EQ 0 THEN phtw2jy_init

c_1=w2jy_c(filt-1)	; a constant per filter retrieve using a common block 

if det eq 'C1' then filter=filt-2 
if det eq 'C2' then filter=filt-9
omega=get_omega(det,indgen(pnum),filter)	; the solid angle of the aperture per pixel

if det eq 'C1' then cap=9.e-14		; capacitance 
if det eq 'C2' then cap=1.4e-13

if det eq 'C1' then fxread,!calg+'PC1RESP_02.FITS',resp_data,resp_hdr
if det eq 'C2' then fxread,!calg+'PC2RESP_02.FITS',resp_data,resp_hdr
resp=resp_data(40,*,0)		; responsivity per pixel (default by orbital position 0.4 - Rev. > 150)

if det eq 'C1' then fxread,!calg+'PC1FLAT.FITS',ff_data,ff_hdr
if det eq 'C2' then fxread,!calg+'PC2FLAT.FITS',ff_data,ff_hdr
ff=ff_data(*,filter,0)		; flatfield per pixel

signal_maxs=dblarr(pnum)	; signal in V/s
for p=0,pnum-1 do signal_maxs(p)=max(mnbr(p,*))
i=where(signal_maxs eq max(signal_maxs)) 
signal_max = max(signal_maxs) * c_1 * obs * omega(i,0) * resp(i) * ff(i) * 1.e6 / cap
if n_elements(i) ge 1 then signal_max=signal_max(0)

if fix(alog(0.50/signal_max)/alog(2)) lt 0 then itime=1.0/2^abs(fix(alog(0.5/signal_max)/alog(2)))
if fix(alog(0.50/signal_max)/alog(2)) ge 0 then itime=2.0^fix(alog(0.50/signal_max)/alog(2))

; Note: Please note that the integration time (itime) is determined here according to the signal
; 	level 
; Now, check the value against the default value to see if other time variables need to be modified

   if dims gt mpts*nlines then begin
	if itime ne 1/8.0 then tfactor=1/8.0/itime else tfactor=1
   endif else begin
	if itime ne 1.0 then tfactor=1.0/itime else tfactor=1
   endelse

	IF NOT KEYWORD_SET(timdif) THEN $
		if dims gt mpts*nlines then timdif=0.500000/tfactor else timdif=18.0/tfactor
	IF NOT KEYWORD_SET(timsweep) THEN BEGIN
		if dims gt mpts*nlines and det eq 'C1' then timsweep=6.5000/tfactor
		if dims gt mpts*nlines and det eq 'C2' then timsweep=3.5000/tfactor
		if dims eq mpts*nlines then timsweep=7.0/tfactor
	ENDIF
	IF NOT KEYWORD_SET(rpchpl) THEN $
		if dims gt mpts*nlines then rpchpl=4 else rpchpl=128
	if dims gt mpts*nlines then $
		time=timdif/2+indgen(dims)*timdif+indgen(dims)/chopnum/3*timsweep else $
		time  = timdif/2 + indgen(dims) * timdif + indgen(dims) * timsweep
	dwel = FLTARR(dims) + timdif

   IF rpar.ownt eq 0 THEN BEGIN
      rpar.ownt = 1
      rpar.inttime = itime
      rpar.onptime = timdif
      rpar.tsweep = timsweep
      rpar.rpchpl = rpchpl 
   ENDIF

; -------------------------------------------------------------------
; Read a general hdr
DEFSYSV, '!CALSAVE', exists=i
IF i NE 1 THEN DEFSYSV, !CALSAVE, ''
 
RESTORE, !CALSAVE + 'AAP_MAP_HDR.SAV'

now = systime()
month = STRMID(now, 4,3)
day = STRMID(now,8,2)

FXADDPAR, hdr, 'FILENAME', admn , ' File name'
FXADDPAR, hdr, 'DATE    ', month+'/'+day+'/'+strmid(now,20,4) , ' Date	of simulation'
FXADDPAR, hdr, 'OBSERVER', 'PIA     ', ' Proposer ID '
FXADDPAR, hdr, 'OBJECT  ', 'Some Obj', ' Target ID as given by proposer'
if dims gt mpts*nlines then $
 FXADDPAR, hdr, 'EOHAAOTN', 'P32     ', ' AOT name ' else $
 FXADDPAR, hdr, 'EOHAAOTN', 'P22     ', ' AOT name '
FXADDPAR, hdr, 'EOHAPLID', 'LETSSIMU', ' Proposal ID '
FXADDPAR, hdr, 'ATTRA   ', AVG(ra)
FXADDPAR, hdr, 'ATTDEC  ', AVG(dec)
FXADDPAR, hdr, 'ATTRNPTS', mpts
FXADDPAR, hdr, 'ATTRNLNS', nlines
FXADDPAR, hdr, 'ATTRDPTS', m_step
FXADDPAR, hdr, 'ATTRDLNS', n_step
FXADDPAR, hdr, 'ATTRORIE', orie	
FXADDPAR, hdr, 'ATTRROTA', rpar.rota
FXADDPAR, hdr, 'PIA_MNAM', admn
FXADDPAR, hdr, 'PIA_VERS', !pia_version
FXADDPAR, hdr, 'COMMENT ', 'PHT '+det+'00 AAP Data level', before='ORIGIN  '
FXADDPAR, hdr, 'DETECTOR', det+'00'
FXADDPAR, hdr, 'BKG_FLAT', mappar.bkg_flat
FXADDPAR, hdr, 'GRD_BACK', mappar.grd_back
FXADDPAR, hdr, 'CWVL_F  ', string(cwvl)+' um'
FXADDPAR, hdr, 'ITIME   ', itime, ' Integration time', before='PIA_VERS'
FXADDPAR, hdr, 'TIMDIF  ', timdif, ' Time spent on each raster (chopper) position', before='PIA_VERS'
FXADDPAR, hdr, 'TIMSWEEP', timsweep, ' Detector sweeping time', before='PIA_VERS'
FXADDPAR, hdr, 'RPCHPL  ', rpchpl, ' Number of ramps per chopper plateau', before='PIA_VERS'

IF unflat GT 0. THEN $
  FOR i=0,N_ELEMENTS(ffct)-1 DO $
  	FXADDPAR, hdr, 'UNFLAT'+STRTRIM(i+1,2), ffct(i), $
		'Resp. unflatness introduced by simulation', $
		before='PIA_VERS'
		  
        phtiaap = {ADMN: admn_tmp, DET: det, $
                DIMS: dims, $
                FILT: [filt], APER: [aper], POLZ: [polz], $
                CWVL: [cwvl],                  $
                HDR: [hdr],                    $
                TIME: [time],    	       $
                CPOS: [cpos],      $
		CSTP: [cstp],     $
		DWEL: [dwel],     $ 
                MNFL: [mnfl], 	               $
                MNFU: [mnfu],                  $
                MNBR: [mnbr],           $
                MNBU: [mnbu],           $
                MDFL: [mdfl],           $
                Q1FL: [q1fl],           $
                Q3FL: [q3fl],           $
                MDBR: [mdbr],           $
                Q1BR: [q1br],           $
                Q3BR: [q3br],           $
                RPID: [rpid],   $ 
                DEC: [dec],       $
                RA: [ra],         $
                DECP: [mnfl]*0.,       $
                RAP: [mnfl]*0.,         $
                ROLL: [aroll],     $
                FLAG: [flag]}

; Change to AAP2000
gen_aap, phtiaap
	
IF KEYWORD_SET(in_buf) THEN $
   fill_aap, phtiaap

;save,phtiaap,filename=admn+'.IAAP'


RETURN
END
