;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       GEN_SRD
;
; PURPOSE:
;	Using the PHTIAAP structure created with GEN_RASTER, this program creates
;	a SRD structure, a Compact Status file and IRPH file (i.e., *.ISRD, *.ICS 
;	and *.IRPH files).
;	The SRD structure contains signals (ramp slopes in V/s)
;	which has user defined level of signal noise and the individual
;	uncertainty level.  It also enables the user to add transients and
;	random glitches to the signal.  In addition, unflatfielding can 
;	be applied as well.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       GEN_SRD, phtiaap, phtisrd, GRAPH=graph, RPAR=rpar
;
; INPUTS:
;	PHTIAAP		The AAP structure created from GEN_RASTER with DERSRD=1
;
;       RPAR            The structure containing the parameters from
;                       MAPSIM_DEFPARR and PIA_MAPSIM_RPAR
;
; KEYWORD PARAMETERS:
;       GRAPH           =1 for an interactive session or
;                       =0 for a non-interactive processing of the user-defined 
;                          parameters 
;
; OUTPUTS:
;	PHTISRD		The resulting SRD structure (save to a file)
;	PHTICS		The Compact Status structure (save to a file)
;	IRPH		The IRPH structure (save to a file)
;
; COMMON BLOCKS:
;	@phtw2jy
;	@phtcs
;	
; PROCEDURES USED:
;       PIA Procedure(s):	READ_CS
;				phtw2jy_init
;				unflat
;
;       Astronomical Library:	FXADDPAR
;				FXREAD
;				FXPAR
;				SXDELPAR
; FUNCTIONS USED:
;       tansient
;	get_omega
;
; MODIFICATION HISTORY:
;       Written by:             Min Hur, IPAC/Caltech       
;				Carlos Gabriel, ESA/Vilspa	Sep 1999
;
;	Bug for case of more than 1 "badpts" fixed (CG)		Dec 1999
;	IRPH file using full admn string (CG)			Dec 1999
;
;	Unflatfielding added (MH)				Jan 2000
;
; 	Update parameters according to user's input in 
;	 "graphical" case (CG)					Feb 2000
;
;	Bug fixed for adding glitches to short observations
;	Glitches algorithm changed (MH)				Mar 2000
;-

PRO gen_srd, phtiaap, phtisrd, GRAPH=graph, RPAR=rpar
; -------------------------------------------------------------------


mnbr=phtiaap.mnbr
mnbu=phtiaap.mnbu
time=phtiaap.time
det=phtiaap.det
dims=phtiaap.dims
filt=phtiaap.filt	; filt is the filter wheel number (refer to IDUM)
AOT = STRTRIM(FXPAR(phtiaap.hdr,'EOHAAOTN'),2)
points = FXPAR(phtiaap.hdr,'ATTRNPTS')
lines = FXPAR(phtiaap.hdr,'ATTRNLNS')

;itime = FXPAR(phtiaap.hdr,'ITIME')
;timdif = FXPAR(phtiaap.hdr,'TIMDIF')
;timsweep = FXPAR(phtiaap.hdr,'TIMSWEEP')
;rpchpl = FXPAR(phtiaap.hdr,'RPCHPL')

;aot=strmid(phtiaap.hdr(18),11,3)
;points=fix(strmid(phtiaap.hdr(31),11,20))
;lines=fix(strmid(phtiaap.hdr(32),11,20))
mapdims=long(points*lines)
admn_tmp=phtiaap.admn
   admn=strmid(admn_tmp,0,2)+'99'+strmid(admn_tmp,4,8)
   
; mimic an IRPH by assigning astrocoords to each indvidual rpid point 
neles = MAX(phtiaap.rpid(1,*)) * MAX(phtiaap.rpid(0,*)) 
irpid = BYTARR(2,neles)
ira = FLTARR(neles)
idec = ira & iroll = ira

count = -1
FOR rp1=1,max(phtiaap.rpid(1,*)) DO $
 FOR rp0=1,max(phtiaap.rpid(0,*)) DO BEGIN
   count = count + 1
   ind = WHERE(phtiaap.rpid(0,*) EQ rp0 AND phtiaap.rpid(1,*) EQ rp1, nfound)
   IF nfound GT 0 THEN BEGIN
	irpid(*,count) = [phtiaap.rpid(0,ind(0)),phtiaap.rpid(1,ind(0))]
	ira(count) = phtiaap.ra(ind(0))
	idec(count) = phtiaap.dec(ind(0))
	iroll(count) = phtiaap.roll(ind(0))
   ENDIF
 ENDFOR
 
	

;irph={ RPID: phtiaap.rpid, $
;       RA:   phtiaap.ra,   $
;       DEC:  phtiaap.dec,  $
;       ROLL: phtiaap.roll }
irph={ RPID: irpid, $
       RA:   ira,   $
       DEC:  idec,  $
       ROLL: iroll }

hdr=phtiaap.hdr
d_tkey=16384.0

if det eq 'C1' then pnum=9
if det eq 'C2' then pnum=4

if aot eq 'P32' and det eq 'C1' then chopnum=13
if aot eq 'P32' and det eq 'C2' then chopnum=7

; ask for noise level
IF KEYWORD_SET(GRAPH) THEN BEGIN 
		  noise=10.
		  print,' Enter the noise level (in %)'
		     read, noise
		  noise = noise / 100.

; ask for the individual signal uncertainty level
		  unc_par = 10.
		  print,' Enter the individual signal uncertainty level (in %)'
		     read, unc_par
		  unc_par = unc_par / 100.
		  
; ask for unflat level
		  unflat=10.
		  print,' Enter the mean responsivity deviation per pixel (in %)'
		  print,' "Unflatfield"'
		     read, unflat
		  unflat = unflat / 100.
		  
		  IF unflat GT 0. THEN $
			unflat, mnbr, ffct, unf=unflat
		
; ask if the user wants to apply the transients to signals
; (one can change parameters later, but for now, use the default one)
		  ans_t = 'no'
		  print,' Would you like to apply transients to signals? (Yes or No)'
		     read, ans_t   

; ask if the user wants to apply the glitches to signals
; (one can change parameters later, but for now, use the default one)
	ans_g = 'no'
	print,' Would you like to simulate glitch effects on data? (Yes or No)'
		     read, ans_g   
; ask if the user for the seed parameter
	seedpar = 0L
	print,' Give a number for start of random calculations (SEED) [=0L]'
		     read, seedpar   

   if det eq 'C1' then begin
      coeff_e=36.0 
      coeff_p=0.48
      beta = 0.30
   endif ELSE IF det eq 'C2' then begin
      coeff_e=38.0
      coeff_p=0.63
      beta = 0.85
   endif
   
; update parameters according to user's input
	
	IF STRUPCASE(STRTRIM(ans_t,2)) EQ 'YES' THEN rpar.trans=1 ELSE $
						     rpar.trans=0
	IF STRUPCASE(STRTRIM(ans_g,2)) EQ 'YES' THEN rpar.glitch=1 ELSE $
						     rpar.glitch=0
	rpar.unflat = unflat * 100.
	rpar.noise = noise * 100.
	rpar.sigunc = unc_par * 100.					     
	rpar.seed = seedpar

ENDIF ELSE IF KEYWORD_SET(RPAR) THEN BEGIN

   seedpar = rpar.seed
   noise = rpar.noise / 100.
   unc_par = rpar.sigunc / 100.
   unflat = rpar.unflat / 100.
   IF unflat GT 0. THEN $
	unflat, mnbr, ffct, unf=unflat
   IF rpar.trans THEN ans_t = 'yes' ELSE ans_t = 'no'
   IF rpar.glitch THEN ans_g = 'yes' ELSE ans_g = 'no'
   coeff_e = rpar.tr_e
   coeff_p = rpar.tr_p
   IF det EQ 'C1' THEN beta = 0.3 ELSE if det EQ 'C2' THEN beta = 0.85
 
ENDIF ELSE BEGIN
	print, 'Either GRAPH or RPAR keyword must be set, returning'
	RETURN
ENDELSE   
; -------------------------------------------------------------------
; convert the MJy/sr into V/s  (refer to the signal2brightness.txt file)

obs=0.91	; an obscuration factor of the secondary mirror of the ISO telescope 
	
@phtw2jy

IF N_ELEMENTS(w2jy_c) EQ 0 THEN phtw2jy_init

c_1=w2jy_c(filt-1)	; a constant per filter retrieve using a common block 

if det eq 'C1' then filter=filt-2 
if det eq 'C2' then filter=filt-9
omega=get_omega(det,indgen(pnum),filter)	; the solid angle of the aperture per pixel

if det eq 'C1' then cap=9.e-14		; capacitance 
if det eq 'C2' then cap=1.4e-13

if det eq 'C1' then fxread,!calg+'PC1RESP_02.FITS',resp_data,resp_hdr
if det eq 'C2' then fxread,!calg+'PC2RESP_02.FITS',resp_data,resp_hdr
resp=resp_data(40,*,0)		; responsivity per pixel (default by orbital position 0.4 - Rev. > 150)

if det eq 'C1' then fxread,!calg+'PC1FLAT.FITS',ff_data,ff_hdr
if det eq 'C2' then fxread,!calg+'PC2FLAT.FITS',ff_data,ff_hdr
ff=ff_data(*,filter,0)		; flatfield per pixel

signal=fltarr(pnum,dims)	; signal in V/s
for i=0,pnum-1 do $
signal(i,*) = mnbr(i,*) * c_1 * obs * omega(i,0) * resp(i) * ff(i) * 1.e6 / cap

signal_err = signal * unc_par
;signal_err=fltarr(pnum,dims)	; signal_err in V/s
;for i=0,pnum-1 do for j=0,dims-1 do $
;signal_err(i,j) = mnbu(i,j) * c_1 * obs * omega(i,0) * resp(i) * ff(i) * 1.e6 / cap


;!p.multi=[0,1,2]
;plot,mnbr,title='Brightness in MJy/sr'
;plot,signal,title='Signal in V/s'

;window,/free
;!p.multi=[0,1,2]
;plot,mnbr,title='Brightness in MJy/sr',psym=4
;errplot,mnbr-mnbu,mnbr+mnbu
;plot,signal,title='Signal in V/s',psym=4
;errplot,signal-signal_err,signal+signal_err
;!p.multi=[0,0,0]

; --------------------------- P22 CASE ---------------------------
IF aot eq 'P22' THEN BEGIN

   no_ownt = 1
   IF KEYWORD_SET(rpar) THEN IF rpar.ownt THEN no_ownt=0
   IF no_ownt THEN BEGIN
;    itime=1.0		; integration time
;    timdif=18.0	; duration of time the detector is observing on position
;    timsweep=7.0	; duration of time the detector is moving between raster positions
;    rpchpl=128
	itime = FXPAR(phtiaap.hdr,'ITIME')
	timdif = FXPAR(phtiaap.hdr,'TIMDIF')
	timsweep = FXPAR(phtiaap.hdr,'TIMSWEEP')
	rpchpl = FXPAR(phtiaap.hdr,'RPCHPL')
   ENDIF ELSE BEGIN
    itime = rpar.inttime
    timdif = rpar.onptime
    timsweep = rpar.tsweep
    rpchpl = rpar.rpchpl
   ENDELSE
   
   jump=timdif+timsweep	; time between start of two raster positions
   
; [Warning:  The times need to be integer multiple of the integration time or this program will not be correct.]
; Also, note that these times (timdif, timdif, and timsweep) must be the same as those found at the end of the
; gen_raster.pro

   if dims eq 1 then srddims=dims*timdif/itime else srddims=(dims*(jump)-timsweep)/itime
   slop=fltarr(pnum,srddims)
   stim=fltarr(srddims)
   srdflag=intarr(pnum,srddims) 
   slun=fltarr(pnum,srddims)

   if dims eq 1 then nr_same = ROUND(timdif / itime) else nr_same = ROUND(jump / itime)	
   if dims eq 1 then nr_sweep = 0 else nr_sweep = ROUND(timsweep/itime)

   stim = INDGEN(N_ELEMENTS(stim)) * itime + itime/2 

   IF STRUPCASE(strmid(ans_t,0,1)) NE 'Y' then begin
     for i=0,pnum-1 do begin
      slop(i,0:nr_same-nr_sweep-1) = signal(i,0)
      slun(i,0:nr_same-nr_sweep-1) = signal_err(i,0)
      FOR j=1,dims-1 DO slop(i,j*nr_same-nr_sweep:(j+1)*nr_same-nr_sweep-1) = signal(i,j) 
      FOR j=1,dims-1 DO slun(i,j*nr_same-nr_sweep:(j+1)*nr_same-nr_sweep-1) = signal_err(i,j) 
     endfor
   ENDIF ELSE BEGIN
     t_arr = stim(0:nr_same-1) - stim(0)
     for i=0,pnum-1 do begin
      slop(i,0:nr_same-nr_sweep-1) = $
       transient(t_arr(0:nr_same-nr_sweep-1),0.,signal(i,0),coeff_e,coeff_p,beta)
      slun(i,0:nr_same-nr_sweep-1) = signal_err(i,0)
      FOR j=1,dims-1 DO slop(i,j*nr_same-nr_sweep:(j+1)*nr_same-nr_sweep-1) = $
      	transient(t_arr, signal(i,j-1), signal(i,j),coeff_e,coeff_p,beta)  
      FOR j=1,dims-1 DO slun(i,j*nr_same-nr_sweep:(j+1)*nr_same-nr_sweep-1) = $
      				signal_err(i,j) 
     endfor
   ENDELSE     


; add user-defined level of noise to signals.

	seed=seedpar
	ran = randomn(seed, pnum, srddims)
	slop = slop + slop * noise * ran 
;	ran = randomn(seed, pnum, srddims)
;	slun = ABS(slop * noise) + slop * noise*noise * ran

;   for i=0,pnum-1 do $
;      for j=0,srddims-timdif/itime-1,jump/itime do $
;      	for l=timdif/itime,jump/itime-1 do srdflag(i,j+l)=4

     
     for i=0,pnum-1 do $
      FOR j=1,dims-1 DO srdflag(i,j*nr_same-nr_sweep:j*nr_same-1) = 4 

; add glitches (every 7 seconds or so randomly)

IF STRUPCASE(STRMID(ans_g,0,1)) EQ 'Y' THEN BEGIN

   if max(stim) lt 20.0 then dg=fix(max(stim)) else dg=20
   gnum=fix(max(stim)/dg)
   glevel=avg(slop) * 0.2
   SEED=seedpar
   glitches=randomn(SEED,pnum,gnum)
   glitches = glitches*0.1+glevel	
   gtime=round(randomn(s,pnum,gnum)*2+dg)/itime
   gelem=intarr(pnum,gnum)	; index of elements to pick out for glitches

; integrate, find position, take only valid points and sum glitches
;   for j=1,gnum-1 do gtime(*,j) = gtime(*,j) + gtime(*,j-1) 
;   gpos = FIX(gtime / itime) < FIX(srddims)
;   FOR p=0,pnum-1 DO BEGIN
;    posis = REFORM(gpos(p,*))
;    slop(p,posis) = slop(p,posis) + glitches(p,*)
;   ENDFOR

   for p=0,pnum-1 do begin
   gelem(p,*)=gtime(p,0)
     for j=1,gnum-1 do $
        for n=1,gnum-1 do if n le j then gelem(p,j)=gelem(p,j)+gtime(p,n)        
   endfor
   
   for p=0,pnum-1 do begin
      for j=0,gnum-1 do begin
	 if gelem(p,j) lt srddims then slop(p,gelem(p,j))=slop(p,gelem(p,j))+glitches(p,j)
      endfor
   endfor
  
  
badpts=finite(slop)
ind = WHERE(badpts EQ 0, ne0)
if ne0 ge 1 and ind(0) ne -1 then begin
   slop(ind)=0.0
   srdflag(ind)=5
endif
  
ENDIF
; end glitches

rchp=long(rpchpl)

chop=fltarr(srddims)

step=fltarr(srddims)+1.0

tkey_start=4.16707e+08

rpid=bytarr(2,srddims)

     for i=0,1 do begin
      rpid(i,0:nr_same-nr_sweep-1) = phtiaap.rpid(i,0)
      FOR j=1,dims-1 DO $
         rpid(i,j*nr_same-nr_sweep:(j+1)*nr_same-nr_sweep-1) = phtiaap.rpid(i,j) 
     endfor


resi=fltarr(pnum,srddims)
   resi(*)=itime
   
nros=fltarr(pnum,srddims)
   nros(*)=32.000

ENDIF

; --------------------------- P32 CASE ---------------------------
IF aot eq 'P32' THEN BEGIN

   no_ownt = 1
   IF KEYWORD_SET(rpar) THEN IF rpar.ownt THEN no_ownt=0
   IF no_ownt THEN BEGIN
;     itime=1/8.0	; integration time
;     rpchpl=4		; number of ramps per chopper plateau
     itime = FXPAR(phtiaap.hdr,'ITIME')		; integration time
     rpchpl = FXPAR(phtiaap.hdr,'RPCHPL')	; number of ramps per chopper plateau
     n_sweeps = 3
   ENDIF ELSE BEGIN
     itime = rpar.inttime
     rpchpl = rpar.rpchpl
     onptime = rpar.onptime
     n_sweeps = FIX(onptime / chopnum / itime / rpchpl) > 3 
; this could be used in the future but it must be in accordance with gen_raster
   ENDELSE
     
   
   timdif=rpchpl*itime	; duration of time the detector is observing on position
   bwraster=chopnum*rpchpl	; number of ramps to pass between raster positions
   				; bwraster is the number of zero signals between each raster positions
   timsweep=bwraster*itime	; time during the detector movement from one raster position to the next 
   n_sweeps = 3
;   n_sweeps = (onptime / chopnum / itime / rpchpl) > 3 
   rdims=dims/chopnum/n_sweeps		; this is equal to mpts*nlines
   jump=n_sweeps*chopnum*rpchpl+bwraster	; number of time indexes between start of two raster positions
  
   srddims=dims*rpchpl+(rdims-1)*bwraster	
   slop=fltarr(pnum,srddims)
   stim=fltarr(srddims)
   srdflag=intarr(pnum,srddims) 
   slun=fltarr(pnum,srddims)

; expand AAP signals to cover the times between raster points
   s_per_rast = chopnum * n_sweeps
   exp_signal = TRANSPOSE(signal(*,0:s_per_rast-1)) 
   exp_signal_err = TRANSPOSE(signal_err(*,0:s_per_rast-1))

   exp_chop = phtiaap.cpos(0:s_per_rast-1)
   exp_rpid = TRANSPOSE(phtiaap.rpid(*,0:s_per_rast-1))
          
   FOR irast=1,rdims - 1 DO BEGIN
       beg = irast*s_per_rast
       last = (irast+1)*s_per_rast - 1
       new_arr = [TRANSPOSE(signal(*,beg:beg+(s_per_rast/n_sweeps)-1)),$
       		  TRANSPOSE(signal(*,beg:last))]
       new_err = [TRANSPOSE(signal_err(*,beg:beg+(s_per_rast/n_sweeps)-1)),$
       		  TRANSPOSE(signal_err(*,beg:last))]
       new_rpid = [TRANSPOSE(phtiaap.rpid(*,beg:beg+(s_per_rast/n_sweeps)-1)),$
       		  TRANSPOSE(phtiaap.rpid(*,beg:last))]
       new_chop = [phtiaap.cpos(beg:beg+(s_per_rast/n_sweeps)-1), $
       			phtiaap.cpos(beg:last)]
		
       
       exp_signal = [exp_signal,new_arr]
       exp_signal_err = [exp_signal_err,new_err]
       
       exp_chop = [exp_chop, new_chop]
       exp_rpid = [exp_rpid, new_rpid]
   ENDFOR
   exp_signal = TRANSPOSE(exp_signal)
   exp_signal_err = TRANSPOSE(exp_signal_err)
   exp_rpid = TRANSPOSE(exp_rpid)
; change dims   
   dims = N_ELEMENTS(exp_signal(0,*))
    
   exp_step = lindgen(dims)
   exp_step = exp_step mod chopnum
   exp_step = exp_step + 1 

   nr_same = rpchpl	
   
   stim = INDGEN(N_ELEMENTS(stim)) * itime + itime/2 

   IF STRUPCASE(strmid(ans_t,0,1)) NE 'Y' then begin
     for i=0,pnum-1 do begin
      FOR j=0,dims-1 DO slop(i,j*nr_same:(j+1)*nr_same-1) = exp_signal(i,j) 
      FOR j=0,dims-1 DO slun(i,j*nr_same:(j+1)*nr_same-1) = exp_signal_err(i,j) 
     endfor
   ENDIF ELSE BEGIN
     t_arr = stim(0:nr_same-1) - stim(0)
     for i=0,pnum-1 do begin
      slop(i,0:nr_same-1) = $
       transient(t_arr(0:nr_same-1),0.,exp_signal(i,0),coeff_e,coeff_p,beta)
      slun(i,0:nr_same-1) = exp_signal_err(i,0)
      FOR j=1,dims-1 DO slop(i,j*nr_same:(j+1)*nr_same-1) = $
      	transient(t_arr, exp_signal(i,j-1), exp_signal(i,j),coeff_e,coeff_p,beta)  
      FOR j=1,dims-1 DO slun(i,j*nr_same:(j+1)*nr_same-1) = $
      				exp_signal_err(i,j) 
     endfor
   ENDELSE     

   
   srd_per_rast = (s_per_rast + chopnum) * rpchpl
   FOR j = 1,rdims-1 DO srdflag(*,j*srd_per_rast-chopnum*rpchpl:j*srd_per_rast-1) = 4

   rchp=long(rpchpl)
   chop=fltarr(srddims)
   step=fltarr(srddims)
   tkey_start=1.0922515e+09
   rpid=bytarr(2,srddims)
   
   FOR j=0,dims-1 DO BEGIN
     chop(j*nr_same:(j+1)*nr_same-1) = exp_chop(j)
     rpid(0,j*nr_same:(j+1)*nr_same-1) = exp_rpid(0,j)
     rpid(1,j*nr_same:(j+1)*nr_same-1) = exp_rpid(1,j)
     step(j*nr_same:(j+1)*nr_same-1) = exp_step(j) 
   ENDFOR

; add user-defined level of noise to signals.

	seed=seedpar
	ran = randomn(seed, pnum, srddims)
	slop = slop + slop * noise * ran 
;	ran = randomn(seed, pnum, srddims)
;	slun = ABS(slop * noise) + slop * noise*noise * ran

; add glitches (every 7 seconds or so randomly)

IF STRUPCASE(STRMID(ans_g,0,1)) EQ 'Y' THEN BEGIN

   if max(stim) lt 20.0 then dg=fix(max(stim)) else dg=20
   gnum=fix(max(stim)/dg)
   glevel=avg(slop) * 0.2
   SEED=seedpar
   glitches=randomn(SEED,pnum,gnum)
   glitches = glitches*0.1+glevel	
   gtime=round(randomn(s,pnum,gnum)*2+dg)/itime
   gelem=intarr(pnum,gnum)	; index of elements to pick out for glitches

; integrate, find position, take only valid points and sum glitches
;   for j=1,gnum-1 do gtime(*,j) = gtime(*,j) + gtime(*,j-1) 
;   gpos = FIX(gtime / itime) < FIX(srddims)
;   FOR p=0,pnum-1 DO BEGIN
;    posis = REFORM(gpos(p,*))
;    slop(p,posis) = slop(p,posis) + glitches(p,*)
;   ENDFOR

   for p=0,pnum-1 do begin
   gelem(p,*)=gtime(p,0)
     for j=1,gnum-1 do $
        for n=1,gnum-1 do if n le j then gelem(p,j)=gelem(p,j)+gtime(p,n)        
   endfor
   
   for p=0,pnum-1 do begin
      for j=0,gnum-1 do begin
	 if gelem(p,j) lt srddims then slop(p,gelem(p,j))=slop(p,gelem(p,j))+glitches(p,j)
      endfor
   endfor
     
ENDIF
; end glitches

badpts=finite(slop)
if n_elements(where(badpts eq 0)) ge 1 and where(badpts eq 0) ne -1 then begin
   slop(where(badpts eq 0))=0.0
   srdflag(where(badpts eq 0))=5
endif
   
resi=fltarr(pnum,srddims)
   resi(*)=itime
   
nros=fltarr(pnum,srddims)
   nros(*)=float(rpchpl)

ENDIF

; -------------------------------------------------------------------

; create an SRD level structure

now=systime()
unit=strmid(now,8,2)+'-'+strmid(now,4,3)+'-'+strmid(now,20,4)+' '+strmid(now,11,8)+'.00'

ordr=1

com1=strmid(hdr(3),8,80-9)
sxdelpar, hdr, 'COMMENT'
FXADDPAR, hdr, 'COMMENT ', com1, before='ORIGIN  '
FXADDPAR, hdr, 'COMMENT ', 'PHT '+det+'00 SRD Data level', $
before='ORIGIN  '
FXADDPAR, hdr, 'PRS_DARK', 1, ' Due to simulation', before='PIA_VERS'
FXADDPAR, hdr, 'PRS_SLIN', 1, ' Due to simulation', before='PIA_VERS'
FXADDPAR, hdr, 'PRS_RIN0', 0., ' Due to simulation', before='PIA_VERS'
FXADDPAR, hdr, 'PRS_RIN1', 1., ' Due to simulation', before='PIA_VERS'
FXADDPAR, hdr, 'PRS_DARK', 1, ' Due to simulation', before='PIA_VERS'
FXADDPAR, hdr, 'PRS_SIMU', 1, ' One for simulated data', before='PIA_VERS'
; put orbital positions
orbit = FIX(STRMID(admn,4,3))
fxaddpar, hdr, 'TREFPHA1', orbit + .45, ' Orbital position set to a default value'
fxaddpar, hdr, 'TREFPHA2', orbit + .45, ' Orbital position set to a default value'
fxaddpar, hdr, 'TREFITKU', 6.1035156e-05, ' ITK unit length in seconds'

fcsn=fltarr(srddims)
fcs1=fltarr(srddims)
fcs2=fltarr(srddims)
temp=fltarr(srddims)
   temp(*)=1.77
tkey=findgen(srddims)
   tkey=tkey*d_tkey+tkey_start
     
slop=[slop,fltarr(2,srddims)]	; PIA requires 6 or 11 for the pix numbers for C200 and C100 respectively
slun=[slun,fltarr(2,srddims)]
srdflag=[srdflag,intarr(2,srddims)]
resi=[resi,fltarr(2,srddims)]
nros=[nros,fltarr(2,srddims)]

	    phtisrd = { UNIT: unit, $		; Acquisition date						
			DET: det, $		; Detector
			ADMN: admn, $		; Internal measurement name
			ORDR: ordr, $		; Position within the TDT
			DIMS: mapdims, $	; Total number of read-outs
			SLDI: long(srddims), $	; Total number of ramps
			RCHP: rchp, $		; Number of ramps / chopper plateau
			HDR: [hdr], $		; Header
			SLOP: [slop], $		; Ramp slopes
			SLUN: [slun], $		; Ramp Slopes uncertainties
			FLAG: [srdflag], $	; Flags, indicating status of data point for processing
			STIM: [stim], $		; Time within measurement
			CHOP: [chop], $		; Chopper position [arcsec]
			STEP: [step], $		; Chopper step
			FCSN: [fcsn], $		; FCS selected
			TEMP: [temp], $		; Temperature [K]
			FCS1: [fcs1], $		; FCS1 electrical power [mW]
			FCS2: [fcs2], $		; FCS2 electrical power [mw]
			TKEY: [tkey], $		; Instrument time key
			RPID: [rpid], $		; x Raster point ID
			RESI: [resi], $		; Effective reset time
			NROS: [nros]}		; Number of read-outs used in ramp
			


; -------------------------------------------------------------------

; create compact status file

if det eq 'C1' and aot eq 'P22' then $
	read_cs,'phtcs_c1_p22.fits',hdr,csn,PATH=!CALSAVE
if det eq 'C2' and aot eq 'P22' then $
	read_cs,'phtcs_c2_p22.fits',hdr,csn,PATH=!CALSAVE
if det eq 'C1' and aot eq 'P32' then $
	read_cs,'phtcs_c1_p32.fits',hdr,csn,PATH=!CALSAVE
if det eq 'C2' and aot eq 'P32' then $
	read_cs,'phtcs_c2_p32.fits',hdr,csn,PATH=!CALSAVE

@phtcs	
	
IND = WHERE(phtcs.unit EQ strmid(admn,4,8), nex)
IF nex GT 0 THEN BEGIN
   MESSAGE, /INFO, $
    'Compact Status file for that name already present, not creating a new one'
ENDIF ELSE BEGIN

 phtcs(csn).unit=strmid(admn,4,8)

 handle_value, phtcs(csn).handle, cs

 cs.c2ps(1) = filt

 cs.intt(1) = fix(7-alog(itime)/alog(2))

 cs.ndr(1) = alog(rpchpl)/alog(2)

 if filt eq 2 then cs.mpc2(1)=2130	; C100_90_RED
 if filt eq 3 then cs.mpc2(1)=1956	; C100_50_UM
 if filt eq 4 then cs.mpc2(1)=2911	; C100_60_UM
 if filt eq 5 then cs.mpc2(1)=1997	; C100_70_UM
 if filt eq 6 then cs.mpc2(1)=2078	; C100_100_UM
 if filt eq 7 then cs.mpc2(1)=1303	; C100_105_UM
 if filt eq 8 then cs.mpc2(1)=2130	; C100_90_UM
 if filt eq 9 then cs.mpc2(1)=1483	; C200_160_UM
 if filt eq 10 then cs.mpc2(1)=2204	; C200_200_UM
 if filt eq 11 then cs.mpc2(1)=1696	; C200_180_UM
 if filt eq 12 then cs.mpc2(1)=2319	; C200_135_UM
 if filt eq 13 then cs.mpc2(1)=1799	; C200_120_UM

 handle_value, phtcs(csn).handle, cs, /set

 phtics={ UNIT: phtcs(csn).unit, $
	 DIMS: phtcs(csn).dims, $
	 HANDLE: [cs] }
	 

ENDELSE

save,irph,filename=!INTDIR+admn_tmp+'.IRPH'


	
; -------------------------------------------------------------------

RETURN
END
