;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	mkripid
;
; PURPOSE:
; 	Generate RPID values for SCP internal file, add attitude keywords and
;	correct chopper position
;
; CATEGORY:
;          PIA - support
;
; CALLING SEQUENCE:
; 	mkrpid, file, npoints, nlines, dpts, dlns, rota, nmeas, ngap, $
;		addchop, pixel=pixel, inverse=inverse
;
; INPUTS:
;	file:		name of PIA internal SCP file
;	npoints:	number of raster points
;	nlines:		number of raster lines
;	dpts:		distance between raster points (arcsec)
;	dlns:		distance between raster points (arcsec)
;	rota:		rotaion angle of raster w.r.t to north
;	nmeas:		number of data points per valid measurement cycle on a
;			raster point
;	ngap:		number of data points in gap between raster points
;	addchop:	shift chopper position (arcsec)
;
; KEYWORD PARAMETERS:
;	pixel:		pixel number
;	inverse:	indicates whether raster is asumed in standard or
;			revese direction  
;	
; OUTPUTS:
;	routine read "file" and changes it according to specifiedd parameters
;
; PROCEDURES USED:
;       ASTRO LIB:
;		fxaddpar
;
; EXAMPLE:
;	Add the raster point id into the file P10001700501.ISRD with the raster
;	parameters for a 9x9 raster with 30arcsec separation. There is a
;	sequence of 300 valid data points and 100 gap points for every raster
;	point/ slew combination. The chopper is not corrected.
;
;	mkrpid, "P10001700501.ISRD", 9, 9, 30, 30, 45, 300, 100, 0
;
; MODIFICATION HISTORY:
; 	Written by:	Detlef Skalay (MPIK)
;	Oct 95 Version 3.5
;			
;-

pro mkrpid, file, npoints, nlines, dpts, dlns, rota, nmeas, ngap, $
	 addchop, pixel=pixel, inverse=inverse

if n_params(file) lt 8 then $
	message, 'syntax: mkrpid, file, npts, nlns, dpts, dlns, rota, '+ $
		'nmeas, ngap, [,addchop], pixel=pixel, inverse=inverse'

restore, file

unit=phtiscp.unit
det=phtiscp.det
admn=phtiscp.admn
ordr=phtiscp.ordr
dims=phtiscp.dims
hdr=phtiscp.hdr
time=phtiscp.time
aver=phtiscp.aver
avun=phtiscp.avun
medi=phtiscp.medi
q1me=phtiscp.q1me
q3me=phtiscp.q3me
flag=phtiscp.flag
chop=phtiscp.chop
step=phtiscp.step
fcsn=phtiscp.fcsn
temp=phtiscp.temp
fcs1=phtiscp.fcs1
fcs2=phtiscp.fcs2
tkey=phtiscp.tkey
rpid=phtiscp.rpid


; change flags and rpid

imax = n_elements(rpid(0,*))-1
nsum = nmeas+ngap
flag(*) = 1
rpid(*) = 0

if keyword_set(inverse) then inverse=1 else inverse=0

for z = 0,nlines-1 do begin
	for y = 0,npoints-1 do begin
		i1 = z*npoints*nsum + y*nsum
		
		if (z mod 2) eq inverse then $
			rpid_y = y+1 $
		else $
			rpid_y = npoints - y
		rpid_z = z+1

		if i1 le imax then begin
			i2 = (i1+nmeas-1) < imax
			i3 = (i1+nsum-1) < imax
			rpid(0,i1:i3) = rpid_y
			rpid(1,i1:i3) = rpid_z
			if keyword_set(pixel) then $
				flag(pixel-1,i1:i2) = 0 $
			else $
				flag(*,i1:i2) = 0
		endif
	endfor
endfor


; change header

fxaddpar,hdr,'ATTRNPTS',FIX(npoints),after='ATTERROR'
fxaddpar,hdr,'ATTRNLNS',FIX(nlines),after='ATTERROR'
fxaddpar,hdr,'ATTRDPTS',FLOAT(dpts),after='ATTERROR'
fxaddpar,hdr,'ATTRDLNS',FLOAT(dlns),after='ATTERROR'
fxaddpar,hdr,'ATTRROTA',FLOAT(rota),after='ATTERROR'


; change chopper positions

if n_elements(addchop) eq 1 then begin
	chop = chop + addchop
	message, /info, 'chopper position shifted: '+string(addchop)
endif


phtiscp = { $
	unit:unit, $
	det:det, $
	admn:admn, $
	ordr:ordr, $
	dims:dims, $
	hdr:hdr, $
	time:time, $
	aver:aver, $
	avun:avun, $
	medi:medi, $
	q1me:q1me, $
	q3me:q3me, $
	flag:flag, $
	chop:chop, $
	step:step, $
	fcsn:fcsn, $
	temp:temp, $
	fcs1:fcs1, $
	fcs2:fcs2, $
	tkey:tkey, $
	rpid:rpid $
}

save, phtiscp, filename=file, /xdr

return
end
