;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;
;+
; NAME:
;	Pair_deglitch
;
; PURPOSE:
;	Flag glitched points within a measurement according to the 
;	two thresholds deglitching algorithm.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	pair_deglitch, tmpdata, med_width, thresh1, thresh2, niter, $
;      			erd_disc, pixel, pdegl_flag
;
; INPUT:
;	tmpdata:	A temporary ERD type structure
;	med_width:	Width to be used for median filtering included in 
;			 the baseline correction
;	thresh1:	Threshold for primary glitch recognition
;	thresh2:	Threshold for recognition of glitch tails
;	niter:		Number of iterations for primary glitch recognition
;	erd_disc:	Flag array corresponding to tmpdata.conp(pixel,*) 
;			 (readout voltages corresponding to one pixel) with 
;			 0's (accepted points) and 1's(discarded points).
;	pixel:		Pixel to be analysed
; 
;
; OUTPUT:
;	pdegl_flag:	Array (as erd_disc) with glitch recognized positions
;			 set to 5.
;
; RESTRICTIONS:
;	tmpdata must be a valid temporary PHT-ERD type structure
;
; PROCEDURES CALLED:
;       Astronomical Library:
;               avg
;		sigma
;	PIA Libraries:
;		decode_pixf
;		get_rastersteps
;		get_rpchopstep
;		median_bb (thanks to JAP)  
; 
; SIDE EFFECTS:
;	Initializes the variable degltext in Common degl_string and fills it
;	with statistical info.	
;
; MODIFICATION HISTORY
;	Written by: Carlos GABRIEL (ESA/VILSPA-SAI)	        November 1997
;	V7.0
;	Avoiding raster/chopper combinations with too
;	 few points (CG)				           March 1998
;	V7.1
;	Using pre-discarded points correctly (CG)	          June 1998
;	V7.2
;	Preventing for no valid points case (CG)	          August 1998
;	V7.3
;	Algorithm modified, a smoothed voltage differences
;	 subtracted from the original differences + simplified 
;	 finding of secondary glitches		JAcosta           May 1999 
;	End condition corrected (CG)		CG		  May 1999
;	V7.4
;-
PRO pair_deglitch, tmpdata, med_width, thresh1, thresh2, niter, $
      erd_disc, pixel, pdegl_flag, rpchopstep, rdstat=rdstat
	
COMMON degl_string, degltext

	pdegl_flag = intarr(N_ELEMENTS(tmpdata.pixf))
	
	ipix = pixel
	degltext = STRARR(1)	
	
	
	time = tmpdata.time
	volt = REFORM(tmpdata.conp(ipix, *))
	pdegl_acc_flag = pdegl_flag
	orig_medwidth = med_width
	
; look for different rastersteps and chopper plateaus
	
	IF N_ELEMENTS(rpchopstep) EQ 0 THEN BEGIN 
	   get_rastersteps, tmpdata.rpid, raststeps
	   get_rpchopstep, raststeps, tmpdata.step, rpchopstep
	ENDIF   

	nele = N_ELEMENTS(volt)
        diff_orig = volt(1:*) - volt(0:nele-2)	
	
; decode PIXF for obtaining the READOUT status

	IF N_ELEMENTS(rdstat) EQ 0 THEN $
         decode_pixf, tmpdata.pixf, suspect, chopon, chops, rdstat


;; loop over every piece of data with same expected flux

	FOR irpch = 1, max(rpchopstep) DO BEGIN
	 
	 indrst = WHERE(rpchopstep EQ irpch, nrst)
	 
	 IF nrst LT 2 THEN BEGIN 
	   MESSAGE, /info, 'No valid points for procedure , raster/chop '+$
	   	STRTRIM(irpch,2)
	   GOTO, jump_over
	 ENDIF
	 
	 ichst = WHERE(erd_disc(indrst(0):indrst(nrst-2)) EQ 0 AND $
	    erd_disc(indrst(1):indrst(nrst-1)) EQ 0 AND $
	    rdstat(indrst(0):indrst(nrst-2)) EQ 0, nichst)
	 	 
	 IF nichst LE 2 THEN BEGIN
	   MESSAGE, /info, 'No valid points for procedure , raster/chop '+$
	   	STRTRIM(irpch,2)
	   GOTO, jump_over
	 ENDIF
	    
; obtain median filtered distribution

	 diff = diff_orig(indrst(ichst)) 

	  
	 IF nichst GT orig_medwidth THEN dmdiff = median_bb(diff, med_width) $ 
	   ELSE dmdiff = AVG(diff)
	 
; subtract median filtered distribution, the median will be 0.

	 diff_norm = diff - dmdiff
	 	 
; iterate a number of times to extract good points
         nind = N_ELEMENTS(diff_norm)
	 indtaken = indgen(nind)
	 diff_norm_ext = diff_norm
	 iter = 0 
	 
	 npts_min = 2

	 REPEAT BEGIN 
	   nind0 = nind
	   diff_norm_ext = diff_norm_ext(indtaken)
	   diff_sig = SIGMA(diff_norm_ext) 
	   abs_tol1 = diff_sig * thresh1
	   indtaken = WHERE(ABS(diff_norm_ext) LE abs_tol1, nind)
	   iter = iter + 1
	 ENDREP UNTIL iter GE niter OR nind LE npts_min OR nind EQ nind0
	    	 
; search for primary and secondary deviant points
     
        indout = WHERE(ABS(diff_norm) GT abs_tol1, nout) 
     
; loop over primary deviant points and search for points deviating more
; than second threshold
	abs_tol2 = diff_sig *  thresh2
	
	; add one element to avoid conflict with reduced dims of diff_norm
	diff_norm = [diff_norm,0.]   
				
	FOR iout = 0, nout-1 DO BEGIN
	   pbeg = indout(iout)
	   pend = pbeg
	   IF pend LT (nichst-1) THEN BEGIN   ; last point could be flagged!	
	    REPEAT BEGIN 
	     pdegl_acc_flag(indrst(ichst(pend))+1) = 5
	     s = STRING(time(indrst(ichst(pend))+1)) + '      '+    $
	         STRING(diff(pend))   + '      '     +    $
	         STRING(ABS(diff(pend))/diff_sig) +       $
	         STRING(pend-pbeg+1)
	     degltext = [degltext,s]
	     pend = pend+1
	     is_below_tol2 = ABS(diff_norm(pend)) LT abs_tol2
	   ENDREP UNTIL pend GE (nichst-1) OR is_below_tol2 
	  ENDIF 	      
	ENDFOR
	

jump_over:	  
	  
	 ENDFOR
	
	     	  
	pdegl_flag = pdegl_acc_flag 
	
	RETURN
	END
	
	   
	        
	    	
