;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+ 
; NAME: 
;       PICKMEAS_pos 
; 
; PURPOSE: 
;       This function allows the user to interactively pick (a) measurement
;	position(s) from the PIA internal buffers.  A file 
;       selection tool with a simple graphical user interface is created.
; 
; CATEGORY: 
;       PIA - graphics
; 
; CALLING SEQUENCE: 
;       Result = PICKMEAS_pos() 
; 
; KEYWORD PARAMETERS: 
; 
;       GROUP:  The widget ID of the widget that calls PICKFILE.  When this 
;               ID is specified, a death of the caller results in the death of 
;               the PICKFILE widget application. 
; 
;       FTYPE:  The structure type to be searched.  
; 
;       TITLE:  A scalar string to be used for the window title.  If it is 
;               not specified, the default title is "Select Measurement" 
; 
;       NOCONFIRM: Return immediately upon selection of a file.  The default 
;               behavior is to display the selection and then return the 
;               file when the user uses the "ok" button. 
;
;	MULTI: 	When set, several positions can be selected
; 
; OUTPUTS: 
;       PICKMEAS returns the position of the selected measurement within the 
;       common block. If no one is selected, PICKMEAS returns a zero. 
; 
; COMMON BLOCKS: 
;       PICKMEAS: COMMON block that maintains state for the widget. 
; 
; SIDE EFFECTS: 
;       This function initiates the XMANAGER if it is not already running. 
; 
; RESTRICTIONS: 
;       Only one instance of the PICKMEAS widget can be running at one time. 
; 
; PROCEDURE: 
;       Create and register the widget and then exit, returning the measurement 
;       name that was picked. 
; 
; EXAMPLE: 
; 
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtaap
;               phtcla
;               phterd
;               phtscp
;               phtsmc
;               phtsmd
;               phtspd
;               phtsrd
;       PIA Procedure(s):
;               decode_admn
;
; MODIFICATION HISTORY: 
;       Written by: Carlos GABRIEL (ESA/SAI)		July 1996 
;	Modified by CG					October 1996
;	V5.x
;	/Modal use according to IDL >= 5.0 (CG)
;	V7.3
;- 
; 
 
;------------------------------------------------------------------------------ 
;       procedure Pickmeas_ev 
;------------------------------------------------------------------------------ 
; This procedure processes the events being sent by the xMANAGER 
;------------------------------------------------------------------------------ 
PRO Pickmeas_pos_ev, event 
 

COMMON pickmeas_cmn, themeas, select, files, list, multflg, multsel

	WIDGET_CONTROL, event.id, GET_UVALUE=uvalue
		
	CASE uvalue OF 
 
	'cancel': BEGIN 
      		themeas = 0
		multsel = ""
      		WIDGET_CONTROL, event.top, /DESTROY 
    	END 
 

  	'ALL': BEGIN 
		themeas = files 
		multsel = themeas 

		decode_admn, multsel, measures
      		WIDGET_CONTROL, select, set_VALUE = measures
    	END 
 
  	'files': BEGIN 
		evt = event.index + 1 
		IF themeas(0) EQ 0 THEN themeas = evt ELSE $
					themeas = [themeas,evt]
		IF multflg THEN BEGIN
		  IF multsel(0) EQ '' THEN multsel = files(event.index) ELSE $
				multsel = [multsel, files(event.index)]
                ENDIF ELSE multsel = files(event.index)

		decode_admn, multsel, measures
      		WIDGET_CONTROL, select, set_VALUE = measures
    	END 
 
  	'ok': GOTO, checkfile 
 
  	'help': PIA_DISP_PRINT_FILE, "", $ 
                GROUP = event.top, /MODAL, $ 
                TITLE = "Measurements Selection Help", $ 
                WIDTH = 50, $ 
                HEIGHT = 12, $ 
                TEXT = [" This file selection widget lets you pick a ", $ 
                        "measurement.  Files are shown on the right.  You can", $ 
                        "select a file by clicking on it with the mouse.", $ 
                        "Pressing the 'OK' button will accept the choice", $ 
                        "and the Cancel button will not."] 
 
ENDCASE 
RETURN 

  checkfile:
    WIDGET_CONTROL, event.top, /DESTROY
    RETURN 
 
 
END ;============= end of Pickmeas event handling routine task ================ 
 
 
 
;------------------------------------------------------------------------------ 
;       procedure Pickmeas_pos 
;------------------------------------------------------------------------------ 
;  This is the actual routine that creates the widget and registers it with the 
;  Xmanager.  It also determines the operating system and sets the specific 
;  file designators for that operating system. 
;------------------------------------------------------------------------------ 
Function Pickmeas_pos, GROUP = GROUP, FTYPE = FTYPE, DTYPE = DTYPE,  $ 
              	   NOCONFIRM = NOCONFIRM, TITLE = TITLE, MULTI = MULTI 
 
@adm
@phterd
@phtsrd
@phtscp
@phtspd
@phtaap
@phtcla
@phtsmc

COMMON pickmeas_cmn, themeas, select, files, list, multflg, multsel
 
themeas = 0
multsel = ''

IF(XRegistered("Pickmeas_pos")) THEN RETURN, themeas 
 
existflag = 0 
 
IF (KEYWORD_SET(NOCONFIRM))     THEN auto_exit = 1      ELSE auto_exit = 0 
IF (N_ELEMENTS(FILE) EQ 0)      THEN FILE = "" 
 

 
        ; Widget pickfile for the rest of IDL 
 
        IF (KEYWORD_SET(DTYPE))     THEN dettyp = DTYPE ELSE dettyp = "ALL" 

        IF (KEYWORD_SET(FTYPE))     THEN ftyp = FTYPE ELSE ftyp = "ERD" 

        IF (KEYWORD_SET(TITLE))      THEN TITLE = TITLE ELSE $  
			  TITLE = "Please Select a Measurement"               
        
        IF (KEYWORD_SET(MULTI))      THEN multflg = 1 ELSE multflg = 0  
        
	CASE ftyp of 
		'ERD': BEGIN
			buffer = phterd		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'SRD': BEGIN
			buffer = phtsrd		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
                'SMC': BEGIN
                        buffer = phtsmc
                        dettyp = STRMID(dettyp,0,2)
                        IF STRMID(dettyp,1,1) EQ '#' OR $
                           STRMID(dettyp,1,1) EQ '%' THEN BEGIN
                         detty = STRMID(dettyp,0,1)
                         indices=where(STRMID(buffer.admn,0,1) eq detty,count)
                        ENDIF ELSE $
                        indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)
                END
		'SCP': BEGIN   
			buffer = phtscp		
			dettyp = STRMID(dettyp,0,2)
			IF STRMID(dettyp,1,1) EQ '#' OR $
			   STRMID(dettyp,1,1) EQ '%' THEN BEGIN
			 detty = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq detty,count)
			ENDIF ELSE $  
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		'SPD': BEGIN   
			buffer = phtspd		
			IF dettyp EQ 'SL' or dettyp EQ 'SS' THEN BEGIN
			 dettyp = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq dettyp,count) 
			ENDIF ELSE BEGIN
			 dettyp = STRMID(dettyp,0,2)
        		 indices=where(STRMID(buffer.admn,0,2) eq dettyp,count) 
			ENDELSE
                END
		'AAP': BEGIN   
			buffer = phtaap		
			IF dettyp EQ 'SL' or dettyp EQ 'SS' THEN BEGIN
			 dettyp = STRMID(dettyp,0,1)
        		 indices=where(STRMID(buffer.admn,0,1) eq dettyp,count) 
			ENDIF ELSE BEGIN
			 dettyp = STRMID(dettyp,0,2)
        		 indices=where(STRMID(buffer.admn,0,2) eq dettyp,count) 
			ENDELSE
                END
		'CLA': BEGIN   
			buffer = phtcla		
			dettyp = STRMID(dettyp,0,2)
        		indices=where(STRMID(buffer.admn,0,2) eq dettyp,count)  
                END
		ELSE: MESSAGE, /INFO, 'Unknown buffer type'
        ENDCASE

	
	
	IF count EQ 0 then files = '' ELSE files = buffer(indices).admn  
        if dettyp eq 'AL' AND N_ELEMENTS(buffer.admn) GT 1 then $
		files = buffer(1:*).admn 

	IF not KEYWORD_SET(group) THEN group = !pia_top_wid
        Pickmeasbase =  WIDGET_BASE(TITLE = TITLE, /COLUMN, /MODAL, Group=group) 

        id_0 =	WIDGET_BASE(Pickmeasbase, /ROW) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Data Type:") 
	id_1 =	WIDGET_TEXT(id_0, VALUE = ftyp)

        id_0 =  WIDGET_BASE(Pickmeasbase, /ROW) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Detector:") 
        id_1 =  WIDGET_TEXT(id_0, VALUE = dettyp, XS = 10, UVALUE= 'dtyp') 

        id_0 =  WIDGET_BASE(Pickmeasbase, /COLUMN) 
        id_1 =  WIDGET_LABEL(id_0, VALUE = "Measurements") 

	numero = N_ELEMENTS(files)
	strlist = strarr(numero)
        FOR i=0,numero-1 DO BEGIN 
		decode_admn,files(i),strl
		strlist(i) = strl
	ENDFOR
        list =  WIDGET_LIST(id_0, VALUE = strlist, YSIZE = 8, $ 
                        UVALUE = 'files') 

        id_0 =  WIDGET_LABEL(Pickmeasbase, VALUE = "Selection:") 
	IF multflg THEN $ 
         select =  WIDGET_TEXT(Pickmeasbase, VALUE = '  ', XS = 20, YS=4, $
			/scroll) ELSE $ 
         select =  WIDGET_TEXT(Pickmeasbase, VALUE = '  ', XS = 20)  

	IF KEYWORD_SET(multi) THEN BEGIN
         id_0 =  WIDGET_BASE(Pickmeasbase, SPACE = 15, /ROW) 
         id_1 =  WIDGET_BUTTON(id_0, VALUE = "  Select all  ", UVALUE = 'ALL')
        ENDIF 

        id_0 =  WIDGET_BASE(Pickmeasbase, SPACE = 15, /ROW) 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = "   Ok   ", UVALUE = 'ok') 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = " Cancel ", UVALUE = 'cancel') 
        id_1 =  WIDGET_BUTTON(id_0, VALUE = "  Help  ", UVALUE = 'help') 
 

        WIDGET_CONTROL, Pickmeasbase, /REALIZE 
 
        xMANAGER,No_BLOCK=1, "Pickmeas_pos", Pickmeasbase, $
        	EVENT_HANDLER = "Pickmeas_pos_ev", $ 
                GROUP_LEADER = GROUP

RETURN, themeas 
 
END ;====================== end of Pickfile routine =========================== 
 
