;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO PROCESS_SCP, SCPDATA, HDR, RESP, SPDPS, SPDSTR, NO_SPD = NO_SPD, $
  	RESPINT=RESPINT, BASDRIFT=BASDRIFT, SPRF=SPRF, SPRU=SPRU
;+
; NAME:
;	PROCESS_SCP
; PURPOSE:
;	Convert PHT data from Signal per Chopper Plateau (SCP) to In-Band-Power 
;	per Chopper Plateau (SPD), using default or actual responses to do
;	flux calibration. It accomodates the data into the structure PHTSPD.
; CATEGORY:
;	PIA - processing
;                
; CALLING SEQUENCE:
;	PROCESS_SCP, SCPDATA, HDR, RESP, SPDPS, SPDSTR, NO_SPD = NO_SPD, 
;		RESPINT=RESPINT 
;
; INPUT PARAMETERS:
;	SCPDATA	: Structure containing the SCP data to be processed
;	SCPHDR  : (Updated) Header element corresponding to SCPDATA
;	RESP	: Responsivity choice flag - 0=default, 1=calculated, 
;				2=absolute photometry, 3=interpolation 
; OPTIONAL KEYWORD :
;	NO_SPD	: No structure added to the dynamical structure phtspd
;	RESPINT	: Structure to be used for time interpolation (must be
;		  set for RESP=3)
;	SPRF	: Spectral response function (for PHT-S) to be used
;
; OPTIONAL OUTPUT:
;	SPDPS	: Position of the produced SPD data within the PHTSPD structure 
;		  (can be used to check if SPD data has been produced)
;	SPDSTR	: Temporary data structure (SPD level)
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtcap
;               phtcs
;               phtresact
;               phtresdef
;               phtscp
;               phtspd
;       PIA Procedure(s):
;               cp_disc
;               der_spdhdr
;               get_wheels
;               phtcap_init
;               pia_respons_abs
;               read_resp
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel (ESA/ESTEC-SAI)
;	Changed (last) by CG				October 1996
;	V5.x
;	External Flatfield correction + avoid writing twice Resp in hdr by 
;	PHT-S and back into SCPHDR (CG)			April 1997
;	V6.3
;	Baseline drift correction included &
;	 quoting responsivity origin in HDR (CG) 	July 1997
;	V6.4
;	Default responsivity used orbit dependent (CG)	January 1998
;	V7.0
;	Writing in hdr actual or default respons &	 
;	 no use of syst.unc. by def. resp. (CG)		March 1998
;	V7.1
;	FCS measurements can be flux calibrated, 
;	 except for absolute photometry mode (JA)	June 1998
;	Using double precision for avoiding underflow (CG) Feb. 1999
;	V7.4
;	Filter to Filter for P added (CG)		July 1999
;	V7.9
;	Writing P single signal values into header
;	(provision for PHT04) (CG)			May 2002
;	V10.0
;-
;
; Needed common blocks :
@adm
@phtcs
@phtcap
@phtscp
@phtspd
@phtresdef
@phtresact
@phtw2jy
@phtffield
@phtftof
@phtfilter

COMMON respabso, respabs, resuabs, respflag
	ON_ERROR, 2                                          

;
; Check the number of parameters
;
	IF N_PARAMS() LT 3 THEN MESSAGE, $
	   'Syntax: PROCESS_SCP, scpdata, scphdr, resp, [SPDPOS]'

	SCPHDR = HDR
;  read default or actual responsivities if necessary
;	IF resp EQ 0 THEN BEGIN
;	   IF N_ELEMENTS(P1RESP) EQ 0 THEN read_resp 
;	ENDIF 

; If phtffield not yet filled, read it from the FITS file
	IF n_elements(c1flatf) EQ 0 THEN read_flatfield

;  Read Capacities and/or FCS powers if necessary
        IF N_ELEMENTS(slcap) EQ 0 THEN phtcap_init

;  get wheel positions + ...
	get_wheels, scpdata.admn, wheels, meast, dr, intt

; Take care of the different array dimensions for different detectors, etc
; Special case of SS/SL --> two files : caseSL
	caseSL = 0

	resp_origin = ['','','']
	CASE scpdata.det OF
; P1
	  'P1': BEGIN
		  CASE resp OF
		    0: BEGIN
		    	def_resp = default_response(scpdata)
			respon = def_resp(*,*,0)
			respun = def_resp(*,*,1)
			chpl = indgen(N_ELEMENTS(def_resp(0,*,0)))  
		    END
		    1: BEGIN
			IF N_ELEMENTS(p1respact) EQ 0 THEN GOTO, NORESPACT
			respon = p1respact
			respun = p1respunc
			resp_origin = [respact_origin.p1n, respact_origin.p1f, $
			    respact_origin.p1m]
			chpl = 0  
		    END
		    2: BEGIN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
		    	IF not KEYWORD_SET(RESPINT) THEN BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			resp_origin = [respact_origin.p1n, respact_origin.p1f, $
			    respact_origin.p1m]

			respon = FLTARR(1,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
		    	respon(0,*) = respint.resp1 + $
		    		(respint.resp2 - respint.resp1) / (tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	respun(0,*) = respint.respun1 + $
		    		(respint.respun2 - respint.respun1) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE
		  capac = p1cap
		  pixnr = 1
		  filt = wheels(2)
		  aper = wheels(1)
		  polz = wheels(0)
; Add signal to header if single one
		  IF N_ELEMENTS(scpdata.aver) EQ 1 THEN BEGIN
		   FXADDPAR, scphdr, 'PRC_SIGN', scpdata.aver, $
		    " Single corrected measurement signal", BEFORE='XTENSION'
		   FXADDPAR, scphdr, 'PRC_USIG', scpdata.avun, $
		    " Single corrected signal uncertainty", BEFORE='XTENSION'
		  ENDIF  
		  IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof(filt-1)
			 flatu = ppftofu(filt-1)
		  ENDIF ELSE BEGIN
			 flatf = ppftof(filt-1) * 0. + 1.
			 flatu = ppftof(filt-1) * 0.
		  ENDELSE
	        END
; P2
	  'P2': BEGIN
		  CASE resp OF
		    0: BEGIN
		    	def_resp = default_response(scpdata)
			respon = def_resp(*,*,0)
			respun = def_resp(*,*,1)
			chpl = indgen(N_ELEMENTS(def_resp(0,*,0)))  
		    END
		    1: BEGIN
			IF N_ELEMENTS(p2respact) EQ 0 THEN GOTO, NORESPACT
			respon = p2respact
			respun = p2respunc
			resp_origin = [respact_origin.p2n, respact_origin.p2f, $
			    respact_origin.p2m]
			chpl = 0  
		    END
		    2: BEGIN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
		    	IF not KEYWORD_SET(RESPINT) THEN  BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			resp_origin = [respact_origin.p2n, respact_origin.p2f, $
			    respact_origin.p2m]
			respon = FLTARR(1,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
		    	respon(0,*) = respint.resp1 + $
		    		(respint.resp2 - respint.resp1) / (tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	respun(0,*) = respint.respun1 + $
		    		(respint.respun2 - respint.respun1) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE
		  capac = p2cap
		  pixnr = 1
		  filt = wheels(2)
		  aper = wheels(1)
		  polz = wheels(0)
; Add signal to header if single one
		  IF N_ELEMENTS(scpdata.aver) EQ 1 THEN BEGIN
		   FXADDPAR, scphdr, 'PRC_SIGN', scpdata.aver, $
		    " Single corrected measurement signal", BEFORE='XTENSION'
		   FXADDPAR, scphdr, 'PRC_USIG', scpdata.avun, $
		    " Single corrected signal uncertainty", BEFORE='XTENSION'
		  ENDIF  
		  IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof(filt-1)
			 flatu = ppftofu(filt-1)
		  ENDIF ELSE BEGIN
			 flatf = ppftof(filt-1) * 0. + 1.
			 flatu = ppftof(filt-1) * 0.
		  ENDELSE
	        END
; P3
	  'P3': BEGIN
		  CASE resp OF
		    0: BEGIN
		    	def_resp = default_response(scpdata)
			respon = def_resp(*,*,0)
			respun = def_resp(*,*,1)
			chpl = indgen(N_ELEMENTS(def_resp(0,*,0)))  
		    END
		    1: BEGIN
			IF N_ELEMENTS(p3respact) EQ 0 THEN GOTO, NORESPACT
			respon = p3respact
			respun = p3respunc
			resp_origin = [respact_origin.p3n, respact_origin.p3f, $
			    respact_origin.p3m]
			chpl = 0  
		    END
		    2: BEGIN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
		    	IF not KEYWORD_SET(RESPINT) THEN BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			resp_origin = [respact_origin.p3n, respact_origin.p3f, $
			    respact_origin.p3m]
			respon = FLTARR(1,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
		    	respon(0,*) = respint.resp1 + $
		    		(respint.resp2 - respint.resp1) / (tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	respun(0,*) = respint.respun1 + $
		    		(respint.respun2 - respint.respun1) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE
		  capac = p3cap
		  pixnr = 1
		  filt = wheels(2)
		  aper = wheels(1)
		  polz = wheels(0)
; Add signal to header if single one
		  IF N_ELEMENTS(scpdata.aver) EQ 1 THEN BEGIN
		   FXADDPAR, scphdr, 'PRC_SIGN', scpdata.aver, $
		    " Single corrected measurement signal", BEFORE='XTENSION'
		   FXADDPAR, scphdr, 'PRC_USIG', scpdata.avun, $
		    " Single corrected signal uncertainty", BEFORE='XTENSION'
		  ENDIF  
		  IF !PIA_FFUSE THEN BEGIN
			 flatf = ppftof(filt-1)
			 flatu = ppftofu(filt-1)
		  ENDIF ELSE BEGIN
			 flatf = ppftof(filt-1) * 0. + 1.
			 flatu = ppftof(filt-1) * 0.
		  ENDELSE
	        END
; C100
	  'C1': BEGIN
		  CASE resp OF
		    0: BEGIN
		    	def_resp = default_response(scpdata)
			respon = def_resp(*,*,0)
			respun = def_resp(*,*,1)
			chpl = indgen(N_ELEMENTS(def_resp(0,*,0)))  
		    END
		    1: BEGIN
			IF N_ELEMENTS(c1respact) EQ 0 THEN GOTO, NORESPACT
			respon = c1respact
			respun = c1respunc
			chpl = 0  
			resp_origin = [respact_origin.c1n, respact_origin.c1f, $
			    respact_origin.c1m]
		    END
		    2: BEGIN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
		    	IF not KEYWORD_SET(RESPINT) THEN BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			resp_origin = [respact_origin.c1n, respact_origin.c1f, $
			    respact_origin.c1m]
			respon = FLTARR(9,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
			FOR Ipix=0,8 DO BEGIN
		    	  respon(ipix,*) = respint.resp1(ipix) + $
		    		(respint.resp2(ipix) - respint.resp1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	  respun(ipix,*) = respint.respun1(ipix) + $
		    	  	(respint.respun2(ipix) - $
		    		 respint.respun1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1) 
			ENDFOR
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE

		  capac = c1cap
		  pixnr = 9
		  filt = wheels(1)
		  aper = wheels(2)
		  polz = wheels(0)
		  IF !PIA_FFUSE THEN BEGIN
		   flatf = c1flatf(*,filt-2)
		   flatu = c1flatu(*,filt-2)
		  ENDIF ELSE BEGIN
		   flatf = c1flatf(*,filt-2)*0.+1.
		   flatu = c1flatu(*,filt-2)*0.
		  ENDELSE
	        END
; C200
	  'C2': BEGIN
		  CASE resp OF
		    0: BEGIN
		    	def_resp = default_response(scpdata)
			respon = def_resp(*,*,0)
			respun = def_resp(*,*,1)
			chpl = indgen(N_ELEMENTS(def_resp(0,*,0)))  
		    END
		    1: BEGIN
			IF N_ELEMENTS(c2respact) EQ 0 THEN GOTO, NORESPACT
			respon = c2respact
			respun = c2respunc
			chpl = 0  
			resp_origin = [respact_origin.c2n, respact_origin.c2f, $
			    respact_origin.c2m]
		    END
		    2: BEGIN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
		    	IF not KEYWORD_SET(RESPINT) THEN BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			resp_origin = [respact_origin.c2n, respact_origin.c2f, $
			    respact_origin.c2m]

			respon = FLTARR(4,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
			FOR Ipix=0,3 DO BEGIN
		    	  respon(ipix,*) = respint.resp1(ipix) + $
		    		(respint.resp2(ipix) - respint.resp1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	  respun(ipix,*) = respint.respun1(ipix) + $
		    	  	(respint.respun2(ipix) - $
		    		 respint.respun1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1) 
			ENDFOR
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE
		  capac = c2cap
		  pixnr = 4
		  filt = wheels(1)
		  aper = wheels(2)
		  polz = wheels(0)
		  IF !PIA_FFUSE THEN BEGIN
		   flatf = c2flatf(*,filt-9)
		   flatu = c2flatu(*,filt-9)
		  ENDIF ELSE BEGIN
		   flatf = c2flatf(*,filt-9)*0. + 1.
		   flatu = c2flatu(*,filt-9)*0.
		  ENDELSE	
	        END
; SS/SL
	  'SL': BEGIN
		  caseSL = 1

;		  IF N_ELEMENTS(slresp) EQ 0 THEN read_resp

		  CASE resp OF
		    0: BEGIN
		    	IF KEYWORD_SET(sprf) and KEYWORD_SET(spru) THEN BEGIN
			  respon = sprf / w2jy_s
			  respun = spru / w2jy_s
			  chpl = 0
			ENDIF ELSE BEGIN
			 respon = v2jy_s / w2jy_s
			 respun = v2jyunc_s / w2jy_s  
			 chpl = 0
			ENDELSE  
		    END
		    1: BEGIN
			IF N_ELEMENTS(slrespact) EQ 0 THEN BEGIN
;				slrespact = v2jy_s / w2jy_s
;				slrespunc = 0.3 * slrespact
				MESSAGE, /INFO, 'Default responses used'
			ENDIF
			respon = v2jy_s / w2jy_s
			respun = v2jyunc_s / w2jy_s
			chpl = 0  
		    END
		    2: BEGIN
			MESSAGE, /INFO, 'I should never come here'
			RETURN
			pia_respons_abs, scpdata, scphdr
			indok = WHERE(respflag NE 2, n)
			IF n EQ 0 THEN GOTO, NORESPACT
			chpl = indgen(N_ELEMENTS(respabs(0,*)))  
			respon = respabs
			respun = resuabs
		    END
		    3: BEGIN
			MESSAGE, /INFO, 'I should never come here'
			RETURN
		    	IF not KEYWORD_SET(RESPINT) THEN BEGIN
		    		MESSAGE, /INFO, 'Interpolation not possible'
		    		RETURN
		    	ENDIF
			tfcs2 = unit2juldate(respint.time2) * 24. * 3600.
			tfcs1 = unit2juldate(respint.time1) * 24. * 3600.
			tinmeas = unit2juldate(scpdata.unit) * 24. * 3600.

			respon = FLTARR(128,N_ELEMENTS(scpdata.time))
			respun = respon
			chpl = indgen(N_ELEMENTS(respon(0,*)))
			FOR Ipix=0,127 DO BEGIN
		    	  respon(ipix,*) = respint.resp1(ipix) + $
		    		(respint.resp2(ipix) - respint.resp1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1)
		    	  respun(ipix,*) = $
		    	  	(respint.resp2(ipix)+respint.respun2(ipix) - $
		    		respint.resp1(ipix)-respint.respun1(ipix)) / $
		    		(tfcs2-tfcs1) * $
		    		(tinmeas+scpdata.time-tfcs1) + $
		    		respint.resp1+respint.respun1
			ENDFOR
		    	END
		    ELSE: BEGIN
			MESSAGE, /INFO, 'Not allowed input parameter'
			RETURN			
		    END	
		  ENDCASE

		  capac = sscap
		  capacsl = slcap
		  pixnr = 128
		  filt = wheels(1)
		  aper = wheels(2)
		  polz = wheels(0)
		  flatf = fltarr(128) + 1.
		  flatu = fltarr(128)
	        END
	  ELSE: BEGIN
	  	  MESSAGE, /INFO, 'Detector not recognized, returning'
	  	  RETURN
	  	END
	  ENDCASE

;  Provide for pseudo respflag if not absolute photometry
        CASE resp of
	  2: flag = respflag
	  ELSE: flag = intarr(pixnr)
	ENDCASE 	

;  Perform Flux Calibration - Ignore FCS signals (absolute photometry case)
	IF resp eq 2 THEN indfov = WHERE(scpdata.fcsn EQ 0)  $ 
	  ELSE indfov = INDGEN(N_ELEMENTS(scpdata.fcsn))

	inba = scpdata.aver(0:pixnr-1,indfov) * capac 
	inbm = scpdata.medi(0:pixnr-1,indfov) * capac 

;  If SS/SL overwrite partly inba and inbm because of different capacitances
        IF scpdata.det EQ 'SL' THEN BEGIN
	  inba(64:127,indfov) = scpdata.aver(64:127,indfov) * capacsl
	  inbm(64:127,indfov) = scpdata.medi(64:127,indfov) * capacsl
        ENDIF

;  Use flgfov to flag source chopper plateaux - Flag as bad also 
;  those ch.pl. which cannot be calibrated in abs.phot. case
 
;  Use flagarr for processing but flagscp for further flagging
	flagscp = scpdata.flag(0:pixnr-1,indfov)

	encode_piaflag, 'SCP',  'STABIL1', flagscp, flagscp, /reset
	encode_piaflag, 'SCP',  'STABIL2', flagscp, flagscp, /reset
	encode_piaflag, 'SCP',  'DRIFTMOD', flagscp, flagscp, /reset
 
	cp_disc, scpdata, flagarr
        flgfov = flagarr(0:pixnr-1,indfov)

; Abs. Photometry case

	IF resp EQ 2 THEN BEGIN

;  If last point of flagarray is on an odd position ignore it to have
;  a 1-to-1 correspondence in the flag
	  nele = N_ELEMENTS(flag(0,*)) / 2 * 2
	  flag = flag(*,0:nele-1)

	  indnacc = WHERE(flag GT 1, n2)
	  IF n2 GT 0 THEN BEGIN
		respon(indnacc) = 1.0
		respun(indnacc) = 1.0
	  	flgfov(indnacc) = 1
          ENDIF
	  
	  PRINT, 'Warning ... Old P05/P25 mode ... Problems to be expected '

        ENDIF  

;  Set all discarded points to 1.0 (to avoid divisions by 0) 

	indnacc = WHERE(flgfov GT 1, n1)
	IF n1 GT 0 THEN BEGIN
		inba(indnacc) = 1.0
		inbm(indnacc) = 1.0
        ENDIF

	mnpw = inba * 0.
	mnpu = mnpw
	mdpw = mnpw
	q1pw = mnpw
	q3pw = mnpw

	
	FOR I = 0, pixnr-1 DO BEGIN

;  Do not use syst. unc. on def.resp -. put it into hdr - not for pht-s
	   IF resp EQ 0 AND pixnr LT 10 THEN BEGIN
	     rel_unc = respun(i,0) / respon(i,0)
	     respun(i,*) = respun(i,*) * 0.
	     pixstr = STRTRIM(STRING(i+1),2)
	     keyw = 'PRC_R_S' + pixstr
	     comm =    ' Rel. syst. unc. by default' +$
	    	          ' response: Pixel '+pixstr
	     IF i EQ 0 THEN                                               $
	         FXADDPAR, scphdr, keyw, rel_unc, comm, BEFORE='XTENSION' $
	     ELSE FXADDPAR, scphdr, keyw, rel_unc, comm, AFTER=keyw_b
	     keyw_b = keyw 
	   ENDIF  

;  Do not use syst. unc. on "flat_field" -. put it into hdr - not for pht-s
	   IF pixnr LT 10 THEN BEGIN
	     rel_unc = flatu(i) / flatf(i)
	     pixstr = STRTRIM(STRING(i+1),2)
	     keyw = 'PRC_F_S' + pixstr
	     comm =    ' Rel.syst.unc. by Filter2Filter:' +$
	    	          ' Pixel '+pixstr
             IF i EQ 0 THEN                                               $
	         FXADDPAR, scphdr, keyw, rel_unc, comm, BEFORE='XTENSION' $
	     ELSE FXADDPAR, scphdr, keyw, rel_unc, comm, AFTER=keyw_b
	     keyw_b = keyw 
	   ENDIF  
	     
;  Capac different for SS and SL
	   IF I LT 64 THEN capaci = capac ELSE capaci = capacsl

;  Compute means and uncertainties - Divide by external flatfield factor 
	   mnpw(i,*) = inba(i,*) / respon(i,chpl) / flatf(i)
	   mnpu(i,*) = abs(mnpw(i,*)) * $
		sqrt( (scpdata.avun(i,indfov)*capaci)^2 / (1.D*inba(i,*))^2 + $
	(respun(i,chpl)*respun(i,chpl)) / (respon(i,chpl)*respon(i,chpl)))
	; + $
	;((flatu(i)/flatf(i))^2) )   	

;  Compute medians and quartiles 
	   mdpw(i,*) = inbm(i,*) / respon(i,chpl) / flatf(i)
	   q1pw(i,*) = scpdata.q1me(i,indfov)*capaci / respon(i,chpl) / flatf(i)
	   q3pw(i,*) = scpdata.q3me(i,indfov)*capaci / respon(i,chpl) / flatf(i)
	ENDFOR

; Correct with baseline drift if required
	IF KEYWORD_SET(basdrift) THEN BEGIN
	   ndif = WHERE(basdrift NE 1., nd)
	  
	   siz_mnp = size(mnpw)
	   siz_bas = size(basdrift)
	   IF nd EQ 0. THEN $
	      PRINT, 'No baseline drift correction at all' $
	   ELSE IF siz_mnp(0) NE siz_bas(0) OR $
	      siz_mnp(1) NE siz_bas(1) OR $
	      siz_mnp(2) NE siz_bas(2) THEN $
	      PRINT, 'Dimensions of arrays and baseline drift do not match' $
	   ELSE BEGIN
	    mnpw = mnpw / basdrift
	    mnpu = mnpu / basdrift
	    mdpw = mdpw / basdrift
	    q1pw = q1pw / basdrift
	    q3pw = q3pw / basdrift
	    FXADDPAR, scphdr, 'PRC_BDRI', 1, $
	      ' Baseline drift correction applied', BEFORE='XTENSION'
	   ENDELSE
	 ENDIF

;  Set to 0 all bad flagged values
	indnacc = WHERE(flgfov GT 1, n)
	IF n GT 0 THEN BEGIN 
	   mnpw(indnacc) = 0.
	   mnpu(indnacc) = 0.
	   mdpw(indnacc) = 0.
	   q1pw(indnacc) = 0.
	   q3pw(indnacc) = 0.
	ENDIF

	IF caseSL EQ 1 THEN BEGIN
	   mnpwss = mnpw(0:63,*) 
	   mnpuss = mnpu(0:63,*) 
	   mdpwss = mdpw(0:63,*) 
	   q1pwss = q1pw(0:63,*) 
	   q3pwss = q3pw(0:63,*) 
	   flgfovss = flgfov(0:63,*)
	   mnpw = mnpw(64:127,*) 
	   mnpu = mnpu(64:127,*) 
	   mdpw = mdpw(64:127,*) 
	   q1pw = q1pw(64:127,*) 
	   q3pw = q3pw(64:127,*) 
	   flgfov = flgfov(64:127,*)
        ENDIF

	DET = SCPDATA.DET

; Accept only data corresponding to valid raster positions
; (i.e. NOT EQ 0)
	indrpok = WHERE(scpdata.RPID(0,indfov) NE 0b AND $
			scpdata.RPID(1,indfov) NE 0b, nok)
	IF nok EQ 0 THEN BEGIN
		MESSAGE, /info, 'Only wrong raster positions, returning'
		RETURN
	ENDIF

;  Accomodate data into the spd structure and the spd handles after
;  creating the temporary ones                  
	SCPADMN = SCPDATA.ADMN
	rep_fill = -1
FILLSTR:
	rep_fill = rep_fill + 1
	PHTSPDT = {PHTSPD_TYPE}
	FOR ihan=SPD_FIRST_HANDLE, SPD_LAST_HANDLE DO $
	   PHTSPDT.(ihan) = HANDLE_CREATE()

;  Change the header according to the selection criteriae 
;	jump over if PHT-SS (info already contained)

	ndim = N_ELEMENTS(respon(0,*))
	IF rep_fill THEN GOTO, EXIT_EXT_hdr 
	IF resp EQ 0 THEN act_or_def = 'default' ELSE act_or_def = 'actual'
	FOR ipix=0,pixnr-1 DO BEGIN 
	    IF ndim GT 1 THEN BEGIN
	    	val = AVG(respon(ipix,*))
	    	rel_val = AVG(respun(ipix,*)/respon(ipix,*))
		pixstr = STRTRIM(STRING(ipix+1),2)
		keyw = 'PRC_R' + pixstr
	    	comm =    ' Average used '+act_or_def+$
	    	          ' response [A/W]: Pixel '+pixstr
	    ENDIF ELSE BEGIN
		val = respon(ipix)
	    	rel_val = respun(ipix)/respon(ipix)
	    	pixstr = STRTRIM(STRING(ipix+1),2)
		keyw = 'PRC_R' + pixstr
		comm = ' Used '+ act_or_def + ' response [A/W]: Pixel '+pixstr	
	    ENDELSE
	    FXADDPAR, scphdr, keyw, val, comm, BEFORE='XTENSION'
	ENDFOR

	IF resp_origin(0) NE '' THEN BEGIN
		FXADDPAR, scphdr, 'PRC_RMEA', resp_origin(0), $
	      ' Responsivity calculation measurement used', BEFORE='XTENSION' 
		FXADDPAR, scphdr, 'PRC_RFIL', resp_origin(1), $
	      ' Responsivity calculation measurement filter', BEFORE='XTENSION'
		FXADDPAR, scphdr, 'PRC_RCAL', resp_origin(2), $
	      ' Responsivity calculation calibrator', BEFORE='XTENSION'
	ENDIF 
EXIT_EXT_hdr:

	IF N_ELEMENTS(scphdr) EQ 0 THEN spdhdr = scpdata.hdr ELSE $
		der_spdhdr, scphdr, spdhdr
	HANDLE_VALUE, PHTSPDT.HDR, spdhdr, /SET
; 	  
	PHTSPDT.UNIT = scpdata.UNIT
	PHTSPDT.DET  = DET
	PHTSPDT.ADMN = SCPADMN
	PHTSPDT.ORDR = SCPDATA.ORDR
	PHTSPDT.DIMS = N_ELEMENTS(mnpw(0,indrpok))

	rpid = scpdata.rpid(*,indfov(indrpok))
	tkey = scpdata.tkey(indfov(indrpok))
	time = scpdata.time(indfov(indrpok))
	step = scpdata.step(indfov(indrpok))
	chop = scpdata.chop(indfov(indrpok))

	HANDLE_VALUE, PHTSPDT.RPID, RPID, /SET
	HANDLE_VALUE, PHTSPDT.TKEY, TKEY , /SET
	HANDLE_VALUE, PHTSPDT.TIME, TIME , /SET
	HANDLE_VALUE, PHTSPDT.FLAG, FLAGSCP, /SET


; just provisorily
	HANDLE_VALUE, PHTSPDT.PLEN, FLGFOV(*,indrpok), /SET
	HANDLE_VALUE, PHTSPDT.CPOS, CHOP, /SET

	mfilt = intarr(phtspdt.dims) + filt
	maper = intarr(phtspdt.dims) + aper
	mpolz = intarr(phtspdt.dims) + polz
	mmeas = intarr(phtspdt.dims) + meast
	mdr = intarr(phtspdt.dims) + dr
	mdwel = intt * mdr 

	HANDLE_VALUE, PHTSPDT.FILT, mfilt , /SET
	HANDLE_VALUE, PHTSPDT.APER, maper , /SET
	HANDLE_VALUE, PHTSPDT.POLZ, mpolz , /SET
	HANDLE_VALUE, PHTSPDT.MEAS, mmeas , /SET
	HANDLE_VALUE, PHTSPDT.NDRS, mdr , /SET
	HANDLE_VALUE, PHTSPDT.CSTP, step  , /SET
	HANDLE_VALUE, PHTSPDT.DWEL, mdwel , /SET

	HANDLE_VALUE, PHTSPDT.MNPW, mnpw(*,indrpok), /SET 
	HANDLE_VALUE, PHTSPDT.MNPU, mnpu(*,indrpok), /SET
	HANDLE_VALUE, PHTSPDT.MDPW, mdpw(*,indrpok), /SET
	HANDLE_VALUE, PHTSPDT.Q1PW, q1pw(*,indrpok), /SET
	HANDLE_VALUE, PHTSPDT.Q3PW, q3pw(*,indrpok), /SET

; Update the PHTSRD structure using the temporary structure
	IF NOT KEYWORD_SET(NO_SPD) THEN BEGIN
	 	PHTSPD = [PHTSPD, PHTSPDT]
; Update the PHTSPD value in the administration file using the index
		SPDPS = N_ELEMENTS(PHTSPD) - 1
		INDADM = WHERE (ADM.NAME EQ PHTSPDT.ADMN, NN)
; SS-SPD has no position in adm file
		IF NN GT 0 THEN ADM(INDADM).PHTSPD = SPDPS
	ENDIF
	IF N_ELEMENTS(SPDSTR) EQ 0 THEN $
		SPDSTR = PHTSPDT ELSE $
		SPDSTR = [SPDSTR,PHTSPDT]

; Now fill SS buffer if S data
	IF caseSL eq 1 then BEGIN
	   caseSL = 0 
	   nam = scpdata.admn
	   strput, nam, 'S', 1
	   scpadmn = nam
	   det = 'SS'
	   mnpw = mnpwss 
	   mnpu = mnpuss 
	   mdpw = mdpwss 
	   q1pw = q1pwss 
	   q3pw = q3pwss 
	   flgfov = flgfovss
           GOTO, FILLSTR
        ENDIF

	RETURN

NORESPACT:    
	XDISPLAYFILE, "", $
           TITLE = "Not actual responses available ", $
           WIDTH = 60, $
           HEIGHT = 2, $
           TEXT = [" ERROR - Cannot find calculated responses"]
         
	RETURN

	END
