;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO PROCESS_SPD, TMPDATA, TMPHDR, TMPAAP, AAPPS 
;+
; NAME:
;	PROCESS_SPD
; PURPOSE:
;	Convert PHT data from In-Band-Power per Chopper Plateau (SPD) to 
;	photometric flux tables (AAP). It accomodates the data into the 
;	dynamical structure PHTAAP, returning also the data as a temporary 
;	structure.
; CATEGORY:
;	PIA - processing
;                
; CALLING SEQUENCE:
;	PROCESS_SPD, SPDDATA, tmpaap, aapps
;
; INPUT PARAMETERS:
;	SPDDATA	: Structure containing the SPD data to be processed
;
; OPTIONAL INPUT:
;	SPDHDR  : (Updated) Header element corresponding to SPDDATA
;
; OPTIONAL OUTPUT:
;	TMPAAP	: A temporary AAP structure
;	AAPPS	: Position of the produced AAP data within the PHTAAP structure 
;		  (can be used to check if AAP data has been produced)
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtaap
;       PIA Procedure(s):
;               flux
;	
; MODIFICATION HISTORY:
;	Written by: C. Gabriel
;	Oct 95 Version 3.5
;	Only "good" points entering the AAP structure (CG)	October 1995
;	Changed call to flux to avoid warning (CG)		March 1996
;	Automatic coordinates setting if not present (CG)	May 1996
;	Avoiding division by 0. (CG)				June 1996
;	Bug fixed (median brightness from avg. power) (CG)	June 1996
;	changed for the aap central wavelength (WMT)            Nov  1996
;	Accept also points with FLAG=1 (CG)			Dec 1997
;	V6.6
;	Changing AAP hdr: new type and PIA version (CG)		March 1998
;	Flag system included (JAc)				March 1998
;	 but commented until in use (CG)			May 1998
;	V7.1 
;	Bug introduced in 7.1 for PHT-S fixed (CG)		May 1998
;	V7.1.1
;	Staring meas. with 1 pixel deselected was wrong 
;	treated in flag value. Fixed now. (CG)			December 1998
;	V7.3
;	Changes according to use of bitflag system (CG)		May 1999
;	V7.4 
;	Deriving "new" AAP structure with RAP and DECP (CG)	March 2000
;	V8.2
;	Putting for all cases info on new AAP structure into
;	the buffer (CG)						May 2000
;	V9.0
;-
;
; Needed common blocks :
@adm
@phtaap
@phtfilter_prop

	ON_ERROR, 2                                          
;
; Check the number of parameters
;
	IF N_PARAMS() LT 1 THEN MESSAGE, $
	   'Syntax: PROCESS_SPD, spddata,[spdhdr, tmpaap, aapps]'

	IF KEYWORD_SET(tmphdr) THEN tmphdr = tmphdr ELSE tmphdr = tmpdata.hdr


	IF tmpdata.dec(0) EQ -99 THEN add_coordinates, tmpdata

	filt = tmpdata.filt(0)
	aper = tmpdata.aper(0)
	polz = tmpdata.polz(0)

; Avoid division by 0.
	tmp_mnpw = tmpdata.mnpw
	ind = WHERE(ABS(tmp_mnpw) LT 1.e-20, n)
	IF n GT 0 THEN tmp_mnpw(ind) = 1.e-20
	
	flux, tmpdata.mnpw, tmpdata.det, filt, aper, 0, oflux
	flun = oflux * tmpdata.mnpu / tmp_mnpw
	flux, tmpdata.mnpw, tmpdata.det, filt, aper, 1, flde
	flde = flde * 1.e-06
	fldu = flde * tmpdata.mnpu / tmp_mnpw

	flux, tmpdata.mdpw, tmpdata.det, filt, aper, 0, fmux
	flux, tmpdata.q1pw, tmpdata.det, filt, aper, 0, f1ux
	flux, tmpdata.q3pw, tmpdata.det, filt, aper, 0, f3ux
	flux, tmpdata.mdpw, tmpdata.det, filt, aper, 1, fmde
	flux, tmpdata.q1pw, tmpdata.det, filt, aper, 1, f1de
	flux, tmpdata.q3pw, tmpdata.det, filt, aper, 1, f3de
	fmde = fmde * 1.e-06
	f1de = f1de * 1.e-06
	f3de = f3de * 1.e-06
	
; Several lines commented until flag system is in use. Redflag = SPD_flag
; Now "de"commented and flag_limit changed to 0

	get_validflag, tmpdata.flag, 'SPD', redflag, $
	   DET = tmpdata.det

	   
; check for the common valid points for consistency with previous versions of 
;  PIA without flag system
        npix = pixel_det(tmpdata.det, /IS_SPD)

	coadflag = INTARR(tmpdata.dims)
   ;reset and mark only those points flagged as bad for all pixels
        FOR ipix=0, npix-1 DO $
	     coadflag = coadflag + redflag(ipix,*) 
        coadflag = coadflag/npix
	
	ind = WHERE(coadflag EQ 0,count)

	IF count EQ 0 THEN BEGIN
		MESSAGE, /INFO, $ 
		    'Not processed to AAP: All points discarded!!!'
		RETURN
        ENDIF
        
	CASE strmid(tmpdata.det, 0, 1) OF
	   'P': begin
		cwave = [filtprop.p1cwl, filtprop.p2cwl, filtprop.p3cwl]
		cwvl = cwave(filt-1)
	      end
	   'C': begin
		cwave = [filtprop.c1cwl, filtprop.c2cwl]
		cwvl = cwave(filt-2)
	      end
	   ELSE: cwvl = 0.0
	ENDCASE
; Update header
	 keyw = STRMID(tmphdr,0,8)
	 com = WHERE(keyw EQ 'COMMENT ',n)
	 IF n GT 0 THEN BEGIN
	   posi = STRPOS(tmphdr(com(0)),'In-band power')
	   IF posi NE -1 THEN BEGIN
	     tmpline = tmphdr(com(0))
	     STRPUT, tmpline, 'Flux/brightn.', posi
	     tmphdr(com(0)) = tmpline
	   ENDIF
	 ENDIF 
	 FXADDPAR, tmphdr, 'PIA_VERS', !PIA_VERSION, $
	  ' PHT-Interactive Analysis version used', BEFORE='XTENSION'	     
	 
	 decp = oflux(*,ind) * 0.
	 rap = decp
	 FXADDPAR, tmphdr, 'PIA_NAAP', 1, ' New type AAP: AAP2000', $
		BEFORE='XTENSION'
	 
         tmpaap = {ADMN: tmpdata.admn, DET: tmpdata.det, $
                DIMS: count, $
                FILT: [filt], APER: [aper], POLZ: [polz], $
                CWVL: [cwvl],                  $
                HDR: [tmphdr],                 $
                TIME: [tmpdata.time(ind)],     $
                CPOS:[tmpdata.cpos(ind)],      $
		CSTP: [tmpdata.cstp(ind)],     $
		DWEL: [tmpdata.dwel(ind)],     $ 
                MNFL: [oflux(*,ind)],          $
                MNFU: [flun(*,ind)],           $
                MNBR: [flde(*,ind)],           $
                MNBU: [fldu(*,ind)],           $
                MDFL: [fmux(*,ind)],           $
                Q1FL: [f1ux(*,ind)],           $
                Q3FL: [f3ux(*,ind)],           $
                MDBR: [fmde(*,ind)],           $
                Q1BR: [f1de(*,ind)],           $
                Q3BR: [f3de(*,ind)],           $
                RPID: [tmpdata.rpid(*,ind)],   $ 
                DEC: [tmpdata.dec(ind)],       $
                RA: [tmpdata.ra(ind)],         $
                DECP: decp,     	       $
                RAP:  rap,                     $
                ROLL: [tmpdata.roll(ind)],     $
                FLAG: [tmpdata.flag(*,ind)]}
	
;  Accomodate data into the aap structure and the aap handles after
;  creating the temporary ones                  

; get the point RA and DEC variables in
	gen_aap, tmpaap
	FILL_AAP, tmpaap, tmphdr

; GET the index where the new aap structure was stored
	AAPPS = N_ELEMENTS(PHTAAP) - 1
	RETURN
	END
