;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO PROCESS_SRD, FILES, HDR, SCPPS, DRIFTCORR=DRIFTCORR 
;+
; NAME:
;	PROCESS_SRD
; PURPOSE:
;	Reduce PHT data from Signal per Ramp to Signal per Chopper Plateu,
;	taken into account diverse selections or flags, and put
;	averages, medians, uncertainties, etc. per chopper plateau into the 
;	structure phtscp 
;	In case of a staring measurement (1 chopper plateaux), data is reduced
;	to signal per raster position. 
; CATEGORY:
;	PIA - processing
;                
; CALLING SEQUENCE:
;	PROCESS_SRD, FILES, HDR, SCPPS, DRIFTCORR = driftcorr
;
; INPUT PARAMETERS:
;	FILES	: Array containing the names (phtsrd.admn) of the buffers
;		  to be processed
;	
; KEYWORDS:
;	DRIFTCORR: If set, drift analysis performed
;
; OPTIONAL INPUT:
;	HDR	: HDR string array - can be used for passing updated header 
;
; OPTIONAL OUTPUT:
;	SCPPS	: Array containing the positions of the produced SCP 
;		  data within the PHTSCP structure 
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtacc
;               phtscp
;               phtsrd
;       PIA Procedure(s):
;               der_scphdr
;               get_rastersteps
;		mann
;       Astronomical Library:
;               avg
;               sigma
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel (ESA/VILSPA-SAI)
;	Oct 95 Version 3.5
;	Drift stability analysis incorporated (CG)	December 1995
;	Long integer for loop variable (for more than 32756 signals!) Jan.96
;	Drift stability analysis changed:
;		at least 8 points required
;		if not stable -> take last points (generally 8 but depending on
;				 flag) and calculate average and slope change 
;	Extension: eff.reset time, eff.nr of readouts & 
;	  distribution standard deviation (CG)		January 1996
;	Bug fixed: if only partial stable mean was taken instead of
;	  weighted mean (CG)				January 1996
;	Changeable parameters (common pht_mann_stab) instead of
;	  fix parameters for stability analysis (CG)	January 1996
;-
;
; Needed common blocks :
@adm
@phtsrd
@phtscp
@phtacc
COMMON pht_mann_st, conflvl, intdiv, minp, mann_stats

	ON_ERROR, 2                                          
;
; Check the number of parameters
;
	IF N_PARAMS() LT 2 THEN MESSAGE, $
	   'Syntax: PROCESS_SRD, FILES, [HDR, SCPPOS]'
; For each measurement chosen
;
	FOR I = 0, N_ELEMENTS(FILES) - 1 DO BEGIN
; 
          ID = STRMID(FILES(I),0,2)
;
          INDEX = WHERE (PHTSRD.ADMN EQ STRMID(FILES(I),0,12))
	  INDEX = INDEX(0)
 

; Take the input HDR if present
          IF KEYWORD_SET(HDR) THEN SRDHDR = HDR ELSE $
	  	HANDLE_VALUE, PHTSRD(INDEX).HDR, SRDHDR

	  handle_value, phtsrd(index).stim, time
	  handle_value, phtsrd(index).slop, slop
	  handle_value, phtsrd(index).slun, slun
	  handle_value, phtsrd(index).flag, flag
	  handle_value, phtsrd(index).chop, chop
	  handle_value, phtsrd(index).step, step
	  handle_value, phtsrd(index).fcsn, fcsn
	  handle_value, phtsrd(index).temp, temp
	  handle_value, phtsrd(index).fcs1, fcs1
	  handle_value, phtsrd(index).fcs2, fcs2
	  handle_value, phtsrd(index).tkey, tkey
	  handle_value, phtsrd(index).rpid, rpid
	  handle_value, phtsrd(index).resi, resi
	  handle_value, phtsrd(index).nros, nros


; Take care of the different array dimensions for different detectors
	  CASE ID OF
; C100
	  'C1': BEGIN
		  DET = 'C1'
		  NPIX = 11
	        END
; C200
          'C2': BEGIN
		  DET = 'C2'
		  NPIX = 6
	        END
; S
	  'SL': BEGIN
		  DET = 'SL'
		  NPIX = 132
	        END
	  ELSE:
	  ENDCASE

	  IDRED = STRMID(ID,0,1)
          CASE IDRED OF
; P
	    'P': BEGIN
		  DET = phtsrd(index).det
		  NPIX = 1
	    END
	  ELSE: 
          ENDCASE

	  ntot = phtsrd(index).sldi
	  oldstep = 1
	  position = 0
	  plat_num = 1

	  in_plat_pos = intarr(ntot)
	  plat_pos = intarr(ntot)
                
; take care of those signals without a chopper step assigned
          fsig = 0L

          indnas = WHERE(step EQ 0, nnas)
          IF nnas GT 0 THEN BEGIN
                plat_pos(indnas) = 0
                fsig = nnas*1L   ; assuming step=0 can be just at begin
          ENDIF

	    FOR isig=fsig,ntot-1 DO BEGIN
		IF step(isig) ne oldstep AND step(isig) NE -1 THEN BEGIN
		  in_plat_pos(isig) = 1
		  oldstep = step(isig)
		  plat_num = plat_num + 1 
		  plat_pos(isig) = plat_num
		  position = 0
	  	ENDIF
	  	position = position + 1
	  	in_plat_pos(isig) = position
		plat_pos(isig) = plat_num
	    ENDFOR

; Fill sig_disc with discarded signals
	  sig_disc = intarr(npix,ntot)
	  sig_dis_com = intarr(ntot)

          FOR ipix = 0,npix-1 DO BEGIN 
	  	erddisc = WHERE(flag(ipix,*) GT 1, ncount)
	  	IF ncount gt 0 THEN BEGIN
			sig_disc(ipix,erddisc) = 1
		ENDIF
          ENDFOR

	  IF sig1acc THEN BEGIN
	  	inddis1 = WHERE(in_plat_pos eq 1)
	  	sig_disc(*,inddis1) = 1
	  	sig_dis_com(inddis1) = 1
	  ENDIF

	  FOR isn=0L,N_ELEMENTS(signacc)-1 DO BEGIN
		inddisc = WHERE(in_plat_pos EQ signacc(isn), ncount)
		IF ncount gt 0 THEN BEGIN 
			sig_disc(*,inddisc) = 1
	  		sig_dis_com(inddisc) = 1
		ENDIF
	  ENDFOR

	  IF minsig GT 0 THEN BEGIN 
		sig_disc(*,0:minsig-1) = 1
		sig_dis_com(0:minsig-1) = 1
	  ENDIF
	  IF maxsig GT 0 THEN BEGIN	
		sig_disc(*,maxsig-1:*) = 1
		sig_dis_com(maxsig-1:*) = 1
	  ENDIF
	  
	  IF (N_ELEMENTS(anysig) GT 1 or anysig(0) NE 0) THEN BEGIN
		IF max(anysig) LT N_ELEMENTS(sig_disc(0,*)) THEN BEGIN
			sig_disc(*,anysig-1) = 1
			sig_dis_com(anysig-1) = 1
			MESSAGE, /INFO, 'Individual signals are discarded'
		ENDIF
	  ENDIF
  
;
;  Look for different raster points first 
;

	  get_rastersteps, rpid, raststeps


;
;  The complication of having unsynchronized chopper plateaux and raster points
;  is solved in the way, that if a chopper plateau has started on a raster
;  point, the rest of this chopper plateau is ignored, in case of a chopped 
;  measurement. In case of a staring measurement the raster points are taken as 
;  if they were chopper plateaux.
;

;  Staring case

	chop_plat_num = plat_num
	IF plat_num EQ 1 and max(raststeps) GT 1 THEN BEGIN

	  ntot = N_ELEMENTS(raststeps)
	  oldstep = 1
	  position = 0
	  plat_num = 1

	  in_plat_pos = intarr(ntot)
	  plat_pos = intarr(ntot)
                
	    FOR isig=0L,ntot-1 DO BEGIN
		IF raststeps(isig) ne oldstep THEN BEGIN
		  in_plat_pos(isig) = 1
		  oldstep = raststeps(isig)
		  plat_num = plat_num + 1 
		  plat_pos(isig) = plat_num
		  position = 0
	  	ENDIF
	  	position = position + 1
	  	in_plat_pos(isig) = position
		plat_pos(isig) = plat_num
	    ENDFOR

        ENDIF 


	  ttime = fltarr(plat_num)
	  taver = fltarr(npix,plat_num)
	  tavun = taver 
	  tmedi = taver
	  tq1me = taver
	  tq3me = taver
	  tflag = intarr(npix,plat_num)
	  tchop = ttime
	  tstep = intarr(plat_num)
	  tfcsn = tstep
	  ttemp = ttime
	  tfcs1 = ttime
	  tfcs2 = ttime
	  ttkey = ttime
	  trpid = bytarr(2,plat_num)
	  tarsi = taver
	  tandr = taver
	  tavsd = taver	  
	  FOR iplat = 0L, plat_num-1  DO BEGIN

	     indall = WHERE (plat_pos-1 EQ iplat, nnc)
             IF nnc EQ 0 THEN BEGIN
                txt = 'Missing plateau position'+ STRING(iplat+1)
                MESSAGE, /INFO, txt
                missplat = 1
                GOTO, escape
             ENDIF
	     indsel = WHERE (plat_pos-1 EQ iplat AND sig_dis_com EQ 0, ncount)
	     
	     IF ncount GT 0 THEN BEGIN

; in case that there are chopper plateaux elements common to two or more 
; different raster points take just the first part of them and forget the 
; rest (only feasible solution for unsynchronized plateaux) 

		indin = WHERE(raststeps(indsel) EQ raststeps(indsel(0)),nindi)
		ttime(iplat) = avg(time(indsel(indin)))
		 FOR ipix=0,npix-1 DO BEGIN
	            ind = $
		      WHERE (plat_pos-1 EQ iplat AND sig_disc(ipix,*) EQ 0, nc)
                    IF nc GT 0 THEN BEGIN
		       indind = WHERE(raststeps(ind) EQ raststeps(ind(0)),nindi)
			weight_mean, slop(ipix,ind(indind)), $
				     slun(ipix,ind(indind)), avwe, unwe, avsi  
			taver(ipix,iplat) = avwe  
			tavun(ipix,iplat) = unwe
			tavsd(ipix,iplat) = avsi	
			tmedi(ipix,iplat) = median(slop(ipix,ind(indind)))
			tarsi(ipix,iplat) = avg(resi(ipix,ind(indind)))
			tandr(ipix,iplat) = median(nros(ipix,ind(indind)))
			indmed = $
		where(slop(ipix,ind(indind)) LT tmedi(ipix,iplat),nquart)
			if nquart GT 0 THEN $
		tq1me(ipix,iplat) = median(slop(ipix,ind(indind(indmed)))) $
			ELSE tq1me(ipix,iplat) = 0.
			indmed = $
		where(slop(ipix,ind(indind)) GE tmedi(ipix,iplat),nquart)
		tq3me(ipix,iplat) = median(slop(ipix,ind(indind(indmed))))
			IF nc EQ 1 THEN $
			  tflag(ipix,iplat) = 1 ELSE $
			  tflag(ipix,iplat) = 0
		    ENDIF ELSE BEGIN
			taver(ipix,iplat) = 0.  
			tavun(ipix,iplat) = 0.	
			tmedi(ipix,iplat) = 0.
                	tq1me(ipix,iplat) = 0.
                	tq3me(ipix,iplat) = 0.
			tflag(ipix,iplat) = 2
			tarsi(ipix,iplat) = 0.  
			tandr(ipix,iplat) = 0	
			tavsd(ipix,iplat) = 0	
		    ENDELSE
		 ENDFOR
                tchop(iplat) = median(chop(indsel(indin)))
                tstep(iplat) = median(step(indsel(indin)))
                tfcsn(iplat) = median(fcsn(indsel(indin)))
                ttemp(iplat) = median(temp(indsel(indin)))
                tfcs1(iplat) = median(fcs1(indsel(indin)))
                tfcs2(iplat) = median(fcs2(indsel(indin)))
                ttkey(iplat) = median(tkey(indsel(indin)))
                trpid(*,iplat) = rpid(*,indsel(0))
             ENDIF ELSE BEGIN
		indin = WHERE(raststeps(indall) EQ raststeps(indall(0)),nindi)
		ttime(iplat) = avg(time(indall(indin)))
		taver(*,iplat) = 0.  
		tavun(*,iplat) = 0.	
		tmedi(*,iplat) = 0.
                tq1me(*,iplat) = 0.
                tq3me(*,iplat) = 0.
                tflag(*,iplat) = 2
		tarsi(*,iplat) = 0.  
		tandr(*,iplat) = 0	
		tavsd(*,iplat) = 0.	
                tchop(iplat) = median(chop(indall(indin)))
                tstep(iplat) = median(step(indall(indin)))
                tfcsn(iplat) = median(fcsn(indall(indin)))
                ttemp(iplat) = median(temp(indall(indin)))
                tfcs1(iplat) = median(fcs1(indall(indin)))
                tfcs2(iplat) = median(fcs2(indall(indin)))
                ttkey(iplat) = median(tkey(indall(indin)))
                trpid(*,iplat) = rpid(*,indall(0))
             ENDELSE
             missplat = 0
escape:
             IF missplat EQ 1 THEN tflag(*,iplat) = 2
          ENDFOR
;  Apply stability testing to real staring measurements (no rasters)
;  If partially or totally stable overwrite contents and put info into
;  the header
	  IF chop_plat_num EQ 1 AND  $
	  	KEYWORD_SET(DRIFTCORR) THEN BEGIN

;  Initialize parameters if not already present in common block
		 IF N_ELEMENTS(conflvl) EQ 0 THEN BEGIN
			conflvl=0.05
			intdiv = 1
			minp=7
		 ENDIF

; Log into header the relevant parameters used
		 newwd = STRING(replicate(32b,15))
		 STRPUT, newwd, STRING(conflvl), 0

		 PRSPIX = 'PRS_DCLV' 
		 junk = PRSPIX +'=       ' + newwd + $
			'/ Confidence level for drift analysis '
		 newl0 = STRING(REPLICATE(32B, 80))
		 STRPUT, newl0, junk, 0			  
		 drihdr_gen = newl0		 

		 newwd = STRING(replicate(32b,15))
		 STRPUT, newwd, STRING(intdiv), 0

		 PRSPIX = 'PRS_DINT' 
		 junk = PRSPIX +'=       ' + newwd + $
			'/ Interval division for drift analysis '
		 newl0 = STRING(REPLICATE(32B, 80))
		 STRPUT, newl0, junk, 0	
		 drihdr_gen = [drihdr_gen, newl0]		  

		 newwd = STRING(replicate(32b,15))
		 STRPUT, newwd, STRING(minp), 0

		 PRSPIX = 'PRS_DMNP' 
		 junk = PRSPIX +'=       ' + newwd + $
			'/ Minimum number of points for drift analysis '
		 newl0 = STRING(REPLICATE(32B, 80))
		 STRPUT, newl0, junk, 0			  
		 drihdr_gen = [drihdr_gen, newl0]		  

		decode_admn, phtsrd(index).admn, tmpdec
		mann_stats = ['Stability analysis ("mann") statistics', $
				'======================================',' ',$
				'Measurement '+tmpdec, ' ', drihdr_gen, ' ']

		FOR irast=1, max(raststeps) DO BEGIN
		 indrst = WHERE(raststeps EQ irast)
		 rpstr = STRTRIM(STRING(FIX(rpid(*,indrst(0)))),2)
	    	 mann_stats = [mann_stats, ' ','Raster point ['+rpstr(0)+','+ $
	    		rpstr(1)+']']

		 FOR ipix=0,npix-1 DO BEGIN
	            ind = WHERE (sig_disc(ipix,indrst) EQ 0, nc)
		    ok = 0
                    IF nc GE minp THEN BEGIN

			mann, slop(ipix,ind), conflvl, tu, td

			IF tu EQ 0 AND td EQ 0 THEN BEGIN 
			   weights = 1./slun(ipix,ind)^2
			   icutoff = WHERE(weights GT 30000., ncut)
			   IF ncut GT 0 THEN $
			   	  	weights(icutoff) = 30000.
 			   res = polyfitw(time(ind),slop(ipix,ind),weights,1)
 			   sslop = res(1) / taver(ipix,0) * 60
; Log into header the relevant parameters used
			   newwd = STRING(replicate(32b,15))
			   STRPUT, newwd, STRING(nc), 0

			   STPIX = STRTRIM(STRING(ipix+1),2)
			   IF STRLEN(STPIX) EQ 1 THEN STPIX = '00'+STPIX $
			   ELSE IF STRLEN(STPIX) EQ 2 THEN STPIX = '0'+STPIX  
			   
			   PRSPIX = 'PRS_D'+STPIX 
			   junk = PRSPIX +'=       ' + newwd + $
			'/ Totally drift stable - Nr. of taken points'
			  newl0 = STRING(REPLICATE(32B, 80))
			  STRPUT, newl0, junk, 0			  
; Log into header the relevant parameters used
			   newwd = STRING(replicate(32b,15))
			   STRPUT, newwd, STRING(sslop), 0
			   PRSPIX = 'PRS_G'+STPIX 
			   junk = PRSPIX+'=       ' + newwd + $
			'/ Drift: Rel variation/min'
			  newl = STRING(REPLICATE(32B, 80))
			  STRPUT, newl, junk, 0			  
			  drihdr = [newl0,newl]
			  ok = 1
			ENDIF ELSE BEGIN
			   mid = N_ELEMENTS(ind) / 2^intdiv
			   indnew = ind(mid:*)
			   npoints = N_ELEMENTS(indnew)
			   IF mid EQ 0 THEN npoints = 0

			   WHILE npoints GE minp DO BEGIN
			     mann, slop(ipix,indnew), conflvl, tu2, td2
				IF tu2 EQ 0 AND td2 EQ 0 THEN BEGIN 
			   	  weights = 1./slun(ipix,indnew)^2
			   	  icutoff = WHERE(weights GT 30000., ncut)
			   	  IF ncut GT 0 THEN $
			   	  	weights(icutoff) = 30000.
 			          res = $
			     polyfitw(time(indnew),slop(ipix,indnew),weights,1)
 				  weight_mean, slop(ipix,indnew), $
				     slun(ipix,indnew), avwe, unwe, avsi  
				  taver(ipix,0) = avwe
				  tavun(ipix,0) = unwe
				  tavsd(ipix,0) = avsi
			          sslop = res(1) / taver(ipix,0) * 60
; Log into header the relevant parameters used
			   	  STPIX = STRTRIM(STRING(ipix+1),2)
			   	  IF STRLEN(STPIX) EQ 1 THEN STPIX = '00'+STPIX $
			   	  ELSE IF STRLEN(STPIX) EQ 2 THEN STPIX = '0'+STPIX  
			   	  newwd = STRING(replicate(32b,15))
			   	  STRPUT, newwd, STRING(npoints), 0
			   	  PRSPIX = 'PRS_D'+STPIX 
			   	  junk = PRSPIX +'=       ' + newwd + $
				  '/ Partially drift stable - Nr of taken points'
			  	  newl0 = STRING(REPLICATE(32B, 80))
			  	  STRPUT, newl0, junk, 0			  
; Log into header the relevant parameters used
			   	  newwd = STRING(replicate(32b,15))
			   	  STRPUT, newwd, STRING(sslop), 0
			   	  PRSPIX = 'PRS_G'+STPIX 
			   	  junk = PRSPIX +'=       ' + newwd + $
				  '/ Drift: Rel variation/min'
			  	  newl = STRING(REPLICATE(32B, 80))
			  	  STRPUT, newl, junk, 0			  
				  ok = 1
				  npoints = 0
			  	  drihdr = [newl0,newl]
				ENDIF ELSE BEGIN
			          mid = N_ELEMENTS(indnew) / 2^intdiv
			   	  indnew = indnew(mid:*)
			   	  npoints = N_ELEMENTS(indnew)
				  IF mid EQ 0 THEN npoints = 0				
				ENDELSE
			     ENDWHILE
			 ENDELSE	

			 IF ok EQ 0 THEN BEGIN
			   indnew = ind(N_ELEMENTS(ind)-minp:*)
			   npoints = N_ELEMENTS(indnew)
			   weights = 1./slun(ipix,indnew)^2
			   icutoff = WHERE(weights GT 30000., ncut)
			   IF ncut GT 0 THEN $
			   	  	weights(icutoff) = 30000.
 			   res = polyfitw(time(indnew),slop(ipix,indnew),weights,1)
 			   sslop = res(1) / taver(ipix,0) * 60
; Log into header the relevant parameters used
			   STPIX = STRTRIM(STRING(ipix+1),2)
			   IF STRLEN(STPIX) EQ 1 THEN STPIX = '00'+STPIX $
			   ELSE IF STRLEN(STPIX) EQ 2 THEN STPIX = '0'+STPIX  
			   newwd = STRING(replicate(32b,15))
			   STRPUT, newwd, STRING(npoints), 0
			   PRSPIX = 'PRS_D'+STPIX 
			   junk = PRSPIX +'=       ' + newwd + $
			'/ Drift unstable - Nr of taken points '
			  newl0 = STRING(REPLICATE(32B, 80))
			  STRPUT, newl0, junk, 0			  
; Log into header the relevant parameters used
			   newwd = STRING(replicate(32b,15))
			   STRPUT, newwd, STRING(sslop), 0
			   PRSPIX = 'PRS_G'+STPIX 
			   junk = PRSPIX +'=       ' + newwd + $
			'/ Drift: Rel variation/min'
			  newl = STRING(REPLICATE(32B, 80))
			  STRPUT, newl, junk, 0			  
			  drihdr = [newl0,newl]
			ENDIF

		   ENDIF ELSE BEGIN
; Case of end with just discarded points
			IF nc EQ 0 THEN BEGIN
				sslop = 0.
			ENDIF ELSE BEGIN
				weights = 1./slun(ipix,ind)^2
				res = polyfitw(time(ind),slop(ipix,ind),weights,1)
				sslop = res(1) / taver(ipix,0) * 60
			ENDELSE
; Log into header the relevant parameters used
			STPIX = STRTRIM(STRING(ipix+1),2)
			IF STRLEN(STPIX) EQ 1 THEN STPIX = '00'+STPIX $
			ELSE IF STRLEN(STPIX) EQ 2 THEN STPIX = '0'+STPIX  
			newwd = STRING(replicate(32b,15))
			STRPUT, newwd, STRING(nc), 0
			PRSPIX = 'PRS_D'+STPIX 
			junk = PRSPIX +'=       ' + newwd + $
			'/ Drift: not enough pts- Nr of taken points'
			newl0 = STRING(REPLICATE(32B, 80))
			STRPUT, newl0, junk, 0			  
; Log into header the relevant parameters used
			newwd = STRING(replicate(32b,15))
			STRPUT, newwd, STRING(sslop), 0
			PRSPIX = 'PRS_G'+STPIX 
			junk = PRSPIX +'=       ' + newwd + $
			'/ Drift: Rel variation/min'
			newl = STRING(REPLICATE(32B, 80))
			STRPUT, newl, junk, 0			  
			drihdr = [newl0,newl]
		   ENDELSE
; Into a file
		rpstr = STRTRIM(STRING(FIX(rpid(*,indrst(0)))),2)
	    	mann_stats = [mann_stats, drihdr]
		ENDFOR

	    ENDFOR

; Now into the header
            KEYWORD = STRMID(srdhdr, 0, 8)
            XTENSION = WHERE(KEYWORD EQ 'XTENSION')
            hdrlen = N_ELEMENTS(srdhdr)

            srdhdr = [ srdhdr( 0 : XTENSION(0) - 1), drihdr_gen, $
                   	  srdhdr( XTENSION(0) : HDRLEN - 1 )]
	  ENDIF

out_stab:
		
;  Accomodate data into the scp structure and the scp handles after
;  creating the temporary ones                  
	  PHTSCPT = {PHTSCP_TYPE}
          PHTSCPT.HDR = HANDLE_CREATE()
          PHTSCPT.TIME = HANDLE_CREATE()
          PHTSCPT.AVER = HANDLE_CREATE()
          PHTSCPT.AVUN = HANDLE_CREATE()
          PHTSCPT.MEDI = HANDLE_CREATE()
          PHTSCPT.Q1ME = HANDLE_CREATE()
          PHTSCPT.Q3ME = HANDLE_CREATE()
          PHTSCPT.FLAG = HANDLE_CREATE()
          PHTSCPT.CHOP = HANDLE_CREATE()
          PHTSCPT.STEP = HANDLE_CREATE()
          PHTSCPT.FCSN = HANDLE_CREATE()
          PHTSCPT.TEMP = HANDLE_CREATE()
          PHTSCPT.FCS1 = HANDLE_CREATE()
          PHTSCPT.FCS2 = HANDLE_CREATE()
          PHTSCPT.TKEY = HANDLE_CREATE()
          PHTSCPT.RPID = HANDLE_CREATE()
          PHTSCPT.ARSI = HANDLE_CREATE()
          PHTSCPT.ANDR = HANDLE_CREATE()
          PHTSCPT.AVSD = HANDLE_CREATE()

;  Change the header according to the selection criteriae 
	  DER_SCPHDR, SRDHDR, SCPHDR
	  HANDLE_VALUE, PHTSCPT.HDR, SCPHDR, /SET
; 	  
          PHTSCPT.UNIT = PHTSRD(INDEX).UNIT
          PHTSCPT.DET  = PHTSRD(INDEX).DET
	  PHTSCPT.ADMN = PHTSRD(INDEX).ADMN
          PHTSCPT.ORDR = PHTSRD(INDEX).ORDR
          PHTSCPT.DIMS = plat_num

	  HANDLE_VALUE, PHTSCPT.TIME, TTIME, /SET, /NO_COPY
	  HANDLE_VALUE, PHTSCPT.AVER, TAVER, /SET, /NO_COPY 
          HANDLE_VALUE, PHTSCPT.AVUN, TAVUN, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.MEDI, TMEDI, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.Q1ME, TQ1ME, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.Q3ME, TQ3ME, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.FLAG, TFLAG, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.CHOP, TCHOP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.STEP, TSTEP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.FCSN, TFCSN, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.TEMP, TTEMP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.FCS1, TFCS1, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.FCS2, TFCS2, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.TKEY, TTKEY, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.RPID, TRPID, /SET, /NO_COPY
	  HANDLE_VALUE, PHTSCPT.ARSI, TARSI, /SET, /NO_COPY 
          HANDLE_VALUE, PHTSCPT.ANDR, TANDR, /SET, /NO_COPY
          HANDLE_VALUE, PHTSCPT.AVSD, TAVSD, /SET, /NO_COPY

; Update the PHTSRD structure using the temporary structure
	  PHTSCP = [PHTSCP, PHTSCPT]
; Update the PHTSRD value in the administration file using the index
	  SCPPOS = N_ELEMENTS(PHTSCP) - 1
	  INDADM = WHERE (ADM.NAME EQ PHTSCPT.ADMN)
	  ADM(INDADM).PHTSCP = SCPPOS
	  IF I EQ 0 THEN SCPPS = SCPPOS ELSE SCPPS = [SCPPS, SCPPOS]	  
	ENDFOR
	RETURN
	END
