;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO PROCESS_TMPERD, TMPERD, NDEG, HDR, SRDPS, DIV_ramp=div_ramp, $
	        DR=dr, OTF=otf  
;+
; NAME:
;	PROCESS_TMPERD
; PURPOSE:
;	Fit voltage ramps of chosen measurements, taken into 
;	account diverse selections or flags and put the resulting
;	ramps slopes, uncertainties, etc. into the structure
;       phtsrd 
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PROCESS_TMPERD, TMPERD, NDEG, HDR, SRDPS, ERDPS [,DIV_RAMP=div_ramp]
;
; INPUT PARAMETERS:
;	TMPERD	: Temporary Structure (ERD level) to be processed
;	NDEG	: Degree of the polynomial to be fitted
;
;
; INPUT KEYWORDS:
;	div_ramp: A factor for subdividing ramps when processing
;
; OPTIONAL INPUT:
;	HDR	: String Array containing the file (updated) header
;
; OPTIONAL OUTPUT:
;	SRDPS	: Array containing the positions of the produced SRD 
;		  data within the PHTSRD structure
;	ERDPS	: Array containing the positions of the input
;		  data within the PHTERD structure
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phterd
;               phtsrd
;       PIA Procedure(s):
;               acc_flg
;               acc_volt
;		check_chop
;               decode_pixf
;               der_shdr
;               fit_ramps
;               red_ramps
;		rpid_check
;
; MODIFICATION HISTORY:
;	Written by JAcosta and CG based on existing process_erd  Nov. 1997
;	V6.6
;	Check for rpid=[0b,0b] added (CG)			Feb.98
;	V7.0
;	Bug for checking unc. for extrem long loops fixed (CG)  Jun.98
;	V7.2
;	Upgrade (larger efficiency) by Jose Acosta (IAC) using
;	bitflag system				May 1999
;	Data reduction info added (CG)			May 1999
;	V7.4
;	Bug for splitting into 2 fixed (CG)		August 1999
;	V8.0+			
;	Call to red_ramps including DR input (CG)	November 1999
;	V8.2
;	Bug for splitting into maximal number fixed (CG) July 2000
;	V9.0
;	Bug for subdivision into 2 ramps fixed (CG)	July 2000
;	V9.1
;-
;
; Needed common blocks :
@phtsrd
@splitramp_par
;	ON_ERROR, 2                                          
;
; Check the number of parameters
;
	IF N_PARAMS() LT 2 THEN MESSAGE, $
	   'Syntax: PROCESS_TMPERD, TMPERD, NDEG, [HDR], [SRDPOS]'



; Look for compact status record
	  GIVE_CS, tmperd, cst, record, conv_cst
	  IF STRPOS(STRUPCASE(conv_cst.cmod(record)),'RECTANGULAR') NE -1 THEN $
	      CHINCR = FLOAT(STRMID(conv_cst.camp(record),0, $ 
	                   STRPOS(conv_cst.camp(record),'arcsec')))*2  $
	  ELSE CHINCR = FLOAT(STRMID(conv_cst.cinc(record),0, $ 
	                   STRPOS(conv_cst.cinc(record),'arcsec')))
; 
; Take the input HDR if present
	  IF N_ELEMENTS(HDR) EQ 0 THEN ERDHDR = hdr ELSE $
	  	erdhdr = tmperd.HDR

; Data reduction
 	  datred = cst.drs(record)

; Detector
          det = tmperd.det

; Take care of the different array dimensions for different detectors
          npix = pixel_det(det,/allpx)

; Check that DR is defined 
          IF N_ELEMENTS(dr) EQ 0 THEN $
         	  DECODE_PIXF, tmperd.pixf, suspect, chopon, chops, dr, otf 
; Check that OTF is defined 
           IF N_ELEMENTS(otf) EQ 0 THEN $
	          DECODE_PIXF, tmperd.pixf, suspect, chopon, chops, dr, otf 
 
; Integration time 
	  TINT = tmperd.TINT

; Subdividing ramps 
	
	IF KEYWORD_SET(div_ramp) THEN BEGIN
	   IF N_ELEMENTS(splitramp_par) EQ 0 THEN splitramp_par_init
	   nndr = tmperd.nndr
	   IF splitramp_par.model EQ 0 THEN BEGIN
	   	pseudoramp = splitramp_par.value 
		IF pseudoramp LT 2 OR pseudoramp GT nndr THEN pseudoramp=0
	   ENDIF ELSE IF splitramp_par.model EQ 1 THEN BEGIN
	      pseudoramp = FIX(ROUND((nndr+1) / splitramp_par.value))  
	      IF pseudoramp LT 2 THEN pseudoramp = 0
	      IF pseudoramp GT nndr THEN pseudoramp = 0
	   ENDIF ELSE pseudoramp = 0
	   DEFSYSV, '!subd_ramp', pseudoramp
	ENDIF  ELSE  DEFSYSV, '!subd_ramp', 0
;	tint = tint / div_ramp 


	  TIME = tmperd.time

; Get index with the ramp ends.
          GET_RAMPEND, time, dr, tint, rampflag, rampend, slopdim, $ 
	     SUB_RAMP =  pseudoramp, datred=datred		     
	  erdflag = tmperd.flag
	      
	  ylin = tmperd.conp
	     
; Accomodate variables into those to be passed
	  ytime = FLTARR(slopdim)
	  yslop = FLTARR(npix,slopdim)
	  yslopr = yslop
	  yresi = yslop
	  ynros = yslop
	  yflag = FIX(yslop)

	  ; Get all severe flags individually per pixel
	  get_validflag, tmperd.flag, 'ERD', redflag, $
	    det=det, /allpix

; Now compute medians of chopper positions, temperatures, etc
        ind_sev_st = SET_INDSEVERE('ERD',['GENDISC'],/REPLACE)
	get_validflag, tmperd.flag(0,*), 'ERD', stflag, INDSEVERE= ind_sev_st  
	RED_RAMPS, tmperd, stflag, chincr, rampend, otf, dr, ychop, ystep, yfcsn, $
	       ytemp, yfcs1, yfcs2, ytkey, yrpid, ycflg, yotf 


; Now fit ramps for all pixels
	  FOR IPIX = 0, npix-1 DO BEGIN
	     erdflag = redflag(ipix,*)
             conp0 = tmperd.conp(ipix,*)
             FIT_RAMPS, time, conp0, ndeg, erdflag, rampend, dr, tylin, $
			tyslop, tyslopr, tyflag, tytime, tyresi, tynros
	     ylin(ipix,*) = tylin
	     yslop(ipix,*) = tyslop
	     yslopr(ipix,*) = tyslopr
	     yresi(ipix,*) = tyresi
	     ynros(ipix,*) = tynros
	     yflag(ipix,*) = tyflag
	     IF ipix EQ 0 THEN ytime = tytime
	  ENDFOR   
						    

;  Put the fitting results into the ERD structure
	  tmperd.FITV = YLIN

;  Accomodate data into the srd structure and the srd handles after
;  creating the temporary ones                  
	  PHTSRDT = {PHTSRD_TYPE}
	  ;create the handles
	  FOR ihan=SRD_FIRST_HANDLE, SRD_LAST_HANDLE DO $
	     PHTSRDT.(ihan) = HANDLE_CREATE()

;  Change the header according to the selection criteriae 
	  DER_SHDR, ERDHDR, NDEG, SRDHDR

; Correct Raster Points ID not assigned (rpid=[0b,0b])
	  rpid_check, yrpid, yflag, /VERBOSE

;
; Check chopper position and step numbering
;
	  cmode = {cste:conv_cst.cste(record), cmod:conv_cst.cmod(record), $
	  	   cinc:conv_cst.cinc(record), camp:conv_cst.camp(record), $
	  	   fcs:yfcsn}
	  
 	   CHECK_CHOP, YCHOP, YCFLG, NCHANGED
	   step_corr = CHECK_STEP(ychop, ystep, ytime, cmode, ycflg, $
	      ADMN=tmperd.ADMN)
	   ystep = step_corr

	   indflag = WHERE(ycflg NE 0, nflag)
	   
	   IF nflag GT 0 THEN BEGIN
	    decode_admn, tmperd.ADMN, name_str
	    PRINT, name_str+ ': '+ STRTRIM(nflag,2) + $
	     ' signals without chopper assignment ' 
	    tmpflag = yflag*0
	    tmpflag(*,indflag) = 2 
	    ENCODE_PIAFLAG, 'SRD', 'NOCHOP', yflag, yflag_new, tmpflag, $ 
	       DET=det, /ALLPIX
	    IF npix EQ 1 THEN yflag(0,*) = yflag_new $
	        ELSE yflag = yflag_new   
	   ENDIF
	    
;
;
; check ON_TARGET flag after 2 sec of changing RPID 
          get_rastersteps, yrpid, raststeps, rpid_flag_new, TIME = YTIME
          rpid_flag_new = rpid_flag_new + yotf
	  ind = WHERE(rpid_flag_new NE 0, count)
	  IF count GT 0 THEN rpid_flag_new(ind) = 1
	   
          encode_piaflag,'SRD','NOTF',YFLAG,YFLAG_NEW,rpid_flag_new, $
                DET=det,/ALLPIX
 	  IF npix EQ 1 THEN yflag(0,*) = yflag_new $
	        ELSE yflag = yflag_new   
               
	  get_rpchopstep, raststeps, ystep, rpchopstep, in_plat_pos
	  plat_num = max(rpchopstep)

;
; 	  
; Prior to write it give uncertainties to the signals with unc=0
	  INDF1 = WHERE(yflag EQ 1, nf1)
	  IF nf1 EQ 0 THEN GOTO, bypass
	  
	  FOR ipix=0,npix-1 DO BEGIN
 	   FOR irpch = 1L, plat_num DO BEGIN
 	    ind_rpch = WHERE(rpchopstep EQ irpch)
	    ind_not0 = WHERE(YSLOP(ipix,ind_rpch) NE 0., nind_not0)
	    IF nind_not0 GT 0 THEN BEGIN
	     rel_unc = YSLOPR(ipix,ind_rpch(ind_not0)) / $
	     		YSLOP(ipix,ind_rpch(ind_not0))
	     erd_flagged = WHERE(yflag(ipix,ind_rpch) EQ 1 OR $
	    		         rel_unc LT 0.001, ncount)
	     IF ncount GT 0 THEN BEGIN
		sloppies = yslop(ipix,ind_rpch)
		sloppuns = yslopr(ipix,ind_rpch) 
	        check_unc, sloppies, sloppuns, abs_min=0.1*MEDIAN([sloppuns])
		yslopr(ipix,ind_rpch) = sloppuns
	     ENDIF
	    ENDIF
	   ENDFOR
	  ENDFOR
	  FXADDPAR, srdhdr, 'PIA_ECOR', 1, $
	 "Unc. assignment performed for ramps with 2 RO", $
	 BEFORE='XTENSION'
bypass:

          PHTSRDT.UNIT = tmperd.UNIT
          PHTSRDT.DET  = DET
	  PHTSRDT.ADMN = tmperd.ADMN
          PHTSRDT.ORDR = tmperd.ORDR
          PHTSRDT.DIMS = N_ELEMENTS(CONP0)
          PHTSRDT.SLDI = N_ELEMENTS(YSLOP) / NPIX
          PHTSRDT.RCHP = tmperd.RCHP
	  HANDLE_VALUE, PHTSRDT.HDR,SRDHDR, /SET
          HANDLE_VALUE, PHTSRDT.SLOP, YSLOP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.SLUN, YSLOPR, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FLAG, YFLAG, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.STIM, YTIME, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.CHOP, YCHOP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.STEP, YSTEP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCSN, YFCSN, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.TEMP, YTEMP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCS1, YFCS1, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCS2, YFCS2, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.TKEY, YTKEY, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.RPID, YRPID, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.RESI, YRESI, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.NROS, YNROS, /SET, /NO_COPY

; Update the PHTSRD structure using the temporary structure
	  PHTSRD = [PHTSRD, PHTSRDT]
; Update the PHTSRD value in the administration file using the index
	  SRDPOS = N_ELEMENTS(PHTSRD) - 1
;	  INDADM = WHERE (ADM.NAME EQ PHTSRDT.ADMN)
;	  ADM(INDADM).PHTSRD = SRDPOS
	  SRDPS = SRDPOS   
	RETURN
	END
