;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	PROCESS_TMPSRD
; PURPOSE:
;	Reduce PHT data from Signal per Ramp to Signal per Chopper Plateu,
;	taken into account diverse selections or flags, and put
;	averages, medians, uncertainties, etc. per chopper plateau into the 
;	structure phtscp. It uses a temporary structure instead of the 
;	dynamical structure as process_srd 
; CATEGORY:
;	PIA - processing
;                
; CALLING SEQUENCE:
;	PROCESS_TMPSRD, FILES, HDR, SCPPS, driftcorr=driftcorr
;
; INPUT PARAMETERS:
;	tmpdata	: structure to be processed
;	
; KEYWORDS:
;	DRIFTCORR: 	If set, drift analysis performed
;	DRIFTMOD:  	If set, drift modelling performed
;	SLOP_VAR:  	IF set, slopes variation analysis performed
;	DRIFTPAR_STR: 	For returning the structure with drift modelling
;			results
;
; OPTIONAL INPUT:
;	HDR	: a header to be taken instead of tmpdata.hdr
;
; OPTIONAL OUTPUT:
;	SCPPS	: Array containing the positions of the produced SCP 
;		  data within the PHTSCP structure 
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtacc
;               phtscp
;		phtdrift
;       PIA Procedure(s):
;               der_scphdr
;               get_rastersteps
;               get_rpchopstep
;		meas_mann
;		slop_var
;       Astronomical Library:
;               avg
;               sigma
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel - ESA/SAI
;	Modified (last) by JA + CG 			 February 1997
;	V6.1
;	Bug for last point manually discarded fixed (CG) March 1997
;	Call to fitpar_str_init using CMODE (JA+CG)	 November 1997
;	V6.6
;	Pattern calibration added (CG)			 June 1999
;	 + changed to bitflag system
;	Recognition of nodding case added (CG)		 June 1999
;	V7.4			 
;	Nodding case also for 3xM raster (CG)		July 1999
;	+ fix for P32					September 1999
;	V8.0
;	Bug fixed: STEP variable in nodding case (CG)	November 1999
;	V8.1
;	Bug fixed: 3x1 P32 case /= nodding (CG)		February 2000
;	V8.2
;	Using biweight mean for computing chopper signal values if
;	 system variables set accordingly (CG)		May 2002
;	V10.0
;-
PRO PROCESS_TMPSRD, TMPDATA, HDR, SCPPS, DRIFTCORR=driftcorr, $
		DRIFTMOD=driftmod, SLOP_VAR=SLOP_VAR, $
		DRIFTPAR_STR = driftpar_str, DRIFTPS, PATTERN=pattern, $
		NO_SCPBUFF =no_scpbuff,  TMPSCP = phtscpt

;
; Needed common blocks :
@adm
@phtscp
@phtacc
@phtdrift
COMMON pht_mann_st, conflvl, intdiv, minp, tmann_min
COMMON pht_drift_st, drift_stats
COMMON pia_drift_par, pia_drift_par


;	ON_ERROR, 2                                          
;
; Check the number of parameters
;
	IF N_PARAMS() LT 1 THEN MESSAGE, $
	   'Syntax: PROCESS_TMPSRD, TMPDATA, [HDR, SCPPOS]'
; For each measurement chosen
;
 
	DRIFTPS = 0
	tmpdata1 = tmpdata

; Take the input HDR if present
          IF KEYWORD_SET(HDR) THEN SRDHDR = HDR ELSE $
	  	SRDHDR = tmpdata.HDR

	  time = tmpdata.stim
	  slop = tmpdata.slop
	  slun = tmpdata.slun
	  flag = tmpdata.flag
	  chop = tmpdata.chop
	  step = tmpdata.step
	  fcsn = tmpdata.fcsn
	  temp = tmpdata.temp
	  fcs1 = tmpdata.fcs1
	  fcs2 = tmpdata.fcs2
	  tkey = tmpdata.tkey
	  rpid = tmpdata.rpid
	  resi = tmpdata.resi
	  nros = tmpdata.nros


; Take care of the different array dimensions for different detectors
	  det = tmpdata.det
	  npix = pixel_det(det,/ALLPX)

	  ntot = tmpdata.sldi
	  oldstep = 1
	  position = 0
	  plat_num = 1

	  in_plat_pos = intarr(ntot)
	  plat_pos = intarr(ntot)
                
; take care of those signals without a chopper step assigned
	  fsig = 0L

	  indnas = WHERE(step EQ 0, nnas)
	  IF nnas GT 0 THEN BEGIN
		plat_pos(indnas) = 0
		fsig = nnas   ; assuming step=0 can be just at begin
	  ENDIF
	  
;	  FOR isig=fsig,ntot-1 DO BEGIN
;		IF step(isig) ne oldstep AND step(isig) NE 0 THEN BEGIN
;		  in_plat_pos(isig) = 1
;		  oldstep = step(isig)
;		  plat_num = plat_num + 1 
;		  plat_pos(isig) = plat_num
;		  position = 0
;	  	ENDIF
;	  	position = position + 1
;	  	in_plat_pos(isig) = position
;		plat_pos(isig) = plat_num
;	  ENDFOR
	  

; Define discarded signals arrays (OLD, only for docu here)
;	  redflag = intarr(npix,ntot)
;	  sig_dis_com = intarr(ntot)

;
;  The complication of having unsynchronized chopper plateaux and raster points
;  is solved in the way, that if a chopper plateau has started on a raster
;  point, the rest of this chopper plateau is ignored, in case of a chopped
;  measurement. In case of a staring measurement the raster points are taken as
;  if they were chopper plateaux.
;
	  get_rastersteps, tmpdata.rpid, raststeps, time=time, rpflag, $
	   /in_rast, in_rast_pos
	  get_rpchopstep, raststeps, tmpdata.step, rpchopstep, in_plat_pos

; define if biweight has to be used or not
	  IF max(tmpdata.step) GT 1 AND !BIWEIGHT_C THEN $
	  	biweight=1 ELSE biweight=0
	  IF max(tmpdata.step) EQ 1 AND !BIWEIGHT_S THEN $
	  	biweight=1 ELSE biweight=0
	  
	  get_validflag, tmpdata.flag, 'SRD', redflag, $
	  	det=tmpdata.det, /ALLPIX	  

	  plat_num = max(rpchopstep)
	  plat_pos = rpchopstep

	  
	  sig_dis_com = redflag(0,*)	  
	  FOR ipix = 1,N_ELEMENTS(redflag(*,0))-1 DO $
	   sig_dis_com = sig_dis_com * redflag(ipix,*)
	  sig_dis_com = reform(sig_dis_com)	  
	  


	  ttime = fltarr(plat_num)
	  taver = fltarr(npix,plat_num)
	  tavun = taver 
	  tmedi = taver
	  tq1me = taver
	  tq3me = taver
	  tflag = intarr(npix,plat_num)
	  tchop = ttime
	  tstep = intarr(plat_num)
	  tfcsn = tstep
	  ttemp = ttime
	  tfcs1 = ttime
	  tfcs2 = ttime
	  ttkey = ttime
	  trpid = bytarr(2,plat_num)
	  tarsi = taver
	  tandr = taver
	  tavsd = taver

	  FOR iplat = 0L, plat_num-1  DO BEGIN
	     indall = WHERE (plat_pos-1 EQ iplat, nnc)
	     IF nnc EQ 0 THEN BEGIN
		txt = 'Missing plateau position'+ STRING(iplat+1)
		MESSAGE, /INFO, txt
		missplat = 1
		GOTO, escape 
             ENDIF
	     indsel = WHERE (plat_pos-1 EQ iplat AND sig_dis_com EQ 0, ncount)
	     IF ncount GT 0 THEN BEGIN

; in case that there are chopper plateaux elements common to two or more
; different raster points take just the first part of them and forget the
; rest (only feasible solution for unsynchronized plateaux)

                indin = WHERE(raststeps(indsel) EQ raststeps(indsel(0)),nindi)
		ttime(iplat) = avg(time(indsel(indin)))
		 FOR ipix=0,npix-1 DO BEGIN
	            ind = $
		      WHERE (redflag(ipix,indsel(indin)) EQ 0, nc)
                    IF nc GT 0 THEN BEGIN
		        ind = indsel(indin(ind))
			weight_mean, slop(ipix,ind), $
			  slun(ipix,ind), avwe, unwe, avsi, biweight=biweight  
			taver(ipix,iplat) = avwe  
			tavun(ipix,iplat) = unwe
			tavsd(ipix,iplat) = avsi	
			tmedi(ipix,iplat) = median(slop(ipix,ind))
			tarsi(ipix,iplat) = avg(resi(ipix,ind))
			tandr(ipix,iplat) = median(nros(ipix,ind))
			indmed = $
		where(slop(ipix,ind) LT tmedi(ipix,iplat),nquart)
			if nquart GT 0 THEN $
		tq1me(ipix,iplat) = median(slop(ipix,ind(indmed))) $
			ELSE tq1me(ipix,iplat) = 0.
			indmed = $
		where(slop(ipix,ind) GE tmedi(ipix,iplat),nquart)
		tq3me(ipix,iplat) = median(slop(ipix,ind(indmed)))
			IF nc EQ 1 THEN $
			  tflag(ipix,iplat) = 1 ELSE $
			  tflag(ipix,iplat) = 0
		    ENDIF ELSE BEGIN
			taver(ipix,iplat) = 0.  
			tavun(ipix,iplat) = 0.	
			tmedi(ipix,iplat) = 0.
                	tq1me(ipix,iplat) = 0.
                	tq3me(ipix,iplat) = 0.
			tarsi(ipix,iplat) = 0.  
			tandr(ipix,iplat) = 0	
			tavsd(ipix,iplat) = 0	
			tflag(ipix,iplat) = 2
		    ENDELSE
		  ENDFOR
                tchop(iplat) = median(chop(indsel(indin)))
                tstep(iplat) = median(step(indsel(indin)))
                tfcsn(iplat) = median(fcsn(indsel(indin)))
                ttemp(iplat) = median(temp(indsel(indin)))
                tfcs1(iplat) = median(fcs1(indsel(indin)))
                tfcs2(iplat) = median(fcs2(indsel(indin)))
                ttkey(iplat) = median(tkey(indsel(indin)))
                trpid(*,iplat) = rpid(*,indsel(0))
             ENDIF ELSE BEGIN
                indin = WHERE(raststeps(indall) EQ raststeps(indall(0)),nindi)
		ttime(iplat) = avg(time(indall(indin)))
		taver(*,iplat) = 0.  
		tavun(*,iplat) = 0.	
		tmedi(*,iplat) = 0.
                tq1me(*,iplat) = 0.
                tq3me(*,iplat) = 0.
                tflag(*,iplat) = 2
		tarsi(*,iplat) = 0.  
		tandr(*,iplat) = 0	
		tavsd(*,iplat) = 0.	
                tchop(iplat) = median(chop(indall(indin)))
                tstep(iplat) = median(step(indall(indin)))
                tfcsn(iplat) = median(fcsn(indall(indin)))
                ttemp(iplat) = median(temp(indall(indin)))
                tfcs1(iplat) = median(fcs1(indall(indin)))
                tfcs2(iplat) = median(fcs2(indall(indin)))
                ttkey(iplat) = median(tkey(indall(indin)))
                trpid(*,iplat) = rpid(*,indall(0))
             ENDELSE
	     missplat = 0
escape: 
             IF missplat EQ 1 THEN tflag(*,iplat) = 2
          ENDFOR
          
	  IF KEYWORD_SET(pattern) THEN BEGIN
; code taking from ccalsrd
		patt = FXPAR(srdhdr, 'PR_PATT', count=count)
		IF count EQ 0 THEN BEGIN
MESSAGE, /INFO, 'Warning: Processing a non "pattern" derived SRD ' 
MESSAGE, /INFO, 'using a "pattern" procedure ?????, will try... ' 
		ENDIF

; Chopper loss correction: derive the signal difference (src-bkg)


		taver=taver*0.
		tavun=taver

		lpix = pixel_det(tmpdata.det)

		give_cs, tmpdata, cst, rec, cst_conv
  
		tc = 2.^(7-cst.intt(rec)) * 2^cst.ndr(rec)

 IF KEYWORD_SET(verbose) THEN $
 	print, "Applying chopper loss correction for tc =", tc

		for npix=0,lpix-1 do begin 
		   sig=tmpdata.slop(npix,*)
		   unsig=tmpdata.slun(npix,*)

		   w=1/(unsig*unsig)
   
		   kdiff=0.
		   onsig=0.
		   offsig=0.

		   choploss, det=tmpdata.det, tchp=tc, sig=sig, selpix=npix, $
		     kdiff=kdiff, onsig=onsig, offsig=offsig, unsig=unsig, $
		     unonsig=unonsig, unoffsig=unoffsig


		   taver[npix,0]=offsig
		   taver[npix,1]=onsig
		   tavun[npix,0]=unoffsig
		   tavun[npix,1]=unonsig
   
		endfor

		FXADDPAR, srdhdr, 'PRS_PATT', 1, $
	   		' Processing SRD to SCP using Pattern procedure ', $
	   		BEFORE='XTENSION'
	  ENDIF
	       	   
;  Apply stability testing to staring measurements (also rasters)
	  struct_mann = -1.
	  IF KEYWORD_SET(DRIFTCORR) THEN BEGIN
;  Check applicability of Mann algorithm
	  	IF N_ELEMENTS(minp) EQ 0 THEN minp=8
	  	IF max(in_plat_pos) GE minp THEN BEGIN
	  	 MEAS_MANN, tmpdata, rpchopstep, redflag, srdhdr, $
	  		taver, tavun, tavsd, struct_mann 
			
	         npix_stab = pixel_det(tmpdata.det)
	         FOR istab=0, npix_stab-1 DO BEGIN		
	          IF struct_mann.stabflag(istab) GE 2 THEN BEGIN 
	           nspix = STRTRIM(istab+1,2) 	
		   txt = 'Mann diagnostic: not transient stabilized measurement'
		   PRINT, dec_admn(tmpdata.admn) + ': Pixel '+ nspix + '- '+ txt 
		   FXADDPAR, srdhdr, 'PRS_MAN'+nspix, 1, $
	   		' Not transient stabilized pixel measurement ', $
	   		BEFORE='XTENSION'
	          ENDIF
	         ENDFOR 
		ENDIF 		
	  ENDIF

;  Compute slope variations 
	  struct_slopv = -1.
	  IF KEYWORD_SET(SLOP_VAR) THEN BEGIN
		slopv_flag = redflag * 2
	  	SLOP_VAR, tmpdata, rpchopstep, slopv_flag, struct_slopv
	  ENDIF
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  Apply drift modelling to all measurements (if chopped use chopped mode!)
;
	  fit_par_str = -1.
	  IF NOT KEYWORD_SET(DRIFTMOD) THEN driftmod = 0
	  
	  IF (driftmod mod 2 eq 1) THEN BEGIN

	  ; initial parameter for this measurement are set from outside.
	  ; It will used mainly for testing purposes
	  ;if keyword_set(driftpar_str) then fit_par_str = driftpar_str $
	  ;     else fitpar_str_init, tmpdata, fit_par_str, error, ffunct, $ 
	  ;          /ONLY_MEASPAR       

	   srd_flag = redflag * 2
	   tmpdata1.flag = srd_flag
	  
	   give_cs, tmpdata1, cst, record, conv_cst, error=cs_error
	   IF cs_error EQ 0 THEN tmpcmode = conv_cst.cmod(record)  $
	     ELSE  tmpcmode = ''

	  
	   fitpar_str_init, tmpdata1, fit_par_str, error, ffunct, /ONLY_MEASPAR, $
	      CMODE = tmpcmode       

	   drift_mod_proc, tmpdata.det, tmpdata.stim, tmpdata.slop, $
	  	tmpdata.slun, srd_flag, tmpdata.rpid, tmpdata.step, $
	  	tmpdata.temp, fit_par_str, srdhdr, $
    		/MULTI_PIX, CMODE = tmpcmode


    
	  ENDIF

;  Accomodate drift info into drift structure
	  IF KEYWORD_SET(DRIFTMOD) or KEYWORD_SET(DRIFTCORR) or $
	    KEYWORD_SET(slop_var) THEN BEGIN
	   phtdrift_t = {phtdrift_type}
	   phtdrift_t.admn = tmpdata.admn
	   phtdrift_t.proct = !STIME
	   phtdrift_t.stab = HANDLE_CREATE()
	   phtdrift_t.slopv = HANDLE_CREATE()
	   phtdrift_t.drift = HANDLE_CREATE()
	   HANDLE_VALUE, phtdrift_t.slopv, struct_slopv, /set
	   HANDLE_VALUE, phtdrift_t.stab, struct_mann, /set
	   HANDLE_VALUE, phtdrift_t.drift, fit_par_str, /set
	   phtdrift = [phtdrift, phtdrift_t]
	   driftps = N_ELEMENTS(phtdrift)-1
	  ENDIF

; Recognize nodding measurement and change step accordingly
	  type = FXPAR(srdhdr, 'ATTTYPE', count=count)
	  aot = FXPAR(srdhdr, 'EOHAAOTN', count = caot)
; Avoid P32 case
	  IF caot GT 0 THEN $
	     IF STRTRIM(aot,2) EQ 'P32' THEN count = 0 
	  IF count GT 0 AND STRMID(type,0,1) EQ 'R' THEN BEGIN
	    rnpts = FXPAR(srdhdr, 'ATTRNPTS')
	    rdlns = FXPAR(srdhdr, 'ATTRDLNS')
	    IF rnpts GE 2 AND rdlns EQ 0 THEN BEGIN
	       tstep = REFORM(FIX(trpid(0,*))) 
	       FXADDPAR, srdhdr, 'PRS_NODD', 1, $
	   		' Nodding case recognized ', $
	   		BEFORE='XTENSION' 
	    ENDIF
	  ENDIF

;  Accomodate data into the scp structure and the scp handles after
;  creating the temporary ones  

;  Change the header according to the selection criteriae 
	  DER_SCPHDR, SRDHDR, SCPHDR
                
	  PHTSCPT = {				$
	  UNIT: TMPDATA.UNIT, 			$
	  DET:  TMPDATA.DET,			$
	  ADMN: TMPDATA.ADMN,			$
	  ORDR: TMPDATA.ORDR,			$
	  DIMS: plat_num,			$
	  HDR: SCPHDR,				$
	  TIME: TTIME,				$
	  AVER: TAVER,				$
	  AVUN: TAVUN,				$
	  MEDI: TMEDI,				$
	  Q1ME: TQ1ME,				$
	  Q3ME: TQ3ME,				$
	  FLAG: TFLAG,				$
	  CHOP: TCHOP,				$
	  STEP: TSTEP,				$
	  FCSN: TFCSN,				$
	  TEMP: TTEMP,				$
	  FCS1: TFCS1,				$
	  FCS2: TFCS2,				$
	  TKEY: TTKEY,				$
	  RPID: TRPID,				$
	  ARSI: TARSI,				$
	  ANDR: TANDR,				$
	  AVSD: TAVSD  }

	  IF NOT KEYWORD_SET(no_scpbuff) THEN BEGIN 
              ; Update the PHTSCP structure using the temporary structure
	      fill_scp, phtscpt
	      SCPPOS = N_ELEMENTS(PHTSCP) - 1
	      SCPPS = SCPPOS 
	  ENDIF


	RETURN
	END
