;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	RAMP_2T_DEGLITCH
;
; PURPOSE:
;	Routine for managing the application of the 2 threshold deglitching 
;	processing to a measurement, collecting statistics and updating the
;	measurement's header 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	ramp_2t_deglitch, tmpdata, erd_disc, newheader, degl_acc
;
; INPUT PARAMETERS:
;	TMPDATA		: Temporary Structure ERD 
;	ERD_DISC	: Flag array (0 = OK, 1 = bad flag)
;	NEWHEADER	: Header to be updated
;
; OUTPUT PARAMETERS:
;	TMPDATA		: Updated temporary Structure ERD (If encode_flag=1)
;	NEWHEADER	: Updated header
;	DEGL_ACC	: Glitches flag array (1=glitch flagged)
;
; KEYWORD INPUT
;	ENCODE_FLAG	: If set, update tmpdata.flag with bitcoded flag
; 
; RESTRICTIONS:
;
;
; METHOD:
;
; PROCEDURES USED:
;      PIA_internal:
;		decode_admn
;		pair_deglitch
;		pdegl_str_init
;		pixel_det
;
; COMMON BLOCKS:
;	pdegl_str
;	degl_summary
;	degl_string
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel (ESA-VILSPA/SAI)		January 1998
;	V7.0
;	Bug for 2 last PHT-S pixels fixed (CG)		February 1998
;	V7.1
;	Updated for use of bitflag system (CG+JA)	May 1999
;	V7.4
;-
PRO ramp_2t_deglitch, tmpdata, erd_disc, newheader, degl_acc, $
	ENCODE_FLAG=encode_flag

COMMON pdegl_str, pdegl_str
COMMON degl_summary, deglsum
COMMON degl_string, degltext

	IF N_ELEMENTS(pdegl_str) EQ 0 THEN PDEGL_STR_INIT

	npix = pixel_det(tmpdata.det)
	
	decode_admn, tmpdata.admn, meastxt
	deglsum=meastxt+ ' - Summary of Ramp (two Thresholds) Deglitching'
	deglsum=[deglsum,$
         '==================================================================']
	deglsum=[deglsum,' ']
	deglsum=[deglsum,'Parameters:']
	deglsum=[deglsum,'-----------']
	deglsum=[deglsum,'1st Sigma threshold: '+string(pdegl_str.thr1)]
	deglsum=[deglsum,'2nd Sigma threshold: '+string(pdegl_str.thr2)]
	deglsum=[deglsum,'Number of iterations: '+string(pdegl_str.nite)]
	deglsum=[deglsum,'Median filtering window: '+string(pdegl_str.medw)]
	deglsum=[deglsum,' ']
	
; look for different rastersteps and chopper plateaus
	get_rastersteps, tmpdata.rpid, raststeps
	get_rpchopstep, raststeps, tmpdata.step, rpchopstep
	

	degl_acc = erd_disc * 0
	FOR i=0,npix-1 DO BEGIN
		pair_deglitch, tmpdata, pdegl_str.medw, $
				pdegl_str.thr1, pdegl_str.thr2, $
				pdegl_str.nite, REFORM(erd_disc(i,*)), i, $
				pdegl_flag, rpchopstep
		ind = WHERE(pdegl_flag NE 0,nind)
		IF nind GT 0 THEN $
		  degl_acc(i,ind) = 1

		IF(n_elements(degltext) gt 1) then begin
			deglsum=[deglsum,' ','Pixel ' +string(i+1)]
			deglsum=[deglsum,' ']
			deglsum=[deglsum, $
'          Time            Strength        Mean Delta	  Strength/Mean    Flagged']
			deglsum = [deglsum, degltext]
		ENDIF
	ENDFOR

; if selected -> encode flag 
       IF KEYWORD_SET(ENCODE_FLAG) THEN BEGIN 
          encode_piaflag, 'ERD', 'GLITCH', tmpdata.flag, newflag, degl_acc, $
              DET = tmpdata.det, /ALLPIX
          tmpdata.flag = newflag    
       ENDIF    
           


	IF N_ELEMENTS(newheader) EQ 0 THEN newheader = tmpdata.hdr

; Log into header the relevant parameters used
	FXADDPAR, newheader, 'PR_D2T1', pdegl_str.thr1, before='XTENSION', $
	   ' Read-out 2 threshold deglitch parameter THR1'
	
	FXADDPAR, newheader, 'PR_D2T2', pdegl_str.thr2, before='XTENSION', $
	   ' Read-out 2 threshold deglitch parameter THR2'
	
	FXADDPAR, newheader, 'PR_D2IT', pdegl_str.nite, before='XTENSION', $
	   ' Read-out 2 threshold deglitch parameter NITE'
	
	FXADDPAR, newheader, 'PR_D2WI', pdegl_str.medw, before='XTENSION', $
	   ' Read-out 2 threshold deglitch parameter MEDW'
	
	
	RETURN
	END
