;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       RAMP_DEGLITCH
;
; PURPOSE:
;	Send a measurement to be ramp deglitched, perform statistics,
;	write into the header
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       RAMP_DEGLITCH, tmpdata, header, silent=silent
;
; INPUT:
;       tmpdata:
;		A real PHT-ERD structure, modified by this process
;	
; INPUT KEYWORDS:
;	silent:
;		Preventing from producing window messages
;
; OPTIONAL INPUT:
;	header:	The header to be modified. If not present, tmpdata.hdr is taken
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               phtacc
;               degl_ramps_cmn
;		degl_summary
;		degl_string
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)		January 1996
;	Modification history:
;	Updated using FXADDPAR - (CG)			December 1996
;	Version 5.5
;	Comment extended for low nr of readouts (CG)	September 1997
;	V6.5
;	Limited to detector pixels, text expanded (CG)	January 1998
;	V6.6
;	Upgraded with rampend info, use of bitflag
;	 system and text output (JAcosta + CG)		May 1999
;	Data reduction info added (CG)			May 1999
;	V7.4 
;-
PRO RAMP_DEGLITCH, tmpdata, newheader, erd_disc, rampend, SILENT=silent, $ 
     ENCODE_FLAG=encode_flag, TEXT_OUT=text_out

@phtacc
COMMON degl_ramps_cmn, fsig, niter, minp
COMMON degl_summary, deglsum
COMMON degl_string, degltext

	IF (N_ELEMENTS(fsig) EQ 0) THEN degl_ramps_init
	
	text_out = ''
	
	deglsum=strarr(1)

; If number of read-outs selected lower than minp do return
	
	junk = WHERE(ndrnacc NE 0, numero)

	IF ndr1nac THEN BEGIN
	  junk = WHERE(ndrnacc EQ 1, nnn) 
	  numero = numero + 1 - nnn
	ENDIF
	
	nrreads = tmpdata.nndr - numero
	IF nrreads LT minp THEN BEGIN
	 IF NOT KEYWORD_SET(SILENT) THEN BEGIN
		XDISPLAYFILE, "", $
                         TITLE = "Number of read-outs too low", $
                         WIDTH = 50, $
                         HEIGHT = 6, $
                TEXT = ["Number of read-outs lower than minimum required.", $
                     " ", "Change corresponding parameter for deglitching", $
                     "if correction should be done.", $
                     "Use 'Test processing' to specify a smaller minimum."]
	 ENDIF ELSE BEGIN
                TEXT_OUT = ["Number of read-outs lower than minimum required.", $
                     "No deglitching performed."]
	 ENDELSE        
               RETURN
        ENDIF

;	Get the number of pixels
	npix = PIXEL_DET(tmpdata.det)
	dims = tmpdata.dims

;keep the possibility to look into acceptance criteria for compatibility with
; versions PIA V<7.3
IF N_ELEMENTS(erd_disc) EQ 0 THEN BEGIN 
 ; Decode the pixel flag word
	DECODE_PIXF, tmpdata.pixf, suspect, chopon, chops, dr, otf 

 ; Look into the acceptance criteria, fill the acc status array
	ACC_FLG, dr, dims, tmpdata.nndr, status, $
			tmpdata.admn, otf
; Check also saturation
	erd_disc = BYTARR(npix, dims) 
	FOR ipix=0, npix-1 DO BEGIN 
	   pixstat = status
	   acc_volt, tmpdata.conp(ipix,*), pixstat, dr
	   erd_disc(ipix,*) =  pixstat
	ENDFOR	
			
ENDIF			
			
; get index with the ramp ends if not provided
IF N_ELEMENTS(rampend) EQ 0 THEN BEGIN 
   IF N_ELEMENTS(dr) EQ 0 THEN $
      DECODE_PIXF, tmpdata.pixf, suspect, chopon, chops, dr 
   give_cs, tmpdata, cst, rec
   GET_RAMPEND, tmpdata.time, dr, tmpdata.tint, rampflag, rampend, $
       datred=cst.drs(rec)
ENDIF      
	  
	npix = PIXEL_DET(tmpdata.det)
	x = tmpdata.time

	decode_admn, tmpdata.admn, meastxt
	deglsum=meastxt+ ' - Summary of Ramp Deglitching'
       	deglsum=[deglsum,'====================================================']
	deglsum=[deglsum,' ']
	deglsum=[deglsum,'Parameters:']
	deglsum=[deglsum,'-----------']
	deglsum=[deglsum,'Sigma threshold: '+string(fsig)]
	deglsum=[deglsum,'Number of iterations: '+string(niter)]
	deglsum=[deglsum,'Minimum mumber of points per ramp: '+string(minp)]

	tmpflag = tmpdata.flag*0
	FOR I=0,npix-1 DO BEGIN
			y = tmpdata.conp(i,*)
			pixstat = erd_disc(i,*)
	  		DEGL_RAMPS, X, Y, pixstat, rampend, deglacc, degltext 
			indglt= WHERE(deglacc GT 0, nglt)
			IF nglt GT 0 THEN tmpflag(i,indglt) = 1
			if(n_elements(degltext) gt 1) then begin
				deglsum=[deglsum,' ','Pixel ' +string(i+1)]
				deglsum=[deglsum,' ']
				deglsum=[deglsum, $
                                '          Time            Strength        ' + $
				'Mean Delta	  Strength/Mean   Pos/ramp']
				deglsum=[deglsum, degltext]
			ENDIF
	ENDFOR
	
; if selected -> encode flag 
       IF KEYWORD_SET(ENCODE_FLAG) THEN BEGIN 
          encode_piaflag, 'ERD', 'GLITCH', tmpdata.flag, newflag, tmpflag, $
              DET = tmpdata.det, /ALLPIX
          tmpdata.flag = newflag    
       ENDIF    
; 		
; 
	IF N_ELEMENTS(newheader) EQ 0 THEN newheader = tmpdata.hdr

; Log into header the relevant parameters used
	FXADDPAR, newheader, 'PR_DGLF', fsig, before='XTENSION', $
	   ' Read-out deglitch parameter FSIGMA'
	
	FXADDPAR, newheader, 'PR_DGLI', niter, before='XTENSION', $
	   ' Read-out deglitch parameter NITER'
	
	FXADDPAR, newheader, 'PR_DGLP', minp, before='XTENSION', $
	   ' Read-out deglitch parameter MINP'
	

	RETURN
	END
