;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO READ_AAR, NAME, HEADER, BUFFER, PATH = PATH, TYPE=type
;+
; NAME:
;       READ_AAR
; PURPOSE:
;       Read a PHT-AAR (Auto Analysis Result) file 
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       READ_AAR, NAME, HEADER, BUFFER [, PATH = PATH ] [, TYPE = TYPE ] 
;
; INPUT:
;       NAME    : Name of AAR FITS file to be read
;
; OPT. INPUT KEYWORD:
;       PATH    : optional path for the file
;
; OPT. OUTPUTS:
;       HEADER  : String array containing the FITS binary table extension
;                 header
;	BUFFER	: The data itself (real structure)
;
; OPT. OUTPUT KEYWORD:
;       TYPE    : The type of the file read as quoted in phtaar
;
; 
; RESTRICTIONS:
;       File must be a valid PHT AAR FITS file
;	Except PHT Photometric Image
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtcs
;               phtaar
;       Astronomical Library:
;               sxpar
; MODIFICATION HISTORY:
;       Written by: Wai-Ming Tai (DIAS)		           Apr, 1996
;	Central Wavelengths added to C and P buffers (CG)  January 1997
;       PPAS, PCAS read error corrected              (IH)  May 1997
;	V6.3
;	Type keyword added (CG)				   Apr 1998
;	AAR types changed, preventing reading AAR < V9.0 (CG) Mar 2000
;	V8.2
;
;-

@phtaar
@adm
@phtcs
@phtfilter_prop
;
        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 3 THEN MESSAGE, $
          'Syntax:  READ_AAR, NAME, HEADER, BUFFER, [PATH = PATH]'

; create filename esp if path is set
        if keyword_set(PATH) then $
          filename = PATH + NAME $
        else filename = !FITDIR + NAME
;
; OPEN THE UNFORMATED FITS FILE AND READ THE FIRST PRIMARY RECORD, 2880 BYTES
        OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN

; read the primary header
        hdr = bytarr(80, 36, /nozero)
        readu, unit, hdr
        header = string( hdr > 32b)
        endline = where( strmid(header,0,8) EQ 'END     ', Nend )

; in the AAR products apart from the photometric image, extended header
; always exist, so we delete the end line in the primary header
        if Nend GT 0 then header = header( 0:endline(0) - 1 )

        while Nend EQ 0 do begin
            readu, unit, hdr
            hdr1 = string( hdr > 32b )
            endline = where( strmid(hdr1,0,8) EQ 'END     ', Nend )
            if Nend GT 0 then hdr1 = hdr1( 0:endline(0) - 1)
            header = [ header, hdr1 ]
        endwhile

; read the extended header
        readu, unit, hdr
        hdr1 = string( hdr > 32b)
        endline = where( strmid(hdr1,0,8) EQ 'END     ', Nend )

        if Nend GT 0 then hdr1 = hdr1( 0:endline(0))
        header = [ header, hdr1 ]

        while Nend EQ 0 do begin
            readu, unit, hdr
            hdr1 = string( hdr > 32b )
            endline = where( strmid(hdr1,0,8) EQ 'END     ', Nend )
            if Nend GT 0 then hdr1 = hdr1( 0:endline(0))
            header = [ header, hdr1 ]
        endwhile

; extract the file type from the header
        filetype = sxpar( header, 'FILENAME')
        admn = filetype
        meaname  = strmid(filetype, 4, 8)
        filetype = strmid(filetype, 0, 4)
        rowcount = sxpar( header, 'NAXIS2')
; look for VERSION - Actual types only valid for version >= 9.0
	olpvers = FXPAR(header, 'OLPVERS', count=count)
	IF count GT 0 THEN BEGIN
	  nnn = FIX(STRMID(olpvers,4,1))
	  IF nnn EQ 6 or nnn EQ 7 or nnn EQ 8 THEN BEGIN
	     MESSAGE, /INFO, 'Old type AAR: < V9.0 , returning'
	     RETURN
	  ENDIF
	ENDIF
; define the buffer type according to their file type
        split = 0
        case filetype of
            'PPAP': BEGIN
                buffer = replicate(fits_ppap_buf, rowcount)
                tmpbuf = replicate(ppap_buf, rowcount)
            END
            'PPAE': BEGIN
                buffer = replicate(fits_ppae_buf, rowcount)
                tmpbuf = replicate(ppae_buf, rowcount)
            END
            'PPAS': BEGIN
                buffer = replicate(ppas_buf, rowcount)
                tmpbuf = replicate(ppas_buf, rowcount)
            END
            'PLAP': begin
                buffer = replicate(fits_plap_buf, rowcount)
                tmpbuf = replicate(plap_buf, rowcount)
                split = 1
            end
            'PLAE': begin
                buffer = replicate(fits_plae_buf, rowcount)
                tmpbuf = replicate(plae_buf, rowcount)
                split = 1
            end
            'PLAS': begin
                buffer = replicate(fits_plas_buf, rowcount)
                tmpbuf = replicate(plas_buf, rowcount)
            end
            'PSAP': begin
                buffer = replicate(fits_psap_buf, rowcount)
                tmpbuf = replicate(psap_buf, rowcount)
                split = 1
            end
            'PSAE': begin
                buffer = replicate(fits_psae_buf, rowcount)
                tmpbuf = replicate(psae_buf, rowcount)
                split = 1
            end
            'PSAS': begin
                buffer = replicate(fits_psas_buf, rowcount)
                tmpbuf = replicate(psas_buf, rowcount)
            end
            'PCAP': BEGIN
                buffer = replicate(fits_pcap_buf, rowcount)
                tmpbuf = replicate(pcap_buf, rowcount)
            END
            'PCAE': BEGIN
                buffer = replicate(fits_pcae_buf, rowcount)
                tmpbuf = replicate(pcae_buf, rowcount)
            END
            'PCAS': BEGIN
                buffer = replicate(pcas_buf, rowcount)
                tmpbuf = replicate(pcas_buf, rowcount)
            END
            ELSE:
        ENDCASE

; reading the entire buffer
        readu, unit, buffer
; close the file
        free_lun, unit

; converting the field type
        ntags = n_tags( buffer )
        for t = 0, ntags - 1 do begin
            s = size( buffer.(t) )
            field_type = s( s(0) + 1 )
            temp = buffer.(t)
            case field_type of
                2: byteorder, temp, /ntohl
                4: byteorder, temp, /xdrtof
                else:
            endcase
            buffer.(t) = temp
        endfor

; putting into the phtaar structure

        tmpaar = { phtaar_type }
        if strmid(admn, 0, 2) EQ 'PS' or strmid(admn, 0, 2) EQ 'PL' then begin
            lambda = fxpar(header, 'LAMBDA*')
; change lambda from meters to micron
            lambda = lambda * 1.E6
            tmpbuf.lambda = lambda
            n_ele = n_tags(buffer)
            for i = 0, n_ele - 1 do $
              tmpbuf.(i) = buffer.(i)
            buffer = tmpbuf
        endif else BEGIN 
            n_ele = n_tags(buffer)
            for i = 0, n_ele - 1 do $
              tmpbuf.(i) = buffer.(i)
        endelse      

        junk = ['PPAP', 'PPAE', 'PCAP', 'PCAE']
        ind = WHERE(junk EQ strmid(admn, 0, 4), nfound)

        IF nfound GT 0 THEN BEGIN
            CASE STRMID(admn, 0,2) of
                'PP': BEGIN
                    cwave = [filtprop.p1cwl, filtprop.p2cwl, filtprop.p3cwl]
                    n = n_elements(tmpbuf.filt)
                    for i = 0, n-1 do $
; note that the tmpstr in here is array of structure
                    tmpbuf(i).cwvl = cwave(buffer(i).filt - 1)
                END
                'PC': BEGIN
                    cwave = [filtprop.c1cwl, filtprop.c2cwl]
                    n = n_elements(tmpbuf.filt)
                    for i = 0, n-1 do $
                      tmpbuf(i).cwvl = cwave(buffer(i).filt - 2)
                END
                ELSE: 
            ENDCASE
        ENDIF
        

        if split gt 0 then begin
            for i = 0, rowcount - 1 do begin
                tmpaar.admn = strmid(admn, 0, 2) + $
                  string(i, format='(I2.2)') + strmid(admn, 4, 8)
                tmpaar.type = filetype
                tmpaar.hdr  = handle_create()
                handle_value, tmpaar.hdr, header, /set
                tmpaar.data = handle_create()
                handle_value, tmpaar.data, tmpbuf(i), /set

                phtaar = [ phtaar, tmpaar ]
            endfor
        endif else begin
            tmpaar.admn = strmid(admn, 0, 2) + '00' + strmid(admn, 4, 8)
            tmpaar.type = filetype
            tmpaar.hdr  = handle_create()
            handle_value, tmpaar.hdr, header, /set
            tmpaar.data = handle_create()
            handle_value, tmpaar.data, tmpbuf, /set

            phtaar = [ phtaar, tmpaar ]
        endelse

	type = tmpaar.type
    END
