;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	READ_ALLPERIGEE
; PURPOSE:
;	Read times from all perigee passage from a file created from Mission
;	Planning PSFs
;
; CATEGORY:
;       PIA - utilities
;
; CALLING SEQUENCE:
;       READ_ALLPERIGEE
;INPUT: 
;	rev: Revolution number
;	date: Date in the format "16-DEC-1995  7:19:14.00". Is is compatible 
;		with tag unit as read from PIA data structures. 
;OUTPUT: 
;	orbpos: Orbital position in units of 2.4hrs
;
;KEYWORDS: 
;	update: it will update file !F_ALLPERIGEE.DAT using read_allpsf
;
;COMMON BLOCKS:
;       all_perigee
;
; PROCEDURES USED:
;       ASTRO LIB:
;		readcol
;       INTERNAL:
;		read_allpsf
;		unit2juldate
;
; MODIFICATION HISTORY:
;       Written by:     Jose Acosta (PIDT - MPIA)       December 1997
;	Added extrapolation (JAc)			February 1998
;
;-
PRO read_allperigee, ERROR=error, UPDATE = update
; THis program read the rev_date file and fill the common structure with 
; perigee passage for each orbit
COMMON ALL_PERIGEE, allperigee

; PSF_DIR is set to mppublic account
PSF_DIR = '~phtcalib/psf/'

error = 0

IF KEYWORD_SET(update) THEN BEGIN
   read_allpsf, PSF_DIR= PSF_DIR ; it will read all psf files
ENDIF

check_file = findfile(!F_ALLPERIGEE)
IF check_file(0) eq '' THEN BEGIN
  PRINT, STRING(7B), $
   'F_ALLPERIGEE not found, impossible to compute orbital position'
  ERROR = 1 
  RETURN
ENDIF  


readcol,!F_ALLPERIGEE, nrev, date, time, date_oop1, time_oop1, $ 
    date_ocl1, time_ocl1, date_oop2, time_oop2,date_ocl2, time_ocl2, $ 
    psf_name,format='I,A,A,A,A,A,A,A,A,A,A,A',/silent
n_psf = n_elements(psf_name)
full_date = date + ' ' + time
full_oop1 = date_oop1 + ' ' + time_oop1
full_ocl1 = date_ocl1 + ' ' + time_ocl1
full_oop2 = date_oop2 + ' ' + time_oop2
full_ocl2 = date_ocl2 + ' ' + time_ocl2

ndate = DBLARR(n_psf)
FOR i=0, n_psf-1 DO ndate(i) = unit2juldate(full_date(i))
obsopen1 = DBLARR(n_psf)
FOR i=0, n_psf-1 DO obsopen1(i) = unit2juldate(full_oop1(i))
obsclose1 = DBLARR(n_psf)
FOR i=0, n_psf-1 DO obsclose1(i) = unit2juldate(full_ocl1(i))
obsopen2 = DBLARR(n_psf)
FOR i=0, n_psf-1 DO obsopen2(i) = unit2juldate(full_oop2(i))
obsclose2 = DBLARR(n_psf)
FOR i=0, n_psf-1 DO obsclose2(i) = unit2juldate(full_ocl2(i))


; extrapolated after maximum revolution
nrev_max = MAX(nrev,nmax)
max_extrev = 980    ; expected maximum revolution of ISO
ndate_max = ndate(nmax)
obsopen1_max = obsopen1(nmax)
obsclose1_max = obsclose1(nmax)
obsopen2_max = obsopen2(nmax)
obsclose2_max = obsclose2(nmax)

nrev_min = MIN(nrev,nmin)
min_extrev = 13     ; extrapolation has to be done for revs. before 22, no PSFs
ndate_min = ndate(nmin)
obsopen1_min = obsopen1(nmin)
obsclose1_min = obsclose1(nmin)
obsopen2_min = obsopen2(nmin)
obsclose2_min = obsclose2(nmin)

;compute shift per revolution
delt_rev = unit2juldate(full_date(nmin+1)) - unit2juldate(full_date(nmin))
FOR ie = 1, nrev_min-min_extrev DO BEGIN 
 nrev = [nrev_min-ie,nrev]
 ndate = [ndate_min-ie*delt_rev,ndate]
 obsopen1 = [obsopen1_min-ie*delt_rev,obsopen1]
 obsclose1 = [obsclose1_min-ie*delt_rev,obsclose1]
 obsopen2 = [obsopen2_min-ie*delt_rev, obsopen2]
 obsclose2 = [obsclose2_min-ie*delt_rev, obsclose2]
 psf_name = ['Extrapolated',psf_name ] 
ENDFOR  


;compute shift per revolution
delt_rev = unit2juldate(full_date(nmax)) - unit2juldate(full_date(nmax-1))

FOR ie = 1, max_extrev-nrev_max DO BEGIN 
 nrev = [nrev,nrev_max+ie]
 ndate = [ndate,ndate_max+ie*delt_rev]
 obsopen1 = [obsopen1,obsopen1_max+ie*delt_rev]
 obsclose1 = [obsclose1,obsclose1_max+ie*delt_rev]
 obsopen2 = [obsopen2,obsopen2_max+ie*delt_rev]
 obsclose2 = [obsclose2,obsclose2_max+ie*delt_rev]
 psf_name = [psf_name, 'Extrapolated'] 
ENDFOR  


allperigee = {rev:0,date:0.D, psf:"", obsopen1:"", obsclose1:"", obsopen2:"",$
          obsclose2:""}
allperigee = REPLICATE(allperigee,N_ELEMENTS(nrev))

;;allperigee = {rev:nrev, date:dblarr(n_psf), psf:psf_name, obsopen1:obsopen1,$ 
;; obsclose1:obsclose1, obsopen2:obsopen2,obsclose2:obsclose2}
allperigee.rev = nrev
allperigee.psf = psf_name
allperigee.date = ndate
allperigee.obsopen1 = obsopen1
allperigee.obsclose1 = obsclose1
allperigee.obsopen2 = obsopen2
allperigee.obsclose2 = obsclose2

RETURN
END
