;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro read_fpgc, infile, ERROR=error 
;+
; NAME:
;     READ_FPGC
; PURPOSE:
;       Read an Focal Plane Geometry Calibration File and analyse contents
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
; INPUT:
; 	infile: name of FPGC file
;
; OPTIONAL INPUT:
;
; KEYWORD PARAMETERS:
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output is not valid
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: I. Heinrichsen
;	Oct 95 Version 3.5
;
;-
  on_error, 2
;
; initialize error status
	error = 1
; initialize debug
	debug = 1
	
; open input file

  openr, unit, infile, /get_lun, ERROR=err
  if (err NE 0) then begin
             goto, openerror
  endif
  
  line = ' '

; read the two header lines
  buffer = strarr(1)
  header0 = ' '
  header1 = ' '
  readf, unit, header0
  readf, unit, header1

; read all data records from FPGC file
  readf, unit, buffer
  while (not eof(unit)) do begin
    readf, unit, line 
       buffer = [buffer, line]
  end

; close FPGC file
  free_lun, unit
  
; extract header information
  instrument=strmid(header1,32,1)
  if (instrument NE 'P') then begin
	goto, notpht
  endif
  if (debug eq 1) then begin
  	print, ' '
  	print, 'Printing Header Information:'
  	print, ' '
  endif	 
  virtab=fix(strmid(header1,33,1))
  case virtab of
  	1:if (debug eq 1) then print, 'PHT virtual apeture used: PHT-P'
  	2:if (debug eq 1) then print, 'PHT virtual apeture used: PHT-C'
  	3:if (debug eq 1) then print, 'PHT virtual apeture used: PHT-S'
  	else: print, 'Virtual apeture undefined for PHT: ', virtab
  endcase
  npts=fix(strmid(header1,40,3))
  nlin=fix(strmid(header1,43,3))
  if (debug eq 1) then print, 'Number of raster points: ', npts
  if (debug eq 1) then print, 'Number of raster legs: ', nlin    
  dist_pts=fix(strmid(header1,64,4))
  dist_lin=fix(strmid(header1,68,4))
  if (debug eq 1) then print, 'Distance between raster points:',dist_pts
  if (debug eq 1) then print, 'Distance between raster lines:', dist_lin
  rota=float(strmid(header1, 74, 5))
  if (debug eq 1) then print, 'Rotation angle of raster in degrees:', rota 


; extract infromation from records
if (debug eq 1) then begin
	print,' '
	print,' '
	print, "Printing Data contents:"
	print,' '
endif
rline = fix(strmid(buffer(*),40,3))
rcolumn = fix(strmid(buffer(*),43,3))
if (debug eq 1) then print, 'Rline:', rline
if (debug eq 1) then print, 'Rcolumns:', rcolumn
ycoor = float(strmid(buffer(*),46,6))
zcoor = float(strmid(buffer(*),53,6))
if (debug eq 1) then print, 'Y Coordinates:', ycoor
if (debug eq 1) then print, 'Z Coordinates:', zcoor

; generate theoretical coordinates
ytheo=fltarr(nlin*npts)
ztheo=fltarr(nlin*npts)

for i=0,npts-1 do begin
	for j=0,nlin-1 do begin
		ztheo(j+i*npts)=(i-nlin/2)*dist_lin
		case (i mod 2) of  
			0: ytheo(j+i*npts)=(j-npts/2)*dist_pts
			1: ytheo(j+i*npts)=-(j-npts/2)*dist_pts
		endcase
	endfor 
endfor
if (debug eq 1) then begin
	print,' '
	print,' '
	print, "Printing theoretical data contents and differences:"
	print,' '
        print, 'Theoretical Y Coordinates: ',ytheo
        print, 'Theoretical Z Coordinates: ',ztheo
endif

;calculate absolute residua
diff=sqrt((ycoor-ytheo)*(ycoor-ytheo)+(zcoor-ztheo)*(zcoor-ztheo))

;calculate relative residua
ytheo_rel=ytheo-ytheo(0)
ztheo_rel=ztheo-ztheo(0)
ycoor_rel=ycoor-ycoor(0)
zcoor_rel=zcoor-zcoor(0)

diff2=sqrt((ycoor_rel-ytheo_rel)*(ycoor_rel-ytheo_rel)+$
           (zcoor_rel-ztheo_rel)*(zcoor_rel-ztheo_rel))
          
if (debug eq 1) then print, 'Absolute Differences: ',diff
if (debug eq 1) then print, 'Relative Differences: ',diff2

plot, ytheo, ztheo, title='Comparison of nominal and FPGC raster coordinates',$
      xtitle='S/C Y-coordinate [arcsec]',  ytitle='S/C Z-coordinate [arcsec]'
oplot, ycoor, zcoor
plot, diff
; everything ok.
	
	error = 0
	return

openerror:
	print, "Unable to open file:", infile
	return

notpht: 
	print, "Instrument ID is not PHT"
	return
end
