;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro read_asc_fcspow, fcspow, det, buffer, inputfile = inputfile

;+
; NAME:
;     READ_ASC_FCSPOW
; PURPOSE:
;       Read a PHT FCSPOW uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_ASC_FCSPOW, FCSPOW, DET, BUFFER, INPUTFILE = INPUTFILE 
;
; INPUT:
;
; OUTPUT:
;       FCSPOW: character string of '1' or '2'
;       DET: character string of 'C1', 'C2', 'P1', 'P2' and 'P3'
;       BUFFER: float array for the FCSPOW table
;
; OPTIONAL INPUT:
;       INPUTFILE: string type for the input uplink file, 
;                  default is either one of the following:
;                    p_fc1_c1.txt, p_fc1_c2.txt,
;                    p_fc2_c1.txt, p_fc2_c2.txt,
;                    p_fc1_p1.txt, p_fc1_p2.txt, p_fc1_p3.txt,
;                    p_fc2_p1.txt, p_fc2_p2.txt, p_fc2_p3.txt
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;-
  on_error, 2

  case det of
    'C1' : begin
      infile = 'p_fc' + fcspow + '_c1.txt'
      nkeys = 15
    end
    'C2' : begin
      infile = 'p_fc' + fcspow + '_c2.txt'
      nkeys = 11
    end
    'P1' : begin
      infile = 'p_fc' + fcspow + '_p1.txt'
      nkeys = 21
    end
    'P2' : begin
      infile = 'p_fc' + fcspow + '_p2.txt'
      nkeys = 5
    end
    'P3' : begin
      infile = 'p_fc' + fcspow + '_p3.txt'
      nkeys = 5
    end
    else: begin
      message, 'DETECTOR not valid. Accepted detectors are C1, C2, P1, P2 and P3 '
      return
    end
  endcase
  infile = !CALU + infile
  if keyword_set(inputfile) then infile = inputfile 

; read in the input file and take out those lines begin with 'comment'
; and with '#', also trim off the leading and trailing spaces.

  openr, unit, infile, /get_lun
  line = ' '


; first of all, count the number of lines and allocate the buffer before
; reading in the file to the buffer.
  c = 0
  while (not eof(unit)) do begin
    readf, unit, line
    c = c+1
  end

; allocate buffer
  buf = strarr(c)

; read in the lines
  c = 0
  point_lun, unit, 0
  while (not eof(unit)) do begin
    readf, unit, line
    buf(c) = line
    c = c+1
  end
  free_lun, unit

; get rid of the # and the COMMENT lines
  ind = where(strmid(buf, 0, 7) ne 'COMMENT' and strmid(buf, 0, 1) ne '#')
  buf = buf(ind)
  len = n_elements(buf)

; we are interested in the value table itself
; getting the nrows, now is in buf(5) 
  nrows_line = buf(5)
  tmp = strlen(nrows_line) - 18
  nrows = long(strmid(nrows_line, 18, tmp))

; the value is the first 6 lines plus the nkeys X 2 + 1
; change the nkeys for different uplink files
  begin_line = 6 + nkeys * 2
  endline = begin_line + nrows -1
  value = buf(begin_line: endline)
  value = strtrim(value, 2)
; note that we get rid of the first dummy line now

; In case there are varying space delimiter between each value,
; we parse the value line one at a time to extract the individual
; value to a token_buf.

  tab = string(9b)

; nrows is the number of row in the table, change this for different
; table 


; We only loop nkeys - 1, as the last token doesn't have any more blank
; space
    ind0 = nrows - 1
    ind = nkeys - 2

    token_buf = strarr(nkeys)
    fcspow_value = replicate(' ', nkeys, nrows)

  for i = 0, ind0 do begin
    line = value(i)

;   replace all the tab characters, just in case
    while strpos(line, tab) GE 0 do begin
      pos = strpos(line, tab)
      strput, line, ' ', pos
    endwhile

    
; ind is calculated before the parental for loop.
    for j = 0, ind do begin
      pos = strpos(line, ' ')
      token_buf(j) = strmid(line, 0, pos)
      len = strlen(line)
      line = strtrim(strmid(line, pos+1, len-pos-1), 1) 
    endfor
    token_buf(nkeys - 1) = line
    fcspow_value(*, i) = token_buf
   endfor 

; trs_ind start at the 2 pos and each having to skip one pos
; eg. [2, 4, 6, 8, 10]
  trs_ind = indgen((nkeys - 1) / 2) * 2 + 2
  fcspow_value(trs_ind, *) = strmid(fcspow_value(trs_ind, *), 3, 1)

; change this to reflect the ouput format
  buffer = float(fcspow_value)


; wrap the begin and end for rows equal to 33

  if nrows eq 33 then begin
      before = fltarr(nkeys, 7)
      after = fltarr(nkeys, 5)
      after(*,*) = 49.99
      before(trs_ind, *) = float(fcspow)
      after(trs_ind, *) = float(fcspow)
      before(0, *) = [1.78E-20, 3.16E-20, 5.62E-20, 1.00E-19, $
                      1.78E-19, 3.16E-19, 5.62E-19]
      after(0, *) = [1.78E-10, 3.16E-10, 5.62E-10, 1.00E-09, $
                      1.78E-09]
      tmpbuffer = fltarr(nkeys, 45)
      tmpbuffer(*, 0:6) = before(*,*)
      tmpbuffer(*, 7:39) = buffer
      tmpbuffer(*, 40:44) = after
      buffer = tmpbuffer
    end

  return

end
