;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro read_asc_pdetect, buffer, inputfile = inputfile

;+
; NAME:
;     READ_ASC_PDETECT
; PURPOSE:
;       Read a PHT PDETECT uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_ASC_PDETECT, BUFFER, INPUTFILE = INPUTFILE 
;
; INPUT:
;       buffer: float array for the all the P and C detectors
;       idark is buffer(0,*)
;       nepread is buffer(1,*)
;       respons is buffer(2,*)
;       eta is buffer(3,*)
;       The array are organised as:
;           index        detector type
;             0            P1
;             1            P2
;             2            P3
;             3            C1
;             4            C2
;             5            SS
;             6            SL
;
; OUTPUT:
;
; OPTIONAL INPUT:
;       INPUTFILE: string type for the input uplink file, 
;                  default is p_pdetect.txt
;
; OPTIONAL INPUT:
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;
;-
  on_error, 2


  if keyword_set(inputfile) then infile = inputfile else infile = 'p_detect.txt'

; read in the input file and take out those lines begin with 'comment'
; and with '#', also trim off the leading and trailing spaces.

  openr, unit, infile, /get_lun
  line = ' '

; the first line is a dummy
  buf = strarr(1)

  while (not eof(unit)) do begin
    readf, unit, line 
    if strmid(line, 0, 7) ne 'COMMENT' then begin
     if strlen(strtrim(line, 2)) gt 0 then begin
       if strmid(line, 0, 1) ne '#' then buf = [buf, strtrim(line, 2)]
     end
    end
  end
  free_lun, unit

; we are interested in the value table itself
  end_table = where(strmid(buf, 0, 12) eq 'END_INST_TAB')

; the value is the first 6 lines plus the nkeys X 2 + 1
; change the nkeys for different uplink files
  nkeys = 9
  begin_line = 7 + nkeys * 2
  value = buf(begin_line: end_table(0) - 1)
; note that we get rid of the first dummy line now

; In case there are varying space delimiter between each value,
; we parse the value line one at a time to extract the individual
; value to a token_buf.

  tab = string(9b)

; the first element in the pdetect_value is a dummy for the for loop
  pdetect_value = [[strarr(nkeys)]]

; nrows is the number of row in the table, change this for different
; table 

; change the nrows for different tables
  nrows = 7

; We only loop nkeys - 1, as the last token doesn't have any more blank
; space
    ind0 = nrows - 1
    ind = nkeys - 2

  for i = 0, ind0 do begin
    line = value(i)

;   replace all the tab characters, just in case
    while strpos(line, tab) GE 0 do begin
      pos = strpos(line, tab)
      strput, line, ' ', pos
    endwhile

    token_buf = strarr(nkeys)
    
; ind is calculated before the parental for loop.
    for j = 0, ind do begin
      pos = strpos(line, ' ')
      token_buf(j) = strmid(line, 0, pos)
      len = strlen(line)
      line = strtrim(strmid(line, pos+1, len-pos-1), 1) 
    endfor
    token_buf(nkeys - 1) = line
    pdetect_value = [[pdetect_value] , [token_buf]]
   endfor 

; Now we should have (nrows + 1) elements in pdetect_value where 
; the first one is
; a dummy but the rest of it would correspond to the pixel number
   pdetect_value = pdetect_value(*, 1 : nrows )

; change this to reflect the ouput format
  buffer = fltarr(4, 7)
  buffer(0, *) = float(pdetect_value(3,*))
  buffer(1,*) = float(pdetect_value(4,*))
  buffer(2,*) = float(pdetect_value(5,*))
  buffer(3,*) = float(pdetect_value(7,*))

  return

end
