;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_CALA, NAME, HDR, ILUNR, PATH = PATH
;+
; NAME:
;       READ_CALA
; PURPOSE:
;	Read a PHT CAL-A Data file filling the data into the 
;	IDL common PHTCLA. 
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_CALA, NAME, [HDR], [ILUNR]
;
; INPUT:
;       NAME 	: Name of PHT-CAL-A FITS file to be read
; OPTIONAL INPUT:
;       PATH    : path for the file
;
; OPT. OUTPUTS:
;	HDR	: String array containing the FITS binary table extension
;		  header
;
;       ILUNR   : Array containing the indices of the measurements in the
;                 PHTSPD structure
;
; RESTRICTIONS:
;       File must be a valid PHT-CAL-A FITS file
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               fxbintable
;               phtcla
;               phtcs
;       PIA Procedure(s):
;               mergehdr
;               phtcs_get_rec
;               read_cs
;       Astronomical Library:
;               fxbclose
;               fxbintable
;               fxbopen
;               fxbread
;               fxhread
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel - ESA/SAI
;	Modified (last) by CG			December 1996
;	V5.4
;-
;
;
@fxbintable
@adm
@phtcs
@phtcla
        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_CALA, NAME, [HDR], [ILUNR], [ PATH = PATH ]'
	DET = STRMID(NAME,1,2)
;  Read the compact status - change -> shape4
        CSNUM = STRMID(NAME, 4,8)
        CSNAME = 'PSTA        .FITS'
        STRPUT, CSNAME, CSNUM, 4
        if keyword_set(path) then $
          READ_CS, CSNAME, CSHDR, CSUN, PATH = PATH $
        else READ_CS, CSNAME, CSHDR, CSUN
 
; return to caller if compact status not found
  if CSUN EQ -1 then begin
        msg = 'Compact status: ' + CSNAME + ' not found, abort reading!'
        message, /info, msg
        return
  endif
;  Check if file has been already read
        ILUNR = WHERE(PHTCLA.admn EQ STRMID(NAME,0,12) , NLUN)
        IF NLUN EQ 0 THEN BEGIN
;
;  Open file. Extension always =1            
;
; create filename esp if path is set
        if keyword_set(PATH) then $
          file = PATH + NAME $
        else file = !FITDIR + NAME

	 OPENR, UNIT, FILE, /BLOCK, /GET_LUN
	 FXHREAD, UNIT, PHDR
	 FREE_LUN, UNIT
 	 FXBOPEN, UNIT, FILE, 1, EHDR
	 MERGEHDR, PHDR, EHDR, HDR
;  Read first the general record prefix
         FXBREAD, UNIT, TKEY, 'GPSCTKEY'
         FXBREAD, UNIT, RPID, 'GPSCRPID'
;  Adjust the keyword names to the positions on the FITS file
        QFLG= 4 
	KYID= 5
	FILT= 8
	APER= 9
	POLZ= 10
        STAT= 11
        DWEL= 12
        CPOS= 13
        FCS1= 14
        FCS2= 15
        TEMP= 16
        BIAS= 18
        MNSG= 19
        MNSU= 20
        MDSG= 21
        Q1SG= 22
        Q3SG= 23
        PLEN= 24
        FLAG= 25
;  Read the columns
	 FXBREAD, UNIT, QFLG, QFLG
	 FXBREAD, UNIT, KYID, KYID
	 FXBREAD, UNIT, FILT, FILT
	 FXBREAD, UNIT, APER, APER
	 FXBREAD, UNIT, POLZ, POLZ
	 FXBREAD, UNIT, STAT, STAT
	 FXBREAD, UNIT, DWEL, DWEL
	 FXBREAD, UNIT, CPOS, CPOS
	 FXBREAD, UNIT, FCS1, FCS1
	 FXBREAD, UNIT, FCS2, FCS2
 	 FXBREAD, UNIT, TEMP, TEMP
	 FXBREAD, UNIT, BIAS, BIAS
	 FXBREAD, UNIT, MNSG, MNSG
	 FXBREAD, UNIT, MNSU, MNSU
	 FXBREAD, UNIT, MDSG, MDSG
	 FXBREAD, UNIT, Q1SG, Q1SG
	 FXBREAD, UNIT, Q3SG, Q3SG
	 FXBREAD, UNIT, PLEN, PLEN
	 FXBREAD, UNIT, FLAG, FLAG
	 
; Set provisorily RPID to [1,1] and FLAG to 0
	for i=0,N_ELEMENTS(RPID(0,*))-1 do RPID(*,i) = RPID(*,i) * 0b  + [1b,1b]
	flag = FLAG*0 

;        
; Nr of measurements, total length, are the measurements staring or chopped?
;
         NMEAS = FXPAR(HDR,'NMEAS')
         NAX = FXPAR(HDR,'NAXIS2')
;        FPCMOD = FXPAR(HDR,'FPCMODE')
;
;  Prepare the arrays containing the measurements' initial and end times
;
         UNITID = STRMID(NAME,0,10)
         PHTCS_GET_REC, CSUN, PHTCST
         DD = PHTCST.DIMS
         PS = INTARR(DD)
         PE = PS
         PS = PS - 1
	 POS = PS
         UTST = LONARR(DD)
;  Read the data into different measurements using the CS ITKs
;         MATCH, TKEY, PHTCST.IKST, PS
         II = 0
         FOR I = 0,DD-1 DO BEGIN
          PP = WHERE(TKEY GE PHTCST.IKST(I) AND TKEY LE PHTCST.IKEN(I), ncount)
          IF (ncount GT 0) THEN BEGIN
                PS(II) = PP(0)
                PE(II) = PP(N_ELEMENTS(PP)-1)
		UTST(II) =  PHTCST.UTST(0,I)
		POS(II) = I
                II = II + 1
          ENDIF
         ENDFOR
         PS = PS(0:II-1)
         PE = PE(0:II-1)
         UTST = UTST(0:II-1)
	 POS = POS(0:II-1)

;  Read the ITK unit length in seconds - right now just from CS header
;  possible --> have to open again the CS file for the case it was read
;  before (thus READ_CS returning just the index)
         OPENR, CSUNIT, !FITDIR + CSNAME, /block, /get_lun
	 FXHREAD, csunit, cshdr
         TREFITKU = FXPAR(cshdr,'TREFITKU')
         CLOSE, csunit
         FREE_LUN, csunit
         FOR I = 0, N_ELEMENTS(PS) - 1 DO BEGIN
              PHTCLAT = {PHTCLA_TYPE}
              PHTCLAT.HDR = HANDLE_CREATE()
              PHTCLAT.TIME = HANDLE_CREATE()
              PHTCLAT.TKEY = HANDLE_CREATE()
              PHTCLAT.RPID = HANDLE_CREATE()
              PHTCLAT.QFLG = HANDLE_CREATE()
              PHTCLAT.KYID = HANDLE_CREATE()
              PHTCLAT.FILT = HANDLE_CREATE()
              PHTCLAT.APER = HANDLE_CREATE()
              PHTCLAT.POLZ = HANDLE_CREATE()
              PHTCLAT.STAT = HANDLE_CREATE()
              PHTCLAT.DWEL = HANDLE_CREATE()
              PHTCLAT.CPOS = HANDLE_CREATE()
              PHTCLAT.FCS1 = HANDLE_CREATE()
              PHTCLAT.FCS2 = HANDLE_CREATE()
              PHTCLAT.TEMP = HANDLE_CREATE()
              PHTCLAT.BIAS = HANDLE_CREATE()
              PHTCLAT.MNSG = HANDLE_CREATE()
              PHTCLAT.MNSU = HANDLE_CREATE()
              PHTCLAT.MDSG = HANDLE_CREATE()
              PHTCLAT.Q1SG = HANDLE_CREATE()
              PHTCLAT.Q3SG = HANDLE_CREATE()
              PHTCLAT.PLEN = HANDLE_CREATE()
              PHTCLAT.FLAG = HANDLE_CREATE()

;  Several detectors can be on one CS P file -> positions to be corrected	      
;  and also on one CS C file -> positions to be corrected	      
;;;;;;;;;;	      IF (strmid(DET,0,1) EQ 'P') THEN BEGIN
		detids = conv_subs(PHTCST.SUBS)
		indeq = WHERE(detids EQ det, ncount)
		IF ncount GT 0 THEN posnew = pos - indeq(0) ELSE BEGIN
			MESSAGE, /INFO, 'CS and Cal A are not corresponding'
			print, detids
			help, det
			RETURN
		ENDELSE 
;;;;;;;;;;	      ENDIF ELSE posnew = pos

              IF (POSNEW(I) LE 9) THEN $
                IDNAME = DET + '0' + $
                	STRTRIM(STRING(POSNEW(I)),1) + STRMID(NAME,4,8) $
               ELSE $
                IDNAME = DET + STRTRIM(STRING(POSNEW(I)),1) + STRMID(NAME,4,8)
;  Fill values
	      pia_conv_date, utst(I), date
              PHTCLAT.UNIT = date
              PHTCLAT.DET  = DET
	      PHTCLAT.ADMN = IDNAME
              PHTCLAT.ORDR = POSNEW(I)
              PHTCLAT.DIMS = PE(I)-PS(I)+1
;  Fill arrays
              DETRED = STRMID(DET,0,1)
              CASE DETRED OF
                'P': BEGIN
                  TBIAS = BIAS(PS(I):PE(I))
		  TMNSG = FLTARR(1,N_ELEMENTS(TBIAS))
		  TMNSU = TMNSG
		  TMDSG = TMNSG
		  TQ1SG = TMNSG
		  TQ3SG = TMNSG
		  TPLEN = TMNSG
		  TFLAG = INTARR(1,N_ELEMENTS(TBIAS))
		  TMNSG(0,*) = MNSG(PS(I):PE(I))
		  TMNSU(0,*) = MNSU(PS(I):PE(I))
		  TMDSG(0,*) = MDSG(PS(I):PE(I))
		  TQ1SG(0,*) = Q1SG(PS(I):PE(I))
		  TQ3SG(0,*) = Q3SG(PS(I):PE(I))
		  TPLEN(0,*) = PLEN(PS(I):PE(I))
		  TFLAG(0,*) = FLAG(PS(I):PE(I))
                END
                ELSE: BEGIN
		  IF (DET EQ 'C2') THEN TBIAS = BIAS(*,PS(I):PE(I)) $
				ELSE TBIAS = BIAS(PS(I):PE(I))
		  TMNSG = MNSG(*,PS(I):PE(I))
		  TMNSU = MNSU(*,PS(I):PE(I))
		  TMDSG = MDSG(*,PS(I):PE(I))
		  TQ1SG = Q1SG(*,PS(I):PE(I))
		  TQ3SG = Q3SG(*,PS(I):PE(I))
		  TPLEN = PLEN(*,PS(I):PE(I))
		  TFLAG = FLAG(*,PS(I):PE(I))
		END
	      ENDCASE

; Fill handles
; Add a dark current subtraction header keyword to avoid new subtraction\
	      FXADDPAR, hdr, 'PRS_DARK', 1, 'Dark Current Subtracted by Pipeline' 
              HANDLE_VALUE, PHTCLAT.HDR, HDR, /SET
              TIME = TKEY(PS(I):PE(I)) - TKEY(PS(I))
              HANDLE_VALUE, PHTCLAT.TIME, TIME * TREFITKU, /SET
              HANDLE_VALUE, PHTCLAT.TKEY, TKEY(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.RPID, RPID(*,PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.QFLG, QFLG(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.KYID, KYID(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.FILT, FILT(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.APER, APER(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.POLZ, POLZ(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.STAT, STAT(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.DWEL, DWEL(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.CPOS, CPOS(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.FCS1, FCS1(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTCLAT.FCS2, FCS2(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTCLAT.TEMP, TEMP(PS(I):PE(I)), /SET

;  Accomodate pixel numbering to the IA (telemetry) convention
	      IF DET EQ 'C2' THEN BEGIN
		c2p_pipe2ia, tbias, ttmp
		tbias = ttmp
		c2p_pipe2ia, tmnsg, ttmp
		tmnsg = ttmp
		c2p_pipe2ia, tmnsu, ttmp
		tmnsu = ttmp
		c2p_pipe2ia, tmdsg, ttmp
		tmdsg = ttmp
		c2p_pipe2ia, tq1sg, ttmp
		tq1sg = ttmp
		c2p_pipe2ia, tq3sg, ttmp
		tq3sg = ttmp
		c2p_pipe2ia, tplen, ttmp
		tplen = ttmp
		c2p_pipe2ia, tflag, ttmp
		tflag = ttmp
	      ENDIF
              HANDLE_VALUE, PHTCLAT.BIAS, TBIAS, /SET
              HANDLE_VALUE, PHTCLAT.MNSG, TMNSG, /SET
              HANDLE_VALUE, PHTCLAT.MNSU, TMNSU, /SET
              HANDLE_VALUE, PHTCLAT.MDSG, TMDSG, /SET
              HANDLE_VALUE, PHTCLAT.Q1SG, TQ1SG, /SET
              HANDLE_VALUE, PHTCLAT.Q3SG, TQ3SG, /SET
              HANDLE_VALUE, PHTCLAT.PLEN, TPLEN, /SET
              HANDLE_VALUE, PHTCLAT.FLAG, TFLAG, /SET
;  Update the PHTCLA structure using the temporary structure
              PHTCLA = [PHTCLA,PHTCLAT]
;  Does exist already an administration record for this measurement?
              ADMIND = WHERE(ADM.NAME EQ IDNAME, N)
;  Update the administration file using temporary structure
              IF (N EQ 1) THEN $
                ADM(ADMIND).PHTCLA = N_ELEMENTS(PHTCLA) - 1 $
               ELSE IF (N EQ 0) THEN BEGIN
                ADMT = {ADM_TYPE}
                ADMT.NAME = IDNAME
                ADMT.PHTCLA = N_ELEMENTS(PHTCLA) - 1
                ADMT.PHTCS = CSUN
                ADM = [ADM, ADMT]
              ENDIF ELSE PRINT, 'READ_CALA: Several identical ADM-records'
         ENDFOR
         FXBCLOSE, UNIT
         FREE_lun, unit
        ENDIF ELSE $
        PRINT, 'CAL-A Data already contained in structure, pos: ', ILUNR
	RETURN
        END
