;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_CS, NAME, HDR, ILUN, PATH = PATH, STATUS = status
;+
; NAME:
;       READ_CS
;
; PURPOSE:
;	Read a PHT Compact Status file filling the data into the IDL common
;	PHTCS. The file has to be a FITS file
;       Note that the PHTCS is organised as an array of structure which is
;       defined as:
;       PHTCS(i) : 
;                   unit : strings
;                   dim  : interger
;                   handle : handle point the collection of array
; 
; CATEGORY:
;	PIA - I/O
;
; CALLING SEQUENCE:
;       READ_CS, NAME, [HDR], [ILUN]
;
; INPUT:
;       NAME 	: Name of CS FITS file to be read
;
; KEYWORD:
;       PATH    : path for the file
;       STATUS  : return 0 if reading is okay, otherwise 1 on error
;
; OUTPUT:
;
; OPT. OUTPUTS:
;	HDR	: String array containing the FITS binary table extension
;		  header
;
;	ILUN	: Unit index in the common
;
;
; RESTRICTIONS:
;       File must be a valid PHT CS FITS file
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               fxbintable
;               phtcs
;       PIA Procedure(s):
;               mergehdr
;       Astronomical Library:
;               fxbclose
;               fxbintable
;               fxbopen
;               fxbread
;               fxhread
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel
;	Modified by: W M Tai
;	Oct 95 Version 3.5
;	Extended for Serendipity Mode CS files
;       optional path added (WMT)                March 1996
;	FREE_LUN included! (CG)			 June 1996
;	STATUS as keyword (CG)			 January 1997
;	V6.0
;-
;
;
@fxbintable
@phtcs
        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_CS, NAME,[ HDR], [ILUN], [PATH = PATH]'

	status = 0
;  Does the common exist? 
         NLUN = 0 
;  Check if file has been already read; in that case just return
;  the index value of the data avoiding unnecessary grow of the common 
           ILUN = WHERE(PHTCS.UNIT EQ STRMID(NAME,4,8) , NLUN)   
           ILUN = ILUN(0)
         IF NLUN EQ 0 THEN BEGIN 
;
;  Open header. Extension is always = 1                   
;
        if keyword_set(path) then $
           	FILE = path + NAME $
        else 	FILE = !FITDIR+NAME

; make sure the file exist before proceeding further
        res = findfile(file, count=count)
 
        if count EQ 0 then begin
	  status = 1
          msg = 'READ_CS.PRO: File ' + NAME + ' not found!'
          message, /info, msg
          return
        ENDIF
 
	   OPENR, UNIT, FILE, /BLOCK, /GET_LUN
	   FXHREAD, UNIT, PHDR
	   CLOSE, UNIT
	   FREE_LUN, unit
           FXBOPEN, UNIT, FILE, 1,EHDR
	   I = 0
	   I = WHERE(LUN EQ UNIT)
           DIMS = NAXIS2(I)
	   MERGEHDR, PHDR, EHDR, HDR
;  Read the Compact Status General Prefix  
           FXBREAD, UNIT, UKST, 'CSGPUKST'
           FXBREAD, UNIT, UKEN, 'CSGPUKEN'
           FXBREAD, UNIT, IKST, 'CSGPIKST'
           FXBREAD, UNIT, IKEN, 'CSGPIKEN'
           FXBREAD, UNIT, UTST, 'CSGPUTST'
           FXBREAD, UNIT, UTEN, 'CSGPUTEN'
           FXBREAD, UNIT, AOTN, 'CSGPOSN'
;  Read the specific PHT part - Extend for PSTI files (CS for serendipity)
	   FXBFIND, unit, 'TTYPE', col, val, nfound
	   ind = WHERE(val EQ 'PSTASUBS',ncount)
	   IF ncount GT 0 THEN BEGIN 
             FXBREAD, UNIT, SUBS, 'PSTASUBS'
             FXBREAD, UNIT, OPFO, 'PSTAOPFO'
             FXBREAD, UNIT, F1TS, 'PSTAF1TS'
             FXBREAD, UNIT, F2TS, 'PSTAF2TS'
             FXBREAD, UNIT, F1PS, 'PSTAF1PS'
             FXBREAD, UNIT, F2PS, 'PSTAF2PS'
             FXBREAD, UNIT, C1PS, 'PSTAC1PS'
             FXBREAD, UNIT, C2PS, 'PSTAC2PS'
             FXBREAD, UNIT, C3PS, 'PSTAC3PS'
             FXBREAD, UNIT, CMOD, 'PSTACMOD'
             FXBREAD, UNIT, CAMP, 'PSTACAMP'
             FXBREAD, UNIT, CINC, 'PSTACINC'
             FXBREAD, UNIT, CRES, 'PSTACRES'
             FXBREAD, UNIT, CSTE, 'PSTACSTE'
             FXBREAD, UNIT, DETA, 'PSTADETA'
             FXBREAD, UNIT, DRS, 'PSTADRS'
             FXBREAD, UNIT, MUX1, 'PSTAMUX1'
             FXBREAD, UNIT, MUX2, 'PSTAMUX2'
             FXBREAD, UNIT, XSTA, 'PSTAXSTA'
             FXBREAD, UNIT, D1OF, 'PSTAD1OF'
             FXBREAD, UNIT, D1GA, 'PSTAD1GA'
             FXBREAD, UNIT, D2GA, 'PSTAD2GA'
             FXBREAD, UNIT, D2OF, 'PSTAD2OF'
             FXBREAD, UNIT, FREQ, 'PSTAFREQ'
             FXBREAD, UNIT, NNDR, 'PSTANNDR'
             FXBREAD, UNIT, NDR, 'PSTANDR'
             FXBREAD, UNIT, INTT, 'PSTAINTT'
             FXBREAD, UNIT, MEAT, 'PSTAMEAT'
             FXBREAD, UNIT, MPC1, 'PSTAMPC1'
             FXBREAD, UNIT, MPC2, 'PSTAMPC2'
             FXBREAD, UNIT, MPC3, 'PSTAMPC3'
             FXBREAD, UNIT, MET, 'PSTAMET'
	ENDIF ELSE BEGIN 
             FXBREAD, UNIT, SUBS, 'PSTISUBS'
             FXBREAD, UNIT, OPFO, 'PSTIOPFO'
             FXBREAD, UNIT, F1TS, 'PSTIF1TS'
             FXBREAD, UNIT, F2TS, 'PSTIF2TS'
             FXBREAD, UNIT, F1PS, 'PSTIF1PS'
             FXBREAD, UNIT, F2PS, 'PSTIF2PS'
             FXBREAD, UNIT, C1PS, 'PSTIC1PS'
             FXBREAD, UNIT, C2PS, 'PSTIC2PS'
             FXBREAD, UNIT, C3PS, 'PSTIC3PS'
             FXBREAD, UNIT, CMOD, 'PSTICMOD'
             FXBREAD, UNIT, CAMP, 'PSTICAMP'
             FXBREAD, UNIT, CINC, 'PSTICINC'
             FXBREAD, UNIT, CRES, 'PSTICRES'
             FXBREAD, UNIT, CSTE, 'PSTICSTE'
             FXBREAD, UNIT, DETA, 'PSTIDETA'
             FXBREAD, UNIT, DRS, 'PSTIDRS'
             FXBREAD, UNIT, MUX1, 'PSTIMUX1'
             FXBREAD, UNIT, MUX2, 'PSTIMUX2'
             FXBREAD, UNIT, XSTA, 'PSTIXSTA'
             FXBREAD, UNIT, D1OF, 'PSTID1OF'
             FXBREAD, UNIT, D1GA, 'PSTID1GA'
             FXBREAD, UNIT, D2GA, 'PSTID2GA'
             FXBREAD, UNIT, D2OF, 'PSTID2OF'
             FXBREAD, UNIT, FREQ, 'PSTIFREQ'
             FXBREAD, UNIT, NNDR, 'PSTINNDR'
             FXBREAD, UNIT, NDR, 'PSTINDR'
             FXBREAD, UNIT, INTT, 'PSTIINTT'
             FXBREAD, UNIT, MEAT, 'PSTIMEAT'
             FXBREAD, UNIT, MPC1, 'PSTIMPC1'
             FXBREAD, UNIT, MPC2, 'PSTIMPC2'
             FXBREAD, UNIT, MPC3, 'PSTIMPC3'
             FXBREAD, UNIT, MET, 'PSTIMET'
	ENDELSE
;  Put all the array into the phtcs structure
;  create a phtcst to hold the value before we put into the phtcs
;  array
           TEMP_PHTCS_STRUC = { $
           UKST: [UKST], $
           UKEN: [UKEN], $
	   IKST: [IKST], $
           IKEN: [IKEN], $
           UTST: [UTST], $
           UTEN: [UTEN], $
           AOTN: [AOTN], $
	   SUBS: [SUBS], $
           OPFO: [OPFO], $
           F1TS: [F1TS], $
           F2TS: [F2TS], $
           F1PS: [F1PS], $
           F2PS: [F2PS], $
	   C1PS: [C1PS], $
           C2PS: [C2PS], $
           C3PS: [C3PS], $
           CMOD: [CMOD], $
           CAMP: [CAMP], $
           CSTE: [CSTE], $
	   CINC: [CINC], $
           CRES: [CRES], $
           DETA: [DETA], $
           DRS: [DRS], $
           MUX1: [MUX1], $
           MUX2: [MUX2], $
	   XSTA: [XSTA], $
           D1OF: [D1OF], $
           D1GA: [D1GA], $
           D2OF: [D2OF], $
           D2GA: [D2GA], $
           FREQ: [FREQ], $
	   NNDR: [NNDR], $
           NDR: [NDR], $
           INTT: [INTT], $
           MEAT: [MEAT], $
           MPC1: [MPC1], $
           MPC2: [MPC2], $
	   MPC3: [MPC3], $
           MET: [MET] }


           PHTCS_REC = {PHTCS_TYPE}
           PHTCS_REC.UNIT = STRMID(NAME, 4, 8)
           PHTCS_REC.DIMS = DIMS(0)
           PHTCS_REC.HANDLE = HANDLE_CREATE()
           HANDLE_VALUE, PHTCS_REC.HANDLE, TEMP_PHTCS_STRUC, /SET
   
           
           PHTCS = [PHTCS, PHTCS_REC]
; ILUN is for administration
;
           ILUN = N_ELEMENTS(PHTCS) - 1
	 FXBCLOSE, UNIT
	 FREE_LUN, unit
        ENDIF
        RETURN
        END
