;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_ERD, NAME, HDR, ILUNR, PATH = PATH 
;+
; NAME:
;       READ_ERD
; PURPOSE:
;	Read a PHT-ERD (Edited Raw Data) file accomodating the data in  
;	structure PHTERD  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_ERD, UNIT, NAME, [HDR], [ILUNR], [PATH = PATH]
;
; INPUT:
;       NAME 	: Name of ERD FITS file to be read
;
; OPT. INPUT:
;	PATH    : optional path for the file
;
; OPT. OUTPUTS:
;	HDR	: String array containing the FITS binary table extension
;		  header
;	ILUNR	: Array containing the indices of the measurements in the
;		  PHTERD structure
; RESTRICTIONS:
;       File must be a valid PHT ERD FITS file
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               chop_info
;               chop_info
;               chop_param
;               chop_param
;               fxbintable
;               phtcs
;               phterd
;       PIA Procedure(s):
;               chop_info_init
;               chop_param_init
;               chop_pos
;               conv_cre
;               conv_subs
;               fcs_pow_conv
;               mergehdr
;               pia_conv_date
;               read_cs
;               tmeas_conv
;       Astronomical Library:
;               fxbintable
;               fxpar
;               match
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel - ESA-SAI
;	Modified (last) by CG		October 1996
;	V5.x
;	Change for detecting corrupted parts of the ERD files (CG)	Oct 97
;	V6.5
;	Keyword FLAG added to ERD dynamical structure (CG)		Nov 97
;	V6.6
;
;-
;
;
@fxbintable
@adm
@phterd
@phtcs
@chop_param
@chop_info
;
        ON_ERROR, 2

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_ERD, NAME, [HDR], [ILUNR], [PATH = PATH]'
;  Read the compact status - change -> shape4	
        CSNUM = STRMID(NAME, 4,8)
        IF STRMID(NAME,0,4) EQ 'P2ES' THEN CSNAME = 'PSTI        .FITS' $
                ELSE         CSNAME = 'PSTA        .FITS'
        STRPUT, CSNAME, CSNUM, 4
	if keyword_set(path) then $
          READ_CS, CSNAME, CSHDR, CSUN, PATH = PATH $
        else READ_CS, CSNAME, CSHDR, CSUN 

; if CSUN is -1, it means compact status doesn't exist and we return
	if CSUN EQ -1 then begin
	  msg = 'Compact status file: ' + CSNAME + ' not found, abort reading!'
	  message, /info, msg
	  return
	endif

; create filename esp if path is set
	if keyword_set(PATH) then $
	  filename = PATH + NAME $
	else filename = !FITDIR + NAME
;
; OPEN THE UNFORMATED FITS FILE AND READ THE FIRST PRIMARY RECORD, 2880 BYTES
          OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN
          phdr = bytarr(80, 72, /nozero)
	  readu, unit, phdr
;  Extract the Detector ID
          ID = STRMID(NAME, 0, 2)
;  Take into account the different formats for the different detectors
          CASE ID OF
             'PP':  BEGIN
; CREATE THE PP EXTENDED FITS RECORD, 1 X 2880 BYTES
                       EHDR = BYTARR(80, 36, /NOZERO)
; READ IN THE EXTENDED FITS HEADER 
                       READU, UNIT, EHDR
		       MERGEHDR, PHDR, EHDR, HDR

                       BUF_TYPE = { PPTYPE, GPSCTKEY: 0L, $
                                            GPSCRPID: BYTARR(2), $
                                            GPSCFILL: 0, $
                                            PIXF: 0, $
                                            PCS1: 0, $
                                            PCS2: 0, $
                                            FIL1: 0, $
                                            TEMP: 0, $
                                            FIL2: INTARR(2), $
                                            CPOS: 0, $
                                            MBV:  0, $
                                            PIX: 0 }
                       ROW_COUNT = FXPAR(HDR, 'NAXIS2')
                       BUF = REPLICATE(BUF_TYPE, ROW_COUNT)
; READ IN THE ENTIRE BUFFER
                       READU, UNIT, BUF
                       GPSCTKEY = BUF.GPSCTKEY
                       BYTEORDER, GPSCTKEY, /NTOHL
                       GPSCRPID = BUF.GPSCRPID

                       PIXF = BUF.PIXF
                       PCS1 = BUF.PCS1
                       PCS2 = BUF.PCS2
                       CPOS = BUF.CPOS
                       TEMP = BUF.TEMP
                       TPIX = BUF.PIX
                       BYTEORDER, PIXF, PCS1, PCS2, CPOS, TEMP, TPIX, /NTOHS
                                              
                       PIXL = INTARR(1,ROW_COUNT)
                       PIXL(0,*) = TPIX
		       DETID = 'P'
                    END
             'P1':  BEGIN
                       EHDR = BYTARR(80, 2*36, /NOZERO)
                       READU, UNIT, EHDR
		       MERGEHDR, PHDR, EHDR, HDR

		       BUF_TYPE = { P1TYPE, GPSCTKEY: 0L, $
					    GPSCRPID: BYTARR(2), $
                                            GPSCFILL: 0, $
                                            PIXF: 0, $
                                            PCS1: 0, $
                                            PCS2: 0, $
                                            CREV: 0, $
                                            TEMP: 0, $
                                            FILL: INTARR(2), $
                                            CPOS: 0, $
                                            MBV:  0, $
                                            PIXR: 0, $
                                            PIXO: 0, $
                                            PIX: INTARR(9) }
                        ROW_COUNT = FXPAR(HDR, 'NAXIS2')
                        BUF = REPLICATE(BUF_TYPE, ROW_COUNT)
; READ IN THE ENTIRE BUFFER
                        READU, UNIT, BUF
                        GPSCTKEY = BUF.GPSCTKEY
                        BYTEORDER, GPSCTKEY, /NTOHL
                        GPSCRPID = BUF.GPSCRPID

                        PIXF = BUF.PIXF
                        PCS1 = BUF.PCS1
                        PCS2 = BUF.PCS2
                        CPOS = BUF.CPOS
                        TEMP = BUF.TEMP
                        TOPEN = BUF.PIXO
                        TRESI = BUF.PIXR
                        PIXL = BUF.PIX
                        BYTEORDER, PIXF, PCS1, PCS2, CPOS, TEMP, $
                                   TOPEN, TRESI, PIXL, /NTOHS

                        PIXLOPEN = FLTARR(1, ROW_COUNT)
                        PIXLRESI = FLTARR(1, ROW_COUNT)
                        PIXLOPEN(0,*) = FLOAT(TOPEN)
                        PIXLRESI(0,*) = FLOAT(TRESI)

		       DETID = 'C1'
                    END
             'P2':  BEGIN

; THERE ARE 2 EXTENDED FITS RECORDS IN P2 + primary
                       EHDR = BYTARR(80, 2*36, /NOZERO)
                       READU, UNIT, EHDR
		       MERGEHDR, PHDR, EHDR, HDR

		       BUF_TYPE = { P2TYPE, GPSCTKEY: 0L, $
					    GPSCRPID: BYTARR(2), $
                                            GPSCFILL: 0, $
                                            PIXF: 0, $
                                            PCS1: 0, $
                                            PCS2: 0, $
                                            CREV: 0, $
                                            TEMP: 0, $
                                            FILL: INTARR(2), $
                                            CPOS: 0, $
                                            MBV1: 0, $
                                            MBV2: 0, $
                                            MBV3: 0, $
                                            MBV4: 0, $
                                            PIX1: 0, $
                                            PIX2: 0, $
                                            PIX3: 0, $
                                            PIX4: 0, $
                                            PIX5: 0, $
                                            PIX6: 0 }
                        ROW_COUNT = FXPAR(HDR, 'NAXIS2')
                        BUF = REPLICATE(BUF_TYPE, ROW_COUNT)
; READ IN THE ENTIRE BUFFER
                        READU, UNIT, BUF
                        GPSCTKEY = BUF.GPSCTKEY
                        BYTEORDER, GPSCTKEY, /NTOHL
                        GPSCRPID = BUF.GPSCRPID

                        PIXF = BUF.PIXF
                        PCS1 = BUF.PCS1
                        PCS2 = BUF.PCS2
                        CPOS = BUF.CPOS
                        TEMP = BUF.TEMP
                        BYTEORDER, PIXF, PCS1, PCS2, CPOS, TEMP, /NTOHS

                        PIX1 = BUF.PIX1
                        PIX2 = BUF.PIX2
                        PIX3 = BUF.PIX3
                        PIX4 = BUF.PIX4
                        PIX5 = BUF.PIX5
                        PIX6 = BUF.PIX6
                        BYTEORDER, PIX1, PIX2, PIX3, PIX4, PIX5, PIX6, /NTOHS

                        PIXL = INTARR(4, ROW_COUNT)
                        PIXL(0,*) = PIX1
                        PIXL(1,*) = PIX2
                        PIXL(2,*) = PIX4
                        PIXL(3,*) = PIX5


                       PIXLOPEN = FLTARR(1,ROW_COUNT)
                       PIXLRESI = FLTARR(1,ROW_COUNT)
                       PIXLOPEN(0,*) = FLOAT(PIX3)
                       PIXLRESI(0,*) = FLOAT(PIX6)

		       DETID = 'C2'
                    END
             'PS':  BEGIN
; READ IN THE EXTENDED FITS HEADER, 2 RECORDS, 2 X 2880 BYTES + primary
                       EHDR = BYTARR(80, 72, /NOZERO)
                       READU, UNIT, EHDR
		       MERGEHDR, PHDR, EHDR, HDR

		       BUF_TYPE = { PSTYPE, GPSCTKEY: 0L, $
					    GPSCRPID: BYTARR(2), $
                                            GPSCFILL: 0, $
                                            PIXF: 0, $
                                            PCS1: 0, $
                                            PCS2: 0, $
                                            CREV: 0, $
                                            TEM1: 0, $
                                            TEM2: 0, $
                                            FILL: 0, $
                                            CPOS: 0, $
                                            MBV1:  0, $
                                            MBV2:  0, $
                                            PIX1: INTARR(66), $
                                            PIX2: INTARR(66) }
                        ROW_COUNT = FXPAR(HDR, 'NAXIS2')
                        BUF = REPLICATE(BUF_TYPE, ROW_COUNT)
; READ IN THE ENTIRE BUFFER
                        READU, UNIT, BUF
                        GPSCTKEY = BUF.GPSCTKEY
                        BYTEORDER, GPSCTKEY, /NTOHL
                        GPSCRPID = BUF.GPSCRPID

                        PIXF = BUF.PIXF
                        PCS1 = BUF.PCS1
                        PCS2 = BUF.PCS2
                        CPOS = BUF.CPOS
                        TEM1 = BUF.TEM1
                        TEM2 = BUF.TEM2                        
                        BYTEORDER, PIXF, PCS1, PCS2, CPOS, TEM1, TEM2, /NTOHS
		       TEMP = INTARR(2,ROW_COUNT)
		       TEMP(0,*) = TEM2
		       TEMP(1,*) = TEM1
                       PIX1 = BUF.PIX1
                       PIX2 = BUF.PIX2
                       BYTEORDER, PIX1, PIX2, /NTOHS
                       PIXL = INTARR(128,ROW_COUNT)
                       PIXL(0:63,*) = PIX1(2:65,*)
                       PIXL(64:127,*) = PIX2(2:65,*)
                       PIXL = ROTATE(PIXL,5)
                       PIXLOPEN = [PIX2(0,*),PIX1(0,*)]  
                       PIXLRESI = [PIX2(1,*),PIX1(1,*)] 
		       DETID = 'SL'
                    END
          ENDCASE
;
;
;  Prepare the arrays containing the measurements' initial and end times
;
          UNITID = STRMID(NAME,0,10)
;  Get CSH information needed
	  DD = PHTCS(CSUN).DIMS
          PS = LONARR(DD(0))
          PE = PS
          DA = PS
	  PPS = PS
;  Read the data into different measurements using the CS ITKs
	  HANDLE_VALUE, PHTCS(CSUN).HANDLE, PHTCST
	  IKST = PHTCST.IKST	  
	  IKEN = PHTCST.IKEN	  

; CHANGE CHANGE  - Start
;	  MATCH, GPSCTKEY, IKST(0:DD-1), PS, PPS
;	  MATCH, GPSCTKEY, IKEN(0:DD-1), PE, PPE
; Must be changed because of non-warranty about absolute matching of
; ERD-CS TKEYs:
	  II = 0
	  handle_pp = LONARR(DD)
	  
	  FOR I=0,DD-1 DO BEGIN
	     PP = $
	      WHERE(GPSCTKEY GE PHTCST.IKST(I) AND GPSCTKEY LE PHTCST.IKEN(I),n)
            IF (PP(0) GE 0) THEN BEGIN
		PPS(II) = I
		PS(II) = PP(0)
		PE(II) = PP(N_ELEMENTS(PP)-1)
		handle_pp(II) = HANDLE_CREATE()
		handle_value, handle_pp(II), PP, /set
		II = II + 1
		IF PE(II-1)-PS(II-1)+1 NE N_ELEMENTS(PP) THEN BEGIN
		  PRINT, 'Problems with time key by measurement '+STRTRIM(II,2)
		  PRINT, "I'll try to cope with them, please check carefully"
		  PRINT, 'TK-Start:',PS(II-1),'  TK-End:',PE(II-1),'  Elements:',$
		     N_ELEMENTS(PP)
		ENDIF
	     ENDIF
	  ENDFOR
	  PS = PS(0:II-1)
	  PE = PE(0:II-1)
	  PPS = PPS(0:II-1)
; CHANGE CHANGE  - End

	  DA = PE - PS + 1
;  Read the ITK unit length in seconds
          TREFITKU = FXPAR(HDR,'TREFITKU')
;  Accomodate data into the erd structure and the erd handles after
;  creating the temporary ones
	  DIMR = N_ELEMENTS(PS) -1
	  isub = 0
              FOR I = 0, DIMR DO BEGIN
	         PHTERDT = {PHTERD_TYPE}        
	         PHTERDT.HDR  = HANDLE_CREATE()
	         PHTERDT.RAWP = HANDLE_CREATE()
                 PHTERDT.CONP = HANDLE_CREATE()
                 PHTERDT.FITV = HANDLE_CREATE()
                 PHTERDT.TIME = HANDLE_CREATE()
                 PHTERDT.PIXF = HANDLE_CREATE()
                 PHTERDT.CHOP = HANDLE_CREATE()
                 PHTERDT.STEP = HANDLE_CREATE()
                 PHTERDT.FCSN = HANDLE_CREATE()
                 PHTERDT.TEMP = HANDLE_CREATE()
                 PHTERDT.FCS1 = HANDLE_CREATE()
                 PHTERDT.FCS2 = HANDLE_CREATE()
                 PHTERDT.TKEY = HANDLE_CREATE()
                 PHTERDT.RPID = HANDLE_CREATE()
                 PHTERDT.FLAG = HANDLE_CREATE()

;
;  Use the subs keyword to complete the DETID for the P-detectors
	      	 IF (strmid(DETID,0,1) EQ 'P') THEN BEGIN
			DETID = conv_subs(PHTCST.SUBS(pps(i)))
;  Several detectors can be on one ERD P file
			IF I EQ 0 THEN BEGIN 
			   ORDR = 0 
			   OLDD = DETID
			   allpdet = DETID
			ENDIF ELSE BEGIN
			   indpdet = WHERE(allpdet EQ detid, npdet)
			   IF npdet GT 0 THEN ORDR = npdet ELSE ORDR = 0			   
			   allpdet = [allpdet,detid]
			ENDELSE
	      	 ENDIF ELSE $
			ORDR = I
;  Idname
              	 IF (ORDR LE 9) THEN $
                  IDNAME = DETID + '0' + STRTRIM(STRING(ORDR),1) + $
			STRMID(NAME,4,8) $
                  ELSE IDNAME = DETID + STRTRIM(STRING(ORDR),1) + $
			STRMID(NAME,4,8)


;  Does exist already an administration record for this measurement?
;                 ADMIND = WHERE(ADM.NAME EQ IDNAME, N)

;  Update the administration file using temporary structure
;                 IF (N EQ 1) THEN BEGIN
;                  ADM(ADMIND).PHTERD = N_ELEMENTS(PHTERD) - 1 
;		  IF (DETID EQ 'SL') THEN BEGIN 
;		   STRPUT, IDNAME, 'S', 1
;		   ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;		   IF (N EQ 1) THEN $
;                	ADM(ADMIND2).PHTERD = N_ELEMENTS(PHTERD) - 1 $
;                   ELSE IF (N EQ 0) THEN BEGIN
;		   	ADMT = {ADM_TYPE}
;		   	ADMT.NAME = IDNAME
;		   	ERDPOS = N_ELEMENTS(PHTERD) - 1
;		   	ADMT.PHTERD = ERDPOS
;		   	ADMT.PHTCS = CSUN
;		   	ADM = [ADM, ADMT] 
;	       	   ENDIF ELSE PRINT, 'READ_ERD: Several identical ADM-records' 
;		   STRPUT, IDNAME, 'L', 1
;		  ENDIF
;	         ENDIF ELSE IF (N EQ 0) THEN BEGIN
;		   ADMT = {ADM_TYPE}
;		   ADMT.NAME = IDNAME
;		   ERDPOS = N_ELEMENTS(PHTERD) - 1
;		   ADMT.PHTERD = ERDPOS
;		   ADMT.PHTCS = CSUN
;		   ADM = [ADM, ADMT] 
;		   IF (DETID EQ 'SL') THEN BEGIN 
;		    STRPUT, IDNAME, 'S', 1
;		    ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;		    IF (N EQ 1) THEN $
;                	ADM(ADMIND2).PHTERD = N_ELEMENTS(PHTERD) - 1 $
;                    ELSE IF (N EQ 0) THEN BEGIN
;		   	ADMT = {ADM_TYPE}
;		   	ADMT.NAME = IDNAME
;		   	ERDPOS = N_ELEMENTS(PHTERD) - 1
;		   	ADMT.PHTERD = ERDPOS
;		   	ADMT.PHTCS = CSUN
;		   	ADM = [ADM, ADMT] 
;	       	    ENDIF ELSE PRINT, 'READ_ERD: Several identical ADM-records' 
;		    STRPUT, IDNAME, 'L', 1
;		   ENDIF
;	         ENDIF ELSE PRINT, 'READ_ERD: Several identical ADM-records' 


;  Fill the temporary ERD structure 
;  Fill now UNIT with Date 
		   pia_conv_date, PHTCST.UTST(0,PPS(I)), DATE
                   PHTERDT.UNIT = DATE          
                   PHTERDT.DET  = DETID     
		   PHTERDT.ADMN = IDNAME
		   PHTERDT.ORDR = ORDR
                   PHTERDT.DIMS = DA(I) 
		   PHTERDT.TINT = 2^(7.-PHTCST.INTT(PPS(I)))
		   PHTERDT.NNDR = 2^PHTCST.NNDR(PPS(I)) - 1
;  Compute number of ramps/chopper plateau -> 0 if staring mode
		   IF PHTCST.CMOD(PPS(I)) EQ 3 THEN PHTERDT.RCHP = 0 ELSE $
		     PHTERDT.RCHP = 2.^PHTCST.NDR(PPS(I)) / PHTCST.DRS(PPS(I))   

; Take now the elements from all found
		   handle_value, handle_pp(I), PP 
		   handle_free, handle_pp(I)

		   CASE ID OF
			'PP': BEGIN
                   		RAWD = PIXL(0,PP)
			      END
		   	ELSE: BEGIN
				RAWO = PIXLOPEN(*,PP)
		   		RAWR = PIXLRESI(*,PP)
                   		RAWD = PIXL(*,PP)
		   		RAWD = [RAWD, RAWO, RAWR]
			      END
		   ENDCASE
		   FLAG = FIX(RAWD*0.)
                   HANDLE_VALUE, PHTERDT.HDR, HDR, /SET 
		   IF DETID EQ 'SL' THEN BEGIN
			SSCONV = CONV_CRE(RAWD(0:63,*),'S2',PHTERDT.ORDR,CSUN) 
			SLCONV = $
			   CONV_CRE(RAWD(64:127,*),'S1',PHTERDT.ORDR,CSUN) 
			SSORCO = $
			   CONV_CRE(RAWD(128:129,*),'S2',PHTERDT.ORDR,CSUN) 
			SLORCO = $
			   CONV_CRE(RAWD(130:131,*),'S1',PHTERDT.ORDR,CSUN) 
			SALL = [SSCONV,SLCONV,SSORCO,SLORCO]
                   	HANDLE_VALUE, PHTERDT.CONP, SALL, /SET, /NO_COPY   
                   ENDIF ELSE $
                   HANDLE_VALUE, PHTERDT.CONP, $
			CONV_CRE(RAWD,DETID,PHTERDT.ORDR,CSUN), /SET, /NO_COPY  
                   HANDLE_VALUE, PHTERDT.RAWP, RAWD, /SET, /NO_COPY  
		   HANDLE_VALUE, PHTERDT.PIXF, PIXF(PP),/SET,/NO_COPY  


       		   HANDLE_VALUE, PHTERDT.TIME, (GPSCTKEY(PP) -$
				 IKST(PPS(I))) *  TREFITKU , /SET, /NO_COPY 
		   HANDLE_VALUE, PHTERDT.TKEY, GPSCTKEY(PP), /SET,$
				 /NO_COPY 

                   HANDLE_VALUE, PHTERDT.CHOP, CPOS(PP), /SET, $
				/NO_COPY 
		   IF DETID EQ 'SL' THEN $
                    HANDLE_VALUE, PHTERDT.TEMP, TEMP(*,PP), /SET, $
				/NO_COPY  ELSE $
                   HANDLE_VALUE, PHTERDT.TEMP, TEMP(PP), /SET, $
				/NO_COPY 
                   HANDLE_VALUE, PHTERDT.FCS1, PCS1(PP), /SET, $
				/NO_COPY 
                   HANDLE_VALUE, PHTERDT.FCS2, PCS2(PP), /SET, $
				 /NO_COPY 
		   HANDLE_VALUE, PHTERDT.RPID, GPSCRPID(*,PP), /SET,$
				 /NO_COPY 
		   HANDLE_VALUE, PHTERDT.FLAG, FLAG, /SET,$
				 /NO_COPY 

; Calling provisory fix of tkey	- IS NOW OUT!	   
;                  HANDLE_VALUE, PHTERDT.TKEY, TKEY 
;		   FIX_TKEY, TKEY, newtkey
;                   HANDLE_VALUE, PHTERDT.TIME, $;
;			(newtkey-newtkey(0))*TREFITKU, /SET
;		   HANDLE_VALUE, PHTERDT.TKEY, newtkey, /SET
;		   PHTCST.IKST(PPS(I)) = newtkey(0) 
;		   PHTCST.IKEN(PPS(I)) = newtkey(N_ELEMENTS(newtkey)-1)
; Calling provisory fix of tkey		   
	 

;  Compute the chopper position, overwrite CHOP and compute step and fcs No
		   CHOP_PARAM_INIT
 		   CHOP_INFO_INIT, IDNAME, phterdt=PHTERDT, cst = PHTCST

		   CHOP_POS, NCP, STEPNO, FCSNO
                   HANDLE_VALUE, PHTERDT.CHOP, NCP, /SET, /NO_COPY 
                   HANDLE_VALUE, PHTERDT.STEP, STEPNO, /SET, /NO_COPY 
                   HANDLE_VALUE, PHTERDT.FCSN, FCSNO, /SET, /NO_COPY 

;  Convert temperatures and overwrite TEMP
		   HANDLE_VALUE, phterdt.temp, unctemp
		   tmeas_conv, unctemp, phterdt.det, ctemp
                   HANDLE_VALUE, PHTERDT.TEMP, ctemp, /SET, /NO_COPY 

;  Convert FCS's and overwrite FCS1 and FCS2
		   HANDLE_VALUE, phterdt.fcs1, uncfcs
		   cfcs = fcs_pow_conv(uncfcs)
                   HANDLE_VALUE, PHTERDT.FCS1, cfcs, /SET, /NO_COPY 
		   HANDLE_VALUE, phterdt.fcs2, uncfcs
		   cfcs = fcs_pow_conv(uncfcs)
                   HANDLE_VALUE, PHTERDT.FCS2, cfcs, /SET, /NO_COPY 

;  Update the PHTERD structure using the temporary structure 
		   IF i EQ 0 THEN 	ILUNR = N_ELEMENTS(phterd) ELSE $ 
		   			ILUNR = [ILUNR,N_ELEMENTS(phterd)] 
		   PHTERD = [PHTERD,PHTERDT]

              ENDFOR 
;  Change PHTCS  structure due to fixing of TKEY
          handle_value, PHTCS(csun).handle, PHTCST, /SET, /NO_COPY
 	  FREE_LUN, UNIT

         RETURN
         END
