;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       READ_FITS_HEADER
;
; PURPOSE:
;       This is part of the pia support routines.
; 	And therefore is designed entirely with the pia in mind.
;	It is used for reading the external fits header, it
;	would trim out the primary endline if the extended
;	header exist and join the primary header and the extended
;	header together.
;
; CATEGORY:
;       PIA - utilities
;
; CALLING SEQUENCE:
;       READ_FITS_HEADER, FILENAME, HEADER, STATUS
;
; INPUT:
;	FILENAME: the fits filename
;	
; OUTPUT:
;	HEADER: the returned fits header
;	STATUS: the status, -1 is returned if there is error else return 0
;
; COMMON BLOCKS:
;
; PROCEDURES USED:
;       PIA LIB:
;	Astronomical Library:
; MODIFICATION HISTORY:
;	Written by: Wai-Ming Tai (DIAS)			March 1996
;	mod to pass the current file pointer, WMT(DIAS) Jan 1997
;
;-
pro read_fits_header, filename, header, status, cur_ptr = cur_ptr

  on_error,2 			; return to caller
  status = 0

; assign fits record size
  hdr = bytarr( 80, 36, /nozero)
; make sure it recovers from io error
  on_ioerror, print_error
  openr, unit, filename, /block, /get_lun

; read in the first record
  readu, unit, hdr

; interpret the header as a string, and check to see if the END line has
; been reached.

  header = string( hdr > 32b)
  endline = where( strmid(header, 0, 8) eq 'END     ', nend)
  if nend gt 0 then header = header( 0:endline(0) )

;
; if the SIMPLE keyword can't be
; found in the first record, then this can't be a FITS file.
;
    simple_line = where(strmid(header,0,8) eq 'SIMPLE  ', n_simple)
    if n_simple eq 0 then begin
	file_status = fstat(unit)
	cur_ptr = file_status.cur_ptr
      free_lun, unit
      message, /info, 'SIMPLE keyword not found in the header'
      status = -1
      goto, done
    endif

; 
; Keep reading until the END line is reached.
;
  while nend eq 0 do begin
    readu, unit, hdr
    hdr1 = string( hdr > 32b)
    endline = where( strmid(hdr1, 0, 8) eq 'END     ', nend)
    if nend gt 0 then begin
; take out the endline first, read one more record and check if
; the xtension keyword exist. 
      IF endline(0) EQ 0 THEN GOTO, jump_out
      theendline = hdr1(endline(0))
      hdr1 = hdr1(0:endline(0) - 1)
    endif
    header = [header, hdr1]
jump_out:
  endwhile

  nend = 0
; read one more record
  readu, unit, hdr

; check if it contains the xtension keyword

  hdr1 = string( hdr > 32b)
  extension = string(hdr(0:7))

  if extension NE 'XTENSION' then begin
; no extension, append the endline and exist
	file_status = fstat(unit)
	cur_ptr = file_status.cur_ptr
    free_lun, unit
    header = [ header, theendline ]
    goto, done
  endif else begin

; reading the extended header
    endline = where( strmid(hdr1, 0, 8) eq 'END     ', nend)
    if nend gt 0 then begin
	file_status = fstat(unit)
	cur_ptr = file_status.cur_ptr
      free_lun, unit
      header = [ header, hdr1(0:endline(0)) ]
      goto, done
    endif else begin

    header = [ header, hdr1 ]
; 
; Keep reading until the END line is reached.
;
    while nend eq 0 do begin
      readu, unit, hdr
      hdr1 = string( hdr > 32b)
      endline = where( strmid(hdr1, 0, 8) eq 'END     ', nend)
      if nend gt 0 then begin
        hdr1 = hdr1(0:endline(0))
      endif
      header = [header, hdr1]
    endwhile
	file_status = fstat(unit)
	cur_ptr = file_status.cur_ptr
    free_lun, unit
    goto, done
    endelse
  endelse

print_error:
  status = -1
  message, /info, 'No Such File or File input error!'
done:
  on_ioerror, null
end


