;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro read_fpgc, infile, npts, nlin, dst_pts, dst_lin, rota, $
                 rline, rcolumn, ycoor, zcoor, TEXTOUT=text, ERROR=error 

;+
; NAME:
;     READ_FPGC
; PURPOSE:
;       Read an Focal Plane Geometry Calibration File
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;	read_fpgc, infile, npts, nlin, dst_pts, dst_lin, rota, $
;                  rline, rcolumn, ycoor, zcoor, $
;		   TEXTOUT=text, ERROR=error 
; INPUT:
; 	infile: name of FPGC file
;
; OPTIONAL INPUT:
;
; KEYWORD PARAMETERS:
;	TEXTOUT		String array containing the vital contents of the 
;  			FGPC file
;	ERROR		Returns an error status:
;			0 = no error
;			1 = error: output is not valid
; OUTPUT:
;	npts		Number of raster points
;	nlin		Number of raster lines
;	dst_pts		Distance between raster points [arcsec]
;       dst_lin    	Distance between raster legs [arcsec]
;	rota		Rotation of raster pattern around raster centre
;			in degrees [0.0 .. 360.0]
;
;	rline[npts*nlin]	Raster point id: line number
;	rcolumn[npts*nlin]	Raster point id: column number
;	ycoor[npts*nlin]	Relative Y-coordinate [arcsec] w.r.t. nominal
;				direction of the raster centre
;	zcoor[npts*nlin]	Relative Z-coordinate [arcsec] w.r.t. nominal
;				direction of the raster centre
;
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: I. Heinrichsen
;	Oct 95 Version 3.5
;
;-
  on_error, 2
;
; initialize error status
;
	error = 1
;
; initialize debug
; 		debug=1 give detailed print-out
;		debug=0 give no print-out
;
	debug = 1
;
; check input
;
 npar   = N_params()
 if npar EQ 0 then begin
    goto, nopar
 endif else begin
 	if ((npar LT 10) or (npar gt 12)) then begin 
 		goto, parerr 
 	endif 
 endelse

;
; check for text
;
	if(n_elements(text) eq 0) then text=strarr(1)
;
; open input file
;
  infile = infile(0)
  
  openr, unit, infile, /get_lun, ERROR=err
  if (err NE 0) then begin
             goto, openerror
  endif
  
  line = ' '

;
; read the two header lines
;
  buffer = strarr(1)
  header0 = ' '
  header1 = ' '
  readf, unit, header0
  readf, unit, header1

;
; read all data records from FPGC file
;
  readf, unit, buffer
  while (not eof(unit)) do begin
    readf, unit, line 
       buffer = [buffer, line]
  end

;
; close FPGC file
;
  free_lun, unit
  
;
; extract header information
;
  time = strmid(header0,0,12)
  instrument = strmid(header1,32,1)
  if (instrument NE 'P') then begin
	goto, notpht
  endif
  if (debug eq 1) then begin
  	text = [text, ' ']
  	text = [text, 'Printing Header Information:']
  	text = [text, ' ']
	text = [text, 'Date/time of execution format [YYDDDHHMMSS]: ' + $
		string(time)]
	text = [text, ' ']
  endif	 
  virtab=fix(strmid(header1,33,1))
  case virtab of
  	1:if (debug eq 1) then text = [text, 'PHT virtual apeture used: PHT-P']
  	2:if (debug eq 1) then text = [text, 'PHT virtual apeture used: PHT-C']
  	3:if (debug eq 1) then text = [text, 'PHT virtual apeture used: PHT-S']
  	else: text = [text, 'Virtual apeture undefined for PHT: ' + $
  		      string(virtab)]
  endcase
  npts=fix(strmid(header1,40,3))
  nlin=fix(strmid(header1,43,3))
  if (debug eq 1) then text = [text, 'Number of raster points: ', +$
  				string(npts)]
  if (debug eq 1) then text = [text, 'Number of raster legs: ', + string(nlin)]    
  dst_pts=fix(strmid(header1,64,4))
  dst_lin=fix(strmid(header1,68,4))
  if (debug eq 1) then text = [text, 'Distance between raster points:', + $
  				string(dst_pts)]
  if (debug eq 1) then text = [text, 'Distance between raster lines:', + $
  				string(dst_lin)]
  rota=float(strmid(header1, 74, 5))
  if (debug eq 1) then text = [text, 'Rotation angle of raster in degrees:', +$
  				string(rota)] 
;
; extract infromation from records
;
if (debug eq 1) then begin
	text=[text,' ']
	text=[text,' ']
	text = [text, "Printing Data contents:"]
	text=[text,' ']
endif
rline = fix(strmid(buffer(*),40,3))
rcolumn = fix(strmid(buffer(*),43,3))
if (debug eq 1) then begin
	text = [text, 'Rline:']
	text = [text, string(rline)]
	text = [text, 'Rcolumns:']
	text = [text, string(rcolumn)]
endif
ycoor = float(strmid(buffer(*),46,6))
zcoor = float(strmid(buffer(*),53,6))
if (debug eq 1) then begin 
	text = [text, 'Y Coordinates:']
	text = [text, string(ycoor)]
	text=[text, 'Z Coordinates:']
	text = [text, string(zcoor)]
endif
;
; everything ok.
;
	error = 0
	return
;
; handle error conditions
;
openerror:
	print, "READ_FPGC: Unable to open file:", infile
	return

notpht: 
	print, "READ_FPGC: Instrument ID is not PHT"
	return

nopar:
    print,'READ_FPGC Syntax: read_fpgc, infile, 
    print,'                             npts, nlin, dst_pts, dst_lin, rota,'
    print,'                             rline, rcolumn, ycoor, zcoor, 
    print,'                             ERROR=error'
    return
parerr:
	print,"READ_FPGC: Not correct number of parameters"
	return
	
end
