;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Read_IRPH, FName, Dec, RA, Roll, UTC, RPId, RPQ, HDR, $
                       CDEC, CRA, CROLL, CORQ, OTFT   
;
;+
;  Name:
;	Read_IRPH
;
;  Purpose:
;	Reads an IRPH attitude file.
; CATEGORY:
;	PIA - I/O
;
;  Calling Sequence:
;	READ_IRPH, fname, Dec, RA, Roll, UTC, Rpid, RPQ, HDR, 
;                  CDEC, CRA, CROLL, CORQ, OTFT   
;
;  Input:
;	FName: Name of an IRPH file
;
;  Output:
;	Dec :     Uncorrected Raster Point declination
;	RA  :     Uncorrected Raster point right ascension
;	Roll:     Uncorrected Raster Pont reference roll angle
;  	UTC :     Expected UTC of start of pointing
;	RPId:     Raster Point ID
;	RPQ :     Raster Point quaternion
;	Hdr :     FITS header information
;       CDEC:     Corrected raster-point reference declination
;       CRA :     Corrected raster-point reference right ascension
;       CROLL:    Corrected raster-point reference roll angle
;       CORQ:     Raster-point correction quaternion
;       OTFT:     On-target flag threshold
;       
;CRA
;  Restrictions:
;	None
;
;  Procedures Called:
;       Astronomical Library:
;               fxbclose
;               fxbopen
;               fxbread
;               fxpar
;
;  Modification History
;	Written by: Huw Morris
;       Modification: New IRPH structure OLP version 6.1 (IH)
;	Aug. 97
;	Warning added for old filetype (CG)	September 1997
;	V6.5
;	Finding of old filetype changed to cope with 
;	 larger files (CG)			January 1998
;	V7.0
;-

        ON_ERROR, 2             ; Return to calling procedure

        IF N_PARAMS() LT 4 THEN BEGIN 
            MESSAGE, $
              'Syntax: READ_IRPH, FILENAME, Dec, RA, Roll '+$
              '[, UTC, Rpid, RPQ, HDR, CDEC, CRA, CROLL, CORQ, OTFT]'
            RETURN
        ENDIF

;  Open the IRPH file. Extension always = 1
        FXBOpen, Unit, Fname, 1, Hdr 
;  Check on number of fields to ensure to work also on old products

        tfields=fxpar(hdr, 'TTYPE*')
        ind = WHERE(STRTRIM(tfields,2) EQ 'CDEC',nind)

        if nind EQ 0 then begin         
; old file
; Read the data. 
	    PRINT, 'Warning: no corrected astronomical coordinates found.'
	    PRINT, ' Old filetype (< OLPV6.1) '
            FXBRead, Unit, Dec, 'DEC'
            FXBRead, Unit, RA, 'RA'
            FXBRead, Unit, Roll, 'ROLL'
            FXBRead, Unit, UTC, 'UTC'
            FXBRead, Unit, RPId, 'RPID'
            FXBRead, Unit, RPQ, 'RPQ'
            FXBRead, Unit, OTFT, 'OTFT'

; set corrected data to uncorrected (best you can do if not available)
            CDEC  = DEC
            CRA   = RA
            CROLL = ROLL
            CORQ  = RPQ
        endif else begin
; new file
; Read the data. 
            FXBRead, Unit, Dec, 'DEC'
            FXBRead, Unit, RA, 'RA'
            FXBRead, Unit, Roll, 'ROLL'
            FXBRead, Unit, UTC, 'UTC'
            FXBRead, Unit, RPId, 'RPID'
            FXBRead, Unit, RPQ, 'RPQ'
            FXBRead, Unit, CDEC, 'CDEC'
            FXBRead, Unit, CRA, 'CRA'
            FXBRead, Unit, CROLL, 'CROLL'
            FXBRead, Unit, CORQ, 'CORQ'
            FXBRead, Unit, OTFT, 'OTFT'
        endelse
;  Close the IRPH file
        FXBClose, Unit
        FREE_LUN, Unit
        RETURN
    END	












