;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_ISRD, NAME, IPOS, PATH = PATH, ADMN = ADMN   ;;, $ 
       ;; N_CHANAME = n_chaname
;+
; NAME:
;       READ_ISRD
; PURPOSE:
;	READ a PHTISRD (Interactive Signal Data measurement) file 
;	into the dynamical structure PHTSRD  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_ISRD, NAME [,IPOS] [, PATH = PATH] [, ADMN = ADMN]
;
; INPUT:
;       NAME 	: SRD-savefile name
;
; OPTIONAL INPUT:
;	PATH    : path for the file
;
; OUTPUT KEYWORD:
;	ADMN    : administration name (phtsrd.admn)
;
; RESTRICTIONS:
;       NAME must be a valid PHTISRD file 
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               adm
;               phtsrd
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/ESTEC-SAI)		
;	Oct 95 Version 3.5
;	Structure extension: RESI + NROS (CG)		January 1996 
;       optional path added (WMT)                       March 1996
;	Added ipos to identify the number in
;	PHTSRD buffer  (J.Acosta)			October 1996
;	Add uncertainties if equal 0 (CG)		April 1997
;	V6.2
;	Keyword admn added & filename in HDR (CG)	May 1997
;       Simplified way of asigning name and saving (JA) May 1997
;	V6.3
;	Loop for uncertainty assignment to LONG 
;	 corrected (V6.5)				September 1997
;
;-
;
;@adm
@phtsrd
;
        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_ISRD, NAME, [PATH = PATH]'

;  check name for extension ISRD, name should not contain it!!
        ext_name = '.ISRD'
	trim_pos = strpos(name,ext_name)
	IF trim_pos ne -1 then tmpname = STRMID(name,0,trim_pos) $ 
	   ELSE tmpname = name

; create filename esp if path is set
	if keyword_set(PATH) then $
	  filename = PATH + TMPNAME + ext_name $
	else BEGIN
	  revers_name = STR_REVERSE(tmpname)
	  IF !version.os EQ "VMS" THEN $
	    ind = STRPOS(revers_name,']') ELSE $
	    ind = STRPOS(revers_name,'/') 
	  IF ind eq -1 THEN tmpname = revers_name ELSE $ 	    
	    tmpNAME = STRMID(revers_name,0,ind)
	  tmpname = STR_REVERSE(tmpname)
	  filename = name
	ENDELSE

;  Restore the save file

	RESTORE, FILENAME
	admn = PHTISRD.admn
	hdr = phtisrd.hdr

; Put filename into header - If = official measname then put a 0

	FXADDPAR, hdr, 'PIA_MNAM', tmpname, $
	 "Name chosen for measurement by PIA user", BEFORE='XTENSION'
		

; Prior to put it into the buffers give uncertainties to the signals with unc=0
; Bypass if check (checkunc) already performed or if no signal with FLAG=1
  
	  ass = FXPAR(hdr, 'PIA_ECOR')
	  IF ass EQ -1 THEN GOTO, bypass
	  
	  yflag = phtisrd.flag
; Use at least ONCE Jose's routine
	  npix = pixel_det(phtisrd.det)
	  flagged = WHERE(yflag(0:npix-1) EQ 1, nfc)
	  IF nfc EQ 0 THEN GOTO, bypass

	  yrpid = phtisrd.rpid
	  ystep = phtisrd.step
	  yslop = phtisrd.slop
	  yslun = phtisrd.slun
	  
          get_rastersteps, yrpid, raststeps
	  get_rpchopstep, raststeps, ystep, rpchopstep, in_plat_pos
	  plat_num = max(rpchopstep)
	  FOR ipix=0,npix-1 DO BEGIN
 	   FOR irpch = 1L, plat_num DO BEGIN
 	    ind_rpch = WHERE(rpchopstep EQ irpch)
	    erd_flagged = WHERE(yflag(ipix,ind_rpch) EQ 1, ncount)
	    IF ncount GT 0 THEN BEGIN
		sloppies = yslop(ipix,ind_rpch(erd_flagged))
		sloppuns = yslun(ipix,ind_rpch(erd_flagged)) 
	        check_unc, sloppies, sloppuns, abs_min=0.1*MEDIAN([sloppuns])
		yslun(ipix,ind_rpch(erd_flagged)) = sloppuns
	    ENDIF
	   ENDFOR
	  ENDFOR
	  phtisrd.slun = yslun
	  FXADDPAR, hdr, 'PIA_ECOR', 1, $
	 "Unc. assignment performed for ramps with 2 RO", $
	 BEFORE='XTENSION'
;

bypass:
	
;  Fill the phtsrd buffer
	fill_srd, PHTISRD, hdr
	
;  Get the position within the PHTSCP structure 
        IPOS = N_elements(phtsrd) - 1

; not used anymore, leave just in case 
;;  Does exist already an administration record for this measurement?
;	       IDNAME = PHTISRD.ADMN
;	       DETID = PHTISRD.DET
;               ADMIND = WHERE(ADM.NAME EQ IDNAME, N)
;
;;  Update the administration file using temporary structure
;               IF (N EQ 1) THEN BEGIN
;                ADM(ADMIND).PHTSRD = N_ELEMENTS(PHTSRD) - 1
;                IF (DETID EQ 'SS') THEN BEGIN
;                  STRPUT, IDNAME, 'L', 1
;                  ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;                  IF (N EQ 1) THEN $
;                        ADM(ADMIND2).PHTSRD = N_ELEMENTS(PHTSRD) - 1 $
;                   ELSE IF (N EQ 0) THEN BEGIN
;                        ADMT = {ADM_TYPE}
;                        ADMT.NAME = IDNAME
;                        SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                        ADMT.PHTSRD = SRDPOS
;                        ADM = [ADM, ADMT]
;                  ENDIF ELSE PRINT, 'READ_ISRD: Several identical ADM-records'
;                ENDIF
;               ENDIF ELSE IF (N EQ 0) THEN BEGIN
;                   ADMT = {ADM_TYPE}
;                   ADMT.NAME = IDNAME
;                   SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                   ADMT.PHTSRD = SRDPOS
;                   ADM = [ADM, ADMT]
;                IF (DETID EQ 'SS') THEN BEGIN
;                  STRPUT, IDNAME, 'L', 1
;                  ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;                  IF (N EQ 1) THEN $
;                        ADM(ADMIND2).PHTSRD = N_ELEMENTS(PHTSRD) - 1 $
;                   ELSE IF (N EQ 0) THEN BEGIN
;                        ADMT = {ADM_TYPE}
;                        ADMT.NAME = IDNAME
;                        SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                        ADMT.PHTSRD = SRDPOS
;                        ADM = [ADM, ADMT]
;                  ENDIF ELSE PRINT, 'READ_ISRD: Several identical ADM-records'
;                ENDIF
;               ENDIF ELSE PRINT, 'READ_ISRD: Several identical ADM-records'

        RETURN
        END
