;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_SPD, NAME, HDR, ILUNR, PATH = PATH
;+
; NAME:
;       READ_SPD
; PURPOSE:
;	Read a PHT-SPD (Standard Processed Data) file accomodating the data 
;	structure PHTSPD  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_SPD, NAME, [HDR], [ILUNR], [PATH = PATH]
;
; INPUT:
;       NAME 	: Name of PHT-SPD FITS file to be read
; OPTIONAL INPUT:
;	PATH    : path for the file
;
; OPT. OUTPUTS:
;	HDR	: String array containing the FITS binary table extension
;		  header
;       ILUNR   : Array containing the indices of the measurements in the
;                 PHTSPD structure
;
; RESTRICTIONS:
;       File must be a valid PHT-SPD FITS file
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               fxbintable
;               phtcs
;               phtspd
;       PIA Procedure(s):
;               c2p_pipe2ia
;               mergehdr
;               phtcs_get_rec
;               read_cs
;       Astronomical Library:
;               fxbclose
;               fxbintable
;               fxbopen
;               fxbread
;               fxhread
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel - ESA-SAI
;	Modified by Wai-Ming Tai (DIAS) and CG 		September 1996
;	V.5.0 - Sept. 96
;	Problem with different nomenclature for SPD files
;	 from pipeline fixed (CG)			September 1997
;	V6.5
;
;-
;
;
@fxbintable
@adm
@phtcs
@phtspd
;        ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_SPD, NAME, [HDR], [ILUNR], [PATH = PATH]'
;  Read the compact status - change -> shape4 
        CSNUM = STRMID(NAME, 4,8)
        CSNAME = 'PSTA        .FITS'
        STRPUT, CSNAME, CSNUM, 4
	if keyword_set(path) then $
          READ_CS, CSNAME, CSHDR, CSUN, PATH = PATH $
        else READ_CS, CSNAME, CSHDR, CSUN

; return to caller if compact status not found
  if CSUN EQ -1 then begin
        PRINT, 'Trying now to read CS from FITDIR ...'
 	READ_CS, CSNAME, CSHDR, CSUN, status=status
        IF (status) THEN BEGIN
	 msg = 'Compact status: ' + CSNAME + ' not found, abort reading!'
	 message, /info, msg
	 return
	ENDIF
  endif

;
;  Open file. Extension always =1            
;
; create filename esp if path is set
	if keyword_set(PATH) then $
	  file = PATH + NAME $
	else file = !FITDIR + NAME

         OPENR,  UNIT, FILE, /BLOCK, /GET_LUN
         FXHREAD, UNIT, PHDR
         FREE_LUN, UNIT
	 FXBOPEN, UNIT, FILE, 1, EHDR
         MERGEHDR, PHDR, EHDR, HDR

;  Read first the general record prefix
         FXBREAD, UNIT, TKEY, 'GPSCTKEY'
         FXBREAD, UNIT, RPID, 'GPSCRPID'
;  Adjust the keyword names to the positions on the FITS file
         KYID = 4
         FILT = 7
         APER = 8
	 POLZ = 9
	 NDRS = 10
	 CSTP = 11
	 DWEL = 12
	 MEAS = 13
	 CPOS = 14
	 MNPW = 15
	 MNPU = 16
  	 MDPW = 17
 	 Q1PW = 18
	 Q3PW = 19
	 PLEN = 20
	 FLAG = 21
;  Read the columns
	 FXBREAD, UNIT, KYID, KYID
	 FXBREAD, UNIT, FILT, FILT
	 FXBREAD, UNIT, APER, APER
	 FXBREAD, UNIT, POLZ, POLZ
	 FXBREAD, UNIT, NDRS, NDRS
	 FXBREAD, UNIT, CSTP, CSTP
; Accomodate step definition to PIA's one
	 IST0 = WHERE(CSTP EQ 0, nst0)
	 IF nst0 GT 0 THEN BEGIN
	   cstp = cstp - min(cstp) + 1
	 ENDIF ELSE BEGIN
	  IST = WHERE(CSTP EQ -1, nst)
	  IF nst GT 0 THEN BEGIN
	   IST1 = WHERE(CSTP EQ 1, NST1)
	   CSTP(ist) = 1
	   IF NST1 GT 0 THEN CSTP(ist1)= 2 ELSE $
	   PRINT, 'Probably error in assigning chopper step'
	   IF (nst + nst1) NE N_ELEMENTS(cstp) THEN $
	     PRINT, 'Error in assigning chopper steps, some undefined'
	  ENDIF
	 ENDELSE
	 
 	 FXBREAD, UNIT, DWEL, DWEL
	 FXBREAD, UNIT, MEAS, MEAS
	 FXBREAD, UNIT, CPOS, CPOS
	 FXBREAD, UNIT, MNPW, MNPW
	 FXBREAD, UNIT, MNPU, MNPU
	 FXBREAD, UNIT, MDPW, MDPW
	 FXBREAD, UNIT, Q1PW, Q1PW
	 FXBREAD, UNIT, Q3PW, Q3PW
	 FXBREAD, UNIT, PLEN, PLEN
	 FXBREAD, UNIT, FLAG, FLAG
; Provisionally take just conversion to integer and set it to 0
	 FLAG = FIX(FLAG) * 0
;
; Nr of measurements, total length, are the measurements staring or chopped?
;
	 NMEAS = FXPAR(HDR,'NMEAS') 
	 NAX = FXPAR(HDR,'NAXIS2') 
;	 FPCMOD = FXPAR(HDR,'FPCMODE') 
;
;  Prepare the arrays containing the measurements' initial and end times
;
         UNITID = STRMID(NAME,0,10)
;        PHTCS_GET_REC, CSUN, PHTCST

	 CONV_CS, phtcs(csun), convcs
         handle_value, convcs.handle, phtcst
; PIA FITS ?
	 IF STRPOS(NAME,'_') EQ -1 THEN pia_fits = 0 ELSE pia_fits = 1 

	 IF pia_fits EQ 0 THEN $
	 indices = WHERE(STRMID(phtcst.subs,0,2) EQ STRMID(NAME,1,2),count) $
         ELSE indices = WHERE(STRMID(phtcst.subs,0,2) EQ STRMID(NAME,0,2),count)
         	 
	 DD = count
         PS = INTARR(DD)
	 PE = PS
	 PS = PS - 1
	 PPS = PS
;  Read the data into different measurements using the CS ITKs
;         MATCH, TKEY, PHTCST.IKST, PS
	 II = 0
	IF DD EQ 1 THEN BEGIN
	  PP = WHERE(TKEY GE PHTCST.IKST(indices) AND $
	             TKEY LE PHTCST.IKEN(indices))
          IF (PP(0) GE 0) THEN BEGIN
		PPS(II) = 0
		PS(II) = PP(0)
		PE(II) = PP(N_ELEMENTS(PP)-1)
		II = II + 1
	  ENDIF
	ENDIF ELSE BEGIN
	 FOR I = 0,DD-1 DO BEGIN
	  PP = WHERE(TKEY GE PHTCST.IKST(indices(I)) AND $
	             TKEY LE PHTCST.IKEN(indices(I)))
          IF (PP(0) GE 0) THEN BEGIN
		PPS(II) = I
		PS(II) = PP(0)
		PE(II) = PP(N_ELEMENTS(PP)-1)
		II = II + 1
	  ENDIF
         ENDFOR
	ENDELSE
	

;  Check if all measurements are found
	 JUNK = WHERE(PS NE -1, DIMR) 
   	 IF (DIMR NE NMEAS) THEN PRINT,'meas(HDR)= ', NMEAS,$
			'  /= number of meas', DIMR 
         TREFITKU = FXPAR(HDR,'TREFITKU')
         FOR I = 0, DIMR - 1 DO BEGIN
 	      PHTSPDT = {PHTSPD_TYPE}
		PHTSPDT.HDR = HANDLE_CREATE()
       	 	PHTSPDT.KYID = HANDLE_CREATE()
       	 	PHTSPDT.FILT = HANDLE_CREATE()
       	 	PHTSPDT.APER = HANDLE_CREATE()
        	PHTSPDT.POLZ = HANDLE_CREATE()
	        PHTSPDT.NDRS = HANDLE_CREATE()
        	PHTSPDT.DWEL = HANDLE_CREATE()
	        PHTSPDT.MEAS = HANDLE_CREATE()
 	       	PHTSPDT.CPOS = HANDLE_CREATE()
  	      	PHTSPDT.TIME = HANDLE_CREATE()
  	      	PHTSPDT.MNPW = HANDLE_CREATE()
   	     	PHTSPDT.MNPU = HANDLE_CREATE()
    	    	PHTSPDT.MDPW = HANDLE_CREATE()
    	    	PHTSPDT.Q1PW = HANDLE_CREATE()
        	PHTSPDT.Q3PW = HANDLE_CREATE()
        	PHTSPDT.DEC = HANDLE_CREATE()
        	PHTSPDT.RA = HANDLE_CREATE()
        	PHTSPDT.ROLL = HANDLE_CREATE()
        	PHTSPDT.FLAG = HANDLE_CREATE()
        	PHTSPDT.CSTP = HANDLE_CREATE()
        	PHTSPDT.PLEN = HANDLE_CREATE()
        	PHTSPDT.TKEY = HANDLE_CREATE()
        	PHTSPDT.RPID = HANDLE_CREATE()
;  Establish Idname
	      IF PIA_FITS THEN det = STRMID(NAME, 0, 2) ELSE $
	      DET = STRMID(NAME, 1, 2)
	      
	      IF (I LE 9) THEN $
	        IDNAME = DET + '0' + STRTRIM(STRING(PPS(I)),1) + STRMID(NAME,4,8) $
	        ELSE IDNAME = DET + STRTRIM(STRING(PPS(I)),1) + STRMID(NAME,4,8) 
;  Fill values
              pia_conv_date, PHTCST.UTST(0, indices(PPS(I))), date
              PHTSPDT.UNIT = date
	      PHTSPDT.DET  = DET
	      PHTSPDT.ADMN = IDNAME
              PHTSPDT.ORDR = I
              PHTSPDT.DIMS = PE(I)-PS(I)+1
;  Fill arrays
	      DETRED = STRMID(DET,0,1)
	      CASE DETRED OF
		'P': BEGIN
		  TMNPW = FLTARR(1,PHTSPDT.DIMS)  
		  TMNPU = TMNPW 
		  TMDPW = TMNPW
		  TQ1PW = TMNPW
		  TQ3PW = TMNPW
		  TPLEN = TMNPW 
		  TFLAG = FIX(TMNPW)  
		  TMNPW(0,*) = MNPW(PS(I):PE(I))  
		  TMNPU(0,*) = MNPU(PS(I):PE(I))  
		  TMDPW(0,*) = MDPW(PS(I):PE(I))  
		  TQ1PW(0,*) = Q1PW(PS(I):PE(I))  
		  TQ3PW(0,*) = Q3PW(PS(I):PE(I))  
		  TPLEN(0,*) = PLEN(PS(I):PE(I))  
		  TFLAG(0,*) = FLAG(PS(I):PE(I))  
		END
		ELSE: BEGIN
		  TMNPW = MNPW(*,PS(I):PE(I))  
		  TMNPU = MNPU(*,PS(I):PE(I))  
		  TMDPW = MDPW(*,PS(I):PE(I))  
		  TQ1PW = Q1PW(*,PS(I):PE(I))  
		  TQ3PW = Q3PW(*,PS(I):PE(I))  
		  TPLEN = PLEN(*,PS(I):PE(I))  
		  TFLAG = FLAG(*,PS(I):PE(I))  
		END
	      ENDCASE
              HANDLE_VALUE, PHTSPDT.HDR, HDR, /SET           
              HANDLE_VALUE, PHTSPDT.TKEY, TKEY(PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.RPID, RPID(*,PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.FILT, FILT(PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.KYID, KYID(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.APER, APER(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.POLZ, POLZ(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.NDRS, NDRS(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.CSTP, CSTP(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTSPDT.DWEL, DWEL(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTSPDT.MEAS, MEAS(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.CPOS, CPOS(PS(I):PE(I)), /SET
 	      TIME = TKEY(PS(I):PE(I)) - TKEY(PS(I)) 
              HANDLE_VALUE, PHTSPDT.TIME, TIME * TREFITKU, /SET
;  Possible two-dimensional arrays

;  Accomodate pixel numbering to the IA (telemetry) convention
	      IF DET EQ 'C2' THEN BEGIN
		c2p_pipe2ia, tmnpw, ttmp
		tmnpw = ttmp
		c2p_pipe2ia, tmnpu, ttmp
		tmnpu = ttmp
		c2p_pipe2ia, tmdpw, ttmp
		tmdpw = ttmp
		c2p_pipe2ia, tq1pw, ttmp
		tq1pw = ttmp
		c2p_pipe2ia, tq3pw, ttmp
		tq3pw = ttmp
		c2p_pipe2ia, tflag, ttmp
		tflag = ttmp
	      ENDIF

	      HANDLE_VALUE, PHTSPDT.MNPW, TMNPW, /SET 
              HANDLE_VALUE, PHTSPDT.MNPU, TMNPU, /SET 
	      HANDLE_VALUE, PHTSPDT.MDPW, TMDPW, /SET 
	      HANDLE_VALUE, PHTSPDT.Q1PW, TQ1PW, /SET 
	      HANDLE_VALUE, PHTSPDT.Q3PW, TQ3PW, /SET 
              HANDLE_VALUE, PHTSPDT.PLEN, TPLEN, /SET 
              HANDLE_VALUE, PHTSPDT.FLAG, TFLAG, /SET 
;  Update the PHTSPD structure using the temporary structure
              PHTSPD = [PHTSPD,PHTSPDT]
;  Does exist already an administration record for this measurement?
;	      ADMIND = WHERE(ADM.NAME EQ IDNAME, N)
;  Update the administration file using temporary structure
;	      IF (N EQ 1) THEN $
;		ADM(ADMIND).PHTSPD = N_ELEMENTS(PHTSPD) - 1 $
;	       ELSE IF (N EQ 0) THEN BEGIN
;                ADMT = {ADM_TYPE}
;		 ADMT.NAME = IDNAME
;                ADMT.PHTSPD = N_ELEMENTS(PHTSPD) - 1
;		 ADMT.PHTCS = CSUN
;		 ADM = [ADM, ADMT]
;	      ENDIF ELSE PRINT, 'READ_SPD: Several identical ADM-records'
 	 ENDFOR
	 FXBCLOSE, UNIT
	 FREE_LUN, UNIT
	RETURN
        END
