;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO READ_SPD_SL, NAME, HDR, ILUNR, PATH = PATH
;+
; NAME:
;       READ_SPD_SL
; PURPOSE:
;	Read a PHT-SPD (Standard Processed Data) file corresponding to 
;	PHT-SS or PHT-SL accomodating the data into the structure PHTSPD  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_SPD_SL, NAME, [HDR], [ILUNR], [PATH = PATH]
;
; INPUT:
;       NAME 	: Name of PHT-SPD FITS file to be read
;
; OPTIONAL INPUT:
;	PATH    : path for the file
;
; OPT. OUTPUTS:
;	HDR	: String array containing the FITS binary table extension
;		  header
;       ILUNR   : Array containing the indices of the measurements in the
;                 PHTSPD structure
;
; RESTRICTIONS:
;       File must be a valid PHT-SPD FITS file
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               fxbintable
;               phtcs
;               phtspd
;       PIA Procedure(s):
;               c2p_pipe2ia
;               mergehdr
;               phtcs_get_rec
;               read_cs
;       Astronomical Library:
;               fxbclose
;               fxbintable
;               fxbopen
;               fxbread
;               fxhread
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)		December 1995
;	optional path added (WMT)                       March 1996
;	abort reading if cs file not found              March 1996
;	Free_lun instead of close (CG)			May 1996
;	change to accomodate the ss/sl spd to scp       Sept 1996
;	structure, we have to merge the sl and ss 
;	files together in the scp structure
;	Extension to IA produced files (CG)		Oct 1997
;	V6.5
;
;-
;
;
@fxbintable
@adm
@phtcs
@phtspd
@phtscp
;       ON_ERROR, 2
;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  READ_SPD_SL, NAME, [HDR], [ILUNR], [PATH = PATH]'

;  Read the compact status - change -> shape4 
        CSNUM = STRMID(NAME, 4,8)
        CSNAME = 'PSTA        .FITS'
        STRPUT, CSNAME, CSNUM, 4
	if keyword_set(path) then $
          READ_CS, CSNAME, CSHDR, CSUN, PATH = PATH $
        else READ_CS, CSNAME, CSHDR, CSUN
;
	slname = name
	ssname = name
	IF STRMID(name,0,1) EQ 'P' THEN BEGIN
	  STRPUT, SSNAME, 'S', 2
	  STRPUT, SLNAME, 'L', 2
	  internal = 0
	ENDIF ELSE BEGIN
	  STRPUT, SSNAME, 'S', 1
	  STRPUT, SLNAME, 'L', 1
	  internal = 1
	ENDELSE
	
;        SLNAME = 'PSLS        .FITS'
;        SSNAME = 'PSSS        .FITS'
;        STRPUT, SSNAME, CSNUM, 4
;        STRPUT, SLNAME, CSNUM, 4

; return to caller if compact status not found
  if CSUN EQ -1 then begin
        PRINT, 'Trying now to read CS from FITDIR ...'
 	READ_CS, CSNAME, CSHDR, CSUN, status=status
        IF (status) THEN BEGIN
	 msg = 'Compact status: ' + CSNAME + ' not found, abort reading!'
	 message, /info, msg
	 return
	ENDIF
  endif

; create the filename esp if path is set
	if keyword_set(path) then begin
          slfile = path + slname 
          ssfile = path + ssname 
	endif else begin
	  slfile = !FITDIR + slNAME
	  ssfile = !FITDIR + ssNAME
	endelse
; check if the slfile and ssfile exist
	slexist = 0
	ssexist = 0
	if is_file_exist(slfile) then slexist = 1
	if is_file_exist(ssfile) then ssexist = 1
; action_flag indicate if both files exist
	action_flag = slexist + ssexist
; return to caller if both files not found
   	if action_flag eq 0 then begin
	  msg = 'Both SPD SS/SL files not found, abort reading!'
	  message, /info, msg
	  return
	endif
;
;  Open file. Extension always =1            
;
; we read in one of the files as default for most of the fields
	if slexist EQ 1 then begin
	  read_fits_header, slfile, hdr 
	  fxbopen, unit, slfile, 1
	endif else begin
	  read_fits_header, ssfile, hdr
	  fxbopen, unit, ssfile, 1
	endelse

;  Read first the general record prefix
         FXBREAD, UNIT, TKEY, 'GPSCTKEY'
         FXBREAD, UNIT, RPID, 'GPSCRPID'
;  Adjust the keyword names to the positions on the FITS file
         NKYID = 4
;         NFILT = 7  ; all other SPD files
;         NAPER = 8  ; have a different structure
	 NPOLZ = 7
	 NNDRS = 8
	 NCSTP = 9
	 NDWEL = 10
	 NMEAS = 11
	 NCPOS = 12
	 NMNPW = 13
	 NMNPU = 14
  	 NMDPW = 15
 	 NQ1PW = 16
	 NQ3PW = 17
	 NPLEN = 18
	 NFLAG = 19
;  Read the columns
;  the following fields are common for both ss and sl
	 FXBREAD, UNIT, PLEN, NPLEN
	 FXBREAD, UNIT, KYID, NKYID
	 FXBREAD, UNIT, POLZ, NPOLZ
	 FXBREAD, UNIT, NDRS, NNDRS
	 FXBREAD, UNIT, CSTP, NCSTP
; Accomodate step definition to PIA's one
	 IST = WHERE(CSTP EQ -1, nst)
	 IF nst GT 0 THEN BEGIN
	   IST1 = WHERE(CSTP EQ 1, NST1)
	   CSTP(ist) = 1
	   IF NST1 GT 0 THEN CSTP(ist1)= 2 ELSE $
	   PRINT, 'Probably error in assigning chopper step'
	   IF (nst + nst1) NE N_ELEMENTS(cstp) THEN $
	     PRINT, 'Error in assigning chopper steps, some undefined'
	 ENDIF
	 	 
 	 FXBREAD, UNIT, DWEL, NDWEL
	 FXBREAD, UNIT, MEAS, NMEAS
	 FXBREAD, UNIT, CPOS, NCPOS
	 FXBREAD, UNIT, MNPW, NMNPW
	 FXBREAD, UNIT, MNPU, NMNPU
	 FXBREAD, UNIT, MDPW, NMDPW
	 FXBREAD, UNIT, Q1PW, NQ1PW
	 FXBREAD, UNIT, Q3PW, NQ3PW
	 FXBREAD, UNIT, FLAG, NFLAG
	 
	 aper = INTARR(N_ELEMENTS(cpos))
	 filt = aper
; Provisionally take just conversion to integer and set it to 0
	 FLAG = FIX(FLAG) * 0
;  end of the common part of ss and sl
	 FXBCLOSE, UNIT
	 
; IF PIA FITS files then read them one by one - merging not needed	 
	 IF internal THEN action_flag = 1
	 
; if action_flag is 2 means both files exist, we read the ss one
; note here that if we only have one file, the file unit is defined
; earlier on. The following lines are useful if both files exist,
; sl file get read first and then ss.
	 if action_flag eq 2 then begin
	   read_fits_header, ssfile, hdr1
	   FXBOPEN, UNIT, SSFILE, 1
	   FXBREAD, UNIT, MNPW1, NMNPW
	   FXBREAD, UNIT, MNPU1, NMNPU
	   FXBREAD, UNIT, MDPW1, NMDPW
	   FXBREAD, UNIT, Q1PW1, NQ1PW
	   FXBREAD, UNIT, Q3PW1, NQ3PW
	   FXBREAD, UNIT, FLAG1, NFLAG
; Provisionally take just conversion to integer and set it to 0
	 FLAG1 = FIX(FLAG1) * 0
	   FXBCLOSE, UNIT
	 endif
;
; Nr of measurements, total length, are the measurements staring or chopped?
;
	 NMEAS = FXPAR(HDR,'NMEAS') 
	 NAX = FXPAR(HDR,'NAXIS2') 
;	 FPCMOD = FXPAR(HDR,'FPCMODE') 
;
;  Prepare the arrays containing the measurements' initial and end times
;
         UNITID = STRMID(NAME,0,10)
         PHTCS_GET_REC, CSUN, PHTCST
	 DD = PHTCST.DIMS
         PS = INTARR(DD)
	 PE = PS
	 PS = PS - 1
	 PPS = PS
;  Read the data into different measurements using the CS ITKs
;         MATCH, TKEY, PHTCST.IKST, PS
	 II = 0
	IF DD EQ 1 THEN BEGIN
	  PP = WHERE(TKEY GE PHTCST.IKST AND TKEY LE PHTCST.IKEN)
          IF (PP(0) GE 0) THEN BEGIN
		PPS(II) = I
		PS(II) = PP(0)
		PE(II) = PP(N_ELEMENTS(PP)-1)
		II = II + 1
	  ENDIF
	ENDIF ELSE BEGIN
	 FOR I = 0,DD-1 DO BEGIN
	  PP = WHERE(TKEY GE PHTCST.IKST(I) AND TKEY LE PHTCST.IKEN(I))
          IF (PP(0) GE 0) THEN BEGIN
		PPS(II) = I
		PS(II) = PP(0)
		PE(II) = PP(N_ELEMENTS(PP)-1)
		II = II + 1
	  ENDIF
         ENDFOR
	ENDELSE

	IF not internal THEN BEGIN

;  Check if all measurements are found
	 JUNK = WHERE(PS NE -1, DIMR) 
   	 IF (DIMR NE NMEAS) THEN PRINT,'meas(HDR)= ', NMEAS,$
			'  /= number of meas', DIMR 
         TREFITKU = FXPAR(HDR,'TREFITKU')
         FOR I = 0, DIMR - 1 DO BEGIN
;  Establish Idname
	      DET = 'SL'
	      IF (I LE 9) THEN $
	        IDNAME = DET + '0' + STRTRIM(STRING(I),1) + STRMID(NAME,4,8) $
	        ELSE IDNAME = DET + STRTRIM(STRING(I),1) + STRMID(NAME,4,8) 
              PHTSCPT = {PHTSCP_TYPE}
              PHTSCPT.HDR = HANDLE_CREATE()
              PHTSCPT.TIME = HANDLE_CREATE()
              PHTSCPT.AVER = HANDLE_CREATE()
              PHTSCPT.AVUN = HANDLE_CREATE()
              PHTSCPT.MEDI = HANDLE_CREATE()
              PHTSCPT.Q1ME = HANDLE_CREATE()
              PHTSCPT.Q3ME = HANDLE_CREATE()
              PHTSCPT.FLAG = HANDLE_CREATE()
              PHTSCPT.CHOP = HANDLE_CREATE()
              PHTSCPT.STEP = HANDLE_CREATE()
              PHTSCPT.FCSN = HANDLE_CREATE()
              PHTSCPT.TEMP = HANDLE_CREATE()
              PHTSCPT.FCS1 = HANDLE_CREATE()
              PHTSCPT.FCS2 = HANDLE_CREATE()
              PHTSCPT.TKEY = HANDLE_CREATE()
              PHTSCPT.RPID = HANDLE_CREATE()
              PHTSCPT.ARSI = HANDLE_CREATE()
              PHTSCPT.ANDR = HANDLE_CREATE()
              PHTSCPT.AVSD = HANDLE_CREATE()
	      pia_conv_date, PHTCST.UTST(0, PPS(I)), date
              PHTSCPT.UNIT = date
              PHTSCPT.DET  = DET
              PHTSCPT.ADMN = IDNAME
              PHTSCPT.ORDR = I
              PHTSCPT.DIMS = PE(I)-PS(I)+1
              HANDLE_VALUE, PHTSCPT.TKEY, TKEY(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSCPT.RPID, RPID(*,PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSCPT.STEP, CSTP(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTSCPT.CHOP, CPOS(PS(I):PE(I)), /SET
              tfcsn = cstp
              tfcsn(*) = 0
              HANDLE_VALUE, PHTSCPT.FCSN, TFCSN(PS(I):PE(I)), /SET
 
              TIME = TKEY(PS(I):PE(I)) - TKEY(PS(I))
              HANDLE_VALUE, PHTSCPT.TIME, TIME * TREFITKU, /SET
 
              TMNPW = MNPW(*,PS(I):PE(I))
              TMNPU = MNPU(*,PS(I):PE(I))
              TMDPW = MDPW(*,PS(I):PE(I))
              TQ1PW = Q1PW(*,PS(I):PE(I))
              TQ3PW = Q3PW(*,PS(I):PE(I))
              TFLAG = FLAG(*,PS(I):PE(I))
;
; if action_flag is equal to 2, both sl/ss files exist
; else we have to set one half of the array to 0 and then
; if sl exist, then we append the other half otherwise
; we prepend the other half
;
	      if action_flag eq 1 then begin
		thdr = hdr
		tmnpw1 = tmnpw
		tmnpw1(*) = 0.
		tmnpu1 = tmnpu
		tmnpu1(*) = 0.
		tmdpw1 = tmdpw
		tmdpw1(*) = 0.
		tq1pw1 = tq1pw
		tq1pw1(*) = 0.
		tq3pw1 = tq3pw
		tq3pw1(*) = 0.
		tflag1 = tflag
		tflag1(*) = 2
;
		if ssexist eq 1 then begin
; append to it if ssexist
		  tmnpw = [tmnpw, tmnpw1]
		  tmnpu = [tmnpu, tmnpu1]
		  tmdpw = [tmdpw, tmdpw1]
		  tq1pw = [tq1pw, tq1pw1]
		  tq3pw = [tq3pw, tq3pw1]
		  tflag = [tflag, tflag1]
		endif else begin
; prepend to if if slexist
		  tmnpw = [tmnpw1, tmnpw]
		  tmnpu = [tmnpu1, tmnpu]
		  tmdpw = [tmdpw1, tmdpw]
		  tq1pw = [tq1pw1, tq1pw]
		  tq3pw = [tq3pw1, tq3pw]
		  tflag = [tflag1, tflag]
		endelse
	      endif else begin
; okay, this is the normal case
                TMNPW1 = MNPW1(*,PS(I):PE(I))
                TMNPU1 = MNPU1(*,PS(I):PE(I))
                TMDPW1 = MDPW1(*,PS(I):PE(I))
                TQ1PW1 = Q1PW1(*,PS(I):PE(I))
                TQ3PW1 = Q3PW1(*,PS(I):PE(I))
                TFLAG1 = FLAG1(*,PS(I):PE(I))
		tmnpw = [tmnpw1, tmnpw]
		tmnpu = [tmnpu1, tmnpu]
		tmdpw = [tmdpw1, tmdpw]
		tq1pw = [tq1pw1, tq1pw]
		tq3pw = [tq3pw1, tq3pw]
		tflag = [tflag1, tflag]
; we only merge the headers if both files exist
		endline = WHERE(STRMID(hdr,0,3) EQ 'END', nend)
		begline = WHERE(STRMID(hdr1,0,8) EQ 'XTENSION',beg2)
		IF nend GT 0 AND beg2 GT 0 THEN $
		 thdr = [hdr(0:endline(0)-1), hdr1(begline(0)+1:*)] $
		ELSE BEGIN
		 PRINT, 'Problems with merging headers, putting everything in'
		 thdr = [hdr,hdr1]
		ENDELSE
		GOTO, jump
; old code, do not use
		keyword = strmid(hdr, 0, 8)
		keyword1 = strmid(hdr1, 0, 8)
		RUNC = 'RUNC64  '
		inde = where(keyword eq RUNC, count)
		IF count EQ 0 THEN RUNC = 'RPUNC64 '
		inds = where(keyword eq 'LAMBDA1 ', count)
		inde = where(keyword eq RUNC, count)
		inds1 = where(keyword1 eq 'LAMBDA1 ', count)
		inde1 = where(keyword1 eq RUNC, count)
		lamhdr = [hdr(inds(0):inde(0)), hdr1(inds1(0):inde1(0))] 

		indss = where(keyword eq 'SATLIM1 ', count)
		indee = where(keyword eq 'FITDR64 ', count)
		indss1 = where(keyword1 eq 'SATLIM1 ', count)
		indee1 = where(keyword1 eq 'FITDR64 ', count)
		sathdr = [hdr(indss(0):indee(0)), hdr1(indss1(0):indee1(0))]
		strput, lamhdr, '        '
		strput, sathdr, '        '
; Accomodate also old SPD files (without Wavelength uncertainty)
		IF N_ELEMENTS(lamhdr) EQ 384 THEN file_fact = 3 ELSE $
		 IF N_ELEMENTS(lamhdr) EQ 512 THEN file_fact = 4 ELSE BEGIN
		   PRINT, 'Wrong things happen with header, trying to continue'
		   GOTO, jump
		  ENDELSE

		for ihdr = 0, 127 do begin
		  ind = ihdr * file_fact
		  tnum = strtrim(string(ihdr+1), 1)
;
		  tmp = lamhdr(ind)
		  strput, tmp, 'LAMBD' + tnum
		  lamhdr(ind) = tmp
;
		  IF file_fact EQ 4 THEN BEGIN
		  ind = ind + 1
		  tmp = lamhdr(ind)
		  strput, tmp, 'LUNC' + tnum
		  lamhdr(ind) = tmp
		  ENDIF
;
		  ind = ind + 1
		  tmp = lamhdr(ind)
		  strput, tmp, 'RESP' + tnum
		  lamhdr(ind) = tmp
;
		  ind = ind + 1
		  tmp = lamhdr(ind)
		  strput, tmp, 'RUNC' + tnum
		  lamhdr(ind) = tmp
;
		  ind = ihdr * 2
		  tmp = sathdr(ind)
		  strput, tmp, 'SATLI' + tnum
		  sathdr(ind) = tmp
;
		  ind = ind + 1
		  tmp = sathdr(ind)
		  strput, tmp, 'FITDR' + tnum
		  sathdr(ind) = tmp
		endfor
		nn = n_elements(hdr)
		thdr = [hdr(0:inds(0)-1), lamhdr, sathdr, hdr(inde1(0)+1:nn-1)]
	      endelse
jump:		
              HANDLE_VALUE, PHTSCPT.HDR, THDR, /SET
              HANDLE_VALUE, PHTSCPT.AVER, TMNPW, /SET
              HANDLE_VALUE, PHTSCPT.AVUN, TMNPU, /SET
              HANDLE_VALUE, PHTSCPT.MEDI, TMDPW, /SET
              HANDLE_VALUE, PHTSCPT.Q1ME, TQ1PW, /SET
              HANDLE_VALUE, PHTSCPT.Q3ME, TQ3PW, /SET
              HANDLE_VALUE, PHTSCPT.FLAG, TFLAG, /SET
              ttime = float(time)
              ttime(*) = 0.
              HANDLE_VALUE, PHTSCPT.TEMP, TTIME, /SET
              HANDLE_VALUE, PHTSCPT.FCS1, TTIME, /SET
              HANDLE_VALUE, PHTSCPT.FCS2, TTIME, /SET
              tarsi = tmnpw
              tarsi(*) = 0.
              HANDLE_VALUE, PHTSCPT.ARSI, TARSI, /SET
              HANDLE_VALUE, PHTSCPT.ANDR, TARSI, /SET
              HANDLE_VALUE, PHTSCPT.AVSD, TARSI, /SET
;  Update the PHTSCP structure using the temporary structure
              PHTSCP = [PHTSCP,PHTSCPT]
 
 	 ENDFOR

	ENDIF ELSE BEGIN

; This is for PIA SPD FITS files	
;  Check if all measurements are found
	 JUNK = WHERE(PS NE -1, DIMR) 
   	 IF (DIMR NE NMEAS) THEN PRINT,'meas(HDR)= ', NMEAS,$
			'  /= number of meas', DIMR 
         TREFITKU = FXPAR(HDR,'TREFITKU')
         FOR I = 0, DIMR - 1 DO BEGIN
 	      PHTSPDT = {PHTSPD_TYPE}
		PHTSPDT.HDR = HANDLE_CREATE()
       	 	PHTSPDT.KYID = HANDLE_CREATE()
       	 	PHTSPDT.FILT = HANDLE_CREATE()
       	 	PHTSPDT.APER = HANDLE_CREATE()
        	PHTSPDT.POLZ = HANDLE_CREATE()
	        PHTSPDT.NDRS = HANDLE_CREATE()
        	PHTSPDT.DWEL = HANDLE_CREATE()
	        PHTSPDT.MEAS = HANDLE_CREATE()
 	       	PHTSPDT.CPOS = HANDLE_CREATE()
  	      	PHTSPDT.TIME = HANDLE_CREATE()
  	      	PHTSPDT.MNPW = HANDLE_CREATE()
   	     	PHTSPDT.MNPU = HANDLE_CREATE()
    	    	PHTSPDT.MDPW = HANDLE_CREATE()
    	    	PHTSPDT.Q1PW = HANDLE_CREATE()
        	PHTSPDT.Q3PW = HANDLE_CREATE()
        	PHTSPDT.DEC = HANDLE_CREATE()
        	PHTSPDT.RA = HANDLE_CREATE()
        	PHTSPDT.ROLL = HANDLE_CREATE()
        	PHTSPDT.FLAG = HANDLE_CREATE()
        	PHTSPDT.CSTP = HANDLE_CREATE()
        	PHTSPDT.PLEN = HANDLE_CREATE()
        	PHTSPDT.TKEY = HANDLE_CREATE()
        	PHTSPDT.RPID = HANDLE_CREATE()
;  Establish Idname
	      det = STRMID(NAME, 0, 2) 
	      
	      IF (I LE 9) THEN $
	        IDNAME = DET + '0' + STRTRIM(STRING(PPS(I)),1) + STRMID(NAME,4,8) $
	        ELSE IDNAME = DET + STRTRIM(STRING(PPS(I)),1) + STRMID(NAME,4,8) 
;  Fill values
              pia_conv_date, PHTCST.UTST(0, PPS(I)), date
              PHTSPDT.UNIT = date
	      PHTSPDT.DET  = DET
	      PHTSPDT.ADMN = IDNAME
              PHTSPDT.ORDR = I
              PHTSPDT.DIMS = PE(I)-PS(I)+1

		  TMNPW = MNPW(*,PS(I):PE(I))  
		  TMNPU = MNPU(*,PS(I):PE(I))  
		  TMDPW = MDPW(*,PS(I):PE(I))  
		  TQ1PW = Q1PW(*,PS(I):PE(I))  
		  TQ3PW = Q3PW(*,PS(I):PE(I))  
		  TPLEN = PLEN(*,PS(I):PE(I))  
		  TFLAG = FLAG(*,PS(I):PE(I))  

              HANDLE_VALUE, PHTSPDT.HDR, HDR, /SET           
              HANDLE_VALUE, PHTSPDT.TKEY, TKEY(PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.RPID, RPID(*,PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.FILT, FILT(PS(I):PE(I)), /SET           
              HANDLE_VALUE, PHTSPDT.KYID, KYID(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.POLZ, POLZ(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.APER, APER(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.NDRS, NDRS(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.CSTP, CSTP(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTSPDT.DWEL, DWEL(PS(I):PE(I)),  /SET
              HANDLE_VALUE, PHTSPDT.MEAS, MEAS(PS(I):PE(I)), /SET
              HANDLE_VALUE, PHTSPDT.CPOS, CPOS(PS(I):PE(I)), /SET
 	      TIME = TKEY(PS(I):PE(I)) - TKEY(PS(I)) 
              HANDLE_VALUE, PHTSPDT.TIME, TIME * TREFITKU, /SET
;  Possible two-dimensional arrays


	      HANDLE_VALUE, PHTSPDT.MNPW, TMNPW, /SET 
              HANDLE_VALUE, PHTSPDT.MNPU, TMNPU, /SET 
	      HANDLE_VALUE, PHTSPDT.MDPW, TMDPW, /SET 
	      HANDLE_VALUE, PHTSPDT.Q1PW, TQ1PW, /SET 
	      HANDLE_VALUE, PHTSPDT.Q3PW, TQ3PW, /SET 
              HANDLE_VALUE, PHTSPDT.PLEN, TPLEN, /SET 
              HANDLE_VALUE, PHTSPDT.FLAG, TFLAG, /SET 
;  Update the PHTSPD structure using the temporary structure
              PHTSPD = [PHTSPD,PHTSPDT]
 	 ENDFOR

	ENDELSE
	FREE_LUN, UNIT
	
	
	RETURN
        END
