;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO read_srd, name, path = path
;+
; NAME:
;       READ_SRD
; PURPOSE:
;	READ a PHTSRD (Interactive Signal Data measurement) fits file 
;	into the dynamical structure PHTSRD  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_SRD, NAME
;
; INPUT:
;       NAME 	: SRD-fitsfile name
;
; OPT. INPUT:
;       PATH    : path for the file
;
; RESTRICTIONS:
;       NAME must be a valid PHTSRD fits file 
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               adm
;               phtsrd
;       PIA Procedure(s):
;               mergehdr
;               read_isrd
;       Astronomical Library:
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: W M Tai (adapted from C Gabriel in read_isrd.pro)
;	Modified (last) by CG		November 1996
;	V5.x
;	Bug fixed (long headers) - CG	January 1997
;	V6.x
;
;-
;
@adm
@phtsrd
;
  ON_ERROR, 2


; create filename esp if path is set
  if keyword_set(PATH) then $
    filename = PATH + NAME $
  else filename = !FITDIR + NAME

; read Compact status file
  csname = 'PSTA' + STRMID(name,4,8) + '.FITS'
  IF NOT keyword_set(PATH) then $
   read_cs, csname  ELSE BEGIN
    read_cs, csname, path=path, status=status
    IF (status) THEN BEGIN
      PRINT, 'Trying now to read CS from FITDIR ...'
      read_cs, csname
    ENDIF
   ENDELSE 
;
; read in the fits header and pass the current file pointer to cur_ptr
;
  read_fits_header, filename, hdr, stat, cur_ptr = cur_ptr

; Extract the detector ID
id = strmid(name, 0, 2)
CASE ID OF

'C1': BEGIN
; C100
  BUF_TYPE = { SLOP: FLTARR(11), $
	       SLUN: FLTARR(11), $
	       FLAG: FLTARR(11), $
               RESI: FLTARR(11), $
               NROS: FLTARR(11), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: bytarr(2)}

      END
'C2': BEGIN
; C200 
  BUF_TYPE = { SLOP: FLTARR(6), $
	       SLUN: FLTARR(6), $
	       FLAG: FLTARR(6), $
               RESI: FLTARR(6), $
               NROS: FLTARR(6), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: bytarr(2)}
      END
'SL': BEGIN
; S
  BUF_TYPE = { SLOP: FLTARR(132), $
	       SLUN: FLTARR(132), $
	       FLAG: FLTARR(132), $
               RESI: FLTARR(132), $
               NROS: FLTARR(132), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: FLTARR(2), $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: bytarr(2)}
      END
ELSE:
ENDCASE

IF STRMID(ID, 0, 1) EQ 'P' THEN BEGIN
; P
  BUF_TYPE = { SLOP: 0.0, $
	       SLUN: 0.0, $
	       FLAG: 0.0, $
               RESI: 0.0, $
               NROS: 0.0, $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: bytarr(2) }
ENDIF

;  Create the structures and the handles
        PHTSRDT = {PHTSRD_TYPE}
        PHTSRDT.HDR = HANDLE_CREATE()
        PHTSRDT.SLOP = HANDLE_CREATE()
        PHTSRDT.SLUN = HANDLE_CREATE()
        PHTSRDT.FLAG = HANDLE_CREATE()
        PHTSRDT.RESI = HANDLE_CREATE()
        PHTSRDT.NROS = HANDLE_CREATE()
        PHTSRDT.STIM = HANDLE_CREATE()
        PHTSRDT.CHOP = HANDLE_CREATE()
        PHTSRDT.STEP = HANDLE_CREATE()
        PHTSRDT.FCSN = HANDLE_CREATE()
        PHTSRDT.TEMP = HANDLE_CREATE()
        PHTSRDT.FCS1 = HANDLE_CREATE()
        PHTSRDT.FCS2 = HANDLE_CREATE()
        PHTSRDT.TKEY = HANDLE_CREATE()
        PHTSRDT.RPID = HANDLE_CREATE()

  phtsrdt.unit = fxpar(hdr, 'SRD_UNIT')
  phtsrdt.det  = fxpar(hdr, 'SRD_DET')
  phtsrdt.ordr = fxpar(hdr, 'SRD_ORDR')
  phtsrdt.dims = fxpar(hdr, 'SRD_DIMS')
  phtsrdt.rchp = fxpar(hdr, 'SRD_RCHP')
  phtsrdt.sldi = fxpar(hdr, 'NAXIS2')
  res = strpos(name, '_')
  phtsrdt.admn = strmid(name, 0, res)
  buf = replicate(buf_type, phtsrdt.sldi)

;
; start to read in the binary data
; we use the current file pointer position from read_fits_header
; to move the newly opened file pointer position to the begining
; of the binary data
;
  OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN
  point_lun, unit, cur_ptr
  readu, unit, buf

  IF STRMID(ID, 0, 1) EQ 'P' THEN BEGIN
    tslop = fltarr(1, phtsrdt.sldi)
    tslun = tslop
    tflag = tslop
    tresi = tslop
    tnros = tslop

    tslop(0,*) = buf.slop
    tslun(0,*) = buf.slun
    tflag(0,*) = buf.flag
    tresi(0,*) = buf.resi
    tnros(0,*) = buf.nros
  endif else begin
    tslop = buf.slop
    tslun = buf.slun
    tflag = buf.flag
    tresi = buf.resi
    tnros = buf.nros
  endelse

  tstim = buf.stim
  tchop = buf.chop
  tstep = buf.step
  tfcsn = buf.fcsn
  ttemp = buf.temp
  tfcs1 = buf.fcs1
  tfcs2 = buf.fcs2
  ttkey = buf.tkey
  trpid = buf.rpid

  byteorder, tslop, /xdrtof
  byteorder, tslun, /xdrtof
  byteorder, tresi, /xdrtof
  byteorder, tnros, /xdrtof
  byteorder, tstim, /xdrtod
  byteorder, tchop, /xdrtof
  byteorder, tstep, /xdrtof
  byteorder, tfcsn, /xdrtof
  byteorder, ttemp, /xdrtof
  byteorder, tfcs1, /xdrtof
  byteorder, tfcs2, /xdrtof
  byteorder, ttkey, /xdrtof
  byteorder, tflag, /ntohs
;  byteorder, trpid, /xdrtof
  
free_lun, unit

;  Fill the handles
        HANDLE_VALUE, PHTSRDT.HDR, HDR, /SET  
        HANDLE_VALUE, PHTSRDT.SLOP, TSLOP, /SET     
	HANDLE_VALUE, PHTSRDT.SLUN, TSLUN, /SET   
        HANDLE_VALUE, PHTSRDT.FLAG, TFLAG, /SET  
        HANDLE_VALUE, PHTSRDT.RESI, TRESI, /SET  
        HANDLE_VALUE, PHTSRDT.NROS, TNROS, /SET  
        HANDLE_VALUE, PHTSRDT.STIM, TSTIM, /SET   
        HANDLE_VALUE, PHTSRDT.CHOP, TCHOP, /SET   
        HANDLE_VALUE, PHTSRDT.STEP, TSTEP, /SET   
        HANDLE_VALUE, PHTSRDT.FCSN, TFCSN, /SET   
        HANDLE_VALUE, PHTSRDT.TEMP, TTEMP, /SET   
        HANDLE_VALUE, PHTSRDT.FCS1, TFCS1, /SET   
        HANDLE_VALUE, PHTSRDT.FCS2, TFCS2, /SET   
        HANDLE_VALUE, PHTSRDT.TKEY, TTKEY, /SET   
        HANDLE_VALUE, PHTSRDT.RPID, TRPID, /SET   

;  Update the PHTSRD structure using the temporary structure
;  Below is copied from CG's read_isrd.pro, WMT, 31/1/95
        PHTSRD = [PHTSRD,PHTSRDT]

;  Does exist already an administration record for this measurement?
;	       IDNAME = PHTSRDT.ADMN
;	       DETID = PHTSRDT.DET
;               ADMIND = WHERE(ADM.NAME EQ IDNAME, N)

;  Update the administration file using temporary structure
;               IF (N EQ 1) THEN BEGIN
;                ADM(ADMIND).PHTSRD = N_ELEMENTS(PHTSRD) - 1
;                IF (DETID EQ 'SS') THEN BEGIN
;                  STRPUT, IDNAME, 'L', 1
;                  ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;                  IF (N EQ 1) THEN $
;                        ADM(ADMIND2).PHTSRD = N_ELEMENTS(PHTSRD) - 1 $
;                   ELSE IF (N EQ 0) THEN BEGIN
;                        ADMT = {ADM_TYPE}
;                        ADMT.NAME = IDNAME
;                        SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                        ADMT.PHTSRD = SRDPOS
;                        ADM = [ADM, ADMT]
;                  ENDIF ELSE PRINT, 'READ_SRD: Several identical ADM-records'
;                ENDIF
;               ENDIF ELSE IF (N EQ 0) THEN BEGIN
;                   ADMT = {ADM_TYPE}
;                   ADMT.NAME = IDNAME
;                   SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                   ADMT.PHTSRD = SRDPOS
;                   ADM = [ADM, ADMT]
;                IF (DETID EQ 'SS') THEN BEGIN
;                  STRPUT, IDNAME, 'L', 1
;                  ADMIND2 = WHERE(ADM.NAME EQ IDNAME, N)
;                  IF (N EQ 1) THEN $
;                        ADM(ADMIND2).PHTSRD = N_ELEMENTS(PHTSRD) - 1 $
;                   ELSE IF (N EQ 0) THEN BEGIN
;                        ADMT = {ADM_TYPE}
;                        ADMT.NAME = IDNAME
;                        SRDPOS = N_ELEMENTS(PHTSRD) - 1
;                        ADMT.PHTSRD = SRDPOS
;                        ADM = [ADM, ADMT]
;                  ENDIF ELSE PRINT, 'READ_SRD: Several identical ADM-records'
;                ENDIF
;               ENDIF ELSE PRINT, 'READ_SRD: Several identical ADM-records'

  RETURN
END
