;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	RED_RAMPS
; PURPOSE:
;	Reduce several variables from read-out level to signal level. Return
;	chopper pos., temp., FCS values, time key and raster point per ramp
;	using the flag status for taking just the selected read-outs
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;    red_ramps, tmperd, st, chincr, rampend, otf, dr, $
;	ychop, ystep, yfcsn, ytemp, yfcs1, yfcs2, ytkey, yrpid, ycflg
;
; INPUT PARAMETERS:
;	TMPERD	: Temporary Structure ERD 
;	ST	: Flag status (0 = OK, 1 = bad flag)
;	CHINCR	: Chopper increment
;	RAMPEND : Ramp end status array (1=end)
;	OTF	: Flag-array for On Target Flag (ERD level)
;	DR	: Destructive Read-out Flag (ERD level)
;
; OUTPUT PARAMETERS:
;	YCHOP	: Array - medians of chopper position / ramp
;	YSTEP	: Array - medians of chopper step / ramp
;	YFCSN	: Array - medians of chopper fcsn value / ramp
;	YTEMP	: Array - interpolated temperatures / ramp
;	YFCS1	: Array - medians of FCS1 el. power / ramp
;	YFCS2	: Array - medians of FCS2 el. power / ramp
;	YTKEY	: Array - initial instr. Time Key / ramp
;	YRPID	: Array - medians of  raster position / ramp
;	YCFLG	: Array - flags for chopper position
;	YOTF	: Array - flags for On Target Flag
;
;
;
; RESTRICTIONS:
;	DR and ST must have the same number of elements as variables in
;	TMPERD
;
; METHOD:

; PROCEDURES USED:
;       Astronomical Library:
;               avg
;               boost_array
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel
;	Modified by CG - PIA V5.x	October 1996
;	Bug for very strange configuration (subdivision + 1 element/ramps 
;	  accepted) fixed (CG)		July 1997
;	V6.4
;	Signal chopper positions defined using
;	 all read-outs + check for consistency 
;	 and corresponding flags (CG)	September 1997
;	Change for 1 read-out in ramp (CG)	February 1998
;	V7.1
;	Upgraded using ramp end info (JAcosta)	May 1999
;	Bug for ramp with no valid reads fixed (CG)	May 1999
;	OTF input, YOTF output added
;	Bug for RPID value when no valid reads fixed (CG)	August 1999
;	V8.0
;	DR input added for fixing bug (OTF flag) in case of two
;	read-outs ramps (CG)					November 1999
;	V8.2
;	
;-
PRO RED_RAMPS, TMPERD, ST, CHINCR, RAMPEND, OTF, DR, $
	YCHOP, YSTEP, YFCSN, YTEMP, YFCS1, YFCS2, YTKEY, YRPID, YCFLG, YOTF 
;
; Needed common blocks : none
;
;
; On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 4 THEN MESSAGE, $
'Syntax: RED_RAMPS, TMPERD, ST, CHINCR, RAMPEND'+$
	'[YCHOP, YSTEP, YFCSN, YTEMP, YFCS1, YFCS2, YTKEY, YRPID, YCFLG]' 

; Extract the variables from TMPERD
	xall = tmperd.time
	chop =tmperd.chop
	step =tmperd.step
	fcsn =tmperd.fcsn
	temp =tmperd.temp 
	fcs1 =tmperd.fcs1
	fcs2 =tmperd.fcs2
	tkey =tmperd.tkey
	rpid =tmperd.rpid
	
	dimtemp = SIZE(temp)
	

; Now proceed ramp per ramp
nramp = N_ELEMENTS(rampend)

ychop = FLTARR(nramp)
ystep = INTARR(nramp)
yfcsn = INTARR(nramp)
ytemp = FLTARR(dimtemp(0),nramp)
yfcs1 = FLTARR(nramp)
yfcs2 = FLTARR(nramp)
ytkey = LONARR(nramp)
yrpid = BYTARR(2,nramp)
ycflg = INTARR(nramp)
yotf = BYTARR(nramp)


iramp0 = 0
FOR iramp=0L, nramp-1 DO BEGIN   ;starts loop over ramps
  npt_ramp = rampend(iramp)-iramp0+1
  ind_ramp = INDGEN(npt_ramp) + iramp0
  xxt = xall(ind_ramp)
  fyt = st(ind_ramp)
  ind_ok = WHERE(fyt EQ 0, nok) 

  IF nok GT 0 THEN $
   ind_val = ind_ramp(ind_ok)
   

  CASE 1 OF 
     nok GT 10: BEGIN 
                 ychop(iramp) = MEDIAN(chop(ind_val))
		 ystep(iramp) = MEDIAN(step(ind_val))
		 yfcsn(iramp) = MEDIAN(fcsn(ind_val))
		 yfcs1(iramp) = MEDIAN(fcs1(ind_val))
		 yfcs2(iramp) = MEDIAN(fcs2(ind_val))
		 ytkey(iramp) = MEDIAN(tkey(ind_val))
		 FOR i=0,dimtemp(0)-1 DO $
		     ytemp(i,iramp) = MEDIAN(temp(i,ind_val))
		 FOR i=0,1 DO yrpid(i,iramp) = MEDIAN(rpid(i,ind_val))
                 IF STDEV(chop(ind_val)) GT chincr*0.55 THEN $ 
		     ycflg(iramp) = 1
		 IF (TOTAL(otf(ind_val))+2) GT nok THEN yotf(iramp) = 1
		END 
     nok GT 0: BEGIN 
                 ychop(iramp) = AVG([chop(ind_val)])
		 ystep(iramp) = AVG([step(ind_val)])
		 yfcsn(iramp) = AVG([fcsn(ind_val)])
		 yfcs1(iramp) = AVG([fcs1(ind_val)])
		 yfcs2(iramp) = AVG([fcs2(ind_val)])
		 ytkey(iramp) = AVG([tkey(ind_val)])
		 FOR i=0,dimtemp(0)-1 DO $ 
		     ytemp(i,iramp) = AVG([temp(i,ind_val)])
		 FOR i=0,1 DO yrpid(i,iramp) = AVG([rpid(i,ind_val)])
		 IF nok LT 2 THEN stdchop = 0 $ 
		    ELSE  stdchop = STDEV([chop(ind_val)])
                 IF stdchop GT chincr*0.55 THEN $ 
		     ycflg(iramp) = 1
		 IF (TOTAL(otf(ind_val)) + 2) GT nok THEN yotf(iramp) = 1
		END		 
     ELSE: BEGIN
     	ycflg(iramp) = 1
	yotf(iramp) = max(otf(ind_ramp))
	FOR i=0,1 DO yrpid(i,iramp) = MEDIAN([rpid(i,ind_ramp)])
	ystep(iramp) = AVG([step(ind_ramp)])
	yfcsn(iramp) = AVG([fcsn(ind_ramp)])
	yfcs1(iramp) = AVG([fcs1(ind_ramp)])
	yfcs2(iramp) = AVG([fcs2(ind_ramp)])
	ytkey(iramp) = AVG([tkey(ind_ramp)])
	FOR i=0,dimtemp(0)-1 DO $ 
		     ytemp(i,iramp) = AVG([temp(i,ind_ramp)])
	
     END		
  ENDCASE
        	
  iramp0 = rampend(iramp)+(1*dr(rampend(iramp)))

ENDFOR	   ;close loop over ramps	

ytemp = REFORM(ytemp)


RETURN
END


