;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO RESPONS_INTERPOL, fcs1, fcs2, resps, NO_WIDGET=no_widget, $
          OUTLIM=outlim
;+
; NAME:
;       RESPONS_INTERPOL
; PURPOSE:
;	Interpolates two PHT-Fcs measurements, writing as actual responsivity
;	the weighted average of both (after user's choice) into the common
;	block phtrespact 
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       RESPONS_INTERPOL, fcs1, fcs2, resps[,NO_WIDGET=no_widget] 
;
; INPUT:
;       fcs1: index within PHTSCP of the first fcs measurement  
;	fcs2: index within PHTSCP of the second fcs measurement  
;
; KEYWORD:
;	no_widget: PIA_respons call with /no_widget
;	outlim	 : Set to 1: if fcs1 is OUT_of_limits 
;			  2: if fcs2 is OUT_of_limits  
;			  3: if both OUT_of_limits
;
; OUTPUT:
;	resps: A structure containing
;		resp1: responsivity calculated with the fcs1 measurement
;		respun1: resp1 uncertainty
;		time1: measurement's date (taken from fcs1.unit)
;		orig1n: admn name 1st FCS
;		orig1f: filter 1st FCS
;		orig1m: method used 1st FCS
;		resp2: responsivity calculated with the fcs2 measurement
;		respun2: resp2 uncertainty
;		time2: measurement's date (taken from fcs2.unit)
;		orig2n: admn name 2nd FCS
;		orig2f: filter 2nd FCS
;		orig2m: method used 2nd FCS
;
; RESTRICTIONS:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               phtresact
;               phtscp
;       PIA Procedure(s):
;               pia_load_iscp
;               pia_respons
;               weight_mean
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel
;	Oct 95 Version 3.5
;	Passing back responses and uncertainties (CG)	February 1996
;	pia_respons call changed (CG)			July 1996
;	Keyword NO_widget added (CG)			June 1997
;	V6.3
;	Output keyword outlim added (CG)		February 1998
;	V7.1
;	Extension of resps for info inclusion (CG)	July 99
;	V7.4
;
;-
@phtscp
@phtresact
        IF N_ELEMENTS(phtscp) LE fcs1 OR $
           N_ELEMENTS(phtscp) LE fcs2 THEN BEGIN   

		MESSAGE, /INFO, 'One of the measurements does not exist'
		RETURN
	ENDIF

	outlim=0		
	
	pia_load_iscp, phtscp(fcs1), tmpdata
	newheader = tmpdata.hdr
	IF KEYWORD_SET(no_widget) THEN no_widget=1 ELSE no_widget=0

	PIA_RESPONS, tmpdata, newheader, NO_WIDGET=no_widget, $
	  out_of_limits=out_of_limits
	
	IF out_of_limits THEN outlim=1

	
	CASE STRMID(tmpdata.det,0,2) OF
		'P1': BEGIN
			resp = p1respact  
			respun = p1respunc
			npix = 1
			orig1n = respact_origin.p1n
			orig1f = respact_origin.p1f
			orig1m = respact_origin.p1m
		END  
		'P2': BEGIN
			resp = p2respact  
			respun = p2respunc
			npix = 1
			orig1n = respact_origin.p2n
			orig1f = respact_origin.p2f
			orig1m = respact_origin.p2m
		END  
		'P3': BEGIN
			resp = p3respact  
			respun = p3respunc
			npix = 1
			orig1n = respact_origin.p3n
			orig1f = respact_origin.p3f
			orig1m = respact_origin.p3m
		END  
		'C1': BEGIN
			resp = c1respact  
			respun = c1respunc
			npix = 9
			orig1n = respact_origin.c1n
			orig1f = respact_origin.c1f
			orig1m = respact_origin.c1m
		END  
		'C2': BEGIN
			resp = c2respact  
			respun = c2respunc
			npix = 4
			orig1n = respact_origin.c2n
			orig1f = respact_origin.c2f
			orig1m = respact_origin.c2m
		END  
		ELSE: MESSAGE, /INFO, 'Not recognized detector'
	ENDCASE	   

	resps={resp1:fltarr(npix), respun1:fltarr(npix), time1:'', $
		orig1n:orig1n, orig1f:orig1f, orig1m:orig1m, $
			resp2:fltarr(npix), respun2:fltarr(npix), time2:'',$
		orig2n:'', orig2f:'', orig2m:''}


	resp0 = resp
	resp0un = respun
	resps.resp1 = resp
	resps.respun1 = respun
	resps.time1 = tmpdata.unit

	pia_load_iscp, phtscp(fcs2), tmpdata
	newheader = tmpdata.hdr
	PIA_RESPONS, tmpdata, newheader, NO_WIDGET=no_widget, $
	  out_of_limits=out_of_limits
	
	IF out_of_limits THEN outlim=outlim+2

	CASE STRMID(tmpdata.det,0,2) OF
		'P1': BEGIN
			resp = p1respact  
			respun = p1respunc
			orig2n = respact_origin.p1n
			orig2f = respact_origin.p1f
			orig2m = respact_origin.p1m
		END  
		'P2': BEGIN
			resp = p2respact  
			respun = p2respunc
			orig2n = respact_origin.p2n
			orig2f = respact_origin.p2f
			orig2m = respact_origin.p2m
		END  
		'P3': BEGIN
			resp = p3respact  
			respun = p3respunc
			orig2n = respact_origin.p3n
			orig2f = respact_origin.p3f
			orig2m = respact_origin.p3m
		END  
		'C1': BEGIN
			resp = c1respact  
			respun = c1respunc
			orig2n = respact_origin.c1n
			orig2f = respact_origin.c1f
			orig2m = respact_origin.c1m
		END  
		'C2': BEGIN
			resp = c2respact  
			respun = c2respunc
			orig2n = respact_origin.c2n
			orig2f = respact_origin.c2f
			orig2m = respact_origin.c2m
		END  
		ELSE: MESSAGE, /INFO, 'Not recognized detector'
	ENDCASE	   

	resps.resp2 = resp
	resps.respun2 = respun
	resps.time2 = tmpdata.unit
	resps.orig2n = orig2n
	resps.orig2m = orig2m
	resps.orig2f = orig2f
			
; actually this is not an interpolation, but it has a future ...
; the problem here is that this could be a map (long measurement) and 
; the time interpolation is not trivial in this case

	FOR IPIXL=0,N_ELEMENTS(resp)-1 DO BEGIN
		rr = [resp0(ipixl),resp(ipixl)]
	      	ru = [resp0un(ipixl),respun(ipixl)]
	      
		WEIGHT_MEAN, rr, ru, mean, stdev
	        resp(ipixl) = mean
		respun(ipixl) = stdev
        ENDFOR

	CASE STRMID(tmpdata.det,0,2) OF
		'P1': BEGIN
			p1respact = resp
			p1respunc = respun
			respact_origin.p1n = orig1n+'+'+orig2n 
			respact_origin.p1f = orig1f+'-'+orig2f 
			respact_origin.p1m = orig1m+'-'+orig2m 
		END
		'P2': BEGIN
			p2respact = resp
			p2respunc = respun
			respact_origin.p2n = orig1n+'+'+orig2n 
			respact_origin.p2f = orig1f+'-'+orig2f 
			respact_origin.p2m = orig1m+'-'+orig2m 
		END
		'P3': BEGIN
			p3respact = resp
			p3respunc = respun
			respact_origin.p3n = orig1n+'+'+orig2n 
			respact_origin.p3f = orig1f+'-'+orig2f 
			respact_origin.p3m = orig1m+'-'+orig2m 
		END
		'C1': BEGIN
			C1respact = resp
			C1respunc = respun
			respact_origin.c1n = orig1n+'+'+orig2n 
			respact_origin.c1f = orig1f+'-'+orig2f 
			respact_origin.c1m = orig1m+'-'+orig2m 
		END
		'C2': BEGIN
			c2respact = resp
			c2respunc = respun
			respact_origin.c2n = orig1n+'/'+orig2n 
			respact_origin.c2f = orig1f+'/'+orig2f 
			respact_origin.c2m = orig1m+'/'+orig2m 
		END
		ELSE: MESSAGE, /INFO, 'Not recognized detector'
	ENDCASE	   
	
	RETURN
	END
