;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	RLIN_PROC
;
; PURPOSE:
;	Apply the signal linearization to an SRD/SCP PHT measurement
;
; CATEGORY:
;	PIA - Processing
;                
; CALLING SEQUENCE:
;	RLIN_PROC, TMP, NEWHDR=NEWHDR, SRD=SRD, SIG_COR=SIG_COR 
;
; INPUT PARAMETERS:
;	tmp:		temporary SRD or SCP structure (it get updated) (**)
;	
; KEYWORDS:
;	SRD:		to be set if measurement of SRD level
;
; OPT.INPUT KEYWORDS:
;	newhdr:		measurement header (it get updated)
;
; OPT.OUTPUT KEYWORDS:
;	sig_cor:	the corrected signals after linearization
;		 	
; KEYWORDS:
;	INFO		getting the messages instead of the console
;
; OUTPUT:
;	sig_in	: 	Array containing signals as observed
;	sig_out : 	Array (same dims as sig_in) containing corrected signals
;			after detector response linearization
;	limits  :	2-elements array containing lower und upper validity
;			bounds
;
; PROCEDURES USED:
;
; (**) NOTE: all affected variables are changed
;	accordingly, eg. slun in the SRD case, and avun, medi, q1me, q3me 
;	in the SCP case.
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel - ESA-VILSPA/SAI 		Dec.98
;	INFO keyword added (CG)					Jun.99
;	Other variables changed (slun ...) CG			Jun.99
;	V7.4
;	rlin_tables under !CALG (CG)				Jul 99
;	V7.9
;	Bug in uncertainty removed (CG)				Jul 99
;	Call to rlin_fct including det as keyword (CG)		Jul 99
;	V8.0
;
;-
PRO rlin_proc, tmp, newhdr=newhdr, SRD=srd, sig_cor=sig_cor, info=info

@phtfilter
	
	IF not KEYWORD_set(newhdr) or N_ELEMENTS(newhdr) EQ 0 THEN $
	  newhdr = tmp.hdr

	INFO = ''
	
	IF KEYWORD_SET(srd) THEN BEGIN
		sig = tmp.slop
		sigunc = tmp.slun 
	ENDIF ELSE BEGIN
		sig = tmp.aver
		sigunc = tmp.avun
		medv = tmp.medi
		med1q = tmp.q1me
		med3q = tmp.q3me
	ENDELSE
	
	sig_cor = sig
	
	slin = FXPAR(newhdr,'PRS_SLIN', count=counts)
	slin = FXPAR(newhdr,'PRC_SLIN', count=countc)

	IF counts + countc NE 0 THEN BEGIN
	  info = ' Signal linearity correction already performed, returning ...'
	  MESSAGE, /INFO, info
	  RETURN
	ENDIF  
; directory containing the tables
	tabdir = !CALG + 'rlin_tables/'

; init filters if necessary
	IF N_ELEMENTS(c1filters) EQ 0 THEN phtfilter_init
        allcfilters = [c1filters, c2filters,'C_cut']
        allpfilters = [p1filters, p2filters, p3filters]
 
; Read the Compact Status if necessary or get the unit - change -> shape4
	give_cs, tmp, cst, record, cst_conv, ERROR = isnot_cs
	IF isnot_cs THEN BEGIN 
	  csfile = 'PSTA'+strmid(tmp.admn,4,8)+'.FITS'
	  read_cs, csfile
	  give_cs, tmp, cst, record, cst_conv
	ENDIF  

	CASE STRMID(tmp.det, 0,2) OF
	  'P1': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'P2': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'P3': BEGIN
	  	filtnr = cst.c3ps(record) - 1
		npixel = 1
		filter = allpfilters(filtnr)
;		asc_fil = allpfilters(filtnr) + '_1_sig.txt'
		END		
	  'C1': BEGIN
	  	filtnr = cst.c2ps(record) - 2
		npixel = 9
		filter = allcfilters(filtnr)
;		asc_fil = allcfilters(filtnr) + $
;		       '_'+ STRTRIM(indgen(npixel)+1,2)+'_sig.txt'
		END
	  'C2': BEGIN
	  	filtnr = cst.c2ps(record) - 2
		npixel = 4
		filter = allcfilters(filtnr)
;		asc_fil = allcfilters(filtnr) + $
;		       '_'+ STRTRIM(indgen(npixel)+1,2)+'_sig.txt'
		END
	  ELSE: BEGIN
	  	MESSAGE, /INFO, 'Wrong detector, returning ...'
		RETURN
		END
	ENDCASE
	
	
	FOR ipix=0,npixel-1 DO BEGIN

	  sigtmp = rlin_fct(sig(ipix,*), filter, ipix+1, det=tmp.det)
	  sig_cor(ipix,*) = sigtmp
	  inot0 = WHERE(sig(ipix,*) NE 0., nn0)
	  IF nn0 GT 0 THEN BEGIN
	   sigunc(ipix,inot0) = sigunc(ipix, inot0) * $
	   			sig_cor(ipix,inot0) / sig(ipix,inot0) 
	  ENDIF
	  IF not KEYWORD_SET(srd) THEN BEGIN
	     medtmp = rlin_fct(medv(ipix,*), filter, ipix+1, det=tmp.det)
	     inot0 = WHERE(sig(ipix,*) NE 0., nn0)
	     IF nn0 GT 0 THEN BEGIN
	        med1q(ipix,inot0) = med1q(ipix, inot0) * $
	   			medtmp(inot0) / medv(ipix,inot0)
	        med3q(ipix,inot0) = med3q(ipix, inot0) * $
	   			medtmp(inot0) / medv(ipix,inot0) 
	     ENDIF
	     medv(ipix,*) = medtmp
	  ENDIF   
	ENDFOR
	
	IF KEYWORD_SET(srd) THEN BEGIN
	   tmp.slop=sig_cor 
	   tmp.slun=sigunc 
	   FXADDPAR, newhdr, 'PRS_SLIN', 1, $
	     ' Signal linearity correction performed', BEFORE='XTENSION'
	ENDIF ELSE BEGIN
	   tmp.aver=sig_cor
	   tmp.avun=sigunc
	   tmp.medi=medv
	   tmp.q1me=med1q
	   tmp.q3me=med3q
	   FXADDPAR, newhdr, 'PRC_SLIN', 1, $
	     ' Signal linearity correction performed', BEFORE='XTENSION'
	ENDELSE
	
	RETURN
	END
		
	
