;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO FIT_RAMPS_SM, IPIX, XALL, YALL, NDEG, DR, ST, TINT, YLIN, YSLOP, $
	               YSLOPR, YFLAG, YTIME, YRESI, YNROS 
;+
; NAME:
;	FIT_RAMPS_SM
;
; PURPOSE:
;	Fit a polynomial to the voltage ramps of a measurement returning 
;	the slopes, their uncertainties and the fitted lines.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;    FIT_RAMPS_SM, IPIX, Xall, Yall, Ndeg, Dr, St, Tint, Ylin, Yslop, Yslopr, $
;	        Yflag, Ytime, Yresi, Ynros
;
; INPUTS:
;	ipix	: The pixel number
;	Yall	: Array containing values of read-outs from a measurement [V]
;	Xall	: Array containing the independent variable [t]  (same size)
;	Ndeg	: Polynomial degree
;	Dr	: Read-out status (0 = NDR, 1 = DR) 
;	St	: Flag status (0 = OK, 1 = bad flag)
;	Tint	: On chip integration time applied in the measurement
;
; OUTPUTS:
;	Ylin	: Array containing the fitted lines
;	Yslop	: Array of found slopes 
;	Yslopr	: Array of uncertainties in slopes
;	Yflag	: Array of flags (0:ok; 1:2 read-outs used; 2:bad)
;	Ytime	: Slopes times   
;	Yresi	: Array with effective reset intervals used
;	Ynros	: Array with effective number of read-outs used
;
; RESTRICTIONS:
;	Yall, Xall, Dr and Tint must have the same number of elements
;
; PROCEDURES USED:
;	None
;
; MODIFICATION HISTORY:
;	Written by:	Carlos Gabriel		
;	Modified (last) by CG:	October 1996
;	V5.x
;	Flagging with 1 pseudo_ramps with less than !subd_ramp/2. 
;	 to avoid problems with too small uncertainties
;	Bug fixed: start time of pseudo-ramp was shifted one read-out (CG)
;	V6.3 
;       modified for use with Serendipity subdivided ramps (SB) June 1997
;       absolute path for the cal file 'c200_debcorr.sav' (SB)  December 1997
;-
;
;On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 13 THEN MESSAGE, $
'Syntax: FIT_RAMPS_SM,IPIX,XALL,YALL,NDEG,DR,ST,TINT,YLIN,YSLOP,YSLOPR'+$
',YFLAG,YTIME,YRESI,YNROS'
;

        COMMON c200_deb_corr, c2_corr, c2_unc, volt_diff
        IF N_ELEMENTS(c2_corr) EQ 0 THEN $
         RESTORE,'/data/serend/Software/calsave/c200_debcorr.sav'

	SUM = N_ELEMENTS(XALL)
	IF SUM NE N_ELEMENTS(YALL) THEN $
		MESSAGE, 'Xall and Yall must have same number of elements'
;
;  Compute an array with the ramp# of each read-out and an array
;  with the time elapsed after begin of the ramp would be the solution for
;  perfect data, e.g. without drop-outs - and without the larger gaps between 
;  chopper plateaus
;
; ;;;;;;;	NRRAMP = FIX(XALL / RAMP_TIME)
; ;;;;;;;	ELTIME = XALL MOD RAMP_TIME
;
;  Instead we have to apply a more complicated approach
;  based on the integration time and on the read-out  status
;
; 
	XXT = XALL(0)
	YYT = YALL(0)
	FYT = ST(0)
	TINIT = XXT
	TINIT0 = TINIT
	II = 1L
	DRIND = DR(0)
	NRAMP = 0L
        NSUB = 0
        YSTART = YALL(0)
;  Problems can arise from comparing time differences - Make first 
;  the integration time just 1/1000 smaller to avoid them, then there
;  are not more than 128 read-outs per ramp and the first read-out
;  of the new ramp can be exactly TINT from the first of the last one
	TIN = TINT * 0.999
;  The second read-out could already belong to the second ramp, so the 
;  third to the third, etc. Keep on reading until at least two read-outs 
;  belonging to one ramp show up 
	TIN = TIN(0)
	WHILE (XALL(II) - TINIT) GT TIN DO BEGIN
		TINIT = XALL(II)
		TINIT0 = TINIT
		II = II + 1
		YFIT = 0.
		SLOPE = 0.
		SIGMA_SLOPE = 0.
		RESI = 0.
		NROS = 0
		FLAG = 2 
		IF (NRAMP EQ 0) THEN BEGIN
			YLIN = YFIT
			YSLOP = SLOPE
			YSLOPR = SIGMA_SLOPE
			YRESI = RESI
			YNROS = NROS
			YFLAG = FLAG
			YTIME = TINIT		
		ENDIF ELSE BEGIN
			YLIN = [YLIN, YFIT]
			YSLOP = [YSLOP, SLOPE]
			YSLOPR = [YSLOPR,SIGMA_SLOPE]
			YRESI = [YRESI,RESI]
			YNROS = [YNROS, NROS]
			YFLAG = [YFLAG,FLAG]
			YTIME = [YTIME, TINIT]
		ENDELSE
		NRAMP = NRAMP + 1L
		DRIND = DR(II)
	ENDWHILE
	TINIT0 = TINIT
;
;  Take read-out after read-out until a ramp is completed: recognized by the
;  read-out status of the former read-out - (and for drop-out cases by a time
;  difference larger than the integration time) - Fit a polynomial (degree  
;  NDEG), initialize the output arrays for the first ramp, increase them 
;  for the other ramps.
;

	ro_limit = !SUBD_ramp / 2.
	delta = XALL(1)
	NNN = N_ELEMENTS(XALL) - 1
	old_rampcase = 2
	FOR I = II, NNN DO BEGIN
	   rampcase = 0
	   IF (DRIND EQ 1) OR (XALL(I) - TINIT0) GT TIN THEN rampcase=1
	   IF !subd_ramp NE 0 AND rampcase NE 1 AND $
	      N_ELEMENTS(xxt) EQ !subd_ramp THEN      rampcase = 2
	   IF rampcase GT 0 THEN BEGIN
                NSUB = NSUB+1
	        INDOK = WHERE(FYT EQ 0, NELEM)
		YF0 = FLTARR(N_ELEMENTS(FYT))
		IF NELEM GT 1 THEN BEGIN  
 		   XXT = XXT(INDOK) - XXT(INDOK(0))
 		   AVGXXT = AVG(XXT)
		   YYT = YYT(INDOK)
		   RESULT = $
		      POLY_FIT(XXT, YYT, NDEG, YFF, YBAND, SIGMA, COVM) 
		   SLOPE = RESULT(1)
		   SIGMA_SLOPE = 0.0
;		   SIGMA_SLOPE = SQRT(COVM(1,1))
                   IF IPIX LE 3 THEN BEGIN 
                      CORR = INTERPOL(C2_CORR(IPIX,NSUB-1,*),VOLT_DIFF,$
                                      AVG(YYT)-YSTART)
                      SLOPE = SLOPE/CORR
                   ENDIF
       		   RESI = max(XXT)
		   NROS = N_ELEMENTS(XXT)
;  To maintain the dimension fill first with 0s
		   YFIT = YF0
		   YFIT(INDOK) = YFF
		   IF NELEM EQ 2 or NELEM LE ro_LIMIT THEN BEGIN
		      FLAG = 1 
		      sigma_slope = 0.
		   ENDIF ELSE FLAG = 0
		   
		ENDIF ELSE BEGIN 
;  To maintain the dimension fill with 0s
		   YFIT = YF0
		   SLOPE = 0.
		   SIGMA_SLOPE = 0.
		   RESI = 0.
		   NROS = 0
		   FLAG = 2 
		   AVGXXT = 0.
		ENDELSE
		IF (NRAMP EQ 0) THEN BEGIN
			YLIN = YFIT
			YSLOP = SLOPE
			YSLOPR = SIGMA_SLOPE
			YRESI = RESI
			YNROS = NROS
			YFLAG = FLAG
			YTIME = TINIT + AVGXXT
		ENDIF ELSE BEGIN
			IF rampcase EQ 2 AND old_rampcase NE 1 $
			    THEN YFIT = YFIT(1:*)
			IF rampcase EQ 1 THEN old_rampcase = 1 ELSE $
						old_rampcase = 2
			YLIN = [YLIN, YFIT]
			YSLOP = [YSLOP, SLOPE]
			YSLOPR = [YSLOPR,SIGMA_SLOPE]
			YRESI = [YRESI,RESI]
			YNROS = [YNROS, NROS]
			YFLAG = [YFLAG,FLAG]
			YTIME = [YTIME,TINIT+AVGXXT]
		ENDELSE
		IF rampcase EQ 1 THEN BEGIN
		 NRAMP = NRAMP + 1L
		 TINIT = XALL(I)
		 TINIT0 = TINIT
		 XXT = XALL(I)
		 YYT = YALL(I)
		 FYT = ST(I)
                 NSUB = 0
                 YSTART = YALL(I)
		ENDIF ELSE BEGIN
		 NRAMP = NRAMP + 1L
		 TINIT = XALL(I-1)
		 XXT = [XALL(I-1), XALL(I)]
		 YYT = [YALL(I-1), YALL(I)]
		 FYT = [ST(I-1), ST(I)]
		ENDELSE 
	   ENDIF ELSE BEGIN
		   XXT = [XXT,XALL(I)]
		   YYT = [YYT,YALL(I)]
		   FYT = [FYT,ST(I)]
	   ENDELSE
	   DRIND = DR(I)
	ENDFOR
; For the last Read-out 
                NSUB = NSUB+1
	        INDOK = WHERE(FYT EQ 0, NELEM)
		YF0 = FLTARR(N_ELEMENTS(FYT))
		IF NELEM GT 1 THEN  BEGIN  
		   XXT = XXT(INDOK) - xxt(indok(0))
		   YYT = YYT(INDOK)
		   AVGXXT = AVG(XXT)
		   RESULT = $
		     POLY_FIT(XXT, YYT, NDEG, YFF, YBAND, SIGMA, COVM) 
		   SLOPE = RESULT(1)
 		   SIGMA_SLOPE = 0.0
;		   SIGMA_SLOPE = SQRT(COVM(1,1))
                   IF IPIX LE 3 THEN BEGIN 
                      CORR = INTERPOL(C2_CORR(IPIX,NSUB-1,*),VOLT_DIFF,$
                                      AVG(YYT)-YSTART)
                      SLOPE = SLOPE/CORR
                   ENDIF
		   RESI = max(XXT)
		   NROS = N_ELEMENTS(XXT)
;  To maintain the dimension fill first with 0s
		   YFIT = YF0
		   YFIT(INDOK) = YFF
		   IF N_ELEMENTS(XXT) EQ 2 or $
		     N_ELEMENTS(XXT) LE ro_LIMIT THEN BEGIN
		     FLAG = 1
		     SIGMA_SLOPE = 0.
		   ENDIF ELSE FLAG = 0
		ENDIF ELSE BEGIN 
;  To maintain the dimension fill first with 0s
		   YFIT = YF0
		   SLOPE = 0.
		   SIGMA_SLOPE = 0.
		   RESI = 0.
		   NROS = 0
		   FLAG = 2 
		   AVGXXT = 0.
		ENDELSE
		IF (NRAMP EQ 0) THEN BEGIN
			YLIN = YFIT
			YSLOP = SLOPE
			YSLOPR = SIGMA_SLOPE
			YRESI = RESI
			YNROS = NROS
			YFLAG = FLAG
			YTIME = TINIT + AVGXXT
		ENDIF ELSE BEGIN
			YLIN = [YLIN, YFIT]
			YSLOP = [YSLOP, SLOPE]
			YSLOPR = [YSLOPR,SIGMA_SLOPE]
			YRESI = [YRESI,RESI]
			YNROS = [YNROS, NROS]
			YFLAG = [YFLAG,FLAG]
			YTIME = [YTIME,TINIT+AVGXXT]
		ENDELSE
	RETURN
	END
