;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_PROC_BATCH_MANAGER_SM_NOX
;
; PURPOSE:
;       Menu and procedure within the PIA to get automatically
;       a sequence of steps to be performed, without major interaction
;	for Serendipity measurements.
;       Data reduction from ERD up to SMC level and point source extraction, 
;	including calibration.
;	This menu is on top of PIA_PROC_BATCH_SM_NOX and loop over several files
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       PIA_PROC_BATCH_MANAGER_SM_NOX
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	none
;
;
; COMMON BLOCKS:
;	phterd:		ERD dynamical structure
;	phtsrd:		SRD dynamical structure
;	phtsmc:		SMC dynamical structure
;       gencho_sm_cmn   for returning several variables
;       phtacc		for selection criteria
;
; PROCEDURES USED:
;	PIA Procedure(s):
;
; MODIFICATION HISTORY:
;       Written by: 	Stefan Bogun (MPIA)			July 1996
;	adapted from PIA_PROC_BATCH_MANAGER written by Carlos Gabriel
;
;	cleaning all buffers (SB, October 1996)
;	also cleaning IIPH buffer (SB, November 1996)
;	correct handling of CANCEL button  (SB, November 1996)
;	processing window remains the same (SB, November 1996)
;	changing path in file selectors works now (SB, December 1996)
;	reading files from !XDRDIR rather than !INTDIR (SB, February 1997)
;       slight corrections due to changed parameters (SB, December 1997)
;-

PRO PIA_PROC_BATCH_MANAGER_SM_NOX, fillist

@gencho_sm
@phterd
@phtsrd
@phtsmc
@phtacc

   IF N_ELEMENTS(gen_erd) EQ 0 THEN gen_erd = 0


; Serendipity parameter settings

   IF N_ELEMENTS(gen_file) EQ 0 THEN gen_file=0
   
   IF gen_file EQ 0 THEN BEGIN

      IF fillist(0) EQ '' THEN BEGIN
         MESSAGE, /INFO, $
          ' No Data selected, BATCH Sequence stopped'
         RETURN
      ENDIF
      
      MESSAGE, /INFO, 'Serendipity batch processing messages'

      FOR I=0, N_ELEMENTS(fillist)-1 DO BEGIN 
         
         filename = STRUPCASE(STRTRIM(fillist(i),2))
         filename = strmid(filename,(STRPOS(filename,'.FITS') - 12), 17)
         MESSAGE, /INFO, 'Reading ERD '+filename
         
         read_erd, filename,hdr,ilunr

; Compute first measurement on the ERD level
         firsterd = ilunr(0)

;
; Compute last measurement to be used
; 
         lasterd = max(ilunr)	    
;
; Compute all measurements to be processed to SMC (FCS to SCP)
;

         meas = indgen(lasterd-firsterd+1) + firsterd

         MESSAGE, /INFO, 'Processing measurements from '+filename	
         PIA_PROC_BATCH_SM_NOX, meas

         MESSAGE, /INFO, 'Now cleaning all buffers '
         erd_init
         srd_init
         scp_init
         smc_init
         cs_init
         iiph_init

         CLOSE, /ALL

      ENDFOR			    
   ENDIF $
    
   ELSE IF gen_file EQ 1 THEN BEGIN
; starting from SRD	

      IF fillist(0) EQ '' THEN BEGIN
         MESSAGE, /INFO, $
          ' No Data selected, BATCH Sequence stopped'
         RETURN
      ENDIF
      
      MESSAGE, /INFO, 'Serendipity batch processing messages'

      FOR I=0, N_ELEMENTS(fillist)-1 DO BEGIN 
         
         filename = STRUPCASE(STRTRIM(fillist(i),2))
         filename = strmid(filename,(STRPOS(filename,'.ISRD') - 12), 17)

         MESSAGE, /INFO, 'Reading ISRD '+filename

         read_isrd, filename, path=!XDRDIR
         srdnam = strmid(filename,0,12)
         ilunr = WHERE(phtsrd.admn EQ srdnam)
         ilunr = ilunr(0)
         isrdnum = strmid(phtsrd(ilunr).admn,4,8)
         icsname = 'CS' + isrdnum + '.ICS'
         read_ics, icsname, path=!XDRDIR

         MESSAGE, /INFO, 'Processing measurements from '+filename
         
         PIA_PROC_BATCH_SM_NOX, ilunr

         MESSAGE, /INFO, 'Now cleaning all buffers '               
         srd_init
         scp_init
         smc_init
         cs_init
         iiph_init

         CLOSE, /ALL

      ENDFOR			    
   ENDIF $
    
   ELSE IF gen_file EQ 2 THEN BEGIN
; starting from SMC	

      IF fillist(0) EQ '' THEN BEGIN
         MESSAGE, /INFO, $
          ' No Data selected, BATCH Sequence stopped'
         RETURN
      ENDIF
      
      MESSAGE, /INFO, 'Serendipity batch processing messages'

      FOR I=0, N_ELEMENTS(fillist)-1 DO BEGIN 

         filename = STRUPCASE(STRTRIM(fillist(i),2))
         filename = strmid(filename,(STRPOS(filename,'.ISMC') - 12), 17)

         MESSAGE, /INFO, 'Reading ISMC '+filename

         read_ismc, filename, path=!XDRDIR
         smcnam = strmid(filename,0,12)
         ilunr = WHERE(phtsmc.admn EQ smcnam)
         ilunr = ilunr(0)

         MESSAGE, /INFO, 'Processing measurements from '+filename
         PIA_PROC_BATCH_SM_NOX, ilunr

         MESSAGE, /INFO, 'Now cleaning all buffers '
         smc_init

         CLOSE, /ALL

      ENDFOR		
      
   ENDIF

   RETURN
END
