;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_PROC_BATCH_SM_NOX
;
; PURPOSE:
;       Menu and procedure within the PIA to get automatically
;       a sequence of steps to be performed for Serendipity.
;       Data reduction from ERD up to SMC level, including point source extr.
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       PIA_PROC_BATCH_SM_NOX
;
; INPUTS:
;	MEAS:	List of measurements in ERD buffer to be processed
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	none
;
;
; COMMON BLOCKS:
;	phtcs:		CS dynamical structure
;	phterd:		ERD dynamical structure
;	phtsrd:		SRD dynamical structure
;	phtscp:		SCP dynamical structure
;	phtsm:		Serendipity dynamical structure
;	phtsmc:		SMC dynamical structure
;	phtresact:	actual responsivities
;	deglitch_param: parameters for deglitching procedure
;	gencho_sm_cmn:	for returning several variables
;	batch_sm_wid:   widget information
;
; PROCEDURES USED:
;	PIA Procedure(s):
;		deglitch_param_init
;		lin_voltages
;		pia_gencho
;		pia_load_ics
;		pia_load_ierd
;		pia_load_iscp
;		pia_load_isrd
;		pia_sm_load_ismc
;		pia_pickfile
;		pia_respons
;		pia_rodisc
;		pia_sigdisc
;		pia_write_ics
;		pia_sm_write_smc
;               pia_sm_psework_init
;               pia_sm_psework_create
;               pia_sm_psework_init
;               pia_sm_psework_to_phtismc
;		pickmeas
;		process_erd
;		process_srd
;		read_erd
;		respons_interpol
;		write_smc
;	Internal:
;
; MODIFICATION HISTORY:
;       Written by: 	Stefan Bogun (MPIA)			July 1996
;	adapted from PIA_PROC_BATCH written by Carlos Gabriel
;	Modified:
;	   WAIT, 10 -> WAIT, 5 (SB)				October 1996
;	   More detailed output in batch window (SB)		October 1996
;	   modified check for saturated pixels (SB)		October 1996
;	   check if enough samples are above slew speed threshold (SB)
;								October 1996
;	   processing window remains the same (SB, November 1996)
;	   two iterations of PSE process (SB)			December 1996
;	   OPENW for log/obj files deleted (SB)			December 1996
;	   PSE processing before saving (SB)			December 1996
;	   new ISMC structure, work structure, point source
;            structure, new processing sequence for ISMC (SB)	December 1996
;          exclude revolutions with zero FCS power from FCS calibration (SB)
;                                                               February 1997
;	   reading internal files from !XDRDIR (rather than from !INTDIR) (SB)
;								February 1997
;	   slight cosmectic changes, point_sources already initialized (SB)
;								February 1997
;          generation of PSE working structure exernal now (SB) February 1997
;          Signal deglitching updated according to PIA V6.2 (SB) April 1997
;          pia_respons_sm call replaced by pia_respons (SB)     April 1997
;          small bug fix (drift recognition failed, SB)         April 1997
;          usage of subdivided ramps possible (SB)              June 1997
;          modified for use with PIA V6.4 (SB)                  July 1997
;          flatfielding introduced (SB)                         July 1997
;          slight corrections due to changed parameters (SB)    December 1997
;          modifications due to SESAME-V4.0 (SB)                December 1997
;
;	   Dark current subtraction now orbital dependent (CG)	April 1998
;	   Also inclusion of Reset Interval Correction (CG)	April 1998
;	   V7.1
;-

PRO PIA_PROC_BATCH_SM_NOX, meas

@phtcs
@phterd
@phtsrd
@phtscp
@phtsm
@phtsmc
@phtresact
@deglitch_param
@gencho_sm
@splitramp_par

; Open Log-File

   OpenW, OutUnit, StrCompress(!PIA_SERDIR+'SESAME.log', /REMO), /GET_LUN , /APPEND

; revolutions with FCS power zero -> to be excluded from FCS calibration
   revol_FCS_excl = ['067','069','071','073','074','076','077','079','080', $
                     '081','082']

   IF !OTFLOW EQ 1 THEN BEGIN
      !OTFLOW=99
      text = [' WARNING - On target flag detected - disabling it now - ',$
      	      ' please take it into account if you are going to ',$
      	      ' reduce non-serendipity data']
      FOR ite=0,N_ELEMENTS(text)-1 DO MESSAGE, /INFO, text(ite)
   ENDIF
     

   IF N_ELEMENTS(gen_file)   EQ 0 THEN gen_file = 0
   IF N_ELEMENTS(gen_erd)    EQ 0 THEN gen_erd = 0
   IF N_ELEMENTS(gen_cal)    EQ 0 THEN gen_cal = 0
   IF N_ELEMENTS(gen_ramp)   EQ 0 THEN gen_ramp = 0
   IF N_ELEMENTS(gen_f_ramp) EQ 0 THEN gen_f_ramp = 0
   IF N_ELEMENTS(gen_lin)    EQ 0 THEN gen_lin = 0
   IF N_ELEMENTS(gen_deg2)   EQ 0 THEN gen_deg2 = 0
   IF N_ELEMENTS(gen_flat)   EQ 0 THEN gen_flat = 0
   IF N_ELEMENTS(gen_dar)    EQ 0 THEN gen_dar = 0
   IF N_ELEMENTS(gen_deg)    EQ 0 THEN gen_deg = 0
   IF N_ELEMENTS(gen_drf)    EQ 0 THEN gen_drf = 0
   IF N_ELEMENTS(gen_f_lin)  EQ 0 THEN gen_f_lin = 0
   IF N_ELEMENTS(gen_f_deg2) EQ 0 THEN gen_f_deg2 = 0
   IF N_ELEMENTS(gen_f_flat) EQ 0 THEN gen_f_flat = 0
   IF N_ELEMENTS(gen_f_dar)  EQ 0 THEN gen_f_dar = 0
   IF N_ELEMENTS(gen_f_deg)  EQ 0 THEN gen_f_deg = 0
   IF N_ELEMENTS(gen_f_drf)  EQ 0 THEN gen_f_drf = 0
   IF N_ELEMENTS(gen_isrd)   EQ 0 THEN gen_isrd = 0
   IF N_ELEMENTS(gen_ismc)   EQ 0 THEN gen_ismc = 0
   IF N_ELEMENTS(gen_smc)    EQ 0 THEN gen_smc = 0
   IF N_ELEMENTS(gen_rep)    EQ 0 THEN gen_rep = 0

   FOR I=0,N_ELEMENTS(meas)-1 DO BEGIN


; Try to catch errors and avoid premature stops during batch processing


      CATCH , ERROR_STATUS

      IF (ERROR_STATUS NE 0)  $
       THEN BEGIN
         IF (gen_file EQ 0)  THEN  SlewName =  phterd(meas(i)).admn
         IF (gen_file EQ 1)  THEN  SlewName =  phtsrd(meas(i)).admn
         IF (gen_file EQ 2)  THEN  SlewName =  phtsmc(meas(i)).admn

         PRINT , ' '
         PRINT , ' ???????????????????????????????????????????????????????? '
         PRINT , ' '
         PRINT , ' ' , !STIME
         PRINT , ' '
         PRINT , ' Problems with ' , SlewName
         PRINT , '   '
         PRINT , ' ERROR Index   = ' , ERROR_STATUS
         PRINT , ' ERROR Message = ' , !ERR_STRING
         PRINT , ' '
         PRINT , ' ... continuing with next measurement '
         PRINT , ' '
         PRINT , ' ???????????????????????????????????????????????????????? '
         PRINT , ' '

         PRINTF , OutUnit , Format='(A)'     , ' '
         PRINTF , OutUnit , Format='(A)'     , ' ???????????????????????????????????????????????????????? '
         PRINTF , OutUnit , Format='(A)'     , '   '
         PRINTF , OutUnit , Format='(A)'     , ' ' + !STIME
         PRINTF , OutUnit , Format='(A)'     , '   '
         PRINTF , OutUnit , Format='(A,A)'   , ' Problems with ' , SlewName
         PRINTF , OutUnit , Format='(A)'     , '   '
         PRINTF , OutUnit , Format='(A,I10)' , ' ERROR Index   = ' , ERROR_STATUS
         PRINTF , OutUnit , Format='(A,A)'   , ' ERROR Message = ' , !ERR_STRING
         PRINTF , OutUnit , Format='(A)'     , ' '
         PRINTF , OutUnit , Format='(A)'     , ' ... continuing with next measurement '
         PRINTF , OutUnit , Format='(A)'     , '   '
         PRINTF , OutUnit , Format='(A)'     , ' ???????????????????????????????????????????????????????? '
         PRINTF , OutUnit , Format='(A)'     , ' '

         GOTO , ENDLOOP

      ENDIF


; starting from ERD
      IF gen_file EQ 0 THEN BEGIN

         decode_admn, phterd(meas(i)).admn, decstr

         MESSAGE, /INFO, 'Starting with '+decstr

; FCS heating power zero?
         revol = strmid(phterd(meas(i)).admn,4,3)
         dummy = where(revol_FCS_excl EQ revol, FCS_excl)

         pia_load_ierd, phterd(meas(i)), tmpdata
         newhdr = tmpdata.hdr
         indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), cscount)
         IF cscount GT 0 THEN $
          PIA_LOAD_ICS, phtcs(indcs(0)), phtics $
         ELSE BEGIN
            MESSAGE, /INFO, 'Compact Status not found in common, can''t ' + $
             'process this measurement'
            GOTO, ENDLOOP
         ENDELSE

; C200_160, C200_180 or C200_200 ?
         subsystem = conv_subs(phtics.handle.subs(i))
         wheel1 = phtics.handle.c1ps(i)
         wheel2 = phtics.handle.c2ps(i)
         IF subsystem NE 'C2' OR wheel1 NE 14 OR wheel2 LT 9 OR wheel2 GT 11 $
          THEN BEGIN
            MESSAGE, /INFO, $
             'Wrong filter or detector selected, ignoring it'
            GOTO, ENDLOOP
         ENDIF

; Is it an FCS measurement ?
         submode = (phtics.handle.CMod(i) AND 2) / 2
         IF (phtics.dims EQ 1 AND submode EQ 1) OR $
          (i EQ 0 AND phtics.dims EQ 2 AND submode EQ 0) OR $
          (i EQ 1 AND phtics.dims EQ 2 AND submode EQ 1) THEN BEGIN
            MESSAGE, /INFO, $
             'Something''s wrong with this measurement, ignoring it'
            GOTO, ENDLOOP
         ENDIF

         IF submode EQ 1 AND gen_cal GE 1 THEN BEGIN
            IF FCS_excl EQ 0 THEN BEGIN
              MESSAGE, /INFO, $
               'FCS Measurement, processing it to SCP'
               genramp = gen_f_ramp
               genlin = gen_f_lin
               gendeg2 = gen_f_deg2
               genflat = gen_f_flat
               gendar = gen_f_dar
               gendeg = gen_f_deg
               gendrf = gen_f_drf
            ENDIF $
            ELSE BEGIN
               MESSAGE, /INFO, $
                '*** Not processing this FCS Measurement' + $
                '    because heating power 0mW           ***'
               GOTO, ENDLOOP
            ENDELSE
         ENDIF $
         ELSE IF submode EQ 1 AND gen_cal EQ 0 THEN BEGIN
            MESSAGE, /INFO, 'FCS Measurement, ignoring it'
            GOTO, ENDLOOP
         ENDIF $
         ELSE BEGIN
            MESSAGE, /INFO, 'Sky Measurement, processing it to SMC'
            genramp = gen_ramp
            genlin = gen_lin
            gendeg2 = gen_deg2
            genflat = gen_flat
            gendar = gen_dar
            gendeg = gen_deg
            gendrf = gen_drf
         ENDELSE

;
; Linearization
;
         IF (genlin NE 0 AND gen_ramp EQ 0) THEN BEGIN

            MESSAGE, /INFO, 'Linearizing ramps ...  '
            lin_voltages, tmpdata, newhdr, /silent
            handle_value, phterd(meas(i)).conp, tmpdata.conp,/set
            handle_value, phterd(meas(i)).hdr, newhdr,/set
         ENDIF

;
; ERD-deglitching
;
         IF (gendeg2 NE 0 AND gen_ramp EQ 0) THEN BEGIN

            MESSAGE, /INFO, 'Deglitching ramps ...  '
            pia_load_ierd, phterd(meas(i)), tmpdata
            ramp_deglitch, tmpdata, newhdr, /silent
            handle_value, phterd(meas(i)).conp, tmpdata.conp,/set
            handle_value, phterd(meas(i)).hdr, newhdr,/set
         ENDIF
;
; Process all data to srd
;

         IF (genramp EQ 0) THEN BEGIN
            MESSAGE, /INFO, $
             'Processing the measurement  ERD->SRD'+ $
             ' using whole ramps'+ $
             ' (This will take some time, take it easy) '

	    pia_load_ierd, phterd(meas(i)), tmp
	    process_tmperd, tmp, 1, 0, srdps
            
            pia_load_isrd, phtsrd(srdps(0)), tmpdata
            newhdr = tmpdata.hdr
         ENDIF $
         ELSE IF (genramp NE 0 AND genlin EQ 0) THEN BEGIN
            MESSAGE, /INFO, $
             'Processing the measurement  ERD->SRD'+ $
             ' splitting each ramp into pairs of readouts'+ $
             ' (This will take some time, take it easy) '

	    IF N_ELEMENTS(splitramp_par) EQ 0 THEN splitramp_par_init
	    splitramp_par.model = 1
	    splitramp_par.value = 3

	    pia_load_ierd, phterd(meas(i)), tmp
	    process_tmperd, tmp, 1, 0, srdps, /div_ramp

            pia_load_isrd, phtsrd(srdps(0)), tmpdata
            newhdr = tmpdata.hdr
         ENDIF $
         ELSE BEGIN
            MESSAGE, /INFO, $
             'Processing the measurement  ERD->SRD'+ $
             ' splitting each ramp into pairs of readouts'+ $
             ' (This will take some time, take it easy) '

	    IF N_ELEMENTS(splitramp_par) EQ 0 THEN splitramp_par_init
	    splitramp_par.model = 1
	    splitramp_par.value = 3
            process_erd_sm, meas(i), 1, hdr, srdps, /div_ramp

            pia_load_isrd, phtsrd(srdps(0)), tmpdata
            newhdr = tmpdata.hdr
         ENDELSE

; Write SRD measurements into PIA internal files

         IF (gen_isrd) THEN BEGIN
            filename = tmpdata.admn+'.ISRD'
            phtisrd = tmpdata
            save, phtisrd, FILE=!INTDIR+filename, /XDR
            MESSAGE, /INFO, $
             'Saving SRD measurement into '+!INTDIR+filename
            IF cscount GT 0 THEN BEGIN
               FILE='CS'+phtics.UNIT+'.ICS'
               junk = FINDFILE(!intdir+FILE, COUNT=count)
               IF count GT 0 THEN $
                MESSAGE, /INFO, $
                'Compact Status file not written, already present' $
               ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
            ENDIF ELSE MESSAGE, /INFO, $
             'Compact Status not found in common, not saved'

         ENDIF

      ENDIF

; starting from SRD

      IF gen_file EQ 1 THEN BEGIN

         decode_admn, phtsrd(meas(i)).admn, decstr

         MESSAGE, /INFO, 'Starting with '+decstr

; FCS heating power zero?
         revol = strmid(phtsrd(meas(i)).admn,4,3)
         dummy = where(revol_FCS_excl EQ revol, FCS_excl)

         pia_load_isrd, phtsrd(meas(i)), tmpdata
         newhdr = tmpdata.hdr
         indcs = WHERE(phtcs.unit EQ STRMID(tmpdata.admn,4,8), cscount)
         IF cscount GT 0 THEN $
          PIA_LOAD_ICS, phtcs(indcs(0)), phtics $
         ELSE BEGIN
            MESSAGE, /INFO, 'Compact Status not found in common, can''t ' + $
             'process this measurement'
            GOTO, ENDLOOP
         ENDELSE

; C200_160, C200_180 or C200_200 ?
         ordr = tmpdata.ordr
         subsystem = conv_subs(phtics.handle.subs(ordr))
         wheel1 = phtics.handle.c1ps(ordr)
         wheel2 = phtics.handle.c2ps(ordr)
         IF subsystem NE 'C2' OR wheel1 NE 14 OR wheel2 LT 9 OR wheel2 GT 11 $
          THEN BEGIN
            MESSAGE, /INFO, $
             'Wrong filter or detector selected, ignoring it'
            GOTO, ENDLOOP
         ENDIF

; Is it an FCS measurement ?
         submode = (phtics.handle.CMod(ordr) AND 2) / 2

         IF (phtics.dims EQ 1 AND submode EQ 1) OR $
          (ordr EQ 0 AND phtics.dims EQ 2 AND submode EQ 0) OR $
          (ordr EQ 1 AND phtics.dims EQ 2 AND submode EQ 1) THEN BEGIN
            MESSAGE, /INFO, $
             'Something''s wrong with this measurement, ignoring it'
            GOTO, ENDLOOP
         ENDIF

         IF submode EQ 1 AND gen_cal EQ 1 THEN BEGIN
            IF FCS_excl EQ 0 THEN BEGIN
               MESSAGE, /INFO, $
                'FCS Measurement, processing it to SCP'
               genflat = gen_f_flat
               gendar = gen_f_dar
               gendeg = gen_f_deg
               gendrf = gen_f_drf
            ENDIF $
            ELSE BEGIN
               MESSAGE, /INFO, $
                 '*** Not processing this FCS Measurement' + $
                 '    because heating power 0mW           ***'
               GOTO, ENDLOOP
            ENDELSE
         ENDIF $
         ELSE IF submode EQ 1 AND gen_cal EQ 0 THEN BEGIN
            MESSAGE, /INFO, $
             'FCS Measurement, ignoring it'
            GOTO, ENDLOOP
         ENDIF $
         ELSE BEGIN
            MESSAGE, /INFO, $
             'Sky Measurement, processing it to SMC'
            genflat = gen_flat
            gendar = gen_dar
            gendeg = gen_deg
            gendrf = gen_drf
         ENDELSE

      ENDIF

      IF gen_file EQ 0 OR gen_file EQ 1 THEN BEGIN
;
; Statistical flatfielding
;
         IF (genflat) THEN BEGIN
            detpix = 4
            MESSAGE, /INFO, 'Applying flatfield correction'

            pia_sm_flatfield, tmpdata.slop(0:detpix-1,*), outslop


; Replace slop contents, log it into the newhdr

            IF (N_ELEMENTS(tmpdata.slop(0:detpix-1,*)) EQ    $
                N_ELEMENTS(outslop)) THEN BEGIN
               tmpdata.slop(0:detpix-1,*) = outslop
               hdrlen = N_ELEMENTS(newhdr)
               hdrend = newhdr(hdrlen-1)
               JUNK = 'PRS_FLAT=          '+$
                '          1 / Flatfield correction     '
               newhdr = [newhdr(0:hdrlen-2),JUNK]
               newhdr = [newhdr,hdrend]
            ENDIF
         ENDIF

; Reset Interval Correction
         IF gen_resint THEN BEGIN
            do_corr_resetint, tmpdata, newhdr, INFO = info
            MESSAGE, /INFO, info
         ENDIF

;
; Subtract dark current
;
         IF (gendar) THEN BEGIN
            detpix = 4
            MESSAGE, /INFO, 'Subtracting Dark Current'

	    dark_done = fxpar(newhdr,'PRS_DARK',count=count)
	    CASE count OF 

		0: darkcur_orb, tmpdata, newheader=Newhdr

		ELSE: MESSAGE, /INFO, $
		 ' Dark Current subtraction already performed, skipped'

	    ENDCASE
         ENDIF
;
;  Deglitching
;

         IF (gendeg) THEN BEGIN

            MESSAGE, /INFO, 'Signals Deglitching ...'

            get_sigdisc, tmpdata, sig_disc

            degl_acc = tmpdata.slop * 0.
            deglitch_param_init
            sizdim1 = n_elements(degl_acc(*,0))
            FOR II = 0,sizdim1 - 1 DO BEGIN
               deglitch_all, tmpdata.slop(II,*), $
                 tmpdata.step, tmpdata.rpid, tmpdata.slun(II,*), $
                 sig_disc(II,*), degacc, slice_chopp = PER_CHOP
               degl_acc(II,*) = degacc
               indnacc = WHERE(degacc NE 0, count)
               IF count GT 0 THEN tmpdata.flag(II,indnacc) = 2
            ENDFOR

; include information in header
            hdrlen = N_ELEMENTS(newhdr)
            hdrend = newhdr(hdrlen-1)
            JUNK = 'PRS_DEGL=          '+$
             '          1 / Deglitching corr. applied'
            newhdr = [newhdr(0:hdrlen-2),JUNK]
            newhdr = [newhdr,hdrend]
         ENDIF

         IF submode EQ 1 THEN BEGIN
            MESSAGE, /INFO, $
             'Processing measurement  SRD->SCP'
            process_tmpsrd, tmpdata, newhdr, scppos, driftcorr=gendrf

; Write SCP data into PIA internal files

            IF (gen_isrd) THEN BEGIN
               pia_load_iscp, phtscp(scppos(0)), tmpscp
               filename = tmpscp.admn+'.ISCP'
               phtiscp = tmpscp
               save, phtiscp, FILE=!INTDIR+filename, /XDR
               MESSAGE, /INFO, $
                'Saving SCP measurement into '+!INTDIR+filename
               IF cscount GT 0 THEN BEGIN
                  FILE='CS'+phtics.UNIT+'.ICS'
                  junk = FINDFILE(!intdir+FILE, COUNT=count)
                  IF count GT 0 THEN $
                   MESSAGE, /INFO, $
                   'Compact Status file not written, already present' $
                  ELSE SAVE, phtics, FILE=!INTDIR+file, /XDR
               ENDIF ELSE MESSAGE, /INFO, $
                'Compact Status not found in common, not saved'
            ENDIF
            GOTO, ENDLOOP
         ENDIF $
         ELSE BEGIN
            IF gen_cal EQ 0 THEN BEGIN
               MESSAGE, /INFO, $
                'Processing measurement  SRD->SMC, ' + $
                'using default responsivities'

               pia_sm, tmpdata, newhdr, 0, smcpos
            ENDIF $
            ELSE IF gen_cal EQ 1 AND phtics.dims EQ 1 THEN BEGIN
               MESSAGE, /INFO, $
                'Processing measurement SRD->SMC, ' + $
                'using default responsivities'
               MESSAGE, /INFO, $
                ' (no FCS measurement available) '

               pia_sm, tmpdata, newhdr, 0, smcpos
            ENDIF $
            ELSE IF gen_cal EQ 1 AND phtics.dims EQ 2 THEN BEGIN
               IF FCS_excl EQ 0 THEN BEGIN
                  MESSAGE, /INFO, $
                   'Processing measurement SRD->SMC, ' + $
                   'using FCS measurement'

; find corresponding SCP measurement
                  scpdat = 'C200' + strmid(tmpdata.admn,4,8)
                  scppos = where(phtscp.admn EQ scpdat, nscp)
                  IF nscp EQ 0 THEN BEGIN
                     scpname = scpdat + '.ISCP'
                     read_iscp, scpname, path=!XDRDIR
                  ENDIF
                  scppos = where(phtscp.admn EQ scpdat, nscp)
                  pia_load_iscp,phtscp(scppos(0)),tmpscp
                  scphdr = tmpscp.hdr
                  PIA_RESPONS, tmpscp, scphdr, /NO_WIDGET

                  pia_sm, tmpdata, newhdr, 1, smcpos
               ENDIF $
               ELSE BEGIN
                  MESSAGE, /INFO, $
                   'Processing measurement SRD->SMC, ' + $
                   'using default responsivities'
                  MESSAGE, /INFO, $
                   ' (FCS heating power 0mW) '

                  pia_sm, tmpdata, newhdr, 0, smcpos
               ENDELSE
            ENDIF $
            ELSE IF gen_cal EQ 2 THEN BEGIN
               MESSAGE, /INFO, 'Responsivities from orbit not yet ' + $
                'implemented. Returning.'
               GOTO, ENDLOOP
            ENDIF $
            ELSE BEGIN
               MESSAGE, /INFO, $
                'Something''s wrong with this measurement, ignoring it'
               GOTO, ENDLOOP
            ENDELSE

            pia_sm_load_ismc, phtsmc(smcpos(0)), tmpdata
            newhdr = tmpdata.hdr

         ENDELSE
      ENDIF

; starting from SMC

      IF gen_file EQ 2 THEN BEGIN

         decode_admn, phtsmc(meas(i)).admn, decstr

         MESSAGE, /INFO, 'Starting with '+decstr

         pia_sm_load_ismc, phtsmc(meas(i)), tmpdata
         newhdr = tmpdata.hdr
      ENDIF

           
      MESSAGE, /INFO, $
       'Starting the slew procesing ...'

      PIA_SM_PSEWORK_CREATE, tmpdata, newhdr, psework

;initialize PSEWORK only if wanted
      IF gen_file LT 2  OR  gen_rep EQ 1 THEN $
         PIA_SM_PSEWORK_INIT, psework, newhdr

      idisc = WHERE(tmpdata.flag(0,*) EQ 2 OR tmpdata.flag(1,*) EQ 2 OR $
                    tmpdata.flag(2,*) EQ 2 OR tmpdata.flag(3,*) EQ 2, ndisc)
      IF ndisc GE tmpdata.dims - 5 THEN BEGIN
         MESSAGE, /INFO, $
          'Complete measurement is saturated.'
         MESSAGE, /INFO, $
          '  Not processing this slew.'
         GOTO, ENDPSE
      ENDIF

      MESSAGE, /INFO, $
       'Discarding low slew speed parts'
      pia_sm_wnd, psework

      FXADDPAR, newhdr, 'PRSM_WND', 1
      FXADDPAR, newhdr, 'PRSM_VEL', smp_vel

      idisc = WHERE( (psework.smfl(0,*) OR psework.smfl(1,*) OR $
                      psework.smfl(2,*) OR psework.smfl(3,*)) AND $
                     NOT flg_pia_1, ndisc)

      IF ndisc GE psework.dims - 20 THEN BEGIN
         MESSAGE, /INFO, $
          'Nearly no data points remaining.'
         MESSAGE, /INFO, $
          '  Terminating slew processing.'
         GOTO, ENDPSE
      ENDIF




; background, sensitivity, noise processing only if needed

      IF gen_file LT 2  OR  gen_rep EQ 1 THEN BEGIN

         MESSAGE, /INFO, $
          ' Background fitting and such ........ '

         pia_sm_SEPREP, psework
; *** added (SB, 08/07/97) ***
         FXADDPAR, newhdr, 'PRSM_BCK', 1
         FXADDPAR, newhdr, 'PRSM_STY', 2
         FXADDPAR, newhdr, 'PRSM_DEG', 1

; update SMC structure
         FXADDPAR, newhdr, 'PRSM_SY1', psework.sens(0)
         FXADDPAR, newhdr, 'PRSM_SY2', psework.sens(1)
         FXADDPAR, newhdr, 'PRSM_SY3', psework.sens(2)
         FXADDPAR, newhdr, 'PRSM_SY4', psework.sens(3)

         FXADDPAR, newhdr, 'PRSM_GN1', psework.gnoi(0)
         FXADDPAR, newhdr, 'PRSM_GN2', psework.gnoi(1)
         FXADDPAR, newhdr, 'PRSM_GN3', psework.gnoi(2)
         FXADDPAR, newhdr, 'PRSM_GN4', psework.gnoi(3)

         FXADDPAR, newhdr, 'PRSM_PRC', 2

      ENDIF

; always search and fit sources

      MESSAGE, /INFO, $
       ' Searching for sources ......... '

      pia_sm_SESAME, psework

ENDPSE:
      pia_sm_psework_to_phtismc, psework, newhdr, tmpdata

; Write SMC data into PIA internal files
      IF (gen_ismc) THEN BEGIN
         filename = tmpdata.admn+'.ISMC'
         phtismc = tmpdata
         save, phtismc, FILE=!INTDIR+filename, /XDR
         MESSAGE, /INFO, $
          'Saving SMC measurement into '+!INTDIR+filename
      ENDIF


; Write SMC data into FITS files
      IF (gen_smc) THEN BEGIN
         WRITE_SMC, tmpdata, newhdr
         filename = 'P2SM' + strmid(tmpdata.admn,4,8) + $
          '.FITS'
         MESSAGE, /INFO, $
          'Saving SMC measurement into '+!INTDIR+filename
      ENDIF

; end of all loops

ENDLOOP:
   ENDFOR

   MESSAGE, /INFO, 'Ready READY Ready READY Ready'

   WAIT, 3

; Close Log-File

   Close, OutUnit
   Free_Lun, OutUnit

   RETURN
END


