;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
PRO PIA_SM, TMPDATA, SRDHDR, RESP, SMCPOS
;+
; NAME:
;	PIA_SM
; PURPOSE:
;       This is the first procedure for the analysis of the Serendipity Mode
;	(SeM) observations. Here the SeM standard structure for is created: 
;	PHTSMC. It is the root procedure that opens the SM Data Display for 
;	the data processing. It allows to display the surface brightness versus
;	time and sky position.
;                
; CALLING SEQUENCE:
;	PIA_SM, TMPDATA, SRDHDR, RESP, SMCPOS
;
; INPUT PARAMETERS:
;	tmpdata	: structure to be use as input for the SM data analysis
;	SRDHDR	: a header to be taken instead of tmpdata.hdr
;	RESP	: Responsivity choice flag - 0=default, 1=from FCS, 
;		  (2=responsivity history over orbit, not available so far).
;	
; OPTIONAL INPUT:
;
; OPTIONAL OUTPUT:
;	SMCPOS	: Array containing the positions of the produced SMC
;		  data within the PHTSMC structure 
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by P. Meroluzzi                      	May 1995
;       Add IIPH reading procedure                      June 1995
;       Add the time conversions and interpolations     July 1995
;       in order to produce the structure SMDPS with
;       sky coordinates and signals
;       Sky coordinates and roll angle                  August 1995 
;       uncertainties added
;       Correction for the file name IIPH               September 1995
;       (FITDIR added)
;	Correction of the flag: the discarded data	January 1996
;	are registered, but not the deglitched ones
;	Correction of the selection criteria (PM)        
;       Correction of the CS selection (PM)
;
;       RoBaS
;       18JUN1996 correct CS file PSTI/PSTA
;
;	general review, many things modified, loops replaced by WHERE syntax,
;	conversion to surface brightness included, SMD structure removed,
;						       (SB, 18/07/96)
;	RA and ROLL flip problem fixed		       (SB, 17/09/96)
;	PHTSMC.FLAG = PHTSRD.FLAG, sig_disc used       (SB, 29/10/96)
;	correction for UTC time offset		       (SB, 06/11/96)
;	new PHTIIPH usage; pointing shift correction   (SB, 14/11/96)
;	little bug in pointing shift correction corr.  (SB, 19/11/96)
;	new PHTSMC structure			       (SB, 28/01/97)
;	responsivity uncertainty not used	       (SB, 30/01/97)
;       initialize SMFL flag here                      (SB, 03/04/97)
;       Previously flatfielded data can be processed   (SB, 01/08/97)
;       Gyro drift correction only for OLP < 6.1       (SB, 12/09/97)
;
;	Default responsivity orbital dependent (CG)	April 1998
;	V7.1
;-

@fxbintable
@adm
@phtcs
@phtsrd
@phtiiph
@phtacc
@chop_info
@phtcap
@phtsmc
@phtresdef
@phtresact
@phtsm

   ON_ERROR, 0                                          
;
; Check the number of parameters
;
   IF N_PARAMS() LT 3 THEN MESSAGE, $
    'Syntax: PIA_SM, TMPDATA, SRDHDR, RESP, [SMCPOS]'

; quit if actual responses want to be used and are not available (SB, 19/07/96)
   IF resp EQ 1 AND N_ELEMENTS(c2respact) EQ 0 THEN GOTO, NORESPACT

; give the values to the variables

   stim = tmpdata.stim
   slop = tmpdata.slop
   slun = tmpdata.slun
   flag = fix(tmpdata.flag)
   chop = tmpdata.chop
   rchp = tmpdata.rchp
   step = tmpdata.step
   fcsn = tmpdata.fcsn
   temp = tmpdata.temp
   fcs1 = tmpdata.fcs1
   fcs2 = tmpdata.fcs2
   tkey = tmpdata.tkey
   rpid = tmpdata.rpid
   sldi = tmpdata.sldi

; Only the C200 observations can be processed in the Serendipity Mode:
; this is checked here

   CASE tmpdata.det OF

; C200
      'C2': BEGIN
         DET = 'C2'
         NPIX = 6
      END
      ELSE: BEGIN
         MESSAGE, /INFO, 'This is not a C200 observation!'
         RETURN
      END

   ENDCASE

   ntot = tmpdata.sldi
   get_sigdisc, tmpdata, sig_disc

; store the values of the variables 
   tflag = flag
   tstim = stim
   tslop = slop
   tslun = slun
   tchop = chop
   tstep = step
   ttemp = temp
   tfcsn = fcsn
   tfcs1 = fcs1
   tfcs2 = fcs2
   ttkey = tkey
   trpid = rpid     

   ii = where(sig_disc NE 0 AND flag NE 3, ni)
   IF ni GT 0 THEN tflag(ii) = 2

;*********************************************************************
;  Collect the information for time conversions                      *
;*********************************************************************

; looking for the IIPH file
   
   IIPHnum = STRMID(tmpdata.admn, 4, 8)
   IIPHname = 'IIPH        .FITS'
   STRPUT, IIPHname, IIPHnum, 4
   
   IF IIPHname EQ " " THEN BEGIN
      MESSAGE, /INFO, "IIPH file does not exist!"
      RETURN
   ENDIF

; reading the IIPH file           
   
   pia_sm_read_iiph, IIPHname, IIPHunit 

; open the structure PHTIIPH and read RA, DEC and roll angle

   IDname = tmpdata.admn
;   Admind = WHERE(adm.name EQ IDname)
;   Admind = Admind(0)
;   adm(admind).phtiiph = iiphunit
   
   IF iiphunit LE 0 THEN BEGIN
      MESSAGE, /INFO, 'IIPH file not found'
   ENDIF

   HANDLE_VALUE, phtiiph(iiphunit).hdr, hdr                   
   HANDLE_VALUE, phtiiph(iiphunit).utk, utk  
   HANDLE_VALUE, phtiiph(iiphunit).utc, utc  
   HANDLE_VALUE, phtiiph(iiphunit).flt, filter  
   HANDLE_VALUE, phtiiph(iiphunit).ra, ra  
   HANDLE_VALUE, phtiiph(iiphunit).dec, dec  
   HANDLE_VALUE, phtiiph(iiphunit).rll, roll 

; modify RA and ROLL (avoid 360 degree-jumps) (SB, 14/11/96)
   nra = phtiiph(iiphunit).dims
   ra1 = [ra(0),ra]
   ra2 = [ra,ra(nra-1)]
   iflip = WHERE(ABS(ra1-ra2) GT 180., nflip)
   IF nflip GT 0 THEN $
    FOR i = 0, nflip-1 DO BEGIN 
      il = iflip(i)-1
      ir = iflip(i)
      IF ra(ir) LT ra(il) THEN ra(ir:*) = ra(ir:*) + 360. $
      ELSE ra(ir:*) = ra(ir:*) - 360.
   ENDFOR

   nroll = nra
   roll1 = [roll(0),roll]
   roll2 = [roll,roll(nroll-1)]
   iflip = WHERE(ABS(roll1-roll2) GT 180., nflip)
   IF nflip GT 0 THEN $
    FOR i = 0, nflip-1 DO BEGIN 
      il = iflip(i)-1
      ir = iflip(i)
      IF roll(ir) LT roll(il) THEN roll(ir:*) = roll(ir:*) + 360. $
      ELSE roll(ir:*) = roll(ir:*) - 360.
   ENDFOR

;  Extract OLP Version 
   OLPVERS  = FXPAR(hdr,'OLPVERS')
   IF STRMID(OLPVERS,0,4) EQ 'OLP_' THEN BEGIN
      OLP_VERSION = STRCOMPRESS( STRMID(OLPVERS,4,4), /REMOVE_ALL) + '0'
      OLPVA = FLOAT( STRMID(OLP_VERSION,0,1) )
      OLPVB = FLOAT( '0.' + STRMID(OLP_VERSION,1,4) )
      OLP_VERSION = OLPVA + OLPVB
   ENDIF $
   ELSE $
    OLP_VERSION = 12.0

; correct coordinates according to first startracker value (SB, 14/11/96)
   IF olp_version LT 6.099 THEN BEGIN

      print, 'PIA_SM: Apply gyro drift correction'
      ifilt = WHERE(filter EQ 0, nfilt)
      IF nfilt GT 0 AND nfilt LT nra THEN BEGIN
         ilast = ifilt(nfilt-1)+1
         utk_t = utk(ilast-1:ilast)
         ra_t = ra(ilast-1:ilast)
         dec_t = dec(ilast-1:ilast)
         roll_t = roll(ilast-1:ilast)

; extrapolate coordinates
         m_t = (ra_t(1)-ra_t(0)) / (utk_t(1)-utk_t(0))
         b_t = ra_t(0) - m_t*utk_t(0)
         ra_c = m_t*utk(ilast+1) + b_t
         
         m_t = (dec_t(1)-dec_t(0)) / (utk_t(1)-utk_t(0))
         b_t = dec_t(0) - m_t*utk_t(0)
         dec_c = m_t*utk(ilast+1) + b_t
         
         m_t = (roll_t(1)-roll_t(0)) / (utk_t(1)-utk_t(0))
         b_t = roll_t(0) - m_t*utk_t(0)
         roll_c = m_t*utk(ilast+1) + b_t

         dist_c = ra*0.d0
         FOR i=1,nra-1 DO dist_c(i)=dist_c(i-1)+ $
          sphdist(ra(i-1),dec(i-1),ra(i),dec(i))
         corr = (dist_c - dist_c(0)) / (dist_c(ilast+1) - dist_c(0))
         corr(ilast+1:*) = 0.d0
         
         ra = ra + corr*(ra(ilast+1)-ra_c)
         dec = dec + corr*(dec(ilast+1)-dec_c)
         roll = roll + corr*(roll(ilast+1)-roll_c)
      ENDIF
   ENDIF

; uncertainties are 0 so far
   rau = ra * 0.D0
   decu = dec * 0.D0
   rolu = roll * 0.D0

; read the CS file and get the information for the time convertion
;
; - change -> shape4
;;;*** modified (SB 17/07/96)
   IF fxpar(tmpdata.hdr, 'EOHAPLID') EQ 'PHTSEREN' THEN $
    CSName = 'PSTI'+IIPHNUM+'.FITS' $
   ELSE $
    CSName = 'PSTA'+IIPHNUM+'.FITS'
   read_cs, CSName, cshdr, CSUnit
;   Admind = WHERE(adm.name eq idname)
;   Admind = Admind(0)
;   adm(admind).phtcs = CSUnit
   IF CSUnit le 0 THEN BEGIN
      MESSAGE, /INFO, 'CS file not found'
      RETURN
   ENDIF
   
; Access the structure pointed to from the unit
   HANDLE_VALUE, phtcs(CSUnit).handle, cs

; Find the CS Index of the required measurement by comparing start times
   CSIndex = WHERE(cs.ikst eq ttkey(0))

; Convert CSIndex from a one dimensional array to a scalar.
   CSIndex = CSIndex(0)

; Check there is agreement between the measurement start time and
; the first readout of a measurement. Change by CG in chop_info_init.pro and here 
; used
   IF (CSIndex eq -1) THEN BEGIN
;print, 'Unable to find required measurement in CS.'
;print, 'Will try nevertheless to cope with it - cross fingers'
      CSsubt = ABS(cs.ikst - ttkey(0))
      cssubt_min = min(cssubt)
      CSIndex = WHERE(cssubt eq min(cssubt))
      IF cssubt(CSindex(0)) GT 2000 THEN BEGIN
         MESSAGE, 'It fails, differences are too large, returning'
         RETURN
      ENDIF
   ENDIF

; At this point, the required variables in the compact status common block
; can be accessed with: CS.xxxx(CSIndex)
   
; Get the start and end times
   tikst = CS.ikst(CSIndex)     
   tiken = CS.iken(CSIndex) 
   utst1 = CS.utst(0,CSIndex)     
   utst2 = CS.utst(1,CSIndex)
   uten1 = CS.uten(0,CSIndex) 
   uten2 = CS.uten(1,CSIndex)
   
;read time conversion factor
   trefitku=fxpar(tmpdata.hdr,'trefitku')

;for test purposes while header not right use the common value
   IF trefitku EQ 0 THEN trefitku = 0.00006103515630D0
   trefitku = DOUBLE(trefitku)    

;*********************************************************************
;  Time conversions                                                  *
;*********************************************************************

; convert the variables into seconds
   ikst = DOUBLE(tikst*trefitku)
   iken = DOUBLE(tiken*trefitku)
   utst = DOUBLE(utst2*1D-07+utst1)
   uten = DOUBLE(uten2*1D-07+uten1)

; adding an offset to utc start and end times
   PRINT, 'Adding 0.75s to the UTC start and end times.'
   utst = utst + 0.75d0
   uten = uten + 0.75d0

   utc_s = DOUBLE(TRANSPOSE(utc(1,*))*1.D-07+TRANSPOSE(utc(0,*))) - utst
   itk_s = ttkey * trefitku - ikst

; interpolate coordinates
   ra_s = INTERPOL(ra, utc_s, itk_s)
   rau_s = INTERPOL(rau, utc_s, itk_s)
   dec_s = INTERPOL(dec, utc_s, itk_s)
   decu_s = INTERPOL(decu, utc_s, itk_s)
   roll_s = INTERPOL(roll, utc_s, itk_s)
   rolu_s = INTERPOL(rolu, utc_s, itk_s)

; shift back to 0-360 degree interval
   ra_s = (ra_s + 1440.d0) MOD 360.D0
   roll_s = (roll_s + 1440.d0) MOD 360.D0

; Conversion to surface brightness (SB, 19/07/96)

;  read default or actual responsivities if necessary -> orbital dep.response
;   IF resp EQ 0 THEN BEGIN
;      IF N_ELEMENTS(SLRESP) EQ 0 THEN read_resp 
;   ENDIF 
   IF resp EQ 0 THEN BEGIN
    def_resp = default_response(tmpdata)
    c2resp = def_resp(*,*,0)
    c2resu = def_resp(*,*,1)
   ENDIF

;  Read Capacities and/or FCS powers if necessary
   IF N_ELEMENTS(slcap) EQ 0 THEN phtcap_init

   wheels = [CS.c1ps(CSIndex),CS.c2ps(CSIndex),CS.c3ps(CSIndex)]
   meast = 2 ^ CS.meat(CSIndex)
   dr = 2 ^ CS.ndr(CSIndex)
   intt = 2. ^ (7 - CS.intt(CSIndex))

   CASE resp OF
      0: BEGIN
         respon = c2resp
         IF N_ELEMENTS(c2resu) EQ 0 THEN $
          respun = 0.3 * respon $
         ELSE $
          respun = c2resu
         chpl = 0  
      END
      1: BEGIN
         respon = c2respact
         respun = c2respunc
         chpl = 0  
      END
      ELSE: BEGIN
         MESSAGE, /INFO, 'Not allowed input parameter'
         RETURN			
      END	
   ENDCASE

;  Check if flatfield has been applied to SRD data
   flat = fxpar(srdhdr, 'PRS_FLAT')
   IF flat EQ 1 THEN respon(*) = respon(1) ; use pixel 1 only

   capac = c2cap
   pixnr = 4
   filt = wheels(1)
   aper = wheels(2)
   polz = wheels(0)

   inbslop = slop(0:pixnr-1,*) * capac
   back = dblarr(pixnr,ntot)
   nois = dblarr(pixnr,ntot)
   tflag = tflag(0:pixnr-1,*)
   smfl = bytarr(pixnr,ntot)

; initialize Serendipity flag
   ifl = WHERE(tflag EQ 1, nfl)
   IF nfl NE 0 THEN smfl(ifl) = flg_pia_1 ; discarded by PIA
   ifl = WHERE(tflag EQ 2, nfl)
   IF nfl NE 0 THEN smfl(ifl) = flg_pia_2 ; discarded by PIA

;  Set all discarded points to 1.0 (to avoid divisions by 0) 
   indnacc = WHERE(tflag EQ 2, n1)
   IF n1 GT 0 THEN inbslop(indnacc) = 1.0

   mnpw = inbslop * 0.
   mnpu = mnpw

; conversion to in-band power
   FOR I = 0, pixnr-1 DO BEGIN
      mnpw(i,*) = inbslop(i,*) / respon(i,chpl)
      mnpu(i,*) = mnpw(i,*) * slun(i,*) * capac / inbslop(i,*)
   ENDFOR

; Avoid division by 0.
;	mnpw = (mnpw > 1.e-20)
   
; conversion to surface brightness
   flux, mnpw, tmpdata.det, filt, aper, 1, flde
   flde = flde * 1.e-06
   fldu = flde * mnpu / mnpw

   IF n1 GT 0. THEN BEGIN
      flde(indnacc) = 0.
      fldu(indnacc) = 0.
   ENDIF

;  Accomodate data into the smc structure and the smc handles after
;  creating the temporary ones                  

   PHTSMCT = {PHTSMC_TYPE}
   PHTSMCT.HDR = HANDLE_CREATE()
   PHTSMCT.TKEY = HANDLE_CREATE()
   PHTSMCT.TIME = HANDLE_CREATE()
   PHTSMCT.FLX = HANDLE_CREATE()
   PHTSMCT.FLXU = HANDLE_CREATE()
   PHTSMCT.BACK = HANDLE_CREATE()
   PHTSMCT.NOIS = HANDLE_CREATE()
   PHTSMCT.FLAG = HANDLE_CREATE()
   PHTSMCT.SMFL = HANDLE_CREATE()
   PHTSMCT.RA = HANDLE_CREATE() 
   PHTSMCT.RAU = HANDLE_CREATE()
   PHTSMCT.DEC = HANDLE_CREATE()
   PHTSMCT.DECU = HANDLE_CREATE() 
   PHTSMCT.ROLL = HANDLE_CREATE()
   PHTSMCT.ROLU = HANDLE_CREATE()
   
   PHTSMCT.UNIT = TMPDATA.UNIT
   PHTSMCT.DET  = DET
   PHTSMCT.ADMN = TMPDATA.ADMN
   PHTSMCT.ORDR = TMPDATA.ORDR
   PHTSMCT.DIMS = N_ELEMENTS(FLDE(0,*))

   HANDLE_VALUE, PHTSMCT.TKEY, TTKEY, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.TIME, TSTIM, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.FLX, FLDE, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.FLXU, FLDU, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.BACK, BACK, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.NOIS, NOIS, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.FLAG, TFLAG, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.SMFL, SMFL, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.RA, RA_S, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.RAU, RAU_S, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.DEC, DEC_S, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.DECU, DECU_S, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.ROLL, ROLL_S, /SET, /NO_COPY
   HANDLE_VALUE, PHTSMCT.ROLU, ROLU_S, /SET, /NO_COPY
;
   DER_SMCHDR, SRDHDR, SMCHDR

; fill in correct header keywords
   conv_cs,phtcs(CSUnit), cs_cv
   handle_value,cs_cv.handle,cs_conv
   nmeas = phtcs(CSUnit).dims
   IF nmeas EQ 1 THEN BEGIN
      filterid = cs.c2ps(0)
      polid    = cs.c1ps(0)
      calfilt  = 0
      calpol   = 0
   ENDIF $
   ELSE BEGIN
      filterid = cs.c2ps(1)
      polid    = cs.c1ps(1)
      calfilt  = cs.c2ps(0)
      calpol   = cs.c1ps(0)
   ENDELSE
   seltrs1 = cs.f1ts(0)
   seltrs2 = cs.f2ts(0)
   selpow1 = float(cs_conv.f1ps(0))
   selpow2 = float(cs_conv.f2ps(0))
   caltype = resp
   respp1 = respon(0)
   respp2 = respon(1)
   respp3 = respon(2)
   respp4 = respon(3)
   runcp1 = respun(0)
   runcp2 = respun(1)
   runcp3 = respun(2)
   runcp4 = respun(3)
   
   FXADDPAR, SMCHDR, 'NMEAS', nmeas
   FXADDPAR, SMCHDR, 'FILTERID', filterid
   FXADDPAR, SMCHDR, 'POLID', polid
   FXADDPAR, SMCHDR, 'SELTRS1', seltrs1
   FXADDPAR, SMCHDR, 'SELTRS2', seltrs2
   FXADDPAR, SMCHDR, 'SELPOW1', selpow1
   FXADDPAR, SMCHDR, 'SELPOW2', selpow2
   FXADDPAR, SMCHDR, 'CALFILT', calfilt
   FXADDPAR, SMCHDR, 'CALPOL', calpol
   FXADDPAR, SMCHDR, 'CALTYPE', caltype
   FXADDPAR, SMCHDR, 'RESPP1', respp1
   FXADDPAR, SMCHDR, 'RESPP2', respp2
   FXADDPAR, SMCHDR, 'RESPP3', respp3
   FXADDPAR, SMCHDR, 'RESPP4', respp4
   FXADDPAR, SMCHDR, 'RUNCP1', runcp1
   FXADDPAR, SMCHDR, 'RUNCP2', runcp2
   FXADDPAR, SMCHDR, 'RUNCP3', runcp3
   FXADDPAR, SMCHDR, 'RUNCP4', runcp4

   HANDLE_VALUE, PHTSMCT.HDR, SMCHDR, /SET

; Update the PHTSMC structure using the temporery structure

   PHTSMC = [PHTSMC, PHTSMCT]

;; Update the PHTSMC value in the administration file using the index

    SMCPOS = N_ELEMENTS(PHTSMC) - 1
;   INDADM = WHERE (ADM.NAME EQ PHTSMCT.ADMN)
;   ADM(INDADM).PHTSMC = SMCPOS

   RETURN

NORESPACT:    
   XDISPLAYFILE, "", $
    TITLE = "Not actual responses available ", $
    WIDTH = 60, $
    HEIGHT = 2, $
    TEXT = [" ERROR - Cannot find calculated responses"]
   RETURN
END
