;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME: 
;	PIA_SM_Disp_FSC_List	
;
; PURPOSE:
;	Display the list of IRAS point sources that are closer than DIST_MAX to
;       the slew specified by the working structure TMPDATA.
;
; CATEGORY:
;	PIA - graphics
;
; CALLING SEQUENCE:
;	PIA_sm_disp_fsc_list, tmpdata, dist_max
;
; INPUTS:
;       TMPDATA:  A Serendipity working structure as created by
;                 PIA_SM_PSEWORK_CREATE.
;
;       DIST_MAX: The maximum distance from (in degrees) at which IRAS point
;                 sources should be searched.
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	No explicit outputs.  A file viewing widget is created.
;
; COMMON BLOCKS:
;	pia_sm_disp_fsc_list_cmn
;
; SIDE EFFECTS:
;	Triggers the XMANAGER if it is not already in use.
;
; PROCEDURE:
;	Open a file and create a widget to display its contents.
;
; PROCEDURES USED:
;       INTERNAL:
;		PIA_Disp_Print_File_evt
;
; MODIFICATION HISTORY:
;	Written by S. Bogun (MPIA)                   	August 1997
;			
;-
; *********************************************************************
; *
; *********************************************************************
PRO PIA_SM_Disp_Fsc_list_evt, event

   COMMON pia_sm_disp_fsc_list_cmn, tmpdata_, dist, assocs, s_assocT, text, $
    dist_limit, saven_id, mdistv_id

   WIDGET_CONTROL, GET_UVALUE = retval, event.id

   CASE retval OF

      "EXIT": WIDGET_CONTROL, event.top, /DESTROY
      
      "PRINT": BEGIN
         lines = N_ELEMENTS(s_assocT) 	
         filename = !PRINT + 'tmp.txt'
         OPENW,unit,filename, /get_lun
         PRINTF,unit
         FOR i=0,lines-1 DO $
          PRINTF,unit,s_assocT(i)
;	  	PRINTF,unit,string(12B)
         FREE_LUN, unit
         
         IF !VERSION.OS NE 'vms' THEN BEGIN
            spawn, '$PIA_INCL/a2ps -1 -p '+filename+ $
             '> $PIA_PRINTDIR/PIA_txt.ps '
            spawn,'lpr -P$pslaser $PIA_PRINTDIR/PIA_txt.ps'
            spawn,'rm $PIA_PRINTDIR/PIA_txt.ps'
         ENDIF ELSE BEGIN
            spawn,'print/queue=pslaser/notify/delete '+filename
         ENDELSE
         
      END

      "SAVE": BEGIN
         WIDGET_CONTROL, saven_id, get_value=savename
         savename = savename(0)
         lines = N_ELEMENTS(s_assocT)	
         OPENW,1,savename
         PRINTF,1
         FOR i=0,lines-1 DO $
          PRINTF,1,s_assocT(i)
         PRINTF,1,string(12B)
         CLOSE,1
      END	

      "RUN": BEGIN
         WIDGET_CONTROL, event.top, /HOURGLASS
         WIDGET_CONTROL, mdistv_id, get_value=s_dist_max
         dist_max = double(s_dist_max(0))/60.d0
         s_dist_max = string(dist_max*60.,format='(F4.1)')
         WIDGET_CONTROL, mdistv_id, set_value=s_dist_max

; Are associations possible?
         IF assocs(0).cntr NE -9999L THEN BEGIN 
            nassocs = n_elements(assocs)

; prepare coordinates
            ra = tmpdata_.ra
            dec = tmpdata_.dec
            time = tmpdata_.time

; define working structure, similar to pia_sm_fsc_assoc output + slew time
            assocT = { IRAS_FSC_ASSOC_SLEW, $
                       CRA:      '           ', $
                       CDEC:     '          ', $
                       CNTR:     0L, $
                       NAME:     '            ', $
                       FNU_12:   0.0, $
                       FNU_25:   0.0, $
                       FNU_60:   0.0, $
                       FNU_100:  0.0, $
                       FQUAL_12: 0, $
                       FQUAL_25: 0, $
                       FQUAL_60: 0, $
                       FQUAL_100:0, $
                       NID:      0, $
                       IDTYPE:   0, $
                       DIST:     0.0D0, $
                       TIME:     0.0 $
                     }

; now use real coordinates and calculate distances
            assocT = replicate(assocT,nassocs)
            FOR i=0,nassocs-1 DO BEGIN 
               dist_min = min(sphdist(ra,dec,assocs(i).ra,assocs(i).dec),imin)
; interpolate data to high resolution in order to compute exact distance from
; slew path
               dist_int = dindgen(21)/20 * (dist(imin+2)-dist(imin-2)) + dist(imin-2)
               ra_int = interpol(ra,dist,dist_int)
               dec_int = interpol(dec,dist,dist_int)
               time_int = interpol(time,dist,dist_int)
               dist_min = min(sphdist(ra_int,dec_int,assocs(i).ra,assocs(i).dec),imin)
               FOR j=2,n_tags(assocs)-1 DO assocT(i).(j) = assocs(i).(j)
               assocT(i).time = time_int(imin)
               assocT(i).dist = dist_min
               assocT(i).cra = deg2ra(assocs(i).ra)
               assocT(i).cdec = deg2dec(assocs(i).dec)
            ENDFOR

; find associations closer than dist_max
            idist = where(assocT.dist LT dist_max, ndist)
            IF ndist GT 0 THEN BEGIN 

               assocT = assocT(idist)
; sort associations by time
               itime = sort(assocT.time)
               assocT = assocT(itime)
               
; print to widget
               head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
                ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
                ' Dist | Time'
               head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
                ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
                ' ['']  | [s]'
               assocT.dist = assocT.dist * 60.
               s_assocT = string(assocT,format='(a11,1x,a10,i7,1x,a12,4f9.2,4i8,i6,i9, ' + $
                                 'f7.2,f9.2)')
               assocT.dist = assocT.dist / 60.

               widget_control,text, SET_VALUE=head1
               widget_control,text, SET_VALUE=head2, /APPEND
               widget_control,text, SET_VALUE=s_assocT, /APPEND
            ENDIF $
            ELSE BEGIN 
               head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
                ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
                ' Dist | Time'
               head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
                ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
                ' ['']  | [s]'
               widget_control,text, SET_VALUE=head1
               widget_control,text, SET_VALUE=head2, /APPEND
               widget_control,text, SET_VALUE='  ***  No associations found.  ***', /APPEND
            ENDELSE
         ENDIF $
         ELSE BEGIN
            head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
             ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
             ' Dist | Time'
            head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
             ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
             ' ['']  | [s]'
            widget_control,text, SET_VALUE=head1
            widget_control,text, SET_VALUE=head2, /APPEND
            widget_control,text, SET_VALUE='  ***  No associations found.  ***', /APPEND
         ENDELSE 
      END

      ELSE:
      
   ENDCASE
END


; *********************************************************************
; *
; *
; *
; *********************************************************************
PRO PIA_SM_DISP_FSC_LIST, tmpdata, dist_max, GROUP = GROUP

   COMMON pia_sm_disp_fsc_list_cmn, tmpdata_, dist, assocs, s_assocT, text, $
    dist_limit, saven_id, mdistv_id

; maximal possible search radius
   dist_limit = 0.1d0

; default file name for saving
   savename = !PIA_SERDIR + '/P2ES'+strmid(tmpdata.admn,4,8)+'_fsc.tbl'

; dist_max for widget
   s_dist_max = string(dist_max*60.,format='(F4.1)')

; create widget structure
   base = WIDGET_BASE(TITLE = 'IRAS Point Source catalog' ,$
                      /COLUMN, $
                      SPACE = 20, $
                      XPAD = 20, $
                      YPAD = 20)

   base1 = WIDGET_BASE(base, /ROW, /FRAME)

   filequit = WIDGET_BUTTON(base1, $
                            VALUE = " Quit ", $
                            UVALUE = "EXIT")

   filep_id = WIDGET_BUTTON(base1, $
                            VALUE = " Print ", $
                            UVALUE = "PRINT")

   files_id = WIDGET_BUTTON(base1, $
                            VALUE = " Save to File ", $
                            UVALUE = "SAVE")

   saven_id = WIDGET_TEXT(base1, $
                          VALUE = savename, xsize=50, $
                          UVALUE = "JUNK", /EDITABLE)		

   mdists_id = WIDGET_LABEL(base1, $
                            VALUE = " Max. search distance [arcmin]: ")

   mdistv_id = WIDGET_TEXT(base1, $
                           VALUE = s_dist_max, xsize=4, $
                           UVALUE = "DIST_MAX", /EDITABLE)		

   exec_id = WIDGET_BUTTON(base1, $
                           VALUE = "   Run   ", $
                           UVALUE = "RUN")

   text = WIDGET_TEXT(base, $ 
                      XSIZE = 88, $
                      YSIZE = 18, $
                      /SCROLL)

; realize widget
   WIDGET_CONTROL, base, /REALIZE, /HOURGLASS

; prepare coordinates
   tmpdata_ = tmpdata
   ra = tmpdata_.ra
   dec = tmpdata_.dec
   time = tmpdata_.time

   dist = ra * 0.D0
   FOR i = 1, tmpdata_.dims - 1 DO $
    dist(i) = dist(i-1) + sphdist(ra(i-1),dec(i-1),ra(i),dec(i))

; search first with maximal possible search radius, interpolate slew positions to
; larger spatial distances
   step = 0.5*dist_limit
   nstep = nint(dist(tmpdata_.dims-1)/step)
   dist_int = dindgen(nstep)*step
   ra_int = interpol(ra,dist,dist_int)
   dec_int = interpol(dec,dist,dist_int)
   time_int = interpol(time,dist,dist_int)

; define working structures, similar to pia_sm_fsc_assoc output + slew time

   assocs = { IRAS_FSC_ASSOC, $
              RA:      0.0D0, $
              DEC:     0.0D0, $
              CNTR:     0L, $
              NAME:     '            ', $
              FNU_12:   0.0, $
              FNU_25:   0.0, $
              FNU_60:   0.0, $
              FNU_100:  0.0, $
              FQUAL_12: 0, $
              FQUAL_25: 0, $
              FQUAL_60: 0, $
              FQUAL_100:0, $
              NID:      0, $
              IDTYPE:   0, $
              DIST:     0.0D0 $
            }

   assocT = { IRAS_FSC_ASSOC_SLEW, $
              CRA:      '           ', $
              CDEC:     '          ', $
              CNTR:     0L, $
              NAME:     '            ', $
              FNU_12:   0.0, $
              FNU_25:   0.0, $
              FNU_60:   0.0, $
              FNU_100:  0.0, $
              FQUAL_12: 0, $
              FQUAL_25: 0, $
              FQUAL_60: 0, $
              FQUAL_100:0, $
              NID:      0, $
              IDTYPE:   0, $
              DIST:     0.0D0, $
              TIME:     0.0 $
            }

; search all associations
   FOR i=0, nstep-1 DO BEGIN 
      assoc1 = pia_sm_fsc_assoc(ra_int(i),dec_int(i),dist_limit,nassoc1)
      IF nassoc1 GT 0 THEN $
       assocs = [assocs,assoc1]
   ENDFOR 

   IF N_ELEMENTS(assocs) GT 1 THEN BEGIN
      assocs = assocs(1:*)

; sort associations by name and throw away mulitple associations
      iobj = sort(assocs.name)
      assocs = assocs(iobj)
      iuniq = uniq(assocs.name)
      nassocs = n_elements(iuniq)
      assocs = assocs(iuniq)

; now use real coordinates and calculate distances
      assocT = replicate(assocT,nassocs)
      FOR i=0,nassocs-1 DO BEGIN 
         dist_min = min(sphdist(ra,dec,assocs(i).ra,assocs(i).dec),imin)
; interpolate data to high resolution in order to compute exact distance from
; slew path
         dist_int = dindgen(21)/20 * (dist(imin+2)-dist(imin-2)) + dist(imin-2)
         ra_int = interpol(ra,dist,dist_int)
         dec_int = interpol(dec,dist,dist_int)
         time_int = interpol(time,dist,dist_int)
         dist_min = min(sphdist(ra_int,dec_int,assocs(i).ra,assocs(i).dec),imin)
         FOR j=2,n_tags(assocs)-1 DO assocT(i).(j) = assocs(i).(j)
         assocT(i).time = time_int(imin)
         assocT(i).dist = dist_min
         assocT(i).cra = deg2ra(assocs(i).ra)
         assocT(i).cdec = deg2dec(assocs(i).dec)
      ENDFOR

; find associations closer than dist_max
      idist = where(assocT.dist LT dist_max, ndist)         
      IF ndist GT 0 THEN BEGIN 

         assocT = assocT(idist)
; sort associations by time
         itime = sort(assocT.time)
         assocT = assocT(itime)

; print to widget
         head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
          ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
          ' Dist | Time'
         head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
          ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
          ' ['']  | [s]'
         assocT.dist = assocT.dist * 60.
         s_assocT = string(assocT,format='(a11,1x,a10,i7,1x,a12,4f9.2,4i8,i6,i9, ' + $
                           'f7.2,f9.2)')
         assocT.dist = assocT.dist / 60.

         widget_control,text, SET_VALUE=head1
         widget_control,text, SET_VALUE=head2, /APPEND
         widget_control,text, SET_VALUE=s_assocT, /APPEND
      ENDIF $
      ELSE BEGIN
         head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
          ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
          ' Dist | Time'
         head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
          ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
          ' ['']  | [s]'
         widget_control,text, SET_VALUE=head1
         widget_control,text, SET_VALUE=head2, /APPEND
         widget_control,text, SET_VALUE='  ***  No associations found.  ***', /APPEND
      ENDELSE 
   ENDIF $
   ELSE BEGIN
      assocs.cntr = -9999L
      head1 = '| RA        | DEC      | CNTR | FSC Name  | fnu_12  | fnu_25 |' + $
       ' fnu_60 | fnu_100| fqual | fqual | fqual | fqual | NID | IDTYPE |' + $
       ' Dist | Time'
      head2 = '| (J2000)   | (J2000)  |      |           | [Jy]    | [Jy]   |' + $
       ' [Jy]   | [Jy]   | 12    | 25    | 60    | 100   |     |        |' + $
       ' ['']  | [s]'
      widget_control,text, SET_VALUE=head1
      widget_control,text, SET_VALUE=head2, /APPEND
      widget_control,text, SET_VALUE='  ***  No associations found.  ***', /APPEND
   ENDELSE 

   Xmanager, /NO_BLOCK, "pia_sm_disp_fsc_list", $
    base, $
    GROUP_LEADER = GROUP, $
    EVENT_HANDLER = "PIA_sm_disp_fsc_list_evt" 

   
END
