



;******************************************************************************
;+
;*NAME:
;
;    FLAGNAN
;
;*CLASS:
;
;*CATEGORY:
;
;*PURPOSE:
;
;    To flag and/or replace IEEE NaN values in an IDL array.
;
;*CALLING SEQUENCE:
;
;    FLAGNAN,ARRAY,IND,COUNT,nvalue
;
;*PARAMETERS:
;
;    ARRAY (REQ) (IO) (012) (RD)
;       IDL array.
;
;    IND (REQ) (O) (1) (I)
;       indices of found NaN values
;
;    COUNT (REQ) (O) (0) (IL)
;       number of found NaN values. (0 if none found.)
;
;    nvalue (OPT) (I) (0) (BILF)
;       optional parameter specifying replacement value.
;       If not specified, user is prompted for value.
;
;*EXAMPLES:
;
;*SYSTEM VARIABLES USED:
;
;*INTERACTIVE INPUT:
;
;*SUBROUTINES CALLED:
;
;   PARCHECK
;
;*FILES USED:
;
;*SIDE EFFECTS:
;
;       If no NaN values are found, or if ARRAY is not of type float, double
;       precision, or complex, then -1 is returned, and COUNT is set to 0.
;
;*RESTRICTIONS:
;
;       ARRAY must be of type float, double-precision, or complex.
;
;*PROCEDURE:
;
;       The bit patterns of the numbers being tested are compared against the
;       IEEE NaN standard.
;
;*INF_1:
;
;*NOTES:
;
;       tested with IDL Version 2.2.0 (sunos sparc)   14 Nov 91
;       tested with IDL Version 2.2.0 (ultrix mipsel) 14 Nov 91
;       tested with IDL Version 2.2.0 (ultrix vax)    14 Nov 91
;       tested with IDL Version 2.3.2 (vax vms)       19 Oct 92
;
;*MODIFICATION HISTORY:
;
;       written by Bill Thompson 2/92
;        8-14-92 rwt add nvalue option, change from a function to a
;               procedure & print out results
;       10-19-92 rwt make ind and count required output parameters
;-
;********************************************************************
 pro flagnan,array,ind,count,nvalue
;
 npar = n_params()
 if (npar eq 0) then begin
    print,' FLAGNAN,ARRAY,IND,COUNT,nvalue'
    retall
 endif  ; npar
;;-- parcheck,npar,[3,4],'FLAGNAN'
;
; call wherenan
;
;
        ON_ERROR,2
;
;  Parse the input array based on the datatype.
;
        SZ = SIZE(ARRAY)
        CASE SZ(SZ(0)+1) OF
;
;  Single precision floating point.
;
                4:  BEGIN
                        LARRAY = LONG(ARRAY,0,N_ELEMENTS(ARRAY))
                        BYTEORDER,LARRAY,/NTOHL
                        E0 = '7F800000'X
                        E = LARRAY AND E0
                        F = LARRAY AND '7FFFFF'X
                        RESULT = WHERE((E EQ E0) AND (F NE 0), COUNT)
                        END
;
;  Double precision floating point.
;
                5:  BEGIN
                        LARRAY = LONG(ARRAY,0,2,N_ELEMENTS(ARRAY))
                        BYTEORDER,LARRAY,/NTOHL
                        E0 = '7FF00000'X
                        E = LARRAY(0,*) AND E0
                        F1 = LARRAY(0,*) AND 'FFFFF'X
                        RESULT = WHERE((E EQ E0) AND ((F1 NE 0) OR      $
                                (LARRAY(1,*) NE 0)), COUNT)
                        END
;
;  Single precision complex floating point.
;
                6:  BEGIN
                        LARRAY = LONG(ARRAY,0,2,N_ELEMENTS(ARRAY))
                        BYTEORDER,LARRAY,/NTOHL
                        E0 = '7F800000'X
                        E1 = LARRAY(0,*) AND E0
                        E2 = LARRAY(1,*) AND E0
                        F1 = LARRAY(0,*) AND '7FFFFF'X
                        F2 = LARRAY(1,*) AND '7FFFFF'X
                        RESULT = WHERE(((E1 EQ E0) AND (F1 NE 0)) OR    $
                                ((E2 EQ E0) AND (F2 NE 0)), COUNT)
                        END
                ELSE:  BEGIN
                        MESSAGE,'Data type must be floating point',/INFORMATIONAL
                        RESULT = -1
                        COUNT = 0
                        END
        ENDCASE
ind = result
;
; if any found, reassign with user-specified value
;
 if (count gt 0) and (npar eq 4) then begin
     print,'  ',strtrim(string(count),2),' special character(s) found,', $
              ' reassigned value =',strtrim(string(nvalue),2)
     array(ind) = nvalue
 endif
;
return
end     ;flagnan









	FUNCTION WHERENAN, ARRAY, COUNT
;+
; NAME:
;	WHERENAN
;
; PURPOSE:
;	Find the positions of all values within an array that correspond to the
;	IEEE NaN (not-a-number) special values.
;
; CATEGORY:
;       Array
;
; CALLING SEQUENCE:
;	Result = WHERENAN( ARRAY [, COUNT ] )
;
; INPUT PARAMETERS:
;	ARRAY	= Array to test against the IEEE NaN special values.  Must be
;		  of either floating point, double-precision, or complex type.
;
; OUTPUTS:
;	The result of the function is the indices of all values of ARRAY
;	corresponding to the IEEE NaN specification, similar to the IDL WHERE
;	function.
;
; OPTIONAL OUTPUT PARAMETERS:
;	COUNT	= Number of values found corresponding to IEEE NaN.
;
; OPTIONAL KEYWORD PARAMETERS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;	If no NaN values are found, or if ARRAY is not of type float, double
;	precision, or complex, then -1 is returned, and COUNT is set to 0.
;
; RESTRICTIONS:
;	ARRAY must be of type float, double-precision, or complex.
;
; PROCEDURE:
;	The bit patterns of the numbers being tested are compared against the
;	IEEE NaN standard.
;
; MODIFICATION HISTORY:
;	William Thompson, Feb. 1992.
;	William Thompson, Oct. 1992, fixed bug regarding order of bytes on VAX
;		machines.
;       Added to idlmeteo from the
;       Astronomy User's Library         Nov-1992    oet@sma.ch
;
;-
;
	ON_ERROR,2
;
;  Check the number of parameters.
;
	IF N_PARAMS() LT 1 THEN MESSAGE,	$
		'Syntax:  Result = WHERENAN(ARRAY [,COUNT])'
;
;  Parse the input array based on the datatype.
;
	SZ = SIZE(ARRAY)
	CASE SZ(SZ(0)+1) OF
;
;  Single precision floating point.
;
		4:  BEGIN
			LARRAY = LONG(ARRAY,0,N_ELEMENTS(ARRAY))
			BYTEORDER,LARRAY,/NTOHL
			E0 = '7F800000'X
			E = LARRAY AND E0
			F = LARRAY AND '7FFFFF'X
			RESULT = WHERE((E EQ E0) AND (F NE 0), COUNT)
			END
;
;  Double precision floating point.
;
		5:  BEGIN
			LARRAY = LONG(ARRAY,0,2,N_ELEMENTS(ARRAY))
			BYTEORDER,LARRAY,/NTOHL
			E0 = '7FF00000'X
			E = LARRAY(0,*) AND E0
			F1 = LARRAY(0,*) AND 'FFFFF'X
			RESULT = WHERE((E EQ E0) AND ((F1 NE 0) OR	$
				(LARRAY(1,*) NE 0)), COUNT)
			END
;
;  Single precision complex floating point.
;
		6:  BEGIN
			LARRAY = LONG(ARRAY,0,2,N_ELEMENTS(ARRAY))
			BYTEORDER,LARRAY,/NTOHL
			E0 = '7F800000'X
			E1 = LARRAY(0,*) AND E0
			E2 = LARRAY(1,*) AND E0
			F1 = LARRAY(0,*) AND '7FFFFF'X
			F2 = LARRAY(1,*) AND '7FFFFF'X
			RESULT = WHERE(((E1 EQ E0) AND (F1 NE 0)) OR	$
				((E2 EQ E0) AND (F2 NE 0)), COUNT)
			END
		ELSE:  BEGIN
			MESSAGE,'Data type must be floating point',/INFORMATIONAL
			RESULT = -1
			COUNT = 0
			END
	ENDCASE
;
	RETURN, RESULT
	END





;====================================================================================
;====================================================================================









Function   PIA_SM_RGauss2DPlane,    Xvec , Yvec , Params , ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D elliptical rotated Gaussian    (Amplitude, sigmax/y, centerx/y, positionangle)      +
;  tilted 2D Background    (a + bx + cx)
;
;


NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

XPrime     = DblArr (NData)
YPrime     = DblArr (NData)

IntParams  = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10
If (IntParams (2)  eq  0.0)  then  IntParams (2) = 1.0D-10
If (IntParams (5)  eq  0.0)  then  IntParams (5) = 1.0D-1

SinAlph    = sin (IntParams(5))
CoSinAlph  = cos (IntParams(5))

XPrime (*) = (CoSinAlph * Xvec (*))   +     (SinAlph * Yvec (*))
YPrime (*) = ( -SinAlph * Xvec (*))   +   (CoSinAlph * Yvec (*))
X0Prime    = (CoSinAlph * Params(3))  +     (SinAlph * Params (4))
Y0Prime    = ( -SinAlph * Params(3))  +   (CoSinAlph * Params (4))

NormDeltaX (*)   = (XPrime (*)  -  X0Prime) / IntParams(1)
NormDeltaY (*)   = (YPrime (*)  -  Y0Prime) / IntParams(2)
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ (*))
FunValues  (*)   = Params(0) * UnitValues (*)

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * (NormDeltaX(*)^2) / IntParams (1)
     ParmDeriv (*,2) = FunValues(*) * (NormDeltaY(*)^2) / IntParams (2)
     ParmDeriv (*,3) = FunValues(*) * NormDeltaX(*) / IntParams(1)
     ParmDeriv (*,4) = FunValues(*) * NormDeltaY(*) / IntParams(2)

     ParmDeriv (*,5) = - FunValues(*) * NormDeltaX (*) * NormDeltaY (*)   *   $
                       ( (IntParams(2) / IntParams(1)) - (IntParams(1) / IntParams(2)) )

     ParmDeriv (*,6) = 1.0
     Parmderiv (*,7) = Xvec (*)
     Parmderiv (*,8) = Yvec (*)
  endif

FunValues  (*) = FunValues (*)  +  Params (6)            +    $
                                  (XVec(*) * Params(7))  +                      $
                                  (YVec(*) * Params(8))

Return, FunValues(*)
End








Function   PIA_SM_Ridge2D,    Xvec , Yvec , Params , ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData = N_Elements (XVec)
NParm = N_Elements (Params)

Height     = DblArr (NData)
NormRadius = DblArr (NData)
FunValues  = DblArr (NData)
UnitValues = DblArr (NData)

XPrime     = DblArr (NData)
YPrime     = DblArr (NData)

IntParams  = Params (*)
If (IntParams (3)  eq  0.0)  then  IntParams (3) = 1.0D-10
If (IntParams (6)  eq  0.0)  then  IntParams (6) = 1.0D-1

SinAlph    = sin (Params(6))
CoSinAlph  = cos (Params(6))

XPrime (*) = (CoSinAlph * Xvec (*))   +     (SinAlph * Yvec (*))
YPrime (*) = ( -SinAlph * Xvec (*))   +   (CoSinAlph * Yvec (*))
X0Prime    = (CoSinAlph * Params(4))  +     (SinAlph * Params (5))
Y0Prime    = ( -SinAlph * Params(4))  +   (CoSinAlph * Params (5))

Height (*) = Params (0) + (Params(1) * YPrime(*)) + (Params(2) * (YPrime(*)^2))

NormRadius (*) = (XPrime (*)  -  X0Prime) / IntParams (3)
UnitValues (*) = exp (-0.5 * (NormRadius (*) ^ 2))
FunValues  (*) = Height (*) * UnitValues (*)

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)
     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = UnitValues(*) * YPrime(*)
     ParmDeriv (*,2) = UnitValues(*) * (YPrime(*)^2)
     ParmDeriv (*,3) = FunValues(*) * (NormRadius(*)^2) / IntParams (3)
     ParmDeriv (*,4) = FunValues(*) * NormRadius(*) / IntParams (3) * CoSinAlph
     ParmDeriv (*,5) = FunValues(*) * NormRadius(*) / IntParams (3) * SinAlph

     Tmp1            = ((Params(1)  +  (2.0D0 * Params(2) * Yvec(*))) * XPrime (*))
     ParmDeriv (*,6) = -(Tmp1(*) * Unitvalues(*))
     Tmp1            = - UnitValues(*) * Normradius(*) / IntParams(3) * (YPrime(*)-Y0Prime)
     ParmDeriv (*,6) = Parmderiv (*,6)  +  (Tmp1(*) * Height(*))  +  $
                       (YPrime(*) * Params(8))  -  (XPrime(*) * Params(9))
     ParmDeriv (*,7) = 1.0D0
     ParmDeriv (*,8) = XPrime (*)
     ParmDeriv (*,9) = YPrime (*)
  endif

FunValues  (*) = FunValues (*)  +  Params (7) +    $
                 (Params (8) * XPrime(*))  +  (Params (9) * YPrime(*))

Return, FunValues(*)
End













Function    PIA_SM_Gauss1DPoly1,    XVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  1D Gaussian    (Amplitude, sigma, center)      +
;  Linear Background    (a + bx)
;
;
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormRadius = DblArr (NData)
FunValues  = DblArr (NData)
UnitValues = DblArr (NData)

IntParams = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10

NormRadius (*) = (XVec (*)  -  Params (2)) / IntParams (1)
UnitValues (*) = exp (-0.5 * (NormRadius (*) ^ 2))
FunValues  (*) = (Params(0) * UnitValues (*))   +              $
                 Params(3) +  (XVec(*)  *  Params (4))

If (N_Params ()  ge 3)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)
     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues (*) * NormRadius(*) * NormRadius(*) / IntParams (1)
     ParmDeriv (*,2) = FunValues (*) * NormRadius(*) / IntParams (1)
     ParmDeriv (*,3) = 1.0
     Parmderiv (*,4) = XVec (*)
  endif

Return, FunValues(*)
End







Function    PIA_SM_Gauss1D,    XVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  1D Gaussian    (Amplitude, sigma, center)
;
;
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormRadius = DblArr (NData)
FunValues  = DblArr (NData)
UnitValues = DblArr (NData)

IntParams = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10

NormRadius (*) = (XVec (*)  -  Params (2)) / IntParams (1)
UnitValues (*) = exp (-0.5 * (NormRadius (*) ^ 2))
FunValues  (*) = (Params(0) * UnitValues (*))

If (N_Params ()  ge 3)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)
     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues (*) * NormRadius(*) * NormRadius(*) / IntParams (1)
     ParmDeriv (*,2) = FunValues (*) * NormRadius(*) / IntParams (1)
  endif

Return, FunValues(*)
End









Function   PIA_SM_Gauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D noncircular Gaussian    (Amplitude, sigmax/y, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

IntParams  = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10
If (IntParams (2)  eq  0.0)  then  IntParams (2) = 1.0D-10

NormDeltaX (*)   = (XVec (*)  -  Params (3)) / IntParams(1) / cos (Params(4) * !DTOR)
;;;;NormDeltaX (*)   = (XVec (*)  -  Params (3)) / IntParams(1)
NormDeltaY (*)   = (YVec (*)  -  Params (4)) / IntParams(2)
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ(*))
FunValues  (*)   = (Params(0) * UnitValues (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * (NormDeltaX(*)^2)  / IntParams(1)
     ParmDeriv (*,2) = FunValues(*) * (NormDeltaY(*)^2)  / IntParams(2)
     ParmDeriv (*,3) = FunValues(*) * NormDeltaX(*) / IntParams(1)
     ParmDeriv (*,4) = FunValues(*) * NormDeltaY(*) / IntParams(2)

     ParmDeriv (*,5) = 1.0
     Parmderiv (*,6) = Xvec (*)
     Parmderiv (*,7) = Yvec (*)
  endif

FunValues (*) = FunValues (*)   +                             $
                (XVec(*) * Params(6))  +                      $
                (YVec(*) * Params(7))  +  Params(5)

Return, FunValues (*)
End












Function   PIA_SM_DblGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D noncircular DoubleGaussian
;            (Amplitude, sigmax/y, factor, sigmax2/y2, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

NormDeltaX2   = DblArr (NData)
NormDeltaY2   = DblArr (NData)
NormRadiusSQ2 = DblArr (NData)
FunValues2    = DblArr (NData)
UnitValues2   = DblArr (NData)

IntParams  = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10
If (IntParams (2)  eq  0.0)  then  IntParams (2) = 1.0D-10
If (IntParams (4)  eq  0.0)  then  IntParams (4) = 1.0D-10
If (IntParams (5)  eq  0.0)  then  IntParams (5) = 1.0D-10

NormDeltaX (*)   = (XVec (*)  -  Params (6)) / IntParams(1) / cos (Params(7) * !DTOR)
;;;;NormDeltaX (*)   = (XVec (*)  -  Params (6)) / IntParams(1)
NormDeltaY (*)   = (YVec (*)  -  Params (7)) / IntParams(2)
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2

NormDeltaX2 (*)   = (XVec (*)  -  Params (6)) / IntParams(4) / cos (Params(7) * !DTOR)
;;;;NormDeltaX2 (*)   = (XVec (*)  -  Params (6)) / IntParams(4)
NormDeltaY2 (*)   = (YVec (*)  -  Params (7)) / IntParams(5)
NormRadiusSQ2 (*) = NormDeltaX2(*) ^ 2    +    NormDeltaY2(*) ^ 2

UnitValues (*)    = exp (-0.5 * NormRadiusSQ(*))
UnitValues2 (*)   = exp (-0.5 * NormRadiusSQ2(*))

FunValues  (*)   = (Params(0) * UnitValues (*))
FunValues2  (*)  = (Params(0) * Params(3) * UnitValues2 (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * (NormDeltaX(*)^2)  / IntParams(1)
     ParmDeriv (*,2) = FunValues(*) * (NormDeltaY(*)^2)  / IntParams(2)

     ParmDeriv (*,3) = Params(0) * UnitValues2(*)
     ParmDeriv (*,4) = FunValues2(*) * (NormDeltaX2(*)^2)  / IntParams(4)
     ParmDeriv (*,5) = FunValues2(*) * (NormDeltaY2(*)^2)  / IntParams(5)

     ParmDeriv (*,6) = (FunValues(*) * NormDeltaX(*) / IntParams(1))   +   $
                       (FunValues2(*) * NormDeltaX2(*) / IntParams(4))
     ParmDeriv (*,7) = (FunValues(*) * NormDeltaY(*) / IntParams(2))    +   $
                       (FunValues2(*) * NormDeltaY2(*) / IntParams(5))

     ParmDeriv (*,8)  = 1.0
     Parmderiv (*,9)  = Xvec (*)
     Parmderiv (*,10) = Yvec (*)
  endif

FunValues (*) = FunValues (*)   +   FunValues2 (*)   +        $
                (XVec(*) * Params(9))   +                     $
                (YVec(*) * Params(10))  +  Params(8)

Return, FunValues (*)
End












Function   PIA_SM_CGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D circular Gaussian    (Amplitude, sigma, xcenter , ycenter)   +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

IntParams  = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-10

NormDeltaX (*)   = (XVec (*)  -  Params (2)) / IntParams(1) / cos (Params(3) * !DTOR)
;; NormDeltaX (*)   = (XVec (*)  -  Params (2)) / IntParams(1)
NormDeltaY (*)   = (YVec (*)  -  Params (3)) / IntParams(1)
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ(*))
FunValues  (*)   = (Params(0) * UnitValues (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * NormRadiusSQ / IntParams(1)
     ParmDeriv (*,2) = FunValues(*) * NormDeltaX (*) / IntParams(1)
     ParmDeriv (*,3) = FunValues(*) * NormDeltaY (*) / IntParams(1)

     ParmDeriv (*,4) = 1.0
     Parmderiv (*,5) = Xvec (*)
     Parmderiv (*,6) = Yvec (*)
  endif

FunValues (*) = FunValues (*)   +                             $
                (XVec(*) * Params(5))  +                      $
                (YVec(*) * Params(6))  +  Params(4)

Return, FunValues (*)
End













Function   PIA_SM_FGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D circular Gaussian with Fixed Width   (Amplitude, xcenter , ycenter)
;
;  Width is set in the 1. statement
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

Sigma_Gauss = 53.0 / 3600.

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

NormDeltaX (*)   = (XVec (*)  -  Params (1)) / Sigma_Gauss / cos (Params(2) * !DTOR)
;; NormDeltaX (*)   = (XVec (*)  -  Params (1)) / Sigma_Gauss
NormDeltaY (*)   = (YVec (*)  -  Params (2)) / Sigma_Gauss
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ(*))
FunValues  (*)   = (Params(0) * UnitValues (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * NormDeltaX (*) / Sigma_Gauss
     ParmDeriv (*,2) = FunValues(*) * NormDeltaY (*) / Sigma_Gauss

     ParmDeriv (*,3) = 1.0
     Parmderiv (*,4) = Xvec (*)
     Parmderiv (*,5) = Yvec (*)
  endif

FunValues (*) = FunValues (*)   +                             $
                (XVec(*) * Params(3))  +                      $
                (YVec(*) * Params(4))  +  Params(5)

Return, FunValues (*)
End


















Function   PIA_SM_SPDblGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;  SubPixel - Interface to
;  2D noncircular DoubleGaussian    (Amplitude, sigmax/y, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;


NParm = N_Elements (Params)

Dimen     = Size (XVec)
NData     = Dimen (1)
NSubPixel = Dimen (2)

FNSubPixel = Float (NSubPixel-1)

FunValues = DblArr (NData)
If (N_Params () ge 4)  then   ParmDeriv  = DblArr (NData , NParm)

For i = 0 , NData-1 do begin

   NaNIndex = WHERENAN (Params , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_SPDblGauss2DPlane> NaNs detected !! , ... returning ! '
         Return, FunValues(*)
      endif


   XSubVec = Transpose (XVec (i , 1:*))
   YSubVec = Transpose (YVec (i , 1:*))

   SubPixValues = PIA_SM_DblGauss2DPlane  (XSubVec, YSubVec, Params, ParmSubDeriv)

   FunValues (i) = Total (SubPixValues (*)) / FNSubPixel

   If (N_Params () ge 4)  $
      then begin
        For j = 0 , NParm-1  do begin
           ParmDeriv (i,j) = Total (ParmSubDeriv (*,j)) / FNSubPixel
        EndFor
      endif

EndFor

Return, FunValues (*)
End














Function   PIA_SM_SPGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;  SubPixel - Interface to
;  2D noncircular Gaussian    (Amplitude, sigmax/y, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;


NParm = N_Elements (Params)

Dimen     = Size (XVec)
NData     = Dimen (1)
NSubPixel = Dimen (2)

FNSubPixel = Float (NSubPixel-1)

FunValues = DblArr (NData)
If (N_Params () ge 4)  then   ParmDeriv  = DblArr (NData , NParm)

For i = 0 , NData-1 do begin

   NaNIndex = WHERENAN (Params , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_SPGauss2DPlane> NaNs detected !! , ... returning ! '
         Return, FunValues(*)
      endif


   XSubVec = Transpose (XVec (i , 1:*))
   YSubVec = Transpose (YVec (i , 1:*))

   SubPixValues = PIA_SM_Gauss2DPlane  (XSubVec, YSubVec, Params, ParmSubDeriv)

   FunValues (i) = Total (SubPixValues (*)) / FNSubPixel

   If (N_Params () ge 4)  $
      then begin
        For j = 0 , NParm-1  do begin
           ParmDeriv (i,j) = Total (ParmSubDeriv (*,j)) / FNSubPixel
        EndFor
      endif

EndFor

Return, FunValues (*)
End











Function   PIA_SM_SPCGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;  SubPixel - Interface to
;  2D ncircular Gaussian    (Amplitude, sigma, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;


NParm = N_Elements (Params)

Dimen     = Size (XVec)
NData     = Dimen (1)
NSubPixel = Dimen (2)

FNSubPixel = Float (NSubPixel-1)

FunValues = DblArr (NData)
If (N_Params () ge 4)  then   ParmDeriv  = DblArr (NData , NParm)

For i = 0 , NData-1 do begin

   NaNIndex = WHERENAN (Params , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_SPCGauss2DPlane> NaNs detected !! , ... returning ! '
         Return, FunValues(*)
      endif


   XSubVec = Transpose (XVec (i , 1:*))
   YSubVec = Transpose (YVec (i , 1:*))

   SubPixValues = PIA_SM_CGauss2DPlane (XSubVec, YSubVec, Params, ParmSubDeriv)

   FunValues (i) = Total (SubPixValues (*)) / FNSubPixel

   If (N_Params () ge 4)  $
      then begin
        For j = 0 , NParm-1  do begin
           ParmDeriv (i,j) = Total (ParmSubDeriv (*,j)) / FNSubPixel
        EndFor
      endif

EndFor

Return, FunValues (*)
End









Function   PIA_SM_SPFGauss2DPlane,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;  SubPixel - Interface to
;  2D fixedwidth circular Gaussian    (Amplitude, x/ycenter    +
;  tilted Plane as Background    (a + bx + cy)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;


NParm = N_Elements (Params)

Dimen     = Size (XVec)
NData     = Dimen (1)
NSubPixel = Dimen (2)

FNSubPixel = Float (NSubPixel-1)

FunValues = DblArr (NData)
If (N_Params () ge 4)  then   ParmDeriv  = DblArr (NData , NParm)

For i = 0 , NData-1 do begin

   NaNIndex = WHERENAN (Params , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_SPFGauss2DPlane> NaNs detected !! , ... returning ! '
         Return, FunValues(*)
      endif


   XSubVec = Transpose (XVec (i , 1:*))
   YSubVec = Transpose (YVec (i , 1:*))

   SubPixValues = PIA_SM_FGauss2DPlane (XSubVec, YSubVec, Params, ParmSubDeriv)

   FunValues (i) = Total (SubPixValues (*)) / FNSubPixel

   If (N_Params () ge 4)  $
      then begin
        For j = 0 , NParm-1  do begin
           ParmDeriv (i,j) = Total (ParmSubDeriv (*,j)) / FNSubPixel
        EndFor
      endif

EndFor

Return, FunValues (*)
End


















Function   PIA_SM_FGauss2D,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D circular Gaussian with Fixed Width   (Amplitude, xcenter , ycenter)
;
;  Width is set in the 1. statement
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

Sigma_Gauss = 0.78

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

NormDeltaX (*)   = (XVec (*)  -  Params (1)) / Sigma_Gauss  / cos (Params(2) * !DTOR)
;;;;NormDeltaX (*)   = (XVec (*)  -  Params (1)) / Sigma_Gauss
NormDeltaY (*)   = (YVec (*)  -  Params (2)) / Sigma_Gauss
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ(*))
FunValues  (*)   = (Params(0) * UnitValues (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * NormDeltaX (*) / Sigma_Gauss
     ParmDeriv (*,2) = FunValues(*) * NormDeltaY (*) / Sigma_Gauss
  endif

Return, FunValues (*)
End











Function   PIA_SM_CGauss2D,    XVec, YVec, Params, ParmDeriv
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  2D circular Gaussian    (Amplitude, sigma, xcenter , ycenter)
;
;  returns function values at all (x,y) and derivatives
;  (only if 4.th parameter is given in call)
;

NData = N_Elements (XVec)
NParm = N_Elements (Params)

NormDeltaX   = DblArr (NData)
NormDeltaY   = DblArr (NData)
NormRadiusSQ = DblArr (NData)
FunValues    = DblArr (NData)
UnitValues   = DblArr (NData)

IntParams  = Params (*)
If (IntParams (1)  eq  0.0)  then  IntParams (1) = 1.0D-20

NormDeltaX (*)   = (XVec (*)  -  Params (2)) / IntParams(1)  / cos (Params(3) * !DTOR)
;;;;NormDeltaX (*)   = (XVec (*)  -  Params (2)) / IntParams(1)
NormDeltaY (*)   = (YVec (*)  -  Params (3)) / IntParams(1)
NormRadiusSQ (*) = NormDeltaX(*) ^ 2    +    NormDeltaY(*) ^ 2
UnitValues (*)   = exp (-0.5 * NormRadiusSQ(*))
FunValues  (*)   = (Params(0) * UnitValues (*))

If (N_Params ()  ge 4)  $
  then begin
     ParmDeriv       = DblArr (NData , NParm)

     ParmDeriv (*,0) = UnitValues(*)
     ParmDeriv (*,1) = FunValues(*) * NormRadiusSQ / IntParams(1)
     ParmDeriv (*,2) = FunValues(*) * NormDeltaX (*) / IntParams(1)
     ParmDeriv (*,3) = FunValues(*) * NormDeltaY (*) / IntParams(1)

;     ParmDeriv (*,4) = 1.0
;     Parmderiv (*,5) = Xvec (*)
;     Parmderiv (*,6) = Yvec (*)
  endif

;FunValues (*) = FunValues (*)   +                             $
;                (XVec(*) * Params(5))  +                      $
;                (YVec(*) * Params(6))  +  Params(4)

Return, FunValues (*)
End











;====================================================================================
;====================================================================================







pro PIA_SM_2D_NLLSQ, Xi, Yi, Zd, Parms, sigmas, chisq, Yfit, LastB1Params,         $
                                        FUNC_NAME=func_name,                       $
					WEIGHTS=Wts, MAX_ITERATIONS=Maxit,         $
					INFO_PRINT=info, TOLERANCE=tol
;+
; NAME:
;       Non_Lin_Lsq
; PURPOSE:
;       Non-linear least squares fit of a function of an
;       arbitrary number of parameters, to 2-D data set.
;       Function may be any non-linear function where
;       the partial derivatives are known or can be approximated.
; CATEGORY:
;       Mathematical Functions
; CALLING:
;       Non_Lin_Lsq, Xi, Yi, Zd, Parms, sigmas, FUNC_NAME="name of function"
; INPUTS:
;       Xi = vector of independent variables.
;       Zd = vector of dependent variable to be fit with func_name (Xi, Yi),
;                                                 same length as Xi.
;       Parms = vector of nterms length containing the initial estimate
;              for each parameter.  If Parms is double precision, calculations
;              are performed in double precision, otherwise in single prec.
;              The initial guess of the parameter values should be as close to
;              the actual values as possible or the solution may not converge.
; KEYWORDS:
;       FUNC_NAME = function name (string)
;              Calling mechanism should be:  F = func_name (Xi, Yi, Parms, Pderiv)
;         where:
;              F = vector of NPOINT values of function.
;              Xi = vector of NPOINT independent variables, input.
;              Yi = vector of NPOINT independent variables, input.
;              Parms = vector of NPARM function parameters, input.
;              Pderiv = array, (NPOINT, NPARM), of partial derivatives.
;                     Pderiv(I,J) = derivative of function at ith point with
;                     respect to jth parameter.  Optional output parameter.
;                     Pderiv should not be calculated if parameter is not
;                     supplied in call (Unless you want to waste some time).
;       WEIGHTS = vector of weights, same length as x and y.
;              For equal (Gaussian) weighting w(i) = 1. (this is default),
;              instrumental (Poisson) weighting w(i) = 1./y(i), etc.
;      /INFO causes Chi-Sq. to be printed each iteration,
;              INFO > 1 causes current parameter estimates to also print.
;       MAX_ITERATIONS = maximum # of gradient search iterations, default=20.
;       TOLERANCE = ratio of Chi-Sq. change to previous Chi-Sq. at which
;                     to terminate iterations ( default = 1.e-4 = 0.1% ).
; OUTPUTS:
;       Parms = vector of parameters giving best fit to the data.
; OPTIONAL OUTPUT PARAMETERS:
;       sigmas = Vector of standard deviations for parameters Parms.
;       chisq = final Chi-Sqr deviation of fit.
;       Yfit = resulting best fit to data.
; PROCEDURE:
;       Copied from "CURFIT", least squares fit to a non-linear
;       function, pages 237-239, Bevington, Data Reduction and Error
;       Analysis for the Physical Sciences.
;       "This method is the Gradient-expansion algorithm which
;       compines the best features of the gradient search with
;       the method of linearizing the fitting function."
; MODIFICATION HISTORY:
;       Written, DMS, RSI, September, 1982.
;       Modified, Frank Varosi, NASA/GSFC, 1992, to use call_function.
;-
	Nparm = N_ELEMENTS( Parms )			;# of params.
	Ndata = N_elements( Zd )
	Nfree = ( Ndata < N_ELEMENTS(Xi) ) - Nparm     ;degrees of freedom

	IF Nfree LE 0 THEN begin
		message,"not enough data points",/INFO
		return
	   endif

	DIAG = INDGEN( Nparm )*(Nparm+1)	;subscripts of diagonal elements

	if N_elements( Maxit ) NE 1 then begin
		Maxit=20
		notify=1
	  endif else notify=0

	if N_elements( tol ) NE 1 then tol = 1.e-4
	if N_elements( func_name ) NE 1 then func_name = "FUNC"
	if N_elements( Wts ) LT Ndata then Wts = replicate( 1, Ndata )
	Parms      = 1.*Parms	;make params floating
	LastParams = 1.*Parms	;make params floating
	Lambda = 0.001		;Initial lambda
	maxtry = 50

        YFit = ZD(*)

	FOR iter = 1,Maxit DO BEGIN	;Big Iteration loop

		YFIT = call_function( func_name, Xi, Yi, Parms, Pderiv )
		BETA = (Zd-YFIT)*Wts # Pderiv
		ALPHA = TRANSPOSE( Pderiv ) # (Wts # (FLTARR(Nparm)+1)*Pderiv)
		chisq1 = TOTAL( Wts*(Zd-YFIT)^2 )/Nfree   ;present chi squared.

; invert modified curvature matrix to find new parameters.

		ntry=0
		REPEAT BEGIN

			C = SQRT( ALPHA(DIAG) # ALPHA(DIAG) )
			ARRAY = ALPHA/C
			ARRAY(DIAG) = 1. + Lambda

			DeltaPara = INVERT (ARRAY , InvertStatus)/C # TRANSPOSE(BETA)

                        NormDeltaPara = DeltaPara / Parms

                        NBIGIndex = Where (Abs(NormDeltaPara) gt 0.5 , NBIG)
                        If (NBIG gt 0)  $
                           then begin
                              DeltaPara (NBIGIndex) = 0.5 * Parms (NBIGIndex)
                           endif

                        Pnew = Parms + DeltaPara

;;;;---			Pnew = Parms + INVERT (ARRAY , InvertStatus)/C # TRANSPOSE(BETA)
                        If (InvertStatus gt 0)  $
                           Then begin
                              print, ' -->>  InvertStatus = ' , InvertStatus
                           EndIf
			Yfit = call_function( func_name, Xi, Yi, Pnew )
			chisq = TOTAL( Wts * ( Zd - Yfit )^2 )/Nfree

			ntry = ntry+1

;;;;----			Lambda = (Lambda*10) < 1.e20	;assume fit got worse
                        Lambda = (Lambda*2.) < 1.e8	;assume fit got worse

		  ENDREP UNTIL (chisq LE chisq1) OR (ntry GT maxtry)

                If ((ntry le maxtry)  AND  (chisq LE chisq1))  $
                   then begin
                      LastB1Params=LastParams(*)
                      LastParams=Parms(*)
                      Parms = Pnew			;save new parameter estimate.
                   endif else begin
                      Parms = Pnew			;save new parameter estimate.
                   endelse

;;;;----		Lambda = Lambda/100		;decrease Lambda
;		Lambda = Lambda / 10.		;decrease Lambda
		Lambda = Lambda / 7.87		;decrease Lambda


		if keyword_set( info ) then begin
			PRINT,iter,"   chisqr =",chisq1,chisq
			if (info GT 1) then PRINT," parms:",Parms
		   endif

		if (chisq EQ 0) then goto,DONE
		IF (chisq1-chisq)/chisq1 LE tol then goto,DONE
	  ENDFOR

	if (notify) OR keyword_set(info) then message,"Failed to converge",/INFO

DONE:	sigmas = SQRT( ARRAY(DIAG) / ALPHA(DIAG) )		;return sigma's

RETURN
END









pro PIA_SM_1D_NLLSQ, Xi, Yd, Parms, sigmas, chisq, Yfit, FUNC_NAME=func_name,  $
					WEIGHTS=Wts, MAX_ITERATIONS=Maxit, $
					INFO_PRINT=info, TOLERANCE=tol
;+
; NAME:
;       Non_Lin_Lsq
; PURPOSE:
;       Non-linear least squares fit of a function of an
;       arbitrary number of parameters, to 1-D data set.
;       Function may be any non-linear function where
;       the partial derivatives are known or can be approximated.
; CATEGORY:
;       Mathematical Functions
; CALLING:
;       Non_Lin_Lsq, Xi, Yd, Parms, sigmas, FUNC_NAME="name of function"
; INPUTS:
;       Xi = vector of independent variables.
;       Yd = vector of dependent variable to be fit with func_name( Xi ),
;                                                 same length as Xi.
;       Parms = vector of nterms length containing the initial estimate
;              for each parameter.  If Parms is double precision, calculations
;              are performed in double precision, otherwise in single prec.
;              The initial guess of the parameter values should be as close to
;              the actual values as possible or the solution may not converge.
; KEYWORDS:
;       FUNC_NAME = function name (string)
;              Calling mechanism should be:  F = func_name( Xi, Parms, Pderiv )
;         where:
;              F = vector of NPOINT values of function.
;              Xi = vector of NPOINT independent variables, input.
;              Parms = vector of NPARM function parameters, input.
;              Pderiv = array, (NPOINT, NPARM), of partial derivatives.
;                     Pderiv(I,J) = derivative of function at ith point with
;                     respect to jth parameter.  Optional output parameter.
;                     Pderiv should not be calculated if parameter is not
;                     supplied in call (Unless you want to waste some time).
;       WEIGHTS = vector of weights, same length as x and y.
;              For equal (Gaussian) weighting w(i) = 1. (this is default),
;              instrumental (Poisson) weighting w(i) = 1./y(i), etc.
;      /INFO causes Chi-Sq. to be printed each iteration,
;              INFO > 1 causes current parameter estimates to also print.
;       MAX_ITERATIONS = maximum # of gradient search iterations, default=20.
;       TOLERANCE = ratio of Chi-Sq. change to previous Chi-Sq. at which
;                     to terminate iterations ( default = 1.e-4 = 0.1% ).
; OUTPUTS:
;       Parms = vector of parameters giving best fit to the data.
; OPTIONAL OUTPUT PARAMETERS:
;       sigmas = Vector of standard deviations for parameters Parms.
;       chisq = final Chi-Sqr deviation of fit.
;       Yfit = resulting best fit to data.
; PROCEDURE:
;       Copied from "CURFIT", least squares fit to a non-linear
;       function, pages 237-239, Bevington, Data Reduction and Error
;       Analysis for the Physical Sciences.
;       "This method is the Gradient-expansion algorithm which
;       compines the best features of the gradient search with
;       the method of linearizing the fitting function."
; MODIFICATION HISTORY:
;       Written, DMS, RSI, September, 1982.
;       Modified, Frank Varosi, NASA/GSFC, 1992, to use call_function.
;-
	Nparm = N_ELEMENTS( Parms )			;# of params.
	Ndata = N_elements( Yd )
	Nfree = ( Ndata < N_ELEMENTS(Xi) ) - Nparm     ;degrees of freedom

	IF Nfree LE 0 THEN begin
		message,"not enough data points",/INFO
		return
	   endif

	DIAG = INDGEN( Nparm )*(Nparm+1)	;subscripts of diagonal elements

	if N_elements( Maxit ) NE 1 then begin
		Maxit=20
		notify=1
	  endif else notify=0

	if N_elements( tol ) NE 1 then tol = 1.e-4
	if N_elements( func_name ) NE 1 then func_name = "FUNC"
	if N_elements( Wts ) LT Ndata then Wts = replicate( 1, Ndata )
	Parms = 1.*Parms	;make params floating
	Lambda = 0.001		;Initial lambda
	maxtry = 20

	FOR iter = 1,Maxit DO BEGIN	;Big Iteration loop

		YFIT = call_function( func_name, Xi, Parms, Pderiv )
		BETA = (Yd-YFIT)*Wts # Pderiv
		ALPHA = TRANSPOSE( Pderiv ) # (Wts # (FLTARR(Nparm)+1)*Pderiv)
		ntry=0
		chisq1 = TOTAL( Wts*(Yd-YFIT)^2 )/Nfree   ;present chi squared.

; invert modified curvature matrix to find new parameters.

		REPEAT BEGIN
			C = SQRT( ALPHA(DIAG) # ALPHA(DIAG) )
			ARRAY = ALPHA/C
			ARRAY(DIAG) = 1.+Lambda
			Pnew = Parms + INVERT( ARRAY )/C # TRANSPOSE( BETA )
			Yfit = call_function( func_name, Xi, Pnew )
			chisq = TOTAL( Wts * ( Yd - Yfit )^2 )/Nfree
			ntry = ntry+1
;;;;----			Lambda = (Lambda*10) < 1.e20	;assume fit got worse
			Lambda = (Lambda*10) < 1.e8	;assume fit got worse

		  ENDREP UNTIL (chisq LE chisq1) OR (ntry GT maxtry)

;;;;----		Lambda = Lambda/100		;decrease Lambda
		Lambda = Lambda / 10.		;decrease Lambda
		Parms=Pnew			;save new parameter estimate.

		if keyword_set( info ) then begin
			PRINT,iter,"   chisqr =",chisq1,chisq
			if (info GT 1) then PRINT," parms:",Parms
		   endif

		if (chisq EQ 0) then goto,DONE
		IF (chisq1-chisq)/chisq1 LE tol then goto,DONE
	  ENDFOR

	if (notify) OR keyword_set(info) then message,"Failed to converge",/INFO

DONE:	sigmas = SQRT( ARRAY(DIAG) / ALPHA(DIAG) )		;return sigma's

RETURN
END







;====================================================================================
;====================================================================================








Pro    PIA_SM_PropIndicsToWinds,     PropertyIndexVec,   $
                             StartWindows, EndWindows, NWindows
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


NWindows = 0

If (PropertyIndexVec(0) lt 0)  then  Return

NIndices  = N_Elements (PropertyIndexVec)

If (NIndices eq 1)  $
   then begin
        StartWindows           = PropertyIndexVec (0)
        EndWindows             = PropertyIndexVec (0)
        NWindows = 1
        Return
   endif

If (NIndices eq 2)  $
   then begin
        StartWindows           = PropertyIndexVec (0)
        EndWindows             = PropertyIndexVec (1)
        NWindows = 1
        Return
   endif

LstInd    = NIndices - 1
LstInd1   = NIndices - 2

;;PropIndex = IntArr (NIndices)
Shift1Vec = IntArr (NIndices)
Shift2Vec = IntArr (NIndices)

PropIndex = PropertyIndexVec (Sort (PropertyIndexVec(*)))



Shift1Vec (0)          = +999
Shift1Vec (1:LstInd)   = PropIndex (1:LstInd)  -    $
                         PropIndex (0:LstInd1) - 1

Shift2Vec (LstInd)     = +999
Shift2Vec (0:LstInd1)  = PropIndex (1:LstInd)  -    $
                         PropIndex (0:LstInd1) - 1

StartWindows           = PropIndex (Where (Shift1Vec gt 0))
EndWindows             = PropIndex (Where (Shift2Vec gt 0))

NWindows               = N_Elements (StartWindows)

Return
End










Pro  PIA_SM_RunningSigma,   NoiseData, RunSigma, FilterWindow,  $
                            Robust=Robust, Zero=Zero
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData = N_Elements (NoiseData)
RunSigma = 0.0 * NoiseData (*)

IF (NData LT FilterWindow)  $
   THEN BEGIN
      If (Keyword_set (Robust))  $
         then begin
            SigmaValue = PIA_SM_Robust_Sigma (NoiseData)
         endif else begin
            PIA_SM_MeanSigma,   NoiseData , Mean , SigmaValue
         endelse
      RunSigma (*) = SigmaValue
      return
   ENDIF


If (FilterWindow le 0)  then FilterWindow = (30 < NData)

FilterWindow2 = FilterWindow / 2

For i = FilterWindow2 , NData-1-FilterWindow2 , 1  do begin

   Data = NoiseData ((i-FilterWindow2) : (i+FilterWindow2))

   If (Keyword_set (Robust))  $
      then begin
         SigmaValue = PIA_SM_Robust_Sigma (Data)
      endif else begin
         PIA_SM_MeanSigma,   Data , Mean , SigmaValue
      endelse

   If (i eq FilterWindow2)    $
      then  RunSigma (0:FilterWindow2) = SigmaValue

   If (i eq (NData-1-FilterWindow2))  $
      then  RunSigma ((NData-1-FilterWindow2):(NData-1)) = SigmaValue

   RunSigma (i) = SigmaValue

EndFor

Return
End











Pro   PIA_SM_FindFit1DSrc,  Data, GoodMask, DetectThresh,                     $
                            PeakVec, SigmaVec, FWHMVec, CentreVec,            $
                            SrcMask, FitData,                                 $
                            _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~



SrcHalfWidth = 2.0
AddHalfSize  = 35
NegClipKappa = 3.0

Params       = DblArr (5)

NData        = N_Elements (Data)
SrcMask      = IntArr (NData)
FitData      = FltArr (NData)

InternalData = Data (*)



PIA_SM_1D_OutlierRazor,  Data , ClippedData, NClipData, OutlierMask,      $
                         /POSITIVEONLY , HighThresh=DetectThresh,      $                       $
                        DEBUG=DEBUG, PLOT=PLOT, _EXTRA=EXTRA

OutlierIndex = Where (OutlierMask(*) gt 0)

;print, Outlierindex

PIA_SM_PropIndicsToWinds,   OutlierIndex, StartWindows, EndWindows, NWindows

;print, startwindows
;print, endwindows

PeakVec   = FltArr (NWindows)
SigmaVec  = FltArr (NWindows)
FWHMVec   = FltArr (NWindows)
CentreVec = FltArr (NWindows)

RevisedMean = PIA_SM_BiWeight_Mean (ClippedData , RevisedSigma , RevisedWeights)

For i = 0 , NWindows-1  do begin

;   print, ' '

   WStart = StartWindows (i)
   WEnd   = EndWindows (i)

   SrcWindowIndex = IndGen (WEnd-Wstart+1) + WStart

   If (i gt 0)  $
      then begin
         WStart = (WStart - AddHalfSize)   >   EndWindows(i-1) + 1
      endif else begin
         WStart = (WStart - AddHalfSize)   >   0
      endelse

   If (i lt NWindows-1)  $
      then begin
         WEnd = (WEnd + AddHalfSize)   <   StartWindows(i+1) - 1
      endif else begin
         WEnd = (WEnd + AddHalfSize)   <   (NData-1)
      endelse

   FitWindowIndex = IndGen (WEnd-Wstart+1) + WStart

;   Print, ' SrcWindowIndex ' , SrcWindowIndex
;   Print, ' FitWindowIndex ' , FitWindowIndex

   PeakVal = Max (InternalData(SrcWindowIndex) , PeakIndex)

   PeakIndex = PeakIndex + SrcWindowIndex (0)

   FitWindowLen = N_Elements (FitWindowIndex)
   XVec = DblArr (FitWindowLen)
   YVec = DblArr (FitWindowLen)

   XVec (*) = Double (FitWindowIndex)
   YVec (*) = Double (InternalData (FitWindowIndex))

   SrcWindowLen = N_Elements (SrcWindowIndex)

   ; /* ignore negative outliers */

   ReplaceInd = Where ((YVec (*) - RevisedMean)  lt  (-NegClipKappa * RevisedSigma) , NReplace)
   If (NReplace gt 0)  $
      then begin
         YVec (ReplaceInd) = RevisedMean
      endif

   ; /* set start parameter */

   Params (0) = Double (PeakVal)
   Params (1) = Double (SrcWindowIndex(SrcWindowLen-1) - SrcWindowIndex(0) + 1.) / 3.0D0
   Params (2) = Double (PeakIndex)
   Params (3) = 0.0
   Params (4) = 0.0

   ; /* do 1D - NonLinLSQ fit */

   PIA_SM_1D_NLLSQ,   Xvec, Yvec, Params, UncParam, ChiSq, YFitVec,  $
                  Func_Name = 'PIA_SM_Gauss1DPoly1',                    $
                  Max_Iterations=100, Tolerance=1.0E-3
;;                  Max_Iterations=200, Tolerance=0.001, /info
;;                  Max_Iterations=100, Tolerance=0.0001, info=10


   ; /* determine source width from fit */

   StartSrc = 0         >  (Long (Params (2) - (SrcHalfWidth * Params(1)) - 0.5))
   EndSrc   = (NData-1) <  (Long (Params (2) + (SrcHalfWidth * Params(1)) + 0.5))

   If (StartSrc gt EndSrc)  $
      then begin
         InternalData (PeakIndex) = InternalData (PeakIndex) - PeakVal
         print , ' +++++++++ ERROR ++++++++++++ '
         goto , NEXTPEAK
      endif


;   If (i gt 0)  $
;      then begin
;         StartSrc = StartSrc    >   (EndWindows(i-1) + 1)
;      endif else begin
;         StartSrc = StartSrc    >   0
;      endelse

;   If (i lt NWindows-1)  $
;      then begin
;         EndSrc = EndSrc   <  (StartWindows(i+1) - 1)
;      endif else begin
;         EndSrc = EndSrc   <   (NData-1)
;      endelse


;   StartSrc = (Long(Params(2)) - 1) > 0
;   EndSrc   = (Long(Params(2)) + 1) <   (NData-1)


   FWHM = 2.0 * sqrt(2.0 * alog(2.0)) * Params (1)

   PeakVec (i)   = Params(0)
   SigmaVec (i)  = Params(1)
   FWHMVec (i)   = FWHM
   CentreVec (i) = Params(2)


   XVec = Double (IndGen (EndSrc-StartSrc+1)  +  StartSrc)
   FitModel = PIA_SM_Gauss1DPoly1 (XVec , Params)

   InternalData (StartSrc:EndSrc) = InternalData (StartSrc:EndSrc) - FitModel

   FitData (StartSrc:EndSrc) = FitModel (*)
   SrcMask (StartSrc:EndSrc) = 1


   XVec = Double (FitWindowIndex)
   FitModel = PIA_SM_Gauss1DPoly1 (XVec , Params)

;   print , ' StartSrc:EndSrc ' ,  StartSrc , EndSrc

;   plot, XVec , Data (FitWindowIndex)
;   oplot, Xvec , FitModel , color=4 , psym=10
;   wait , 0.5
;   cursor , vv , bb

;   plot, internaldata
;   wait , 0.5


NEXTPEAK :

EndFor

EXITLOOP:

Return
End














Pro    PIA_SM_StartPara2DFit,     Xvec , Yvec , Zvec , XStart , YStart , ZStart
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
; derive coefficient for 2D - parabola z = a + b*x + c*y + d*x*y + e*x^2 + f*y^2
;
;
SVDTolerance = 1.0E-6

NData       = N_Elements (ZVec)
LstDataInd  = NData - 1



; /* Prepare the Matrix with X/Y-Vec - Powers */

MCoeff =  6
AMatrix  = FltArr (MCOeff , NData)
For j = 0 , LstDataInd  do begin

      AMatrix (0 , j) = 1.0
      AMatrix (1 , j) = XVec (j)
      AMatrix (2 , j) = YVec (j)
      AMatrix (3 , j) = XVec (j) * YVec (j)
      AMatrix (4 , j) = XVec (j) * XVec (j)
      AMatrix (5 , j) = YVec (j) * YVec (j)

EndFor



; /* Do Singular Value Decomposition */

SVDC,  AMatrix,  Wvector, UMatrix, VMatrix


; /* Set small singular values to zero */

SVDThresh            = SVDTolerance * Max (WVector(*))
SmallIndex           = Where (WVector  le  SVDThresh , NSmall)
If (NSmall gt 0)  then  WVector (SmallIndex) = 0.0


; /* solve for the coefficients */

Coeffs = SVSOL (UMatrix, WVector, VMatrix, ZVec)

XStart =  (-Coeffs(1) + (0.5 * Coeffs(2) * Coeffs(3) / Coeffs(5)))   /    $
          ((2.0 * Coeffs(4)) - (0.5 * Coeffs(3) * Coeffs(3) / Coeffs(5)))

YStart =  (-Coeffs(2) - (XStart * Coeffs(3)))  /  (2.0 * Coeffs(5))

ZStart =  Coeffs(0) + (XStart * Coeffs(1))        + (YStart * Coeffs(2)) +     $
                      (XStart * YStart * Coeffs(3)) +                          $
                      (XStart * XStart * Coeffs(4)) +                          $
                      (YStart * YStart * Coeffs(5))


Return
END















Pro     PIA_SM_SourceSearcher,  Flux, Mask, DetectThresh,   $
                                IndivPeaks, IndivFWHMs, IndivCentres, IndivFitData,  $
                                IndivSrcMasks, CumuSrcMask,  _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Dimen  = Size (Flux)
NPixel = Dimen (1)
NData  = Dimen (2)

FluxData = FltArr (NData)
GoodMask = IntArr (NData)

CumuSrcMask     = IntArr (NData)
CumuSrcMask (*) = 0

IndivSrcMasks  =  IntArr (NPixel , NData)
IndivFitData   =  FltArr (NPixel , NData)
IndivPeaks     =  FltArr (NPixel , NData)
IndivFWHMs     =  FltArr (NPixel , NData)
IndivCentres   =  FltArr (NPixel , NData)
IndivNoPeaks   =  IntArr (NPixel)


For  i = 0 , NPixel-1  do begin

     FluxData (*) = Flux (i,*)
     GoodMask (*) = Mask (i,*)

     PIA_SM_FindFit1DSrc,  FluxData, GoodMask, DetectThresh,       $
                           PeakVec, SigmaVec, FWHMVec, CentreVec, SrcMask, FitData

     NoPeaks = N_Elements (PeakVec)

     IndivFitData  (i,*)  = FitData (*)
     IndivSrcMasks (i,*)  = SrcMask (*)

     CumuSrcMask (*)                = CumuSrcMask (*) + SrcMask (*)

     IndivPeaks    (i,0:NoPeaks-1)  = PeakVec (*)
     IndivCentres  (i,0:NoPeaks-1)  = CentreVec (*)
     IndivFWHMs    (i,0:NoPeaks-1)  = FWHMVec (*)

     IndivNoPeaks (i)               = NoPeaks

EndFor

Return
End






;====================================================================================
;====================================================================================








Pro  PIA_SM_SourceEstimator,   TotalFlux, FluxErr, DiffFlux, BackFlux, GoodMask, CumuSrcMask,      $
                               RAPosit, DecPosit, RollAngle, RASubPixel, DecSubPixel,     $
                               Fit1DResults, Peaks, Sigmas, FWHMs, Centres,               $
                               GlobalNoise, Fit2DInputs, Fit2DResults,                    $
                               SrcParams, UncSrcParams,                                   $
                               StartWinds, EndWinds, FitStartWinds, FitEndWinds, NWinds,  $
                               UsePixelIndex, ChiSqVec, NoFitPointsVec, Distances,        $
                               Ridge=Ridge,                                 $
                               Gauss2DPlane=Gauss2DPlane, RGauss2DPlane=RGauss2DPlane,    $
                               CGauss2DPlane=CGauss2DPlane, FGauss2DPlane=FGauss2DPlane,  $
                               SubPixel=SubPixel, PLOT=PLOT, DEBUG=DEBUG, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~




Dimen   = Size (DiffFlux)
NPixels = Dimen (1)
NData   = Dimen (2)

AddHalfSize  = 35

NegClipKappa = 3.0

Fit2DTolerance =  1.0E-6

MaxNoiseSigma =  Max (GlobalNoise)


InternalDebug = 0
If (Keyword_set (Debug))  $
   then begin
      InternalDebug = 1
   endif


;  /* select the fitting function */


InitialFunctionName = 'PIA_SM_CGauss2DPlane'
NInitialParams = 7 ; Gauss2DPlane
InitialParams  = FltArr (NInitialParams)


If (Keyword_Set (Gauss2DPlane))  $
   then begin
      NParams = 8   ; Gauss2DPlane
      FunctionName = 'PIA_SM_Gauss2DPlane'
      If (Keyword_Set (SubPixel))  then   FunctionName = 'PIA_SM_SPGauss2DPlane'
      ParamIndex =  [0,1,2,3,4,5,6,7]
   endif

If (Keyword_Set (CGauss2DPlane))  $
   then begin
      NParams = 7   ; CGauss2DPlane
      FunctionName = 'PIA_SM_CGauss2DPlane'
      If (Keyword_Set (SubPixel))  then   FunctionName = 'PIA_SM_SPCGauss2DPlane'
      ParamIndex =  [0,1,1,2,3,4,5,6]
   endif

If (Keyword_Set (FGauss2DPlane))  $
   then begin
      NParams = 6   ; FGauss2DPlane
      FunctionName = 'PIA_SM_FGauss2DPlane'
      If (Keyword_Set (SubPixel))  then   FunctionName = 'PIA_SM_SPFGauss2DPlane'
      ParamIndex =  [0,0,0,1,2,3,4,5]
   endif

If (Keyword_Set (RGauss2DPlane))  $
   then begin
      NParams = 9   ; RGauss2DPlane
      FunctionName = 'PIA_SM_RGauss2DPlane'
   endif

If (Keyword_Set (Ridge))  $
   then begin
      NParams = 10   ; Ridge
      FunctionName = 'PIA_SM_Ridge2D'
   endif



SrcIndices  = Where (CumuSrcMask gt 0)
PIA_SM_PropIndicsToWinds,   SrcIndices, StartWinds, EndWinds, NWinds

If (NWinds le 0)  $
   then begin
      Return
   endif



OrigStartWinds =  StartWinds (*)
OrigEndWinds   =  EndWinds (*)

FitStartWinds =  StartWinds (*)
FitEndWinds   =  EndWinds (*)

;  /* ... for all sources */

For i = 0 , NWinds-1  do begin

   SrcStart = StartWinds (i)
   SrcEnd   = EndWinds (i)

;  /* determine the fit windows */

   If (i gt 0)  $
      then begin
         SrcStart = (SrcStart - AddHalfSize)   >   OrigEndWinds(i-1) + 2
      endif else begin
         SrcStart = (SrcStart - AddHalfSize)   >   0
      endelse

   If (i lt NWinds-1)  $
      then begin
         SrcEnd = (SrcEnd + AddHalfSize)   <   OrigStartWinds(i+1) - 2
      endif else begin
         SrcEnd = (SrcEnd + AddHalfSize)   <   (NData-1)
      endelse

   IF (i EQ 0)  $
      then BEGIN
         MaxWindLen = SrcEnd - SrcStart + 1
      ENDIF ELSE BEGIN
         MaxWindLen = max ([MaxWindLen , (SrcEnd - SrcStart + 1)])
      ENDELSE

   FitStartWinds (i)              = SrcStart
   FitEndWinds (i)                = SrcEnd
ENDFOR


NMaxParams =  8

NoFitPointsVec = FltArr (NWinds)

Fit2DResults   = FltArr (NWinds , 4*MaxWindLen)
Fit2DInputs    = FltArr (NWinds , 4*MaxWindLen)

ChiSqVec       = FltArr (NWinds)
Distances      = FltArr (NWinds)

SrcParams      = FltArr (NWinds , NMaxParams)
UncSrcParams   = FltArr (NWinds , NMaxParams)

Distances      = FltArr (NWinds)
Distances (*)  = -999.

Params       = FltArr (NParams)

;;>>ParabolaPos =   FltArr (NWinds , 2)



;  /* select the pixels for fitting */

PIA_SM_SelectDetectPixels,   GlobalNoise, UsePixelIndex


If (InternalDebug eq 1)  $
   then begin
      print, ' <pia_sm_sourceestimator> FunctionName = ' , FunctionName
      print, ' <pia_sm_sourceestimator> GlobalNoise = ' , GlobalNoise(*)
      print, ' <pia_sm_sourceestimator> UsePixelIndex = ' , Usepixelindex(*)
   endif



;  /* ... for all sources */

For i = 0 , NWinds-1  do begin

   SrcFitStart = FitStartWinds (i)
   SrcFitEnd   = FitEndWinds (i)

   SrcStart = StartWinds (i)
   SrcEnd   = EndWinds (i)

   SrcCentre =  Long (0.5 * (OrigStartWinds(i) + OrigEndWinds(i)))


;  /* get the pixel / subpixel data */

   PIA_SM_CopySrcData,   RAPosit, DecPosit, TotalFlux, DiffFlux,              $
                         FluxErr, BackFlux, UsePixelIndex,                    $
                         SrcStart, SrcEnd, SrcFitStart, SrcFitEnd,            $
                         XVec, YVec, ZVec, ZZVec, UncZVec, TotVec,           $
                         BackVec, WeightVec, XYZLen

   If (Keyword_Set (SubPixel)) $
      then begin
      	 PIA_SM_CopySubPixelSrcData,   RAPosit, DecPosit, RASubPixel, DecSubPixel,          $
      				       TotalFlux, DiffFlux, FluxErr, BackFlux,              $
      				       UsePixelIndex,                                       $
                                       SrcStart, SrcEnd, SrcFitStart, SrcFitEnd,            $
                                       SubXVec, SubYVec, SubZVec, SubZZVec, SubUncZVec,     $
                                       SubTotVec, SubBackVec, SubWeightVec, SubXYZLen
      endif


;  /* Startposition */

    RAStartPos =  0.25 * (RAPosit(0,SrcCentre) + RAPosit(1,SrcCentre) +    $
                          RAPosit(2,SrcCentre) + RAPosit(3,SrcCentre))

    DecStartPos =  0.25 * (DecPosit(0,SrcCentre) + DecPosit(1,SrcCentre) +    $
                           DecPosit(2,SrcCentre) + DecPosit(3,SrcCentre))



   If (InternalDebug eq 1)  $
      then begin
         print, ' <pia_sm_sourceestimator> RAStartPos / DecStartPos = ' , RAStartPos , DecStartPos
      endif


;  /* ignore negative outliers */

   NegPeaksInd = Where (ZVec (*)  lt  (-NegClipKappa * MaxNoiseSigma) , NNegPeak)

   If (NNegPeak gt 0)  $
      then  begin
         ZVec (NegPeaksInd) = 0.0
      endif

;  /* not enough data points */

   If (SrcFitEnd - SrcFitStart  LT  (NParams+1))  $
      then begin
         Params (*) = -999.9

         Fit2DInputs (i,0:xyzlen-1)  = ZVec (*)
         Fit2DResults (i,0:xyzlen-1) = ZVec (*)

         SrcParams (i,*)             = Params (ParamIndex)
         UncSrcParams (i,*)          = Params (ParamIndex)

         ChiSqVec (i)                = -9999.
         NoFitPointsVec (i)          = -XYZLen

         StartWinds (i)              = SrcStart
         EndWinds (i)                = SrcEnd

         print, ' <pia_sm_sourceestimator> Too few pixels for 2D fit ... '
         goto, NEXTPEAK
      endif


; /* set start params for 2D fit */

   If (Keyword_Set (Gauss2DPlane))  $
      then begin
      	 Params (0) = Max (ZZVec(*) , MaxInd)

      	 TmpZVec    = ZZVec (Sort (ZZVec))
      	 Params (0) = TmpZVec (XYZLen-1)

      	 Params (1) = 2.23D0/60.
      	 Params (2) = 2.33D0/60.

      	 Params (3) = RAStartPos
      	 Params (4) = DecStartPos

      	 Params (5) = 0.0001
      	 Params (6) = 0.0001
      	 Params (7) = 0.0001
      endif

   If (Keyword_Set (CGauss2DPlane))  $
      then begin
      	 Params (0) = Max (ZZVec(*) , MaxInd)

      	 TmpZVec    = ZZVec (Sort (ZZVec))
      	 Params (0) = TmpZVec (XYZLen-1)

      	 Params (1) = 2.33D0/60.

      	 Params (2) = RAStartPos
      	 Params (3) = DecStartPos

      	 Params (4) = 0.0001
      	 Params (5) = 0.0001
      	 Params (6) = 0.0001
      endif


   If (Keyword_Set (FGauss2DPlane))  $
      then begin
      	 Params (0) = Max (ZZVec(*) , MaxInd)

      	 TmpZVec    = ZZVec (Sort (ZZVec))
      	 Params (0) = TmpZVec (XYZLen-1)

      	 Params (1) = RAStartPos
      	 Params (2) = DecStartPos

      	 Params (3) = 0.0001
      	 Params (4) = 0.0001
      	 Params (5) = 0.0001
      endif

   If (Keyword_Set (RGauss2DPlane))  $
      then begin
      	 Params (0) = Max (ZZVec(*) , MaxInd)

      	 TmpZVec    = ZZVec (Sort (ZZVec))
      	 Params (0) = TmpZVec (XYZLen-1)
      	 Params (1) = 2.13D0/60.
      	 Params (2) = 2.23D0/60.

      	 Params (3) = XVec(MaxInd)
      	 Params (4) = YVec(MaxInd)

         Params (5) = 0.2
      	 Params (6) = 0.0001
      	 Params (7) = 0.0001
      	 Params (8) = 0.0001
      endif

   If (Keyword_Set (Ridge))  $
      then begin
      	 Params (0) = Max (ZZVec(*), MaxInd)
      	 TmpZVec    = ZZVec (Sort (ZZVec))
      	 Params (0) = TmpZVec (XYZLen-1)
      	 Params (1) = 0.1
      	 Params (2) = 0.0
      	 Params (3) = 2.5D0/60.
      	 Params (4) = XVec(MaxInd)
      	 Params (5) = YVec(MaxInd)
      	 Params (6) = 1.0
      	 Params (7) = 0.0001
      	 Params (8) = 0.0001
      	 Params (9) = 0.0001
      endif



;  /* incorrect start params */

   If (Params (0)  EQ  0.0)  $
      then begin
         Params (*) = -999.9

         Fit2DInputs (i,0:xyzlen-1)  = ZVec (*)
         Fit2DResults (i,0:xyzlen-1) = ZVec (*)

         SrcParams (i,*)             = Params (ParamIndex)
         UncSrcParams (i,*)          = Params (ParamIndex)

         ChiSqVec (i)                = -9999.
         NoFitPointsVec (i)          = -XYZLen

         StartWinds (i)              = SrcStart
         EndWinds (i)                = SrcEnd

         print, ' <pia_sm_sourceestimator> useless start params ... '
         goto, NEXTPEAK
      endif





   If (InternalDebug eq 1)  $
      then begin
         print, ' <pia_sm_sourceestimator> start params for 2D fit = ' , params (*)
      endif






; /* Initial simple fit to determine good start params */


InitialParams (0) = Max (ZZVec(*) , MaxInd)

TmpZVec    = ZZVec (Sort (ZZVec))
InitialParams (0) = TmpZVec (XYZLen-1)

InitialParams (1) = 2.29D0/60.

InitialParams (2) = RAStartPos
InitialParams (3) = DecStartPos

InitialParams (4) = 0.00001
InitialParams (5) = 0.00001
InitialParams (6) = 0.00001


ZInitVec = call_function (InitialFunctionName , XVec, YVec, InitialParams)
ZFitVec = ZInitVec (*)

NaNIndex = WHERENAN (InitialParams , NoNaNs)
If (NoNaNs gt 0)  $
   then begin
      Print, ' <pia_sm_sourceestimator> NaNs in INITIALPARAMS detected !! '
      Print, ' <pia_sm_sourceestimator> skipping that source ...  '

      Fit2DResults (i , 0:xyzlen-1)  =  0.0
      Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

      goto,  NEXTPEAK
   endif

NaNIndex = WHERENAN (ZInitVec , NoNaNs)
If (NoNaNs gt 0)  $
   then begin
      Print, ' <pia_sm_sourceestimator> NaNs in ZINITVEC detected !! '
      Print, ' <pia_sm_sourceestimator> skipping that source ...  '

      Fit2DResults (i , 0:xyzlen-1)  =  0.0
      Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

      goto,  NEXTPEAK
   endif


PIA_SM_2D_NLLSQ,   XVec, YVec, ZVec, InitialParams, InitialUncSrcParams,       $
                      FinalChiSq, ZFitVec, LastParams,               $
                      WEIGHTS=WeightVec, Func_Name=InitialFunctionName,     $
                      Max_Iterations=100, Tolerance=Fit2DTolerance


; /* results of initial fit will be start params for final fitting */

NaNIndex = WHERENAN (Params , NoNaNs)
If (NoNaNs gt 0) $
   then begin
         Print, ' <pia_sm_sourceestimator> Source ' , i , ';   NaNs detected !! '
         Print, ' <pia_sm_sourceestimator> Last Minimum Params : ' , Lastparams
         Print, ' <pia_sm_sourceestimator> Resulting Fit Params : ' , Initialparams
         Print, ' <pia_sm_sourceestimator> trying new start parameters '
   ENDIF ELSE BEGIN
      If (Keyword_Set (CGauss2DPlane))  $
         then begin
            Params  = InitialParams (*)
         endif

      If (Keyword_Set (FGauss2DPlane))  $
         then begin
            Params (0) = InitialParams (0)

            Params (1) = InitialParams (2)
            Params (2) = InitialParams (3)

            Params (3) = InitialParams (4)
            Params (4) = InitialParams (5)
            Params (5) = InitialParams (6)
         endif

      If (Keyword_Set (Gauss2DPlane))  $
         then begin
            Params (0) = 0.8 * InitialParams (0)

            Params (1) = 1.11 * InitialParams (1)
            Params (2) = 1.11 * InitialParams (1)

            Params (3) = InitialParams (2)
            Params (4) = InitialParams (3)

            Params (5) = InitialParams (4)
            Params (6) = InitialParams (5)
            Params (7) = InitialParams (6)
         endif

   ENDELSE


; /* arrays for subpixel fitting */

If (Keyword_Set (SubPixel)) $
   then begin
      XVec      =  SubXVec
      YVec      =  SubYVec
      ZVec      =  SubZVec
      ZZVec     =  SubZZVec
      UncZVec   =  SubUncZVec
      BackVec   =  SubBackVec
      WeightVec =  SubWeightVec
      XYZLen    =  SubXYZLen
   endif


; /* do the fitting for each source */

   FitIter = -1

REDOFIT:


   StartParams = Params (*)


   ZInitVec = call_function (FunctionName , XVec, YVec, StartParams)
   ZFitVec = ZInitVec (*)

   NaNIndex = WHERENAN (StartParams , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_sourceestimator> NaNs in INITPARAMS detected !! '
         Print, ' <pia_sm_sourceestimator> skipping that source ...  '
         goto,  NEXTPEAK
      endif

   NaNIndex = WHERENAN (ZInitVec , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_sourceestimator> NaNs in ZINITVEC detected !! '
         Print, ' <pia_sm_sourceestimator> skipping that source ...  '
         goto,  NEXTPEAK
      endif


   If (Keyword_Set (PLOT))  $
      then begin
         Plot, Zvec, Color=1, psym=10 , /xst , /yst
         OPlot, ZInitvec, Color=5, psym=10
         wait, 1
      endif



   If (InternalDebug eq 1)  $
      then begin
         print , '  '
         print , ' <pia_sm_sourceestimator> new source -------------------------- iter = ' , fititer
      endif



   PIA_SM_2D_NLLSQ,   XVec, YVec, ZVec, Params, UncParams,           $
                      FinalChiSq, ZFitVec, LastParams,               $
                      WEIGHTS=WeightVec, Func_Name=FunctionName,     $
                      Max_Iterations=200, Tolerance=Fit2DTolerance
;;;                      Max_Iterations=200, Tolerance=Fit2DTolerance, /info

   NaNIndex = WHERENAN (Params , NoNaNs)
   If ((NoNaNs gt 0)  and  (FitIter le 10)) $
      then begin

         Print, ' <pia_sm_sourceestimator> Source ' , i , ';   NaNs detected !! '
         Print, ' <pia_sm_sourceestimator> Last Minimum Params : ' , Lastparams
         Print, ' <pia_sm_sourceestimator> Resulting Fit Params : ' , Params
         Print, ' <pia_sm_sourceestimator> trying new start parameters '


      	 If (Keyword_Set (Gauss2DPlane))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
               Params (1) = 1.1D0/60. + (Float(FitIter+1)/10.)
               Params (2) = 1.2D0/60. + (Float(FitIter+1)/10.)

      	       Params (3) = XVec(MaxInd)
      	       Params (4) = YVec(MaxInd)

      	       Params (5) = 0.0
      	       Params (6) = 0.0
      	       Params (7) = 0.0
      	    endif

      	 If (Keyword_Set (RGauss2DPlane))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
               Params (1) = 1.1D0/60. + (Float(FitIter+1)/10.)
               Params (2) = 1.2D0/60. + (Float(FitIter+1)/10.)

      	       Params (3) = XVec(MaxInd)
      	       Params (4) = YVec(MaxInd)

      	       Params (5) = 0.2
      	       Params (6) = 0.0
      	       Params (7) = 0.0
      	       Params (8) = 0.0
      	    endif

      	 If (Keyword_Set (Ridge))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
      	       Params (1) = 0.1
      	       Params (2) = 0.0
      	       Params (3) = 2.5D0/60.
      	       Params (4) = XVec(MaxInd)
      	       Params (5) = YVec(MaxInd)
      	       Params (6) = 1.0
      	       Params (7) = 0.0
      	       Params (8) = 0.0
      	       Params (9) = 0.0
      	    endif

         Print, ' <pia_sm_sourceestimator> New Start Params ' , params

         FitIter = FitIter + 1
         If (FitIter le 10)  then  goto,   REDOFIT
      endif

   If ((NoNaNs gt 0)  and  (FitIter gt 10)) $
      then begin
         Params (NaNIndex)  = -999.9
         SrcParams (i,*)    = -999.9
         UncSrcParams (i,*) = -999.9

         Fit2DResults (i , 0:xyzlen-1)  =  0.0
         Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)
      endif




   FitIter = -1

REDOFIT2:

   StartParams = Params (*)

   Params (0) = 1.07 * Params (0)
   If (Keyword_Set (CGauss2DPlane))  THEN  Params (1) = 1.07 * Params (1)
   If (Keyword_Set (Gauss2DPlane))  THEN  Params (2) = 1.13 * Params (2)

   ZInitVec = call_function (FunctionName , XVec, YVec, Params)
   ZFitVec = ZInitVec (*)

   NaNIndex = WHERENAN (StartParams , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_sourceestimator> NaNs in INITPARAMS detected !! '
         Print, ' <pia_sm_sourceestimator> skipping that source ...  '
         goto,  NEXTPEAK
      endif

   NaNIndex = WHERENAN (ZInitVec , NoNaNs)
   If (NoNaNs gt 0)  $
      then begin
         Print, ' <pia_sm_sourceestimator> NaNs in ZINITVEC detected !! '
         Print, ' <pia_sm_sourceestimator> skipping that source ...  '
         goto,  NEXTPEAK
      endif

   If (Keyword_Set (PLOT))  $
      then begin
         OPlot, ZInitvec, Color=8, psym=10
      endif


   PIA_SM_2D_NLLSQ,   XVec, YVec, ZVec, Params, UncParams,       $
                      FinalChiSq, ZFitVec, LastParams,           $
                      WEIGHTS=WeightVec, Func_Name=FunctionName, $
                      Max_Iterations=200, Tolerance=Fit2DTolerance
;;                      Max_Iterations=200, Tolerance=Fit2DTolerance,  /info


   NaNIndex = WHERENAN (Params , NoNaNs)
   If ((NoNaNs gt 0)  and  (FitIter le 10)) $
      then begin

         Print, ' <pia_sm_sourceestimator> Source ' , i , ';   NaNs detected !! '
         Print, ' <pia_sm_sourceestimator> Last Minimum Params : ' , Lastparams
         Print, ' <pia_sm_sourceestimator> Resulting Fit Params : ' , Params
         Print, ' <pia_sm_sourceestimator> trying new start parameters '

      	 If (Keyword_Set (Gauss2DPlane))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
               Params (1) = 1.17D0/60. + (Float(FitIter+1)/10.)
               Params (2) = 1.27D0/60. + (Float(FitIter+1)/10.)

      	       Params (3) = XVec(MaxInd)
      	       Params (4) = YVec(MaxInd)

      	       Params (5) = 0.0
      	       Params (6) = 0.0
      	       Params (7) = 0.0
      	    endif

      	 If (Keyword_Set (RGauss2DPlane))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
               Params (1) = 1.17D0/60. + (Float(FitIter+1)/10.)
               Params (2) = 1.27D0/60. + (Float(FitIter+1)/10.)

      	       Params (3) = XVec(MaxInd)
      	       Params (4) = YVec(MaxInd)

      	       Params (5) = 0.2
      	       Params (6) = 0.0
      	       Params (7) = 0.0
      	       Params (8) = 0.0
      	    endif

      	 If (Keyword_Set (Ridge))  $
      	    then begin
      	       TmpZVec    = ZVec (Sort (ZVec))
      	       Params (0) = TmpZVec (XYZLen-1)
      	       Params (1) = 0.1
      	       Params (2) = 0.0
      	       Params (3) = 2.5D0/60.
      	       Params (4) = XVec(MaxInd)
      	       Params (5) = YVec(MaxInd)
      	       Params (6) = 1.0
      	       Params (7) = 0.0
      	       Params (8) = 0.0
      	       Params (9) = 0.0
      	    endif

         Print, ' <pia_sm_sourceestimator> New Start Params ' , params

         FitIter = FitIter + 1
         If (FitIter le 10)  then  goto,   REDOFIT2
      endif

   If ((NoNaNs gt 0)  and  (FitIter gt 10)) $
      then begin
         Params (NaNIndex)  = -999.9
         SrcParams (i,*)    = -999.9
         UncSrcParams (i,*) = -999.9

         ZFitVec = ZInitVec (*)

         Fit2DResults (i , 0:xyzlen-1)  =  0.0
         Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

      endif




   If (InternalDebug eq 1)  $
      then begin
         print , '  '
         Print, ' <pia_sm_sourceestimator> @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ '
         Print, ' <pia_sm_sourceestimator> Final Params '
         Print, ' <pia_sm_sourceestimator> @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ '
         Print, Params (*)
         Print, UncParams (*)
         Print, ' <pia_sm_sourceestimator> @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ '
      endif



;  /* copy final FitParams to Array */

   If (Keyword_Set (Gauss2DPlane))  $
      then begin
         SrcParams (i , *)    =  Params (*)

         UncSrcParams (i , *) =  UncParams (*)

         Fit2DResults (i , 0:xyzlen-1)  =  ZFitVec (*)
         Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

         ChiSqVec  (i)                  =  FinalChiSq
         NoFitPointsVec (i)             = XYZLen
      endif


   If (Keyword_Set (CGauss2DPlane))  $
      then begin
         SrcParams (i , 0)    =  Params (0)
         SrcParams (i , 1)    =  Params(1)
         SrcParams (i , 2:NMaxParams-1)    =  Params (1:*)

         UncSrcParams (i , 0) =  UncParams (0)
         UncSrcParams (i , 1) =  UncParams(1)
         UncSrcParams (i , 2:NMaxParams-1) =  UncParams (1:*)

         Fit2DResults (i , 0:xyzlen-1)  =  ZFitVec (*)
         Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

         ChiSqVec  (i)                  =  FinalChiSq
         NoFitPointsVec (i)             = XYZLen
      endif


   If (Keyword_Set (FGauss2DPlane))  $
      then begin
         SrcParams (i , 0)    =  Params (0)
         SrcParams (i , 1)    =  53.0 / 3600.
         SrcParams (i , 2)    =  53.0 / 3600.
         SrcParams (i , 3:NMaxParams-1)    =  Params (1:*)

         UncSrcParams (i , 0) =  UncParams (0)
         UncSrcParams (i , 1) =  0.0
         UncSrcParams (i , 2) =  0.0
         UncSrcParams (i , 3:NMaxParams-1) =  UncParams (1:*)

         Fit2DResults (i , 0:xyzlen-1)  =  ZFitVec (*)
         Fit2DInputs (i , 0:xyzlen-1)   =  ZVec (*)

         ChiSqVec  (i)                  =  FinalChiSq
         NoFitPointsVec (i)             = XYZLen
      endif


   If (Keyword_Set (RGauss2DPlane))  $
      then begin

      endif

   If (Keyword_Set (Ridge))  $
      then begin

      endif



;;
;;    SrcParams (i,*)             = Params (*)
;;    UncSrcParams (i,*)          = UncParams (*)
;;
;;    ChiSqVec (i)                = FinalChiSq
;;    NoFitPointsVec (i)          = XYZLen
;;
;;    Fit2DInputs (i,0:xyzlen-1)  = ZVec (*)
;;    Fit2DResults (i,0:xyzlen-1) = ZFitVec (*)
;;
;;    StartWinds (i)              = SrcStart
;;    EndWinds (i)                = SrcEnd




   If (Keyword_Set (FGauss2DPlane))  $
      then BEGIN
      	 RAPos  = Params (1)
      	 DecPos = Params (2)
      endif

   If (Keyword_Set (CGauss2DPlane))  $
      then BEGIN
      	 RAPos  = Params (2)
      	 DecPos = Params (3)
      endif

   If (Keyword_Set (Gauss2DPlane))  $
      then BEGIN
      	 RAPos  = Params (3)
      	 DecPos = Params (4)
      endif



   FOR l = 0 , XYZLen-1 , 1 do BEGIN
      IF (l EQ 0) $
         THEN BEGIN
            MinDist = 60.0 * sphdist (XVec(l), YVec(l), RAPos, DecPos)
         ENDIF ELSE BEGIN
            MinDist = min ([MinDist , 60.0 * sphdist (XVec(l), YVec(l), RAPos, DecPos)])
         ENDELSE
   ENDFOR

   Distances (i) =  MinDist



   If (Keyword_Set (PLOT))  $
      then begin
         Plot, Zvec, Color=1, psym=10 , /xst , /yst
         OPlot, ZInitvec, Color=5, psym=10
         OPlot, ZFitvec, Color=3, psym=10
         wait, 1
      endif


;;>>   PIA_SM_FitParabola , Fit2DResults , i ,                         $
;;>>                          SrcFitStart , SrcFitEnd ,  RAStartPos , DecStartPos , $
;;>>                          XVec , YVec , ZVec , ParabRAPos , ParabDecPos
;;>>
;;>>   ParabolaPos (i , 0) =  ParabRAPos
;;>>   ParabolaPos (i , 1) =  ParabDecPos


   NEXTPEAK:

EndFor

Return
End












Pro  PIA_SM_SelectDetectPixels ,  DataVals, UseIndex, ThreshHold=ThreshHold
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData        = N_Elements (DataVals)
Useindex     = IntArr (NData)

SortIndex    = Sort (DataVals)
SortedVals   = DataVals (SortIndex)

Thresh        = 6.
IF (N_Elements (ThreshHold) GT 0)  THEN   Thresh = ThreshHold

UseIndex (*)  = 1

If (Keyword_set (Debug))  $
   then begin
      print, ' <PIA_SM_SelectDetectPixels> Thresh = ' , Thresh
      print, ' <PIA_SM_SelectDetectPixels> UseIndex (*) = ' , UseIndex (*)
      print, ' <PIA_SM_SelectDetectPixels> DataVals (*) = ' , DataVals (*)
      print, ' <PIA_SM_SelectDetectPixels> SortedVals (*) = ' , SortedVals (*)
   ENDIF


If (NData eq 4)  $
   then begin
      Delta1 = abs (SortedVals (3) - SortedVals (2))
      Delta2 = abs (SortedVals (2) - SortedVals (1))
      Delta3 = abs (SortedVals (1) - SortedVals (0))

      IF (Delta1 GT (Thresh * max ([Delta2 , Delta3])))  THEN  UseIndex (SortIndex(NData-1)) = 0
   endif



If (Keyword_set (Debug))  $
   then begin
      print, ' <PIA_SM_SelectDetectPixels> Thresh = ' , Thresh
      print, ' <PIA_SM_SelectDetectPixels> delta1 delta2 delta3 = ' , delta1 , delta2 , delta3
      print, ' <PIA_SM_SelectDetectPixels> UseIndex (*) = ' , UseIndex (*)
   endif



Return
End













Pro   PIA_SM_PixelCoord,   RACenter, DecCenter, RollAngle,   $
                           RAPixel, DecPixel,                $
                           RAPixelCorn, DecPixelCorn,        $
                           Rectangular=Rectangular
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


NData  = N_Elements (RACenter)
NPixel = 4

RAPixel  = DblArr (NPixel , NData)
DecPixel = DblArr (NPixel , NData)

RAPixelCorn  = DblArr (NPixel , NData , 5)
DecPixelCorn = DblArr (NPixel , NData , 5)

RelRA  = DblArr (NPixel , NData)
RelDec = DblArr (NPixel , NData)

RelCornRA  = DblArr (NPixel , NData , 5)
RelCornDec = DblArr (NPixel , NData , 5)


; /* Pixel Center Coordinates ! */

HalfLenX = 45.3            ;  [arcsec]
HalfLenY = 45.3            ;  [arcsec]

RelXCoord = [-HalfLenX , +HalfLenX , +HalfLenX , -HalfLenX]
RelYCoord = [-HalfLenY , -HalfLenY , +HalfLenY , +HalfLenY]

SinPhi   = Double (sin (RollAngle(*) * !DTOR))
CoSinPhi = Double (cos (RollAngle(*) * !DTOR))

For i = 0 , NPixel-1  do begin
   RelRA (i,*)  = - ((RelXCoord(i) * CoSinPhi(*))  -  (RelYCoord(i) *   SinPhi(*)))
   RelRA (i,*)  = RelRA (i,*) / 3600.

   RelDec (i,*) = + ((RelXCoord(i) * SinPhi(*))    +  (RelYCoord(i) * CoSinPhi(*)))
   RelDec (i,*) = RelDec (i,*) / 3600.
EndFor

For i = 0 , NPixel-1  do begin
   If (Not Keyword_set (Rectangular))  $
      then begin
         DecPixel (i,*) = DecCenter (*)  +  RelDec (i,*)
         RAPixel (i,*)  = RACenter (*)   +  RelRA (i,*) / cos (DecPixel(i,*) * !DTOR)
      endif
   If (Keyword_set (Rectangular))  $
      then begin
         DecPixel (i,*) = DecCenter (*)  +  RelDec (i,*)
         RAPixel (i,*)  = (RACenter (*) / cos (DecCenter(*) * !DTOR))  +  RelRA (i,*)
      endif
EndFor


; /* Corner Coordinates ! */

LenX = 90.6            ;  [arcsec]
LenY = 90.6            ;  [arcsec]

CornRelXCoord = DblArr (NPixel , 5)
CornRelYCoord = DblArr (NPixel , 5)

CornRelXCoord (0,*) = [0.0 , -LenX, -LenX ,  0.0  , 0.0]
CornRelYCoord (0,*) = [0.0 ,  0.0,  -LenY , -LenY , 0.0]

CornRelXCoord (1,*) = [0.0 ,  0.0,   +LenX , +LenX , 0.0]
CornRelYCoord (1,*) = [0.0 , -LenY,  -LenY ,  0.0 ,  0.0]

CornRelXCoord (2,*) = [0.0 , +LenX, +LenX ,  0.0  , 0.0]
CornRelYCoord (2,*) = [0.0 ,  0.0,  +LenY , +LenY , 0.0]

CornRelXCoord (3,*) = [0.0 ,  0.0,  -LenX , -LenX , 0.0]
CornRelYCoord (3,*) = [0.0 , +LenY, +LenY ,  0.0  , 0.0]


For j = 0 , 4  do begin

   For i = 0 , NPixel-1  do begin
      RelCornRA (i,*,j)  = - ((CornRelXCoord(i,j) * CoSinPhi(*))  -  (CornRelYCoord(i,j) *   SinPhi(*)))
      RelCornRA (i,*,j)  = RelCornRA (i,*,j) / 3600.

      RelCornDec (i,*,j) = + ((CornRelXCoord(i,j) * SinPhi(*))    +  (CornRelYCoord(i,j) * CoSinPhi(*)))
      RelCornDec (i,*,j) = RelCornDec (i,*,j) / 3600.
   EndFor

   For i = 0 , NPixel-1  do begin
      If (Not Keyword_set (Rectangular))  $
   	 then begin
   	    DecPixelCorn (i,*,j) = DecCenter (*)  +  RelCornDec (i,*,j)
   	    RAPixelCorn (i,*,j)  = RACenter (*)   +  RelCornRA (i,*,j) / cos (DecPixel(i,*) * !DTOR)
   	 endif
      If (Keyword_set (Rectangular))  $
   	 then begin
   	    DecPixelCorn (i,*,j) = DecCenter (*)  +  RelCornDec (i,*,j)
   	    RAPixelCorn (i,*,j)  = (RACenter (*) / cos (DecCenter(*) * !DTOR))  +  RelCornRA (i,*,j)
   	 endif
   EndFor

EndFor


Return
End













Pro   PIA_SM_SubPixelCoord,   RACenter, DecCenter, RollAngle,   $
                              RASubPixel, DecSubPixel,          $
                              Rectangular=Rectangular,          $
                              NSubPixel=NSubPixelPar
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

SinPhi   = Double (sin (RollAngle(*) * !DTOR))
CoSinPhi = Double (cos (RollAngle(*) * !DTOR))

NData  = N_Elements (RACenter)
NPixel = 4

NSubPixel = 9
If (N_Elements (NSubPixelPar))  then  NSubPixel = NSubPixelPar

RASubPixel   = DblArr (NPixel , NData , NSubPixel)
DecSubPixel  = DblArr (NPixel , NData , NSubPixel)

RelSubPixRA  = DblArr (NPixel , NData , NSubPixel)
RelSubPixDec = DblArr (NPixel , NData , NSubPixel)


; /* SubPixel Coordinates ! */

LenX = 90.6            ;  [arcsec]
LenY = 90.6            ;  [arcsec]

DeltaX = LenX / Float (NSubPixel)
DeltaY = LenY / Float (NSubPixel)

SubPixRelXCoord = DblArr (NPixel , NSubPixel)
SubPixRelYCoord = DblArr (NPixel , NSubPixel)

RelSubPixRA  = DblArr (NPixel , NData , NSubPixel)
RelSubPixDec = DblArr (NPixel , NData , NSubPixel)

RASubPixel  = DblArr (NPixel , NData , NSubPixel)
DecSubPixel = DblArr (NPixel , NData , NSubPixel)


For j = 0 , NSubPixel-1  do begin
   SubPixRelXCoord (0,j) = -((Double(j)+0.5)*DeltaX)
   SubPixRelXCoord (1,j) = +((Double(j)+0.5)*DeltaX)
   SubPixRelXCoord (2,j) = +((Double(j)+0.5)*DeltaX)
   SubPixRelXCoord (3,j) = -((Double(j)+0.5)*DeltaX)

   SubPixRelYCoord (0,j) = -((Double(j)+0.5)*DeltaY)
   SubPixRelYCoord (1,j) = -((Double(j)+0.5)*DeltaY)
   SubPixRelYCoord (2,j) = +((Double(j)+0.5)*DeltaY)
   SubPixRelYCoord (3,j) = +((Double(j)+0.5)*DeltaY)
EndFor


For j = 0 , NSubPixel-1  do begin
   For i = 0 , NPixel-1  do begin
      RelSubPixRA (i,*,j)  = - ((SubPixRelXCoord(i,j) * CoSinPhi(*))  -  (SubPixRelYCoord(i,j) *   SinPhi(*)))
      RelSubPixRA (i,*,j)  = RelSubPixRA (i,*,j) / 3600.

      RelSubPixDec (i,*,j) = + ((SubPixRelXCoord(i,j) * SinPhi(*))    +  (SubPixRelYCoord(i,j) * CoSinPhi(*)))
      RelSubPixDec (i,*,j) = RelSubPixDec (i,*,j) / 3600.
   EndFor
EndFor


For j = 0 , NSubPixel-1  do begin
   For i = 0 , NPixel-1  do begin
      If (Not Keyword_set (Rectangular))  $
   	 then begin
   	    DecSubPixel (i,*,j) = DecCenter (*)  +  RelSubPixDec (i,*,j)
   	    RASubPixel (i,*,j)  = RACenter (*)   +  RelSubPixRA (i,*,j) / cos (DecSubPixel(i,*) * !DTOR)
   	 endif
      If (Keyword_set (Rectangular))  $
   	 then begin
   	    DecSubPixel (i,*,j) = DecCenter (*)  +  RelSubPixDec (i,*,j)
   	    RASubPixel (i,*,j)  = (RACenter (*) / cos (DecCenter(*) * !DTOR))  +  RelSubPixRA (i,*,j)
   	 endif
   EndFor
EndFor

Return
End









Pro  PIA_SM_CopySubPixelSrcData,   RAPixPosit, DecPixPosit, RASubPixel, DecSubPixel,      $
                                   TotalFlux, DiffFlux, FluxErr, BackFlux, UsePixelIndex, $
                                   SrcStart, SrcEnd, SrcFitStart, SrcFitEnd,              $
                                   XVec, YVec, ZVec, ZZVec, UncZVec, TotVec, BackVec,     $
                                   WeightVec, VecLen
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Dimen   = Size (DiffFlux)
NPixels = Dimen (1)
NData   = Dimen (2)

Dimen   = Size (RASubPixel)
NSubPix = Dimen(3)

OffSet  = SrcFitEnd - SrcFitStart + 1
VecLen  = OffSet * NPixels

XVec = FltArr (VecLen , (NSubPix^2)+1)
YVec = FltArr (VecLen , (NSubPix^2)+1)

Weights = FltArr (OffSet)

For i = 0 , NPixels-1  do begin

;;   NewXcoord = - Transpose (RAPixPosit (i , SrcStart:SrcEnd)) *    $
;;                 cos (Transpose (DecPixPosit(i,SrcStart:SrcEnd) * !DTOR))

   NewXcoord = Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd))

;;   NewXcoord = + Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd)) /    $
;;                 cos (Transpose (DecPixPosit(i,SrcFitStart:SrcFitEnd) * !DTOR))

   NewYcoord = + Transpose (DecPixPosit (i , SrcFitStart:SrcFitEnd))


   XVec (i*Offset , 0) = NewXcoord (*)
   YVec (i*Offset , 0) = NewYcoord (*)

   XSubVec = FltArr (NsubPix^2)
   YSubVec = FltArr (NsubPix^2)

   XSubVec (*) = 0.0
   YSubVec (*) = 0.0

   For j = 0 , OffSet-1  do begin

      jj = SrcStart + j

      For k = 0 , NSubPix-1 do begin
         For l = 0 , NSubPix-1 do begin

            XSubVec (k*NSubPix+l) = RASubPixel (i , jj , k)
            YSubVec (k*NSubPix+l) = DecSubPixel (i , jj , l)

         EndFor
      EndFor

      XVec ((i*Offset)+j , 1:*) = XSubVec (*)
      YVec ((i*Offset)+j , 1:*) = YSubVec (*)

   EndFor



   FitPart = Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))
   SrcPart = Transpose (TotalFlux(i,SrcStart:SrcEnd) - BackFlux(i,SrcStart:SrcEnd))

   FitOffset = SrcStart - SrcFitStart
   SrcWidth = SrcEnd - SrcStart + 1

   FitPart (FitOffset:FitOffset+SrcWidth-1) = SrcPart (*)


   if (i eq 0)   $
      then begin
         ZVec  = [FitPart]
         ZZVec = [FitPart]
;;>>         ZVec = [Transpose (TotalFlux(i,SrcStart:SrcEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
;;>>         ZZVec = [Transpose (TotalFlux(i,SrcStart:SrcEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
      endif else begin
         ZVec  = [ZVec , FitPart]
         ZZVec = [ZZVec , FitPart]
;;>>         ZVec = [ZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
;;>>         ZZVec = [ZZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
      endelse

;;>>    if (i eq 0)   $
;;>>       then begin
;;>>          ZVec = [Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>          ZZVec = [Float(UsePixelIndex(i)) * Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>       endif else begin
;;>>          ZVec = [ZVec , Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>          ZZVec = [ZZVec , Float(UsePixelIndex(i)) * Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>       endelse

;   if (i eq 0)   $
;      then begin
;         ZVec = [Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
;      endif else begin
;         ZVec = [ZVec , Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
;      endelse

   if (i eq 0)   $
      then begin
         TotVec = [Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
         UncZVec = [Transpose (FluxErr (i , SrcFitStart:SrcFitEnd))]
         BackVec = [Transpose (BackFlux (i , SrcFitStart:SrcFitEnd))]
      endif else begin
         TotVec = [TotVec , Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
         UncZVec = [UncZVec , Transpose (FluxErr (i , SrcFitStart:SrcFitEnd))]
         BackVec = [BackVec , Transpose (BackFlux (i , SrcFitStart:SrcFitEnd))]
      endelse


   Weights (*) = 1.0
   If (UsePixelIndex (i) eq 0)  then  Weights (*) = 0.0
   if (i eq 0)   $
      then begin
         WeightVec = [Weights]
      endif else begin
         WeightVec = [WeightVec , Weights]
      endelse

EndFor



Return
End













Pro  PIA_SM_CopySrcData,   RAPixPosit, DecPixPosit,                                  $
                           TotalFlux, DiffFlux, FluxErr, BackFlux, UsePixelIndex,    $
                           SrcStart, SrcEnd, SrcFitStart, SrcFitEnd,                 $
                           XVec, YVec, ZVec, ZZVec, UncZVec, TotVec, BackVec,        $
                           WeightVec, VecLen
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Dimen   = Size (DiffFlux)
NPixels = Dimen (1)
NData   = Dimen (2)

VecLen  = NPixels * (SrcFitEnd - SrcFitStart + 1)

Weights = FltArr (SrcFitEnd - SrcFitStart + 1)

For i = 0 , Npixels-1  do begin

;;   NewXcoord = - Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd)) *    $
;;                 cos (Transpose (DecPixPosit(i,SrcFitStart:SrcFitEnd) * !DTOR))

   NewXcoord = Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd))

;;   NewXcoord = + Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd)) /    $
;;                 cos (Transpose (DecPixPosit(i,SrcFitStart:SrcFitEnd) * !DTOR))

   NewYcoord = + Transpose (DecPixPosit (i , SrcFitStart:SrcFitEnd))




   FitPart = Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))
   SrcPart = Transpose (TotalFlux(i,SrcStart:SrcEnd) - BackFlux(i,SrcStart:SrcEnd))

   FitOffset = SrcStart - SrcFitStart
   SrcWidth = SrcEnd - SrcStart + 1

   FitPart (FitOffset:FitOffset+SrcWidth-1) = SrcPart (*)

   if (i eq 0)   $
      then begin
         XVec = [NewXCoord]
;         XVec = [Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd))]
         YVec = [NewYCoord]
;         YVec = [Transpose (DecPixPosit (i , SrcFitStart:SrcFitEnd))]
      endif else begin
         XVec = [XVec , NewXCoord]
;         XVec = [XVec , Transpose (RAPixPosit (i , SrcFitStart:SrcFitEnd))]
         YVec = [YVec , NewYCoord]
;         YVec = [YVec , Transpose (DecPixPosit (i , SrcFitStart:SrcFitEnd))]
      endelse

   if (i eq 0)   $
      then begin
         ZVec  = [FitPart]
         ZZVec = [FitPart]
;;>>         ZVec = [ZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
;;>>         ZZVec = [ZZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
      endif else begin
         ZVec  = [ZVec , FitPart]
         ZZVec = [ZZVec , FitPart]
;;>>         ZVec = [ZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
;;>>         ZZVec = [ZZVec , Transpose (TotalFlux(i,SrcFitStart:SrcFitEnd) - BackFlux(i,SrcFitStart:SrcFitEnd))]
      endelse

;;>>    if (i eq 0)   $
;;>>       then begin
;;>>          ZVec = [Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>          ZZVec = [Float(UsePixelIndex(i)) * Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>       endif else begin
;;>>          ZVec = [ZVec , Transpose (DiffFlux (i , SrcStart:SrcEnd))]
;;>>          ZZVec = [ZZVec , Float(UsePixelIndex(i)) * Transpose (DiffFlux (i , SrcFitStart:SrcFitEnd))]
;;>>       endelse
;;>>

   if (i eq 0)   $
      then begin
         TotVec = [Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
         BackVec = [Transpose (BackFlux (i , SrcFitStart:SrcFitEnd))]
         UncZVec = [Transpose (FluxErr (i , SrcFitStart:SrcFitEnd))]
      endif else begin
         TotVec = [TotVec , Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
         BackVec = [BackVec , Transpose (BackFlux (i , SrcFitStart:SrcFitEnd))]
         UncZVec = [UncZVec , Transpose (FluxErr (i , SrcFitStart:SrcFitEnd))]
      endelse


;   if (i eq 0)   $
;      then begin
;         ZVec = [Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
;      endif else begin
;         ZVec = [ZVec , Transpose (TotalFlux (i , SrcFitStart:SrcFitEnd))]
;      endelse


   Weights (*) = 1.0
   If (UsePixelIndex (i) eq 0)  then  Weights (*) = 0.0
   if (i eq 0)   $
      then begin
         WeightVec = [Weights]
      endif else begin
         WeightVec = [WeightVec , Weights]
      endelse

EndFor

Return
End










Function  PIA_SM_Median,  Data
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData = N_Elements (Data)

Result = 0.0
If ((NData mod 2)  ne  0)  $
   then begin
      Result      =  Median (Data)
   endif else begin
      TmpData     =  Data (*)
      TmpData (*) =  TmpData (Sort (TmpData (*)))
      Middle      =  (NData - 1) / 2
      Result      =  0.5 * (TmpData (Middle) + TmpData (Middle+1))
   endelse

Return, Result
End








Function   PIA_SM_ReverseVector,  Vector
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData        = N_Elements (Vector)
LstDataInd   = NData - 1
Result       = FltArr (NData)

For i = 0 , LstDataInd do begin
   k = LstDataInd - i
   Result (k) = Vector (i)
EndFor

Return, Result
End





Pro    PIA_SM_RightInsert,   Vector1, Vector2
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData1       = N_Elements (Vector1)
LstData1Ind  = NData1 - 1

NData2       = N_Elements (Vector2)
LstData2Ind  = NData2 - 1

For i = 0 , LstData2Ind do begin
   k = LstData1Ind - i
   Vector1 (k) = Vector2 (i)
EndFor

Return
End















Pro  PIA_SM_Filter,   Data, Result, FilterWeights, Params=FilterParams,  $
                  BoxAve=BoxAve, BoxMed=BoxMed,                      $
                  Gauss=Gauss, Hanning=Hanning,                      $
                  Mexican=Mexican, SavGol=SavGol,                    $
                  AveMedAve=AveMedAve, BlockAve=BlockAve,            $
                  PaddConstant=PaddConstant, PaddMirror=PaddMirror,  $
                  PaddExtrapolate=PaddExtrapolate, DEBUG=DEBUG
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  General Filtering Routine
;
;  Filtering is done using the given FilterWeights - Vector
;  or with special filters called by keyword parameters and
;  the appropriate filterparams
;

; /* Initialization */

NData        = N_Elements (Data)
LstDataInd   = NData - 1

Result       = FltArr (NData)
If (N_Elements (FilterWeights) gt 0)    $
   then  begin
      FilterSize   = N_Elements (FilterWeights)
      FilterSize2  = FilterSize / 2
   endif

Method = 'general'
If (Keyword_Set (BoxAve))  $
   then begin
      Method            = 'general'
      FilterSize        = Long (FilterParams(0))   >   3
      If (FilterSize mod 2  eq  0) then FilterSize = FilterSize + 1
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      FilterWeights (*) = 1.0
   endif

If (Keyword_Set (BoxMed))  $
   then begin
      Method            = 'boxmed'
      FilterSize        = Long (FilterParams(0))   >   3
      If (FilterSize mod 2  eq  0) then FilterSize = FilterSize + 1
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      FilterWeights (*) = 1.0
   endif

If (Keyword_Set (Hanning))  $
   then begin
      Method            = 'general'
      FilterWeights     = [0.25 , 0.5 , 0.25]
      FilterSize        = N_Elements (FilterWeights)
      FilterSize2       = FilterSize / 2
   endif

If (Keyword_Set (Gauss))  $
   then begin
      Method            = 'general'
      FilterSize        = Long (FilterParams(0))
      If (FilterSize mod 2  eq  0) then FilterSize = FilterSize + 1
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      TmpX              = FIndGen (FilterSize2) + 1
      Sigma             = abs (Float (FilterParams(1)))
      If (Sigma eq 0.0)  then Sigma = 1.0
      FilterWeights     = [PIA_SM_ReverseVector (-TmpX(*)) , 0.0 , TmpX(*)] / Sigma
      FilterWeights (*) = exp (-0.5  *  FilterWeights(*)^2)
   endif

If (Keyword_Set (Mexican))  $
   then begin
      Method            = 'general'
      FilterSize        = Long (FilterParams(0))
      If (FilterSize mod 2  eq  0) then FilterSize = FilterSize + 1
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      TmpX              = FIndGen (FilterSize2) + 1
      Sigma             = abs (Float (FilterParams(1)))
      If (Sigma eq 0.0)  then Sigma = 1.0
      TmpX              = [PIA_SM_ReverseVector (-TmpX(*)) , 0.0 , TmpX(*)] / Sigma
      FilterWeights (*) = (1.0 - TmpX (*)^2)* exp (-0.5 * TmpX(*)^2)
   endif

If (Keyword_Set (SavGol))  $
   then begin
      Method            = 'savgol'
      FilterSize        = Long (FilterParams(0))   >   3
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      FilterWeights (*) = 1.0
      FilterDegree      = Long (FilterParams(1))   >   2
   endif

If (Keyword_Set (AveMedAve))  $
   then begin
      Method            = 'avemedave'
      FilterSize        = Long (FilterParams(0))
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      FilterWeights (*) = 1.0
   endif

If (Keyword_Set (BlockAve))  $
   then begin
      Method            = 'blockave'
      FilterSize        = Long (FilterParams(0))
      FilterSize2       = FilterSize / 2
      FilterWeights     = FltArr (FilterSize)
      FilterWeights (*) = 1.0
   endif


; /* Get the data padding */

If ((NOT Keyword_set (PaddConstant))    AND    (NOT Keyword_set (PaddMirror))   AND $
    (NOT Keyword_set (PaddExtrapolate)))  $
   then begin
      LeftPad      = FltArr (FilterSize)
      LeftPad (*)  = Data (0)

      RightPad     = FltArr (FilterSize)
      RightPad (*) = Data (LstDataInd)
   endif

If (Keyword_set (PaddConstant))  $
   then begin
      LeftPad      = FltArr (FilterSize)
      LeftPad (*)  = Data (0)

      RightPad     = FltArr (FilterSize)
      RightPad (*) = Data (LstDataInd)
   endif

If (Keyword_set (PaddMirror))  $
   then begin
      LeftPad      = FltArr (FilterSize)
      LeftPad (*)  = PIA_SM_ReverseVector (Data (0:FilterSize-1))

      RightPad     = FltArr (FilterSize)
      RightPad (*) = PIA_SM_ReverseVector (Data (NData-FilterSize:LstDataInd))
   endif

If (Keyword_set (PaddExtrapolate))  $
   then begin

      XVec = FindGen (2*FilterSize)
      YVec = Data (0:2*FilterSize-1)
      FitPara =  PIA_SM_ROBUST_LINEFIT (XVec , YVec)
      XVec = PIA_SM_ReverseVector (-FindGen (FilterSize) - 1.0)
      LeftPad      = FltArr (FilterSize)
      LeftPad (*)  = FitPara(0) + (FitPara(1)*XVec(*))

      XVec = PIA_SM_ReverseVector (LstDataInd - FindGen (2*FilterSize))
      YVec = Data (NData-2*FilterSize:LstDataInd)
      FitPara =  PIA_SM_ROBUST_LINEFIT (XVec , YVec)
      XVec = LstDataInd + FindGen (FilterSize) + 1.0
      RightPad     = FltArr (FilterSize)
      RightPad (*) = FitPara(0) + (FitPara(1)*XVec(*))

   endif



If (Keyword_set (Debug))  $
   then begin
      print , ' <ms_filter> , Method        = ' , Method
      print , ' <ms_filter> , FilterWeights = ' , FilterWeights
      print , ' <ms_filter> , LeftPad       = ' , LeftPad
      print , ' <ms_filter> , RightPad      = ' , RightPad
   endif


; /* padded TmpData will be filtered */

TmpData      = [LeftPad (*) , Data (*) , RightPad (*)]
NDataTmp     = N_Elements (TmpData)
LstTmpDataInd = NDataTmp - 1

TmpResult    = FltArr (NDataTmp)
DataFiltered = FltArr (FilterSize)

; /* Filternormalization */

If (Method  eq  'general')  $
   then begin
      FilterSum    = Total (FilterWeights)
      If (FilterSum ne 0.0) $
         then begin
            InvFilterSum = 1.0 / FilterSum
         endif else begin
            InvFilterSum = 1.0
         endelse
   endif


; /* Do filtering */


; /* For SavGol, we get everything at once */

If (Method  eq  'savgol')  $
   then begin
      TmpResult (*) = pia_sm_poly_smooth (TmpData (*) , FilterSize , Degree=FilterDegree)

      Result = TmpResult (FilterSize : LstTmpDataInd-Filtersize)

      Return
   endif


; /* BlockAve must be done separately */

If (Method  eq  'blockave')  $
   then begin
   NBlocks = (NDataTmp / FilterSize) + 1

      For  i = 0 , NBlocks-1 , 1  do begin
         TakeFrom  = (i*FilterSize)
         TakeUntil = (((i+1)*FilterSize) - 1) <  (NdataTmp-1)

         DivideBy =  Float (TakeUntil - TakeFrom + 1)

         TmpResult (TakeFrom:TakeUntil) =  Total (TmpData(TakeFrom:TakeUntil)) / DivideBy
      ENDFOR

      Result = TmpResult (FilterSize : LstTmpDataInd-Filtersize)

      Return
   endif

; /* AveMedAve must be done separately */

If (Method  eq  'avemedave')  $
   then begin
      Result = TmpResult (FilterSize : LstTmpDataInd-Filtersize)

      Return
   endif




For  i = FilterSize , LstTmpDataInd-FilterSize  do begin
   TakeFrom  = i - FilterSize2
   TakeUntil = i + FilterSize2

   Case  Method  of
      'general'  :   $
         begin
            DataFiltered (*) = FilterWeights (*)  *  TmpData (Takefrom:TakeUntil)
            TmpResult (i) = Total (DataFiltered (*)) * InvFilterSum
         end
      'boxmed'   :   $
         begin
            TmpResult (i) = Med (TmpData (Takefrom:TakeUntil))
         end
      else       :   $
         begin
            print, ' <MS_Filter> ==> Error : Filtering Method not found '
            Return
         end
   EndCase
EndFor

Result = TmpResult (FilterSize : LstTmpDataInd-Filtersize)

Return
End
















Function    PIA_SM_EvalPoly,   X, PolCoeffs
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;
;  Evaluate Polynomial given by its Coefficients
;  'PolCoeffs' at Abscissa 'X'
;

NCoff  = N_Elements (PolCoeffs)
NCoff1 = NCoff - 1

NXVals = N_Elements (X)
PolyVal = FltArr (NXVals)

PolyVal (*) = PolCoeffs (NCoff1)
For  ii = 1 , NCoff1  do begin
   i = NCoff1 - ii
   PolyVal (*) = (PolyVal(*) * X(*))  +  PolCoeffs (i)
EndFor

Return,  PolyVal(*)
End











Pro    PIA_SM_CosmicCleaner, Data, CosmicMask, Result,        $
                             FilterSize=FilterSize,           $
                             Neighbors=Neighbors,             $
                             FindThresh=FindThresh,           $
                             RemoveThresh=RemoveThresh,       $
                             Degree=Degree,                   $
                             DEBUG=DEBUG, PLOT=PLOT,          $
                            _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;
;  To do : If detected peak is broader (connected) than FilterWindow
;  IGNORE !
;
;
;
;

; /* Initialization */

SVDTolerance = 1.0E-6

NData       = N_Elements (Data)
LstDataInd  = NData - 1
CosmicMask  = IntArr (NData)
Result      = Data (*)

Repair = 0
If (N_Params ()  ge  3)   $
   then begin
      Repair = 1
   endif

WindowWing = 2
If (N_Elements (Neighbors) eq 1)  then  WindowWing = Neighbors

Order = 2
If (N_Elements (Degree) eq 1)  then  Order = Degree
MCoeff = Order + 1

DetectThresh = 4.0
If (N_Elements (FindThresh) eq 1)  then  DetectThresh = FindThresh

RepairThresh = 20.0
If (N_Elements (RemoveThresh) eq 1)  then  RepairThresh = RemoveThresh

FilterWidth = 1
If (N_Elements (FilterSize) eq 1)  then  FilterWidth = FilterSize
If ((FilterWidth mod 2) eq 0)  then  FilterWidth = FilterWidth + 1
FilterWidth2 = FilterWidth / 2

; /* Prepare the FilterFitWindow */

TotalWidth  = WindowWing + FilterWidth + WindowWing
TotalWidth2 = TotalWidth / 2

XVec     = FIndGen(WindowWing)+FilterWidth2+1
XVec     = [-XVec , +XVec]
XVec (*) = XVec (Sort(Xvec))
XVec (*) = XVec (Uniq(Xvec))

XEvalVec     = FltArr (FilterWidth)
XEvalVec (*) = FIndGen (FilterWidth)
XEvalVec (*) = XEvalVec (*) - Float (FilterWidth2)

FitWidth     = 2 * WindowWing
YFit         = FltArr (FilterWidth)

YVecL        = FltArr (WindowWing)
YVecR        = FltArr (WindowWing)

Coeffs       = FltArr (MCoeff)
IndepPowers  = FltArr (MCoeff)

; /* Prepare the Matrix with XVec - Powers */

;AMatrix  = FltArr (FitWidth , MCoeff)
AMatrix  = FltArr (MCOeff , FitWidth)
For j = 0 , FitWidth-1 do begin
   For i = 0 , MCoeff-1  do begin
;   AMatrix (* , i) = IndepPowers (i)
      AMatrix (i , j) = Xvec(j) ^ i
   EndFor
EndFor

;print, Amatrix

; /* Where are the Peaks */

InitMean    = PIA_SM_BiWeight_Mean (Data , InitSigma , InitWeights)
ClippedData = Data (Where ((Data(*) - InitMean) lt abs(DetectThresh*InitSigma)))
RevisedMean = PIA_SM_BiWeight_Mean (ClippedData , RevisedSigma , RevisedWeights)

PeaksIndex  = Where ((Data - RevisedMean) ge abs(DetectThresh*RevisedSigma) , NPeaks)

;print, ' PeaksIndex = ' , PEAKSINDEX

; /* Do Singular Value Decomposition */

SVDC,  AMatrix,  Wvector, UMatrix, VMatrix

; /* Set small singular values to zero */

SVDThresh            = SVDTolerance * Max (WVector(*))
SmallIndex           = Where (WVector  le  SVDThresh , NSmall)
If (NSmall gt 0)  then  WVector (SmallIndex) = 0.0

; /* Run through all the peaks */

TmpData = Data (*)
For i = 0 , NPeaks-1  do begin

   ; /* Get the data in the window */

   LFitFrom  = (PeaksIndex(i) - FilterWidth2 - TotalWidth2      >    0)
   LFitUntil = (PeaksIndex(i) - FilterWidth2 - 1                >    0)


   RFitFrom  = (PeaksIndex(i) + FilterWidth2 + 1                <    LstDataInd)
   RFitUntil = (PeaksIndex(i) + FilterWidth2 + TotalWidth2      <    LstDataInd)


   If (PeaksIndex (i)  eq           0)  then  YVecL (*) = YVecR (*)
   If (PeaksIndex (i)  eq  LstDataInd)  then  YVecR (*) = YVecL (*)

   If ( (PeaksIndex (i)  gt          0)   and   $
        (PeaksIndex (i)  le  WindowWing) )       $
      then begin
         YVecL (*) = Data (0)
         PIA_SM_RightInsert,  YVecL , Tmpdata (LFitFrom:LFitUntil)
      endif

   If ( (PeaksIndex (i)  ge  (LstDataInd-WindowWing))   and   $
        (PeaksIndex (i)  lt               LstDataInd) )       $
      then begin
         YVecR (*)                      = Data (LstDataInd)
         YVecR (0:(RFitUntil-RFitFrom)) = Tmpdata (RFitFrom:RFitUntil)
      endif

   If ( (PeaksIndex (i)  gt               WindowWing)   and   $
        (PeaksIndex (i)  lt  (LstdataInd-WindowWing)) )       $
      then begin
         YVecL (*) = TmpData (LFitFrom:LFitUntil)
         YVecR (*) = TmpData (RFitFrom:RFitUntil)
      endif

   YVec     = [YVecL(*) , YVecR (*)]

;   Print, '  '
;   print, ' Xvec = ' ,  Xvec
;   print, ' YVec = ' ,  YVec

   ; /* solve for the coefficients */

   Coeffs = SVSOL (UMatrix, WVector, VMatrix, YVec)

   ; /* evaluate polynomial in filterwindow */

   YFit = PIA_SM_EvalPoly (XEvalVec , Coeffs)

   TakeFrom  = PeaksIndex(i) - FilterWidth2
   TakeUntil = PeaksIndex(i) + FilterWidth2

    If (Keyword_set (DEBUG))  $
       then begin
         Print, ' <pia_sm_cosmicCleaner> PeaksIndex(i) ' , PeaksIndex(i)
         Print, ' <pia_sm_cosmicCleaner> TakeFrom , TakeUntil' ,  TakeFrom , TakeUntil
         Print, ' <pia_sm_cosmicCleaner> Data = ' ,  Data(Takefrom:TakeUntil)

         Print, ' <pia_sm_cosmicCleaner> LFitFrom:LFitUntil ' ,  LFitFrom , LFitUntil
         Print, ' <pia_sm_cosmicCleaner> RFitFrom:RFitUntil ' ,  RFitFrom , RFitUntil

         Print, ' <pia_sm_cosmicCleaner> Coeffs = ' , coeffs

         print, ' <pia_sm_cosmicCleaner> XEvalVec = ' , XEvalVec
         print, ' <pia_sm_cosmicCleaner> YFit = ' , YFit
      endif

   LocOutlierInd = Where (Data(Takefrom:TakeUntil) - YFit(*) gt (RepairThresh*RevisedSigma) ,   $
                         NOutlier)

   If (NOutlier gt 0)  $
      then begin
         OutlierIndex = LocOutlierInd (*) + PeaksIndex(i) - FilterWidth2
         CosmicMask (OutlierIndex) = 1
         TmpData (OutlierIndex) = YFit(LocOutlierInd)
      endif

   XTVec = [Xvec , XEvalvec]
   XTVec = XTVec (Sort (XTVec))
   YTvec = [YVecL ,  Data (Takefrom:TakeUntil) , YVecR]

;   print, ' XTVec ' , XTVec
;   print, ' YTVec ' , YTVec


    If (Keyword_set (Plot))  $
       then begin
          Plot , XTVec , YTVec , psym=10 , color=2
          OPlot , Xvec , YVec  , Psym=5
          OPlot , XEvalVec , YFit , Psym=2 , color=3
          ErrPlot , XEvalVec , YFit-(RepairThresh*RevisedSigma) , YFit+(RepairThresh*RevisedSigma)
          wait, 0.6
;;        Cursor , xx , yy
       endif

EndFor

If (Repair  eq  1)   $
   then begin
      Result (*) = TmpData (*)
   endif

Return
End
















PRO  PIA_SM_ClippMask,    Data , OutlierMask,                              $
                          HighCut=HighCut, Absolute=Absolute, Plot=Plot,   $
                          _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData =  N_Elements (Data)

OutlierMask = IntArr (NData)
OutlierMask (*) = 0



Thresh = 1.0
IF (Keyword_Set (HighCut))  THEN  Thresh =  HighCut

IndexAbove = Where (Data(*) GT  Thresh , NAbove)
IF (Keyword_Set (Absolute))  THEN  IndexAbove = Where (abs (Data(*)) GT  Thresh, NAbove)

PIA_SM_PropIndicsToWinds,   IndexAbove,  StartWindows, EndWindows, NWindows


FOR i = 0 , NWindows-1  DO BEGIN

   LeftDecreaseValue =  Data(StartWindows(i))
   LeftDecreaseIndex =  StartWindows(i)
   FOR  j =  StartWindows(i) , 0 , -1    DO BEGIN

      IF (Data (j)  GT  LeftDecreaseValue)   $
         THEN  BEGIN
            GOTO,  RightSide
         ENDIF

      IF (Data (j)  EQ  LeftDecreaseValue)   $
         THEN  BEGIN
            GOTO,  RightSide
         ENDIF

      IF (Data (j)  LT  LeftDecreaseValue)   $
         THEN  BEGIN
            LeftDecreaseIndex =  j
            LeftDecreaseValue = Data (j)
         ENDIF

   ENDFOR


RightSide:


   RightDecreaseValue =  Data(EndWindows(i))
   RightDecreaseIndex =  EndWindows(i)
   FOR  j =  EndWindows(i) , NData-1  , +1   DO BEGIN

      IF (Data (j)  GT  RightDecreaseValue)   $
         THEN  BEGIN
            GOTO,  NextWindow
         ENDIF

      IF (Data (j)  EQ  RightDecreaseValue)   $
         THEN  BEGIN
            GOTO,  NextWindow
         ENDIF

      IF (Data (j)  LT  RightDecreaseValue)   $
         THEN  BEGIN
            RightDecreaseIndex =  j
            RightDecreaseValue = Data (j)
         ENDIF


   ENDFOR

NextWindow:

   OutlierMask (LeftDecreaseIndex:RightDecreaseIndex) = 1


ENDFOR


IF (Keyword_Set (Plot))  $
   THEN  BEGIN
      Plot, Data (*)
      Oplot, Max(Data) - (0.1 * OutlierMask (*) * (Max(Data) - Min(Data)))
   ENDIF


Return
END












PRO  PIA_SM_OutlierInterval,  Data, OutlierMask, StartWindows , EndWindows,     $
                              NWindows, HighCut=HighCut, LowCut=LowCut,         $
                              Extend=Extend, Absolute=Absolute,                 $
                              Plot=Plot, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

NData =  N_Elements (Data)

OutlierMask = IntArr (NData)
OutlierMask (*) = 0
NWindows =  0


HighThresh = 1.0
IF (Keyword_Set (HighCut))  THEN  HighThresh =  HighCut

LowThresh = 1.0
IF (Keyword_Set (LowCut))  THEN  LowThresh =  LowCut

ExtendMask =  0
IF (Keyword_Set (Extend))  THEN ExtendMask =  Extend


; /* get the Indices where Data above HighCut */

IndexHigh = Where (Data(*) GT  HighThresh , NHigh)
IF (Keyword_Set (Absolute))  THEN  IndexHigh = Where (abs (Data(*)) GT  HighThresh, NHigh)

IF (NHigh LE 0)  THEN Return


; /* get the Indices where Data above HighCut */

IndexLow = Where (Data(*) GT  LowThresh , NLow)
IF (Keyword_Set (Absolute))  THEN  IndexLow = Where (abs (Data(*)) GT  LowThresh, NLow)


; /* make Start/End windows out of Indices */

PIA_SM_PropIndicsToWinds,   IndexHigh , StartHighWindows, EndHighWindows, NHighWindows

PIA_SM_PropIndicsToWinds,   IndexLow , StartLowWindows, EndLowWindows, NLowWindows


WindStart =         -9999L
WindEnd   =  NData + 9999L
NWindows  =  0

FOR i = 0 , NHighWindows-1  DO BEGIN

   FOR j = 0 , NLowWindows-1  DO BEGIN

      jj =  NLowWindows-1 - j

      IF (StartHighWindows (i)  GE  StartLowWindows (j))    $
         THEN  WindStart =  StartLowWindows (j)

      IF (EndHighWindows (i)  LE  EndLowWindows (jj))    $
         THEN  WindEnd =  EndLowWindows (jj)

   ENDFOR

   WindStart =  (WindStart-ExtendMask)  >  0
   WindEnd   =  (WindEnd+ExtendMask)    <  (NData-1)


   IF  (i EQ 0)  $
       THEN BEGIN
          StartWindows =  [WindStart]
          ENDWindows   =  [WindEnd]
          NWindows     =  NWindows + 1
       ENDIF ELSE begin
          StartWindows =  [StartWindows , WindStart]
          ENDWindows   =  [ENDWindows , WindEnd]
          NWindows     =  NWindows + 1
       ENDELSE

NextWindow:

   OutlierMask (WindStart:WindEnd) = 1

ENDFOR

StartWindows =  StartWindows (Sort(StartWindows))
EndWindows   =  EndWindows (Sort(EndWindows))


StartWindows =  StartWindows (Uniq(StartWindows))
EndWindows   =  EndWindows (Uniq(EndWindows))

NWindows =  N_Elements (StartWindows)


IF (Keyword_Set (Plot))  $
   THEN  BEGIN
      Plot, Data (*)
      Oplot, Max(Data) - (0.1 * OutlierMask (*) * (Max(Data) - Min(Data)))
   ENDIF


Return
END



























pro   pia_sm_radec,  ra, dec,  ihr, imin, xsec,  ideg, imn, xsc, idecsign
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;
;Convert from decimal to hrs,min
;
;+
; NAME:
;       RADEC
; PURPOSE:
;       To convert right ascension and declination from decimal degrees to
;       sexigesimal hours (for R.A.)  and degrees( for Dec.).
;
; CALLING SEQUENCE:
;       pia_sm_radec, ra, dec, ihr, imin, xsec, ideg, imn, xsc, idegsign
;
; INPUTS:
;       ra   - right ascension in decimal DEGREES, scalar or vector
;       dec  - declination in decimal DEGREES, scalar or vector, same number
;               of elements as RA
;
; OUTPUTS:
;       ihr  - right ascension hours   (INTEGER*2)
;       imin - right ascension minutes (INTEGER*2)
;       xsec - right ascension seconds  (REAL*4 or REAL*8)
;       ideg - declination degrees (INTEGER*2)
;       imn  - declination minutes (INTEGER*2)
;       xsc  - declination seconds (REAL*4 or REAL*8)
;       idecsign - sign of declination (INTEGER*2)
;
; RESTRICTIONS:
;       RADEC does minimal parameter checking.
;
; REVISON HISTORY:
;       Written by B. Pfarr, STX, 4/24/87
;       Sign of declination : M. Stickel , MPIA/IDC 3/97
;-
  On_error,2

  if (N_params() LT 2 ) then begin
    print,'Syntax - radec, ra, dec, ihr, imin, xsec, ideg, imn, xsc, idecsign'
    return
  endif

;    Compute RA

  ra = ra mod 360.          ;Make sure between 0 and 24 hours
  ra = ra + 360*(ra lt 0)
  ihr = fix(ra/15.)
  xmin =abs(ra*4.0-ihr*60.0)
  imin = fix(xmin)
  xsec = (xmin-imin)*60.0

;    Compute Dec

  ideg = fix(dec)
  xmn = abs(dec-ideg)*60.0
  imn = fix(xmn)
  xsc = (xmn-imn)*60.0

; Now test for the special case of zero degrees

  zero_deg = ( ideg EQ 0 ) and (dec LT 0)
  imn = imn - 2*imn*fix( zero_deg*(imn NE 0) )
  xsc = xsc - 2*xsc*zero_deg*(imn EQ 0)

  idecsign = +1
  If (dec lt 0.0)  $
     then begin
        ideg = abs (ideg)
        imn = abs (imn)
        xsc = abs (xsc)

        idecsign = -1
     endif

  return
  end










pro pia_sm_Bprecess, ra, dec, ra_1950, dec_1950, MU_RADEC = mu_radec,  $
                  PARALLAX = parallax,  RAD_VEL = rad_vel, EPOCH = epoch
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;+
; NAME:
;       BPRECESS
; PURPOSE:
;       Calculate the mean place of a star at B1950.0 on the FK4 system from the
;       mean place at J2000.0 on the FK5 system.
;
; CALLING SEQUENCE:
;       bprecess, ra, dec, ra_1950, dec_1950, [ MU_RADEC = , PARALLAX =
;                                       RAD_VEL =, EPOCH =   ]
;
; INPUTS:
;       RA,DEC - Input J2000 right ascension and declination in *degrees*.
;               Scalar or N element vector
;
; OUTPUTS:
;       RA_1950, DEC_1950 - The corresponding B1950 right ascension and
;               declination in *degrees*.    Same number of elements as
;               RA,DEC but always double precision.
;
; OPTIONAL INPUT-OUTPUT KEYWORDS
;       MU_RADEC - 2xN element double precision vector containing the proper
;                  motion in seconds of arc per tropical *century* in right
;                  ascension and declination.
;       PARALLAX - N_element vector giving stellar parallax (seconds of arc)
;       RAD_VEL  - N_element vector giving radial velocity in km/s
;
;       The values of MU_RADEC, PARALLAX, and RADVEL will all be modified
;       upon output to contain the values of these quantities in the
;       B1950 system.  The parallax and radial velocity will have a very
;       minor influence on the B1950 position.
;
;       EPOCH - scalar giving epoch of original observations, default 2000.0d
;           This keyword value is only used if the MU_RADEC keyword is not set.
; NOTES:
;       The algorithm is taken from the Explanatory Supplement to the
;       Astronomical Almanac 1992, page 186.
;       Also see Aoki et al (1983), A&A, 128,263
;
;       BPRECESS distinguishes between the following two cases:
;       (1) The proper motion is known and non-zero
;       (2) the proper motion is unknown or known to be exactly zero (i.e.
;               extragalactic radio sources).   In this case, the reverse of
;               the algorithm in Appendix 2 of Aoki et al. (1983) is used to
;               ensure that the output proper motion is  exactly zero. Better
;               precision can be achieved in this case by inputting the EPOCH
;               of the original observations.
;
;       The error in using the IDL procedure PRECESS for converting between
;       B1950 and J1950 can be up to 1.5", mainly in right ascension.   If
;       better accuracy than this is needed then BPRECESS should be used.
;
;       An unsystematic comparison of BPRECESS with the IPAC precession
;       routine available at ned.ipac.caltech.edu always gives differences
;       less than 0.1".
; EXAMPLE:
;       The SAO2000 catalogue gives the J2000 position and proper motion for
;       the star HD 119288.   Find the B1950 position.
;
;       RA(2000) = 13h 42m 12.740s      Dec(2000) = 8d 23' 17.69''
;       Mu(RA) = -.0257 s/yr      Mu(Dec) = -.090 ''/yr
;
;       IDL> mu_radec = 100D* [ -15D*.0257, -0.090 ]
;       IDL> ra = ten(13, 42, 12.740)*15.D
;       IDL> dec = ten(8, 23, 17.69)
;       IDL> bprecess, ra, dec, ra1950, dec1950, mu_radec = mu_radec
;       IDL> print, adstring(ra1950, dec1950,2)
;               ===> 13h 39m 44.526s    +08d 38' 28.63"
;
; REVISION HISTORY:
;       Written,    W. Landsman                October, 1992
;       Vectorized, W. Landsman                February, 1994
;       Treat case where proper motion not known or exactly zero  November 1994
;       Handling of arrays larger than 32767   Lars L. Christensen, march, 1995
;-
  On_error,2

  if N_params() LT 4 then begin
     print,'Syntax - bprecess, ra,dec, ra_1950, dec_1950, [MU_RADEC =
     print,'                            PARALLAX = , RAD_VEL = ]
     print,'  Input RA and Dec should be given in DEGREES for J2000'
     print,'  Proper motion, MU_RADEC, (optional) in arc seconds per *century*'
     print,'  Parallax (optional) in arc seconds'
     print,'  Radial Velocity (optional) in km/s '
     return

  endif

  N = N_elements( ra )
  if N EQ 0 then message,'ERROR - First parameter (RA vector) is undefined'

  if not keyword_set( RAD_VEL) then rad_vel = dblarr(N) else begin
        rad_vel = rad_vel*1.
        if N_elements( RAD_VEL) NE N then message, $
        'ERROR - RAD_VEL keyword vector must contain ' + strtrim(N,2) +' values'
  endelse

  if keyword_set( MU_RADEC) then begin
         if (N_elements( mu_radec) NE 2*N ) then message, $
    'ERROR - MU_RADEC keyword (proper motion) be dimensioned (2,' + $
                 strtrim(N,2) + ')'
        mu_radec = mu_radec*1.
  endif

  if not keyword_set( Parallax) then parallax = dblarr(N) else $
        parallax = parallax*1.

  if not keyword_set(Epoch) then epoch = 2000.0d0

  radeg = 180.D/!DPI
  sec_to_radian = 1.d0/radeg/3600.d0

 M =  [ [+0.9999256795D, -0.0111814828D, -0.0048590040D,  $
         -0.000551D,  -0.238560D,     +0.435730D     ], $
       [ +0.0111814828D, +0.9999374849D, -0.0000271557D,  $
         +0.238509D,     -0.002667D,      -0.008541D     ], $
       [ +0.0048590039D, -0.0000271771D, +0.9999881946D , $
         -0.435614D,      +0.012254D,      +0.002117D      ], $
       [ -0.00000242389840D, +0.00000002710544D, +0.00000001177742D, $
         +0.99990432D,    -0.01118145D,    -0.00485852D    ], $
       [ -0.00000002710544D, -0.00000242392702D, +0.00000000006585D, $
         +0.01118145D,     +0.99991613D,    -0.00002716D    ], $
       [ -0.00000001177742D, +0.00000000006585D,-0.00000242404995D, $
         +0.00485852D,   -0.00002717D,    +0.99996684D] ]

 A = 1D-6*[ -1.62557D, -0.31919D, -0.13843D]        ;in radians
 A_dot = 1D-3*[1.244D, -1.579D, -0.660D ]           ;in arc seconds per century

 ra_rad = ra/radeg       &      dec_rad = dec/radeg
 cosra =  cos( ra_rad )  &       sinra = sin( ra_rad )
 cosdec = cos( dec_rad ) &      sindec = sin( dec_rad )

 dec_1950 = dec*0.
 ra_1950 = ra*0.

 for i = 0L, N-1 do begin

 r0 = [ cosra(i)*cosdec(i), sinra(i)*cosdec(i), sindec(i) ]

 if keyword_set(mu_radec) then begin

 mu_a = mu_radec( 0, i )
 mu_d = mu_radec( 1, i )
 r0_dot = [ -mu_a*sinra(i)*cosdec(i) - mu_d*cosra(i)*sindec(i) , $ ;Velocity vector
             mu_a*cosra(i)*cosdec(i) - mu_d*sinra(i)*sindec(i) , $
             mu_d*cosdec(i) ] + 21.095 * rad_vel(i) * parallax(i) * r0

 endif else r0_dot = [0.0d0, 0.0d0, 0.0d0]

  R_0 = [ r0, r0_dot ]
  R_1 =  M # R_0

 ; Include the effects of the E-terms of aberration to form r and r_dot.

 r1 = R_1(0:2)
 r1_dot = R_1(3:5)

 if not keyword_set(Mu_radec) then begin
        r1 = r1 + sec_to_radian * r1_dot * (epoch - 1950.0d)/100.
        A = A + sec_to_radian * A_dot * (epoch - 1950.0d)/100.
 endif

 x1 = R_1(0)   &   y1 = R_1(1)    &  z1 = R_1(2)
 rmag = sqrt( x1^2 + y1^2 + z1^2 )


 s1 = r1/rmag    & s1_dot = r1_dot/rmag

 s = s1
 for j = 0,2 do begin
    r = s1 + A - (total(s * A))*s
    s = r/rmag
 endfor
 x = r(0)          & y = r(1)     &  z = r(2)
 r2 = x^2 + y^2 + z^2
 rmag = sqrt( r2 )

 if keyword_set(Mu_radec) then begin
         r_dot = s1_dot + A_dot - ( total( s * A_dot))*s
         x_dot = r_dot(0)  & y_dot= r_dot(1)  &  z_dot = r_dot(2)
         mu_radec(0,i) = ( x*y_dot - y*x_dot) / ( x^2 + y^2)
         mu_radec(1,i) = ( z_dot* (x^2 + y^2) - z*(x*x_dot + y*y_dot) ) /  $
                     ( r2*sqrt( x^2 + y^2) )
 endif

 dec_1950(i) = asin( z / rmag)
 ra_1950(i) = atan( y, x)

  if parallax(i) GT 0. then begin
      rad_vel(i) = ( x*x_dot + y*y_dot + z*z_dot )/ (21.095*Parallax(i)*rmag)
      parallax(i) = parallax(i) / rmag
  endif

 endfor

 neg = where( ra_1950 LT 0, NNeg )
 if Nneg GT 0 then ra_1950(neg) = ra_1950(neg) + 2.D*!DPI

 ra_1950 = ra_1950*radeg & dec_1950 = dec_1950*radeg

; Make output scalar if input was scalar

 sz = size(ra)
 if sz(0) EQ 0 then begin
        ra_1950 = ra_1950(0)     &      dec_1950 = dec_1950(0)
 endif

 return
 end










