;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
FUNCTION PIA_SM_FSC_ASSOC, RA, DEC, RADIUS, NASSOC
;+
; NAME:
;	PIA_SM_FSC_ASSOC
; PURPOSE:
;       This routine searches the IRAS point source catalog for all objects
;       closer than RADIUS to the position specified by RA and DEC. RA and
;       DEC are specified in J2000. PIA_SM_FSC_ASSOC returns
;       a structure array containing all associations to the specified position.
;                
; CALLING SEQUENCE:
;	RESULT = PIA_SM_FSC_ASSOC(RA, DEC, RADIUS, NASSOC)
;
; INPUT PARAMETERS:
;	RA	: float scalar, containing the right ascension of the search
;                 position (degrees, J2000).
;	DEC	: float scalar, containing the declination of the search
;                 position (degrees, J2000).
;	RADIUS	: float scalar, containing the search radius (degrees).
;	
; OPTIONAL INPUT:
;
; OUTPUTS:
;       PIA_SM_FSC_ASSOC returns a structure array containing the matching 
;       associations plus the spatial distance.
;
;       NASSOC: integer scalar, containing the number of associations.
;
; OPTIONAL OUTPUT:
;
; RESTRICTIONS:
;       PIA_SM_FSC_ASSOC needs the IRAS point source catalog stored in a special
;       format in different save files (splitted into bins of 15 x 15 degrees).
;
; METHOD:
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;	Written by S. Bogun (MPIA)                   	June 1997
;
;Needed common blocks : 
;       fsc_assoc: handles for the FSC buffer
;-

   ON_ERROR, 2                                         
;
; Check the number of parameters
;
   IF N_PARAMS() LT 4 THEN MESSAGE, $
    'Syntax: PIA_SM_FSC_ASSOC, RA, DEC, RADIUS, NASSOC'

   COMMON fsc_assoc, handle

   IF finite(ra) EQ 0 OR finite(dec) EQ 0 OR $
    ra LT 0.d0 OR ra GT 360.d0 OR dec LT -90.d0 OR dec GT 90. THEN BEGIN 
      message, /info, 'No valid celestial position! Returning.'
      nassoc = 0
      RETURN, -1
   ENDIF

; find the needed file for the given position
   fsc_files_prefix = 'iras_fsc_qs_'

   step_dec = 15.0D0
   overlap_dec = 0.250001d0
   ndec = nint(180./step_dec)

   idec = FIX((dec+90.000001)/step_dec)
   idec = (idec > 0) < (ndec-1)
   decm = -90.d0+(idec+0.5)*step_dec

   nra = nint(cos(decm*!dtor)*360./step_dec)
   IF idec EQ 0 OR idec EQ ndec-1 THEN nra = 1
   step_ra = 360.d0/nra
   overlap_ra = (0.250001d0 / cos(decm*!dtor)) < 360. 

   ira = FIX((ra+0.000001)/step_ra)

   IF n_elements(handle) EQ 0 THEN $
      handle = lonarr(nint(360./step_dec),ndec)

; load file if not yet in buffer, otherwise get from buffer
   IF handle(ira,idec) EQ 0 THEN BEGIN
      file = fsc_files_prefix+string(ira,idec,format=$
             '(i2.2,"_",i2.2)')+'.sav'
      restore,'/data/serend/results/iras_catalogs/'+file
      handle(ira,idec) = handle_create(value=fsc)
   ENDIF $
   ELSE $
      handle_value, handle(ira,idec), fsc

; warning: if search radius larger than overlap, association list could be
; incomplete
   IF radius GT overlap_dec THEN message, /info, $
    '*** Warning: search radius is too large. Association list could be ' +$
    'imcomplete! ***'

; search for the associations
   dist = sphdist(fsc.ra,fsc.dec,ra,dec)
   iassoc = where(dist LE radius, nassoc)
   IF nassoc EQ 0 THEN return, -1

   fsc_assoc = { IRAS_FSC_ASSOC, $
                 RA:      0.0D0, $
                 DEC:     0.0D0, $
                 CNTR:     0L, $
                 NAME:     '            ', $
                 FNU_12:   0.0, $
                 FNU_25:   0.0, $
                 FNU_60:   0.0, $
                 FNU_100:  0.0, $
                 FQUAL_12: 0, $
                 FQUAL_25: 0, $
                 FQUAL_60: 0, $
                 FQUAL_100:0, $
                 NID:      0, $
                 IDTYPE:   0, $
                 DIST:     0.0D0 $
               }

   fsc_assoc = replicate(fsc_assoc,nassoc)

   FOR i=0,n_tags(fsc)-1 DO fsc_assoc.(i) = fsc(iassoc).(i)

   fsc_assoc(0).dist = dist(iassoc(0))
   IF nassoc GT 1 THEN fsc_assoc.dist = dist(iassoc)

   IF nassoc GT 1 THEN BEGIN
      isort = sort(fsc_assoc.dist)
      fsc_assoc = fsc_assoc(isort)
   ENDIF

   return, fsc_assoc
   END 

