


Pro    PIA_SM_CompNoise,    Data,  Mask,  GlobalStatistics,            $
                            NoMask=NoMask, Debug=Debug, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;+
; NAME:
;         PIA_SM_CompNoise
;
; PURPOSE:
;         Computes Global Statistics (Mean, Sigma)
;
; CATEGORY:
;        ISOPHOT, PIA, Serendipity Survey
;
; CALLING SEQUENCE:
;         PIA_SM_CompNoise, Data, Mask, GlobalStatistics, [/NoMask, /Debug]
;
; INPUTS:
;        Data (i,*) : data streams
;        Mask (i,*) : associated mask streams, >0 means good
;
;                     i = 0 , ... , NoDetectors-1
;
; KEYWORD PARAMETERS:
;        /NoMask    : No Mask available, use all samples in all streams
;        /Debug     : switch on printouts for debugging
;
; OUTPUTS:
;        GlobalStatistics (i, 0) : mean of each data stream
;        GlobalStatistics (i, 1) : standard deviation of the distribution
;                                  for each data stream
;
;                     i = 0 , ... , NoDetectors-1
;
; COMMON BLOCKS:
;
; PROCEDURES USED:
;
; NOTES:
;
; REVISION HISTORY:
;        Written   Ver 1.0     M. Stickel, MPIA/IDC    December 18, 1996
;-


InternalDebug = 0
If (Keyword_set (Debug))  $
   then begin
      InternalDebug = 1
   endif

Dimen        = Size (Data)
NoPixels     = Dimen (1)
NoDataPoints = Dimen(2)

If (InternalDebug eq 1)  $
   then begin
      print, ' <PIA_SM_Sensitivity> NoPixels = ', NoPixels
   endif

GlobalStatistics = FltArr (NoPixels , 2)

For i = 0 , NoPixels-1  do begin
   If ( (Keyword_set (NoMask))   or  (N_elements (Mask) eq 0) )   $
      then begin
         UseData = Transpose (Data (i,*))
      endif else begin
         UseData = Transpose (Data (i , Where (Mask (i,*) gt 0)))
      endelse

   NData = Float (N_Elements (UseData))

   If (InternalDebug eq 1)  $
      then begin
         print, ' <PIA_SM_Noise> Index i = ', i
         print, ' <PIA_SM_Noise> NData   = ', NData
;         print, ' <PIA_SM_Noise> UseData = ', UseData
      endif

   Mean     = Total (UseData(*)) / NData
   Variance = Total ((UseData(*) - Mean) ^ 2) -   $
              (((Total (UseData(*) - Mean)) ^ 2) / NData)
   Variance = Variance / (Ndata - 1.0)
   StdDeviation = sqrt (Variance)

   If (InternalDebug eq 1)  $
      then begin
         print, ' <PIA_SM_Noise> Index i = ', i
         print, ' <PIA_SM_Noise> Mean    = ', Mean
         print, ' <PIA_SM_Noise> StdDev  = ', StdDeviation
      endif

   GlobalStatistics (i , 0) = Mean
   GlobalStatistics (i , 1) = StdDeviation
EndFor

If (InternalDebug eq 1)  $
   then begin
      print, ' <PIA_SM_Noise> MeanVals = ', GLobalStatistics (* , 0)
      print, ' <PIA_SM_Noise> StdDevs  = ', GLobalStatistics (* , 1)
   endif

Return
End









Pro   PIA_SM_Noise,    WorkStructure, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;+
; NAME:
;         PIA_SM_Noise
;
; PURPOSE:
;         PIA_SM - Interface to PIA_SM_CompNoise
;         Interpretes switches and prepares data and masks accordingly
;
; CATEGORY:
;        ISOPHOT, PIA, Serendipity Survey
;
; CALLING SEQUENCE:
;         PIA_SM_Noise, WorkStructure
;
; INPUTS:
;        PHT_SM_WorkStructure
;
; KEYWORD PARAMETERS:
;        any other keyword parameters are passed to all other routines
;
; OUTPUTS:
;        Results are in-place written to
;                    WorkStructure.gnoi
;
; COMMON BLOCKS:
;
; PROCEDURES USED:
;        PIA_SM_CompNoise
;
; NOTES:
;
; REVISION HISTORY:
;        Written   Ver 1.0     M. Stickel, MPIA/IDC    January 29, 1997
;-

; /* include Serendipity Dynamical Structure /*

@phtsm


; /* ... debugging needed ? */

InternalDebug = 1
If (Keyword_set (Debug))  $
   then begin
      InternalDebug = 1
   endif

; /* copy needed data from workstructure to temporary arrays */

SM_Flux   = WorkStructure.flx (*,*)
SM_Back   = WorkStructure.back (*,*)
SM_Flags  = WorkStructure.smfl (*,*)

; /* get size of data */

Dimen = Size (SM_Flux)
NoPixels = Dimen (1)
NoData   = Dimen (2)

; /* prepare mask */

PIA_SM_ConvSMFlagsMask,   SM_Flags, GoodMask, /Good

;; GoodMask  = IntArr (NoPixels , NoData)
;; For i = 0 , NoPixels-1  do begin
;;    GoodMask (i,0) = 0
;;    GoodIndex = Where (SM_Flags (i,*)  AND  NOT  flg_pia_1 , NGood)
;;    If (NGood gt 0)  $
;;       then begin
;;          GoodMask (i,GoodIndex) = 1
;;       endif
;; EndFor



; /* init sensitivities */

Globalstatistics       = FltArr (NoPixels,2)
Globalstatistics (*,*) = 0.0

; /* do it */

SM_Diff = SM_Flux - SM_Back
PIA_SM_CompNoise,   SM_Diff, GoodMask, GlobalStatistics,  $
                    _EXTRA=EXTRA
;;                    Debug=InternalDebug, _EXTRA=EXTRA

; /* put 'em back */

WorkStructure.gnoi (*) = GlobalStatistics (*,1)

Return
End





