;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       PIA_SM_PSEWORK_CREATE
;
; PURPOSE:
;       Creates a PIA Serendipity working structure using a PIA phtismc 
;       structure
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       PIA_SM_PSEWORK_CREATE, PHTISMC, HDR, PSEWORK
;
; INPUTS:
;       PHTISMC: PIA phtismc structure
;       HDR:     PHTISMC header
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;	PSEWORK: PIA Serendipity working structure
;
; COMMON BLOCKS:
;	phtsm:		Serendipity dynamical structure
;
; PROCEDURES USED:
;       sphdist
;
; MODIFICATION HISTORY:
;       Written by: 	Stefan Bogun (MPIA)			February 1997
;-

PRO PIA_SM_PSEWORK_CREATE, phtismc, hdr, psework

@phtsm
   
   unit = phtismc.unit
   det = phtismc.det
   admn = phtismc.admn
   ordr = phtismc.ordr
   dims = phtismc.dims
   hdr = phtismc.hdr
   tkey = phtismc.tkey
   time = phtismc.time
   flx = phtismc.flx
   flxu = phtismc.flxu
   back = phtismc.back
   nois = phtismc.nois
   gnoi = dblarr(4)
   flag = phtismc.flag
   smfl = phtismc.smfl
   ra = phtismc.ra
   rau = phtismc.rau
   dec = phtismc.dec
   decu = phtismc.decu
   roll = phtismc.roll
   rolu = phtismc.rolu
   sens = dblarr(4) + 1.d0
   rap = dblarr(4,dims)
   decp = dblarr(4,dims)
   vel = dblarr(dims)
   htr = dblarr(4,dims)
   grad = dblarr(3,dims)
   afst  = bytarr(dims)
   
; load pixel sensitivities
   sens(0) = FXPAR(hdr, 'PRSM_SY1')
   sens(1) = FXPAR(hdr, 'PRSM_SY2')
   sens(2) = FXPAR(hdr, 'PRSM_SY3')
   sens(3) = FXPAR(hdr, 'PRSM_SY4')

; load global noise values
   gnoi(0) = FXPAR(hdr, 'PRSM_GN1')
   gnoi(1) = FXPAR(hdr, 'PRSM_GN2')
   gnoi(2) = FXPAR(hdr, 'PRSM_GN3')
   gnoi(3) = FXPAR(hdr, 'PRSM_GN4')

; calculate pixel positions
   phi_pix=[135.,225.,315.,45.]
   FOR i = 0, 3 DO BEGIN
      decp(i,*) = dec + 90.6/3600./sqrt(2.) * $
       cos((roll+phi_pix(i))/!RADEG)
      rap(i,*) = ra + 90.6/3600./sqrt(2.) * $
       sin((roll+phi_pix(i))/!RADEG) / cos(dec/!RADEG)
   ENDFOR
   
; calculate slew speed
   dst = ra * 0.D0
   FOR i = 1, dims - 1 DO $
    dst(i) = dst(i-1) + sphdist(ra(i-1),dec(i-1),ra(i),dec(i))
   vel = DERIV(time,dst)

; create working structure for PSE
   psework = { $
              unit:unit, $
              det: det, $
              admn: admn, $
              ordr: ordr, $
              dims: dims, $
              hdr: hdr, $
              tkey: tkey, $
              time: time, $
              flx: flx, $
              flxu: flxu, $
              back: back, $
              nois: nois, $
              gnoi: gnoi, $
              flag: flag, $
              smfl: smfl, $
              ra: ra, $
              rau: rau, $
              dec: dec, $
              decu: decu, $
              roll: roll, $
              rolu: rolu, $
              sens: sens, $
              rap: rap, $
              decp: decp, $
              vel: vel, $
              htr: htr, $
              grad: grad, $
              afst: afst $
             }

   RETURN
END







