;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        PRO PIA_SM_READ_IIPH, NAME, ILUN, PATH = PATH
;+
; NAME:
;       PIA_SM_READ_IIPH
; PURPOSE:
;	Read a IIPH file accomodating the data in structure PHTIIPH  
; CALLING SEQUENCE:
;       PIA_SM_READ_IIPH, NAME, [ILUN], [PATH = PATH]
;
; INPUT:
;       NAME 	: Name of IIPH FITS file to be read
;
; OPT. INPUT:
;       PATH    : optional path for the file
;
; OPT. OUTPUTS:
;	ILUN	: Array containing the indices of the measurements in the
;		  PHTIIPH structure
;
; RESTRICTIONS:
;       File must be a valid IIPH FITS file
;
; PROCEDURES USED:
;       MERGEHDR 
;
; MODIFICATION HISTORY:
;       Written by P. Merluzzi                      	June 1995
;       Sky coordinates and roll angle                  August 1995
;       uncertainties added
;	modified according to increased primary header size
;							SB, December 1995
;       correct the sky coordinates arrays              SB & PM January 1996
;	modifications according to new IIPH file structure
;	and many other modifications                    SB, July 1996
;	correct calculation of OLP version		SB, November 1996
;	modified due to changes of IIPH format		SB, November 1996
;	modified due to changes of IIPH format for OLP V6.1
;                                                 	SB, September 1997
;-
;
@fxbintable
@adm
@phtcs
@phtiiph

        ON_ERROR, 0

;
;  Check the number of parameters.
;
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  PIA_SM_READ_IIPH, NAME, [ILUN], [PATH = PATH]'
;  Does the common exist?
         NLUN = 0
;  Check if file has been already read; in that case just return
;  the index value of the data avoiding unnecessary grow of the common
         ILUN = WHERE(PHTIIPH.UNIT EQ STRMID(NAME,4,8) , NLUN)
         ILUN = ILUN(0)
         IF NLUN EQ 0 THEN BEGIN

; create filename esp if path is set
        if keyword_set(PATH) then $
          filename = PATH + NAME $
        else filename = !FITDIR + NAME

; make sure the file exist before proceeding further
        res = findfile(filename, count=count)

        if count EQ 0 then begin
          msg = 'PIA_SM_READ_IIPH: File ' + NAME + ' not found!'
          message, /info, msg
          return
        ENDIF

        OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN
        FXHREAD, UNIT, PHDR
        CLOSE, UNIT
        FREE_LUN, unit
        FXBOPEN, UNIT, FILENAME, 1, EHDR
        I = 0
        I = WHERE(LUN EQ UNIT)
        I = I(0)
        DIMS = NAXIS2(I)
        MERGEHDR, PHDR, EHDR, HDR
        
;  Extract OLP Version 
        OLPVERS  = FXPAR(hdr,'OLPVERS')
        IF STRMID(OLPVERS,0,4) EQ 'OLP_' THEN BEGIN
           OLP_VERSION = STRCOMPRESS( STRMID(OLPVERS,4,4), /REMOVE_ALL) + '0'
	   OLPVA = FLOAT( STRMID(OLP_VERSION,0,1) )
	   OLPVB = FLOAT( '0.' + STRMID(OLP_VERSION,1,4) )
           OLP_VERSION = OLPVA + OLPVB
        ENDIF $
        ELSE $
           OLP_VERSION = 12.0

;  Take into account the different format
        IF olp_version LT 6.099 THEN BEGIN

; ... then read the data
	   MESSAGE, /INFO, 'Warning: OLP_VERSION < 6.1. Reading uncorrected '+ $
	      'pointing data!'
           FXBREAD, UNIT, UTK, 'UTK'
           FXBREAD, UNIT, UTC, 'UTC'
           FXBREAD, UNIT, RPID, 'RPID'
           FXBREAD, UNIT, OTF, 'OTF'
           FXBREAD, UNIT, FILTER, 'FILTER'
           FXBREAD, UNIT, SPIKE, 'SPIKE'
           FXBREAD, UNIT, ATTQ, 'ATTQ'
           FXBREAD, UNIT, STRQ, 'STRQ'
           FXBREAD, UNIT, RA, 'RA'
           FXBREAD, UNIT, DEC, 'DEC'
           FXBREAD, UNIT, ROLL, 'ROLL'
        ENDIF $
        ELSE BEGIN
           FXBREAD, UNIT, UTK, 'UTK'
           FXBREAD, UNIT, UTC, 'UTC'
           FXBREAD, UNIT, RPID, 'RPID'
           FXBREAD, UNIT, OTF, 'OTF'
           FXBREAD, UNIT, FILTER, 'FILTER'
           FXBREAD, UNIT, SPIKE, 'SPIKE'
           FXBREAD, UNIT, ATTQ, 'ATTQ'
           FXBREAD, UNIT, STRQ, 'STRQ'
           FXBREAD, UNIT, RA, 'CRA'
           FXBREAD, UNIT, DEC, 'CDEC'
           FXBREAD, UNIT, ROLL, 'CROLL'
        ENDELSE

; Extract the number from the file name
        IIPHNUM = STRMID(NAME, 4, 8)

; Create the structure and the handles
        PHTIIPHT = {PHTIIPH_TYPE}
        PHTIIPHT.HDR = HANDLE_CREATE()
        PHTIIPHT.UTK = HANDLE_CREATE()
        PHTIIPHT.UTC = HANDLE_CREATE()
        PHTIIPHT.RPD = HANDLE_CREATE()
        PHTIIPHT.OTF = HANDLE_CREATE()
        PHTIIPHT.FLT = HANDLE_CREATE()
        PHTIIPHT.SPK = HANDLE_CREATE()
        PHTIIPHT.ATQ = HANDLE_CREATE()
        PHTIIPHT.STR = HANDLE_CREATE()
        PHTIIPHT.RA  = HANDLE_CREATE()
        PHTIIPHT.DEC = HANDLE_CREATE()
        PHTIIPHT.RLL = HANDLE_CREATE()

        PHTIIPHT.UNIT = IIPHNUM
        PHTIIPHT.DIMS = DIMS
        HANDLE_VALUE, PHTIIPHT.HDR, HDR, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.UTK, UTK, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.UTC, UTC, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.RPD, RPID, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.OTF, OTF, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.FLT, FILTER, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.SPK, SPIKE, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.ATQ, ATTQ, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.STR, STRQ, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.RA, RA, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.DEC, DEC, /SET, /NO_COPY
        HANDLE_VALUE, PHTIIPHT.RLL, ROLL, /SET, /NO_COPY

; Update the PHTIIPH structure using the temporery structure
        PHTIIPH = [PHTIIPH, PHTIIPHT]
        ILUN = N_ELEMENTS(PHTIIPH) - 1
        FXBCLOSE, UNIT
        FREE_LUN, unit

	ENDIF       
        RETURN
        END
