
Pro    PIA_SM_CompSens,   Data,  Mask, ScaleFactors,               $
                          NoMask=NoMask, Debug=Debug, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;+
; NAME:
;         PIA_SM_CompSens
;
; PURPOSE:
;         Computes Sensitivities (ScaleFactors)
;
; CATEGORY:
;        ISOPHOT, PIA, Serendipity Survey
;
; CALLING SEQUENCE:
;         PIA_SM_CompSens, Data, Mask, ScaleFactors, [/NoMask, /Debug]
;
; INPUTS:
;        Data (i,*) : data streams, scalefactors are averages thereof
;        Mask (i,*) : associated mask streams, >0 means good
;
;                     i = 0 , ... , NoDetectors-1
;
; KEYWORD PARAMETERS:
;        /NoMask    : No Mask available, use all samples in all streams
;        /Debug     : switch on printouts for debugging
;
; OUTPUTS:
;        ScaleFactors (i) : computed and renormalized sensitivities
;
;                     i = 0 , ... , NoDetectors-1
;
; COMMON BLOCKS:
;
; PROCEDURES USED:
;
; NOTES:
;
; REVISION HISTORY:
;        Written   Ver 1.0     M. Stickel, MPIA/IDC    December 18, 1996
;-



InternalDebug = 0
If (Keyword_set (Debug))  $
   then begin
      InternalDebug = 1
   endif

Dimen        = Size (Data)
NoPixels     = Dimen (1)
ScaleFactors = FltArr (NoPixels)

If (InternalDebug  eq  1)  $
   then begin
      print, ' <PIA_SM_Sensitivity> NoPixels = ', NoPixels
   endif

For i = 0 , NoPixels-1  do begin
   If ( (Keyword_set (NoMask))   or  (N_elements (Mask) eq 0) )  $
      then begin
         UseData = Transpose (Data (i,*))
      endif else begin
         UseData =  Transpose (Data (i , Where (Mask (i,*) gt 0)))
      endelse

   NData = Float (N_Elements (UseData))
   ScaleFactors (i) = Total (UseData (*)) / NData

   If (InternalDebug eq 1)  $
      then begin
         print, ' <PIA_SM_Sensitivity> Index i = ', i
         print, ' <PIA_SM_Sensitivity> NData   = ', NData
         print, ' <PIA_SM_Sensitivity> UseData = ', UseData
      endif
EndFor

MeanScaleFactor  = Total (ScaleFactors (*)) / Float (NoPixels)
ScaleFactors (*) = MeanScaleFactor / ScaleFactors (*)

If (InternalDebug eq 1)  $
   then begin
      print, ' <PIA_SM_Sensitivity> ScaleFactors    = ', ScaleFactors (*)
      print, ' <PIA_SM_Sensitivity> MeanScaleFactor = ', MeanScaleFactor
   endif

Return
End








Pro   PIA_SM_Sensitivity,   WorkStructure,                  $
                            Use_Flux=Use_Flux,              $
                            Use_Back=Use_Back, _EXTRA=EXTRA
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;+
; NAME:
;         PIA_SM_Sensitivity
;
; PURPOSE:
;         PIA_SM_Interface to PIA_SM_CompSens
;         Interpretes switches and prepares data and masks accordingly
;
; CATEGORY:
;        ISOPHOT, PIA, Serendipity Survey
;
; CALLING SEQUENCE:
;         PIA_SM_Sensitivity, WorkStructure, [/Use_Flux, /Use_Back]
;
; INPUTS:
;        PHT_SM_WorkStructure
;
; KEYWORD PARAMETERS:
;        any other keyword parameters are passed to all other routines
;
; OUTPUTS:
;        Results are in-place written to
;                    WorkStructure.sens     (Sensitivity Factors)
;
; COMMON BLOCKS:
;
; PROCEDURES USED:
;        PIA_SM_CompSens
;
; NOTES:
;
; REVISION HISTORY:
;        Written   Ver 1.1     M. Stickel, MPIA/IDC    April 10, 1997
;        Written   Ver 1.0     M. Stickel, MPIA/IDC    January 29, 1997
;-


; /* include Serendipity Dynamical Structure /*

@phtsm

; /* .
; /* ... debugging needed ? */

InternalDebug = 1
If (Keyword_set (Debug))  $
   then begin
      InternalDebug = 1
   endif

; /* copy needed data from workstructure to temporary arrays */

SM_Flux   = WorkStructure.flx (*,*)
SM_Back   = WorkStructure.back (*,*)
SM_Flags  = WorkStructure.smfl (*,*)

; /* get size of data */

Dimen = Size (SM_Flux)
NoPixels = Dimen (1)
NoData   = Dimen (2)

; /* prepare mask */

PIA_SM_ConvSMFlagsMask,   SM_Flags, GoodMask, /Good

;; GoodMask  = IntArr (NoPixels , NoData)
;; GoodMask (*,*) = 0
;; For i = 0 , NoPixels-1  do begin
;;    GoodIndex = Where (SM_Flags (i,*)  AND  NOT  flg_pia_1 , NGood)
;;    If (NGood gt 0)  $
;;       then begin
;;          GoodMask (i,GoodIndex) = 1
;;       endif
;; EndFor


; /* init sensitivities */

ScaleFactors = FltArr (NoPixels)
ScaleFactors = 1.0

; /* use flux data for sensitivities */


If ((NOT  Keyword_Set (Use_Flux))   AND   (NOT  Keyword_Set (Use_Back))) $
   then begin
      PIA_SM_CompSens,   SM_Back,  GoodMask,  ScaleFactors,          $
                         _EXTRA=EXTRA
;;                         Debug=InternalDebug, _EXTRA=EXTRA
   endif


If (Keyword_Set (Use_Flux))  $
   then begin
      PIA_SM_CompSens,   SM_Flux,  GoodMask,  ScaleFactors,          $
                         _EXTRA=EXTRA
;;                         Debug=InternalDebug, _EXTRA=EXTRA
   endif

; /* use background for sensitivities */

If (Keyword_Set (Use_Back))  $
   then begin
      PIA_SM_CompSens,   SM_Back,  GoodMask,  ScaleFactors,          $
                         _EXTRA=EXTRA
;;                         Debug=InternalDebug, _EXTRA=EXTRA
   endif

; /* put 'em back */

WorkStructure.sens (*) = ScaleFactors (*)

Return
End





