;+
; NAME:
;       PIA_SM_SLEWSHOW
;
; PURPOSE:
;       PIA_SM_SLEWSHOW shows the slew specified by the phtismc structure
;	on one ISSA map. If the slew hits more than one plate, then you can
;       navigate through all plates. A PS file can be generated as well as 
;       instantanious output on a printer.
;       
; CATEGORY:
;       Infrared Space Observatory
;
; CALLING SEQUENCE:
;       PIA_SM_SLEWSHOW, PHTISMC
;
; INPUTS:
;       PHTISMC: PHTISMC data structure
;
; OPTIONAL INPUT PARAMETERS:
;      None.
;
; OPTIONAL INPUT KEYWORDS:
;      None.
;
; OUTPUTS:
;       Always a XWindow plot. A  PS file can be generated.
;
; OPTIONAL OUTPUT PARAMETERS:
;	None
;
; OPTIONAL OUTPUT KEYWORDS:
;       None.
;
; COMMON BLOCKS:
;       xslews_vars
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;	System variables that have to be defined:
;       !PIA_SERDIR: output directory (is set by PIA)
;	$PRINTER: the UNIX variable PRINTER determines the printer device
;
; MODIFICATION HISTORY:
;       created by: Christian Straka        MPIA Heidelberg
;                   July, 1997
;-

;find the point sources hit by the slewpath
;for more information see /data/serend/Software/SeS-2.2/pia_sm_psc_assoc.pro
FUNCTION get_pointsources, tmpdata
; prepare coordinates
   tmpdata_ = tmpdata
   ra = tmpdata_.ra
   dec = tmpdata_.dec
   time = tmpdata_.time

   dist = ra * 0.D0
   FOR i = 1, tmpdata_.dims - 1 DO $
    dist(i) = dist(i-1) + sphdist(ra(i-1),dec(i-1),ra(i),dec(i))

; search first with maximal possible search radius, interpolate slew 
; positions to larger spatial distances
   dist_limit = 0.1d0
   step = 0.5*dist_limit
   nstep = nint(dist(tmpdata_.dims-1)/step)
   dist_int = dindgen(nstep)*step
   ra_int = interpol(ra,dist,dist_int)
   dec_int = interpol(dec,dist,dist_int)
   time_int = interpol(time,dist,dist_int)

; define working structures, similar to pia_sm_psc_assoc output + slew time

   assocs = { IRAS_PSC_ASSOC, $
              RA:      0.0D0, $
              DEC:     0.0D0, $
              CNTR:     0L, $
              NAME:     '           ', $
              FNU_12:   0.0, $
              FNU_25:   0.0, $
              FNU_60:   0.0, $
              FNU_100:  0.0, $
              FQUAL_12: 0, $
              FQUAL_25: 0, $
              FQUAL_60: 0, $
              FQUAL_100:0, $
              NID:      0, $
              IDTYPE:   0, $
              DIST:     0.0D0 $
            }

; search all associations
   FOR i=0, nstep-1 DO BEGIN 
      assoc1 = pia_sm_psc_assoc(ra_int(i),dec_int(i),dist_limit,nassoc1)
      IF nassoc1 GT 0 THEN $
       assocs = [assocs,assoc1]
   ENDFOR 

   IF N_ELEMENTS(assocs) GT 1 THEN BEGIN
      assocs = assocs(1:*)

; sort associations by name and throw away mulitple associations
      iobj = sort(assocs.name)
      assocs = assocs(iobj)
      iuniq = uniq(assocs.name)
      nassocs = n_elements(iuniq)
      assocs = assocs(iuniq)
   ENDIF

   return, assocs
END


PRO pia_sm_slewshow_event, event
   COMMON xslews_vars,issa,index,ral,decl,id_1,id_3,id_6,filedir,titdir,$
    assoc,oldpos,pht,id_12,slewfile

   min = 0
   max = n_elements(issa)-1
   filename = filedir + string(index+1,format='(i2.2)')
   title = titdir + string(index+1,format='(i2.2)') + '/' + $
    string(n_elements(issa),format='(i2.2)')

   WIDGET_CONTROL, event.id, GET_UVALUE=type
   CASE type OF
      'Quit':	BEGIN
         ;restore initial path
         popd
         WIDGET_CONTROL, event.top, /DESTROY
      END
      'FORWARD': BEGIN
         IF index LT max THEN index = index + 1
         IF index EQ max THEN widget_control,id_3,sensitive=0
         widget_control,id_1,sensitive=1
         filename = filedir + string(index+1,format='(i2.2)') 
         title = titdir + string(index+1,format='(i2.2)') + '/' + $
          string(n_elements(issa),format='(i2.2)')
         widget_control,id_6,set_value=(filename+ '.ps')
         erase
         slewmap_plot,ral,decl,issa(index),slewfile=slewfile,title=title,$
          /EXTWIN, sources=assoc,mincol=40,slewcol=3
         ;restore settings for PIA
         !p.position = oldpos
       END
      'BACK':	BEGIN
         IF index GT min THEN index = index - 1
         IF index EQ min THEN widget_control,id_1,sensitive=0
         widget_control,id_3,sensitive=1
         erase
         filename = filedir + string(index+1,format='(i2.2)') 
         title = titdir + string(index+1,format='(i2.2)') + '/' + $
          string(n_elements(issa),format='(i2.2)')
         widget_control,id_6,set_value=(filename + '.ps')
         slewmap_plot,ral,decl,issa(index),slewfile=slewfile,title=title,$
          /EXTWIN, sources=assoc,mincol=40,slewcol=3
         !p.position = oldpos
       END
       'SOURCES': BEGIN
         widget_control,id_12,sensitive=0 
         assoc = get_pointsources(pht)
         filename = filedir + string(index+1,format='(i2.2)') 
         title = titdir + string(index+1,format='(i2.2)') + '/' + $
          string(n_elements(issa),format='(i2.2)')
         widget_control,id_6,set_value=(filename+ '.ps')
         erase
         slewmap_plot,ral,decl,issa(index),slewfile=slewfile,title=title,$
          /EXTWIN, sources=assoc,mincol=40,slewcol=3
         !p.position = oldpos
        END
       'SAVE':	BEGIN
         tvlct,r,g,b,/get
         set_plot,'ps'
         ;load new colortable for print file
         loadct, 3
         widget_control,id_6,get_value=filename
         filename = strmid(filename(0),0,strlen(filename(0))-3)
         slewmap_plot,ral,decl,issa(index),slewfile=slewfile,title=title,$
          outputfile=filename, sources=assoc,/PS
         !p.position = oldpos
         tvlct,r,g,b
       END
       'PRINT':	BEGIN
         tmpfile = filedir + 'tmp'
         tvlct,r,g,b,/get
         ;load new colortable for print file
         loadct,3
         slewmap_plot,ral,decl,issa(index),slewfile=slewfile,title=title,$
          outputfile=tmpfile, sources=assoc,/PS
         !p.position = oldpos
         tvlct,r,g,b
         spawn,'lpr -P$pslaser ' + tmpfile + '.ps'
	 spawn,'rm ' + tmpfile + '.ps'
       END
      ELSE: MESSAGE, /INFO, 'Function not yet implemented!'
   ENDCASE
   return
END

PRO pia_sm_slewshow,phtismc
   on_error,2
   COMMON xslews_vars,issa,index,ral,decl,id_1,id_3,id_6,filedir,titdir,$
    assoc,oldpos,pht,id_12,slewfile
   IF XRegistered("pia_sm_slewshow") NE 0 THEN RETURN
   IF N_PARAMS() LT 1 THEN BEGIN
      MESSAGE, 'XSLEWSHOW, PHTISMC'
   ENDIF

   pht = phtismc
   oldpos = !p.position
   ra = phtismc.ra
   dec = phtismc.dec
   WIDGET_CONTROL, default_font="fixed"
 
   assoc = 0
   index = 0
   cd, current=cwd
   pushd, cwd

;find the plates
   issa=['Init']
   ral = ra
   decl = dec
   ra2 = ra
   dec2 = dec
   precess,ra2,dec2,2000.,1950.
   posno = n_elements(ra2)-1
   slpos = indgen(posno/20)*20
   ; check for hit plates every 200 points
   FOR s=0, posno, 200 DO BEGIN
      issa1 = ffield(ra2(s), dec2(s))
      issa = [issa,issa1]
   ENDFOR

   issa=issa(1:*)
   issa = issa( uniq(issa) ) 

;generate filename
   filedir = !PIA_SERDIR + phtismc.admn + '_map_'
   filename = filedir + string(1,format='(i2.2)') + '.ps'
   titdir = 'Slew: ' + phtismc.admn + '   Page: '
   title = titdir + string(1,format='(i2.2)') + '/' + $
    string(n_elements(issa),format='(i2.2)')

; setup widget hirachy
   id_base = WIDGET_BASE( TITLE='XSLEWSHOW',/COLUMN)

; window
   id_draw =  WIDGET_DRAW(id_base, xs=531,ys=625,/fr)

;Buttons
   id_0 =  WIDGET_BASE(id_base, /ROW,/align_center)
   id_1 = WIDGET_BUTTON(id_0, VALUE='   <-   ',UVALUE='BACK')
   id_2 = widget_label(id_0, value = ' ')
   id_3 = WIDGET_BUTTON(id_0, VALUE='   ->   ',UVALUE='FORWARD')
   id_11 = widget_label(id_0, value = '       ')
   id_12 = WIDGET_BUTTON(id_0, VALUE=' Overlay IRAS sources ',UVALUE='SOURCES')
   id_4 = WIDGET_BASE(id_base, /ROW,/align_center)
   id_5 = widget_label(id_4, value = 'Save File:')
   id_6 = widget_text(id_4, value=filename, uval='SAVEFILE',/edit,/scroll,xs=32)
   id_7 = WIDGET_BUTTON(id_4, VALUE='  Save  ',UVALUE='SAVE') 
   id_8 = WIDGET_BUTTON(id_4, VALUE='  Print  ',UVALUE='PRINT') 
   id_9 = widget_label(id_4, value = '    ')
   id_10 = WIDGET_BUTTON(id_4, VALUE='   Quit   ',UVALUE='Quit')  

;realize widget
   widget_control, id_1, sensitive=0
   IF n_elements(issa)-1 LE 0 THEN widget_control, id_3, sensitive=0
   WIDGET_CONTROL, id_base, /REALIZE
   widget_control, id_draw, get_value=win
   wset, win

;call slewmap_routine
   slewfile = strmid(phtismc.admn,4,strlen(phtismc.admn))
   slewmap_plot,ral,decl,issa(0),slewfile=slewfile,title=title,sources=assoc, $
    /EXTWIN, mincol=40,slewcol=3
   !p.position = oldpos
    XManager, /NO_BLOCK, "pia_sm_slewshow", id_base
   RETURN
END






