;+
; NAME:
;       PIA_SM_WND
; PURPOSE:
;       Set flag for all data where the slew velocity is less than smp_vel
;
; CALLING SEQUENCE:
;       PIA_SM_WND, TMPDATA
;
; INPUT PARAMETERS:
;       tmpdata : structure to be use as input for the SM data analysis
;
; OPTIONAL INPUT:
;
; OPTIONAL OUTPUT:
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       25JUL1996 RoBaS - created document, first code
;       12JUG1996 - output to logfile
;       21AUG1996 - fixed for SeM-1.0
;       02SEP1996 - better algorithme to find first/last sample to use (x0,x1)
;       29OCT1996 - proper handling of PIA flag
;       30JAN1997 - new data structure and logfile handling
;       12FEB1997 - initialise smp_file, smp_slew
;       24FEB1997 - don't initialize smp_file, smp_slew (SB)
;
;Needed common blocks : phtsm
;-
;
PRO pia_sm_wnd, tmpdata

@phtsm


;
; get data into variables
;
  smflag = tmpdata.smfl                 ; flag from PIA_SM - empty at this point
  vel = tmpdata.vel                     ; slew speed

;
; create temp. flag and find samples at low speed
;
  tmpfl = BYTARR(tmpdata.dims)          ; declare
  tmpfl(*) = 0                          ; set to zero
  ind = WHERE(vel LT smp_vel/60., n)    ; find samples at low speed
  IF n EQ 0 THEN BEGIN                  ; check if any
    GOTO, end_routine
  ENDIF
  tmpfl(ind) = 1b                     ; set temp. flag

;
; All samples with low slew speed are flagged at this point. However, if
; the final position of the satellite is not correct it moves around to
; point to the next target. When the speed cutoff is low it may be that
; during this pointing the speed is above the cutoff again. These
; samples must be disgarded as well.
;
; The first high speed part of the slew is supposed to be the real one.
;
  ind0 = WHERE(tmpfl EQ 0, n0)          ; find the high speed part of slew
  IF n0 NE 0 THEN BEGIN                 ; do we have a high speed part ?
    j = ind0(0)                         ; high speed starts at sample j
    ind1 = WHERE(tmpfl(j:*) EQ 1b, n1)  ; find end of high speed part
    IF n1 NE 0 THEN BEGIN               ; is there an end ?
      k = ind1(0)                       ; end of high speed part at sample k from j
      i = j+k                           ; start of subsequent low speed section
      tmpfl(i:*) = tmpfl(i:*) OR 1b     ; flag all subsequent samples as low speed
    ENDIF
  ENDIF

;
; we now have all low speed samples flagged as 1, the high speed samples
; are flagged as 0.
;
; Next step is to copy the flags to smfl.
;
  ind = WHERE(tmpfl NE 0, n)            ; unusable part of slew
  IF n NE 0 THEN BEGIN                  ; if any
    smflag(*,ind) = smflag(*,ind) OR flg_speed ; add flag to four detectors
  ENDIF
  tmpdata.smfl = smflag                 ; store changes

;
; and write a note to the logfile
;
  ind = WHERE(tmpfl EQ 0, n)            ; usable part of slew
  IF n NE 0 THEN BEGIN                  ; if any
    x0 = ind(0)                         ; get first
    x1 = ind(n-1)                       ;     and last sample to use
    returnval = 0
  ENDIF

END_ROUTINE:
  RETURN
END

