;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO PROCESS_ERD_SM, FILES, NDEG, HDR, SRDPS, ERDPS, DIV_ramp=div_ramp 
;+
; NAME:
;	PROCESS_ERD_SM
; PURPOSE:
;	Fit voltage ramps of chosen measurements, taken into 
;	account diverse selections or flags and put the resulting
;	ramps slopes, uncertainties, etc. into the structure
;       phtsrd 
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	PROCESS_ERD_SM, FILES, NDEG, HDR, SRDPS, ERDPS
;
; INPUT PARAMETERS:
;	FILES	: Array containing the names (phterd.admn) of the buffers
;		  to be processed or the positions within the phterd buffer
;	NDEG	: Degree of the polynomial to be fitted
;
;  Example: FILES=['C10000100104','C10100100104','C20011111101']	 
;	processes two C100 measurements (first and second of original
;	P1ER00100104 FITS file and first of P2ER11111101 
;
; INPUT KEYWORDS:
;	div_ramp: A factor for subdividing ramps when processing
;
; OPTIONAL INPUT:
;	HDR	: String Array containing the file (updated) header
;
; OPTIONAL OUTPUT:
;	SRDPS	: Array containing the positions of the produced SRD 
;		  data within the PHTSRD structure
;	ERDPS	: Array containing the positions of the input
;		  data within the PHTERD structure
;
; RESTRICTIONS:
;
; METHOD:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phterd
;               phtsrd
;       PIA Procedure(s):
;               acc_flg
;               acc_volt
;               decode_pixf
;               der_shdr
;               fit_ramps
;               red_ramps
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel
;	Oct 95 Version 3.5
;	Extension for effective integration times and number of 
;	 used readouts (CG)					January 1996
;	Forcing flag array as integer array (CG)		July 1996
;	Keyword div_ramp added for separating ramps (CG)	July 1996
;	Filling FITV on ERD level only for not separated ramps  February 1997
;	V6.1
;	Bug (inexistent variable for uncertainties) fixed (CG)	April 1997
;	V6.2.1 
;	Ramp splitting using CB splitramp_par (CG)		May 1997
;	V6.3
;	Chopper step check included (CG)			June 1997
;	V6.4
;	Chopper position check included and step corrected (CG) September 1997
;	V6.5
;       modified for use with Serendipity subdivided ramps (SB) December 1997
;-
;
; Needed common blocks :
@adm
@phterd
@phtsrd
@splitramp_par
;	ON_ERROR, 2                                          
;
; Check the number of parameters
;
	IF N_PARAMS() LT 2 THEN MESSAGE, $
	   'Syntax: PROCESS_ERD_SM, FILES, NDEG, [HDR], [SRDPOS]'


; For each measurement chosen
;
	FOR I = 0, N_ELEMENTS(FILES) - 1 DO BEGIN

; 
; Take the input HDR if present
	  IF KEYWORD_SET(HDR) THEN ERDHDR = HDR
;
; Get type of input "files" - If integer take it, else search for the string
	  type = size(files(i))
	  type = type(N_ELEMENTS(type)-2)
	  IF (type EQ 2 OR type EQ 3) THEN index = files(i) ELSE $
          	INDEX = WHERE (PHTERD.ADMN EQ STRMID(FILES(I),0,12))
	  INDEX = INDEX(0)

; Look for compact status record
	  GIVE_CS, phterd(index), cst, record, conv_cst
	  IF STRPOS(STRUPCASE(conv_cst.cmod(record)),'RECTANGULAR') NE -1 THEN $
	      CHINCR = FLOAT(STRMID(conv_cst.camp(record),0, $ 
	                   STRPOS(conv_cst.camp(record),'arcsec')))*2  $
	  ELSE CHINCR = FLOAT(STRMID(conv_cst.cinc(record),0, $ 
	                   STRPOS(conv_cst.cinc(record),'arcsec')))
; Detector
          det = phterd(index).det
; Decode the pixel flag word
	  HANDLE_VALUE, PHTERD(INDEX).PIXF, JUNK
	  DECODE_PIXF, JUNK, SUSPECT, CHOPON, CHOPS, DR, OTF 
; Look into the acceptance criteria, fill the acc status array
	  ACC_FLG, DR, PHTERD(INDEX).DIMS, PHTERD(INDEX).NNDR, STATUS, $
		PHTERD(INDEX).ADMN, OTF
; Integration time 
	  TINT = PHTERD(INDEX).TINT

; Subdividing ramps 
	
	IF KEYWORD_SET(div_ramp) THEN BEGIN
	   IF N_ELEMENTS(splitramp_par) EQ 0 THEN splitramp_par_init
	   nndr = phterd(index).nndr
	   IF splitramp_par.model EQ 0 THEN pseudoramp = splitramp_par.value $
	   ELSE IF splitramp_par.model EQ 1 THEN BEGIN
	      pseudoramp = ((nndr+1) / splitramp_par.value) + 1
	      IF pseudoramp LT 2 THEN pseudoramp = 0
	      IF pseudoramp GT nndr THEN pseudoramp = 0
	   ENDIF ELSE pseudoramp = 0
	   DEFSYSV, '!subd_ramp', pseudoramp
	ENDIF  ELSE  DEFSYSV, '!subd_ramp', 0
;	tint = tint / div_ramp 



	  HANDLE_VALUE, PHTERD(INDEX).TIME, TIME
 	  HANDLE_VALUE, PHTERD(INDEX).CONP, CONP
; Take care of the different array dimensions for different detectors
          npix = pixel_det(det,/allpx)

; Accept read-outs for first pixel to get dimensions
	  STFLAG = STATUS
	  CONP0 = CONP(0,*)
	  ACC_VOLT, CONP0, STFLAG, DR 
	  CONDIM = N_ELEMENTS(CONP0)
	  YLIN = FLTARR(NPIX,CONDIM)
; Now fit ramps for first pixel using temporary variables 
; to get the second dimensions for two dimensional arrays
          FIT_RAMPS_SM, 0, TIME, CONP0, NDEG, DR, STFLAG, TINT, TYLIN,     $
			TYSLOP, TYSLOPR, TYFLAG, TYTIME, TYRESI, TYNROS

; Reduce ramps: compute medians of Chopper positions, ... per ramp
; and interpolate between temperatures
	  RED_RAMPS, PHTERD(INDEX), DR, STFLAG, TINT, CHINCR, $   ;input
	  		YCHOP, YSTEP, YFCSN, YTEMP, $     	;output
			YFCS1, YFCS2, YTKEY, YRPID, YCFLG        ;  "

; Accomodate variables into those to be passed
	  SLOPDIM = N_ELEMENTS(TYSLOP)
	  YTIME = FLTARR(SLOPDIM)
	  YSLOP = FLTARR(NPIX,SLOPDIM)
	  YSLOPR = YSLOP
	  YRESI = YSLOP
	  YNROS = YSLOP
	  YFLAG = FIX(YSLOP)
	  IF !subd_ramp EQ 0 THEN YLIN(0,*) = TYLIN 
	  YSLOP(0,*) = TYSLOP
	  YSLOPR(0,*) = TYSLOPR
	  YRESI(0,*) = TYRESI
	  YNROS(0,*) = TYNROS
	  YFLAG(0,*) = TYFLAG
	  YTIME = TYTIME
; Loop over other pixels (if any)
	  FOR IPIX = 1, NPIX-1 DO BEGIN
		    STFLAG = STATUS
		    CONP0 = CONP(IPIX,*)		    
; Do not use the acc_volt for open and resistor pixels
		  IF IPIX LT (NPIX - 2) AND IPIX LT 128 THEN $
	    	    ACC_VOLT, CONP0, STFLAG, DR
                    FIT_RAMPS_SM, I, TIME, CONP0, NDEG, DR, STFLAG, TINT, TYLIN, $
			TYSLOP, TYSLOPR, TYFLAG, TYTIME, TYRESI, TYNROS
		    IF !subd_ramp EQ 0 THEN YLIN(IPIX,*) = TYLIN   
		    YSLOP(IPIX,*) = TYSLOP
		    YSLOPR(IPIX,*) = TYSLOPR
		    YRESI(IPIX,*) = TYRESI
		    YNROS(IPIX,*) = TYNROS
		    YFLAG(IPIX,*) = TYFLAG
	  ENDFOR

;  Put the fitting results into the ERD structure
	  HANDLE_VALUE, PHTERD(INDEX).FITV, YLIN, /SET, /NO_COPY 

;  Accomodate data into the srd structure and the srd handles after
;  creating the temporary ones                  
	  PHTSRDT = {PHTSRD_TYPE}
          PHTSRDT.HDR = HANDLE_CREATE()
          PHTSRDT.SLOP = HANDLE_CREATE()
          PHTSRDT.SLUN = HANDLE_CREATE()
          PHTSRDT.FLAG = HANDLE_CREATE()
          PHTSRDT.STIM = HANDLE_CREATE()
          PHTSRDT.CHOP = HANDLE_CREATE()
          PHTSRDT.STEP = HANDLE_CREATE()
          PHTSRDT.FCSN = HANDLE_CREATE()
          PHTSRDT.TEMP = HANDLE_CREATE()
          PHTSRDT.FCS1 = HANDLE_CREATE()
          PHTSRDT.FCS2 = HANDLE_CREATE()
          PHTSRDT.TKEY = HANDLE_CREATE()
          PHTSRDT.RPID = HANDLE_CREATE()
          PHTSRDT.RESI = HANDLE_CREATE()
          PHTSRDT.NROS = HANDLE_CREATE()
;  Change the header according to the selection criteriae 
	  IF KEYWORD_SET(HDR) THEN ERDHDR = ERDHDR ELSE $
	  	HANDLE_VALUE, PHTERD(INDEX).HDR, ERDHDR
	  DER_SHDR, ERDHDR, NDEG, SRDHDR

;
; Check chopper position and step numbering
;
	  
	  cmode = {cste:conv_cst.cste(record), cmod:conv_cst.cmod(record), $
	  	   cinc:conv_cst.cinc(record), camp:conv_cst.camp(record), $
	  	   fcs:yfcsn}
	  
 	   CHECK_CHOP, YCHOP, YCFLG, NCHANGED
	   step_corr = CHECK_STEP(ychop, ystep, ytime, cmode, ycflg, $
	      ADMN=PHTERD(INDEX).ADMN)
	   ystep = step_corr

	   indflag = WHERE(ycflg NE 0, nflag)
	   
	   IF nflag GT 0 THEN BEGIN
	    decode_admn, PHTERD(INDEX).ADMN, name_str
	    PRINT, name_str+ ': '+ STRTRIM(nflag,2) + $
	     ' signals without chopper assignment ' + $
	     'at time: ' + STRTRIM(ytime(indflag),2)
	    yflag(*,indflag) = 2
	   ENDIF
	    

;
; 	  
; Prior to write it give uncertainties to the signals with unc=0
	  INDF1 = WHERE(yflag EQ 1, nf1)
	  IF nf1 EQ 0 THEN GOTO, bypass
	  
          get_rastersteps, yrpid, raststeps
	  get_rpchopstep, raststeps, ystep, rpchopstep, in_plat_pos
	  plat_num = max(rpchopstep)
	  FOR ipix=0,npix-1 DO BEGIN
 	   FOR irpch = 1, plat_num DO BEGIN
 	    ind_rpch = WHERE(rpchopstep EQ irpch)
	    ind_not0 = WHERE(YSLOP(ipix,ind_rpch) NE 0., nind_not0)
	    IF nind_not0 GT 0 THEN BEGIN
	     rel_unc = YSLOPR(ipix,ind_rpch(ind_not0)) / $
	     		YSLOP(ipix,ind_rpch(ind_not0))
	     erd_flagged = WHERE(yflag(ipix,ind_rpch) EQ 1 OR $
	    		         rel_unc LT 0.001, ncount)
	     IF ncount GT 0 THEN BEGIN
		sloppies = yslop(ipix,ind_rpch)
		sloppuns = yslopr(ipix,ind_rpch) 
	        check_unc, sloppies, sloppuns
		yslopr(ipix,ind_rpch) = sloppuns
	     ENDIF
	    ENDIF
	   ENDFOR
	  ENDFOR
	  FXADDPAR, srdhdr, 'PIA_ECOR', 1, $
	 "Unc. assignment performed for ramps with 2 RO", $
	 BEFORE='XTENSION'
bypass:

          PHTSRDT.UNIT = PHTERD(INDEX).UNIT
          PHTSRDT.DET  = DET
	  PHTSRDT.ADMN = PHTERD(INDEX).ADMN
          PHTSRDT.ORDR = PHTERD(INDEX).ORDR
          PHTSRDT.DIMS = N_ELEMENTS(CONP0)
          PHTSRDT.SLDI = N_ELEMENTS(YSLOP) / NPIX
          PHTSRDT.RCHP = PHTERD(INDEX).RCHP
	  HANDLE_VALUE, PHTSRDT.HDR,SRDHDR, /SET
          HANDLE_VALUE, PHTSRDT.SLOP, YSLOP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.SLUN, YSLOPR, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FLAG, YFLAG, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.STIM, YTIME, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.CHOP, YCHOP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.STEP, YSTEP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCSN, YFCSN, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.TEMP, YTEMP, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCS1, YFCS1, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.FCS2, YFCS2, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.TKEY, YTKEY, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.RPID, YRPID, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.RESI, YRESI, /SET, /NO_COPY
          HANDLE_VALUE, PHTSRDT.NROS, YNROS, /SET, /NO_COPY

; Update the PHTSRD structure using the temporary structure
	  PHTSRD = [PHTSRD, PHTSRDT]
; Update the PHTSRD value in the administration file using the index
	  SRDPOS = N_ELEMENTS(PHTSRD) - 1
;	  INDADM = WHERE (ADM.NAME EQ PHTSRDT.ADMN)
;	  ADM(INDADM).PHTSRD = SRDPOS
	  IF I EQ 0 THEN SRDPS = SRDPOS ELSE SRDPS = [SRDPS, SRDPOS]	  
	  IF I EQ 0 THEN ERDPS = INDEX ELSE ERDPS = [ERDPS, INDEX]	  
	ENDFOR
	RETURN
	END
