;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO read_smc, name, path = path
;+
; NAME:
;       READ_SMC
; PURPOSE:
;	READ a PHTSMC (Serendipity Surface Brightness Data) fits file 
;	into the dynamical structure PHTSMC  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       READ_SMC, NAME
;
; INPUT:
;       NAME 	: SMC-fitsfile name
;
; OPT. INPUT:
;	PATH    : path for the file
;
; RESTRICTIONS:
;       NAME must be a valid PHTSMC fits file 
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               adm
;               phtsmc
;       PIA Procedure(s):
;               mergehdr
;       Astronomical Library:
;               fxpar
;
; MODIFICATION HISTORY:
;       Written by: S. Bogun, MPIA, August 96
;       based on READ_SRD written by W. M. Tai and C. Gabriel
;	updated according to new SMC structure		(SB, 28/01/97)	
;	reading SMC files from !XDRDIR (rather than from !FITDIR)
;							(SB, 06/02/97)
;
;-
;
@adm
@phtsmc
;
  ON_ERROR, 2

; Check if the file has been read
  ilunr = where(strmid(phtsmc.admn,4,8) eq strmid(name, 4, 8) , nlun)
  if nlun eq 0 then begin

; create filename esp if path is set
  if keyword_set(PATH) then $
    filename = PATH + NAME $
  else filename = !XDRDIR + NAME

  OPENR, UNIT, FILENAME, /BLOCK, /GET_LUN

; read in the primary header, 2 X 2880
  phdr = bytarr(80, 3*36, /nozero)
  readu, unit, phdr

; READ IN THE EXTENDED FITS HEADER 
  EHDR = BYTARR(80, 2*36, /NOZERO)
  READU, UNIT, EHDR

  MERGEHDR, PHDR, EHDR, HDR

; C200
BUF_TYPE = { TKEY: 0.0, $
             TIME: 0.0D0, $
             FLX: DBLARR(4), $
             FLXU: DBLARR(4), $
             BACK: DBLARR(4), $
             NOIS: DBLARR(4), $
             FLAG: INTARR(4), $
             SMFL: BYTARR(4), $
             RA: 0.0D0, $
             RAU: 0.0D0, $
             DEC: 0.0D0, $
             DECU: 0.0D0, $
             ROLL: 0.0D0, $
             ROLU: 0.0D0 }

;  Create the structures and the handles
PHTSMCT = {PHTSMC_TYPE}
PHTSMCT.HDR = HANDLE_CREATE()
PHTSMCT.TKEY = HANDLE_CREATE()
PHTSMCT.TIME = HANDLE_CREATE()
PHTSMCT.FLX = HANDLE_CREATE()
PHTSMCT.FLXU = HANDLE_CREATE()
PHTSMCT.BACK = HANDLE_CREATE()
PHTSMCT.NOIS = HANDLE_CREATE()
PHTSMCT.FLAG = HANDLE_CREATE()
PHTSMCT.SMFL = HANDLE_CREATE()
PHTSMCT.RA = HANDLE_CREATE() 
PHTSMCT.RAU = HANDLE_CREATE()
PHTSMCT.DEC = HANDLE_CREATE()
PHTSMCT.DECU = HANDLE_CREATE() 
PHTSMCT.ROLL = HANDLE_CREATE()
PHTSMCT.ROLU = HANDLE_CREATE()

tmpdate = fxpar(hdr, 'SMC_DATE')
tmptime = fxpar(hdr, 'SMC_TIME')
phtsmct.unit = tmpdate + ' ' + tmptime
phtsmct.det  = fxpar(hdr, 'SMC_DET')
phtsmct.dims = fxpar(hdr, 'SMC_DIMS')

measnr = fxpar(hdr, 'NMEAS')

phtsmct.admn = 'C2' + string(measnr-1,format = '(i2.2)') + strmid(name, 4, 8)
buf = replicate(buf_type, phtsmct.dims)
readu, unit, buf

ttkey = buf.tkey
ttime = buf.time
tflx = buf.flx
tflxu = buf.flxu
tback = buf.back
tnois = buf.nois
tflag = buf.flag
tsmfl = buf.smfl
tra = buf.ra
trau = buf.rau
tdec = buf.dec
tdecu = buf.decu
troll = buf.roll
trolu = buf.rolu

byteorder, ttkey, /xdrtof
byteorder, ttime, /xdrtof
byteorder, tflx, /xdrtof
byteorder, tflxu, /xdrtof
byteorder, tback, /xdrtof
byteorder, tnois, /xdrtof
byteorder, tflag, /xdrtof
byteorder, tsmfl, /xdrtof
byteorder, tra, /xdrtof
byteorder, trau, /xdrtof
byteorder, tdec, /xdrtof
byteorder, tdecu, /xdrtof
byteorder, troll, /xdrtof
byteorder, trolu, /xdrtof
  
free_lun, unit

;  Fill the handles
HANDLE_VALUE, PHTSMCT.HDR, HDR, /SET
HANDLE_VALUE, PHTSMCT.TKEY, TTKEY, /SET
HANDLE_VALUE, PHTSMCT.TIME, TTIME, /SET
HANDLE_VALUE, PHTSMCT.FLX, TFLX, /SET
HANDLE_VALUE, PHTSMCT.FLXU, TFLXU, /SET
HANDLE_VALUE, PHTSMCT.BACK, TBACK, /SET
HANDLE_VALUE, PHTSMCT.NOIS, TNOIS, /SET
HANDLE_VALUE, PHTSMCT.FLAG, TFLAG, /SET
HANDLE_VALUE, PHTSMCT.SMFL, TSMFL, /SET
HANDLE_VALUE, PHTSMCT.RA, TRA, /SET
HANDLE_VALUE, PHTSMCT.RAU, TRAU, /SET
HANDLE_VALUE, PHTSMCT.DEC, TDEC, /SET
HANDLE_VALUE, PHTSMCT.DECU, TDECU, /SET
HANDLE_VALUE, PHTSMCT.ROLL, TROLL, /SET
HANDLE_VALUE, PHTSMCT.ROLU, TROLU, /SET


;  Update the PHTSMC structure using the temporary structure
;  Below is copied from CG's read_ismc.pro, WMT, Feb 95
        PHTSMC = [PHTSMC,PHTSMCT]

;  Does exist already an administration record for this measurement?
	       IDNAME = PHTSMCT.ADMN
	       DETID = PHTSMCT.DET
; not used anymore, leave just in case 
;               ADMIND = WHERE(ADM.NAME EQ IDNAME, N)
;
;  Update the administration file using temporary structure
;               IF (N EQ 1) THEN $
;                ADM(ADMIND).PHTSMC = N_ELEMENTS(PHTSMC) - 1 $
;               ELSE IF (N EQ 0) THEN BEGIN
;                   ADMT = {ADM_TYPE}
;                   ADMT.NAME = IDNAME
;                   SMCPOS = N_ELEMENTS(PHTSMC) - 1
;                   ADMT.PHTSMC = SMCPOS
;                   ADM = [ADM, ADMT]
;                ENDIF $
;                ELSE PRINT, 'READ_SMC: Several identical ADM-records'
  endif else $
      print, 'SMC Data already contained in structure, pos: ', ILUNR

  RETURN
END






