;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME: 
;	radec2slewcoord
;
; PURPOSE:
;	Converts slew coordinates (X,Y) into celestial coordinates (RA,DEC).
;       (X,Y) are cartesian coordinates with X being parallel to the slewing
;       direction and Y perpendicular. The conversion is only valid locally.
;
; CATEGORY:
;	PIA - Serendipity
;
; CALLING SEQUENCE:
;	slewcoord2radec, X, Y, ra_slew, dec_slew, index, ra, dec
;
; INPUTS:
;       X, Y:               The slew coordinates to be converted into (RA,DEC).
;                           (0,0) is converted to (RA_SLEW(INDEX),
;                           DEC_SLEW(INDEX)).
;
;       RA_SLEW, DEC_SLEW:  The slew coordinates (or an excerpt of the slew
;                           around the reference point 
;                           (RA_SLEW(INDEX),DEC_SLEW(INDEX)). Must be arrays.
;
;       INDEX:              An scalar integer or float pointing to one slew 
;                           position specified by (RA_SLEW,DEC_SLEW).
;
; KEYWORD PARAMETERS:
;	None.
;
; OUTPUTS:
;       RA, DEC:            The celestial coordinates corresponding to (X,Y).
;                           Same dimensions as X, Y.
;
; COMMON BLOCKS:
;	None.
;
; RESTRICTIONS:
;       The conversion is only correct locally.
;
; MODIFICATION HISTORY:
;	Written by S. Bogun (MPIA)                   	June 1997
;			
;-
; *********************************************************************
; *
; *********************************************************************

PRO SLEWCOORD2RADEC, X, Y, RA_SLEW, DEC_SLEW, INDEX, RA, DEC

   ON_ERROR, 2                                         
;
; Check the number of parameters
;
   IF N_PARAMS() LT 7 THEN MESSAGE, $
    'Syntax: SLEWCOORD2RADEC, X, Y, ra_slew, dec_slew, index, ra, dec'

   IF n_elements(ra_slew) LT 2 OR n_elements(dec_slew) LT 2 THEN message, $
    'ra_slew and dec_slew must be arrays'

;  half size of subarray to compute slew direction vector
   isize = 6

   index_i = nint(index)
   n_slew = n_elements(ra_slew)

; Compute reference point
   ra0 = interpolate(ra_slew,index)
   dec0 = interpolate(dec_slew,index)

; Compute slew direction angle (phi)
   ilow = (index_i-isize) > 0
   ihigh = (index_i+isize) < (n_slew-1)
   ra_slew_d = ra_slew(ilow:ihigh)
   dec_slew_d = dec_slew(ilow:ihigh)

   x_slew_d = -(ra_slew_d-ra0)*cos(dec_slew_d*!dtor)
   y_slew_d = dec_slew_d-dec0


   dx_slew_d = deriv(x_slew_d)
   dy_slew_d = deriv(y_slew_d)

   phi = avg(atan(dy_slew_d,dx_slew_d))

; Convert coordinates
   x_c = X*cos(phi) - Y*sin(phi)
   y_c = X*sin(phi) + Y*cos(phi)

   dec = y_c+dec0
   ra = -x_c/cos(dec*!dtor) + ra0

   RETURN
END
