	FUNCTION SPHDIST, RA1, DEC1, RA2, DEC2
;+
; NAME:
;	SPHDIST
;
; PURPOSE:
;	SPHDIST calculates the distance between two points on a sphere.
;	SPHDIST is also able to calculate the distance between a single point
;	and a series of points or between two series of points. 
;
; CATEGORY:
;       Astronomy.
;
; CALLING SEQUENCE:
;	RESULT = SPHDIST( RA1, DEC1, RA2, DEC2 )
;
; INPUTS:
;	RA1:	Flt scalar or array; specifying the right ascension of point 1
;		[degrees].
;	DEC1:	Flt scalar or array; specifying the declination of point 1
;		[degrees].
;	RA2:	Flt scalar or array; specifying the right ascension of point 2
;		[degrees].
;	DEC2:	Flt scalar or array; specifying the declination of point 2
;		[degrees].
;
; OPTIONAL INPUT PARAMETERS:
;       None.
;
; OPTIONAL INPUT KEYWORDS:
;	None.
;
; OUTPUTS:
;	SPHDIST returns the distance on the sphere between [RA1,DEC1] and
;	[RA2,DEC2] [degrees]. If the input variables are arrays, SPHDIST
;	returns an array of the same size.
;
; OPTIONAL OUTPUT PARAMETERS:
;	None.
;
; OPTIONAL OUTPUT KEYWORDS:
;	None.
;
; COMMON BLOCKS:
;	None.
;
; SIDE EFFECTS:
;       None.
;
; RESTRICTIONS:
;	RA1 and DEC1 must have the same dimensions, as well as RA2 and DEC2.
;	If RA1, DEC1 are arrays, then RA2, DEC2 must either have the same size
;	or must be scalars. In this case, the distance of [RA2,DEC2] to each
;	of the points specified in [RA1,DEC1] is returned. The same applies to
;	the opposite case.
;
; MODIFICATION HISTORY:
;       created by: Stefan Bogun            MPIA Heidelberg
;       	    March, 1995
;-

	ON_ERROR, 2

	IF N_PARAMS() LT 2 THEN MESSAGE, $
	   'Syntax: RESULT = SPHDIST( RA1, DEC1, RA2, DEC2 )'

	IF ( N_ELEMENTS(RA1) NE N_ELEMENTS(DEC1) ) OR $
	   ( N_ELEMENTS(RA2) NE N_ELEMENTS(DEC2) ) THEN MESSAGE, $
	   'Array dimensions are not compatible.'

	RA11	 = RA1
	DEC11	 = DEC1
	RA21	 = RA2
	DEC21	 = DEC2

	IF N_ELEMENTS(RA11) EQ 1 THEN BEGIN
	   RA11	 = RA21*0 + RA11
	   DEC11 = DEC21*0 + DEC11
	ENDIF

	IF N_ELEMENTS(RA21) EQ 1 THEN BEGIN
	   RA21	 = RA11*0 + RA21
	   DEC21 = DEC11*0 + DEC21
	ENDIF

	RAD	 = RA11 - RA21
	IN	 = WHERE( RAD LT 0, NN )
	IF NN GT 0 THEN BEGIN
	   RAN1	 = RA11(IN)
	   RAN2  = RA21(IN)
	   DECN1 = DEC11(IN)
	   DECN2 = DEC21(IN)

	   RA11(IN)  = RAN2
	   RA21(IN)  = RAN1
	   DEC11(IN) = DECN2
	   DEC21(IN) = DECN1
	ENDIF

	ID	 = WHERE( ABS(RAD) GT 180., ND )
	IF ND GT 0 THEN RA21(ID) = RA21(ID) + 360.

	RESULT	 = ABS( !RADEG*ACOS( SIN(DEC11/!RADEG)*SIN(DEC21/!RADEG) + $
	   COS(DEC11/!RADEG)*COS(DEC21/!RADEG)*COS((RA11-RA21)/!RADEG) ) )

	RETURN, RESULT
	END
