;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO write_smc, tmpdata, tmphdr, filename = filename
;+
; NAME:
;     WRITE_SMC
; PURPOSE:
;	Write a PHT-SMC (Serendipity Surface Brightness Data) measurement from
;	a temporary structure PHTSMC onto a fits file
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_SMC, TMPDATA
;
; INPUT:
;       tmpdata	: PHTSMC structure
;
; OPTIONAL INPUT:
;       tmphdr	: A temporary header
;       filename: filename to store the fitsfile
;
;
; RESTRICTIONS:
;       tmpdata must be a valid PHTSMC structure
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               phtsmc
;
; MODIFICATION HISTORY:
;       Written by: S. Bogun, MPIA, August 96
;	based on WRITE_SRD written by W. M. Tai and C. Gabriel
;	updated according to new SMC structure		(SB, 28/01/97)	
;
;-
@phtsmc
;
  ON_ERROR, 2
ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE


; C200 
BUF_TYPE = { TKEY: 0.0, $
             TIME: 0.0D0, $
             FLX: DBLARR(4), $
             FLXU: DBLARR(4), $
             BACK: DBLARR(4), $
             NOIS: DBLARR(4), $
             FLAG: INTARR(4), $
             SMFL: BYTARR(4), $
             RA: 0.0D0, $
             RAU: 0.0D0, $
             DEC: 0.0D0, $
             DECU: 0.0D0, $
             ROLL: 0.0D0, $
             ROLU: 0.0D0 }

comment = 'PHT C200 Serendipity Surface Brightness data'
tform_code = 'P2SM'

; Calculate the naxis size from the buf_type structure
  nchars = size_struct(buf_type)
  strnaxis1 = strtrim(string(nchars))
  naxis1 = string(replicate(32b, 20 - strlen(strnaxis1))) + strnaxis1

; Create the tag name for the extended header
  tags = tag_names(buf_type)
  ntags = n_tags( buf_type)

; this is a dummy, we will get rid of it later
  ttype = ['        ']

; calculate the interger, floating and double type code for fits header
; note that we only interested in three types here
  for t=0, ntags-1 do begin
    s = size(buf_type.(t))
    n_elem = s( s(0)+2)
    field_type = s( s(0)+1)
    case field_type of
      1:  fits_type = 'B'
      2:  fits_type = 'I'
      4:  fits_type = 'E'
      5:  fits_type = 'D'
    else:
    endcase

; put everything together, note the max number of blank space is 8
    sfield = strtrim(string(n_elem), 2)
    ttype = [ttype, sfield + fits_type + string(replicate(32b, 7 - strlen(sfield)))]
  endfor
; put the correct ttype by taking out the first dummy element
  ttype = ttype(1:ntags)

;
; Get the header from the phtsmc record if parameter missed
;
  IF KEYWORD_SET(tmphdr) THEN hdr = tmphdr ELSE $
   hdr = tmpdata.HDR  ; get the extended header
  keyword = strmid(hdr, 0, 8)
;
; Creating the NAXIS, COMMENT, FILENAME and DATE lines for the primary header

  naxis = 'NAXIS   =                    0 / number of data axes                            '
  comm_line = 'COMMENT                                                                         '
  fname_line = 'FILENAME=                      / File name assigned in ISO archive              '
  date_line = "DATE    = '13/07/94'           / Date of creating file                          "
  fname = "'P2SM" + strmid(tmpdata.admn,4,8) + "'  "
  strput, fname_line, fname, 10
  strput, comm_line, comment, 11
  monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
	 'SEP', 'OCT', 'NOV', 'DEC']
  monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
	 '09', '10', '11', '12']
  sysdate = systime()
  monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
  date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
       strmid(sysdate, 22, 2)
  strput, date_line, date, 11
;
; Find the index of the COMMENT, FILENAME  and DATE lines
;
  comment_ind = where(keyword EQ 'COMMENT ')
  file_ind = where(keyword EQ 'FILENAME')
  date_ind = where(keyword EQ 'DATE    ')
  xtension = where(keyword EQ 'XTENSION')
  hdr(comment_ind) = comm_line
  hdr(file_ind) = fname_line
  hdr(date_ind) = date_line

; extract the unit from tmpdata and split to smcdate and smctime
  unit = strtrim(tmpdata.unit, 2)
  unitlen = strlen(unit)
  tmppos = strpos(unit, ' ')
  tmp = "'" + strmid(unit, 0, tmppos) + "'"
  smcdate = blankline
  strput, smcdate , 'SMC_DATE='
  strput, smcdate, tmp, 10
  strput, smcdate, '/ SMC_DATE field in the PHTSMC', 31
  tmp = "'" + strmid(unit, tmppos + 1, unitlen - tmppos - 1) + "'"
  smctime = blankline
  strput, smctime , 'SMC_TIME='
  strput, smctime, tmp, 10
  strput, smctime, '/ SMC_TIME field in the PHTSMC', 31

; create a DET line
  detline = blankline
  strput, detline, 'SMC_DET ='
  det = "'" + strtrim(tmpdata.det, 2) + "'"
  strput, detline, det, 10
  strput, detline, '/ DETECTOR', 31

; create a DIMS line
  dimsline = blankline
  strput, dimsline, 'SMC_DIMS='
  dims = strtrim(string(tmpdata.dims), 2)
  len = strlen(dims)
  strput, dimsline, dims, (30 - len)>10
  strput, dimsline, '/', 31


; put into the primary header
phdr = [hdr(0:1), naxis, hdr( 2 : xtension(0) - 1 ), $
        smcdate, smctime, detline, dimsline, endline]

;
; pad the rest of the record with blank line for a 2880 bytes fits record
;
  trail = n_elements(phdr) mod 36
  if trail gt 0 then phdr = [phdr, replicate(blankline, 36 - trail)]

eheader = [ $
"XTENSION= 'BINTABLE'           / Binary table FITS extension                    ", $
"BITPIX  =                    8 / 8 BITS character format                        ", $
"NAXIS   =                    2 / Tables are 2-d character array                 ", $
"NAXIS1  =                      / Characters in a row                            ", $
"NAXIS2  =                      / Number of rows in a table                      ", $
"PCOUNT  =                    0 / Parameter count always 0                       ", $
"GCOUNT  =                    1 / Group count always 1                           ", $
"TFIELDS =                      / No of columns in table                         "]

; NAXIS1
;
  temp = eheader(3)
  strput, temp, naxis1, 10
  eheader(3) = temp
; NAXIS2
  naxis2 = string(tmpdata.dims)
  len = strlen(naxis2)
  temp = eheader(4)
  strput, temp, naxis2, (30 - len)>10
  eheader(4) = temp
; TFIELDS
  TFIELDS = strtrim(ntags, 2)
  temp = eheader(7)
  len = strlen(tfields)
  strput, temp, tfields, (30 - len)>10
  eheader(7) = temp
;
; The rest of the extended header
;

  tform = tform_code + tag_names(buf_type)
  itkey = WHERE(tag_names(buf_type) EQ 'TKEY',ntkey)
  IF ntkey GT 0 THEN BEGIN
     temp = tform(itkey(0))
     strput, temp, 'GPSC', 0
     tform(itkey(0)) = temp
  ENDIF
  
; create a dummy for hdr_buf
  hdr_buf = ['    ']

  for i = 0, ntags - 1 do begin
; create the form line
    temp = "TFORM   = '        '           /" + string(replicate(32b, 48))
    strput, temp, strtrim(i+1, 2), 5
    strput, temp, ttype(i), 11
    hdr_buf = [hdr_buf, temp]
; create the type line
    temp = "TTYPE   = '        '           /" + string(replicate(32b, 48))
    strput, temp, strtrim(i+1, 2), 5
    strput, temp, tform(i), 11
    strput, temp, phtsmc_desc(smc_first_handle+i+1), 33
    hdr_buf = [hdr_buf, temp]
  endfor
  hdr_buf = hdr_buf(1:2*ntags)

  eheader = [eheader(0:7), hdr_buf, endline]
  eend = n_elements(eheader)
  trail = eend mod 36
  if trail gt 0 then eheader = [eheader, replicate(blankline, 36 - trail)]
  tkey=tmpdata.tkey
  time=tmpdata.time
  flx=tmpdata.flx
  flxu=tmpdata.flxu
  back=tmpdata.back
  nois=tmpdata.nois
  flag=tmpdata.flag
  smfl=tmpdata.smfl
  ra=tmpdata.ra
  rau=tmpdata.rau
  dec=tmpdata.dec
  decu=tmpdata.decu
  roll=tmpdata.roll
  rolu=tmpdata.rolu

  byteorder, tkey, /ftoxdr
  byteorder, time, /ftoxdr
  byteorder, flx, /ftoxdr
  byteorder, flxu, /ftoxdr
  byteorder, back, /ftoxdr
  byteorder, nois, /ftoxdr
  byteorder, flag, /ftoxdr
  byteorder, smfl, /ftoxdr
  byteorder, ra, /ftoxdr
  byteorder, rau, /ftoxdr
  byteorder, dec, /ftoxdr
  byteorder, decu, /ftoxdr
  byteorder, roll, /ftoxdr
  byteorder, rolu, /ftoxdr

  BUF = REPLICATE(BUF_TYPE, TMPDATA.dims)
  buf.tkey = tkey
  buf.time = time
  buf.flx = flx
  buf.flxu = flxu
  buf.back = back
  buf.nois = nois
  buf.flag = flag
  buf.smfl = smfl
  buf.ra = ra
  buf.rau = rau
  buf.dec = dec
  buf.decu = decu
  buf.roll = roll
  buf.rolu = rolu

  naxis2 = tmpdata.dims
  trail = (nchars * naxis2) mod 2880
  pad = bytarr( 2880 - trail )

; added filename as an alternative filename
  if keyword_set(filename) then $
    fitsfile = filename else $
  fitsfile = !INTDIR + 'P2SM' + strmid(tmpdata.admn,4,8) + '.FITS'
  fname = "'P2SM" + strmid(tmpdata.admn,4,8) + "'  "

  openw, unit, fitsfile, /block, /get_lun
  writeu, unit, byte(phdr), byte(eheader), buf, pad

  free_lun, unit
END
