;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;	SLOP_VAR
;
; PURPOSE:
;	Calculate slop variations in time intervals  
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	SLOP_VAR, tmpdata, rpchopsteps, sig_disc, slop_var_str
;
; INPUT:
;	tmpdata: 	An SRD temporary structure
;	rpchopsteps:	The raster/chopper combination steps
;	sig_disc:	A flag array corresponding to tmpdata.slop
;
; KEYWORD PARAMETERS:
;	
; OUTPUT:
;	slop_var_str:	A structure containing the slopes variations per time
;			interval and pixel, uncertainties, averages, initial
;			and final signals.	
;	
; COMMON BLOCKS:
;	(none) 
;		
; METHOD:
;	Divide into time intervals per raster point and chopper position:
;	time intervals are given by 2^n seconds, (n > 1).
;
; MODIFICATION HISTORY:
;	Written by: C. Gabriel - ESA/SAI	  December 1996
;	Version 5.5
;	Bug fixed - JA+CG			  February 1997
;	V6.1
;			
;-
PRO SLOP_VAR, tmpdata, rpchopsteps, sig_disc, slop_var_str

	pixnr = pixel_det(tmpdata.det)
	
; Determine first division into timeslots taking raster/chopper position 
; into account
	   	  
	stim = tmpdata.stim
	tot_timeslots = 0
	all_sinis = FLTARR(pixnr,max(rpchopsteps))
	all_sfins = all_sinis
	
        ISTEP_CUR = 1
	FOR ISTEP=1, max(rpchopsteps) DO BEGIN  ; start loop per rpid/chopp

	   indrp = WHERE(rpchopsteps EQ istep, nrp)
	   
; initial/final signals
	   IF nrp GE 3 THEN BEGIN
		   s_ini_final, stim(indrp), tmpdata.slop(*,indrp), $
		      sig_disc(*,indrp), sini, sfin, $
		      det = tmpdata.det, error=sinerror, minnpts=3
	           all_sinis(*,istep-1) = sini
	           all_sfins(*,istep-1) = sfin
	   ENDIF ELSE sinerror = INTARR(pixnr) + 1


	   IF ISTEP eq  1 then t0 = 0. $
	      ELSE t0 = (stim(indrp(0)) + stim(indrp(0)-1))/2.
	   time = stim(indrp) - t0 
	   tmax = time(N_ELEMENTS(time)-1)
	   
	   IF tmax lt 2. THEN goto, CONT1
	   nmax = ROUND(ALOG(tmax) / ALOG(2))
	   
	   IF nmax le 0 THEN goto, CONT1
	   
	   timeslots = [2.^(indgen(nmax)+1)] 
	   ;IF ISTEP EQ 1 THEN timeslots(0) = 0. ELSE $
	   ;		   timeslots = timeslots(1:*) 
	   
	   
	   abs_timeslots = timeslots + t0

	   IF ISTEP_CUR EQ 1 THEN BEGIN
	     all_timeslots = [0.,abs_timeslots]
	     tslots = timeslots
	     all_t0 = t0 
	     ind_timeslot = n_elements(timeslots)
	     all_end = ind_timeslot
	   ENDIF  ELSE BEGIN
	     all_timeslots = [all_timeslots,abs_timeslots]
	     tslots = [tslots, timeslots]
	     all_t0 = [all_t0,t0]
	     ind_timeslot = ind_timeslot + n_elements(timeslots) 
	     all_end = [all_end, ind_timeslot]
	   ENDELSE
	   
	   ISTEP_CUR = ISTEP_CUR + 1
	   
	   
	   CONT1:

	ENDFOR 

	; Redefinition of several arrays
	
	all_slopes = FLTARR(pixnr,N_ELEMENTS(all_timeslots)-1)
	all_slopun = all_slopes
	all_averag = all_slopes
	all_flags = all_slopes
	;all_sinis = FLTARR(pixnr,N_ELEMENTS(all_end)-1)
	;all_sfins = all_sinis

	
; perform the double loop over pixels and timeslots
	
	;start loop over timeslots  
	FOR ITIME=0,N_ELEMENTS(all_timeslots) - 2 DO BEGIN  
	   ind = WHERE(stim GE (all_timeslots(itime)) AND $
		   	stim LE (all_timeslots(itime+1)), ntimes)
		   	
		
; initial/final signals
;	   IF ntimes GE 3 THEN BEGIN
;		   s_ini_final, stim(ind), tmpdata.slop(*,ind), $
;		      sig_disc(*,ind), sini, sfin, dersini, $
;		      det = tmpdata.det, error=sinerror, minnpts=3
;	   ENDIF ELSE sinerror = INTARR(pixnr) + 1
		    

	   IF ntimes GT 1 THEN BEGIN  ; 
	      FOR IPIX=0,pixnr-1 DO BEGIN
		
		indp = WHERE(sig_disc(ipix,ind) EQ 0, npoints)

		IF npoints GT 2 THEN BEGIN
		  ind2 = ind(indp)
	  	    IF !NO_WEIGHTS THEN $
	  	       weights=tmpdata.slun(ipix,ind2)*0.+1. $
			   ELSE BEGIN 
			   	weights = 1./tmpdata.slun(ipix,ind2)^2
			   	icutoff = WHERE(weights GT 30000., ncut)
			   	IF ncut GT 0 THEN $
			   	  	weights(icutoff) = 30000.
			   ENDELSE
; averages
		    weight_mean, tmpdata.slop(ipix,ind2), $
			 tmpdata.slun(ipix,ind2), avwe, unwe, avsi
; slopes variations
		    res = poly_fit(stim(ind2),tmpdata.slop(ipix,ind2),1,$
		    			yf,yb,sg,covm)

		    all_averag(ipix, itime) = avwe
		    all_slopes(ipix, itime) = res(1) * 60.
		    all_slopun(ipix, itime) = SQRT(covm(1,1)) * 60. 
		    all_flags(ipix, itime) = 0
		ENDIF ELSE BEGIN 
		    all_averag(ipix, itime) = 0.
		    all_slopes(ipix, itime) = 0.
		    all_slopun(ipix, itime) = 0.
		    all_flags(ipix, itime) = 2
		ENDELSE

		;IF sinerror(ipix) EQ 0 THEN BEGIN  
		;  all_sinis(ipix,itime) = sini(ipix)
		;  all_sfins(ipix,itime) = sfin(ipix)
		;ENDIF ELSE BEGIN
		;  all_sinis(ipix,itime) = 0.
		;  all_sfins(ipix,itime) = 0.
		;  all_flags(ipix,itime) = max([all_flags(ipix,itime),1])
		;ENDELSE
		   
	      ENDFOR   ;close loop per pixel
	    ENDIF ELSE BEGIN
		  all_averag(*, itime) = 0.
		  all_slopes(*, itime) = 0.
		  all_slopun(*, itime) = 0.
		  ;all_sinis(*,itime) = 0.
		  ;all_sfins(*,itime) = 0.
		  all_flags(*, itime) = 2
	    ENDELSE
	       

	 ENDFOR   ; close loop per timeslot 

	 
; Slop variation structure definition
	 slop_var_str = {admn:tmpdata.admn, timeslots:tslots, $
	 		t0: all_t0, ind_fin: all_end, $
	 		avrg:all_averag, slop:all_slopes, slun:all_slopun, $
	 		sini:all_sinis, sfin:all_sfins, flag:all_flags } 


RETURN
END	 
		    
	  
