;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Subtr_bckg, tmpdata, srcst, srcsubt, bckstep=bckstep, ISPD = ispd 
;+
; NAME:
;	SUBTR_BCKG
;
; PURPOSE:
;	Return subtractions of chopper plateaux, using neighbour chopper 
;	chopper plateaux positions and interpolating for the
;	background to be subtracted if required
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	Subtr_bckg, tmpdata, srcst, srcsubt, bckstep=bckstep, SPD=spd
;
; INPUTS:
;	tmpdata: A structure containing data (SCP or SPD level)
;	srcst: The chopper step to subtract from (if 98 central position is
; 		taken for triangular and sawtooth chopping, or 2 for others)
;
; KEYWORD PARAMETERS:
;	BCKSTEP: The chopper step to be subtracted - (if not present or 99
;		 interpolation of all other steps) 	
;	ISPD:	 (0 if SCP level, 1 if SPD, 2 if AAP)
;	
; OUTPUTS:
;	srcsubt: Structure containing
;		name: measurement name
;		level: 'SCP' or 'SPD'
;		aver: array (pixel #,ch.pl) of averages (src - bck)
;		avun: aver uncertainty array
;		bcav: array (pixel #,ch.pl) of background as used for
;		      subtraction
;		bcun: bcav uncertainty array
;		medi: array (pixel #,ch.pl) of medians (src - bck)
;		m1qu: array (pixel #,ch.pl) of first quartiles (src - bck)
;		m3qu: array (pixel #,ch.pl) of third quartiles (src - bck)
;		flag: array (pixel #,ch.pl)
;	
; OPTIONAL OUTPUTS:
;	
; SIDE EFFECTS:
;	
; RESTRICTIONS:
;	
; METHOD:
;	
; EXAMPLE:
;	
; CALLED ROUTINES:
;	
; COMMON BLOCKS:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               phtcs
;       PIA Procedure(s):
;               conv_subs
;               cp_discl
;               read_cs
;               spd_disc
;	
; MODIFICATION HISTORY:
;	Written by: C.Gabriel
;	Oct 95 Version 3.5
;	Bug for SS ('SL' in CS) fixed (CG)		December 1995
;	cp_discl instead of cp_disc used (CG)		March 1996
;	Avoiding crash if no interpolation possible
;	 because of only 1 bck point (CG)		March 1999
;	V7.4
;-
@phtcs
	
	IF srcst EQ 98 THEN BEGIN

		GIVE_CS, tmpdata, phtcst, indcs

; Get out the chopper mode, if sawtooth or triangular chopping look for 
; central positions and number of steps
		cmod = phtcst.cmod(indcs)
		IF cmod LE 3 THEN BEGIN
		   	srcstep = phtcst.cste(indcs) + 1 
		   	IF cmod GE 2 THEN $
		   	  srcstep = [srcstep, 3*phtcst.cste(indcs)+1]
		ENDIF ELSE srcstep = 2
	ENDIF ELSE srcstep = srcst


        IF KEYWORD_SET(bckstep) THEN bckstep=bckstep ELSE bckstep=99

	IF NOT KEYWORD_SET(ispd) THEN ispd = 0
; set variables for both SCP and SPD level

	CASE ispd OF 
	     0: BEGIN
		    level = 'SCP'
		    step = tmpdata.step
		    aver = tmpdata.aver
                    avun = tmpdata.avun
		    medi = tmpdata.medi
		    m1qu = tmpdata.q1me
		    m3qu = tmpdata.q3me
                    ; flag
		    cp_discl, tmpdata, flagarr 
		END
	     1: BEGIN
		    level = 'SPD'
		    step = tmpdata.cstp
		    aver = tmpdata.mnpw
                    avun = tmpdata.mnpu
		    medi = tmpdata.mdpw
		    m1qu = tmpdata.q1pw
		    m3qu = tmpdata.q3pw
                    ; flag
		    spd_disc, tmpdata, flagarr
		END 
	     2: BEGIN
		    level = 'AAP'
		    step = tmpdata.cstp
		    aver = tmpdata.mnfl
                    avun = tmpdata.mnfu
		    medi = tmpdata.mdfl
		    m1qu = tmpdata.q1fl
		    m3qu = tmpdata.q3fl
		    aver_br = tmpdata.mnbr
                    avun_br = tmpdata.mnbu
		    medi_br = tmpdata.mdbr
		    m1qu_br = tmpdata.q1br
		    m3qu_br = tmpdata.q3br
                    ; flag
		    aap_disc, tmpdata, flagarr
		END 
	ENDCASE	

	dim1 = N_ELEMENTS(flagarr(*,0))
	dim2 = max(step)


	aver = aver(0:dim1-1,*) 
        avun = avun(0:dim1-1,*) 
	medi = medi(0:dim1-1,*) 
	m1qu = m1qu(0:dim1-1,*) 
	m3qu = m3qu(0:dim1-1,*) 

	IF level EQ 'AAP' THEN BEGIN
	   aver_br = aver_br(0:dim1-1,*) 
           avun_br = avun_br(0:dim1-1,*) 
	   medi_br = medi_br(0:dim1-1,*) 
	   m1qu_br = m1qu_br(0:dim1-1,*) 
	   m3qu_br = m3qu_br(0:dim1-1,*) 	
	ENDIF

	IF srcstep(0) GT 0 THEN BEGIN

; Fill indices
	  IF N_ELEMENTS(srcstep) EQ 1 THEN BEGIN	
	    indsrc = WHERE(step EQ srcstep, nsrc)
	    indices = WHERE(step NE srcstep, n)
	  ENDIF ELSE BEGIN
	    indsrc = WHERE(step EQ srcstep(0) or step EQ srcstep(1), nsrc)
	    indices = WHERE(step NE srcstep(0) AND step NE srcstep(1), n)
	  ENDELSE
; Source point times - Normalize

	  last = N_ELEMENTS(tmpdata.time) - 1

	  timsrc = tmpdata.time(indsrc) / tmpdata.time(last)
; Interpolate if desired

	  IF bckstep EQ 99 THEN BEGIN 

	    intpnt = indsrc - indgen(N_ELEMENTS(indsrc)) - 0.5 
; interpolations are
	    IF n GT 1 THEN BEGIN

             intave = INTERPOLATE(aver(*,indices),indgen(dim1),intpnt, /grid)
             intmed = INTERPOLATE(medi(*,indices),indgen(dim1),intpnt, /grid)
             intm1q = INTERPOLATE(m1qu(*,indices),indgen(dim1),intpnt, /grid)
             intm3q = INTERPOLATE(m3qu(*,indices),indgen(dim1),intpnt, /grid)  
; Take the 1 sigma envelope as first approximation
             intavu = INTERPOLATE(avun(*,indices),indgen(dim1),intpnt, /grid)
	     IF level EQ 'AAP' THEN BEGIN
               intave_br = INTERPOLATE(aver_br(*,indices),indgen(dim1),intpnt, $
	              /grid)
               intmed_br = INTERPOLATE(medi_br(*,indices),indgen(dim1),intpnt, $
	              /grid)
               intm1q_br = INTERPOLATE(m1qu_br(*,indices),indgen(dim1),intpnt, $
	              /grid)
               intm3q_br = INTERPOLATE(m3qu_br(*,indices),indgen(dim1),intpnt, $
	              /grid)  
; Take the 1 sigma envelope as first approximation
               intavu_br = INTERPOLATE(avun_br(*,indices),indgen(dim1),intpnt, $
	              /grid)
	     ENDIF

	     flagarr = flagarr ^ 2

             intflg = INTERPOLATE(flagarr(*,indices),indgen(dim1),intpnt, /grid)
	     indbad = WHERE(intflg GT 1, nbad)
	     IF nbad GT 0 THEN intflg(indbad) = 2
	     flagarr = SQRT(flagarr)

	    ENDIF ELSE BEGIN

	     intave = aver(*,indices)
	     intavu = avun(*,indices)
             intmed = medi(*,indices)
             intm1q = m1qu(*,indices)
             intm3q = m3qu(*,indices)
             intflg = flagarr(*,indices)
	     IF level EQ 'AAP' THEN BEGIN
               intave_br = aver_br(*,indices)
               intmed_br = medi_br(*,indices)
               intm1q_br = m1qu_br(*,indices)
               intm3q_br = m3qu_br(*,indices)
               intavu_br = avun_br(*,indices)
	     ENDIF  ;endif of AAP level

	    ENDELSE	     

	  ENDIF ELSE IF bckstep EQ 0 THEN BEGIN  

		  intave = aver(*,indices)
		  intavu = avun(*,indices)
                  intmed = medi(*,indices)
                  intm1q = m1qu(*,indices)
                  intm3q = m3qu(*,indices)
                  intflg = flagarr(*,indices)
		  IF level EQ 'AAP' THEN BEGIN
        	    intave_br = aver_br(*,indices)
        	    intmed_br = medi_br(*,indices)
        	    intm1q_br = m1qu_br(*,indices)
        	    intm3q_br = m3qu_br(*,indices)
        	    intavu_br = avun_br(*,indices)
		  ENDIF  ;endif of AAP level

          ENDIF ELSE BEGIN

		  indices = WHERE(step EQ bckstep)

		  intave = aver(*,indices)
		  intavu = avun(*,indices)
                  intmed = medi(*,indices)
                  intm1q = m1qu(*,indices)
                  intm3q = m3qu(*,indices)
                  intflg = flagarr(*,indices)
		  IF level EQ 'AAP' THEN BEGIN
        	    intave_br = aver_br(*,indices)
        	    intmed_br = medi_br(*,indices)
        	    intm1q_br = m1qu_br(*,indices)
        	    intm3q_br = m3qu_br(*,indices)
        	    intavu_br = avun_br(*,indices)
		  ENDIF  ;endif of AAP level


          ENDELSE

	  nn = N_ELEMENTS(intave(0,*))

	  if nn GT nsrc + 1 THEN BEGIN 

			MESSAGE, /INFO,$
				' Cannot perform subtraction, I get confused'
			PRINT,$
				' Source points =',nsrc, '   Bckg.:=', nn
			
			RETURN

	   ENDIF ELSE BEGIN

; Set equal lengths for both source and background
	     nbck = N_ELEMENTS(intave(0,*))
	     IF nbck GT nsrc THEN nbck = nsrc
	     IF nsrc GT nbck THEN nsrc = nbck
	
	     srcave = aver(*,indsrc(0:nsrc-1)) - intave(*,0:nbck-1)
	     srcavu = sqrt(avun(*,indsrc(0:nsrc-1))^2 + intavu(*,0:nbck-1)^2)  
	     srcmed = medi(*,indsrc(0:nsrc-1)) - intmed(*,0:nbck-1)
	     srcm1q = m1qu(*,indsrc(0:nsrc-1)) - intm1q(*,0:nbck-1)
	     srcm3q = m3qu(*,indsrc(0:nsrc-1)) - intm3q(*,0:nbck-1)
             IF level EQ 'AAP' THEN BEGIN
		srcave_br = aver_br(*,indsrc(0:nsrc-1)) - intave_br(*,0:nbck-1)
		srcavu_br = SQRT(avun_br(*,indsrc(0:nsrc-1))^2 + $ 
		                  intavu_br(*,0:nbck-1)^2)  
		srcmed_br = medi_br(*,indsrc(0:nsrc-1)) - intmed_br(*,0:nbck-1)
		srcm1q_br = m1qu_br(*,indsrc(0:nsrc-1)) - intm1q_br(*,0:nbck-1)
		srcm3q_br = m3qu_br(*,indsrc(0:nsrc-1)) - intm3q_br(*,0:nbck-1)
             ENDIF  ;endif of AAP level

; Use both flags to flag the src-bckg 

	     srcflg = flagarr(*,indsrc(0:nsrc-1))^2 + intflg(*,0:nbck-1)^2
	     indbad = WHERE(srcflg GT 3, nbad)
	     indok = WHERE(srcflg EQ 0, nok)
	     srcflg = (FIX(srcflg) * 0) + 1
	     IF nbad GT 0 THEN srcflg(indbad) = 2
	     IF nok GT 0 THEN srcflg(indok) = 0

	     IF level NE 'AAP' THEN $
	     SRCSUBT = {NAME:tmpdata.admn,           $
			level:level,                 $
			srcstep:srcstep,             $
			bckstep:bckstep,             $
			aver:srcave,                 $
			avun:srcavu,                 $
			bcav:intave(*,0:nbck-1),     $
			bcun:intavu(*,0:nbck-1),     $
			medi:srcmed,                 $
			m1qu:srcm1q,                 $
			m3qu:srcm3q,                 $
			flag:srcflg }                $
            ELSE                                     $
	     SRCSUBT = {NAME:tmpdata.admn,           $
			level:level,                 $
			srcstep:srcstep,             $
			bckstep:bckstep,             $
			aver:srcave,                 $
			avun:srcavu,                 $
			bcav:intave(*,0:nbck-1),     $
			bcun:intavu(*,0:nbck-1),     $
			medi:srcmed,                 $
			m1qu:srcm1q,                 $
			m3qu:srcm3q,                 $
			aver_br:srcave_br,           $
			avun_br:srcavu_br,           $
			bcav_br:intave_br(*,0:nbck-1),$
			bcun_br:intavu_br(*,0:nbck-1),$
			medi_br:srcmed_br,           $
			m1qu_br:srcm1q_br,           $
			m3qu_br:srcm3q_br,           $
			flag:srcflg }                

           ENDELSE
        ENDIF
RETURN
END

