;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       SUBTR_darstr_MEAS
;
; PURPOSE:
;	Subtract an SCP level dark current or straylight measurement from 
;	a source or FCS measurement, replacing the contents into the 
;	dynamical common block PHTSCP 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       SUBTR_darstr_MEAS, src_meas, bck_meas, subt_meas, STRAY=stray
;
; INPUT:
;       src_meas:
;		A real SCP structure containing the source measurement
;       bck_meas:
;		A real SCP structure containing the background measurement
; INPUT KEYWORDS:
;	STRAY:
;		indicating subtraction of straylight measurement
;	REPL_INDEX:
;		giving the position within the scp common to put the 
;		subtracted measurement
;
; OUTPUT:
;	subt_meas:
;		A real SCP structure containing the subtr. measurement
;
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtscp
;		phtspd
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)		December 1996
;	REPL_INDEX keyword added (CG)			June 1997
;	V6.4
;	Measurement name added if Warning (CG)		August 1999
;	V8.0
;-
PRO subtr_darstr_meas, meas1, meas2, subt_meas, $
		STRAY=stray, REPL_INDEX=repl_index

@adm
@phtscp
;
 On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 3 THEN MESSAGE, $
  'Syntax: SUBTR_darstr_MEAS, meas1, meas2, subt_meas[,STRAY=stray] '

	subt_meas = meas1
	aper_norm = 1.
	  	  
 	IF N_ELEMENTS(meas2.aver(0,*)) EQ 1 THEN BEGIN
	    npix = N_ELEMENTS(meas1.aver(*,0))
	    FOR Ipix = 0, npix-1 DO BEGIN
	   	subt_meas.aver(ipix,*) = meas1.aver(ipix,*) - $
	   				 aper_norm * meas2.aver(ipix,*) 
	   	subt_meas.medi(ipix,*) = meas1.medi(ipix,*) - $
	   				 aper_norm * meas2.medi(ipix,*) 
	   	subt_meas.q1me(ipix,*) = meas1.q1me(ipix,*) - $
	   				 aper_norm * meas2.q1me(ipix,*) 
	   	subt_meas.q3me(ipix,*) = meas1.q3me(ipix,*) - $
	   				 aper_norm * meas2.q3me(ipix,*) 
	   	subt_meas.avun(ipix,*) = SQRT(meas1.avun(ipix,*)^2 + $
	   				(aper_norm * meas2.avun(ipix,*))^2 )
		subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					(aper_norm * meas2.flag(ipix,*))^2 )) 
	    ENDFOR
	 ENDIF ELSE MESSAGE, /INFO, 'Do not know how to handle this case'
        
; check: if dark current subtraction already applied give a warning
	 newheader = subt_meas.hdr
         done = fxpar(newheader,'PRS_DARK')
	 IF done NE 0 THEN BEGIN
	    PRINT, 'Warning!!!: A dark current subtraction has been already' 
	    PRINT, 'Warning!!!:    performed - this is actually not OK  '
	    PRINT, 'Measurement ' + dec_admn(subt_meas.admn) 
	 ENDIF
	 
	 decode_admn, meas2.admn, dstring
	 IF not KEYWORD_SET(STRAY) THEN $
	    fxaddpar, newheader, 'PRS_DARK', 1, $ 
		 ' Dark Current subtracted: '+dstring, before='XTENSION'  $
	 ELSE BEGIN
	    fxaddpar, newheader, 'PRC_STRL', 1, $ 
		 ' Straylight subtracted: '+dstring, before='XTENSION'
	    fxaddpar, newheader, 'PRS_DARK', 2, $ 
		 ' Dark Current implicitely subtracted ', before='XTENSION' 
	 ENDELSE
;  Update the PHTSCP structure using the temporary structure
	 IF NOT KEYWORD_SET(repl_index) THEN BEGIN 
	  ind = WHERE(phtscp.admn EQ subt_meas.admn, nind)
	  IF nind GT 1 THEN BEGIN
		decode_admn, phtscp(ind).admn, admis
		pia_multchoice, index, $
			elemnames = admis
	  ENDIF ELSE index = 0
	  repl_index = ind(index)
	 ENDIF
	     
         fill_scp, subt_meas, newheader, REPLACE=repl_index 
           


RETURN
END
                           
