;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       SUBTR_MEAS
;
; PURPOSE:
;	Subtract an SCP or SPD level measurement from another one, writing the
;	results into the dynamical common block PHTSCP 
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;       SUBTR_MEAS, src_meas, bck_meas, subt_meas, SPD=SPD
;
; INPUT:
;       src_meas:
;		A real SCP or SPD structure containing the source measurement
;       bck_meas:
;		A real SCP or SPD structure containing the background measurement
; INPUT KEYWORDS:
;	SPD:
;		indicating SPD level subtraction
;	AAP:
;		indicating AAP level subtraction
;	NO_AP:
;		not using apertures normalization
;	NO_WIDGET: 
;		to avoid showing a widget with subtraction info.
;	INDPHT
;		the index of subtracted measure in PHTXXX structure 
;
; OUTPUT:
;	subt_meas:
;		A real SCP or SPD structure containing the subtr. measurement
;
;
; SIDE EFFECTS:
;	The dynamical structure phtscp (or phtspd) is increased on one record
;	containing the subtracted measurement.
;
; RESTRICTIONS:
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtscp
;		phtspd
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)		January 1996
;	Extension for keywords ARSI, ANDR and AVSD (CG) January 1996
;	Apertures normalization introduced - 
;	  No_AP keyword added  (CG)			April 1996
;	Information added on buffer name (CG)		April 1996
;	Extended for AAP subtraction (CG)		November 1997
;	V6.6
;	For normalisation by different apertures take
;	 omegas instead of aperture sizes (CG)		January 1998
;	V7.0
;	Number of pixels for PHT-S SPD/AAP fixed (CG)	July 1998
;	V7.2
;	Use CASE instead of IF ELSE, use FILL_xxx routines, 
;	and update of PIA_MNAM keyword in the header (JAc)	Dec 1998
;	V7.3
;	Bug for aperture normalization case fixed +
;	 warning added in case of measurements with
;	 different filters/apertures (CG)			June 1999
;	V7.4
;-
PRO subtr_meas, meas1, meas2, subt_meas, SPD=SPD, NO_AP=NO_AP, AAP=AAP, $
   NO_WIDGET = NO_WIDGET, INDPHT = indpht

@adm
@phtscp
@phtspd
@phtaap
@phtomega
;
 On_ERROR,2                                          ;Return to caller
;
; Check the number of parameters
;
	IF N_PARAMS() LT 3 THEN MESSAGE,$
	        'Syntax: SUBTR_MEAS, meas1, meas2, subt_meas[,/SPD]'  + $ 
                '[,NO_AP=NO_AP] [,/AAP] [,/NO_WIDGET] [,INDPHT=indpht]'

	subt_meas = meas1
	det = STRMID(meas1.admn,0,2)
	IF KEYWORD_SET(SPD) or KEYWORD_SET(AAP) THEN is_spd = 1 ELSE is_spd=0
	detpix = pixel_det(det, IS_SPD=is_spd)

; Check if omega is defined
	IF N_ELEMENTS(c1om) EQ 0 THEN read_omega

	IF KEYWORD_SET(NO_AP) OR STRMID(meas1.admn,0,1) EQ 'S' THEN BEGIN
	  	aper_norm=FLTARR(128) + 1. 
	  	aper_norm_unc = FLTARR(128)
	ENDIF ELSE BEGIN
	  	  get_wheels, meas1.admn, wheels1
	  	  get_wheels, meas2.admn, wheels2
	  	
	  	  
		  IF (STRMID(det,0,1) EQ 'C' AND $
		          wheels1(1) NE wheels2(1)) OR $ 
		     (STRMID(det,0,1) EQ 'P' AND $
		          wheels1(2) NE wheels2(2)) THEN BEGIN
	MESSAGE, /INFO, 'Warning, subtracting meas with different filters:'
	MESSAGE, /INFO, dec_admn(meas1.admn) + ' vs ' + dec_admn(meas2.admn)
		  ENDIF
		  
		  IF (STRMID(det,0,1) EQ 'P' AND $
		          wheels1(1) NE wheels2(1)) THEN BEGIN
	MESSAGE, /INFO, 'Warning, subtracting meas with different apertures:'
	MESSAGE, /INFO, 'Using aperture areas for normalizing '
		  ENDIF
		  
		  
	  	  CASE STRMID(det,0,2) of
	  	  
	  	   'C1': BEGIN
	  	   	area1 = REFORM(c1om(*,wheels1(1)-2,*)) 
	  	   	area2 = REFORM(c1om(*,wheels2(1)-2,*)) 
	  	   END
	  	   'C2': BEGIN
	  	   	area1 = REFORM(c2om(*,wheels1(1)-9,*,*)) 
	  	   	area2 = REFORM(c2om(*,wheels2(1)-9,*,*)) 
	  	   END
	  	   ELSE: BEGIN
	  	          area1 = REFORM(pom(*,wheels1(2)-1,*)) 
	  	          area1 = area1(wheels1(1)-1,*)
	  	          area2 = REFORM(pom(*,wheels2(2)-1,*)) 
	  	          area2 = area2(wheels2(1)-1,*)	  	        
	  	   END
	  	  ENDCASE   
	  	  
	  	  aper_norm = area1(*,0)/area2(*,0)
	  	  aper_norm_unc = aper_norm * $
	  	  SQRT( (area1(*,1)/area1(*,0))^2 + (area2(*,1)/area2(*,0))^2 )  
	ENDELSE	   

CASE 1 OF 

;  SPD part
	KEYWORD_SET(SPD): BEGIN 

 	  IF N_ELEMENTS(meas2.mnpw(0,*)) EQ 1 THEN BEGIN
	    npix = detpix
	    FOR Ipix = 0, npix-1 DO BEGIN
	   	subt_meas.mnpw(ipix,*) = meas1.mnpw(ipix,*) - $
	   				aper_norm(ipix) * meas2.mnpw(ipix,*) 
	   	subt_meas.mdpw(ipix,*) = meas1.mdpw(ipix,*) - $
	   				aper_norm(ipix) * meas2.mdpw(ipix,*) 
	   	subt_meas.q1pw(ipix,*) = meas1.q1pw(ipix,*) - $
	   				aper_norm(ipix) * meas2.q1pw(ipix,*) 
	   	subt_meas.q3pw(ipix,*) = meas1.q3pw(ipix,*) - $
	   				aper_norm(ipix) * meas2.q3pw(ipix,*) 

		subt_meas.mnpu(ipix,*) = SQRT( meas1.mnpu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*meas2.mnpu(ipix,*))^2  + $
	   		      (aper_norm_unc(ipix) * meas2.mnpw(ipix,*))^2) )

		subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					meas2.flag(ipix,*)^2 )) 
	    ENDFOR
	  ENDIF ELSE BEGIN
	    npix = detpix
	    FOR Ipix = 0, npix-1 DO BEGIN
		indok = WHERE(meas2.flag(ipix,*) LT 2, ncount)
		IF ncount GT 1 THEN BEGIN
		  weight_mean, meas2.mnpw(ipix,indok), meas2.mnpu(ipix,indok),$
		 		mnpw, mnpu
	   	  subt_meas.mnpw(ipix,*) = meas1.mnpw(ipix,*) -              $
	   	  			aper_norm(ipix) * mnpw
	   	 			
	   	  subt_meas.mdpw(ipix,*) = meas1.mdpw(ipix,*) -              $
	   	 	 aper_norm(ipix) * median(meas2.mdpw(ipix,indok)) 
	   	  subt_meas.q1pw(ipix,*) = meas1.q1pw(ipix,*) -              $
	   	 	 aper_norm(ipix) * median(meas2.q1pw(ipix,indok)) 
	   	  subt_meas.q3pw(ipix,*) = meas1.q3pw(ipix,*) -              $
	   	 	 aper_norm(ipix) * median(meas2.q3pw(ipix,indok)) 
	   	  subt_meas.mnpu(ipix,*) = SQRT( meas1.mnpu(ipix,*)^2 +      $
	   		 ((aper_norm(ipix)*mnpu)^2  +                        $
	   		      (aper_norm_unc(ipix) * mnpw)^2) )

		  subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 +   $
					median(meas2.flag(ipix,indok))^2 )) 

		ENDIF ELSE IF ncount EQ 1 THEN BEGIN

	   	  subt_meas.mnpw(ipix,*) = meas1.mnpw(ipix,*) - $
	   	  			aper_norm(ipix) * meas2.mnpw(ipix,indok)
	   	  subt_meas.medi(ipix,*) = meas1.medi(ipix,*) - $
	   	 			aper_norm(ipix) * meas2.medi(ipix,indok) 
	   	  subt_meas.q1pw(ipix,*) = meas1.q1pw(ipix,*) - $
	   	 			aper_norm(ipix) * meas2.q1pw(ipix,indok) 
	   	  subt_meas.q3pw(ipix,*) = meas1.q3pw(ipix,*) - $
	   	 			aper_norm(ipix) * meas2.q3pw(ipix,indok) 
	   	  subt_meas.mnpu(ipix,*) = SQRT( meas1.mnpu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*meas2.mnpu(ipix,indok))^2  + $
	   		      (aper_norm_unc(ipix) * meas2.mnpw(ipix,indok))^2) )
	   		      
		  subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					meas2.flag(ipix,indok)^2 )) 
		   
		ENDIF ELSE BEGIN
		  subt_meas.mnpw(ipix,*) = 0.
		  subt_meas.mnpu(ipix,*) = 0.
		  subt_meas.q1pw(ipix,*) = 0.
		  subt_meas.q3pw(ipix,*) = 0.
		  subt_meas.mdpw(ipix,*) = 0.
		  subt_meas.flag(ipix,*) = 2. 
		ENDELSE
	   ENDFOR
	  ENDELSE 
	   

          ; Update header
          newhdr = meas1.hdr
	  FXADDPAR, newhdr, 'PRD_SUBM', 1, 'SPD level meas subtraction', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRD_SM_S',meas1.admn,'Source Measurement', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRD_SM_B',meas2.admn,'Background Measurement', $
	       BEFORE='XTENSION' 

	  ;change the admn name
	  subt_meas.ADMN = subt_meas.ADMN + 'S'	

          ; change the user name in the header
	  FXADDPAR,newhdr, 'PIA_MNAM', subt_meas.ADMN, $
                "Name chosen for measurement by PIA user", BEFORE='XTENSION'


	  ;store the temporary structure in the PHTSPD structure 
	  INDPHT = N_ELEMENTS(phtspd)
	  FILL_SPD, subt_meas, newhdr
	  ;update the header of temporary structure
	  change_tmphdr, subt_meas, 'SPD', newhdr, subt_meas

              
	   decode_admn, subt_meas.admn, decnam
	  IF NOT KEYWORD_SET(NO_WIDGET) THEN BEGIN 
                XDISPLAYFILE, "", $
                     TITLE = "Measurement in Buffer", $
                     WIDTH = 50, $
                     HEIGHT = 5, $
                     TEXT = ["The subtracted measurement is now in the ", $
                         "SPD buffer as: " + decnam ]

	  ENDIF	     
	
		END
		
;  AAP part
	KEYWORD_SET(AAP): BEGIN

 	  IF N_ELEMENTS(meas2.mnfl(0,*)) EQ 1 THEN BEGIN
	    npix = detpix
	    FOR Ipix = 0, npix-1 DO BEGIN
	   	subt_meas.mnfl(ipix,*) = meas1.mnfl(ipix,*) - $
	   				aper_norm(ipix) * meas2.mnfl(ipix,*) 
	   	subt_meas.mdfl(ipix,*) = meas1.mdfl(ipix,*) - $
	   				aper_norm(ipix) * meas2.mdfl(ipix,*) 
	   	subt_meas.q1fl(ipix,*) = meas1.q1fl(ipix,*) - $
	   				aper_norm(ipix) * meas2.q1fl(ipix,*) 
	   	subt_meas.q3fl(ipix,*) = meas1.q3fl(ipix,*) - $
	   				aper_norm(ipix) * meas2.q3fl(ipix,*) 
	   	subt_meas.mnfu(ipix,*) = SQRT( meas1.mnfu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*meas2.mnfu(ipix,*))^2  + $
	   		      (aper_norm_unc(ipix) * meas2.mnfl(ipix,*))^2) )
	   	
	   	subt_meas.mnbr(ipix,*) = meas1.mnbr(ipix,*) - $
	   				aper_norm(ipix) * meas2.mnbr(ipix,*) 
	   	subt_meas.mdbr(ipix,*) = meas1.mdbr(ipix,*) - $
	   				aper_norm(ipix) * meas2.mdbr(ipix,*) 
	   	subt_meas.q1br(ipix,*) = meas1.q1br(ipix,*) - $
	   				aper_norm(ipix) * meas2.q1br(ipix,*) 
	   	subt_meas.q3br(ipix,*) = meas1.q3br(ipix,*) - $
	   				aper_norm(ipix) * meas2.q3br(ipix,*) 
	   	subt_meas.mnbu(ipix,*) = SQRT( meas1.mnbu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*meas2.mnbu(ipix,*))^2  + $
	   		      (aper_norm_unc(ipix) * meas2.mnbr(ipix,*))^2) )
	   		      
		subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					meas2.flag(ipix,*)^2 )) 
	    ENDFOR
	  ENDIF ELSE BEGIN
	    npix = detpix
	    FOR Ipix = 0, npix-1 DO BEGIN
		indok = WHERE(meas2.flag(ipix,*) LT 2, ncount)
		IF ncount GT 1 THEN BEGIN
		  weight_mean, meas2.mnfl(ipix,indok), meas2.mnfu(ipix,indok),$
		 		mnfl, mnfu
	   	  subt_meas.mnfl(ipix,*) = meas1.mnfl(ipix,*) - $ 
		        aper_norm(ipix) * mnfl
	   	 			
	   	  subt_meas.mdfl(ipix,*) = meas1.mdfl(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.mdfl(ipix,indok)) 
	   	  subt_meas.q1fl(ipix,*) = meas1.q1fl(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.q1fl(ipix,indok)) 
	   	  subt_meas.q3fl(ipix,*) = meas1.q3fl(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.q3fl(ipix,indok)) 
	   	  subt_meas.mnfu(ipix,*) = SQRT( meas1.mnfu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*mnfu)^2  + $
	   		      (aper_norm_unc(ipix) * mnfl)^2) )

		  subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					median(meas2.flag(ipix,indok))^2 )) 

		  weight_mean, meas2.mnbr(ipix,indok), meas2.mnbu(ipix,indok),$
		 		mnbr, mnbu
	   	  subt_meas.mnbr(ipix,*) = meas1.mnbr(ipix,*) - $ 
		        aper_norm(ipix) * mnbr

	   	  subt_meas.mnbu(ipix,*) = SQRT( meas1.mnbu(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*mnbu)^2  + $
	   		      (aper_norm_unc(ipix) * mnbr)^2) )
	   	  subt_meas.mdbr(ipix,*) = meas1.mdbr(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.mdbr(ipix,indok)) 
	   	  subt_meas.q1br(ipix,*) = meas1.q1br(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.q1br(ipix,indok)) 
	   	  subt_meas.q3br(ipix,*) = meas1.q3br(ipix,*) - $
	   	 	aper_norm(ipix) * median(meas2.q3br(ipix,indok)) 

		ENDIF ELSE IF ncount EQ 1 THEN BEGIN

	   	  subt_meas.mnfl(ipix,*) = meas1.mnfl(ipix,*) - $
	   	  	aper_norm(ipix) * meas2.mnfl(ipix,indok)
	   	  subt_meas.medi(ipix,*) = meas1.medi(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.medi(ipix,indok) 
	   	  subt_meas.q1fl(ipix,*) = meas1.q1fl(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.q1fl(ipix,indok) 
	   	  subt_meas.q3fl(ipix,*) = meas1.q3fl(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.q3fl(ipix,indok) 
	   	  subt_meas.mnfu(ipix,*) = SQRT( meas1.mnfu(ipix,*)^2 + $
	   		   ( (aper_norm(ipix)*meas2.mnfu(ipix,indok))^2  + $
	   		    (aper_norm_unc(ipix) * meas2.mnfl(ipix,indok))^2) )
		  subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					meas2.flag(ipix,indok)^2 )) 
					
	   	  subt_meas.mnbr(ipix,*) = meas1.mnbr(ipix,*) - $
	   	  	aper_norm(ipix) * meas2.mnbr(ipix,indok)
	   	  subt_meas.medi(ipix,*) = meas1.medi(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.medi(ipix,indok) 
	   	  subt_meas.q1br(ipix,*) = meas1.q1br(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.q1br(ipix,indok) 
	   	  subt_meas.q3br(ipix,*) = meas1.q3br(ipix,*) - $
	   	 	aper_norm(ipix) * meas2.q3br(ipix,indok) 
	   	  subt_meas.mnbu(ipix,*) = SQRT( meas1.mnbu(ipix,*)^2 + $
	   		  ( (aper_norm(ipix)*meas2.mnbu(ipix,indok))^2  + $
	   		    (aper_norm_unc(ipix) * meas2.mnbr(ipix,indok))^2) )
		   
		ENDIF ELSE BEGIN
		  subt_meas.mnfl(ipix,*) = 0.
		  subt_meas.mnfu(ipix,*) = 0.
		  subt_meas.q1fl(ipix,*) = 0.
		  subt_meas.q3fl(ipix,*) = 0.
		  subt_meas.mdfl(ipix,*) = 0.
		  subt_meas.mnbr(ipix,*) = 0.
		  subt_meas.mnfu(ipix,*) = 0.
		  subt_meas.q1br(ipix,*) = 0.
		  subt_meas.q3br(ipix,*) = 0.
		  subt_meas.mdbr(ipix,*) = 0.
		  subt_meas.flag(ipix,*) = 2. 
		ENDELSE
	   ENDFOR
	  ENDELSE
	   

          ; Update header
          newhdr = meas1.hdr
	  FXADDPAR, newhdr, 'PRA_SUBM', 1, 'AAP level meas subtraction', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRA_SM_S',meas1.admn,'Source Measurement', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRA_SM_B',meas2.admn,'Background Measurement', $
	       BEFORE='XTENSION' 

	  ;change the admn name
	  subt_meas.ADMN = subt_meas.ADMN + 'S'	

          ; change the user name in the header
	  FXADDPAR,newhdr, 'PIA_MNAM', subt_meas.ADMN, $
                "Name chosen for measurement by PIA user", BEFORE='XTENSION'


	  ; store the temporary structure in the PHTAAP structure 
	  INDPHT = N_ELEMENTS(phtaap)
	  FILL_AAP, subt_meas, newhdr
  	  ;update the header of temporary structure
	  change_tmphdr, subt_meas, 'AAP', newhdr, subt_meas


	   decode_admn, subt_meas.admn, decnam
	   IF NOT KEYWORD_SET(NO_WIDGET) THEN BEGIN 
             XDISPLAYFILE, "", $
                         TITLE = "Measurement in Buffer", $
                         WIDTH = 50, $
                         HEIGHT = 5, $
             TEXT = ["The subtracted measurement is now in the ", $
                     "AAP buffer as: " + decnam ]
	   ENDIF	     

		END
	
	ELSE: BEGIN	
;SCP part	

           ; There are several possibilities for the subtraction: 
           ;	always: source measurement --> fltarr(npix,n)
           ; 1st: Measurement to be subtracted is staring --> fltarr(npix,1) /\>
           ;	   subtract same value to all n elements, output fltarr(npix,n)
           ; 2nd: Measurement to be subtracted is 
	   ;        fcs-chopped --> fltarr(npix,m) /\>
           ;	 in this case meas should be processed to SPD and then subtract

 	  IF N_ELEMENTS(meas2.aver(0,*)) EQ 1 THEN BEGIN
	    npix = detpix
	    FOR Ipix = 0, npix-1 DO BEGIN
	   	subt_meas.aver(ipix,*) = meas1.aver(ipix,*) - $
	   				 aper_norm(ipix) * meas2.aver(ipix,*) 
	   	subt_meas.medi(ipix,*) = meas1.medi(ipix,*) - $
	   				 aper_norm(ipix) * meas2.medi(ipix,*) 
	   	subt_meas.q1me(ipix,*) = meas1.q1me(ipix,*) - $
	   				 aper_norm(ipix) * meas2.q1me(ipix,*) 
	   	subt_meas.q3me(ipix,*) = meas1.q3me(ipix,*) - $
	   				 aper_norm(ipix) * meas2.q3me(ipix,*) 
	   	subt_meas.avun(ipix,*) = SQRT( meas1.avun(ipix,*)^2 + $
	   		    ( (aper_norm(ipix)*meas2.avun(ipix,*))^2  + $
	   		      (aper_norm_unc(ipix) * meas2.aver(ipix,*))^2) )
	   		      
		subt_meas.flag(ipix,*) = FIX(SQRT(meas1.flag(ipix,*)^2 + $
					(meas2.flag(ipix,*))^2 )) 
	    ENDFOR
	  ENDIF ELSE BEGIN
	   MESSAGE, /INFO, $
	    'Have still to decide how to implement a subtraction of this kind'
	   RETURN
	  ENDELSE
	   
          ; Update header
          newhdr = meas1.hdr
	  FXADDPAR, newhdr, 'PRC_SUBM', 1, 'SCP level meas subtraction', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRC_SM_S',meas1.admn,'Source Measurement', $
	       BEFORE='XTENSION' 
	  FXADDPAR, newhdr, 'PRC_SM_B',meas2.admn,'Background Measurement', $
	       BEFORE='XTENSION' 

	  ;change the admn name
	  subt_meas.ADMN = subt_meas.ADMN + 'S'	

          ; change the user name in the header
	  FXADDPAR,newhdr, 'PIA_MNAM', subt_meas.ADMN, $
                "Name chosen for measurement by PIA user", BEFORE='XTENSION'
		
	  ;store the temporary structure in the PHTSCP structure 
	  INDPHT = N_ELEMENTS(phtscp)
          FILL_SCP, subt_meas, newhdr
	  ;update the header of temporary structure
	  change_tmphdr, subt_meas, 'SCP', newhdr, subt_meas
	  

	   decode_admn, subt_meas.admn, decnam
	  IF NOT KEYWORD_SET(NO_WIDGET) THEN BEGIN 
              XDISPLAYFILE, "", $
                   TITLE = "Measurement in Buffer", $
                   WIDTH = 50, $
                   HEIGHT = 5, $
                   TEXT = ["The subtracted measurement is now in the ", $
                     "SCP buffer as: " + decnam ]
	  ENDIF	  
	  
	END
	     
	ENDCASE


RETURN
END
                           

