;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO TMEAS_CONV, RAW_TMP, DET_MODE, CONV_TMP
;+
; NAME:
;	TMEAS_CONV, RAW_TMP, DET_MODE, CONV_TMP
; PURPOSE:
;  To convert measured temperatures from telemetry
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	TMEAS_CONV, RAW_TMP, DET_MODE, CONV_TMP
;
; INPUT:
;  RAW_TMP  - integer, raw (telemetered) temperature(s)
;  DET_MODE - char str, the modeword for this subsystem
;
; OUTPUT:
;	CONV_TMP - real, converted temperature(s)
;
; DESCRIPTION:
;  Uses the subsystem-dependent conversion parameters to
;  convert the raw telemetered detector temperature values
;  using the algorithm defined in "Converting Algorithms for
;  Telemetry Data" (TN 2096-8100DS/01).
;
;              Namely : T [ `C] =             B
;                                    -----------------------------
;                                    log10(Rs) + (K /log10 Rs) - A
;
;              where
;
;                       Rs [Ohm] = 1000 * [          1              - Rk]
;                                           -----------------------
;                                           [      1           -  1 ]
;                                            ----------------    ---
;                                             C1 * TMDATA - C2    Rp
;
;  The quantities A, B, K, C1, C2, RP_1 (1/Rp)  and Rk
;  are accessed from an array using the modeword as a key.
;
;
; PROCEDURES USED:
;	NONE
;
; MODIFICATION HISTORY:
;	Written by: W. M. Tai
;	Oct 95 Version 3.5
;-
;
;
	ON_ERROR, 2
        IF N_PARAMS() LT 1 THEN MESSAGE, $
          'Syntax:  TMEAS_CONV, RAW_TMP, N_TMPS, MODE, CONV_TMP'

;  For tables A, B and K, they are based on FM - FPU
   A = [6.01522233, 7.58937051, 5.10725437, 6.53685466, 6.54013656, $
        5.43967173, 4.51090074]
	B = [3.61029820, 3.12832510, 3.49330104, 2.94105591, 2.90355780, $
		  3.05199111, 3.11645880]
	K = [8.77235310, 14.39802964, 6.29231315, 10.65720895, 10.66791386, $
		  7.33580094, 4.91872389]
;
	C1= [0.1742,  0.2099,  0.1819,  0.1728,  0.1704,  $
		  0.0989, 0.0483]
	C2= [344.5515, 388.1270, 288.1145, 341.6283, 336.8819, $
		  146.3797, -78.3216]
	RK = [1.12, 1.34, 1.34, 1.12, 1.12, 1.34, 0.67]
	RP_1 = [0.002, 0.002, 0.0, 0.002, 0.002, 0.0, 0.0]

;  Here we pass the modeword in char str to the corresponding table index
   CASE DET_MODE OF
	  'P1': MODE = 0
	  'P2': MODE = 1
	  'P3': MODE = 2
	  'SS': MODE = 3
	  'SL': MODE = 4
	  'C1': MODE = 5
	  'C2': MODE = 6
	ENDCASE
	
;  Calculate sensor resistance as intermediate to converted temperature
   RS = 1000*(1/(1/( C1(MODE)*RAW_TMP - C2(MODE) ) - RP_1(MODE)) - RK(MODE))
	LOG_RS = ALOG10(RS)

;  Use RS to calculate converted temperature
   CONV_TMP = B(MODE) / ( LOG_RS + K(MODE)/LOG_RS - A(MODE) )

	END
