;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO Vignet, SubInst, In_Slop, In_Slun, Chop, Fcsn, Pixel, Filter, $
	            Out_Slop, Out_Slun, FCORRECTION= fcorrection

;+
; Name:
;	Vignet
;
; Purpose:
;	To correct for chopper amplitude dependent vignetting
; CATEGORY:
;	PIA - processing
;
; Input:
;	SubInst: PHT Sub-instrument: One of P1, P2, P3, C1, C2, SL
;	In_Slop: Array of uncorrected averaged slopes for plateaux
;	In_Slun: Corresponding array of uncertainties associated with each
;                slope
;	Chop: Chopper position for each slope
;	Fcsn: FCS active for each slope
;	Pixel: Pixel number for PHT-C100 (0 - 8), PHT-C200 (0 - 3)
;	       Aperture number for PHT-P (0 - 13)
;	Filter: Filter position number (1 - Max_filter)
;
; Output:
;	Out_Slop: Array of corrected averaged slopes for plateaux
;	Out_Slun: Corresponding array of uncertainties associated with each
;	          slope
;
; KEYWORDS:
;	FCORRECTION: the computed correction factors (same dimm as in_slop)
;
; Restrictions:
;	All input arrays must be of the same size
;	The vignetting correction tables must have been read in first
;
; Procedures Used:
;       Common Block(s) or @ procedure(s):
;               phtvignet
;
; Modification History:
;	Written by: Huw Morris
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;	
;-
 


@phtvignet

	ON_ERROR, 2             ; Return to calling procedure

; In case of an early exit from this procedure set up the output arrays
; as being equal to the input arrays
	Out_Slop = In_Slop
	Out_Slun = In_Slun

	Status = 'ok'            ; No errors yet

; Make sure In_Slop, In_Slun, and Chop are all the same size
	Num_Slop = N_ELEMENTS(In_Slop)
	Num_Slun = N_ELEMENTS(In_Slun)
	Num_Chop = N_ELEMENTS(Chop)

	Max_Num = MAX([Num_Slop, Num_Slun, Num_Chop])
	Min_Num = MIN([Num_Slop, Num_Slun, Num_Chop])

; Exit from this routine if there is an error
	IF (Max_Num NE Min_Num) THEN BEGIN
	  MESSAGE, 'Input arrays not of same size'
	  RETURN
	ENDIF

; initialize vigneting correction tables if not yet done
IF N_ELEMENTS(pc2vign) EQ 0 THEN vignet_tables_read


; Find the correct table for the subinstrument
	CASE SubInst OF

	  'P1': BEGIN
; Filter values 1-10
	    IF (Filter LT 1 OR Filter GT 10) THEN BEGIN
	      Status = 'error'
	    ENDIF ELSE BEGIN
	      Temp_Filter = Filter - 1
	    ENDELSE
	    VTable = PP1Vign
	    Max_Pix = 13
	  END

	  'P2': BEGIN
; Filter values 11-12
	    IF (Filter LT 11 OR Filter GT 12) THEN BEGIN
	      Status = 'error'
	    ENDIF ELSE BEGIN
	      Temp_Filter = Filter - 11
	    ENDELSE
	    VTable = PP2Vign
	    Max_Pix = 13
	  END

	  'P3': BEGIN
; Filter values 13-14
	    IF (Filter LT 13 OR Filter GT 14) THEN BEGIN
	      Status = 'error'
	    ENDIF ELSE BEGIN
	      Temp_Filter = Filter - 13
	    ENDELSE
	    VTable = PP3Vign
	    Max_Pix = 13
	  END

	  'C1': BEGIN
; Filter values 2-8                    ; C100_OPEN not counted
	    IF (Filter LT 2 OR Filter GT 8) THEN BEGIN
	      Status = 'error'
	    ENDIF ELSE BEGIN
	      Temp_Filter = Filter - 2
	    ENDELSE
	    VTable = PC1Vign
	    Max_Pix = 8
	  END

	  'C2': BEGIN
; Filter values 9-13                    ; C200_RETICLE not counted
	    IF (Filter LT 9 OR Filter GT 13) THEN BEGIN
	      Status = 'error'
	    ENDIF ELSE BEGIN
	      Temp_Filter = Filter - 9
	    ENDELSE
	    VTable = PC2Vign
	    Max_Pix = 3
	  END
	
	  'SL': RETURN                    ; No vignetting for PHT-S

	  ELSE: BEGIN
	    MESSAGE, 'Subinstrument not known'
	    RETURN
	  END

	ENDCASE

; Check the filter is in range
	IF (Status EQ 'error') THEN BEGIN
	  MESSAGE, 'Filter out of range'
	  RETURN
	ENDIF

; Check the pixel number is in range
	IF (Pixel GT Max_Pix) THEN BEGIN
	  MESSAGE, 'Pixel out of range'
	  RETURN
	ENDIF


; Set up an array of chopper positions - each element corresponds to
; a column in the vignetting table
	Chop_Table = INDGEN(25)*15 - 180

; Set up an array of vignetting corrections for each chopper position
; in the above array
	Correction = FLTARR(25)
	Correction(*) = VTable(Pixel, *, Temp_Filter)

; Now interpolate the actual chopper positions against the table
	Vig_Correction = INTERPOL(Correction, Chop_Table, Chop) 

; Vignetting not applicable to FCS
	ind = WHERE(fcsn NE 0, nind)
	IF nind GT 0 THEN vig_correction(ind) = 1.
	
	Out_Slop = In_Slop * Vig_Correction
	Out_Slun = In_Slun * Vig_Correction
	
	FCORRECTION = vig_correction

	END
