;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	PRO WEIGHT_MEAN, DATA, UNC, MEAN, STDV, SIGMA, MEDIAN=MEDIAN, $ 
		SET_WEIGHT = set_weight, MINNR=minnr, BIWEIGHT=biweight 
;+
; NAME:
;	WEIGHT_MEAN
; PURPOSE:
;  To perform a weighted mean returning also the standard deviation
;
; CATEGORY:
;	PIA - processing
;
; CALLING SEQUENCE:
;	WEIGHT_MEAN, DATA, UNC, MEAN, [STDV], [SIGMA] [, /MEDIAN] $
;	 [,SET_WEIGTH = SET_WEIGHT] [,MINNR = MINNR] [,BIWEIGHT = BIWEIGHT]
;
; INPUT:
;  	DATA  - A one-dimensional array of values
;  	UNC   - An array of same dimensions as data with uncertainties
;
; OPTIONAL INPUT:
;	BIWEIGHT - If set, returns biweight mean instead of weighted mean 
;	MEDIAN - If set, returns median instead of weighted mean 
;	SET_WEIGHT - A vector defining the weighting factors
;       MINNR  - If set, the minimum number of points required to weight the
;              mean. By default, if less than 15 elements, then simple average
;              is taken, unless median or biweight has been chosen
;
; OUTPUT:
;	MEAN - The weighted mean
;
; OPTIONAL OUTPUT:
;	STDV - The standard deviation of the mean
;	SIGMA - The standard deviation of the distribution
;
; DESCRIPTION:
;
; PROCEDURES USED:
;	NONE
;
; RESTRICTIONS:
;	At least 15 points or MINNR are required for weighted mean, if less 
;	normal average is returned.
;
; MODIFICATION HISTORY:
;	Written by: C.Gabriel
;	Modified: Threshold and limit added - CG	December 1996 
;	Limit changed - CG				December 1996
;	V5.5
;	If no weight use weights=1. (CG)		June 1997
;	V6.4
;	SET_WEIGHT keyword introduced to use as proper 
;	weight (JA) 					July 98
;	Keyword MINNR added (CG)			September 1998
;	V7.3
;	Biweight call introduced via keyword (CG)	May 2002
;	V10.0
;-
;
;
	ON_ERROR, 2
        IF N_PARAMS() LT 3 THEN MESSAGE, $
	 'Syntax:  WEIGHT_MEAN, DATA, UNC, MEAN [,STDV][,SIGMA]'+$
	 '[,MEDIAN=median][SET_WEIGHT=set_weight][,MINNR=minnr]'+$
	 '[,BIWEIGHT=biweight]'

	IF NOT KEYWORD_SET (biweight) THEN biweight = 0
	IF NOT KEYWORD_SET (median) THEN median = 0
	IF NOT KEYWORD_SET (minnr) THEN minnr = 15
	
	IF N_ELEMENTS(set_weight) EQ 0 THEN nouse_unc = 0 ELSE nouse_unc = 1

;	
	defsysv, '!NO_WEIGHTS', exists=exi
	IF exi EQ 0 THEN BEGIN 
	  defsysv, '!NO_WEIGHTS', 0
	ENDIF  

	noweight = !NO_WEIGHTS

	IF NOT nouse_unc THEN BEGIN 
             ; Avoid unc = 0.
	     ind = where(unc EQ 0., nzero)
             IF nzero GT 0 THEN check_unc, data, unc
	ENDIF

; Set threshold to the uncertainties
	nunc = N_ELEMENTS(unc)
	CASE 1 OF
	  noweight EQ 1: weight = unc * 0. + 1.
	  nouse_unc EQ 1: weight = set_weight
	  nunc GE minnr: BEGIN 
		        sigunc = STDEV(unc, munc) 
	                ind = WHERE(unc LT (munc-2*sigunc), nsusp)
	                IF nsusp GT 0 THEN unc(ind) = munc
	                ; to use weights
 	                weight = 1. / (unc*unc)
		      END
	  ELSE: weight = unc * 0. + 1. 
	ENDCASE
	  
	totwei = TOTAL(weight)


	; depending on the number of data, use different ways to derive 
	; mean. 
	; #data > 4 : use weighted mean or median if required
	; 1< #data <= 4: use weigthed mean, even if median is required
	; #data = 1 : set mean to the value 

	nelem = N_ELEMENTS(data)	
	CASE 1 OF 
	  nelem GT 4: BEGIN  
	               IF biweight THEN BEGIN 
	                 mean = biweight_mean(data, bsig)
	                 dif = data - mean
		         sigma = bsig 	          
			 stdv = sigma/sqrt(nelem-1)
	               ENDIF ELSE IF median THEN BEGIN 
	                 mean = median(data)
	                 dif = data - mean
		         sigma = sqrt(TOTAL(weight*dif*dif)/totwei)	          
			 stdv = sigma/sqrt(nelem-1)
	               ENDIF ELSE BEGIN 
	                 mean = TOTAL(weight * data) / totwei
	                 dif = data - mean
		         sigma = sqrt(TOTAL(weight*dif*dif)/totwei)
		         stdv = sigma/sqrt(nelem-1)
		       ENDELSE  		
	              END
          nelem GT 1: BEGIN
	               mean = TOTAL(weight * data) / totwei
	               dif = data - mean
		       sigma = sqrt(TOTAL(weight*dif*dif)/totwei)
		       stdv = sigma/sqrt(nelem-1)
	              END
          ELSE:       BEGIN 
      		       MEAN = DATA(0)
		       STDV = UNC(0)
		       SIGMA = UNC(0)
	              END
	ENDCASE
		

	RETURN
	END
