;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   PRO write_aap_multi, list_meas, accvector=accvector, filename=filename
;+
; NAME:
;       WRITE_AAP_MULTI
; PURPOSE:
;       Write a FITS binary table from one or multiple PHT-AAP 
;	measurements of any or all types of PHT-C, PHT-P or PHT-S.  
;	
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       WRITE_AAP_MULTI, LIST_MEAS [, ACCVECTOR=ACCVECTOR] [, FILENAME=FILENAME]
;
; INPUT:
;       list_meas: list of selected measurement positions from the buffer
; 
; OPTIONAL INPUT:
;
; OPTIONAL KEYWORD:
;	accvector: acceptance vector for each field
;		   (refer to the variable defined below, ccol_comments)
;
;	filename: name of the FITS file including directory (default=!intdir)
;		  (If not specified, it uses date and time for the name in case of
;		  multiple measurements or uses the detector id and two digit meas.
;		  number followed by the TDT for a sigle measurement.)
;
; OUTPUT: 
;	A binary FITS file in the working directory
;
; RESTRICTIONS:
;
; PROCEDURES USED: 
;	pia_load_iaap
;	dec_admn (function)
;	
;       Common Block(s) or @ procedure(s):
;		phtaap
;		phtslwavlen
;
; MODIFICATION HISTORY:
;       Written by: Min Y. Hur	(IPAC/Caltech)			May 2000
;	V9.0
;
;-
@phtaap
@phtslwavlen	; common block with SL wavelengths 
		; (first 64 for SS, last 64 for SL)
		; name for the variable is "slwavelen"

ON_ERROR, 2

IF not KEYWORD_SET(filename) THEN BEGIN
	now=systime(0)
	month=strmid(now,4,3)
	day='00'
	if strmid(now,8,1) eq ' ' then day='0'+strmid(now,9,1) $
	   else day=strmid(now,8,2)
  fitsfile=month+day+strmid(now,11,2)+strmid(now,14,2)+'_AAP.FITS'
  filename = !INTDIR+fitsfile
ENDIF ELSE BEGIN
	  revers_name = STR_REVERSE(filename)
	  IF !version.os EQ "VMS" THEN $
	    ind = STRPOS(revers_name,']') ELSE $
	    ind = STRPOS(revers_name,'/') 
	  IF ind eq -1 THEN fitsfile = revers_name ELSE $ 	    
	    fitsfile = STRMID(revers_name,0,ind)
	  fitsfile = STR_REVERSE(fitsfile)
ENDELSE  

num_meas=n_elements(list_meas)

IF not KEYWORD_SET(accvector) THEN $
accvector=intarr(18)+1	

num_fields=n_elements(where(accvector ne 0))

; length = the length of the binary table or the number of rows
length=intarr(num_meas)
name=strarr(num_meas)
FOR n=0,num_meas-1 DO BEGIN
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   name(n)=dec_admn(tmpdata.admn)
   length(n)=n_elements(tmpdata.mnbr)
   IF num_meas eq 1 THEN BEGIN
;      fitsfile=tmpdata.admn+'_AAP.FITS'
;      filename=!INTDIR+fitsfile
      onehdr=tmpdata.hdr
   ENDIF
ENDFOR
tot_length=total(length)
   	
columns=fltarr(num_fields,tot_length)

; Measurement ID number
IF accvector(0) eq 1 THEN BEGIN
   meas_id=intarr(length(0))+1
   if num_meas gt 0 then for n=1,num_meas-1 do $
      meas_id=[meas_id,intarr(length(n))+n+1]
   columns(0,*)=meas_id
ENDIF
   
; Pixel number identification
IF accvector(1) eq 1 THEN BEGIN
n=0
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   dims=tmpdata.dims
   if det eq 'C1' then pixel=indgen(9)+1
   if det eq 'C2' then pixel=indgen(4)+1
   if det eq 'SS' then pixel=indgen(64)+1
   if det eq 'SL' then pixel=indgen(64)+65
   if strmid(det,0,1) eq 'P' then pixel=1
   pixel_id=intarr(length(n))
   for i=0,length(n)-1 do pixel_id(i)=pixel(i/dims)
   pixel_no=pixel_id
if num_meas ge 1 then for n=1,num_meas-1 do begin
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   dims=tmpdata.dims
   if det eq 'C1' then pixel=indgen(9)+1
   if det eq 'C2' then pixel=indgen(4)+1
   if det eq 'SS' then pixel=indgen(64)+1
   if det eq 'SL' then pixel=indgen(64)+65
   if strmid(det,0,1) eq 'P' then pixel=1
   pixel_id=intarr(length(n))
   for i=0,length(n)-1 do pixel_id(i)=pixel(i/dims)
   pixel_no=[pixel_no,pixel_id]
endfor
columns(fix(total(accvector(0:1)))-1,*)=pixel_no
ENDIF

	; apertures in " only for info
	p_asy = [ 0.0,  0.0,  0.0,   0.0, 127.0,   0.0,   0.0, $
		0.0, 0.0,  0.0,  0.0,  0.0, 32.0,  0.0]
	p_asz = [52.0, 79.0, 99.0, 120.0, 127.0, 180.0, 180.0, $
		5.0, 7.6, 10.0, 13.8, 18.0, 20.0, 23.0]

; Aperture_Y : zero if circular
; Aperture_Z : diameter if Aperture_Y=0
n=0
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   apery_ind=fltarr(length(n))
   aperz_ind=fltarr(length(n))
   if strmid(det,0,1) eq 'S' then begin
      apery_ind(*)=24.0
      aperz_ind(*)=24.0
   endif
   if det eq 'C1' then begin
      apery_ind(*)=43.5
      aperz_ind(*)=43.5
   endif
   if det eq 'C2' then begin
      apery_ind(*)=89.4
      aperz_ind(*)=89.4
   endif
   if strmid(det,0,1) eq 'P' then begin
      aindex=tmpdata.aper
      apery_ind(*)=p_asy(aindex - 1)
      aperz_ind(*)=p_asz(aindex - 1)
   endif
   aper_y=apery_ind
   aper_z=aperz_ind
if num_meas ge 1 then for n=1,num_meas-1 do begin
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   apery_ind=fltarr(length(n))
   aperz_ind=fltarr(length(n))
   if strmid(det,0,1) eq 'S' then begin
      apery_ind(*)=24.0
      aperz_ind(*)=24.0
   endif
   if det eq 'C1' then begin
      apery_ind(*)=43.5
      aperz_ind(*)=43.5
   endif
   if det eq 'C2' then begin
      apery_ind(*)=89.4
      aperz_ind(*)=89.4
   endif
   if strmid(det,0,1) eq 'P' then begin
      aindex=tmpdata.aper
      apery_ind(*)=p_asy(aindex - 1)
      aperz_ind(*)=p_asz(aindex - 1)
   endif
   aper_y=[aper_y,apery_ind]
   aper_z=[aper_z,aperz_ind]
endfor   
IF accvector(2) eq 1 THEN columns(fix(total(accvector(0:2)))-1,*)=aper_y
IF accvector(3) eq 1 THEN columns(fix(total(accvector(0:3)))-1,*)=aper_z

; Wavelength
IF accvector(4) eq 1 THEN BEGIN
n=0
   wave_ind=fltarr(length(n))
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   if det eq 'SS' then $
      for i=0,length(n)-1 do wave_ind(i)=slwavelen(i-(i/64)*64)
   if det eq 'SL' then $
      for i=0,length(n)-1 do wave_ind(i)=slwavelen(i-(i/64)*64+64)
   if strmid(det,0,1) ne 'S' then $
      wave_ind(*)=tmpdata.cwvl
   wavelen=wave_ind
if num_meas ge 1 then for n=1,num_meas-1 do begin
   wave_ind=fltarr(length(n))
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det =tmpdata.det
   if det eq 'SS' then $
      for i=0,length(n)-1 do wave_ind(i)=slwavelen(i-(i/64)*64)
   if det eq 'SL' then $
      for i=0,length(n)-1 do wave_ind(i)=slwavelen(i-(i/64)*64+64)
   if strmid(det,0,1) ne 'S' then $
      wave_ind(*)=tmpdata.cwvl
   wavelen=[wavelen,wave_ind]
endfor
columns(fix(total(accvector(0:4)))-1,*)=wavelen
ENDIF

; Delta_Wavelength
IF accvector(5) eq 1 THEN BEGIN

	pht_p_width = [0.22, 1., 1.55, 3.38, 0.84, 1.86, 0.77, 6.51, $
			2.33, 2.86, 9.43, 9.18, 25.48, 40.15] / 2
			
	pht_c_width = [-2., 51.4, 57.8, 23.9, 49.5, 43.6, 37.4, 51.4, $
			89.4, 67.3, 71.7,  82.5, 47.3] / 2
		    ; the first element is -1.0 because there's no filter 1 for C
		    
	delta_slwave=fltarr(128)
	for i=0,126 do delta_slwave(i)=(slwavelen(i+1)-slwavelen(i))/2.0
	delta_slwave(63)=delta_slwave(62)
	delta_slwave(127)=delta_slwave(126)			
n=0
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   filt=tmpdata.filt
   delta_ind=fltarr(length(n))
   if strmid(det,0,1) eq 'P' then delta_ind(*)=pht_p_width(filt-1)
   if strmid(det,0,1) eq 'C' then delta_ind(*)=pht_c_width(filt-1)
   if det eq 'SS' then $
      for i=0,length(n)-1 do delta_ind(i)=delta_slwave(i-(i/64)*64)
   if det eq 'SL' then $
      for i=0,length(n)-1 do delta_ind(i)=delta_slwave(i-(i/64)*64+64)
   delta_wave=delta_ind		
if num_meas ge 1 then for n=1,num_meas-1 do begin
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   det=tmpdata.det
   filt=tmpdata.filt
   delta_ind=fltarr(length(n))
   if strmid(det,0,1) eq 'P' then delta_ind(*)=pht_p_width(filt-1)
   if strmid(det,0,1) eq 'C' then delta_ind(*)=pht_c_width(filt-1)
   if det eq 'SS' then $
      for i=0,length(n)-1 do delta_ind(i)=delta_slwave(i-(i/64)*64)
   if det eq 'SL' then $
      for i=0,length(n)-1 do delta_ind(i)=delta_slwave(i-(i/64)*64+64)
   delta_wave=[delta_wave,delta_ind]	
endfor
columns(fix(total(accvector(0:5)))-1,*)=delta_wave
ENDIF

; MNFL, MNFU, MNBR, MNBU, and Flag
; Declination and Right Ascension per point
n=0
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   mnfl2=tmpdata.mnfl
   mnfu2=tmpdata.mnfu
   mdfl2=tmpdata.mdfl
   mnbr2=tmpdata.mnbr
   mnbu2=tmpdata.mnbu
   mdbr2=tmpdata.mdbr
   decp=tmpdata.decp
   rap=tmpdata.rap
   flag2=tmpdata.flag
   dims=tmpdata.dims
   mnfl_ind=dblarr(length(n))
   mnfu_ind=dblarr(length(n))
   mdfl_ind=dblarr(length(n))
   mnbr_ind=dblarr(length(n))
   mnbu_ind=dblarr(length(n))
   mdbr_ind=dblarr(length(n))
   decp_ind=dblarr(length(n))
   rap_ind=dblarr(length(n))
   flag_ind=intarr(length(n))
   for i=0,length(n)-1 do begin
      mnfl_ind(i)=mnfl2(i/dims,i-(i/dims)*dims)
      mnfu_ind(i)=mnfu2(i/dims,i-(i/dims)*dims)
      mdfl_ind(i)=mdfl2(i/dims,i-(i/dims)*dims)
      mnbr_ind(i)=mnbr2(i/dims,i-(i/dims)*dims)
      mnbu_ind(i)=mnbu2(i/dims,i-(i/dims)*dims)
      mdbr_ind(i)=mdbr2(i/dims,i-(i/dims)*dims)
      decp_ind(i)=decp(i/dims,i-(i/dims)*dims)
      rap_ind(i) = rap(i/dims,i-(i/dims)*dims)
      flag_ind(i)=flag2(i/dims,i-(i/dims)*dims)
   endfor
   mnfl=mnfl_ind
   mnfu=mnfu_ind
   mdfl=mdfl_ind
   mnbr=mnbr_ind
   mnbu=mnbu_ind
   mdbr=mdbr_ind
   dec=decp_ind
   ra=rap_ind
   flag=flag_ind
if num_meas ge 1 then for n=1,num_meas-1 do begin
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   mnfl2=tmpdata.mnfl
   mnfu2=tmpdata.mnfu
   mdfl2=tmpdata.mdfl
   mnbr2=tmpdata.mnbr
   mnbu2=tmpdata.mnbu
   mdbr2=tmpdata.mdbr
   decp=tmpdata.decp
   rap=tmpdata.rap
   flag2=tmpdata.flag
   dims=tmpdata.dims
   mnfl_ind=dblarr(length(n))
   mnfu_ind=dblarr(length(n))
   mdfl_ind=dblarr(length(n))
   mnbr_ind=dblarr(length(n))
   mnbu_ind=dblarr(length(n))
   mdbr_ind=dblarr(length(n))
   decp_ind=dblarr(length(n))
   rap_ind=dblarr(length(n))
   flag_ind=intarr(length(n))
   for i=0,length(n)-1 do begin
      mnfl_ind(i)=mnfl2(i/dims,i-(i/dims)*dims)
      mnfu_ind(i)=mnfu2(i/dims,i-(i/dims)*dims)
      mdfl_ind(i)=mdfl2(i/dims,i-(i/dims)*dims)
      mnbr_ind(i)=mnbr2(i/dims,i-(i/dims)*dims)
      mnbu_ind(i)=mnbu2(i/dims,i-(i/dims)*dims)
      mdbr_ind(i)=mdbr2(i/dims,i-(i/dims)*dims)
      decp_ind(i)=decp(i/dims,i-(i/dims)*dims)
      rap_ind(i) = rap(i/dims,i-(i/dims)*dims)
      flag_ind(i)=flag2(i/dims,i-(i/dims)*dims)
   endfor
   mnfl=[mnfl,mnfl_ind]
   mnfu=[mnfu,mnfu_ind]
   mdfl=[mdfl,mdfl_ind]
   mnbr=[mnbr,mnbr_ind]
   mnbu=[mnbu,mnbu_ind]
   mdbr=[mdbr,mdbr_ind]
   dec=[dec,decp_ind]
   ra = [ra,rap_ind]
   flag=[flag,flag_ind]
endfor
IF accvector(6) eq 1 THEN columns(fix(total(accvector(0:6)))-1,*)=mnfl
IF accvector(7) eq 1 THEN columns(fix(total(accvector(0:7)))-1,*)=mnfu
IF accvector(8) eq 1 THEN columns(fix(total(accvector(0:8)))-1,*)=mdfl
IF accvector(9) eq 1 THEN columns(fix(total(accvector(0:9)))-1,*)=mnbr
IF accvector(10) eq 1 THEN columns(fix(total(accvector(0:10)))-1,*)=mnbu
IF accvector(11) eq 1 THEN columns(fix(total(accvector(0:11)))-1,*)=mdbr

; Raster point ID
; Roll angle
n=0
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   rpid=tmpdata.rpid
   rpidy=rpid(0,*)
   rpidz=rpid(1,*)
   roll2=tmpdata.roll
   dims=tmpdata.dims
   rpidy_ind=intarr(length(n))
   rpidz_ind=intarr(length(n))
   roll_ind=dblarr(length(n))
   for i=0,length(n)-1 do begin
      rpidy_ind(i)=rpidy(i-(i/dims)*dims)
      rpidz_ind(i)=rpidz(i-(i/dims)*dims)
      roll_ind(i)=roll2(i-(i/dims)*dims)
   endfor
   rpid_y=rpidy_ind
   rpid_z=rpidz_ind
   roll=roll_ind
if num_meas ge 1 then for n=1,num_meas-1 do begin
   pia_load_iaap,phtaap(list_meas(n)),tmpdata
   rpid=tmpdata.rpid
   rpidy=rpid(0,*)
   rpidz=rpid(1,*)
   roll2=tmpdata.roll
   dims=tmpdata.dims
   rpidy_ind=intarr(length(n))
   rpidz_ind=intarr(length(n))
   roll_ind=dblarr(length(n))
   for i=0,length(n)-1 do begin
      rpidy_ind(i)=rpidy(i-(i/dims)*dims)
      rpidz_ind(i)=rpidz(i-(i/dims)*dims)
      roll_ind(i)=roll2(i-(i/dims)*dims)
   endfor
   rpid_y=[rpid_y,rpidy_ind]
   rpid_z=[rpid_z,rpidz_ind]
   roll=[roll,roll_ind]
endfor
IF accvector(12) eq 1 THEN columns(fix(total(accvector(0:12)))-1,*)=rpid_y
IF accvector(13) eq 1 THEN columns(fix(total(accvector(0:13)))-1,*)=rpid_z
IF accvector(14) eq 1 THEN columns(fix(total(accvector(0:14)))-1,*)=dec
IF accvector(15) eq 1 THEN columns(fix(total(accvector(0:15)))-1,*)=ra
IF accvector(16) eq 1 THEN columns(fix(total(accvector(0:16)))-1,*)=roll
IF accvector(17) eq 1 THEN columns(fix(total(accvector(0:17)))-1,*)=flag


ccol_names=["Meas_ID", $	; 0
	   "Pixel_no",  $	; 1
	   "Aperture_Y", $	; 2
	   "Aperture_Z", $	; 3
	   "Wavelength", $	; 4
	   "Delt_Wave", $	; 5
	   "MNFL", $		; 6
	   "MNFU", $		; 7
	   "MDFL", $		; 8
	   "MNBR", $		; 9
	   "MNBU", $		; 10
	   "MDBR", $		; 11
	   "RPID_Y", $		; 12
	   "RPID_Z", $		; 13
	   "DEC", $		; 14
	   "RA", $		; 15
	   "ROLL", $		; 16
	   "FLAG"]		; 17
	   
col_names=ccol_names(where(accvector eq 1))
	   	
ccol_types = ["I", "I", "R", "R", "R", "R", "R", "R", "R", "R", "R", "R", "I", "I", "R", "R", "R", "I"]

col_types=ccol_types(where(accvector eq 1))

ccol_comments = ["Meas. ID - refer to the keywords MEASID#", $
		"Pixel number", $
		"Aperture_Y in arcsec (zero means a circular aperture)", $
		"Aperture_Z in arcsec (diameter if Aperture_Y is zero)", $
		"Wavelength in um", $
		"Uncertainty in wavelength in um", $
		"Mean flux in Jy", $
		"Mean flux uncertainty in Jy", $
		"Median flux in Jy", $
		"Mean brightness in MJy/sr", $
		"Mean brightness uncertainty in MJy/sr", $
		"Median brightness in MJy/sr", $
		"Raster point ID in Y", $
		"Raster point ID in Z", $
		"Declination in degrees", $
		"Right Ascension in degrees", $
		"Roll Angle in degrees", $
		"Flag"]
		
col_comments=ccol_comments(where(accvector eq 1))		
	   

fxhmake,  hdr, /extend, /date 

IF num_meas EQ 1 THEN phdr=onehdr ELSE phdr=hdr 	
	fxaddpar, phdr, 'ORIGIN','ESA',' European Space Agency'
	fxaddpar, phdr, 'TELESCOP','ISO',' Infrared Space Observatory'
	fxaddpar, phdr, 'INSTRUME','PHT',' Instrument used'
	fxaddpar, phdr, 'COMMENT',' File created with PIA'
	fxaddpar, phdr, 'FILENAME', fitsfile,' File name assigned in ISO archive'
	fxaddpar, phdr, 'PIA_VERS', !pia_version, ' Internal (user) PIA-CAL Version'

    end_no=['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', $
        '15', '16', '17', '18', '19', '20', '21', '22', '23', '24', '25', '26', $
        '27', '28', '29', '30', '31', '32', '33', '34', '35', '36', '37', '38', $
        '39', '40', '41', '42', '43', '44', '45', '46', '47', '48', '49', '50', $
        '51', '52', '53', '54', '55', '56', '57', '58', '59', '60', '61', '62', $
        '63', '64', '65', '66', '67', '68', '69', '70', '71', '72', '73', '74', $
        '75', '76', '77', '78', '79', '80', '81', '82', '83', '84', '85', '86', $
        '87', '88', '89', '90']

   if num_meas le 89 then for n=0,num_meas-1 do $
   	fxaddpar, phdr, 'MEASID'+end_no(n), name(n), $
	' '+end_no(n)+' : Det - TDT - Meas. #',before='PIA_VERS' else $
	fxaddpar, phdr, 'MEASID_','More measurements not listed'
	
fxwrite, filename, phdr 
	
	var = INTARR(N_ELEMENTS(col_names))
	rows = N_ELEMENTS(columns(0,*))

	sxdelpar, hdr, 'DATE'
	fxbhmake, hdr, rows, 'DATA', 'FITS file'
	
	FOR I=0, N_ELEMENTS(var)-1 DO BEGIN
	  IF col_types(i) EQ 'I' OR col_types(i) EQ 'J' THEN $
	  				coli = (LONG(columns(i,0))) $
	  			 ELSE coli = (columns(i,0))
	  fxbaddcol, v, hdr, coli, col_names(i), col_comments(i) 
	  var(i) = v
	ENDFOR
	
	fxbcreate, unit, filename, hdr
	
	irow=1
	FOR icol=0,N_ELEMENTS(var)-1 DO BEGIN
	 FOR irow=1,rows DO BEGIN
	  IF col_types(icol) EQ 'I' OR col_types(icol) EQ 'J' THEN $
	  	coli = (LONG(columns(icol,irow-1))) $		 
		ELSE coli = (columns(icol,irow-1))
	  fxbwrite, unit, coli, var(icol), irow
	 ENDFOR
	ENDFOR  
	
	FXBFINISH, unit
	FREE_LUN, unit
	
	RETURN

END
