;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   PRO write_aap_sl, tmpdata, tmphdr, filename = filename
;+
; NAME:
;       WRITE_AAP_SL
; PURPOSE:
;       Write a PHT-AAP (Astrophysical Applications Product) measurement from a  
;       temporary structure of AAP type onto a FITS binary table file
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       WRITE_AAP_SL, TMPDATA[, tmphdr]
;
; INPUT:
;       tmpdata : PHTAAP temporary structure
; 
; OPTIONAL INPUT:
;       tmphdr  : PHTAAP header
;
; OPTIONAL KEYWORD:
;       filename: filename to store the FITS file
;
; RESTRICTIONS:
;       tmpdata must be a valid PHT_S AAP temporary structure
;	(either staring or chopped subtracted)
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;		phtslwavlen
;
; MODIFICATION HISTORY:
;       Written by: Min Y. Hur				Feb 1999
;	Small modifications by CG			Feb 1999
;	V7.4
;	Modified to reflect the new AAP structure (MH)	Mar 2000
;	V8.2
;
;-
@phtslwavlen	; common block with SL wavelengths 
		; (first 64 for SS, last 64 for SL)
		; name for the variable is "slwavelen"

ON_ERROR, 2

;; originally written for saving all PHT-S measurements present in the
;; buffer - commented out to be used with tmpdata as single input (CG) 

;num=n_elements(phtaap)
;for k=1,num-1 do begin
;pia_load_iaap,phtaap(k),tmpdata	; 1 signals the position within the AAP
                            	    	; buffer, and it starts from 1


IF (TMPDATA.DET ne 'SS' and TMPDATA.DET ne 'SL') THEN BEGIN
   print,' This routine works only with PHT-S data.'
   goto, discon
ENDIF  

; If chopped observation, the background subtraction should have been 
; performed before this FITS binary table can be created.

inda=where(tmpdata.cstp eq 2, count)
if (tmpdata.dims ne 1 and count gt 0) then begin
   print,' ',tmpdata.admn,' is a chopped measurement and needs'
   print,' the background subtraction before you can save the result'
   print,' to a FITS binary table.'
   goto, discon
endif

;IF KEYWORD_SET(tmphdr) THEN BEGIN
;   hdr = tmphdr 
;   found = STRPOS(tmpdata.hdr, "PIA_VERS")
;   line=tmpdata.hdr(where(found ne -1))
;   pia_vers=strmid(line,11,8)
;   fxaddpar, hdr, 'PIA_VERS',pia_vers,'PHT-Interactive Analysis version used'
;ENDIF ELSE hdr = tmpdata.HDR  

IF not KEYWORD_SET(tmphdr) THEN tmphdr=tmpdata.hdr

if (tmpdata.det eq 'SS') then wave=slwavelen(0:63) else wave=slwavelen(64:127)
rpid=tmpdata.rpid
mnfl=tmpdata.mnfl
mnfu=tmpdata.mnfu
mdfl=tmpdata.mdfl
q1fl=tmpdata.q1fl
q3fl=tmpdata.q3fl
mnbr=tmpdata.mnbr
mnbu=tmpdata.mnbu
mdbr=tmpdata.mdbr
q1br=tmpdata.q1br
q3br=tmpdata.q3br
dec=tmpdata.dec
ra=tmpdata.ra
decp=tmpdata.decp
rap=tmpdata.rap
roll=tmpdata.roll
flag=tmpdata.flag

IF keyword_set(filename) THEN fitsfile=filename ELSE $
fitsfile=tmpdata.det+strmid(tmpdata.admn,4,8)+'_AAP.FITS'

; Use the buffer structure header or the input header via keyword for 
; a primary header and write it out.
;
sxdelpar, tmphdr, 'XTENSION'
sxdelpar, tmphdr, 'PIA_VERS'
fxaddpar, tmphdr, 'PIA_VERS',!pia_version,'PHT-Interactive Analysis version used',$
   before='OLPVERS '
fxwrite,fitsfile,tmphdr

; For single staring and chopped observations
if (tmpdata.dims eq 1) then begin

; Create a binary table extension header for a table with 1 row.
;
fxbhmake,header,1,'TESTEXT','Test binary table extension'
;
; Create the column for the data arrays.
;
fxbaddcol,wavecol,header,wave,'WAVELEN','Wavelengths',tunit='um'
fxbaddcol,rpidcol,header,rpid,'RPID','Raster point ID'
fxbaddcol,mnflcol,header,mnfl,'MNFL','Mean flux',tunit='Jy'
fxbaddcol,mnfucol,header,mnfu,'MNFU','Mean flux uncertainty',tunit='Jy'
fxbaddcol,mdflcol,header,mdfl,'MDFL','Median of flux',tunit='Jy'
fxbaddcol,q1flcol,header,q1fl,'Q1FL','First quartile of flux',tunit='Jy'
fxbaddcol,q3flcol,header,q3fl,'Q3FL','Third quartile of flux',tunit='Jy'
fxbaddcol,mnbrcol,header,mnbr,'MNBR','Mean brightness',tunit='MJy/sr'
fxbaddcol,mnbucol,header,mnbu,'MNBU','Mean brightness uncertainty',tunit='MJy/sr'
fxbaddcol,mdbrcol,header,mdbr,'MDBR','Median of brightness',tunit='MJy/sr'
fxbaddcol,q1brcol,header,q1br,'Q1BR','First quartile of brightness',tunit='MJy/sr'
fxbaddcol,q3brcol,header,q3br,'Q3BR','Third quartile of brightness',tunit='MJy/sr'
fxbaddcol, deccol,header, dec,'DEC','Declination',tunit='degrees'
fxbaddcol,  racol,header,  ra,'RA','Right ascension',tunit='degrees'
fxbaddcol,decpcol,header,decp,'DECP','Declination per point',tunit='degrees'
fxbaddcol, rapcol,header, rap,'RAP','Right ascension per point',tunit='degrees'
fxbaddcol,rollcol,header,roll,'ROLL','Roll angle',tunit='degrees'
fxbaddcol,flagcol,header,flag,'FLAG','Flag indicating status of data point for processing'
;
; Write out the extension header.
;
fxbcreate,unit,fitsfile,header
;
; Write out the data.  
;
fxbwrite,unit,wave,wavecol,1
fxbwrite,unit,rpid,rpidcol,1
fxbwrite,unit,mnfl,mnflcol,1
fxbwrite,unit,mnfu,mnfucol,1
fxbwrite,unit,mdfl,mdflcol,1
fxbwrite,unit,q1fl,q1flcol,1
fxbwrite,unit,q3fl,q3flcol,1
fxbwrite,unit,mnbr,mnbrcol,1
fxbwrite,unit,mnbu,mnbucol,1
fxbwrite,unit,mdbr,mdbrcol,1
fxbwrite,unit,q1br,q1brcol,1
fxbwrite,unit,q3br,q3brcol,1
fxbwrite,unit, dec, deccol,1
fxbwrite,unit,  ra,  racol,1
fxbwrite,unit,decp,decpcol,1
fxbwrite,unit, rap, rapcol,1
fxbwrite,unit,roll,rollcol,1
fxbwrite,unit,flag,flagcol,1
;
; Close the binary extension.
;
fxbfinish,unit
endif

; For a raster observation
if (tmpdata.dims gt 1 and count eq 0) then begin

; Create a binary table extension header for a table with number of rows
; equal to the number of raster positions.

rpts=tmpdata.dims

fxbhmake,header,rpts,'TESTEXT','Test binary table extension'

; Create the column for the data arrays.

j=1
fxbaddcol,wavecol,header,wave,'WAVELEN','Wavelengths',tunit='um'
fxbaddcol,rpidcol,header,rpid(*,j-1),'RPID','Raster point ID'
fxbaddcol,mnflcol,header,mnfl(*,j-1),'MNFL','Mean flux',tunit='Jy'
fxbaddcol,mnfucol,header,mnfu(*,j-1),'MNFU','Mean flux uncertainty',tunit='Jy'
fxbaddcol,mdflcol,header,mdfl(*,j-1),'MDFL','Median of flux',tunit='Jy'
fxbaddcol,q1flcol,header,q1fl(*,j-1),'Q1FL','First quartile of flux',tunit='Jy'
fxbaddcol,q3flcol,header,q3fl(*,j-1),'Q3FL','Third quartile of flux',tunit='Jy'
fxbaddcol,mnbrcol,header,mnbr(*,j-1),'MNBR','Mean brightness',tunit='MJy/sr'
fxbaddcol,mnbucol,header,mnbu(*,j-1),'MNBU','Mean brightness uncertainty',tunit='MJy/sr'
fxbaddcol,mdbrcol,header,mdbr(*,j-1),'MDBR','Median of brightness',tunit='MJy/sr'
fxbaddcol,q1brcol,header,q1br(*,j-1),'Q1BR','First quartile of brightness',tunit='MJy/sr'
fxbaddcol,q3brcol,header,q3br(*,j-1),'Q3BR','Third quartile of brightness',tunit='MJy/sr'
fxbaddcol, deccol,header, dec,'DEC','Declination',tunit='degrees'
fxbaddcol,  racol,header,  ra,'RA','Right ascension',tunit='degrees'
fxbaddcol,decpcol,header,decp(*,j-1),'DECP','Declination per point',tunit='degrees'
fxbaddcol, rapcol,header, rap(*,j-1),'RAP','Right ascension per point',tunit='degrees'
fxbaddcol,rollcol,header,roll,'ROLL','Roll angle',tunit='degrees'
fxbaddcol,flagcol,header,flag(*,j-1),'FLAG','Flag indicating status of data point for processing'

; Write out the extension header.
;
fxbcreate,unit,fitsfile,header
;
; Write out the data.  

while (j lt rpts+1) do begin
fxbwrite,unit,wave,wavecol,j
fxbwrite,unit,rpid(*,j-1),rpidcol,j
fxbwrite,unit,mnfl(*,j-1),mnflcol,j
fxbwrite,unit,mnfu(*,j-1),mnfucol,j
fxbwrite,unit,mdfl(*,j-1),mdflcol,j
fxbwrite,unit,q1fl(*,j-1),q1flcol,j
fxbwrite,unit,q3fl(*,j-1),q3flcol,j
fxbwrite,unit,mnbr(*,j-1),mnbrcol,j
fxbwrite,unit,mnbu(*,j-1),mnbucol,j
fxbwrite,unit,mdbr(*,j-1),mdbrcol,j
fxbwrite,unit,q1br(*,j-1),q1brcol,j
fxbwrite,unit,q3br(*,j-1),q3brcol,j
fxbwrite,unit, dec, deccol,j
fxbwrite,unit,  ra,  racol,j
fxbwrite,unit,decp(*,j-1),decpcol,j
fxbwrite,unit, rap(*,j-1), rapcol,j
fxbwrite,unit,roll,rollcol,j
fxbwrite,unit,flag(*,j-1),flagcol,j
j=j+1
endwhile

; Close the binary extension.
;
fxbfinish,unit

endif
;endfor

discon:

END
