;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_asc_fcspow, filename, fcspow, trs_id, det, table, $
	inst_tab=inst_tab, version=version,author=author

;+
; NAME:
;     WRITE_ASC_FCSPOW
; PURPOSE:
;	Write a PHT FCSPOW to an uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ASC_FCSPOW, FILENAME, FCSPOW, TRS_NUM, DETECTOR, FCSPOW_TABLE
;                   INST_TAB = INST_TAB, 
;                   VERSION = VERSION, AUTHOR = AUTHOR
;
; INPUT:
;       filename: uplink file name 
;       fcspow: character string of '1' or '2'
;       trs_id: character string of '1' or '2'
;       det: character string of 'C1', 'C2', 'P1', 'P2', 'P3'
;       talble: fcspow table, 4 dimensional array of
;               45 * detector pixels * 2 * 2
;
; OPTIONAL INPUT:
;       INST_TAB : string to identify the instrument table, eg P_FC1_C1
;       VERSION : the version number, string type, (00.00)
;                 default is 00.01
;       AUTHOR : author, string type, default is PHT IDT
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               phtoptpow
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;
;-
  on_error, 2
@phtoptpow

  case fcspow of
    '1': fcs_ind = 0
    '2': fcs_ind = 1
    else: begin
      message, 'FCSPOW not valid. Accepted FCS value are 1 or 2'
      return
    end
  endcase

  case trs_id of
    '1': begin
      trs_ind = 0
      trs_key = 'TRS1_ON'
    end
    '2': begin
      trs_ind = 1
      trs_key = 'TRS2_ON'
    end
    else: begin
      message, 'TRS not valid. Accepted TRS value are 1 or 2'
      return
    end
  endcase

  
  if keyword_set(inst_tab) then tinst_tab = inst_tab else $
    tinst_tab = 'P_' + 'FC' + fcspow + '_' + det
  if keyword_set(version) then tver = version else tver = '00.01'

; prepare the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
; date format is yy/mm/dd
    date = strmid(sysdate, 22, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 8, 2)
  
  if keyword_set(author) then tauthor = author else tauthor = 'PHT IDT'

  openw, unit, filename, /get_lun

  printf, unit, 'START_INST_TAB  = ', tinst_tab
  printf, unit, 'VERSION         = V', tver
  printf, unit, 'DATE            = ', date
  printf, unit, 'AUTHOR          = ', tauthor

  case det of
    'C1' : begin
      pixels = 6
      if keyword_set(first_index) then ii = first_index else ii = 0
      if keyword_set(last_index) then iii = last_index else iii = 44
      num_of_rows = iii - ii + 1

      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      tmpline = 'COMMENT                        P_FC' + fcspow + '_C1'
      printf, unit, tmpline
      printf, unit, 'COMMENT  Provides the transformation factors between power in W/pixel on'
      printf, unit, 'COMMENT  the C100 detector and the electrical power to be applied to'
      tmpline = 'COMMENT  the fine calibration source FCS' + fcspow + $
                ' for this flux.'
      printf, unit, tmpline
      printf, unit, 'COMMENT  This relation is given for each C100 filter independently.'
      printf, unit, 'COMMENT  In addition the command mnemonic for the correct selection of the'
      printf, unit, 'COMMENT  thermal radiation source (trs) mode is given.'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'

      printf, unit, 'NUM_OF_KEYS     = 15'
      printf, unit, 'NUM_OF_ROWS     = ', strtrim(string(num_of_rows),2)

      tmpline = 'COMMENT         Electrical power in [mW] to be applied to FCS' + fcspow + $
                ' for filter'
      tmpline1 = 'COMMENT         trs mode mnemonic to be used for the given flux for filter'

      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME01      = FLUX'
      printf, unit, 'COMMENT         Flux in W/pixel on the C100 detector'
      printf, unit, 'KEY_FORM01      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME02      = ELECT_POWER_2'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_90_RED (Position 2 of wheel)'
      printf, unit, 'KEY_FORM02      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME03      = TRS_MODE_2'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_90_RED'
      printf, unit, 'KEY_FORM03      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME04      = ELECT_POWER_3'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_50_UM'
      printf, unit, 'KEY_FORM04      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME05      = TRS_MODE_3'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_50_UM'
      printf, unit, 'KEY_FORM05      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME06      = ELECT_POWER_4'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_60_UM'
      printf, unit, 'KEY_FORM06      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME07      = TRS_MODE_4'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_60_UM'
      printf, unit, 'KEY_FORM07      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME08      = ELECT_POWER_5'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_70_UM'
      printf, unit, 'KEY_FORM08      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME09      = TRS_MODE_5'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_70_UM'
      printf, unit, 'KEY_FORM09      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME10      = ELECT_POWER_6'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_100_UM'
      printf, unit, 'KEY_FORM10      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME11      = TRS_MODE_6'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_100_UM'
      printf, unit, 'KEY_FORM11      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME12      = ELECT_POWER_7'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_105_UM'
      printf, unit, 'KEY_FORM12      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME13      = TRS_MODE_7'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_105_UM'
      printf, unit, 'KEY_FORM13      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME14      = ELECT_POWER_8'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C100_90_UM'
      printf, unit, 'KEY_FORM14      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME15      = TRS_MODE_8'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C100_90_UM'
      printf, unit, 'KEY_FORM15      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT -----------------------------------------------------------------------------------------'
      printf, unit, 'COMMENT               90_RED      50         60         70         100       105         90   '
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT FLUX [W/pix]  2   M2    3    M3    4    M4    5    M5    6    M6    7    M7    8    M8'
      printf, unit, 'COMMENT -----------------------------------------------------------------------------------------'
      end
    'C2' : begin
      pixels = 4
      if keyword_set(first_index) then ii = first_index else ii = 0
      if keyword_set(last_index) then iii = last_index else iii = 44
      num_of_rows = iii - ii + 1

      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      tmpline = 'COMMENT                        P_FC' + fcspow + '_C2'
      printf, unit, tmpline
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT  Provides the transformation factors between power in W/pixel on'
      printf, unit, 'COMMENT  the C200 detector and the electrical power to be applied to'
      tmpline = 'COMMENT  the fine calibration source FCS' + fcspow + $
                ' for this flux.'
      printf, unit, tmpline
      printf, unit, 'COMMENT  This relation is given for each C200 filter independently.'
      printf, unit, 'COMMENT  In addition the command mnemonic for the correct selection of the'
      printf, unit, 'COMMENT  thermal radiation source (trs) mode is given.'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'

      printf, unit, 'NUM_OF_KEYS     = 11'
      printf, unit, 'NUM_OF_ROWS     = ', strtrim(string(num_of_rows),2)
      tmpline = 'COMMENT         Electrical power in [mW] to be applied to FCS' + fcspow + $
                ' for filter'
      tmpline1 = 'COMMENT         trs mode mnemonic to be used for the given flux for filter'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME01      = FLUX'
      printf, unit, 'COMMENT         Flux in W/pixel on the C200 detector'
      printf, unit, 'KEY_FORM01      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME02      = ELECT_POWER_9'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C200_160_UM'
      printf, unit, 'KEY_FORM02      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME03      = TRS_MODE_9'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C200_160_UM'
      printf, unit, 'KEY_FORM03      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME04      = ELECT_POWER_10'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C200_200_UM'
      printf, unit, 'KEY_FORM04      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME05      = TRS_MODE_10'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C200_200_UM'
      printf, unit, 'KEY_FORM05      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME06      = ELECT_POWER_11'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C200_180_UM'
      printf, unit, 'KEY_FORM06      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME07      = TRS_MODE_11'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C200_180_UM'
      printf, unit, 'KEY_FORM07      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME08      = ELECT_POWER_12'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C200_135_UM'
      printf, unit, 'KEY_FORM08      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME09      = TRS_MODE_12'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C200_135_UM'
      printf, unit, 'KEY_FORM09      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME10      = ELECT_POWER_13'
      printf, unit, tmpline
      printf, unit, 'COMMENT         C200_120_UM'
      printf, unit, 'KEY_FORM10      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME11      = TRS_MODE_13'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         C200_120_UM'
      printf, unit, 'KEY_FORM11      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT -------------------------------------------------------------------'
      printf, unit, 'COMMENT               160         200        180        135        120   '
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT FLUX[W/pix]  9    M9    10   M10   11   M11   12   M12   13   M13'
      printf, unit, 'COMMENT -------------------------------------------------------------------'
      end
    'P1'  : begin
      pixels = 9
      if keyword_set(first_index) then ii = first_index else ii = 0
      if keyword_set(last_index) then iii = last_index else iii = 44
      num_of_rows = iii - ii + 1
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      tmpline = 'COMMENT                        P_FC' + fcspow + '_P1'
      printf, unit, tmpline
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT  Provides the transformation factors between power in W/mm^2 on'
      printf, unit, 'COMMENT  the P1 detector and the electrical power to be applied to'
      tmpline = 'COMMENT  the fine calibration source FCS' + fcspow + ' for this flux.'
      printf, unit, tmpline
      printf, unit, 'COMMENT  This relation is given for each P1 filter independently.'
      printf, unit, 'COMMENT  In addition the command mnemonic for the correct selection of the'
      printf, unit, 'COMMENT  thermal radiation source (trs) mode is given.'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      printf, unit, 'NUM_OF_KEYS     = 21'
      printf, unit, 'NUM_OF_ROWS     = ', strtrim(string(num_of_rows),2)
      tmpline = 'COMMENT         Electrical power in [mW] to be applied to FCS' + fcspow + $
                ' for filter'
      tmpline1 = 'COMMENT         trs mode mnemonic to be used for the given flux for filter'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME01      = FLUX'
      printf, unit, 'COMMENT         Flux in W/mm^2 on th P1 detector'
      printf, unit, 'KEY_FORM01      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME02      = ELECT_POWER_1'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_3P29_UM'
      printf, unit, 'KEY_FORM02      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME03      = TRS_MODE_1'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_3P29_UM'
      printf, unit, 'KEY_FORM03      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME04      = ELECT_POWER_2'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_3P6_UM'
      printf, unit, 'KEY_FORM04      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME05      = TRS_MODE_2'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_3P6_UM'
      printf, unit, 'KEY_FORM05      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME06      = ELECT_POWER_3'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_4P85_UM'
      printf, unit, 'KEY_FORM06      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME07      = TRS_MODE_3'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_4P85_UM'
      printf, unit, 'KEY_FORM07      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME08      = ELECT_POWER_4'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_7P3_UM'
      printf, unit, 'KEY_FORM08      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME09      = TRS_MODE_4'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_7P3_UM'
      printf, unit, 'KEY_FORM09      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME10      = ELECT_POWER_5'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_7P7_UM'
      printf, unit, 'KEY_FORM10      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME11      = TRS_MODE_5'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_7P7_UM'
      printf, unit, 'KEY_FORM11      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME12      = ELECT_POWER_6'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_10_UM'
      printf, unit, 'KEY_FORM12      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME13      = TRS_MODE_6'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_10_UM'
      printf, unit, 'KEY_FORM13      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME14      = ELECT_POWER_7'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_11P3_UM'
      printf, unit, 'KEY_FORM14      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME15      = TRS_MODE_7'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_11P3_UM'
      printf, unit, 'KEY_FORM15      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME16      = ELECT_POWER_8'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_12P8_UM'
      printf, unit, 'KEY_FORM16      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME17      = TRS_MODE_8'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_12P8_UM'
      printf, unit, 'KEY_FORM17      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME18      = ELECT_POWER_9'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_16P0_UM'
      printf, unit, 'KEY_FORM18      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME19      = TRS_MODE_9'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_16P0_UM'
      printf, unit, 'KEY_FORM19      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME20      = ELECT_POWER_10'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P1_11P5_UM'
      printf, unit, 'KEY_FORM20      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME21      = TRS_MODE_10'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P1_11P5_UM'
      printf, unit, 'KEY_FORM21      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT -----------------------------------------------------------------------------------------------------------------------'
      printf, unit, 'COMMENT            3.29       3.6        4.85        7.3        7.7       10.0       11.3       12.8       16.0        11.5'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT FLUX      1    M1    2    M2    3    M3    4    M4    5    M5    6    M6    7    M7    8    M8    9    M9    10  M10'
      printf, unit, 'COMMENT -----------------------------------------------------------------------------------------------------------------------'
      end
    'P2'  : begin
      pixels = 1
      if keyword_set(first_index) then ii = first_index else ii = 0
      if keyword_set(last_index) then iii = last_index else iii = 44
      num_of_rows = iii - ii + 1
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      tmpline = 'COMMENT                        P_FC' + fcspow + '_P2'
      printf, unit, tmpline
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT  Provides the transformation factors between power in W/mm^2 on'
      printf, unit, 'COMMENT  the P2 detector and the electrical power to be applied to'
      tmpline = 'COMMENT  the fine calibration source FCS' + fcspow + ' for this flux.'
      printf, unit, tmpline
      printf, unit, 'COMMENT  This relation is given for each P2 filter independently.'
      printf, unit, 'COMMENT  In addition the command mnemonic for the correct selection of the'
      printf, unit, 'COMMENT  thermal radiation source (trs) mode is given.'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      printf, unit, 'NUM_OF_KEYS     = 5'
      printf, unit, 'NUM_OF_ROWS     = ', strtrim(string(num_of_rows),2)
      tmpline = 'COMMENT         Electrical power in [mW] to be applied to FCS' + fcspow + $
                ' for filter'
      tmpline1 = 'COMMENT         trs mode mnemonic to be used for the given flux for filter'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME01      = FLUX'
      printf, unit, 'COMMENT         Flux in W/mm^2 on the P2 detector'
      printf, unit, 'KEY_FORM01      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME02      = ELECT_POWER_11'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P2_20_UM'
      printf, unit, 'KEY_FORM02      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME03      = TRS_MODE_11'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P2_20_UM'
      printf, unit, 'KEY_FORM03      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME04      = ELECT_POWER_12'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P2_25_UM'
      printf, unit, 'KEY_FORM04      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME05      = TRS_MODE_12'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P2_25_UM'
      printf, unit, 'KEY_FORM05      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT              20         25   '
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT  FLUX      11  M11    12  M11   '
      printf, unit, 'COMMENT ---------------------------------------------------'
      end
    'P3'  : begin
      pixels = 1
      if keyword_set(first_index) then ii = first_index else ii = 0
      if keyword_set(last_index) then iii = last_index else iii = 44
      num_of_rows = iii - ii + 1
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      tmpline = 'COMMENT                        P_FC' + fcspow + '_P3'
      printf, unit, tmpline
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT  Provides the transformation factors between power in W/mm^2 on'
      printf, unit, 'COMMENT  the P3 detector and the electrical power to be applied to'
      tmpline = 'COMMENT  the fine calibration source FCS' + fcspow + ' for this flux.'
      printf, unit, tmpline
      printf, unit, 'COMMENT  This relation is given for each P3 filter independently.'
      printf, unit, 'COMMENT  In addition the command mnemonic for the correct selection of the'
      printf, unit, 'COMMENT  thermal radiation source (trs) mode is given.'
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT ***************************************************************'
      printf, unit, 'COMMENT'
      printf, unit, 'NUM_OF_KEYS     = 5'
      printf, unit, 'NUM_OF_ROWS     = ', strtrim(string(num_of_rows),2)
      tmpline = 'COMMENT         Electrical power in [mW] to be applied to FCS' + fcspow + $
                ' for filter'
      tmpline1 = 'COMMENT         trs mode mnemonic to be used for the given flux for filter'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME01      = FLUX'
      printf, unit, 'COMMENT         Flux in W/mm^2 on the P3 detector'
      printf, unit, 'KEY_FORM01      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME02      = ELECT_POWER_13'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P3_60_UM'
      printf, unit, 'KEY_FORM02      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME03      = TRS_MODE_13'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P3_60_UM'
      printf, unit, 'KEY_FORM03      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME04      = ELECT_POWER_14'
      printf, unit, tmpline
      printf, unit, 'COMMENT         P3_100_UM'
      printf, unit, 'KEY_FORM04      = F'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'KEY_NAME05      = TRS_MODE_14'
      printf, unit, tmpline1
      printf, unit, 'COMMENT         P3_100_UM'
      printf, unit, 'KEY_FORM05      = C'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT --------------------------------------------------------'
      printf, unit, 'COMMENT              60         100   '
      printf, unit, 'COMMENT'
      printf, unit, 'COMMENT   FLUX     13   M13   14   M14   '
      printf, unit, 'COMMENT -------------------------------------------------'
      end
    else: begin
      message, 'DETECTOR not valid. Accepted detectors are C1, C2, P1, P2 and P3'
      return
    end
  endcase
  opt_i = 0
  for i = ii, iii do begin
     str = ''
; This for loop is to make sure that idl printf would output a line without 
; inserting newline marker in the middle. That happens when use the printf
; directly with the array variable. ie
; printf, unit, string(format='(f10.2)', table(...)) + trskey
     for j = 0, pixels do $
       str = str + ' ' + strtrim(string(format='(f10.2)', $
             table(i,j,trs_ind, fcs_ind)),2) + ' ' + trs_key
     str = strtrim(string(format='(E10.2)', longpow(opt_i)),2) + str 
     printf, unit, str
     opt_i = opt_i + 1
  endfor
  printf, unit, 'END_INST_TAB'

  free_lun, unit
END
