;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_asc_pdetect, det, idark = idark, nepread = nepread,$
                         respons = respons, eta = eta, $
                         inputfile = inputfile, outputfile = outputfile, $
                         inst_tab = inst_tab, version=version, author=author 

;+
; NAME:
;     WRITE_ASC_PDETECT
; PURPOSE:
;       Write a PHT PDETECT to an uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ASC_PDETECT, DETECTOR, IDARK = IDARK, NEPREAD = NEPREAD, 
;                   RESPONS = RESPONS, ETA = ETA,
;                   INPUTFILE = INPUTFILE, OUTPUTFILE = OUTPUTFILE,
;                   INST_TAB = INST_TAB,
;                   VERSION = VERSION, AUTHOR = AUTHOR
;
; INPUT:
;       detector: character strings of the detector, (P1, P2, P3, C100,
;                 C200, SS and SL)
;
; OPTIONAL INPUT:
;       IDARK: dark current, floating point
;       NEPREAD: nepread detector noise, floating point
;       RESPONS: responsivity, floating point
;       ETA: quantum efficiency of detector, floating point
;       INPUTFILE: string type for the input uplink file, 
;                  default is p_detect.txt
;       OUTPUTFILE: string type for the output uplink file, 
;                  default is p_detect_pia.txt
;       INST_TAB : string to identify the instrument table, eg P_DETECT
;       VERSION : the version number, string type, (00.00)
;                 default is 00.01
;       AUTHOR : author, string type, default is PHT IDT
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;
;-
  on_error, 2


  if keyword_set(inputfile) then infile = inputfile else infile = 'p_detect.txt'
  if keyword_set(outputfile) then outfile = outputfile else $
     outfile = 'p_detect_pia.txt'

; read in the input file and take out those lines begin with 'comment'
; and with '#', also trim off the leading and trailing spaces.

  openr, unit, infile, /get_lun
  line = ' '

; the first line is a dummy
  buf = strarr(1)

  while (not eof(unit)) do begin
    readf, unit, line 
    if strmid(line, 0, 7) ne 'COMMENT' then begin
     if strlen(strtrim(line, 2)) gt 0 then begin
       if strmid(line, 0, 1) ne '#' then buf = [buf, strtrim(line, 2)]
     end
    end
  end
  free_lun, unit

; we are interested in the value table itself
  end_table = where(strmid(buf, 0, 12) eq 'END_INST_TAB')

; the value 25 is the first 6 lines plus the 9 keys X 2 + 1
  value = buf(25: end_table(0) - 1)

; In case there are varying space delimiter between each value,
; we parse the value line one at a time to extract the individual
; value to a token_buf.

  tab = string(9b)

; pdetect_value is an array of structure of values
  pdetect_value = [{detect, pdetect: strarr(9)}]

  for i = 0, 6 do begin
    line = value(i)

;   replace all the tab characters, just in case
    while strpos(line, tab) GE 0 do begin
      pos = strpos(line, tab)
      strput, line, ' ', pos
    endwhile

; the first element in the token_buf is a dummy
    token_buf = [' ']
    
    for j = 0, 8 do begin
      pos = strpos(line, ' ')
      if pos ne -1 then begin
        token = strmid(line, 0, pos)
        token_buf = [token_buf, strmid(line, 0, pos)]
        len = strlen(line)
        line = strtrim(strmid(line, pos+1, len-pos-1), 1) 
      endif else token_buf = [token_buf, line]
    endfor
    pdetect_value = [pdetect_value , {detect, pdetect: token_buf(1:9)}]
   endfor 

; take out the first dummy in pdetect_value
   pdetect_value = pdetect_value(1: n_elements(pdetect_value) - 1)

; change the various pdetect value

  ind = where(pdetect_value.pdetect(0) eq det)
  if keyword_set(idark) then $
     pdetect_value(ind(0)).pdetect(3) = strtrim(string(format='(f10.1)', idark), 2)
  if keyword_set(nepread) then $
     pdetect_value(ind(0)).pdetect(4) = strtrim(string(format='(f10.2)', nepread), 2)
  if keyword_set(respons) then $
     pdetect_value(ind(0)).pdetect(5) = strtrim(string(format='(f10.2)', respons), 2)
  if keyword_set(eta) then $
     pdetect_value(ind(0)).pdetect(7) = strtrim(string(format='(f10.4)', eta), 2)

; write out the file
  if keyword_set(inst_tab) then tinst_tab = inst_tab else $
    tinst_tab = 'P_DETECT'
  if keyword_set(version) then tver = version else tver = '00.01'

; prepare the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
         'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
         '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
; date format is yy/mm/dd
    date = strmid(sysdate, 22, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 8, 2)

  if keyword_set(author) then tauthor = author else tauthor = 'PHT IDT'

  openw, unit, outfile , /get_lun

  printf, unit, 'START_INST_TAB  = ', tinst_tab
  printf, unit, 'VERSION         = V', tver
  printf, unit, 'DATE            = ', date
  printf, unit, 'AUTHOR          = ', tauthor

  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT **********************************************************************'
  printf, unit, 'COMMENT                                                      '
  printf, unit, 'COMMENT                    P_DETECT'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT        Provides characteristics of the PHT Detectors'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT PHT detector performance values according to ISOPHOT-FM Performance'
  printf, unit, 'COMMENT Data sheet by S.Kirches and J. Wolf from 94/06/28'
  printf, unit, 'COMMENT '
  printf, unit, 'COMMENT **********************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'NUM_OF_KEYS     = 9'
  printf, unit, 'NUM_OF_ROWS     = 7'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME01      = Detector_mnemonic'
  printf, unit, 'KEY_FORM01      = C'
  printf, unit, 'COMMENT           S1 of Perf. Data  Sheet corresponds to SL, S2 to SS'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME02      = deltaU'
  printf, unit, 'COMMENT           Delta U [V] dynamic range of the detector'
  printf, unit, 'KEY_FORM02      = F'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME03      = Capacitance'
  printf, unit, 'COMMENT           Capacitance [fF] [as F * 1.0E-15] of the CRE of the detector'
  printf, unit, 'COMMENT		  Origin: Test Reports PR-PHT-241 882BF (SL), 754BF (SS)'
  printf, unit, 'KEY_FORM03      = E'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME04      = Idark'
  printf, unit, 'COMMENT           Dark current  [as A * 1.0E-17]'
  printf, unit, 'COMMENT		  Origin: Performance Data Sheet; e = 1.60219 * 1.0E-19 C'
  printf, unit, 'KEY_FORM04      = E'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME05      = NEPread'
  printf, unit, 'COMMENT           NEPread detector noise including read-out noise'
  printf, unit, 'COMMENT           Units are in WHz**1/2  * 1.0E-17'
  printf, unit, 'COMMENT		  Origin: Performance Data Sheet, Variable called  NEPdet'
  printf, unit, 'KEY_FORM05      = E'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME06      = Respons'
  printf, unit, 'COMMENT           Responsivity [A/W] of the detector'
  printf, unit, 'COMMENT		  Origin: Performance Data Sheet'
  printf, unit, 'KEY_FORM06      = F'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME07      = Treflec'
  printf, unit, 'COMMENT           Treflect losses at reflecting surfaces inside ISO'
  printf, unit, 'COMMENT           and ISOPHOT. Product of TISOreflec and TPHTreflec'
  printf, unit, 'COMMENT           for PHT-S Treflec must not be used separately, because its'
  printf, unit, 'COMMENT           factor is included in the general transmission term per'
  printf, unit, 'COMMENT           pixel; key_word04 in table P_SS_SL'
  printf, unit, 'KEY_FORM07      = F'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME08      = ETA'
  printf, unit, 'COMMENT           Quantum efficiency of detector'
  printf, unit, 'COMMENT		  Origin: Performance Data Sheet'
  printf, unit, 'KEY_FORM08      = F'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'KEY_NAME09      = LOG2_CP_OPT'
  printf, unit, 'COMMENT           Log2 of the optimum chopper dwell time for detector'
  printf, unit, 'KEY_FORM09      = I'
  printf, unit, 'COMMENT ----------------------------------------------------------------------'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT deltaU  Capacitance  Idark  NEPread Respons  Treflec   ETA  LOG2_CP_OPT'
  printf, unit, 'COMMENT ------------------------------------------------------------------------'

  for i = 0, 6 do begin
    str = string(replicate(32b, 80))
    strput, str, pdetect_value(i).pdetect(0), 0
    len = strlen(pdetect_value(i).pdetect(1))
    strput, str, pdetect_value(i).pdetect(1), (13 - len)
    len = strlen(pdetect_value(i).pdetect(2))
    strput, str, pdetect_value(i).pdetect(2), (23 - len)
    len = strlen(pdetect_value(i).pdetect(3))
    strput, str, pdetect_value(i).pdetect(3), (33 - len)
    len = strlen(pdetect_value(i).pdetect(4))
    strput, str, pdetect_value(i).pdetect(4), (42 - len)
    len = strlen(pdetect_value(i).pdetect(5))
    strput, str, pdetect_value(i).pdetect(5), (50 - len)
    len = strlen(pdetect_value(i).pdetect(6))
    strput, str, pdetect_value(i).pdetect(6), (59 - len)
    len = strlen(pdetect_value(i).pdetect(7))
    strput, str, pdetect_value(i).pdetect(7), (68 - len)
    len = strlen(pdetect_value(i).pdetect(8))
    strput, str, pdetect_value(i).pdetect(8), (74 - len)

    printf, unit, str
  endfor
  printf, unit, 'END_INST_TAB'

  free_lun, unit
end
