;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_asc_pfiltr, column, buffer, inputfile = inputfile, $
			outputfile = outputfile, $
			inst_tab = inst_tab, version=version,$
			author=author

;+
; NAME:
;     WRITE_ASC_PFILTR
; PURPOSE:
;       Write a PHT PFILTR uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ASC_PFILTR, BUFFER, INPUTFILE = INPUTFILE,  
;                   INST_TAB = INST_TAB,
;                   VERSION = VERSION, AUTHOR = AUTHOR
;
; INPUT:
;       column: Column in the table (0 < column < 12)
;       BUFFER: a floating array of 14
;
; OPTIONAL INPUT:
;       INPUTFILE: string type for the input uplink file, 
;                  default is p_pfiltr.txt
;       OUTPUTFILE: string type for the output uplink file, 
;                  default is p_pfiltr_pia.txt
;       INST_TAB : string to identify the instrument table, eg P_SS_SL
;       VERSION : the version number, string type, (00.00)
;                 default is 00.01
;       AUTHOR : author, string type, default is PHT IDT
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;
;-
  on_error, 2


  if keyword_set(inputfile) then infile = inputfile else infile = 'p_pfiltr.txt'
    if keyword_set(outputfile) then outfile = outputfile else $
	 outfile = 'p_pfiltr_pia.txt'

; read in the input file and take out those lines begin with 'comment'
; and with '#', also trim off the leading and trailing spaces.

  openr, unit, infile, /get_lun
  line = ' '

; the first line is a dummy
  buf = strarr(1)

  while (not eof(unit)) do begin
    readf, unit, line 
    if strmid(line, 0, 7) ne 'COMMENT' then begin
     if strlen(strtrim(line, 2)) gt 0 then begin
       if strmid(line, 0, 1) ne '#' then buf = [buf, strtrim(line, 2)]
     end
    end
  end
  free_lun, unit

; we are interested in the value table itself
  end_table = where(strmid(buf, 0, 12) eq 'END_INST_TAB')

; the value 31 is the first 6 lines plus the 12 keys X 2 + 1
  value = buf(31: end_table(0) - 1)

; In case there are varying space delimiter between each value,
; we parse the value line one at a time to extract the individual
; value to a token_buf.

  tab = string(9b)

  pfiltr_value = [[strarr(12)]]

  for i = 0, 13 do begin
    line = value(i)

;   replace all the tab characters, just in case
    while strpos(line, tab) GE 0 do begin
      pos = strpos(line, tab)
      strput, line, ' ', pos
    endwhile

    token_buf = strarr(12)
    
    for j = 0, 10 do begin
      pos = strpos(line, ' ')
      token_buf(j) = strmid(line, 0, pos)
      len = strlen(line)
      line = strtrim(strmid(line, pos+1, len-pos-1), 1) 
    endfor
    token_buf(11) = line
    pfiltr_value = [[pfiltr_value] , [token_buf]]
   endfor 

; Now we should have 15 elements in pfiltr_value where the first one is
; a dummy but the rest of it would correspond to the column number
   pfiltr_value = pfiltr_value(*, 1 : 14)

; put a dummy column number in the buffer
;   buffer = [0, buffer]
;    column = column - 1

; 
   form = strarr(11)
   ind = [1, 2, 3, 5, 6]
   form(ind-1) = '(f10.3)'
   ind = [4,8,9]
   form(ind-1) = '(f10.2)'
   form(7-1) = '(f10.6)'
   ind = [10,11]
   form(ind-1) = '(e10.2)'
   pfiltr_value(column,*) = strtrim(string(format=form(column-1),buffer),2)

; write out the file
  if keyword_set(inst_tab) then tinst_tab = inst_tab else $
    tinst_tab = 'P_PFILTR'
  if keyword_set(version) then tver = version else tver = '00.01'

; prepare the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
         'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
         '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
; date format is yy/mm/dd
    date = strmid(sysdate, 22, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 8, 2)

  if keyword_set(author) then tauthor = author else tauthor = 'PHT IDT'

  openw, unit, outfile , /get_lun

  printf, unit, 'START_INST_TAB  = ', tinst_tab
  printf, unit, 'VERSION         = V', tver
  printf, unit, 'DATE            = ', date
  printf, unit, 'AUTHOR          = ', tauthor
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT                       P_PFILTR'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Provides values for the PHT-P detectors used to derive'
  printf, unit, 'COMMENT  illuminated power'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Each line represents a P-Filter (sorted by wheel position)'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Column (Omega) depends on Aperture (not Filter) but it is put'
  printf, unit, 'COMMENT  in this Table (In this case, column 1 indicates Aperture wheel'
  printf, unit, 'COMMENT  position)'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'NUM_OF_KEYS     = 12'
  printf, unit, 'NUM_OF_ROWS     = 14'
  printf, unit, 'KEY_NAME01      = P-Filter wheel position '
  printf, unit, 'KEY_FORM01      = I'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME02      = T'
  printf, unit, 'COMMENT           filter transmission '
  printf, unit, 'KEY_FORM02      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME03      = KT'
  printf, unit, 'COMMENT           Color correction factor for filter transmission '
  printf, unit, 'KEY_FORM03      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME04      = L'
  printf, unit, 'COMMENT           Spectral resolution: lambda centre/delta lambda '
  printf, unit, 'KEY_FORM04      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME05      = Lc'
  printf, unit, 'COMMENT           Central Wavelength'
  printf, unit, 'KEY_FORM05      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME06      = R'
  printf, unit, 'COMMENT           System Response'
  printf, unit, 'KEY_FORM06      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME07      = KR'
  printf, unit, 'COMMENT           Colour correction factor for system response'
  printf, unit, 'KEY_FORM07      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME08      = Omega '
  printf, unit, 'COMMENT           (solid angle subtended) !Odered by Aperture!'
  printf, unit, 'KEY_FORM08      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME09      = Tpol                         '
  printf, unit, 'COMMENT           Transmission of polarizers'
  printf, unit, 'KEY_FORM09      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME10      = EFF                         '
  printf, unit, 'COMMENT           Efficiency of polarizer'
  printf, unit, 'KEY_FORM10      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME11      = MAX_FCS1_FLUX                         '
  printf, unit, 'COMMENT           Maximum flux possible with FCS1 for the given filter'
  printf, unit, 'COMMENT           in [W/mm^2]'
  printf, unit, 'KEY_FORM11      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME11      = MAX_FCS2_FLUX                         '
  printf, unit, 'COMMENT           Maximum flux possible with FCS2 for the given filter'
  printf, unit, 'COMMENT           in [W/mm^2]'
  printf, unit, 'KEY_FORM11      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  T     KT      L      Lc     R      KR    Omega    Tpol  Eff   MAX_FCS1   MAX_FCS2'
  printf, unit, 'COMMENT ----------------------------------------------------------------------------------'

; position index
  ind = [4, 11, 18, 26, 33, 40, 47, 57, 63, 69, 79, 90]

  for i = 0, 13 do begin

    str = string(replicate(32b, 90))

    for j = 0, 11 do begin
      len = strlen(pfiltr_value(j, i))
      strput, str, pfiltr_value(j, i), (ind(j) - len)
    endfor

    printf, unit, strtrim(str)
  endfor

  printf, unit, 'END_INST_TAB'

  free_lun, unit
  return

end
