;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_asc_ppsf, filename, table, inst_tab=inst_tab, version=version,author=author

;+
; NAME:
;     WRITE_ASC_PPSF
; PURPOSE:
;	Write a PHT PPSF to an uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ASC_FCSPOW, FILENAME, PPSF_TABLE
;                   INST_TAB = INST_TAB, 
;                   VERSION = VERSION, AUTHOR = AUTHOR
;
; INPUT:
;       filename: uplink file name 
;       talble: ppsf table, 2 dimensional array 
;
; OPTIONAL INPUT:
;       INST_TAB : string to identify the instrument table, eg P_FC1_C1
;       VERSION : the version number, string type, (00.00)
;                 default is 00.01
;       AUTHOR : author, string type, default is PHT IDT
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Oct 95 Version 3.5
;
;-
  on_error, 2

  if keyword_set(inst_tab) then tinst_tab = inst_tab else $
    tinst_tab = 'P_PPSF'
  if keyword_set(version) then tver = version else tver = '00.01'

; prepare the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
; date format is yy/mm/dd
    date = strmid(sysdate, 22, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 8, 2)
  
  if keyword_set(author) then tauthor = author else tauthor = 'PHT IDT'

  openw, unit, filename, /get_lun

  printf, unit, 'START_INST_TAB  = ', tinst_tab
  printf, unit, 'VERSION         = V', tver
  printf, unit, 'DATE            = ', date
  printf, unit, 'AUTHOR          = ', tauthor

  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT                           P_PPSF'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Provides Point spread function values for the PHT-P detectors'
  printf, unit, 'COMMENT  for each possible Filter/Aperture combination'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Each line represents a P-Filter (sorted by wheel position)'
  printf, unit, 'COMMENT  Each column represents a P-Aperture (sorted by wheel position)'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'NUM_OF_KEYS     = 14'
  printf, unit, 'NUM_OF_ROWS     = 14'
  printf, unit, 'KEY_NAME01      = Point spread function for Aperture No. 1   52"'
  printf, unit, 'KEY_FORM01      = F'
  printf, unit, 'KEY_NAME02      = Point spread function for Aperture No. 2   79"'
  printf, unit, 'KEY_FORM02      = F'
  printf, unit, 'KEY_NAME03      = Point spread function for Aperture No. 3   99"'
  printf, unit, 'KEY_FORM03      = F'
  printf, unit, 'KEY_NAME04      = Point spread function for Aperture No. 4  120"'
  printf, unit, 'KEY_FORM04      = F'
  printf, unit, 'KEY_NAME05      = Point spread function for Aperture No. 5  127" x 127"'
  printf, unit, 'KEY_FORM05      = F'
  printf, unit, 'KEY_NAME06      = Point spread function for Aperture No. 6  180"'
  printf, unit, 'KEY_FORM06      = F'
  printf, unit, 'KEY_NAME07      = Point spread function for Aperture No. 7  180"'
  printf, unit, 'KEY_FORM07      = F'
  printf, unit, 'KEY_NAME08      = Point spread function for Aperture No. 8    5"'
  printf, unit, 'KEY_FORM08      = F'
  printf, unit, 'KEY_NAME09      = Point spread function for Aperture No. 9    7.6"'
  printf, unit, 'KEY_FORM09      = F'
  printf, unit, 'KEY_NAME10      = Point spread function for Aperture No. 10  10"'
  printf, unit, 'KEY_FORM10      = F'
  printf, unit, 'KEY_NAME11      = Point spread function for Aperture No. 11  13.8"'
  printf, unit, 'KEY_FORM11      = F'
  printf, unit, 'KEY_NAME12      = Point spread function for Aperture No. 12  18"'
  printf, unit, 'KEY_FORM12      = F'
  printf, unit, 'KEY_NAME13      = Point spread function for Aperture No. 13  20" x 32"'
  printf, unit, 'KEY_FORM13      = F'
  printf, unit, 'KEY_NAME14      = Point spread function for Aperture No. 14  23"'
  printf, unit, 'KEY_FORM14      = F'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT   79"    99"    120"   127"   180"   180"   5"     7.6"   10"    13.8"  18"    32"    23"'
  printf, unit, 'COMMENT ------------------------------------------------------------------------------------------'

  for i = 0, 13 do begin
     str = ''
     for j = 0, 13 do $
       str = str + ' ' + strtrim(string(format='(f10.4)', $
             table(j,i)),2) 
     printf, unit, str
  endfor
  printf, unit, 'END_INST_TAB'

  free_lun, unit
END
