;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_asc_psssl, det = det, pixel = pixel, fpsf = fpsf, lc = lc,$
                         trans = trans, rrespons = rrespons, $
                         inputfile = inputfile, outputfile = outputfile, $
                         inst_tab = inst_tab, version=version, author=author 

;+
; NAME:
;     WRITE_ASC_PSSSL
; PURPOSE:
;       Write a PHT PSSSL to an uplink file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ASC_PSSSL, DET=det, PIXEL = PIXEL, FPSF = FPSF, LC = LC, 
;                   TRANS = TRANS, RRESPONS = RRESPONS,
;                   INPUTFILE = INPUTFILE, OUTPUTFILE = OUTPUTFILE,
;                   INST_TAB = INST_TAB,
;                   VERSION = VERSION, AUTHOR = AUTHOR
;
; INPUT:
;
; OPTIONAL INPUT:
;       DET: string type of 'SS' or 'SL'. If det is set as keyword,
;         the following four optional parameters have to be an array of
;         64 elements, if not they have to contain 128 elements.
;       PIXEL: individual pixel number, this override the detector value if
;              set, integer
;       FPSF: point spread function for pixel, floating point
;       LC: Central wavelength, floating point
;       TRANS: total transmission for the pixel, floating point
;       RRESPONS: relative system response for pixel, floating point
;       INPUTFILE: string type for the input uplink file, 
;                  default is p_detect.txt
;       OUTPUTFILE: string type for the output uplink file, 
;                  default is p_ss_sl.txt
;       INST_TAB : string to identify the instrument table, eg P_SS_SL
;       VERSION : the version number, string type, (00.00)
;                 default is 00.01
;       AUTHOR : author, string type, default is PHT IDT
;
; PROCEDURES USED:
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;
;-
  on_error, 2


  if keyword_set(inputfile) then infile = inputfile else infile = 'p_ss_sl.txt'
  if keyword_set(outputfile) then outfile = outputfile else $
     outfile = 'p_ss_sl_pia.txt'

; read in the input file and take out those lines begin with 'comment'
; and with '#', also trim off the leading and trailing spaces.

  openr, unit, infile, /get_lun
  line = ' '

; the first line is a dummy
  buf = strarr(1)

  while (not eof(unit)) do begin
    readf, unit, line 
    if strmid(line, 0, 7) ne 'COMMENT' then begin
     if strlen(strtrim(line, 2)) gt 0 then begin
       if strmid(line, 0, 1) ne '#' then buf = [buf, strtrim(line, 2)]
     end
    end
  end
  free_lun, unit

; we are interested in the value table itself
  end_table = where(strmid(buf, 0, 12) eq 'END_INST_TAB')

; the value 17 is the first 6 lines plus the 5 keys X 2 + 1
  value = buf(17: end_table(0) - 1)

; In case there are varying space delimiter between each value,
; we parse the value line one at a time to extract the individual
; value to a token_buf.

  tab = string(9b)

; psssl_value is an array of structure of values
  psssl_value = [{sssl, psssl: strarr(5)}]

  for i = 0, 127 do begin
    line = value(i)

;   replace all the tab characters, just in case
    while strpos(line, tab) GE 0 do begin
      pos = strpos(line, tab)
      strput, line, ' ', pos
    endwhile

; the first element in the token_buf is a dummy
    token_buf = [' ']
    
    for j = 0, 3 do begin
      pos = strpos(line, ' ')
      token_buf = [token_buf, strmid(line, 0, pos)]
      len = strlen(line)
      line = strtrim(strmid(line, pos+1, len-pos-1), 1) 
    endfor
    token_buf = [token_buf, line]
    psssl_value = [psssl_value , {sssl, psssl: token_buf(1:5)}]
   endfor 

; Now we should have 129 elements in psssl_value where the first one is
; a dummy but the rest of it would correspond to the pixel number
   psssl_value = psssl_value(1: n_elements(psssl_value) - 1)

; change the various psssl value
  ind = indgen(128)
  if keyword_set(det) then $
    if det EQ 'SS' then ind = indgen(64) else ind = indgen(64) + 64
  if keyword_set(pixel) then ind = pixel - 1

  if keyword_set(fpsf) then $
     psssl_value(ind).psssl(1) = strtrim(string(format='(f10.6)', fpsf), 2)
  if keyword_set(lc) then $
     psssl_value(ind).psssl(2) = strtrim(string(format='(E14.5)', lc), 2)
  if keyword_set(trans) then $
     psssl_value(ind).psssl(3) = strtrim(string(format='(f10.6)', trans), 2)
  if keyword_set(rrespons) then $
     psssl_value(ind).psssl(4) = strtrim(string(format='(f10.4)', rrespons), 2)

; write out the file
  if keyword_set(inst_tab) then tinst_tab = inst_tab else $
    tinst_tab = 'P_SS_SL'
  if keyword_set(version) then tver = version else tver = '00.01'

; prepare the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
         'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
         '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
; date format is yy/mm/dd
    date = strmid(sysdate, 22, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 8, 2)

  if keyword_set(author) then tauthor = author else tauthor = 'PHT IDT'

  openw, unit, outfile , /get_lun

  printf, unit, 'START_INST_TAB  = ', tinst_tab
  printf, unit, 'VERSION         = V', tver
  printf, unit, 'DATE            = ', date
  printf, unit, 'AUTHOR          = ', tauthor

  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT                                                      '
  printf, unit, 'COMMENT                          P_SS_SL'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Table contains S Detectors values for:'
  printf, unit, 'COMMENT'
  printf, unit, 'COMMENT  Pixels  1 -  64 represent detector SS'
  printf, unit, 'COMMENT                            (short wavelegth 2.5 - 5 micron)'
  printf, unit, 'COMMENT  Pixels 65 - 128 represent detector SL'
  printf, unit, 'COMMENT                             (long wavelength 6-12 micron)'
  printf, unit, 'COMMENT '
  printf, unit, 'COMMENT ***************************************************************'
  printf, unit, 'COMMENT'
  printf, unit, 'NUM_OF_KEYS     = 5'
  printf, unit, 'NUM_OF_ROWS     = 128'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME01      = N'
  printf, unit, 'COMMENT           N: Pixel number'
  printf, unit, 'KEY_FORM01      = I'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME02      = fpsf'
  printf, unit, 'COMMENT           fpsf: Point spread function for pixel'
  printf, unit, 'KEY_FORM02      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME03      = Lc'
  printf, unit, 'COMMENT           Lc [m]: Central wavelength'
  printf, unit, 'KEY_FORM03      = E'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME04      = T'
  printf, unit, 'COMMENT           T: Total transmission for the pixel'
  printf, unit, 'KEY_FORM04      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'KEY_NAME05      = R'
  printf, unit, 'COMMENT           R[A/W]: Relative system response for pixel'
  printf, unit, 'KEY_FORM05      = F'
  printf, unit, 'COMMENT --------------------------------------------------------'
  printf, unit, 'COMMENT '
  printf, unit, 'COMMENT '
  printf, unit, 'COMMENT '
  printf, unit, 'COMMENT     N         fpsf          Lc              T             R'
  printf, unit, 'COMMENT ----------------------------------------------------------------'

  for i = 0, 127 do begin
    str = string(replicate(32b, 80))
    len = strlen(psssl_value(i).psssl(0))
    strput, str, psssl_value(i).psssl(0), (13 - len)
    len = strlen(psssl_value(i).psssl(1))
    strput, str, psssl_value(i).psssl(1), (27 - len)
    len = strlen(psssl_value(i).psssl(2))
    strput, str, psssl_value(i).psssl(2), (44 - len)
    len = strlen(psssl_value(i).psssl(3))
    strput, str, psssl_value(i).psssl(3), (58 - len)
    len = strlen(psssl_value(i).psssl(4))
    strput, str, psssl_value(i).psssl(4), (68 - len)

    printf, unit, strtrim(str)
  endfor
  printf, unit, 'END_INST_TAB'

  free_lun, unit
end
