;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_crelin, filename, det, table, pipelinename=pipelinename, comment=comment, version=version

;+
; NAME:
;     WRITE_CRELIN
; PURPOSE:
;	Write a PHT CRELIN to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_CRELIN, FILENAME, DETECTOR, CRELIN_TABLE,
;                   PIPELINENAME = PIPELINENAME,
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       det: character string of 'C1', 'C2', 'P'
;       talble: crelin table, 4 dimensional array of
;               121 * detector pixels * 3 * 2
;
; OPTIONAL INPUT:
;       PIPELINENAME: string type, default is the pipeline product name
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;	Changing pixel order to SPD pipeline convention 
;	  when writing out (CG) 			Dec.96
;	V5.5
;
;-
  on_error, 2
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  case det of
    'C1' : begin
      pixels = '9'
      tcomment = 'C100 linearisations'
      pfilename = 'PC1CRELIN'
      end
    'C2' : begin
      pixels = '4'
      tcomment = 'C200 linearisations'
      pfilename = 'PC2CRELIN'
; change pixel order
      newcrelin_table = table
	FOR ifreq = 0,2 DO $
	   FOR ierr = 0,1 DO BEGIN
	      newcrel = TRANSPOSE(table(*,*,ifreq,ierr))
	      C2P_IA2PIPE, newcrel, pipecrel
	      pipecrel = TRANSPOSE(pipecrel)
	      newcrelin_table(*,*,ifreq,ierr) = pipecrel
	   ENDFOR
     table = newcrelin_table      
      end
    'P'  : begin
      pixels = '3'
      tcomment = 'P linearisations'
      pfilename = 'PPCRELIN'
      end
    else: begin
      message, 'DETECTOR not valid. Accepted detectors are C1, C2 and P'
      return
    end
  endcase

  hdr = [ $
  "SIMPLE  =                    T / file does conform to FITS standard" + $
  string(replicate(32B, 13)), $
  "BITPIX  =                  -32 / number of bits per data pixel" + $
  string(replicate(32B, 18)), $
  "NAXIS   =                    4 / number of data axes" + $
  string(replicate(32B, 28)), $
  "NAXIS1  =                  121 / length of data axis   1" + $
  string(replicate(32B, 24)), $
  "NAXIS2  =                      / length of data axis   2" + $
  string(replicate(32B, 24)), $
  "NAXIS3  =                    3 / length of data axis   3" + $
  string(replicate(32B, 24)), $
  "NAXIS4  =                    2 / length of data axis   4" + $
  string(replicate(32B, 24)), $
  "EXTEND  =                    T / FITS dataset may contain extensions" + $
  string(replicate(32B, 12)), $
  "ORIGIN  = 'ESA     '           / European Space Agency" + $
  string(replicate(32B, 26)), $
  "TELESCOP= 'ISO     '           / Infrared Space Observatory" + $
  string(replicate(32B, 21)), $
  "INSTRUME= 'PHT     '           / Instrument used" + $
  string(replicate(32B, 32)), $
  "COMMENT" + $
  string(replicate(32B, 73)), $
  "FILENAME=                      / File name assigned in ISO archive" + $
  string(replicate(32B, 14)), $
  "DATE    = '12/02/93'           / Date of creating file" + $
  string(replicate(32B, 26)), $
  "FILEVERS= '        '           / Version ID assigned in ISO archive" + $
  string(replicate(32B, 13)), $
  "PIA_VERS= '  '                 / Internal (user) PIA-CAL Version" + $
  string(replicate(32B, 16)), $
  "END" + $
  string(replicate(32B, 77))]

; Write out the comment line
  temp = hdr(11)
  if keyword_set(comment) then tcomment = comment 
  strput, temp, tcomment, 11
  hdr(11) = temp

; Write out the filename 
; Note that if the pipeline filename is given in the parameter, it takes
; the pipelinename otherwise it would be the default pipeline name.
    if keyword_set(pipelinename) then pfilename = pipelinename
    fname_line = hdr(12)
    fname = "'" + pfilename + "'"
    strput, fname_line,  fname, 10
    hdr(12) = fname_line

; Write out the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
    date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 22, 2)
    date_line = hdr(13)
    strput, date_line, date, 11
    hdr(13) = date_line
  
; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
  tver = "'" + tver + "'"
  temp = hdr(15)
  strput, temp, tver, 10
  hdr(15) = temp

; Write out the NAXIS2
  temp = hdr(4)
  strput, temp, pixels, 29
  hdr(4) = temp

; calculate the size of the header
  tsize = 36 - (n_elements(hdr) mod 36)
  hdr = [hdr, replicate(blankline, tsize)]

  fitsfile = !CALSAVE + filename + '.FITS'
  openw, unit, fitsfile, /block, /get_lun

; 2904 = 4 * naxis1 * naxis3 * naxis4 
; Where 4 is the 4 bytes floating number and 
; naxis1 = 121, naxis3 = 3, naxis4 = 2
  trail = (2904 * fix(pixels)) mod 2880
  pad = bytarr( 2880 - trail )
  buf = float(table)
  byteorder, buf, /ftoxdr
  writeu, unit, byte(hdr), buf, pad
   
  free_lun, unit
END
