;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO write_dark, filename, tdarkp1, tdarkp2, tdarkp3, tdark1c, tdark2c, $
		  tdarks, pipelinename = pipelinename, $
		  comment=comment, version= version, FITS = fits
;+
; NAME:
;     WRITE_DARK
; PURPOSE:
;	Write a PHT DARK CURRENT to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_DARK, FILENAME, DARKP1, DARKP2, DARKP3, DARK1C, DARK2C, DARKS
;                   COMMENT = COMMENT, VERSION = VERSION, FITS = fits
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       darkp1, darkp2, darkp3: single float type 
;       dark1c, dark2c, darks: array of float type, size of 8, 4, 128
;
; OPTIONAL INPUT:
;       PIPELINENAME: string type, default is the pipeline product name
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;	FITS: IF present and not equal 0, write out FITS file
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               phtdarkcur
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;
;-
@phtdarkcur

  ON_ERROR, 2
; Check the number of parameters
  if n_params() lt 7 then message, $
    'Syntax: WRITE_RESP, FILENAME, DARKP1, DARKP2, DARKP3, DARK1C, DARK2C, DARKS, COMMENT=COMMENT, VERSION=VERSION'
  ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  hdr = [ $
  "SIMPLE  =                    T / file does conform to FITS standard" + $
  string(replicate(32b, 13)), $
  "BITPIX  =                    8 / number of bits per data pixel" + $
  string(replicate(32b, 18)), $
  "NAXIS   =                    0 / number of data axes" + $
  string(replicate(32b, 28)), $
  "EXTEND  =                    T / FITS dataset may contain extensions" + $
  string(replicate(32b, 12)), $
  "ORIGIN  = 'ESA     '           / European Space Agency" + $
  string(replicate(32b, 26)), $
  "TELESCOP= 'ISO     '           / Infrared Space Observatory" + $
  string(replicate(32b, 21)), $
  "INSTRUME= 'PHT     '           / Instrument used" + $
  string(replicate(32b, 32)), $
  "COMMENT" + $
  string(replicate(32b, 73)), $
  "FILENAME=                      / File name assigned in ISO archive" + $
  string(replicate(32b, 14)), $
  "DATE    = '        '           / Date of creating file" + $
  string(replicate(32b, 26)), $
  "FILEVERS= '        '           / Version ID assigned in ISO archive" + $
  string(replicate(32b, 13)), $
  "PIA_VERS= '  '                 / Internal (user) PIA-CAL Version" + $
  string(replicate(32B, 16)) ]

; Write out the comment line
  temp = hdr(7)
  if keyword_set(comment) then tcomment = comment else $
    tcomment = 'PHT Dark currents'
  strput, temp, tcomment, 11
  hdr(7) = temp

; Write out the filename 
; Note that if the pipeline filename is given in the parameter, it takes
; the pipelinename otherwise it would be the default pipeline name.
    if keyword_set(pipelinename) then pfilename = pipelinename else $
	pfilename = 'PDARKCURR'
    fname_line = hdr(8)
    fname = "'" + pfilename + "'"
    strput, fname_line,  fname, 10
    hdr(8) = fname_line

; Write out the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
    date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 22, 2)
    date_line = hdr(9)
    strput, date_line, date, 11
    hdr(9) = date_line
  
; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
    tvers = "'" + tver + "'"
    temp = hdr(11)
    strput, temp, tvers, 10
    hdr(11) = temp

; The DARKP1 lines
  line = "DARKP1  =                      /" + $
  	    string(replicate(32b, 48))
      nresp = string(format='(F10.6)', tdarkp1)
      len = strlen(nresp)
      strput, line, nresp, (30 - len)>10
      hdr = [hdr, line]
  
; The DARKP2 lines
  line = "DARKP2  =                      /" + $
  	    string(replicate(32b, 48))
      nresp = string(format='(F10.6)', tdarkp2)
      len = strlen(nresp)
      strput, line, nresp, (30 - len)>10
      hdr = [hdr, line]
  
; The DARKP3 lines
  line = "DARKP3  =                      /" + $
  	    string(replicate(32b, 48))
      nresp = string(format='(F10.6)', tdarkp3)
      len = strlen(nresp)
      strput, line, nresp, (30 - len)>10
      hdr = [hdr, line]
  
; The DARK1C* lines
  line = "DARK1C  =                      /" + $
  	     string(replicate(32b, 48))
    for i = 0, 8 do begin
      temp = line
      strput, temp, strtrim(string(i+1), 1), 6
      nresp = string(format='(F10.6)', tdark1c(i))
      len = strlen(nresp)
      strput, temp, nresp, (30 - len)>10
      hdr = [hdr, temp]
    endfor
  
; The DARKC2* lines
  line = "DARK2C  =                      /" + $
  	     string(replicate(32b, 48))
    for i = 0, 3 do begin
      temp = line
      strput, temp, strtrim(string(i+1), 1), 6
      nresp = string(format='(F10.6)', tdark2c(i))
      len = strlen(nresp)
      strput, temp, nresp, (30 - len)>10
      hdr = [hdr, temp]
    endfor
;
; The DARKS* lines
  line = "DARKS   =                      /" + $
  	    string(replicate(32b, 48))
    for i = 0, 127 do begin
      temp = line
      strput, temp, strtrim(string(i+1), 1), 5
      nresp = string(format='(F10.6)', tdarks(i))
      len = strlen(nresp)
      strput, temp, nresp, (30 - len)>10
      hdr = [hdr, temp]
    endfor
  
; append the endline 
    hdr = [hdr, endline]

; overwrite the phtdarkhdr common
    darkcurhdr = hdr

;pad the fits 2880 bytes record    
    trail = (36 - (n_elements(hdr) mod 36))
    if trail gt 0 then hdr = [hdr, replicate(blankline, trail)]
  
; write the hdr to a FITS file if desired
    IF KEYWORD_SET(fits) THEN BEGIN
     fitsfile = !CALSAVE + filename + '.FITS'
     openw, unit, fitsfile, /get_lun, /block
     writeu, unit, byte(hdr)
     free_lun, unit
    ENDIF
  END
