;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   PRO write_dietrans, filename, inst, tpdfix, tpduof, tpdgn1, tpdgn2, $
			pipelinename = pipelinename, $
			comment = comment, version = version
;+
; NAME:
;     WRITE_DIETRANS
; PURPOSE:
;	Write a PHT DIE 1 or 2 Transfer Function to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_DIETRANS, FILENAME, INSTRUMENT, PDIEFIX, PDIEUOF,PDIEGN1,PDIEGN2
;                   COMMENT = COMMENT, VERSION = VERSION 
;
; INPUT:
;       INSTRUMENT: character strings of '1', '2'
;       PDIEFIX, PDIEUOF, PDIEGN1, PDIEGN2: float array
;       
;
; OPTIONAL INPUT:
;       PIPELINENAME: string type, default is the pipeline product name
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;
;-
;
;
  on_error, 2
  ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  case inst of
    '1' : begin
      die = '1'
      tcomment = 'PHT DIE 1 Transfer Function'
      pfilename = 'PDIE1TRANS'
      tform = [ $
        "TFORM1  = '1E      '           /                                                ", $
        "TTYPE1  = 'PDIE1FIX'           / Fixed offset                                   ", $
        "TFORM2  = '1E      '           /                                                ", $
        "TTYPE2  = 'PDIE1UOF'           / Offset dependent on signal gain                ", $
        "TFORM3  = '1E      '           /                                                ", $
        "TTYPE3  = 'PDIE1GN1'           / Signal gain                                    ", $
        "TFORM4  = '1E      '           /                                                ", $
        "TTYPE4  = 'PDIE1GN2'           / Offset gain                                    "]
      end
    '2' : begin
      die = '2'
      tcomment = 'PHT DIE 2 Transfer Function'
      pfilename = 'PDIE2TRANS'
      tform = [ $
        "TFORM1  = '1E      '           /                                                ", $
        "TTYPE1  = 'PDIE2FIX'           / Fixed offset                                   ", $
        "TFORM2  = '1E      '           /                                                ", $
        "TTYPE2  = 'PDIE2UOF'           / Offset dependent on signal gain                ", $
        "TFORM3  = '1E      '           /                                                ", $
        "TTYPE3  = 'PDIE2GN1'           / Signal gain                                    ", $
        "TFORM4  = '1E      '           /                                                ", $
        "TTYPE4  = 'PDIE2GN2'           / Offset gain                                    "]
      end
    else: begin
      message, 'Instrument is not valid. Accepted instrument are 1 and 2'
      return
    end
  endcase
  phdr = [ $
  "SIMPLE  =                    T / file does conform to FITS standard" + $
  string(replicate(32b, 13)), $
  "BITPIX  =                    8 / number of bits per data pixel" + $
  string(replicate(32b, 18)), $
  "NAXIS   =                    0 / number of data axes" + $
  string(replicate(32b, 28)), $
  "EXTEND  =                    T / FITS dataset may contain extensions" + $
  string(replicate(32b, 12)), $
  "ORIGIN  = 'ESA     '           / European Space Agency" + $
  string(replicate(32b, 26)), $
  "TELESCOP= 'ISO     '           / Infrared Space Observatory" + $
  string(replicate(32b, 21)), $
  "INSTRUME= 'PHT     '           / Instrument used" + $
  string(replicate(32b, 32)), $
  "COMMENT" + $
  string(replicate(32b, 73)), $
  "FILENAME=                      / File name assigned in ISO archive" + $
  string(replicate(32b, 14)), $
  "DATE    = '        '           / Date of creating file" + $
  string(replicate(32b, 26)), $
  "FILEVERS= '        '           / Version ID assigned in ISO archive" + $
  string(replicate(32b, 13)), $
  "PIA_VERS= '  '                 / Internal (user) PIA-CAL Version" + $
  string(replicate(32B, 16)), endline ]

; Write out the comment line
  temp = phdr(7)
  if keyword_set(comment) then tcomment = comment 
  strput, temp, tcomment, 11
  phdr(7) = temp

; Write out the filename 
; pfilename would set to pipelinename if given in the optional
; parameter
    if keyword_set(pipelinename) then pfilename = pipelinename
    fname_line = phdr(8)
    fname = "'" + pfilename + "'"
    strput, fname_line,  fname, 10
    phdr(8) = fname_line

; Write out the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
    date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 22, 2)
    date_line = phdr(9)
    strput, date_line, date, 11
    phdr(9) = date_line
  
; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
  tver = "'" + tver + "'"
  temp = phdr(11)
  strput, temp, tver, 10
  phdr(11) = temp


; Create the Extended FITS header
;
  eheader = [ $
  "XTENSION= 'BINTABLE'           / Binary table FITS extension                    ", $
  "BITPIX  =                    8 / 8 BITS character format                        ", $
  "NAXIS   =                    2 / Tables are 2-d character array                 ", $
  "NAXIS1  =                   16 / Characters in a row                            ", $
  "NAXIS2  =                      / Number of rows in a table                      ", $
  "PCOUNT  =                    0 / Parameter count always 0                       ", $
  "GCOUNT  =                    1 / Group count always 1                           ", $
  "TFIELDS =                    4 / No of columns in table                         ", $
  tform, endline]
  
; NAXIS2
  dims = n_elements(tpdfix)
  naxis2 = string(dims)
  len = strlen(naxis2)
  temp = eheader(4)
  strput, temp, naxis2, (30 - len)>10
  eheader(4) = temp

; we calculate the padding size for the primary header and the extened
; header. Note the module 36 for the size is a precaution.
  phdrsize = 36 - (n_elements(phdr) mod 36)
  ehdrsize = 36 - (n_elements(eheader) mod 36)
  hdr = [phdr, replicate(blankline, phdrsize), eheader, $
	  replicate(blankline, ehdrsize)]

  buf_type = { diefix : 0.0, dieuof : 0.0, diegn1 : 0.0, diegn2 : 0.0}

  buf = replicate(buf_type, dims)

  tpdiefix = tpdfix
  tpdieuof = tpduof
  tpdiegn1 = tpdgn1
  tpdiegn2 = tpdgn2

  byteorder, tpdiefix, tpdieuof, tpdiegn1, tpdiegn2, /ftoxdr

  buf.diefix = tpdiefix
  buf.dieuof = tpdieuof
  buf.diegn1 = tpdiegn1
  buf.diegn2 = tpdiegn2

  fitsfile = !CALSAVE + filename + '.FITS'
  openw, unit, fitsfile, /block, /get_lun

  trail = (16 * dims) mod 2880
  pad = bytarr( 2880 - trail )
  writeu, unit, byte(hdr), buf, pad
   
  free_lun, unit
END
