;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  pro write_fcspow, filename, det, table, pipelinename=pipelinename, $
   comment=comment, version=version

;+
; NAME:
;     WRITE_FCSPOW
; PURPOSE:
;	Write a PHT FCSPOW to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_FCSPOW, FILENAME, DETECTOR, FCSPOW_TABLE,
;		    PIPELINENAME = PIPELINENAME,
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       det: character string of 'C1', 'C2', 'P1', 'P2', 'P3'
;       talble: fcspow table, 4 dimensional array of
;               45 * nr of filters for det * 2 * 2
;
; OPTIONAL INPUT:
;	PIPELINENAME: string type, default is the pipeline product name
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;
;-
  on_error, 2
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  case det of
    'C1' : begin
      pixels = '7'
      trs=1
      tcomment = 'C100 FCS power tables'
      pfilename = 'PC1FCSPOW'
      end
    'C2' : begin
      pixels = '5'
      trs=1
      tcomment = 'C200 FCS power tables'
      pfilename = 'PC2FCSPOW'
      end
    'P1'  : begin
      pixels = '10'
      trs=0
      tcomment = 'P1 FCS power tables'
      pfilename = 'PP1FCSPOW'
      end
    'P2'  : begin
      pixels = '2'
      trs=0
      tcomment = 'P2 FCS power tables'
      pfilename = 'PP2FCSPOW'
      end
    'P3'  : begin
      pixels = '2'
      trs=1
      tcomment = 'P3 FCS power tables'
      pfilename = 'PP3FCSPOW'
      end
    else: begin
      message, 'DETECTOR not valid. Accepted detectors are C1, C2, P1, P2 and P3'
      return
    end
  endcase

  IF KEYWORD_SET(COMMENT) THEN tcomment=tcomment+' - ' + comment
  lowpf1 = FLTARR(FIX(pixels))
  higpf1 = lowpf1 & lowpf2 = lowpf1 & higpf2 = lowpf1
  FOR I=0,N_ELEMENTS(lowpf1)-1 DO BEGIN
    ind = WHERE(table(*,i,trs,0) NE 0, nind)
    IF nind GT 0 THEN BEGIN
      lowpf1(i) = table(ind(0),i,trs,0)
      indh = WHERE(table(*,i,trs,0) GT 50., nind)
      IF nind GT 0 THEN $
       higpf1(i) = table(indh(0)-1,i,trs,0) ELSE higpf1(i)=50. 
    ENDIF ELSE BEGIN
      lowpf1(i) = 0.
      higpf1(i) = 0.
    ENDELSE
    
    ind = WHERE(table(*,i,trs,1) NE 0, nind)
    IF nind GT 0 THEN BEGIN
      lowpf2(i) = table(ind(0),i,trs,1)
      indh = WHERE(table(*,i,trs,1) GT 50., nind)
      IF nind GT 0 THEN $      
       higpf2(i) = table(indh(0)-1,i,trs,1) ELSE higpf2(i)=50. 
    ENDIF ELSE BEGIN
      lowpf2(i) = 0.
      higpf2(i) = 0.
    ENDELSE
     
   ENDFOR
      	  
  CREATE_PFITS, pfilename, table, USERNAME="PIA", COMMENT=tcomment, $
  	lowpf1, higpf1, lowpf2, higpf2

END
