;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   PRO write_illum, filename, inst, ilcor, illun, $
		    pipelinename = pipelinename, $
	            username = username, comment = comment, version = version
;+
; NAME:
;     WRITE_ILLUM
; PURPOSE:
;	Write a PHT FCS Illumination to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_ILLUM, FILENAME, INSTRUMENT, ILCOR, ILLUN,
;		    PIPELINENAME = PIPELINENAME,
;                   USERNAME = USERNAME, COMMENT = COMMENT, VERSION = VERSION 
;
; INPUT:
;	FILENAME: filename for the output file without extension
;       INSTRUMENT : character strings of '1' or '2'
;       ILCOR, ILLUN : float array of C100 or C200 pixel size
;       
;
; OPTIONAL INPUT:
;       PIPELINENAME: string type, default is the pipeline product name
;       USERNAME : the username line in the fits header, string type.
;                  the maximum character length is 8.
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;	Change pixel order for C200 to pipeline SPD convention (CG)  Dec.96
;	V5.5
;
;-
;
;
  on_error, 2
  ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  if keyword_set(username) then begin
    if strlen(username) > 8 then begin
      message, 'Maximum length for username is 8 characters long'
      return
    endif
  endif
  case inst of
    '1' : begin
      tcomment = 'PHT C100 FCS illumination'
      pfilename = 'PC1ILLUM'
      naxis1 = '72'
      array_size = 9
      buf_type = { ilcor: fltarr(9), illun: fltarr(9) }
      tform = [ $
        "TFORM1  = '9E      '           /                                                ", $
        "TTYPE1  = 'PC1ILCOR'           / Illumination correction per pixel              ", $
        "TFORM2  = '9E      '           /                                                ", $
        "TTYPE2  = 'PC1ILLUN'           / Uncertainty in PC1ILCOR                        "]
      end
    '2' : begin
      tcomment = 'PHT C200 FCS illumination'
      pfilename = 'PC2ILLUM'
      naxis1 = '32'
      array_size = 4
; Change pixel order to SPD convention
      c2p_ia2pipe, ilcor, ilcornew
      c2p_ia2pipe, illun, illunnew
      ilcor = ilcornew
      illun = illunnew
; Change pixel order to SPD convention
      
      buf_type = { ilcor: fltarr(4), illun: fltarr(4) }
      tform = [ $
        "TFORM1  = '4E      '           /                                                ", $
        "TTYPE1  = 'PC2ILCOR'           / Illumination correction per pixel              ", $
        "TFORM2  = '4E      '           /                                                ", $
        "TTYPE2  = 'PC2ILLUN'           / Uncertainty in PC2ILCOR                        "]
      end
    else: begin
      message, 'Instrument is not valid. Accepted instrument are 1 and 2'
      return
    end
  endcase
  phdr = [ $
  "SIMPLE  =                    T / file does conform to FITS standard" + $
  string(replicate(32b, 13)), $
  "BITPIX  =                    8 / number of bits per data pixel" + $
  string(replicate(32b, 18)), $
  "NAXIS   =                    0 / number of data axes" + $
  string(replicate(32b, 28)), $
  "EXTEND  =                    T / FITS dataset may contain extensions" + $
  string(replicate(32b, 12)), $
  "ORIGIN  = 'ESA     '           / European Space Agency" + $
  string(replicate(32b, 26)), $
  "TELESCOP= 'ISO     '           / Infrared Space Observatory" + $
  string(replicate(32b, 21)), $
  "INSTRUME= 'PHT     '           / Instrument used" + $
  string(replicate(32b, 32)), $
  "COMMENT" + $
  string(replicate(32b, 73)), $
  "FILENAME=                      / File name assigned in ISO archive" + $
  string(replicate(32b, 14)), $
  "DATE    = '        '           / Date of creating file" + $
  string(replicate(32b, 26)), $
  "FILEVERS= '        '           / Version ID assigned in ISO archive" + $
  string(replicate(32b, 13)), $
  "PIA_VERS= '  '                 / Internal (user) PIA-CAL Version" + $
  string(replicate(32B, 16)), $
  "USERNAME= '        '           / Unofficial data product" + $
  string(replicate(32b, 24)), endline ]

; Write out the comment line
  temp = phdr(7)
  if keyword_set(comment) then tcomment = comment 
  strput, temp, tcomment, 11
  phdr(7) = temp

; Write out the filename 
; Note that if the pipeline filename is given in the parameter, it takes
; the pipelinename otherwise it would be the default pipeline name.
    if keyword_set(pipelinename) then pfilename = pipelinename
    fname_line = phdr(8)
    fname = "'" + pfilename + "'"
    strput, fname_line,  fname, 10
    phdr(8) = fname_line

; Write out the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
    date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 22, 2)
    date_line = phdr(9)
    strput, date_line, date, 11
    phdr(9) = date_line
  
; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
  temp = phdr(11)
  tver = "'" + tver + "'"
  strput, temp, tver, 10
  phdr(11) = temp

; Write out the username line
  if keyword_set(username) then begin
    temp = phdr(12)
    strput, temp, username, 11
    phdr(12) = temp
  endif

; Create the Extended FITS header
;
  eheader = [ $
  "XTENSION= 'BINTABLE'           / Binary table FITS extension                    ", $
  "BITPIX  =                    8 / 8 BITS character format                        ", $
  "NAXIS   =                    2 / Tables are 2-d character array                 ", $
  "NAXIS1  =                      / Characters in a row                            ", $
  "NAXIS2  =                      / Number of rows in a table                      ", $
  "PCOUNT  =                    0 / Parameter count always 0                       ", $
  "GCOUNT  =                    1 / Group count always 1                           ", $
  "TFIELDS =                    2 / No of columns in table                         ", $
  tform, endline]
  
; NAXIS1
  len = strlen(naxis1)
  temp = eheader(3)
  strput, temp, naxis1, (30 - len)>10
  eheader(3) = temp

; NAXIS2
  dims = n_elements(ilcor)/array_size
  naxis2 = string(dims)
  len = strlen(naxis2)
  temp = eheader(4)
  strput, temp, naxis2, (30 - len)>10
  eheader(4) = temp

; Calculate the padding size of the primary and extended header
  phdrsize = 36 - (n_elements(phdr) mod 36)
  ehdrsize = 36 - (n_elements(eheader) mod 36)
  hdr = [phdr, replicate(blankline, phdrsize), eheader, $
	  replicate(blankline, ehdrsize)]

  buf = replicate(buf_type, dims)

  byteorder, ilcor, illun, /ftoxdr

  buf.ilcor = ilcor
  buf.illun = illun

  fitsfile = !CALSAVE + filename + '.FITS'
  openw, unit, fitsfile, /block, /get_lun

  trail = (fix(naxis1) * dims) mod 2880
  pad = bytarr( 2880 - trail )
  writeu, unit, byte(hdr), buf, pad
   
  free_lun, unit
END
