;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.!
 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             !
 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             !
 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
pro write_map, filename, map, hdr, mappar, path = path, comment=comment, $
	version=version , username = username
;+
; NAME:
;     WRITE_MAP
; PURPOSE:
;       Write a PHT MAP to a FITS file.
; CATEGORY:
;       PIA - I/O
; CALLING SEQUENCE:
;       WRITE_MAP, FILENAME, MAP, HEADER, MAPPAR, PATH = PATH
;                   COMMENT = COMMENT, VERSION = VERSION, USERNAME = USERNAME
;
; INPUT:
;       filename: fits file name with the '.FITS' extension
;	map: a N X M float array, it would accept any type of array
;	hdr: fits header from the phtaap
;	mappar: the parameter structure from the pia_mapw.pro
;
; OPTIONAL INPUT:
;	PATH : the path to write the file
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;       USERNAME: the username
;; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               addpar2hdr
;		addcomm2hdr
;		get_sysdate
;	Astronomical Library:
;		writefits  (modified !!!)
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai,	FEB, 1996
;	Use of writefits instead of openw (Jacosta) 	April 1997
;	Bug in writefits corrected (writefits -> writefits_mod) -> 
;		No byte swapping in this routine (CG)	July 1998
;	BUNIT=JY/BEAM for maps in Jy 
; V7.3
;-

  s = size(map)

  stype = s(s(0)+1)              ;Type of data
  case stype of
        1:      bitpix = 8
        2:      bitpix = 16
        3:      bitpix = 32
        4:      bitpix = -32
        5:      bitpix = -64
        6:      bitpix = 64
        7:      bitpix = 8
        else:   message,'Illegal Image Datatype'
  endcase

  header = ['END' + string(replicate(32b,77))]
  addpar2hdr, header, 'SIMPLE', 'T','file does conform to FITS standard'
  addpar2hdr, header, 'BITPIX', bitpix, 'number of bits per data pixel'
  addpar2hdr, header, 'NAXIS', S(0), 'number of data axes'

  if ( s(0) GT 0 ) then begin
   for i = 1, s(0) do addpar2hdr,header,'NAXIS' + strtrim(i,2),s(i), $
     'length of data axis   ' +  strtrim(i,2)
  endif

  addpar2hdr, header, 'EXTEND', 'T', 'Units'
  IF mappar.datatype GE 4 THEN bunit = 'JY/BEAM' ELSE bunit = 'MJY/SR'
  addpar2hdr, header, 'BUNIT', bunit, 'FITS dataset may contain extensions'
  addpar2hdr, header, 'ORIGIN', 'ESA', 'European Space Agency'
  addpar2hdr, header, 'TELESCOP', 'ISO', 'Infrared Space Observatory'
  addpar2hdr, header, 'INSTRUME', 'PHT', 'Instrument used'

; add the comment line
  if keyword_set(comment) then tcomment = comment else $
    tcomment = 'PHT Photometric Image'
  addcomm2hdr, header, tcomment

  addpar2hdr, header, 'FILENAME', filename, 'File name'
  get_sysdate, sdate
  addpar2hdr, header, 'DATE', sdate, 'Creation date'

; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
   addpar2hdr, header, 'FILEVERS', tver, 'Version id'
 
; Write out the username line
  if keyword_set(username) then tuser = username else tuser = 'PIA'
   addpar2hdr, header, 'USERNAME', tuser

; the ctype, crpix...
  addpar2hdr, header, 'CTYPE1', 'RA---TAN', 'Coordinate name'
  addpar2hdr, header, 'CTYPE2', 'DEC--TAN', 'Coordinate name'
;  addpar2hdr, header, 'CTYPE3', 'LAMBDA', 'Coordinate name'

  addpar2hdr, header, 'CRPIX1', mappar.trans.crpix(0)+1., 'Origin of Ra-axis in pixels'
  addpar2hdr, header, 'CRPIX2', mappar.trans.crpix(1)+1., 'Origin of Dec-axis in pixels'
;  addpar2hdr, header, 'CRPIX3', 1.00000, 'n/a'

  addpar2hdr, header, 'CRVAL1', mappar.trans.crval(0), 'Right Ascension of raster centre'
  addpar2hdr, header, 'CRVAL2', mappar.trans.crval(1), 'Declination of raster centre'
;  addpar2hdr, header, 'CRVAL3', 1.00000E-04, 'Wavelength in metres'

  addpar2hdr, header, 'CDELT1', -mappar.pixsize(0)/3600., 'Pixel size (degree)'
  addpar2hdr, header, 'CDELT2', mappar.pixsize(1)/3600., 'Pixel size (degree)'
;  addpar2hdr, header, 'CDELT3', 0.00000, 'n/a'

  addpar2hdr, header, 'CROTA1', 360. - mappar.rota, 'Roll Angle'
  addpar2hdr, header, 'CROTA2', 360. - mappar.rota, 'Roll Angle'
; addpar2hdr, header, 'CROTA3', 0.00000, 'n/a'

; addpar2hdr, header, 'APERTURE', '127"x127', 'Aperture used' 
  addpar2hdr, header, 'FILTER1', mappar.filter, 'Filter'
; addpar2hdr, header, 'EXBRGT1', 5.22E+00, 'Expected brightness'

; delete the end line
  nlines = n_elements(header)
  header = header(0:nlines-2)

  keyword = strmid(hdr,0,8)
  ind = where(keyword eq 'OBJECT  ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'EQUINOX ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'TMRATE  ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

; All the EOH... lines
  ind = where(strmid(keyword,0, 3) eq 'EOH', nfound)
  if nfound gt 0 then header = [header, hdr(ind)]

; write the ATT... lines
  ind = where(keyword eq 'ATTUTCSL', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTUTCS ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTOTFTH', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTRA   ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTDEC  ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTTYPE ', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTRNPTS', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTRNLNS', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTRDPTS', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTRDLNS', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTGUIDE', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTSAANG', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

  ind = where(keyword eq 'ATTERROR', nfound)
  if nfound eq 1 then header = [header, hdr(ind(0))]

; All the TREF... lines
  ind = where(strmid(keyword,0, 4) eq 'TREF', nfound)
  if nfound gt 0 then header = [header, hdr(ind)]

; All the TREF... lines
  ind = where(strmid(keyword,0, 2) eq 'PR', nfound)
  if nfound gt 0 then header = [header, hdr(ind)]
  
  extast, header, astr, nopar
  cd = astr.cd

  header = [header, 'END' +string(replicate(32b,77))]

  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE

;pad the fits 2880 bytes record
  trail = (36 - (n_elements(header) mod 36))
  if trail gt 0 then header = [header, replicate(blankline, trail)]
 
; prepare the data array

; It is not necessary anymore to do byte-swapping.
; The routine writefits (after my modification because of the bug
; mentioned by Wai here below) takes care of it. 

; the reason for looping over the array is there a bug in
; the ultrix and linux version for ftoxdr over array
; so might as well loop over the array instead of using
; host_to_ieee which is a handy interface to 
; the byteorder routine anyway.
;  n = n_elements(map) - 1
;  for i = 0, n do begin
;    t = map(i)
;    byteorder, t, /ftoxdr
;    map(i) = t
;  endfor
;  host_to_ieee, newdata

;
;  If necessary, then pad out to an integral multiple of 2880 bytes.
;
;  nbytes = n_elements(map) * (abs(bitpix) / 8 )
;  npad = nbytes mod 2880
;  if npad ne 0 then npad = 2880 - npad
    
  if KEYWORD_SET(PATH) then fitsfile = path + filename else $
    fitsfile = filename
;  openw, unit, fitsfile, /get_lun, /block
;  writeu, unit, byte(header), map, bytarr(npad)
;  free_lun, unit
  
  writefits_mod, fitsfile, map, header

END
