;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO write_resp, filename, tp1resp, tp2resp, tp3resp, tc1resp, tc2resp, $
		  pipelinename=pipelinename, $
		  tsresp, comment=comment, version= version
;+
; NAME:
;     WRITE_RESP
; PURPOSE:
;	Write a PHT responsivities to a FITS file.
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_RESP, FILENAME, P2RESP, P2RESP, P3RESP, C1RESP, C2RESP, SRESP,
;		    PIPELINENAME = PIPELINENAME,
;                   COMMENT = COMMENT, VERSION = VERSION
;
; INPUT:
;       filename: fits file name without the '.FITS' extension
;       p1resp, p2resp, p3resp: single double type 
;       c1resp, c2resp, sresp: array of double type, size of 8, 4, 128
;
; OPTIONAL INPUT:
;       PIPELINENAME: string type, default is the pipeline product name
;       COMMENT : the comment line in the fits header, string type
;       VERSION : the version number, string type, less than 8 chars
;                 default is 0001
;
; PROCEDURES USED: 
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;	to pipeline SPD C200 pixel order + PHT-S out (CG)	December 1996
;	V5.5
;
;-
;
  ON_ERROR, 2
; Check the number of parameters
  if n_params() lt 7 then message, $
    'Syntax: WRITE_RESP, FILENAME, P2RESP, P2RESP, P3RESP, C1RESP,'+ $
  ' C2RESP, SRESP, PIPELINENAME=PIPELINENAME, COMMENT=COMMENT, VERSION=VERSION'
  ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
  BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE
  hdr = [ $
  "SIMPLE  =                    T / file does conform to FITS standard" + $
  string(replicate(32b, 13)), $
  "BITPIX  =                  -32 / number of bits per data pixel" + $
  string(replicate(32b, 18)), $
  "NAXIS   =                    0 / number of data axes" + $
  string(replicate(32b, 28)), $
  "EXTEND  =                    T / FITS dataset may contain extensions" + $
  string(replicate(32b, 12)), $
  "ORIGIN  = 'ESA     '           / European Space Agency" + $
  string(replicate(32b, 26)), $
  "TELESCOP= 'ISO     '           / Infrared Space Observatory" + $
  string(replicate(32b, 21)), $
  "INSTRUME= 'PHT     '           / Instrument used" + $
  string(replicate(32b, 32)), $
  "COMMENT    PHT Default responsivities" + $
  string(replicate(32b, 43)), $
  "FILENAME=                      / File name assigned in ISO archive" + $
  string(replicate(32b, 14)), $
  "DATE    = '        '           / Date of creating file" + $
  string(replicate(32b, 26)), $
  "FILEVERS= '        '           / Version ID assigned in ISO archive" + $
  string(replicate(32b, 13)), $
  "PIA_VERS= '  '                 / Internal (user) PIA-CAL Version" + $
  string(replicate(32B, 16))]

; Write out the comment line
  if keyword_set(comment) then begin
    temp = 'COMMENT' + string(replicate(32b, 73))
    strput, temp, comment, 11
    hdr(7) = temp
  endif

; Write out the filename 
; Note that if the pipeline filename is given in the parameter, it takes
; the pipelinename otherwise it would be the default pipeline name.
    if keyword_set(pipelinename) then pfilename = pipelinename else $
	pfilename = 'PRESPONSE'
    fname_line = hdr(8)
    fname = "'" + pfilename + "'"
    strput, fname_line,  fname, 10
    hdr(8) = fname_line

; Write out the date
    monthstr = ['JAN', 'FEB', 'MAR', 'APR', 'MAY', 'JUN', 'JUL', 'AUG', $
  	 'SEP', 'OCT', 'NOV', 'DEC']
    monthseq = ['01', '02', '03', '04', '05', '06', '07', '08', $
  	 '09', '10', '11', '12']
    sysdate = systime()
    monthind = where(monthstr eq strupcase(strmid(sysdate, 4, 3)))
    date = strmid(sysdate, 8, 2) + '/' + monthseq(monthind(0)) + '/' + $
         strmid(sysdate, 22, 2)
    date_line = hdr(9)
    strput, date_line, date, 11
    hdr(9) = date_line
  
; Write out the version line
  if keyword_set(version) then tver = version else tver = '0001'
    tver = "'" + tver + "'"
    temp = hdr(11)
    strput, temp, tver, 10
    hdr(11) = temp

  ; The P1RESP lines
  prespline = "P1RESP  =                      / Responsivity (A/W)" + $
  	    string(replicate(32b, 29))
      nresp = string(format='(D14.6)', tp1resp)
      len = strlen(nresp)
      strput, prespline, nresp, (30 - len)>10
      hdr = [hdr, prespline]
  
  ; The P2RESP lines
  prespline = "P2RESP  =                      / Responsivity (A/W)" + $
  	    string(replicate(32b, 29))
      nresp = string(format='(D14.6)', tp2resp)
      len = strlen(nresp)
      strput, prespline, nresp, (30 - len)>10
      hdr = [hdr, prespline]
  
  ; The P3RESP lines
  prespline = "P3RESP  =                      / Responsivity (A/W)" + $
  	    string(replicate(32b, 29))
      nresp = string(format='(D14.6)', tp3resp)
      len = strlen(nresp)
      strput, prespline, nresp, (30 - len)>10
      hdr = [hdr, prespline]
  
  ; The C1RESP* lines
  c1respline = "C1RESP  =                      / Responsivity (A/W)" + $
  	     string(replicate(32b, 29))
    for i = 0, 8 do begin
      temp = c1respline
      strput, temp, strtrim(string(i+1), 1), 6
      nresp = string(format='(D14.6)', tc1resp(i))
      len = strlen(nresp)
      strput, temp, nresp, (30 - len)>10
      hdr = [hdr, temp]
    endfor
  
  ; The C2RESP* lines
  c2respline = "C2RESP  =                      / Responsivity (A/W)" + $
  	     string(replicate(32b, 29))

; Change to C200 pipeline SPD pixel order 
    c2p_ia2pipe, tc2resp, tc2resp_new
; 
    for i = 0, 3 do begin
      temp = c2respline
      strput, temp, strtrim(string(i+1), 1), 6
      nresp = string(format='(D14.6)', tc2resp_new(i))
      len = strlen(nresp)
      strput, temp, nresp, (30 - len)>10
      hdr = [hdr, temp]
    endfor

  ;
  ; The SRESP* lines
;  srespline = "SRESP   =                      / Responsivity (A/W)" + $
;  	    string(replicate(32b, 29))
;    for i = 0, 127 do begin
;      temp = srespline
;      strput, temp, strtrim(string(i+1), 1), 5
;      nresp = string(format='(D14.6)', tsresp(i))
;      len = strlen(nresp)
;      strput, temp, nresp, (30 - len)>10
;      hdr = [hdr, temp]
;    endfor
  
  ; append the endline and pad the fits 2880 bytes record
    hdr = [hdr, endline]
    eend = n_elements(hdr)
    trail = eend mod 36
    if trail gt 0 then hdr = [hdr, replicate(blankline, 36 - trail)]
  
    fitsfile = !CALSAVE + filename + '.FITS'
    openw, unit, fitsfile, /get_lun, /block
    writeu, unit, byte(hdr)
    free_lun, unit
  END
