;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  PRO write_srd, tmpdata, tmphdr, filename = filename
;+
; NAME:
;     WRITE_SRD
; PURPOSE:
;	Write a PHT-SRD (Signal Data) measurement from a temporary 
;	structure PHTSRD onto a fits file
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_SRD, TMPDATA
;
; INPUT:
;       tmpdata	: PHTSRD structure
;
; OPTIONAL INPUT:
;       tmphdr	: A temporary header
;       filename: filename to store the fitsfile
;
;
; RESTRICTIONS:
;       tmpdata must be a valid PHTSRD structure
;
; PROCEDURES USED: 
;       Common Block(s) or @ procedure(s):
;               phtsrd
;
; MODIFICATION HISTORY:
;       Written by: W. M. Tai
;	Modified by: C Gabriel
;	Oct 95 Version 3.5
;	Extension for effective reset time and nr of read-outs (CG)	Jan.96
;	trim to srd_unit to 18 chars    (WMT)            March 96
;	Storing into PIA_SAVEDIR 	(CG)		 March 96
;	filename keyword added          (WMT)            Apr   96
;	No byteswap to flag (CG)			 August96
;
;	Y2K Compliance (CG)				 Jun 1999
;	V7.4
;	Bug for array data fixed (CG)			 March 2000 
;	V8.2
;-
@phtsrd
;
  ON_ERROR, 2
ENDLINE = 'END' + STRING(REPLICATE(32B, 77))  ; END LINE
BLANKLINE = STRING(REPLICATE(32B, 80))        ; BLANK LINE


CASE TMPDATA.DET OF

'C1': BEGIN
; C100
  BUF_TYPE = { SLOP: FLTARR(11), $
	       SLUN: FLTARR(11), $
	       FLAG: FLTARR(11), $
	       RESI: FLTARR(11), $
	       NROS: FLTARR(11), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: BYTARR(2) }
  comment = 'PHT C100 SRD'
  tform_code = 'P1SR'

      END
'C2': BEGIN
; C200 
  BUF_TYPE = { SLOP: FLTARR(6), $
	       SLUN: FLTARR(6), $
	       FLAG: FLTARR(6), $
	       RESI: FLTARR(6), $
	       NROS: FLTARR(6), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: BYTARR(2) }
  comment = 'PHT C200 SRD'
  tform_code = 'P2SR'
      END
'SL': BEGIN
; S
  BUF_TYPE = { SLOP: FLTARR(132), $
	       SLUN: FLTARR(132), $
	       FLAG: FLTARR(132), $
	       RESI: FLTARR(132), $
	       NROS: FLTARR(132), $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: FLTARR(2), $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: BYTARR(2) }
  comment = 'PHT-S SRD'
  tform_code = 'PSSR'
      END
ELSE:
ENDCASE
IF STRMID(TMPDATA.DET, 0, 1) EQ 'P' THEN BEGIN
; P
  BUF_TYPE = { SLOP: 0.0, $
	       SLUN: 0.0, $
	       FLAG: 0.0, $
	       RESI: 0.0, $
	       NROS: 0.0, $
	       STIM: 0D, $
	       CHOP: 0.0, $
	       STEP: 0.0, $
	       FCSN: 0.0, $
	       TEMP: 0.0, $
	       FCS1: 0.0, $
	       FCS2: 0.0, $
	       TKEY: 0.0, $
	       RPID: BYTARR(2) }
  comment = 'PHT-P SRD'
  tform_code = 'PPSR'
ENDIF
; Calculate the naxis size from the buf_type structure
  nchars = size_struct(buf_type)
  strnaxis1 = strtrim(string(nchars))
  naxis1 = string(replicate(32b, 20 - strlen(strnaxis1))) + strnaxis1

; Create the tag name for the extended header
  tags = tag_names(buf_type)
  ntags = n_tags( buf_type)

; this is a dummy, we will get rid of it later
  ttype = ['        ']

; calculate the interger, floating and double type code for fits header
; note that we only interested in three types here
  for t=0, ntags-1 do begin
    s = size(buf_type.(t))
    n_elem = s( s(0)+2)
    field_type = s( s(0)+1)
    case field_type of
      1:  fits_type = 'B'
      2:  fits_type = 'I'
      4:  fits_type = 'E'
      5:  fits_type = 'D'
    else:
    endcase

; put everything together, note the max number of blank space is 8
    sfield = strtrim(string(n_elem), 2)
    ttype = [ttype, sfield + fits_type + string(replicate(32b, 7 - strlen(sfield)))]
  endfor
; put the correct ttype by taking out the first dummy element
  ttype = ttype(1:ntags)

;
; Get the header from the phtsrd record if parameter missed
;
  IF KEYWORD_SET(tmphdr) THEN hdr = tmphdr ELSE $
   hdr = tmpdata.HDR  ; get the extended header
  keyword = strmid(hdr, 0, 8)
;
; Creating the NAXIS, COMMENT, FILENAME and DATE lines for the primary header

  naxis = 'NAXIS   =                    0 / number of data axes                            '
  comm_line = 'COMMENT                                                                         '
  fname_line = 'FILENAME=                      / File name assigned in ISO archive              '
  fname = "'" + tmpdata.admn + "_SRD'  "
  strput, fname_line, fname, 10
  strput, comm_line, comment, 11
  date_line = "DATE    = '1999-01-01'         / Date of creating file                          "
  get_sysdate, date 
  strput, date_line, date, 11
;
; Find the index of the COMMENT, FILENAME  and DATE lines
;
  comment_ind = where(keyword EQ 'COMMENT ')
  file_ind = where(keyword EQ 'FILENAME')
  date_ind = where(keyword EQ 'DATE    ')
  xtension = where(keyword EQ 'XTENSION')
  hdr(comment_ind) = comm_line
  hdr(file_ind) = fname_line
  hdr(date_ind) = date_line

; create a UNIT line 
  unitline = blankline
  strput, unitline, 'SRD_UNIT='
  unit = "'" + strmid(strtrim(tmpdata.unit, 2), 0, 18) + "'"
  strput, unitline, unit, 10
  strput, unitline, '/ UNIT field in the PHTSRD', 31

; create a DET line
  detline = blankline
  strput, detline, 'SRD_DET ='
  det = "'" + strtrim(tmpdata.det, 2) + "'"
  strput, detline, det, 10
  strput, detline, '/ DETECTOR', 31

; create a ORDR line
  ordrline = blankline
  strput, ordrline, 'SRD_ORDR='
  ordr = strtrim(string(tmpdata.ordr), 2)
  len = strlen(ordr)
  strput, ordrline, ordr, (30 - len)>10
  strput, ordrline, '/', 31

; create a DIMS line
  dimsline = blankline
  strput, dimsline, 'SRD_DIMS='
  dims = strtrim(string(tmpdata.sldi), 2)
  len = strlen(dims)
  strput, dimsline, dims, (30 - len)>10
  strput, dimsline, '/', 31

; create a RCHP line 
  rchpline = blankline
  strput, rchpline, 'SRD_RCHP='
  rchp = strtrim(string(tmpdata.rchp), 2)
  len = strlen(rchp)
  strput, rchpline, rchp, (30 - len)>10
  strput, rchpline, '/', 31

; put into the primary header
phdr = [hdr(0:1), naxis, hdr( 2 : xtension(0) - 1 ), $
	unitline, detline, ordrline, dimsline, rchpline, endline]

;
; pad the rest of the record with blank line for a 2880 bytes fits record
;
  trail = n_elements(phdr) mod 36
  if trail gt 0 then phdr = [phdr, replicate(blankline, 36 - trail)]

eheader = [ $
"XTENSION= 'BINTABLE'           / Binary table FITS extension                    ", $
"BITPIX  =                    8 / 8 BITS character format                        ", $
"NAXIS   =                    2 / Tables are 2-d character array                 ", $
"NAXIS1  =                      / Characters in a row                            ", $
"NAXIS2  =                      / Number of rows in a table                      ", $
"PCOUNT  =                    0 / Parameter count always 0                       ", $
"GCOUNT  =                    1 / Group count always 1                           ", $
"TFIELDS =                      / No of columns in table                         "]

; NAXIS1
;
  temp = eheader(3)
  strput, temp, naxis1, 10
  eheader(3) = temp
; NAXIS2
  naxis2 = string(tmpdata.sldi)
  len = strlen(naxis2)
  temp = eheader(4)
  strput, temp, naxis2, (30 - len)>10
  eheader(4) = temp
; TFIELDS
  TFIELDS = strtrim(ntags, 2)
  temp = eheader(7)
  len = strlen(tfields)
  strput, temp, tfields, (30 - len)>10
  eheader(7) = temp
;
; The rest of the extended header
;

  tform = tform_code + tag_names(buf_type)

; create a dummy for hdr_buf
  hdr_buf = ['    ']

  for i = 0, ntags - 1 do begin
; create the form line
    temp = "TFORM   = '        '           /" + string(replicate(32b, 48))
    strput, temp, strtrim(i+1, 2), 5
    strput, temp, ttype(i), 11
    hdr_buf = [hdr_buf, temp]
; create the type line
    temp = "TTYPE   = '        '           /" + string(replicate(32b, 48))
    strput, temp, strtrim(i+1, 2), 5
    strput, temp, tform(i), 11
    hdr_buf = [hdr_buf, temp]
  endfor

  hdr_buf = hdr_buf(1:2*ntags)

  eheader = [eheader(0:7), hdr_buf, endline]
  eend = n_elements(eheader)
  trail = eend mod 36
  if trail gt 0 then eheader = [eheader, replicate(blankline, 36 - trail)]
  slop = tmpdata.slop
  slun = tmpdata.slun
  resi = tmpdata.resi
  nros = tmpdata.nros
  flag = tmpdata.flag
  stim = tmpdata.stim
  chop = tmpdata.chop
  step = tmpdata.step
  fcsn = tmpdata.fcsn
  temp = tmpdata.temp
  fcs1 = tmpdata.fcs1
  fcs2 = tmpdata.fcs2
  tkey = tmpdata.tkey
  rpid = tmpdata.rpid
  byteorder, slop, /ftoxdr
  byteorder, slun, /ftoxdr
  byteorder, resi, /ftoxdr
  byteorder, nros, /ftoxdr
  byteorder, flag, /htons
  byteorder, stim, /ftoxdr
  byteorder, chop, /ftoxdr
  byteorder, step, /htons
  byteorder, fcsn, /htons
  byteorder, temp, /ftoxdr
  byteorder, fcs1, /ftoxdr
  byteorder, fcs2, /ftoxdr
  byteorder, tkey, /ftoxdr
  BUF = REPLICATE(BUF_TYPE, TMPDATA.SLDI)
  IF STRMID(TMPDATA.DET, 0, 1) EQ 'P' THEN BEGIN
;   There is a bug in IDL 3.6.1 that 1 x Y array is very different
;   from >2 X Y array. That is assignment is not allowed in the first
;   case.
    buf.slop = transpose(slop)
    buf.slun = transpose(slun)
    buf.resi = transpose(resi)
    buf.nros = transpose(nros)
    buf.flag = transpose(flag)
  ENDIF ELSE BEGIN
    buf.slop = slop
    buf.slun = slun
    buf.resi =  resi 
    buf.nros =  nros 
    buf.flag = flag
  ENDELSE
  buf.stim = stim
  buf.chop = chop
  buf.step = step
  buf.fcsn = fcsn
  buf.temp = temp
  buf.fcs1 = fcs1
  buf.fcs2 = fcs2
  buf.tkey = tkey
  buf.rpid = rpid
  naxis2 = tmpdata.sldi
  trail = (nchars * naxis2) mod 2880
  pad = bytarr( 2880 - trail )

; added filename as an alternative filename
  if keyword_set(filename) then $
    fitsfile = filename else $
  fitsfile = !INTDIR + tmpdata.admn + '_SRD.FITS'

  openw, unit, fitsfile, /block, /get_lun
  writeu, unit, byte(phdr), byte(eheader), buf, pad

  free_lun, unit
END
