;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;!                                                                             !
;! This routine belongs to the PHT Interactive Analysis Package PIA.           !
;!                                                                             !
;! PIA is a joint development by the ESA Astrophysics Division and the ISOPHOT !
;! Consortium led by the Max Planck Institute for Astronomy (MPIA), Heidelberg.! 
;! Contributing ISOPHOT Consortium institutes are DIAS, RAL, AIP, MPIK, and    !
;! MPIA.                                                                       !
;!                                                                             ! 
;! Its use and distribution are free, as long as this header text is not       !
;! removed, or changed. When publishing ISOPHOT Data reduced with this analysis!
;! package, please mention this in the acknowledgement.                        !
;!                                                                             !
;! No warranties for installation/ support/ maintenance are given.             ! 
;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;+
; NAME:
;       WRITE_SUBTR
; PURPOSE:
;	WRITES source - background subtracted values from srcsubt accomodating 
;	them into the dynamical structure PHTSCP or PHTSPD (ispd=1)  
; CATEGORY:
;	PIA - I/O
; CALLING SEQUENCE:
;       WRITE_SUBTR, srcsubt, data, ispd, newhdr, no_info=no_info,  $
;              OUTMEAS=outmeas 
;
; INPUT:
;       srcsubt:
;		A structure containing the subtracted values
;	data:
;		The PHTSCP or PHTSPD temporary structure used for the
;		subtraction
;	ispd:
;		0 for SCP, 1 for SPD, 2 for AAP
;
; OPTIONAL INPUT:
;	NEWHDR:	
;		Modified header to be used as input in the new structure, it
;		will be modified in the output 
;
; KEYWORDS:
;	NO_INFO:
;		Suppressing message 
;	OUTMEAS:
;		subtracted SCP (SPD) temporary structure
;	NO_INTO_BUFFER:
;		do not put the measurement into the corresponding buffer
;
; PROCEDURES USED:
;       Common Block(s) or @ procedure(s):
;               adm
;               phtscp
;               phtspd
;
; MODIFICATION HISTORY:
;       Written by: C. Gabriel (ESA/VILSPA-SAI)		December 1995
;	Extension for ARSI, ANDR and AVSD (CG)		January 1996
;	Information added on buffer name (CG)		April 1996
;	NO_INFO keyword added (CG)			July 1996
;	Added outmeas keyword to provide the 
;	resulting subtracted structure  (JA+CG)		May 1997
;	Added REDUCE_MED keyword to reduce subtracted 
;	signals to one value using median   (JA)	Aug 1997
;	Keyword NO_INTO_BUFFER added (CG)		Oct 1997
;	V6.5
;	Chopper position from source instead from 
;	background (CG)					Oct 1997
;	V6.6
; 	Bug in HDR keyword for SCP case fixed (CG)	Jan 1998
;	V7.0
;	Bug in SCP & NO_INTO_BUFFER fixed (CG)		Mar 1998
;	V7.1
;	Bug for 1 element case fixed (CG)		Mar 1999
;	V7.4
;-
;

PRO WRITE_SUBTR, srcsubt, data, ispd, newhdr, NO_INFO=no_info, OUTMEAS= outmeas, $
          REDUCE_MED = reduce_med, NO_INTO_BUFFER=no_into_buffer

@adm
@phtscp
@phtspd
@phtaap

	IF ispd EQ 0 and srcsubt.level NE 'SCP' OR                 $
	   ispd EQ 1 and srcsubt.level NE 'SPD' OR                 $
	   ispd EQ 2 and srcsubt.level NE 'AAP' THEN BEGIN
		MESSAGE, /info, $
	  'Subtracted array and data array are not compatible, returning'
	  	RETURN
	ENDIF

	IF NOT KEYWORD_SET(reduce_med) THEN reduce_med = 0

	IF ispd GT 0 THEN step=data.cstp ELSE step=data.step

	IF N_ELEMENTS(srcsubt.srcstep) EQ 1 THEN $
	   ind = WHERE(step EQ srcsubt.srcstep, n) ELSE $
	   ind = WHERE(step EQ srcsubt.srcstep(0) OR $
	   		step EQ srcsubt.srcstep(1), n) 

	IF ISPD EQ 0 THEN BEGIN
	   npix = N_ELEMENTS(data.arsi(*,0))
	   arsi_avg = FLTARR(npix)
	   andr_avg = arsi_avg
	   avsd_avg = arsi_avg
	   IF n GT 1 THEN BEGIN
		FOR Ipix=0, npix-1 DO BEGIN
		   arsi_avg(ipix) = AVG(data.arsi(ipix,ind))
		   andr_avg(ipix) = MEDIAN(data.andr(ipix,ind))
		   avsd_avg(ipix) = AVG(data.avsd(ipix,ind)) 
		ENDFOR
	   ENDIF ELSE BEGIN
		FOR Ipix=0, npix-1 DO BEGIN
		   arsi_avg(ipix) = data.arsi(ipix,ind)
		   andr_avg(ipix) = data.andr(ipix,ind)
		   avsd_avg(ipix) = data.avsd(ipix,ind) 
		ENDFOR
	   ENDELSE
	 ENDIF
		
		
	bcksub_calc, srcsubt.aver, srcsubt.avun, srcsubt.flag, aver, avun, $
	   MEDIAN = reduce_med
	bcksub_calc, srcsubt.medi, srcsubt.avun, srcsubt.flag, medi, unc, $
	   MEDIAN = reduce_med
	
	m1qu = medi - unc
	m3qu = medi + unc

	IF ISPD EQ 2 THEN BEGIN 
	  bcksub_calc, srcsubt.aver_br, srcsubt.avun_br, srcsubt.flag,  $ 
	      aver_br, avun_br, MEDIAN = reduce_med
	  bcksub_calc, srcsubt.medi_br, srcsubt.avun_br, srcsubt.flag,  $
	      medi_br, unc_br, MEDIAN = reduce_med
	  m1qu_br = medi_br - unc_br
	  m3qu_br = medi_br + unc_br
	
	ENDIF   
	   
	
	CASE ispd OF 
	; SCP case
	0: BEGIN      ; SCP case
         
;  Create the structures and the handles
           phtscpt = {PHTSCP_TYPE}
	   FOR ihan = scp_first_handle, scp_last_handle DO  $
	       phtscpt.(ihan) = HANDLE_CREATE()

;  Fill the temporary PHTSCPT structure
           phtscpt.unit = data.UNIT
           phtscpt.det  = data.DET
           phtscpt.admn = data.ADMN + 'S'
           phtscpt.ordr = data.ORDR
           phtscpt.dims = 1L
                           
; Update header
	   IF N_ELEMENTS(NEWHDR) EQ 0 THEN newhdr = data.HDR

; change the name in the header 
	   pia_mnam = FXPAR(newhdr,'PIA_MNAM', count=count)
	   IF count EQ 0 THEN pia_mnam = phtscpt.admn
	   
           fxaddpar, newhdr, 'PIA_MNAM', pia_mnam + 'S'       

           fxaddpar, newhdr, 'PRC_SUBT',1,' SCP level bck subtraction', $
	      BEFORE='XTENSION'                      
	   srcst = srcsubt.srcstep
	   fxaddpar, newhdr,'PRC_SS_1',STRTRIM(srcst(0),2),' Source Step 1', $
	      BEFORE='XTENSION' 
	   IF N_ELEMENTS(srcst) GT 1 THEN $
	      fxaddpar, newhdr,'PRC_SS_2',STRTRIM(srcsubt.srcstep(1),2), $ 
	      ' Source Step 2', BEFORE='XTENSION' 
	   IF srcsubt.bckstep EQ 99 THEN nr = 'I' ELSE $
	      nr = STRTRIM(srcsubt.bckstep,2)
	   fxaddpar, newhdr,'PRC_SBCK',nr,' Bckgr step (I=int)', $
	      BEFORE='XTENSION'  
	
;  Fill the handles
           HANDLE_VALUE, PHTSCPT.HDR, NEWHDR, /SET  
           HANDLE_VALUE, PHTSCPT.TIME, median([data.TIME(ind)]), /SET  
; put chopper position corresponding to source
           HANDLE_VALUE, PHTSCPT.CHOP, data.CHOP(1), /SET   
           HANDLE_VALUE, PHTSCPT.STEP, data.STEP(0), /SET   
           HANDLE_VALUE, PHTSCPT.FCSN, data.FCSN(0), /SET   
           HANDLE_VALUE, PHTSCPT.TEMP, data.TEMP(0), /SET   
           HANDLE_VALUE, PHTSCPT.FCS1, data.FCS1(0), /SET   
           HANDLE_VALUE, PHTSCPT.FCS2, data.FCS2(0), /SET   
           HANDLE_VALUE, PHTSCPT.TKEY, median([data.TKEY(ind)]), /SET   
           HANDLE_VALUE, PHTSCPT.RPID, data.RPID(*,0), /SET 
           HANDLE_VALUE, PHTSCPT.ARSI, ARSI_avg, /SET  
           HANDLE_VALUE, PHTSCPT.ANDR, ANDR_avg, /SET  
           HANDLE_VALUE, PHTSCPT.AVSD, AVSD_avg, /SET  

           HANDLE_VALUE, PHTSCPT.AVER, AVER, /SET  
           HANDLE_VALUE, PHTSCPT.AVUN, AVUN, /SET     
           HANDLE_VALUE, PHTSCPT.MEDI, MEDI, /SET   
	   HANDLE_VALUE, PHTSCPT.Q1ME, m1qu, /SET   
	   HANDLE_VALUE, PHTSCPT.Q3ME, m3qu, /SET   
           HANDLE_VALUE, PHTSCPT.FLAG, intarr(N_ELEMENTS(aver)), /SET  
;  Update the PHTSCP structure using the temporary structure
           PHTSCP = [PHTSCP,PHTSCPT]
           
;  Write the output temporary structure
           pia_load_iscp, phtscp(N_ELEMENTS(phtscp)-1), tmpdata 
	   OUTMEAS = tmpdata

	   IF KEYWORD_SET(NO_INTO_BUFFER) THEN $
	     scp_init, index=N_ELEMENTS(phtscp)-1 ELSE BEGIN

	     decode_admn, phtscpt.admn, decnam

	     IF not KEYWORD_SET(NO_INFO) THEN $
               XDISPLAYFILE, "", $
                         TITLE = "Measurement in Buffer", $
                         WIDTH = 50, $
                         HEIGHT = 5, $
                         TEXT = ["The subtracted measurement is now in the ", $
                                 "SCP buffer as: " + decnam ]
           ENDELSE
	 
	   END        ; end SCP case
	   
	1: BEGIN      ; SPD case
	
	   PHTSPDT = {PHTSPD_TYPE}
	   FOR ihan =spd_first_handle, spd_last_handle DO $
	     PHTSPDT.(ihan) = HANDLE_CREATE()

;  Fill the temporary SPD structure
           PHTSPDT.UNIT = data.UNIT
           PHTSPDT.DET  = data.DET
           PHTSPDT.ADMN = data.ADMN + 'S'
           PHTSPDT.ORDR = data.ORDR
           PHTSPDT.DIMS = 1L
	   
; Update header
	   IF N_ELEMENTS(NEWHDR) EQ 0 THEN newhdr = data.HDR
	   
; change the name in the header 
	   pia_mnam = FXPAR(newhdr,'PIA_MNAM', count=count)
	   IF count EQ 0 THEN pia_mnam = phtspdt.admn
	   
           fxaddpar, newhdr, 'PIA_MNAM',pia_mnam + 'S'       
	 
           fxaddpar, newhdr, 'PRD_SUBT',1,' SPD level bck subtraction', $
	      BEFORE='XTENSION'                      
	   srcst = srcsubt.srcstep
	   fxaddpar, newhdr,'PRD_SS_1',STRTRIM(srcst(0),2),' Source Step 1', $
	      BEFORE='XTENSION' 
	   IF N_ELEMENTS(srcst) GT 1 THEN $
	      fxaddpar, newhdr,'PRD_SS_2',STRTRIM(srcsubt.srcstep(1),2), $ 
	      ' Source Step 2', BEFORE='XTENSION' 
	   IF srcsubt.bckstep EQ 99 THEN nr = 'I' ELSE $
	      nr = STRTRIM(srcsubt.bckstep,2)
	   fxaddpar, newhdr,'PRD_SBCK',nr,' Bckgr step (I=int)', $
	      BEFORE='XTENSION'  

	
;  Fill the handles
           HANDLE_VALUE, PHTSPDT.HDR, NEWHDR, /SET  ;
           HANDLE_VALUE, PHTSPDT.TKEY, median(data.TKEY(ind)), /SET  ;
           HANDLE_VALUE, PHTSPDT.KYID, data.KYID(0), /SET  ;
           HANDLE_VALUE, PHTSPDT.FILT, data.FILT(0), /SET  ;  
	   HANDLE_VALUE, PHTSPDT.APER, data.APER(0), /SET  ; 
           HANDLE_VALUE, PHTSPDT.TIME, median(data.TIME(ind)), /SET  ;
           HANDLE_VALUE, PHTSPDT.POLZ, data.POLZ(0), /SET  ;
           HANDLE_VALUE, PHTSPDT.NDRS, data.NDRS(0), /SET  ; 
           HANDLE_VALUE, PHTSPDT.DWEL, data.DWEL(0), /SET  ;   
	   HANDLE_VALUE, PHTSPDT.MEAS, data.MEAS(0), /SET  ; 
           HANDLE_VALUE, PHTSPDT.CPOS, data.CPOS(ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.CSTP, data.CSTP(ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.DEC, data.DEC(ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.RA, data.RA(ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.ROLL, data.ROLL(ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.RPID, data.RPID(*,ind(0)), /SET  ;
           HANDLE_VALUE, PHTSPDT.PLEN, data.PLEN(*,ind(0)), /SET ; 

           HANDLE_VALUE, PHTSPDT.MNPW, aver, /SET  ;
           HANDLE_VALUE, PHTSPDT.MNPU, avun, /SET  ;
           HANDLE_VALUE, PHTSPDT.MDPW, medi, /SET  ;
           HANDLE_VALUE, PHTSPDT.Q1PW, m1qu, /SET  ;
           HANDLE_VALUE, PHTSPDT.Q3PW, m3qu, /SET  ;
           HANDLE_VALUE, PHTSPDT.FLAG, intarr(N_ELEMENTS(aver)), /SET  ;

;  Update the PHTSPD structure using the temporary structure
           PHTSPD = [PHTSPD,PHTSPDT]

;  Write the output temporary structure
           pia_load_ispd, phtspd(N_ELEMENTS(phtspd)-1), tmpdata 
	   OUTMEAS = tmpdata
	   IF KEYWORD_SET(NO_INTO_BUFFER) THEN $
	     spd_init, index=N_ELEMENTS(phtspd)-1 ELSE BEGIN

	     decode_admn, phtspdt.admn, decnam

	     IF not KEYWORD_SET(NO_INFO) THEN $
               XDISPLAYFILE, "", $
                         TITLE = "Measurement in Buffer", $
                         WIDTH = 50, $
                         HEIGHT = 5, $
                         TEXT = ["The subtracted measurement is now in the ", $
                                 "SPD buffer as: " + decnam ]
	   ENDELSE
	   
	   END        ; end SPD case
	   
	; AAP case   
	2:  BEGIN 
	
	   PHTAAPT = {PHTAAP_TYPE}
	   FOR ihan =aap_first_handle, aap_last_handle DO $
	     PHTAAPT.(ihan) = HANDLE_CREATE()

;  Fill the temporary AAP structure
           PHTAAPT.DET  = data.DET
           PHTAAPT.ADMN = data.ADMN + 'S'
           PHTAAPT.DIMS = 1L
	   PHTAAPT.FILT = data.FILT
	   PHTAAPT.APER = data.APER
	   PHTAAPT.POLZ = data.POLZ
	   PHTAAPT.CWVL = data.CWVL
	   
; Update header
	   IF N_ELEMENTS(NEWHDR) EQ 0 THEN newhdr = data.HDR
	   
; change the name in the header 
	   pia_mnam = FXPAR(newhdr,'PIA_MNAM', count=count)
	   IF count EQ 0 THEN pia_mnam = phtaapt.admn
	   
           fxaddpar, newhdr, 'PIA_MNAM',pia_mnam + 'S'       
           fxaddpar, newhdr, 'PRA_SUBT',1,' AAP level bck subtraction', $
	      BEFORE='XTENSION'                      
	   srcst = srcsubt.srcstep
	   fxaddpar, newhdr,'PRA_SS_1',STRTRIM(srcst(0),2),' Source Step 1', $
	      BEFORE='XTENSION' 
	   IF N_ELEMENTS(srcst) GT 1 THEN $
	      fxaddpar, newhdr,'PRA_SS_2',STRTRIM(srcsubt.srcstep(1),2), $ 
	      ' Source Step 2', BEFORE='XTENSION' 
	   IF srcsubt.bckstep EQ 99 THEN nr = 'I' ELSE $
	      nr = STRTRIM(srcsubt.bckstep,2)
	   fxaddpar, newhdr,'PRA_SBCK',nr,' Bckgr step (I=int)', $
	      BEFORE='XTENSION'  

	
;  Fill the handles
           HANDLE_VALUE, PHTAAPT.HDR, NEWHDR, /SET  ;
	   HANDLE_VALUE, PHTAAPT.TIME, median([data.TIME(ind)]), /SET  ;
           HANDLE_VALUE, PHTAAPT.CSTP, data.CSTP(ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.CPOS, data.CPOS(ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.DWEL, data.DWEL(0), /SET  ;   

           HANDLE_VALUE, PHTAAPT.MNFL, aver, /SET  ;
           HANDLE_VALUE, PHTAAPT.MNFU, avun, /SET  ;
           HANDLE_VALUE, PHTAAPT.MDFL, medi, /SET  ;
           HANDLE_VALUE, PHTAAPT.Q1FL, m1qu, /SET  ;
           HANDLE_VALUE, PHTAAPT.Q3FL, m3qu, /SET  ;
           HANDLE_VALUE, PHTAAPT.MNBR, aver_br, /SET  ;
           HANDLE_VALUE, PHTAAPT.MNBU, avun_br, /SET  ;
           HANDLE_VALUE, PHTAAPT.MDBR, medi_br, /SET  ;
           HANDLE_VALUE, PHTAAPT.Q1BR, m1qu_br, /SET  ;
           HANDLE_VALUE, PHTAAPT.Q3BR, m3qu_br, /SET  ;
           HANDLE_VALUE, PHTAAPT.RPID, data.RPID(*,ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.DEC, data.DEC(ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.RA, data.RA(ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.ROLL, data.ROLL(ind(0)), /SET  ;
           HANDLE_VALUE, PHTAAPT.FLAG, intarr(N_ELEMENTS(aver)), /SET  ;

;  Update the PHTSPD structure using the temporary structure
           PHTAAP = [PHTAAP,PHTAAPT]

;  Write the output temporary structure
           pia_load_iaap, phtaap(N_ELEMENTS(phtaap)-1), tmpdata 
	   OUTMEAS = tmpdata
	   IF KEYWORD_SET(NO_INTO_BUFFER) THEN $
	     aap_init, index=N_ELEMENTS(phtaap)-1 ELSE BEGIN

	     decode_admn, phtaapt.admn, decnam

	     IF not KEYWORD_SET(NO_INFO) THEN $
               XDISPLAYFILE, "", $
                         TITLE = "Measurement in Buffer", $
                         WIDTH = 50, $
                         HEIGHT = 5, $
                         TEXT = ["The subtracted measurement is now in the ", $
                                 "aap buffer as: " + decnam ]
	   ENDELSE
	   
	   END        ; end AAP case
	   
        ENDCASE
	
         END
